local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )
local inventory = include( "sim/inventory" )
local abilitydefs = include( "sim/abilitydefs" )

local AI_Core = { ClassType = "AI_Core" }

function AI_Core:onWarp( sim, oldcell, cell )
	if oldcell and not cell then
		self:deactivate( sim )
	end

	if cell and self:getTraits().startOn == true then
		self:activate( sim )
		self:setPlayerOwner(sim:getNPC())
	end
end

function AI_Core:activate( sim )
	if self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"

		if self:getSounds().spot_old then
			self:getSounds().spot = self:getSounds().spot_old		
			self:getSounds().spot_old = nil
		end

		sim:addTrigger( simdefs.TRG_START_TURN, self )
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_on" )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
	end
end

function AI_Core:deactivate( sim )	
	sim:triggerEvent( simdefs.TRG_UNIT_DISABLED, self )

	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "inactive"

		self:getSounds().spot_old = self:getSounds().spot		
		self:getSounds().spot = nil			

		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		self:destroyTab()
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_off" )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
	end
end

function AI_Core:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_START_TURN and evData:isNPC() and self:getPlayerOwner() == evData then
		local x,y = self:getLocation()
		evData:addCPUs( 1, sim )
	end
end

-----------------------------------------------------
-- Interface functions

local function createAiCore( unitData, sim )
	return simunit.createUnit( unitData, sim, AI_Core )
end

simfactory.register( createAiCore )

return
{
	createAiCore = createAiCore,
}