local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )
local speechdefs = include("sim/speechdefs")
local mathutil = include( "modules/mathutil" )

local W93_combatAI_gainHash =
{
	name = STRINGS.PROGEXTEND.UI.AI_SCAN,
        proxy = true,
	HUDpriority = 1,

	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		local tooltip = util.tooltip( hud._screen )
		local section = tooltip:addSection()
		local canUse, reason = abilityUser:canUseAbility( sim, self, abilityOwner, targetUnitID )		
	        section:addLine( STRINGS.PROGEXTEND.UI.AI_COMBAT2 )
		section:addAbility( STRINGS.PROGEXTEND.UI.AI_SCAN_A, string.format(STRINGS.PROGEXTEND.UI.AI_SCAN_DESC, self:calculateHashes(abilityOwner, abilityUser, targetUnitID), math.max(abilityUser:getSkillLevel( "hacking" ) - 2, 0)), self.profile_icon )
		section:addAbility( STRINGS.PROGEXTEND.UI.AI_SCAN2, STRINGS.PROGEXTEND.UI.AI_SCAN_DESC2, "gui/icons/skills_icons/skills_icon_small/icon-item_analyze_small.png" )
		section:addAbility( STRINGS.PROGEXTEND.UI.AI_SCAN3, STRINGS.PROGEXTEND.UI.AI_SCAN_DESC5, "gui/icons/action_icons/Action_icon_Small/icon-action_lock_small.png" )
		section:addLine( string.format(STRINGS.PROGEXTEND.AI.TOKEN, sim:getPC():getTraits().aiToken, sim:getPC():getTraits().aiTokenMax) )
		if reason then
			section:addRequirement( reason )
		end
		return tooltip
	end,

	getName = function( self, sim, abilityOwner, abilityUser, targetUnitID )
		return string.format( STRINGS.PROGEXTEND.UI.AI_COMBAT)
	end,
		
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_scan_small.png",

        getProfileIcon = function( self, sim, abilityOwner )
		return abilityOwner:getUnitData().profile_icon or self.profile_icon
        end,

	calculateHashes = function( self, abilityOwner, unit, targetUnit )
		local bonus = unit:getTraits().hash_bonus or 0
            	if unit ~= abilityOwner then
                	bonus = bonus + (abilityOwner:getTraits().hash_bonus or 0)
           	end
		return bonus
	end,

	isTarget = function( self, abilityOwner, unit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, unit )
		local x0, y0 = unit:getLocation()
		local units = {}
		for _, targetUnit in pairs(sim:getAllUnits()) do
			local x1, y1 = targetUnit:getLocation()
			if x1 and self:isTarget( abilityOwner, unit, targetUnit ) and sim:getNPC():findMainframeAbility("W93_AI_assembly") and sim:getNPC():hasMainframeAbility("W93_AI_assembly") then
				local range = mathutil.dist2d( x0, y0, x1, y1 )
				if range <= 1 and simquery.isConnected( sim, sim:getCell( x0, y0 ), sim:getCell( x1, y1 ) ) then
					table.insert( units, targetUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, unit )
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		if abilityOwner ~= unit and abilityOwner:getUnitOwner() ~= unit then
			return false
		end

		if not sim:getNPC():findMainframeAbility("W93_AI_assembly") or not sim:getNPC():hasMainframeAbility("W93_AI_assembly") then
			return false
		end

		if (unit:getSkillLevel( "hacking" ) or 0) < 2 then
			return false, string.format( STRINGS.UI.TOOLTIP_REQUIRES_SKILL_LVL, "HACKING", 2 )
		end

		if abilityOwner ~= unit then
			if abilityOwner:getTraits().cooldown and abilityOwner:getTraits().cooldown > 0 then
				return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, abilityOwner:getTraits().cooldown)
			end
			if abilityOwner:getTraits().usesCharges and abilityOwner:getTraits().charges < 1 then
				return false, STRINGS.UI.REASON.CHARGES
			end
			if abilityOwner:getTraits().pwrCost and unit:getPlayerOwner():getCpus() < abilityOwner:getTraits().pwrCost then 
				return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
			end
		end

		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit then
			assert( self:isTarget( abilityOwner, unit, targetUnit ))
			if targetUnit:getTraits().mainframe_console_lock > 0 then
				return false, STRINGS.UI.REASON.CONSOLE_LOCKED
			end
			if not targetUnit:getTraits().hijacked then
				return false, STRINGS.PROGEXTEND.UI.REASON_NOT_HIJACKED
			end

			local dir = targetUnit:getFacing()
			local x0, y0 = targetUnit:getLocation()
			local x1, y1 = simquery.getDeltaFromDirection(dir)
			local consoleFront = sim:getCell( x0 + x1, y0 + y1 )

			if consoleFront ~= sim:getCell(unit:getLocation()) then
				return false, STRINGS.PROGEXTEND.UI.REASON_DIRECTION
			end
		end

		return abilityutil.checkRequirements( abilityOwner, unit )
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		sim:emitSpeech( unit, speechdefs.EVENT_HIJACK )
		local targetUnit = sim:getUnit( targetUnitID )
		local x1, y1 = targetUnit:getLocation()
    		local x0,y0 = unit:getLocation()
		local facing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
		local tokens = 1 + self:calculateHashes(abilityOwner, unit, targetUnitID)
		sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = unit:getID(), targetID = targetUnit:getID(), facing = facing, sound = "SpySociety/Actions/monst3r_jackin" , soundFrame = 16, useTinker=true } )
		sim:getPC():unlockAllMainframe( "fiend" )
		sim:getPC():getTraits().aiToken = math.min(sim:getPC():getTraits().aiToken + tokens, sim:getPC():getTraits().aiTokenMax)
		sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
	    	sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = util.sformat( STRINGS.PROGEXTEND.UI.PLUS_TOKEN, tokens), unit = unit, color={r=255/255,g=255/255,b=51/255,a=1}} )
            	sim:triggerEvent(simdefs.TRG_UNIT_HIJACKED, { unit = targetUnit, sourceUnit = unit, action = "gainHash" } )
		targetUnit:processEMP( 4 - math.max(unit:getSkillLevel( "hacking" ) - 2, 0), true )
		sim:processReactions( unit )
		inventory.useItem( sim, unit, abilityOwner )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
	end
}

return W93_combatAI_gainHash