local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

-------------------------------------------------------------------------
-- Killing an enemy using shootSingle restores full AP.
local melee_abilities = { ["melee"] = true, ["overwatchMelee"] = true, ["meleeOverwatch"] = true}

local ITB_SilicaAugment =
{
	name = "Archimedes Augment",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit
        sim:addTrigger( "TRG_UNIT_KILLED_BY_ARCHIMEDES", self )
        local user = unit:getUnitOwner()
		local abilities = user:getAbilities()
		if #abilities > 0 then
			for i = #abilities, 1, -1 do
				local ability = abilities[i]
				if melee_abilities[ability._abilityID] then
					table.remove(abilities, i)
				end
			end
		end		
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( "TRG_UNIT_KILLED_BY_ARCHIMEDES", self )
        self.abilityOwner = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
		if evType == "TRG_UNIT_KILLED_BY_ARCHIMEDES" then
			local target = self.abilityOwner:getUnitOwner()
			local x1, y1 = target:getLocation()
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = target } )
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.BREACH_AGENTS.UI.ARCHIMEDES_ATTACK,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )				
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT_NAME, x = x1, y = y1, color=cdefs.AUGMENT_TXT_COLOR} )

			target:getTraits().mp = target:getTraits().mpMax
			if not target:getTraits().sneaking then
				local SPRINT_BONUS = 3
					if target:hasTrait("sprintBonus") then
						SPRINT_BONUS = target:getTraits().sprintBonus
					end
				target:getTraits().mp = target:getTraits().mpMax + SPRINT_BONUS
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.SPRINTING, unit = target, color= cdefs.MOVECLR_DEFAULT })
			end			
        end
    end,
}

return ITB_SilicaAugment