local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

-------------------------------------------------------------------------
-- agent begins the mission cloaked; cloak lasts until agent attacks, sprints, or until alarm level 1

local function activateCloak( item )
	local owner = item:getUnitOwner()
	owner:setInvisible(true)
	owner:getTraits().BethanyCloak = true --this gets cleared if agent is decloaked for whatever reason before alarm level 1
end

local ITB_BethanyAugment =
{
	name = "Bethany Augment",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit --item
        sim:addTrigger( simdefs.TRG_UNIT_WARP, self ) --not sure if want?
		sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		activateCloak( unit )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
        self.abilityOwner = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
		local owner = self.abilityOwner:getUnitOwner()
		if (evType == simdefs.TRG_ALARM_STATE_CHANGE) and owner and owner:getTraits().BethanyCloak then
			owner:setInvisible(false)
		elseif (evType == simdefs.TRG_UNIT_WARP) and owner and (evData.unit == owner) then
			if not owner:getTraits().sneaking then
				owner:setInvisible(false)
				owner:getTraits().BethanyCloak = nil
			end
        end
    end,
}

return ITB_BethanyAugment