local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

-------------------------------------------------------------------------
-- Agent's attack is refreshed if they attack while in smoke. reset smoke attack bonus at turn end

-- check if missing AP is solely due to encumberance, to decide whether to display floaty text.
local function onlyEncumbered( unit, sim )
	if unit:getTraits().inventoryMaxSize then
		local overload = unit:getInventoryCount() - unit:getTraits().inventoryMaxSize
		if overload > (unit:getTraits().overloadCount or 0) then
			local tempAP = unit:getTraits().mpMax - overload
			if tempAP == unit:getTraits().mp then
				return true
			end
		end	
	end
	return false
end

-- do I want to use this?
local function applyItemMPPenalty( agent, sim )

	local traits = agent:getTraits()
	local totalPenalty = 0
	for i,unit in ipairs (agent:getChildren()) do
		if unit:getTraits().mp_penalty then
			traits.mp = traits.mp - unit:getTraits().mp_penalty
			totalPenalty = totalPenalty + unit:getTraits().mp_penalty
		end
	end
	if totalPenalty  > 0 then
	local x1,y1 = agent:getLocation()
		agent:getSim():dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, { txt=util.sformat(STRINGS.UI.FLY_TXT.HEAVY, totalPenalty ), x=x1,y=y1, color={r=163/255,g=0,b=0,a=1}, skipQue=false } ) 
	end

	if agent:getPlayerOwner() ~= nil and agent:getPlayerOwner() == sim:getPC() then 
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = agent } )
	end  
	
end


local ITB_CamilaAugment =
{
	name = "Camila Augment Ability",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit
        sim:addTrigger( simdefs.TRG_END_TURN, self )
		sim:addTrigger( "TRG_POST_START_TURN", self )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( simdefs.TRG_END_TURN, self )
		sim:removeTrigger( "TRG_POST_START_TURN", self )
        self.abilityOwner = nil
	end,

    onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_END_TURN then
			if self.abilityOwner and self.abilityOwner:getTraits().usedSmokeBonus then
				self.abilityOwner:getTraits().usedSmokeBonus = nil
			end
		elseif evType == "TRG_POST_START_TURN" and evData:isPC() then
			if self.abilityOwner then
				local agent = self.abilityOwner:getUnitOwner()
				if agent and agent:getTraits().mpMax then
					if not onlyEncumbered( agent, sim ) then
						local x1, y1 = agent:getLocation()
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.BREACH_AGENTS.TOOLTIPS.AP_RESIST,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
					end
					agent:getTraits().mp = agent:getMPMax()
					agent:getTraits().overloadCount = nil
					agent:checkOverload(sim)
					applyItemMPPenalty( agent, sim )
				end
			end
		end
    end,
}

return ITB_CamilaAugment