local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")

-----------------------------------------------------------------
-- Agent deploys in a random unseen tile instead of starting room
-- sim:dispatchEvent wrap handles removing Gana from the list of teleported units

local function canEnemySee( sim, x, y )
	for i, unit in pairs(sim:getAllUnits()) do
		if unit:getPlayerOwner() ~= sim:getPC() then
			local canSeeCell, canSenseCell = sim:canUnitSee(unit, x, y)
			if canSeeCell or canSenseCell then
				return true
			end
		end
	end
	return false
end

local function canPath( cellquery, unit, startcell, endcell )
    for i,cellUnit in ipairs(endcell.units) do
        if cellUnit:getTraits().dynamicImpass and cellUnit:getTraits().emitterID then
            return false
        end
    end

    return simquery.canStaticPath( cellquery, unit, startcell, endcell )
end

local function getSafeCell( sim, startCell )
	local safeCells = {}
	local exitroomCell = nil
	local startroomCell = nil
	local randomCell = nil
	
	sim:forEachCell(function(cell)	 --find exit
		if cell.exitID then 
			exitroomCell = cell
			return
		end	
	end)

	-- find regular entry
	sim:forEachCell(function(cell)
		if cell.deployID then 
			startroomCell = cell
			return
		end
	end)
	
	assert(startroomCell)
	assert(exitroomCell)

	-- collect cells that are not in the start room, not seen by the enemy, and can be pathed away from
	
	local candidateCells = simquery.floodFill( sim, nil, startCell, math.huge, nil, canPath  )
	for i, cell in pairs(candidateCells) do
        -- sim:forEachCell(function(cell)
		local x, y = cell.x, cell.y

		if 
		(cell.procgenRoom.roomIndex ~= startroomCell.procgenRoom.roomIndex) and (cell.procgenRoom.roomIndex ~= exitroomCell.procgenRoom.roomIndex)

		and not simquery.cellHasTag(sim, cell, "noguard")
		
		and not (#cell.units > 0)
		and not (cell.impass > 0 )
		
		-- and not sim:canPlayerSee( sim:getNPC(), x, y) --this doesn't work because cameras and turrets are nil-owned
		and not canEnemySee( sim, x, y )

		then
			table.insert(safeCells, cell)
		end
	end
	-- end)
	
	if #safeCells > 0 then
		randomCell = safeCells[sim:nextRand(1,#safeCells)]
	end
	return randomCell
end

local ITB_GanaAugment =
{
	name = "Gana Augment",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit --item
		self.abilityUser = unit:getUnitOwner() --agent
        sim:addTrigger( "Gana_finishedAgentConnection", self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( "Gana_finishedAgentConnection", self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
        self.abilityOwner = nil
		self.abilityUser = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
		if evType == "Gana_finishedAgentConnection" then
			-- log:write("LOG onfile gana")
			local unit = self.abilityUser		
			local script = sim:getLevelScript()
			local delay = 3
			if not (#sim:getPC():getUnits() > 1) then
				delay = 2
			end		
			local randomCell = getSafeCell(sim, sim:getCell(unit:getLocation()) )
			if randomCell then
				log:write("Found random cell")
				script:queue( { type="pan", x=randomCell.x, y=randomCell.y } )		
				sim:warpUnit( unit, randomCell )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, delay * cdefs.SECONDS )
				sim:dispatchEvent( simdefs.EV_TELEPORT, { units = { unit }, warpOut = false } )
			else
				log:write("didn't find free cell")
				local randomCell = sim:getCell(unit:getLocation())
				sim:dispatchEvent( simdefs.EV_TELEPORT, { units = { unit }, warpOut = false } )
			end
			unit:getTraits().hasSight = true
			sim:dispatchEvent( simdefs.EV_UNIT_HIDE, { unit = unit, hide = false } )
			sim:refreshUnitLOS( unit )
			unit:updateFacing( unit:getFacing() )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit  } )
		end
    end,
}

return ITB_GanaAugment
