local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")

-----------------------------------------------------------------
-- sim:dispatchEvent wrap handles removing Gana from the list of teleported units

local function GanaNotAlone( sim, GanaUnit )
	for i, unit in pairs(sim:getPC():getUnits()) do
		if (unit ~= GanaUnit) and not unit:isKO() then
			return true
		end
	end
	return false
end

local ITB_GanaAugment2 =
{
	name = "Gana Augment 2",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit --item
		self.abilityUser = unit:getUnitOwner() --agent
        sim:addTrigger( "Gana_a_finishedAgentConnection", self )
		sim:addTrigger("MAINFRAME_INIT", self )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( "Gana_a_finishedAgentConnection", self )
		sim:removeTrigger("MAINFRAME_INIT", self )
        self.abilityOwner = nil
		self.abilityUser = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

	onTrigger = function( self, sim, evType, evData )
		if (evType == "MAINFRAME_INIT") and (sim:getTurnCount() < 1) then
			local programs = sim:getPC()._mainframeAbilities
			for i, program in pairs(programs) do
				if program._abilityID == "W93_void" then
					sim:getTags().Gana_void = true
					-- log:write("LOG gana void")
					break
				end
			end
		elseif (evType == "Gana_a_finishedAgentConnection") and GanaNotAlone(sim, self.abilityUser) and not sim:getTags().firstGroupTeleportDone then
			log:write("LOG archived Gana going into deploy mode")
			sim:getTags().firstGroupTeleportDone = true
			self.abilityUser:getPlayerOwner():addMainframeAbility(sim, "ITB_deploymentMatrix" )
			sim:removeTrigger( "Gana_a_finishedAgentConnection", self )
			sim.deploymentMatrix = sim.deploymentMatrix or {} table.insert(sim.deploymentMatrix, self.abilityUser:getUnitData().agentID)
			self.abilityUser:getTraits().GanaHidden = nil
			sim:warpUnit( self.abilityUser, nil )
			sim:despawnUnit( self.abilityUser )
		elseif self.abilityUser and self.abilityUser:getTraits().GanaHidden and self.abilityUser:getLocation() then
			sim:dispatchEvent( simdefs.EV_UNIT_HIDE, { unit = self.abilityUser, hide = false } )
		end
	end,
}

return ITB_GanaAugment2