local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local level = include( "sim/level" )
-------------------------------------------------------------------------
-- Allow Henry to path through other units (but not other units through him) by fiddling with dynamicImpass in stupid ways

local cellOccupied = function( sim, unit, startcell, endcell )	-- Check for dynamic impass.  Note that even if dynamicImpass exists, if the unit owner doesn't know it's there, this won't fail.
	for i,cellUnit in ipairs(endcell.units) do
		if cellUnit:getTraits().dynamicImpass then
			-- Owned emitters will turn themselves off, so they are not considered impassable.
			if cellUnit:getTraits().emitterID == nil or cellUnit:isGhost() or not cellUnit:canControl( unit ) then
				return true
			end
		end
	end
	return false
end

local PC_AGENT_SELECTED =
{
	uiEvent = level.EV_UNIT_SELECTED,
	fn = function( sim, unitID )
		local unit = sim:getUnit( unitID )
		if unit and unit:isPC() then
			return unit
		end
	end
}

-- this feels risky but ok
local function onSelected(script, sim)
	while true do
		local _, unitID = script:waitFor( PC_AGENT_SELECTED )
		local unit = sim:getUnit( unitID )
		if unit:getTraits().ITB_HenryImpass then
			sim:triggerEvent("Henry_selected")
			sim:getTags().HenrySelected = true
			sim.HenryAgent = unit
			-- log:write("selected")
		else
			sim:triggerEvent("Henry_unselected")
			sim:getTags().HenrySelected = nil
			sim.HenryAgent = nil
			-- log:write("unselected")
		end
		-- sim:forEachUnit(
			-- function( u )
				-- sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = u } )
			-- end
		-- )
	end
	-- script:addHook(onSelected)
end

local function setNoImpass( unit )
	unit:getTraits().had_dynamicImpass = 
	unit:getTraits().had_dynamicImpass or unit:getTraits().dynamicImpass
	unit:getTraits().dynamicImpass = false
	log:write("setting no impass on Henry")
	log:write(util.stringize(unit:getTraits().dynamicImpass,1))
end

local function resetImpass( unit )
	-- if unit:getTraits().had_dynamicImpass then
		-- unit:getTraits().dynamicImpass = unit:getTraits().	had_dynamicImpass
	-- end
	unit:getTraits().dynamicImpass = true
	unit:getTraits().had_dynamicImpass = nil
	log:write("resetting impass on Henry")
	log:write(util.stringize(unit:getTraits().dynamicImpass,1))
end

local ITB_HenryImpassHelp =
{
	name = "ArchHenry Augment",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit --item
        -- sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP_PRE, self )
		sim:addTrigger( simdefs.TRG_END_TURN, self )
		sim:addTrigger( simdefs.TRG_UNIT_KO, self )
		sim:addTrigger( "Henry_selected", self )
		sim:addTrigger( "Henry_unselected", self )
		self.hook = sim:getLevelScript():addHook( "HENRY_IMPASS", onSelected, nil, unit )
		if self.abilityOwner:getTraits().installed and self.abilityOwner:getUnitOwner() then
			local owner = self.abilityOwner:getUnitOwner()
			setNoImpass(owner)
		end
	end,
        
	onDespawnAbility = function( self, sim, unit )
        -- sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP_PRE, self )
		sim:removeTrigger( simdefs.TRG_END_TURN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		sim:removeTrigger( "Henry_selected", self )
		sim:removeTrigger( "Henry_unselected", self )
		if self.abilityOwner:getUnitOwner() then
			local owner = self.abilityOwner:getUnitOwner()
			resetImpass(owner)
		end
        self.abilityOwner = nil
		sim:getLevelScript():removeHook( self.hook )
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
		local owner = self.abilityOwner:getUnitOwner()
		if (evType == simdefs.TRG_UNIT_WARP_PRE) then
			if owner and (evData.unit == owner) then
				if evData.to_cell then	
					local cellOccupied = cellOccupied( sim, owner, nil, evData.to_cell )
					if cellOccupied then
						owner:getTraits().notSeenByEnemy = true
					else
						owner:getTraits().notSeenByEnemy = nil
					end
					-- modinit/canUnitSeeUnit does the rest
				end
			end
		elseif (evType == simdefs.TRG_END_TURN) and owner and not owner:isKO() then
			if evData:isNPC() then
				log:write("NPC turn end")
				setNoImpass(owner)
			else
				log:write("PC turn end")
				resetImpass(owner)
			end
		elseif (evType == "Henry_selected") and owner and not owner:isKO() then
			setNoImpass(owner)
		elseif (evType == "Henry_unselected") and owner and not owner:isKO() then
			resetImpass(owner)
		elseif (evType == simdefs.TRG_UNIT_KO) and evData.unit:getTraits().ITB_HenryImpass and evData.ticks then
			-- set impass to false the way it normally would for a KO unit. When Henry is revived, wakeup code handles the impass reset
			setNoImpass(evData.unit)
        end
    end,	
}

return ITB_HenryImpassHelp