local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local unitdefs = include( "sim/unitdefs" )

-------------------------------------------------------------------
-- If overwatched, unit gains AP equal to 2x number of enemies overwatched by when he escapes overwatch

local function unitOverwatchedBy( unit )
	local overwatchEnemies = {}
    local aiPlayer = unit:getSim():getNPC()
    local situation = aiPlayer and aiPlayer:findExistingCombatSituation(unit)
    if situation and situation:isValid() then
    	for k,unit in pairs(situation.units) do
    		if not unit:isDown() and not unit:getTraits().pacifist then
    			table.insert(overwatchEnemies, unit)
			end
		end
    end
    return #overwatchEnemies
end

local ITB_HenryOverwatch =
{
	name = "Henry Augment 2",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit --item
        sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP_PRE, self )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP_PRE, self )
        self.abilityOwner = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
		local unit = self.abilityOwner:getUnitOwner()
		if (evType == simdefs.TRG_UNIT_WARP_PRE) then
			if unit and (evData.unit == unit) then
				if simquery.isUnitUnderOverwatch(unit) then
					local enemyCount = unitOverwatchedBy( unit )
					self.wasOverwatchedBy = enemyCount			
				end
			end
		elseif (evType == simdefs.TRG_UNIT_WARP) then
			if unit and (evData.unit == unit) then
				if self.wasOverwatchedBy then
					local multiplier = self.abilityOwner:getTraits().APmultiplier
					unit:addMP( self.wasOverwatchedBy * multiplier )
					self.wasOverwatchedBy = nil
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit  } )
					local x1, y1 = unit:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT2_NAME,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
		
				end
			end
        end
    end,
}

return ITB_HenryOverwatch