local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local unitdefs = include( "sim/unitdefs" )

-------------------------------------------------------------------
-- Can path/"squeeze" through an enemy  for 2 AP. Flips enemy facing direction.

-- Hugs and kisses to Puppetsquid and his Goose Protocol mod, from which most of this code is adapted from

local function checkForUnits( sim, cell )
	for i,unit in ipairs(cell.units) do
		if unit:getTraits().dynamicImpass ~= nil then
			return true
		end
	end
    return false
end

local peek_tooltip = class( abilityutil.hotkey_tooltip )

function peek_tooltip:init( hud, unit, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
	self._cost = 0
	if unit:getTraits().gooseRibbon then
	    		self._cost = 2
			else
				self._cost = 1
			end
end

function peek_tooltip:activate( screen )
	abilityutil.hotkey_tooltip.activate( self, screen )
	self._game.hud:previewAbilityAP( self._unit, self._cost )
end

function peek_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
	self._game.hud:previewAbilityAP( self._unit, 0 )
end


local squeeze_base =
		{
			testDir = simdefs.DIR_N,
			
			name = STRINGS.BREACH_AGENTS.ABILITIES.SCURRY,
			baseMPCost = 2,
			usesAction = false,
			onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			
				local legOwner, isEnemy = self:getLegs ( sim, abilityOwner )
				local tiptxt = STRINGS.BREACH_AGENTS.ABILITIES.SCURRY_TIP_ALLY
				if legOwner and isEnemy then
					tiptxt = STRINGS.BREACH_AGENTS.ABILITIES.SCURRY_TIP
				end
				self.usesAction = false
				
				local tip = util.sformat( tiptxt, self:getMPCost(sim,abilityUser)  )
				return peek_tooltip( hud, abilityUser, self, sim, abilityOwner, tip )
			end,
			
			getProfileIcon = function( self, sim, abilityOwner, abilityUser, hasTarget )
				return self.profile_icon
			end,
			
			usesMP = true,
			
			getMPCost = function( self, sim, goose )
				return self.baseMPCost
			end,
			
			profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_acrobatics_small.png",
			canUseWhileDragging = false,

			getName = function( self, sim, unit )		
				return self.name
			end,
			
			getCells = function (self, sim, abilityOwner)
				local x0,y0 = abilityOwner:getLocation()
				local xD,yD = simquery.getDeltaFromDirection( self.testDir )
				local startCell = sim:getCell( abilityOwner:getLocation() )
				local midCell = sim:getCell( x0+xD, y0+yD )
				local targCell = sim:getCell( x0+xD+xD, y0+yD+yD )
				local canPath = false
				if midCell and targCell then
					if 
					-- ( simquery.checkDynamicImpass(sim, midCell)) and
					checkForUnits( sim, midCell ) and
					not (targCell.impass > 0) and simquery.couldUnitSeeCell(sim, abilityOwner, targCell ) then
						canPath = true
					end
				end
				return startCell, midCell, targCell, canPath
			end,
			
			getLegs = function(self, sim, unit, cell, midCell)
				local cell = cell
				local midCell = midCell
				if not cell or not midCell then
					cell, midCell, targCell, canPath = self:getCells ( sim, unit )
				end
				local legUnit = nil
				local enemy = nil
				if midCell and ( simquery.checkDynamicImpass(sim, midCell)) then
					local skills = unit:getSkills()
					for i,cellUnit in ipairs(midCell.units) do
						if not cellUnit:isKO() then
							legUnit = cellUnit
						end
					end
					if legUnit then
						if legUnit:getTraits().isGuard or legUnit:getTraits().vip then
							enemy = true
						else
							enemy = false
						end
					end
				end
				return legUnit, enemy
			end,

			acquireTargets = function( self, targets, game, sim, abilityOwner, unit )		
			
				local targCells = {}
				local x0,y0 = unit:getLocation()
				local xD,yD = simquery.getDeltaFromDirection( self.testDir )
				local midCell = sim:getCell( x0+xD, y0+yD )
				if midCell then
					table.insert( targCells, sim:getCell( x0+xD, y0+yD ) )	
				end
				return targets.cellTarget( game, targCells, self, abilityOwner, unit )
			end,

			canUseAbility = function( self, sim, unit, abilityUser )
				local cost = self:getMPCost(sim,unit)
				local cell, midCell, targCell, canPath = self:getCells ( sim, unit )
				
				if midCell and targCell then
					if not canPath then
						return false
					end
					
					local legOwner, isEnemy = self:getLegs ( sim, unit, cell, midCell )
					
					if unit:getMP() < cost then
						return false, string.format(STRINGS.UI.REASON.REQUIRES_AP,cost)
					end
					
					if simquery.checkDynamicImpass(sim, targCell) then
						return false, STRINGS.BREACH_AGENTS.ABILITIES.SCURRY_BLOCKED
					end
					
				else
					return false
				end
				return true
			end,

			executeAbility = function( self, sim, abilityOwner, userUnit )
				local player = sim:getPC()
				local cell, midCell, targCell = self:getCells ( sim, abilityOwner )
				local legOwner, isEnemy = self:getLegs ( sim, abilityOwner, cell, midCell )
				local unit = userUnit
				
				unit:useMP( self:getMPCost(sim,unit),sim )
				sim:dispatchEvent( simdefs.EV_UNIT_START_WALKING, { unit = unit, reverse=true  } )
				sim:warpUnit(userUnit, targCell)
				sim:dispatchEvent( simdefs.EV_UNIT_STOP_WALKING, { unit = unit  } )
				
				sim:processReactions( unit )
				
				if legOwner then
					if isEnemy then
						-- local senses = legOwner:getBrain():getSenses()
						-- legOwner:getTraits().interrupted = nil
						-- senses:clearTargets()
						local newFacing = simquery.getReverseDirection(legOwner:getFacing())
						local x1, y1 = legOwner:getLocation()
----------------------------------------
						sim:dispatchEvent( simdefs.EV_UNIT_START_WALKING, { unit = legOwner, reverse=false  } )
						-- local cell = sim:getCell(x1, y1)
						-- sim:dispatchEvent( simdefs.EV_UNIT_WARPED, { unit = legOwner, from_cell = cell, to_cell = cell, facing = legOwner:getFacing(), reverse=true } )
					
						sim:dispatchEvent( simdefs.EV_UNIT_STOP_WALKING, { unit = legOwner  } )
						-- legOwner:updateFacing( newFacing ) --this flips the unit.
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = legOwner } )	
						
						
-----------------------------------------					
						
						
						sim:getNPC():spawnInterest(x1, y1, simdefs.SENSE_HEARING, simdefs.REASON_NOISE, legOwner)
							
						-- local actions = include("sim/btree/actions")
						-- actions.DoLookAround(sim, legOwner)						
					end
				end
				-- sim:dispatchEvent( simdefs.EV_UNIT_STOP_WALKING, { unit = unit  } )
				-- sim:processReactions( unit ) -- move this to before guard refresh so you can escape guard overwatch.
			end,
		}


local henry_squeeze = 
{
	ITB_henry_squeeze_up = util.extend( squeeze_base )
		 {
			testDir = simdefs.DIR_N,
		},
	ITB_henry_squeeze_down = util.extend( squeeze_base )
		{
			testDir = simdefs.DIR_S,
		},
	ITB_henry_squeeze_left = util.extend( squeeze_base )
		{
			testDir = simdefs.DIR_E,
		},
	ITB_henry_squeeze_right = util.extend( squeeze_base )
		{
			testDir = simdefs.DIR_W,
		},
}
return henry_squeeze