local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

-------------------------------------------------------------------------
-- If agent hasn't moved this turn, attacking refreshes attack and grants infinite attacks. Handled here and in modinit.

local ITB_SilicaAugment =
{
	name = "Silica Augment",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit
        sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_START_TURN, self )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
        self.abilityOwner = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_START_TURN and evData:isPC() then
			local SilicaAgent = self.abilityOwner:getUnitOwner()
			if SilicaAgent then
				SilicaAgent:getTraits().notYetMoved = true
				local x1, y1 = SilicaAgent:getLocation()
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.BREACH_AGENTS.UI.INFINITE_ATTACKS,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} ) 
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT2_NAME, x = x1, y = y1, color=cdefs.AUGMENT_TXT_COLOR} )				
			end
		elseif evType == simdefs.TRG_UNIT_WARP then
			if self.abilityOwner and evData.unit == self.abilityOwner:getUnitOwner() and evData.unit:getPlayerOwner() and evData.to_cell and evData.unit:getTraits().notYetMoved then
				evData.unit:getTraits().notYetMoved = nil
				local SilicaAgent = evData.unit
				local x1, y1 = SilicaAgent:getLocation()
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.BREACH_AGENTS.UI.INFINITE_ATTACKS_LOST,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} ) 				
			end  
        end
    end,
}

return ITB_SilicaAugment