local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local itemdefs = include("sim/unitdefs/itemdefs")
local guarddefs = include("sim/unitdefs/guarddefs")
local propdefs = include("sim/unitdefs/propdefs")

-- credit to Momo's Agent Valkyrie mod for the basis of this code, used with permission <3
local ITB_throwIceMine =
	{
		name = STRINGS.ABILITIES.THROW,

		getName = function( self, sim, unit, userUnit )
			return self.name
		end,
	
		createToolTip = function( self,sim,unit,targetCell)
			return abilityutil.formatToolTip( STRINGS.ABILITIES.THROW,  STRINGS.ABILITIES.THROW_DESC, 1 )
		end,
	
		profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png",
		usesAction = true,

		acquireTargets = function( self, targets, game, sim, grenadeUnit, unit)
			if not self:canUseAbility( sim, grenadeUnit, unit ) then
				return nil
			end
			return targets.throwTarget( game, grenadeUnit:getTraits().range or 0, sim, unit, unit:getTraits().maxThrow, grenadeUnit:getTraits().targeting_ignoreLOS)
		end, 


		canUseAbility = function( self, sim, grenadeUnit, unit, targetCell )
		
            if unit:getTraits().movingBody then
                return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
            end
			
			local ok, reason = abilityutil.canConsumeAmmo( sim, grenadeUnit )
            if not ok then
                return false
            end

			if grenadeUnit:getTraits().restrictedUse then

				local userUnitAgentID = unit:getUnitData().agentID
				local canUse =false
				for i,set in pairs(grenadeUnit:getTraits().restrictedUse )do
					if set.agentID == userUnitAgentID then
						canUse=true
					end
				end


				if not canUse then
					return false,  STRINGS.UI.REASON.RESTRUCTED_USE
				end				
			end

			if grenadeUnit:getTraits().cooldown and grenadeUnit:getTraits().cooldown > 0 then
				return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,grenadeUnit:getTraits().cooldown)
			end

			if grenadeUnit:getTraits().usesCharges and grenadeUnit:getTraits().charges < 1 then
				return false, util.sformat(STRINGS.UI.REASON.CHARGES)
			end

			if grenadeUnit:getTraits().pwrCost and unit:getPlayerOwner():getCpus() < grenadeUnit:getTraits().pwrCost then
				return false, util.sformat(STRINGS.UI.REASON.NOT_ENOUGH_PWR)
			end

			if targetCell then
				local targetX,targetY = unpack(targetCell)
				local unitX, unitY = unit:getLocation()
	    		local raycastX, raycastY = sim:getLOS():raycast(unitX, unitY, targetX, targetY)
				if raycastX ~= targetX or raycastY ~= targetY then
					return false
				end
				local throwRange = grenadeUnit:getTraits().throwRange or math.max
				local range = mathutil.dist2d(unitX, unitY, targetX, targetY)
				if range > throwRange then
					return false
				end
				-- weirdly, this bit doesn't do anything
				if simquery.checkDynamicImpass(sim, targetCell) then
					return false
				end
			end

			return true
		end,
	   
		-----------------
		
		executeAbility = function( self, sim, grenadeUnit, userUnit, targetCell )
			local sim = grenadeUnit:getSim()
			local x0,y0 = userUnit:getLocation()
					
			local x1,y1 = unpack(targetCell)
			local cell = sim:getCell( userUnit:getLocation() )
			
			if simquery.checkDynamicImpass(sim, sim:getCell(x1,y1)) then
				local script = sim:getLevelScript()
				script:queue( { type="showWarning", txt1=STRINGS.BREACH_AGENTS.UI.INVALID_TILE, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
				return
			end
		
			local oldFacing = userUnit:getFacing()
			local newFacing = simquery.getDirectionFromDelta(x1-x0, y1-y0)
			simquery.suggestAgentFacing(userUnit, newFacing)
			if userUnit:getBrain() then	
				if grenadeUnit:getTraits().baseDamage then
					sim:emitSpeech(userUnit, speechdefs.HUNT_GRENADE)
				end
				sim:refreshUnitLOS( userUnit )
				sim:processReactions( userUnit )
			end

			if userUnit:isValid() and not userUnit:getTraits().interrupted then

				local newUnit = nil
				local player = userUnit:getPlayerOwner()

				local x0, y0 = userUnit:getLocation()
				
				local oldWeapon = simquery.getEquippedGun( userUnit )
				grenadeUnit:getTraits().slot = "gun"
				if not oldWeapon == grenadeUnit then
					inventory.unequipItem( userUnit, oldWeapon )
					inventory.equipItem( userUnit, grenadeUnit )
				end
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
				
				local pinning, pinnee = simquery.isUnitPinning(userUnit:getSim(), userUnit)
				
				-- log:write("LOG EV_UNIT_START_SHOOTING")
				sim:dispatchEvent( simdefs.EV_UNIT_START_SHOOTING, { unitID = userUnit:getID(), newFacing=newFacing, oldFacing=oldFacing, pinning=pinning } )

				local dmgt = abilityutil.createShotDamage( grenadeUnit, userUnit )
				local evData = { unitID = userUnit:getID(), x0 = x0, y0 = y0, x1=x1, y1=y1, dmgt = dmgt } 	
	
				-- log:write("LOG EV_UNIT_SHOT")
				sim:dispatchEvent( simdefs.EV_UNIT_SHOT, evData )
	
				local mineUnit = simfactory.createUnit( propdefs.ITB_iceMine, sim )
				sim:spawnUnit( mineUnit )
				sim:warpUnit( mineUnit, sim:getCell(x1, y1) )
				
				-- log:write("LOG EV_UNIT_STOP_SHOOTING")
				sim:dispatchEvent( simdefs.EV_UNIT_STOP_SHOOTING, { unitID = userUnit:getID(), facing=newFacing, pinning=pinning} )	

				inventory.useItem( sim, userUnit, grenadeUnit )

				if userUnit:isValid() then
					userUnit:setInvisible(false)
				end				
				sim:processReactions()
				
			end
			if userUnit:isValid() and not userUnit:getTraits().interrupted then
				simquery.suggestAgentFacing(userUnit, newFacing)
			end

		end,
	}

return ITB_throwIceMine


-------------- Backup/reference: The below execute block does the same with slightly different FX - a temporary grenade unit is spawned and "thrown" by the gun, goes flying, then despawns and is replaced by the final unit. After getting it to work, I scrapped the grenade spawn after all because I like the grenade-less look better.

	-- executeAbility = function( self, sim, grenadeUnit, userUnit, targetCell )
		-- local sim = grenadeUnit:getSim()
		-- local x0,y0 = userUnit:getLocation()
				
		-- local x1,y1 = unpack(targetCell)
		-- local cell = sim:getCell( userUnit:getLocation() )
		
		-- if simquery.checkDynamicImpass(sim, sim:getCell(x1,y1)) then
			-- local script = sim:getLevelScript()
			-- script:queue( { type="showWarning", txt1=STRINGS.BREACH_AGENTS.UI.INVALID_TILE, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
			-- return
		-- end
		
		-- userUnit:getTraits().throwing = true --GRENADE EFFECT
	
		-- local oldFacing = userUnit:getFacing()
		-- local newFacing = simquery.getDirectionFromDelta(x1-x0, y1-y0)
		-- simquery.suggestAgentFacing(userUnit, newFacing)
		-- if userUnit:getBrain() then	
			-- if grenadeUnit:getTraits().baseDamage then
				-- sim:emitSpeech(userUnit, speechdefs.HUNT_GRENADE)
			-- end
			-- sim:refreshUnitLOS( userUnit )
			-- sim:processReactions( userUnit )
		-- end

		-- if userUnit:isValid() and not userUnit:getTraits().interrupted then

			-- local newUnit = nil
			-- local player = userUnit:getPlayerOwner()

			-- newUnit = simfactory.createUnit( itemdefs.item_crybaby, sim )

			-- local x0, y0 = userUnit:getLocation()
			
			-- local oldWeapon = simquery.getEquippedGun( userUnit )
			-- grenadeUnit:getTraits().slot = "gun"
			-- if not oldWeapon == grenadeUnit then
				-- inventory.unequipItem( userUnit, oldWeapon )
				-- inventory.equipItem( userUnit, grenadeUnit )
			-- end
			-- sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
			
			-- local pinning, pinnee = simquery.isUnitPinning(userUnit:getSim(), userUnit)
			
			-- log:write("LOG EV_UNIT_START_SHOOTING")
			-- sim:dispatchEvent( simdefs.EV_UNIT_START_SHOOTING, { unitID = userUnit:getID(), newFacing=newFacing, oldFacing=oldFacing, pinning=pinning } )

			-- local dmgt = abilityutil.createShotDamage( grenadeUnit, userUnit )
			-- local evData = { unitID = userUnit:getID(), x0 = x0, y0 = y0, x1=x1, y1=y1, dmgt = dmgt } 	

			-- log:write("LOG EV_UNIT_SHOT")
			-- sim:dispatchEvent( simdefs.EV_UNIT_SHOT, evData )

			-- sim:spawnUnit( newUnit )
			-- sim:warpUnit(newUnit, sim:getCell(x0, y0))
	
			-- assert( player )
			-- newUnit:setPlayerOwner(player)
			-- log:write("LOG EV_UNIT_THROWN")
			
			-- -- the throw event is responsible for the FX of the canister flying at the target cell.
			
			-- sim:dispatchEvent( simdefs.EV_UNIT_THROWN, { unit = newUnit, x=x1, y=y1 } )

			-- if x0 ~= x1 or y0 ~= y1 then
				-- sim:warpUnit(newUnit, sim:getCell(x1, y1))
			-- end
			
			-- newUnit:getTraits().throwingUnit = userUnit:getID()
			
			-- log:write("LOG EV_UNIT_STOP_SHOOTING")
			-- sim:dispatchEvent( simdefs.EV_UNIT_STOP_SHOOTING, { unitID = userUnit:getID(), facing=newFacing, pinning=pinning} )	
			
			-- if newUnit:getTraits().keepPathing == false and player:getBrain() then
				-- player:useMP(player:getMP(), sim)
			-- end
			
			-- userUnit:getTraits().throwing = nil
			-- inventory.useItem( sim, userUnit, grenadeUnit )
			
			-- -- now that flying unit has arrived, despawn it spawn the actual ice mine.
			-- sim:warpUnit( newUnit, nil )
			-- sim:despawnUnit( newUnit )
			-- local mineUnit = simfactory.createUnit( itemdefs.item_flash_pack, sim ) --placeholder
			-- sim:spawnUnit( mineUnit )
			-- sim:warpUnit( mineUnit, sim:getCell(x1, y1) )
			-- -- mineUnit:activate() -- comment this back in!
			
			-- sim:processReactions()
			
		-- end
		-- if userUnit:isValid() and not userUnit:getTraits().interrupted then
			-- simquery.suggestAgentFacing(userUnit, newFacing)
		-- end

	-- end,
	---------------