local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

local coit_conversion_item =
{
	name = STRINGS.CORPORATE_ITEMS.ABILITIES.CONVERSION.NAME,
	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.CORPORATE_ITEMS.ABILITIES.CONVERSION.NAME, STRINGS.CORPORATE_ITEMS.ABILITIES.CONVERSION.DESC , 0 )
	end,

	profile_icon = "gui/icons/coit_conversion_item.png",

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, itemUnit, unit )
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not player:getTraits().coit_conversion then
			return false --Don't show if no conversion program
		end
		if not itemUnit:getTraits().cooldown then
			return false --Don't show for items that don't have a cooldown
		end
		if itemUnit:getTraits().cooldown <= 0 then
			return false, STRINGS.CORPORATE_ITEMS.ABILITIES.CONVERSION.NO_CD
		end
		local conversion_program = player:getTraits().coit_conversion
		if conversion_program.cooldown > 0 then
			return false, STRINGS.CORPORATE_ITEMS.ABILITIES.CONVERSION.CONVERSION_CD
		end

		return true
	end,
	
	executeAbility = function( self, sim, itemUnit, unit )
		local player = unit:getPlayerOwner()
		if player then
			local conversion_program = player:getTraits().coit_conversion
			if conversion_program then
				conversion_program.maxCooldown = itemUnit:getTraits().cooldown
				conversion_program:setCooldown( sim )
				
				itemUnit:getTraits().cooldown = 0
				if unit then
					local x0, y0 = unit:getLocation()
					sim:emitSound( { path = "SpySociety/Actions/recharge_item", range = simdefs.SOUND_RANGE_0 }, x0, y0, unit )
				end
			end
		end
	end,
}

return coit_conversion_item
