local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local coit_daemon_blocker =
{
	name = STRINGS.CORPORATE_ITEMS.ABILITIES.DAEMON_BLOCKER.NAME,
	
	alwaysShow = true,
	maa_activeAbility = true,
	usesAction = false, --we use this to better indicate toggle status
	
	createToolTip = function( self, sim, unit )
		log:write("Create tooltip")
		local userUnit = unit:getUnitOwner()
		if unit:getTraits().toggle then
			log:write("ON tooltip")
			return abilityutil.formatToolTip( STRINGS.CORPORATE_ITEMS.ABILITIES.DAEMON_BLOCKER.NAME_OFF, STRINGS.CORPORATE_ITEMS.ABILITIES.DAEMON_BLOCKER.DESC_OFF )
		else
			log:write("OFF tooltip")
			return abilityutil.formatToolTip( STRINGS.CORPORATE_ITEMS.ABILITIES.DAEMON_BLOCKER.NAME_ON, STRINGS.CORPORATE_ITEMS.ABILITIES.DAEMON_BLOCKER.DESC_ON )
		end
	end,
	
	profile_icon = "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png",
	getProfileIcon =  function( self, sim, unit )
			if unit:getTraits().toggle then
				return "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png"
			else
				return "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png"
			end
		end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		if not unit:getTraits().installed then
			return false
		end
		
		if unit:getTraits().ammo and unit:getTraits().ammo <= 0 then
			return false, STRINGS.CORPORATE_ITEMS.ABILITIES.DAEMON_BLOCKER.NO_AMMO
		end

		return true
	end,
	
	canBlockDaemon = function( self, sim )
		if not self.unit then
			return false
		end
		if self.unit:getTraits().ammo and self.unit:getTraits().ammo <= 0 then
			return false
		end
		
		return true
	end,
	
	daemonBlocked = function( self, sim )
		if self.unit then
			if self.unit:getTraits().ammo then
				self.unit:getTraits().ammo = self.unit:getTraits().ammo - 1
			end
			
			if self.unit:getTraits().ammo <= 0 then
				local player = self.unit:getPlayerOwner()
				self.unit:getTraits().toggle = false
				self.usesAction = false
				if player:getTraits().coit_daemon_block then
					for i, ability in ipairs( player:getTraits().coit_daemon_block ) do
						if ability == self then
							table.remove( player:getTraits().coit_daemon_block, i)
							break
						end
					end
				end
			end
		end
	end,

	executeAbility = function( self, sim, unit )
		local player = unit:getPlayerOwner()
		if unit:getTraits().toggle then
			unit:getTraits().toggle = false
			self.usesAction = false
			if player:getTraits().coit_daemon_block then
				for i, ability in ipairs( player:getTraits().coit_daemon_block ) do
					if ability == self then
						table.remove( player:getTraits().coit_daemon_block, i)
						break
					end
				end
			end
		else
			unit:getTraits().toggle = true
			self.usesAction = true
			if not player:getTraits().coit_daemon_block then
				player:getTraits().coit_daemon_block = {}
			end
			table.insert( player:getTraits().coit_daemon_block, self )
		end
	end,

	onSpawnAbility = function( self, sim, unit )
		self.unit = unit
		
		local userUnit = unit:getUnitOwner()
		if unit:getTraits().maxAmmo then
			unit:getTraits().ammo = unit:getTraits().maxAmmo
		end
	end,

	onDespawnAbility = function( self, sim, unit )
		local player = unit:getPlayerOwner()
		if player then
			if player:getTraits().coit_daemon_block then
				for i, ability in ipairs( player:getTraits().coit_daemon_block ) do
					if ability == self then
						table.remove( player:getTraits().coit_daemon_block, i)
						break
					end
				end
			end
		end
	end,
}

return coit_daemon_blocker
