local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local coit_daemon_siphon =
{
	name = STRINGS.CORPORATE_ITEMS.ITEMS.DAEMON_SIPHON_NAFRICA.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( "TRG_COIT_TRUE_DAEMON_INSTALL", self, unit )
		sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( "TRG_COIT_TRUE_DAEMON_INSTALL", self, unit )
		sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == "TRG_COIT_TRUE_DAEMON_INSTALL" then
				--we listen first to the true daemon trigger to figure out if the following daemon install is a reversable daemon
				self.true_daemon = false
				if evData then
					self.true_daemon = true
				end
			elseif evType == simdefs.TRG_DAEMON_INSTALL then
				if self.true_daemon or not unit:getTraits().trueDaemon then
					if not unit:getTraits().coit_proc_max or not self.procs or self.procs < unit:getTraits().coit_proc_max then
						self.procs = (self.procs or 0) + 1
						local userUnit = unit:getUnitOwner()
						local x1, y1 = userUnit:getLocation()
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.CORPORATE_ITEMS.UI.FLY.DAEMON_SIPHON, x = x1, y = y1, color={r=255/255,g=255/255,b=51/255,a=1}} )
						
						--reduce item cooldowns for north african variant
						if unit:getTraits().coit_cd_reduction_item then
							for _, item in pairs( userUnit:getChildren() ) do
								if item:getTraits().cooldown then
									item:getTraits().cooldown = math.max(item:getTraits().cooldown - unit:getTraits().coit_cd_reduction_item, 0)
								end
							end
						end
						
						--boost ap for south african variant
						if unit:getTraits().coit_ap_boost then
							userUnit:addMP( unit:getTraits().coit_ap_boost )
							sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
						end
					end					
				end
			elseif evType == simdefs.TRG_END_TURN then
				if evData:isNPC() then
					self.procs = nil
				end
			end
		end
	end,
}

return coit_daemon_siphon
