local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local mainframe = include( "sim/mainframe" )

local coit_laptop =
{
	name = STRINGS.CORPORATE_ITEMS.ABILITIES.PORTABLE_DAEMON_SERVER.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_UNIT_PICKEDUP, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_DEPLOYED, self, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_PICKEDUP, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_DEPLOYED, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local x0, y0 = unit:getLocation()
		if evType == simdefs.TRG_UNIT_DEPLOYED then
			if evData.unit == unit then
				local player = unit:getPlayerOwner()
				
				---------------------
				----daemon server----
				---------------------
				if unit:getTraits().coit_add_reverse_odds then
					player:getTraits().coit_add_reversal_odds = (player:getTraits().coit_add_reversal_odds or 0) + (unit:getTraits().coit_add_reverse_odds or 0)
				end
				if unit:getTraits().coit_pwr_capacity then
					player:getTraits().PWRmaxBouns = (player:getTraits().PWRmaxBouns or 0) + unit:getTraits().coit_pwr_capacity
					player:addCPUs( math.min( player:getMaxCpus() - player:getCpus(), 0 ) ) --remove any power over maxPWR
				end
				---------------------
			end
		elseif evType == simdefs.TRG_UNIT_PICKEDUP then
			if evData.item == unit and unit:getTraits().deployed then
				local player = unit:getPlayerOwner()
				
				---------------------
				----daemon server----
				---------------------
				if unit:getTraits().coit_add_reverse_odds then
					player:getTraits().coit_add_reversal_odds = (player:getTraits().coit_add_reversal_odds or 0) - (unit:getTraits().coit_add_reverse_odds or 0)
				end
				if unit:getTraits().coit_pwr_capacity then
					player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns - unit:getTraits().coit_pwr_capacity
					player:addCPUs( math.min( player:getMaxCpus() - player:getCpus(), 0 ) ) --remove any power over maxPWR
				end
				---------------------
			end
		elseif evType == simdefs.TRG_START_TURN then
			if evData:isPC() then
				if unit:getTraits().coit_trigger_cooldown and unit:getTraits().deployed then
					unit:getTraits().coit_trigger_cooldown = unit:getTraits().coit_trigger_cooldown - 1
					
					if unit:getTraits().coit_trigger_cooldown <= 0 then
						--------------------
						----scanning amp----
						--------------------
						if unit:getTraits().coit_mainframe_reveal then
							local potentialUnits = {}
							for _, checkUnit in pairs( sim:getAllUnits() ) do
								if checkUnit:getTraits().mainframe_item and not sim:getPC():hasSeen( checkUnit ) then
									table.insert( potentialUnits, checkUnit )
								end
							end

							if #potentialUnits > 0 then
								local selectedUnit = potentialUnits[ sim:nextRand(1, #potentialUnits) ]
								sim:getPC():glimpseUnit( sim, selectedUnit:getID() )
								sim:dispatchEvent( simdefs.EV_CAM_PAN, { selectedUnit:getLocation() } )
								sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30 )
								sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = selectedUnit, coit_reveal = true } )
							end
						end
						--------------------
						--------------
						----router----
						--------------
						if unit:getTraits().coit_random_ice_break then
							local potentialUnits = {}
							for _, checkUnit in pairs( sim:getAllUnits() ) do
								if checkUnit:getTraits().mainframe_item and checkUnit:getTraits().mainframe_status == "active" and ( not checkUnit:getPlayerOwner() or not checkUnit:getPlayerOwner():isPC() ) and sim:getPC():hasSeen( checkUnit ) then
									table.insert( potentialUnits, checkUnit )
								end
							end
							
							if #potentialUnits > 0 then
								local selectedUnit = potentialUnits[ sim:nextRand(1, #potentialUnits) ]
								sim:dispatchEvent( simdefs.EV_CAM_PAN, { selectedUnit:getLocation() } )
								mainframe.breakIce( sim, selectedUnit, unit:getTraits().coit_random_ice_break )
							end
						end
						--------------
						--------------------
						----drain server----
						--------------------
						if unit:getTraits().coit_drain_ai_pwr then
							local pwr = math.min( unit:getTraits().coit_drain_ai_pwr, sim:getNPC():getCpus() )
							sim:getNPC():addCPUs( -pwr )
							if unit:getTraits().coit_steal_pwr then
								sim:getPC():addCPUs( pwr )
								if x0 and y0 then
									sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.PLUS_PWR, pwr), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
								end
							end
						end
						--------------------
						
						unit:getTraits().coit_trigger_cooldown = unit:getTraits().coit_trigger_cooldownMax
					end
				end
			end
		end
	end,
}

return coit_laptop
