local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")

local coit_use_chip =
{
	name = STRINGS.CORPORATE_ITEMS.ABILITIES.USE_CHIP.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, unit, targetCell)
		return abilityutil.formatToolTip( unit:getTraits().use_ability_name or STRINGS.CORPORATE_ITEMS.ABILITIES.USE_CHIP.NAME, unit:getTraits().use_ability_desc or STRINGS.CORPORATE_ITEMS.ABILITIES.USE_CHIP.DESC )
	end,

	proxy = true,

	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chip_hyper_buster_small.png",
	
	getProfileIcon =  function( self, sim, unit )
		return unit:getUnitData( ).profile_icon or self.profile_icon
	end,

	isTarget = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local cell = sim:getCell( abilityUser:getLocation() )
		local units = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTarget( sim, abilityUser, cellUnit ) then
					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		if not userUnit then
			return false
		end
		if unit:getUnitOwner() ~= userUnit then
			return false
		end
		local player = sim:getPC()
		if unit:getTraits().movingBody then
				return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
		end

		return abilityutil.checkRequirements( unit, userUnit )
	end,
	
	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,
	
	onTrigger = function( self, sim, evType, evData, unit )
		sim:getPC():getTraits().coit_alarm_block_turn = nil
	end,

	executeAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local player = unit:getPlayerOwner()
		local x0, y0 = userUnit:getLocation()

		if targetUnit then
			if unit:getTraits().coit_gain_cpu then
				local pwr = unit:getTraits().coit_gain_cpu
				player:addCPUs( pwr )
				sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.PLUS_PWR, pwr), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
			end
			
			if unit:getTraits().coit_alarm_block then
				player:getTraits().coit_alarm_block_turn = true
			end
			
			inventory.useItem( sim, userUnit, unit )
		end
	end,
}

return coit_use_chip
