local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local coit_weapon_power_buff =
{
	name = STRINGS.CORPORATE_ITEMS.ABILITIES.WEAPON_POWER_BUFF.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
	alwaysShow = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		return abilityutil.formatToolTip( STRINGS.CORPORATE_ITEMS.ABILITIES.WEAPON_POWER_BUFF.NAME, STRINGS.CORPORATE_ITEMS.ABILITIES.WEAPON_POWER_BUFF.DESC )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		
		if unit:getTraits().coit_weapon_power_buff_cost and player:getCpus( ) < unit:getTraits().coit_weapon_power_buff_cost then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end

		return true
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if unit:getTraits().coit_weapon_power_buffs then
			for _, buff in ipairs( unit:getTraits().coit_weapon_power_buffs ) do
				if buff.trait then
					unit:getTraits()[buff.trait] = (unit:getTraits()[buff.trait] or 0) + (buff.value or 0)
					if not unit:getTraits().coit_temporary_buffs then
						unit:getTraits().coit_temporary_buffs = {}
					end
					table.insert( unit:getTraits().coit_temporary_buffs, {trait = buff.trait, value = buff.value} )
				end
			end
			
			if unit:getTraits().coit_weapon_power_buff_cost then
				local player = unit:getPlayerOwner()
				local cpus = unit:getTraits().coit_weapon_power_buff_cost
				player:addCPUs( -cpus )
				local x0, y0 = userUnit:getLocation()
				sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
			end
		end
	end,
}

return coit_weapon_power_buff
