local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilitydefs = include( "sim/abilitydefs" )
local inventory = include("sim/inventory")

local oldDoorMechanism = abilitydefs.lookupAbility("doorMechanism")
local oldCanUseAbility = oldDoorMechanism.canUseAbility
local oldExecuteAbility = oldDoorMechanism.executeAbility

local doorMechanism = util.extend(oldDoorMechanism)
{
	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local exits = {}
		local fromCell = sim:getCell( abilityUser:getLocation() )
		local x0,y0 = fromCell.x, fromCell.y
            	local applyFn = abilityOwner:getTraits().applyFn
           	if type(applyFn) == "string" then
                	applyFn = simquery[ applyFn ] -- Sooo brutal, but due to circular includes, itemdefs cannot directly reference simquery functions
            	end

		for dir, exit in pairs( fromCell.exits ) do
			if abilityOwner:getTraits().decodeVault then
				if applyFn( exit ) and exit.locked and (exit.keybits == simdefs.DOOR_KEYS.VAULT or exit.keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT) then
					table.insert( exits, { x = fromCell.x, y = fromCell.y, dir = dir } )
				end
			else
				if applyFn( exit ) then
					table.insert( exits, { x = fromCell.x, y = fromCell.y, dir = dir } )
				end
			end
		end

		return targets.exitTarget( game, exits, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit, abilityUser, x, y, dir )
		local val, txt = oldCanUseAbility( self, sim, unit, abilityUser, x, y, dir )

		if unit:getTraits().ammo and unit:getTraits().ammo <= 0 then
			return false, STRINGS.UI.REASON.NO_AMMO
		end

		return val, txt
	end,

	executeAbility = function( self, sim, unit, userUnit, x0, y0, dir )
		local fromCell = sim:getCell( x0, y0 )
            	assert( fromCell and dir )

		userUnit:resetAllAiming()
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR, { unitID = userUnit:getID(), facing = dir } )
		local doorDevice = nil
		if unit:getTraits().decodeVault then
			doorDevice = include( SCRIPT_PATHS.W93_NIAA .. "/W93_lock_decoder" )
		else
			doorDevice = include( string.format( "sim/units/%s", unit:getTraits().doorDevice ))
		end
            	doorDevice.applyToDoor( sim, fromCell, dir, unit, userUnit )
		inventory.useItem( sim, userUnit, unit )

		sim:emitSound( simdefs.SOUND_PLACE_TRAP, x0, y0, userUnit)	

		sim:processReactions( userUnit )
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR_PST, { unitID = userUnit:getID(), facing = dir } )	
		simquery.LEVER_resetStatus( sim, userUnit, "ANY", false )
	end,
}
return doorMechanism