local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local abilitydefs = include( "sim/abilitydefs" )

local oldJackin = abilitydefs.lookupAbility("jackin")
local oldExecuteAbility = oldJackin.executeAbility
local oldCanUseAbility = oldJackin.canUseAbility

local jackin = util.extend(oldJackin)
{
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		local tooltip = util.tooltip( hud._screen )
		local section = tooltip:addSection()
		local canUse, reason = abilityUser:canUseAbility( sim, self, abilityOwner, targetUnitID )		
		local targetUnit = sim:getUnit( targetUnitID )
	        section:addLine( targetUnit:getName() )
		if (targetUnit:getTraits().cpus or 0) > 0 then
			if abilityOwner:getTraits().PWR_conversion then 
				section:addAbility( self:getName(sim, abilityOwner, abilityUser, targetUnitID),
					util.sformat(STRINGS.ABILITIES.HIJACK_CONSOLE_DESC_CONVERT_TO_CRED, abilityOwner:getName()), "gui/items/icon-action_hack-console.png" )
			else
				section:addAbility( self:getName(sim, abilityOwner, abilityUser, targetUnitID),
					STRINGS.ABILITIES.HIJACK_CONSOLE_DESC, "gui/items/icon-action_hack-console.png" )
			end
		end

		if sim:getNPC():findMainframeAbility("W93_AI_assembly") and sim:getNPC():hasMainframeAbility("W93_AI_assembly") then
			section:addAbility( STRINGS.PROGEXTEND.UI.AI_SCAN_A, STRINGS.FUNCLIB.UI.HIJACK_CONSOLE_AI_DESC, "gui/icons/skills_icons/skills_icon_small/icon-item_scan_small.png" )
			section:addLine( string.format(STRINGS.PROGEXTEND.AI.TOKEN, sim:getPC():getTraits().aiToken, sim:getPC():getTraits().aiTokenMax) )
		end

		if reason then
			section:addRequirement( reason )
		end
		return tooltip
	end,

	canUseAbility = function( self, sim, unit, userUnit, target )
		if unit:getTraits().LEVERinstallRequirement and userUnit then
			local isStatus = false

			if not userUnit:getTraits().LEVER_status then
				return false, STRINGS.FUNCLIB.UI.REASON_REQ
			end

			for _, req in pairs(unit:getTraits().LEVERinstallRequirement) do
				if req == userUnit:getTraits().LEVER_status then
					isStatus = true
				end
			end

			if isStatus == false then
				return false, STRINGS.FUNCLIB.UI.REASON_REQ
			end
		end

		local targetUnit = sim:getUnit( target )
		if targetUnit and unit:getTraits().PWR_conversion and targetUnit:getTraits().noPWRconvert then
			return false, STRINGS.FUNCLIB.UI.REASON_USED_ECONCHIP
		end

		return oldCanUseAbility( self, sim, unit, userUnit, target )
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		oldExecuteAbility( self, sim, unit, userUnit, target )

		simquery.LEVER_resetStatus( sim, userUnit, "LEGIT", true )
	end,
}
return jackin