local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )

local maa_adrenaline_amp =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ADRENALINE_AMP.NAME,

	getName = function( self, sim, unit )
		return self.name
	end,

	onSpawnAbility = function( self, sim, unit )
		self.seerUnits = {}
		self.periphUnits = {}
		unit:getTraits().ammo = 0
		sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_HIT, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_HIT, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_UNIT_NEWINTEREST or evType == simdefs.TRG_UNIT_NEWTARGET then
				if not userUnit:isKO() then
					--check that it's peripheral as well
					local ignore = false
					local target = nil
					local type = nil
					if evType == simdefs.TRG_UNIT_NEWINTEREST then
						ignore = evData.interest.sense ~= simdefs.SENSE_PERIPHERAL
						target = evData.interest.sourceUnit
						type = "peripheral"
					else
						target = evData.target
						type = "sight"
					end

					local seer = evData.unit
					if not ignore and userUnit == target then --and array.find( self.seerUnits, seer ) == nil then
						local charge = 0
						if type == "peripheral" and not array.find( self.seerUnits, seer ) and not array.find( self.periphUnits, seer ) then
							charge = unit:getTraits().adrenaline_gain_periph
						elseif type == "sight" and not array.find( self.seerUnits, seer ) and not array.find( self.periphUnits, seer ) then
							charge = unit:getTraits().adrenaline_gain_full
						elseif type == "sight" and not array.find( self.seerUnits, seer ) then
							charge = unit:getTraits().adrenaline_gain_full - unit:getTraits().adrenaline_gain_periph
						end
						if charge > 0 then
							if type == "peripheral" then
								table.insert( self.periphUnits, seer )
							else
								table.insert( self.seerUnits, seer )
							end
							
							unit:getTraits().ammo = math.min( unit:getTraits().ammo + charge, unit:getTraits().maxAmmo )
							local x0, y0 = userUnit:getLocation()
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, { unit = userUnit , txt=util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.ADRENALINE_UP, charge ), x=x0,y=y0,color={r=255/255,g=178/255,b=102/255,a=1}  } )
						end
					end
				end
			elseif evType == simdefs.TRG_START_TURN then
				--reset who's seen me
				self.seerUnits = {}
				self.periphUnits = {}
				--reset the buffs on PC start turn (so you can buff your overwatch)
				if evData:isPC() then
					userUnit:getTraits().maa_buff_melee_damage = nil
					userUnit:getTraits().maa_buff_melee_pierce = nil
				end
			elseif evType == simdefs.TRG_UNIT_HIT then
				--Check if Nika punched someone. Reset her buffs if so.
				if evData.melee and evData.sourceUnit == userUnit then
					userUnit:getTraits().maa_buff_melee_damage = nil
					userUnit:getTraits().maa_buff_melee_pierce = nil
				end
			end
		end
	end,
}

return maa_adrenaline_amp
