local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_chameleon_walk =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.CHAMELEON_WALK.NAME,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_invisicloak_small.png",
	alwaysShow = true,
	maa_activeAbility = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local level = 0
		local reactor_unit = nil
		if userUnit then
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
		end
		if reactor_unit then
			level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.CHAMELEON_WALK.NAME, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.CHAMELEON_WALK.DESC, 2*level ) )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not userUnit then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end

		local reactor_unit = nil
		for _, child in ipairs( userUnit:getChildren() ) do
			if child:getTraits().maa_power_allocated then
				reactor_unit = child
				break
			end
		end
		if not reactor_unit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_DOWN.NO_REACTOR
		end
		if (reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] or 0) <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.NO_POWER
		end
    if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
      return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
    end

		return true
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local reactor_unit = nil
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
			if reactor_unit then
        local level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]

        local already_invisible = userUnit:getTraits().invisible

        userUnit:setInvisible(true, 1)
        userUnit:resetAllAiming()

        if not already_invisible or userUnit:getTraits().cloakDistance then
          if not userUnit:getTraits().cloakDistance or userUnit:getTraits().cloakDistance < 2*level then
            userUnit:getTraits().cloakDistance = 2*level
          end
        end

        sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )

        sim:processReactions(userUnit)

				if unit:getTraits().cooldownMax then
					 unit:getTraits().cooldown = unit:getTraits().cooldownMax
				end
			end
		end
	end,
}

return maa_chameleon_walk
