local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local maa_daemon_cloak =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DAEMON_CLOAK_AUGMENT.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( "TRG_MAA_TRUE_DAEMON_INSTALL", self, unit )
		sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( "TRG_MAA_TRUE_DAEMON_INSTALL", self, unit )
		sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if evType == "TRG_MAA_TRUE_DAEMON_INSTALL" then
			--we listen first to the true daemon trigger to figure out if the following daemon install is a reversable daemon
			self.true_daemon = false
			if evData then
				self.true_daemon = true
			end
		elseif evType == simdefs.TRG_DAEMON_INSTALL then
			if unit:getTraits().installed then
				if self.true_daemon or not unit:getTraits().trueDaemon then
					local userUnit = unit:getUnitOwner()
					if unit:getTraits().cloakInVision or not sim:canPlayerSeeUnit( sim:getNPC(), userUnit ) then
						local already_invisible = userUnit:getTraits().invisible

						userUnit:setInvisible(true, unit:getTraits().duration)
						userUnit:resetAllAiming()

						if unit:getTraits().cloakDistanceMax then
							if not already_invisible or userUnit:getTraits().cloakDistance then
								if not userUnit:getTraits().cloakDistance or userUnit:getTraits().cloakDistance < unit:getTraits().cloakDistanceMax then
									userUnit:getTraits().cloakDistance = unit:getTraits().cloakDistanceMax
								end
							end
						end

						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )

						sim:processReactions(userUnit)
					end
				end
			end
		end
	end,
}

return maa_daemon_cloak
