local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )

local function deactivateDefibrillator( self, sim )
	if self.defi_unit then
		sim:warpUnit( self.defi_unit, nil )
		sim:despawnUnit( self.defi_unit )
		self.defi_unit = nil
	end
end

local maa_defibrillator =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DEFIBRILLATOR.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		--listen for agent revive, so we can destory the defi unit
		--listen for the agent getting KOed or killed to create the defi unit
		sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
		sim:addTrigger( "TRG_MAA_SOFT_REVIVE", self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self, unit )
		sim:removeTrigger( "TRG_MAA_SOFT_REVIVE", self, unit )
		deactivateDefibrillator( self, sim )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_UNIT_KO then
				if evData.unit == userUnit and evData.ticks ~= nil and not self.defi_unit then
					local template = unitdefs.lookupTemplate( unit:getTraits().maa_defiUnit )
					self.defi_unit = simfactory.createUnit( template, sim )
					sim:spawnUnit( self.defi_unit )
					local cell = sim:getCell( userUnit:getLocation() )
					sim:warpUnit( self.defi_unit, cell )

					local player = unit:getPlayerOwner()
					self.defi_unit:setPlayerOwner(player)

					self.defi_unit:activate( unit )
				elseif evData.unit == userUnit and evData.ticks == nil and self.defi_unit then
					deactivateDefibrillator( self, sim )
				end
			elseif evType == "TRG_MAA_SOFT_REVIVE" then
				if evData.unit == userUnit and self.defi_unit then
					deactivateDefibrillator( self, sim )
				end
			end
		end
	end,
}

return maa_defibrillator
