local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local itemdefs = include("sim/unitdefs/itemdefs")
local guarddefs = include("sim/unitdefs/guarddefs")
local propdefs = include("sim/unitdefs/propdefs")


local maa_deploy_drone =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DEPLOY_DRONE.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, unit, targetCell)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DEPLOY_DRONE.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DEPLOY_DRONE.DESC )
	end,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png",
	usesAction = true,

	acquireTargets = function( self, targets, game, sim, grenadeUnit, unit)
		if not self:canUseAbility( sim, grenadeUnit, unit ) then
			return nil
		end
		return targets.throwTarget( game, grenadeUnit:getTraits().range or 0, sim, unit, unit:getTraits().maxThrow, grenadeUnit:getTraits().targeting_ignoreLOS)
	end,


	canUseAbility = function( self, sim, grenadeUnit, unit, targetCell )
		if unit:getTraits().movingBody then
			return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
		end

		if grenadeUnit:getTraits().restrictedUse then

			local userUnitAgentID = unit:getUnitData().agentID
			local canUse =false
			for i,set in pairs(grenadeUnit:getTraits().restrictedUse )do
				if set.agentID == userUnitAgentID then
					canUse=true
				end
			end


			if not canUse then
				return false, STRINGS.UI.REASON.RESTRUCTED_USE
			end
		end
		
		--check if the user has the drone controller
		local controller = nil
		for _, childUnit in ipairs( unit:getChildren() ) do
			if childUnit:getTraits().maa_is_umi_drone_controller then
				controller = childUnit
			end
		end
		
		if not controller then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DEPLOY_DRONE.NO_CONTROLLER
		end
		
		--check if the drone has power left to be deployed
		local newUnit = nil
		for _, agent in ipairs( sim:getPC():getSummons() ) do
			if agent:getTraits().maa_isUmiDrone then
				newUnit = agent
				break
			end
		end
		if newUnit then
			local battery = nil
			for _, childUnit in ipairs( newUnit:getChildren() ) do
				if childUnit:getTraits().maa_is_drone_battery then
					battery = childUnit
					break
				end
			end
			
			if not battery or battery:getTraits().ammo <= 0 then
				return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DEPLOY_DRONE.NO_BATTERY
			end
		end

		if grenadeUnit:getTraits().cooldown and grenadeUnit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,grenadeUnit:getTraits().cooldown)
		end

		if grenadeUnit:getTraits().usesCharges and grenadeUnit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end

		if grenadeUnit:getTraits().pwrCost and unit:getPlayerOwner():getCpus() < grenadeUnit:getTraits().pwrCost then
			return false, util.sformat(STRINGS.UI.REASON.NOT_ENOUGH_PWR)
		end

		if targetCell then
			local targetX,targetY = unpack(targetCell)
			local unitX, unitY = unit:getLocation()
			local raycastX, raycastY = sim:getLOS():raycast(unitX, unitY, targetX, targetY)
			if raycastX ~= targetX or raycastY ~= targetY then
				return false
			end
		end

		return true
	end,

	executeAbility = function( self, sim, grenadeUnit, userUnit, targetCell )
		local sim = grenadeUnit:getSim()
		local x0, y0 = userUnit:getLocation()

		local x1, y1 = unpack(targetCell)
		local cell = sim:getCell( userUnit:getLocation() )
		userUnit:getTraits().throwing = true

		local facing = simquery.getDirectionFromDelta(x1-x0, y1-y0)
		simquery.suggestAgentFacing(userUnit, facing)
		if userUnit:getBrain() then --huh?
			if grenadeUnit:getTraits().baseDamage then
				sim:emitSpeech(userUnit, speechdefs.HUNT_GRENADE)
			end
			sim:refreshUnitLOS( userUnit )
			sim:processReactions( userUnit )
		end

		if userUnit:isValid() and not userUnit:getTraits().interrupted then
			sim:dispatchEvent( simdefs.EV_UNIT_THROW, { unit = userUnit, x1=x1, y1=y1, facing=facing } )


			local newUnit = nil
			--local unitTemplate = unitdefs.lookupTemplate( "maa_spider" )
			--newUnit = simfactory.createUnit( unitTemplate, sim )
			--sim:spawnUnit( newUnit )
			--newUnit:setPlayerOwner( userUnit:getPlayerOwner() )
			--find the drone
			for _, agent in ipairs( sim:getPC():getSummons() ) do --yes, this only works for PC, sue me...
				if agent:getTraits().maa_isUmiDrone then
					newUnit = agent
					break
				end
			end
			if not newUnit then
				for _, summonDef in ipairs( sim:getParams().agency.unitDefs_childs ) do
					if summonDef.id == "maa_umi_drone" then
						newUnit = sim:getPC():deploySummon( sim, summonDef )
						break
					end
				end
			end

			newUnit:setPlayerOwner( sim:getPC() )
			--sim:warpUnit( newUnit, cell, facing )
			sim:warpUnit( newUnit, sim:getCell(x1, y1) )
			if newUnit:isKO() then
				newUnit:setKO( sim, nil )
			end
			
			--newUnit:removeAbility(sim, "carryable")
			local battery = nil
			for _, childUnit in ipairs( newUnit:getChildren() ) do
				if childUnit:getTraits().maa_is_drone_battery then
					battery = childUnit
					break
				end
			end
			if battery then
				battery:getTraits().ammo = (battery:getTraits().ammo or 1) - 1
			end

			sim:emitSound( simdefs.SOUND_ITEM_PUTDOWN, cell.x, cell.y, userUnit)
			sim:emitSound( simdefs.SOUND_PRIME_EMP, cell.x, cell.y, userUnit)

			--newUnit:getTraits().primed = true
			--newUnit:detonate( sim )
			sim:dispatchEvent( simdefs.EV_HUD_REFRESH )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = newUnit } )

			if newUnit:getTraits().trigger_mainframe then
				newUnit:getTraits().mainframe_item = true
				newUnit:getTraits().mainframe_status = "on"
				newUnit:setPlayerOwner( userUnit:getPlayerOwner() )
			end

			userUnit:resetAllAiming()

			inventory.useItem( sim, userUnit, grenadeUnit )

			--remove this item from the users inventory and add a reference of it to the drone
			userUnit:removeChild( grenadeUnit )
			if grenadeUnit:getLocation( ) then
				sim:warpUnit( grenadeUnit, nil )
			end
			newUnit:getTraits().maa_droneCan = grenadeUnit

			--if grenadeUnit.throw then
			--	grenadeUnit:throw(userUnit, sim:getCell(x1, y1) )
			--end

			sim:dispatchEvent( simdefs.EV_UNIT_STOP_THROW, { unitID = userUnit:getID(), x1=x1, y1=y1, facing=facing } )


			sim:processReactions( userUnit )
		end
		userUnit:getTraits().throwing = nil
		if userUnit:isValid() and not userUnit:getTraits().interrupted then
			simquery.suggestAgentFacing(userUnit, facing)
		end

	end,
	
	onDespawnAbility = function( self, sim, unit )
		--if the portable drone leaves the level, the agentunit should too
		local newUnit = nil
		for _, agent in ipairs( sim:getPC():getSummons() ) do
			if agent:getTraits().maa_isUmiDrone then
				newUnit = agent
				break
			end
		end
		
		if newUnit then
			sim:despawnUnit( newUnit )
		end
	end,
}

return maa_deploy_drone
