local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )

local maa_distract_camera =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DISTRACT_CAMERA.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self,sim,unit,targetCell)
		return abilityutil.formatToolTip(STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DISTRACT_CAMERA.NAME,  STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DISTRACT_CAMERA.DESC )
	end,

	proxy = true,
	maa_two_state = true,

	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_Sticky_Cam_small.png",
	--usesAction = true,

	isTargetConsole = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		--this ability has two states: activating at a console and selecting the cell to distract to
		if not self.maa_state then
			--pick consoles as targets
			local cell = sim:getCell( abilityUser:getLocation() )
			local units = {}
	  		for dir, exit in pairs(cell.exits) do
	  			for _, cellUnit in ipairs( exit.cell.units ) do
	  				if self:isTargetConsole( sim, abilityUser, cellUnit ) then
	  					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
	  						table.insert( units, cellUnit )
	  					end
	  				end
	  			end
	  		end

	  		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
		else
			--pick cells seen by hacked cameras
			local cells = {}
			for _, simUnit in pairs( sim:getAllUnits() ) do
				if simUnit:getTraits().mainframe_camera and simUnit:getTraits().mainframe_status == "active" and simUnit:getPlayerOwner() == abilityUser:getPlayerOwner() then
					local paired_cells = sim:getLOS():calculateUnitLOS( sim:getCell(simUnit:getLocation()), simUnit )
					for id, addCell in pairs( paired_cells ) do
						local cellcoords = { x=addCell.x, y=addCell.y }

						table.insert( cells, cellcoords )
					end
				end
			end

			return targets.maa_areaCellTarget( game, 0, sim, cells )
		end
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		if not self.maa_state then
			if not userUnit then
				return false
			end
			if unit:getUnitOwner() ~= userUnit then
				return false
			end
			local player = sim:getPC()
			if userUnit:getTraits().movingBody then
				return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
			end
			if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
				return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
			end

			return true
		else
			local targetCell = targetUnitID --just convenience
			if targetCell then
				local targetX, targetY = unpack(targetCell)
				if targetX and targetY then
					local unitX, unitY = userUnit:getLocation()
		    		local raycastX, raycastY = sim:getLOS():raycast(unitX, unitY, targetX, targetY)
					if raycastX ~= targetX or raycastY ~= targetY then
						return false
					end
				end
			end

			return abilityutil.checkRequirements( unit, userUnit )
		end
	end,

	executeAbility = function( self, sim, unit, userUnit, targetCell ) --this hole stuff is very badly coded! it only works if no threads are running inside this function, so no dispatchEvents!
		if targetCell then
			local x0, y0 = unpack(targetCell)
			if x0 and y0 then
				sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_CAMERA )

				if unit:getTraits().cooldownMax then
					unit:getTraits().cooldown = unit:getTraits().cooldownMax
				end
			end
		end
  end,
}

return maa_distract_camera
