local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local inventory = include("sim/inventory")


local maa_drink_whiskey =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRINK_WHISKEY.NAME,

	profile_icon = "gui/icons/maa_bottle.png",
	alwaysShow = true,
	getName = function( self, sim, unit )
		return self.name
	end,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local desc = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRINK_WHISKEY.DESC
		if userUnit:getTraits().maa_isDrunk then
				desc = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRINK_WHISKEY.DESC_2
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRINK_WHISKEY.NAME, desc )
	end,

  onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end

		if unit:getTraits().charges and unit:getTraits().charges <= 0 then
			return false, STRINGS.UI.REASON.CHARGES
		end

		return abilityutil.checkRequirements( unit, userUnit )
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()

		if userUnit:getTraits().maa_isDrunk then
      userUnit:getTraits().maa_buff_melee_pierce_turn = (userUnit:getTraits().maa_buff_melee_pierce_turn or 0) + (unit:getTraits().maa_pierce_buff or 1)
			userUnit:getTraits().maa_buff_melee_damage_turn = (userUnit:getTraits().maa_buff_melee_damage_turn or 0) + (unit:getTraits().maa_damage_buff or 1)
			userUnit:addMP( unit:getTraits().maa_mp_buff or 1 )
    else
			userUnit:giveAbility( "maa_drunk" )
			local drunk_ability = userUnit:hasAbility( "maa_drunk" )
			drunk_ability.cooldown = (unit:getTraits().maa_drunk_duration or 10)
    end

		if unit:getTraits().charges then
			unit:getTraits().charges = unit:getTraits().charges - 1
		end
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local userUnit = unit:getUnitOwner()
		if evType == simdefs.TRG_START_TURN and evData:isPC() then
			userUnit:getTraits().maa_buff_melee_pierce_turn = nil
			userUnit:getTraits().maa_buff_melee_damage_turn = nil
		end
	end,
}
return maa_drink_whiskey
