local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local maa_drone_battery =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_BATTERY.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
		
		local userUnit = unit:getUnitOwner()
		if userUnit then
			unit:getTraits().ammo = userUnit:getTraits().maa_drone_battery or 0
			unit:getTraits().maxAmmo = userUnit:getTraits().maa_drone_battery or 0
		end
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if userUnit then
				if evType == simdefs.TRG_START_TURN then
					if evData:isPC() and userUnit:getLocation() and not userUnit:isDown( ) then
						if unit:getTraits().ammo and unit:getTraits().ammo > 0 then
							--the drone is active, remove one power from the battery
							unit:getTraits().ammo = unit:getTraits().ammo - 1
						else
							--the drone is active, but the battery is empty, shut down
							userUnit:setKO( sim, 2 )
						end
					elseif userUnit:isKO() and ( userUnit:getTraits().maa_drone_shutdown or (unit:getTraits().ammo or 0) <= 0) then
						--keep the drone shut down
						userUnit:setKO( sim, 2 )
					end
				elseif evType == simdefs.TRG_UNIT_KO then
					if evData.unit == userUnit and userUnit:getLocation() and not evData.ticks then
						--drone is deployed and wakes up, consume power
						--unit:getTraits().ammo = unit:getTraits().ammo - 1
					end
				end
			end
		end
	end,
}

return maa_drone_battery
