local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_drone_door_hack = 
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HACK_DOOR.NAME,
	
	createToolTip = function( self,sim,unit,targetCell)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HACK_DOOR.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HACK_DOOR.DESC )
	end,

	profile_icon = "gui/icons/item_icons/icon-item_passcard.png",

	proxy = true,

	getName = function( self, sim, unit )
		return string.format(STRINGS.ABILITIES.SET_NAME,unit:getName())
	end,
	
	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local exits = {}
		local fromCell = sim:getCell( userUnit:getLocation() )
		local x0, y0 = fromCell.x, fromCell.y

		for dir, exit in pairs( fromCell.exits ) do
			if exit ~= nil and exit.door and exit.closed and exit.locked and exit.keybits == simdefs.DOOR_KEYS.SECURITY then
				table.insert( exits, { x = fromCell.x, y = fromCell.y, dir = dir } )
			end
		end

		return targets.exitTarget( game, exits, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit, x, y, dir )
		if not userUnit then
			return false
		end
		
		if not userUnit:getLocation() then
			return false
		end
		
		local player = userUnit:getPlayerOwner()
		if not player then
			return false
		end

		if unit:getTraits().maa_drone_door_hack_cooldown and unit:getTraits().maa_drone_door_hack_cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().maa_drone_door_hack_cooldown)
		end

		if unit:getTraits().maa_drone_door_hack_cost and player:getCpus() < unit:getTraits().maa_drone_door_hack_cost then 
			return false,  STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, unit, userUnit, x0, y0, dir )
		local player = userUnit:getPlayerOwner()
		local x1, y1 = userUnit:getLocation()
		
		local fromCell = sim:getCell( x0, y0 )
		local exit = fromCell.exits[dir]
		exit.locekd = false
		if unit:getTraits().maa_drone_door_hack_cooldownMax then
			unit:getTraits().maa_drone_door_hack_cooldown = unit:getTraits().maa_drone_door_hack_cooldownMax
		end
		if unit:getTraits().maa_drone_door_hack_cost then
			player:addCPUs( -unit:getTraits().maa_drone_door_hack_cost, sim, x1, y1 )
		end
		
		sim:modifyExit( fromCell, dir, simdefs.EXITOP_UNLOCK, userUnit, userUnit:getTraits().sneaking )
	end,
	
	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == simdefs.TRG_END_TURN then
				if evData:isNPC() then
					if unit:getTraits().maa_drone_door_hack_cooldown and unit:getTraits().maa_drone_door_hack_cooldown > 0 then
						unit:getTraits().maa_drone_door_hack_cooldown = unit:getTraits().maa_drone_door_hack_cooldown - 1
					end
				end
			end
		end
	end,
}
return maa_drone_door_hack
