local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )
local simquery = include("sim/simquery")
local mission_util = include( "sim/missions/mission_util" )

local maa_drone_id_daemon =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ID_DAEMON.NAME,
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_analyze_small.png",

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ID_DAEMON.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ID_DAEMON.DESC )
	end,

	proxy = true,

	getName = function( self, sim, unit )
		return self.name
	end,

	isTargetConsole = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,
	
	isTarget = function( self, sim, userUnit, targetUnit)
		if (targetUnit:getTraits().mainframe_ice or 0) > 0 and ( targetUnit:getTraits().mainframe_program or sim:getHideDaemons()) and not targetUnit:getTraits().daemon_sniffed then
			return true
		end
	end,

	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local localPlayer = unit:getPlayerOwner()
		local units = {}
		for _, checkUnit in pairs( sim:getAllUnits() ) do
			local x1, y1 = checkUnit:getLocation()
			
			if x1 and y1 and localPlayer:getCell( x1, y1 ) then
				if self:isTarget( sim, userUnit, checkUnit ) then
					table.insert( units, checkUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit )
		local cell = sim:getCell( userUnit:getLocation() )
		local console_adjacent = false
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTargetConsole( sim, userUnit, cellUnit ) then
					if simquery.canUnitReach( sim, userUnit, exit.cell.x, exit.cell.y ) then
						console_adjacent = true
						break
					end
				end
			end
		end
		
		if not console_adjacent then
			return false
		end
		
		if unit:getTraits().maa_drone_id_daemon_cooldown and unit:getTraits().maa_drone_id_daemon_cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().maa_drone_id_daemon_cooldown )
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local target = sim:getUnit(target)
		if not target:getTraits().mainframe_program then
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HUD_INCIDENT_NEGATIVE.path )
			mission_util.showDialog( sim, STRINGS.UI.DIALOGS.NO_DAEMON_TITLE, STRINGS.UI.DIALOGS.NO_DAEMON_BODY )
			target:getTraits().daemon_sniffed = true	
			sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = target, ice = target:getTraits().mainframe_ice, delta = 0} )	
		else
			target:getTraits().daemon_sniffed = true	
			sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = target, ice = target:getTraits().mainframe_ice, delta = 0} )		
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_DAEMON_REVEAL.path )
		end
		
		if unit:getTraits().maa_drone_id_daemon_cooldownMax then
			unit:getTraits().maa_drone_id_daemon_cooldown = unit:getTraits().maa_drone_id_daemon_cooldownMax
		end
	end,
	
	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,
	
	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == simdefs.TRG_END_TURN then
				if evData:isNPC() then
					if unit:getTraits().maa_drone_id_daemon_cooldown and unit:getTraits().maa_drone_id_daemon_cooldown > 0 then
						unit:getTraits().maa_drone_id_daemon_cooldown = unit:getTraits().maa_drone_id_daemon_cooldown - 1
					end
				end
			end
		end
	end,
}

return maa_drone_id_daemon
