local util = include( "modules/util" )
local array = include("modules/array")
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local abilitydefs = include( "sim/abilitydefs" )
local mathutil = include( "modules/mathutil" )
local shootSingle = abilitydefs._abilities["shootSingle"]

local function isShootTarget( self, sim, abilityOwner, abilityUser, targetUnit )
	if not simquery.isShootable( abilityUser, targetUnit ) then
		return false
	end
	if not sim:canPlayerSeeUnit( abilityUser:getPlayerOwner(), targetUnit ) then
		return false
	end

	return true
end

local maa_drone_mark_target =
{
	name = shootSingle.name,
	profile_icon = shootSingle.profile_icon,
	usesAction = shootSingle.usesAction,
	proxy = shootSingle.proxy,
	getName = shootSingle.getName,
	onTooltip = shootSingle.onTooltip,
	
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local units = {}
		local x0, y0 = userUnit:getLocation()
		for _, targetUnit in pairs(sim:getAllUnits()) do
			if isShootTarget( self, sim, unit, userUnit, targetUnit) then 

				local x1,y1 = targetUnit:getLocation()
				local distance =  mathutil.dist2d( x0, y0, x1, y1 )

    				if sim:canUnitSeeUnit( userUnit, targetUnit ) then
					table.insert( units, targetUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,
	
	canUseAbility = function( self, sim, ownerUnit, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit and not sim:canUnitSeeUnit( unit, targetUnit ) then
			return false
		end

		if targetUnit and targetUnit:getArmor() <= 0 then
			return false
		end
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if ownerUnit:getTraits().maa_drone_mark_target_cost and player:getCpus() < ownerUnit:getTraits().maa_drone_mark_target_cost then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, ownerUnit, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local player = unit:getPlayerOwner()
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()

		local oldFacing = unit:getFacing()
		local newFacing = simquery.getDirectionFromDelta(x1-x0,y1-y0)

		local pinning, pinnee = simquery.isUnitPinning(unit:getSim(), unit)
		if pinning and pinnee == targetUnit then
			newFacing = pinnee:getFacing()
		else
			pinning = false
		end

		if targetUnit:getPlayerOwner() then
			targetUnit:getPlayerOwner():glimpseUnit( sim, unit:getID() )
		end

		simquery.suggestAgentFacing(unit, newFacing)
		sim:dispatchEvent( simdefs.EV_UNIT_START_SHOOTING, { unitID=unit:getID(), newFacing=newFacing, oldFacing=oldFacing, targetUnitID=targetUnit:getID(), pinning=pinning } )

		if unit:getTraits().monster_hacking then 
			unit:getTraits().monster_hacking = false
			unit:getSounds().spot = nil
		end
						
		if unit:getTraits().data_hacking then 
			unit:stopHacking(sim)
		end
						

		if unit:isValid() then
			sim:startTrackerQueue(true)				
			targetUnit:getTraits().maa_armor_mark = (targetUnit:getTraits().maa_armor_mark or 0) + 1
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
			
			if ownerUnit:getTraits().maa_drone_mark_target_cost then
				player:addCPUs( -ownerUnit:getTraits().maa_drone_mark_target_cost, sim, x0, y0 )
			end

			sim:dispatchEvent( simdefs.EV_UNIT_STOP_SHOOTING, { unitID = unit:getID(), facing=newFacing, pinning=pinning} )			
			sim:startTrackerQueue(false)
			sim:processReactions( unit )	
		end
	end,
}

return maa_drone_mark_target
