local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local speechdefs = include("sim/speechdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_drone_reactivate =
{

	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.NAME,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
	alwaysShow = true,
	maa_activeAbility = true,

	getName = function( self, sim, unit )
		return self.name
	end,

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.DESC )
	end,


	canUseAbility = function( self, sim, unit, userUnit)
		local userUnit = unit:getUnitOwner()
		if not unit:getTraits().installed then
			return false
		end
		local droneUnit = nil
		for _, agent in ipairs( sim:getPC():getSummons() ) do
			if agent:getTraits().maa_isUmiDrone then
				droneUnit = agent
				break
			end
		end
		if not droneUnit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.NO_DRONE
		end
		if not droneUnit:getLocation() then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.NO_DRONE
		end
		if not droneUnit:isDown( ) then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.DRONE_ACTIVE
		end

		local battery = nil
		for _, childUnit in ipairs( droneUnit:getChildren() ) do
			if childUnit:getTraits().maa_is_drone_battery then
				battery = childUnit
				break
			end
		end
		if battery and battery:getTraits().ammo <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRONE_REACTIVATE.DRONE_NO_POWER
		end

		local player = userUnit:getPlayerOwner()
		if unit:getTraits().maa_drone_reactivation_cost and player:getCpus() < unit:getTraits().maa_drone_reactivation_cost then
			return false,  STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit)
		local droneUnit = nil
		for _, agent in ipairs( sim:getPC():getSummons() ) do
			if agent:getTraits().maa_isUmiDrone then
				droneUnit = agent
				break
			end
		end
		if droneUnit and not simquery.isUnitPinned( sim, droneUnit ) then
			droneUnit:setKO( sim, nil )
			droneUnit:getTraits().maa_drone_shutdown = false

			local battery = nil
			for _, childUnit in ipairs( droneUnit:getChildren() ) do
				if childUnit:getTraits().maa_is_drone_battery then
					battery = childUnit
					break
				end
			end
			if battery then
				battery:getTraits().ammo = (battery:getTraits().ammo or 1) - 1
			end

			local player = userUnit:getPlayerOwner()
			local x1, y1 = userUnit:getLocation()
			if unit:getTraits().maa_drone_reactivation_cost then
				player:addCPUs( -unit:getTraits().maa_drone_reactivation_cost, sim, x1, y1 )
			end
		end
	end,
}

return maa_drone_reactivate
