local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )
local simquery = include("sim/simquery")
local mission_util = include( "sim/missions/mission_util" )

local maa_drone_reboot_device =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.REBOOT_DEVICE.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_equip_small.png",

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.REBOOT_DEVICE.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.REBOOT_DEVICE.DESC )
	end,

	proxy = true,

	getName = function( self, sim, unit )
		return self.name
	end,

	isTargetConsole = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,
	
	isTarget = function( self, sim, userUnit, targetUnit)
		if targetUnit:getTraits().mainframe_item and targetUnit:getTraits().mainframe_status and targetUnit:getTraits().mainframe_status == "active" and userUnit:getPlayerOwner() ~= targetUnit:getPlayerOwner() and not targetUnit:getTraits().noTakeover then
			return true
		end
	end,

	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local localPlayer = unit:getPlayerOwner()
		local units = {}
		for _, checkUnit in pairs( sim:getAllUnits() ) do
			local x1, y1 = checkUnit:getLocation()
			
			if x1 and y1 and localPlayer:getCell( x1, y1 ) then
				if self:isTarget( sim, userUnit, checkUnit ) then
					table.insert( units, checkUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit )
		local cell = sim:getCell( userUnit:getLocation() )
		local console_adjacent = false
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTargetConsole( sim, userUnit, cellUnit ) then
					if simquery.canUnitReach( sim, userUnit, exit.cell.x, exit.cell.y ) then
						console_adjacent = true
						break
					end
				end
			end
		end
		
		if not console_adjacent then
			return false
		end
		
		if unit:getTraits().maa_drone_reboot_device_cooldown and unit:getTraits().maa_drone_reboot_device_cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().maa_drone_reboot_device_cooldown )
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local target = sim:getUnit(target)
		target:processEMP( 1, true, nil, true )
		sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = target, ice = target:getTraits().mainframe_ice, delta = 0} )
		
		if unit:getTraits().maa_drone_reboot_device_cooldownMax then
			unit:getTraits().maa_drone_reboot_device_cooldown = unit:getTraits().maa_drone_reboot_device_cooldownMax
		end
	end,
	
	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,
	
	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == simdefs.TRG_END_TURN then
				if evData:isNPC() then
					if unit:getTraits().maa_drone_reboot_device_cooldown and unit:getTraits().maa_drone_reboot_device_cooldown > 0 then
						unit:getTraits().maa_drone_reboot_device_cooldown = unit:getTraits().maa_drone_reboot_device_cooldown - 1
					end
				end
			end
		end
	end,
}

return maa_drone_reboot_device
