local array = include( "modules/array" )
local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )

local maa_drone_scan =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.THROW_SCAN.NAME,

	createToolTip = function( self, sim, unit )
		local desc = util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.THROW_SCAN.DESC, unit:getTraits().mma_drone_scan_range or 1 )
		if unit:getTraits().maa_drone_scan_cooldownMax then
			desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.THROW_SCAN.DESC_COOLDOWN, unit:getTraits().maa_drone_scan_cooldownMax )
			
			if unit:getTraits().maa_drone_scan_cooldown and unit:getTraits().maa_drone_scan_cooldown > 0 then
				desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.THROW_SCAN.DESC_COOLDOWN_2, unit:getTraits().maa_drone_scan_cooldown )
			end
		end
		if unit:getTraits().maa_drone_scan_cost then
			desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.THROW_SCAN.DESC_PWR_COST, unit:getTraits().maa_drone_scan_cost )
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.THROW_SCAN.NAME, desc )
	end,
	
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_scanner_small.png",

	alwaysShow = true,
	maa_activeAbility = true,
	
	getName = function( self, sim, unit )
		return self.name
	end,
	
	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,
	
	canUseAbility = function( self, sim, unit, userUnit )
	
		if not userUnit:getLocation() then
			return false
		end
		
		local player = userUnit:getPlayerOwner()
		if not player then
			return false
		end
		
		if unit:getTraits().maa_drone_scan_cost and player:getCpus() < unit:getTraits().maa_drone_scan_cost then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
		
		if unit:getTraits().maa_drone_scan_cooldown and unit:getTraits().maa_drone_scan_cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().maa_drone_scan_cooldown )
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, unit, userUnit )
		local player = userUnit:getPlayerOwner()
		local x1, y1 = userUnit:getLocation()
		if x1 and y1 and player then
			local cells = simquery.rasterCircle( sim, x1, y1, unit:getTraits().mma_drone_scan_range or 1 )
			for i, x, y in util.xypairs( cells ) do
				local cell = sim:getCell( x, y )
				if cell then
					player:glimpseCell( sim, cell )
					for _, cellUnit in ipairs(cell.units) do
						player:glimpseUnit( sim, cellUnit:getID() )
					end
				end
			end
			
			--refresh los
			sim:dispatchEvent( simdefs.EV_LOS_REFRESH, { player = player, cells = cells } )
			--scan animation
			sim:dispatchEvent( "MAA_DRONE_SCAN", {unit=userUnit, cells=cells, range=(unit:getTraits().mma_drone_scan_range or 1)} )
			
			--pay up
			if unit:getTraits().maa_drone_scan_cost then
				player:addCPUs( -unit:getTraits().maa_drone_scan_cost, sim, x1, y1 )
			end
			if unit:getTraits().maa_drone_scan_cooldownMax then
				unit:getTraits().maa_drone_scan_cooldown = unit:getTraits().maa_drone_scan_cooldownMax
			end
		end
	end,
	
	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == simdefs.TRG_END_TURN then
				if evData:isNPC() then
					if unit:getTraits().maa_drone_scan_cooldown and unit:getTraits().maa_drone_scan_cooldown > 0 then
						unit:getTraits().maa_drone_scan_cooldown = unit:getTraits().maa_drone_scan_cooldown - 1
					end
				end
			end
		end
	end,
}

return maa_drone_scan
