local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_drunk =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRUNK.NAME,

	alwaysShow = true,
	HUDpriority = -1,

	profile_icon = "gui/icons/maa_bottle.png",

	getName = function( self, sim, unit )
		return self.name
	end,

	-- onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
	-- 	return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRUNK.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRUNK.DESC )
	-- end,

	createToolTip = function( self, sim, unit )
		local desc = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRUNK.DESC
		if self.cooldown and self.cooldown > 0 then
			desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRUNK.DESC_2, self.cooldown )
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.DRUNK.NAME, desc )
	end,

	canUseAbility = function( self, sim, unit )
		return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PASSIVE
	end,

	onSpawnAbility = function( self, sim, unit )
		unit:getTraits().maa_isDrunk = true
		for _, childUnit in pairs( unit:getChildren() ) do
			if childUnit:getTraits().slot == "gun" and childUnit:getTraits().equipped then
				inventory.unequipItem( unit, childUnit )
				break
			end
		end

		if unit:hasAbility( "sprint" ) then
			self.sprint_removed = true
			unit:removeAbility( sim, "sprint" )
		end

		unit:getTraits().meleeDamage = (unit:getTraits().meleeDamage or 0) + 1

		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		unit:getTraits().maa_isDrunk = false

		if self.sprint_removed then
			unit:giveAbility( "sprint" )
		end

		unit:getTraits().meleeDamage = unit:getTraits().meleeDamage - 1

		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	executeAbility = function( self, sim, unit )

	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if evType == simdefs.TRG_START_TURN and evData:isPC() then
			if self.cooldown then
				self.cooldown = self.cooldown - 1
				if self.cooldown <= 0 then
					unit:removeAbility( sim, "maa_drunk" )
				end
			end
		end
	end,
}

return maa_drunk
