local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )
local simquery = include("sim/simquery")
local mission_util = include( "sim/missions/mission_util" )
local mathutil = include( "modules/mathutil" )

local maa_extend_drone_control =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.EXTEND_DRONE_DURATION.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_augment.png",

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.EXTEND_DRONE_DURATION.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.EXTEND_DRONE_DURATION.DESC )
	end,

	proxy = true,

	getName = function( self, sim, unit )
		return self.name
	end,
	
	isTarget = function( self, sim, userUnit, targetUnit)
		if targetUnit:getTraits().takenDrone and targetUnit:getTraits().controlTimer and targetUnit:getTraits().controlTimer > 0 then
			return true
		end
	end,

	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local localPlayer = unit:getPlayerOwner()
		local units = {}
		for _, checkUnit in pairs( sim:getAllUnits() ) do
			local x1, y1 = checkUnit:getLocation()
			local x0, y0 = userUnit:getLocation()
			
			if x1 and y1 and x0 and y0 and localPlayer:getCell( x1, y1 ) then
				local distance = mathutil.dist2d( x0, y0, x1, y1 )
				if distance <= (unit:getTraits().range or 0) then
					if self:isTarget( sim, userUnit, checkUnit ) then
						table.insert( units, checkUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit )	
		local player = userUnit:getPlayerOwner()
		if not player then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
		end
		if unit:getTraits().maa_extend_drone_duration_cost and player:getCpus() < unit:getTraits().maa_extend_drone_duration_cost then 
			return false,  STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local target = sim:getUnit(target)
		target:getTraits().controlTimer = target:getTraits().controlTimer + 1
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = target } )
		
		local player = userUnit:getPlayerOwner()
		local x1, y1 = userUnit:getLocation()
		if unit:getTraits().cooldownMax then
			unit:getTraits().cooldown = unit:getTraits().cooldownMax
		end
		if unit:getTraits().maa_extend_drone_duration_cost then
			player:addCPUs( -unit:getTraits().maa_extend_drone_duration_cost, sim, x1, y1 )
		end
	end,
	
	onSpawnAbility = function( self, sim, unit )
		--sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		--sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,
	
	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == simdefs.TRG_END_TURN then
				if evData:isNPC() then
					if unit:getTraits().maa_extend_drone_duration_cooldown and unit:getTraits().maa_extend_drone_duration_cooldown > 0 then
						unit:getTraits().maa_extend_drone_duration_cooldown = unit:getTraits().maa_extend_drone_duration_cooldown - 1
					end
				end
			end
		end
	end,
}

return maa_extend_drone_control
