local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local maa_feedback_processor =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.FEEDBACK_PROCESSOR.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_ALARM_INCREASE, self, unit )
		local start_trigger = sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		start_trigger.priority = -1
		sim:addTrigger( "MAA_TRG_PRE_END_TURN", self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_ALARM_INCREASE, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( "MAA_TRG_PRE_END_TURN", self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_ALARM_INCREASE then
				if not self.suspend_buff then
					userUnit:addMP( unit:getTraits().maa_alarm_mp_bonus or 1 )
					local x1, y1 = userUnit:getLocation()
					sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.FEEDBACK_PROCESSING, x=x1, y=y1, color={r=1,g=1,b=1,a=1}} )
				end
			elseif evType == simdefs.TRG_START_TURN then
				if evData:isPC() then
					self.suspend_buff = nil
				end
			elseif evType == "MAA_TRG_PRE_END_TURN" then
				if evData:isNPC() then
					self.suspend_buff = true
				end
			end
		end
	end,
}

return maa_feedback_processor
