local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_headshot =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.NAME,
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_accuracy_small.png",
	alwaysShow = true,
	maa_activeAbility = true,
	
	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.DESC, userUnit:getMPMax() )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not unit:getTraits().installed then
			return false
		end
		if userUnit:getMP() < userUnit:getMPMax() then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.NO_MP
		end

		return true
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local x0, y0 = userUnit:getLocation()
		userUnit:getTraits().maa_preparing_headshot = true
		userUnit:getTraits().maa_preparing_headshot_active = true
		userUnit:useMP( userUnit:getTraits().mp, sim )
		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_PREPARING, x=x0, y=y0, color={r=1,g=1,b=41/255,a=1}} )
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_ACTION, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_ACTION, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				if not userUnit:getTraits().maa_preparing_headshot then
					--if the unit has broken his preparation last turn, we reset it this turn
					userUnit:getTraits().maa_preparing_headshot = true
				else
					--if the unit did not break the preparation we make it ready to use headshot
					userUnit:getTraits().maa_headshot_ready = true
					local x0, y0 = userUnit:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_READY, x=x0, y=y0, color={r=1,g=1,b=41/255,a=1}} )
				end
			elseif evType == simdefs.TRG_UNIT_WARP then
				if evData.unit == userUnit then
					local x0, y0 = userUnit:getLocation()
					if x0 and y0 then
						if userUnit:getTraits().maa_headshot_ready then
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_STOP, x=x0, y=y0, color={r=163/255,g=0,b=0,a=1}} )
						elseif userUnit:getTraits().maa_preparing_headshot_active then
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_PREP_STOP, x=x0, y=y0, color={r=163/255,g=0,b=0,a=1}} )
						end
					end
					--the unit has moved and therefor broken the preparation or the preped headshot
					userUnit:getTraits().maa_preparing_headshot = false
					userUnit:getTraits().maa_headshot_ready = false
					userUnit:getTraits().maa_preparing_headshot_active = false
				end
			elseif evType == simdefs.TRG_ACTION then
				if evData.ClassType == "abilityAction" and not evData.pre then
					--any ability thats actively used
					local actionUser = sim:getUnit( evData[2] )
					local actionOwner = sim:getUnit( evData[1] )
					if userUnit == actionUser and unit ~= actionOwner then
						local x0, y0 = userUnit:getLocation()
						if userUnit:getTraits().maa_headshot_ready then
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_STOP, x=x0, y=y0, color={r=163/255,g=0,b=0,a=1}} )
						elseif userUnit:getTraits().maa_preparing_headshot_active then
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_PREP_STOP, x=x0, y=y0, color={r=163/255,g=0,b=0,a=1}} )
						end
						--the unit has used an ability, so we break the prep
						userUnit:getTraits().maa_preparing_headshot = false
						userUnit:getTraits().maa_headshot_ready = false
						userUnit:getTraits().maa_preparing_headshot_active = false
					end
				end
			end
		end
	end,
}

return maa_headshot
