local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")

local function isTargetTrackable( self, sim, abilityOwner, abilityUser, targetUnit )
	if not simquery.isShootable( abilityUser, targetUnit ) then
		return false
	end
	if not sim:canUnitSeeUnit( abilityUser, targetUnit ) then
		return false
	end

	return true
end

local maa_headshot_new =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.NAME,
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_augment_shalem_small.png",
	proxy = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.DESC )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local units = {}
		local x0, y0 = userUnit:getLocation()
		for _, targetUnit in pairs(sim:getAllUnits()) do
			if isTargetTrackable( self, sim, unit, userUnit, targetUnit ) then
				table.insert( units, targetUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		if not userUnit then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end
		if self.track_target and self.track_target == targetUnit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.HEADSHOT.ALREADY_TRACKED
		end
		if targetUnit and not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit then
			if unit:getTraits().cooldownMax then
				unit:getTraits().cooldown = unit:getTraits().cooldownMax
			end
			self.track_target = targetUnit
			userUnit:getTraits().maa_tracked_target = nil
			if self.eyeballUnit then
				sim:warpUnit( self.eyeballUnit )
				sim:despawnUnit( self.eyeballUnit )
				self.eyeballUnit = nil
			end
		end
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
		if self.eyeballUnit then
			sim:warpUnit( self.eyeballUnit )
			sim:despawnUnit( self.eyeballUnit )
			self.eyeballUnit = nil
		end
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			local player = unit:getPlayerOwner()
			if evType == simdefs.TRG_UNIT_WARP then
				if evData.unit == userUnit then
					local x0, y0 = userUnit:getLocation()
					if x0 and y0 then
						if userUnit:getTraits().maa_tracked_target then
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.HEADSHOT_STOP, x=x0, y=y0, color={r=163/255,g=0,b=0,a=1}} )
						end
					end
					--the unit has moved and therefor broken the tracking
					userUnit:getTraits().maa_tracked_target = nil
					self.track_target = nil
					if self.eyeballUnit then
						sim:warpUnit( self.eyeballUnit )
						sim:despawnUnit( self.eyeballUnit )
						self.eyeballUnit = nil
					end
				elseif userUnit and self.track_target and evData.unit == self.track_target then
					player:glimpseUnit( sim, self.track_target:getID() )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.track_target } )
				end
			elseif evType == simdefs.TRG_START_TURN and evData:isPC() then
				if self.track_target and userUnit then
					userUnit:getTraits().maa_tracked_target = self.track_target
					local target_cell = sim:getCell( self.track_target:getLocation() )

					local eyeball = include( "sim/units/eyeball" )
					self.eyeballUnit = eyeball.createEyeball( sim )
					self.eyeballUnit:setPlayerOwner( userUnit:getPlayerOwner() )
					self.eyeballUnit:getTraits().LOSarc = math.pi * 2
					self.eyeballUnit:getTraits().LOSrange = 0.4
					sim:spawnUnit( self.eyeballUnit )
					sim:warpUnit( self.eyeballUnit, target_cell)
				end
			elseif evType == simdefs.TRG_END_TURN and evData:isPC() then
				if self.eyeballUnit then
					sim:warpUnit( self.eyeballUnit )
					sim:despawnUnit( self.eyeballUnit )
					self.eyeballUnit = nil
				end
			end
		end
	end,
}

return maa_headshot_new
