local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")

local maa_ice_breaker =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ICE_BREAKER.NAME,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_generic_head_small2.png",
	alwaysShow = true,
	proxy = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local level = 0
		local reactor_unit = nil
		if userUnit then
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
		end
		if reactor_unit then
			level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ICE_BREAKER.NAME, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ICE_BREAKER.DESC, level ) )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

  acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local cell = sim:getCell( userUnit:getLocation() )
    local cells = { cell }
		for dir, exit in pairs(cell.exits) do
        if simquery.isOpenExit( exit ) then
            table.insert( cells, exit.cell )
        end
    end

		local units = {}
		for i, cell in pairs(cells) do
			for _, cellUnit in ipairs( cell.units ) do
				if (cellUnit:getTraits().mainframe_ice or 0) > 0 then
					if cellUnit:getTraits().mainframe_status ~= "off" or (cellUnit:getTraits().mainframe_camera and cellUnit:getTraits().mainframe_booting) then
						if not cellUnit:getTraits().isDrone or not cellUnit:isKO() then
							table.insert( units, cellUnit )
						end
					end

				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not userUnit then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end

		local reactor_unit = nil
		for _, child in ipairs( userUnit:getChildren() ) do
			if child:getTraits().maa_power_allocated then
				reactor_unit = child
				break
			end
		end
		if not reactor_unit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_DOWN.NO_REACTOR
		end
		if (reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] or 0) <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.NO_POWER
		end
    if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
      return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
    end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
    local mainframe = include( "sim/mainframe" )
		local userUnit = unit:getUnitOwner()
    if userUnit then
			local reactor_unit = nil
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
			if reactor_unit then
        local level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]

        local target = sim:getUnit(target)
    		local x0,y0 = userUnit:getLocation()
    		local x1,y1 = target:getLocation()
      	local newFacing = simquery.getDirectionFromDelta(x1-x0,y1-y0)

    		sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = userUnit:getID(), facing = newFacing, sound=simdefs.SOUNDPATH_USE_CONSOLE, soundFrame=10 } )

    		local iceCount = level
    		mainframe.breakIce( sim, target, math.min( target:getTraits().mainframe_ice, iceCount ) )

    		if userUnit:isValid() then
    			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
    		end

				if unit:getTraits().cooldownMax then
					 unit:getTraits().cooldown = unit:getTraits().cooldownMax
				end
			end
		end
	end,
}

return maa_ice_breaker
