local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local maa_idle_cycle_processing =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.IDLE_CYCLE_PROCESSOR.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		local trigger = sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		trigger.priority = -1
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			if evType == simdefs.TRG_START_TURN then
				if evData:isPC() then
					local player = unit:getPlayerOwner()
					local userUnit = unit:getUnitOwner()
					local pwr = player:getCpus()
					if sim:getTurnCount() == 0 then
						pwr = sim:getParams().difficultyOptions.startingPower + (player:getTraits().extraStartingPWR or 0)
					end
					local mp_gain = math.floor( pwr / (unit:getTraits().maa_pwr_threshold or 6) ) * (unit:getTraits().maa_mp_per_pwr or 1)
					mp_gain = mp_gain - (unit:getTraits().maa_mp_penalty or 0)
					userUnit:addMP( mp_gain )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
				end
			end
		end
	end,
}

return maa_idle_cycle_processing
