local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")
local inventory = include("sim/inventory")

local maa_inject_lethal =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INJECT_LETHAL.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INJECT_LETHAL.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INJECT_LETHAL.DESC )
	end,


	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_mindwipesyringe_small.png",
	proxy = true,
	alwaysShow = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local cell = sim:getCell( userUnit:getLocation() )

		local units = {}


		for i, cellUnit in ipairs(cell.units) do
			if (cellUnit:isKO() and not cellUnit:isDead()) and cellUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() and not cellUnit:getTraits().isDrone then
				table.insert( units, cellUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end

		if unit:getTraits().ammo and unit:getTraits().ammo <= 0 then
			return false, STRINGS.UI.REASON.NO_AMMO
		end
				
		local ok, reason = abilityutil.checkRequirements( unit, userUnit )
		if not ok then
			return false, reason
		end
		
		local cell = sim:getCell( userUnit:getLocation() )
		local units = {}
		for i, cellUnit in ipairs(cell.units) do
			if (cellUnit:isKO() and not cellUnit:isDead()) and cellUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() and cellUnit:getTraits().canKO then
				table.insert( units, cellUnit )
			end
		end					

		if #units < 1 then
			return false, STRINGS.UI.REASON.NO_VIABLE_TARGET
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		target = sim:getUnit(target)
		target:getTraits().regenerative_nanocells = nil

		target:killUnit( sim )
		
		inventory.useItem( sim, userUnit, unit )
	end,
}

return maa_inject_lethal
