local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")
local inventory = include("sim/inventory")

local maa_install_tracking_chip =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_TRACKING_CHIP.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_TRACKING_CHIP.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_TRACKING_CHIP.DESC )
	end,


	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chip_daemonscan_small.png",
	proxy = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()

		local cell = sim:getCell( userUnit:getLocation() )
		local units = {}

		for i, cellUnit in ipairs(cell.units) do
			if cellUnit:getTraits().isGuard and not cellUnit:getTraits().isDrone and cellUnit:isKO() then
				table.insert( units, cellUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		if not userUnit then
			return false
		end
		if unit:getUnitOwner() ~= userUnit then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
		end
		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit and targetUnit:getTraits().tagged then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_TRACKING_CHIP.ALREADY_TAGGED
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		target = sim:getUnit(target)
		if target then
			target:setTagged( false )
		end

		inventory.useItem( sim, userUnit, unit )
	end,
}

return maa_install_tracking_chip
