local util = include( "modules/util" )
local array = include( "modules/array" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local inventory = include("sim/inventory")

local maa_install_weapon_mod =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.NAME,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
	
	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.DESC )
	end,
	
	alwaysShow = true,
	getName = function( self, sim, unit )
		return self.name
	end,

	onSpawnAbility = function( self, sim, unit )
		
	end,

	onDespawnAbility = function( self, sim, unit )
		
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		
		local weapon = array.findIf( userUnit:getChildren(), function( u ) return u:getTraits().maa_is_shalem_mod_rifle ~= nil end )
		if not weapon then
			return false --Don't show if weapon is not present
		end
		
		--can't install mods when the weapon is on cooldown
		if weapon:getTraits().cooldown and weapon:getTraits().cooldown > 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.ON_COOLDOWN
		end
		
		--can't install mods when the weapon was used this turn
		if weapon:getTraits().maa_weapon_used then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.USED
		end
		return true
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local weapon = array.findIf( userUnit:getChildren(), function( u ) return u:getTraits().maa_is_shalem_mod_rifle ~= nil end )
		
		if weapon then
			--check if a mod already occupies the slot
			for i, weapon_mod in ipairs( weapon:getChildren() ) do
				if weapon_mod:getTraits().maa_mod_slot and weapon_mod:getTraits().maa_mod_slot == unit:getTraits().maa_mod_slot then
					local unitData = weapon_mod:getUnitData()
					if unitData.maa_doRemove then
						unitData.maa_doRemove( weapon_mod, weapon )
					end
					
					--put the mod back into the users inventory
					weapon:removeChild( weapon_mod )
					userUnit:addChild( weapon_mod )
				end
			end
			
			--put the new mod into the weapon
			userUnit:removeChild( unit )
			weapon:addChild( unit )
			weapon:getTraits().maa_weapon_modded = true
			
			--refresh the rifle
			sim:triggerEvent( "TRG_MAA_REFRESH_MOD_WEAPON", nil )
			inventory.equipItem( userUnit, weapon )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
		end
	end,
}

return maa_install_weapon_mod
