local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_kinetic_boost =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.NAME,
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_speed_small.png",
	alwaysShow = true,
	maa_activeAbility = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local level = 0
		local reactor_unit = nil
		if userUnit then
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
		end
		if reactor_unit then
			level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.NAME, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.DESC, level ) )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not userUnit then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end

		local reactor_unit = nil
		for _, child in ipairs( userUnit:getChildren() ) do
			if child:getTraits().maa_power_allocated then
				reactor_unit = child
				break
			end
		end
		if not reactor_unit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_DOWN.NO_REACTOR
		end
		if (reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] or 0) <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.NO_POWER
		end
    if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
      return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
    end

		return true
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local reactor_unit = nil
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
			if reactor_unit then
        local level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]
        userUnit:addMP( level or 1 )
        local x1, y1 = userUnit:getLocation()
        sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
        sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.KINETIC_BOOST, x=x1, y=y1, color={r=1,g=1,b=1,a=1}} )
				self.penalty = level or 1
				if unit:getTraits().cooldownMax then
					 unit:getTraits().cooldown = unit:getTraits().cooldownMax
				end
			end
		end
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_START_TURN then
				if evData:isPC() and userUnit then
					if self.penalty then
						userUnit:addMP( -self.penalty )
						self.penalty = nil
					end
				end
			end
		end
	end,
}

return maa_kinetic_boost
