local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_activate_heartshocktrap =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.LOCATE_HEARTSHOCKTRAP.NAME,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_scanner_small.png",
	alwaysShow = true,
	maa_activeAbility = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.LOCATE_HEARTSHOCKTRAP.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.LOCATE_HEARTSHOCKTRAP.DESC, 0 )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end
		local hasTarget = nil
		for _, testUnit in pairs( sim:getAllUnits() ) do
			if testUnit:getTraits().maa_trap_installed then
				hasTarget = true
				break
			end
		end
		if not hasTarget then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.ACTIVATE_HEARTSHOCKTRAP.NO_TRAP
		end
		if unit:getTraits().pwr_cost_locate and player:getCpus() < unit:getTraits().pwr_cost_locate then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end

		return true
	end,

	onSpawnAbility = function( self, sim, unit )
		unit:getTraits().trapID = nil
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local trapTargets = {}
		for _, testUnit in pairs( sim:getAllUnits() ) do
			if testUnit:getTraits().maa_trap_installed then
				table.insert( trapTargets, testUnit )
			end
		end
		for _, target in ipairs( trapTargets ) do
			local target = sim:getUnit( trap:getTraits().maa_trap_targetID )
			if target then
				local player = unit:getPlayerOwner()
				player:glimpseUnit( sim, target:getID() )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = target } )

				local cpus = unit:getTraits().pwr_cost_locate
				player:addCPUs( -cpus )
				local x0, y0 = userUnit:getLocation()
				sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
				sim:dispatchEvent( simdefs.EV_CAM_PAN, { target:getLocation() } )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30 )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = target, maa_reveal = true } )
			end
		end
	end,
}

return maa_activate_heartshocktrap
