local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local UPGRADE_OPTIONS =
{
	CANCEL = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.CANCEL,
	COOLDOWN = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.COOLDOWN,
	CHARGE = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.CHARGE,
	AMMO = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.AMMO,
	POWER = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.POWER,
	SKILL_REQ = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.SKILL_REQ,
	ARMOR_PIERCE = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.ARMOR_PIERCE,
	DAMAGE = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.UPGRADE_OPTIONS.DAMAGE,
}

local function showDialog( sim, headerTxt, bodyTxt, options, t, result )
	sim._choiceCount = sim._choiceCount + 1
	local choice = sim._choices[ sim._choiceCount ]
	options = options or { "OK" }

	if not choice then
		-- Choice has not yet been made.
		sim:dispatchChoiceEvent( "MAA_EV_CHOICE_DIALOG", util.extend( { headerTxt = headerTxt, bodyTxt = bodyTxt, options = options }) ( t ), true )
		choice = sim._choices[ sim._choiceCount ]
	end

	return choice
end

local maa_mod_item =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.NAME,
	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.DESC )
	end,

	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_technician_small.png",

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, itemUnit, unit )
		local toolkit = array.findIf( unit:getChildren(), function( u ) return u:getTraits().maa_tony_toolkit ~= nil end )
		if not toolkit then
			return false --Don't show if no toolkit
		end
		local legal_user = false
		local userAgentID = unit:getUnitData().agentID
		if not userAgentID then
			return false
		end
		if toolkit:getTraits().restrictedUse then
			for _, user in ipairs( toolkit:getTraits().restrictedUse ) do
				if user.agentID == userAgentID then
					legal_user = true
					break
				end
			end
		end
		if not legal_user then
			return false
		end
		if itemUnit:getTraits().maa_modded_item then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.ALREADY_MODDED
		end

		return true
	end,

	executeAbility = function( self, sim, itemUnit, unit )
		local toolkit = array.findIf( unit:getChildren(), function( u ) return u:getTraits().maa_tony_toolkit ~= nil end )

		if toolkit then
			local upgrade_option = {}
			table.insert(upgrade_option, UPGRADE_OPTIONS.CANCEL)
			if itemUnit:getTraits().cooldownMax and itemUnit:getTraits().cooldownMax > 1 then
				table.insert(upgrade_option, UPGRADE_OPTIONS.COOLDOWN)
			end
			if itemUnit:getTraits().chargesMax then
				table.insert(upgrade_option, UPGRADE_OPTIONS.CHARGE)
			end
			if itemUnit:getTraits().maxAmmo then
				table.insert(upgrade_option, UPGRADE_OPTIONS.AMMO)
			end
			if itemUnit:getTraits().pwrCost then
				table.insert(upgrade_option, UPGRADE_OPTIONS.POWER)
			end
			if itemUnit:getRequirements() then
				for _, _ in pairs( itemUnit:getRequirements() ) do
					table.insert(upgrade_option, UPGRADE_OPTIONS.SKILL_REQ)
					break
				end
			end
			if itemUnit:getTraits().armorPiercing or (itemUnit:getTraits().damage and itemUnit:getTraits().damage > 0) or (itemUnit:getTraits().baseDamage and itemUnit:getTraits().baseDamage > 0) then
				table.insert(upgrade_option, UPGRADE_OPTIONS.ARMOR_PIERCE)
			end
			if (itemUnit:getTraits().damage and itemUnit:getTraits().damage > 0) or (itemUnit:getTraits().baseDamage and itemUnit:getTraits().baseDamage > 0) then
				table.insert(upgrade_option, UPGRADE_OPTIONS.DAMAGE)
			end

			if #upgrade_option > 0 then
				local option = showDialog( sim, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.MODAL_TITLE, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MOD_ITEM.MODAL_TEXT, itemUnit:getName() ), upgrade_option )

				local modification_done = false
				if upgrade_option[option] == UPGRADE_OPTIONS.CANCEL then
					--do nothing
				elseif upgrade_option[option] == UPGRADE_OPTIONS.COOLDOWN then
					itemUnit:getTraits().maa_mod_cooldownMax = (itemUnit:getTraits().maa_mod_cooldownMax or 0) - 1
					itemUnit:getTraits().cooldownMax = itemUnit:getTraits().cooldownMax - 1
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.COOLDOWN
					modification_done = true
				elseif upgrade_option[option] == UPGRADE_OPTIONS.CHARGE then
					itemUnit:getTraits().maa_mod_chargesMax = (itemUnit:getTraits().maa_mod_chargesMax or 0) + 1
					itemUnit:getTraits().chargesMax = itemUnit:getTraits().chargesMax + 1
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.CHARGE
					modification_done = true
				elseif upgrade_option[option] == UPGRADE_OPTIONS.AMMO then
					itemUnit:getTraits().maa_mod_maxAmmo = (itemUnit:getTraits().maa_mod_maxAmmo or 0) + 1
					itemUnit:getTraits().maxAmmo = itemUnit:getTraits().maxAmmo + 1
					itemUnit:getTraits().ammo = (itemUnit:getTraits().ammo or 0) + 1
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.AMMO
					modification_done = true
				elseif upgrade_option[option] == UPGRADE_OPTIONS.POWER then
					itemUnit:getTraits().maa_mod_pwrCost = (itemUnit:getTraits().maa_mod_pwrCost or 0) - 1
					itemUnit:getTraits().pwrCost = itemUnit:getTraits().pwrCost - 1
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.POWER
					modification_done = true
				elseif upgrade_option[option] == UPGRADE_OPTIONS.SKILL_REQ then
					itemUnit:getTraits().maa_mod_requirements = true
					itemUnit._unitData.requirements = nil
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.SKILL_REQ
					modification_done = true
				elseif upgrade_option[option] == UPGRADE_OPTIONS.ARMOR_PIERCE then
					itemUnit:getTraits().maa_mod_armorPiercing = (itemUnit:getTraits().maa_mod_armorPiercing or 0) + 1
					itemUnit:getTraits().armorPiercing = (itemUnit:getTraits().armorPiercing or 0) + 1
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.ARMOR_PIERCE
					modification_done = true
				elseif upgrade_option[option] == UPGRADE_OPTIONS.DAMAGE then
					itemUnit:getTraits().maa_mod_damage = (itemUnit:getTraits().maa_mod_damage or 0) + 1
					if itemUnit:getTraits().damage and itemUnit:getTraits().stun then
						itemUnit:getTraits().stun = itemUnit:getTraits().stun + 1
					end
					if itemUnit:getTraits().damage then
						itemUnit:getTraits().damage = itemUnit:getTraits().damage + 1
					end
					if itemUnit:getTraits().baseDamage then
						itemUnit:getTraits().baseDamage = itemUnit:getTraits().baseDamage + 1
					end
					itemUnit:getTraits().maa_modded_item = UPGRADE_OPTIONS.DAMAGE
					modification_done = true
				end

				if modification_done then
					toolkit:getTraits().ammo = toolkit:getTraits().ammo - 1
					if toolkit:getTraits().ammo <= 0 then
						inventory.trashItem( sim, unit, toolkit )
					end

					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Weapons/LowBore/reload_handgun" )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/EMP_explo" )
					sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= unit, anim="pick_up"} )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/monst3r_jackin" )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )

				end
			end
		end
	end,
}

return maa_mod_item
