local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )
local simquery = include("sim/simquery")
local mathutil = include( "modules/mathutil" )

local maa_mp_advance =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MP_ADVANCE.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hide_small.png",
	
	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MP_ADVANCE.NAME, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MP_ADVANCE.DESC, unit:getTraits().pwr_cost_advance ), 1 )
	end,
	
	proxy = true,
	maa_mp_ability = true,

	getName = function( self, sim, unit )
		return self.name
	end,
	
	isTarget = function( self, sim, userUnit, targetUnit )
		if targetUnit:getTraits().maa_mainframe_entrance then
			return true
		end
		
		return false
	end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local cell = sim:getCell( abilityUser:getLocation() )
		local units = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTarget( sim, abilityUser, cellUnit ) then
					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
	end,
	
	canUseAbility = function( self, sim, unit, userUnit )
		local player = unit:getPlayerOwner()
		if not unit:getTraits().installed then
			return false
		end
		
		if unit:getTraits().pwr_cost_advance and player:getCpus() < unit:getTraits().pwr_cost_advance then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
		
		if unit:getTraits().cd_advance and unit:getTraits().cd_advance > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cd_advance)
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, unit, userUnit, target )
		--find the closest undiscovered mainframe device
		local x0, y0 = userUnit:getLocation()
		local closest_device = nil
		local closest_distance = nil
		if x0 and y0 then
			for _, checkUnit in pairs( sim:getAllUnits() ) do
				if checkUnit:getTraits().mainframe_item and not sim:getPC():hasSeen( checkUnit ) then
					local x1, y1 = checkUnit:getLocation()
					if x1 and y1 then
						local distance = mathutil.dist2d( x0, y0, x1, y1 )
						if not closest_distance or distance < closest_distance then
							closest_distance = distance
							closest_device = checkUnit
						end
					end
				end
			end
		end
		
		if closest_device then
			sim:getPC():glimpseUnit( sim, closest_device:getID() )
			sim:dispatchEvent( simdefs.EV_CAM_PAN, { closest_device:getLocation() } )
			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30 )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = closest_device, maa_reveal = true } )
			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60 )
		end
		
		if unit:getTraits().cdMax_advance then
			unit:getTraits().cd_advance = unit:getTraits().cdMax_advance
		end
		
		local player = unit:getPlayerOwner()
		local cpus = unit:getTraits().pwr_cost_advance
		player:addCPUs( -cpus )
		local x0, y0 = userUnit:getLocation()
		sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
	end,
}

return maa_mp_advance
