local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )
local simquery = include("sim/simquery")

local maa_mp_control_device =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MP_CONTROL.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_augment.png",
	
	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local power_cost = unit:getTraits().pwr_cost_control_base or 0
		power_cost = power_cost - (unit:getTraits().pwr_cost_control_reduction_per_turn or 0) * (userUnit:getTraits().maa_turns_inside_mainframe or 0)
		
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MP_CONTROL.NAME, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.MP_CONTROL.DESC, power_cost, (unit:getTraits().pwr_cost_control_reduction_per_turn or 0) ), 0 )
	end,
	
	proxy = true,
	maa_mp_ability = true,

	getName = function( self, sim, unit )
		return self.name
	end,
	
	isTarget = function( self, sim, userUnit, targetUnit )
		if userUnit:getTraits().mma_controled_device and userUnit:getTraits().mma_controled_device == targetUnit then
			return false
		end
		if targetUnit:getTraits().noTakeover then
			return false
		end
		if targetUnit:getTraits().mainframe_item and targetUnit:getTraits().mainframe_status and targetUnit:getTraits().mainframe_status == "active" and userUnit:getPlayerOwner() ~= targetUnit:getPlayerOwner() then
			return true
		end
		
		return false
	end,
	
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local localPlayer = unit:getPlayerOwner()
		local units = {}
		for _, checkUnit in pairs( sim:getAllUnits() ) do
			local x1, y1 = checkUnit:getLocation()
			
			if x1 and y1 and localPlayer:getCell( x1, y1 ) then
				if self:isTarget( sim, userUnit, checkUnit ) then
					table.insert( units, checkUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,
	
	canUseAbility = function( self, sim, unit, userUnit )
		local player = unit:getPlayerOwner()
		if not unit:getTraits().installed then
			return false
		end
		
		if not userUnit:getTraits().maa_inside_mainframe then
			return false
		end
		
		local power_cost = unit:getTraits().pwr_cost_control_base or 0
		power_cost = power_cost - (unit:getTraits().ppwr_cost_control_reduction_per_turn or 0) * (userUnit:getTraits().maa_turns_inside_mainframe or 0)
		
		if player:getCpus() < power_cost then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
		
		if unit:getTraits().cd_control and unit:getTraits().cd_control > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cd_control)
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, unit, userUnit, target )
		--if the user already controls a device, lose control over that device first
		if userUnit:getTraits().mma_controled_device then
			--drones have a specific way to lose control
			if userUnit:getTraits().mma_controled_device.loseControl then
				--grab the ice
				local ice_before = userUnit:getTraits().mma_controled_device:getTraits().mainframe_ice
				
				userUnit:getTraits().mma_controled_device:loseControl( sim )
				
				--reset the ice
				userUnit:getTraits().mma_controled_device:getTraits().mainframe_ice = ice_before
				--skip drone reboot
				userUnit:getTraits().mma_controled_device:setKO( sim, nil )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit:getTraits().mma_controled_device } )
			else
				userUnit:getTraits().mma_controled_device:takeControl( sim:getNPC() )
			end
			sim:getPC():glimpseUnit( sim, userUnit:getTraits().mma_controled_device:getID() )
			userUnit:getTraits().mma_controled_device = nil
		end
		local targetUnit = sim:getUnit( target )
		userUnit:getTraits().mma_controled_device = targetUnit
		targetUnit:takeControl( sim:getPC() )
		
		if unit:getTraits().cdMax_control then
			unit:getTraits().cd_control = unit:getTraits().cdMax_control
		end
		
		local player = unit:getPlayerOwner()
		local cpus = unit:getTraits().pwr_cost_control_base - (userUnit:getTraits().maa_turns_inside_mainframe or 0)
		player:addCPUs( -cpus )
		local x0, y0 = userUnit:getLocation()
		sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
	end,
}

return maa_mp_control_device
