local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")

local maa_neutal_interrogation =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.NEUTAL_INTERROGATION.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.NEUTAL_INTERROGATION.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.NEUTAL_INTERROGATION.DESC )
	end,


	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_generic_head_small.png",
	proxy = true,
	alwaysShow = true,
	ghostable = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local cell = sim:getCell( userUnit:getLocation() )

		local units = {}


		for i, cellUnit in ipairs(cell.units) do
			if cellUnit:getTraits().isGuard and not cellUnit:getTraits().isDrone and cellUnit:isKO() and not cellUnit:getTraits().maa_neural_interrogated then
				table.insert( units, cellUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()

		if not unit:getTraits().installed then
			return false
		end
		if not userUnit then
			return false
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		target = sim:getUnit(target)

		local potentialUnits = {}
		for _, checkUnit in pairs( sim:getAllUnits() ) do
			if ( checkUnit:getTraits().mainframe_item or checkUnit:getTraits().isGuard ) and not sim:getPC():hasSeen( checkUnit ) then
				table.insert( potentialUnits, checkUnit )
			end
		end

		if #potentialUnits > 0 then
			local selectedUnit = potentialUnits[ sim:nextRand(1, #potentialUnits) ]
			sim:getPC():glimpseUnit( sim, selectedUnit:getID() )
			sim:dispatchEvent( simdefs.EV_CAM_PAN, { selectedUnit:getLocation() } )
			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30 )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = selectedUnit, maa_reveal = true } )
			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60 )
			sim:dispatchEvent( simdefs.EV_CAM_PAN, { userUnit:getLocation() } )
		else
			local x0, y0 = userUnit:getLocation()
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.NO_INTERROGATION_TARGETS, x=x0, y=y0, color={r=163/255,g=0,b=0,a=1}} )
		end
		target:getTraits().maa_neural_interrogated = true
	end,
}

return maa_neutal_interrogation
