local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_overcharge_routine =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_OVERCHARGE.NAME,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chargeweapon_small.png",
	alwaysShow = true,
	maa_activeAbility = true,

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_OVERCHARGE.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_OVERCHARGE.DESC, 0 )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		if not unit:getTraits().installed then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown)
		end

		return true
	end,

	executeAbility = function( self, sim, unit )
		local player = sim:getPC()
		if not player:getTraits().maa_incognita_overcharge_penalties then
			player:getTraits().maa_incognita_overcharge_penalties = 0
		end
		player:getTraits().maa_incognita_overcharge_penalties = player:getTraits().maa_incognita_overcharge_penalties + 1
		player:getTraits().maa_incognita_overcharge_buff_active = true

		if unit:getTraits().maa_breaking_buff then
	    		player:getTraits().firewallBreakPenalty = ( player:getTraits().firewallBreakPenalty or 0 ) - unit:getTraits().maa_breaking_buff
	    	end
	    	if unit:getTraits().mma_cd_buff then
	    		for _, program in ipairs( player:getAbilities() ) do
	    			if program.cooldown then
	    				program.cooldown = math.max ( program.cooldown - unit:getTraits().mma_cd_buff, 0 )
	    			end
	    		end
	    	end
	    	if unit:getTraits().cooldownMax then
	    		unit:getTraits().cooldown = unit:getTraits().cooldownMax
	    	end
				local dialogParams =
				{
					STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_OVERCHARGE.POPUP_DESC_1,
					STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_OVERCHARGE.POPUP_DESC_2,
					STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_OVERCHARGE.POPUP_DESC_3,
					"gui/icons/item_icons/icon-item_chargeweapon.png"
				}
				sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local player = sim:getPC()
		if evType == simdefs.TRG_END_TURN then
			if evData:isPC() then
				if player:getTraits().maa_incognita_overcharge_buff_active then
					player:getTraits().maa_incognita_overcharge_buff_active = nil

					if unit:getTraits().maa_breaking_buff then
				    		player:getTraits().firewallBreakPenalty = ( player:getTraits().firewallBreakPenalty or 0 ) + unit:getTraits().maa_breaking_buff
				    	end
				end

				if not player:getTraits().maa_incognita_overcharge_penalties_applied then
					player:getTraits().maa_incognita_overcharge_penalties_applied = 0
				end

				if player:getTraits().maa_incognita_overcharge_penalties_applied < ( player:getTraits().maa_incognita_overcharge_penalties or 0 ) then
					if unit:getTraits().maa_program_cost_penalty then
						player:getTraits().program_cost_modifier = ( player:getTraits().program_cost_modifier or 0 ) + unit:getTraits().maa_program_cost_penalty * ( ( player:getTraits().maa_incognita_overcharge_penalties or 0 ) - player:getTraits().maa_incognita_overcharge_penalties_applied )
					end
					player:getTraits().maa_incognita_overcharge_penalties_applied = ( player:getTraits().maa_incognita_overcharge_penalties or 0 )
				end
			end
		end
	end,
}

return maa_overcharge_routine
