local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )

local maa_particle_uplink =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PARTICLE_UPLINK.NAME,

	getName = function( self, sim, unit )
		return self.name
	end,

	onSpawnAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local userAgentID = userUnit:getUnitData().agentID
			
			unit:getTraits().partnerAugment = nil

			if unit:getTraits().maa_partner_installed then
				for _, simUnit in pairs( sim:getAllUnits() ) do
					if simUnit:getTraits().maa_partner_installed and simUnit:getTraits().maa_partner_installed == userAgentID then
						unit:getTraits().partnerAugment = simUnit
						simUnit:getTraits().partnerAugment = unit
					end
				end
			end
			--if we don't find it, it probably hasn't spawned yet and its up to the partner to do the connection
			
			sim:addTrigger( simdefs.TRG_ACTION, self, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		end
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_ACTION, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local userAgentID = userUnit:getUnitData().agentID
			if evType == simdefs.TRG_ACTION then
				if not unit:getTraits().maa_partner_installed then
				--We listen to any action and check if the augment is getting installed. Permanently link the agents on install.
					if evData.ClassType == "abilityAction" and not evData.pre then
						if unit:getTraits().installed then
							--We don't really need to specify further. If any action happened and the augment is now installed, then the user installed the augment obviously.
							for _, simUnit in pairs( sim:getAllUnits() ) do
								if simUnit ~= unit and simUnit:getTraits().maa_is_particle_uplink and not simUnit:getTraits().maa_partner_installed and simUnit:getTraits().installed then
									local partnerUser = simUnit:getUnitOwner()
									local partnerAgentID = nil
									if partnerUser then
										partnerAgentID = partnerUser:getUnitData().agentID
									end
									if partnerAgentID then
										simUnit:getTraits().maa_partner_installed = userAgentID
										unit:getTraits().maa_partner_installed = partnerAgentID
										simUnit:getTraits().partnerAugment = unit
										unit:getTraits().partnerAugment = simUnit
									end
								end
							end
						end
					end
				end
			elseif evType == simdefs.TRG_START_TURN and evData:isPC() then
				unit:getTraits().maa_swap_cooldown = math.max(0, unit:getTraits().maa_swap_cooldown - 1)
			end
		end
	end,
}

return maa_particle_uplink
