local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")

local maa_place_heartshocktrap =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PLACE_HEARTSHOCKTRAP.NAME,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png",
	--alwaysShow = true,
	proxy = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PLACE_HEARTSHOCKTRAP.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PLACE_HEARTSHOCKTRAP.DESC, 0 )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	isTarget = function( self, sim, userUnit, targetUnit )
		if targetUnit:getTraits().heartMonitor then
			return true
		end

		return false
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local cell = sim:getCell( abilityUser:getLocation() )
		local units = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTarget( sim, abilityUser, cellUnit ) then
					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		if not simquery.isAgent( userUnit ) then
			return false
		end
		if unit:getUnitOwner() ~= userUnit then
			return false
		end
		--if unit:getTraits().maa_trap_active then
		--	return false --should never happen, just to be sures
		--end
		local target = sim:getUnit( targetUnitID )
		if target then
			if target:getTraits().maa_trap_installed then
				return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PLACE_HEARTSHOCKTRAP.ALREADY_TRAP
			end
		end
		
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown)
		end

		if target then
			local sight, periph = sim:canUnitSeeUnit( target, userUnit )
			if sight then
				return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PLACE_HEARTSHOCKTRAP.SIGHTED
			end
		end

		return true
	end,

	onSpawnAbility = function( self, sim, unit )
		--sim:addTrigger( "MAA_TRG_ITEM_LOOT", self, unit )
		unit:getTraits().activatorID = nil
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		--if evType == "MAA_TRG_ITEM_LOOT" then
		--	--Is the heartshocktrap transfered? Then remove the trap_active trait.
		--	if evData.item and evData.item == unit then
		--		unit:getTraits().maa_trap_active = false
		--	end
		--end
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		if unit:getTraits().maa_trap_targetID then
			local old_target = sim:getUnit( unit:getTraits().maa_trap_targetID )
			old_target:getTraits().maa_trap_installed = nil
		end
		local target = sim:getUnit(target)
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = target:getLocation()
		local newFacing = simquery.getDirectionFromDelta(x1-x0,y1-y0)
		userUnit:setInvisible(false)
		userUnit:setDisguise(false)
		userUnit:setFacing( newFacing )
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR, { unitID = userUnit:getID() } )
		sim:emitSound( simdefs.SOUND_PLACE_TRAP, x0, y0, userUnit)
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR_PST, { unitID = userUnit:getID() } )
		--userUnit:removeChild( unit )
		--target:addChild( unit )
		--unit:getTraits().maa_trap_active = true
		unit:getTraits().maa_trap_targetID = target:getID()
		target:getTraits().maa_trap_installed = unit:getTraits().damage
		
		inventory.useItem( sim, userUnit, unit )
	end,
}

return maa_place_heartshocktrap
