local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local maa_power_battery =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PORTABLE_POWER_BATTERY.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_PICKEDUP, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_DEPLOYED, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_PICKEDUP, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_DEPLOYED, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if evType == simdefs.TRG_START_TURN then
			if unit:getTraits().deployed and evData:isPC() then
				if unit:getTraits().maa_powerMax and unit:getTraits().maa_power_generated and unit:getTraits().maa_power_generated < unit:getTraits().maa_powerMax then
					local x0, y0 = unit:getLocation()
					if x0 and y0 then
						local cpus = math.min( (unit:getTraits().maa_power_per_turn or 0), unit:getTraits().maa_powerMax - unit:getTraits().maa_power_generated )
						unit:getPlayerOwner():addCPUs( unit:getTraits().cpus )
						sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.PLUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
						unit:getTraits().maa_power_generated = unit:getTraits().maa_power_generated + cpus
					end
				end
			end
		elseif evType == simdefs.TRG_UNIT_DEPLOYED then
			if evData.unit == unit then
				local player = unit:getPlayerOwner()
				player:getTraits().PWRmaxBouns = (player:getTraits().PWRmaxBouns or 0) + (unit:getTraits().maa_add_maxPwr or 0)
			end
		elseif evType == simdefs.TRG_UNIT_PICKEDUP then
			if evData.item == unit and unit:getTraits().deployed then
				local player = unit:getPlayerOwner()
				player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns - (unit:getTraits().maa_add_maxPwr or 0)
				player:addCPUs( math.min( player:getMaxCpus() - player:getCpus(), 0 ) ) --remove any power over maxPWR
			end
		end
	end,
}

return maa_power_battery
