local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")

local maa_protocol78 =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PROTOCOL78.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self,sim,unit,targetCell)
		return abilityutil.formatToolTip(STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PROTOCOL78.NAME,  STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PROTOCOL78.DESC )
	end,

	proxy = true,

	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chip_econ_small.png",

	isTarget = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local cell = sim:getCell( abilityUser:getLocation() )
		local units = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTarget( sim, abilityUser, cellUnit ) then
					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		if not userUnit then
			return false
		end
		if unit:getUnitOwner() ~= userUnit then
			return false
		end
		local player = sim:getPC()
		if unit:getTraits().movingBody then
				return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )

		if targetUnit then
			targetUnit:getTraits().mainframe_status = "off"
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit })
			if unit:getTraits().cooldownMax then
				unit:getTraits().cooldown = unit:getTraits().cooldownMax
			end
      			
      			local daemon = sim:getNPC():hasMainframeAbility( "maa_protocol78_daemon" )
      			if not daemon then
      				sim:getNPC():addMainframeAbility( sim, "maa_protocol78_daemon" )
      				local ability = sim:getNPC():hasMainframeAbility( "maa_protocol78_daemon" )
      				if ability and unit:getTraits().maa_credits_per_alarm then
      					ability.credit_increase = unit:getTraits().maa_credits_per_alarm
      				end
      			else
      				daemon.instances = daemon.instances + 1
							sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/HUD/voice/level1/alarmvoice_daemon")
      			end

		end
	end,
}

return maa_protocol78
