local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local mainframe_common = include("sim/abilities/mainframe_common")

local maa_protocol78_daemon = util.extend( mainframe_common.createDaemon( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PROTOCOL78_DAEMON ) )
{
	icon = "gui/icons/daemon_icons/protocol78.png",
	title = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PROTOCOL78_DAEMON.TITLE,
	noDaemonReversal = true,

	onSpawnAbility = function( self, sim, player )
		self.permanent = true
		self.instances = 1
		self.credit_increase = 5
		self.last_tracker = sim:getTracker()

		sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=false, name = self.name, icon=self.icon, txt = self.activedesc, title = self.title } )
		--sim:dispatchEvent( simdefs.EV_SHOW_REVERSE_DAEMON, { showMainframe=false, name = self.name, icon=self.icon, txt = self.activedesc, title = self.title } )
		--local dialogParams =
		--{
		--	self.title,
		--	self.name,
		--	self.activedesc,
		--    	self.icon
		--}
		--sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )

		sim:addTrigger( simdefs.TRG_END_TURN, self )
		sim:addTrigger( simdefs.TRG_ALARM_INCREASE, self )
	end,

	onDespawnAbility = function( self, sim )
		sim:removeTrigger( simdefs.TRG_END_TURN, self )
		sim:removeTrigger( simdefs.TRG_ALARM_INCREASE, self )
	end,

	onTooltip = function( self, hud, sim, player )
		local tooltip = util.tooltip( hud._screen )
		local section = tooltip:addSection()

		local fname = self.name
		if self.instances > 1 then
			fname = fname .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PROTOCOL78_DAEMON.NAME_2, self.instances )
		end
		section:addLine( fname )
		section:addAbility( util.sformat(self.shortdesc, self.instances, (self.instances * self.credit_increase) ), self.desc, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

		if self.dlcFooter then
			section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
		end

		return tooltip
	end,

	onTrigger = function( self, sim, evType, evData, userUnit )
		if evType == simdefs.TRG_END_TURN then
			if evData:isNPC() then
				sim:trackerAdvance( self.instances )
			end
		elseif evType == simdefs.TRG_ALARM_INCREASE then
			local ticks = sim:getTracker() - self.last_tracker
			self.last_tracker = sim:getTracker()

			if sim:getTrackerStage() >= simdefs.TRACKER_MAXSTAGE then
				sim:forEachUnit(
					function( unit )
						if unit:getTraits().safeUnit and unit:getTraits().credits and unit:getTraits().credits > 0 then
							unit:getTraits().credits = nil
						end
					end )
				sim:getNPC():removeAbility(sim, self )
			elseif ticks and ticks > 0 then
				sim:forEachUnit(
					function( unit )
						if unit:getTraits().safeUnit and unit:getTraits().credits and unit:getTraits().credits > 0 then
							unit:getTraits().credits = unit:getTraits().credits + (ticks * self.credit_increase * self.instances)
						end
					end )
			end
		end
	end,
}

return maa_protocol78_daemon
