local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")

local maa_pu_lifelink =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_LIFELINK.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_LIFELINK.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_LIFELINK.DESC )
	end,


	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_select_small.png",

	canUseAbility = function( self, sim, unit )
		return false
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
	
	end,
	
	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
		
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self, unit )
		if self.penalty_applied then
			userUnit:addMPMax( unit:getTraits().maa_lifelink_penalty or 0 )
			self.penalty_applied = nil
		end
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local partnerUnit = nil
			if unit:getTraits().partnerAugment then
				partnerUnit = unit:getTraits().partnerAugment:getUnitOwner()
			end
			local userAgentID = userUnit:getUnitData().agentID
			if evType == simdefs.TRG_UNIT_KO then
				if partnerUnit and evData.unit == partnerUnit then
					if not self.penalty_applied and evData.ticks then
						--partner down, apply penalty
						userUnit:addMPMax( -(unit:getTraits().maa_lifelink_penalty or 0) )
						self.penalty_applied = true
						--remove all AP when this happens
						userUnit:addMP( -userUnit:getMP() )
					elseif self.penalty_applied and not evData.ticks then
						--partner revived, remove penalty
						userUnit:addMPMax( unit:getTraits().maa_lifelink_penalty or 0 )
						self.penalty_applied = nil
					end
				end
			elseif evType == simdefs.TRG_START_TURN and evData:isPC() then
				if sim:getTurnCount() == 0 then
					--check on first turn, if partner is present and apply penalty if not
					if unit:getTraits().maa_partner_installed and not unit:getTraits().partnerAugment then
						userUnit:addMPMax( -(unit:getTraits().maa_lifelink_penalty or 0) )
						self.penalty_applied = true
					end
				end
			end
		end
	end,
}

return maa_pu_lifelink
