local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local array = include( "modules/array" )

local function teleportOut( sim, agent )
	local status = false
	coroutine.yield(status)
	sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = agent:getID(), noSightingFx=true } )
	sim:dispatchEvent( simdefs.EV_TELEPORT, { units={agent}, warpOut =true } )
	sim:warpUnit( agent, nil )
	status = true
end

local maa_pu_linked_pockets =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_LINKED_POCKETS.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_LINKED_POCKETS.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_LINKED_POCKETS.DESC )
	end,


	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_trade.png",
	proxy = true,
	alwaysShow = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local units = {}

		local linked_partner = unit:getTraits().partnerAugment
		if linked_partner and linked_partner:getUnitOwner() and linked_partner:getUnitOwner():getLocation() then
			table.insert( units, linked_partner:getUnitOwner() )
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()

		if not unit:getTraits().installed then
			return false
		end
		if not userUnit then
			return false
		end
		local linked_partner = unit:getTraits().partnerAugment
		if not linked_partner then
			return false
		end
		if simquery.canGive( sim, userUnit, linked_partner:getUnitOwner() ) then
			return false --don't show linked pockets if they can already trade
		end
		if linked_partner:getUnitOwner():isDown() then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.PARTNER_KO
		end
		if player:getCpus() < (unit:getTraits().maa_pocket_pwr_cost or 0) then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local partnerUnit = unit:getTraits().partnerAugment:getUnitOwner()
		local player = unit:getPlayerOwner()
		if partnerUnit then
			self.transfer_check = util.tdupe( userUnit:getChildren() )
			sim:dispatchEvent( "MAA_EV_ITEMS_PANEL", { unit = userUnit, targetUnit = partnerUnit } )
		end
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( "MAA_CLOSE_TRANSFER_PANEL", self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( "MAA_CLOSE_TRANSFER_PANEL", self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local player = unit:getPlayerOwner()
			if evType == "MAA_CLOSE_TRANSFER_PANEL" then
				if self.transfer_check then
					local actually_traded = false
					local count = #self.transfer_check
					for _, child in ipairs( userUnit:getChildren() ) do
						if array.find( self.transfer_check, child ) then
							count = count - 1
						else
							--we found an item, that is in the inventory, that wasn't before
							actually_traded = true
							break
						end
					end
					if count > 0 then
						--there was an item in the inventory before, that isn't now anymore
						actually_traded = true
					end

					if actually_traded then
						--pay up
						local cpus = (unit:getTraits().maa_pocket_pwr_cost or 0)
						local x0, y0 = userUnit:getLocation()
						player:addCPUs( -cpus )
						sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
					end
					self.transfer_check = nil
				end
			end
		end
	end,
}

return maa_pu_linked_pockets
