local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")

local maa_pu_push =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.DESC )
	end,


	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_select_small.png",
	proxy = true,
	alwaysShow = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local units = {}

		local linked_partner = unit:getTraits().partnerAugment
		if linked_partner and linked_partner:getUnitOwner() and linked_partner:getUnitOwner():getLocation() then
			table.insert( units, linked_partner:getUnitOwner() )
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()

		if not unit:getTraits().installed then
			return false
		end
		if not userUnit then
			return false
		end
		local linked_partner = unit:getTraits().partnerAugment
		if not linked_partner then
			return false
		end
		if linked_partner:getUnitOwner():isDown() then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.PARTNER_KO
		end
		if userUnit:getMP() < (unit:getTraits().maa_push_mp_cost or 1) then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.NO_MP
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local partnerUnit = unit:getTraits().partnerAugment:getUnitOwner()
		if partnerUnit then
			partnerUnit:addMP( 1 )
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = partnerUnit } )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = partnerUnit })
			userUnit:addMP( -(unit:getTraits().maa_push_mp_cost or 1) )
		end
	end,
}

return maa_pu_push
