local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")

local function teleportOut( sim, agent )
	local status = false
	coroutine.yield(status)
	sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = agent:getID(), noSightingFx=true } )
	sim:dispatchEvent( simdefs.EV_TELEPORT, { units={agent}, warpOut =true } )
	sim:warpUnit( agent, nil )
	status = true
end

local maa_pu_swap =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_SWAP.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_SWAP.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_SWAP.DESC )
	end,


	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_equip_small.png",
	proxy = true,
	alwaysShow = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local units = {}

		local linked_partner = unit:getTraits().partnerAugment
		if linked_partner and linked_partner:getUnitOwner() and linked_partner:getUnitOwner():getLocation() then
			table.insert( units, linked_partner:getUnitOwner() )
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()

		if not unit:getTraits().installed then
			return false
		end
		if not userUnit then
			return false
		end
		local linked_partner = unit:getTraits().partnerAugment
		if not linked_partner then
			return false
		end
		if linked_partner:getUnitOwner():isDown() then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.PU_PUSH.PARTNER_KO
		end
		if unit:getTraits().maa_swap_cooldown and unit:getTraits().maa_swap_cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().maa_swap_cooldown )
		end
		if player:getCpus() < (unit:getTraits().maa_swap_pwr_cost or 0) then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local partnerUnit = unit:getTraits().partnerAugment:getUnitOwner()
		local player = unit:getPlayerOwner()
		if partnerUnit then
			local cell_own = sim:getCell( userUnit:getLocation() )
			local cell_partner = sim:getCell( partnerUnit:getLocation() )
			--warp both agents to nil
			sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = userUnit:getID(), noSightingFx=true } )
			sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = partnerUnit:getID(), noSightingFx=true } )
			sim:dispatchEvent( simdefs.EV_TELEPORT, { units={userUnit, partnerUnit}, warpOut =true } )
			sim:warpUnit( userUnit, nil )
			sim:warpUnit( partnerUnit, nil )

			--now teleport them back in
			sim:warpUnit( userUnit, cell_partner )
			sim:warpUnit( partnerUnit, cell_own )
			sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = userUnit:getID(), noSightingFx=true } )
			sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = partnerUnit:getID(), noSightingFx=true } )
			sim:dispatchEvent( simdefs.EV_TELEPORT, { units={userUnit, partnerUnit}, warpOut =false } )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = partnerUnit } )

			--process reactions
			sim:processReactions( userUnit )
			sim:processReactions( partnerUnit )

			--pay up
			unit:getTraits().maa_swap_cooldown = unit:getTraits().maa_swap_cooldownMax
			unit:getTraits().partnerAugment:getTraits().maa_swap_cooldown = unit:getTraits().partnerAugment:getTraits().maa_swap_cooldownMax

			local x0, y0 = userUnit:getLocation()
			local cpus = (unit:getTraits().maa_swap_pwr_cost or 0)
			player:addCPUs( -cpus )
			sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
		end
	end,
}

return maa_pu_swap
