local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local unitdefs = include( "sim/unitdefs" )

local maa_refresh_weapon_mods =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.NAME, --dummy name
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( "TRG_MAA_REFRESH_MOD_WEAPON", self, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( "TRG_MAA_REFRESH_MOD_WEAPON", self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit and evType == "TRG_MAA_REFRESH_MOD_WEAPON" then
			--bring the weapon back to its template state (this will also remove any buffs from the likes of tonys tools. But I think its fair to make this weapon non-upgradeable)
			local used_state = unit:getTraits().maa_weapon_used
			local modded_state = unit:getTraits().maa_weapon_modded
			
			local template = unitdefs.lookupTemplate( unit:getUnitData().upgradeName )
			for trait, value in pairs( unit:getTraits() ) do
				unit:getTraits()[trait] = nil
			end
			for trait, value in pairs( template.traits ) do
				unit:getTraits()[trait] = value
			end
			--unit:getTraits() = util.tcopy( template.traits )
			
			local callback_order = { "clip", "scope", "special" }
			for _, slot_type in ipairs( callback_order ) do
				for _, weapon_mod in ipairs( unit:getChildren() ) do
					if weapon_mod:getTraits().maa_mod_slot and weapon_mod:getTraits().maa_mod_slot == slot_type then
						local unitData = weapon_mod:getUnitData()
						if unitData.maa_doUpgrade then
							unitData.maa_doUpgrade( weapon_mod, unit )
						end
						break
					end
				end
			end
			
			unit:getTraits().maa_weapon_used = used_state
			unit:getTraits().maa_weapon_modded = modded_state
		elseif unit and evType == simdefs.TRG_START_TURN then
			if evData:isPC() then
				unit:getTraits().maa_weapon_used = nil
				unit:getTraits().maa_weapon_modded = nil
			end
		end
	end,
}

return maa_refresh_weapon_mods
