local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")

local maa_reset_overcharge_routine =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_RESET.NAME,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_incognita_small.png",
	proxy = true,
	
	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_RESET.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_RESET.DESC, 0 )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,
	
	isTarget = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().mainframe_console then
			return false
		end

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end

		return true
	end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local cell = sim:getCell( abilityUser:getLocation() )
		local units = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTarget( sim, abilityUser, cellUnit ) then
					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit )
		local player = sim:getPC()
		if not unit:getTraits().installed then
			return false
		end
		if not player:getTraits().maa_incognita_overcharge_penalties or player:getTraits().maa_incognita_overcharge_penalties <= 0 then
			return false
		end
		if unit:getTraits().ammo and unit:getTraits().ammo <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INCOGNITA_RESET.NO_CHARGE
		end

		return true
	end,

	executeAbility = function( self, sim, unit )
		local player = sim:getPC()
		self.lockout_duration = 2
		sim:setMainframeLockout( true )
		
		player:getTraits().maa_incognita_overcharge_penalties = 0
		if player:getTraits().maa_incognita_overcharge_penalties_applied then
			if unit:getTraits().maa_program_cost_penalty then
				player:getTraits().program_cost_modifier = ( player:getTraits().program_cost_modifier or 0 ) - unit:getTraits().maa_program_cost_penalty * ( player:getTraits().maa_incognita_overcharge_penalties_applied )
					end
			player:getTraits().maa_incognita_overcharge_penalties_applied = 0
		end
		
		unit:getTraits().ammo = math.max( (unit:getTraits().ammo or 0) - 1, 0 )
	end,

	onSpawnAbility = function( self, sim, unit )
		unit:getTraits().ammo = unit:getTraits().maxAmmo
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local player = sim:getPC()
		if evType == simdefs.TRG_END_TURN then
			if evData:isPC() then
				if self.lockout_duration then
					self.lockout_duration = self.lockout_duration - 1
					if self.lockout_duration <= 0 then
						self.lockout_duration = nil
						sim:setMainframeLockout( false )
					end
				end
			end
		end
	end,
}

return maa_reset_overcharge_routine
