local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_servo_punch =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.SERVO_PUNCH.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_melee_small-old.png",
	alwaysShow = true,
	maa_activeAbility = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local level = 0
		local reactor_unit = nil
		if userUnit then
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
		end
		if reactor_unit then
			level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]
		end
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.SERVO_PUNCH.NAME, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.SERVO_PUNCH.DESC, level ) )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not userUnit then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end

		local reactor_unit = nil
		for _, child in ipairs( userUnit:getChildren() ) do
			if child:getTraits().maa_power_allocated then
				reactor_unit = child
				break
			end
		end
		if not reactor_unit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_DOWN.NO_REACTOR
		end
		if (reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] or 0) <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.KINETIC_BOOST.NO_POWER
		end
    if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
      return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
    end

		return true
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
    sim:addTrigger( simdefs.TRG_UNIT_HIT, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
    sim:removeTrigger( simdefs.TRG_UNIT_HIT, self, unit )
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local reactor_unit = nil
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
			if reactor_unit then
        local level = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]

    		local x0, y0 = userUnit:getLocation()
    		userUnit:getTraits().maa_buff_melee_damage = (userUnit:getTraits().maa_buff_melee_damage or 0) + level
    		self._sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.SERVO_PUNCH_DAMAGE_UP, x=x0, y=y0, color={r=255/255,g=178/255,b=102/255,a=1}} )

				if unit:getTraits().cooldownMax then
					 unit:getTraits().cooldown = unit:getTraits().cooldownMax
				end
			end
		end
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_START_TURN then
				if evData:isPC() and userUnit then
					userUnit:getTraits().maa_buff_melee_damage = nil
				end
      elseif evType == simdefs.TRG_UNIT_HIT then
        if evData.melee and evData.sourceUnit == userUnit then
					userUnit:getTraits().maa_buff_melee_damage = nil
				end
			end
		end
	end,
}

return maa_servo_punch
