local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )

local maa_sharp_power_up =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_UP.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
	alwaysShow = true,
	maa_activeAbility = true,

	createToolTip = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_UP.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_UP.DESC )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		local player = unit:getPlayerOwner()
		if not player then
			return false
		end
		if not userUnit then
			return false
		end
		if not unit:getTraits().installed then
			return false
		end
		
		local reactor_unit = nil
		for _, child in ipairs( userUnit:getChildren() ) do
			if child:getTraits().maa_power_allocated then
				reactor_unit = child
				break
			end
		end
		if not reactor_unit then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_DOWN.NO_REACTOR
		end
		if ( unit:getTraits().maa_target_level or reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] ) >= unit:getTraits().maa_max_level then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_UP.MAX_LEVEL
		end
		if (reactor_unit:getTraits().maa_available_power or 0) <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.POWER_UP.NO_POWER
		end

		return true
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			local reactor_unit = nil
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits().maa_power_allocated then
					reactor_unit = child
					break
				end
			end
			if reactor_unit then
				unit:getTraits().maa_target_level = ( unit:getTraits().maa_target_level or reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] ) + 1
				reactor_unit:getTraits().maa_available_power = reactor_unit:getTraits().maa_available_power - 1
				
				if unit:getTraits().maa_target_level == reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ] then
					reactor_unit:getTraits().maa_can_power_down = true
				end
			end
		end
	end,
}

return maa_sharp_power_up
