local util = include( "modules/util" )
local simdefs = include("sim/simdefs")

local skill_order = {"stealth", "hacking", "inventory", "anarchy"}

local maa_sharp_reactor =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.NUCLEAR_REACTOR.NAME,
	profile_icon = nil,

	getName = function( self, sim, unit )
		return self.name
	end,
	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	executeAbility = nil, -- Passives by definition have no execute.

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )

		local userUnit = unit:getUnitOwner()
		if userUnit then
			unit:getTraits().maa_available_power = 0

			local skill_backup = util.tcopy( userUnit:getSkills() )
			userUnit._skills = {}
			for _, skill in ipairs( skill_order ) do
				local skill_level = unit:getTraits().maa_power_allocated[ skill ]
				userUnit:giveSkill( skill, skill_level+1, sim )
				unit:getTraits().maa_available_power = unit:getTraits().maa_available_power - skill_level
			end
			for _, skill_back in ipairs( skill_backup ) do
				table.insert( userUnit:getSkills(), skill_back )
			end
			
			
			unit:getTraits().maa_available_power = unit:getTraits().maa_available_power + (userUnit:getTraits().maa_sharp_reactor or 0)
			for i, unit_skill in ipairs( userUnit:getSkills( ) ) do
				if unit_skill:getID() == "maa_sharp_reactor" then
					unit:getTraits().maa_reactor_level = unit_skill:getCurrentLevel( )
					table.remove( userUnit._skills, i)
				end
			end
		end
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		if unit:getTraits().installed then
			local userUnit = unit:getUnitOwner()
			if evType == simdefs.TRG_END_TURN then
				if userUnit and evData:isPC() then
					unit:getTraits().maa_can_power_down = true
					for _, child in ipairs( userUnit:getChildren() ) do
						if child:getTraits().maa_target_level then
							unit:getTraits().maa_power_allocated[ child:getTraits().maa_sharp_augment_type ] = child:getTraits().maa_target_level
							child:getTraits().maa_target_level = nil
						end
					end

					for _, unit_skill in ipairs( userUnit:getSkills( ) ) do
						if unit:getTraits().maa_power_allocated[ unit_skill:getID() ] then
							local diff = (unit:getTraits().maa_power_allocated[ unit_skill:getID() ] + 1) - unit_skill:getCurrentLevel()
							if diff > 0 then
								for i = 1, diff do
									unit_skill:levelUp( sim, userUnit )
								end
							elseif diff < 0 then
								for i = 1, math.abs(diff) do
									unit_skill:levelDown( sim, userUnit )
								end
							end
						end
					end
				end
			end
		end
	end,
}

return maa_sharp_reactor
