local util = include( "modules/util" )
local array = include( "modules/array" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local inventory = include("sim/inventory")

local maa_uninstall_weapon_mods =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.UNINSTALL_MODS.NAME,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
	
	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.UNINSTALL_MODS.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.UNINSTALL_MODS.DESC )
	end,
	
	alwaysShow = true,
	getName = function( self, sim, unit )
		return self.name
	end,

	onSpawnAbility = function( self, sim, unit )
		
	end,

	onDespawnAbility = function( self, sim, unit )
		
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		
		if #unit:getChildren() <= 0 then
			--no mods installed
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.UNINSTALL_MODS.NO_MODS
		end
		
		--can't remove mods when the weapon is on cooldown
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.ON_COOLDOWN
		end
		
		--can't remove mods when the weapon was used this turn
		if unit:getTraits().maa_weapon_used then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.INSTALL_MOD.USED
		end
		
		return true
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		
		if unit and userUnit then
			for i=#unit:getChildren(), 1, -1 do
				local weapon_mod = unit:getChildren()[i]
				local unitData = weapon_mod:getUnitData()
				if unitData.maa_doRemove then
					unitData.maa_doRemove( weapon_mod, unit )
				end
				if userUnit:getInventoryCount( ) >= 8 then --shitty to hardcode this :/
					local cell = sim:getCell( userUnit:getLocation() )
					unit:removeChild( weapon_mod )
					sim:warpUnit( item, cell )
				else
					unit:removeChild( weapon_mod )
					userUnit:addChild( weapon_mod )
				end
			end

			--refresh the rifle
			sim:triggerEvent( "TRG_MAA_REFRESH_MOD_WEAPON", nil )
		end
	end,
}

return maa_uninstall_weapon_mods
