local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")
local store = include( "sim/units/store" )
local simability = include( "sim/simability" )
local inventory = include("sim/inventory")
local cdefs = include( "client_defs" )

local maa_use_lock_explosive_door =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.USE_LOCK_EXPLOSIVE.NAME_DOOR,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flash_grenade_small.png",
	proxy = true,

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.USE_LOCK_EXPLOSIVE.NAME_DOOR, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.USE_LOCK_EXPLOSIVE.DESC_DOOR )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		--copy paste from doorMechanism
		local exits = {}
		local fromCell = sim:getCell( abilityUser:getLocation() )
		local x0, y0 = fromCell.x, fromCell.y
		local applyFn = abilityOwner:getTraits().applyFn
		if type(applyFn) == "string" then
			applyFn = simquery[ applyFn ] -- Sooo brutal, but due to circular includes, itemdefs cannot directly reference simquery functions
		end

		for dir, exit in pairs( fromCell.exits ) do
			if applyFn( exit ) then
				table.insert( exits, { x = fromCell.x, y = fromCell.y, dir = dir } )
			end
		end

		return targets.exitTarget( game, exits, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit, abilityUser, x, y, dir )
		-- Must have a user owner.
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		
		if x and y then
			local legal_door = false
			local ammo_cost = 1
			local cell = sim:getCell( x, y )
			if cell.exits[ dir ] and cell.exits[ dir ].keybits then
				for _, key in ipairs( unit:getTraits().maa_target_keybits ) do
					if cell.exits[ dir ].keybits == key[1] then
						legal_door = true
						ammo_cost = key[2]
						break
					end
				end
			end
			if not legal_door then
				return false
			end
			
			if unit:getTraits().ammo and unit:getTraits().ammo < ammo_cost then
				return false, STRINGS.UI.REASON.NO_AMMO
			end
		end

		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,unit:getTraits().cooldown)
		end

		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end
            
		return abilityutil.checkRequirements( unit, userUnit )
	end,

	executeAbility = function( self, sim, unit, userUnit, x0, y0, dir )
		local fromCell = sim:getCell( x0, y0 )
		assert( fromCell and dir )
		
		local exit = fromCell.exits[dir]
		local rdir = simquery.getReverseDirection( dir )
		local reverseExit = exit.cell.exits[ rdir ]
		local xd, yd = simquery.getDeltaFromDirection(dir) --place the animation on the other side of the door
		local x1 = x0 + xd
		local y1 = y0 + yd
		
		local ammo_cost = 1
		for _, key in ipairs( unit:getTraits().maa_target_keybits ) do
			if key[1] == fromCell.exits[dir].keybits then
				ammo_cost = key[2]
				break
			end
		end

		userUnit:resetAllAiming()
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR, { unitID = userUnit:getID(), facing = dir } )
		
		--inventory.useItem( sim, userUnit, unit )
		if unit:getTraits().ammo then
			unit:getTraits().ammo = math.max(unit:getTraits().ammo - ammo_cost, 0)
		end
		
		sim:emitSound( simdefs.SOUND_PLACE_TRAP, x0, y0, userUnit)	

		sim:processReactions( userUnit )
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR_PST, { unitID = userUnit:getID(), facing = dir } )
		sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )
		
		--sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, range = 1} )
		sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x1, y = y1, units = nil, range = 1} )
		--trick vanilla assert
		exit.locked = false
		reverseExit.locked = false
		sim:modifyExit( fromCell, dir, simdefs.EXITOP_BREAK_DOOR ) 
		
		sim:processReactions( userUnit )
		sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )
		
		if unit:getTraits().maa_alarm then
			sim:trackerAdvance( unit:getTraits().maa_alarm )
		end
	end,
}

return maa_use_lock_explosive_door

