local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local simquery = include("sim/simquery")
local store = include( "sim/units/store" )
local simability = include( "sim/simability" )
local inventory = include("sim/inventory")
local cdefs = include( "client_defs" )

local maa_use_lock_explosive_safe =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.USE_LOCK_EXPLOSIVE.NAME_SAFE,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flash_grenade_small.png",
	proxy = true,

	createToolTip = function( self, sim, unit )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.USE_LOCK_EXPLOSIVE.NAME_SAFE, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.USE_LOCK_EXPLOSIVE.DESC_SAFE )
	end,

	getName = function( self, sim, unit )
		return self.name
	end,

	isTarget = function( self, sim, userUnit, targetUnit )
		if not targetUnit:getTraits().safeUnit then
			return false
		end
		if targetUnit:getPlayerOwner() == userUnit:getPlayerOwner() or targetUnit:getTraits().mainframe_status ~= "active" or targetUnit:getTraits().open then
			return false
		end

		return true
	end,

	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		local cell = sim:getCell( abilityUser:getLocation() )
		local units = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if self:isTarget( sim, abilityUser, cellUnit ) then
					if simquery.canUnitReach( sim, abilityUser, exit.cell.x, exit.cell.y ) then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, abilityUser )
	end,

	canUseAbility = function( self, sim, unit, abilityUser, targetUnitID )
		-- Must have a user owner.
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end

		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN,unit:getTraits().cooldown)
		end

		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, util.sformat(STRINGS.UI.REASON.CHARGES)
		end
		
		if unit:getTraits().ammo and unit:getTraits().ammo < 1 then
			return false, STRINGS.UI.REASON.NO_AMMO
		end

		return abilityutil.checkRequirements( unit, userUnit )
	end,

	executeAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0, y0 = targetUnit:getLocation()
		local x1, y1 = userUnit:getLocation()
		local dir = simquery.getDirectionFromDelta( x0 - x1, y0 - y1 )

		userUnit:resetAllAiming()
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR, { unitID = userUnit:getID(), facing = dir } )

		--inventory.useItem( sim, userUnit, unit )
		if unit:getTraits().ammo then
			unit:getTraits().ammo = math.max(unit:getTraits().ammo - 1, 0)
		end

		sim:emitSound( simdefs.SOUND_PLACE_TRAP, x0, y0, userUnit)

		sim:processReactions( userUnit )
		sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR_PST, { unitID = userUnit:getID(), facing = dir } )
		sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )

		--sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, range = 1} )
		sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, units = nil, range = (unit:getTraits().soundRange or 1)} )
		sim:emitSound( {path = nil, range = (unit:getTraits().soundRange or 1)}, x0, y0 )
		sim:processReactions( userUnit )
		sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )

		targetUnit:getTraits().open = true
		targetUnit:getTraits().mainframe_status = "off"
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit })

		if unit:getTraits().maa_alarm then
			sim:trackerAdvance( unit:getTraits().maa_alarm )
		end
	end,
}

return maa_use_lock_explosive_safe
