local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local inventory = include("sim/inventory")


local maa_use_sniper_cloak =
{
	name = STRINGS.ABILITIES.CLOAK,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
	alwaysShow = true,
	getName = function( self, sim, unit )
		if unit:getTraits().cooldown == 0 then
			return STRINGS.ABILITIES.CLOAK_USE
		else
			if unit:getTraits().duration and unit:getTraits().duration > 0 then
				return string.format(STRINGS.ABILITIES.CLOAK_DURATION, unit:getTraits().duration)
			else
				return string.format(STRINGS.ABILITIES.CLOAK_COOLDOWN, unit:getTraits().cooldown)
			end
		end
	end,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self, unit )
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown)
		end

		if not unit:getTraits().cloakInVision then
			if sim:canPlayerSeeUnit( sim:getNPC(), userUnit ) then
				return false, STRINGS.UI.REASON.CANT_CLOAK_IN_VISION
			end
		end

		if unit:getTraits().maa_maxAlarm then
			local max_alarm = nil
			if type(unit:getTraits().maa_maxAlarm) == "number" then
				max_alarm = unit:getTraits().maa_maxAlarm
			else
				max_alarm = simdefs.TRACKER_MAXSTAGE
			end
			if sim:getTrackerStage() >= max_alarm then
				return false, util.sformat(STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.SNIPER_CLOAK.ALARM_HIGH, max_alarm)
			end
		end

		return abilityutil.checkRequirements( unit, userUnit )
	end,

	executeAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()

		userUnit:setInvisible(true, unit:getTraits().duration)
		userUnit:resetAllAiming()

		if unit:getTraits().cloakDistanceMax then
			userUnit:getTraits().cloakDistance = unit:getTraits().cloakDistanceMax
		end

		inventory.useItem( sim, userUnit, unit )

		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )

		sim:processReactions(userUnit)

		self.cloak_active = true --we need to keep track if the source of invis is this cloak, otherwise we would bust an other cloaks invis on max alarm
		self.cloaker = userUnit
	end,

	onTrigger = function( self, sim, evType, evData, unit )
		local userUnit = unit:getUnitOwner()
		if evType == simdefs.TRG_ALARM_STATE_CHANGE then
			local max_alarm = nil
			if type(unit:getTraits().maa_maxAlarm) == "number" then
				max_alarm = unit:getTraits().maa_maxAlarm
			else
				max_alarm = simdefs.TRACKER_MAXSTAGE
			end

			if userUnit and userUnit:getTraits().invisible and evData >= max_alarm and self.cloak_active then
				userUnit:setInvisible( false )
				self.cloak_active = false
				self.cloaker = nil
			end
		elseif evType == simdefs.TRG_START_TURN then
			if not userUnit or ( not userUnit:getTraits().invisible and self.cloak_active ) then
				self.cloak_active = false
				self.cloaker = nil
			end
			if evData:isNPC() then
				if self.cloak_active and unit:getTraits().cooldown then
					unit:getTraits().cooldown = unit:getTraits().cooldown + 1 --no cd reset while cloak is active
				end
			end
		elseif evType == simdefs.TRG_UNIT_WARP then
			if evData.unit == unit then
				if self.cloak_active and self.cloaker then
					self.cloak_active = false
					self.cloaker:setInvisible( false )
					self.cloaker = nil
				end
			end
		end
	end,
}
return maa_use_sniper_cloak
