local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local array = include( "modules/array" )
local use_stim = include( "sim/abilities/use_stim" )

local baseDoInjection = use_stim.doInjection
local baseOnTrigger = use_stim.onTrigger
local baseOnSpawAnbility = use_stim.onSpawnAbility
local baseOnDespawnAbility = use_stim.onDespawnAbility

local maa_use_turbo_stim = util.extend( use_stim )
{
	doInjection = function( self, sim, unit, userUnit, target )
		baseDoInjection( self, sim, unit, userUnit, target )

		if unit:getTraits().sprint_bonus_mp then
			target:getTraits().sprintBonus = (target:getTraits().sprintBonus or 0) + unit:getTraits().sprint_bonus_mp
			if not unit:getTraits().sneaking then
				target:addMP( unit:getTraits().sprint_bonus_mp )
			end
			self.applied_sprint_bonus = unit:getTraits().sprint_bonus_mp
		end
		if unit:getTraits().sprint_noise_incr then
			target:getTraits().dashSoundRange = (target:getTraits().dashSoundRange or 0) + unit:getTraits().sprint_noise_incr
			self.applied_sprint_noise = unit:getTraits().sprint_noise_incr
		end
		self.stim_target = target
	end,

	onTrigger = function( self, sim, evType, evData, unit, ... )
		if baseOnTrigger then
			baseOnTrigger( self, sim, evType, evData, unit, ... )
		end

		if evType == simdefs.TRG_END_TURN and self.stim_target then
			if self.applied_sprint_bonus then
				self.stim_target:getTraits().sprintBonus = self.stim_target:getTraits().sprintBonus - self.applied_sprint_bonus
				self.applied_sprint_bonus = nil
			end
			if self.applied_sprint_noise then
				self.stim_target:getTraits().dashSoundRange = self.stim_target:getTraits().dashSoundRange - self.applied_sprint_noise
				self.applied_sprint_noise = nil
			end
		end
	end,

	onSpawnAbility = function( self, sim, unit, ... )
		if baseOnSpawAnbility then
			baseOnSpawAnbility( self, sim, unit, ... )
		end
		if not sim._triggers[ simdefs.TRG_END_TURN ] or array.findIf( sim._triggers[ simdefs.TRG_END_TURN ], function( t ) return t._obj == self end ) == nil then
			sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
		end
	end,

	onDespawnAbility = function( self, sim, unit, ... )
		if baseOnDespawnAbility then
			baseOnDespawnAbility( self, sim, unit, ... )
		end
		sim:removeTrigger( simdefs.TRG_END_TURN, self, unit )
	end,
}

return maa_use_turbo_stim
