local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")

local maa_vampire_scan =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.DESC )
	end,


	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_dracul_small.png",
	proxy = true,
	alwaysShow = true,
	ghostable = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local cell = sim:getCell( userUnit:getLocation() )

		local units = {}


		for i,cellUnit in ipairs(cell.units) do
			if cellUnit:getTraits().iscorpse and not cellUnit:getTraits().wasDrone and not cellUnit:getTraits().neural_scanned then
				table.insert( units, cellUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()

		if not unit:getTraits().installed then
			return false
		end
		if not userUnit then
			return false
		end
		local vamp_gun = nil
		for _, child in ipairs( userUnit:getChildren() ) do
			if child:getTraits() and child:getTraits().maa_is_vampire_gun then
				vamp_gun = child
				break
			end
		end
		if not vamp_gun then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.NO_GUN
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown )
		end
		if unit:getTraits().ammo and unit:getTraits().ammo <= 0 then
			return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.NO_SCANS
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		target = sim:getUnit(target)

		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/transferData" )

		if target:getTraits().iscorpse then
			local vamp_gun = nil
			for _, child in ipairs( userUnit:getChildren() ) do
				if child:getTraits() and child:getTraits().maa_is_vampire_gun then
					vamp_gun = child
					break
				end
			end
			if vamp_gun then
				local potential_buffs = {}
				table.insert( potential_buffs, {"armorPiercing", 1, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_ARMOR} )
				table.insert( potential_buffs, {"maxAmmo", 1, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_AMMO} )
				if vamp_gun:getTraits().cooldownMax and vamp_gun:getTraits().cooldownMax > 0 then
					table.insert( potential_buffs, {"cooldownMax", -2, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_COOLDOWN} )
				end
				if vamp_gun:getTraits().maxAmmo and (vamp_gun:getTraits().maa_autoreload or 0 ) then
					if vamp_gun:getTraits().maxAmmo > (vamp_gun:getTraits().maa_autoreload or 0 ) then
						table.insert( potential_buffs, {"maa_autoreload", 1, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_AUTOLOAD} )
					end
				end
				if (vamp_gun:getTraits().maa_ap_refresh or 0 ) < vamp_gun:getTraits().maxAmmo then
					table.insert( potential_buffs, {"maa_ap_refresh", 1, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_AP} )
				end
				if (vamp_gun:getTraits().maa_mp_boost or 0 ) < 6 then
					table.insert( potential_buffs, {"maa_mp_boost", 3, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_MP} )
				end

				local buff_selected = potential_buffs[ sim:nextRand(1, #potential_buffs) ]
				if not vamp_gun:getTraits()[ buff_selected[1] ] then
					vamp_gun:getTraits()[ buff_selected[1] ] = 0
				end
				vamp_gun:getTraits()[ buff_selected[1] ] = vamp_gun:getTraits()[ buff_selected[1] ] + buff_selected[2]

				if vamp_gun:getTraits().cooldownMax and vamp_gun:getTraits().cooldownMax <= 0 then
					vamp_gun:getTraits().cooldownMax = nil
					vamp_gun:getTraits().cooldown = nil
				end

				local dialogParams =
				{

					STRINGS.DLC1.NEURAL_SCAN_MODAL_1,
					STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_MODAL_2,
					util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_SCAN.VAMPIRE_UPGRADE_MODAL_3, buff_selected[3] ),
					"gui/icons/item_icons/icon-item_generic_arm.png"
				}
				sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )

				if unit:getTraits().cooldownMax then
					unit:getTraits().cooldown = unit:getTraits().cooldownMax
				end
				if unit:getTraits().ammo then
					unit:getTraits().ammo = math.max( (unit:getTraits().ammo or 0) - 1, 0 )
				end
			end

			target:getTraits().neural_scanned = true
		end
	end,

	onSpawnAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		unit:getTraits().ammo = unit:getTraits().maxAmmo or 0
	end,
}

return maa_vampire_scan
