local util = include( "modules/util" )
local abilityutil = include( "sim/abilities/abilityutil" )
local simdefs = include("sim/simdefs")
local abilitydefs = include( "sim/abilitydefs" )

local BUFF_TABLE =
{
	[0] = function( sim, unit, userUnit)
		userUnit:addMPMax( -4 )
	end,
	[1] = function( sim, unit, userUnit)
		userUnit:addMPMax( 2 )
	end,
	[2] = function( sim, unit, userUnit)
		userUnit:addMPMax( 2 )
	end,
	[3] = function( sim, unit, userUnit)
		userUnit:addMPMax( 2 )
	end,
	[4] = function( sim, unit, userUnit)
		userUnit:addMPMax( 1 )
		userUnit:getTraits().apMax = (userUnit:getTraits().apMax or 0) + 1
	end,
	[5] = function( sim, unit, userUnit)
		userUnit:addMPMax( 1 )
		userUnit:getTraits().apMax = (unit:getTraits().apMax or 0) + 1
		unit:getTraits().addArmorPiercingRanged = (unit:getTraits().addArmorPiercingRanged or 0) + 1
	end,
	[6] = function( sim, unit, userUnit)
		userUnit:addMPMax( 1 )
		unit:getTraits().addArmorPiercingRanged = (unit:getTraits().addArmorPiercingRanged or 0) + 1
	end,
	[7] = function( sim, unit, userUnit)
		userUnit:addMPMax( 1 )
		unit:getTraits().addArmorPiercingRanged = (unit:getTraits().addArmorPiercingRanged or 0) + 1
	end,
	[8] = function( sim, unit, userUnit)
		userUnit:addMPMax( 2 )
	end,
}

local STAT_RESET_TABLE =
{
	[0] = 0,
	[1] = 0,
	[2] = 1,
	[3] = 2,
	[4] = 3,
	[5] = 3,
	[6] = 4,
	[7] = 4,
	[8] = 4,
}

local maa_vampire_thirst =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_THIRST.NAME,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local target = sim:getUnit(targetID)
		if target:isKO() then
			return abilityutil.formatToolTip(STRINGS.DLC1.NEURAL_SCAN, STRINGS.DLC1.NEURAL_SCAN_DESC_KO )
		else
			return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_THIRST.NAME, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.VAMPIRE_THIRST.DESC )
		end
	end,


	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_dracul_small.png",
	proxy = true,
	alwaysShow = true,
	ghostable = true,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		local cell = sim:getCell( userUnit:getLocation() )

		local units = {}


		for i, cellUnit in ipairs(cell.units) do
			if ((cellUnit:getTraits().iscorpse and not cellUnit:getTraits().wasDrone) or ((cellUnit:isKO() or cellUnit:isDead()) and cellUnit:getTraits().isGuard and not cellUnit:getTraits().isDrone)) and not cellUnit:getTraits().neural_scanned then
				table.insert( units, cellUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()

		if not unit:getTraits().installed then
			return false
		end
		if not userUnit then
			return false
		end

		return true
	end,

	dlcExecuteAbility = abilitydefs._abilities["neural_scan"].executeAbility,

	executeAbility = function( self, sim, unit, userUnit, targetID )
		target = sim:getUnit(targetID)

		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/transferData" )

		if target:getTraits().iscorpse then
			if unit:getTraits().maa_thirst_state < 8 then
				unit:getTraits().maa_thirst_state = unit:getTraits().maa_thirst_state + 1
			end
			--apply buff according to buff_table
			BUFF_TABLE[ unit:getTraits().maa_thirst_state ]( sim, unit, userUnit )

			target:getTraits().neural_scanned = true
		else
			--just call the dlc's neural scan execute for KOed guards
			self.dlcExecuteAbility( self, sim, unit, userUnit, targetID )
		end
	end,

	onSpawnAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if userUnit then
			--run down the buff_table
			for i=0, unit:getTraits().maa_thirst_state do
				BUFF_TABLE[ i ]( sim, unit, userUnit )
			end
		end
	end,

	onDespawnAbility = function( self, sim, unit )
		--I'm not tracking stats for the edge case the player drills the augment. If they do so, Draco keeps the buff/debuff until end of mission, who cares.

		--Reduce the state according to table
		unit:getTraits().maa_thirst_state = STAT_RESET_TABLE[unit:getTraits().maa_thirst_state] or 0
	end,
}

return maa_vampire_thirst
