local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local mathutil = include( "modules/mathutil" )
local abilityutil = include( "sim/abilities/abilityutil" )
local speechdefs = include("sim/speechdefs")

local maa_wireless_money_transfer =
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.WIRELESS_MONEY_TRANSFER.NAME,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_steal_small.png",
	proxy = true,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit(targetID)
		local desc = STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.WIRELESS_MONEY_TRANSFER.DESC
		if targetUnit then
			local credits = targetUnit:getTraits().credits or targetUnit:getTraits().cashOnHand or 0
			desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.WIRELESS_MONEY_TRANSFER.DESC_2, credits )
		end

		return abilityutil.formatToolTip( STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.WIRELESS_MONEY_TRANSFER.NAME, desc )
	end,

	isTarget = function( self, sim, userUnit, targetUnit )
		if (targetUnit:getTraits().credits or 0) > 0 then
			return true
		end
		if (targetUnit:getTraits().cashOnHand or 0) > 0 then
			return true
		end
		if targetUnit:getTraits().isGuard and not targetUnit:getTraits().isDrone then
			return true
		end
		if targetUnit:getTraits().safeUnit then
			return true
		end
		--for _, child in ipairs( targetUnit:getChildren() ) do
		--	if (child:getTraits().cashOnHand or 0) > 0 or (child:getTraits().credits or 0) > 0 then
		--		return true
		--	end
		--end
		return false
	end,

	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local units = {}
		local x0, y0 = userUnit:getLocation()
		for _, targetUnit in pairs(sim:getAllUnits()) do
			if self.isTarget( self, sim, userUnit, targetUnit) then

				local x1,y1 = targetUnit:getLocation()
				local distance =  mathutil.dist2d( x0, y0, x1, y1 )

    				if sim:canUnitSeeUnit( userUnit, targetUnit ) then
					table.insert( units, targetUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )

		if targetUnit then
			if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
				return false
			end

			if ( targetUnit:getTraits().credits or targetUnit:getTraits().cashOnHand or 0 ) <= 0 then
				return false, STRINGS.MORE_ARCHIVE_AGENTS.ABILITIES.WIRELESS_MONEY_TRANSFER.NO_CREDITS
			end

			if targetUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() and targetUnit:getTraits().mainframe_status == "active" and not targetUnit:getTraits().open then
				return false, STRINGS.ABILITIES.TOOLTIPS.UNLOCK_WITH_INCOGNITA
			end

			if targetUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() and targetUnit:getTraits().security_box and targetUnit:getTraits().security_box_locked==true then
				return false, STRINGS.UI.REASON.UNLOCK_WITH_VAULT_TERMINAL
			end
		end

		return true
	end,

	executeAbility = function( self, sim, unit, userUnit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )

		if targetUnit and userUnit then
			local x1, y1 = targetUnit:getLocation()
			local credits = targetUnit:getTraits().credits
			if credits and credits > 0 then
				targetUnit:getTraits().credits = nil
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/HUD/gameplay/gain_money")
				userUnit:getPlayerOwner():addCredits(credits, sim, x1, y1)
				sim:emitSpeech( targetUnit, speechdefs.EVENT_LOOT )
				sim._resultTable.credits_gained.safes = sim._resultTable.credits_gained.safes and sim._resultTable.credits_gained.safes + credits or credits
				sim:triggerEvent( simdefs.TRG_SAFE_LOOTED, { unit = userUnit, targetUnit = unit } )

				if targetUnit:getTraits().open ~= true then
					if not targetUnit:getTraits().noOpenAnim then
						targetUnit:getTraits().open = true
					end

					targetUnit:getTraits().mainframe_item = nil

					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )

					sim:getStats():incStat("safes_looted")
				end
			end

			local cashOnHand = targetUnit:getTraits().cashOnHand
			if cashOnHand and cashOnHand > 0 then
				local cash = math.floor( simquery.calculateCashOnHand( sim, targetUnit ) * (1 + (userUnit:getTraits().stealBonus or 0)) )
				sim._resultTable.credits_gained.pickpocket = sim._resultTable.credits_gained.pickpocket and sim._resultTable.credits_gained.pickpocket + cash or cash
				userUnit:getPlayerOwner():addCredits( cash, sim, x1, y1 )
				targetUnit:getTraits().cashOnHand = nil
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_CREDITS_PICKUP.path )
			end
			--for _, child in targetUnit:getChildren() do
			--	local cell = sim:getCell( child:getLocation() )
			--	if child:getTraits().cashOnHand and child:getTraits().cashOnHand > 0 then
			--		local cash = math.floor( simquery.calculateCashOnHand( sim, child ) * (1 + (userUnit:getTraits().stealBonus or 0)) )
			--		sim._resultTable.credits_gained.pickpocket = sim._resultTable.credits_gained.pickpocket and sim._resultTable.credits_gained.pickpocket + cash or cash
			--		unit:getPlayerOwner():addCredits( cash, sim, cell.x, cell.y )
			--		child:getTraits().cashOnHand = nil
			--		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_CREDITS_PICKUP.path )
			--	end
			--	if child:getTraits().credits and child:getTraits().credits > 0 then
			--		local cash = child:getTraits().credits
			--		sim._resultTable.credits_gained.safes = sim._resultTable.credits_gained.safes and sim._resultTable.credits_gained.safes + cash or cash
			--		userUnit:getPlayerOwner():addCredits( cash, sim, cell.x, cell.y )
			--		child:getTraits().credits = nil
			--		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_CREDITS_PICKUP.path )
			--	end
			--end
		end
	end,
}

return maa_wireless_money_transfer
