local util = include( "client_util" )
local array = include( "modules/array" )
local simdefs = include("sim/simdefs")
local cdefs = include( "client_defs" )
local simquery = include("sim/simquery")
local speechdefs = include( "sim/speechdefs" )
local mathutil = include( "modules/mathutil" )
local mainframe = include ( "sim/mainframe" )
local mainframe_common = include("sim/abilities/mainframe_common")

--PE Copy Pasta--
local nonbreakIceTooltip = include( "hud/tooltip_nonbreakice" )
local booster_tooltip = class(nonbreakIceTooltip)

function booster_tooltip:init( mainframePanel, targetWidget, unit, reason )
	util.tooltip.init( self, mainframePanel._hud._screen )
	self._targetWidget = targetWidget
	self.mainframePanel = mainframePanel

	local localPlayer = mainframePanel._hud._game:getLocalPlayer()
	local equippedProgram = nil
	if localPlayer then 
		equippedProgram = localPlayer:getEquippedProgram()
		if equippedProgram then
			local programWidget = mainframePanel._panel.binder.programsPanel:findWidget( equippedProgram:getID() )		
			if programWidget and programWidget:isVisible() then            	
				self._ux0, self._uy0 = programWidget.binder.btn:getAbsolutePosition()
				if equippedProgram:canUseAbility( mainframePanel._hud._game.simCore, localPlayer ) then
					self.programWidget = programWidget
					self.equippedProgram = equippedProgram
				end
			end
		end
	end

	local section = self:addSection()
	section:addLine( "<ttheader>"..util.sformat( "TARGET {1}", unit:getName() ).."</>" )

	if equippedProgram then
		if equippedProgram:getDef().credit_cost then
			section:addAbility( string.format(STRINGS.UI.TOOLTIPS.CURRENTLY_EQUIPPED, equippedProgram:getDef().name), util.sformat(equippedProgram:getDef().tipdesc, equippedProgram:getCpuCost(), math.max((equippedProgram.break_firewalls or 0) - (localPlayer:getTraits().firewallBreakPenalty or 0),1), equippedProgram:getDef().credit_cost),  "gui/icons/arrow_small.png" )
		else
			section:addAbility( string.format(STRINGS.UI.TOOLTIPS.CURRENTLY_EQUIPPED, equippedProgram:getDef().name), util.sformat(equippedProgram:getDef().tipdesc, equippedProgram:getCpuCost(), math.max((equippedProgram.break_firewalls or 0) - (localPlayer:getTraits().firewallBreakPenalty or 0),1)),  "gui/icons/arrow_small.png" )
		end
	end 

	if reason then
		section:addRequirement( reason )
	end
end


-------------------------------------
-------------------------------------

local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY
local REGION_LOCATIONS =
{
	NORTH_AFRICA = { STRINGS.MAP_NAMES.ALGIERS, STRINGS.MAP_NAMES.CASABLANCA, STRINGS.MAP_NAMES.CAIRO, STRINGS.MAP_NAMES.MONROVIA, },
	SOUTH_AFRICA = { STRINGS.MAP_NAMES.CAPE_TOWN, STRINGS.MAP_NAMES.ANTANANARIVO, STRINGS.MAP_NAMES.MOMBASA, STRINGS.MAP_NAMES.GABORONE, },
	SOUTH_AMERICA = { STRINGS.MAP_NAMES.BOGOTA, STRINGS.MAP_NAMES.LIMA, STRINGS.MAP_NAMES.SANTA_CRUZ, STRINGS.MAP_NAMES.BRASILIA, STRINGS.MAP_NAMES.PORTO_ALEGRE, STRINGS.MAP_NAMES.SANTA_ROSA, },
	
	CANADA = { STRINGS.MAP_NAMES.ANCHORAGE, STRINGS.MAP_NAMES.VANCOUVER, STRINGS.MAP_NAMES.THUNDER_BAY, STRINGS.MAP_NAMES.ST_JOHNS, },
	USA = { STRINGS.MAP_NAMES.SAN_FRANCISCO, STRINGS.MAP_NAMES.MEXICO_CITY, STRINGS.MAP_NAMES.KANSAS_CITY, STRINGS.MAP_NAMES.NEW_YORK, STRINGS.MAP_NAMES.ATLANTA, STRINGS.MAP_NAMES.HAVANA, },
	
	EUROPE = { STRINGS.MAP_NAMES.REYKJAVIK, STRINGS.MAP_NAMES.LONDON, STRINGS.MAP_NAMES.MADRID, STRINGS.MAP_NAMES.STOCKHOLM, STRINGS.MAP_NAMES.ROME, STRINGS.MAP_NAMES.BUDAPEST },
	RUSSIA = { STRINGS.MAP_NAMES.DUBAI, STRINGS.MAP_NAMES.BEIRUT, STRINGS.MAP_NAMES.ISTANBUL, STRINGS.MAP_NAMES.MOSCOW },
	AUSTRALIA = { STRINGS.MAP_NAMES.PERTH, STRINGS.MAP_NAMES.SYDNEY },
	INDIA = { STRINGS.MAP_NAMES.KABUL, STRINGS.MAP_NAMES.DHAKA, STRINGS.MAP_NAMES.NEW_DELHI, STRINGS.MAP_NAMES.COLOMBO },
	SOUTH_EAST_ASIA = { STRINGS.MAP_NAMES.BANGKOK, STRINGS.MAP_NAMES.SINGAPORE, STRINGS.MAP_NAMES.JAKARTA, STRINGS.MAP_NAMES.MANILA, STRINGS.MAP_NAMES.CHRISTCHURCH },
	CHINA = { STRINGS.MAP_NAMES.TOKYO, STRINGS.MAP_NAMES.SEOUL, STRINGS.MAP_NAMES.CHENGDU, STRINGS.MAP_NAMES.ASTANA, STRINGS.MAP_NAMES.HONG_KONG },
}

local mainframe_abilities =
{
	coit_blossom = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.BLOSSOM.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BLOSSOM.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BLOSSOM.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BLOSSOM.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BLOSSOM.TIP_DESC,
		
		icon = "gui/icons/programs/coit_blossom.png",
		icon_100 = "gui/icons/programs/coit_blossom_shop.png",
		
		cooldown = 0,
		maxCooldown = 1,
		passive = true,
		value = 550,
		
		coit_daemon_cooldown = true, --this is for manipulating the hud
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		onSpawnAbility = function( self, sim )
			sim:addTrigger( "TRG_COIT_TRUE_DAEMON_INSTALL", self )
			sim:addTrigger( "TRG_COIT_PROGRAM_PREUSED", self )
			sim:addTrigger( "TRG_COIT_PROGRAM_USED", self )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			
			self.checkDaemon = false
			self.intermediateDaemon = false
			
			if sim:getPC() then
				self.active = true
				sim:getTags().nextProgFree = (sim:getTags().nextProgFree or 0) + 1
				sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or 0) - 1
				self.coolDownMod = -2
			end
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_COIT_TRUE_DAEMON_INSTALL", self )
			sim:removeTrigger( "TRG_COIT_PROGRAM_PREUSED", self )
			sim:removeTrigger( "TRG_COIT_PROGRAM_USED", self )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			
			if self.active then
				sim:getTags().nextProgFree = math.max( (sim:getTags().nextProgFree or 1) - 1, 0)
				sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or -1) + 1
				self.coolDownMod = 0
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			--This is a very weird workaround. We use seeds free program flag and add a pwr cost modifier of 1 on top so we have 5 pwr reduction in total. Otherwise we would have to append every program -> no thanks
			if evType == "TRG_COIT_TRUE_DAEMON_INSTALL" and evData then
				if self.checkDaemon then
					self.intermediateDaemon = true
				end
				if self.cooldown and self.cooldown > 0 then
					self.cooldown = self.cooldown - 1
					
					if not self.active and self.cooldown <= 0 then
						self.active = true
						sim:getTags().nextProgFree = (sim:getTags().nextProgFree or 0) + 1
						sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or 0) - 1
						self.coolDownMod = -2
					end
				end
			elseif evType == "TRG_COIT_PROGRAM_PREUSED" then
				self.checkDaemon = true
			elseif evType == "TRG_COIT_PROGRAM_USED" then
				if self.active and not self.intermediateDaemon then
					self.active = false
					sim:getTags().nextProgFree = math.max( (sim:getTags().nextProgFree or 1) - 1, 0)
					sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or -1) + 1
					self.coolDownMod = 0
					
					self:setCooldown( sim )

					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, { txt=STRINGS.CORPORATE_ITEMS.PROGRAMS.BLOSSOM.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.icon } )
				end
				self.checkDaemon = false
				self.intermediateDaemon = false
			elseif evType == simdefs.TRG_START_TURN then --I hate it. But on ability spawn pc doesnt exist yet
				if evData:isPC() and sim:getTurnCount() == 0 and not self.active then
					self.active = true
					sim:getTags().nextProgFree = (sim:getTags().nextProgFree or 0) + 1
					sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or 0) - 1
					self.coolDownMod = -2
				end
			elseif evType == simdefs.TRG_END_TURN then
				--vanilla clears the cost modifier each turn, so we need to reapply
				if self.active then
					sim:getTags().nextProgFree = (sim:getTags().nextProgFree or 0) + 1
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()

			local sub = ""


			if self:getCpuCost() > 0 then 
				sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							
			end 

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			
			if self.maxCooldown and self.maxCooldown > 0  then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.DAEMON_COOLDOWN, self.maxCooldown )  )
			end

			if self.equipped then
				section:addLine( STRINGS.PROGRAMS.EQUIPPED, string.format( "" ))
			end

	   		section:addAbility( util.sformat(self.shortdesc,self.break_firewalls), util.sformat(self.desc,self.break_firewalls), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				local canUse, reason = self:canUseAbility( sim, player )
				if not canUse and reason then
					section:addRequirement( reason )
				end
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},	
	
	coit_faust_v2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.TIP_DESC,
		
		icon = "gui/icons/programs/coit_faust_v2.png",
		icon_100 = "gui/icons/programs/coit_faust_v2_shop.png",
		
		passive = true,
		value = 600,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		onSpawnAbility = function( self, sim )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			self.pwr_generated = 0
		end, 
		
		executeAbility = function( self, sim )
			local player = sim:getCurrentPlayer()			

			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			
			local pwr_before = player:getCpus()
			player:addCPUs( 2 )
			self.pwr_generated = self.pwr_generated + player:getCpus() - pwr_before
			
			if self.pwr_generated >= 6 then
				self.pwr_generated = self.pwr_generated - 6
				
				local programList = sim:getIcePrograms()
				local daemon = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))			

				sim:getNPC():addMainframeAbility( sim, daemon ) 
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and sim:getTurnCount() > 0 then
				self:executeAbility(sim)	
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = DEFAULT_ABILITY.onTooltip( self, screen, sim, player )
			local section = tooltip._sections[#tooltip._sections]
			
			if self.pwr_generated then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.FAUST_V2.TOOLTIP_PWR_GEN, 6 - self.pwr_generated )  )
			end
			
			return tooltip
		end,
	},
	
	coit_sacrifice = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.SACRIFICE.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SACRIFICE.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SACRIFICE.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SACRIFICE.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SACRIFICE.TIP_DESC,
		
		icon = "gui/icons/programs/coit_sacrifice.png",
		icon_100 = "gui/icons/programs/coit_sacrifice_shop.png",
		
		value = 350,
		
		PROGRAM_NO_BREAKER_LIST = 5,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),
		
		equip_program = true, 
		equipped = false, 
		targetingIcon = "gui/icons/action_icons/Action_icon_Small/icon-action_ko.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		
		pwr_gain = 4,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not targetUnit:getTraits().isAgent then 				
					return false
				end
				
				if targetUnit:getTraits().isGuard then 				
					return false
				end

				if targetUnit:getPlayerOwner() ~= sim:getPC() then
					return false
				end

				if not sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) then
					return false
				end

				if targetUnit:isGhost() then
					return false
				end

				if targetUnit:isDown() then
					return false
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			if targetUnit then
				targetUnit:setKO( sim, 2 )
				sim:getPC():addCPUs( self.pwr_gain, sim )
				local x0, y0 = targetUnit:getLocation()
				sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.PLUS_PWR, self.pwr_gain), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
			end
			
			self:setCooldown( sim )
			self:useCPUs( sim )
		end,
	},
	
	coit_claws = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.CLAWS.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CLAWS.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CLAWS.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CLAWS.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CLAWS.TIP_DESC,
		
		icon = "gui/icons/programs/coit_claws.png",
		icon_100 = "gui/icons/programs/coit_claws_shop.png",
		
		cpu_cost = 1,
		break_firewalls = 4, 
		equip_program = true, 
		equipped = false, 
		cooldown = 0,
		maxCooldown = 1,
		coit_daemon_cooldown = true, --this is for manipulating the hud
		
		value = 350,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),
			
		onSpawnAbility = function( self, sim )
			sim:addTrigger( "TRG_COIT_TRUE_DAEMON_INSTALL", self )
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_COIT_TRUE_DAEMON_INSTALL", self )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == "TRG_COIT_TRUE_DAEMON_INSTALL" and evData then
				if self.cooldown and self.cooldown > 0 then
					self.cooldown = self.cooldown - 1
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()

			local sub = ""


			if self:getCpuCost() > 0 then 
				sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							
			end 

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			
			if self.maxCooldown and self.maxCooldown > 0  then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.DAEMON_COOLDOWN, self.maxCooldown )  )
			end

			if self.equipped then
				section:addLine( STRINGS.PROGRAMS.EQUIPPED, string.format( "" ))
			end

	   		section:addAbility( util.sformat(self.shortdesc,self.break_firewalls), util.sformat(self.desc,self.break_firewalls), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				local canUse, reason = self:canUseAbility( sim, player )
				if not canUse and reason then
					section:addRequirement( reason )
				end
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},
	
	coit_brimstone_v2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.BRIMSTONE_V2.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BRIMSTONE_V2.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BRIMSTONE_V2.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BRIMSTONE_V2.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BRIMSTONE_V2.TIP_DESC,
		
		icon = "gui/icons/programs/coit_brimstone_v2.png",
		icon_100 = "gui/icons/programs/coit_brimstone_v2_shop.png",
		
		cpu_cost = 3,
		break_firewalls = 1, 
		equip_program = true, 
		equipped = false, 
		daemonReversalAdd = 0,
		maxReversalAdd = 40,
		
		value = 450,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )
			
			if targetUnit:getTraits().mainframe_ice and targetUnit:getTraits().mainframe_ice <= 0 then
				self.daemonReversalAdd = math.min( self.daemonReversalAdd + 5, self.maxReversalAdd )
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = DEFAULT_ABILITY.onTooltip( self, screen, sim, player )
			local section = tooltip._sections[#tooltip._sections]
			
			section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.BRIMSTONE_V2.TOOLTIP_REVERSAL_ODDS, self.daemonReversalAdd )  )
			
			return tooltip
		end,
	},
	
	coit_crowbar = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.CROWBAR.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CROWBAR.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CROWBAR.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CROWBAR.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CROWBAR.TIP_DESC,
		
		icon = "gui/icons/programs/coit_crowbar.png",
		icon_100 = "gui/icons/programs/coit_crowbar_shop.png",
		
		cpu_cost = 3,
		break_firewalls = 1, 
		equip_program = true, 
		equipped = false, 
		cooldown = 0,
		maxCooldown = 2,
		
		value = 500,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),

		executeAbility = function( self, sim, targetUnit )
			local targetDaemon = nil
			if targetUnit then			
				--temporarily remove the daemon on the device and put it back if it was not fully hacked
				targetDaemon = targetUnit:getTraits().mainframe_program
				targetUnit:getTraits().mainframe_program = nil
			end
			DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )
			
			if targetUnit and targetDaemon then
				if targetUnit:getTraits().mainframe_ice and targetUnit:getTraits().mainframe_ice > 0 then
					--device not hacked, put back daemon
					targetUnit:getTraits().mainframe_program = targetDaemon
				else
					--device got hacked, kill plastech possesser
					sim:dispatchEvent( simdefs.EV_KILL_DAEMON, {unit = targetUnit})
				
					if targetUnit:getTraits().daemonHost then
						sim:getUnit(targetUnit:getTraits().daemonHost):killUnit(sim)
						targetUnit:getTraits().daemonHost =nil
					end
				end
			end
		end,
	},
	
--	coit_shapeshift = util.extend( DEFAULT_ABILITY )
--	{
--		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.SHAPESHIFT.NAME,
--		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SHAPESHIFT.DESC,
--		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SHAPESHIFT.HUD_DESC,
--		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SHAPESHIFT.SHORT_DESC,
--		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SHAPESHIFT.TIP_DESC,
--		
--		icon = "gui/icons/programs/coit_shapeshift.png",
--		icon_100 = "gui/icons/programs/coit_shapeshift_shop.png",
--			
--		value = 450,
--		
--		PROGRAM_NO_BREAKER_LIST = 10,
--		
--		coit_locations = util.tconcat( REGION_LOCATIONS.NORTH_AFRICA, REGION_LOCATIONS.SOUTH_AFRICA, REGION_LOCATIONS.SOUTH_AMERICA ),
--		
--		
--	},
	
	coit_recycling = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.TIP_DESC,
		
		icon = "gui/icons/programs/coit_recycling.png",
		icon_100 = "gui/icons/programs/coit_recycling_shop.png",
		
		cooldown = 0,
		maxCooldown = 5,
		
		value = 1000,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		equip_program = true, 
		equipped = false, 
		targetingIcon = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			
			if sim:getPC() then
				sim:getPC():getTraits().coit_console_boot = ( sim:getPC():getTraits().coit_console_boot or 0 ) - 1
				sim:getPC():getTraits().coit_hijack_boost = ( sim:getPC():getTraits().coit_hijack_boost or 0 ) + 1
				self.initiated = true
			end
		end, 

		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			
			if self.initiated then
				sim:getPC():getTraits().coit_console_boot = sim:getPC():getTraits().coit_console_boot + 1
				sim:getPC():getTraits().coit_hijack_boost = sim:getPC():getTraits().coit_hijack_boost - 1
			end
		end,
		
		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			local result, reason = DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )
			if not result then
				return result, reason
			end
			if targetUnit then
				if not targetUnit:getTraits().mainframe_console then		
					return false
				end
				
				if targetUnit:getTraits().mainframe_status ~= "active" then 		
					return false, STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.BOOTING
				end
				
				--if not sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) then
				--	return false
				--end
				
				if (targetUnit:getTraits().cpus or 0) ~= 0 or targetUnit:getTraits().hijacked == false then
					return false, STRINGS.CORPORATE_ITEMS.PROGRAMS.RECYCLING.NOT_HIJACKED
				end
			end

			return true	
		end,

		executeAbility = function( self, sim, targetUnit )
			if targetUnit then
				targetUnit:getTraits().cpus = 1
				targetUnit:getTraits().hijacked = false
				targetUnit:getTraits().noPWRconvert = true
				targetUnit:processEMP( 1 )
				targetUnit:setPlayerOwner( sim:getNPC() )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
				
				self:setCooldown( sim )
				self:useCPUs( sim )
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			
			if evType == simdefs.TRG_START_TURN then
				if not self.initiated then
					sim:getPC():getTraits().coit_console_boot = ( sim:getPC():getTraits().coit_console_boot or 0 ) - 1
					sim:getPC():getTraits().coit_hijack_boost = ( sim:getPC():getTraits().coit_hijack_boost or 0 ) + 1
					self.initiated = true
				end
			end
		end,
	},
	
	coit_compass = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.COMPASS.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.COMPASS.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.COMPASS.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.COMPASS.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.COMPASS.TIP_DESC,
		
		icon = "gui/icons/programs/coit_compass.png",
		icon_100 = "gui/icons/programs/coit_compass_shop.png",
		
		passive = true,
		value = 700,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		executeAbility = function( self, sim )
			local player = sim:getCurrentPlayer()	
			local exploreCount = 0
			local totalCount = 0
			sim:forEachCell(
				function( cell )
					if not cell.isSolid then
						totalCount = totalCount + 1
						if player:getCell( cell.x, cell.y ) ~= nil then
							exploreCount = exploreCount + 1
						end
					end
				end )

			local explorePercent = exploreCount / totalCount
			
			local cpus = 0
			local plus = "+"
			if explorePercent >= 0.75 then
				cpus = 2
			elseif explorePercent >= 0.5 then
				cpus = 1
			elseif explorePercent < 0.25 then
				cpus = -1
				plus = ""
			end
			
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.COMPASS.WARNING, cpus, math.floor(explorePercent*100), plus ), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			
			player:addCPUs( cpus )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and sim:getTurnCount() > 0 then
				self:executeAbility(sim)	
			end
		end,
	},
	
	coit_knife = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE.TIP_DESC,
		
		icon = "gui/icons/programs/coit_knife.png",
		icon_100 = "gui/icons/programs/coit_knife_shop.png",
		
		cpu_cost = 1,
		break_firewalls = 3, 
		equip_program = true, 
		equipped = false, 
		cooldown = 0,
		maxCooldown = 7,
		value = 550,
		
		coit_program_cooldown = true, --this is for manipulating the hud
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		onSpawnAbility = function( self, sim )
			sim:addTrigger( "TRG_COIT_PROGRAM_USED", self )
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_COIT_PROGRAM_USED", self )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == "TRG_COIT_PROGRAM_USED" and evData.program ~= "coit_knife" and evData.program ~= "W93_gear_shift" then --yeah, yeah, the gear shift
				if self.cooldown and self.cooldown > 0 then
					self.cooldown = self.cooldown - 1
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()

			local sub = ""


			if self:getCpuCost() > 0 then 
				sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							
			end 

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			
			if self.maxCooldown and self.maxCooldown > 0  then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.PROGRAM_COOLDOWN, self.maxCooldown )  )
			end

			if self.equipped then
				section:addLine( STRINGS.PROGRAMS.EQUIPPED, string.format( "" ))
			end

	   		section:addAbility( util.sformat(self.shortdesc,self.break_firewalls), util.sformat(self.desc,self.break_firewalls), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				local canUse, reason = self:canUseAbility( sim, player )
				if not canUse and reason then
					section:addRequirement( reason )
				end
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},
	
	coit_knife_v2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE_V2.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE_V2.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE_V2.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE_V2.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.KNIFE_V2.TIP_DESC,
		
		icon = "gui/icons/programs/coit_knife_v2.png",
		icon_100 = "gui/icons/programs/coit_knife_v2_shop.png",
		
		cpu_cost = 3,
		break_firewalls = 5, 
		equip_program = true, 
		equipped = false, 
		cooldown = 0,
		maxCooldown = 5,
		value = 550,
		
		coit_program_cooldown = true, --this is for manipulating the hud
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		onSpawnAbility = function( self, sim )
			sim:addTrigger( "TRG_COIT_PROGRAM_USED", self )
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_COIT_PROGRAM_USED", self )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == "TRG_COIT_PROGRAM_USED" and evData.program ~= "coit_knife_v2" and evData.program ~= "W93_gear_shift" then --yeah, yeah, the gear shift
				if self.cooldown and self.cooldown > 0 then
					self.cooldown = self.cooldown - 1
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()

			local sub = ""


			if self:getCpuCost() > 0 then 
				sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							
			end 

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			
			if self.maxCooldown and self.maxCooldown > 0  then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.PROGRAM_COOLDOWN, self.maxCooldown )  )
			end

			if self.equipped then
				section:addLine( STRINGS.PROGRAMS.EQUIPPED, string.format( "" ))
			end

	   		section:addAbility( util.sformat(self.shortdesc,self.break_firewalls), util.sformat(self.desc,self.break_firewalls), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				local canUse, reason = self:canUseAbility( sim, player )
				if not canUse and reason then
					section:addRequirement( reason )
				end
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},
	
	coit_roller = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.ROLLER.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.ROLLER.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.ROLLER.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.ROLLER.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.ROLLER.TIP_DESC,
		
		icon = "gui/icons/programs/coit_roller.png",
		icon_100 = "gui/icons/programs/coit_roller_shop.png",
		
		cpu_cost = 3,
		break_firewalls = 2, 
		range = 1,
		value = 500,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( "TRG_COIT_PROGRAM_USED", self )
			self.baseRange = self.range
		end, 

		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( "TRG_COIT_PROGRAM_USED", self )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == "TRG_COIT_PROGRAM_USED" and evData.program ~= "W93_gear_shift" then
				self.range = self.range + 1
				
				if evData.program == self:getID() then
					self.range = self.baseRange
				end
			end
		end,
		
		acquireTargets = function( self, targets, game, sim, unit )
			local targetHandler = targets.areaCellTarget( game, self.range, sim, sim:getPC() )
			targetHandler:setUnitPredicate(
				function( u )
					return mainframe.canBreakIce( sim, u, self )
				end )
			targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
			return targetHandler
		end, 

		startTargeting = function( self, sim, unit, userUnit, targetCell )
			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
		end,

		getTargetUnits = function( self, sim, cellx, celly )
			local cells = simquery.rasterCircle( sim, cellx, celly, self.range )
			local units = {}
			for i, x, y in util.xypairs( cells ) do
				local cell = sim:getCell( x, y )
				if cell then
					for _, cellUnit in ipairs(cell.units) do
						if sim:getCurrentPlayer():getLastKnownCell( sim, x, y ) ~= nil then
							if mainframe.canBreakIce( sim, cellUnit, self ) then
								table.insert( units, cellUnit )
							end
						end
					end
				end
			end
			return units
		end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			
			local player = sim:getCurrentPlayer()
			local currentProgram = player:getEquippedProgram()

			player:equipProgram( sim, self:getID() )

			local cellx, celly = unpack(targetCell)
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )

			local targetUnits = self:getTargetUnits( sim, cellx, celly )

			sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = targetUnits, range = self.range } )		

			local daemonUnits = {}
			for _, unit in ipairs(targetUnits) do
				if unit:getTraits().mainframe_item or not sim:isVersion("0.17.15") then
					-- Keep track of daemons that *Would* be invoked, so that we can invoke them only after
					-- everything has been broken.
					local daemonProgram = unit:getTraits().mainframe_program

					unit:getTraits().mainframe_program = nil
					mainframe.breakIce( sim, unit, self.break_firewalls )
					unit:getTraits().mainframe_program = daemonProgram

					if daemonProgram and unit:getTraits().mainframe_ice <= 0 then
						table.insert( daemonUnits, unit )
					end
				end
			end

			self:useCPUs( sim )
			player:equipProgram( sim, currentProgram and currentProgram:getID() )

			for i, unit in ipairs( daemonUnits ) do
				mainframe.invokeDaemon( sim, unit )
			end

			self:setCooldown( sim ) 
		end,
	},
	
	coit_refraction_field = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.REFRACTION_FIELD.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.REFRACTION_FIELD.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.REFRACTION_FIELD.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.REFRACTION_FIELD.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.REFRACTION_FIELD.TIP_DESC,
		
		icon = "gui/icons/programs/coit_refraction_field.png",
		icon_100 = "gui/icons/programs/coit_refraction_field_shop.png",
		
		cpu_cost = 6,
		range = 5,
		cooldown = 0,
		maxCooldown = 10,
		value = 800,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		acquireTargets = function( self, targets, game, sim, unit )
			local targetHandler = targets.areaCellTarget( game, self.range, sim, sim:getPC() )
			targetHandler:setUnitPredicate(
				function( u )
					return u:getTraits().isAgent
				end )
			targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
			return targetHandler
		end, 

		startTargeting = function( self, sim, unit, userUnit, targetCell )
			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
		end,

		getTargetUnits = function( self, sim, cellx, celly )
			local cells = simquery.rasterCircle( sim, cellx, celly, self.range )
			local units = {}
			for i, x, y in util.xypairs( cells ) do
				local cell = sim:getCell( x, y )
				if cell then
					for _, cellUnit in ipairs(cell.units) do
						if cellUnit:getTraits().isAgent then
							table.insert( units, cellUnit )
						end
					end
				end
			end
			return units
		end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			
			local player = sim:getCurrentPlayer()
			local currentProgram = player:getEquippedProgram()

			player:equipProgram( sim, self:getID() )

			local cellx, celly = unpack(targetCell)
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )

			local targetUnits = self:getTargetUnits( sim, cellx, celly )

			sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = targetUnits, range = self.range } )		

			local guards_affected = false
			for _, unit in ipairs(targetUnits) do
				if unit:getTraits().isGuard and not unit:isDown() and unit:getPlayerOwner():isNPC() then
					guards_affected = true
					
					--alert all affected guards
					if not unit:getTraits().pacifist and not unit:isAlerted() then
						unit:setAlerted(true)
	    	        			local x0, y0 = unit:getLocation()
			    			sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_HUNTING, unit)
						sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = unit:getID() } )
						unit:getSim():emitSpeech( unit, speechdefs.HUNT_NOISE)
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
					end
				end
			end
			
			if not guards_affected then
				for _, unit in ipairs(targetUnits) do
					if unit:getPlayerOwner():isPC() then
						unit:setInvisible(true, 1)
						unit:resetAllAiming()
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit  } )
						sim:processReactions(unit)
					end
				end
			end

			self:useCPUs( sim )
			player:equipProgram( sim, currentProgram and currentProgram:getID() )
			self:setCooldown( sim ) 
		end,
	},
	
	coit_overdrive_v2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERDRIVE_V2.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERDRIVE_V2.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERDRIVE_V2.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERDRIVE_V2.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERDRIVE_V2.TIP_DESC,
		
		icon = "gui/icons/programs/coit_overdrive_v2.png",
		icon_100 = "gui/icons/programs/coit_overdrive_v2_shop.png",
		
		passive = true,
		value = 1100,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			
			if sim:getPC() then
				sim:getPC():getTraits().coit_cooldownMod = ( sim:getPC():getTraits().coit_cooldownMod or 0 ) - 1
				self.initiated = true
			end
		end, 

		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			if self.initiated then
				sim:getPC():getTraits().coit_cooldownMod = sim:getPC():getTraits().coit_cooldownMod + 1
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			
			if evType == simdefs.TRG_START_TURN then
				if not self.initiated then
					sim:getPC():getTraits().coit_cooldownMod = ( sim:getPC():getTraits().coit_cooldownMod or 0 ) - 1
					self.initiated = true
				end
			end
		end,
	},
	
--	coit_teleport = util.extend( DEFAULT_ABILITY )
--	{
--		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.TELEPORT.NAME,
--		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.TELEPORT.DESC,
--		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.TELEPORT.HUD_DESC,
--		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.TELEPORT.SHORT_DESC,
--		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.TELEPORT.TIP_DESC,
--		
--		icon = "gui/icons/programs/coit_teleport.png",
--		icon_100 = "gui/icons/programs/coit_teleport_shop.png",
--		
--		value = 800,
--		
--		PROGRAM_NO_BREAKER_LIST = 10,
--		
--		coit_locations = util.tconcat( REGION_LOCATIONS.CANADA, REGION_LOCATIONS.USA ),
--		
--		--I put this on hold. There are some things conceptionally difficult with this ability
--	},

	coit_soul_harvest = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOUL_HARVEST.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOUL_HARVEST.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOUL_HARVEST.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOUL_HARVEST.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOUL_HARVEST.TIP_DESC,
		
		icon = "gui/icons/programs/coit_soul_harvest.png",
		icon_100 = "gui/icons/programs/coit_soul_harvest_shop.png",
		
		cooldown = 0,
		maxCooldown = 4,
		pwr_gain = 3,
		
		value = 300,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.EUROPE, REGION_LOCATIONS.RUSSIA, REGION_LOCATIONS.AUSTRALIA ),
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end, 

		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		end,
		
		executeAbility = function( self, sim, targetUnit )
			local player = sim:getCurrentPlayer()
			player:addCPUs( self.pwr_gain )
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_gainCPU" )
			
			self:setCooldown( sim )
			self:useCPUs( sim )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_UNIT_KILLED then
				if evData.unit and evData.unit:getTraits().isGuard then
					self.maxCooldown = math.max( self.maxCooldown - 1, 1 )
				end
			end
		end,
	},
	
	coit_siren_v2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.SIREN_V2.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SIREN_V2.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SIREN_V2.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SIREN_V2.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SIREN_V2.TIP_DESC,
		
		icon = "gui/icons/programs/coit_siren_v2.png",
		icon_100 = "gui/icons/programs/coit_siren_v2_shop.png",
		
		pwr_gain = 2,
		passive = true,
		
		value = 500,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.EUROPE, REGION_LOCATIONS.RUSSIA, REGION_LOCATIONS.AUSTRALIA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		executeAbility = function( self, sim )
			local player = sim:getCurrentPlayer()			

			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.CORPORATE_ITEMS.PROGRAMS.SIREN_V2.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			
			player:addCPUs( self.pwr_gain )
			sim:trackerAdvance( 1 )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and sim:getTurnCount() > 0 then
				self:executeAbility(sim)	
			end
		end,
	},
	
	coit_nail = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.NAIL.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NAIL.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NAIL.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NAIL.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NAIL.TIP_DESC,
		
		icon = "gui/icons/programs/coit_nail.png",
		icon_100 = "gui/icons/programs/coit_nail_shop.png",
		
		cpu_cost = 3,
		break_firewalls = 1,
		equip_program = true, 
		equipped = false, 
		value = 350,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.EUROPE, REGION_LOCATIONS.RUSSIA, REGION_LOCATIONS.AUSTRALIA ),
		
		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )

			if targetUnit:getTraits().mainframe_ice and targetUnit:getTraits().mainframe_ice <= 0 then
				local player = sim:getCurrentPlayer()
				player:addCPUs( 2 )
			end
		end,
	},
	
	coit_static = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.STATIC.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.STATIC.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.STATIC.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.STATIC.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.STATIC.TIP_DESC,
		
		icon = "gui/icons/programs/coit_static.png",
		icon_100 = "gui/icons/programs/coit_static_shop.png",
		
		passive = true,
		value = 800,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.EUROPE, REGION_LOCATIONS.RUSSIA, REGION_LOCATIONS.AUSTRALIA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			
			if sim:getPC() then
				sim:getPC():getTraits().coit_melee_damage = ( sim:getPC():getTraits().coit_melee_damage or 0 ) + 2
				self.initiated = true
			end
		end, 
		
		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			if self.initiated then
				sim:getPC():getTraits().coit_melee_damage = sim:getPC():getTraits().coit_melee_damage - 2
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			
			if evType == simdefs.TRG_START_TURN then
				if not self.initiated then
					sim:getPC():getTraits().coit_melee_damage = ( sim:getPC():getTraits().coit_melee_damage or 0 ) + 2
					self.initiated = true
				end
			end
		end,
	},
	
	coit_bullet = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.BULLET.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BULLET.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BULLET.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BULLET.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.BULLET.TIP_DESC,
		
		icon = "gui/icons/programs/coit_bullet.png",
		icon_100 = "gui/icons/programs/coit_bullet_shop.png",
		
		cpu_cost = 1,
		break_firewalls = 1,
		cooldown = 0,
		maxCooldown = 1,
		equip_program = true, 
		equipped = false, 
		value = 550,
		
		coit_kill_cooldown = true, --this is for manipulating the hud
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.EUROPE, REGION_LOCATIONS.RUSSIA, REGION_LOCATIONS.AUSTRALIA ),
		
		onSpawnAbility = function( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		end,
		
		executeAbility = function( self, sim, targetUnit )
			self.break_firewalls = targetUnit:getTraits().mainframe_ice
			DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_UNIT_KILLED then
				if evData.unit and evData.unit:getTraits().isGuard and self.cooldown and self.cooldown > 0 then
					self.cooldown = math.max( self.cooldown - 1, 0 )
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()

			local sub = ""


			if self:getCpuCost() > 0 then 
				sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							
			end 

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			
			if self.maxCooldown and self.maxCooldown > 0  then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.KILL_COOLDOWN, self.maxCooldown )  )
			end

			if self.equipped then
				section:addLine( STRINGS.PROGRAMS.EQUIPPED, string.format( "" ))
			end

	   		section:addAbility( util.sformat(self.shortdesc,self.break_firewalls), util.sformat(self.desc,self.break_firewalls), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				local canUse, reason = self:canUseAbility( sim, player )
				if not canUse and reason then
					section:addRequirement( reason )
				end
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},
	
	coit_soulkiller = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOULKILLER.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOULKILLER.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOULKILLER.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOULKILLER.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.SOULKILLER.TIP_DESC,
		
		icon = "gui/icons/programs/coit_soulkiller.png",
		icon_100 = "gui/icons/programs/coit_soulkiller_shop.png",
		
		cpu_cost = 1,
		cooldown = 0,
		maxCooldown = 1,
		value = 550,
		
		coit_kill_cooldown = true, --this is for manipulating the hud
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.EUROPE, REGION_LOCATIONS.RUSSIA, REGION_LOCATIONS.AUSTRALIA ),
		
		onSpawnAbility = function( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		end,
		
		executeAbility = function( self, sim, targetUnit )
			self:setCooldown( sim )
			local player = sim:getCurrentPlayer()
			self:useCPUs( sim )

			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_wings_activate")

			for _, unit in pairs(sim:getAllUnits()) do
				if unit:getPlayerOwner() == player and simquery.isAgent( unit ) then
					unit:getTraits().mp = unit:getTraits().mp + 8
					local x0, y0 = unit:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FORMATS.PLUS_AP,8),x=x0,y=y0,color={r=1,g=1,b=1,a=1}} )		

					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )

					local x1, y1 = unit:getLocation()
					sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = self.name, x = x1, y = y1,color={r=255/255,g=255/255,b=51/255,a=1}} )	
				end
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_UNIT_KILLED then
				if evData.unit and evData.unit:getTraits().isGuard and self.cooldown and self.cooldown > 0 then
					self.cooldown = math.max( self.cooldown - 1, 0 )
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()

			local sub = ""


			if self:getCpuCost() > 0 then 
				sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							
			end 

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			
			if self.maxCooldown and self.maxCooldown > 0  then
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.KILL_COOLDOWN, self.maxCooldown )  )
			end

			if self.equipped then
				section:addLine( STRINGS.PROGRAMS.EQUIPPED, string.format( "" ))
			end

	   		section:addAbility( util.sformat(self.shortdesc,self.break_firewalls), util.sformat(self.desc,self.break_firewalls), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				local canUse, reason = self:canUseAbility( sim, player )
				if not canUse and reason then
					section:addRequirement( reason )
				end
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},
	
	coit_overclock = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERCLOCK.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERCLOCK.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERCLOCK.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERCLOCK.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERCLOCK.TIP_DESC,
		
		icon = "gui/icons/programs/coit_overclock.png",
		icon_100 = "gui/icons/programs/coit_overclock_shop.png",
		
		cooldown = 0,
		maxCooldown = 1,
		value = 850,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.INDIA, REGION_LOCATIONS.SOUTH_EAST_ASIA, REGION_LOCATIONS.CHINA ),
		
		executeAbility = function( self, sim, targetUnit )
			self:setCooldown( sim )
			self:useCPUs( sim )
			
			local player = sim:getCurrentPlayer()
			local cpus = player:getMaxCpus() - player:getCpus()
			player:addCPUs( cpus )
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_gainCPU" )
			
			self.IncognitaLock_turns = math.ceil( cpus / 5 )
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				if self.IncognitaLock_turns then
					if self.IncognitaLock_turns <= 0 and self.lock_applied then
						sim:setMainframeLockout( false )
						self.lock_applied = nil
					elseif self.IncognitaLock_turns > 0 and not self.lock_applied then
						sim:setMainframeLockout( true )
						self.lock_applied = true
					end
					if self.IncognitaLock_turns > 0 then
						self.IncognitaLock_turns = self.IncognitaLock_turns - 1
					end
				end
			end
		end,
		
		onTooltip = function( self, screen, sim, player )
			local tooltip = DEFAULT_ABILITY.onTooltip( self, screen, sim, player )
			local section = tooltip._sections[#tooltip._sections]
			
			if self.IncognitaLock_turns and self.IncognitaLock_turns > 0 then
				local turns = self.IncognitaLock_turns
				if self.lock_applied then
					turns = turns + 1
				end
				section:addLine( util.sformat( STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERCLOCK.TOOLTIP_LOCKDOWN, turns )  )
			end
			
			return tooltip
		end,
	},
	
	coit_volt_chain = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.VOLT_CHAIN.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.VOLT_CHAIN.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.VOLT_CHAIN.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.VOLT_CHAIN.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.VOLT_CHAIN.TIP_DESC,
		
		icon = "gui/icons/programs/coit_volt_chain.png",
		icon_100 = "gui/icons/programs/coit_volt_chain_shop.png",
		
		cpu_cost = 3,
		break_firewalls = 1,
		range = 3,
		equip_program = true, 
		equipped = false, 
		value = 600,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.INDIA, REGION_LOCATIONS.SOUTH_EAST_ASIA, REGION_LOCATIONS.CHINA ),
		
		executeAbility = function( self, sim, targetUnit )
			local player = sim:getCurrentPlayer()
			local buffed_range = self.range + math.floor( player:getCpus() / 5 )
			DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )
			
			local affected_targets = {}
			table.insert( affected_targets, targetUnit )
			local lastTarget = targetUnit
			
			--is it fine doing this dirty? this is some massive nested linear search
			for i=1, 100 do --caping at 100 jumps should be plenty
				local bestTarget = nil
				local bestRange = nil
				local x0, y0 = lastTarget:getLocation()
				for _, unit in pairs( sim:getAllUnits() ) do
					local x1, y1 = unit:getLocation()
					if x1 and y1 and not array.find(affected_targets, unit) and player:getLastKnownCell( sim, x1, y1 ) and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getPlayerOwner() ~= player then
						local distance = mathutil.dist2d( x0, y0, x1, y1 )
						if distance <= buffed_range and ( not bestRange or distance < bestRange ) then
							bestTarget = unit
							bestRange = distance
						end
					end
				end
				
				if not bestTarget then
					break
				end
				local firewallsToBreak = self.break_firewalls

				if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
					firewallsToBreak = math.max( firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1 )
				end

				if (firewallsToBreak or 0) > 0 then
					mainframe.breakIce( sim, bestTarget, firewallsToBreak )
				end
				lastTarget = bestTarget
				table.insert( affected_targets, lastTarget )
			end
		end,
	},
	
	coit_needle = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.NEEDLE.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NEEDLE.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NEEDLE.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NEEDLE.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.NEEDLE.TIP_DESC,
		
		icon = "gui/icons/programs/coit_needle.png",
		icon_100 = "gui/icons/programs/coit_needle_shop.png",
		
		cpu_cost = 2,
		break_firewalls = 1,
		cooldown = 0,
		maxCooldown = 1,
		equip_program = true, 
		equipped = false, 
		value = 850,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.INDIA, REGION_LOCATIONS.SOUTH_EAST_ASIA, REGION_LOCATIONS.CHINA ),
		
		executeAbility = function( self, sim, targetUnit )
			self.break_firewalls = targetUnit:getTraits().mainframe_ice
			DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )
			
			self.cooldown = self.cooldown + math.ceil( self.break_firewalls / 2 )
		end,
	},
	
	coit_overload = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERLOAD.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERLOAD.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERLOAD.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERLOAD.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.OVERLOAD.TIP_DESC,
		
		icon = "gui/icons/programs/coit_overload.png",
		icon_100 = "gui/icons/programs/coit_overload_shop.png",
		
		cpu_cost = 2,
		break_firewalls = 4,
		cooldown = 0,
		maxCooldown = 5,
		equip_program = true, 
		equipped = false, 
		value = 650,
		
		PROGRAM_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.INDIA, REGION_LOCATIONS.SOUTH_EAST_ASIA, REGION_LOCATIONS.CHINA ),
		
		executeAbility = function( self, sim, targetUnit )
			if (targetUnit:getTraits().mainframe_camera or targetUnit:getTraits().isDrone) and targetUnit:getTraits().mainframe_ice and targetUnit:getTraits().mainframe_ice <= self.break_firewalls then
				self:setCooldown( sim )
				self:useCPUs( sim )
				
				--targetUnit:onDamage( 1, sim )
				local player = sim:getCurrentPlayer()
				targetUnit:setFacing( simdefs.DIR_E )
				sim:damageUnit( targetUnit, 1, false, "emp", targetUnit )
			else
				DEFAULT_ABILITY.executeAbility( self, sim, targetUnit )
			end
		end,
	},
	
	coit_charge_v2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.CHARGE_V2.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CHARGE_V2.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CHARGE_V2.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CHARGE_V2.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CHARGE_V2.TIP_DESC,
		
		icon = "gui/icons/programs/coit_charge_v2.png",
		icon_100 = "gui/icons/programs/coit_charge_v2_shop.png",
		
		passive = true,
		value = 1500,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.INDIA, REGION_LOCATIONS.SOUTH_EAST_ASIA, REGION_LOCATIONS.CHINA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			
			self.buffedItems = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getTraits().usesCharges and unit:getTraits().charges and unit:getTraits().chargesMax then
					unit:getTraits().charges = unit:getTraits().charges + 1
					unit:getTraits().chargesMax = unit:getTraits().chargesMax + 1
					table.insert( self.buffedItems, unit )
				end
			end
			
			sim:getTags().coit_chargeBuff = self
		end, 

		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:getTags().coit_chargeBuff = nil
			
			for _, item in ipairs( self.buffedItems ) do
				if item then
					if unit:getTraits().charges then
						unit:getTraits().charges = math.max( unit:getTraits().charges - 1, 0 )
					end
					if unit:getTraits().chargesMax then
						unit:getTraits().chargesMax = math.max( unit:getTraits().chargesMax - 1, 0 )
					end
				end
			end
		end,
	},
	
	coit_conversion = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.CORPORATE_ITEMS.PROGRAMS.CONVERSION.NAME,
		desc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CONVERSION.DESC,
		huddesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CONVERSION.HUD_DESC,
		shortdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CONVERSION.SHORT_DESC,
		tipdesc = STRINGS.CORPORATE_ITEMS.PROGRAMS.CONVERSION.TIP_DESC,
		
		icon = "gui/icons/programs/coit_conversion.png",
		icon_100 = "gui/icons/programs/coit_conversion_shop.png",
		
		cooldown = 0,
		maxCooldown = 0,
		passive = true,
		value = 1500,
		
		PROGRAM_NO_BREAKER_LIST = 10,
		
		coit_locations = util.tconcat( REGION_LOCATIONS.INDIA, REGION_LOCATIONS.SOUTH_EAST_ASIA, REGION_LOCATIONS.CHINA ),
		
		canUseAbility = function( self, sim )
			return false 
		end,
		
		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			
			if sim:getPC() then
				sim:getPC():getTraits().coit_conversion = self
				self.initiated = true
			end
		end,
		
		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			
			if self.initiated then
				sim:getPC():getTraits().coit_conversion = nil
			end
		end,
		
		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			
			if evType == simdefs.TRG_START_TURN then
				if not self.initiated then
					sim:getPC():getTraits().coit_conversion = self
					self.initiated = true
				end
			end
		end,
	},
}

return mainframe_abilities
