local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local speechdefs = include("sim/speechdefs")
local mathutil = include( "modules/mathutil" )
local inventory = include("sim/inventory")

local oldMelee = include("sim/abilities/melee")
local oldCanUseAbility = oldMelee.canUseAbility
				
local melee = util.extend(oldMelee)
{
	canUseAbility = function( self, sim, unit, userUnit, targetID )
		local tazerUnit = simquery.getEquippedMelee( unit )
		if tazerUnit == nil or tazerUnit:getTraits().nonStandardMelee then
			return false
		end
		if unit:getTraits().mpAttackRestrict and unit:getTraits().mpAttackRestrict > unit:getMP() then
			if tazerUnit:getTraits().lethalMelee then
				return false, STRINGS.UI.COMBAT_PANEL_KILL, string.format(STRINGS.FUNCLIB.UI.COMBAT_PANEL_MP, unit:getTraits().mpAttackRestrict)
			else
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, string.format(STRINGS.FUNCLIB.UI.COMBAT_PANEL_MP, unit:getTraits().mpAttackRestrict)
			end
		end
		if unit:getTraits().pacifist then
			if (tazerUnit:getTraits().baseDamage and tazerUnit:getTraits().baseDamage > 0) or (tazerUnit:getTraits().damage and tazerUnit:getTraits().damage > 0)  then 
				if tazerUnit:getTraits().lethalMelee then
					return false, STRINGS.UI.COMBAT_PANEL_KILL, STRINGS.FUNCLIB.UI.COMBAT_PANEL_PACIFIST
				else
					return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.FUNCLIB.UI.COMBAT_PANEL_PACIFIST
				end
			end
		end
		if unit:countAugments( "augment_kpc_pistons" ) > 0 and unit:getMP() < 3 then 
			if tazerUnit:getTraits().lethalMelee then
				return false, STRINGS.UI.COMBAT_PANEL_KILL, STRINGS.FUNCLIB.UI.COMBAT_PANEL_NO_ATTACK
			else
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.FUNCLIB.UI.COMBAT_PANEL_NO_ATTACK
			end
		end
		if tazerUnit:getTraits().behindMelee and targetID then
			local target = sim:getUnit(targetID)
			local x0,y0 = unit:getLocation()
			local x1,y1 = target:getLocation()
			local distance = mathutil.dist2d( x0, y0, x1, y1 )
			local fc = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
			local dif = fc - target:getFacing()
			if distance > 1 then
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.UI.COMBAT_PANEL_NO_RANGE
			elseif (dif > 1 and dif < 7) or (dif > -7 and dif < -1 ) and tazerUnit:getTraits().behindMelee == "LENIENT" then
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.FUNCLIB.UI.COMBAT_PANEL_NO_BEHIND
			elseif dif ~= 0 and tazerUnit:getTraits().behindMelee == "STRICT" then
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.FUNCLIB.UI.COMBAT_PANEL_NO_BEHIND
			end
		end
		if tazerUnit:getTraits().restrictedUse then
			local userUnitAgentID = unit:getUnitData().agentID
			local canUse = false
			for i, set in pairs(tazerUnit:getTraits().restrictedUse ) do
				if set.agentID == userUnitAgentID then
					canUse=true
				end
			end

			if not canUse then
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.UI.REASON.RESTRUCTED_USE
			end				
		end

		if tazerUnit:getTraits().lethalMelee then
			unit:getTraits().ignorePinning = false
		end
		local ok, res, res2 = oldCanUseAbility( self, sim, unit, userUnit, targetID )
		unit:getTraits().ignorePinning = false
		
		if res == STRINGS.UI.COMBAT_PANEL_FAIL_KO and tazerUnit:getTraits().lethalMelee then
			res = STRINGS.UI.COMBAT_PANEL_KILL
		end
		
		return ok, res, res2
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local targetUnit = sim:getUnit(target)
		local tazer = simquery.getEquippedMelee( userUnit )
		local meleeDamage = simquery.calculateMeleeDamage(sim, tazer, targetUnit)
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		local distance = mathutil.dist2d( x0, y0, x1, y1 )

		if meleeDamage > 0 and not tazer:getTraits().lethalMelee then
			if tazer:getTraits().tagsTarget then
				targetUnit:getTraits().tagged = true
			end
		end

		if distance > 1 then 
			local cell = simquery.findNearestEmptyCell( sim, x0, y0, targetUnit )
			sim:warpUnit( targetUnit, cell )
			x1,y1 = targetUnit:getLocation()
		end

		local dir = simquery.getDirectionFromDelta(x1-x0,y1-y0)
		unit:setFacing(dir)

		if not unit:isValid() or unit:isDown() then
			return
		end
		unit:setInvisible( false )
		unit:setDisguise(false)
		if not unit:isValid() or unit:isDown() then
			return
		end

		sim:emitSpeech( unit, speechdefs.EVENT_ATTACK_MELEE )		

		local pinning, pinnee = simquery.isUnitPinning(sim, userUnit)
		if pinning and pinnee ~= targetUnit then
			pinning = false
		end
		local grapple = not userUnit:getPlayerOwner():isNPC() and meleeDamage > 0 and not pinning
		local lethal = false

		if tazer:getTraits().lethalMelee then
			grapple = false
			lethal = true
		end

		if grapple then
			targetUnit:getTraits().grappler = unit:getID()
			sim:refreshUnitLOS( targetUnit )
                	targetUnit:destroyTab()
		end

        	if unit and unit:isPC() and targetUnit and targetUnit:isKO() then
            		for _, ability in ipairs( sim:getNPC():getAbilities() ) do 
                		if ability:getID() == "transistordaemonolivia" then
                    			if targetUnit:getTraits().heartMonitor and not targetUnit:getTraits().improved_heart_monitor then
                        			targetUnit:getTraits().heartMonitor = "disabled"
                    			end
                    			unit:getTraits().ap = unit:getTraits().ap + 1
                    			lethal = true
                		end
            		end
        	end

		sim:dispatchEvent( simdefs.EV_UNIT_MELEE, { unit = unit, targetUnit = targetUnit, grapple = grapple, pinning = pinning, lethal = lethal} )	

		local facing = (unit:getFacing() + 4) % simdefs.DIR_MAX
		if grapple == true then
			targetUnit:getTraits().grappler = nil
			unit:setFacing(facing)
			targetUnit:setFacing(facing)
		end
		if lethal == true then
			unit:setFacing(facing)
		end
							
		if meleeDamage > 0 then	
			if sim:isVersion("0.17.12") and tazer:getTraits().armorPWRcost and targetUnit:getArmor() then
				local PWRcostPoints = returnPWRcostPoints(sim, userUnit, targetUnit)
				if PWRcostPoints > 0 then
					tazer:getPlayerOwner():addCPUs( -(tazer:getTraits().armorPWRcost * PWRcostPoints), sim, x1,y1)	
				end
			end

			if lethal == true then
				local dmgt =
				{
					damage = tazer:getTraits().baseDamage or 1,
					ko = false,
					noTargetAlert = false,
				}
				sim:hitUnit( unit, targetUnit, dmgt )
			else
				local koTime = math.max( 0, meleeDamage )
				if tazer:getTraits().drainsAllPWR then 
					local player = unit:getPlayerOwner()
					local cpus = player:getCpus()
					player:addCPUs( -cpus )
					local x0, y0 = unit:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR, cpus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1},} )
				end 
				if unit:isPC() and not pinning then
					local hadSight = targetUnit:getTraits().hasSight
					targetUnit:getTraits().hasSight = nil
					sim:warpUnit( targetUnit, sim:getCell(unit:getLocation()), targetUnit:getFacing())
					targetUnit:getTraits().hasSight = hadSight
				end
				targetUnit:setKO( sim, koTime )
				if pinning then
					sim:dispatchEvent( simdefs.EV_UNIT_HIT, {unit = targetUnit, taze=true} )
				end
				targetUnit:getTraits().lastHit = { x=x0, y=y0, unit=unit }

				if unit:getTraits().convertKOtoPWR then
					unit:getPlayerOwner():addCPUs(koTime, sim, x0,y0, 30 )						
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(unit:getTraits().convertKOtoPWR, koTime), x=x0,y=y0, color=cdefs.AUGMENT_TXT_COLOR} )
				end
			end

			if unit:countAugments( "augment_predictive_brawling" ) > 0 then
				local BRAWLING_BONUS = 6
				if unit:getPlayerOwner() ~= sim:getCurrentPlayer() then
					if not unit:getTraits().floatTxtQue then
						unit:getTraits().floatTxtQue = {}
					end
					table.insert(unit:getTraits().floatTxtQue,{txt=util.sformat(STRINGS.UI.FLY_TXT.PREDICTIVE_BRAWLING,BRAWLING_BONUS),color={r=1,g=1,b=41/255,a=1}})
				else
					sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.PREDICTIVE_BRAWLING,BRAWLING_BONUS),x=x0,y=y0,color={r=1,g=1,b=41/255,a=1}} ) 
				end
				unit:addMP( BRAWLING_BONUS )
			end

			if not sim:isVersion("0.17.12") and tazer:getTraits().armorPWRcost and targetUnit:getArmor() then
				tazer:getPlayerOwner():addCPUs( -(tazer:getTraits().armorPWRcost * targetUnit:getArmor()), sim, x1,y1)	
			end

			inventory.useItem( sim, userUnit, tazer )

			if unit:getTraits().tempMeleeBoost then
				unit:getTraits().tempMeleeBoost = 0
			end
		end

		if meleeDamage <= 0 or (grapple == false and lethal == false and pinning == false) then
			sim:dispatchEvent( simdefs.EV_UNIT_HIT, {unit = targetUnit, result = 0} )
		end
		if grapple then
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, {unit = unit} )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, {unit = targetUnit} )
		end

		sim:dispatchEvent( simdefs.EV_UNIT_MELEE, { finishMelee=true })
		sim:triggerEvent(simdefs.TRG_UNIT_HIT, {targetUnit=targetUnit, sourceUnit=unit, x=x1, y=y1, melee=true})

		if not sim:isVersion("0.17.12") and unit:getTraits().kinetic_capacitor_bonus_charged then
			unit:getTraits().kinetic_capacitor_bonus_charged = nil
		end

		if tazer:getTraits().soundRange then
			local soundRange = { path = nil, range = tazer:getTraits().soundRange }
			sim:emitSound( soundRange, x0, y0, unit )
		end

		unit:useAP( sim )

		if unit:countAugments( "augment_kpc_pistons" ) > 0 and not lethal then
			local BRAWLING_DRAWBACK = -3
			if unit:getPlayerOwner() ~= sim:getCurrentPlayer() then
				if not unit:getTraits().floatTxtQue then
					unit:getTraits().floatTxtQue = {}
				end
				table.insert(unit:getTraits().floatTxtQue,{txt=util.sformat(STRINGS.FUNCLIB.UI.BRAWLING_DRAWBACK, BRAWLING_DRAWBACK),color={r=216/255,g=37/255,b=19/255,a=1}})
			else
				sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FUNCLIB.UI.BRAWLING_DRAWBACK, BRAWLING_DRAWBACK),x=x0,y=y0,color={r=216/255,g=37/255,b=19/255,a=1}} ) 
			end
			unit:addMP( BRAWLING_DRAWBACK )
		end

		if tazer:getTraits().PWRbonus then
			sim:getPC():addCPUs( tazer:getTraits().PWRbonus, sim, x0,y0 )
			sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.PLUS_PWR, tazer:getTraits().PWRbonus), x=x0,y=y0, color={r=163/255,g=243/255,b=248/255,a=1}} )    
                end

		simquery.LEVER_resetStatus( sim, userUnit, "ANY", false )
	end,
}
return melee