local abilitydefs = include( "sim/abilitydefs" )
local melee = abilitydefs._abilities["melee"]
local simquery = include("sim/simquery")

local baseCanUseAbility = melee.canUseAbility
melee.canUseAbility = function( self, sim, unit, userUnit, targetID )
	local result, reason1, reason2 = baseCanUseAbility( self, sim, unit, userUnit, targetID )

	if result then
		--N-Umi 1 tazer
		local tazerUnit = simquery.getEquippedMelee( unit )
		if tazerUnit and targetID then
			if tazerUnit:getTraits().maa_ammo_per_armor then
				local targetUnit = sim:getUnit( targetID )
				local meleeDamage, armorPierce, armor = simquery.calculateDamageAndArmorForMelee(sim, tazerUnit, targetUnit)
				--how much armorPierce is needed from the weapon?
				local diff = (tazerUnit:getTraits().armorPiercing or 0) + armor - armorPierce
				if diff > 0 then
					--armorPierce from the weapon is needed
					if diff > (tazerUnit:getTraits().ammo or 0) then
						--not enough charges to pay for the armor
						return false, STRINGS.UI.COMBAT_PANEL_NEED_CHARGES, STRINGS.UI.COMBAT_PANEL_NEED_CHARGES_2
					end
				end
			end
		end
	end

	return result, reason1, reason2
end

local baseExecuteAbility = melee.executeAbility
melee.executeAbility = function( self, sim, unit, userUnit, target )
	local targetUnit = sim:getUnit(target)

	--Central 2 blacklisted disruptor
	local advance_tracker = 0
	local ammo_cost = 0
	local tazer = simquery.getEquippedMelee(unit)
	local meleeDamage,  armorPierce, armor = simquery.calculateDamageAndArmorForMelee(sim, tazer, targetUnit)
	if tazer:getTraits().maa_alarm_per_armor then
		local diff = armorPierce - armor

		advance_tracker = advance_tracker + math.max(tazer:getTraits().armorPiercing - diff, 0)
	end
	if tazer:getTraits().maa_ammo_per_armor then
		local diff = (tazer:getTraits().armorPiercing or 0) + armor - armorPierce
		if diff > 0 then
			ammo_cost = diff
		end
	end

	--Rush 2 tazer
	if tazer:getTraits().maa_alarm_on_use and targetUnit and not targetUnit:isKO() then --prevent player from gaining infinite AP with StimIV
		advance_tracker = advance_tracker + tazer:getTraits().maa_alarm_on_use
	end

	--check if the user can melee silently, if so check if they KOed a not-koed target and apply the silent_wakeup trait if so
	local apply_silent_melee = false
	if userUnit and userUnit:getTraits().maa_silent_melee then
		if targetUnit and not targetUnit:isKO() then
			apply_silent_melee = true
		end
	end

	baseExecuteAbility(  self, sim, unit, userUnit, target )

	if userUnit:getTraits().maa_melee_cooldown and tazer:getTraits().cooldown then
		tazer:getTraits().cooldown = tazer:getTraits().cooldown + userUnit:getTraits().maa_melee_cooldown
	end

	if apply_silent_melee and targetUnit:isKO() then
		targetUnit:getTraits().maa_silent_wakeup = true
	end

	if tazer:getTraits().ammo then
		tazer:getTraits().ammo = tazer:getTraits().ammo - ammo_cost
	end

	if advance_tracker > 0 then
		sim:trackerAdvance( advance_tracker )
	end
end
