local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local abilitydefs = include( "sim/abilitydefs" )
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local inventory = include("sim/inventory")

local oldMoveBody = abilitydefs.lookupAbility("moveBody")

local moveBody = util.extend(oldMoveBody)
{
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetID )
		local tooltip = util.tooltip( hud._screen )
		local section = tooltip:addSection()

		local target = sim:getUnit(targetID)
		if abilityOwner:getTraits().movingBody then
			section:addLine( "<ttheader>"..util.sformat( STRINGS.ABILITIES.MOVE_BODY_DROP_TIP , target:getName() ).."</>\n" .. STRINGS.ABILITIES.MOVE_BODY_DROP_DESC)
		else
			section:addLine( "<ttheader>"..util.sformat( STRINGS.ABILITIES.MOVE_BODY_DRAG_TIP , target:getName() ).."</>\n" .. STRINGS.ABILITIES.MOVE_BODY_DRAG_DESC)
		end
		local canUse, reason = self:canUseAbility( sim, abilityOwner, abilityUser, targetID )
		if not canUse and reason then
			section:addRequirement( reason )
		end

		if SCRIPT_PATHS.W93_FuncLibActive and SCRIPT_PATHS.W93_FuncLibChanges and abilityOwner:getTraits().movingBody and simquery.isUnitUnderOverwatch(abilityOwner) and abilityOwner:countAugments( "W93_augment_hostage_monitor" ) > 0 then 
			tooltip:addSection():addWarning( STRINGS.UI.DOOR_TRACKED, STRINGS.UI.DOOR_TRACKED_TT, "gui/hud3/hud3_tracking_icon_sm.png" , cdefs.COLOR_WATCHED_BOLD  )
		end 

		return tooltip
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local targetUnit = sim:getUnit(target)

		if unit:getTraits().movingBody == targetUnit then
			unit:getTraits().movingBody = nil
			targetUnit:setFacing(unit:getFacing() )
			unit:setFacing(simquery.getReverseDirection(unit:getFacing() ) )
			sim:dispatchEvent( simdefs.EV_UNIT_DROP_BODY, { unit = unit, targetUnit = targetUnit} )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, {unit = targetUnit} )
			if SCRIPT_PATHS.W93_FuncLibActive and SCRIPT_PATHS.W93_FuncLibChanges and unit:countAugments( "W93_augment_hostage_monitor" ) > 0 then
				sim:processReactions(userUnit)
			end
		else
			local x0,y0 = unit:getLocation()
			local x1,y1 = targetUnit:getLocation()
			local distance = mathutil.dist2d( x0, y0, x1, y1 )

			unit:setFacing(targetUnit:getFacing() )
			if not unit:isValid() then
				return
			end
				
			userUnit:getTraits().movingBody = targetUnit
			sim:dispatchEvent( simdefs.EV_UNIT_DRAG_BODY, { unit = unit, targetUnit = targetUnit} )

			local susp, aug = userUnit:countAugments( "augment_skeletal_suspension" )
			if susp > 0 and aug[1]:getTraits().cooldownMax and aug[1]:getTraits().cooldown <= 0 then
				inventory.useItem( sim, userUnit, aug[1] )
				userUnit:addMP( aug[1]:getTraits().mpRestored or 3 )
				sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.ITEMS.AUGMENTS.SKELETAL_SUSPENSION,x=x0,y=y0,color={r=1,g=1,b=41/255,a=1}} ) 
			end
		end
		simquery.LEVER_resetStatus( sim, userUnit, "ANY", false )
	end,
}
return moveBody