local mainframe_common = include("sim/abilities/mainframe_common")
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local util = include( "modules/util" )

local createIPenaltyDaemon = mainframe_common.createIPenaltyDaemon
local DEFAULT_IPENALTY_DAEMON = mainframe_common.DEFAULT_IPENALTY_DAEMON

local ipenalty_abilities =
{
	 sitrep_exhaustion = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.EXHAUSTION ) )
	 {
	 	icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
	 	random_weight = 10,
	
	 	onSpawnAbility = function( self, sim, player )
	 		sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
	 		sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or 0) + 1
	 	end,
	
		onDespawnAbility = function( self, sim )
	 		sim:getPC():getTraits().program_cost_modifier = (sim:getPC():getTraits().program_cost_modifier or 0) - 1
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.EXHAUSTION.NAME, STRINGS.SITREP.PENALTY_DAEMONS.EXHAUSTION.DESC)
		end,
	},

	sitrep_insomnia = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.INSOMNIA ) )
	{
		icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
		random_weight = 10,
	
		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			self.lock = false
		end,
	
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			if self.lock then
				sim:setMainframeLockout( false )
			end
		end,
	
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_END_TURN then
				if (sim:getTurnCount() - 1) % 4 == 0 then
					--end of odd NPC turn
					sim:setMainframeLockout( true )
					self.lock = true
				elseif (sim:getTurnCount() + 2) % 4 == 0 then
					--end of even PC turn
					sim:setMainframeLockout( false )
					self.lock = false
				end
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.INSOMNIA.NAME, STRINGS.SITREP.PENALTY_DAEMONS.INSOMNIA.DESC)
		end,
	},

	sitrep_power_leak = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.POWER_LEAK ) )
	{
		icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
		random_weight = 10,
	
		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
			local trigger = sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			trigger.priority = -1
		end,
	
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		end,
	
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				sim:getPC():addCPUs( -sim:getPC():getCpus() )
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.POWER_LEAK.NAME, STRINGS.SITREP.PENALTY_DAEMONS.POWER_LEAK.DESC)
		end,
	},

	sitrep_scream = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.SCREAM ) )
	{
		icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
		random_weight = 10,
	
		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
			sim:addTrigger( "TRG_SITREP_PROGRAM_USED", self )
			self.counter = 0
		end,
	
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_SITREP_PROGRAM_USED", self )
		end,
	
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == "TRG_SITREP_PROGRAM_USED" then
				self.counter = self.counter +1
				if self.counter >= 3 then
					local trackerCount = 1
					sim:trackerAdvance( trackerCount )
					self.counter = 0
				end
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.SCREAM.NAME, STRINGS.SITREP.PENALTY_DAEMONS.SCREAM.DESC)
		end,
	},

	sitrep_malware = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.MALWARE ) )
	{
		icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
		random_weight = 10,
	
		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
			sim:addTrigger( "TRG_SITREP_PROGRAM_USED", self )
			self.counter = 0
		end,
	
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_SITREP_PROGRAM_USED", self )
		end,
	
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == "TRG_SITREP_PROGRAM_USED" then
				self.counter = self.counter +1
				if self.counter >= 5 then
					local daemonList = sim:getIcePrograms()
					if daemonList:getTotalWeight() > 0 then
						local wt = sim:nextRand( 1, daemonList:getTotalWeight() )
						local daemon = daemonList:getChoice( wt )
						sim:getNPC():addMainframeAbility( sim, daemon, sim:getNPC(), nil )
						self.counter = 0
					end
				end
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.MALWARE.NAME, STRINGS.SITREP.PENALTY_DAEMONS.MALWARE.DESC)
		end,
	},

	sitrep_overheat = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.OVERHEAT ) )
	{
		icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
		random_weight = 10,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( "TRG_UNIT_HACKED", self )
			self.blocked = false
		end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( "TRG_UNIT_HACKED", self )
			if self.blocked then
				sim:setMainframeLockout( false )
			end
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_END_TURN then
				if self.blocked then
					self.blocked = false
					sim:setMainframeLockout( false )
				end
			elseif evType == "TRG_UNIT_HACKED" then
				if not self.blocked then
					sim:setMainframeLockout( true )
					self.blocked = true
				end
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.OVERHEAT.NAME, STRINGS.SITREP.PENALTY_DAEMONS.OVERHEAT.DESC)
		end,
	},

	sitrep_memory_leak = util.extend( createIPenaltyDaemon( STRINGS.SITREP.PENALTY_DAEMONS.MEMORY_LEAK ) )
	{
		icon = "gui/icons/daemons/sitrep_incognita_penalty.png",
		random_weight = 10,
	
		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )
			--self.locked_slots = {}
			local candidates = {}
			for index, program in ipairs( sim:getPC():getAbilities() )  do
				if not program.passive then
					table.insert( candidates, index )
				end
			end
			if #candidates >= 1 then
				local lucky_winner_index = candidates[sim:nextRand( 1, #candidates )]
				--table.insert( self.locked_slots, lucky_winner_index )
				sim:getPC():lockdownMainframeAbility( lucky_winner_index, "sitrep_memoryLeak" )
			end
		end,
	
		onDespawnAbility = function( self, sim )
			--for _, slot in ipairs(self.locked_slots) do
				sim:getPC():unlockAllMainframe( "sitrep_memoryLeak" )
			--end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.PENALTY_DAEMONS.MEMORY_LEAK.NAME, STRINGS.SITREP.PENALTY_DAEMONS.MEMORY_LEAK.DESC)
		end,
	},
}

return ipenalty_abilities
