local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local abilitydefs = include( "sim/abilitydefs" )

local oldObservePath = abilitydefs.lookupAbility("observePath")
local oldExecuteAbility = oldObservePath.executeAbility

local observePath = util.extend(oldObservePath)
{
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		if config.RECORD_MODE then
			return nil
		end
		
		local player = userUnit:getPlayerOwner()
		local oneConsole = nil
		local cell = sim:getCell( userUnit:getLocation() )
		local nearbyConsoles = {}
		for dir, exit in pairs(cell.exits) do
			for _, cellUnit in ipairs( exit.cell.units ) do
				if cellUnit:getTraits().mainframe_console or cellUnit:getTraits().open_secure_boxes then
					if simquery.canUnitReach( sim, userUnit, exit.cell.x, exit.cell.y ) and cellUnit:getTraits().mainframe_status ~= "off" and cellUnit:getPlayerOwner() == userUnit:getPlayerOwner() then
						table.insert( nearbyConsoles, cellUnit )
						oneConsole = true
						self.myConsoleDir = cellUnit:getFacing()
						self.myConsole = cellUnit
					end
				end
			end
		end
		
		local units = {}
		local x0, y0 = userUnit:getLocation()
		for _, targetUnit in pairs(sim:getAllUnits()) do
			if self:isTarget( userUnit, targetUnit ) then
				if ( userUnit:countAugments( "LEVER_augment_quantum_communicator" ) > 0 and sim:canPlayerSeeUnit( sim:getCurrentPlayer(), targetUnit ) ) or (sim:canUnitSeeUnit( userUnit, targetUnit ) and ((userUnit:getSkillLevel("stealth") and userUnit:getSkillLevel("stealth") > 1) or not sim:getParams().difficultyOptions.flav_skills)) then
					--or (sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) and oneConsole) then
					table.insert( units, targetUnit )
				else
					for _, cameraUnit in pairs(sim:getAllUnits()) do 
						if (oneConsole or not sim:getParams().difficultyOptions.flav_skills) and (cameraUnit:getTraits().mainframe_camera or cameraUnit:getTraits().isDrone) and cameraUnit:getPlayerOwner() == sim:getCurrentPlayer() then 
							if sim:canUnitSeeUnit(cameraUnit, targetUnit) then 
								table.insert( units, targetUnit )
								break
							end
						elseif cameraUnit:getTraits().peekID == userUnit:getID() and sim:canUnitSeeUnit( cameraUnit, targetUnit )
								and ((userUnit:getSkillLevel("stealth") and userUnit:getSkillLevel("stealth") > 1) or not sim:getParams().difficultyOptions.flav_skill) then -- or oneConsole
							table.insert( units, targetUnit )
							break
						end
					end
				end
			end
		end
		return targets.unitTarget( game, units, self, unit, userUnit )
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetID )
		if targetID then 
			local targetUnit = sim:getUnit( targetID )
                	if not self:isTarget( userUnit, targetUnit ) then
                    		return false
                	end

			if unit:getMP() < 1 and userUnit:countAugments( "LEVER_augment_distinctive_perceptors" ) == 0 then
				return false, string.format(STRINGS.UI.REASON.REQUIRES_AP,1)
			end

			if (sim:getParams().difficultyOptions.flav_skills and userUnit:countAugments( "LEVER_augment_quantum_communicator" ) == 0 and unit:getSkillLevel("stealth") and unit:getSkillLevel("stealth") == 1) then
				local cell = sim:getCell( userUnit:getLocation() )
				local reason
				for dir, exit in pairs(cell.exits) do
					for _, cellUnit in ipairs( exit.cell.units ) do
						if cellUnit:getTraits().mainframe_console or cellUnit:getTraits().open_secure_boxes then
							local dir = cellUnit:getFacing()
							local x0, y0 = cellUnit:getLocation()
							local x1, y1 = simquery.getDeltaFromDirection(dir)
							local consoleFront = sim:getCell( x0 + x1, y0 + y1 )
												
							if sim:getCell(userUnit:getLocation()) ~= consoleFront then
								reason = STRINGS.FUNCLIB.UI.REASON_CONSOLE_FRONT
							elseif cellUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() then
								reason = STRINGS.FUNCLIB.UI.REASON_NOT_HIJACKED
							else
								return true
							end						
						end
					end
				end
				return false, reason
			end
		end

		return true 
	end, 

	executeAbility = function( self, sim, unit, userUnit, target )
		if userUnit:countAugments( "LEVER_augment_distinctive_perceptors" ) > 0 then
			unit:addMP( 1 )
		end

		oldExecuteAbility( self, sim, unit, userUnit, target )
	end
}
return observePath