local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilitydefs = include( "sim/abilitydefs" )
local abilityutil = include( "sim/abilities/abilityutil" )

local oldOverwatch = abilitydefs.lookupAbility("overwatch")
local oldCanUseAbility = oldOverwatch.canUseAbility
local oldExecuteAbility = oldOverwatch.executeAbility

local overwatch = util.extend(oldOverwatch)
{
	canUseAbility = function( self, sim, unit )
		if unit:countAugments( "LEVER_augment_melee_specialist" ) > 0 or unit:countAugments( "LEVER_augment_melee_master" ) > 0 then
			return false, STRINGS.FUNCLIB.UI.REASON_MELEE_ONLY
		end

		if unit:getTraits().mpAttackRestrict and unit:getTraits().mpAttackRestrict > unit:getMP() then
			return false, string.format(STRINGS.FUNCLIB.UI.COMBAT_PANEL_MP, unit:getTraits().mpAttackRestrict)
		end

		local weaponUnit = simquery.getEquippedGun( unit )
		if not weaponUnit then
			return false, STRINGS.UI.REASON.NO_GUN
            	else
			if weaponUnit:getTraits().usesCharges then
				if weaponUnit:getTraits().charges < 1 then
					return false, STRINGS.UI.COMBAT_PANEL_NEED_CHARGES_2
				end
			end

			local canUse, reason = abilityutil.checkRequirements( weaponUnit, unit )
			if not canUse then
				return false, reason
			end
		end

		return oldCanUseAbility( self, sim, unit )
	end,

	executeAbility = function( self, sim, unit, ownerUnit, targetID )
		oldExecuteAbility( self, sim, unit, ownerUnit, targetID )

		simquery.LEVER_resetStatus( sim, ownerUnit, "ANY", false )
	end,
}
return overwatch
