local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local abilitydefs = include( "sim/abilitydefs" )
local abilityutil = include( "sim/abilities/abilityutil" )

local oldPrimeEmp = abilitydefs.lookupAbility("prime_emp")
local oldExecuteAbility = oldPrimeEmp.executeAbility

local emp_tooltip = class( abilityutil.hotkey_tooltip )

function emp_tooltip:init( hud, unit, abilityOwner, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
    	self._emp = abilityOwner
	self._range = abilityOwner:getTraits().range
end

function emp_tooltip:activate( screen )
	abilityutil.hotkey_tooltip.activate( self, screen )

	local x0, y0 = self._unit:getLocation()
	local coords = {}

	if self._emp:getTraits().flash_pack or self._emp:getTraits().breaching_charges then
		local cells = self._emp:getExplodeCells(x0, y0)
		for i,cell in ipairs(cells)do
			table.insert(coords,cell.x)
			table.insert(coords,cell.y)
		end
	else
		coords = simquery.rasterCircle( self._game.simCore, x0, y0, self._range )
	end
	self._hiliteID = self._game.boardRig:hiliteCells( coords, {0.2,0.2,0.2,0.2} )

    	local targets = self._emp:getTargets( x0, y0 )
    	for i, target in ipairs(targets) do
        	self._game.boardRig:getUnitRig( target:getID() ):getProp():setRenderFilter( cdefs.RENDER_FILTERS["focus_target"] )
    	end
end

function emp_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
	self._game.boardRig:unhiliteCells( self._hiliteID )
	self._hiliteID = nil

    	local targets = self._emp:getTargets( self._unit:getLocation() )
    	for i, target in ipairs(targets) do
        	self._game.boardRig:getUnitRig( target:getID() ):refreshRenderFilter()
    	end
end 

local prime_emp = util.extend(oldPrimeEmp)
{
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
		if abilityOwner:getTraits().flash_pack then
			return emp_tooltip( hud, abilityUser, abilityOwner, self, sim, abilityOwner, STRINGS.ABILITIES.PRIME_FLASH_PACK_DESC )
		elseif abilityOwner:getTraits().scan_pack then
			return emp_tooltip( hud, abilityUser, abilityOwner, self, sim, abilityOwner, STRINGS.FUNCLIB.UI.PRIME_SCAN_PACK_DESC )
		elseif abilityOwner:getTraits().breaching_charges then
			return emp_tooltip( hud, abilityUser, abilityOwner, self, sim, abilityOwner, STRINGS.FUNCLIB.UI.PRIME_BREACHING_CHARGES_DESC )
		else
			return emp_tooltip( hud, abilityUser, abilityOwner, self, sim, abilityOwner, STRINGS.ABILITIES.PRIME_EMP_DESC )
		end
	end,

	getName = function( self, sim, unit )
		if unit:getTraits().flash_pack then
			return STRINGS.ABILITIES.PRIME_FLASH_PACK
		elseif unit:getTraits().scan_pack then
			return STRINGS.FUNCLIB.UI.PRIME_SCAN_PACK
		elseif unit:getTraits().breaching_charges then
			return STRINGS.FUNCLIB.UI.PRIME_BREACHING_CHARGES
		else
			return STRINGS.ABILITIES.PRIME_EMP
		end
	end,

	executeAbility = function( self, sim, unit, userUnit )
		oldExecuteAbility( self, sim, unit, userUnit )

		simquery.LEVER_resetStatus( sim, userUnit, "ANY", false )
	end,
}
return prime_emp