local array = include( "modules/array" )
local util = include( "modules/util" )
local simquery = include("sim/simquery")
local abilityutil = include("sim/abilities/abilityutil")

local recharge =
{
	name = STRINGS.ABILITIES.RECHARGE,
	createToolTip = function( self,sim,unit,targetCell)
		userUnit = unit:getUnitOwner()
		if unit:getTraits().usesCharges then
			return abilityutil.formatToolTip( STRINGS.ABILITIES.RECHARGE,  util.sformat(STRINGS.FUNCLIB.UI.RECHARGE, userUnit:getTraits().pwr_bank or 0))
		else
			return abilityutil.formatToolTip( STRINGS.ABILITIES.RECHARGE,  util.sformat(STRINGS.FUNCLIB.UI.RECHARGE_ALT, userUnit:getTraits().pwr_bank or 0))
		end
	end,

	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",

	alwaysShow = true,

	getName = function( self, sim, unit )
		return  STRINGS.ABILITIES.RECHARGE
	end,

	canUseAbility = function( self, sim, itemUnit, unit )
		if itemUnit:getTraits().usesCharges then
			if itemUnit:getTraits().charges == itemUnit:getTraits().chargesMax then
				return false, STRINGS.UI.REASON.CHARGE_FULL
			end
		else
			-- Unit must have ammo in need of reloading.			
			if not itemUnit:getTraits().cooldown or itemUnit:getTraits().cooldown <= 0 then
				return false, STRINGS.UI.REASON.CHARGE_FULL
			end
		end

		-- Unit must have an ammo clip, or be flagged infinite-ammo.
		local hasAmmo = array.findIf( unit:getChildren(), 
			function( u ) 
				if u:getTraits().ammo_clip ~= nil and u:getTraits().ammo_clip > 0 then
					return u:getTraits().ammo_clip
				else
					return nil
				end
			end )
		if unit:getTraits().pwr_bank and unit:getTraits().pwr_bank == 1 then
			if unit:getPlayerOwner():getCpus() < 8 then
				return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
			else
				return true
			end
		elseif not hasAmmo then
			return false, STRINGS.UI.REASON.NO_POWER_PACK
		end
		return true
	end,
		
	executeAbility = function( self, sim, itemUnit, unit )
		abilityutil.doRecharge( sim, itemUnit )
		simquery.LEVER_resetStatus( sim, unit, "LEGIT", true )
	end
}
return recharge