local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilitydefs = include( "sim/abilitydefs" )
local abilityutil = include( "sim/abilities/abilityutil" )

local oldShootSingle = abilitydefs.lookupAbility("shootSingle")
local oldCanUseAbility = oldShootSingle.canUseAbility
local oldExecuteAbility = oldShootSingle.executeAbility

local shootSingle = util.extend(oldShootSingle)
{
	canUseAbility = function( self, sim, ownerUnit, unit, targetUnitID )
		if unit:countAugments( "LEVER_augment_melee_specialist" ) > 0 or unit:countAugments( "LEVER_augment_melee_master" ) > 0 then
			return false, STRINGS.FUNCLIB.UI.REASON_MELEE_ONLY
		end

		if unit:getTraits().mpAttackRestrict and unit:getTraits().mpAttackRestrict > unit:getMP() then
			return false, string.format(STRINGS.FUNCLIB.UI.COMBAT_PANEL_MP, unit:getTraits().mpAttackRestrict)
		end

		if not ownerUnit then
			return false, STRINGS.UI.REASON.NO_GUN		
		end

		local canUse, reason = abilityutil.checkRequirements( ownerUnit, unit )
		if not canUse then
			return false, reason
		end

		return oldCanUseAbility( self, sim, ownerUnit, unit, targetUnitID )
	end,

	executeAbility = function( self, sim, ownerUnit, unit, targetUnitID )
		oldExecuteAbility( self, sim, ownerUnit, unit, targetUnitID )

		simquery.LEVER_resetStatus( sim, unit, "ANY", false )
	end,
}
return shootSingle