local abilitydefs = include( "sim/abilitydefs" )
local shootSingle = abilitydefs._abilities["shootSingle"]
local simquery = include("sim/simquery")
local simdefs = include( "sim/simdefs" )

local baseCanUseAbility = shootSingle.canUseAbility
shootSingle.canUseAbility = function( self, sim, ownerUnit, unit, targetUnitID )
	local weaponUnit = simquery.getEquippedGun( unit )
	local targetUnit = sim:getUnit( targetUnitID )
	
	local ap_added = false
	if weaponUnit and weaponUnit:getTraits().coit_multishot and weaponUnit:getTraits().coit_multishot == "active" then
		unit:getTraits().ap = unit:getTraits().ap + 1
		ap_added = true
	end
	
	local temporary_trait_buffs = {}
	--trace weapons bonus against tagged targets
	if weaponUnit and weaponUnit:getTraits().coit_weapon_trace_buffs and targetUnit and targetUnit:getTraits().tagged then
		for _, buff in ipairs( weaponUnit:getTraits().coit_weapon_trace_buffs ) do
			if type(buff.value) == "boolean" and not weaponUnit:getTraits()[buff.trait] then
				weaponUnit:getTraits()[buff.trait] = buff.value
				table.insert( temporary_trait_buffs, {trait = buff.trait, value = buff.value } )
			elseif type(buff.value) == "number" then
				weaponUnit:getTraits()[buff.trait] = (weaponUnit:getTraits()[buff.trait] or 0) + buff.value
				table.insert( temporary_trait_buffs, {trait = buff.trait, value = buff.value } )
			end
		end
	end

	local result, reason = baseCanUseAbility( self, sim, ownerUnit, unit, targetUnitID )
	
	--remove temporary buffs again
	for _, buff in ipairs( temporary_trait_buffs ) do
		if type(buff.value) == "boolean" then
			weaponUnit:getTraits()[buff.trait] = nil
		elseif type(buff.value) == "number" then
			assert( weaponUnit:getTraits()[buff.trait] )
			weaponUnit:getTraits()[buff.trait] = weaponUnit:getTraits()[buff.trait] - buff.value
		end
	end
	
	if ap_added then
		unit:getTraits().ap = unit:getTraits().ap - 1
	end
	
	if result then
		if targetUnit and weaponUnit:getTraits().coit_can_mark then
			if targetUnit:getArmor() <= 0 then
				return false
			end
		end
	end
	
	--if result then
	--	if weaponUnit:getTraits().coit_ammo_cost_per_armor then
	--		local targetUnit = sim:getUnit( targetUnitID )
	--		if targetUnit then
	--			local shot = simquery.calculateShotSuccess( sim, unit, targetUnit, weaponUnit )
	--			local target_armor = 0
	--			local bonus_pierce = (shot.armorPiercing or weaponUnit:getTraits().armorPiercing) - weaponUnit:getTraits().armorPiercing
	--			
	--			if targetUnit:getArmor() then
	--			    	target_armor = target_armor + targetUnit:getArmor()
	--		    	end
	--			if weaponUnit:getTraits().ignoreArmor then
	--				target_armor = 0
	--			else
	--		    		target_armor = math.max( 0, target_armor - unit:countArmorPiercingUpgrades("addArmorPiercingRanged") )
	--			end
	--			
	--			local pierce_needed = target_armor - bonus_pierce
	--			local ammo_cost = 1
	--			if pierce_needed > 0 then
	--				ammo_cost = math.max( math.ceil( pierce_needed * weaponUnit:getTraits().coit_ammo_cost_per_armor ), 1 )
	--			end
	--			if weaponUnit:getTraits().ammo and weaponUnit:getTraits().ammo < ammo_cost then
	--				return false
	--			end
	--		end
	--	end
	--end
	
	return result, reason
end

local baseExecuteAbility = shootSingle.executeAbility
shootSingle.executeAbility = function( self, sim, ownerUnit, unit, targetUnitID )
	if ownerUnit:getTraits().slot ~= "gun" then
		ownerUnit = simquery.getEquippedGun( ownerUnit )
		assert( ownerUnit )
	end
	local weaponUnit = ownerUnit
	local targetUnit = sim:getUnit( targetUnitID )
	
	local temporary_trait_buffs = {}
	--trace weapons bonus against tagged targets
	if weaponUnit and weaponUnit:getTraits().coit_weapon_trace_buffs and targetUnit and targetUnit:getTraits().tagged then
		for _, buff in ipairs( weaponUnit:getTraits().coit_weapon_trace_buffs ) do
			if type(buff.value) == "boolean" and not weaponUnit:getTraits()[buff.trait] then
				weaponUnit:getTraits()[buff.trait] = buff.value
				table.insert( temporary_trait_buffs, {trait = buff.trait, value = buff.value } )
			elseif type(buff.value) == "number" then
				weaponUnit:getTraits()[buff.trait] = (weaponUnit:getTraits()[buff.trait] or 0) + buff.value
				table.insert( temporary_trait_buffs, {trait = buff.trait, value = buff.value } )
			end
		end
		--weaponUnit:getTraits().coit_temporary_buffs_applied = true
	end
	
	local ammo_cost = 0
	
	if weaponUnit:getTraits().coit_ammo_cost_per_armor then
		local shot = simquery.calculateShotSuccess( sim, unit, targetUnit, weaponUnit )
		local target_armor = 0
		local bonus_pierce = (shot.armorPiercing or weaponUnit:getTraits().armorPiercing or 0) - (weaponUnit:getTraits().armorPiercing or 0)
		
		if targetUnit:getArmor() then
		    	target_armor = target_armor + targetUnit:getArmor()
	    	end
		if weaponUnit:getTraits().ignoreArmor then
			target_armor = 0
		else
	    		target_armor = math.max( 0, target_armor - unit:countArmorPiercingUpgrades("addArmorPiercingRanged") )
		end
		
		local pierce_needed = target_armor - bonus_pierce
		ammo_cost = 1
		if pierce_needed > 0 then
			ammo_cost = math.max( math.ceil( pierce_needed * weaponUnit:getTraits().coit_ammo_cost_per_armor ), 1 )
		end
		ammo_cost = ammo_cost - 1 --we already pay one with inventory.useItem
	end
	
	baseExecuteAbility( self, sim, ownerUnit, unit, targetUnitID )
	
	--remove temporary buffs again
	for _, buff in ipairs( temporary_trait_buffs ) do
		if type(buff.value) == "boolean" then
			weaponUnit:getTraits()[buff.trait] = nil
		elseif type(buff.value) == "number" then
			assert( weaponUnit:getTraits()[buff.trait] )
			weaponUnit:getTraits()[buff.trait] = weaponUnit:getTraits()[buff.trait] - buff.value
		end
	end
	--weaponUnit:getTraits().coit_temporary_buffs_applied = nil
	
	if targetUnit and weaponUnit:getTraits().coit_can_mark then
		if not targetUnit:getTraits().coit_armor_mark then
			targetUnit:getTraits().coit_armor_mark = {}
		end
		table.insert( targetUnit:getTraits().coit_armor_mark, sim:getTurnCount() )
		
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
	end
	
	if ammo_cost > 0 then
		weaponUnit:getTraits().ammo = weaponUnit:getTraits().ammo - ammo_cost
	end
end
