local mainframe_common = include("sim/abilities/mainframe_common")
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local serverdefs = include( "modules/serverdefs" )

local createGuardDaemon = mainframe_common.createGuardDaemon
local DEFAULT_GUARD_DAEMON = mainframe_common.DEFAULT_GUARD_DAEMON

local guard_abilities =
{
	sitrep_console_alarm = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.CONSOLE_ALARM ) )
	{
		icon = "gui/icons/daemons/sitrep_tracer.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			sim:addTrigger( simdefs.TRG_UNIT_HIJACKED, self )
		end,
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_HIJACKED, self )
		end,
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_HIJACKED and evData.unit:getTraits().mainframe_console then
				--we put the investigation point on the console and not the console user. could make for fun internationale distractions...
				local x0, y0 = evData.unit:getLocation()
				sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_ALARMEDSAFE, evData.unit)
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.SITREP.SITREPS.CONSOLE_ALARM.WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )

				--also raise the alarm
				local trackerCount = 1
				sim:trackerAdvance( trackerCount )
			else
				DEFAULT_GUARD_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			end
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.CONSOLE_ALARM.NAME, STRINGS.SITREP.SITREPS.CONSOLE_ALARM.DESC)
		end,
	},

	sitrep_turrets_360 = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.TURRETS_360 ) )
	{
		icon = "gui/icons/daemons/sitrep_widescreen.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			local corpData = serverdefs.getCorpData( situation )
			if corpData.region == "europe" then
				return true
			end
			
			return false
		end,

		onSpawnAbility = function( self, sim, player )
			for id, unit in pairs( sim:getAllUnits() ) do
				if unit:getTraits().mainframe_turret then
					unit:getTraits().LOSarc = 2 * math.pi
					sim:refreshUnitLOS( unit )
				end
			end
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.TURRETS_360.NAME, STRINGS.SITREP.SITREPS.TURRETS_360.DESC)
		end,
	},

	sitrep_guard_vision = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.GUARD_VISION ) )
	{
		icon = "gui/icons/daemons/sitrep_eagle.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSimParams = function( self, params )
			if not params.onUnitCreation then
				params.onUnitCreation = {}
			end
			fn = function (sim, unit)
				if unit:getPlayerOwner() == sim:getNPC() then
					if unit:getTraits().isGuard and unit:getTraits().hasSight then
						if unit:getTraits().LOSrange and unit:getTraits().LOSrange > 0 then
							unit:getTraits().LOSrange = unit:getTraits().LOSrange + 2
						end
						if unit:getTraits().LOSperipheralArc and unit:getTraits().LOSperipheralArc > 0 then
							unit:getTraits().LOSperipheralArc = unit:getTraits().LOSperipheralArc + math.pi/4
						end
						if unit:getTraits().LOSarc and unit:getTraits().LOSarc > 0 then
							unit:getTraits().LOSarc = unit:getTraits().LOSarc + math.pi/4
						end
					end
				end
			end
			table.insert(params.onUnitCreation, fn)
		end,
		onSpawnAbility = function( self, sim, player )
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.GUARD_VISION.NAME, STRINGS.SITREP.SITREPS.GUARD_VISION.DESC)
		end,
	},

	sitrep_safe_fortify = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.SAFE_FORTIFY ) )
	{
		icon = "gui/icons/daemons/sitrep_titanium_case.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			for i, unit in pairs(sim:getAllUnits()) do
				if unit:getTraits().safeUnit and unit:getPlayerOwner() ~= sim:getPC() and not unit:getTraits().open and unit:getTraits().mainframe_ice then
					unit:increaseIce( sim, 2 )
					unit:getTraits().magnetic_reinforcement = true
				end
			end
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.SAFE_FORTIFY.NAME, STRINGS.SITREP.SITREPS.SAFE_FORTIFY.DESC)
		end,
	},

	sitrep_turtle = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.TURTLE ) )
	{
		icon = "gui/icons/daemons/sitrep_turtle.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			local targets = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getPlayerOwner() ~= sim:getPC() and unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getTraits().mainframe_status ~= "off" then
					table.insert(targets,unit)
				end
			end
			for i=1, 3 do
				if #targets > 0 then
					local target = table.remove( targets, math.floor(sim:nextRand(#targets)) )
					target:increaseIce( sim, target:getTraits().mainframe_ice )
				else
					break
				end
			end
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.TURTLE.NAME, STRINGS.SITREP.SITREPS.TURTLE.DESC)
		end,
	},

	sitrep_camera_failsafe = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.CAMERA_FAILSAFE ) )
	{
		icon = "gui/icons/daemons/sitrep_sharp_eye.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			local corpData = serverdefs.getCorpData( situation )
			if corpData.region == "europe" then
				return false
			end
			
			return true
		end,
		
		onSpawnAbility = function( self, sim, player )
			for id, unit in pairs( sim:getAllUnits() ) do
				if unit:getTraits().mainframe_camera then
					unit:giveAbility( "sitrep_failsafe" )
				end
			end
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.CAMERA_FAILSAFE.NAME, STRINGS.SITREP.SITREPS.CAMERA_FAILSAFE.DESC)
		end,
	},

	sitrep_chronjob_daemon = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.CHRONJOB ) )
	{
		icon = "gui/icons/daemons/sitrep_chronjob.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			local targets = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getPlayerOwner() ~= sim:getPC() and unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getTraits().mainframe_status ~= "off" and not unit:getTraits().isDrone and not unit:getTraits().safeUnit then
					table.insert( targets, unit )
				end
			end
			for i=1, math.ceil(#targets / 3) do
				if #targets > 0 then
					local target = table.remove( targets, math.floor(sim:nextRand(#targets)) )
					target:giveAbility( "sitrep_chronjob" )
				else
					break
				end
			end
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.CHRONJOB.NAME, STRINGS.SITREP.SITREPS.CHRONJOB.DESC)
		end,
	},

	sitrep_failsafe_drones = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.FAILSAFE_DRONES ) )
	{
		icon = "gui/icons/daemons/sitrep_shirudo.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			local corpData = serverdefs.getCorpData( situation )
			if corpData.region == "asia" then
				return true
			end
			
			return false
		end,

		onSimParams = function( self, params )
			if not params.onUnitCreation then
				params.onUnitCreation = {}
			end
			fn = function (sim, unit)
				if unit:getPlayerOwner() == sim:getNPC() and unit:getTraits().isDrone then
					unit:getTraits().failsafeAi = true
				end
			end
			table.insert(params.onUnitCreation, fn)
		end,
		onSpawnAbility = function( self, sim, player )
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.FAILSAFE_DRONES.NAME, STRINGS.SITREP.SITREPS.FAILSAFE_DRONES.DESC)
		end,
	},

	sitrep_vault_exit = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.VAULT_EXIT ) )
	{
		icon = "gui/icons/daemons/sitrep_balrog.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSimParams = function( self, params )
			params.sitrep_vault_exit = true
		end,
		onSpawnAbility = function( self, sim, player )
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.VAULT_EXIT.NAME, STRINGS.SITREP.SITREPS.VAULT_EXIT.DESC)
		end,
	},

	sitrep_daemonitors = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.DAEMONITORS ) )
	{
		icon = "gui/icons/daemons/sitrep_daemonitors.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			local corpData = serverdefs.getCorpData( situation )
			if corpData.region == "sa" then
				return true
			end
			
			return false
		end,

		onSimParams = function( self, params )
			if not params.onUnitCreation then
				params.onUnitCreation = {}
			end
			fn = function (sim, unit)
				if unit:getPlayerOwner() == sim:getNPC() and unit:getTraits().heartMonitor and unit:getTraits().heartMonitor == "enabled" then
					unit:giveAbility( "sitrep_daemonitor" )
				end
			end
			table.insert(params.onUnitCreation, fn)
		end,
		onSpawnAbility = function( self, sim, player )
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.DAEMONITORS.NAME, STRINGS.SITREP.SITREPS.DAEMONITORS.DESC)
		end,
	},

	sitrep_reboot_max_ice = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.REBOOT_ICE_MAX ) )
	{
		icon = "gui/icons/daemons/sitrep_phoenix.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			for id, unit in pairs( sim:getAllUnits() ) do
				if unit:getPlayerOwner() ~= sim:getPC() and unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice then
					if unit:getTraits().mainframe_iceMax and unit:getTraits().mainframe_iceMax > 0 then
						unit:getTraits().sitrep_reboot_ice = unit:getTraits().mainframe_iceMax
					else
						unit:getTraits().sitrep_reboot_ice = 3
					end
				end
			end
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.REBOOT_ICE_MAX.NAME, STRINGS.SITREP.SITREPS.REBOOT_ICE_MAX.DESC)
		end,
	},

	sitrep_improved_heart_monitors = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.IMPROVED_HEART_MONITORS ) )
	{
		icon = "gui/icons/daemons/sitrep_cardial_monitoring.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			local corpData = serverdefs.getCorpData( situation )
			if corpData.region == "asia" then
				return false
			end
			
			return true
		end,

		onSimParams = function( self, params )
			if not params.onUnitCreation then
				params.onUnitCreation = {}
			end
			fn = function (sim, unit)
				if unit:getPlayerOwner() == sim:getNPC() and unit:getTraits().heartMonitor and unit:getTraits().heartMonitor == "enabled" then
					unit:giveAbility( "improved_heart_monitor_passive" )
				end
			end
			table.insert(params.onUnitCreation, fn)
		end,
		onSpawnAbility = function( self, sim, player )
		end,
		onDespawnAbility = function( self, sim )
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.IMPROVED_HEART_MONITORS.NAME, STRINGS.SITREP.SITREPS.IMPROVED_HEART_MONITORS.DESC)
		end,
	},

	sitrep_superspreader = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.SUPERSPREADER ) )
	{
		icon = "gui/icons/daemons/sitrep_superspreader.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			sim:getNPC():getTraits().sitrep_daemon_spreader = true
		end,
		onDespawnAbility = function( self, sim )
			sim:getNPC():getTraits().sitrep_daemon_spreader = false
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.SUPERSPREADER.NAME, STRINGS.SITREP.SITREPS.SUPERSPREADER.DESC)
		end,
	},
	
	sitrep_static_field = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.STATIC_FIELD ) )
	{
		icon = "gui/icons/daemons/sitrep_static_field.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			sim:getPC():getTraits().sitrep_item_cd_mod = ( sim:getPC():getTraits().sitrep_item_cd_mod or 0 ) + 2
		end,
		onDespawnAbility = function( self, sim )
			sim:getPC():getTraits().sitrep_item_cd_mod = sim:getPC():getTraits().sitrep_item_cd_mod - 2
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.STATIC_FIELD.NAME, STRINGS.SITREP.SITREPS.STATIC_FIELD.DESC)
		end,
	},
	
	sitrep_netwatch = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.NETWATCH ) )
	{
		icon = "gui/icons/daemons/sitrep_netwatch.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			sim:addTrigger( "TRG_UNIT_HACKED", self )
			self.counter = 0
		end,
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_UNIT_HACKED", self )
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == "TRG_UNIT_HACKED" then
				self.counter = self.counter + 1
				if self.counter >= 3 then
					local trackerCount = 1
					sim:trackerAdvance( trackerCount )
					self.counter = 0
				end
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.NETWATCH.NAME, STRINGS.SITREP.SITREPS.NETWATCH.DESC)
		end,
	},
	
	sitrep_frost_field = util.extend( createGuardDaemon( STRINGS.SITREP.SITREPS.FROST_FIELD ) )
	{
		icon = "gui/icons/daemons/sitrep_frost_field.png",
		random_weight = 10,
		
		canBeSelected = function( situation, campaign )
			return true
		end,

		onSpawnAbility = function( self, sim, player )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			for i, unit in pairs( sim:getPC():getUnits() ) do
				if unit:getMP() then
					unit:addMP( 3 )
					unit:addMPMax( 3 )
				end
			end
		end,
		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				for i, unit in pairs( sim:getPC():getUnits() ) do
					if unit:getMP() and unit:getMP() > 4 then --lets not root them agents
						unit:addMP( -1 )
						unit:addMPMax( -1 )
					end
				end
			end
		end,
		
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITREPS.FROST_FIELD.NAME, STRINGS.SITREP.SITREPS.FROST_FIELD.DESC)
		end,
	},
}

return guard_abilities
