local agentdefs = include("sim/unitdefs/agentdefs")
local unitdefs = include("sim/unitdefs")
local serverdefs = include( "modules/serverdefs" )
local util = include("client_util")
local simfactory = include( "sim/simfactory" )
local simdefs = include("sim/simdefs")

local modifier_temp =
{
	situationModifier = true,
}

local situation_modifier =
{
	low = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.LOW.NAME,

		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 2 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "low" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			local maxSlots = serverdefs.getProgramSlots( campaign )
			local allowedSlots = maxSlots - 1
			local currentSlots = 0
			for _, program in pairs(program_loadout) do
				if program then
					currentSlots = currentSlots + 1
				end
			end
			return currentSlots <= allowedSlots
		end,
		onLocationTooltip = function( campaign, section )
			local allowedSlots = serverdefs.getProgramSlots( campaign ) - 1
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.LOW.NAME, util.sformat( STRINGS.SITREP.SITUATION_MODIFIER.LOW.DESC, allowedSlots ), "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},

	manual = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.MANUAL.NAME,

		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 3 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "manual" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			local maxSlots = serverdefs.getProgramSlots( campaign )
			local allowedSlots = maxSlots - 2
			local currentSlots = 0
			for _, program in pairs(program_loadout) do
				if program then
					currentSlots = currentSlots + 1
				end
			end
			return currentSlots <= allowedSlots
		end,
		onLocationTooltip = function( campaign, section )
			local allowedSlots = serverdefs.getProgramSlots( campaign ) - 2
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.MANUAL.NAME, util.sformat( STRINGS.SITREP.SITUATION_MODIFIER.MANUAL.DESC, allowedSlots ), "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},

	precise = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.PRECISE.NAME,
		
		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 3 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "precise" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			local allowedSlots = simdefs.MAX_AGENT_COUNT - 1
			local currentSlots = 0
			for _, agentDef in ipairs(agent_loadout) do
				if agentDef then
					currentSlots = currentSlots + 1
				end
			end
			return currentSlots <= allowedSlots
		end,
		onLocationTooltip = function( campaign, section )
			local allowedSlots = simdefs.MAX_AGENT_COUNT - 1
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.PRECISE.NAME, util.sformat( STRINGS.SITREP.SITUATION_MODIFIER.PRECISE.DESC, allowedSlots ), "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},
	
	surgical = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.SURGICAL.NAME,
		
		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 4 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "surgical" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			local allowedSlots = simdefs.MAX_AGENT_COUNT - 2
			local currentSlots = 0
			for _, agentDef in ipairs(agent_loadout) do
				if agentDef then
					currentSlots = currentSlots + 1
				end
			end
			return currentSlots <= allowedSlots
		end,
		onLocationTooltip = function( campaign, section )
			local allowedSlots = simdefs.MAX_AGENT_COUNT - 2
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.SURGICAL.NAME, util.sformat( STRINGS.SITREP.SITUATION_MODIFIER.SURGICAL.DESC, allowedSlots ), "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},
	
	light = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.LIGHT.NAME,
		
		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 3 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "light" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			for _, agentDef in ipairs(agent_loadout) do
				if agentDef then
					local item_count = 0
					
					for i, item in ipairs(agentDef.upgrades) do
						local itemDef, upgradeParams
						if type(item) == "string" then
							itemDef = unitdefs.lookupTemplate( item )
						else
							upgradeParams = item.upgradeParams
							itemDef = unitdefs.lookupTemplate( item.upgradeName )
						end
						if itemDef then
							local itemUnit = simfactory.createUnit( util.extend( itemDef )( upgradeParams and util.tcopy( upgradeParams )), nil )
							if not (itemUnit:getTraits().augment and itemUnit:getTraits().installed) then
								item_count = item_count + 1
							end
						end
					end
					
					if item_count > 3 then
						return false
					end
				end
			end
			
			return true
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.LIGHT.NAME, STRINGS.SITREP.SITUATION_MODIFIER.LIGHT.DESC, "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},
	
	mechanical = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.MECHANICAL.NAME,
		
		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 2 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "mechanical" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			for _, agentDef in ipairs(agent_loadout) do
				if agentDef then
					local item_count = 0
					
					for i, item in ipairs(agentDef.upgrades) do
						local itemDef, upgradeParams
						if type(item) == "string" then
							itemDef = unitdefs.lookupTemplate( item )
						else
							upgradeParams = item.upgradeParams
							itemDef = unitdefs.lookupTemplate( item.upgradeName )
						end
						if itemDef then
							local itemUnit = simfactory.createUnit( util.extend( itemDef )( upgradeParams and util.tcopy( upgradeParams )), nil )
							if not (itemUnit:getTraits().augment and itemUnit:getTraits().installed) then
								if itemDef.requirements and itemDef.requirements["anarchy"] and itemDef.requirements["anarchy"] >= 2 then
									return false
								end
							end
						end
					end
				end
			end
			
			return true
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.MECHANICAL.NAME, STRINGS.SITREP.SITUATION_MODIFIER.MECHANICAL.DESC, "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},
	
	slow = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.SLOW.NAME,
		
		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 2 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "slow" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			for _, agentDef in ipairs(agent_loadout) do
				if agentDef then
					local item_count = 0
					
					for i, item in ipairs(agentDef.upgrades) do
						local itemDef, upgradeParams
						if type(item) == "string" then
							itemDef = unitdefs.lookupTemplate( item )
						else
							upgradeParams = item.upgradeParams
							itemDef = unitdefs.lookupTemplate( item.upgradeName )
						end
						if itemDef then
							local itemUnit = simfactory.createUnit( util.extend( itemDef )( upgradeParams and util.tcopy( upgradeParams )), nil )
							if not (itemUnit:getTraits().augment and itemUnit:getTraits().installed) then
								if itemDef.requirements and itemDef.requirements["stealth"] and itemDef.requirements["stealth"] >= 2 then
									return false
								end
							end
						end
					end
				end
			end
			
			return true
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.SLOW.NAME, STRINGS.SITREP.SITUATION_MODIFIER.SLOW.DESC, "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},
	
	smart = util.extend( modifier_temp )
	{
		random_weight = 10,
		name = STRINGS.SITREP.SITUATION_MODIFIER.SMART.NAME,
		
		canBeSelected = function( situation, campaign )
			if situation.sitreps and #situation.sitreps >= 2 then
				return true
			end
			
			return false
		end,
		onSelect = function( situation )
			table.remove( situation.sitreps, 1 )
			table.insert( situation.sitreps, 1, "smart" )
		end,
		checkInfiltration = function( campaign, agent_loadout, program_loadout )
			for _, agentDef in ipairs(agent_loadout) do
				if agentDef then
					local item_count = 0
					
					for i, item in ipairs(agentDef.upgrades) do
						local itemDef, upgradeParams
						if type(item) == "string" then
							itemDef = unitdefs.lookupTemplate( item )
						else
							upgradeParams = item.upgradeParams
							itemDef = unitdefs.lookupTemplate( item.upgradeName )
						end
						if itemDef then
							local itemUnit = simfactory.createUnit( util.extend( itemDef )( upgradeParams and util.tcopy( upgradeParams )), nil )
							if not (itemUnit:getTraits().augment and itemUnit:getTraits().installed) then
								if itemDef.requirements and itemDef.requirements["hacking"] and itemDef.requirements["hacking"] >= 2 then
									return false
								end
							end
						end
					end
				end
			end
			
			return true
		end,
		onLocationTooltip = function( campaign, section )
			section:addAbility( STRINGS.SITREP.SITUATION_MODIFIER.SMART.NAME, STRINGS.SITREP.SITUATION_MODIFIER.SMART.DESC, "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png")
		end,
	},
}

return situation_modifier
