local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilitydefs = include( "sim/abilitydefs" )

local oldUseAggression = abilitydefs.lookupAbility("use_aggression")
local oldIsTarget = oldUseAggression.isTarget
local oldExecuteAbility = oldUseAggression.executeAbility

local use_aggression = util.extend(oldUseAggression)
{
	isTarget = function(self, sim, userUnit, targetUnit )
	
		local canUse = oldIsTarget(self, sim, userUnit, targetUnit )
		if canUse == false then
			if simquery.isAgent(targetUnit) and targetUnit:isDead() then
				if targetUnit:getTraits().isDying then
					canUse = true
				end
			end
		end
		return canUse	
	
	end,
	
	executeAbility = function( self, sim, unit, userUnit, target )
		local targetUnit = sim:getUnit(target)
		if targetUnit ~= userUnit then
			if targetUnit:getTraits().isDying then
				sim.stimmingDeadAgent = true
				-- log:write("LOG stimmingdeadagent")
			end
		end

		if targetUnit:getTraits().isDying then
			-- log:write("log defib is dying")
			local x1,y1 = targetUnit:getLocation()
			local newTimer = sim:getParams().difficultyOptions.deathtimer
			targetUnit:getTraits().isDying = newTimer
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.PERMADEATH.DYING_AGENT_STIMMED,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
		end
		
		oldExecuteAbility( self, sim, unit, userUnit, target )
		
		sim.stimmingDeadAgent = nil -- this append is necessary to keep oneliners working correctly
	end,
}

return use_aggression