local array = include( "modules/array" )
local util = include( "modules/util" )
local inventory = include("sim/inventory")
local simdefs = include("sim/simdefs")
local simquery = include( "sim/simquery" )
local abilityutil = include( "sim/abilities/abilityutil" )

function abilityutil.doReload( sim, weaponUnit )
	local userUnit = weaponUnit:getUnitOwner()
	local ammoClip = nil
	local ammoConsumed = false

	if weaponUnit:getTraits().infiniteAmmo then
		ammoConsumed = true
	elseif weaponUnit:getTraits().noReload then 
		ammoConsumed = false 
	elseif userUnit then
		ammoClip = array.findIf( userUnit:getChildren(), 
		function( u ) 
			if u:getTraits().ammo_clip ~= nil and u:getTraits().ammo_clip > 0 then
				return u:getTraits().ammo_clip
			else
				return nil
			end
		end )

		if ammoClip then
			ammoClip:getTraits().ammo_clip = ammoClip:getTraits().ammo_clip - 1
			if ammoClip:getTraits().ammo then
				ammoClip:getTraits().ammo = ammoClip:getTraits().ammo - 1
			end
			if ammoClip:getTraits().charges then
				ammoClip:getTraits().charges = ammoClip:getTraits().charges - 1
			end
			if ammoClip:getTraits().ammo_clip <= 0 and ammoClip:getTraits().disposable ~= false then
				inventory.trashItem( sim, userUnit, ammoClip )
			end
			ammoConsumed = true
		end
	end

	if ammoConsumed then
		if ammoClip and ammoClip:getTraits().overRecharge then
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().maxAmmo + 1
		elseif ammoClip and ammoClip:getTraits().singleRecharge then
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().ammo + 1
		else
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().maxAmmo
		end
        	if userUnit then
		    	local x0, y0 = userUnit:getLocation()		    
		    	sim:emitSound( { path = weaponUnit:getUnitData().sounds.reload, range = simdefs.SOUND_RANGE_0 }, x0, y0, userUnit )
            		if weaponUnit == simquery.getEquippedGun( userUnit ) then
		        	sim:dispatchEvent( simdefs.EV_UNIT_RELOADED, { unit = userUnit } )
		    	end
        	end
	end

	return ammoConsumed
end

function abilityutil.doRecharge( sim, itemUnit )
	local userUnit = itemUnit:getUnitOwner()
	local ammoClip = nil
	local ammoConsumed = false

	if itemUnit:getTraits().infiniteAmmo then
		ammoConsumed = true
	elseif itemUnit:getTraits().noReload then 
		ammoConsumed = false 
	elseif userUnit then
		ammoClip = array.findIf( userUnit:getChildren(), 
		function( u ) 
			if u:getTraits().ammo_clip ~= nil and u:getTraits().ammo_clip > 0 then
				return u:getTraits().ammo_clip
			else
				return nil
			end
		end )

		if ammoClip and (not userUnit:getTraits().pwr_bank or (userUnit:getTraits().pwr_bank and userUnit:getTraits().pwr_bank == 0) ) then
			ammoClip:getTraits().ammo_clip = ammoClip:getTraits().ammo_clip - 1
			if ammoClip:getTraits().ammo then
				ammoClip:getTraits().ammo = ammoClip:getTraits().ammo - 1
			end
			if ammoClip:getTraits().charges then
				ammoClip:getTraits().charges = ammoClip:getTraits().charges - 1
			end
			if ammoClip:getTraits().ammo_clip <= 0 and ammoClip:getTraits().disposable ~= false then
				inventory.trashItem( sim, userUnit, ammoClip )
			end
			ammoConsumed = true
		elseif userUnit:getTraits().pwr_bank and userUnit:getTraits().pwr_bank == 1 then
	    		sim:getCurrentPlayer():addCPUs( -8, sim )
			ammoConsumed = true
		end
	end

	if ammoConsumed then
		if userUnit:getTraits().pwr_bank and userUnit:getTraits().pwr_bank == 1 then
			itemUnit:getTraits().charges = itemUnit:getTraits().chargesMax
		elseif ammoClip and ammoClip:getTraits().overRecharge and itemUnit:getTraits().usesCharges then
			itemUnit:getTraits().charges = itemUnit:getTraits().chargesMax + 1
		elseif ammoClip and ammoClip:getTraits().singleRecharge and itemUnit:getTraits().usesCharges then
			itemUnit:getTraits().charges = itemUnit:getTraits().charges + 1
		elseif itemUnit:getTraits().usesCharges then
			itemUnit:getTraits().charges = itemUnit:getTraits().chargesMax
		elseif ammoClip and ammoClip:getTraits().fullRecharge then
			itemUnit:getTraits().cooldown = 0
		else
			itemUnit:getTraits().cooldown = math.max(itemUnit:getTraits().cooldown - 2,0)
		end
        	if userUnit then
			local x0, y0 = userUnit:getLocation()
			sim:emitSound( { path = "SpySociety/Actions/recharge_item", range = simdefs.SOUND_RANGE_0 }, x0, y0, userUnit )
			if userUnit:getTraits().pwr_bank and userUnit:getTraits().pwr_bank == 1 then
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.UI.FLY_TXT.MINUS_PWR,8),x=x0,y=y0,color={r=1,g=1,b=1,a=1}} )
			end
        	end
	end

	return ammoConsumed
end

function abilityutil.canConsumeAmmo( sim, weaponUnit, consume )
	if weaponUnit:getTraits().ammo then
		local owner = weaponUnit:getUnitOwner()
		local number, aug = 0, {}
		if owner and owner.countAugments then
			number, aug = owner:countAugments( "W93_augment_subdermal_dart_pocket" )
		end
		if weaponUnit:getTraits().canSleep and number > 0 then
			if weaponUnit:getTraits().ammo + aug[1]:getTraits().ammo < (weaponUnit:getTraits().ammoRequired or 1) then
				return false, STRINGS.UI.REASON.NO_AMMO
			end
		elseif weaponUnit:getTraits().ammo < (weaponUnit:getTraits().ammoRequired or 1) then
			return false, STRINGS.UI.REASON.NO_AMMO
		end
	end

	if weaponUnit:getTraits().cooldown then
		if weaponUnit:getTraits().cooldown > 0 then
			return false, util.sformat( STRINGS.UI.REASON.COOLDOWN, weaponUnit:getTraits().cooldown )        
		end
	end

	if weaponUnit:getTraits().energyWeapon == "used" then
		return false, STRINGS.UI.REASON.ALREADY_USED
	end

	if weaponUnit:getTraits().pwrCost then
		if weaponUnit:getPlayerOwner():getCpus() < weaponUnit:getTraits().pwrCost then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR, STRINGS.UI.FLY_TXT.NOT_ENOUGH_PWR
		end
	end

	return true
end

function abilityutil.checkRequirements( abilityOwner, abilityUser )
	if abilityOwner:getRequirements() then 
		for skill,level in pairs( abilityOwner:getRequirements() ) do
			if not abilityUser:hasSkill(skill, level) and not abilityUser:getTraits().useAnyItem then 
				local skilldefs = include( "sim/skilldefs" )
				local skillDef = skilldefs.lookupSkill( skill )            	
				return false, string.format( STRINGS.UI.TOOLTIP_REQUIRES_SKILL_LVL, util.toupper(skillDef.name), level )
			end 
		end
	end

	if abilityOwner:getTraits().restrictedUse then
		local userUnitAgentID = abilityUser:getUnitData().agentID
		local canUse = false
		for i, set in pairs(abilityOwner:getTraits().restrictedUse ) do
			if set.agentID == userUnitAgentID then
				canUse=true
			end
		end

		if not canUse then
			return false, STRINGS.UI.REASON.RESTRUCTED_USE
		end				
	end

	return true
end