local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )

local hoverColor = util.color( 1,1,1,1 )
hoverColor.unpack = util.color.unpack

local SPELL_TEMPLATE = {
	usesAction = true,
	noDefaultTarget = true,
	triggersOverwatch  = true,
	iconColorHover = hoverColor,
		
	getName = function( self, sim, ownerUnit, userUnit, targetUnitID )
		local name = (self.strings and self.strings.name) or self.name
	
		if self.spellSlotLevel then
			return util.sformat( STRINGS.WIZARD.SPELLS.LEVEL, self.spellSlotLevel, name )
		else
			return name
		end
	end,
	
	getLevel = function( self, sim, unit )
		return self.spellSlotLevel or self.spellLevel
	end,
	
	getDescParams = function( self, sim, unit )
		return self:getLevel( self, sim, unit )
	end,
	
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, target, addDescriptions, ... )
		local screen = hud._screen or hud
		local tooltip = util.tooltip( screen )
		local section = tooltip:addSection()
		local FLAVOUR_COLOUR = "<c:61AAAA>"
		local strings = self.strings or self
		
		section:addLine( "<ttheader>"..util.toupper(self:getName()).."</>" )
		local flavor = ""
		if strings.school then
			--flavor = flavor..util.toupper( util.sformat( STRINGS.WIZARD.TOOLTIPS.SCHOOL_OF, self.spellLevel, strings.school ) )
			section:addLine( util.sformat( STRINGS.WIZARD.TOOLTIPS.SCROLL_LEVEL, self.spellLevel, strings.school ) )
		end
		if strings.flavor then
			if #flavor > 0 then
				flavor = flavor.."\n"
			end
			flavor = flavor..strings.flavor
		end
		if #flavor > 0 then
			section:addDesc( FLAVOUR_COLOUR..flavor.."</>" )
		end
		if strings.desc then
			section:addDesc( util.sformat( strings.desc, self:getDescParams( sim, abilityUser ) ) )
		end
		if self.getEffectRadius then
			local radius = self:getEffectRadius( self, sim, abilityOwner, abilityUser, target, ... )
			local throwRange = self:getThrowRange()
			if self.squareArea then
				section:addDesc( util.sformat( STRINGS.WIZARD.SPELLS.SQUARE_AREA, radius, throwRange ) )
			elseif radius > 0 then
				section:addDesc( util.sformat( STRINGS.WIZARD.SPELLS.EFFECT_RADIUS, radius, throwRange ) )
			else
				section:addDesc( util.sformat( STRINGS.WIZARD.SPELLS.RANGE, throwRange ) )
			end
		end
		
		if type(addDescriptions) == "table" then
			for i, description in ipairs( addDescriptions ) do
				section:addDesc( description )
			end
		end
		
		if self.getConcentrationDuration then
			local concentration = self:getConcentrationDuration( sim, abilityOwner, abilityUser, target, ... )
			if concentration and concentration > 0 then
				section:addDesc( util.sformat( STRINGS.WIZARD.SPELLS.CONCENTRATION, concentration ) )
			end
		end
		
		if strings.upcast then
			section:addDesc( util.sformat( strings.upcast, self:getLevel() ) )
		end
		
		if sim then
			local canUse, reason = self:canUseAbility( sim, abilityOwner, abilityUser, target, ... )
			if reason then
				section:addDesc( "<c:ff0000>" .. reason .. "</>" )
			end
		end
		
		return tooltip, section
	end,
	
	canUseAbility = function( self, sim, abilityOwner, unit )
		return self:canCastSpell( sim, abilityOwner, unit )
	end,
	
	canCastSpell = function( self, sim, abilityOwner, unit )
		if self.spellSlotExpended and not self.noSpellSlotRequired then
			return false
		end
		
		if unit:isKO() then
			return false
		end
		
		if self.spellSwitchedOut then
			return false
		end

		if unit:getAP() < 1 and not self.noAttackRequired then 
			return false, STRINGS.UI.REASON.ATTACK_USED
		end
		
		if unit:getTraits().movingBody then
            return false, STRINGS.UI.REASON.DROP_BODY_TO_USE
        end
		
		return true
	end,
	
	castSpell = function( self, sim, abilityOwner, userUnit, target )
		if userUnit:getTraits().monster_hacking then 
			userUnit:getTraits().monster_hacking = false
			userUnit:getSounds().spot = nil
		end
							
		if userUnit:getTraits().data_hacking then 
			userUnit:stopHacking(sim)
		end
		
		userUnit:setInvisible( false )
		userUnit:setDisguise( false, nil, true )
		userUnit:resetAllAiming()
		
		if userUnit:isValid() and not userUnit:isKO() then
			if self.getConcentrationDuration then
				local concentration = self:getConcentrationDuration( sim, abilityOwner, userUnit, target )
				if concentration and concentration > 0 then
					self.arcaneFocus:startConcentration( sim, self, concentration )
				end
			end
		
			self.spellSlotExpended = true
			sim:startTrackerQueue(true)
			if self.manaCost and not self.noSpellSlotRequired then
				self.arcaneFocus:useMana( self.manaCost )
			end
			
			if target then
				-- Show throw animation
				local x0, y0 = userUnit:getLocation()
				local x1, y1 = target.x, target.y
				
				-- Unit target
				if target.getLocation then
					x1, y1 = target:getLocation()
				end
				-- Cell target
				if x1 == nil then
					x1, y1 = target[1], target[2]
				end
				
				local newFacing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )

				if newFacing == simdefs.DIR_MAX then
					self.newFacing = userUnit:getFacing()
				else
					simquery.suggestAgentFacing( userUnit, newFacing )
					self.newFacing = newFacing
				end

				sim:dispatchEvent( simdefs.EV_UNIT_THROW, { unit = userUnit, x1=x1, y1=y1, facing=newFacing } )
			end
		
			return true
		end
		
		return false
	end,
	
	endThrow = function( self, sim, abilityOwner, userUnit )
		--sim:dispatchEvent( simdefs.EV_UNIT_STOP_THROW, { unitID = userUnit:getID(), x1=x1, y1=y1, facing=newFacing } )		
		sim:startTrackerQueue(false)	
		
		if not self.noAttackRequired then
			userUnit:useAP( sim )
		else
			sim:processReactions( userUnit )
		end
	end,
}

local SPELL_UNITTARGET = util.extend( SPELL_TEMPLATE ) {
	showTargets = true,
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local units = {}
		for _, targetUnit in pairs(sim:getAllUnits()) do
			if self:isTarget( sim, unit, userUnit, targetUnit ) then
				table.insert( units, targetUnit )
			end
		end

		local targeting = targets.unitTarget( game, units, self, unit, userUnit )
		
		if game.hud._state ~= game.hud.STATE_ABILITY_TARGET then
			targeting.startTargeting = function( targetSelf, ... )
			end
		end
		
		return targeting
	end,
}

local SPELL_CELLTARGET = util.extend( SPELL_TEMPLATE ) {
	throwRange = 8,
	effectRadius = 0,
	ignoreLOS = false,
	staticImpass = false,
	dynamicImpass = false,
	areaPassesThroughWalls = false,

	showTargets = true,
	
	getThrowRange = function( self )
		return self.throwRange
	end,
	
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		if not self:canUseAbility( sim, unit, userUnit ) then
			return nil
		end
		
		if self.squareArea then
			return targets.squareSpellTarget( game, self:getEffectRadius( sim, unit, userUnit ), sim, userUnit, self )
		else
			return targets.areaSpellTarget( game, self:getEffectRadius( sim, unit, userUnit ), sim, userUnit, self )
		end
	end,
	
	getEffectRadius = function( self, sim, abilityOwner, unit, targetCell )
		return self.effectRadius
	end,
}

abilityutil.SPELL_TEMPLATE = SPELL_TEMPLATE
abilityutil.SPELL_UNITTARGET = SPELL_UNITTARGET
abilityutil.SPELL_CELLTARGET = SPELL_CELLTARGET