local abilityutil = include( "sim/abilities/abilityutil" )
local array = include( "modules/array" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")

function abilityutil.doReload( sim, weaponUnit )
	local userUnit = weaponUnit:getUnitOwner()
	local ammoClip = nil

	local ammoConsumed = false 
	if weaponUnit:getTraits().infiniteAmmo then
		ammoConsumed = true
	elseif weaponUnit:getTraits().noReload then 
		ammoConsumed = false 
	elseif userUnit then
		ammoClip = array.findIf( userUnit:getChildren(), 
			function( u ) 
			if u:getTraits().ammo_clip ~= nil and u:getTraits().ammo_clip > 0 then
				return u:getTraits().ammo_clip
			else
				return nil
			end
		end )

		if ammoClip then
			ammoClip:getTraits().ammo_clip = ammoClip:getTraits().ammo_clip - 1
			if ammoClip:getTraits().ammo then
				ammoClip:getTraits().ammo = ammoClip:getTraits().ammo - 1
			end
			if ammoClip:getTraits().charges then
				ammoClip:getTraits().charges = ammoClip:getTraits().charges - 1
			end
			if ammoClip:getTraits().ammo_clip <= 0 and ammoClip:getTraits().disposable ~= false then
				inventory.trashItem( sim, userUnit, ammoClip )
			end
			ammoConsumed = true
		end
	end

	if ammoConsumed then
		if ammoClip and ammoClip:getTraits().overRecharge then
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().maxAmmo + 1
		elseif ammoClip and ammoClip:getTraits().singleRecharge then
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().ammo + 1
		else
			weaponUnit:getTraits().ammo = weaponUnit:getTraits().maxAmmo
		end
        	if userUnit then
		    	local x0, y0 = userUnit:getLocation()		    
		    	sim:emitSound( { path = weaponUnit:getUnitData().sounds.reload, range = simdefs.SOUND_RANGE_0 }, x0, y0, userUnit )
            		if weaponUnit == simquery.getEquippedGun( userUnit ) then
		        	sim:dispatchEvent( simdefs.EV_UNIT_RELOADED, { unit = userUnit } )
		    	end
        	end
	end

	return ammoConsumed
end
