local Actions = include("sim/btree/actions")
local btree = include("sim/btree/btree")
local cdefs = include("client_defs")
local speechdefs = include( "sim/speechdefs" )
local simdefs = include( "sim/simdefs" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local simquery = include( "sim/simquery" )
local array = include("modules/array")
local astar = include( "modules/astar" )
local astar_handlers = include( "sim/astar_handlers" )
local mainframe = include("sim/mainframe")
local guarddefs = include("sim/unitdefs/guarddefs")

Actions.IsPhantasmalFrightened = function(sim, unit)
	return unit.huntedByPhantasmalKiller ~= nil
end

Actions.TurnToFacePhantasm = function(sim, unit)
	local phantasm = unit.huntedByPhantasmalKiller
	if phantasm then
		local x,y = phantasm:getLocation()
		if x then
			unit:turnToFace(x,y)
		end
	end
	return simdefs.BSTATE_COMPLETE
end

Actions.ReactToPhantasm = function(sim, unit)
	local interest = unit:getBrain():getInterest()
	
	if interest and not interest.noticed then
		if interest.reason == "SAW_PHANTASM" then
			sim:emitSpeech( unit, speechdefs.FLEE_STARTLED )
			unit:turnToFace( interest.x, interest.y )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
		else
			sim:emitSpeech( unit, speechdefs.HUNT_GENERIC )
		end
		
		interest.noticed = true
	elseif unit:getBrain():getTarget() then
		sim:emitSpeech( unit, speechdefs.HUNT_GENERIC )
	end

	return simdefs.BSTATE_COMPLETE
end

Actions.FleePhantasm = class(Actions.MoveTo, function(self, name)
	btree.BaseAction.init(self, name or "FleePhantasm")
end)

function Actions.FleePhantasm:getDestination()
	local unit = self.unit
	local sim = self.sim
	local phantasm = unit.huntedByPhantasmalKiller
	
	if not phantasm or not unit:getTraits().hasSeenSimulacrum then
		return
	end
	
	local x0,y0 = unit:getLocation()
	local x1,y1 = unit:getTraits().hasSeenSimulacrum.x, unit:getTraits().hasSeenSimulacrum.y
	local thisCell = sim:getCell( x0, y0 )
	local cells = simquery.floodFill( sim, unit, thisCell, unit:getTraits().mp, nil, simquery.canSoftPath, nil, sim )
	local bestCell = nil
	local bestDist = 0
	for i, cell in ipairs( cells ) do
		local d = mathutil.distSqr2d( cell.x, cell.y, x1, y1 )
		if d > bestDist then
			bestDist = d
			bestCell = cell
		end
	end
	
	return bestCell
end

function Actions.FleePhantasm:onPathNoMP()
	return simdefs.BSTATE_COMPLETE
end

function Actions.DismissPhantasm(sim, unit)
	if not unit:getTraits().hasSeenSimulacrum then
		local status = Actions.DoLookAround(sim, unit)
		if status == simdefs.BSTATE_COMPLETE then
			Actions.FinishSearch(sim, unit)
		end
	end
	
	return simdefs.BSTATE_COMPLETE
end