local agent_actions = include( "hud/agent_actions" )
local simdefs = include( "sim/simdefs" )

local baseShouldShowAbility = agent_actions.shouldShowAbility
function agent_actions.shouldShowAbility( game, ability, abilityOwner, abilityUser )
	local result = baseShouldShowAbility( game, ability, abilityOwner, abilityUser )

	if result and abilityOwner:getTraits().augment and abilityOwner:getTraits().installed and not ability.maa_activeAbility then
		return false
	end

	return result
end

local baseGeneratePotentialActions = agent_actions.generatePotentialActions
function agent_actions.generatePotentialActions( hud, actions, unit, cellx, celly )
	baseGeneratePotentialActions( hud, actions, unit, cellx, celly )

	if unit:getTraits().maa_inside_mainframe then
		local sim = hud._game.simCore
		local localPlayer = hud._game:getLocalPlayer()
		local x0, y0 = unit:getLocation()

		if not cellx or not celly or x0 ~= cellx or y0 ~= celly then
			return
		end

		for _, checkUnit in pairs( sim:getAllUnits() ) do
			local x1, y1 = checkUnit:getLocation()

			if x1 and y1 and localPlayer:getCell( x1, y1 ) then
				--is the distance 2 or less then its already been handled
				if math.abs( x0 - x1 ) + math.abs( y0 - y1 ) > 2 then
					for j, ability in ipairs( checkUnit:getAbilities() ) do
						if checkUnit ~= unit and agent_actions.shouldShowProxyAbility( hud._game, ability, checkUnit, unit, actions ) then
							table.insert( actions, { ability = ability, abilityOwner = checkUnit, abilityUser = unit, priority = ability.HUDpriority } )
						end
					end
				end
			end
		end
	end
end

local basePerformAbility = agent_actions.performAbility
function agent_actions.performAbility( game, abilityOwner, abilityUser, ability, ... )
	local playAction = false
	if ability.maa_two_state then
		if not ability.maa_state then
			ability.maa_state = true
			game.hud.maa_pending_ability = ability
			local abilityTargets = {...}
			game.hud:transitionItemTarget( abilityOwner, abilityUser )
		else
			ability.maa_state = nil
			playAction = true
		end
	else
		playAction = true
	end

	if playAction then
		basePerformAbility( game, abilityOwner, abilityUser, ability, ... )
	end
end
