local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include("sim/simdefs")
local speechdefs = include("sim/speechdefs")
local SCRIPTS = include('client/story_scripts')
local DEFAULT_DRONE = commondefs.DEFAULT_DRONE
local SOUNDS = commondefs.SOUNDS

local WIZARD_SOUNDS =
{
    bio = "",
    escapeVo = "",
	speech="SpySociety/Agents/dialogue_player",  
	step = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_NORMAL, 
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_SOFT,  

	wallcover = "SpySociety/Movement/foley_trench/wallcover",
	crouchcover = "SpySociety/Movement/foley_trench/crouchcover",
	fall = "SpySociety/Movement/foley_trench/fall",	
	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 35,						
	getup = "SpySociety/Movement/foley_trench/getup",	
	grab = "SpySociety/Movement/foley_trench/grab_guard",	
	pin = "SpySociety/Movement/foley_trench/pin_guard",
	pinned = "SpySociety/Movement/foley_trench/pinned",
	peek_fwd = "SpySociety/Movement/foley_trench/peek_forward",	
	peek_bwd = "SpySociety/Movement/foley_trench/peek_back",					
	move = "SpySociety/Movement/foley_trench/move",		
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",			
}

local agents = {
	wizard_agent = {
		type = "simunit", 
        agentID = "wizard_agent",
		name = STRINGS.WIZARD.WIZARD.NAME,
		file = STRINGS.WIZARD.WIZARD.FILE,
		fullname = STRINGS.WIZARD.WIZARD.ALT_1.FULLNAME,
		loadoutName = STRINGS.UI.ON_FILE,
		yearsOfService = STRINGS.WIZARD.WIZARD.YEARS_OF_SERVICE,
		age = STRINGS.WIZARD.WIZARD.ALT_1.AGE,
		homeTown = STRINGS.WIZARD.WIZARD.HOMETOWN,
		gender = "male",
		class = "Engineer",
		toolTip =  STRINGS.WIZARD.WIZARD.ALT_1.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/wizard_36.png",
		profile_icon_64x64= "gui/profile_icons/wizard_64x64.png",		
		splash_image = "gui/agents/wizard_1024.png",
		team_select_img = {
			"gui/agents/team_select_1_wizard.png",
		},

		profile_anim = "wizard/wizard_face",
		kanim = "kanim_wizard",
		hireText =  STRINGS.WIZARD.WIZARD.RESCUED,
		centralHireSpeech = nil,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax = 8 },
		skills = { "stealth", "hacking", "magic", "anarchy" }, 
		startingSkills = { },
		abilities = util.tconcat( { "sprint" }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = { }, -- Dont add items here, add them to the upgrades table in createDefaultAgency()
		sounds = WIZARD_SOUNDS,
		speech = STRINGS.WIZARD.WIZARD.BANTER,
		blurb =  STRINGS.WIZARD.WIZARD.ALT_1.BIO,
		upgrades = { "augment_wizard", "item_wiz_spellbook", "item_wiz_spellscroll_free" },
		couldLearnSkill = function(self, skillId)
			return skillId == "inventory"
		end,
	},

	wizard_agent_archived = {
		type = "simunit", 
        agentID = "wizard_agent",
		name = STRINGS.WIZARD.WIZARD.NAME,
		file = STRINGS.WIZARD.WIZARD.FILE,
		fullname = STRINGS.WIZARD.WIZARD.ALT_1.FULLNAME,
		codename = STRINGS.WIZARD.WIZARD.ALT_2.FULLNAME,
		loadoutName = STRINGS.UI.ON_ARCHIVE,
		yearsOfService = STRINGS.WIZARD.WIZARD.YEARS_OF_SERVICE,
		age = STRINGS.WIZARD.WIZARD.ALT_2.AGE,
		homeTown = STRINGS.WIZARD.WIZARD.HOMETOWN,
		gender = "male",
		class = "Engineer",
		toolTip =  STRINGS.WIZARD.WIZARD.ALT_2.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/wizard_arch_36.png",
		profile_icon_64x64= "gui/profile_icons/wizard_arch_64x64.png",		
		splash_image = "gui/agents/wizard_arch_1024.png",
		team_select_img = {
			"gui/agents/team_select_1_wizard_arch.png",
		},

		profile_anim = "wizard/wizard_face_archived",
		kanim = "kanim_wizard_archived",
		hireText =  STRINGS.WIZARD.WIZARD.RESCUED,
		centralHireSpeech = nil,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax = 8 },
		skills = { "stealth", "hacking", "magic2", "anarchy" }, 
		startingSkills = { },
		abilities = util.tconcat( { "sprint" }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = { }, -- Dont add items here, add them to the upgrades table in createDefaultAgency()
		sounds = WIZARD_SOUNDS,
		speech = STRINGS.WIZARD.WIZARD.BANTER,
		blurb =  STRINGS.WIZARD.WIZARD.ALT_2.BIO,
		upgrades = { "augment_wizard_archived", "item_wiz_crystal" },
		couldLearnSkill = function(self, skillId)
			return skillId == "inventory"
		end,
	}
}

return agents