local util = include( "modules/util" )
local commondefs = include("sim/unitdefs/commondefs")
local simdefs = include("sim/simdefs")
local speechdefs = include( "sim/speechdefs" )
local SCRIPTS = include('client/story_scripts')

local CYBERNETICIST_SOUNDS =
{
	bio = "",
	escapeVo = "",
	speech="SpySociety/Agents/dialogue_player",
	step = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_NORMAL,
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_SOFT,

	wallcover = "SpySociety/Movement/foley_suit/wallcover",
	crouchcover = "SpySociety/Movement/foley_suit/crouchcover",
	fall = "SpySociety/Movement/foley_suit/fall",
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 16,
	getup = "SpySociety/Movement/foley_suit/getup",
	grab = "SpySociety/Movement/foley_suit/grab_guard",
	pin = "SpySociety/Movement/foley_suit/pin_guard",
	pinned = "SpySociety/Movement/foley_suit/pinned",
	peek_fwd = "SpySociety/Movement/foley_suit/peek_forward",
	peek_bwd = "SpySociety/Movement/foley_suit/peek_back",
	move = "SpySociety/Movement/foley_suit/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",
}

local SPIDER_DRONE_SOUNDS =
{
	bio = "",
	escapeVo = "",
	speech="SpySociety/Agents/dialogue_player",
	stealthstep = simdefs.SOUNDPATH_FOOTSTEP_DRONE_HARDWOOD_NORMAL,
	step = simdefs.SOUNDPATH_FOOTSTEP_DRONE_HARDWOOD_NORMAL,

	wallcover = "SpySociety/Movement/foley_trench/wallcover",
	crouchcover = "SpySociety/Movement/foley_trench/crouchcover",
	fall = "SpySociety/Movement/foley_walkingdrone/fall",
	fall_knee = "SpySociety/Movement/foley_walkingdrone/fall",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/foley_walkingdrone/fall",
	fall_handframe = 20,
	land = "SpySociety/Objects/drone/drone_hitground",
	land_frame = 9,
	getup = "SpySociety/Movement/foley_walkingdrone/getup",
	grabbed = "SpySociety/Movement/foley_walkingdrone/grabbed",
	pin = "SpySociety/Movement/foley_walkingdrone/pin_guard",
	pinned = "SpySociety/Movement/foley_walkingdrone/pinned",
	peek_fwd = "SpySociety/Actions/Engineer/wireless_emitter",
	peek_bwd = "SpySociety/Actions/Engineer/wireless_emitter",
	move ="SpySociety/Movement/foley_walkingdrone/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_metal",

	-- next sounds not used in current iteration I guess:
	explode = "SpySociety/Objects/drone/drone_explo",
	explode_frame = 1,
	open = "SpySociety/Objects/drone/drone_open",
	open_frame = 12,
	close = "SpySociety/Objects/drone/drone_close",
	close_frame = 10,
	walk_pre = "SpySociety/Objects/drone/drone_close",
	walk_pst = "SpySociety/Objects/drone/drone_open",
	run_pst = "SpySociety/Objects/drone/drone_open",
	snk_pst = "SpySociety/Objects/drone/drone_open",
}

---------------------------------------------------------------------------

local agents =
{
	maa_numi_1 =
	{
		type = "simunit",
      		agentID = "mod_04_n_umi",
		name = STRINGS.AMRE.AGENTS.NU.NAME,
		file = STRINGS.AMRE.AGENTS.NU.FILE,
		fullname = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.NUMI.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_B,
		yearsOfService = STRINGS.AMRE.AGENTS.NU.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.NUMI.ALT_3.AGE,
		homeTown = STRINGS.AMRE.AGENTS.NU.HOMETOWN,
		gender = "female",
		class = "Engineer",
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.NUMI.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/lady_nu_36.png",
		profile_icon_64x64= "gui/profile_icons/nu_64x64.png",
		splash_image = "gui/agents/nu_1024.png",

		team_select_img = {
			"gui/agents/team_select_1_nu.png",
		},

		profile_anim = "portraits/lady_sharpshooter_face",
		profile_build = "portraits/nu_lady_face",
		kanim = "kanim_lady_cyberneticist",
		hireText = STRINGS.AMRE.AGENTS.NU.RESCUED,
		--centralHireSpeech = SCRIPTS.INGAME.CENTRAL_AGENT_ESCAPE_INTERNATIONALE,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax = 8 },
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		--startingSkills = { hacking = 2},
		startingSkills = { },
		abilities = util.tconcat( {  "sprint" }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = {}, -- Dont add items here, add them to the upgrades table in createDefaultAgency()
		sounds = CYBERNETICIST_SOUNDS,
		speech = STRINGS.AMRE.AGENTS.NU.BANTER,
		blurb = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.NUMI.ALT_3.BIO,
		upgrades = { "maa_drone_controller", "maa_sankaku_tazer", "maa_portable_drone" },
		maa_summons = { "maa_spider" },
	},

	maa_spider =
	{
		type = "simunit",
		agentID = "maa_umi_drone",
		name = STRINGS.AMRE.ITEMS.SPIDER_BOT,
		fullname = STRINGS.AGENTS.HOSTAGE.ALT_1.FULLNAME,
		yearsOfService = STRINGS.AGENTS.HOSTAGE.YEARS_OF_SERVICE,
		age = STRINGS.AGENTS.HOSTAGE.AGE,
		homeTown = STRINGS.AGENTS.HOSTAGE.HOMETOWN,
		toolTip = STRINGS.AMRE.ITEMS.SPIDER_BOT_TIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/mod_sp_drone_36.png",
		splash_image = "gui/agents/agentDeckard_768.png",
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/mod_spider_drone",
		kanim = "kanim_drone_mod",
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { inventoryMaxSize = 1, neural_scanned = true, mp=8, mpMax =8, noUpgrade = true, hidesInCover = true, noDoorAnim = false, notDraggable = true, dashSoundRange = 4, hits = "spark", isAiming = false, noLoopOverwatch = true, canBeCritical = false, canKO = false, empDeath = true, hasHearing = false, augmentMaxSize = 1, dynamicImpass = false, maa_isUmiDrone = true, maa_cant_steal = true, },
		skills = { "maa_battery" },
		children = { },
		abilities =  { "peek", "escape", "disarmtrap", "maa_recover_drone" },
		sounds = SPIDER_DRONE_SOUNDS,
		speech = speechdefs.stealth_1,
		blurb = "",
		upgrades = { "maa_drone_battery" },
		maa_skillpaths = { "drone_combat", "drone_scout", "drone_support" },
		upgrade_screen_rebuy = "maa_portable_drone",
	},
}

return agents
