local util = include( "modules/util" )
local commondefs = include("sim/unitdefs/commondefs")
local simdefs = include("sim/simdefs")
local speechdefs = include( "sim/speechdefs" )
local SCRIPTS = include('client/story_scripts')

local OLIVIA_SOUNDS =
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Olivia",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Olivia",
	speech="SpySociety/Agents/dialogue_player",
	step = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_NORMAL,
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_SOFT,

	wallcover = "SpySociety/Movement/foley_suit/wallcover",
	crouchcover = "SpySociety/Movement/foley_suit/crouchcover",
	fall = "SpySociety/Movement/foley_suit/fall",
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 16,
	getup = "SpySociety/Movement/foley_suit/getup",
	grab = "SpySociety/Movement/foley_suit/grab_guard",
	pin = "SpySociety/Movement/foley_suit/pin_guard",
	pinned = "SpySociety/Movement/foley_suit/pinned",
	peek_fwd = "SpySociety/Movement/foley_suit/peek_forward",
	peek_bwd = "SpySociety/Movement/foley_suit/peek_back",
	move = "SpySociety/Movement/foley_suit/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",
}

local DEREK_SOUNDS =
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Derek",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Derek",
	speech="SpySociety/Agents/dialogue_player",
	step = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_NORMAL,
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_SOFT,

	wallcover = "SpySociety/Movement/foley_suit/wallcover",
	crouchcover = "SpySociety/Movement/foley_suit/crouchcover",
	fall = "SpySociety/Movement/foley_suit/fall",
	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 35,
	getup = "SpySociety/Movement/foley_suit/getup",
	grab = "SpySociety/Movement/foley_suit/grab_guard",
	pin = "SpySociety/Movement/foley_suit/pin_guard",
	pinned = "SpySociety/Movement/foley_suit/pinned",
	peek_fwd = "SpySociety/Movement/foley_suit/peek_forward",
	peek_bwd = "SpySociety/Movement/foley_suit/peek_back",
	move = "SpySociety/Movement/foley_suit/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",
}

local DRACO_SOUNDS = util.extend(DEREK_SOUNDS)
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Draco",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Draco",
}

local RUSH_SOUNDS = util.extend(OLIVIA_SOUNDS)
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Rush",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Rush",
}

local agents =
{
	maa_olivia_ult_ab =
	{
		type = "simunit",
		agentID = 1000,
		name = STRINGS.DLC1.AGENTS.OLIVIA.NAME,
		fullname = STRINGS.DLC1.AGENTS.OLIVIA.ALT_1.FULLNAME,
		codename = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.OLIVIA.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_ULT_AB,
		file = STRINGS.DLC1.AGENTS.OLIVIA.FILE,
		yearsOfService = STRINGS.DLC1.AGENTS.OLIVIA.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.OLIVIA.ALT_3.AGE,
		homeTown =  STRINGS.DLC1.AGENTS.OLIVIA.HOMETOWN,
		gender = "female",
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.OLIVIA.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/olivia_36.png",
		profile_icon_64x64= "gui/profile_icons/olivia_64x64.png",
		splash_image = "gui/agents/olivia_1024.png",
		profile_anim = "portraits/olivia_face",
		gender = "female",
		team_select_img = {
			"gui/agents/team_select_1_olivia.png",
		},
		kanim = "kanim_olivia",
		hireText =  STRINGS.DLC1.AGENTS.OLIVIA.RESCUED,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax =8, },
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		startingSkills = { anarchy = 2 },
		abilities = util.tconcat( {  "sprint",  }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = {},
		sounds = OLIVIA_SOUNDS,
		speech = STRINGS.DLC1.AGENTS.OLIVIA.BANTER,
		blurb = STRINGS.DLC1.AGENTS.OLIVIA.ALT_1.BIO.."\n\n"..STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.OLIVIA.ALT_3.BIO,
		upgrades = { "augment_ambient_turbine", "maa_neural_interrogator", "item_tazer", "item_crybaby", "maa_tracking_chip" },
		logs = {},
	},
	
	maa_derek_ult_ab =
	{
		type = "simunit",
		agentID = 1001,
		name = STRINGS.DLC1.AGENTS.DEREK.NAME,
		fullname = STRINGS.DLC1.AGENTS.DEREK.ALT_1.FULLNAME,
		codename = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DEREK.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_ULT_AB,
		yearsOfService = STRINGS.DLC1.AGENTS.DEREK.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DEREK.ALT_3.AGE,
		homeTown = STRINGS.DLC1.AGENTS.DEREK.HOMETOWN,
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DEREK.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/derek_36.png",
		profile_icon_64x64= "gui/profile_icons/derek_64x64.png",
		gender = "male",
		splash_image = "gui/agents/derek_1024.png",
		profile_anim = "portraits/derek_build",
		team_select_img = {
			"gui/agents/team_select_1_derek.png",
		},
		kanim = "kanim_derek",
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax=8 },
		children = {}, -- Dont add items here, add them to the upgrades table in createDefaultAgency()
		abilities = util.tconcat( {  "sprint",  }, commondefs.DEFAULT_AGENT_ABILITIES ),-- "stealth"
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		startingSkills = { hacking = 2 },
		sounds = DEREK_SOUNDS,
		speech = STRINGS.DLC1.AGENTS.DEREK.BANTER,
		hireText = STRINGS.DLC1.AGENTS.DEREK.RESCUED,
		blurb = STRINGS.DLC1.AGENTS.DEREK.ALT_1.BIO.."\n\n"..STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DEREK.ALT_3.BIO,
		upgrades = { "augment_particle_envelope", "maa_virus_injector", "item_dartgun", "item_transport_beacon", "maa_derek_smoke" },
		logs = {},
	},
	
	maa_draco_ult_ab =
	{
		type = "simunit",
		agentID = 1003,
		name = STRINGS.DLC1.AGENTS.DRACO.NAME,
		fullname = STRINGS.DLC1.AGENTS.DRACO.ALT_1.FULLNAME,
		codename = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_ULT_AB,
		file = STRINGS.DLC1.AGENTS.DRACO.FILE,
		yearsOfService = STRINGS.DLC1.AGENTS.DRACO.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.AGE,
		homeTown = STRINGS.DLC1.AGENTS.DRACO.HOMETOWN,
		gender = "male",
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/draco_36.png",
		profile_icon_64x64= "gui/profile_icons/draco_64x64.png",
		splash_image = "gui/agents/draco_1024.png",
		profile_anim = "portraits/dracul_build",
		team_select_img = {
			"gui/agents/team_select_ultimate_ofdraco.png",
		},
		kanim = "kanim_dracul",
		hireText = STRINGS.DLC1.AGENTS.DRACO.RESCUED,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax =8, },
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		startingSkills = { inventory = 2 },
		abilities = util.tconcat( {  "sprint",  }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = {},
		sounds = DRACO_SOUNDS,
		speech = STRINGS.DLC1.AGENTS.DRACO.BANTER,
		blurb = STRINGS.DLC1.AGENTS.DRACO.ALT_1.BIO.."\n\n"..STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.BIO,
		upgrades = { "augment_neural_mapper", "maa_vampire_scanner", "item_tazer", "maa_vampire_gun" },
		logs = {},
	},
	
	maa_rush_ult_ac =
	{
		type = "simunit",
		agentID = 1002,
		name = STRINGS.DLC1.AGENTS.RUSH.NAME,
		fullname = STRINGS.DLC1.AGENTS.RUSH.ALT_1.FULLNAME,
		codename = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_ULT_AC,
		file = STRINGS.DLC1.AGENTS.RUSH.FILE,
		yearsOfService = STRINGS.DLC1.AGENTS.RUSH.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.AGE,
		homeTown = STRINGS.DLC1.AGENTS.RUSH.HOMETOWN,
		gender = "female",
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/rush_36.png",
		profile_icon_64x64= "gui/profile_icons/rush_64x64.png",
		splash_image = "gui/agents/rush_1024.png",
		profile_anim = "portraits/rush_face",
		team_select_img = {
			"gui/agents/team_select_ultimate_ofrush.png",
		},
		kanim = "kanim_rush",
		hireText = STRINGS.DLC1.AGENTS.RUSH.RESCUED,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax =8, },
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		startingSkills = { stealth = 2 },
		abilities = util.tconcat( {  "sprint",  }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = {},
		sounds = RUSH_SOUNDS,
		speech = STRINGS.DLC1.AGENTS.RUSH.BANTER,
		blurb = STRINGS.DLC1.AGENTS.RUSH.ALT_1.BIO.."\n\n"..STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_4.BIO,
		upgrades = { "augment_kinetic_capacitor", "maa_feedback_processor", "maa_corporate_tazer", "item_stim", "maa_lock_explosive" },
		logs = {},
	},
}

return agents
