local util = include( "modules/util" )
local commondefs = include("sim/unitdefs/commondefs")
local simdefs = include("sim/simdefs")
local speechdefs = include( "sim/speechdefs" )
local SCRIPTS = include('client/story_scripts')

local OLIVIA_SOUNDS =
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Olivia",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Olivia",
	speech="SpySociety/Agents/dialogue_player",
	step = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_NORMAL,
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_SOFT,

	wallcover = "SpySociety/Movement/foley_suit/wallcover",
	crouchcover = "SpySociety/Movement/foley_suit/crouchcover",
	fall = "SpySociety/Movement/foley_suit/fall",
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 16,
	getup = "SpySociety/Movement/foley_suit/getup",
	grab = "SpySociety/Movement/foley_suit/grab_guard",
	pin = "SpySociety/Movement/foley_suit/pin_guard",
	pinned = "SpySociety/Movement/foley_suit/pinned",
	peek_fwd = "SpySociety/Movement/foley_suit/peek_forward",
	peek_bwd = "SpySociety/Movement/foley_suit/peek_back",
	move = "SpySociety/Movement/foley_suit/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",
}

local DEREK_SOUNDS =
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Derek",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Derek",
	speech="SpySociety/Agents/dialogue_player",
	step = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_NORMAL,
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_SOFT,

	wallcover = "SpySociety/Movement/foley_suit/wallcover",
	crouchcover = "SpySociety/Movement/foley_suit/crouchcover",
	fall = "SpySociety/Movement/foley_suit/fall",
	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 35,
	getup = "SpySociety/Movement/foley_suit/getup",
	grab = "SpySociety/Movement/foley_suit/grab_guard",
	pin = "SpySociety/Movement/foley_suit/pin_guard",
	pinned = "SpySociety/Movement/foley_suit/pinned",
	peek_fwd = "SpySociety/Movement/foley_suit/peek_forward",
	peek_bwd = "SpySociety/Movement/foley_suit/peek_back",
	move = "SpySociety/Movement/foley_suit/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",
}

local DRACO_SOUNDS = util.extend(DEREK_SOUNDS)
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Draco",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Draco",
}

local RUSH_SOUNDS = util.extend(OLIVIA_SOUNDS)
{
	bio = "SpySociety_DLC001/VoiceOver/Central/Bio_Rush",
	escapeVo = "SpySociety_DLC001/VoiceOver/Central/Escape_Rush",
}

local agents =
{
	maa_draco_ult_ofc =
	{
		type = "simunit",
		agentID = 1003,
		name = STRINGS.DLC1.AGENTS.DRACO.NAME,
		fullname = STRINGS.DLC1.AGENTS.DRACO.ALT_1.FULLNAME,
		codename = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_ULT_OFC,
		file = STRINGS.DLC1.AGENTS.DRACO.FILE,
		yearsOfService = STRINGS.DLC1.AGENTS.DRACO.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.AGE,
		homeTown = STRINGS.DLC1.AGENTS.DRACO.HOMETOWN,
		gender = "male",
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/draco_36.png",
		profile_icon_64x64= "gui/profile_icons/draco_64x64.png",
		splash_image = "gui/agents/draco_1024.png",
		profile_anim = "portraits/dracul_build",
		profile_build = "portraits/dracul_onfile_build",
		team_select_img = {
			"gui/agents/team_select_ultimate_ofdraco.png",
		},
		kanim = "kanim_dracul_onfile",
		hireText = STRINGS.DLC1.AGENTS.DRACO.RESCUED,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax =8, },
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		startingSkills = { inventory = 2 },
		abilities = util.tconcat( {  "sprint",  }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = {},
		sounds = DRACO_SOUNDS,
		speech = STRINGS.DLC1.AGENTS.DRACO.BANTER,
		blurb = STRINGS.ONFILE_PEIA.AGENTS.DRACO.ALT_2.BIO.."\n\n"..STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.DRACO.ALT_4.BIO,
		upgrades = { "augment_neural_mapper_onfile", "maa_vampire_thirst", "item_tazer", "item_light_pistol_KO", "item_smokegrenade_draco" },
		logs = {},
	},
	
	maa_rush_ult_ofb =
	{
		type = "simunit",
		agentID = 1002,
		name = STRINGS.DLC1.AGENTS.RUSH.NAME,
		fullname = STRINGS.DLC1.AGENTS.RUSH.ALT_1.FULLNAME,
		codename = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.FULLNAME,
		loadoutName = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.GENERAL.LOADOUT_ULT_OFB,
		file = STRINGS.DLC1.AGENTS.RUSH.FILE,
		yearsOfService = STRINGS.DLC1.AGENTS.RUSH.YEARS_OF_SERVICE,
		age = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.AGE,
		homeTown = STRINGS.DLC1.AGENTS.RUSH.HOMETOWN,
		gender = "female",
		toolTip = STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.TOOLTIP,
		onWorldTooltip = commondefs.onAgentTooltip,
		profile_icon_36x36= "gui/profile_icons/rush_36.png",
		profile_icon_64x64= "gui/profile_icons/rush_64x64.png",
		splash_image = "gui/agents/rush_1024.png",
		profile_anim = "portraits/rush_face",
		team_select_img = {
			"gui/agents/team_select_ultimate_ofrush.png",
		},
		kanim = "kanim_rush",
		hireText = STRINGS.DLC1.AGENTS.RUSH.RESCUED,
		traits = util.extend( commondefs.DEFAULT_AGENT_TRAITS ) { mp=8, mpMax =8, },
		skills = util.extend( commondefs.DEFAULT_AGENT_SKILLS ) {},
		startingSkills = { stealth = 2 },
		abilities = util.tconcat( {  "sprint",  }, commondefs.DEFAULT_AGENT_ABILITIES ),
		children = {},
		sounds = RUSH_SOUNDS,
		speech = STRINGS.DLC1.AGENTS.RUSH.BANTER,
		blurb = STRINGS.ONFILE_PEIA.AGENTS.RUSH.ALT_2.BIO.."\n\n"..STRINGS.MORE_ARCHIVE_AGENTS.AGENTS.RUSH.ALT_3.BIO,
		upgrades = { "augment_kinetic_dynamo_mod", "maa_echo_injector", "item_power_tazer_1", "item_soundbug_bomb", "maa_turbo_stim" },
		logs = {},
	},
}

return agents
