local agentrig = include( "gameplay/agentrig" )
local simquery = include("sim/simquery")
local simdefs = include("sim/simdefs")

agentrig.walkState = upvalueUtil.find( agentrig.rig.init, "walking_state" )

function agentrig.walkState:performStep( ev )
	local from_cell = ev.eventData.from_cell
	local to_cell = ev.eventData.to_cell
	
	local isStealth = ev.eventData.unit:getTraits().sneaking
	local soundRange = simquery.getMoveAudioRange( ev.eventData.unit, to_cell )
	local walkTime = simquery.getWalkTime( ev.eventData.unit, from_cell, to_cell )
	local isSeen = self._rig._boardRig:canPlayerSeeUnit( ev.eventData.unit )

	-- Need to make flag frontmost, in case we're walking past some dead dudes.
	if self._rig._flagUI then
		self._rig._flagUI:moveToFront()
	end

	local foot_frames = simdefs.FOOTSTEP_FRAMES_RUN
	if isSeen then
		self._rig:setPlayMode( KLEIAnim.LOOP )

		if self._rig:getUnit():getTraits().movingBody then
			self._rig:setCurrentAnim("body_drag", simquery.getReverseDirection(self._rig:getUnit():getFacing() ) )
		elseif self._rig:getUnit():getTraits().walk then
			if not ev.eventData.reverse then
				if self._rig._prop:getCurrentAnim() == "walk180" and self._rig._prop:getFrame() < self._rig._prop:getFrameCount()-1 then
					self._rig._prop:setListener( KLEIAnim.EVENT_ANIM_END,
						function( anim, animname )
							self._rig._prop:setListener( KLEIAnim.EVENT_ANIM_END, nil )
							self._rig:setCurrentAnim( "walk" )
						end )
				else
					self._rig:setCurrentAnim( "walk" )
				end
				foot_frames = simdefs.FOOTSTEP_FRAMES_WALK
			end
		elseif self._rig:getUnit():getTraits().sneaking then
			self._rig:setCurrentAnim( "snk" )
			foot_frames = simdefs.FOOTSTEP_FRAMES_SNK
		else
			self._rig:setCurrentAnim( "run" )
			foot_frames = simdefs.FOOTSTEP_FRAMES_RUN
		end
	end
		
	if isSeen and not self._wasSeen then
		self:createGhostFade( walkTime * 0.7, 0.8, 1 ) -- Fade in from FOW
	elseif not isSeen and self._wasSeen then
		self:createGhostFade( walkTime * 0.7, 1, 0.8 ) -- Fade out into FOW
	end

	-- Update correct anim facing and render filter according to rig location (if visibility changes, we may become ghosted)
	if self._wasSeen or isSeen then
		self._rig:setLocation( to_cell.x, to_cell.y )
		self._rig._facing = simquery.getDirectionFromDelta( to_cell.x - from_cell.x, to_cell.y - from_cell.y )

		self._rig:refreshProp( true )
	end

	if self._wasSeen or isSeen or self._rig._boardRig:canPlayerHear( to_cell.x, to_cell.y, soundRange ) or (self._rig:getUnit():getTraits().tagged and not isStealth) then

		if self._wasSeen or isSeen then
			-- Perform the actual movement interpolation if we are or were visible.
			local x0, y0 = self._rig._boardRig:cellToWorld( from_cell.x, from_cell.y )
			local x1, y1 = self._rig._boardRig:cellToWorld( to_cell.x, to_cell.y )
			self._rig._prop:setLoc( x0, y0 )

			local ease = MOAIEaseType.LINEAR
			if ev.eventData.reverse then
				ease = MOAIEaseType.LINEAR
			end

			self._easeDriver = self._rig._prop:seekLoc( x1, y1, 0, walkTime, ease )

			if self._losProp == nil then
				self._losProp = MOAITransform.new()
				self:refreshLOSCaster( self._rig._unitID )
			end
			self._losProp:setLoc( x0, y0 )
			self._losDriver = self._losProp:seekLoc( x1, y1, 0, walkTime, ease )

		else
			-- Otherwise, just fake the timing; we need to play sounds.
			local timer = MOAITimer.new ()
			timer:setSpan( 0, walkTime )
			timer:start()
			self._easeDriver = timer
		end

		if ev.eventData.unit:getSounds().move_loop then
			local x1, y1 = self._rig._boardRig:cellToWorld( to_cell.x, to_cell.y )
			MOAIFmodDesigner.setSoundProperties( "move_loop", nil, {x1,y1,0}, nil )
		end	
		if ev.eventData.unit:getTraits().movingBody then
			local x1, y1 = self._rig._boardRig:cellToWorld( to_cell.x, to_cell.y )
			MOAIFmodDesigner.setSoundProperties( "drag_loop", nil, {x1,y1,0}, nil )
		end	

		local stepSound
		if isStealth then
			stepSound = ev.eventData.unit:getSounds().stealthStep			
		else
			stepSound = ev.eventData.unit:getSounds().step
		end

		local frames = foot_frames
		local sounds =  { stepSound	}

		if self._rig:getUnit():getSounds().move then
			table.insert(frames,2)
			table.insert(sounds,self._rig:getUnit():getSounds().move)
		end

		self:wait(self._easeDriver,
		{ 
			{
				frames = frames,
				sounds = sounds,
				x = to_cell.x,
				y = to_cell.y,
			},
		} )

		self._easeDriver = nil

	else
		self:waitDuration( walkTime )
	end

	self._rig:refreshHUD( self._rig:getUnit() )
	self:destroyGhostFade()

	self._wasSeen = isSeen
	
end