local util = include( "client_util" )
local agentrig = include( "gameplay/agentrig" ).rig
local simquery = include( "sim/simquery" )
local animmgr = include( "anim-manager" )


local function GetAnimResource( build )
    if gSupportNewXANIM then
	    build = string.gsub( build, "data/anims/characters/", "data/xanims/characters/" )
	    build = string.gsub( build, ".adef", ".xanm" )
	    build = string.gsub( build, ".abld", ".xanm" )
    end
	return KLEIResourceMgr.GetResource(build)
end

local function applyKanim( kanim, prop )
	if type(kanim.build) == "table" then
		for i,build in ipairs(kanim.build) do
			prop:bindBuild( GetAnimResource(build) )
		end
	elseif type(kanim.build) == "string" then
		prop:bindBuild( GetAnimResource(kanim.build) )
	end

	for i,anim in ipairs(kanim.anims) do
		prop:bindAnim( GetAnimResource(anim) )
	end

	for _,build in ipairs(kanim.wireframe or {}) do
		prop:bindWireframeBuild( GetAnimResource(build) )
	end
end

local function unapplyKanim( kanim, prop )
	if type(kanim.build) == "table" then
		for i,build in ipairs(kanim.build) do
			prop:unbindBuild( GetAnimResource(build) )
		end
	elseif type(kanim.build) == "string" then
		prop:unbindBuild( GetAnimResource(kanim.build) )
	end

	for i,anim in ipairs(kanim.anims) do
		prop:unbindAnim( GetAnimResource(anim) )
	end

	for _,build in ipairs(kanim.wireframe or {}) do
		prop:unbindWireframeBuild( GetAnimResource(build) )
	end
end

function agentrig:refreshAnim( unit )
	local weapon = simquery.getEquippedGun(unit)

	if weapon and weapon:getUnitData() ~= self._weaponUnitData then

		local rawUnit = unit:getSim():getUnit(unit:getID() )
		local unloadedAnims = {}
		if self._weaponUnitData then
			unapplyKanim( animmgr.lookupAnimDef( self._weaponUnitData.weapon_anim ), self._prop )
			util.tmerge( unloadedAnims, self._kanim[ self._weaponUnitData.agent_anim ] )
		elseif rawUnit and rawUnit:isNPC() then
			util.tmerge( unloadedAnims, self._kanim[ "anims_1h" ] )
		end

		for _,anim in pairs(unloadedAnims) do
			assert(GetAnimResource(anim), anim)
			self._prop:unbindAnim( GetAnimResource(anim) )
		end

		self._weaponUnitData = weapon:getUnitData()

		local loadedAnims = {}
		if self._weaponUnitData then
			applyKanim( animmgr.lookupAnimDef( self._weaponUnitData.weapon_anim ), self._prop )
			util.tmerge( loadedAnims, self._kanim[ self._weaponUnitData.agent_anim ] )		
		end

		for _,anim in pairs(loadedAnims) do
			assert(GetAnimResource(anim), anim)
			self._prop:bindAnim( GetAnimResource(anim) )
		end

	elseif not weapon then
		local rawUnit = unit:getSim():getUnit(unit:getID() )
		local unloadedAnims = {}
		if self._weaponUnitData then
			unapplyKanim( animmgr.lookupAnimDef( self._weaponUnitData.weapon_anim ), self._prop )
			util.tmerge( unloadedAnims, self._kanim[ self._weaponUnitData.agent_anim ] )

		elseif rawUnit and rawUnit:getPlayerOwner() and rawUnit:getPlayerOwner():isNPC() then
			util.tmerge( unloadedAnims, self._kanim[ "anims" ] )
		end

		for _,anim in pairs(unloadedAnims) do
			assert(GetAnimResource(anim), anim)
			self._prop:unbindAnim( GetAnimResource(anim) )
		end

		self._weaponUnitData = nil

		local loadedAnims = {}
		local defaultAnims = "anims"
		if rawUnit and rawUnit:getPlayerOwner() and rawUnit:getPlayerOwner():isNPC() and rawUnit:getTraits().vip and rawUnit:isAlerted() then
			defaultAnims = "anims_panic"
		end
		util.tmerge( loadedAnims, self._kanim[ defaultAnims ] )

		for _,anim in pairs(loadedAnims) do
			assert(GetAnimResource(anim), anim)
			self._prop:bindAnim( GetAnimResource(anim) )
		end

	end

    local movingBodyID = unit:getTraits().movingBody and unit:getTraits().movingBody:getID()
	if self._draggingBodyID ~= movingBodyID then
		if self._draggingBodyID then
			local bodyRig = self._boardRig:getUnitRig( self._draggingBodyID )
			if bodyRig then
				local grp_build = bodyRig._kanim.grp_build[1]
				self._prop:unbindBuild(GetAnimResource(grp_build))
                self._prop:setCurrentAnim( "dead" )
				local bodyUnit = bodyRig:getUnit()
				local gender = bodyUnit:getUnitData().gender or "male"
				if bodyUnit:getTraits().disguiseOn then
					gender = "male"
				end				
				self._prop:unbindAnim(GetAnimResource(self._kanim.grp_anims["anims"][gender]) )
				if self._weaponUnitData then
					self._prop:unbindAnim(GetAnimResource(self._kanim.grp_anims[self._weaponUnitData.agent_anim][gender]) )
				else
					self._prop:unbindAnim(GetAnimResource(self._kanim.grp_anims["anims_unarmed"][gender]) )
				end
			end
		end

		if movingBodyID then
			local bodyRig = self._boardRig:getUnitRig( movingBodyID )
			if bodyRig then
				local grp_build = bodyRig._kanim.grp_build[1]
				self._prop:bindBuild(GetAnimResource(grp_build))
				local bodyUnit = bodyRig:getUnit()
				local gender = bodyUnit:getUnitData().gender or "male"
				if bodyUnit:getTraits().disguiseOn then
					gender = "male"
				end
				self._prop:bindAnim(GetAnimResource(self._kanim.grp_anims["anims"][gender]) )
				if self._weaponUnitData then
					self._prop:bindAnim(GetAnimResource(self._kanim.grp_anims[self._weaponUnitData.agent_anim][gender]) )
				else
					self._prop:bindAnim(GetAnimResource(self._kanim.grp_anims["anims_unarmed"][gender]) )
				end
			end
		end

		self._draggingBodyID = movingBodyID
	end


end
