local util = include( "client_util" )
local cdefs = include( "client_defs" )
local mathutil = include( "modules/mathutil" )
local rig_util = include( "gameplay/rig_util" )
local mui_defs = include( "mui/mui_defs" )
local mui = include( "mui/mui" )
local simfactory = include( "sim/simfactory" )
local itemdefs = include("sim/unitdefs/itemdefs")
local agentdefs = include("sim/unitdefs/agentdefs")
local abilitydefs = include( "sim/abilitydefs" )

local agentsadded = include('client/fe/agentsadded')

local SET_COLOR = {r=244/255,g=255/255,b=120/255, a=1}

function agentsadded:updateItems( agentDef, agentWidget )
	local SECOND_ROW_OFFSET = 13
	local ITEMS_IN_ROW = 3
	local itemCount = #agentDef.upgrades
	
	for i, widget in agentWidget.binder:forEach( "item" ) do
		if agentDef.upgrades[i] then
			widget:setVisible(true)

			local unitData = itemdefs[ agentDef.upgrades[i] ]
			local newItem = simfactory.createUnit( unitData, nil )					  
			widget:setImage( unitData.profile_icon )

			local tooltip = util.tooltip( self.screen )
			local section = tooltip:addSection()
			newItem:getUnitData().onTooltip( section, newItem )
			widget:setTooltip( tooltip )
			widget:setColor(SET_COLOR.r,SET_COLOR.g,SET_COLOR.b,SET_COLOR.a)
			
			-- If there are more than 3 items, shift items up to make room for a second row
			if widget._rowOffset then
				local x, y = widget:getPosition()
				widget:setPosition( x, y - SECOND_ROW_OFFSET )
				widget._rowOffset = nil
			end
			if itemCount > ITEMS_IN_ROW then
				local x, y = widget:getPosition()
				widget:setPosition( x, y + SECOND_ROW_OFFSET )
				widget._rowOffset = SECOND_ROW_OFFSET
			end
		else
			widget:setVisible(false)
		end
	end
end

function agentsadded:populateProgram(programWidget, ability)
	local PWRtooltip = STRINGS.UI.TOOLTIPS.TEAM_SELECT_PWR_DESC

	programWidget.binder.programIcon:setImage(ability.icon)	 
	programWidget.binder.programName:setText(util.toupper(ability.name))
	programWidget.binder.programTxt:spoolText(ability.desc)
	if ability.cpu_cost > 0 or not ability.passive then
		programWidget.binder.powerTxt:setText(tostring(ability.cpu_cost))
		programWidget.binder.firewallTooltip:setTooltip(PWRtooltip..util.sformat(STRINGS.UI.TOOLTIPS.TEAM_SELECT_PWR, ability.cpu_cost))
	else
		programWidget.binder.powerTxt:setText("-")
		programWidget.binder.firewallTooltip:setTooltip(PWRtooltip..STRINGS.UI.TOOLTIPS.TEAM_SELECT_NOPWR)
	end
end

function agentsadded:refreshAgents()
	for i, agentWidget in self.panel:forEach( "agent" ) do
		if self.agency.unitDefs[ self.agentScroll + i ] then
			local agentDef = agentdefs[ self.agency.unitDefs[ self.agentScroll + i ].template ]
		
			agentWidget.binder.agent.binder.agentNameBody:setText(util.toupper(agentDef.name))
			agentWidget.binder.agent.binder.agentImg:setImage(agentDef.team_select_img[1])
			self:updateItems( agentDef, agentWidget.binder.agent )
		else
			agentWidget:setVisible(false)
		end
	end
end

function agentsadded:refreshPrograms()
	for i, programWidget in self.panel:forEach("Program") do
		if self.agency.abilities[ self.programScroll + i ] then
			local programDef = abilitydefs.lookupAbility( self.agency.abilities[ self.programScroll + i ] )
			
			self:populateProgram(programWidget.binder.Program, programDef)
		else
			programWidget:setVisible(false)
		end
	end
end

function agentsadded:init( agency )
	local screen = mui.createScreen( "modal-agents-added.lua" )
	mui.activateScreen( screen )
	self.isdone = false
	MOAIFmodDesigner.playSound( "SpySociety/HUD/gameplay/unlock_agent" )

	self.screen = screen
	self.agency = agency
	self.ok_button = screen:findWidget("okBtn.btn")
	self.panel = screen.binder.pnl.binder
	self.agentScroll = 0
	self.programScroll = 0
	
	self:refreshAgents()
	self:refreshPrograms()

	self.ok_button.onClick = function() self:Hide() end 
	self.ok_button:setText(STRINGS.UI.CONTINUE)
	
	inputmgr.addListener( self, 1 )
end

function agentsadded:IsDone()
	return self.isdone
end

function agentsadded:onInputEvent( event )
	if event.eventType == mui_defs.EVENT_KeyDown then
		if event.key == mui_defs.K_ENTER then
			self:Hide()
			return true

		elseif util.isKeyBindingEvent( "pause", event ) then
			self:Hide()
			return true
		end
	end
end

function agentsadded:Hide(  )
	inputmgr.removeListener( self )

	mui.deactivateScreen( self.screen ) 
	MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT ) 
	self.isdone = true
end

return agentsadded
