local util = include( "client_util" )
local mui_defs = include( "mui/mui_defs")
local world_hud = include( "hud/hud-inworld" )
local rig_util = include( "gameplay/rig_util" )
local level = include( "sim/level" )
local mainframe = include( "sim/mainframe" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )

local MODE_HIDDEN = 0
local MODE_VISIBLE = 1

------------------------------------------------------------------------------
-- Local functions

local function onClickAiAbility( panel, ability, abilityOwner, ai )
	panel._hud:transitionAbilityTarget( abilityOwner, abilityOwner, ai, ability )
end

local function updateAiButtonFromProgram( self, widget, ability, abilityOwner )
	local sim = self._hud._game.simCore
	local ai = sim:getNPC():hasMainframeAbility("W93_AI_assembly")

	widget.binder.powerTxt:setVisible(true)
	widget.binder.powerTxt:setColor(200/255,50/255,50/255)

	widget.binder.turnsTxt:setVisible(true)
	widget.binder.turnsTxt:setColor(200/255,50/255,50/255)

	if ability.identified then
		widget.binder.powerTxt:setText( ai.getPwrCost(ai, ability) or "-" )
		if not ai.getPwrCost(ai, ability) then
			widget.binder.powerTxt:setText("-")		
			widget.binder.turnsTxt:setVisible(false)
		end
		widget.binder.turnsTxt:setText(STRINGS.PROGRAMS.PWR)
	
		widget.binder.hazzardBG:setVisible(false)
		if ability.cooldown then
			if ability.cooldown > 0 then 
				widget.binder.turnsTxt:setVisible(true)
				widget.binder.turnsTxt:setText(STRINGS.PROGRAMS.TURNS)
				widget.binder.powerTxt:setVisible(true)
				widget.binder.powerTxt:setText( ability.cooldown )
				widget.binder.hazzardBG:setVisible(true)
			end		
		end
	else
		widget.binder.powerTxt:setText("?")		
		widget.binder.turnsTxt:setVisible(false)
	end
	
	widget:setAlias( ability.name )
	widget.binder.btn:setTooltip( function() return ai.onAbilityTooltip( ai, self._hud, sim, abilityOwner, ability ) end )
	if (sim:getPC():getTraits().aiToken <= 0) or (ai.HashTable[ability.type] > sim:getPC():getTraits().aiToken and ability.identified) or (type(ai.PWRtable[ability.type]) ~= "number" and ability.identified) or (ai.PWRtable[ability.type] > sim:getPC():getCpus() and ability.identified) then
		widget.binder.btn:setDisabled( true )
	else
		widget.binder.btn:setDisabled( false )
	end

	self:aiProgramWidgetSetColor( widget, ability, sim )

	widget.binder.btn.onClick = util.makeDelegate( nil, onClickAiAbility, self, ability, abilityOwner, ai )
		
	if ability.icon and ability.identified then
		widget.binder.img:setVisible(true)
		widget.binder.img:setImage( ability.icon )
	else
		widget.binder.img:setVisible(true)
		widget.binder.img:setImage( "gui/icons/ai_icons/icon-ai_unknown.png" )
	end

	widget:setVisible( self._hud:canShowElement( "mainframe" ) )
end 


local function updateAiProgramButtons( self, widgetName, player, primeRefresh )
	local panel = self._panel
	local sim = self._hud._game.simCore
	local ai = nil
	if sim:getNPC():findMainframeAbility("W93_AI_assembly") then
		ai = sim:getNPC():hasMainframeAbility("W93_AI_assembly")
	end

	for i=1, 15, 1 do
		panel.binder.aiProgramsPanel.binder["aiProgram"..i]:setVisible(false)
	end

	if ai then
		for i, ability in ipairs(ai.getAiAbilities(ai))do
			local widget = panel.binder.aiProgramsPanel.binder["aiProgram"..i]
			updateAiButtonFromProgram( self, widget, ability, player )
			widget:setVisible( true )
			if primeRefresh then
				widget:createTransition( "activate_above")
			end
		end
	end
end

------------------------------------------------------------------------------

local panel = class()

function panel:init( screen, hud )
	self._screen = screen
	self._hud = hud
	self._panel = screen.binder.aiPanel
	self._mode = MODE_HIDDEN
	
	self:hide()
end

function panel:aiProgramWidgetSetColor( widget, ability, sim, colorOb )
	local colorImg = util.color.WHITE
	local activeColor = colorOb
	local ai = sim:getNPC():hasMainframeAbility("W93_AI_assembly")

    	if not colorOb then
	    	if (sim:getPC():getTraits().aiToken <= 0) or (ai.HashTable[ability.type] > sim:getPC():getTraits().aiToken and ability.identified) or (type(ai.PWRtable[ability.type]) ~= "number" and ability.identified) or (ai.PWRtable[ability.type] > sim:getPC():getCpus() and ability.identified) then
            		colorOb, activeColor = util.color.DARK_RED, util.color.DARK_RED
            		colorImg = ability.color or util.color.DARK_RED
	    	else
		    	if ability.color then
                		colorOb, activeColor, colorImg = ability.color, util.color.RED, ability.color
		    	else 
                		colorOb, activeColor = util.color.RED, util.color.WHITE
		    	end
	    	end
    	end

	widget.binder.img:setColor(colorImg:unpack())

	widget.binder.btn:setColor(colorOb:unpack())
	widget.binder.btn:setColorInactive(colorOb:unpack())

	widget.binder.btn:setColorActive(activeColor:unpack())
	widget.binder.btn:setColorHover(activeColor:unpack())

	widget.binder.powerTxt:setColor(colorOb:unpack())
	widget.binder.turnsTxt:setColor(colorOb:unpack())
	widget.binder.digi:setColor( colorOb.r,colorOb.g,colorOb.b, 100/255 )
	widget.binder.descTxt:setColor(colorOb:unpack())	
end

function panel:hide()
	self._mode = MODE_HIDDEN
	self:refresh()
end

function panel:show()
	local localPlayer = self._hud._game:getLocalPlayer()
	if localPlayer == nil or self._hud._game.simCore:isGameOver() then
		return
	end

	if not self._hud:isMainframe() then
		self._hud:showMainframe()
	end

	self._mode = MODE_VISIBLE
	self:refresh(true)
end

function panel:toggle()
	if self._mode == MODE_HIDDEN then
		self:show()
	else
		self:hide()
	end
end

function panel:refresh(primeRefresh)
	if self._mode == MODE_HIDDEN or not self._hud:isMainframe() then
		self._panel.binder.aiProgramsPanel:setVisible( false )
		self._mode = MODE_HIDDEN
		self._hud._PE_ai = false
		if self._hud:isMainframe() then
			self._hud._mainframe_panel:show()
		end
		self._hud._home_panel:refresh()
		return
	end
	self._hud._PE_ai = true
	self._hud._mainframe_panel:hide()
	self._panel.binder.aiProgramsPanel:setVisible( true )
	self._hud._home_panel:refresh()
	updateAiProgramButtons( self, "program", self._hud._game.simCore:getNPC(), primeRefresh )
	self._panel.binder.aiPanel:setVisible( true )
end

function panel:onSimEvent( ev )
	if ev.eventType == "PE_TOGGLE_AIPANEL" then
		if ev.eventData.action == "CLOSE" then
			self:hide()
		elseif ev.eventData.action == "OPEN" then
			self:show()
		elseif ev.eventData.action == "TOGGLE" then
			self:toggle()
		end
	elseif ev.eventType == "PE_UPDATE_AI_INFO" or
		ev.eventType == simdefs.EV_HUD_REFRESH then
			self:refresh()
	elseif ev.eventType == simdefs.EV_TURN_END or
		ev.eventType == simdefs.EV_ITEMS_PANEL or
		ev.eventType == simdefs.EV_EXEC_DIALOG or
		ev.eventType == simdefs.EV_INSTALL_AUGMENT_DIALOG or
		ev.eventType == simdefs.EV_GRAFTER_DIALOG or
		(ev.eventType == simdefs.EV_LOOT_ACQUIRED and not ev.eventData.silent) or
		ev.eventType == simdefs.EV_FADE_TO_BLACK or
		ev.eventType == simdefs.EV_CMD_DIALOG then
			self:hide()
	end
end

return
{
	panel = panel
}