local serverdefs = include( "modules/serverdefs" )
local aiplayer = include( "sim/aiplayer" )
local mainframe_abilities = include( "sim/abilities/mainframe_abilities" )
local npc_abilities = include( "sim/abilities/npc_abilities" )
local simability = include( "sim/simability" )
local simdefs = include( "sim/simdefs" )
local util = include( "client_util" )

function aiplayer:onStartTurn( sim )
	local units = util.tdupe( self._units )
	for i,unit in ipairs( units ) do
		if unit:isValid() then
			unit:onStartTurn( sim )
		end
	end

	--if sim:getTags().clearPWREachTurn then			
	--	self:addCPUs( -self:getCpus( ), sim )
	--end

	if sim:getTurnCount() <= 1 then
		local counter_ai = sim:getParams().difficultyOptions.W93_AI or -1
		if ((counter_ai == 0 and (sim:getParams().world == "omni" or sim:getParams().world == "omni2")) or ( counter_ai <= sim:getParams().difficulty and counter_ai > 0 )) then
			sim:getNPC():addMainframeAbility( sim, "W93_AI_assembly", nil, 0 )
		end
	end
end

function aiplayer:getMaxCpus( )
	return math.max(self._maxCpus + (self._traits.PWRmaxBouns or 0),0)
end

function aiplayer:addMainframeAbility(sim, abilityID, hostUnit, reversalOdds )
	local monst3rReverseOdds = (reversalOdds or 10)
	local reflect = nil

	for i, ability in ipairs(sim:getPC():getAbilities()) do
		if ability:getID() == "W93_reflect" then
			reflect = ability
			break
		end
	end

	if monst3rReverseOdds > 0 then
		monst3rReverseOdds = monst3rReverseOdds + (sim:getPC():getTraits().globalReverseOdds or 0)
	end

	if self:isNPC() then 
		for _, ability in ipairs( sim:getPC():getAbilities() ) do 
			if ability.daemonReversalAdd and monst3rReverseOdds > 0 then 
				monst3rReverseOdds = monst3rReverseOdds + ability.daemonReversalAdd
			end 
		end 
	end

	local count = 0
	for _, ability in ipairs( self._mainframeAbilities ) do
		if ability:getID() == abilityID then
			count = count + 1
		end
	end

	local ability = simability.create( abilityID )

	if ability and count < (ability.max_count or math.huge) then
		local monst3rReverse = nil 

		if self:isNPC() and sim:nextRand( 1, 100 ) < monst3rReverseOdds then 
			monst3rReverse = true 
		elseif reflect and reflect:getCpuCost() <= sim:getPC():getCpus() and sim:getPC():getTraits().reverseAll then
			monst3rReverse = true
		end

		if monst3rReverse and (not sim:isVersion("0.17.5") or not ability.noDaemonReversal )  then  
			local newAbilityID = serverdefs.REVERSE_DAEMONS[ sim:nextRand( 1, #serverdefs.REVERSE_DAEMONS ) ] 
			ability = simability.create( newAbilityID )
			sim:triggerEvent( simdefs.TRG_DAEMON_REVERSE, ability )
		end 
		
		table.insert( self._mainframeAbilities, ability )
		ability:spawnAbility( self._sim, self, hostUnit )

		if self:isNPC() then			
			sim:dispatchEvent( simdefs.EV_MAINFRAME_INSTALL_PROGRAM, {idx = #self._mainframeAbilities, ability=ability} )	
			sim:triggerEvent( simdefs.TRG_DAEMON_INSTALL, ability )		
		end		
	end
end

function aiplayer:findMainframeAbility(abilityID)
	local testAbility = mainframe_abilities[abilityID]
	if not testAbility then
		testAbility = npc_abilities[abilityID]
	end
	return testAbility
end