local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local serverdefs = include( "modules/serverdefs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local store = include( "sim/units/store" )
local mainframe = include( "sim/mainframe" )

	function PE_updateAgentAbilityAI( remove, agentdefs )
		if remove then
			for i, agentDef in pairs(agentdefs) do
				local j = array.find(agentDef.abilities, "W93_combatAI_gainHash")
				if j then
					table.remove(agentDef.abilities, j)
				end
				j = array.find(agentDef.abilities, "W93_combatAI_gainHashHeavy")
				if j then
					table.remove(agentDef.abilities, j)
				end
				j = array.find(agentDef.abilities, "W93_combatAI_drainPWR")
				if j then
					table.remove(agentDef.abilities, j)
				end
			end
		else
			for i, agentDef in pairs(agentdefs) do
				if not array.find(agentDef.abilities, "W93_combatAI_gainHash") and array.find(agentDef.abilities, "jackin") then
					table.insert(agentDef.abilities, "W93_combatAI_gainHash")
				end
				if not array.find(agentDef.abilities, "W93_combatAI_gainHashHeavy") and array.find(agentDef.abilities, "jackin") then
					table.insert(agentDef.abilities, "W93_combatAI_gainHashHeavy")
				end
				if not array.find(agentDef.abilities, "W93_combatAI_drainPWR") and array.find(agentDef.abilities, "jackin") then
					table.insert(agentDef.abilities, "W93_combatAI_drainPWR")
				end
			end
		end
	end

	function PE_setColors()
		cdefs.TRACKER_COLOURS =
		{
			util.color.fromBytes( 250, 253, 105 ), -- 250 253 105
			util.color.fromBytes( 250, 253, 105 ),
			util.color.fromBytes( 251, 203, 98 ),
			util.color.fromBytes( 225, 152, 45 ),
			util.color.fromBytes( 246, 90, 21 ),
			util.color.fromBytes( 205, 24, 10 ),
			util.color.fromBytes( 205, 24, 10 ),
			util.color.fromBytes( 180, 15, 5 ),
			util.color.fromBytes( 180, 15, 5 ),
		}

		cdefs.COLOR_AI_WARNING = { r=255/255,g=216/255,b=0/255,a=255/255 }
	end

	function PE_resetServers()
		local serverroom_1 = include( "sim/prefabs/shared/serverroom_1" )
		local serverroom_2 = include( "sim/prefabs/shared/serverroom_2" )
		local serverroom_3 = include( "sim/prefabs/shared/serverroom_3" )

		serverroom_1.units =
		{
    			{
        			maxCount = 5,
        			spawnChance = 1,
        			{
            				{
                				x = 4,
                				y = 1,
                				template = [[console_core]],
                				unitData =
                				{
							facing = 0,
						},
            				},
            				1,
        			},
        			{
            				{
                				x = 3,
                				y = 3,
                				template = [[console]],
                				unitData =
                				{
                    				facing = 0,
                				},
            				},
            				1,
        			},
        			{
					{
                				x = 5,
                				y = 3,
                				template = [[console]],
                				unitData =
                				{
                    				facing = 4,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 5,
                				y = 5,
                				template = [[server_terminal]],
                				unitData =
                				{
                    					facing = 4,
                    					tags =
                    					{
                        					"serverFarm",
                    					},
                				},
            				},
            				1,
				},
			},
		}

		serverroom_2.units =
		{
    			{
        			maxCount = 5,
        			spawnChance = 1,
        			{
            				{
                				x = 9,
                				y = 6,
                				template = [[console_core]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 7,
                				y = 6,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 5,
                				y = 6,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 4,
                				},
            				},
            				1,
        			},
        			{
            				{
               					x = 6,
                				y = 3,
                				template = [[server_terminal]],
                				unitData =
                				{
                    					facing = 2,
                    					tags =
                    					{
                        					"serverFarm",
                    					},
                				},
            				},
            				1,
        			},
    			},
		}

		serverroom_3.units =
		{
    			{
        			maxCount = 5,
        			spawnChance = 1,
        			{
            				{
                				x = 6,
                				y = 8,
                				template = [[console_core]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 1,
                				y = 8,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 11,
                				y = 8,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 4,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 6,
                				y = 1,
                				template = [[server_terminal]],
                				unitData =
                				{
                    					facing = 2,
                    					tags =
                    					{
                        					"serverFarm",
                    					},
                				},
            				},
            				1,
        			},
    			},
		}
	end

	function PE_setServers()
		local serverroom_1 = include( "sim/prefabs/shared/serverroom_1" )
		local serverroom_2 = include( "sim/prefabs/shared/serverroom_2" )
		local serverroom_3 = include( "sim/prefabs/shared/serverroom_3" )

		serverroom_1.units =
		{
    			{
        			maxCount = 5,
        			spawnChance = 1,
        			{
            				{
                				x = 3,
                				y = 5,
                				template = [[extra_server_terminal]],
                				unitData =
                				{
							facing = 0,
						},
            				},
            				1,
        			},
        			{
            				{
                				x = 4,
                				y = 1,
                				template = [[console_core]],
                				unitData =
                				{
							facing = 0,
						},
            				},
            				1,
        			},
        			{
            				{
                				x = 3,
                				y = 3,
                				template = [[console]],
                				unitData =
                				{
                    				facing = 0,
                				},
            				},
            				1,
        			},
        			{
					{
                				x = 5,
                				y = 3,
                				template = [[console]],
                				unitData =
                				{
                    				facing = 4,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 5,
                				y = 5,
                				template = [[server_terminal]],
                				unitData =
                				{
                    					facing = 4,
                    					tags =
                    					{
                        					"serverFarm",
                    					},
                				},
            				},
            				1,
				},
			},
		}

		serverroom_2.units =
		{
    			{
        			maxCount = 5,
        			spawnChance = 1,
        			{
					{
                				x = 6,
                				y = 9,
                				template = [[extra_server_terminal]],
                				unitData =
                				{
							facing = 6,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 9,
                				y = 6,
                				template = [[console_core]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 7,
                				y = 6,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 5,
                				y = 6,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 4,
                				},
            				},
            				1,
        			},
        			{
            				{
               					x = 6,
                				y = 3,
                				template = [[server_terminal]],
                				unitData =
                				{
                    					facing = 2,
                    					tags =
                    					{
                        					"serverFarm",
                    					},
                				},
            				},
            				1,
        			},
    			},
		}

		serverroom_3.units =
		{
    			{
        			maxCount = 5,
        			spawnChance = 1,
        			{
            				{
                				x = 6,
                				y = 3,
                				template = [[extra_server_terminal]],
                				unitData =
                				{
                    					facing = 6,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 6,
                				y = 8,
                				template = [[console_core]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 1,
                				y = 8,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 0,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 11,
                				y = 8,
                				template = [[console]],
                				unitData =
                				{
                    					facing = 4,
                				},
            				},
            				1,
        			},
        			{
            				{
                				x = 6,
                				y = 1,
                				template = [[server_terminal]],
                				unitData =
                				{
                    					facing = 2,
                    					tags =
                    					{
                        					"serverFarm",
                    					},
                				},
            				},
            				1,
        			},
    			},
		}
	end

	function PE_updateBasePrograms()
		local simdefs = include( "sim/simdefs" )
		local mainframe_common = include("sim/abilities/mainframe_common")
		local mainframe_abilities_base = include("sim/abilities/mainframe_abilities")
		local npc_abilities = include("sim/abilities/npc_abilities")
		local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY
		local createDaemon = mainframe_common.createDaemon
		local createReverseDaemon = mainframe_common.createReverseDaemon
		local createCountermeasureInterest = mainframe_common.createCountermeasureInterest

	mainframe_abilities_base.fusion_17_10 = util.extend( mainframe_abilities_base.fusion_17_10 )
	{
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
        	value = 700,
	}

	mainframe_abilities_base.seed = util.extend( mainframe_abilities_base.seed )
	{
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
        	value = 1000,
	}

	mainframe_abilities_base.emergency_drip = util.extend( DEFAULT_ABILITY ) 
	{
		name = STRINGS.PROGRAMS.EMERGENCY_DRIP.NAME,
		desc = STRINGS.PROGRAMS.EMERGENCY_DRIP.DESC,
		shortdesc = STRINGS.PROGRAMS.EMERGENCY_DRIP.SHORT_DESC,
		huddesc = STRINGS.PROGRAMS.EMERGENCY_DRIP.HUD_DESC,
		icon = "gui/icons/programs_icons/Program0016.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_0016.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		cpu_cost = nil,
		credit_cost = 0, 
		value = 800,
        	MIN_CPU = 4,
        	cooldown = 0,        
        	maxCooldown = 3, 
        	passive = true,

		executeAbility = function( self, sim )
			local player = sim:getCurrentPlayer()			
			if not player:isNPC() and player:getCpus() < self.MIN_CPU and self.cooldown <= 0 then
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=util.sformat( STRINGS.PROGRAMS.EMERGENCY_DRIP.WARNING, 5 ), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
				player:addCPUs( 5 )
				self:setCooldown( sim )				
			end
		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )	
			sim:addTrigger( "TRG_PRE_START_TURN", self )	
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( "TRG_PRE_START_TURN", self )	
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == "TRG_PRE_START_TURN" and evData:isPC() then				
				if self.cooldown <= 0 then 
					self:executeAbility( sim )
				end 
			end
		end,
	}

	mainframe_abilities_base.pwr_manager = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.PWR_MANAGER.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.PWR_MANAGER.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.PWR_MANAGER.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.PWR_MANAGER.SHORT_DESC,
		tipdesc = STRINGS.PROGRAMS.PWR_MANAGER.TIP_DESC,
 		passive = true,
		icon = "gui/icons/programs_icons/Program0019.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_0019.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		value = 300,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )	
            		if not player:getTraits().extraStartingPWR then
            			player:getTraits().extraStartingPWR = 0
            		end
			if not player:getTraits().PWRmaxBouns then
				player:getTraits().PWRmaxBouns = 0
			end
            		player:getTraits().extraStartingPWR = player:getTraits().extraStartingPWR + 5
			player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns + 15
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )	
			player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns - 15
    		end,
	}

	mainframe_abilities_base.pwr_manager_2 = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.PWR_MANAGER_2.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.PWR_MANAGER_2.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.PWR_MANAGER_2.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.PWR_MANAGER_2.SHORT_DESC,
		tipdesc = STRINGS.PROGRAMS.PWR_MANAGER_2.TIP_DESC,
 		passive = true,
		icon = "gui/icons/programs_icons/Program0019.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_0019.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
        	value = 450,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )	
            		if not player:getTraits().extraStartingPWR then
            			player:getTraits().extraStartingPWR = 0
            		end
			if not player:getTraits().PWRmaxBouns then
				player:getTraits().PWRmaxBouns = 0
			end
            		player:getTraits().extraStartingPWR = player:getTraits().extraStartingPWR + 15
			player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns + 5
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )	
			player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns - 5
    		end,
	}

	mainframe_abilities_base.dynamo = util.extend( mainframe_abilities_base.dynamo )
	{
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
        	value = 700,
	}

	mainframe_abilities_base.root = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.ROOT.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.ROOT.DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.ROOT.SHORT_DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.ROOT.HUD_DESC,
		icon = "gui/icons/programs_icons/icon-program_Root.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_Root.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		value = 600,
		passive = true,
		
		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				local pwr = 2 - math.floor(evData:getCpus() / 8)
				evData:addCPUs( pwr )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.ROOT.WARNING, pwr), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.icon } )
			end 
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )		
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
    		end,
	}

	mainframe_abilities_base.faust = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.FAUST.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.FAUST.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.FAUST.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FAUST.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FAUST.TIP_DESC,
		lockedText = STRINGS.UI.TEAM_SELECT.UNLOCK_CENTRAL_MONSTER,

		icon = "gui/icons/programs_icons/icon-program-faust.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_faust.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		value = 600,
		passive = true,

		onSpawnAbility = function( self, sim, player  )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )	
			sim:addTrigger( simdefs.TRG_END_TURN, self )
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )	
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
    		end,

		executeAbility = function( self, sim )
			local player = sim:getCurrentPlayer()			
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGRAMS.FAUST.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			player:addCPUs( 2 )
		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				self:executeAbility(sim)
			elseif evType == simdefs.TRG_END_TURN and evData:isPC() then
				if evData:getCpus() % 5 == 0 and sim:getTracker() > 1 then
					local programList = nil
					local daemon = nil
					if sim:isVersion("0.17.5") then
						programList = sim:getIcePrograms()
						daemon = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
					else
						programList = serverdefs.PROGRAM_LIST
						daemon = programList[sim:nextRand(1, #programList)]			
					end	

					sim:getNPC():addMainframeAbility( sim, daemon )
				end 	
			end
		end,
	}

	mainframe_abilities_base.brimstone = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.BRIMSTONE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.BRIMSTONE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.BRIMSTONE.HUD_DESC,
		shortdesc = STRINGS.PROGRAMS.BRIMSTONE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.BRIMSTONE.TIP_DESC,

		icon = "gui/icons/programs_icons/Program0025.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_brimstone.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		cpu_cost = 3,
		break_firewalls = 1, 
		equip_program = true, 
		equipped = false, 
		value = 450,
		daemonReversalAdd = 0,
		custom_firewall_break = true,
		lockedText = STRINGS.UI.TEAM_SELECT.UNLOCK_CENTRAL_MONSTER,
		stack = 0,

		getFirewallBreak = function( self, sim, player, unit )
			return self.break_firewalls + (self.stack * 3)
		end,

		getCpuCost = function( self )
			local cpuCost = self.cpu_cost
			local zeroCpu = false

			if cpuCost == 0 then 
				zeroCpu = true 
			end

			if self._sim then
				local pwrModd = 0
				for i, ability in ipairs( self._sim:getPC():getAbilities()) do
					if ability.pwrMod then
						pwrModd = pwrModd + ability.pwrMod
					end
				end
				if zeroCpu then 
					cpuCost = math.max(0, cpuCost + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				else 
					cpuCost = math.max(1, cpuCost + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				end 
				cpuCost = cpuCost  + (pwrModd or 0) - self.stack

				if (self._sim:getTags().nextProgFree or 0) > 0 then
					cpuCost = cpuCost - 4
				end
			end

	   		return math.max(0, cpuCost)
		end,

		onSpawnAbility = function( self, sim, player  )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self )
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self )
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_DAEMON_INSTALL then
				self.stack = self.stack + 1				
			end
		end,

		executeAbility = function( self, sim, targetUnit )
			local firewallsToBreak = self.break_firewalls + (self.stack * 3)

			if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        		firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
			end

			self:useCPUs( sim )
	       		self:setCooldown( sim )
			self.stack = 0
			if targetUnit and (firewallsToBreak or 0) > 0 then
				mainframe.breakIce( sim, targetUnit, firewallsToBreak )
			end
		end,
	}

	mainframe_abilities_base.dataBlast = util.extend( mainframe_abilities_base.dataBlast )
	{
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
        	value = 700,
	}

	mainframe_abilities_base.feast = util.extend( mainframe_abilities_base.feast )
	{
        	desc = STRINGS.PROGEXTEND.PROGRAMS.FEAST.DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FEAST.SHORT_DESC,
		break_firewalls = 5,
	}

	mainframe_abilities_base.flare = util.extend( mainframe_abilities_base.flare )
	{
		desc = STRINGS.PROGEXTEND.PROGRAMS.FLARE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLARE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLARE.SHORT_DESC,
		tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC,
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)

			local targets = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getPlayerOwner() ~= sim:getPC() and unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getTraits().mainframe_status ~= "off" then
					table.insert(targets, unit)						
				end
			end
			
			if #targets > 0 then			
				local target = targets[math.floor(sim:nextRand()*#targets)+1]
				target:increaseIce(sim, 4)
			end
		end,
	}

	mainframe_abilities_base.fool = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.FOOL.NAME,
		desc = STRINGS.PROGRAMS.FOOL.DESC,
		huddesc = STRINGS.PROGRAMS.FOOL.HUD_DESC,
		shortdesc = STRINGS.PROGRAMS.FOOL.SHORT_DESC,
		tipdesc = STRINGS.PROGRAMS.FOOL.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_Jester.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_Jester.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		cooldown = 0,
		maxCooldown = 1,
		cpu_cost = 0,
		equip_program = true, 
		equipped = false, 
		value = 100,
		
		onSpawnAbility = function( self, sim, player  )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )	
			player:getTraits().daemonDurationModd = (player:getTraits().daemonDurationModd or 0) + 1
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )	
           		player:getTraits().daemonDurationModd = (player:getTraits().daemonDurationModd or 0) - 1
    		end,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)			
			targetUnit:processEMP( 1, false, false, true )

			local programList = nil
			local daemon = nil
			if sim:isVersion("0.17.5") then
				programList = sim:getIcePrograms()
				daemon = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
			else
				programList = serverdefs.PROGRAM_LIST
				daemon = programList[sim:nextRand(1, #programList)]			
			end	

			sim:getNPC():addMainframeAbility( sim, daemon )			
		end,			
	}

	mainframe_abilities_base.sniffer = util.extend( mainframe_abilities_base.sniffer )
	{
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
        	value = 100,
		maxCooldown = 1
	}

	mainframe_abilities_base.rogue = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.ROGUE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.ROGUE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.ROGUE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.ROGUE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.ROGUE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_rogue.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_rogue.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		cpu_cost = 4,
		cooldown = 0,
		maxCooldown = 2,		
		value = 600,

		canUseAbility = function( self, sim, abilityOwner )
			local player = sim:getCurrentPlayer()
			if player ~= abilityOwner or player == nil then
				return false
			end

			local availableSafes = {}
			for _, unit in pairs( sim:getAllUnits() ) do 
				if unit:getTraits().safeUnit and not unit:getTraits().security_box then 
					if unit:getPlayerOwner() ~= player and unit:getTraits().mainframe_status == "active" and not unit:getTraits().open then 
						table.insert( availableSafes, unit )
					end
				end
			end

			if #availableSafes == 0 then 
				return false, STRINGS.PROGRAMS.ROGUE.WARNING
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner )
		end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
			local availableSafes = {}

			for _, unit in pairs( sim:getAllUnits() ) do 
				if unit:getTraits().safeUnit and not unit:getTraits().security_box then 
					if unit:getPlayerOwner() ~= player and unit:getTraits().mainframe_status == "active" and not unit:getTraits().open then 
						table.insert( availableSafes, unit )
					end
				end
			end

			if #availableSafes > 0 then
				local safe = availableSafes[ sim:nextRand( 1, #availableSafes ) ]	            
				sim:getPC():glimpseUnit(sim, safe:getID() )
    				sim:dispatchEvent( simdefs.EV_CAM_PAN, { safe:getLocation() } )	
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_camera" )
				sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units={safe:getID()},reveal = true} )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*0.65)
            			self:useCPUs( sim )
            			self:setCooldown( sim )
				if safe:getTraits().mainframe_program then
					safe:getTraits().mainframe_program = nil
					sim:dispatchEvent( simdefs.EV_KILL_DAEMON, {unit = safe})
				
					if safe:getTraits().daemonHost then
						sim:getUnit(safe:getTraits().daemonHost):killUnit(sim)
						safe:getTraits().daemonHost = nil
					end
				end
            			mainframe.breakIce( sim, safe, safe:getTraits().mainframe_ice or 100 )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = safe } )
			end
		end,
	}

	mainframe_abilities_base.wildfire = util.extend( mainframe_abilities_base.wildfire )
	{
		desc = STRINGS.PROGEXTEND.PROGRAMS.WILDFIRE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.WILDFIRE.HUD_DESC,
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		value = 600,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
			self:setCooldown( sim )
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_run_fusion" )
            		self:useCPUs( sim )
			player:addCPUs( 5 )

			local targets = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getPlayerOwner() ~= sim:getPC() and unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getTraits().mainframe_status ~= "off" then
					table.insert(targets, unit)						
				end
			end
			
			if #targets > 0 then			
				local target = targets[math.floor(sim:nextRand()*#targets)+1]
        			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )				
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.WILDFIRE.WARNING, target:getName()), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.icon } )
				target:increaseIce(sim, 5)
			end
		end,
	}

	mainframe_abilities_base.wings = util.extend( mainframe_abilities_base.wings )
	{
		executeAbility = function( self, sim )
			self:setCooldown( sim )
            		self:useCPUs( sim )
            		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_wings_activate")

			for _, unit in pairs(sim:getPC():getUnits()) do
				if unit:getPlayerOwner() == sim:getPC() and simquery.isAgent( unit ) and unit:getTraits().mp then
					unit:getTraits().mp = unit:getTraits().mp + 2
					local x0,y0 = unit:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FORMATS.PLUS_AP,2),x=x0,y=y0,color={r=1,g=1,b=1,a=1}} )		
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
					sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.PROGRAMS.WINGS.NAME, x = x0, y = y0,color={r=255/255,g=255/255,b=51/255,a=1}} )	
				end
			end
		end
	}

	mainframe_abilities_base.charge = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.CHARGE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.CHARGE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.CHARGE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CHARGE.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program-charge.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_charge.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		passive = true,
		value = 1500,

		onSpawnAbility = function( self, sim, player )
			player:getTraits().program_cost_modifier = (player:getTraits().program_cost_modifier or 0) - 1
		end,

		onDespawnAbility = function( self, sim, player )
			player:getTraits().program_cost_modifier = player:getTraits().program_cost_modifier + 1
		end,

		canUseAbility = function( self, sim )
			return false 	
		end,
	}

	mainframe_abilities_base.overdrive = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGRAMS.OVERDRIVE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.OVERDRIVE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.OVERDRIVE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.OVERDRIVE.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program-overdrive.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_overdrive.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},
		coolDownMod = -1,
		passive = true,
		value = 1000,

		canUseAbility = function( self, sim )
			return false 	
		end,
	}

	npc_abilities.energize = util.extend( createReverseDaemon( STRINGS.PROGEXTEND.REVERSE_DAEMONS.FORTUNE ) )
	{
		icon = "gui/icons/daemon_icons/Program0021.png",
		dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},

		standardDaemon = false,
		reverseDaemon = true,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = true,

		onSpawnAbility = function( self, sim, player )
			self.duration = 10

			for i, unit in pairs(sim:getAllUnits()) do
				if unit:getTraits().safeUnit and unit:getPlayerOwner() ~= sim:getPC() and not unit:getTraits().open and unit:getTraits().mainframe_ice then
					if unit:getTraits().credits then
						unit:getTraits().credits = math.floor(unit:getTraits().credits * 1.5)
					end
				end
			end
			sim:dispatchEvent( simdefs.EV_SHOW_REVERSE_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
			sim:addTrigger( simdefs.TRG_END_TURN, self )	
		end,

		onDespawnAbility = function( self, sim )		
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
			for i, unit in pairs(sim:getAllUnits()) do
				if unit:getTraits().safeUnit and unit:getPlayerOwner() ~= sim:getPC() and not unit:getTraits().open and unit:getTraits().mainframe_ice then					
					if unit:getTraits().credits then
						unit:getTraits().credits = math.floor(unit:getTraits().credits / 1.5)
					end				
				end
			end	
		end,
	
		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,	
	}

	npc_abilities.alertPulse = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.PULSEV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_pulse.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			local pcplayer = sim:getPC()
			self.items = {}

			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc, } )	

			local pcplayer = sim:getPC()
			for i, unit in pairs( pcplayer:getUnits() ) do 
				for i, item in pairs( unit:getChildren() ) do 
					table.insert( self.items, item )
				end 
			end 

			for _, item in ipairs(self.items) do
				if item:getTraits().cooldownMax then 
					item:getTraits().cooldownMax = item:getTraits().cooldownMax + 3
				end 
			end	
		end,

		onDespawnAbility = function( self, sim, unit )
			for _, item in ipairs(self.items) do
				if item:getTraits().cooldownMax then 
					item:getTraits().cooldownMax = item:getTraits().cooldownMax - 3
				end 
			end
		end,	
	}

	npc_abilities.alertModulate = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.MODULATEV2 ) )
	{
		icon = "gui/icons/daemon_icons/Daemons00013.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc, } )
			if not sim:getPC():getTraits().program_cost_modifier then
				sim:getPC():getTraits().program_cost_modifier = 0
			end
            		sim:getPC():getTraits().program_cost_modifier = sim:getPC():getTraits().program_cost_modifier + 2
			sim:addTrigger( simdefs.TRG_END_TURN, self )	
		end,

		onDespawnAbility = function( self, sim )
            		sim:getPC():getTraits().program_cost_modifier = sim:getPC():getTraits().program_cost_modifier - 2
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
		end,	
	}

	npc_abilities.siphon = util.extend( createDaemon( STRINGS.DAEMONS.SIPHON ) )
	{
		icon = "gui/icons/daemon_icons/Daemons0005.png",

		onSpawnAbility = function( self, sim, player )
			self._cpu = math.min(sim:nextRand(2, 5), sim:getPC():getCpus())
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self._cpu ), } )	

			sim:getPC():addCPUs( -self._cpu )
			sim:getNPC():addCPUs( self._cpu )
			player:removeAbility(sim, self )
		end,

		onDespawnAbility = function( self, sim, unit )
		end,
	}

	npc_abilities.fortify = util.extend( createDaemon( STRINGS.DAEMONS.RUBIKS ) )
	{
		icon = "gui/icons/daemon_icons/Daemons0001.png",

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc, } )	
            		sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getPlayerOwner() ~= sim:getPC() then
					unit:increaseIce(sim,1)
				end
			end
			player:removeAbility(sim, self )
		end,

		onDespawnAbility = function( self, sim, unit )
		end,
	}

	end

	function PE_updateDlcPrograms()
		local simdefs = include( "sim/simdefs" )
		local mainframe_common = include("sim/abilities/mainframe_common")
		local mainframe_abilities_base = include("sim/abilities/mainframe_abilities")
		local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY

		mainframe_abilities_base.burst = util.extend( DEFAULT_ABILITY )
		{
			name = STRINGS.DLC1.PROGRAMS.BURST.NAME,
			desc = STRINGS.DLC1.PROGRAMS.BURST.DESC,
			huddesc = STRINGS.DLC1.PROGRAMS.BURST.HUD_DESC,
			shortdesc = STRINGS.DLC1.PROGRAMS.BURST.SHORT_DESC,
			tipdesc = STRINGS.DLC1.PROGRAMS.BURST.TIP_DESC,
			dlcFooter = { "Programs Extended","gui/icons/programs_extended.png","wodzu_93"},

			PROGRAM_LIST = 20,
			PROGRAM_NO_BREAKER_LIST = 20,

			icon = "gui/icons/programs_icons/icon-program_burst.png",
			icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_burst.png",
			cpu_cost = 0,
			credit_cost = 0,
			cooldown = 0,
			maxCooldown = 2,
			value = 800,

			onTrigger = function( self, sim, evType, evData )
				DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )		
				if evType == simdefs.TRG_START_TURN and evData:isPC() then
					if self.cooldown > 0 then 
						local player = sim:getCurrentPlayer()
						local skipQue = false
						for _, unit in ipairs(self._units) do
							if unit:getPlayerOwner() == player and simquery.isAgent( unit ) and unit:getTraits().mp then
								unit:getTraits().mp = unit:getTraits().mp -3
								local x0,y0 = unit:getLocation()

								sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat("<c:FFFF33>"..STRINGS.DLC1.PROGRAMS.BURST.WARNING .."</>\n"..STRINGS.FORMATS.AP,-3),x=x0,y=y0,color={r=1,g=1,b=1,a=1},skipQue=skipQue} )		
								skipQue = true

								sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )

								local x1, y1 = unit:getLocation()
								sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
							end
						end
					end 
				end		
			end,

			executeAbility = function( self, sim, unit, userUnit, targetCell )
				local player = sim:getCurrentPlayer()
				player:addCPUs( 3 )
				self:setCooldown( sim )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_gainCPU" )
            			self:useCPUs( sim )
            			self._units = {}

           			local skipQue = false
				for _, unit in pairs(sim:getAllUnits()) do
					if unit:getPlayerOwner() == player and simquery.isAgent( unit ) and unit:getTraits().mp then
						unit:getTraits().mp = unit:getTraits().mp +3
						local x0,y0 = unit:getLocation()
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FORMATS.PLUS_AP,3),x=x0,y=y0,color={r=1,g=1,b=1,a=1}} )		
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
						local x1, y1 = unit:getLocation()
						sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.DLC1.PROGRAMS.BURST.NAME, x = x1, y = y1,color={r=255/255,g=255/255,b=51/255,a=1},skipQue=skipQue} )	
						skipQue = true
						table.insert(self._units,unit)
					end
				end
			end,
		}
	end

	function PE_setShops()
		store.STORE_ITEM.storeType.server["progAmount"] = 8
		store.STORE_ITEM.storeType["extraserver"] =
		{
			itemAmount = 0, 
			progAmount = 6, 				
			weaponAmount = 0, 
			augmentAmount = 0, 				
			noBreakers = true
		}
	end

	function PE_setRarity( t )
		local adjustment = 0

		if type(t) == "boolean" and t == true then
			adjustment = 1
		end
		if type(t) == "number" then
			adjustment = t
		end
		if adjustment == 0 then return end

		for i=1,33,1 do
			store.STORE_ITEM.progList:removeChoice(1)
		end
		for i=1,7,1 do
			store.STORE_ITEM.progList_17_5:removeChoice(1)
		end
		for i=1,24,1 do
			store.STORE_ITEM.noBreakerProgList:removeChoice(1)
		end
		for i=1,5,1 do
			store.STORE_ITEM.noBreakerProgList_17_5:removeChoice(1)
		end

		if adjustment == nil or adjustment == 1 then
			store.STORE_ITEM.progList:addChoice( "lockpick_1", 20 )
			store.STORE_ITEM.progList:addChoice( "lockpick_2", 10 )
			store.STORE_ITEM.progList:addChoice( "dagger", 20 )
			store.STORE_ITEM.progList:addChoice( "dagger_2", 10 )
			store.STORE_ITEM.progList:addChoice( "parasite", 20 )
			store.STORE_ITEM.progList:addChoice( "parasite_2", 10 )
			store.STORE_ITEM.progList:addChoice( "rapier", 20 )
			store.STORE_ITEM.progList:addChoice( "dataBlast", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_2", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_3", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_4", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_5", 10 )
			store.STORE_ITEM.progList:addChoice( "hammer", 20 )
			store.STORE_ITEM.progList:addChoice( "brimstone", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "mercenary", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "flare", 5 )
			store.STORE_ITEM.progList_17_5:addChoice( "feast", 5 )
			store.STORE_ITEM.progList_17_5:addChoice( "rogue", 10 )

			store.STORE_ITEM.noBreakerProgList:addChoice( "remoteprocessor", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "emergency_drip", 5 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "fusion_17_10", 5 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "seed", 5 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "faust", 5 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "pwr_manager", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "pwr_manager_2", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wildfire", 5 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "dynamo", 5 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "root", 5 )

			store.STORE_ITEM.noBreakerProgList:addChoice( "hunter", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "mainframePing", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "esp", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "taurus", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "sniffer", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "oracle", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wings", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "shade", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "leash", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "lightning", 20 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "overdrive", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "charge", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "halt", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "fool", 10 )
		elseif adjustment == 2 then
			store.STORE_ITEM.progList:addChoice( "lockpick_1", 10 )
			store.STORE_ITEM.progList:addChoice( "lockpick_2", 10 )
			store.STORE_ITEM.progList:addChoice( "dagger", 10 )
			store.STORE_ITEM.progList:addChoice( "dagger_2", 10 )
			store.STORE_ITEM.progList:addChoice( "parasite", 10 )
			store.STORE_ITEM.progList:addChoice( "parasite_2", 10 )
			store.STORE_ITEM.progList:addChoice( "rapier", 10 )
			store.STORE_ITEM.progList:addChoice( "dataBlast", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_2", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_3", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_4", 10 )
			store.STORE_ITEM.progList:addChoice( "wrench_5", 10 )
			store.STORE_ITEM.progList:addChoice( "hammer", 10 )
			store.STORE_ITEM.progList:addChoice( "brimstone", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "mercenary", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "flare", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "feast", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "rogue", 10 )

			store.STORE_ITEM.progList:addChoice( "remoteprocessor", 10 )
			store.STORE_ITEM.progList:addChoice( "emergency_drip", 10 )
			store.STORE_ITEM.progList:addChoice( "fusion_17_10", 10 )
			store.STORE_ITEM.progList:addChoice( "seed", 10 )
			store.STORE_ITEM.progList:addChoice( "faust", 10 )
			store.STORE_ITEM.progList:addChoice( "pwr_manager", 10 )
			store.STORE_ITEM.progList:addChoice( "pwr_manager_2", 10 )
			store.STORE_ITEM.progList:addChoice( "wildfire", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "dynamo", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "root", 10 )

			store.STORE_ITEM.progList:addChoice( "hunter", 10 )
			store.STORE_ITEM.progList:addChoice( "mainframePing", 10 )
			store.STORE_ITEM.progList:addChoice( "esp", 10 )
			store.STORE_ITEM.progList:addChoice( "taurus", 10 )
			store.STORE_ITEM.progList:addChoice( "sniffer", 10 )
			store.STORE_ITEM.progList:addChoice( "oracle", 10 )
			store.STORE_ITEM.progList:addChoice( "wings", 10 )
			store.STORE_ITEM.progList:addChoice( "shade", 10 )
			store.STORE_ITEM.progList:addChoice( "leash", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "lightning", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "overdrive", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "charge", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "halt", 10 )
			store.STORE_ITEM.progList_17_5:addChoice( "fool", 10 )

			store.STORE_ITEM.noBreakerProgList:addChoice( "lockpick_1", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "lockpick_2", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "dagger", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "dagger_2", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "parasite", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "parasite_2", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "rapier", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "dataBlast", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wrench_2", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wrench_3", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wrench_4", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wrench_5", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "hammer", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "brimstone", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "mercenary", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "flare", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "feast", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "rogue", 10 )

			store.STORE_ITEM.noBreakerProgList:addChoice( "remoteprocessor", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "emergency_drip", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "fusion_17_10", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "seed", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "faust", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "pwr_manager", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "pwr_manager_2", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wildfire", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "dynamo", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "root", 10 )

			store.STORE_ITEM.noBreakerProgList:addChoice( "hunter", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "mainframePing", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "esp", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "taurus", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "sniffer", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "oracle", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "wings", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "shade", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "leash", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "lightning", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "overdrive", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "charge", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "halt", 10 )
			store.STORE_ITEM.noBreakerProgList_17_5:addChoice( "fool", 10 )
		end
	end

	function PE_setRarityDlc( t )
		local adjustment = 0

		if type(t) == "boolean" and t == true then
			adjustment = 1
		end
		if type(t) == "number" then
			adjustment = t
		end
		if adjustment == 0 then return end

		local dlc = include( "dlc1/mainframe_abilities")
		store.STORE_ITEM.progList:removeByItem( "golem" )
		store.STORE_ITEM.noBreakerProgList:removeByItem( "bless" )
		store.STORE_ITEM.noBreakerProgList:removeByItem( "burst" )
		store.STORE_ITEM.noBreakerProgList:removeByItem( "cycle" )
		store.STORE_ITEM.noBreakerProgList:removeByItem( "aces" )

		if adjustment == 1 then
			store.STORE_ITEM.progList:addChoice( "golem_17_10", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "bless", 20 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "burst", 5 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "cycle", 5 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "aces", 10 )
		elseif adjustment == 2 then
			store.STORE_ITEM.progList:addChoice( "golem_17_10", 10 )
			store.STORE_ITEM.progList:addChoice( "bless", 10 )
			store.STORE_ITEM.progList:addChoice( "burst", 10 )
			store.STORE_ITEM.progList:addChoice( "cycle", 10 )
			store.STORE_ITEM.progList:addChoice( "aces", 10 )

			store.STORE_ITEM.noBreakerProgList:addChoice( "golem_17_10", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "bless", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "burst", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "cycle", 10 )
			store.STORE_ITEM.noBreakerProgList:addChoice( "aces", 10 )
		end
	end