local simactions = include( "sim/simactions" )
local CHEATS = include( "fe/cheatmenu" )

local cheatActions = 
{
	spawnUnit = function( sim, templateName, x, y, hire )
		local util = include( "client_util" )
		local simfactory = include( "sim/simfactory" )
		local unitdefs = include( "sim/unitdefs" )
		local upgradedefs = include( "sim/upgradedefs" )
		
		local unitData = util.tcopy( unitdefs.lookupTemplate( templateName ) )
		
		for i, upgrade in ipairs(unitData.upgrades or {}) do
			if type(upgrade) == "string" then
				upgradedefs.applyUpgrade( upgrade, unitData )
			else
				upgradedefs.applyUpgrade( upgrade.upgradeName, unitData, upgrade.upgradeParams )
			end
		end
		
		local newItem = simfactory.createUnit( unitData, sim )
		
		if newItem:getTraits().isGuard then
			newItem:setPlayerOwner(sim:getNPC() )
			newItem:setPather(sim:getNPC().pather)
		elseif newItem:getTraits().isAgent then
			newItem:setPlayerOwner(sim:getPC() )
		end
		
		newItem:getTraits().installed = nil		
		local cell = sim:getCell( x, y )
		if cell then
			sim:spawnUnit( newItem )
			sim:warpUnit( newItem, cell )
		end
		if newItem:getTraits().isGuard then
			sim:getNPC():returnToIdleSituation(newItem)
		end
		
		if hire then
			local agentID = unitData.agentID
			if not sim:getPC()._deployed[ agentID ] then
				sim:getPC()._deployed[ agentID ] = { 
					agentDef = {
						id = agentID,
						template = templateName,
						upgrades = {},
					}
				}
			end
			
			sim:getPC()._deployed[ agentID ].id = newItem:getID()
		end
	end,
	
	
	
	spawnProgram = function( sim, programName )
		local simquery = include("sim/simquery")
		local maxPrograms = simquery.getMaxPrograms( sim )

		if #sim:getPC():getAbilities() < maxPrograms then
			sim:getPC():addMainframeAbility( sim, programName )
		end
	end,
	
	
	addCpuPoints = function( sim )
		sim:getPC():addCPUs(1)
	end,

	addMoney = function( sim )
		sim:getPC():addCredits(100)
	end,

	addAlarm = function( sim )
		sim:trackerAdvance(1)
	end,

	maxAlarm = function( sim )
		sim:trackerAdvance( math.max( 1, sim.getDefs().TRACKER_MAXCOUNT - sim:getTracker() ))
	end,
	
	simCreateInterest = function( sim, unitID, x, y )
		local unit = sim:getUnit( unitID )
		if unit and unit:getBrain() then
			unit:getBrain():spawnInterest(x, y, sim:getDefs().SENSE_DEBUG, sim:getDefs().REASON_NOTICED)
		end
	end,

	simKillOthers = function( sim, x, y )
		local cell = sim:getCell( x, y )
		local survivor = cell and cell.units[1]
		if survivor then
			sim:forEachUnit(function(unit)
				if unit:getPlayerOwner() == survivor:getPlayerOwner() and unit ~= survivor then
					unit:killUnit(sim)
				end
			end)
		end
	end,
	
	simWin = function( sim )
		sim:win()
	end,

	seeEverything = function( sim )
		local cells = {}
		sim:forEachCell(function(cell)
			sim:getPC():glimpseCell( sim, cell )
			table.insert( cells, cell.x )
			table.insert( cells, cell.y )
		end)
		sim:forEachUnit(function(u)
			sim:getPC():glimpseUnit( sim, u:getID() )
		end)
		sim:dispatchEvent( sim:getDefs().EV_LOS_REFRESH, { player = sim:getCurrentPlayer(), cells = cells } )
	end,

	spawnProgram = function( sim, programName )
		local simquery = include("sim/simquery")
        local maxPrograms = simquery.getMaxPrograms( sim )

        if #sim:getPC():getAbilities() < maxPrograms then
            sim:getPC():addMainframeAbility( sim, programName )
        end
	end,

	spawnDaemon = function( sim, daemonName )
		sim:getNPC():addMainframeAbility(sim, daemonName, sim:getNPC() )
	end,
	
	teleportSelected = function( sim, unitID, x, y )
		local unit = sim:getUnit( unitID )
		local cell = sim:getCell( x, y )
		if cell and cell ~= sim:getCell( unit:getLocation() ) then
			sim:warpUnit(unit, cell)
		end
		sim:processReactions(unit)
	end,

	simKill = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			cell.units[1]:killUnit( sim )
		end
	end,

	simKO = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			cell.units[1]:setKO(sim, 3)
		end
	end,
	
	
	deleteAIroutine = function( sim, routineType, routineName )
		local assembly = sim:getNPC():hasMainframeAbility("W93_AI_assembly")
		if assembly then
			if routineType == "PWR" or routineType == "PWR2" then
				for i, routine in pairs(assembly.slotsPWR) do
					if routine.name == routineName then
						if routine.removeAbility then
							routine.removeAbility( assembly, sim )
						end
						table.remove(assembly.slotsPWR, i)
						break
					end
				end
			elseif routineType == "PROACTIVE" then
				for i, routine in pairs(assembly.slotsPro) do
					if routine.name == routineName then
						if routine.removeAbility then
							routine.removeAbility( assembly, sim )
						end
						table.remove(assembly.slotsPro, i)
						break
					end
				end
			elseif routineType == "REACTIVE" then
				for i, routine in pairs(assembly.slotsRea) do
					if routine.name == routineName then
						if routine.removeAbility then
							routine.removeAbility( assembly, sim )
						end
						table.remove(assembly.slotsRea, i)
						break
					end
				end
			else
				for i, routine in pairs(assembly.slotsUti) do
					if routine.name == routineName then
						if routine.removeAbility then
							routine.removeAbility( assembly, sim )
						end
						table.remove(assembly.slotsUti, i)
						break
					end
				end
			end
			sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
		else
			log:write("Warning: Attempt to delete AI subroutine when no Assembly Ability is installed.")
		end
	end,
	
	
	spawnAIroutine = function( sim, routineType, routineName )
		local assembly = sim:getNPC():hasMainframeAbility("W93_AI_assembly")
		if assembly then
			local util = include("modules/util")
		
			local onSpawnAbility = assembly.onSpawnAbility
			local subroutines = upvalueUtil.find( onSpawnAbility, "subroutines" )
		
			if subroutines then
				for k, routine in pairs(subroutines[routineType]) do
					if routine.name == routineName then
						routine = util.tdupe(routine)
						routine.identified = false
						
						if routineType == "PWR" or routineType == "PWR2" then
							table.insert(assembly.slotsPWR, routine)
						elseif routineType == "PROACTIVE" then
							routine.cooldown = 0
							table.insert(assembly.slotsPro, routine)
							assembly.activePrograms = assembly.activePrograms + 1
						elseif routineType == "REACTIVE" then
							routine.cooldown = 0
							table.insert(assembly.slotsRea, routine)
							assembly.activePrograms = assembly.activePrograms + 1
						else
							table.insert(assembly.slotsUti, routine)
						end
						
						if routine.onSpawnAbility then
							routine.onSpawnAbility( assembly, sim, routine )
						end
						
						sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
						break
					end
				end
			else
				log:write("Finding upvalue subroutines failed!")
			end
			sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
		else
			log:write("Warning: Attempt to create AI subroutine when no Assembly Ability is installed.")
		end
	end,
	
	
	addHashKey = function( sim, x, y )
		sim:getPC():getTraits().aiToken = (sim:getPC():getTraits().aiToken or 0) + 1
		sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
	end,
	
	drainAIPWR = function( sim, x, y )
		sim:getNPC():addCPUs( -1 )
		sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
	end,
	
	addAIPwr = function( sim, x, y )
		sim:getNPC():addCPUs( 1 )
		sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
	end,
	
	clearAI = function( sim )
		local assembly = sim:getNPC():hasMainframeAbility("W93_AI_assembly")
		if assembly then
			local util = include("modules/util")
			local routines = util.tmerge({},assembly.slotsPWR,assembly.slotsPro,assembly.slotsRea,assembly.slotsUti)
			for i, routine in pairs(routines) do
				if routine.removeAbility then
					routine.removeAbility( assembly, sim )
				end
			end
			util.tclear(assembly.slotsPWR)
			util.tclear(assembly.slotsPro)
			util.tclear(assembly.slotsRea)
			util.tclear(assembly.slotsUti)
			sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
		else
			log:write("Warning: Attempt to delete AI subroutine when no Assembly Ability is installed.")
		end
	end,
	
	
	rotateUnit = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			local unit = cell.units[1]
			local simdefs = include( "sim/simdefs" )
			unit:updateFacing((unit:getFacing() + 1) % simdefs.DIR_MAX)
		end
	end,
	
	despawnUnit = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			local unit = cell.units[1]
			sim:warpUnit(unit,nil)
			unit:destroyTab()
			sim:despawnUnit(unit)
		end
	end,
	
	
	makeChildrenCarryable = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			for i, child in pairs(cell.units[1]:getChildren()) do
				if not child:hasAbility( "carryable" ) then
					child:giveAbility("carryable")
				end
				child:getTraits().installed = false
			end
		end
	end,
	
	
	clearPatrol = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			local unit = cell.units[1]
			unit:getTraits().patrolPath = nil
			unit:getBrain():reset()
			sim:processReactions()
		end
	end,
	
	addPatrolPoint = function( sim, unitID, x, y )
		local cell = sim:getCell( x, y )
		if cell then
			local unit = sim:getUnit( unitID )
			if not unit:getTraits().patrolPath then
				unit:getTraits().patrolPath = {}
			end
			table.insert( unit:getTraits().patrolPath, {x = x, y = y} )
			sim:processReactions()
		end
	end,
	
	addPatrolPointWithFacing = function( sim, unitID, x, y )
		local cell = sim:getCell( x, y )
		if cell then
			local unit = sim:getUnit( unitID )
			if not unit:getTraits().patrolPath then
				unit:getTraits().patrolPath = {}
			end
			table.insert( unit:getTraits().patrolPath, {x = x, y = y, facing = unit:getFacing()} )
			sim:processReactions()
		end
	end,
				
	generateStationaryPath = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			local unit = cell.units[1]
			if unit and unit:getBrain() then
				local player = unit:getPlayerOwner()
				if player:isNPC() and player:getIdleSituation() == unit:getBrain():getSituation() then
					player:getIdleSituation():generateStationaryPath( unit, unit:getLocation() )
				end
			end
		end
	end,


	levelSkill = function( sim, x, y, skillId )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			local unit = cell.units[1]
			local skill = unit:getSkills()[skillId]
			if skill then
				skill:levelUp( sim, unit )
			end
		end
	end,

	levelAll = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			local unit = cell.units[1]
			for i, skill in ipairs(unit:getSkills()) do
				while skill:getCurrentLevel() < skill:getNumLevels() do
					skill:levelUp( sim, unit )
				end
			end
		end
	end,

	clearSkills = function( sim, x, y )
		local cell = sim:getCell( x, y )
		if cell and cell.units[1] then
			for i, skill in ipairs(cell.units[1]:getSkills()) do
				while skill:getCurrentLevel() > 1 do
					skill:levelDown( sim, cell.units[1] )
				end
			end
		end
	end,
}

CHEATS.cheatActions = cheatActions

simactions.cheatAction = function( sim, fnName, ... )
	assert( CHEATS.cheatActions[fnName] )
	CHEATS.cheatActions[fnName]( sim, ... )
end