local util = include( "client_util" )
local unitdefs = include( "sim/unitdefs" )
local simdefs = include( "sim/simdefs" )

local CHEATS = include( "fe/cheatmenu" )

local cheat_item = class()

function cheat_item:init( name, fn, ... )
    self.name = name
    self.fn = fn
    self.fnParams = { ... }
end

function cheat_item:getName()
    return self.name
end

function cheat_item:createWidget( cheatmenu )
    local widget = cheatmenu.listbox:addItem( self )
    widget.binder.txt:setText( self:getName() )
    return widget
end

function cheat_item:onActivate( cheatmenu )
    cheatmenu:executeBinding( self:getName(), self.fn, self.fnParams )
end

local cheat_toggle = class( cheat_item )

function cheat_toggle:init( name, toggleFn )
    assert( type(toggleFn.get) == "function" and type(toggleFn.set) == "function" )

    cheat_item.init( self, name )
    self.toggleFn = toggleFn
end

function cheat_toggle:createWidget( cheatmenu )
    local widget = cheatmenu.listbox:addItem( self )
    setfenv( self.toggleFn.get, cheatmenu.debugenv )
    if self.toggleFn.get() then
        widget.binder.txt:setText( "[X] "..self:getName() )
    else
        widget.binder.txt:setText( "[ ] "..self:getName() )
    end
    return widget
end

function cheat_toggle:onActivate( cheatmenu )
    cheatmenu:executeBinding( self:getName(), self.toggleFn.set, { not self.toggleFn.get() } )
    cheatmenu:refreshListbox()
end

function cheat_toggle:onClick( cheatmenu )
    self:onActivate( cheatmenu )
end


local cheat_submenu = class( cheat_item )

function cheat_submenu:init( name, submenu )
    self.name = name
    self.submenu = submenu
end

function cheat_submenu:getName()
    return self.name .. " >>"
end

function cheat_submenu:getChild( i )
    if type(self.submenu[i]) == "string" then
        return cheat_item( self.submenu[i] )
    else
        return self.submenu[i]
    end   
end

function cheat_submenu:onClick( cheatmenu )
    self:onActivate( cheatmenu )
end

function cheat_submenu:onActivate( cheatmenu )
    cheatmenu:pushMenu( self )
end

local spawn_menu = class( cheat_submenu )

function spawn_menu:init( name, defs )
    local submenu = {}
    for k, unitData in pairs( defs ) do
        table.insert( submenu, cheat_item( string.format( "%s (%s)", k, unitData.name or "MISSING NAME" ), CHEATS.spawnUnit, k ))
    end
    table.sort( submenu, function( s1, s2 ) return s1:getName() < s2:getName() end )

    cheat_submenu.init( self, name, submenu )
end

local hire_menu = class( cheat_submenu )

function hire_menu:init( name, defs )
    local submenu = {}
    for k, unitData in pairs( defs ) do
        table.insert( submenu, cheat_item( string.format( "%s (%s)", k, unitData.name or "MISSING NAME" ), CHEATS.spawnUnit, k, true ))
    end
    table.sort( submenu, function( s1, s2 ) return s1:getName() < s2:getName() end )

    cheat_submenu.init( self, name, submenu )
end

local spawn_program_menu = class( cheat_submenu )

function spawn_program_menu:init( name )
    local submenu = {}
    local mainframe_abilities = include( "sim/abilities/mainframe_abilities" )
    for programName, ability in pairs( mainframe_abilities ) do
        table.insert( submenu, cheat_item( string.format( "%s (%s)", programName, ability.name ), CHEATS.spawnProgram, programName ))
    end
    table.sort( submenu, function( s1, s2 ) return s1:getName() < s2:getName() end )

    cheat_submenu.init( self, name, submenu )
end

local spawn_daemon_menu = class( cheat_submenu )

function spawn_daemon_menu:init( name )
    local submenu = {}
    local npc_abilities = include( "sim/abilities/npc_abilities" )
    for daemonName, ability in pairs( npc_abilities ) do
        table.insert( submenu, cheat_item( string.format( "%s (%s)", daemonName, ability.name ), CHEATS.spawnDaemon, daemonName ))
    end
    table.sort( submenu, function( s1, s2 ) return s1:getName() < s2:getName() end )

    cheat_submenu.init( self, name, submenu )
end

local spawn_location_menu = class( cheat_submenu )

function spawn_location_menu:init( name )
    local submenu = {}
    local serverdefs = include( "modules/serverdefs" )
    for j, corpName in pairs( serverdefs.CORP_NAMES ) do
        for i, tag in ipairs( serverdefs.ESCAPE_MISSION_TAGS ) do
            local tags = { tag, corpName }
            table.insert( submenu, cheat_item( corpName.." "..tag, CHEATS.spawnLocation, tags ))
        end
    end
    table.sort( submenu, function( s1, s2 ) return s1:getName() < s2:getName() end )

    cheat_submenu.init( self, name, submenu )
end

CHEATS.deleteAIroutine = function( routineType, routineName )
    if sim then
        game:doAction( "cheatAction", "deleteAIroutine", routineType, routineName )
    end
end

CHEATS.spawnAIroutine = function( routineType, routineName )
    if sim then
        game:doAction( "cheatAction", "spawnAIroutine", routineType, routineName )
    end
end

CHEATS.spawn_ai_program_menu = class( cheat_submenu )

function CHEATS.spawn_ai_program_menu:init( name )
    local submenu = {}
	
    local npc_abilities = include( "sim/abilities/npc_abilities" )
    
	for programName, ability in pairs( npc_abilities ) do
	
        if (programName == "W93_AI_assembly") then
			local onSpawnAbility = ability.onSpawnAbility
			local subroutines = upvalueUtil.find( onSpawnAbility, "subroutines" )
			
			if subroutines then
				for routineType, routineList in pairs(subroutines) do
					for k, routine in pairs(routineList) do
						table.insert( submenu, cheat_item( string.format( "[%s] (%s)", routineType, routine.name ), CHEATS.spawnAIroutine, routineType, routine.name ))
					end
				end
			end
		end
	end

    cheat_submenu.init( self, name, submenu )
end

CHEATS.delete_ai_program_menu = class( cheat_submenu )

function CHEATS.delete_ai_program_menu:init( name )
    local submenu = {}
	
    local npc_abilities = include( "sim/abilities/npc_abilities" )
    
	for programName, ability in pairs( npc_abilities ) do
        if (programName == "W93_AI_assembly") then
			local onSpawnAbility = ability.onSpawnAbility
			local subroutines = upvalueUtil.find( onSpawnAbility, "subroutines" )
			
			if subroutines then
				for routineType, routineList in pairs(subroutines) do
					for k, routine in pairs(routineList) do
						table.insert( submenu, cheat_item( string.format( "[%s] (%s)", routineType, routine.name ), CHEATS.deleteAIroutine, routineType, routine.name ))
					end
				end
			end
		end
	end

    cheat_submenu.init( self, name, submenu )
end

local function onItemSelected( cheatmenu, old_idx, new_idx, cheatItem )
    local item = cheatmenu.listbox:getItem( old_idx )
    if item then
        item.widget.binder.bg:setColor( 0, 0, 0, 0.4 )
    end

    local item = cheatmenu.listbox:getItem( new_idx )
    if item then
        item.widget.binder.bg:setColor( 1, 0, 0, 0.4 )
    end
    
    -- Preserve last selected index for this menu
    local menu = cheatmenu.menuStack[ #cheatmenu.menuStack ]
    cheatmenu.lastSelection[ menu ] = new_idx
end


local function onItemClicked( cheatmenu, idx, cheatItem )
    if cheatItem.onClick then
        cheatmenu.debugenv:updateEnv()
        cheatItem:onClick( cheatmenu, cheatmenu.debugenv )
    end
end


function CHEATS.menu:init( screen, debugenv, statedebug )
	self.statedebug = statedebug
    self.screen = screen
    self.debugenv = debugenv
    self.listbox = screen.binder.listbox
    self.listbox.onItemSelected = util.makeDelegate( nil, onItemSelected, self )
    self.listbox.onItemClicked = util.makeDelegate( nil, onItemClicked, self )

    self.menuStack = {}
    self.lastSelection = {}
	
	local CHEATS = {
		cheat_item( "Unlock Rewards", CHEATS.unlockAllRewards ),
		cheat_item( "Reveal All", CHEATS.seeEverything ),
		cheat_item( "Teleport Unit (Selected)", CHEATS.teleportSelected ),
		cheat_item( "Kill Unit (Mouse Over)", CHEATS.simKill ),
		cheat_item( "KO Unit (Mouse Over)", CHEATS.simKO ),
		cheat_submenu( "Spawn Units",
			{
				spawn_menu( "Items", unitdefs.tool_templates ),
				spawn_menu( "Props", unitdefs.prop_templates ),
				spawn_menu( "Guards", unitdefs.npc_templates ),
				spawn_menu( "Agents", unitdefs.agent_templates ),
			}),
		spawn_program_menu( "Spawn Programs" ),
		spawn_daemon_menu( "Spawn Daemons" ),
		spawn_location_menu( "Spawn Location" ),
		cheat_submenu( "UI",
			{
				cheat_toggle( "Show UI", CHEATS.toggleUI ),
				cheat_item( "Print Widgets (Mouse Over)", CHEATS.printWidgets ),
			}),
		cheat_submenu( "Programmer",
			{
				cheat_toggle( "Track Leaks", CHEATS.toggleLeakTracking )
			})
	}
	util.tmerge(CHEATS,rawget(simdefs,"CHEATS"))

    self:pushMenu( cheat_submenu( "*", CHEATS ) )
    self:refresh()
end

CHEATS.cheat_item = cheat_item
CHEATS.hire_menu = hire_menu
CHEATS.cheat_submenu = cheat_submenu
CHEATS.cheat_toggle = cheat_toggle
CHEATS.spawn_menu = spawn_menu
CHEATS.spawn_daemon_menu = spawn_daemon_menu
CHEATS.spawn_location_menu = spawn_location_menu
CHEATS.spawn_program_menu = spawn_program_menu