local cdefs = include( "client_defs" )
local simquery = include( "sim/simquery" )
local util = include( "modules/util" )
local combat_panel = include( "hud/combat_panel" )

function combat_panel:refreshShoot( weaponUnit, unit, targetUnit )
	assert( unit and targetUnit )

	local COLOR_BLUE_DARK = util.color(50/255,87/255,87/255,150/255)
	local COLOR_BLUE_LIGHT = util.color(150/255,255/255,255/255,1)

	local COLOR_RED_DARK = util.color(80/255,0/255,0/255,150/255)
	local COLOR_RED_LIGHT = util.color(163/255,0/255,0/255,1)

	local sim = self._hud._game.simCore
	local color, textColor = COLOR_BLUE_DARK, COLOR_BLUE_LIGHT
	local shot = sim:getQuery().calculateShotSuccess( sim, unit, targetUnit, weaponUnit )

	local immune = false
	if shot.ko and not targetUnit:getTraits().canKO then 
		immune = true 
	end 

	local targetTxt = targetUnit:getName()
	if targetUnit:isKO() then
		targetTxt = STRINGS.UI.COMBAT_PANEL_KILL_KO_TARGET .." ".. targetTxt
	end
	self._panel.binder.targetTxt:setText( targetTxt )

	if weaponUnit:getTraits().restrictedUse then
		local userUnitAgentID = unit:getUnitData().agentID
		local canUse = false
		for i, set in pairs(weaponUnit:getTraits().restrictedUse ) do
			if set.agentID == userUnitAgentID then
				canUse=true
			end
		end
		if not canUse then
			self._panel.binder.targetTxt:setText( STRINGS.UI.REASON.RESTRUCTED_USE )		
			color = COLOR_RED_DARK
			textColor = COLOR_RED_LIGHT
		end
	end

	if not sim:canUnitSeeUnit( unit, targetUnit ) and not weaponUnit:getTraits().xray then 
		self._panel.binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_NO_LOS )		
		color = COLOR_RED_DARK
		textColor = COLOR_RED_LIGHT
	elseif unit:countAugments( "augment_kinetic_dynamo_mod" ) > 0 then
		self._panel.binder.damageTxt:setText( STRINGS.FUNCLIB.UI.COMBAT_PANEL_CANNOT_SHOOT )		
		color = COLOR_RED_DARK
		textColor = COLOR_RED_LIGHT
	elseif weaponUnit:getTraits().canTag then
		self._panel.binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_TAG )
	elseif immune then 
		self._panel.binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_IMMUNE )
		color = COLOR_RED_DARK
		textColor = COLOR_RED_LIGHT		
	elseif shot.armorBlocked then 
		self._panel.binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_ARMORED )
		color = COLOR_RED_DARK
		textColor = COLOR_RED_LIGHT		
	elseif shot.ko then
		self._panel.binder.damageTxt:setText( util.sformat( STRINGS.FORMATS.AMT_KO, shot.damage ))
	elseif not shot.ko and weaponUnit:getTraits().canEmp then
		self._panel.binder.damageTxt:setText( util.sformat( STRINGS.FUNCLIB.UI.COMBAT_PANEL_EMP, weaponUnit:getTraits().modeEmpdamage ) )
	else
		self._panel.binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_KILL )
	end

	self._panel:setVisible( true )
	self._panel.binder.hitBackground:setColor(color:unpack())  
	self._panel.binder.targetTxt:setColor(textColor:unpack())
	self._panel.binder.damageTxt:setColor(textColor:unpack())
end

function combat_panel:refreshMelee( abilityOwner, unit, targetUnit, weaponUnit )
    	assert( unit and targetUnit )

	local sim = self._hud._game.simCore
	local color, textColor = util.color(50/255,87/255,87/255,150/255), util.color(150/255,255/255,255/255,1)
    	local meleeAbility = nil
	local ok, reason1, reason2 = nil, nil, nil
	if not weaponUnit then
		weaponUnit = simquery.getEquippedMelee( unit )
		meleeAbility = unit:hasAbility( "melee" )
		assert( meleeAbility )
		ok, reason1, reason2 = meleeAbility:canUseAbility( sim, unit, unit, targetUnit:getID())
	else
		meleeAbility = weaponUnit:hasAbility( weaponUnit:getTraits().weaponAbility )
		assert( meleeAbility )
		ok, reason1, reason2 = meleeAbility:canUseAbility( sim, weaponUnit, unit, targetUnit:getID())
	end

    	if ok then
    		local koDamage = simquery.calculateMeleeDamage(sim, weaponUnit, targetUnit)
		if weaponUnit:getTraits().lethalMelee then
			self._panel.binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_KILL )
		else
			self._panel.binder.damageTxt:setText( util.sformat( STRINGS.FORMATS.AMT_KO, koDamage ))
		end
    		self._panel.binder.targetTxt:setText( targetUnit:getName())
	else
        	color, textColor = util.color(80/255,0/255,0/255,150/255), util.color(163/255,0/255,0/255,1)
		self._panel.binder.damageTxt:setText( reason1 )
		self._panel.binder.targetTxt:setText( reason2 or targetUnit:getName() )
    	end

	self._panel:setVisible( true )
	self._panel.binder.hitBackground:setColor(color:unpack())  
	self._panel.binder.targetTxt:setColor(textColor:unpack())
	self._panel.binder.damageTxt:setColor(textColor:unpack())
end