local util = include( "modules/util" )
local skilldefs = include( "sim/skilldefs" )

local function onGuardTooltip( tooltip, unit )
	if unit:getTraits().hypnoticPattern then
		tooltip:addAbility( util.sformat(STRINGS.WIZARD.TOOLTIPS.HYPNOTIZED, unit:getTraits().hypnoticPattern), util.sformat(STRINGS.WIZARD.TOOLTIPS.HYPNOTIZED_DESC, unit:getTraits().hypnoticPattern), "gui/icons/spells/hypnotic_gaze.png" )
	end
	if unit:getTraits().trueStrike and unit:getTraits().trueStrike > 0 then
		tooltip:addAbility( util.sformat(STRINGS.WIZARD.TOOLTIPS.TRUE_STRIKE, unit:getTraits().trueStrike), util.sformat(STRINGS.WIZARD.TOOLTIPS.TRUE_STRIKE_DESC, unit:getTraits().trueStrike), "gui/icons/thought_icons/status_engaged.png" )
	end
	local enfeeblement = unit.getModifiers and unit:getModifiers():has( "LOSrange", "enfeeblement" )
	if enfeeblement then
		tooltip:addAbility( STRINGS.WIZARD.TOOLTIPS.FEEBLE, util.sformat(STRINGS.WIZARD.TOOLTIPS.FEEBLE_DESC, -enfeeblement.val), "gui/icons/spells/enfeeblement.png" )
	end
end

local function onAgentTooltip( tooltip, unit )
	if unit:getTraits().enhanceAbility then
		local skill = skilldefs.lookupSkill( unit:getTraits().enhanceAbility.skillId )
		tooltip:addAbility( util.sformat( STRINGS.WIZARD.TOOLTIPS.ENHANCED, util.toupper( skill.name ), unit:getTraits().enhanceAbility.level ), util.sformat( STRINGS.WIZARD.TOOLTIPS.ENHANCED_DESC, skill.name, unit:getTraits().enhanceAbility.level ), "gui/icons/spells/enhance_ability.png" )
	end
	
	if unit:getTraits().hasteAbility then
		local mpBonus = unit:getTraits().hasteAbility.mpBonus
		local sprintBonus = unit:getTraits().hasteAbility.sprintBonus
		local mpPenalty = unit:getTraits().hasteAbility.mpBonus + unit:getTraits().hasteAbility.mpPenalty
		tooltip:addAbility( STRINGS.WIZARD.TOOLTIPS.HASTE, util.sformat( STRINGS.WIZARD.TOOLTIPS.HASTE_DESC, mpBonus, sprintBonus, mpPenalty ), "gui/icons/spells/haste.png" )
	end
	
	local penalty = unit:getTraits().hastePenalty
	if penalty then
		if penalty.penalizeApNextTurn and penalty.penalizeMPNextTurn <= 0 then
			tooltip:addAbility( STRINGS.WIZARD.TOOLTIPS.HASTE_RECOVERY_ATTACK, STRINGS.WIZARD.TOOLTIPS.HASTE_RECOVERY_ATTACK_DESC, "gui/icons/spells/haste.png" )
		elseif penalty.penalizeMPNextTurn > 0 and not penalty.penalizeApNextTurn then
			tooltip:addAbility( STRINGS.WIZARD.TOOLTIPS.HASTE_RECOVERY_MP, util.sformat( STRINGS.WIZARD.TOOLTIPS.HASTE_RECOVERY_MP_DESC, math.ceil(penalty.penalizeMPNextTurn) ), "gui/icons/spells/haste.png" )
		elseif penalty.penalizeApNextTurn and penalty.penalizeMPNextTurn > 0 then
			tooltip:addAbility( STRINGS.WIZARD.TOOLTIPS.HASTE_RECOVERY, util.sformat( STRINGS.WIZARD.TOOLTIPS.HASTE_RECOVERY_DESC, math.ceil(penalty.penalizeMPNextTurn) ), "gui/icons/spells/haste.png" )
		end
	end
end

return 
{
	onGuardTooltip = onGuardTooltip,
	onAgentTooltip = onAgentTooltip,
	--onItemTooltip = onItemTooltip,
}