local util = include( "modules/util" )
local array = include( "modules/array" )
local simdefs = include( "sim/simdefs" )
local propdefs = include( "sim/unitdefs/propdefs" )

propdefs.console.onWorldTooltip = function( tooltip, unit, hud )
	tooltip:addLine( unit:getName() )

	if (unit:getTraits().cpus or 0) > 0 then
        	local selectedUnit = hud:getSelectedUnit()
        	if selectedUnit then
            		local ability = selectedUnit:hasAbility( "jackin" )
            		if ability then
				tooltip:addAbility( ability:getName( unit:getSim(), selectedUnit, selectedUnit, unit:getID()), STRINGS.ABILITIES.HIJACK_CONSOLE_DESC, "gui/items/icon-action_hack-console.png" )
			end
	    	end
	end

	if unit:getTraits().noPWRconvert then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.REPRIME_PENALTY, STRINGS.FUNCLIB.UI.REPRIME_PENALTY_DESC,  "gui/icons/action_icons/Action_icon_Small/icon-action_lock_small.png" )
	end
end

local function onAgentTooltip( tooltip, unit )
 	if unit:getTraits().wounds and unit:getTraits().woundsMax and unit:getTraits().woundsMax > 1 and unit:getTraits().woundsMax > unit:getTraits().wounds then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.DERMAL_ARMOR, string.format(STRINGS.FUNCLIB.UI.DERMAL_ARMOR_DESC, unit:getTraits().woundsMax - unit:getTraits().wounds - 1),  "gui/hud3/hud3_shieldflag.png" )
	end
end

local function onGuardTooltip( tooltip, unit )
	if unit:getSim():getParams().difficultyOptions.W93_FuncLib_Tooltips then
		local armorPiercing = array.findIf( unit:getChildren(), function( u ) 
			if u:getTraits().armorPiercing ~= nil and u:getTraits().armorPiercing > 0 then
				return u:getTraits().armorPiercing
			else return nil end end )

		local baseDamage = array.findIf( unit:getChildren(), function( u ) 
			if u:getTraits().baseDamage ~= nil and not u:getTraits().canSleep then
				return u:getTraits().baseDamage
			else return nil end end )

		if ((baseDamage and baseDamage:getTraits().baseDamage > 1) or (armorPiercing and armorPiercing:getTraits().armorPiercing)) then
			tooltip:addAbility( STRINGS.FUNCLIB.UI.HEAVY_DAMAGE, util.sformat(STRINGS.FUNCLIB.UI.HEAVY_DAMAGE_DESC, (baseDamage and baseDamage:getTraits().baseDamage or 0), (armorPiercing and armorPiercing:getTraits().armorPiercing or 0)),  "gui/icons/action_icons/Action_icon_Small/icon-item_overwatch_small.png" )
		end
	end
end

local function onItemTooltip( tooltip, unit, userUnit )
	if unit:getTraits().baseDamage and unit:getTraits().baseDamage > 1 and not unit:getTraits().canSleep then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.HEAVY_DAMAGE2, util.sformat(STRINGS.FUNCLIB.UI.HEAVY_DAMAGE_DESC2, unit:getTraits().baseDamage),  "gui/icons/action_icons/Action_icon_Small/icon-item_overwatch_small.png" )
	end

	if unit:getTraits().ignoreArmor and unit:getTraits().ignoreArmor == true and unit:getTraits().melee then
		tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.IGNORES_ARMOR, STRINGS.ITEMS.TOOLTIPS.IGNORES_ARMOR_DESC, "gui/icons/arrow_small.png" )
	end

	if unit:getTraits().proxyKObonus and unit:getTraits().proxyRange and unit:getTraits().proxyRange > 0 then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.PROXY_KO, util.sformat(STRINGS.FUNCLIB.UI.PROXY_KO_DESC1, unit:getTraits().proxyKObonus, math.abs(unit:getTraits().proxyRange)),  "gui/icons/skills_icons/skills_icon_small/icon-item_accuracy_small.png" )
	elseif unit:getTraits().proxyKObonus and unit:getTraits().proxyRange and unit:getTraits().proxyRange < 0 then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.PROXY_KO, util.sformat(STRINGS.FUNCLIB.UI.PROXY_KO_DESC2, unit:getTraits().proxyKObonus, math.abs(unit:getTraits().proxyRange)),  "gui/icons/skills_icons/skills_icon_small/icon-item_accuracy_small.png" )
	end

	if unit:getTraits().soundRange then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.SOUND_RANGE, util.sformat(STRINGS.FUNCLIB.UI.SOUND_RANGE_DESC, unit:getTraits().soundRange),  "gui/icons/device_icons/sound_bug.png" )
	end

	if unit:getTraits().xray then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.XRAY, STRINGS.FUNCLIB.UI.XRAY_DESC, "gui/icons/action_icons/Action_icon_Small/icon-action_scanner_small.png" )
	end

	if unit:getTraits().cpuTurn and unit:getTraits().cpuTurnMax and unit:getTraits().cpuTurnMax > 1 and not unit:getTraits().installed then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.LAPTOP_CYCLE, util.sformat(STRINGS.FUNCLIB.UI.LAPTOP_CYCLE_DESC, unit:getTraits().cpuTurnMax, unit:getTraits().cpuTurn),  "gui/icons/item_icons/icon-item_cloaking_rig.png" )
	end

	if unit:getTraits().doorDevice == "simtrap" and unit:getTraits().stun and unit:getTraits().ignoreArmor ~= nil then
		if unit:getTraits().armorPiercing and unit:getTraits().armorPiercing > 0 then
			tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING, string.format(STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_DESC, unit:getTraits().armorPiercing), "gui/icons/arrow_small.png" )
		elseif unit:getTraits().ignoreArmor == true then
			tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.IGNORES_ARMOR, STRINGS.ITEMS.TOOLTIPS.IGNORES_ARMOR_DESC, "gui/icons/arrow_small.png" )
		else
			tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_NONE, STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_NONE_DESC, "gui/icons/arrow_small.png" )
		end
	end

	if unit:getTraits().ammoRequired then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.AMMO_REQUIRED, util.sformat(STRINGS.FUNCLIB.UI.AMMO_REQUIRED_DESC, unit:getTraits().ammoRequired),  "gui/icons/action_icons/Action_icon_Small/icon-item_reload_small.png" )
	end

	if unit:getTraits().usesLeft then
        	tooltip:addAbility( STRINGS.FUNCLIB.UI.BREAKABLE, util.sformat(STRINGS.FUNCLIB.UI.BREAKABLE_DESC, unit:getTraits().usesLeft), "gui/icons/arrow_small.png" )
    	end
end

local function onItemWorldTooltip( tooltip, unit )
	if unit:getTraits().cpuTurn and unit:getTraits().cpuTurnMax and unit:getTraits().cpuTurnMax > 1 then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.LAPTOP_CYCLE, util.sformat(STRINGS.FUNCLIB.UI.LAPTOP_CYCLE_DESC, unit:getTraits().cpuTurnMax, unit:getTraits().cpuTurn),  "gui/icons/item_icons/icon-item_cloaking_rig.png" )
	end
end

local function onMainframeTooltip( tooltip, unit )
 	if unit:getTraits().wounds and unit:getTraits().woundsMax and unit:getTraits().woundsMax > 1 and unit:getTraits().woundsMax > unit:getTraits().wounds then
		tooltip:addAbility( STRINGS.FUNCLIB.UI.DERMAL_ARMOR, string.format(STRINGS.FUNCLIB.UI.DERMAL_ARMOR_DESC, unit:getTraits().woundsMax - unit:getTraits().wounds - 1),  "gui/hud3/hud3_shieldflag.png" )
	end
end

return 
{
	onAgentTooltip = onAgentTooltip,
	onGuardTooltip = onGuardTooltip,
	onItemTooltip = onItemTooltip,
	onItemWorldTooltip = onItemWorldTooltip,
	onMainframeTooltip = onMainframeTooltip,
}