----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )
local inventory = include( "sim/inventory" )
local abilitydefs = include( "sim/abilitydefs" )

-----------------------------------------------------
-- Local functions

local daemon_server = { ClassType = "daemon_server" }

function daemon_server:onWarp( sim, oldcell, cell )
	if oldcell then
		self:deactivate( sim )
	end

	if cell and self:getTraits().startOn == true then
		self:activate( sim )
	end
end

function daemon_server:activate( sim )
	if  self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"

		if self:getSounds().spot_old then
			self:getSounds().spot = self:getSounds().spot_old		
			self:getSounds().spot_old = nil
		end

		sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_on" )
	end
end

function daemon_server:deactivate( sim )	
	sim:triggerEvent( simdefs.TRG_UNIT_DISABLED, self )

	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "inactive"

		self:getSounds().spot_old = self:getSounds().spot		
		self:getSounds().spot = nil			

		sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		self:destroyTab()
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_off" )
	end
end

function daemon_server:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_ALARM_STATE_CHANGE then

		if not sim:hasTag( "scannedAtAlarmLevel"..evData ) then

			sim:getTags()["scannedAtAlarmLevel"..evData] = true

	            	local dialogParams =
	            	{
				STRINGS.WORLDEXTEND.UI.DAEMON_SERVER_HEAD,
	                	STRINGS.WORLDEXTEND.UI.DAEMON_SERVER_TITLE,
	                	STRINGS.WORLDEXTEND.UI.DAEMON_SERVER_TXT,
	                	"gui/icons/item_icons/icon-item_FTM_scanner.png",
	                	color = {r=1,g=0,b=0,a=1}
	            	}
			sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )

			local possibleUnits = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if unit:getTraits().mainframe_item and unit:getPlayerOwner() ~= sim:getPC() and not unit:getTraits().mainframe_program then
					table.insert( possibleUnits, unit )		
				end
			end

			if #possibleUnits > 0 then 
				local index = sim:nextRand(1, #possibleUnits)
				local unit = possibleUnits[ index ]
				table.remove( possibleUnits, index )

				if sim:isVersion("0.17.5") then
					local programList = sim:getIcePrograms()
					unit:getTraits().mainframe_program = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
				else
					unit:getTraits().mainframe_program = PROGRAM_LIST[ sim:nextRand(1, #PROGRAM_LIST) ]
				end	

				sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = unit, ice = unit:getTraits().mainframe_ice, delta = 0} )
			end 

			local daemon = nil
			if sim:isVersion("0.17.5") then
				daemon = sim:getIcePrograms():getChoice( sim:nextRand( 1, sim:getIcePrograms():getTotalWeight() ))
			else
				daemon = programList[sim:nextRand(1, #serverdefs.PROGRAM_LIST)]			
			end	

			sim:getNPC():addMainframeAbility( sim, daemon )
		end
	end
end

function daemon_server:removeChild( childUnit )
    simunit.removeChild( self, childUnit )
    if childUnit:getTraits().ammo_clip then
	self:processEMP( nil, false, false, true )
        self:getSim():triggerEvent( simdefs.TRG_UNIT_DISABLED, self )
    end
end

-----------------------------------------------------
-- Interface functions

local function createDaemonServer( unitData, sim )
	return simunit.createUnit( unitData, sim, daemon_server )
end

simfactory.register( createDaemonServer )

return
{
	createDaemonServer = createDaemonServer,
}