					
local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local STRINGS = include("strings")

-- local hasplotarmor = {[99] = true, [100] = true, [107] = true, [108] = true, }
local hasplotarmor = {}
local exempt_situations = {
	["mid_1"] = true,
	["mid_2"] = true,
	["ending_1"] = true,
}

local abilitydeathtracker =
{

	name = "deathtracker", 
	getName = function( self, sim, unit )
		return self.name
	end,

	createToolTip = nil,

	profile_icon = nil,

	alwaysShow = false,

	canUseWhileDragging = true,

	usesAction = false,


	canUseAbility = function( self, sim, unit, userUnit, targetID )
		return true
	end,

	-- executeAbility = function( self, sim, unit, userUnit, target )

	-- end,

	onSpawnAbility = function( self, sim, unit )
		self.abilityOwner = unit
		hasplotarmor = {[99] = true, [100] = true, [107] = true, [108] = true, }
		
		local isEndless = sim:getParams().difficultyOptions.maxHours == math.huge
		local params = sim:getParams()
		if isEndless then
			-- if params.difficultyOptions.endless_mid_mission then -- check if it's Endless mode and midway mission is enabled 
			-- log:write("LOG: endless mid mission")
				if sim:getTags().needPowerCells == true then -- check that midway mission has already happened! we do not need Monst3r and Central anymore
					hasplotarmor = nil
					hasplotarmor = {}
				end
			-- else
				-- hasplotarmor = nil
				-- hasplotarmor = {}
				-- log:write("LOG: no endless mid mission")
			-- end
		end
		
		-- log:write("LOG: onspawnability")
		
		if sim:getParams().difficultyOptions.deathtimer and sim:getParams().difficultyOptions.deathtimer > 0 then
		
			if self.abilityOwner:getTraits().canBeCritical == false then
				self.abilityOwner:getTraits().canBeCritical = true
			end
		end
		
		sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		sim:addTrigger( simdefs.TRG_START_TURN, self )
		sim:addTrigger( simdefs.TRG_UNIT_KO, self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		
	end,


	onDespawnAbility = function( self, sim, unit)
		sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		self.abilityOwner = nil
	end,

	-- followDeathTracker = function( self, sim )
		-- if not sim:getPC() then return end
		-- for i, unit in pairs(sim:getPC():getUnits()) do
			-- if unit:getTraits().isDying then
				-- dyingunit = unit
				-- local x1, y1 = dyingunit:getLocation()
				-- if x1 and y1 then 
					 -- script:queue( { type="hideHUDInstruction" } )
					 -- script:queue( { type="displayHUDInstruction", text="Medical attention required", subtext="Turns left: ".. unit:getTraits().isDying, x=x1, y=y1 } )
				-- end
			-- end
		-- end
	-- end,
	
	onTrigger = function( self, sim, evType, evData )
		if sim:getParams().difficultyOptions.permadeath and (sim:getParams().difficultyOptions.permadeath == true) then
		
		local script = sim:getLevelScript()
		
		if evType == simdefs.TRG_UNIT_KO then
		local woundedunit = self.abilityOwner
			if evData.unit and woundedunit:isPC() and woundedunit:isDead() then
				
				-- log:write("LOG: agent is KO")	
			
			-- log:write("LOG: agentID")
			-- log:write(util.stringize(woundedunit._unitData.agentID,2))
				
				if sim:getParams().difficultyOptions.deathtimer and sim:getParams().difficultyOptions.deathtimer > 0 and not
				( (hasplotarmor[woundedunit:getUnitData().agentID] or (woundedunit:getTraits().monst3r or woundedunit:getTraits().central )) 
				and
				exempt_situations[sim:getParams().situationName] )
				and not woundedunit:getTraits().transistorKO and woundedunit:getTraits().isDying == nil then
					-- log:write("LOG inducing death timer")
					woundedunit:getTraits().isDying = sim:getParams().difficultyOptions.deathtimer
					if woundedunit:getTraits().deathtimer_heavydamage then
						woundedunit:getTraits().isDying = woundedunit:getTraits().deathtimer_heavydamage
					end
					x1, y1 = woundedunit:getLocation() 
					script:queue( { type="displayHUDInstruction", text="Medical attention required", subtext="Turns left: ".. woundedunit:getTraits().isDying, x=x1, y=y1 } )
					if sim:getParams().difficultyOptions.permadeath_beeper then
						script:queue( { soundPath="SpySociety/Actions/guard/guard_heart_stage1", type="operatorVO" } )
					end
					-- replace with proper strings later
					-- STRINGS.UI.TOOLTIP_AGENT_DOWN_DESC = util.sformat(STRINGS.PERMADEATH.TOOLTIP_AGENT_DYING_DESC, woundedunit:getTraits().isDying)
				end

			end
			
			if evData.unit and woundedunit:isPC() and evData.ticks == nil and not woundedunit:isDead() then
				if woundedunit:getTraits().isDying then
					woundedunit:getTraits().isDying = nil
					woundedunit:getTraits().deathtimer_heavydamage = nil
					script:queue( { type="hideHUDInstruction" } )
				end
				-- log:write("LOG: reviving unit")
			end
			
		elseif evType == simdefs.TRG_START_TURN and evData:isPC() then
			-- log:write("LOG: start of turn trigger")
			local dying = nil
			for k,u in pairs(sim:getPC():getUnits()) do
				if u:getTraits().isDying then
					dying = true --super blunt check
					break
				end
			end
			if dying == nil then
				script:queue( { type="hideHUDInstruction" } ) --lazy fix to remove 'stuck' HUD tips after dying agent teleports out
			end
			
			local dyingunit = self.abilityOwner
			dyingunit:getTraits().friendlyPin = nil
			-- log:write("self.unitOwner")
			-- log:write(util.stringize(dyingunit,2))
			
			if dyingunit:getTraits().isDying and dyingunit:getLocation() then
			
				if dyingunit:getTraits().isDying > 0 then
					
					local cell = sim:getCell( dyingunit:getLocation() )
					if not cell then
						return
					end
					for i, cellUnit in ipairs(cell.units) do
						if cellUnit ~= nil and cellUnit:getTraits().isAgent and (cellUnit:getPlayerOwner() == sim:getPC() ) and not cellUnit:getTraits().isDrone and not cellUnit:isKO() and not cellUnit:getTraits().iscorpse then
					
							-- log:write("Friendlypin")
							dyingunit:getTraits().friendlyPin = true
						end
					end
					
					if not dyingunit:getTraits().friendlyPin then
						dyingunit:getTraits().isDying = dyingunit:getTraits().isDying - 1
						log:write("death ticks modified")
						if dyingunit:getTraits().isDying > 0 then
							if sim:getParams().difficultyOptions.permadeath_beeper then
								script:queue( { soundPath="SpySociety/Actions/guard/guard_heart_stage1", type="operatorVO" } )
							end
						end
					else
						local x1, y1 = dyingunit:getLocation()
						if x1 and y1 then
							sim:dispatchEvent(	simdefs.EV_UNIT_FLOAT_TXT, {
							unit = evData.target,
							txt = util.sformat(STRINGS.PERMADEATH.AGENT_STABILISED),
							x = x1, y = y1,
							color = {r = 255/255, g = 255/255, b = 51/255, a = 1 },
							} )
						end
					end
					
					sim:forEachUnit(
					function(unit)
						if unit:getTraits().isDying then
							dyingunit = unit
							local x1, y1 = dyingunit:getLocation()
							script:queue( { type="hideHUDInstruction" } )
							script:queue( { type="displayHUDInstruction", text="Medical attention required", subtext="Turns left: ".. dyingunit:getTraits().isDying, x=x1, y=y1 } )
							-- STRINGS.UI.TOOLTIP_AGENT_DOWN_DESC = util.sformat(STRINGS.PERMADEATH.TOOLTIP_AGENT_DYING_DESC, dyingunit:getTraits().isDying)
						end
					end)
				end
				
				if dyingunit:getTraits().isDying == 0 then
					if sim:getParams().difficultyOptions.permadeath_beeper then
						script:queue( { soundPath="SpySociety/Actions/guard/guard_heart_flatline", type="operatorVO" } )
					end
					dyingunit:getTraits().canBeCritical = false
					dyingunit:onDamage(1)
					log:write("killed unit")
					script:queue( { type="hideHUDInstruction" } )	
					
					
				end
			end
			
			
		elseif evType == simdefs.TRG_UNIT_KILLED then
			-- log:write("LOG unit death")
			-- log:write(util.stringize(evData.unit._unitData.agentID,1))		
			if evData.unit._unitData.agentID and evData.unit:getTraits().corpseTemplate then
				sim.permadeathkilled = sim.permadeathkilled or {}
				sim.permadeathkilled[evData.unit._unitData.agentID] = true
				log:write("LOG permadeath permadeathkilled1")
				log:write(util.stringize(sim.permadeathkilled,2))
			end
			-- save them in a table so we can check at mission end if we should remove them from the detention pool
			-- log:write(util.stringize(sim.permadeathkilled,1))	
			
		elseif (evType == simdefs.TRG_UNIT_WARP and evData.unit:getTraits().isDying) then
			local dyingunit = nil
			sim:forEachUnit(
				function(unit)
					if unit:getTraits().isDying and unit:isValid() and unit:getLocation() then
						dyingunit = unit
						-- script:queue( { type="hideHUDInstruction" } )
						local x1, y1 = unit:getLocation()
						if x1 and y1 then
							script:queue( { type="displayHUDInstruction", text="Medical attention required", subtext="Turns left: ".. unit:getTraits().isDying, x=x1, y=y1 } ) --refresh HUD if wounded agent got dragged away
							-- STRINGS.UI.TOOLTIP_AGENT_DOWN_DESC = util.sformat(STRINGS.PERMADEATH.TOOLTIP_AGENT_DYING_DESC, unit:getTraits().isDying)
						end
					end	
			end)
		end
	
	end
	
	end,
	
	
}

return abilitydeathtracker