local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local function getTargets(sim)
	local units = {}
	sim:forEachUnit(function(u)
		if u:getTraits().canBeShot and not u:isDead() and not (u:getPlayerOwner() and u:getPlayerOwner():isPC()) then
			table.insert(units,u)
		end
	end)
	return units
end

local emp_tooltip = class( abilityutil.hotkey_tooltip )

function emp_tooltip:init( hud, sim, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._sim = sim
end

function emp_tooltip:activate( screen )
	abilityutil.hotkey_tooltip.activate( self, screen )
	
	local targets = getTargets( self._sim )
    for i, target in ipairs(targets) do
        self._game.boardRig:getUnitRig( target:getID() ):getProp():setRenderFilter( cdefs.RENDER_FILTERS["focus_target"] )
    end
end

function emp_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
	local targets = getTargets( self._sim )
    for i, target in ipairs(targets) do
        self._game.boardRig:getUnitRig( target:getID() ):refreshRenderFilter()
    end
end    

local debug_nuke = 
{
	name = "LETHAL NUKE",

	onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
		return emp_tooltip( hud, sim, self, sim, abilityOwner, "LETHAL NUKE" )
	end,
	
	getName = function()
		return "LETHAL NUKE"
	end,

	usesAction = true,
	alwaysShow = true,
	
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",

	canUseAbility = function( self, sim, abilityOwner, abilityUser )
		if not abilityUser then
			return false
		end

		local cell = sim:getCell( abilityOwner:getLocation() )
		if cell then
			-- On the ground.
			return cell == sim:getCell( abilityUser:getLocation() )
		else
			-- Carried. (can be dropped)
			if array.find( abilityUser:getChildren(), abilityOwner ) ~= nil then
				return true
			end
		end
	end,
	
	executeAbility = function( self, sim, unit, userUnit )
		local cell = sim:getCell( unit:getLocation() ) or sim:getCell( userUnit:getLocation() )
		local x0, y0 = cell.x, cell.y
		
		sim:startTrackerQueue(true)				
		sim:startDaemonQueue()
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HOLOCIRCUIT_OVERLOAD.path )
		
		local maxRange = 0
		local targets = getTargets(sim)
		for i, u in ipairs(targets) do
			local x1, y1 = u:getLocation()
			local range = mathutil.dist2d(x0,y0,x1,y1)
			if range > maxRange then
				maxRange = math.ceil(range)
			end
		end

		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, range = maxRange + 1 } )
        sim:dispatchEvent( simdefs.EV_KO_GROUP, true )
		for i, u in ipairs(targets) do
			u:killUnit(sim)
		end
        sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
		
		sim._trackerQueue = nil			
		sim:processDaemonQueue()
	end,
}
return debug_nuke