local util = include( "modules/util" )
local detention_center = include( "sim/missions/mission_detention_centre" )
local unitdefs = include( "sim/unitdefs" )
local array = include("modules/array")
local itemdefs = include( "sim/unitdefs/itemdefs" )
local simfactory = include( "sim/simfactory" )
local mission_util = include( "sim/missions/mission_util" )
local simdefs = include( "sim/simdefs" )


local baseInit = detention_center.init
function detention_center:init( scriptMgr, sim )
	baseInit( self, scriptMgr, sim )

	if sim:getParams().difficultyOptions.maa_detention_equipment and sim:getParams().difficultyOptions.maa_detention_equipment > 0 then
		--Handle case, where a player could loot chain detention centers by not rescueing the agent.
		if not sim:getParams().agency.maa_first_time_loot_presented then
			sim:getParams().agency.maa_first_time_loot_presented = {}
		end

		--Do we have an agent or prisoner?
		local agent = nil
		for _, unit in pairs( sim:getAllUnits() ) do
			if unit:hasTag( "agent" ) then
				agent = unit
				break
			elseif unit:hasTag( "prisoner" ) then
				break
			end
		end

		local agentID = nil
		--Assign the loot
		local loot = {}
		if agent then
			for _, agentDefPot in ipairs(sim:getParams().agency.unitDefsPotential) do
				if agent:getTraits().rescueID == agentDefPot.id then
					agentID = agentDefPot.id
					--log:write("MAA-DEBUG: DC agent ID: " .. agentID)
					if not agentDefPot.captureTime then
						if not array.find( sim:getParams().agency.maa_first_time_loot_presented, agentDefPot.id ) then
							--Is it a first time agent? Then assign loot from the template
							local agent_template = unitdefs.lookupTemplate( agentDefPot.template)
							for _, upgrade in ipairs( agent_template.upgrades ) do
								--log:write("MAA-DEBUG: DC agent upgrade: " .. upgrade)
								local item_template = unitdefs.lookupTemplate( upgrade )
								if not( item_template.traits and item_template.traits.installed ) then
									table.insert( loot, upgrade )
								end
							end
						end
					else
						--Is it a lost agent? Then assign loot from agentDefPot
						for _, upgrade in ipairs(agentDefPot.upgrades) do
							if type(upgrade) == "string" then
								local item_template = unitdefs.lookupTemplate( upgrade )
								if item_template and not( item_template.traits and item_template.traits.installed ) then
									table.insert( loot, upgrade )
								end
							else
								if upgrade.upgradeParams and not( upgrade.upgradeParams.traits and upgrade.upgradeParams.traits.installed ) then
									table.insert( loot, upgrade )
								end
							end
						end
					end
					break
				end
			end
		end
		if sim:getParams().difficultyOptions.maa_detention_equipment == 2 and #loot == 0 then
			--No loot? Then create backup loot for the vault locker.
			--Copy pasta from security...
			local itemList = {}
			for k,v in pairs(itemdefs) do
				if (v.floorWeight or 0) > 2 then
					table.insert(itemList,v)
				end
			end

			local item = itemList[ sim:nextRand( 1, #itemList ) ]
			table.insert( loot, item )
		end

		--If the level has a big safe we remove its content and place the loot in there, otherwise we just add a safe.
		local safe = mission_util.findUnitByTag( sim, "maa_equipment_safe" )
		if sim:getParams().difficultyOptions.maa_detention_equipment == 1 and #loot > 0 then
			local big_safe = nil
			for _, simUnit in pairs( sim:getAllUnits() ) do
				if simUnit:getTraits().tier2safe then
					big_safe = simUnit
					break
				end
			end
			if big_safe then
				--remove the safe we added
				sim:warpUnit( safe, nil )
				sim:despawnUnit( safe )
				--remove the content of the big safe
				for i=#big_safe:getChildren(), 1, -1 do
					local loot = false
					for _, loot_item in ipairs( simdefs.LOOT_TABLES.lab_safe ) do
						if loot_item[1] == big_safe:getChildren()[i]:getUnitData().id then
							loot = true
							break
						end
					end
					if loot then
						big_safe:removeChild( big_safe:getChildren()[i] )
					end
				end
				big_safe:getTraits().credits = nil
				--make the big safe our new safe
				big_safe:addTag( "maa_equipment_safe" )
				safe = big_safe
			end
		end
		--Place the loot
		if #loot > 0 then
			for _, loot_item in ipairs( loot ) do
				local templateName = loot_item.upgradeName or loot_item
				local template = unitdefs.lookupTemplate( templateName )
				local newItemData = util.extend( template )( loot_item.upgradeParams and util.tcopy( loot_item.upgradeParams ) )
				local newItem = simfactory.createUnit( newItemData, sim )
				sim:spawnUnit( newItem )
				safe:addChild( newItem )
			end
		else
			--remove the safe if no loot
			sim:warpUnit( safe, nil )
			sim:despawnUnit( safe )
		end

		if agentID then
			local agentDef = sim:getPC()._deployed[ agentID ].agentDef
			for i=#agentDef.upgrades, 1, -1 do
				local upgrade = agentDef.upgrades[i]
				if type(upgrade) == "string" then
					local item_template = unitdefs.lookupTemplate( upgrade )
					if item_template and not( item_template.traits and item_template.traits.installed ) then
						table.remove( agentDef.upgrades, i )
					end
				else
					if upgrade.upgradeParams and not( upgrade.upgradeParams.traits and upgrade.upgradeParams.traits.installed ) then
						table.remove( agentDef.upgrades, i )
					end
				end
			end
			--this has to be moved to mission_scoring. altering the agency here does not get reset on rewind and breaks the game...
			-- 	--Add agent id to the loot presented list
			-- 	if not array.find( sim:getParams().agency.maa_first_time_loot_presented, agentID ) then
			-- 		table.insert( sim:getParams().agency.maa_first_time_loot_presented, agentID )
			-- 	end
			sim.maa_add_agent_to_loot_presented = agentID
		 end
	end
end

local basePregeneratePrefabs = detention_center.pregeneratePrefabs
function detention_center.pregeneratePrefabs( cxt, tagSet )
	basePregeneratePrefabs( cxt, tagSet )

	if cxt.params.difficultyOptions.maa_detention_equipment and cxt.params.difficultyOptions.maa_detention_equipment == 2 then
		table.insert( tagSet[1], "maa_equipment_locker" )
	end
end

local baseGeneratePrefabs = detention_center.generatePrefabs
function detention_center.generatePrefabs( cxt, candidates )
	if baseGeneratePrefabs then
		baseGeneratePrefabs( cxt, candidates )
	end
	if cxt.params.difficultyOptions.maa_detention_equipment and cxt.params.difficultyOptions.maa_detention_equipment == 1 then
		local prefabs = include( "sim/prefabs" )
		prefabs.generatePrefabs( cxt, candidates, "maa_equipment_safe", 1 )
	end
end
