local doorrig = include( "gameplay/doorrig2" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")

function doorrig:refreshProp()
	local gfxOptions = self._game:getGfxOptions()
	local boardRig = self._boardRig
	local simdefs = boardRig._game.simCore:getDefs()
	local bMainFrameMode = gfxOptions.bMainframeMode or gfxOptions.bTacticalView
	local bShortWallMode = gfxOptions.bShortWallMode

	local x1,y1 = self:getLocation1()
	local x2,y2 = self:getLocation2()

	local ex, ey, EX, EY
	local nx, ny, NX, NY

	if x2-x1 == 1 then
		ex, ey = self._boardRig:cellToWorld( x1 + 0.5, y1 - 0.5 )
		EX, EY = self._boardRig:cellToWorld( x1 + 0.5, y1 + 0.5 )
		nx,ny = 0,1
		NX,NY = 0,-1
	elseif x1-x2 == 1 then
		ex, ey = self._boardRig:cellToWorld( x2 + 0.5, y2 - 0.5 )
		EX, EY = self._boardRig:cellToWorld( x2 + 0.5, y2 + 0.5 )
		nx,ny = 0,-1
		NX,NY = 0,1
	elseif y2-y1 == 1 then
		ex, ey = self._boardRig:cellToWorld( x1 - 0.5, y1 + 0.5 )
		EX, EY = self._boardRig:cellToWorld( x1 + 0.5, y1 + 0.5 )
		nx,ny = 1,0
		NX,NY = -1,0
	elseif y1-y2 == 1 then
		ex, ey = self._boardRig:cellToWorld( x1 - 0.5, y2 + 0.5 )
		EX, EY = self._boardRig:cellToWorld( x1 + 0.5, y2 + 0.5 )
		nx,ny = -1,0
		NX,NY = 1,0
	else
		crash()
	end

	local ccell_1 = boardRig:getLastKnownCell( x1,y1 )
	local ccell_2 = boardRig:getLastKnownCell( x2,y2 )

	local offset, count = 0, 0

	if ccell_1 or ccell_2 then

		--cell1 is not exit and cell2 is exit then 

		local exit1 = ccell_1 and ccell_1.exits[ self._simdir1 ]
		local exit2 = ccell_2 and ccell_2.exits[ self._simdir2 ]

		-- This is slightly complicated by the fact that either either of cell or to_cell may be ghosted (or non-existent)
		-- We want to use the *most up to date* information (eg. the non-ghosted info, or most recent newest ghosted info).
		assert( ccell_1 or ccell_2 ) -- One of these MUST be non-nil, otherwise we shouldn't be visible.
		local showClosed, showLocked
		if ccell_1 and not ccell_1.ghostID then
			showClosed, showLocked = exit1.closed, exit1.locked
		elseif ccell_2 and not ccell_2.ghostID then
			showClosed, showLocked = exit2.closed, exit2.locked
		elseif not ccell_1 then
			showClosed, showLocked = exit2.closed, exit2.locked
		elseif not ccell_2 then
			showClosed, showLocked = exit1.closed, exit1.locked
		elseif ccell_1.ghostID > ccell_2.ghostID then
			showClosed, showLocked = exit1.closed, exit1.locked
		else
			showClosed, showLocked = exit2.closed, exit2.locked
		end

		if exit1 and (exit1.keybits == simdefs.DOOR_KEYS.GUARD or exit1.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE) then
			showLocked = true
		elseif exit2 and (exit2.keybits == simdefs.DOOR_KEYS.GUARD or exit2.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE) then
			showLocked = true
		end

		if (not bMainFrameMode and bShortWallMode) then
			self._prop:setScl(1,1,cdefs.SHORT_WALL_SCALE)
		else
			self._prop:setScl(1,1,1)
		end

		if bMainFrameMode then
			if showLocked then
				assert( self._offsets and self._offsets['mainframe_locked'] )
				offset, count = unpack( self._offsets['mainframe_locked'] )
			elseif showClosed then
				assert( self._offsets and self._offsets['mainframe_unlocked'] )
				offset, count = unpack( self._offsets['mainframe_unlocked'] )
			else
				assert( self._offsets and self._offsets['mainframe_open'] )
				offset, count = unpack( self._offsets['mainframe_open'] )
			end
			self:setUVTransform( cdefs.WALL_MAINFRAME )
		elseif showClosed and showLocked then
			if self._offsets and self._offsets['locked'] then
				offset, count = unpack( self._offsets['locked'] )
			end
			self:setUVTransform( self._wallUVs.locked )
		elseif showClosed then
			if self._offsets and self._offsets['unlocked'] then
				offset, count = unpack( self._offsets['unlocked'] )
			end
			self:setUVTransform( self._wallUVs.unlocked )
		elseif showLocked then
			if self._offsets and self._offsets['broken'] and self._wallUVs.broken then
				offset, count = unpack( self._offsets['broken'] )
				self:setUVTransform( self._wallUVs.broken )
			end
		end

		if self.lock1 then
			local orientation = self._boardRig._game:getCamera():getOrientation()

			self.lock1:setCurrentFacingMask( 2^((self.lock1._facing - orientation*2) % simdefs.DIR_MAX) )
			self.lock2:setCurrentFacingMask( 2^((self.lock2._facing - orientation*2) % simdefs.DIR_MAX) )

			if bMainFrameMode then
				self.lock1:setVisible(false)
				self.lock2:setVisible(false)
			elseif showClosed then
				self.lock1:setVisible(true)
				self.lock2:setVisible(true)
			else
				self.lock1:setVisible(false)
				self.lock2:setVisible(false)
			end

			if showLocked then
				self.lock1:setCurrentAnim( "idle" )
				self.lock2:setCurrentAnim( "idle" )
			else
				self.lock1:setCurrentAnim( "idle_unlocked" )
				self.lock2:setCurrentAnim( "idle_unlocked" )
			end			
		end	
	else
		if self._secure or self._vault_door then
			self.lock1:setVisible(false)
			self.lock2:setVisible(false)
		end
	end

	local prop, mesh = self._prop, self._mesh
	mesh:setElementOffset( offset )
	mesh:setElementCount( count )
	prop:setVisible( count > 0 )
end