local simdefs = include("sim/simdefs")
local sim = include("sim/engine")
local simquery = include("sim/simquery")
local util = include("modules/util")

-- unoptimized repeating code alert

SLF_nullblock = 
{
	onSpawnAbility = function( self, sim, unit )
		self.abilityOwner = unit
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		sim:addTrigger( simdefs.TRG_UNIT_KO, self )
		sim:addTrigger( simdefs.TRG_UNIT_EMP, self )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		sim:removeTrigger( simdefs.TRG_UNIT_EMP, self )
	end,

	onTrigger = function( self, sim, evType, evData )
		--null drone is captured
		if evType == simdefs.TRG_ICE_BROKEN and evData.unit:getTraits().mainframe_suppress_range and evData.unit:getTraits().mainframe_ice <= 0 then
			--clear all null tiles around unit
			local sim = evData.unit:getSim()
			local cell = sim:getCell(evData.unit:getLocation())
			local cells = simquery.fillCircle(sim, cell.x, cell.y, evData.unit:getTraits().mainframe_suppress_range, 0)
			local nullzone = {}
			if not sim._board.cell_groups[ "nullzone" ] then
				sim._board.cell_groups["nullzone"] = {}
			end
			for _, cell in pairs(cells) do
				for i, n_cell in pairs(sim._board.cell_groups["nullzone"]) do
					if n_cell == cell then
						table.remove(sim._board.cell_groups["nullzone"], i)
					end
				end
			end
		end

		if evType == simdefs.TRG_UNIT_EMP then
			log:write("ACW - "..util.stringize(evData, 1))
		end

		--null drone KO and wakeup
		if (evType == simdefs.TRG_UNIT_KO or evType == simdefs.TRG_UNIT_EMP) and evData.unit and evData.unit:getTraits().mainframe_suppress_range then
			

			--EMP'd or KO'd (guard/drone)
			if evData.ticks then
				--clear all null tiles around unit
				local sim = evData.unit:getSim()
				local cell = sim:getCell(evData.unit:getLocation())
				local cells = simquery.fillCircle(sim, cell.x, cell.y, evData.unit:getTraits().mainframe_suppress_range, 0)
				local nullzone = {}
				if not sim._board.cell_groups[ "nullzone" ] then
					sim._board.cell_groups["nullzone"] = {}
				end
				for _, cell in pairs(cells) do
					for i, n_cell in pairs(sim._board.cell_groups["nullzone"]) do
						if n_cell == cell then
							table.remove(sim._board.cell_groups["nullzone"], i)
						end
					end
				end
			end
			--waking up
			--make new null tiles around unit
			local sim = evData.unit:getSim()
			local cell = sim:getCell(evData.unit:getLocation())
			local cells = simquery.fillCircle(sim, cell.x, cell.y, evData.unit:getTraits().mainframe_suppress_range, 0)
			local nullzone = {}
			if not sim._board.cell_groups[ "nullzone" ] then
				sim._board.cell_groups["nullzone"] = {}
			end
			for _, cell in pairs(cells) do
				table.insert(sim._board.cell_groups["nullzone"], cell)
			end
		end

		--whenever a null-active unit moves, assign cells in the null-zone a special tag.
		if evData.unit and not evData.unit:isKO() and evData.unit:getTraits().mainframe_suppress_range and evData.unit:getPlayerOwner() == sim:getNPC() then
			local stc, enc = evData.from_cell, evData.to_cell
			if stc then
				--clear all null tiles around startCell
				local sim = evData.unit:getSim()
				local cells = simquery.fillCircle(sim, stc.x, stc.y, evData.unit:getTraits().mainframe_suppress_range, 0)
				local nullzone = {}
				if not sim._board.cell_groups[ "nullzone" ] then
					sim._board.cell_groups["nullzone"] = {}
				end
				for _, cell in pairs(cells) do
					for i, n_cell in pairs(sim._board.cell_groups["nullzone"]) do
						if n_cell == cell then
							table.remove(sim._board.cell_groups["nullzone"], i)
						end
					end
				end
			end
			if enc then
				--make new null tiles around endCell
				local sim = evData.unit:getSim()
				local cells = simquery.fillCircle(sim, enc.x, enc.y, evData.unit:getTraits().mainframe_suppress_range, 0)
				local nullzone = {}
				if not sim._board.cell_groups[ "nullzone" ] then
					sim._board.cell_groups["nullzone"] = {}
				end
				for _, cell in pairs(cells) do
					table.insert(sim._board.cell_groups["nullzone"], cell)
				end
			end
		end
	end,
}

return SLF_nullblock