----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local cdefs = include( "client_defs" )
local simunit = include( "sim/simunit" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )
local inventory = include( "sim/inventory" )
local abilitydefs = include( "sim/abilitydefs" )
local mainframe = include( "sim/mainframe" )

-----------------------------------------------------
-- Local functions

local function removeParasite(player, hostUnit)
	if hostUnit:getTraits().parasite then
		for i, ability in ipairs(player:getAbilities()) do
			if ability.parasite_hosts then
            			array.removeElement( ability.parasite_hosts, hostUnit:getID() )
			end
		end
		hostUnit:getTraits().parasite = nil
	end

	if hostUnit:getTraits().crossfeed_target then
		for i, ability in ipairs(player:getAbilities()) do
			if ability.crossfeed_target then
            			ability.crossfeed_target = nil
				ability.cooldown = 0
			end
		end
		hostUnit:getTraits().crossfeed_target = nil
	elseif hostUnit:getTraits().crossfeed_source then
		for i, ability in ipairs(player:getAbilities()) do
			if ability.crossfeed_source then
            			ability.crossfeed_source = nil
				ability.cooldown = 0
			end
		end
		hostUnit:getTraits().crossfeed_source = nil
	end
end

local drone_uplink = { ClassType = "drone_uplink" }

function drone_uplink:onWarp( sim, oldcell, cell )
	if oldcell then
		self:deactivate( sim )
	end

	if cell and self:getTraits().startOn == true then
		self:activate( sim )
		self:setPlayerOwner(sim:getNPC())
	end
end

function drone_uplink:activate( sim )
	if  self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"

		if self:getSounds().spot_old then
			self:getSounds().spot = self:getSounds().spot_old		
			self:getSounds().spot_old = nil
		end

		sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_on" )
	end
end

function drone_uplink:deactivate( sim )	
	sim:triggerEvent( simdefs.TRG_UNIT_DISABLED, self )

	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "inactive"

		self:getSounds().spot_old = self:getSounds().spot		
		self:getSounds().spot = nil			

		sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		self:destroyTab()
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_off" )
	end
end

function drone_uplink:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_ICE_BROKEN and evData and self:getPlayerOwner() == sim:getNPC() then
		local unit = evData.unit
		if unit:getTraits().mainframe_ice <= 0 and unit:getTraits().isDrone and not unit:isNeutralized() then
			local firewallBoost = self:getTraits().drone_uplink or 1
	        	sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.WORLDEXTEND.UI.DRONE_UPLINK_WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
			if unit:getTraits().parasite then
				removeParasite(sim:getPC(), unit)
			end
			unit:deactivate( sim )
			while unit:isKO() do
				unit:tickKO( sim )
			end
			sim:processReactions(unit)
			unit:increaseIce(sim, firewallBoost)
			self:processEMP(1, true, false, true)
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
		end
	end
end

function drone_uplink:removeChild( childUnit )
	simunit.removeChild( self, childUnit )
	if childUnit:getTraits().ammo_clip then
		self:processEMP( nil, false, false, true )
		self:getSim():triggerEvent( simdefs.TRG_UNIT_DISABLED, self )
	end
end

-----------------------------------------------------
-- Interface functions

local function createDroneUplink( unitData, sim )
	return simunit.createUnit( unitData, sim, drone_uplink )
end

simfactory.register( createDroneUplink )

return
{
	createDroneUplink = createDroneUplink,
}