local util = include("client_util")
local array = include("modules/array")
local simdefs = include("sim/simdefs")
local engine = include( "sim/engine" )
local simquery = include("sim/simquery")

local baseTryShootAt = engine.tryShootAt
function engine:tryShootAt( sourceUnit, targetUnit, dmgt0, equipped )
	local refresh_vision = false
	if equipped:getTraits().maa_reduceTargetMaxMP_factor and targetUnit and targetUnit:getMPMax() then
		local reduction_amount = math.floor( targetUnit:getMPMax() * equipped:getTraits().maa_reduceTargetMaxMP_factor )
		reduction_amount = math.min( targetUnit:getMPMax() - 5, reduction_amount )
		targetUnit:addMPMax( -reduction_amount )
	end
	if equipped:getTraits().maa_reduceTargetLOSRange_factor and targetUnit and targetUnit:getTraits().LOSrange then
		local reduction_amount = math.floor( targetUnit:getTraits().LOSrange * equipped:getTraits().maa_reduceTargetLOSRange_factor )
		reduction_amount = math.min( targetUnit:getTraits().LOSrange - 5, reduction_amount )
		targetUnit:getTraits().LOSrange = targetUnit:getTraits().LOSrange - reduction_amount

		if targetUnit:getTraits().LOSperipheralRange then
			local reduction_amount_periph = math.floor( targetUnit:getTraits().LOSperipheralRange * equipped:getTraits().maa_reduceTargetLOSRange_factor )
			reduction_amount_periph = math.min( targetUnit:getTraits().LOSperipheralRange - 6, reduction_amount_periph )
			targetUnit:getTraits().LOSperipheralRange = targetUnit:getTraits().LOSperipheralRange - reduction_amount_periph
			refresh_vision = true
		end
	end
	if equipped:getTraits().maa_reduceTargetLOSArc_factor  and targetUnit and targetUnit:getTraits().LOSarc then
		local reduction_amount = targetUnit:getTraits().LOSarc * equipped:getTraits().maa_reduceTargetLOSArc_factor
		reduction_amount = math.min( targetUnit:getTraits().LOSarc - (math.pi / 8), reduction_amount )
		targetUnit:getTraits().LOSarc = targetUnit:getTraits().LOSarc - reduction_amount

		if targetUnit:getTraits().LOSperipheralArc then
			local reduction_amount_periph = targetUnit:getTraits().LOSperipheralArc * equipped:getTraits().maa_reduceTargetLOSArc_factor
			reduction_amount_periph = math.min( targetUnit:getTraits().LOSperipheralArc - (math.pi / 6), reduction_amount_periph )
			targetUnit:getTraits().LOSperipheralArc = targetUnit:getTraits().LOSperipheralArc - reduction_amount_periph
			refresh_vision = true
		end
	end
	baseTryShootAt( self, sourceUnit, targetUnit, dmgt0, equipped )
	if refresh_vision then
		self:refreshUnitLOS( targetUnit )
	end

	if targetUnit:isDead() then
		if equipped:getTraits().maa_mp_boost then
			sourceUnit:addMP( equipped:getTraits().maa_mp_boost )
		end
		if equipped:getTraits().maa_ap_refresh and equipped:getTraits().maa_ap_refresh > (equipped:getTraits().maa_ap_refreshs_turn or 0) then
			sourceUnit:getTraits().ap = ( sourceUnit:getTraits().ap or 0 ) + 1
			equipped:getTraits().maa_ap_refreshs_turn = (equipped:getTraits().maa_ap_refreshs_turn or 0) + 1
		end
	end
end

local baseDamageUnit = engine.damageUnit
function engine:damageUnit( targetUnit, srcDamage, kodamage, fx, sourceUnit )
	if srcDamage and srcDamage > 0 and targetUnit and sourceUnit and sourceUnit:getTraits().isGuard and not sourceUnit:getTraits().isDrone then
		local reflexes = nil
		for _, child in ipairs( targetUnit:getChildren() ) do
			if child:getTraits().maa_lightning_reflexes then
				reflexes = child
				break
			end
		end
		local x1,y1 = targetUnit:getLocation()
		if reflexes and reflexes:getTraits().dodged_units and array.find( reflexes:getTraits().dodged_units, sourceUnit ) then
			srcDamage = 0
			self:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.LIGHTNIG_REFLEXES, x=x1, y=y1, color={r=1,g=1,b=0,a=1}} )
		elseif reflexes and reflexes:getTraits().ammo > 0 then
			srcDamage = 0
			reflexes:getTraits().ammo = reflexes:getTraits().ammo - 1
			if not reflexes:getTraits().dodged_units then
				reflexes:getTraits().dodged_units = {}
			end
			table.insert( reflexes:getTraits().dodged_units, sourceUnit )
			self:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MORE_ARCHIVE_AGENTS.UI.FLY_TXT.LIGHTNIG_REFLEXES, x=x1, y=y1, color={r=1,g=1,b=0,a=1}} )
		end
	end
	
	baseDamageUnit( self, targetUnit, srcDamage, kodamage, fx, sourceUnit )
end

local baseTrackerAdvance = engine.trackerAdvance
function engine:trackerAdvance( delta, txt, scan )
	if not self._trackerQueue then
		local player = self:getPC()
		if player:getTraits().maa_alarm_block then
			for i=1, delta do --Yes, it is intended to not apply the alarmMultiplier on the block. I think that makes the augment still fair when playing with alarmMultiplier
				if player:getCpus() >= player:getTraits().maa_alarm_block then
					delta = delta - 1
					player:addCPUs( -player:getTraits().maa_alarm_block )
				end
			end
		end
	end
	
	if delta > 0 then
		baseTrackerAdvance( self, delta, txt, scan )
	end
end

local baseEndTurn = engine.endTurn
function engine:endTurn()
	self:triggerEvent( "MAA_TRG_PRE_END_TURN", self:getCurrentPlayer() )
	
	baseEndTurn( self )
end

local baseSpawnUnit = engine.spawnUnit
function engine:spawnUnit( unit )
	baseSpawnUnit( self, unit )
	
	--late skill levelup for skills that need the unit's children to be instantiated
	if unit:getSkills( ) then
		for _, skill in ipairs( unit:getSkills( ) ) do
			for i=1, skill:getCurrentLevel() do
				if skill:getLevel( i ).onLearn_late then
					skill:getLevel( i ).onLearn_late( self, unit )
				end
			end
		end
	end
end

local baseInit = engine.init --we need to implement SC's skillburn for summons as well
function engine:init( params, levelData, ... )
	baseInit( self, params, levelData, ... )

	if params.agency.unitDefs_childs then
		for v, agentDef in pairs ( params.agency.unitDefs_childs )do
			for skillIdx, skill in pairs( agentDef.skills or {} ) do
				skill.skillChange = nil
			end	
		end
	end
end
