local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local mission_util = include( "sim/missions/mission_util" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local itemdefs = include( "sim/unitdefs/itemdefs" )
local propdefs = include( "sim/unitdefs/propdefs" )
local serverdefs = include( "modules/serverdefs" )
local SCRIPTS = include('client/story_scripts')
---------------------------------------------------------------------------------------------
-- KIA COMMENT STUFF
local function AGENT_KIA()
	return
	{
		trigger = simdefs.TRG_UNIT_KILLED,
		fn = function( sim, triggerData )
			if triggerData and triggerData.unit:getTraits().corpseTemplate then
				-- log:write("log triggerData.unit_unitdata")
				-- log:write(util.stringize( triggerData.unit._unitData, 3 ))	
				if triggerData.unit._unitData.hireText then -- check that it's an agent but not prisoner/hostage
					return triggerData.unit 
				end				
            		end
       		end
    	}
end

-- KIA
local function KIA_comment( script, sim )	
	local _, agent = script:waitFor( AGENT_KIA() )
	log:write("received permadeath trigger")
	if not sim:getWinner() and agent:getTraits().corpseTemplate then
        	local dead_central = false 
        	local activeAgents = 0

        	for _, unit in pairs( sim:getPC():getAgents() ) do
            		if unit:isKO() and unit:getTraits().central then
               			dead_central = true                
            		end
					if unit:getTraits().corpseTemplate and not unit:isKO() then
						activeAgents = activeAgents + 1
            		end
        	end
        		if not dead_central and activeAgents > 0 then
            			script:queue( 2*cdefs.SECONDS )
						script:queue( { type="clearOperatorMessage" } )
            			script:queue( { script=SCRIPTS.INGAME.KIA_COMMENT[sim:nextRand(1, #SCRIPTS.INGAME.KIA_COMMENT)], type="newOperatorMessage" } ) 			     
        		end
			script:addHook( KIA_comment )
    		end
	end
-- SPAWNING GHOSTS OF DEAD AGENTS

local function getCellGhost( player, cellx, celly )
	assert( player._ghost_cells, util.stringize( player, 1 ))
	return player._ghost_cells[ simquery.toCellID( cellx, celly ) ]
end

local function isKnownCell( sim, cellx, celly)
	return getCellGhost(sim:getPC(), cellx, celly)
	or sim:canPlayerSee( sim:getPC(), cellx, celly )
end

local tooltips = {
"coRrUptEd_sIGnaL",
"ERROR",
"FALSE POSITIVE",
"???",
"aberrant_signal",
}

local function setRandomAnim( sim, ghost )

	local facings = {0,1,2,3,4,5,6,7}
	local facing = facings[sim:nextRand(1,#facings)]
	ghost:setFacing(facing)
	--the stuff below doesn't work so blah
	-- local rand = sim:nextRand(1,8)
	-- log:write("LOG rand is "..tostring(rand))
	-- if rand == 1 then
		-- -- sim:dispatchEvent( simdefs.EV_UNIT_START_SHOOTING, { unitID = ghost:getID() } )
		-- ghost:getTraits().isAiming = true
	-- elseif rand == 2 then
		-- sim:dispatchEvent( simdefs.EV_UNIT_OVERWATCH_MELEE, {unit = ghost} )
	-- elseif rand == 3 then
		-- sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = ghost:getID(), facing = facing } )	
	-- elseif rand == 4 then
		-- self:dispatchEvent( simdefs.EV_UNIT_START_WALKING, { unit = ghost  } )
	-- elseif rand == 5 then
		-- -- sim:dispatchEvent( simdefs.EV_UNIT_PEEK, { unitID = ghost:getID() } )
		-- sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= ghost, anim="peek_fwrd" } )
	-- elseif rand == 6 then
		-- -- ghost:getTraits().isDead = true
		-- sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= ghost, anim="dead" } )
	-- elseif rand == 7 then
		-- sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= ghost, anim="pin_pre" } )
	-- elseif rand == 8 then
		-- sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= ghost, anim="peek_pst_fwrd" } )
	-- end
	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = ghost } )

end

local function spawnGhost( script, sim )
	script:waitFor(mission_util.PC_START_TURN)
	local haunting_chance = 0.5
	
	local agency = sim:getParams().agency
	if agency.permadeathpool and (#agency.permadeathpool > 0) and (sim:nextRand()<= haunting_chance ) then
		--pick agent
		local deadID = agency.permadeathpool[sim:nextRand(1,#agency.permadeathpool)]
		-- log:write("LOG deadID")
		-- log:write(util.stringize(deadID,2))
		-- log:write("LOG permadeathpool")
		-- log:write(util.stringize(agency.permadeathpool,2))
		local template = nil
		local agentDef = nil
		local agentdefs = include("sim/unitdefs/agentdefs")
		for k, v in pairs(agentdefs) do
			if v.agentID and (v.agentID == deadID) then
				template = k
				agentDef = v
				-- log:write("LOG template is "..tostring(template))
			end
		end
					
		-- pick spawn cell
		local cellx
		local celly
		local allcells = {}
		for y, xcords in ipairs(sim._board) do
			for x, cell in ipairs(xcords) do
				if not isKnownCell( sim, x, y ) then
					table.insert(allcells, simquery.toCellID( x, y ))
				end
			end
		end
		if agentDef and template and (#allcells > 0) then
			log:write("spawning ghost")
			local cellIdx = sim:nextRand(1, #allcells)
			if allcells[cellIdx] then
				local x, y = simquery.fromCellID( allcells[cellIdx] )
				local cell = sim:getCell(x, y)
				
				local prop_template = unitdefs.prop_templates.agent_capture
				local ghostAgent = simfactory.createUnit( prop_template, sim ) 
				local agentTemplate = unitdefs.lookupTemplate(template)
				ghostAgent._unitData.kanim = agentTemplate.kanim
				ghostAgent:getTraits().template = agentDef.template
				-- ghostAgent._traits = agentDef.traits
				ghostAgent._unitData = agentTemplate
				ghostAgent._unitData.name = tooltips[sim:nextRand(1,#tooltips)]
				
				-- sim:dispatchEvent( simdefs.EV_CAM_PAN, { x, y } ) --for testing, comment out before release
				
				--spawn and set animation
				sim:spawnUnit( ghostAgent )
				-- ghostAgent:setPlayerOwner(sim:getNPC())
				sim:warpUnit( ghostAgent, cell )
				-- setRandomAnim( sim, ghostAgent )
				local deployed = sim:getPC():getUnits()
				local living_agent = deployed[sim:nextRand(1,#deployed)]
				local x0, y0 = ghostAgent:getLocation()
				local x1, y1 = living_agent:getLocation()
				local facing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
				ghostAgent:setFacing(facing)
							
				-- glimpse and despawn
				sim:getPC():glimpseUnit( sim, ghostAgent:getID() )
				
				sim:warpUnit( ghostAgent, nil )
				sim:despawnUnit( ghostAgent )
				sim.permadeath_ghost_cell = cell
			end
		end
		
	end
	script:addHook( spawnGhost )
	
end		

local function despawnGhost( script, sim )
	script:waitFor(mission_util.PC_END_TURN)
	if sim.permadeath_ghost_cell then
		sim:getPC():glimpseCell( sim, sim.permadeath_ghost_cell )	
		sim.permadeath_ghost_cell = nil
	end
	script:addHook( despawnGhost )
end


--Interface Functions

function init( scriptMgr, sim )
	scriptMgr:addHook( "KIA_comment", KIA_comment )
	if sim:getParams().difficultyOptions.permadeath_ghosts then
		scriptMgr:addHook( "permadeath_spawnGhost", spawnGhost )
		scriptMgr:addHook( "permadeath_despawnGhost", despawnGhost )
	end
end

return
{
	init = init,
}