--------------------------------------------------------------------
-- Master file for civiliansAlertGuards. In charge of making sure everything
-- is set up properly for this feature. modinit is responsible for
-- passing relevant campaign generation options.
--
-- Feature description:
-- When a civilian is alerted, he will notify all nearby guards of
-- what he saw. A civlian can be alerted in at least 3 cases
-- 1:
--  Civilian sees an agent in direct line-of-sight.
--  Response: Notify guards, guards will go into alert and turn
--   to face the agent seen by the civilian.
-- 2:
--  Civilian wakes up after being K.O.ed
--  Response: Notify guards, guards will go on alert and investigate
--   the civilians current location.
-- 3:
--  Alarm level hits four (I think it's four, it might be 5).
--  Repsonse: Whole facility is no on alert, don't notify the guards
--   (since they already know anyway).
--
-- Note that while the civlian is fleeing, if he sees an agent he should
-- notify guards again.

local actions = include("sim/btree/actions")
local simdefs = include("sim/simdefs")
-- local util = include("modules/util")

--------------------------------------------------------------------
-- Mod campaign option information, set by modinit
local CIVILIANS_ALERT_GUARDS = false
local function setModInfo(civiliansAlertGuards)
	CIVILIANS_ALERT_GUARDS = civiliansAlertGuards or false
end

--------------------------------------------------------------------
-- helper functions

--------------------------------------------------------------------
-- Extend actions
local old_Panic = actions.Panic
actions.Panic = function(sim, unit)
	local returnValue = old_Panic(sim, unit)
	if CIVILIANS_ALERT_GUARDS == true then
		if unit:getBrain():getTarget() then -- Only alert the guards if we actually see an agent
			local target = unit:getBrain():getTarget()
			local x0,y0 = unit:getLocation()
			local x1,y1 = target:getLocation()
			sim:triggerEvent(simdefs.TRG_NEW_INTEREST, { x=x0, y=y0, range=simdefs.SOUND_RANGE_2, target=target, interest={x=x1, y=y1, sourceUnit=unit} })
		end
	end
	return returnValue
end

return {
	setModInfo = setModInfo
}
