----------------------------------------------------------------------
-- The surrender ability. Allows agents to K.O theirselves when under
-- overwatch.

-- Surrender is intended to be usuable by all agents.

local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local util = include("client_util") -- "client_util" also includes "modules/util"

local SURRENDER_KO = 1 -- How long an Agent is K.O.ed if they surrender

local surrender =
{
	-- Constants set by modinit
	WOUNDED = 0,
	WOUNDED_MAX = 0,

	name = STRINGS.NEW_CORP_TACTICS.TOOLTIPS.SURRENDER,
	profile_icon = "gui/icons/nct_icon-action_surrender.png",
	proxy = false, -- Tells the program to show on agent/owner (instead of the target)
	alwaysShow = nil, -- When true, shows a grayed out version of the ability when it can't be used
	ghostable = nil, -- Ghosts are red images that show you where something you saw was last.

	-- Allows us to customize the name if necessary
	getName = function (self, sim, unit)
		return self.name
	end,

	-- Creates the tooltip.
	onTooltip = function (self, hud, sim, player)
		local tooltip = util.tooltip( hud._screen )
		local section = tooltip:addSection()

		section:addLine("<ttheader>"..self.name.."</>")
		section:addDesc(STRINGS.NEW_CORP_TACTICS.TOOLTIPS.SURRENDER_DESC)
		section:addAbility(STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE, util.sformat("Knocks the agent out for {1} {1:turn|turns}", SURRENDER_KO), "gui/icons/arrow_small.png")
		if self.WOUNDED > 0 then
			section:addAbility(STRINGS.NEW_CORP_TACTICS.TOOLTIPS.WOUNDED, util.sformat(STRINGS.NEW_CORP_TACTICS.TOOLTIPS.WOUNDED_DESC, self.WOUNDED), "gui/icons/arrow_small.png")
		end
		return tooltip
	end,

	-- Finds only eligible targets for the ability. In this case, the agent that with the ability.
	acquireTargets = function (self, targets, game, sim, abilityOwner, abilityUser)
		return targets.unitTarget(game, {abilityUser}, self, abilityUser, abilityUser)
	end,

	-- Is the agent able to surrender?
	canUseAbility = function (self, sim, unit, abilityUser) -- 'unit' should be the agent that has this ability. abilityUser should be the agent to right?
		if unit ~= abilityUser then
			return false, "unit ~= abilityUser 'This shouldn't happen, please contact mod developer'"
		end
		if simquery.isUnitUnderOverwatch(unit) == false then
			return false
		end
		return true
	end,

	-- Surrender, K.O.ing the agent
	executeAbility = function (self, sim, unit, userUnit, target)
		local targetUnit = sim:getUnit(target)
		local x1, y1 = unit:getLocation()
		unit:setInvisible(false)
		unit:setDisguise(false)
		unit:setKO(sim, SURRENDER_KO)
		sim:triggerEvent(simdefs.TRG_UNIT_HIT, {targetUnit=targetUnit, sourceUnit=unit, x=x1, y=y1, melee=true})
		sim:processReactions(unit)

		-- Mark the agent to get wounded once a guard picks them up.
		-- Reduction of MP occurs in dragAgents.lua inside the actions.MarkInterestInvestigated function.
		local currentWounds = unit:getTraits().wounded or 0
		local reduceMPby = math.min(self.WOUNDED, self.WOUNDED_MAX - currentWounds)
		if reduceMPby > 0 then
			unit:getTraits().woundMe = reduceMPby
		end
	end

}
return surrender
