----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local mathutil = include( "modules/mathutil" )

-----------------------------------------------------
-- Local functions

local frostspire = { ClassType = "frostspire" }

function frostspire:activate( sim )
	if  self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"

		if self:getSounds().spot_old then
			self:getSounds().spot = self:getSounds().spot_old		
			self:getSounds().spot_old = nil
		end

		sim:addTrigger( simdefs.TRG_START_TURN, self )
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_on" )
	end
end

function frostspire:deactivate( sim )	
	sim:triggerEvent( simdefs.TRG_UNIT_DISABLED, self )

	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "inactive"

		self:getSounds().spot_old = self:getSounds().spot		
		self:getSounds().spot = nil			

		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		self:destroyTab()
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_off" )
	end
end

function frostspire:onWarp( sim, oldcell, cell )
	if oldcell then
		self:deactivate( sim )
	end

	if cell and self:getTraits().startOn == true then
		self:activate( sim )
		self:setPlayerOwner( sim:getNPC() )
	end
end 

function frostspire:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer():isPC() and sim:getTurnCount() > 1 then
		if self:getTraits().mainframe_status == "active" and self:getPlayerOwner():isNPC() then
			local x0,y0 = self:getLocation()
			sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = nil, range = self:getTraits().range } )
			local cells = simquery.rasterCircle( sim, x0, y0, self:getTraits().range )
    			for i, x, y in util.xypairs( cells ) do
        			local cell = sim:getCell( x, y )
        			if cell then
            				for _, cellUnit in ipairs(cell.units) do
                				if simquery.isEnemyAgent( self:getPlayerOwner(), cellUnit) and cellUnit:getMP() and cellUnit:getMP() > 5 then
							local freezeMP = self:getTraits().freezeMP or 1
							freezeMP = math.min(cellUnit:getMP() - 5, freezeMP)
							local x1, y1 = cellUnit:getLocation()
                    					cellUnit:useMP(freezeMP, sim)
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FORMATS.AP, -freezeMP),x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )		
                				end
					end
            			end
			end
		end
	end
end


function frostspire:removeChild( childUnit )
	simunit.removeChild( self, childUnit )
	if childUnit:getTraits().ammo_clip then
		self:processEMP( nil, false, false, true )
		self:getSim():triggerEvent( simdefs.TRG_UNIT_DISABLED, self )
	end
end
-----------------------------------------------------
-- Interface functions

local function createFrostSpire( unitData, sim )
	return simunit.createUnit( unitData, sim, frostspire )
end

simfactory.register( createFrostSpire )

return
{
	createFrostSpire = createFrostSpire,
}

