local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local commondefs = include("sim/unitdefs/commondefs")
local itemdefs = include("sim/unitdefs/itemdefs")
local speechdefs = include( "sim/speechdefs" )
local guarddefs = include( "sim/unitdefs/guarddefs" )

---------------------------------------------------------------------------------------------------------
-- NPC templates

local SOUNDS = commondefs.SOUNDS

local DEFAULT_IDLES = commondefs.DEFAULT_IDLES

local DEFAULT_ABILITIES = commondefs.DEFAULT_ABILITIES

local DEFAULT_DRONE = commondefs.DEFAULT_DRONE

local onGuardTooltip = commondefs.onGuardTooltip

local npc_templates =
{
	neptune_sysadmin =
	{
		type = "simunit",
		name = STRINGS.MOD_CE and STRINGS.MOD_CE.GUARDS.SYS_ADMIN or "SYSTEM ADMIN",
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_sysadmin_face",	
		profile_image = "neptune_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			mainframeRecapture = 2,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},			
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "WimpBrain",
	},
	
	neptune_sysadmin_new =
	{
		type = "simunit",
		name = STRINGS.MOD_CE and STRINGS.MOD_CE.GUARDS.SYS_ADMIN or "SYSTEM ADMIN",
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_sysadmin_face",	
		profile_image = "neptune_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			meleeRecapture = true,
			consoleScan = true,
			investigateHackedDevices = true,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},			
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "ceGuardbrainSysadmin",
	},
	
	neptune_guard =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_guard_build",
		profile_image = "neptune_guard.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_security",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			
			recap_icon = "guard",
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
					
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
		blurb = "<ttheader>EMPLOYED</>\nIt is corporate policy to invest in life insurance for each an every one of it's valued employees. In event of accident causing injury or permanent system failure, dependents will receive insurance benefits in an order as noted by form CSF_312.\n<ttrule>Read all fine Print</>\n",
	},
	
	neptune_guard_weak =
	{
		type = "simunit",
		name = STRINGS.GUARDS.GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_guard_build",
		profile_image = "neptune_guard.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_security",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			
			resistKO = -1,
			LOSperipheralArc = 11*math.pi/16,
			lookaroundArc = 11*math.pi/16,
			mpMax= 9,
			mp= 9,
			recap_icon = "guard",
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
					
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",		
		blurb = "<ttheader>EMPLOYED</>\nIt is corporate policy to invest in life insurance for each an every one of it's valued employees. In event of accident causing injury or permanent system failure, dependents will receive insurance benefits in an order as noted by form CSF_312.\n<ttrule>Read all fine Print</>\n",
	},
	
	neptune_guard_tier_2 =
	{
		type = "simunit",
		name = STRINGS.GUARDS.ELITE_GUARD,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_elite_build",	
		profile_image = "neptune_guard_2.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_security_elite",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,	
			cashOnHand = 110, 

			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,
            lookaroundArc = math.pi / 2 + math.pi / 4,
            lookaroundOffset = math.pi / 4 - math.pi / 8,
			
			recap_icon = "guard_armour",
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},
	
	neptune_frost_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CORPS.GUARDS.FROST_TROOPER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_frost_build",	
		profile_image = "neptune_frost.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_frost_guard",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 110,
			
			frostZoneRange = 5,
			frostZone = 1,
			frostGrenades = true,
			
			recap_icon = "frost",
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "neptune_frost_zone"},
		children = { itemdefs.item_npc_pistol2, itemdefs.item_neptune_icegrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},
	
	neptune_seeker_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CORPS.GUARDS.SEEKER_TROOPER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_seeker_build",	
		profile_image = "neptune_seeker.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_seeker",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 110,
			
			neptune_seeker = true,
			
			recap_icon = "frost",
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor" },
		children = { itemdefs.item_neptune_seekergun },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "neptuneGuardbrainSeeker",
	},
	
	neptune_frost_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CORPS.GUARDS.FROST_TROOPER_2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_frost_elite_build",	
		profile_image = "neptune_frost_2.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_frost_guard_elite",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 110,
			armor = 1,
			
			frostZoneRange = 8,
			frostZone = 2,
			frostGrenades = true,
			
			recap_icon = "frost_2",
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "neptune_frost_zone"},
		children = { itemdefs.item_npc_pistol2, itemdefs.item_neptune_icegrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
	},	
	
	neptune_seeker_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CORPS.GUARDS.SEEKER_TROOPER_2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/neptune_seeker_elite_build",	
		profile_image = "neptune_seeker_2.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_neptune_seeker_elite",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 110,
			armor = 1,
			
			neptune_seeker = true,
			
			recap_icon = "frost",
		},
		dropTable =
		{
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 6},
			{ "item_emp_pack_3", 4 },
			{ "item_shocktrap_2", 4 },
			{ "item_shocktrap_3", 4 },
			{nil, 68}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 15 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor" },
		children = { itemdefs.item_neptune_seekergun_2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "neptuneGuardbrainSeeker",
	},
}

return npc_templates