local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )

local mod_hackdevice = 
	{
	name = STRINGS.ABILITIES.SCAN_DEVICE,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
    proxy = true,
	alwaysShow = true,
	HUDpriority = 1,
	noAI_PWRcost = 2,
	triggersOverwatch = true,
	-- iconColor= util.color( 163/255, 0/255, 0/255 ),
	-- iconColorHover= util.color( 1,1,1 ),
	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit( targetID )
		-- return abilityutil.formatToolTip(STRINGS.ABILITIES.SCAN, string.format( STRINGS.ABILITIES.TOOLTIPS.SCAN, targetUnit:getName() ))
		local targetUnit = sim:getUnit( targetID )
		local hackingskill = abilityUser:getSkillLevel("hacking")
		local icebreaker = 0
		if hackingskill == 1 then
				icebreaker = 1
		elseif hackingskill == 2 or hackingskill == 3 then 
				icebreaker = 2
		elseif hackingskill > 3 then
				icebreaker = 3
		end
		
		local tooltip = nil
		
		if self.balancedHacking then
			tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.SLOWHACKING1_WITH_INCOG, icebreaker, self.noAI_PWRcost)
		else
			tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.SLOWHACKING1, icebreaker)
		end
		
		if abilityUser:getTraits().mod_data_hacking and ( abilityUser:getTraits().mod_data_hacking == targetUnit:getID() ) then
			tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.STOP_HACKING)
		end
		
		return tooltip

	end,
		
	getName = function( self, sim, unit )
		return STRINGS.MANUALHACKING.TOOLTIPS.SLOWHACKINGNAME
	end,

    getProfileIcon = function( self, sim, abilityOwner )
        return self.profile_icon
    end,

	isTarget = function( self, abilityOwner, unit, targetUnit )

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end
		
		if not targetUnit:getTraits().mainframe_ice then
			return false
		end
		
		if targetUnit:getTraits().mainframe_ice then
			if not (targetUnit:getTraits().mainframe_ice > 0) then
				return false
			end
		end
		
		-- if targetUnit:getTraits().mainframe_camera then
			-- return false
		-- end --commented out for now since you CAN use buster chips on cameras so why wouldn't you be able to hack them

		-- if (targetUnit:getTraits().hacker ~= nil) and (targetUnit:getTraits().hacker ~= abilityOwner:getID()) then 
			-- return false, STRINGS.MANUALHACKING.TOOLTIPS.ALREADYHACKING 
		-- end

		return true
	end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, unit )

			local x0, y0 = unit:getLocation()
			local units = {}
			for _, targetUnit in pairs(sim:getAllUnits()) do
				local x1, y1 = targetUnit:getLocation()
				if x1 and self:isTarget( abilityOwner, unit, targetUnit ) then
					local range = mathutil.dist2d( x0, y0, x1, y1 )
					
						if range <= 1 and simquery.isConnected( sim, sim:getCell( x0, y0 ), sim:getCell( x1, y1 ) ) then
							table.insert( units, targetUnit )
						end
					
				end
			end

			return targets.unitTarget( game, units, self, abilityOwner, unit )
		end,
		
	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
	
		 if abilityOwner ~= unit and abilityOwner:getUnitOwner() ~= unit then
                return false
            end
		
		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit then
			assert( self:isTarget( abilityOwner, unit, targetUnit ))
			if targetUnit:getTraits().mod_beinghacked and not (abilityOwner:getTraits().mod_data_hacking and abilityOwner:getTraits().mod_data_hacking == targetUnit:getID()) then
				return false, STRINGS.UI.REASON.ALREADY_HACKING or "already hacking"
			end
			if targetUnit:getTraits().mainframe_ice and targetUnit:getTraits().mainframe_ice <= 0 then
				return false
			end
			if targetUnit:getTraits().mod_beingsniffed then
				return false, STRINGS.MANUALHACKING.TOOLTIPS.CANNOT_HACK_WHILE_SNIFFING
			end
		end
		
		if self.hackingDisabled then
			return false
		end	
		
		return abilityutil.checkRequirements( abilityOwner, unit)
	end,
		
	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local mainframe = include( "sim/mainframe" )
		local userUnit = unit
		local targetUnit = sim:getUnit( targetUnitID )
		assert( targetUnit, "No target : "..tostring(targetUnitID))
		
		if userUnit:getTraits().mod_data_hacking and (userUnit:getTraits().mod_data_hacking ~= targetUnit:getID() ) then
			-- reset hacking already being done on different object
			local oldTarget = sim:getUnit(userUnit:getTraits().mod_data_hacking)
			if oldTarget then
				oldTarget:getTraits().hacker = nil
				oldTarget:getTraits().mod_beinghacked = nil
			end
		end
		
		if userUnit:getTraits().mod_data_hacking and (userUnit:getTraits().mod_data_hacking == targetUnit:getID() ) then

			-- stop hacking!
			userUnit:getTraits().mod_data_hacking = nil
			userUnit:getTraits().data_hacking = nil
			targetUnit:getTraits().mod_beinghacked = nil
			targetUnit:getTraits().hacker = nil
			userUnit:getSounds().spot = nil
			if not userUnit:getTraits().isDrone then
				sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = userUnit } )
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
			targetUnit:getTraits().progress = nil
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
			
		else
			local x0,y0 = userUnit:getLocation()
			local x1,y1 = targetUnit:getLocation()			
			local tempfacing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
			userUnit:getTraits().data_hacking = true
			userUnit:getTraits().mod_data_hacking = targetUnit:getID()
			targetUnit:getTraits().mod_beinghacked = userUnit:getID()
			userUnit:setFacing(tempfacing)	
			if not userUnit:getTraits().isDrone then
				sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = userUnit:getID(), facing = tempfacing, sound = simdefs.SOUNDPATH_USE_CONSOLE , soundFrame = 10, useTinkerMonst3r=true } )
			end
				
			if not targetUnit:getTraits().progress then --can't remember if I still need this but... \o/
				targetUnit:getTraits().progress = 0	
				-- targetUnit:getTraits().progressMax = math.huge
			end

			if sim:isVersion("0.17.11") then
				userUnit:getTraits().isMeleeAiming = false
				userUnit:setAiming(false)	
			end		

			targetUnit:getTraits().mod_beinghacked = userUnit:getID()
			userUnit:getTraits().data_hacking = targetUnit:getID()			
			userUnit:getTraits().data_hacking_spotsound = true
			userUnit:getSounds().spot = "SpySociety/Actions/monst3r_hacking"
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit })
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit })
			sim:processReactions( userUnit ) -- this will get you shot if you start hacking while being overwatched! :)
			
			--Age of Lever compatibility: reset status if anyone other than LEGIT is caught doing it
			if SCRIPT_PATHS.W93_FuncLibActive then
				simquery.LEVER_resetStatus( sim, userUnit, "LEGIT", true)
			end
		
		end

	end,
	
	onSpawnAbility = function( self, sim, unit )
		self.abilityOwner = unit
		sim:addTrigger( simdefs.TRG_START_TURN, self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_END_TURN, self )
		sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )
		if (sim:getParams().difficultyOptions.MH_incognita == nil) and (sim:getParams() and sim:getParams().difficultyOptions.MH_weakenIncognita or (sim:getParams().difficultyOptions.incognita_active and ( sim:getParams().difficultyOptions.incognita_active == 1) )) then
			self.balancedHacking = true --legacy for backwards compatibility
			-- log:write("old slow hack")
		end
		-- if sim:getParams().difficultyOptions.MH_incognita and (sim:getParams().difficultyOptions.MH_incognita < 2) then
			-- -- log:write("new slow hack")
			-- self.balancedHacking = true
		-- end
		if sim:getParams().difficultyOptions.MH_PWRhack and (sim:getParams().difficultyOptions.MH_PWRhack > 0) then
			self.balancedHacking = true
			self.noAI_PWRcost = sim:getParams().difficultyOptions.MH_PWRhack
		end	
		if not (sim:getParams().difficultyOptions.icebreaker and (sim:getParams().difficultyOptions.icebreaker == true)) then 
			self.hackingDisabled = true
			self.alwaysShow = false
		end		
	end,
	
	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_END_TURN, self )
		sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		self.abilityOwner = nil
		self.balancedHacking = nil
	end,
	
	onTrigger = function( self, sim, evType, evData )

		if evType == simdefs.TRG_UNIT_WARP and ( evData.unit:getTraits().mod_data_hacking or evData.unit:getTraits().mod_beinghacked ) then

			--warped unit could be agent OR device moving away on enemy turn
			local userUnit = nil
			local targetUnit = nil
			if evData.unit:getTraits().mod_data_hacking then
				userUnit = evData.unit
				targetUnit = sim:getUnit(evData.unit:getTraits().mod_data_hacking)
			elseif evData.unit:getTraits().hacker or evData.unit:getTraits().mod_beinghacked then
				local hackerman = evData.unit:getTraits().hacker or evData.unit:getTraits().mod_beinghacked
				userUnit = sim:getUnit(hackerman)
				targetUnit = evData.unit
			end
			
			if userUnit and targetUnit and userUnit:isValid() and userUnit:getLocation() and targetUnit:isValid() and targetUnit:getLocation() then
				local canReach = simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() )
			-- log:write("can reach")
			-- log:write(util.stringize(canReach,2))
			
				-- if not simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() ) then
				-- log:write("LOG targetUnit resetting")
					userUnit:getTraits().mod_data_hacking = nil
					userUnit:getTraits().data_hacking = nil
					targetUnit:getTraits().progress = nil
					targetUnit:getTraits().hacker = nil
					targetUnit:getTraits().mod_beinghacked = nil
					userUnit:getSounds().spot = nil
					-- sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = userUnit } ) -- this makes the agent refreshing smoother but makes the moving guard/drone animation stutter so... \o/
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
				-- end
			end
		
		elseif evType == simdefs.TRG_END_TURN then 
		-- block to reset hacking that's still happening by mistake
			if evData:isPC() then
				for _, Unit in pairs(sim:getAllUnits()) do
					if Unit:getTraits().mainframe_ice ~= nil and Unit:getTraits().mainframe_ice <= 0 then
						if Unit:getTraits().hacker or Unit:getTraits().mod_beinghacked then 
							Unit:getTraits().mod_beinghacked = nil
							if not Unit:getTraits().databank then
								local hackerman = Unit:getTraits().hacker or Unit:getTraits().mod_beinghacked
								local hacker = sim:getUnit( hackerman ) 
								if hacker then
									hacker:getTraits().data_hacking = nil
									Unit:getTraits().hacker = nil
									hacker:getSounds().spot = nil
									if not hacker:getTraits().isDrone then
										sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = hacker } )
									end
									sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = hacker } )
								end
							end
							sim:dispatchEvent(simdefs.EV_UNIT_REFRESH, {unit = Unit } )
						end
					end
				end
			end
			
		elseif evType == simdefs.TRG_ICE_BROKEN then
			local hackedUnit = evData.unit 
			if hackedUnit:getTraits().mod_beinghacked and (hackedUnit:getTraits().mainframe_ice and (hackedUnit:getTraits().mainframe_ice <= 0)) then
				local hackerman = sim:getUnit(hackedUnit:getTraits().mod_beinghacked)
				if hackerman then
					-- log:write("LOG stop hacking")
					hackerman:getTraits().mod_data_hacking = nil
					hackerman:getSounds().spot = nil
					hackerman:getTraits().data_hacking = nil
					if not hackerman:getTraits().isDrone then
						sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = hackerman } )
					end
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= hackerman } )
				end
				hackedUnit:getTraits().mod_beinghacked = nil
				hackedUnit:getSounds().spot = nil
				hackedUnit:getTraits().hacker = nil		
			end
		elseif evType == simdefs.TRG_START_TURN then
			
			local userUnit = self.abilityOwner
			local mainframe = include( "sim/mainframe" )
			-- log:write("LOG: userUnit")
			-- log:write(util.stringize(userUnit,2))
		
			if userUnit:getTraits().mod_data_hacking 
			-- and userUnit:getTraits().data_hacking 
			then 
				local targetID = userUnit:getTraits().mod_data_hacking or 0
				-- log:write("LOG: targetunit")
				-- log:write(util.stringize(targetUnit,2))
				-- log:write(util.stringize(userUnit:getTraits().data_hacking,2))
				local targetUnit = sim:getUnit( targetID )
			
				if not simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() ) then
					userUnit:getTraits().mod_data_hacking = nil
					userUnit:getTraits().data_hacking = nil
					targetUnit:getTraits().progress = nil
					userUnit:getSounds().spot = nil
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
					targetUnit:getTraits().hacker = nil
					targetUnit:getTraits().mod_beinghacked = nil
					 -- stop hacking if target has moved out of range!
				end
			
				if targetUnit:getTraits().progress and evData:isPC() then
					local ice = targetUnit:getTraits().mainframe_ice
					local icebreaker = 0
					local hackedall = false
					local hackingskill = userUnit:getSkillLevel("hacking")
					-- log:write("LOG: hacking skill")
					-- log:write(util.stringize(hackingskill,2))
					
					if ice and ice > 0 and hackingskill then
						if hackingskill == 1 then
							icebreaker = 1
						elseif hackingskill == 2 or hackingskill == 3 then 
							icebreaker = 2
						elseif hackingskill > 3 then
							icebreaker = 3
							-- if hackingskill == 5 and icebreaker >= ice then
								-- self.daemonReversalAdd = 100
							-- end
							-- 20% chance to reverse daemon if daemon!
						end
						if icebreaker >= ice then
							hackedall = true
						end
					end
					-- log:write("LOG: ice, icebreaker1")
					-- log:write(util.stringize(ice,2))
					-- log:write(util.stringize(icebreaker,2))
					
					if self.balancedHacking and (sim:getPC():getCpus() < self.noAI_PWRcost) then
						icebreaker = 0 -- no hacking if using the "balanced" version of this ability and not enough PWR!
						hackedall = false
						local x1, y1 = targetUnit:getLocation()
						sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT, {
						unit = targetUnit,
						txt = util.sformat(STRINGS.UI.REASON.NOT_ENOUGH_PWR ),
						x = x1, y = y1,
						color = {r = 255/255, g = 255/255, b = 51/255, a = 1 },
					} )
					end
										
					-- HACKING execution block
					-- if hackedall and hackingskill > 4 then 
						-- if targetUnit:getTraits().mainframe_program then
							-- targetUnit:getTraits().mainframe_program = nil
							-- sim:dispatchEvent( simdefs.EV_KILL_DAEMON, {unit = targetUnit})
							
							-- if targetUnit:getTraits().daemonHost then
								-- sim:getUnit(targetUnit:getTraits().daemonHost):killUnit(sim)
								-- targetUnit:getTraits().daemonHost =nil
							-- end
						-- end
					
					-- end
					--daemon killing moved to daemonsniff!

					
					if icebreaker > 0 then
						mainframe.breakIce( sim, targetUnit, math.min( ice, icebreaker ) )
						-- log:write("LOG manual hacking breaking ice")
						if self.balancedHacking then
							sim:getPC():addCPUs( (-self.noAI_PWRcost
							), sim )
						end
					end
					
					if hackedall then
						-- log:write("LOG icebreaker: all firewalls down")
						userUnit:getTraits().mod_data_hacking = nil
						userUnit:getTraits().data_hacking = nil
						targetUnit:getTraits().mod_beinghacked = nil
						targetUnit:getTraits().hacker = nil
						userUnit:getSounds().spot = nil
						if not userUnit:getTraits().isDrone then
							sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = userUnit } )
						end
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
						targetUnit:getTraits().progress = nil
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
						-- no 'process reactions' here: finish hacking from previous turn without being shot
					end
					-- // execution block
				end
			end
		end
	
	end,
	}

return mod_hackdevice