local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )


local mod_hackdevice_fast = 
	{
	name = STRINGS.ABILITIES.SCAN_DEVICE,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
    proxy = true,
	alwaysShow = true,
	HUDpriority = 2,
	cpu_cost = 3,
	triggersOverwatch = true,
	iconColor= util.color( 163/255, 0/255, 0/255 ),
	iconColorHover= util.color( 1,1,1 ),
	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit( targetID )
		local hackingskill = abilityUser:getSkillLevel("hacking")
		local icebreaker = 0
		if hackingskill == 1 then
			icebreaker = 1
		elseif hackingskill == 2 or hackingskill == 3 then 
			icebreaker = 2
		elseif hackingskill > 3 then
			icebreaker = 3
		end
		if hackingskill and hackingskill < 4 then
			return util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.FASTHACKING1, icebreaker)
		else
			return util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.FASTHACKING2, icebreaker)
		end
	end,
		
	getName = function( self, sim, unit )
		return STRINGS.MANUALHACKING.TOOLTIPS.FASTHACKINGNAME
	end,

    getProfileIcon = function( self, sim, abilityOwner )
        return self.profile_icon
    end,

	isTarget = function( self, abilityOwner, unit, targetUnit )

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end
		
		if not targetUnit:getTraits().mainframe_ice then
			return false
		end
		
		if targetUnit:getTraits().mainframe_ice then
			if not (targetUnit:getTraits().mainframe_ice > 0) then
				return false
			end
		end

		-- if targetUnit:getTraits().mainframe_camera then
			-- return false
		-- end

		if self.hackingDisabled then
			return false
		end		
		
		return true
	end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, unit )

		local x0, y0 = unit:getLocation()
		local units = {}
		for _, targetUnit in pairs(sim:getAllUnits()) do
			local x1, y1 = targetUnit:getLocation()
			if x1 and self:isTarget( abilityOwner, unit, targetUnit ) then
				local range = mathutil.dist2d( x0, y0, x1, y1 )
				
					
					if range <= 1 and simquery.isConnected( sim, sim:getCell( x0, y0 ), sim:getCell( x1, y1 ) ) then
						table.insert( units, targetUnit )
					end
				
			end
		end

		return targets.unitTarget( game, units, self, abilityOwner, unit )
	end,
		
	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
	
		if abilityOwner ~= unit and abilityOwner:getUnitOwner() ~= unit then
                return false
        end
		
		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit then
			assert( self:isTarget( abilityOwner, unit, targetUnit ))
		end
		
		local PWR = unit:getPlayerOwner():getCpus()
		if PWR < self.cpu_cost then
			return false,  STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
		
		return abilityutil.checkRequirements( abilityOwner, unit)
	end,
		
	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local mainframe = include( "sim/mainframe" )
		local userUnit = unit
		local targetUnit = sim:getUnit( targetUnitID )
		assert( targetUnit, "No target : "..tostring(targetUnitID))
		
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = targetUnit:getLocation()			
		local tempfacing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )

		userUnit:setFacing(tempfacing)	
		if not userUnit:getTraits().isDrone then
			sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = userUnit:getID(), facing = tempfacing, sound = "SpySociety/Actions/monst3r_jackin" , soundFrame = 16, } )
		end

		local ice = targetUnit:getTraits().mainframe_ice
		local icebreaker = 0
		local hackedall = false
		local hackingskill = userUnit:getSkillLevel("hacking")		

		if ice and ice > 0 and hackingskill then
			if hackingskill == 1 then
				icebreaker = 1
			elseif hackingskill == 2 or hackingskill == 3 then 
				icebreaker = 2
			elseif hackingskill > 3 then
				icebreaker = 3
			end

		end
		
		if icebreaker >= ice then
			hackedall = true
		end

		mainframe.breakIce( sim, targetUnit, math.min( ice, icebreaker ) )
		if hackedall and hackingskill and hackingskill < 5 then
			sim:getNPC():addMainframeAbility(sim, "castle", nil, 0 )
		end
		if hackedall then
			userUnit:getTraits().mod_data_hacking = nil
			
		end
		sim:getPC():addCPUs( (-3), sim )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit })
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit })
			
		sim:processReactions( userUnit )
		
		--Age of Lever compatibility: reset status if anyone other than LEGIT is caught doing it
		if SCRIPT_PATHS.W93_FuncLibActive then
			simquery.LEVER_resetStatus( sim, userUnit, "LEGIT", true)
		end			
		

	end,
	
	onSpawnAbility = function( self, sim, unit )
		if not (sim:getParams().difficultyOptions.icebreaker and (sim:getParams().difficultyOptions.icebreaker == true)) then 
			self.hackingDisabled = true
			self.alwaysShow = false
		end
	end,	
	
	}

return mod_hackdevice_fast