local util = include( "modules/util" )
local hud = include( "hud/hud" )
local hudtarget = include( "hud/targeting")
local cdefs = include( "client_defs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local mui_defs = include( "mui/mui_defs")
local mui_tooltip = include("mui/mui_tooltip")
local agent_actions = include( "hud/agent_actions" )
local agent_panel = include( "hud/agent_panel" )

local oldCreateHud = hud.createHud

local function clearMovementRange( self )
	self._game.boardRig:clearMovementTiles()
	self._game.boardRig:clearCloakTiles()
	self._revealCells = nil
	self._cloakCells = nil
end

local function showMovement( hud, unit, moveTable, pathCost )
	local sim = hud._game.simCore

	if hud._movePreview then
		hud._game.boardRig:unchainCells( hud._movePreview.hiliteID )
		local rig = hud._game.boardRig:getUnitRig( hud._movePreview.unitID )
		local prevUnit = sim:getUnit( hud._movePreview.unitID )
		if rig and not hud._abilityPreview then
			rig:previewMovement( 0 )
		end
		hud._movePreview = nil

		hud._home_panel:refreshAgent( prevUnit )
	end

	if moveTable then
		hud._movePreview = { unitID = unit:getID(), pathCost = pathCost }
		if unit:getMP() >= pathCost then
			hud._movePreview.hiliteID = hud._game.boardRig:chainCells( moveTable )
			local rig = hud._game.boardRig:getUnitRig( unit:getID() )
			if rig then
				rig:previewMovement( pathCost )
				hud._abilityPreview = false 
			end
		else
			hud._movePreview.hiliteID = hud._game.boardRig:chainCells( moveTable, {r=0.2, g=0.2, b=0.2, a=0.8}, nil, true )
		end

		hud._home_panel:refreshAgent( unit )
	elseif MOAIEnvironment.OS == "NX" then
		selectedx = -1
		selectedy = -1
	end
end

local function previewMovement(hud, unit, cellx, celly)
	local sim = hud._game.simCore 
	local simdefs = sim:getDefs()

	hud._bValidMovement = false

	if unit and sim:getCurrentPlayer() and unit:getPlayerOwner() == sim:getCurrentPlayer() and unit:hasTrait("mp") and unit:canAct() then
		local startcell = sim:getCell( unit:getLocation() )
		local endcell = startcell
		if cellx and celly then
			endcell = sim:getCell( cellx, celly )
		end

		if startcell ~= endcell and endcell then
			local moveTable, pathCost = sim:getQuery().findPath( sim, unit, startcell, endcell, math.max( 15, unit:getMP() ) )
			if moveTable then
				hud._bValidMovement = unit:getMP() >= pathCost
				table.insert( moveTable, 1, { x = startcell.x, y = startcell.y } )
				showMovement(hud, unit, moveTable, pathCost )
				return
			end
		end
	end

	showMovement( hud, nil )
end

local function transition( hud, state, stateData )	
	if state == hud._state then
        	return
	end

	local sim = hud._game.simCore

	if hud._state == 9 then
		hud._game:skip()
	end

	if hud._stateData and hud._stateData.hiliteID then
		hud._game.boardRig:unhiliteCells( hud._stateData.hiliteID )
	elseif hud._stateData and hud._stateData.ability then
		if hud._stateData.ability.endTargeting then
			hud._stateData.ability:endTargeting( hud )
		end
		if hud._stateData.targetHandler and hud._stateData.targetHandler.endTargeting then
			hud._stateData.targetHandler:endTargeting( hud )
		end
	end

	if state == 4 and stateData and stateData.ability then 
		if stateData.ability.startTargeting then
			stateData.ability:startTargeting( hud )
		end
		if stateData.targetHandler and stateData.targetHandler.startTargeting then
			stateData.targetHandler:startTargeting( agent_panel.buttonLocator( hud ) )
		end
	end

	hud._state = state
	hud._stateData = stateData
	hud._screen._ignoreEvents = false
	
	if hud._state == 0 then
		hud._hideCubeCursor = false
		MOAISim.setCursor( cdefs.CURSOR_DEFAULT )
		
		if hud:getSelectedUnit() and not hud:getSelectedUnit()._isPlayer then
			hud:showMovementRange( hud:getSelectedUnit() )
			previewMovement( hud, hud:getSelectedUnit(), hud._tooltipX, hud._tooltipY )
		end
	else
        	if hud._state == 5 then
			hud._hideCubeCursor = true
			MOAISim.setCursor( cdefs.CURSOR_TARGET )
        	end

		clearMovementRange( hud )
		showMovement( hud, nil )
	end

	hud:refreshHud( )

	if hud._state == 5 and MOAIEnvironment.Use_Gamepad and MOAIEnvironment.Use_Gamepad == 1 then
		if hud._stateData.targetHandler and hud._stateData.targetHandler.targetToWorld then
			hud._screen._ignoreEvents = true
			local event = {}
			event.px = 1
			event.py = 1
			event.eventType = mui_defs.EVENT_LeftStickMove
			hud._stateData.targetHandler:onInputEvent( event )
			hud._screen:setFocus( hud._screen._currentFocusedWidget )
			showMovement( hud, nil )
			hud._screen.binder.black:setVisible(false)
		end
	end
end

hud.createHud = function( ... )
	local hudObject = oldCreateHud( ... )

    	local oldRefreshHud = hudObject.refreshHud
    	local oldOnSimEvent = hudObject.onSimEvent
	local oldShowMainframe = hudObject.showMainframe
	local oldHideMainframe = hudObject.hideMainframe

	local function refreshTrackerMusic( hud, stage )
		stage = math.max( stage or 0, hud._musicStage or 0 )
		local isClimax = stage >= simdefs.TRACKER_MAXSTAGE or hud._game.simCore:getClimax()

		if not isClimax then
			for i, unit in pairs( hud._game.simCore:getPC():getAgents() ) do
				if simquery.isUnitUnderOverwatch( unit ) then
					isClimax = true
					break
				end
			end
		end

		MOAIFmodDesigner.setMusicProperty("intensity", math.min(stage,6) )
		if isClimax then
			MOAIFmodDesigner.setMusicProperty("kick",1)
    		else
			MOAIFmodDesigner.setMusicProperty("kick",0)
		end

		hud._musicStage = stage
	end

	hudObject._ai_panel = SCRIPT_PATHS.aiPanel.panel( hudObject._screen, hudObject )
	hudObject._screen.binder.resourcePnl.binder.hashDisplay:setVisible(false)
	hudObject._screen.binder.resourcePnl.binder.hashDisplay:setColor(140/255,255/255,255/255,1)
	hudObject._screen.binder.resourcePnl.binder.hashDisplay:setTooltip( mui_tooltip( STRINGS.PROGEXTEND.UI.HASH_DISPLAY_NAME, STRINGS.PROGEXTEND.UI.HASH_DISPLAY_DESC, "PE_HASHKEY_DISPLAY" ) )

	function hudObject:onClickAiBtn( button, event )
		if hudObject._state ~= 0 and hudObject._state ~= 9 then
    			transition( self, 0 )
    		end

    		if self._state == 0 then
			self._ai_panel:toggle()
		end
	end

	function hudObject:showMainframe()
		oldShowMainframe(hudObject)
		hudObject._ai_panel:refresh()
	end

	function hudObject:hideMainframe()
		oldHideMainframe(hudObject)
		hudObject._ai_panel:refresh()
	end

    	function hudObject:refreshHud()
        	oldRefreshHud( self )
        	refreshTrackerMusic( self )
		
		if self._game.simCore:getCurrentPlayer() == self._game.simCore:getPC() and self._game.simCore:getNPC():findMainframeAbility("W93_AI_assembly") then
			if self._game.simCore:getNPC():hasMainframeAbility("W93_AI_assembly") then
				self._screen.binder.resourcePnl.binder.hashDisplay:setText(util.sformat(STRINGS.PROGEXTEND.UI.HASH_DISPLAY_BAR, self._game.simCore:getPC():getTraits().aiToken or 0, self._game.simCore:getPC():getTraits().aiTokenMax or 0 ))
				self._screen.binder.resourcePnl.binder.hashDisplay:setVisible(true)
			end
		else
			self._screen.binder.resourcePnl.binder.hashDisplay:setVisible(false)
		end
    	end

    	function hudObject:onSimEvent( ev )
		hudObject._ai_panel:onSimEvent( ev )
        	oldOnSimEvent( self, ev )
        	if ev.eventType == simdefs.EV_ADVANCE_TRACKER or ev.eventType == "used_radio" then
            		refreshTrackerMusic( self )
		end
    	end

	function hudObject:transitionAbilityTarget( abilityOwner, abilityUser, ability, extra )
		if hudObject._state == 4 then
			hudObject:transitionNull()
    		elseif hudObject._state == 9 then
        		return
    		end

		if not ability.acquireTargets then
			agent_actions.performAbility( hudObject._game, abilityOwner, abilityUser, ability, extra )
			MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_MENU_CONFIRM )
			if MOAIEnvironment.Use_Gamepad and MOAIEnvironment.Use_Gamepad == 1 then
				if self._isMainframe then
					if ability.equip_program then
						self:goMainframe()
						MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_MAINFRAME_MODE_SWITCH, nil, 0.1 )
					else

					end
				end
			end
		else
			local targetHandler = ability:acquireTargets( hudtarget, hudObject._game, hudObject._game.simCore, abilityOwner, abilityUser )
			if not targetHandler:hasTargets() then
				hudObject:showWarning( STRINGS.UI.WARNING_NO_TARGETS )
			else
				local defaultTarget = targetHandler:getDefaultTarget()
				if defaultTarget and not ability.noDefaultTarget then
					agent_actions.performAbility( hudObject._game, abilityOwner, abilityUser, ability, defaultTarget )
				else
					if ability.startTargeting then
						ability:startTargeting( hudtarget, hudObject._game, hudObject._game.simCore, abilityOwner, abilityUser )
					end

					transition( hudObject, 4, { abilityOwner = abilityOwner, abilityUser = abilityUser, ability = ability, targetHandler = targetHandler } )
					if MOAIEnvironment.Use_Gamepad and MOAIEnvironment.Use_Gamepad == 1 then
						if self._isMainframe then
							self:goMainframeTarget()
						end
					end
				end
			end
		end
	end

    	return hudObject
end