local mui = include( "mui/mui" )
local modalDialog = include( "states/state-modal-dialog" )
local util = include( "modules/util" )
local abilitydefs = include( "sim/abilitydefs" )
local guiex = include( "guiex" )
local hud = include( "hud/hud" )
local items_panel = include( "hud/items_panel" )

-- FROM INTERACTIVE EVENTS BY CYBERBOY2000

local function showChoiceDialog( hud, headerTxt, bodyTxt, options, dType, tooltips, playerCredits, enemyProfile )
	assert( hud._choice_dialog == nil )

	local screen

	screen = mui.createScreen( "modal-event.lua" )

	hud._choice_dialog = screen
	mui.activateScreen( screen )

	screen.binder.pnl.binder.headerTxt:setText( headerTxt )
	screen.binder.pnl.binder.bodyTxt:setText("<c:8CFFFF>".. bodyTxt.."</>" )

	--selected agent
	local unit = hud:getSelectedUnit()
	if unit ~= nil and unit.getUnitData then
		screen.binder.pnl.binder.yourface.binder.portrait:bindBuild( unit:getUnitData().profile_build or unit:getUnitData().profile_anim )
		screen.binder.pnl.binder.yourface.binder.portrait:bindAnim( unit:getUnitData().profile_anim )
		screen.binder.pnl.binder.yourface:setVisible(true)
	else
		screen.binder.pnl.binder.yourface:setVisible(false)
	end

	screen.binder.pnl.binder.Item:setVisible(false)

	if enemyProfile == true then
		screen.binder.pnl.binder.theirface.binder.portrait:bindBuild( "portraits/portrait_security_build" )
		screen.binder.pnl.binder.theirface.binder.portrait:bindAnim( "portraits/portrait_animation_template" )
		screen.binder.pnl.binder.theirface.binder.portrait:setColor(0,0,0,1)
		screen.binder.pnl.binder.theirface:setVisible(true)
	elseif enemyProfile then
		screen.binder.pnl.binder.theirface.binder.portrait:bindBuild( enemyProfile.build )
		screen.binder.pnl.binder.theirface.binder.portrait:bindAnim( enemyProfile.anim )
		screen.binder.pnl.binder.theirface:setVisible(true)
		if enemyProfile.color then
			screen.binder.pnl.binder.theirface.binder.portrait:setColor(enemyProfile.color.r,enemyProfile.color.g,enemyProfile.color.b,1)
		end
	else
		screen.binder.pnl.binder.theirface:setVisible(false)
	end

	--prevent copy pasta issue
	if not screen.binder.pnl:findWidget( "optionBtn5" ) then
		local btn5 = screen.binder.pnl:findWidget( "maa_optionBtn5" )
		btn5._name = "optionBtn5"
	end

	local use_list = false
	if #options > 5 then
		use_list = true
	end

	-- Fill out the dialog options.
	local result = nil
	local x = 1 --RaXaH: What is this used for? Remove?
	for i, btn in screen.binder.pnl.binder:forEach( "optionBtn" ) do
		if use_list or options[i] == nil then
			btn:setVisible( false )
		else
			btn:setVisible( true )
			btn:setText("<c:8CFFFF>"..  options[i] .."</>")
			btn.onClick = util.makeDelegate( nil, function() result = i end )
			x = x + 1
		end
	end

	if use_list then
		local lb = screen:findWidget("optionList")
		lb:setVisible(true)
		lb:clearItems()
		for i=#options, 1, -1 do
			local entry = lb:addItem(nil , nil)
			local btn = entry.binder.optionListBtn
			btn:setText("<c:8CFFFF>"..  options[i] .."</>")
			btn.onClick = util.makeDelegate( nil, function() result = i end )
		end
	end

	if (dType == "level" or dType == "augment") and tooltips ~= nil then
		for i, btn in screen.binder.pnl.binder:forEach( "optionBtn" ) do
			btn:setTooltip(tooltips[i])
		end
	end

	-- We are running in the vizThread coroutine.  Yield until a response is chosen by the UI.
	-- Note that the click handler will be triggered by the main coroutine, but we use a closure
	-- to inform us what the chosen result is.
	while result == nil do
		coroutine.yield()
	end

	mui.deactivateScreen( screen )
	hud._choice_dialog = nil

	hud._game.simCore:setChoice( result )

	return result
end

local function showDerekDialog( hud, headerTxt, bodyTxt, options, dType, tooltips, playerCredits, enemyProfile )
	assert( hud._choice_dialog == nil )

	local screen

	screen = mui.createScreen( "modal-event.lua" )

	hud._choice_dialog = screen
	mui.activateScreen( screen )

	screen.binder.pnl.binder.headerTxt:setText( headerTxt )
	screen.binder.pnl.binder.bodyTxt:setText("<c:8CFFFF>".. bodyTxt.."</>" )

	--selected agent
	local unit = hud:getSelectedUnit()
	if unit ~= nil and unit.getUnitData then
		screen.binder.pnl.binder.yourface.binder.portrait:bindBuild( unit:getUnitData().profile_build or unit:getUnitData().profile_anim )
		screen.binder.pnl.binder.yourface.binder.portrait:bindAnim( unit:getUnitData().profile_anim )
		screen.binder.pnl.binder.yourface:setVisible(true)
	else
		screen.binder.pnl.binder.yourface:setVisible(false)
	end

	screen.binder.pnl.binder.Item:setVisible(false)

	if enemyProfile == true then
		screen.binder.pnl.binder.theirface.binder.portrait:bindBuild( "portraits/portrait_security_build" )
		screen.binder.pnl.binder.theirface.binder.portrait:bindAnim( "portraits/portrait_animation_template" )
		screen.binder.pnl.binder.theirface.binder.portrait:setColor(0,0,0,1)
		screen.binder.pnl.binder.theirface:setVisible(true)
	elseif enemyProfile then
		screen.binder.pnl.binder.theirface.binder.portrait:bindBuild( enemyProfile.build )
		screen.binder.pnl.binder.theirface.binder.portrait:bindAnim( enemyProfile.anim )
		screen.binder.pnl.binder.theirface:setVisible(true)
		if enemyProfile.color then
			screen.binder.pnl.binder.theirface.binder.portrait:setColor(enemyProfile.color.r,enemyProfile.color.g,enemyProfile.color.b,1)
		end
	else
		screen.binder.pnl.binder.theirface:setVisible(false)
	end

	-- Fill out the dialog options.
	local result = nil

	--first option should always be cancel, every other option is a program
	for i, btn in screen.binder.pnl.binder:forEach( "optionBtn" ) do
		if i == 1 and options[i] then
			btn:setVisible( true )
			btn:setText("<c:8CFFFF>"..  options[i] .."</>")
			btn.onClick = util.makeDelegate( nil, function() result = i end )
		else
			btn:setVisible( false )
		end
	end

	--local programs = hud._game.simCore:getUnit( options[2] ).programs
	for i, widget in screen.binder.pnl.binder:forEach( "program" ) do
		if options[i+1] then
			local aug = hud._game.simCore:getUnit( options[i+1][1] )
			local prog = aug.programs[ options[i+1][2] ]
			widget:setVisible( true )
			guiex.updateButtonFromItem( hud._screen, nil, widget, prog )
			widget.binder.btn.onClick = util.makeDelegate( nil, function() result = i+1 end )
		else
			widget:setVisible( false )
		end
	end

	-- We are running in the vizThread coroutine.  Yield until a response is chosen by the UI.
	-- Note that the click handler will be triggered by the main coroutine, but we use a closure
	-- to inform us what the chosen result is.
	while result == nil do
		coroutine.yield()
	end

	mui.deactivateScreen( screen )
	hud._choice_dialog = nil

	hud._game.simCore:setChoice( result )

	return result
end

local baseCreateHud = hud.createHud
hud.createHud = function( ... )
    local hudObject = baseCreateHud( ... )

    local baseOnSimEvent = hudObject.onSimEvent

    function hudObject:onSimEvent( ev )
		if ev.eventType == "MAA_EV_CHOICE_DIALOG" then
			return showChoiceDialog( self, ev.eventData.headerTxt, ev.eventData.bodyTxt, ev.eventData.options, ev.eventData.type, ev.eventData.tooltips, ev.eventData.playerCredits, ev.eventData.enemyProfile )
		elseif ev.eventType == "MAA_EV_PROGRAM_DIALOG" then
			return showDerekDialog( self, ev.eventData.headerTxt, ev.eventData.bodyTxt, ev.eventData.options, ev.eventData.type, ev.eventData.tooltips, ev.eventData.playerCredits, ev.eventData.enemyProfile )
		elseif ev.eventType == "MAA_EV_ITEMS_PANEL" then
			self:showItemsPanel( items_panel.transfer( self, ev.eventData.unit, ev.eventData.targetUnit ))
		else
			baseOnSimEvent( self, ev )
		end
    end

    return hudObject
end

local baseRefreshHud = hud.refreshHud
function hud:refreshHud()
	if self._state == 0 then
		if self.maa_pending_ability then
			self.maa_pending_ability.maa_state = nil
		end
	end

	baseRefreshHud( self )
end

-- local baseTransitionNull = hud.transitionNull
-- function hud:transitionNull()
-- 	print("transitionNull")
-- 	if self.maa_pending_ability then
-- 		print("reset pending abilities")
-- 		self.maa_pending_ability.maa_state = nil
-- 	end
--
-- 	baseTransitionNull( self )
-- end
