local inventory = include("sim/inventory")
local serverdefs = include( "modules/serverdefs" )
local simdefs = include( "sim/simdefs" )
local simquery = include("sim/simquery")

local baseUseItem = inventory.useItem

function inventory.useItem( sim, unit, item )
	--temporarily apply overdrive 2.0 bonus
	local item_cdMax = item:getTraits().cooldownMax
	if sim:getPC():getTraits().coit_cooldownMod and item_cdMax and item_cdMax > 0 then
		item:getTraits().cooldownMax = math.max( item_cdMax + sim:getPC():getTraits().coit_cooldownMod, 1 )
	end
	local delta_item_cdMax = nil
	if item_cdMax then
		delta_item_cdMax = item:getTraits().cooldownMax - item_cdMax
	end
	
	baseUseItem( sim, unit, item )
	
	if delta_item_cdMax then
		item:getTraits().cooldownMax = item:getTraits().cooldownMax - delta_item_cdMax
	end
	
	if item:getTraits().coit_spawnsDaemon then
		local programList = nil
					
		programList = sim:handleOverrideAbility(serverdefs.OMNI_PROGRAM_LIST)

		if sim and sim:getParams().difficultyOptions.daemonQuantity == "LESS" then
			programList = sim:handleOverrideAbility(serverdefs.OMNI_PROGRAM_LIST_EASY)
		end

		daemon = programList[sim:nextRand(1, #programList)]

		sim:getNPC():addMainframeAbility( sim, daemon )
	end
	if item:getTraits().coit_notifiyGuard then
		if unit and unit:getLocation() then
			local x0, y0 = unit:getLocation()
			if x0 and y0 then
				sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_CAMERA )
			end
		end
	end
	if item:getTraits().coit_increase_alarm then
		sim:trackerAdvance( item:getTraits().coit_increase_alarm )
	end
	if item:getTraits().ammo and item:getTraits().coit_no_ammo_usage then
		item:getTraits().ammo = item:getTraits().ammo + 1
	end
	if item:getTraits().charges and item:getTraits().coit_no_ammo_usage then
		item:getTraits().charges = item:getTraits().charges + 1
	end

	
	if item:getTraits().coit_temporary_buffs then
		for _, buff in ipairs( item:getTraits().coit_temporary_buffs ) do
			if item:getTraits()[buff.trait] then
				item:getTraits()[buff.trait] = math.max( item:getTraits()[buff.trait] - (buff.value or 0), 0 )
			end
		end
		
		item:getTraits().coit_temporary_buffs = nil
	end
	
	if item:getTraits().coit_multishot then
		if item:getTraits().coit_multishot == "idle" then
			item:getTraits().coit_multishot = "active"
		elseif item:getTraits().coit_multishot == "active" then
			--refund ap
			if unit and unit:getTraits().ap then
				unit:getTraits().ap = unit:getTraits().ap + 1
			end
		end
	end
end
