local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )
local cdefs = include( "client_defs" )
local simquery = include( "sim/simquery" )
local NEVER_SOLD = 10000

local function CamilaAugTooltip( tooltip, unit, userUnit )
    commondefs.onItemTooltip( tooltip, unit, userUnit )
	tooltip:addAbility(  STRINGS.BREACH_AGENTS.TOOLTIPS.SMOKE_ATTACK, STRINGS.BREACH_AGENTS.TOOLTIPS.SMOKE_ATTACK_DESC, "gui/icons/item_icons/items_icon_small/icon-item_smoke_grenade_small.png" )
	tooltip:addAbility(  STRINGS.BREACH_AGENTS.TOOLTIPS.AP_RESIST, STRINGS.BREACH_AGENTS.TOOLTIPS.AP_RESIST_DESC, "gui/icons/skills_icons/skills_icon_small/icon-item_strength_small.png" )
	-- local guarddefs = include("sim/unitdefs/guarddefs")
	-- if guarddefs.neptune_frost_guard then
		-- tooltip:addAbility(  STRINGS.BREACH_AGENTS.TOOLTIPS.NEPTUNE_IMMUNE, STRINGS.BREACH_AGENTS.TOOLTIPS.NEPTUNE_IMMUNE_DESC, "gui/icons/skills_icons/skills_icon_small/icon-item_toughness_small.png" )
	-- end
	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1],unit:getTraits().dlcFooter[2] )
	end	
end

-- unused
local function BethanyGunTooltip( tooltip, unit, userUnit )
    commondefs.onItemTooltip( tooltip, unit, userUnit )
	tooltip:addAbility(  STRINGS.BREACH_AGENTS.TOOLTIPS.SHIELD_NO_AP_LOSS, STRINGS.BREACH_AGENTS.TOOLTIPS.SHIELD_NO_AP_LOSS_DESC, "gui/icons/item_icons/items_icon_small/augment_bethany2_small.png" )
	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1],unit:getTraits().dlcFooter[2] )
	end
end

local function SilicaAugTooltip( tooltip, unit, userUnit )
    commondefs.onItemTooltip( tooltip, unit, userUnit )
	local agent = unit:getUnitOwner()
	if agent and agent:getTraits().notYetMoved then
		tooltip:addAbility(  STRINGS.BREACH_AGENTS.TOOLTIPS.NOT_YET_MOVED, STRINGS.BREACH_AGENTS.TOOLTIPS.NOT_YET_MOVED_DESC, "gui/icons/action_icons/Action_icon_Small/icon-item_melee_small.png" )
		-- icon-item_melee_small
		-- icon-item_melee_small-old
		-- icon-item_overwatch_small
	end
	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1],unit:getTraits().dlcFooter[2] )
	end	
end

local tool_templates =
{
	-- CAMILA ---------------------------------------
	ITB_augment_camila = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.CAMILA_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.CAMILA_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.CAMILA_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"ITB_smokeAttackBonus",true},{"ignoresFrostClouds", true}}, --ignoresFrostClouds is handled by Neptune mod
			installed = true,
		},
		value = 400,
		onTooltip = CamilaAugTooltip,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_camila_small.png",
    	profile_icon_100 = "gui/icons/item_icons/icon-item_augment_camila.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_CamilaAugment" }),
	},

	-- maybe dust bomb instead? acts like smoke but EMPs units
	-- lasts two turns
	ITB_item_smokegrenade_camila = util.extend( commondefs.grenade_template )
	{
        type = "smoke_grenade",
		name = STRINGS.BREACH_AGENTS.ITEMS.CAMILA_SMOKE_GRENADE_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.CAMILA_SMOKE_GRENADE_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.CAMILA_SMOKE_GRENADE_FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_smoke_grenade_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_smoke_grenade.png",
		kanim = "kanim_stickycam",
		sounds = {explode="SpySociety/Grenades/smokegrenade_explo", bounce="SpySociety/Grenades/bounce_smokegrenade"},
		traits = { on_spawn = "ITB_smoke_cloud_camila" , range=1, noghost = true, explodes = 0, usesCharges = true, charges = 1, chargesMax = 1, disposable = false, throwUnit = "ITB_item_smokegrenade_camila", },
		abilities = { "carryable", "recharge", "throw" },
		value = 300,
		-- floorWeight = 2,
		locator=true,
	},

	-- SILICA ---------------------------------------
	-- all guns act as xray rifles
	ITB_augment_silica = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"OFSilica",true}},
			modTrait = {{"mpMax",-1}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_silica1_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_silica1.png",
	},

	-- attacking if unit hasn't moved gives infinite attacks
	ITB_augment_silica_2 = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT2_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT2_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT2_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"ARCHSilica",true}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_silica2_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_silica2.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_SilicaAugment" }),
		onTooltip = SilicaAugTooltip,
	},

	item_tag_pistol_silica = util.extend( commondefs.weapon_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.SILICA_PISTOL_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.SILICA_PISTOL_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.SILICA_PISTOL_FLAVOR,
		icon = "itemrigs/FloorProp_Pistol.png",	
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_microchip_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_microchip.png",				
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", baseDamage = 0, canTag= true, noTargetAlert=true, ignoreArmor=true, nopwr_guards = {}, doNotInterruptMove = true, usesCharges = true, charges = 2, chargesMax = 2,},
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_dart", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		abilities = util.tmerge( { "recharge" }, commondefs.weapon_template.abilities ),
		weapon_anim = "kanim_precise_revolver",
		agent_anim = "anims_1h",
		value = 300,
	},	
	
	item_flurrydart_silica = util.extend( commondefs.weapon_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.SILICA_FLURRYDART_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.SILICA_FLURRYDART_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.SILICA_FLURRYDART_FLAVOR,
		icon = "itemrigs/FloorProp_Precision_SMG.png",		
		--profile_icon = "gui/items/item_pistol_56.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flurrydart_small.png",	
		profile_icon_100 = "gui/icons/item_icons/icon-item_flurrydart.png",			
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", baseDamage = 2, canSleep = true,  pwrCost = 2, energyWeapon = "idle", shots=3, nopwr_guards = {}},
		sounds = {shoot="SpySociety/Weapons/LowBore/shoot_handgun_silenced", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_precise_smg",
		agent_anim = "anims_2h",
		value = 600,
	},	

	-- BETHANY -----------------------------------------------
	-- cloak before alarm level 1
	ITB_augment_bethany = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_bethany_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_bethany.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_BethanyAugment" }),
	},

	-- starts with shield
	ITB_augment_bethany_2 = util.extend( commondefs.augment_template )
	{
		type = "ITB_shieldingRig",
		name = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_AUGMENT2_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_AUGMENT2_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_AUGMENT2_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			modTrait = {{"shields", 1},{"shieldsMax",1}},
			addTrait = {{"neutralize_shield",1}},
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_bethany2_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_bethany2.png",
	},
	-- ice gun. costs AP to carry unless user has shield
	-- actual ice gun
	ITB_item_icegun_bethany = util.extend( commondefs.weapon_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_ICEGUN_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_ICEGUN_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_ICEGUN_FLAVOR,
		icon = "itemrigs/FloorProp_Precision_SMG.png",		
		--profile_icon = "gui/items/item_pistol_56.png",
		-- onTooltip = BethanyGunTooltip,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_icegun_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_icegun.png",			
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", baseDamage = 3, canSleep = true, pwrCost = 3, freezeGun = true,
		-- shieldNotHeavy = true,
		-- mp_penalty = 2,
			-- ammo = 3, maxAmmo = 3,
		cooldown = 0, cooldownMax = 10,
			possibleFrozenAnims = {
				{"hitfrt", 0},
				-- {"hitbck", 16},
				{"death", 11},
				-- {"body_fall", 0},
				-- {"body_fall", 3},
				{"throw", 2},
				-- {"lean_throw", 1}, -- guards don't have this
				{"door_kick_pre", 10}, --!!
				{"door_kick_pre", 2},
				{"door_kick_pre", 8},
				{"door_kick_pre", 16},
			},
			frozenRenderFilter = { 
				shader=KLEIAnim.SHADER_FOW,
				r=85/255,
				g=222/255,
				b=227/255,
				a=0.75,
				lum=1.0, },
		},
		sounds = {shoot="SpySociety/Weapons/LowBore/shoot_handgun_silenced", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_bethany_icegun",
		agent_anim = "anims_2h",
		value = 600,
	},
	
	ITB_item_iceminegun_bethany = util.extend( commondefs.weapon_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_MINEGUN_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_MINEGUN_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.BETHANY_MINEGUN_FLAVOR,
		icon = "itemrigs/FloorProp_Precision_SMG.png",		
		--profile_icon = "gui/items/item_pistol_56.png",
		-- onTooltip = BethanyGunTooltip,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_icegun_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_icegun.png",			
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", pwrCost = 3,
		-- shieldNotHeavy = true,
		-- mp_penalty = 2,
			-- ammo = 3, maxAmmo = 3,
			ko_duration = 3,
		cooldown = 0, cooldownMax = 6, throwRange = 5,
		}, --throwRange trait does nothing
		abilities = { "carryable", "ITB_throwIceMine", "recharge", },
		sounds = {shoot="SpySociety/Weapons/LowBore/shoot_handgun_silenced", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_bethany_icegun",
		agent_anim = "anims_2h",
		value = 600,
		locator=true,
		-- createUpgradeParams = function( self, unit )
			-- return { traits = { ammo =  unit:getTraits().ammo } }
		-- end,
	},	
	
---------------------------------------------
	-- ARCHIMEDES
	-- using an item restores AP equal to item cooldown
	ITB_augment_archimedes = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"ITB_Archimedes_OF",true}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_generic_arm_small.png",
    	profile_icon_100 = "gui/icons/item_icons/icon-item_generic_arm.png",
	},	
	
	-- killing restores full ap, can't melee
	ITB_augment_archimedes_2 = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT2_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT2_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT2_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"ITB_Archimedes",true}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_generic_leg_small2.png",
    	profile_icon_100 = "gui/icons/item_icons/icon-item_generic_leg2.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_ArchimedesAugment" }),
	},

	ITB_item_archimedes_rifle = util.extend( commondefs.weapon_reloadable_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.RIFLE_ARCHIMEDES_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.RIFLE_ARCHIMEDES_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.RIFLE_ARCHIMEDES_FLAVOR,
		icon = "itemrigs/FloorProp_Precision_Rifle.png",
		--profile_icon = "gui/items/item_rifle_56.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_rifle_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_rifle.png",
		equipped_icon = "gui/items/equipped_rifle.png",
		-- traits = { weaponType="rifle", baseDamage = 1, ammo = 2, maxAmmo = 2, cooldown = 0, },
		traits = { weaponType="rifle", baseDamage = 1, cooldown = 0, cooldownMax = 3, pwrCost = 4, restrictedUse={{agentID="itb_archimedes",name=STRINGS.BREACH_AGENTS.AGENTS.ARCHIMEDES.NAME}} },
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_rifle_silenced", reload="SpySociety/Weapons/Precise/reload_rifle", use="SpySociety/Actions/item_pickup",shell="SpySociety/Weapons/Shells/shell_rifle_wood"},
		weapon_anim = "kanim_precise_rifle",
		agent_anim = "anims_2h",
		value =500,
		soldAfter = NEVER_SOLD,
	},
	
	-- Biogenic Dart without the armour piercing
	ITB_item_archimedes_pistol = util.extend( commondefs.weapon_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.BIODART_ARCHIMEDES_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.BIODART_ARCHIMEDES_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.BIODART_ARCHIMEDES_FLAVOR,
		icon = "itemrigs/FloorProp_Precision_Pistol.png",	
		--profile_icon = "gui/items/icon-item_gun_pistol.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_dart_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_dart.png",				
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", baseDamage = 3, cooldown = 0, cooldownMax = 7, canSleep = true },
        abilities = util.tmerge( { "recharge" }, commondefs.weapon_template.abilities ),
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_dart", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_precise_revolver",
		agent_anim = "anims_1h",
		value = 600,
		soldAfter = NEVER_SOLD,
	},	
	--------------------------------------------------------
	-- Lily
	-- +2 ap and free peeking before alarm level 1, -1 to melee KO damage
	ITB_augment_lily = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.LILY_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.LILY_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.LILY_AUGMENT_FLAVOR,
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"APboost",2},{"freePeeking",true},{"meleeDamageDebuff",true}},
			modTrait = {{"meleeDamage",(-1)}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_lily_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_lily.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_LilyAugment" }),
	},

	--------------------------------------------------------
	-- Henry
	-- can "squeeze" through other units (OLD)
	-- can path through other units. (NEW)
	ITB_augment_henry = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT_FLAVOR,
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			addTrait = {{"ITB_HenryImpass",true}},
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_kwan_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_kwan.png",
    	-- abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_giveHenryScurry" }),	 --because addAbilities  trait doesn't support more than one ability, and this is easier than rewriting the ability to be owned by item instead of agent
		abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_HenryImpassHelp" }), --handles auxiliary stuff
	},

	-- When escaping overwatch, gain AP equal to multiplier * number of previously overwatching enemies. (OLD)
	-- Overwatching guards won't track/turn to follow. (NEW)
	-- 
	ITB_augment_henry_2 = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT2_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT2_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.HENRY_AUGMENT2_FLAVOR,
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			addTrait = {{"ITB_HenryUntrackable",true}},
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_kwan2_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_kwan2.png",
    	-- abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_HenryOverwatch" }),
		abilities = commondefs.augment_template.abilities,
	},	
	---------------------------------------------------------
	-- Gana
	ITB_augment_gana = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.GANA_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.GANA_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.GANA_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"ITB_Gana",true}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_gana_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_gana.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_GanaAugment" }),
	},
	
	ITB_augment_gana_2 = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.GANA_AUGMENT2_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.GANA_AUGMENT2_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.GANA_AUGMENT2_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			addTrait = {{"ITB_Gana_a",true}},
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_gana2_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_gana2.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_GanaAugment2" }),
	},	

	ITB_item_gana_locktorch = util.extend(commondefs.item_template)
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.GANA_LOCK_OPENER_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.GANA_LOCK_OPENER_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.GANA_LOCK_OPENER_FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_lock_thermite_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_lock_thermite.png",
		traits = {
			charges = 1, chargesMax = 1, usesCharges = true, applyFn = "isSecurityExit", profile_icon="gui/icons/item_icons/items_icon_small/icon-item_lock_decoder_small.png", soundRange = simdefs.SOUND_RANGE_1,
		},
		requirements = { },
		abilities = { "ITB_GanaLockOpener", "carryable", "recharge" },
		value = 400,
		floorWeight = 1,
	},
	---------------------------------------------------------
	-- PROSPERO
	ITB_augment_prospero = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.PROSPERO_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.PROSPERO_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.PROSPERO_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_gana_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_gana.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_ProsperoAugment" }),
	},	

	ITB_augment_prospero_2 = util.extend( commondefs.augment_template )
	{
		name = STRINGS.BREACH_AGENTS.ITEMS.PROSPERO_AUGMENT_NAME,
		desc = STRINGS.BREACH_AGENTS.ITEMS.PROSPERO_AUGMENT_TIP,
		flavor = STRINGS.BREACH_AGENTS.ITEMS.PROSPERO_AUGMENT_FLAVOR,

		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
		},
		value = 400,
		profile_icon = "gui/icons/item_icons/items_icon_small/augment_gana_small.png",
    	profile_icon_100 = "gui/icons/item_icons/augment_gana.png",
    	abilities = util.tconcat( commondefs.augment_template.abilities, { "ITB_ProsperoAugment2" }),
	},	
}

for id, template in pairs(tool_templates) do
	template.id = id
end

return tool_templates