local util = include("modules/util")
local commondefs = include("sim/unitdefs/commondefs")
-- local simdefs = include( "sim/simdefs" )

local _melee_ontooltip = commondefs.melee_template.onTooltip
local _item_ontooltip = commondefs.item_template.onTooltip

local glitchtrap_base = util.extend(commondefs.item_template) {
    name = STRINGS.HEATSIGNATURE.GLITCHTRAP,
    desc = STRINGS.HEATSIGNATURE.GLITCHTRAP_TIP,
    flavor = STRINGS.HEATSIGNATURE.GLITCHTRAP_FLAVOR,
	rig = "glitchtraprig",
    kanim = "glitchtrap", -- needed for EMP highlight, a consequence of potentially being a mainframe object
    profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_glitchtrap_small.png",
    profile_icon_100 = "gui/icons/item_icons/icon-item_glitchtrap.png",
    abilities = {"carryable", "glitchtrap"},
    requirements = {},
    traits = {targeting_ignoreLOS = true, range = 6},
    ITEM_LIST = true
}

local empbeam_base = util.extend(commondefs.item_template) {
    name = STRINGS.HEATSIGNATURE.EMPBEAM,
    desc = STRINGS.HEATSIGNATURE.EMPBEAM_TIP,
    flavor = STRINGS.HEATSIGNATURE.EMPBEAM_FLAVOR,
    icon = "itemrigs/FlashPack.png",
    profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_empbeam_small.png",
    profile_icon_100 = "gui/icons/item_icons/icon-item_empbeam.png",
    abilities = {"carryable", "empbeam"},
    requirements = {},
    traits = {targeting_ignoreLOS = true, range = 6},
    ITEM_LIST = true
}

local function upgradeParamsUsesLeft(self, unit)
    return {traits = {usesLeft = unit:getTraits().usesLeft}}
end

local function upgradeParamsAmmo(self, unit)
    return {traits = {ammo = unit:getTraits().ammo}}
end

local tool_templates = {
    item_facebreaker = util.extend(commondefs.item_template) {
        name = STRINGS.HEATSIGNATURE.FACEBREAKER,
        desc = STRINGS.HEATSIGNATURE.FACEBREAKER_TIP,
        flavor = STRINGS.HEATSIGNATURE.FACEBREAKER_FLAVOR,
        icon = "itemrigs/FloorProp_InvisiCloakTimed.png",
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_facebreaker_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_facebreaker.png",
        abilities = {"carryable", "facebreaker"},
        requirements = {},
        traits = {
            damage = 1,
            restrictedUse = {{agentID = "saderfiasco", name = STRINGS.HEATSIGNATURE.SADERFIASCO.NAME}}
        },
        value = 0,
        -- notSoldAfter = 0,
        onTooltip = function(tooltip, unit, userUnit)
            commondefs.item_template.onTooltip(tooltip, unit, userUnit)
            local footer = tooltip._children[#tooltip._children]
            tooltip._children[#tooltip._children] = nil
            tooltip:addAbility(
                STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_NONE,
                STRINGS.ITEMS.TOOLTIPS.ARMOR_PIERCING_NONE_DESC,
                "gui/icons/arrow_small.png"
            )
            table.insert(tooltip._children, footer)
        end
    },
    item_glitchtrap_selfcharging = util.extend(glitchtrap_base) {
        name = STRINGS.HEATSIGNATURE.GLITCHTRAP_SELFCHARGING,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_glitchtrap_selfcharge_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_glitchtrap_selfcharge.png",
        traits = {cooldown = 0, cooldownMax = 2},
        value = 500,
        ITEM_LIST = false -- notSoldAfter = 0,
    },
    item_glitchtrap_extremerange = util.extend(glitchtrap_base) {
        name = STRINGS.HEATSIGNATURE.GLITCHTRAP_EXTREMERANGE,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_glitchtrap_range_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_glitchtrap_range.png",
        traits = {disposable = true, range = 18},
        value = 300
    },
    item_glitchtrap_highcap = util.extend(glitchtrap_base) {
        name = STRINGS.HEATSIGNATURE.GLITCHTRAP_HIGHCAP,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_glitchtrap_capacity_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_glitchtrap_capacity.png",
        traits = {usesLeft = 5},
        createUpgradeParams = upgradeParamsUsesLeft,
        value = 500
    },
    item_glitchtrap_rechargable = util.extend(glitchtrap_base) {
        name = STRINGS.HEATSIGNATURE.GLITCHTRAP_RECHARGABLE,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_glitchtrap_recharge_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_glitchtrap_recharge.png",
        abilities = util.tconcat(glitchtrap_base.abilities, {"recharge"}),
        traits = {usesCharges = true, charges = 3, chargesMax = 3},
        value = 700
    },
    item_glitchtrap_remote = util.extend(glitchtrap_base) {
        type = "simmainframeunit",
        name = STRINGS.HEATSIGNATURE.GLITCHTRAP_REMOTE,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_glitchtrap_remote_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_glitchtrap_remote.png",
        uses_mainframe = {
            toggle = {
                name = STRINGS.HEATSIGNATURE.ABILITIES.GLITCHTRAP_ACTIVATE,
                tooltip = STRINGS.HEATSIGNATURE.ABILITIES.GLITCHTRAP_ACTIVATE_DESC,
                fn = "toggle"
            }
        },
        traits = {usesLeft = 3, range = 12, mainframe_item = true, mainframe_status = "off", agent_filter = true},
        value = 500
    },
    item_flashgrenade_sticky_launcher = util.extend(commondefs.grenade_template) {
        name = STRINGS.HEATSIGNATURE.GRENADE_STICKY_FLASH_LAUNCHER,
        desc = STRINGS.HEATSIGNATURE.GRENADE_STICKY_FLASH_TIP,
        flavor = STRINGS.HEATSIGNATURE.GRENADE_STICKY_FLASH_FLAVOR,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flash_grenade_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_flash_grenade.png",
        kanim = "kanim_flashgrenade",
        sounds = {explode = "SpySociety/Grenades/flashbang_explo", bounce = "SpySociety/Grenades/bounce"},
        abilities = util.tconcat(commondefs.grenade_template.abilities, {"reload", "primeGrenades"}),
        requirements = {},
        traits = {
            baseDamage = 2,
            canSleep = true,
            range = 2,
            disposable = false,
            ammo = 2,
            maxAmmo = 2,
            throwUnit = "item_flashgrenade_sticky"
        },
        value = 600,
        createUpgradeParams = upgradeParamsAmmo
    },
    item_flashgrenade_sticky = util.extend(commondefs.grenade_template) {
        type = "simgrenade_stickystun",
        name = STRINGS.HEATSIGNATURE.GRENADE_STICKY_FLASH,
        desc = STRINGS.HEATSIGNATURE.GRENADE_STICKY_FLASH_TIP,
        flavor = STRINGS.HEATSIGNATURE.GRENADE_STICKY_FLASH_FLAVOR,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flash_grenade_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_flash_grenade.png",
        kanim = "kanim_flashgrenade",
        sounds = {explode = "SpySociety/Grenades/flashbang_explo", bounce = "SpySociety/Grenades/bounce"},
        requirements = {},
        traits = {baseDamage = 2, canSleep = true, range = 2, sightable = true},
        value = 0
    },
    item_swapper = util.extend(commondefs.item_template) {
        name = STRINGS.HEATSIGNATURE.SWAPPER,
        desc = STRINGS.HEATSIGNATURE.SWAPPER_TIP,
        flavor = STRINGS.HEATSIGNATURE.SWAPPER_FLAVOR,
        icon = "itemrigs/FloorProp_InvisiCloakTimed.png",
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_swapper_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_swapper.png",
        abilities = {"carryable", "swapteleport"},
        requirements = {},
        traits = {range = 6, usesCharges = true, charges = 1, chargesMax = 1},
        value = 400
    },
    item_empbeam_selfcharging = util.extend(empbeam_base) {
        name = STRINGS.HEATSIGNATURE.EMPBEAM_SELFCHARGING,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_empbeam_selfcharge_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_empbeam_selfcharge.png",
        traits = {cooldown = 0, cooldownMax = 5},
        value = 800,
        soldAfter = 24
    },
    item_empbeam_extremerange = util.extend(empbeam_base) {
        name = STRINGS.HEATSIGNATURE.EMPBEAM_EXTREMERANGE,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_empbeam_range_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_empbeam_range.png",
        traits = {disposable = true, range = 18},
        value = 300
    },
    item_empbeam_highcap = util.extend(empbeam_base) {
        name = STRINGS.HEATSIGNATURE.EMPBEAM_HIGHCAP,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_empbeam_capacity_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_empbeam_capacity.png",
        traits = {usesLeft = 3},
        createUpgradeParams = upgradeParamsUsesLeft,
        value = 300,
        notSoldAfter = 48
    },
    item_empbeam_rechargable = util.extend(empbeam_base) {
        name = STRINGS.HEATSIGNATURE.EMPBEAM_RECHARGABLE,
        profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_empbeam_recharge_small.png",
        profile_icon_100 = "gui/icons/item_icons/icon-item_empbeam_recharge.png",
        abilities = util.tconcat(empbeam_base.abilities, {"recharge"}),
        traits = {usesCharges = true, charges = 2, chargesMax = 2},
        value = 0, -- 600
        ITEM_LIST = false
    }
}

return tool_templates
