local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )

local FLAVOUR_COLOUR = "<c:61AAAA>"
local ITEM_HEADER_COLOUR = "<ttheader>"

local function onScrollWorldTooltip( tooltip, unit )
	tooltip:addLine( ITEM_HEADER_COLOUR..unit:getName().."</>" )
	tooltip:addDesc(  string.format("<c:61AAAA>"..unit:getUnitData().flavor).."</c>" )
	tooltip:addDesc( unit:getUnitData().desc )
end

local function onScrollTooltip( tooltip, unit, userUnit )
	local simquery = include( "sim/simquery" )
	local name = util.toupper( unit:getName() )
	local spell
	local spell_strings
	if unit:getTraits().storedSpell and not unit:getTraits().hiddenScroll then
		local spellId = unit:getTraits().storedSpell
		local abilitydefs = include( "sim/abilitydefs" )
		spell = abilitydefs.lookupAbility( spellId )
		spell_strings = (spell and spell.strings) or spell
	end
	
	tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>" )
	if spell_strings and spell_strings.school then
		tooltip:addLine( util.sformat( STRINGS.WIZARD.TOOLTIPS.SCROLL_LEVEL, spell.spellLevel, spell_strings.school ) )
	elseif spell then
		tooltip:addLine( util.sformat( STRINGS.WIZARD.TOOLTIPS.SCROLL_LEVEL_NO_SCHOOL, spell.spellLevel ) )
	end
	
	tooltip:addDesc( FLAVOUR_COLOUR..unit:getUnitData().flavor.."</>" )
	tooltip:addDesc( unit:getUnitData().desc )

	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1],unit:getTraits().dlcFooter[2] )
	end	
end

local function onBookTooltip( tooltip, unit, userUnit )
	local simquery = include( "sim/simquery" )
	local name = util.toupper( unit:getName() )
	
	tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>" )
	if unit:getUnitData().flavor then
		tooltip:addDesc( FLAVOUR_COLOUR..unit:getUnitData().flavor.."</>" )
	end
	
	if unit:getUnitData().desc then
		tooltip:addDesc( unit:getUnitData().desc )
	end
	if unit:getUnitData().desc2 then
		tooltip:addDesc( unit:getUnitData().desc2 )
	end
	if unit:getUnitData().desc3 then
		tooltip:addDesc( unit:getUnitData().desc3 )
	end
	
	--if unit:getTraits().usesCharges then
	--	tooltip:addAbility( util.sformat(STRINGS.WIZARD.TOOLTIPS.BOOK_CHARGES, unit:getTraits().charges,unit:getTraits().chargesMax), STRINGS.WIZARD.TOOLTIPS.BOOK_CHARGES_DESC, "gui/icons/arrow_small.png" )
	--end
	
	if unit:getTraits().maa_modded_item and STRINGS.MORE_ARCHIVE_AGENTS then
		tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ITEM_MODDED, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ITEM_MODDED_DESC, unit:getTraits().maa_modded_item ), "gui/icons/skills_icons/skills_icon_small/icon-item_technician_small.png" )
	end
	
	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1], unit:getTraits().dlcFooter[2] )
	end
end

local function onSorceryTooltip( tooltip, unit, userUnit )
	local simquery = include( "sim/simquery" )
	local name = util.toupper( unit:getName() )
	
	tooltip:addLine( ITEM_HEADER_COLOUR..name.."</>" )
	if unit:getUnitData().flavor then
		tooltip:addDesc( FLAVOUR_COLOUR..unit:getUnitData().flavor.."</>" )
	end
	
	if unit:getUnitData().desc then
		tooltip:addDesc( unit:getUnitData().desc )
	end
	if unit:getUnitData().desc2 then
		tooltip:addDesc( unit:getUnitData().desc2 )
	end
	
	if unit:getTraits().usesCharges then
		tooltip:addAbility( util.sformat(STRINGS.WIZARD.TOOLTIPS.MANA_CHARGES, unit:getTraits().charges,unit:getTraits().chargesMax), STRINGS.WIZARD.TOOLTIPS.MANA_CHARGES_DESC, "gui/icons/arrow_small.png" )
	end	
	
	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1], unit:getTraits().dlcFooter[2] )
	end
end

local tool_templates = {
	augment_wizard = util.extend( commondefs.augment_template ) {
		name = STRINGS.WIZARD.ITEMS.ARCANE_FOCUS,
		desc = STRINGS.WIZARD.ITEMS.ARCANE_FOCUS_TIP,
		flavor = STRINGS.WIZARD.ITEMS.ARCANE_FOCUS_FLAVOR,
		traits =  util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			spellsPrepared = {},
		},
		icon = "itemrigs/FloorProp_DataDisc.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_augment.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_augment.png",
		createUpgradeParams = function( self, unit )
			return { traits = {
				spellsPrepared = unit:getTraits().spellsPrepared
			}}
		end,
		abilities = util.tconcat( commondefs.augment_template.abilities, { "wizard_arcane_focus" }),		
	},
	
	augment_wizard_archived = util.extend( commondefs.augment_template ) {
		name = STRINGS.WIZARD.ITEMS.SORCERY_BLOOD,
		desc = STRINGS.WIZARD.ITEMS.SORCERY_BLOOD_TIP,
		flavor = STRINGS.WIZARD.ITEMS.SORCERY_BLOOD_FLAVOR,
		traits =  util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			spellsKnown = {},
			pastRewards = {},
			nextRewardCount = 1,
			nextReward = nil
		},
		icon = "itemrigs/FloorProp_DataDisc.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_augment.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_augment.png",
		createUpgradeParams = function( self, unit )
			return { traits = {
				spellsKnown = unit:getTraits().spellsKnown,
				pastRewards = unit:getTraits().pastRewards,
				nextRewardCount = unit:getTraits().nextRewardCount,
				nextReward = unit:getTraits().nextReward,
			}}
		end,
		
		onTooltip = onSorceryTooltip,
		abilities = util.tconcat( commondefs.augment_template.abilities, { "wizard_archived" }),		
	},
	
	item_wiz_spellbook = util.extend( commondefs.item_template ) {
		name = STRINGS.WIZARD.ITEMS.BOOK,
		desc = STRINGS.WIZARD.ITEMS.BOOK_TIP,
		desc2 = STRINGS.WIZARD.ITEMS.BOOK_TIP2,
		desc3 = STRINGS.WIZARD.ITEMS.BOOK_TIP3,
		flavor = STRINGS.WIZARD.ITEMS.BOOK_FLAVOR,
		traits = {
			spellsKnown = {},
			usesCharges = true,
			charges = 2,
			chargesMax = 2,
			restrictedUse = {{ agentID = "wizard_agent", name = STRINGS.WIZARD.WIZARD.NAME }},
		},
		icon = "itemrigs/FloorProp_DataDisc.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_book.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_book.png",	
		value = 0,
		createUpgradeParams = function( self, unit )
			return { traits = {
				spellsKnown = unit:getTraits().spellsKnown
			}}
		end,
		
		onTooltip = onBookTooltip,
		--abilities = { "carryable", "wizard_spellbook", "wizard_readbook" },		
		abilities = { "carryable", "wizard_spellbook", "wizard_book_swap", "wizard_book_recharge" },		
	},
	
	item_wiz_spellscroll = util.extend( commondefs.item_template ) {
		type = "wizard_spellscroll",
		name = STRINGS.WIZARD.ITEMS.SCROLL,
		desc = STRINGS.WIZARD.ITEMS.SCROLL_TIP,
		flavor = STRINGS.WIZARD.ITEMS.SCROLL_FLAVOR,
		icon = "itemrigs/FloorProp_DataDisc.png",		
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_page.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_page.png",
		traits = {
			hiddenScroll = true,
			storedSpell = nil,
			copyingCostPerLevel = 50,
		},
		
		value = 50,
		createUpgradeParams = function( self, unit )
			return { traits = {
				storedSpell = unit:getTraits().storedSpell,
				hiddenScroll = unit:getTraits().hiddenScroll,
			}}
		end,
		
		--onWorldTooltip = onScrollWorldTooltip,
		onTooltip = onScrollTooltip,
		abilities = { "carryable" },			
	},
	
	item_wiz_spellscroll_free = util.extend( commondefs.item_template ) {
		type = "wizard_free_spellscroll",
		name = STRINGS.WIZARD.ITEMS.SCROLL_FREE,
		desc = STRINGS.WIZARD.ITEMS.SCROLL_FREE_TIP,
		flavor = STRINGS.WIZARD.ITEMS.SCROLL_FREE_FLAVOR,
		icon = "itemrigs/FloorProp_DataDisc.png",		
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_page_free.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_page_free.png",
		traits = {
			freeWizardSpells = 2,
		},
		
		value = 200,
		--onWorldTooltip = onScrollWorldTooltip,
		--onTooltip = onScrollTooltip,
		abilities = { "carryable" },			
	},
	
	item_wiz_crystal = util.extend( commondefs.item_template ) {
		name = STRINGS.WIZARD.ITEMS.CRYSTAL,
		desc = STRINGS.WIZARD.ITEMS.CRYSTAL_TIP,
		flavor = STRINGS.WIZARD.ITEMS.CRYSTAL_FLAVOR,
		icon = "itemrigs/FloorProp_DataDisc.png",		
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_crystal.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_crystal.png",
		traits = {
			restoreMana = 2,
			disposable = true,
		},
		
		value = 400,
		abilities = { "carryable", "wizard_crystal" },
	},
	
	item_wiz_crystal_debug = util.extend( commondefs.item_template ) {
		name = "CHEAT CRYSTAL",
		desc = "For cheating",
		flavor = "Cheater",
		icon = "itemrigs/FloorProp_DataDisc.png",		
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_wizard_crystal.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_wizard_crystal.png",
		traits = {
			restoreMana = 99,
			canExceedMaxMana = true,
		},
		
		value = 99999,
		abilities = { "carryable", "wizard_crystal" },
	},
}	
	
	
return tool_templates