local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )

local items =
{
	--Draco 1
	maa_vampire_scanner = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_SCANNER.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_SCANNER.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_SCANNER.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_dracul_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_augment_dracul.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			ammo = 3, maxAmmo = 3, maa_ammo_text = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_SCANNER.AMMO_TEXT,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_vampire_scan" }),
	},
	maa_vampire_gun = util.extend( commondefs.weapon_reloadable_template )
	{
		name =  STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_GUN.NAME,
		desc =  STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_GUN.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_GUN.FLAVOR,
		icon = "itemrigs/FloorProp_Pistol.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_pistol_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_pistol.png",
		equipped_icon = "gui/items/equipped_pistol.png",
		traits =
		{
			weaponType = "pistol", baseDamage = 1, ammo = 1, maxAmmo = 1, cooldown = 0, cooldownMax = 4,
			maa_is_vampire_gun = true,
			restrictedUse = {{agentID=1003, name=STRINGS.DLC1.AGENTS.DRACO.NAME}}
		},
		niaa_traits = { soundRange = 6 }, --Traits we only add if New Items and Augements is installed. The trait is handled in FuncLib, but I feel like the mechanic belongs to NIaA exclusively
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_dart", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_light_revolver",
		agent_anim = "anims_1h",
		value = 1600,
		--make stat changes to this gun permanent
		createUpgradeParams = function( self, unit )
			local params = { traits = { autoEquip = (unit:getTraits().equipped == true) } } --TODO: get this from commondefs
			params.traits.ammo = unit:getTraits().ammo
			params.traits.maxAmmo = unit:getTraits().maxAmmo
			params.traits.cooldownMax = unit:getTraits().cooldownMax
			if not unit:getTraits().cooldownMax then
				params.traits.maa_mod_remove_cooldown = true
			end
			if unit:getTraits().maa_autoreload then
				params.traits.maa_autoreload = unit:getTraits().maa_autoreload
			end
			if unit:getTraits().maa_ap_refresh then
				params.traits.maa_ap_refresh = unit:getTraits().maa_ap_refresh
			end
			if unit:getTraits().maa_mp_boost then
				params.traits.maa_mp_boost = unit:getTraits().maa_mp_boost
			end
			if unit:getTraits().armorPiercing then
				params.traits.armorPiercing = unit:getTraits().armorPiercing
			end

			return params
		end,
	},

	--Olivia 1
	maa_neural_interrogator = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.NEURAL_INTERROGATOR.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.NEURAL_INTERROGATOR.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.NEURAL_INTERROGATOR.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_generic_head_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_generic_head.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_neural_interrogation" }),
	},
	maa_tracking_chip = util.extend(commondefs.item_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TRACKING_CHIP.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TRACKING_CHIP.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TRACKING_CHIP.FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chip_daemonscan_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_chip_daemonscan.png",
		traits =
		{
			cooldown = 0, cooldownMax = 5,
		},
		abilities = { "carryable", "maa_install_tracking_chip" },
		requirements = { anarchy = 2 },
		value = 300,
	},
	
	--Derek 1
	maa_virus_injector = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VIRUS_INJECTOR.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VIRUS_INJECTOR.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VIRUS_INJECTOR.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_generic_head_small2.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_generic_head2.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			ammo = 1, maxAmmo = 1, maa_ammo_text = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VIRUS_INJECTOR.AMMO_TEXT,
			maa_virus_injector = true, maa_hacking_turns = 2,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_craft_program" }),
	},
	maa_derek_smoke = util.extend( commondefs.grenade_template )
	{
        	type = "smoke_grenade",
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DEREK_SMOKE.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DEREK_SMOKE.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DEREK_SMOKE.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_smoke_grenade_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_smoke_grenade.png",
		kanim = "kanim_stickycam",
		sounds = {explode="SpySociety/Grenades/smokegrenade_explo", bounce="SpySociety/Grenades/bounce_smokegrenade"},
		abilities = { "carryable", "recharge", "throw" },
		traits =
		{
			throwUnit="maa_smokegrenade", range=2, disposable = false,
			usesCharges = true, charges = 1, chargesMax = 1,
		},
		value = 400,
		locator=true,
	},
	maa_smokegrenade = util.extend( commondefs.grenade_template )
	{
        type = "smoke_grenade",
		name = STRINGS.ITEMS.GRENADE_SMOKE,
		desc = STRINGS.ITEMS.GRENADE_SMOKE_TOOLTIP,
		flavor = STRINGS.ITEMS.GRENADE_SMOKE_FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_smoke_grenade_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_smoke_grenade.png",
		kanim = "kanim_stickycam",
		sounds = {explode="SpySociety/Grenades/smokegrenade_explo", bounce="SpySociety/Grenades/bounce_smokegrenade"},
		traits = { on_spawn = "smoke_cloud" , range=3, noghost = true, explodes = 0 },
		locator=true,
	},

	--Rush 1
	maa_echo_injector = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.ECHO_INJECTOR.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.ECHO_INJECTOR.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.ECHO_INJECTOR.FLAVOR,
		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_augment_nika_small.png",
		profile_icon_100 = "gui/icons/skills_icons/icon-item_augment_nika.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			echo_mp_bonus = 2,
			echo_max_bonus = 3,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_echo_injector" }),
	},
	maa_turbo_stim = util.extend(commondefs.item_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TURBO_STIM.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TURBO_STIM.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TURBO_STIM.FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_stim_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_stim.png",
		traits =
		{
			cooldown = 0, cooldownMax = 4,
			sprint_bonus_mp = 4,
			sprint_noise_incr = 2,
			mpRestored = 0, --so we can use vanilla use_stim
		},
		abilities = { "carryable", "recharge", "maa_use_turbo_stim" },
		requirements = { stealth = 2 },
		value = 250,
	},
	
	--Olivia 2
	maa_daemon_tazer_charger = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DAEMON_TAZER_CHARGER.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DAEMON_TAZER_CHARGER.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DAEMON_TAZER_CHARGER.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_olivia_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_augment_olivia.png",	
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			ammo = 0, maxAmmo = 3, maa_ammo_text = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DAEMON_TAZER_CHARGER.AMMO_TEXT,
			trueDaemon = true,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_daemon_tazer_charger", "maa_dtc_cooldownMax", "maa_dtc_damage", "maa_dtc_armor_pierce" }),
	},
	maa_tazer_olivia = util.extend(commondefs.melee_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TAZER_OLIVIA.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TAZER_OLIVIA.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.TAZER_OLIVIA.FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_tazer_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_tazer.png",		
		traits = { damage = 2,  cooldown = 0, cooldownMax = 3, melee = true, tazer = true, maa_olivia_tazer = true },
		niaa_traits = { soundRange = 2 }, --Traits we only add if New Items and Augements is installed. The trait is handled in FuncLib, but I feel like the mechanic belongs to NIaA exclusively
		value = 500,
	},
	
	--Derek 2
	maa_positive_particle_uplink = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PARTICLE_UPLINK.NAME_1,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PARTICLE_UPLINK.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PARTICLE_UPLINK.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_derek_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_augment_derek.png",	
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			modTrait = {{"mpMax", -1}},
			maa_is_particle_uplink = true,
			maa_push_mp_cost = 1.5, 
			maa_swap_pwr_cost = 4, maa_swap_cooldown = 0, maa_swap_cooldownMax = 6,
			maa_pocket_pwr_cost = 1,
			maa_lifelink_penalty = 1,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_particle_uplink", "maa_pu_push", "maa_pu_swap", "maa_pu_linked_pockets", "maa_pu_lifelink"}),
		createUpgradeParams = function( self, unit )
			local params = commondefs.augment_template.createUpgradeParams( self, unit )
			params.traits.maa_partner_installed = unit:getTraits().maa_partner_installed

			return params
		end,
	},
	--maa_negative_particle_uplink see below
	
	--Draco 2
	maa_vampire_thirst = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_THIRST.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_THIRST.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_THIRST.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_augment_dracul_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_augment_dracul.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_thirst_state = 2,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_vampire_thirst" }),
		createUpgradeParams = function( self, unit )
			local params = commondefs.augment_template.createUpgradeParams( self, unit )
			params.traits.maa_thirst_state = unit:getTraits().maa_thirst_state

			return params
		end,
	},
	
	--Rush 2
	maa_feedback_processor = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.FEEDBACK_PROCESSOR.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.FEEDBACK_PROCESSOR.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.FEEDBACK_PROCESSOR.FLAVOR,
		profile_icon = "gui/icons/skills_icons/icon-item_augment_central.png",
		profile_icon_100 = "gui/icons/skills_icons/icon-item_augment_central.png",	
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_alarm_mp_bonus = 1,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_feedback_processor" }),
	},
	maa_corporate_tazer = util.extend(commondefs.melee_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.CORPORATE_TAZER.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.CORPORATE_TAZER.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.CORPORATE_TAZER.FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_tazer_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_tazer.png",		
		traits = { damage = 2, maa_alarm_on_use = 1, melee = true, tazer = true, },
		niaa_traits = { soundRange = 2 }, --Traits we only add if New Items and Augements is installed. The trait is handled in FuncLib, but I feel like the mechanic belongs to NIaA exclusively
		value = 500,
	},
	maa_lock_explosive = util.extend(commondefs.item_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.LOCK_EXPLOSIVE.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.LOCK_EXPLOSIVE.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.LOCK_EXPLOSIVE.FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flash_grenade_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_flash_grenade.png",	
		traits =
		{
			applyFn = "isClosedDoor",
			maa_target_keybits = { {simdefs.DOOR_KEYS.VAULT, 3}, {simdefs.DOOR_KEYS.SECURITY, 2}, {simdefs.DOOR_KEYS.OFFICE, 1} },
			maa_alarm = 1,
			soundRange = 8,
			ammo = 3, maxAmmo = 3,
		},
		abilities = { "carryable", "reload", "maa_use_lock_explosive_door", "maa_use_lock_explosive_safe" },
		value = 550,
		createUpgradeParams = function( self, unit )
			local params = { traits = {} }
			params.traits.ammo = unit:getTraits().ammo

			return params
		end,
	},
}

local maa_negative_particle_uplink = util.extend( items.maa_positive_particle_uplink )
{
	name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PARTICLE_UPLINK.NAME_2,
	traits = util.extend( items.maa_positive_particle_uplink.traits )
	{
		installed = false,
	},
}
items["maa_negative_particle_uplink"] = maa_negative_particle_uplink

return items
