local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simdefs = include( "sim/simdefs" )

local items =
{
	--N-Umi 1
	maa_drone_controller = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_CONTROLLER.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_CONTROLLER.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_CONTROLLER.FLAVOR,
		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_augment_internationale_small.png",
		profile_icon_100 = "gui/icons/skills_icons/icon-item_augment_internationale.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_is_umi_drone_controller = true,
			range = 5, maa_extend_drone_duration_cost = 2, cooldown = 0, cooldownMax = 3,
			maa_drone_reactivation_cost = 2,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_drone_controller", "maa_extend_drone_control", "maa_drone_reactivate" }),
	},
	maa_portable_drone = util.extend( commondefs.grenade_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PORTABLE_DRONE.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PORTABLE_DRONE.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.PORTABLE_DRONE.FLAVOR,
		profile_icon = "gui/icons/icon-item_spider_drone_small.png",
		profile_icon_100 = "gui/icons/icon-item_spider_drone.png",
		icon = "itemrigs/FloorProp_InvisiCloakTimed.png",
		kanim = "kanim_stickycam",
		abilities = {"carryable" , "maa_deploy_drone"},
		sounds = { activate = "SpySociety/Grenades/stickycam_deploy", bounce = "SpySociety/Grenades/bounce" },
		traits = {
			disposable = false, targeting_ignoreLOS = true, agent_filter = true,
			maa_deployUnit = "maa_spider", mp_penalty = 1,
			restrictedUse = { {agentID = "mod_04_n_umi", name = STRINGS.AMRE.AGENTS.NU.NAME} },
			maa_is_umi_drone_item = true,
			cooldown = 0, maa_cooldownMaxDeploy = 2, cooldownMax = 2, --???
		},
		value = 800,
		locator=true,
	},
	maa_sankaku_tazer = util.extend(commondefs.melee_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.SANKAKU_TAZER.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.SANKAKU_TAZER.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.SANKAKU_TAZER.FLAVOR,
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_tazer_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_tazer.png",
		traits = {
			damage = 2, armorPiercing = 2, melee = true, tazer = true,
			cooldown = 0, cooldownMax = 5,
			maa_ammo_per_armor = 1,
			ammo = 2, maxAmmo = 2, maa_ammo_text = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.SANKAKU_TAZER.AMMO_TEXT, maa_no_ammo_usage = true,
		},
		niaa_traits = { soundRange = 2 }, --Traits we only add if New Items and Augements is installed. The trait is handled in FuncLib, but I feel like the mechanic belongs to NIaA exclusively
		value = 500,
	},

	maa_drone_battery = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_BATTERY.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_BATTERY.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_BATTERY.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chargeweapon_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_chargeweapon.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			ammo = 0, maxAmmo = 0, maa_ammo_text = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_BATTERY.AMMO_TEXT,
			maa_is_drone_battery = true,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_drone_battery", "maa_deactivate_drone" }),
	},
	maa_drone_turret = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TURRET.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TURRET.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TURRET.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_pistol_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_pistol.png",
		equipped_icon = "gui/items/equipped_pistol.png",
		onTooltip = commondefs.weapon_template.onTooltip,
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			weaponType="pistol", baseDamage = 1, ammo = 2, maxAmmo = 2,
			equipped = true, slot = "gun",
			maa_is_umi_drone_turret = true,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "shootSingle", "equippable", "maa_drone_turret_reload" }),
		sounds = {shoot="SpySociety/Weapons/LowBore/shoot_handgun", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_light_revolver",
		agent_anim = "anims_1h",
		createUpgradeParams = function( self, unit )
			local params = commondefs.augment_template.createUpgradeParams( self, unit )
			params.traits.ammo = unit:getTraits().ammo

			return params
		end,
	},
	maa_drone_toolkit_scout = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TOOLKIT_SCOUT.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TOOLKIT_SCOUT.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TOOLKIT_SCOUT.FLAVOR,
		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_technician_small.png",
		profile_icon_100 = "gui/icons/skills_icons/icon-skill_technician.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_is_umi_drone_toolkit_scout = true,
			maa_drone_scan_cost = 1, maa_drone_scan_cooldown = 0, maa_drone_scan_cooldownMax = 4, mma_drone_scan_range = 7,
			maa_drone_door_hack_cost = 2, maa_drone_door_hack_cooldown = 0, maa_drone_door_hack_cooldownMax = 2,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, {  }),
		sounds = {shoot="SpySociety/Weapons/LowBore/shoot_handgun", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_light_revolver",
		agent_anim = "anims_1h",
	},
	maa_drone_cloaking_frame = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_CLOAKING_FRAME.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_CLOAKING_FRAME.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_CLOAKING_FRAME.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_invisicloak_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_invisi_cloak.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_is_umi_drone_cloak = true,
			duration = 1, cooldown = 0, cooldownMax = 20, cloakInVision = true, pwrCost = 5, --pwrCost is handled in FuncLib
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, { "maa_useInvisiCloak_augment" }),
	},
	maa_drone_toolkit_support = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TOOLKIT_SUPPORT.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TOOLKIT_SUPPORT.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_TOOLKIT_SUPPORT.FLAVOR,
		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_technician_small.png",
		profile_icon_100 = "gui/icons/skills_icons/icon-skill_technician.png",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_is_umi_drone_toolkit_support = true,
			maa_drone_icebreak_cooldown = 0, maa_drone_icebreak_cooldownMax = 3, icebreak = 2,
			maa_drone_mark_target_cost = 2,
			maa_drone_pwr_gen_amount = 1,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, {  }),
		sounds = {shoot="SpySociety/Weapons/LowBore/shoot_handgun", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_light_revolver",
		agent_anim = "anims_1h",
	},
	maa_drone_hacking_tools = util.extend( commondefs.augment_template )
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_HACKING_TOOLS.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_HACKING_TOOLS.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.DRONE_HACKING_TOOLS.FLAVOR,
		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_safecracker_small.png",
		profile_icon_100 = "gui/icons/skills_icons/icon-skill_safecracker",
		traits = util.extend( commondefs.DEFAULT_AUGMENT_TRAITS ){
			installed = true,
			maa_is_umi_drone_hacking_tool = true,
			maa_drone_id_daemon_cooldown = 0, maa_drone_id_daemon_cooldownMax = 2,
			maa_drone_reboot_device_cooldown = 0, maa_drone_reboot_device_cooldownMax = 4,
			maa_drone_swap_daemon_cooldown = 0, maa_drone_swap_daemon_cooldownMax = 3, maa_drone_swap_daemon_cost = 1,
		},
		abilities = util.tconcat( commondefs.augment_template.abilities, {  }),
	},

	maa_drone_skillpath_combat = util.extend(commondefs.item_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.COMBAT.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.COMBAT.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.COMBAT.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_SMG_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_smg.png",
	},
	maa_drone_skillpath_scout = util.extend(commondefs.item_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.SCOUT.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.SCOUT.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.SCOUT.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_scanner_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_scanner.png",
	},
	maa_drone_skillpath_support = util.extend(commondefs.item_template)
	{
		name = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.SUPPORT.NAME,
		desc = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.SUPPORT.DESC,
		flavor = STRINGS.MORE_ARCHIVE_AGENTS.SKILLPATHS.SUPPORT.FLAVOR,
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_heart_monitor.png",
	},
}

for id, template in pairs(items) do
	template.id = id
end

return items
