local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local simunit = include( "sim/simunit" )
local simfactory = include( "sim/simfactory" )
local speechdefs = include( "sim/speechdefs" )
local simquery = include( "sim/simquery" )

local defibrillator = { ClassType = "maa_defibrillator" }

function defibrillator:activate( parent_augment )
	local sim = self:getSim()
	self:getTraits().parent_unit = parent_augment
	self:getTraits().pwrCost = (parent_augment:getTraits().pwrCost or 0)
	self:getTraits().agent_unit = parent_augment:getUnitOwner()
	self:getTraits().mainframe_item = true
	self:getTraits().mainframe_status = "on"

	self:getTraits().deployed = true

	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
end

function defibrillator:toggle( sim )
	local target = self:getTraits().agent_unit

	local x1,y1 = target:getLocation()
	if target:isKO() and target:isDead() then
		assert( target:getWounds() >= target:getTraits().woundsMax ) -- Cause they're dead, should have more wounds than max
		target:getTraits().dead = nil
		target:addWounds( target:getTraits().woundsMax - target:getWounds() - 1 )

		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.REVIVED,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )

		if not simquery.isUnitCellFull( sim, target ) and not simquery.isUnitDragged( sim, target ) then
			target:setKO( sim, nil )
		else
			if target:getTraits().koTimer then
				target:getTraits().koTimer = 1
			end
			sim:triggerEvent( "TRG_MAA_SOFT_REVIVE", { unit = target } )
		end
		target:getTraits().mp = math.max( 0, target:getMPMax() - (target:getTraits().overloadCount or 0) )

		sim:emitSpeech( target, speechdefs.EVENT_REVIVED )
	end

	sim:getPC():addCPUs(- self:getTraits().pwrCost)
	self:getTraits().mainframe_item = nil
	self:getTraits().mainframe_status = nil

	self:getTraits().parent_unit:getTraits().maa_count_used = (self:getTraits().parent_unit:getTraits().maa_count_used or 0) + 1
	self:getTraits().parent_unit:getTraits().pwrCost = (self:getTraits().parent_unit:getTraits().pwrCost or 0) + (self:getTraits().parent_unit:getTraits().maa_cost_increase or 1)
end

local function createDefibrillator( unitData, sim )
	return simunit.createUnit( unitData, sim, defibrillator )
end

simfactory.register( createDefibrillator )
