local mainframe = include( "sim/mainframe" )
local mathutil = include( "modules/mathutil" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local util = include( "modules/util" )

local oldInit = mainframe.init

function mainframe.init( sim )
	oldInit( sim )

	if sim:getCurrentPlayer() and not EDITOR then
		sim:triggerEvent( "TRG_PRE_START_TURN", sim:getCurrentPlayer() )
	end
end

function mainframe.canBreakIce( sim, targetUnit, equippedProgram )
	local player = sim:getCurrentPlayer()
    	if player == nil then
        	return false
    	end

    	if sim:getMainframeLockout() then
        	return false, STRINGS.UI.REASON.INCOGNITA_LOCKED_DOWN
    	end

    	if (targetUnit:getTraits().mainframe_ice or 0) <= 0 then
        	return false
    	end

    	if targetUnit:getTraits().isDrone and targetUnit:isKO() then
        	return false
    	end

	if targetUnit:getTraits().mainframe_status == "off" then
        	if not (targetUnit:getTraits().mainframe_camera and targetUnit:getTraits().mainframe_booting) then
            		return false
        	end
	end

	if targetUnit:getTraits().iceImmune then
		return false, STRINGS.FUNCLIB.UI.JAMMED
	end

	if equippedProgram == nil then
		equippedProgram = player:getEquippedProgram()
	end

	if equippedProgram == nil then 
		return false, STRINGS.UI.REASON.NO_PROGRAM
	end 

	if sim:getCurrentPlayer() == nil or equippedProgram:getCpuCost() > player:getCpus() then
		if equippedProgram.flail_target and (targetUnit ~= equippedProgram.flail_target) then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		elseif not equippedProgram.flail_target then
			return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
		end
	end

	local ok, result = equippedProgram:canUseAbility( sim, player, targetUnit )
	if not ok then
		return ok, result	
	end

	if equippedProgram.sniffer then 
		if not sim:getHideDaemons() and not targetUnit:getTraits().mainframe_program then 
			return false, STRINGS.UI.REASON.NO_DAEMON
		elseif targetUnit:getTraits().daemon_sniffed then 
			return false, STRINGS.UI.REASON.DAEMON_REVEALED
		end
	end

	if equippedProgram.daemon_killer then 
		if not sim:getHideDaemons() and not targetUnit:getTraits().mainframe_program then 
			return false, STRINGS.UI.REASON.NO_DAEMON
		end
	end

	if equippedProgram.wrench then 
		if targetUnit:getTraits().mainframe_ice ~= equippedProgram.break_firewalls then 
			return false, util.sformat( STRINGS.UI.REASON.WRONG_WRENCH, equippedProgram.break_firewalls )
		end 
	end

	local x0, y0 = targetUnit:getLocation()
	for unitID, unit in pairs( sim:getAllUnits() ) do
		local range = unit:getTraits().mainframe_suppress_range
		if range and unit:getPlayerOwner() ~= sim:getPC() and not unit:isKO() and unit:getLocation() and unit ~= targetUnit then
			local distSqr = mathutil.distSqr2d( x0, y0, unit:getLocation() )
			if distSqr <= range * range then
				return false, "nulldrone", unit:getName()
			end
		end
	end

	return true
end

function mainframe.breakIce( sim, unit, cost )
	local currentPlayer = sim:getPC()

	sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = unit, ice = unit:getTraits().mainframe_ice, delta = -cost } )
	unit:getTraits().mainframe_ice = unit:getTraits().mainframe_ice - cost

	sim:triggerEvent( simdefs.TRG_ICE_BROKEN, { unit = unit, delta = -(cost+math.min(0,unit:getTraits().mainframe_ice)) } )

	if unit:getTraits().mainframe_ice > 0 then
		sim:getCurrentPlayer():glimpseUnit( sim, unit:getID() )
		sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units={unit:getID()}} )
	else
		if sim._resultTable.devices[unit:getID()] then
			sim._resultTable.devices[unit:getID()].hacked = true
		end
		local x1,y1 = unit:getLocation()
		local guardTable = {}
		for i, unit in pairs( sim:getNPC():getUnits() ) do
			if unit:getTraits().investigateHackedDevices and unit:getBrain() and unit:getBrain():getSenses() and not unit:isKO() and not unit:isDead() and unit:getBrain():getSituation().ClassType ~= simdefs.SITUATION_FLEE then
				table.insert( guardTable, unit )
			end
		end
		local closestGuard = simquery.findClosestUnit( guardTable, x1, y1 )
		if closestGuard then
			closestGuard:getBrain():spawnInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_HACKEDDEVICE, unit)
		end
		for _, unit in pairs( currentPlayer:getUnits() ) do
			runBreakIceAugments( sim, unit )
		end

		if unit:getTraits().shieldArmor and unit:getTraits().shields > 0 then
			unit:getTraits().shields = 0
			sim:dispatchEvent( simdefs.EV_UNIT_HIT_SHIELD,  {unit = unit, shield = unit:getTraits().shields} )
			sim:getCurrentPlayer():glimpseUnit( sim, unit:getID() )
		end

		sim:getStats():incStat( "security_hacked" )

		if unit:getTraits().tracker_alert == true then
			unit:getTraits().tracker_alert = false
		end

		if unit:getTraits().revealUnits then 
			local unitlist = {}
			sim:forEachUnit(
				function ( u )
					if u:getTraits()[ unit:getTraits().revealUnits ] ~= nil then
						table.insert(unitlist,u:getID())		
						currentPlayer:glimpseUnit( sim, u:getID() )				
					end
				end )
			sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units=unitlist,reveal = true} )
		end
		
		if unit:getTraits().showOutline then 
			sim._showOutline = true
			sim:dispatchEvent( simdefs.EV_WALL_REFRESH )

    			local x0,y0 = unit:getLocation()
    			local color = {r=1,g=1,b=41/255,a=1}
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.FACILITY_REVEALED,x=x0,y=y0,color=color,alwaysShow=true} )
		end
		
		if unit:getTraits().revealDaemons then 
			sim:forEachUnit(
			function ( u )
				if u:getTraits().mainframe_program ~= nil then
					u:getTraits().daemon_sniffed = true 
				end
			end )
		end
		mainframe.invokeDaemon( sim, unit )
		
		if unit:getTraits().selfFirewallBuff then
			unit:getTraits().mainframe_iceMax = unit:getTraits().mainframe_iceMax + unit:getTraits().selfFirewallBuff
		end
		
		if not unit:getTraits().noTakeover then
			if unit:getTraits().failsafeAi then

				if (currentPlayer:getTraits().controlTicks or 0) > 0 then
					unit:getTraits().controlTimerMax = 0
					unit:takeControl( currentPlayer )
				else
					unit:getTraits().mainframe_ice = unit:getTraits().mainframe_iceMax
					unit:setKO( sim, unit:getTraits().failsafeAiDuration or 2 )
				end
			else

				unit:takeControl( currentPlayer )

			end
		end

		if unit:getTraits().parasite then
			mainframe.removeParasite(currentPlayer,unit)
		end
		
		sim:triggerEvent( "TRG_UNIT_HACKED", { unit = unit } )
	end
end

function runBreakIceAugments( sim, sourceUnit )
	local x2, y2 = sourceUnit:getLocation()

	local t, netDownlink = sourceUnit:countAugments( "augment_net_downlink" )
	if t > 0 and not sourceUnit:isKO() then
        	local gainMp = math.min( (netDownlink[1]:getTraits().bonusDownlinkMp or 2), (netDownlink[1]:getTraits().maxDownlinkMp or 6) - (sourceUnit:getTraits().netDownlinkMp or 0))
        	if gainMp > 0 then
        		sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = sourceUnit  } )
		    	sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.ITEMS.AUGMENTS.NET_DOWNLINK, x = x2, y = y2, color={r=255/255,g=255/255,b=51/255,a=1 }} )
		    	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = sourceUnit } )
            		sourceUnit:getTraits().netDownlinkMp = (sourceUnit:getTraits().netDownlinkMp or 0) + gainMp
		    	sourceUnit:getTraits().mp = sourceUnit:getTraits().mp + gainMp
        	end
	end 

	local MAX_PRISM_BONUS = 2
	local prismBonus = sourceUnit:getTraits().prismBonus or 0
	if sourceUnit:countAugments( "augment_prism_2" ) > 0 and not sourceUnit:isKO() and prismBonus < MAX_PRISM_BONUS then
		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.ITEMS.AUGMENTS.PRISM_2, x = x2, y = y2 ,color={r=255/255,g=255/255,b=51/255,a=1 }} )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = sourceUnit } )
		sourceUnit:getPlayerOwner():addCPUs( 1, sim,x2,y2)
        	sourceUnit:getTraits().prismBonus = ( sourceUnit:getTraits().prismBonus or 0 )+ 1
	end
end