local mainframe = include("sim/mainframe")
local unitdefs = include("sim/unitdefs")
local simquery = include("sim/simquery")
local simdefs = include("sim/simdefs")

--Our custom breakIce function

function mainframe.performFailsafeAi( sim, unit )
	local currentPlayer = sim:getPC()
	if (currentPlayer:getTraits().controlTicks or 0) > 0 then
		unit:getTraits().controlTimerMax = 0
		unit:takeControl( currentPlayer )
	else
		sim:dispatchEvent( "EV_CE_FAILSAFE_AI_HACKED", {unit = unit} )
		unit:getTraits().mainframe_ice = unit:getTraits().mainframe_iceMax
		sim:dispatchEvent( simdefs.EV_UNIT_CAPTURE, { unit = unit } )
		unit:setKO( sim, unit:getTraits().failsafeAiDuration or 2 )
		currentPlayer:glimpseUnit( sim, unit:getID() )
	end
end

function mainframe.investigateHackedDevice( sim, unit )
	local x1,y1 = unit:getLocation()
	local guardTable = {}
	for i, unit in pairs( sim:getNPC():getUnits() ) do
		if unit:getTraits().investigateHackedDevices and unit:getBrain() and unit:getBrain():getSenses() and not unit:isKO() and not unit:isDead() and unit:getBrain():getSituation().ClassType ~= simdefs.SITUATION_FLEE then
			table.insert( guardTable, unit )
		end
	end
	
	local closestGuard = simquery.findClosestUnit( guardTable, x1, y1 )
	if closestGuard then
		local interest = closestGuard:getBrain():getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_HACKEDDEVICE, unit)
		if interest then
			local rememberedInterest = closestGuard:getBrain():getSenses():getRememberedInterest()
			
			if rememberedInterest then
				if rememberedInterest.reason == simdefs.REASON_HACKEDDEVICE then
					interest.remember = true
					rememberedInterest.remember = nil
				end
			else
				interest.remember = true
			end
			
			sim:processReactions( closestGuard )
		end
	end
end

---------------PLEASE FIXME!!---------------
--Need to check _G because of Function Library...
mainframe.runBreakIceAugments = mainframe.runBreakIceAugments or rawget(_G, "runBreakIceAugments") or function( sim, sourceUnit )
	local x2, y2 = sourceUnit:getLocation()
	local AUGMENT_TXT_COLOR = {r=255/255,g=255/255,b=51/255,a=1 }
	
	if sourceUnit:countAugments( "augment_net_downlink" ) > 0 and not sourceUnit:isKO() then
		local MAX_DOWNLINK_AP, BONUS_DOWNLINK_AP = 6, 2
		local gainMp = math.min( BONUS_DOWNLINK_AP, MAX_DOWNLINK_AP - (sourceUnit:getTraits().netDownlinkMp or 0))
		if gainMp > 0 then
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = sourceUnit  } )
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.ITEMS.AUGMENTS.NET_DOWNLINK, x = x2, y = y2, color=AUGMENT_TXT_COLOR} )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = sourceUnit } )
			sourceUnit:getTraits().netDownlinkMp = (sourceUnit:getTraits().netDownlinkMp or 0) + gainMp
			sourceUnit:getTraits().mp = sourceUnit:getTraits().mp + gainMp
		end
	end 

	local MAX_PRISM_BONUS = 2
	local prismBonus = sourceUnit:getTraits().prismBonus or 0
	if sourceUnit:countAugments( "augment_prism_2" ) > 0 and not sourceUnit:isKO() and prismBonus < MAX_PRISM_BONUS then
		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.ITEMS.AUGMENTS.PRISM_2, x = x2, y = y2 ,color=AUGMENT_TXT_COLOR} )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = sourceUnit } )
		sourceUnit:getPlayerOwner():addCPUs( 1, sim,x2,y2)
		sourceUnit:getTraits().prismBonus = ( sourceUnit:getTraits().prismBonus or 0 )+ 1
	end
end
	
function mainframe.breakIce( sim, unit, cost )
	local currentPlayer = sim:getPC()

	sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = unit, ice = unit:getTraits().mainframe_ice, delta = -cost } )
	unit:getTraits().mainframe_ice = unit:getTraits().mainframe_ice - cost

	sim:triggerEvent( simdefs.TRG_ICE_BROKEN, { unit = unit, delta = -(cost+math.min(0,unit:getTraits().mainframe_ice)) } )

	if unit:getTraits().mainframe_ice > 0 then
		sim:getCurrentPlayer():glimpseUnit( sim, unit:getID() )
		sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units={unit:getID()}} )
	else
		if sim._resultTable.devices[unit:getID()] then
			sim._resultTable.devices[unit:getID()].hacked = true
		end
		
		mainframe.investigateHackedDevice( sim, unit )
		
		for _, unit in pairs( currentPlayer:getUnits() ) do
			-- Check augments and perform their behaviours -- which better not modify the currentPlayer units list!
			mainframe.runBreakIceAugments( sim, unit )
		end

		if unit:getTraits().shieldArmor and unit:getTraits().shields > 0 then
			unit:getTraits().shields = 0
			sim:dispatchEvent( simdefs.EV_UNIT_HIT_SHIELD,  {unit = unit, shield = unit:getTraits().shields} )
			sim:getCurrentPlayer():glimpseUnit( sim, unit:getID() )
		end

		sim:getStats():incStat( "security_hacked" )

		if unit:getTraits().tracker_alert == true then
			unit:getTraits().tracker_alert = false
		end

		if unit:getTraits().revealUnits then 
			local unitlist = {}
			sim:forEachUnit(
				function ( u )
					if u:getTraits()[ unit:getTraits().revealUnits ] ~= nil then
						table.insert(unitlist,u:getID())		
						currentPlayer:glimpseUnit( sim, u:getID() )				
					end
				end )

			sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units=unitlist,reveal = true} )
		end
		
		if unit:getTraits().showOutline then 
			sim._showOutline = true
			sim:dispatchEvent( simdefs.EV_WALL_REFRESH )

			local x0,y0 = unit:getLocation()
			local color = {r=1,g=1,b=41/255,a=1}
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.FACILITY_REVEALED,x=x0,y=y0,color=color,alwaysShow=true} )
		end
		
		if unit:getTraits().revealDaemons then 
			sim:forEachUnit(
			function ( u )
				if u:getTraits().mainframe_program ~= nil then
					u:getTraits().daemon_sniffed = true 
				end
			end )
		end
		mainframe.invokeDaemon( sim, unit )
		
		if unit:getTraits().selfFirewallBuff then
			unit:getTraits().mainframe_iceMax = unit:getTraits().mainframe_iceMax + unit:getTraits().selfFirewallBuff
		end
		
		-- for items that can be deactivated when controlled.
		if not unit:getTraits().noTakeover then
			if unit:getTraits().failsafeAi then
				mainframe.performFailsafeAi( sim, unit )
			else
				unit:takeControl( currentPlayer )
			end
		end

		-- for items infested with parasite
		if unit:getTraits().parasite then
			mainframe.removeParasite(currentPlayer,unit)
		end
		
		sim:triggerEvent( "TRG_UNIT_HACKED", { unit = unit } )
	end
end

local oldinit = mainframe.init

mainframe.init = function( sim )
	ce_createTriggers( sim )
	if unitdefs.lookupTemplate("ce_dummy_pather") then
		for i, unit in pairs(sim:getAllUnits()) do
			if unit:getTraits().mpMax then
				unit:getTraits().mp = unit:getMPMax()
			end
		end
	end
	if unitdefs.lookupTemplate("ce_dummy_patrol") then
		sim.ce_shouldGeneratePatrols = true
		local idle = sim:getNPC():getIdleSituation()
		local guards = sim:getNPC():getUnits()

		for i,guard in ipairs(guards) do
		   if guard:getBrain() and guard:getBrain():getSituation().ClassType == simdefs.SITUATION_IDLE and not guard:getTraits().patrolPath then
				if guard:getTraits().reconPatrol then 
					local oldMP = guard:getTraits().mpMax
					guard:getTraits().mpMax = 25
					sim:getNPC():getIdleSituation():generatePatrolPath( guard, guard:getLocation() )
					guard:getTraits().mpMax = oldMP
				else 
					if sim:nextRand( 1, 3 ) > 1 or guard:getTraits().always_patrol then
						-- 66% of guards have a patrol, otherwise they're stationary
						sim:getNPC():getIdleSituation():generatePatrolPath( guard, guard:getLocation() )
					else
						sim:getNPC():getIdleSituation():generateStationaryPath( guard, guard:getLocation() )
					end
					local patrolPath = guard:getTraits().patrolPath
					if patrolPath and #patrolPath > 0 and patrolPath[1].facing then
						guard:updateFacing( patrolPath[1].facing )
					end
				end
			end
		end
	end
	
	oldinit( sim )
end