local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "client_util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local cdefs = include( "client_defs" )
local mainframe = include( "sim/mainframe" )
local modifiers = include( "sim/modifiers" )
local mission_util = include( "sim/missions/mission_util" )
local serverdefs = include("modules/serverdefs")
local mainframe_common = include("sim/abilities/mainframe_common")
local mainframe_abilities_base = include("sim/abilities/mainframe_abilities")

local function build_shroud_desc()
	local shroud_desc_alarmcap
	local shroud_desc_backfire
	local shroud_desc_pwrpenalty
	if simdefs.shroudalarmcap and simdefs.shroudalarmcap > 0 then
		shroud_desc_alarmcap = string.format( STRINGS.SHROUD.DESC_ALARMCAP, simdefs.shroudalarmrate and simdefs.shroudalarmrate or 0, simdefs.shroudalarmcap )
	else
		shroud_desc_alarmcap = STRINGS.SHROUD.DESC_NO_ALARMCAP
	end
	if simdefs.shroudbackfire and simdefs.shroudbackfire > 0 then
		shroud_desc_backfire = STRINGS.SHROUD.DESC_BACKFIRE
	else
		shroud_desc_backfire = STRINGS.SHROUD.DESC_NO_BACKFIRE
	end
	if simdefs.shroudpwrpenaltystart and simdefs.shroudpwrpenaltystart > 0 then
		shroud_desc_pwrpenalty = string.format( STRINGS.SHROUD.DESC_PWRPENALTY, simdefs.shroudpwrpenaltystart )
	else
		shroud_desc_pwrpenalty = ""
	end
	
	--[[print ("shroudbackfire: "          .. simdefs.shroudbackfire                                 )
	print ("shroudbackfirecap: "       .. (simdefs.shroudbackfirecap and "true" or "false")      )
	print ("shroudalarmrate: "         .. simdefs.shroudalarmrate                                )
	print ("shroudalarmcap: "          .. simdefs.shroudalarmcap                                 )
	print ("shroudpwrpenaltystart: "   .. simdefs.shroudpwrpenaltystart                          )
	print ("shroudnocountermeasures: " .. (simdefs.shroudnocountermeasures and "true" or "false"))
	print ("shroudallalert: "          .. simdefs.shroudallalert                                 )]]
	
	return string.format( STRINGS.SHROUD.DESC, shroud_desc_alarmcap, shroud_desc_backfire, shroud_desc_pwrpenalty )
end

return util.extend( mainframe_common.DEFAULT_ABILITY ) 
{
	name = STRINGS.SHROUD.NAME,
	desc = build_shroud_desc(),
	shortdesc = STRINGS.SHROUD.SHORT_DESC,
	huddesc = (simdefs.shroudalarmcap and simdefs.shroudalarmcap > 0) and STRINGS.SHROUD.HUD_DESC or STRINGS.SHROUD.HUD_DESC_CAP,
	icon = "gui/icons/programs_icons/icon-program_shroud.png",
	icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_Shroud.png",
	value = 600,

	passive = true,
	
	stopAutoAlarm = true,
	storedAutoAlarm = 0,
	timerAutoAlarm = 0,
	stopCountermeasures = true,

	-- PROGRAM_LIST = 8,	
	-- PROGRAM_NO_BREAKER_LIST = 8,

	executeAbility = function( self, sim )
		local player = sim:getCurrentPlayer()
		if not player or not player:isNPC() then
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.SHROUD.STARTRISE_WARN, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action",icon=self.icon } )
		end
		self.stopAutoAlarm = false
		self.huddesc = STRINGS.SHROUD.HUD_DESC_OFF
		if self.stopCountermeasures then
			sim:getTags().noCountermeasures = false
			self.stopCountermeasures = false
		end
		sim:removeTrigger( "shroud_trg_alarm", self )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
		sim:addTrigger( simdefs.TRG_END_TURN, self )
	end,

	canUseAbility = function( self, sim )
		return false
	end,

	onSpawnAbility = function( self, sim, player )
		mainframe_common.DEFAULT_ABILITY.onSpawnAbility( self, sim )
		sim:addTrigger( "shroud_trg_alarm", self )	
		sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self )
		sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
		-- doesnt look like this is loaded when the function executes
		-- if sim:getTracker() ~= 0 then
			-- self.stopAutoAlarm = false
			-- self.huddesc = STRINGS.SHROUD.HUD_DESC_OFF
		-- end

		-- Apply PWR Penalty
		if not player:getTraits().PWRmaxBouns then
			player:getTraits().PWRmaxBouns = 0
		end
		player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns - simdefs.shroudpwrpenaltystart

		if simdefs.shroudnocountermeasures and not sim:getTags().noCountermeasures then
			sim:getTags().noCountermeasures = true
			self.stopCountermeasures = true
		else
			self.stopCountermeasures = nil
		end
		
		self.desc = build_shroud_desc()
		self.huddesc = (simdefs.shroudalarmcap and simdefs.shroudalarmcap > 0) and STRINGS.SHROUD.HUD_DESC or STRINGS.SHROUD.HUD_DESC_CAP
	end,

	onDespawnAbility = function( self, sim, player )
		mainframe_common.DEFAULT_ABILITY.onDespawnAbility( self, sim )
		self.stopAutoAlarm = false

		-- Undo PWR Penalty
		player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns + simdefs.shroudpwrpenaltystart

		sim:removeTrigger( "shroud_trg_alarm", self )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
		sim:removeTrigger( simdefs.TRG_END_TURN, self )
	end,


	onTrigger = function( self, sim, evType, evData )
		mainframe_common.DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

		if self.stopAutoAlarm then 
			-- check if Shroud should deactivate now
			if evType == "shroud_trg_alarm" then
				if evData.txt ~= nil and evData.txt ~= STRINGS.UI.ALARM_INCREASE or (simdefs.shroudallalert and simdefs.shroudallalert >= 2) then
					self:executeAbility(sim)
				end
			elseif evType == simdefs.TRG_UNIT_NEWTARGET
			 or (simdefs.shroudallalert and simdefs.shroudallalert >= 2 and evType == simdefs.TRG_UNIT_ALERTED) then
				self:executeAbility(sim)
			elseif evType == simdefs.TRG_UNIT_NEWINTEREST then
				local reason = evData.interest.reason
				if reason == simdefs.REASON_FOUNDCORPSE or
				 reason == simdefs.REASON_FOUNDOBJECT or
				 reason == simdefs.REASON_WITNESS or -- guard down
				 reason == simdefs.REASON_SMOKE or
				 (simdefs.shroudallalert and simdefs.shroudallalert >= 1 and reason == simdefs.REASON_KO) then -- waking up
					self:executeAbility(sim)
				end
			end
		else
			-- handle backfire
			if evType == simdefs.TRG_END_TURN and sim:getCurrentPlayer():isNPC()
			 and simdefs.shroudbackfire and simdefs.shroudbackfire > 0 then
				if not simdefs.shroudbackfirecap then
					sim:trackerAdvance( simdefs.shroudbackfire, STRINGS.SHROUD.ALARM_BACKFIRE )
				elseif self.storedAutoAlarm > 0 then
					local backfire = math.min( self.storedAutoAlarm, simdefs.shroudbackfire )
					self.storedAutoAlarm = self.storedAutoAlarm - backfire
					sim:trackerAdvance( backfire, STRINGS.SHROUD.ALARM_BACKFIRE )
				end
			end
		end
	end,

	-- call before auto-incrementing alarm
	-- returns true if this increment should be stopped
	calcNextStop = function( self, sim )
		self.timerAutoAlarm = self.timerAutoAlarm + 1
		if sim:getTrackerStage() >= simdefs.shroudalarmcap or self.timerAutoAlarm < simdefs.shroudalarmrate then
			if sim:getTrackerStage() >= simdefs.shroudalarmcap then
				self.huddesc = STRINGS.SHROUD.HUD_DESC_CAP
			end
			self.storedAutoAlarm = self.storedAutoAlarm + 1
			return true
		else
			self.timerAutoAlarm = 0
			return false
		end
	end,
	
	build_shroud_desc = build_shroud_desc,
}
