local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local mainframe = include( "sim/mainframe" )
local mainframe_abilities_base = include("sim/abilities/mainframe_abilities")

function funcLibUpdatePrograms()
	mainframe_abilities_base.dataBlast = util.extend( mainframe_abilities_base.dataBlast )
	{
		break_firewalls = 1,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
			local currentProgram = player:getEquippedProgram()
			local cellx, celly = unpack(targetCell)
			local targetUnits = self:getTargetUnits( sim, cellx, celly )
			player:equipProgram( sim, self:getID() )
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )
			sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = targetUnits, range = self.range } )		

			local daemonUnits = {}
			for _, unit in ipairs(targetUnits) do
				if unit:getTraits().mainframe_item or not sim:isVersion("0.17.15") then
					local daemonProgram = unit:getTraits().mainframe_program
					local firewallsToBreak = self.break_firewalls
	        			if sim:getPC():getTraits().firewallBreakPenalty then
	        				firewallsToBreak = math.max(self.break_firewalls - sim:getPC():getTraits().firewallBreakPenalty, 1 )
	       				end
					if firewallsToBreak >= unit:getTraits().mainframe_ice then
                				unit:getTraits().mainframe_program = nil
					end
					mainframe.breakIce( sim, unit, firewallsToBreak )
					unit:getTraits().mainframe_program = daemonProgram

					if daemonProgram and unit:getTraits().mainframe_ice <= 0 then
						table.insert( daemonUnits, unit )
	             		   	end
				end
			end

			self:useCPUs( sim )
			player:equipProgram( sim, currentProgram and currentProgram:getID() )

			for i, unit in ipairs( daemonUnits ) do
				mainframe.invokeDaemon( sim, unit )
			end

			if sim:isVersion("0.17.6") then
				self:setCooldown( sim ) 
			end
		end,
	}

	mainframe_abilities_base.mainframePing = util.extend( mainframe_abilities_base.mainframePing )
	{
		range = simdefs.SOUND_MAINFRAME_PING.range,

		acquireTargets = function( self, targets, game, sim, unit )
			return targets.simpleAreaTarget( game, self.range, sim )
		end, 

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
			local cellx, celly = unpack(targetCell)
			self:setCooldown( sim )
			sim:dispatchEvent( simdefs.EV_SCANRING_VIS, { x = cellx, y = celly, range = self.range } )		
			sim:emitSound( { path = "SpySociety/Actions/mainframe_noisemaker", range = self.range, distanceOffset = -5, ignoreSight = true }, cellx, celly, nil, {{ x = cellx, y = celly }} )
			sim:processReactions()
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_MAINFRAME_PING.path )
            		self:useCPUs( sim )
			player:addCredits( -self.credit_cost )
		end,
	}
end