local hud = include( "hud/hud" )
local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "client_util" )
local simability = include( "sim/simability" )
local simdefs = include("sim/simdefs")
local simfactory = include( "sim/simfactory" )
local simquery = include("sim/simquery")
local cdefs = include( "client_defs" )
local mainframe = include( "sim/mainframe" )
local modifiers = include( "sim/modifiers" )
local mission_util = include( "sim/missions/mission_util" )
local serverdefs = include("modules/serverdefs")
local mainframe_common = include("sim/abilities/mainframe_common")
local mainframe_abilities_base = include("sim/abilities/mainframe_abilities")
local nonbreakIceTooltip = include( "hud/tooltip_nonbreakice" )

-------------------------------------------------------------------------------
-- These are PC mainframe abilities.  They are owned and executed by the player.
local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY

local mainframe_abilities =
{
	AF_frontier = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.AFPROGRAMS.FRONTIER.NAME,
		desc = STRINGS.AFPROGRAMS.FRONTIER.DESC,
		huddesc = STRINGS.AFPROGRAMS.FRONTIER.HUDDESC,
		shortdesc = STRINGS.AFPROGRAMS.FRONTIER.SHORTDESC,
		icon = "gui/icons/programs/ProgramFrontier2.png",
		icon_100 = "gui/icons/programs/store_icons/ProgramFrontier2_Store.png",
        
		value = 450,
		passive = true,

		PROGRAM_NO_BREAKER_LIST = 5,
		
		canUseAbility = function( self, sim )
			return false 	
		end,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			--sim:addTrigger( simdefs.TRG_LOS_REFRESH, self )
			sim:addTrigger( simdefs.TRG_UNIT_APPEARED, self )
			
			self.discoveredDevice = nil
			
			sim:forEachUnit(
			function( mainframeUnit )
				if (mainframeUnit:getTraits().mainframe_item or mainframeUnit:getTraits().mainframe_console) and not mainframeUnit:getTraits().frontierSeen then
					if sim:getPC():hasSeen( mainframeUnit ) then
						mainframeUnit:getTraits().frontierSeen = true
						if mainframeUnit:getTraits().frontierTempSightable then
							mainframeUnit:getTraits().sightable = false
						end
						self.discoveredDevice = mainframeUnit:getName()
						self:executeAbility(sim)
					else
						if not mainframeUnit:getTraits().sightable then
							mainframeUnit:getTraits().sightable = true
							mainframeUnit:getTraits().frontierTempSightable = true
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = mainframeUnit, reveal = false } )
						end
					end
				end
			end )
		end, 

		onDespawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			--sim:removeTrigger( simdefs.TRG_LOS_REFRESH, self )	
			sim:removeTrigger( simdefs.TRG_START_TURN, self )	
			sim:removeTrigger( simdefs.TRG_UNIT_APPEARED, self )
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			
			if evType == simdefs.TRG_START_TURN and sim:getTurnCount() == 0 then
				local player = sim:getCurrentPlayer()	
				
				if not player:getTraits().extraStartingPWR then
					player:getTraits().extraStartingPWR = 0
				end
				
				sim:forEachUnit(
				function( mainframeUnit )
					if (mainframeUnit:getTraits().mainframe_item or mainframeUnit:getTraits().mainframe_console) and not mainframeUnit:getTraits().frontierSeen then
						if sim:getPC():hasSeen( mainframeUnit ) then
							mainframeUnit:getTraits().frontierSeen = true
							if mainframeUnit:getTraits().frontierTempSightable then
								mainframeUnit:getTraits().sightable = false
							end
							player:getTraits().extraStartingPWR = player:getTraits().extraStartingPWR + 1
							self.discoveredDevice = mainframeUnit:getName()
							self:executeAbility(sim)
						else
							if not mainframeUnit:getTraits().sightable then
								mainframeUnit:getTraits().sightable = true
								mainframeUnit:getTraits().frontierTempSightable = true
								sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = mainframeUnit, reveal = false } )
							end
						end
					end
				end )
			end
			
			--[[
			if evType == simdefs.TRG_LOS_REFRESH then
				log:write("Frontier event")
				
				if evData.cells ~= nil then
				
					log:write(util.stringize(evData.cells, 3))
					
					for i=1, #evData.cells, 2 do -- iterate over the thing, starting at index 1, increasing by 2
					  local cell = sim:getCell(evData.cells[i], evData.cells[i+1])
					  for i, cellUnit in pairs(cell.units) do
							if (cellUnit:getTraits().mainframe_item or cellUnit:getTraits().mainframe_console) and not cellUnit:getTraits().frontierSeen then
								cellUnit:getTraits().frontierSeen = true
								self:executeAbility(sim)
							end
						end
					end
				end
			end
			]]
			
			if evType == simdefs.TRG_UNIT_APPEARED 
			and (evData.unit:getTraits().mainframe_item or evData.unit:getTraits().mainframe_console) 
			and not evData.unit:getTraits().frontierSeen
			and evData.seerID == sim:getPC():getID() then
			
                evData.unit:getTraits().frontierSeen = true
				if evData.unit:getTraits().frontierTempSightable then
					evData.unit:getTraits().sightable = false
				end
				self.discoveredDevice = evData.unit:getName()
                self:executeAbility(sim)
            end 
		end,

		executeAbility = function( self, sim, targetUnit )
			
			sim:getCurrentPlayer():addCPUs( 1, sim )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.AFPROGRAMS.FRONTIER.NAME .. " | +1 PWR\n" .. self.discoveredDevice, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
		end,
	},
}

return 
{
	mainframe_abilities = mainframe_abilities,
}
