local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "client_util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local cdefs = include( "client_defs" )
local mainframe = include( "sim/mainframe" )
local modifiers = include( "sim/modifiers" )
local mission_util = include( "sim/missions/mission_util" )
local serverdefs = include("modules/serverdefs")
local mainframe_common = include("sim/abilities/mainframe_common")
local simfactory = include("sim/simfactory")

-------------------------------------------------------------------------------
local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY

-- name is a lie, actually, player.deployUnit handles the spawn...
local function spawnGana( sim, cell, unit )
	if cell.units and #cell.units > 0 then
		for i, cellUnit in pairs(cell.units) do
			if (unit ~= cellUnit) and not cellUnit:isKO() and cellUnit:getTraits().canKO then
				cellUnit:setKO(sim, 1)
			end
		end
	end
	unit:getTraits().GanaSpawned = true
	if sim:getTags().Gana_void then
		local itemdefs = include( "sim/unitdefs/itemdefs" )
		local item = simfactory.createUnit( itemdefs.W93_item_icebreaker_starter, sim )
		sim:spawnUnit(item)
		unit:addChild( item )
		sim:getTags().Gana_void = nil
	end
	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit =unit  } )
	sim:dispatchEvent( simdefs.EV_TELEPORT, { units = { unit }, warpOut = false } )
end

local function cellIsDeployable( sim, cell )
	local KOimmune = false
	local armored = false
	local result = true
	if simquery.cellHasTag(sim, cell, "interruptNonCentral") then
		result = false
		return result, KOimmune, armored
	end
	if (cell.impass or 0) > 0 then
		result = false
		return result, KOimmune, armored
	end
	if cell.units then
		for i, unit in pairs(cell.units) do
			if unit:getTraits().dynamicImpass and not unit:isKO() then
				if not unit:getTraits().canKO then
					KOimmune = true
					result = false
					break
				end
				if (unit:getArmor() or 0) > 0 then
					armored = true
					result = false
					break
				end
			end
		end
	end
	return result, KOimmune, armored
end

-- if player never spawned in Gana, spawn him in just long enough to escape him again. Cheap way of making sure he doesn't get treated as MIA.
local function handleUnspawnedGana( sim, cell )
	local player = sim:getPC()
	if sim.deploymentMatrix and (#sim.deploymentMatrix > 0) then
		for i = #sim.deploymentMatrix, 1, -1 do
			local agentID = sim.deploymentMatrix[i]
			-- log:write(tostring(agentID))
			local unit = player:deployUnit( sim, agentID, cell )
			table.remove(sim.deploymentMatrix, i)
			for ID, deployData in pairs( player:getDeployed() ) do
				if (ID == agentID) and not deployData.escapedUnit then
					deployData.escapedUnit = unit
					simlog( "%s escaped!", unit:getName())
					sim:warpUnit( unit, nil )
					sim:despawnUnit( unit )
					sim:triggerEvent( simdefs.TRG_UNIT_ESCAPED, unit )
				end
			end
		end
	end
end

return {
	ITB_deploymentMatrix = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.BREACH_AGENTS.PROGRAMS.DEPLOYMENT_MATRIX.NAME,
		desc = STRINGS.BREACH_AGENTS.PROGRAMS.DEPLOYMENT_MATRIX.DESC,
		huddesc = STRINGS.BREACH_AGENTS.PROGRAMS.DEPLOYMENT_MATRIX.HUD_DESC,
		shortdesc = STRINGS.BREACH_AGENTS.PROGRAMS.DEPLOYMENT_MATRIX.SHORT_DESC,
		tipdesc = STRINGS.BREACH_AGENTS.PROGRAMS.DEPLOYMENT_MATRIX.TIP_DESC,


		icon = "gui/icons/program_icons/ITB_deploymentMatrix.png",
		icon_100 = "gui/icons/program_icons/ITB_deploymentMatrix_shop.png",
		-- cpu_cost = 0,
		credit_cost = 0,
		value = 0,

		acquireTargets = function( self, targets, game, sim, unit )
			-- return targets.simpleAreaTarget( game, 0, sim )
			return targets.areaCellTarget( game, 0, sim, sim:getPC() )
		end, 

		onSpawnAbility = function( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )	
			DEFAULT_ABILITY.onSpawnAbility( self, sim )			
		end, 

		onDespawnAbility = function( self, sim )
			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )	
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_UNIT_WARP and evData.unit:isPC() and (evData.to_cell == nil) then
				local remainingAgents = {}
				for i, unit in pairs(sim:getPC():getUnits()) do
					if unit:hasAbility("escape") and not unit:isKO() and not unit:isDead() and not (unit == evData.unit) then
						table.insert(remainingAgents, unit)
					end
				end
				if #remainingAgents < 1 then
					log:write("Last agent despawning. Removing Deployment Matrix.")
					handleUnspawnedGana( sim, evData.from_cell )
					sim:getPC():removeAbility(sim, "ITB_deploymentMatrix" )
				end
			end
		end,
		
		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
			local cellx, celly = unpack(targetCell)
			local cell = sim:getCell(cellx, celly)
			local script = sim:getLevelScript()

			local canDeploy, KOimmune, armored = cellIsDeployable( sim, cell)
			if not canDeploy then	
				if armored or KOimmune then
					local txt1 = STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY
					local txt2 = STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY_DESC_KO
					if armored then
						txt2 = STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY_DESC_ARMOR
					end
					script:queue( { type="showWarning", txt1=txt1, txt2 = txt2, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
				else
					local txt1 = STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY
					local txt2 = STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY_IMPASS_DESC
					script:queue( { type="showWarning", txt1=txt1, txt2 = txt2, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
				end
				return
			end
			-- this shouldn't be easy, you can't even peek inside them. but some mods add a way, so just in case...
			if simquery.cellHasTag(sim, cell, "guard_spawn") then
				if((sim.playerBeingStubborn or 0) < 7) then
					sim.playerBeingStubborn = sim.playerBeingStubborn or 0
					sim.playerBeingStubborn = sim.playerBeingStubborn + 1
					local txt1 = STRINGS.BREACH_AGENTS.UI.SHOULD_NOT_DEPLOY
					local txt2 = STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY_GUARD_EXIT[sim:nextRand(1,#STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY_GUARD_EXIT)]
					script:queue( { type="showWarning", txt1=txt1, txt2 = txt2, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
					return
				else
					script:queue( { type="showWarning", txt1=STRINGS.BREACH_AGENTS.UI.CANNOT_DEPLOY_BUT_OKAY, txt2 = "", sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  })
				end
			end
			
			-- This technicaly supports more than one "Gana" agent, just in case... but you don't get to pick the order
			if sim.deploymentMatrix and (#sim.deploymentMatrix > 0) then
				for i = #sim.deploymentMatrix, 1, -1 do
					local agentID = sim.deploymentMatrix[i]
					local unit = player:deployUnit( sim, agentID, cell )
					table.remove(sim.deploymentMatrix, i)
					sim:dispatchEvent( simdefs.EV_UNIT_HIDE, { unit = unit, hide = true } )
					sim:dispatchEvent( simdefs.EV_SCANRING_VIS, { x= cellx,y= celly, range= 1 } )		
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_lightning_strike" )					
					sim:dispatchEvent( simdefs.EV_SCRIPT_EXIT_MAINFRAME )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )
					sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = cellx, y = celly, units = nil, range = 1} )
					spawnGana( sim, cell, unit )
					sim:processReactions()
					break
				end
			end
			if sim.deploymentMatrix and (#sim.deploymentMatrix < 1) then
				player:removeAbility(sim, "ITB_deploymentMatrix" )
			end

		end,
	},
}