local hud = include( "hud/hud" )
local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "client_util" )
local simability = include( "sim/simability" )
local simdefs = include("sim/simdefs")
local simfactory = include( "sim/simfactory" )
local simquery = include("sim/simquery")
local cdefs = include( "client_defs" )
local mainframe = include( "sim/mainframe" )
local modifiers = include( "sim/modifiers" )
local mission_util = include( "sim/missions/mission_util" )
local serverdefs = include("modules/serverdefs")
local mainframe_common = include("sim/abilities/mainframe_common")
local mainframe_abilities_base = include("sim/abilities/mainframe_abilities")
local nonbreakIceTooltip = include( "hud/tooltip_nonbreakice" )
local booster_tooltip = class(nonbreakIceTooltip)

local dice_breaker = 
{
	"W93_avalanche",
	"W93_axe",
	"W93_bandwidth",
	"W93_breach",
	"W93_breach_2",
	"W93_bribe",
	"W93_club",
	"W93_dao",
	"W93_flail",
	"W93_omniwrench",
	"W93_plague",
	"W93_rally",
	"W93_spear",
	"W93_split",
	"brimstone",
	"dagger",
	"dagger_2",
	"dataBlast",
	"feast",
	"flare",
	"golem_17_10",
	"hammer",
	"lockpick_1",
	"lockpick_2",
	"mercenary",
	"parasite",
	"parasite_2",
	"rapier",
}

local dice_pwr =
{
	"W93_condense",
	"W93_crossfeed",
	"W93_fission",
	"W93_gear_shift",
	"W93_nosferatu",
	"W93_power_surge",
	"W93_power_uplink",
	"W93_pwr_bill",
	"W93_resonance",
	"W93_sap",
	"burst",
	"cycle",
	"dynamo",
	"emergency_drip",
	"faust",
	"fusion_17_10",
	"remoteprocessor",
	"root",
	"seed",
	"wildfire",
}

local dice_other =
{
	"W93_bank",
	"W93_convert",
	"W93_fade",
	"W93_fortify",
	"W93_greed",
	"W93_heartbreaker",
	"W93_hitman",
	"W93_lantern",
	"W93_lottery",
	"W93_overview",
	"W93_poke",
	"W93_reflect",
	"W93_rental",
	"W93_ring",
	"W93_scapegoat",
	"W93_search",
	"W93_sonar",
	"W93_volcano",
	"aces",
	"bless",
	"charge",
	"esp",
	"fool",
	"halt",
	"hunter",
	"leash",
	"lightning",
	"mainframePing",
	"oracle",
	"overdrive",
	"pwr_manager",
	"pwr_manager_2",
	"rogue",
	"shade",
	"sniffer",
	"taurus",
	"wings",
	"wrench_2",
	"wrench_3",
	"wrench_4",
	"wrench_5",
}

function booster_tooltip:init( mainframePanel, targetWidget, unit, reason )
	util.tooltip.init( self, mainframePanel._hud._screen )
	self._targetWidget = targetWidget
	self.mainframePanel = mainframePanel

	local localPlayer = mainframePanel._hud._game:getLocalPlayer()
	local equippedProgram = nil
	if localPlayer then 
		equippedProgram = localPlayer:getEquippedProgram()
		if equippedProgram then
			local programWidget = mainframePanel._panel.binder.programsPanel:findWidget( equippedProgram:getID() )		
			if programWidget and programWidget:isVisible() then            	
				self._ux0, self._uy0 = programWidget.binder.btn:getAbsolutePosition()
				if equippedProgram:canUseAbility( mainframePanel._hud._game.simCore, localPlayer ) then
					self.programWidget = programWidget
					self.equippedProgram = equippedProgram
				end
			end
		end
	end

	local section = self:addSection()
	section:addLine( "<ttheader>"..util.sformat( "TARGET {1}", unit:getName() ).."</>" )

	if equippedProgram then
		if equippedProgram:getDef().credit_cost then
			section:addAbility( string.format(STRINGS.UI.TOOLTIPS.CURRENTLY_EQUIPPED, equippedProgram:getDef().name), util.sformat(equippedProgram:getDef().tipdesc, equippedProgram:getCpuCost(), math.max((equippedProgram.break_firewalls or 0) - (localPlayer:getTraits().firewallBreakPenalty or 0),1), equippedProgram:getDef().credit_cost),  "gui/icons/arrow_small.png" )
		else
			section:addAbility( string.format(STRINGS.UI.TOOLTIPS.CURRENTLY_EQUIPPED, equippedProgram:getDef().name), util.sformat(equippedProgram:getDef().tipdesc, equippedProgram:getCpuCost(), math.max((equippedProgram.break_firewalls or 0) - (localPlayer:getTraits().firewallBreakPenalty or 0),1)),  "gui/icons/arrow_small.png" )
		end
	end 

	if reason then
		section:addRequirement( reason )
	end
end

-------------------------------------------------------------------------------
-- These are PC mainframe abilities.  They are owned and executed by the player.
local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY

local mainframe_abilities =
{
	W93_avalanche = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.AVALANCHE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.AVALANCHE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.AVALANCHE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.AVALANCHE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.AVALANCHE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_avalanche.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_avalanche.png",
		cpu_cost = 3,
		break_firewalls = 1, 
		equip_program = true, 
		equipped = false, 
		value = 500,
		custom_firewall_break = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 2,

		getFirewallBreak = function( self, sim, player, unit )
			return math.floor(self.break_firewalls + (player:getCpus() or 0) / 8)
		end,

		executeAbility = function( self, sim, targetUnit )
			local firewallsToBreak = math.floor(self.break_firewalls + (sim:getPC():getCpus() or 0) / 8)

			if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        		firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
			end

			self:useCPUs( sim )
			if targetUnit and (firewallsToBreak or 0) > 0 then
				mainframe.breakIce( sim, targetUnit, firewallsToBreak )
			end
	       		self:setCooldown( sim )
		end,
	},

	W93_axe = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.AXE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.AXE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.AXE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.AXE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.AXE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_axe.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_axe.png",
		cpu_cost = 6,
		break_firewalls = 4,
		equip_program = true, 
		equipped = false, 
		cooldown = 0,
		maxCooldown = 3,
		value = 600,

		PROGRAM_LIST = 10,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)
			local firewallsToBreak = self.break_firewalls

			if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        		firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
			end

			for i, unit in pairs(sim:getAllUnits()) do
				local x1, y1 = unit:getLocation()
				if unit:getName() == targetUnit:getName() and unit ~= targetUnit and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and sim:getPC():getLastKnownCell( sim, x1, y1 ) ~= nil then
					mainframe.breakIce( sim, unit, firewallsToBreak )
				end
			end
		end,
	},

	W93_bandwidth = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.BANDWIDTH.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.BANDWIDTH.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.BANDWIDTH.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.BANDWIDTH.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.BANDWIDTH.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_bandwidth.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_bandwidth.png",
		cpu_cost = 5,
		break_firewalls = 2,
		value = 600,
		range = 2,

		PROGRAM_LIST = 10,	

		acquireTargets = function( self, targets, game, sim, unit )
            		local targetHandler = targets.areaCellTarget( game, self.range + 2*math.floor(sim:getPC():getCpus()/8), sim, sim:getPC() )
            		targetHandler:setUnitPredicate(
                		function( u )
                    			return mainframe.canBreakIce( sim, u, self )
                		end )
            		targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
            		return targetHandler
		end, 

        	startTargeting = function( self, sim, unit, userUnit, targetCell )
    			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
        	end,

        	getTargetUnits = function( self, sim, cellx, celly )
            		local cells = simquery.rasterCircle( sim, cellx, celly, self.range + 2*math.floor(sim:getPC():getCpus()/8) )
            		local units = {}
            		for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
                		if cell then
                    			for _, cellUnit in ipairs(cell.units) do
                        			if sim:getCurrentPlayer():getLastKnownCell( sim, x, y ) ~= nil then
                            				if mainframe.canBreakIce( sim, cellUnit, self ) then
                                				table.insert( units, cellUnit )
                            				end
                        			end
                    			end
                		end
            		end
            		return units
        	end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
            		local currentProgram = player:getEquippedProgram()

    			player:equipProgram( sim, self:getID() )

			local cellx, celly = unpack(targetCell)
            		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )

            		local targetUnits = self:getTargetUnits( sim, cellx, celly )

           		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = targetUnits, range = self.range + 2*math.floor(sim:getPC():getCpus()/8) } )		

            		local daemonUnits = {}
            		for _, unit in ipairs(targetUnits) do
                		local daemonProgram = unit:getTraits().mainframe_program
				local firewallsToBreak = self.break_firewalls
	        		if sim:getPC():getTraits().firewallBreakPenalty then
	        			firewallsToBreak = math.max(self.break_firewalls - sim:getPC():getTraits().firewallBreakPenalty, 1 )
	       			end
				if firewallsToBreak >= unit:getTraits().mainframe_ice then
                			unit:getTraits().mainframe_program = nil
				end
                		mainframe.breakIce( sim, unit, firewallsToBreak )
                		unit:getTraits().mainframe_program = daemonProgram

                		if daemonProgram and unit:getTraits().mainframe_ice <= 0 then
                    			table.insert( daemonUnits, unit )
                		end
	        	end

            		self:useCPUs( sim )
			self:setCooldown( sim )
    			player:equipProgram( sim, currentProgram and currentProgram:getID() )

            		for i, unit in ipairs( daemonUnits ) do
                		mainframe.invokeDaemon( sim, unit )
            		end
		end,
	},

	W93_bank = util.extend( DEFAULT_ABILITY ) 
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.BANK.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.BANK.DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.BANK.SHORT_DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.BANK.HUD_DESC,
		icon = "gui/icons/programs_icons/icon-program_bank.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_bank.png",
		credit_cost = 0,
		uses_remaining = 8,
		value = 400,
		passive = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim, player  )	
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
			self.uses_remaining = 8
		end,

    		onDespawnAbility = function( self, sim, player )	
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
    		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		getCpuCost = function( self )
			return self.cpu_cost
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_ICE_BROKEN and evData["unit"]:getTraits().mainframe_ice <= 0 and self.uses_remaining > 0 then
				local x1, y1 = evData["unit"]:getLocation()
				self.uses_remaining = self.uses_remaining - 1
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.BANK.WARNING, evData["unit"]:getTraits().mainframe_iceMax * 10, self.uses_remaining), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/HUD/gameplay/gain_money", icon=self.icon } )
				if x1 then
					sim:getPC():addCredits(evData["unit"]:getTraits().mainframe_iceMax * 10, sim, x1, y1)
				else
					sim:getPC():addCredits(evData["unit"]:getTraits().mainframe_iceMax * 10, sim)
				end
			end
		end,

		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()
			local sub = util.sformat( STRINGS.PROGEXTEND.PROGRAMS.BANK.TOOLTIP, self.uses_remaining )							

			section:addLine( "<ttheader>"..self.name.."</>", sub)
			section:addAbility( self.shortdesc, self.desc, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},

	W93_breach = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.BREACH.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.BREACH.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.BREACH.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.BREACH.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.BREACH.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_breach.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_breach.png",
		cpu_cost = 3,
		break_firewalls = 1, 
		break_firewalls_bonus = 0,
		equip_program = true, 
		equipped = false, 
		value = 400,
		custom_firewall_break = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 5,

		getFirewallBreak = function( self, sim, player, unit )
			return self.break_firewalls + self.break_firewalls_bonus
		end,

		executeAbility = function( self, sim, targetUnit )
			self:useCPUs( sim )

			local firewallsToBreak = self.break_firewalls + self.break_firewalls_bonus

			if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        		firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
			end

			if targetUnit and (firewallsToBreak or 0) > 0 then
				mainframe.breakIce( sim, targetUnit, firewallsToBreak )
			end
	       		self:setCooldown( sim )
			self.break_firewalls_bonus = self.break_firewalls_bonus + 1
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_START_TURN then
				self.break_firewalls_bonus = 0
			end
		end,
	},

	W93_breach_2 = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.BREACH_2.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.BREACH_2.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.BREACH_2.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.BREACH_2.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.BREACH_2.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_breach_2.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_breach_2.png",
		cpu_cost = 4,
		break_firewalls = 1,
		break_firewalls_bonus = 0,
		equip_program = true, 
		equipped = false, 
		value = 700,
		custom_firewall_break = true,

		PROGRAM_LIST = 10,	

		getFirewallBreak = function( self, sim, player, unit )
			return self.break_firewalls + self.break_firewalls_bonus
		end,	

		executeAbility = function( self, sim, targetUnit )
			self:useCPUs( sim )

			local firewallsToBreak = self.break_firewalls + self.break_firewalls_bonus

			if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        		firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
			end

			if targetUnit and (firewallsToBreak or 0) > 0 then
				mainframe.breakIce( sim, targetUnit, firewallsToBreak )
			end
	       		self:setCooldown( sim )
			self.break_firewalls_bonus = self.break_firewalls_bonus + 2
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_START_TURN then
				self.break_firewalls_bonus = 0
			end
		end,
	},

	W93_bribe = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.BRIBE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.BRIBE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.BRIBE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.BRIBE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.BRIBE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_bribe.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_bribe.png",
		cpu_cost = 0,
		credit_cost = 40,
		break_firewalls = 3, 
		equip_program = true, 
		equipped = false, 
		value = 0,

		PROGRAM_LIST = 5,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)
			local x1, y1 = targetUnit:getLocation()
			if self.credit_cost > 0 then
				sim:getPC():addCredits(-self.credit_cost, sim, x1, y1)
			end
		end,
	},


	W93_club = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.CLUB.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.CLUB.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.CLUB.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CLUB.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CLUB.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_club.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_club.png",
		cpu_cost = 3,
		break_firewalls = 1, 
		break_firewalls_bonus = 0,
		equip_program = true, 
		equipped = false, 
		value = 600,
		custom_firewall_break = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 5,

		getFirewallBreak = function( self, sim, player, unit )
			return self.break_firewalls + math.floor(self.break_firewalls_bonus)
		end,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			self.break_firewalls_bonus = sim:getTrackerStage( math.min( simdefs.TRACKER_MAXCOUNT, self._sim:getTracker() ) ) / 2
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )	
    		end,

		executeAbility = function( self, sim, targetUnit )
			self:useCPUs( sim )

			local firewallsToBreak = self.break_firewalls + math.floor(self.break_firewalls_bonus)

			if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        		firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
			end

			if targetUnit and (firewallsToBreak or 0) > 0 then
				mainframe.breakIce( sim, targetUnit, firewallsToBreak )
			end
	       		self:setCooldown( sim )
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				self.break_firewalls_bonus = math.min(self.break_firewalls_bonus + 0.5, simdefs.TRACKER_MAXSTAGE / 2)
			end
		end,
	},

	W93_condense = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.CONDENSE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.CONDENSE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.CONDENSE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CONDENSE.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program_condense.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_condense.png",
		cpu_cost = 0,
	        cooldown = 0,        
        	maxCooldown = 3, 
		value = 750,
		MH_exception = true,
	
		PROGRAM_LIST = 5,
		PROGRAM_NO_BREAKER_LIST = 5,

		executeAbility = function( self, sim )
			if sim:getPC():getTraits().PWRmaxBouns then
				sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns - 5
				sim:getPC():addCPUs( 10 )
			end
			self:setCooldown( sim )
            		self:useCPUs( sim )
		end,

		canUseAbility = function( self, sim, abilityOwner )
			local player = sim:getCurrentPlayer()
			if player ~= abilityOwner or player == nil then
				return false
			end

			if sim:getPC():getMaxCpus() <= 5 then
				return false, STRINGS.PROGEXTEND.PROGRAMS.CONDENSE.WARNING
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner )
		end,

    		onSpawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			if not player:getTraits().PWRmaxBouns then
				player:getTraits().PWRmaxBouns = 0
			end
			player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns + 5
    		end,

		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			if sim:getPC():getMaxCpus() >= 5 then
				player:getTraits().PWRmaxBouns = player:getTraits().PWRmaxBouns - 5
			end
		end,
	},

	W93_convert = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.CONVERT.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.CONVERT.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.CONVERT.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CONVERT.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CONVERT.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_convert.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_convert.png",
		cpu_cost = 6,
	        cooldown = 0,   
        	maxCooldown = 3, 
		equip_program = true, 
		equipped = false,
		daemon_killer = true,
		value = 700,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,

		executeAbility = function( self, sim, targetUnit )
			if targetUnit:getTraits().mainframe_program == nil then
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HUD_INCIDENT_NEGATIVE.path )
				mission_util.showDialog( sim, STRINGS.UI.DIALOGS.NO_DAEMON_TITLE, STRINGS.UI.DIALOGS.NO_DAEMON_BODY )
			else
				local algorythm = serverdefs.REVERSE_DAEMONS[sim:nextRand(1, #serverdefs.REVERSE_DAEMONS)]
				targetUnit:getTraits().mainframe_program = algorythm
				self:setCooldown( sim )
                		self:useCPUs( sim )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_DAEMON_REVEAL.path )
			end
		end,
	},

	W93_crossfeed = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_crossfeed.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_crossfeed.png",
		cpu_cost = 1,
	        cooldown = 0,        
        	maxCooldown = 0, 
        	value = 500,
		equip_program = true, 
		equipped = false, 
		mode = 0,
		crossfeed_source = nil,
		crossfeed_target = nil,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )	
    		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				if self.crossfeed_source and self.crossfeed_target then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
					evData:addCPUs( 1 )
					self.crossfeed_target:increaseIce(sim,1)
					mainframe.breakIce( sim, self.crossfeed_source, 1 )

					if self.crossfeed_source and self.crossfeed_source:getTraits().mainframe_ice > 0 then
						self.maxCooldown = self.crossfeed_source:getTraits().mainframe_ice
						self.cooldown = self.crossfeed_source:getTraits().mainframe_ice
					else
						if self.crossfeed_source then
							self.crossfeed_source:getTraits().crossfeed_source = nil
						end
						self.crossfeed_source = nil
						if self.crossfeed_target then
							self.crossfeed_target:getTraits().crossfeed_target = nil
						end
						self.crossfeed_target = nil
						self.cooldown = 0
						self.maxCooldown = 0
					end
				end
			end
			if evType == simdefs.TRG_UNIT_EMP or (evType == simdefs.TRG_ICE_BROKEN and (evData.delta or 0) < 0) then
				if self.crossfeed_source and ((self.crossfeed_source:getTraits().mainframe_ice <= 0 and evType == simdefs.TRG_ICE_BROKEN) or (self.crossfeed_source:getTraits().mainframe_booting and evType == simdefs.TRG_UNIT_EMP)) then
					if self.crossfeed_target then
						self.crossfeed_target:getTraits().crossfeed_target = nil
						self.crossfeed_target = nil
					end
					self.crossfeed_source:getTraits().crossfeed_source = nil
					self.crossfeed_source = nil
					self.cooldown = 0
					self.maxCooldown = 0
				end
				if self.crossfeed_target and ((self.crossfeed_target:getTraits().mainframe_ice <= 0 and evType == simdefs.TRG_ICE_BROKEN) or (self.crossfeed_target:getTraits().mainframe_booting and evType == simdefs.TRG_UNIT_EMP)) then
					if self.crossfeed_source then
						self.crossfeed_source:getTraits().crossfeed_source = nil
						self.crossfeed_source = nil
					end
					self.crossfeed_target:getTraits().crossfeed_target = nil
					self.crossfeed_target = nil
					self.cooldown = 0
					self.maxCooldown = 0
				end
			end
			if (evType == simdefs.TRG_ICE_BROKEN and (evData.delta or 0) > 0) and self.crossfeed_source and self.crossfeed_source:getTraits().mainframe_ice > 0 then
				self.maxCooldown = self.crossfeed_source:getTraits().mainframe_ice
				self.cooldown = self.crossfeed_source:getTraits().mainframe_ice
			end
			if evType == simdefs.TRG_UNIT_KILLED and (evData.unit == self.crossfeed_source or evData.unit == self.crossfeed_target) then
				if self.crossfeed_source then
					self.crossfeed_source:getTraits().crossfeed_source = nil
					self.crossfeed_source = nil
				end
				if self.crossfeed_target then
					self.crossfeed_target:getTraits().crossfeed_target = nil
					self.crossfeed_target = nil
				end
				self.cooldown = 0
				self.maxCooldown = 0
			end
		end,

		executeAbility = function( self, sim, targetUnit )
        		if targetUnit and self.mode == 0 and not self.crossfeed_source then
				if not targetUnit:getTraits().crossfeed_target then
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HOST_PARASITE.path )
					self.mode = 1
					self.maxCooldown = targetUnit:getTraits().mainframe_ice
					targetUnit:getTraits().crossfeed_source = true 
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_parasite_installed")
					self.crossfeed_source = targetUnit
					self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.HUD_DESC2
					self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.SHORT_DESC2
					self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.TIP_DESC2
				else
					targetUnit:getTraits().crossfeed_target = nil
					self.crossfeed_target = nil
				end
			elseif targetUnit and self.mode == 0 and self.crossfeed_source then
				self.maxCooldown = 0
				targetUnit:getTraits().crossfeed_source = nil
				self.crossfeed_source = nil
        		elseif targetUnit and self.mode == 1 and not self.crossfeed_target then
				if not targetUnit:getTraits().crossfeed_source then
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HOST_PARASITE.path )
					self.mode = 0
					targetUnit:getTraits().crossfeed_target = true 
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_parasite_installed")
					self.crossfeed_target = targetUnit
					self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.HUD_DESC
					self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.SHORT_DESC
					self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.TIP_DESC
				else
					targetUnit:getTraits().crossfeed_source = nil
					self.crossfeed_source = nil
					self.maxCooldown = 0
					self.mode = 0
					self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.HUD_DESC
					self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.SHORT_DESC
					self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.TIP_DESC
				end
			elseif targetUnit and self.mode == 1 and self.crossfeed_target then
				targetUnit:getTraits().crossfeed_target = nil
				self.crossfeed_target = nil
				self.mode = 0
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.SHORT_DESC
				self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.CROSSFEED.TIP_DESC
			end

			if self.crossfeed_source and self.crossfeed_target then
				self:setCooldown( sim )
            			self:useCPUs( sim )
			end
		end,
	},

	W93_dao = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.DAO.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.DAO.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.DAO.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.DAO.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.DAO.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_dao.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_dao.png",
		cpu_cost = 2,
		break_firewalls = 2,
		cooldown = 0,
		maxCooldown = 2, 
		equip_program = true, 
		equipped = false, 
		value = 500,
		coolDownMod = -1,
		pwrMod = -1,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )	
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				if evData % 2 == 0 then
					self.coolDownMod = -1
					self.pwrMod = -1
				else
					self.coolDownMod = 1
					self.pwrMod = 1
				end
			end
		end,
	},

	W93_dice = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.DICE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.DICE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.DICE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.DICE.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_dice.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_dice.png",
		value = 500,	
		passive = true,
		MH_exception = true,
		used = false,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_GAME_OVER, self )
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_GAME_OVER, self )	
    		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and self.used == false then
				evData:equipProgram( sim, nil )
				while #evData:getAbilities() > 1 do
					for _, ability in pairs(evData:getAbilities()) do
						if ability ~= self then
							evData:removeAbility( sim, ability )
						end
					end
				end

				local dice_pwr_copy = util.tdupe( dice_pwr )
				local dice_breaker_copy = util.tdupe( dice_breaker )
				local dice_other_copy = util.tdupe( dice_other )

				local idx = sim:nextRand( 1, #dice_pwr_copy )
				evData:addMainframeAbility( sim, dice_pwr_copy[idx] )
				table.remove(dice_pwr_copy, idx)

				idx = sim:nextRand( 1, #dice_breaker_copy )
				evData:addMainframeAbility( sim, dice_breaker_copy[idx] )
				table.remove(dice_breaker_copy, idx)

				local diff = sim:getParams().difficulty

				if diff >= 4 then 
					idx = sim:nextRand( 1, #dice_breaker_copy )
					evData:addMainframeAbility( sim, dice_breaker_copy[idx] )
					table.remove(dice_breaker_copy, idx)	
				end

				if diff >= 2 then
					idx = sim:nextRand( 1, #dice_other_copy )
					evData:addMainframeAbility( sim, dice_other_copy[idx] )
					table.remove(dice_other_copy, idx)
				end

				local dice_all = util.tconcat( dice_pwr_copy, dice_breaker_copy, dice_other_copy)
				if diff >= 6 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				if diff >= 8 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				if diff >= 10 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				if diff >= 12 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				if diff >= 14 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				if diff >= 16 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				if diff >= 18 and #evData:getAbilities() < simquery.getMaxPrograms( sim ) then
					idx = sim:nextRand( 1, #dice_all )
					evData:addMainframeAbility( sim, dice_all[idx] )
					table.remove(dice_all, idx)
				end

				self.used = true
			end

			if evType == simdefs.TRG_GAME_OVER then
				for _, ability in pairs(sim:getPC():getAbilities()) do
					if ability ~= self then
						ability.no_save = true
					end
				end
			end
		end,
	},


	W93_fade = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.FADE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.FADE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.FADE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FADE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FADE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_fade.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_fade.png",
		cpu_cost = 3,
		cooldown = 0,
		maxCooldown = 9,
        	value = 850,
		equip_program = true, 
		equipped = false, 
		targetingIcon = "gui/icons/item_icons/items_icon_small/icon-item_invisicloak_small.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 10,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not targetUnit:getTraits().isAgent then 				
					return false
				end
				
				if targetUnit:getTraits().isGuard then 				
					return false
				end

				if targetUnit:getPlayerOwner() ~= sim:getPC() then
					return false
				end

				if not sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) then
					return false
				end

				if targetUnit:isGhost() then
					return false
				end

				if targetUnit:getTraits().invisible then
					return false
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			targetUnit:setInvisible(true, 1)
			targetUnit:resetAllAiming()
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit  } )
			sim:processReactions(targetUnit)

            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_fission = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.FISSION.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.FISSION.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.FISSION.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FISSION.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program_fission.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_fission.png",
		cpu_cost = 0,
	        cooldown = 0,
        	maxCooldown = 9,
        	value = 900,
		MH_exception = true,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )	
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )	
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_ICE_BROKEN and evData["unit"]:getTraits().mainframe_ice <= 0 then
				if self.cooldown >= 1 then
					self.cooldown = math.max(0, self.cooldown - 2)
				end
			end
		end,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim)
			sim:getCurrentPlayer():addCPUs( 6, sim )
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_run_fusion" )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.FISSION.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
		end,
	},

	W93_flail = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_flail.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_flail.png",
		cpu_cost = 0,
		break_firewalls = 1,
        	value = 750,
		equip_program = true, 
		equipped = false, 
		mode = 0,
		flail_target = nil,
		custom_firewall_break = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )	
    		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			local player = sim:getCurrentPlayer()
			if player == nil or player ~= abilityOwner then
				return false
			end

			if (self.cooldown or 0) > 0 then 
				return false, STRINGS.UI.REASON.EQUIPPED_ON_COOLDOWN
			end

			if sim:getMainframeLockout() then 
				return false, STRINGS.UI.REASON.INCOGNITA_LOCKED_DOWN
			end 

			for i, x in ipairs(player:getAbilities()) do 
				if x == self then 
					for j, y in ipairs(player:getLockedAbilities()) do 
						if y == i then 
							return false, STRINGS.ABILITIES.TEMPORARY_LOSS 
						end 
					end 
				end 
			end 

			return true 
		end,

		getFirewallBreak = function( self, sim, player, unit )
			return self.break_firewalls + unit:getTraits().mainframe_ice - 1
		end,

		getCpuCost = function( self )
			local cpuCost = self.cpu_cost
			local zeroCpu = false

			if cpuCost == 0 then 
				zeroCpu = true 
			end

	        	if self._sim then
	        		local pwrModd = 0

				if self.flail_target then
					cpuCost = math.ceil(self.flail_target:getTraits().mainframe_ice * 1.5)
				end

				for i, ability in ipairs( self._sim:getPC():getAbilities())do
					if ability.pwrMod then
						pwrModd = pwrModd + ability.pwrMod
					end
				end

				if zeroCpu then 
					cpuCost = math.max(0, cpuCost + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				else 
					cpuCost = math.max(1, cpuCost + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				end 
				cpuCost = cpuCost  + (pwrModd or 0)

				if (self._sim:getTags().nextProgFree or 0) > 0 then
					cpuCost = cpuCost - 4
				end
			end

	   		return math.max(0, cpuCost)
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_UNIT_EMP or evType == simdefs.TRG_ICE_BROKEN then
				if self.flail_target and (self.flail_target:getTraits().mainframe_ice <= 0 or self.flail_target:getTraits().mainframe_booting) then
					self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC
					self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC
					self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC
					self.flail_target:getTraits().flail_target = nil
					self.flail_target = nil
					self.mode = 0
				end
			end
			if evType == simdefs.TRG_START_TURN and self.flail_target then
				self.flail_target:getTraits().flail_target = nil
				self.flail_target = nil
				self.mode = 0
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC
				self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC
			end
			if evType == simdefs.TRG_UNIT_KILLED and evData.unit == self.flail_target then
				self.flail_target:getTraits().flail_target = nil
				self.flail_target = nil
				self.mode = 0
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC
				self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC
			end
		end,

		executeAbility = function( self, sim, targetUnit )
        		if targetUnit and self.mode == 0 and not self.flail_target then
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HOST_PARASITE.path )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_parasite_installed")
				self.mode = 1
				targetUnit:getTraits().flail_target = true 
				self.flail_target = targetUnit
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC2
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC2
				self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC2
			elseif targetUnit and self.mode == 0 and self.flail_target then
				self.flail_target:getTraits().flail_target = nil
				self.flail_target = nil
        		elseif targetUnit and self.mode == 1 and (not self.flail_target or (self.flail_target and targetUnit:getTraits().flail_target)) then
				targetUnit:getTraits().flail_target = nil
				self.flail_target = nil
				self.mode = 0
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC
				self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC
			elseif targetUnit and self.mode == 1 and not targetUnit:getTraits().flail_target and self.flail_target then
				if self:getCpuCost() > sim:getCurrentPlayer():getCpus() then
					return
				end
            			self:useCPUs( sim )
				self:setCooldown( sim )
				mainframe.breakIce( sim, self.flail_target, math.max(self.break_firewalls + targetUnit:getTraits().mainframe_ice - 1 - (sim:getPC():getTraits().firewallBreakPenalty or 0), 1) )
				if self.flail_target then
					self.flail_target:getTraits().flail_target = nil
					self.flail_target = nil
				end
				self.mode = 0
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.SHORT_DESC
				self.tipdesc = STRINGS.PROGEXTEND.PROGRAMS.FLAIL.TIP_DESC
			end
		end,
	},

	W93_fortify = util.extend( DEFAULT_ABILITY ) 
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.SHORT_DESC,
		cpu_cost = 1,

		icon = "gui/icons/programs_icons/icon-program_fortify.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_fortify.png",
		value = 450,
		mode = 0,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 10,

		onSpawnAbility = function( self, sim, player  )	
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

    		onDespawnAbility = function( self, sim, player )	
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
    		end,

		canUseAbility = function( self, sim, abilityOwner )
			local player = sim:getCurrentPlayer()
			if player == nil or player ~= abilityOwner then
				return false
			end

			if (self.cooldown or 0) > 0 then 
				return false, STRINGS.UI.REASON.EQUIPPED_ON_COOLDOWN
			end

			if sim:getMainframeLockout() then 
				return false, STRINGS.UI.REASON.INCOGNITA_LOCKED_DOWN
			end 

			for i, x in ipairs(player:getAbilities()) do 
				if x == self then 
					for j, y in ipairs(player:getLockedAbilities()) do 
						if y == i then 
							return false, STRINGS.ABILITIES.TEMPORARY_LOSS 
						end 
					end 
				end 
			end 

			return true 
		end,

		executeAbility = function( self, sim, targetUnit )
			if self.mode == 0 then
				self.mode = 1
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.HUD_DESC2
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.WARNING_ON, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate", icon=self.icon } )
			else
				self.mode = 0
				self.huddesc =STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.HUD_DESC
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.WARNING_OFF, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate", icon=self.icon } )
			end
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_ICE_BROKEN and evData["unit"]:getTraits().mainframe_ice <= 0 and not evData["unit"]:getTraits().isGuard and self.mode == 1 and sim:getPC():getCpus() >= self:getCpuCost() then
				evData["unit"]:getTraits().mainframe_no_recapture = true
				local x0, y0 = evData["unit"]:getLocation()
				self:useCPUs( sim )
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.PROGEXTEND.UI.FORTIFY),x=x0,y=y0,color={r=1,g=1,b=41/255,a=1}} )
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.FORTIFY.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_PWRreverse_off",icon=self.icon } )
			end
		end,
	},

	W93_gear_shift = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.DESC,
		desc_low = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.DESC_LOW,
		desc_high = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.DESC_HIGH,
		desc_top = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.DESC_TOP,
		desc_use = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.DESC_USE,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.SHORT_DESC,
		shortdesc_low = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.SHORT_DESC_LOW,
		shortdesc_high = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.SHORT_DESC_HIGH,
		shortdesc_top = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.SHORT_DESC_TOP,
		shortdesc_use = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.SHORT_DESC_USE,

		icon = "gui/icons/programs_icons/icon-program_gear_shift.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_gear_shift.png",
		cpu_cost = 0,
	        cooldown = 0,
        	maxCooldown = 5,
        	value = 600,
		mode = 0,
		wasUsed = false,
		MH_exception = true,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				local PWR = 0

				if self.mode == 0 and evData:getCpus() < 10 then
					PWR = 1
				elseif self.mode == 1 and evData:getCpus() > 8 and evData:getCpus() < 15 then
					PWR = 2
				elseif self.mode == 1 and evData:getCpus() < 8 then
					PWR = -1
				elseif self.mode == 2 and evData:getCpus() > 14 then
					PWR = 3
				elseif self.mode == 2 and evData:getCpus() < 14 then
					PWR = -2
				end

				if PWR ~= 0 then
					evData:addCPUs( PWR, sim )
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.WARNING_PWR, PWR), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.icon } )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_run_fusion" )
				end
			end
			if evType == simdefs.TRG_END_TURN and evData:isPC() and self.wasUsed == true then
				self.wasUsed = false
				DEFAULT_ABILITY.executeAbility(self, sim)
				if self.mode == 0 then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.WARNING_1, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate", icon=self.icon } )
				elseif self.mode == 1 then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.WARNING_2, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate", icon=self.icon } )
				elseif self.mode == 2 then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.WARNING_3, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate", icon=self.icon } )
				end
			end
		end,

		executeAbility = function( self, sim, targetUnit )
			if self.mode == 0 then
				self.mode = 1
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.HUD_DESC2
			elseif self.mode == 1 then
				self.mode = 2
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.HUD_DESC3
			elseif self.mode == 2 then
				self.mode = 0
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.GEAR_SHIFT.HUD_DESC
			end
			self.wasUsed = true
		end,

		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()
			local sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			section:addLine( util.sformat( STRINGS.PROGRAMS.COOLDOWN, self.maxCooldown )  )

	        	if sim then
				if self.mode == 0 then
					section:addLine( self.shortdesc_low )
				elseif self.mode == 1 then
					section:addLine( self.shortdesc_high )
				elseif self.mode == 2 then
					section:addLine( self.shortdesc_top )
				end
			end

			section:addAbility( self.shortdesc, self.desc, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				section:addAbility( self.shortdesc_low, self.desc_low, "gui/icons/arrow_small.png" )
				section:addAbility( self.shortdesc_high, self.desc_high, "gui/icons/arrow_small.png" )
				section:addAbility( self.shortdesc_top, self.desc_top, "gui/icons/arrow_small.png" )
				section:addAbility( self.shortdesc_use, self.desc_use, self.icon_100 )

			    	local canUse, reason = self:canUseAbility( sim, player )
			    	if not canUse and reason then
				    	section:addRequirement( reason )
			    	end
	        	end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},

	W93_greed = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.GREED.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.GREED.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.GREED.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.GREED.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.GREED.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_greed.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_greed.png",
		cpu_cost = 4,
		cooldown = 0,
		maxCooldown = 5,
        	value = 400,
		equip_program = true, 
		equipped = false, 
		targetingIcon = "gui/icons/cashflow_icons/safes.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 10,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not targetUnit:getTraits().safeUnit then 				
					return false
				end

				if targetUnit:getTraits().open then 				
					return false
				end

				if not targetUnit:getTraits().credits then
					return false
				end

				if targetUnit:getTraits().greedBoost then
					return false
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			local x1, y1 = targetUnit:getLocation()

			targetUnit:getTraits().credits = targetUnit:getTraits().credits * (sim:getTrackerStage() * 0.2 + 1)
			targetUnit:getTraits().greedBoost = sim:getTrackerStage() * 20
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.PROGEXTEND.UI.GREED_BOOST,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )

            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_heartbreaker = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.HEARTBREAKER.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.HEARTBREAKER.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.HEARTBREAKER.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.HEARTBREAKER.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.HEARTBREAKER.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_heartbreaker.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_heartbreaker.png",
		cpu_cost = 1,
		cooldown = 0,
		maxCooldown = 4,
        	value = 300,
		equip_program = true,
		equipped = false, 
		targetingIcon = "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		MH_exception = true,

		PROGRAM_LIST = 15,
		PROGRAM_NO_BREAKER_LIST = 10,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) then
					return false
				end

				if targetUnit:isGhost() then
					return false
				end

				if not targetUnit:getTraits().isGuard then 				
					return false
				end
				
				if targetUnit:getTraits().isDrone then 				
					return false
				end
				
				if targetUnit:isDown() then 				
					return false
				end

				if not targetUnit:getTraits().heartMonitor then
					return false
				end 
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			local x1, y1 = targetUnit:getLocation()
			if targetUnit:hasAbility( "improved_heart_monitor_passive" ) then
				targetUnit:removeAbility( sim, "improved_heart_monitor_passive" )
				targetUnit:getTraits().improved_heart_monitor = nil
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.PROGEXTEND.UI.MONITOR_DOWNGRADED,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
			else
				targetUnit:getTraits().heartMonitor = nil
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.MONITOR_DISABLED,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
			end

            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_hitman = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.HITMAN.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.HITMAN.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.HITMAN.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.HITMAN.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.HITMAN.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_hitman.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_hitman.png",
		credit_cost = 250,
		cpu_cost = 0,
		cooldown = 0,
		maxCooldown = 6,
        	value = 0,
		equip_program = true,
		equipped = false, 
		targetingIcon = "gui/icons/item_icons/items_icon_small/icon-item_gun_rifle_small.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) then
					return false
				end

				if targetUnit:isGhost() then
					return false
				end

				if not targetUnit:getTraits().isGuard then 				
					return false
				end
				
				if targetUnit:getTraits().isDrone then 				
					return false
				end
				
				if targetUnit:isDown() then 				
					return false
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
            		self:useCPUs( sim )
			self:setCooldown( sim )

			local x1, y1 = targetUnit:getLocation()
			if self.credit_cost > 0 then
				sim:getPC():addCredits(-self.credit_cost, sim, x1, y1)
			end

			targetUnit:killUnit( sim )
		end,
	},

	W93_lantern = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.LANTERN.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.LANTERN.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.LANTERN.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.LANTERN.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.LANTERN.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_lantern.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_lantern.png",
		cpu_cost = 1,
		cooldown = 0,
		maxCooldown = 3,
		value = 250,
		range = 5,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,	

		createObserveTab = function( self, unit )
			if not unit:getBrain() then return end

			local subtext = ""
			local brainClassType = unit:getBrain():getSituation().ClassType
			local x0, y0 = unit:getLocation()

			if brainClassType == simdefs.SITUATION_INVESTIGATE then
				if unit:getBrain():getInterest().x == x0 and unit:getBrain():getInterest().y == y0 then
					subtext = STRINGS.GUARD_STATUS.START_INVESTIGATING
				else
					subtext = STRINGS.GUARD_STATUS.INVESTIGATING
				end
			elseif brainClassType == simdefs.SITUATION_HUNT then
				local interest = unit:getBrain():getInterest()
				if interest and interest.reason == simdefs.REASON_KO and interest.x == x0 and interest.y == y0 then
					subtext = STRINGS.GUARD_STATUS.START_HUNTING
				else
					subtext = STRINGS.GUARD_STATUS.HUNTING
				end
			elseif brainClassType == simdefs.SITUATION_FLEE then
				subtext = STRINGS.GUARD_STATUS.FLEEING
			elseif brainClassType == simdefs.SITUATION_COMBAT then
				if unit:getTraits().vip then
					subtext = STRINGS.GUARD_STATUS.FLEEING
				else
					subtext = STRINGS.GUARD_STATUS.COMBAT
				end
			elseif brainClassType == simdefs.SITUATION_IDLE then
				if not unit:getTraits().patrolPath or (#unit:getTraits().patrolPath == 1 and unit:getTraits().patrolPath[1].x == x0 and unit:getTraits().patrolPath[1].y == y0) then
					subtext = STRINGS.GUARD_STATUS.IDLE
				else
					subtext = STRINGS.GUARD_STATUS.PATROLLING
				end
			else
				subtext = "UNKNOWN"
			end

			unit:createTab( STRINGS.GUARD_STATUS.STATUS, subtext )
		end,

		acquireTargets = function( self, targets, game, sim, unit )
            		local targetHandler = targets.simpleAreaTarget( game, self.range, sim )
            		targetHandler:setUnitPredicate(
                		function( u )
                    			return u:getTraits().isGuard
                		end )
            		targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
            		return targetHandler
		end, 

        	startTargeting = function( self, sim, unit, userUnit, targetCell )
    			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
        	end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
            		local currentProgram = player:getEquippedProgram()

    			player:equipProgram( sim, self:getID() )

			local tagged = false
			local cellx, celly = unpack(targetCell)
			local cells = simquery.rasterCircle( sim, cellx, celly, self.range )
           		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = nil, range = self.range } )

			for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
				if cell then
					for j, unit in pairs(cell.units) do
						local tx,ty = unit:getLocation()
						if tx == x and ty == y and unit:getTraits().isGuard and not unit:isDown() then
							player:glimpseCell( sim, cell )
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit, reveal = true } )
							unit:getTraits().patrolObserved = true
            						self:createObserveTab( unit )
							sim:dispatchEvent( simdefs.EV_UNIT_OBSERVED, unit )
							player:glimpseUnit( sim, unit:getID() )
							tagged = true
						end
					end
				end
			end

			if tagged then
            			self:useCPUs( sim )
            			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )
			else
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HUD_INCIDENT_NEGATIVE.path )
			end
			self:setCooldown( sim )
    			player:equipProgram( sim, currentProgram and currentProgram:getID() )
		end,
	},

	W93_lottery = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.LOTTERY.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.LOTTERY.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.LOTTERY.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.LOTTERY.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program_lottery.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_lottery.png",
		cpu_cost = 1,
	        cooldown = 0,        
        	maxCooldown = 8, 
		value = 150,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,

		executeAbility = function( self, sim )
			local ability = simability.create( serverdefs.REVERSE_DAEMONS[ sim:nextRand( 1, #serverdefs.REVERSE_DAEMONS ) ] )
			table.insert( sim:getNPC():getAbilities(), ability )
			ability:spawnAbility( sim, sim:getNPC(), nil )
			DEFAULT_ABILITY.executeAbility(self, sim)	
		end,
	},

	W93_nosferatu = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.NOSFERATU.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.NOSFERATU.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.NOSFERATU.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.NOSFERATU.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program_nosferatu.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_nosferatu.png",
        	value = 550,
		passive = true,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				local power = 0
				local units = sim:getPC():getUnits()
				for i, unit in ipairs(units) do
					local cell = sim:getCell( unit:getLocation() )
					if cell then
						for i, cellUnit in ipairs(cell.units) do
							if cellUnit:isKO() and not cellUnit:isDead() and not unit:isKO() then
								power = power + 1
							end
						end
					end
				end
				if power > 0 then
					sim:getCurrentPlayer():addCPUs( power + 1, sim )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_run_fusion" )
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.NOSFERATU.WARNING,power + 1), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
				end
			end
		end,
	},

	W93_omniwrench = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program-wrench.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_wrench.png",
		cpu_cost = 0,
		break_firewalls = 1,
		cooldown = 0,
		maxCooldown = 2,
		equip_program = true, 
		equipped = false, 
		value = 600,
		used = 3,
		custom_firewall_break = true,

		PROGRAM_LIST = 10,	

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			self.tipdesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.TIP_DESC, self.used)
		end,

		getFirewallBreak = function( self, sim, player, unit )
			return self.break_firewalls + unit:getTraits().mainframe_ice - 1
		end,

		executeAbility = function( self, sim, targetUnit )
			self.cpu_cost = targetUnit:getTraits().mainframe_ice + (sim:getPC():getTraits().program_cost_modifier or 0)

			if targetUnit then
				if self:getCpuCost() <= sim:getCurrentPlayer():getCpus() then
					self:setCooldown( sim )
					self:useCPUs( sim )
					mainframe.breakIce( sim, targetUnit, math.max(1, self.break_firewalls - 1 + targetUnit:getTraits().mainframe_ice - (sim:getPC():getTraits().firewallBreakPenalty or 0) ) )
					self.used = self.used - 1
					if self.used < 0 then
						self.used = 3
						local programList = nil
						local daemon = nil
						if sim:isVersion("0.17.5") then
							programList = sim:getIcePrograms()
							daemon = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
						else
							programList = serverdefs.PROGRAM_LIST
							daemon = programList[sim:nextRand(1, #programList)]			
						end	
						sim:getNPC():addMainframeAbility( sim, daemon )
					end
					self.tipdesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.OMNIWRENCH.TIP_DESC, self.used)
				end
			end

			self.cpu_cost = 0
		end,
	},

	W93_overview = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.DESC,
		huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.HUD_DESC, STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.WARNING),
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.TIP_DESC,
		icon = "gui/icons/programs_icons/icon-program_overview.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_overview.png",
		cpu_cost = 1,
		cooldown = 0,
		maxCooldown = 4,
        	value = 250,
		equip_program = true, 
		equipped = false, 
		targetingIcon = "gui/icons/item_icons/items_icon_small/icon-item_Sticky_Cam_small.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		targetUnit = nil,
		cameraUnit = nil,
		MH_exception = true,

		PROGRAM_LIST = 15,
		PROGRAM_NO_BREAKER_LIST = 10,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		spawnCamera = function( self, sim, targetUnit )
			local x0, y0 = targetUnit:getLocation()
			local cell = sim:getCell( x0, y0 )
			local eyeball = include( "sim/units/eyeball" )
			local eyeballUnit = eyeball.createEyeball( sim )
			eyeballUnit:setPlayerOwner( sim:getPC() )
			eyeballUnit:setFacing( targetUnit:getFacing() )
			eyeballUnit:getTraits().LOSarc = math.pi
			sim:spawnUnit( eyeballUnit )
			sim:warpUnit( eyeballUnit, cell )
			self.cameraUnit = eyeballUnit
			self.targetUnit = targetUnit
			self.huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.HUD_DESC, targetUnit:getName())
		end,

		onSpawnAbility = function( self, sim, player  )	
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end,

    		onDespawnAbility = function( self, sim, player )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )	
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

			if evType == simdefs.TRG_UNIT_WARP or evType == simdefs.TRG_UNIT_KO or evType == simdefs.TRG_UNIT_KILLED then
				if evData.unit == self.targetUnit and self.cameraUnit then
					sim:warpUnit( self.cameraUnit )
					sim:despawnUnit( self.cameraUnit )
					self.cameraUnit = nil
					if evType == simdefs.TRG_UNIT_WARP and self.targetUnit:isValid() and not self.targetUnit:getTraits().dead and self.targetUnit:getLocation() ~= nil then
						self:spawnCamera( sim, evData.unit )
						return
					end
					self.targetUnit = nil
					self.huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.HUD_DESC, STRINGS.PROGEXTEND.PROGRAMS.OVERVIEW.WARNING)
				end
			end
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if (not targetUnit:getTraits().isGuard) and (not targetUnit:getTraits().safeUnit) then 				
					return false
				end

				if targetUnit:getTraits().isGuard and (not sim:canPlayerSeeUnit( sim:getPC(), targetUnit )) then
					return false
				end

				if targetUnit:isGhost() then
					return false
				end

				if targetUnit == self.targetUnit then
					return false
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			self:spawnCamera( sim, targetUnit )

            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_plague = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.PLAGUE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.PLAGUE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.PLAGUE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.PLAGUE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.PLAGUE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_plague.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_plague.png",
		cpu_cost = 1,
		cooldown = 0,
		maxCooldown = 3,
		break_firewalls = 1,
		range = 5,
        	value = 300,
		equip_program = true, 
		equipped = false,
		plague_hosts = {},

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		end, 

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

		getCpuCost = function( self )
			local cpuCost = self.cpu_cost
			local zeroCpu = false

			if cpuCost == 0 then 
				zeroCpu = true 
			end

			if self._sim then
				local pwrModd = 0
				for i, ability in ipairs( self._sim:getPC():getAbilities()) do
					if ability.pwrMod then
						pwrModd = pwrModd + ability.pwrMod
					end
				end
				if zeroCpu then 
					cpuCost = math.max(0, cpuCost + #self.plague_hosts + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				else 
					cpuCost = math.max(1, cpuCost + #self.plague_hosts + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				end 
				cpuCost = cpuCost  + (pwrModd or 0)

				if (self._sim:getTags().nextProgFree or 0) > 0 then
					cpuCost = cpuCost - 4
				end
			end

	   		return math.max(0, cpuCost)
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )

            			local hosts = util.tdupe(self.plague_hosts)

				if #hosts > 0 then
					for i, hostID in ipairs(hosts) do
						local hostUnit = sim:getUnit( hostID )
						if hostUnit and hostUnit:getTraits().plague and hostUnit:getTraits().mainframe_ice and hostUnit:getTraits().mainframe_ice > 0 then
							mainframe.breakIce( sim, hostUnit, self.break_firewalls )
							if hostUnit:getTraits().mainframe_ice <= 0 then
								array.removeElement( self.plague_hosts, hostUnit:getID() )
								hostUnit:getTraits().plague = nil
								local cellx, celly = hostUnit:getLocation()
								local cells = simquery.rasterCircle( sim, cellx, celly, self.range )
           							sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = {}, range = self.range } )		
            							for i, x, y in util.xypairs( cells ) do
                							local cell = sim:getCell( x, y )
                							if cell and sim:getPC():getLastKnownCell( sim, x, y ) ~= nil then
                    								for _, cellUnit in ipairs(cell.units) do
                        								if cellUnit:getPlayerOwner() ~= sim:getPC() and (not cellUnit:getTraits().plague) and cellUnit:getTraits().mainframe_ice and cellUnit:getTraits().mainframe_ice > 0 then
												cellUnit:getTraits().plague = true 
												sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_parasite_installed")
												table.insert(self.plague_hosts, cellUnit:getID())
                    									end
                								end
            								end
								end
							end
						end
                			end
            			end
			end

			if evType == simdefs.TRG_ICE_BROKEN and evData["unit"]:getTraits().mainframe_ice <= 0 and evData["unit"]:getTraits().plague then
            			array.removeElement( self.plague_hosts, evData["unit"]:getID() )
				evData["unit"]:getTraits().plague = nil
			end
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit and targetUnit:getTraits().plague then
				return false, STRINGS.PROGEXTEND.PROGRAMS.PLAGUE.WARNING
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
            		self:useCPUs( sim )
			self:setCooldown( sim )
			targetUnit:getTraits().plague = true 
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_parasite_installed")
			table.insert(self.plague_hosts, targetUnit:getID())
		end,
	},

	W93_poke = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.POKE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.POKE.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.POKE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.POKE.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.POKE.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_poke.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_poke.png",
		cpu_cost = 2,
		cooldown = 0,
		maxCooldown = 6,
        	value = 350,
		equip_program = true,
		equipped = false, 
		targetingIcon = "gui/icons/programs_icons/store_icons/StorePrograms_poke.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not sim:canPlayerSeeUnit( sim:getPC(), targetUnit ) then
					return false
				end

				if targetUnit:isGhost() then
					return false
				end

				if not targetUnit:getTraits().isGuard then 				
					return false
				end
				
				if targetUnit:isDown() then 				
					return false
				end

				if targetUnit:getTraits().thoughtVis == "combat" then
					return false, STRINGS.PROGEXTEND.UI.REASON_ON_OVERWATCH
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			local x1, y1 = targetUnit:getLocation()
			targetUnit:updateFacing( simquery.getReverseDirection(targetUnit:getFacing()) )
			targetUnit:getBrain():spawnInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_SCANNED, targetUnit)

            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_power_surge = util.extend( DEFAULT_ABILITY ) 
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.POWERSURGE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.POWERSURGE.DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.POWERSURGE.SHORT_DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.POWERSURGE.HUD_DESC,
		icon = "gui/icons/programs_icons/icon-program_power_surge.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_power_surge.png",
		cpu_cost = 0,
		cooldown = 0,
		maxCooldown = 4,
		credit_cost = 0,
		value = 750,
        	passive = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim, player  )	
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

    		onDespawnAbility = function( self, sim, player )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )	
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
    		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_ICE_BROKEN and evData["unit"]:getTraits().mainframe_ice <= 0 and self.cooldown <= 0 then
				self:setCooldown( sim )
				sim:getCurrentPlayer():addCPUs( 5, sim )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.POWERSURGE.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			end
		end,
	},

	W93_power_uplink = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.PWRUPLINK.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.PWRUPLINK.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.PWRUPLINK.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.PWRUPLINK.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.PWRUPLINK.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_power_uplink.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_power_uplink.png",
		cpu_cost = 2,
	        cooldown = 0,        
        	maxCooldown = 5, 
        	value = 600,
		equip_program = true, 
		equipped = false, 
		uplink_hosts = {},
		MH_exception = true,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
    		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				local uplinkCount = 0
            			for i, hostID in ipairs(self.uplink_hosts) do
                			local hostUnit = sim:getUnit( hostID )
					if hostUnit and hostUnit:getTraits().uplink and (hostUnit:getTraits().mainframe_ice <= 0 or hostUnit:getTraits().mainframe_booting) then
						hostUnit:getTraits().uplink = nil
					end
                			if hostUnit and hostUnit:getTraits().uplink and hostUnit:getTraits().mainframe_ice > 0 then
						uplinkCount = uplinkCount+1
					end
            			end
				if uplinkCount > 0 then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.PWRUPLINK.WARNING,uplinkCount), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
					evData:addCPUs( uplinkCount )
				end
			end
			if evType == simdefs.TRG_UNIT_EMP or evType == simdefs.TRG_ICE_BROKEN then
            			for i, hostID in ipairs(self.uplink_hosts) do
                			local hostUnit = sim:getUnit( hostID )
					if hostUnit and hostUnit:getTraits().uplink and (hostUnit:getTraits().mainframe_ice <= 0 or hostUnit:getTraits().mainframe_booting) then
						hostUnit:getTraits().uplink = nil
					end
            			end
			end
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			local uplinkCount = 0
            		for i, hostID in ipairs(self.uplink_hosts) do
                		local hostUnit = sim:getUnit( hostID )
				if hostUnit and hostUnit:getTraits().uplink then
					uplinkCount = uplinkCount + 1
				end
            		end
			if uplinkCount >= 3 then
				return false, "MAXIMUM NUMBER OF UPLINKS REACHED"
			end

			if targetUnit and targetUnit:getTraits().uplink then
				return false, "ALREADY HAS POWER UPLINK"
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )
		end,

		executeAbility = function( self, sim, targetUnit )
        		if not targetUnit:getTraits().uplink then
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HOST_PARASITE.path )
            			self:useCPUs( sim )
				self:setCooldown( sim )
				targetUnit:getTraits().uplink = true 
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_parasite_installed")
				table.insert(self.uplink_hosts, targetUnit:getID())
        		end
		end,
	},

	W93_pwr_bill = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.PWR_BILL.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.PWR_BILL.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.PWR_BILL.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.PWR_BILL.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program_pwr_bill.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_pwr_bill.png",
		credit_cost = 80,
		cpu_cost = 0,
	        cooldown = 0,        
        	maxCooldown = 3, 
        	value = 0,
		MH_exception = true,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim)
			sim:getCurrentPlayer():addCPUs( 8, sim )
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_run_fusion" )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.PWR_BILL.WARNING, 8, self.credit_cost), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			if self.credit_cost > 0 then
				sim:getPC():addCredits(-self.credit_cost, sim)
			end
		end,
	},

	W93_rally = util.extend( DEFAULT_ABILITY ) 
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.RALLY.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.RALLY.DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.RALLY.SHORT_DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.RALLY.HUD_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.RALLY.TIP_DESC,
		icon = "gui/icons/programs_icons/icon-program_rally.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_rally.png",
		break_firewalls = 2,
		cpu_cost = 1,
	        cooldown = 0,        
        	maxCooldown = 0, 
		rallyPenalty = 0,
		value = 600,
		equip_program = true, 
		equipped = false,
	
		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim, player  )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )	
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
    		end,

		getCpuCost = function( self )
			local cpuCost = self.cpu_cost
			local zeroCpu = false

			if cpuCost == 0 then 
				zeroCpu = true 
			end

			if self._sim then
				local pwrModd = 0
				for i, ability in ipairs( self._sim:getPC():getAbilities()) do
					if ability.pwrMod then
						pwrModd = pwrModd + ability.pwrMod
					end
				end
				if zeroCpu then 
					cpuCost = math.max(0, cpuCost + self.rallyPenalty + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				else 
					cpuCost = math.max(1, cpuCost + self.rallyPenalty + (self._sim:getPC():getTraits().program_cost_modifier or 0))
				end 
				cpuCost = cpuCost  + (pwrModd or 0)

				if (self._sim:getTags().nextProgFree or 0) > 0 then
					cpuCost = cpuCost - 4
				end
			end

	   		return math.max(0, cpuCost)
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if ((evType == simdefs.TRG_UNIT_KO and evData.ticks) or evType == simdefs.TRG_UNIT_KILLED) and evData["unit"]:getTraits().isGuard then
				self.cooldown = 0
				self.maxCooldown = 0
				self.rallyPenalty = 0
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.RALLY.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_PWRreverse_off",icon=self.icon } )
			end
		end,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)
			self.rallyPenalty = self.rallyPenalty + 1
			self.maxCooldown = self.maxCooldown + 1
		end,
	},

	W93_reflect = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_reflect.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_reflect.png",
		cpu_cost = 8,
        	cooldown = 5,        
        	maxCooldown = 5, 
		value = 800,
		daemonReversalAdd = 10,
		modeToggle = 2,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,

		onSpawnAbility = function( self, sim, player  )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )	
			self.modeToggle = 0
			sim:addTrigger( simdefs.TRG_DAEMON_REVERSE, self )
			sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )	
			self.modeToggle = 0
			sim:removeTrigger( simdefs.TRG_DAEMON_REVERSE, self )
			sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )

    		end,

		onTrigger = function( self, sim, evType, evData )
			local player = sim:getCurrentPlayer()
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_DAEMON_REVERSE and self.modeToggle == 1 and self.cooldown <= 0 and sim:getPC():getCpus() >= self:getCpuCost() then
				self:useCPUs( sim )
				self:setCooldown( sim )
				self.modeToggle = 2
				self.daemonReversalAdd = 0
				sim:getPC():getTraits().reverseAll = nil
				self.icon = "gui/icons/programs_icons/icon-program_reflect.png"
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.HUD_DESC3
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.SHORT_DESC3
			end
			if evType == simdefs.TRG_START_TURN and self.modeToggle == 2 and self.cooldown <= 0 then
				self.modeToggle = 0
				self.daemonReversalAdd = 10
				sim:getPC():getTraits().reverseAll = nil
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.SHORT_DESC
			end
			if (evType == simdefs.TRG_DAEMON_INSTALL or simdefs.TRG_DAEMON_REVERSE or simdefs.TRG_ICE_BROKEN or evType == simdefs.TRG_START_TURN) and self.modeToggle == 1 and self.cooldown <= 0 and sim:getPC():getCpus() < self:getCpuCost() then
				self.modeToggle = 0
				self.daemonReversalAdd = 10
				sim:getPC():getTraits().reverseAll = nil
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.SHORT_DESC
			end
		end,

		executeAbility = function( self, sim )
			local player = sim:getCurrentPlayer()

			if self.modeToggle == 0 and sim:getPC():getCpus() >= self:getCpuCost() then
				self.modeToggle = 1
				self.daemonReversalAdd = 0
				sim:getPC():getTraits().reverseAll = true
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.HUD_DESC2
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.SHORT_DESC2
			elseif self.modeToggle == 1 then
				self.modeToggle = 0
				self.daemonReversalAdd = 10
				sim:getPC():getTraits().reverseAll = nil
				self.huddesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.HUD_DESC
				self.shortdesc = STRINGS.PROGEXTEND.PROGRAMS.REFLECT.SHORT_DESC
			end
		end,
	},

	W93_rental = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.RENTAL.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.RENTAL.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.RENTAL.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.RENTAL.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_rental.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_rental.png",
		credit_cost = 75,
		cpu_cost = 0,
        	cooldown = 0, 
        	maxCooldown = 3,
		value = 0,
		rental = 3,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and sim:getPC():getTraits().W93_Rental then
				sim:getPC():getTraits().controlTicks = sim:getPC():getTraits().controlTicks - self.rental
				sim:getPC():getTraits().W93_Rental = nil
			end
		end,

		executeAbility = function( self, sim )
			if not sim:getPC():getTraits().controlTicks then
				sim:getPC():getTraits().controlTicks = self.rental
			else
				sim:getPC():getTraits().controlTicks = sim:getPC():getTraits().controlTicks + self.rental
			end
			sim:getPC():getTraits().W93_Rental = true
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=util.sformat(STRINGS.PROGEXTEND.PROGRAMS.RENTAL.WARNING, self.rental, self.credit_cost), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wings_activate", icon=self.icon } )
            		self:useCPUs( sim )
			self:setCooldown( sim )
			if self.credit_cost > 0 then
				sim:getPC():addCredits(-self.credit_cost, sim)
			end
		end,
	},

	W93_resonance = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.DESC,
		desc_passive = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.DESC_PASSIVE,
		desc_active = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.DESC_ACTIVE,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.SHORT_DESC,
		shortdesc_passive = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.SHORT_DESC_PASSIVE,
		shortdesc_active = STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.SHORT_DESC_ACTIVE,
		icon = "gui/icons/programs_icons/icon-program_resonance.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_resonance.png",
		cpu_cost = 2,
	        cooldown = 0,        
        	maxCooldown = 2, 
        	value = 700,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 5,

		onSpawnAbility = function( self, sim )
			DEFAULT_ABILITY.onSpawnAbility( self, sim )
			sim:addTrigger( "TRG_PRE_END_TURN", self )	
		end, 

    		onDespawnAbility = function( self, sim, player )
    			DEFAULT_ABILITY.onDespawnAbility( self, sim )
			sim:removeTrigger( "TRG_PRE_END_TURN", self )	
    		end,

		getCpuCost = function( self )
			if self._sim then
	   			return self._sim:getPC():getCpus()
			end
			return 0
		end,

		onTrigger = function( self, sim, evType, evData )
			DEFAULT_ABILITY.onTrigger( self, sim, evType, evData )
			if evType == "TRG_PRE_END_TURN" and evData:isPC() then
				local agents = evData:getUnits()
				local pwrTable = {}
				local pwr = 0

				for i, agent in pairs (agents) do
					if type(agent:getMP()) == "number" and not agent:isDown() then
						table.insert(pwrTable, agent:getMP())
					end
				end

				for i, value1 in pairs (pwrTable) do
					for j, value2 in pairs (pwrTable) do
						if math.abs(value1 - value2) < 0.5 and value1 >= 0.5 and value2 >= 0.5 and i > j then
							pwr = pwr + 1
						end
					end
				end

				if pwr > 0 then
					pwr = math.min(pwr, 3)
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.WARNING,pwr), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
					evData:addCPUs( pwr )
				end
			end
		end,

		executeAbility = function( self, sim )
			for i, unit in pairs(sim:getPC():getUnits()) do
				if unit:getMP() then
					unit:addMP(sim:getPC():getCpus()/self.cpu_cost)
					sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
				end
			end

			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.RESONANCE.WARNING2,sim:getPC():getCpus()/self.cpu_cost), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wings_activate", icon=self.icon } )
            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,

		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()
			local sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			section:addLine( util.sformat( STRINGS.PROGRAMS.COOLDOWN, self.maxCooldown )  )
			section:addAbility( self.shortdesc, self.desc, "gui/icons/arrow_small.png" )

			if sim then
				section:addAbility( self.shortdesc_passive, self.desc_passive, "gui/icons/arrow_small.png" )
				section:addAbility( self.shortdesc_active, self.desc_active, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			    	local canUse, reason = self:canUseAbility( sim, player )
			    	if not canUse and reason then
				    	section:addRequirement( reason )
			    	end
	        	end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},

	W93_ring = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.RING.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.RING.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.RING.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.RING.SHORT_DESC,
		icon = "gui/icons/programs_icons/icon-program_ring.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_ring.png",
		cpu_cost = 2,
        	value = 400,
		range = 3,
		rangeMin = 2,
		width = 4,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,	

		acquireTargets = function( self, targets, game, sim, unit )
            		local targetHandler = targets.customAreaTarget( game, sim, {"+", self.rangeMin, self.range, self.width} )
            		targetHandler:setUnitPredicate(
                		function( u )
                    			return (u:getTraits().isGuard and not u:isDown())
                		end )
            		targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
            		return targetHandler
		end, 

        	startTargeting = function( self, sim, unit, userUnit, targetCell )
    			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
        	end,

        	getTargetUnits = function( self, sim, cellx, celly )
            		local cells = simquery.fillOrthogonalCross( sim, cellx, celly, self.rangeMin, self.range, self.width )
            		local units = 0
            		for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
                		if cell then
                    			for _, cellUnit in ipairs(cell.units) do
                        			if sim:getCurrentPlayer():getLastKnownCell( sim, x, y ) ~= nil then
                            				if cellUnit:getTraits().isGuard and not cellUnit:isDown() then
                                				units = units + 1
                            				end
                        			end
                    			end
                		end
            		end
            		return units
        	end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
            		local currentProgram = player:getEquippedProgram()
			local cellx, celly = unpack(targetCell)
            		local targetUnits = self:getTargetUnits( sim, cellx, celly )

    			player:equipProgram( sim, self:getID() )
           		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_wings_activate")
			
			local cells = simquery.fillOrthogonalCross( sim, cellx, celly, 0, 1, 2 )
            		local units = {}
            		for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
                		if cell then
                    			for _, cellUnit in ipairs(cell.units) do
                            			if cellUnit:getTraits().isAgent and cellUnit:getPlayerOwner() == sim:getPC() and not cellUnit:isDown() then
                                			cellUnit:getTraits().mp = cellUnit:getTraits().mp + targetUnits
							sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = cellUnit } )
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(STRINGS.FORMATS.PLUS_AP,targetUnits),x=x,y=y,color={r=1,g=1,b=1,a=1}} )	
                            			end
                    			end
                		end
            		end

            		self:useCPUs( sim )
			self:setCooldown( sim )
    			player:equipProgram( sim, currentProgram and currentProgram:getID() )
		end,
	},

	W93_sap = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.SAP.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.SAP.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.SAP.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.SAP.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.SAP.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_sap.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_sap.png",
		break_firewalls = 0,
		cpu_cost = 0,
		cooldown = 0,
		maxCooldown = 3,
		value = 900,
		range = 5,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 5,	

		acquireTargets = function( self, targets, game, sim, unit )
            		local targetHandler = targets.areaCellTarget( game, self.range, sim, sim:getPC() )
            		targetHandler:setUnitPredicate(
                		function( u )
                    			return mainframe.canBreakIce( sim, u, self )
                		end )
            		targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
            		return targetHandler
		end, 

        	startTargeting = function( self, sim, unit, userUnit, targetCell )
    			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
        	end,

        	getTargetUnits = function( self, sim, cellx, celly )
            		local cells = simquery.rasterCircle( sim, cellx, celly, self.range )
            		local units = {}
            		for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
                		if cell then
                    			for _, cellUnit in ipairs(cell.units) do
                        			if sim:getCurrentPlayer():getLastKnownCell( sim, x, y ) ~= nil then
                            				if mainframe.canBreakIce( sim, cellUnit, self ) then
                                				table.insert( units, cellUnit )
                            				end
                        			end
                    			end
                		end
            		end
            		return units
        	end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
            		local currentProgram = player:getEquippedProgram()

    			player:equipProgram( sim, self:getID() )

			local cellx, celly = unpack(targetCell)
            		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )

            		local targetUnits = self:getTargetUnits( sim, cellx, celly )
           		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = nil, range = self.range } )		

            		local devices = 0
            		for _, unit in ipairs(targetUnits) do
				devices = devices + 1
	        	end

			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.SAP.WARNING, devices), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU",icon=self.icon } )
			player:addCPUs( devices )
            		self:useCPUs( sim )
			self:setCooldown( sim )
    			player:equipProgram( sim, currentProgram and currentProgram:getID() )
		end,
	},

	W93_scapegoat = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.SCAPEGOAT.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.SCAPEGOAT.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.SCAPEGOAT.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.SCAPEGOAT.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.SCAPEGOAT.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_scapegoat.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_scapegoat.png",
		cpu_cost = 2,
		cooldown = 0,
		maxCooldown = 5,
        	value = 400,
		equip_program = true,
		equipped = false, 
		targetingIcon = "gui/icons/programs_icons/store_icons/StorePrograms_scapegoat.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		targetRawUnit = true,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,

		canUseAbility = function( self, sim, abilityOwner, targetUnit )
			if targetUnit then
				if not targetUnit:getTraits().isAgent then
					return false
				end

				if targetUnit:getTraits().isGuard then
					return false
				end

				if targetUnit:getPlayerOwner() ~= sim:getPC() then
					return false
				end
				
				if targetUnit:isDown() then 				
					return false
				end

				if targetUnit:getTraits().isMainframeGhost then
					return false
				end
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner, targetUnit )	
		end,

		executeAbility = function( self, sim, targetUnit )
			local x1, y1 = targetUnit:getLocation()
			targetUnit:useMP( targetUnit:getMP(), sim )
			for _, guard in pairs(sim:getNPC():getUnits()) do
				if guard:getTraits().isGuard and not guard:isDown() then
					guard:getBrain():spawnInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_SENSEDTARGET, targetUnit)
				end
			end
            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_search = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.SEARCH.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.SEARCH.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.SEARCH.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.SEARCH.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_search.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_search.png",
		cpu_cost = 3,
		credit_cost = 0,
		cooldown = 0,
		maxCooldown = 4,		
		value = 200,
		locatedNoDatabases = false,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		canUseAbility = function( self, sim, abilityOwner )
			local player = sim:getCurrentPlayer()
			if player ~= abilityOwner or player == nil then
				return false
			end

			local availableDatabases = {}
			for _, unit in pairs( sim:getAllUnits() ) do 
				if unit:getTraits().revealDaemons or unit:getTraits().revealUnits or unit:getTraits().showOutline then 
					if unit:getPlayerOwner() ~= player then 
						table.insert( availableDatabases, unit )
					end
				end
			end

			if #availableDatabases == 0 then 
				return false, STRINGS.PROGEXTEND.PROGRAMS.SEARCH.WARNING
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner )
		end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
			local availableDatabases = {}

			for _, unit in pairs( sim:getAllUnits() ) do 
				if unit:getTraits().revealDaemons or unit:getTraits().revealUnits or unit:getTraits().showOutline then 
					if unit:getPlayerOwner() ~= player then 
						table.insert( availableDatabases, unit )
					end
				end
			end

			if #availableDatabases > 0 then
				local database = availableDatabases[ sim:nextRand( 1, #availableDatabases ) ]	            
				sim:getPC():glimpseUnit(sim, database:getID() )
    				sim:dispatchEvent( simdefs.EV_CAM_PAN, { database:getLocation() } )	
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_object_camera" )
				sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units={database:getID()},reveal = true} )
            			self:useCPUs( sim )
            			self:setCooldown( sim )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*0.65)
				if database:getTraits().mainframe_program then
					database:getTraits().mainframe_program = nil
					sim:dispatchEvent( simdefs.EV_KILL_DAEMON, {unit = database})
				
					if database:getTraits().daemonHost then
						sim:getUnit(database:getTraits().daemonHost):killUnit(sim)
						database:getTraits().daemonHost = nil
					end
				end
            			mainframe.breakIce( sim, database, database:getTraits().mainframe_ice )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = database } )
			end
		end,
	},

	W93_sonar = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.SONAR.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.SONAR.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.SONAR.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.SONAR.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.SONAR.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_sonar.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_sonar.png",
		cpu_cost = 1,
		cooldown = 0,
		maxCooldown = 4,
		value = 250,
		range = 5,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,	

		acquireTargets = function( self, targets, game, sim, unit )
            		local targetHandler = targets.simpleAreaTarget( game, self.range, sim )
            		targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
            		return targetHandler
		end, 

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
            		local currentProgram = player:getEquippedProgram()

    			player:equipProgram( sim, self:getID() )

			local cellx, celly = unpack(targetCell)
            		local cells = simquery.rasterCircle( sim, cellx, celly, self.range )

			local units = sim:getAllUnits()
			for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
				if cell then
					player:glimpseCell( sim, cell )
					for j, unit in pairs(units) do
						local tx,ty = unit:getLocation()
						if tx == x and ty == y then
							player:glimpseUnit( sim, unit:getID() )
						end
					end
				end
			end 
            		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )
           		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = nil, range = self.range } )
           		sim:dispatchEvent( simdefs.EV_SCRIPT_EXIT_MAINFRAME )
            		self:useCPUs( sim )
			self:setCooldown( sim )
		end,
	},

	W93_spear = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.SPEAR.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.SPEAR.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.SPEAR.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.SPEAR.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.SPEAR.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_spear.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_spear.png",
		break_firewalls = 2,
		cpu_cost = 4,
		cooldown = 0,
		maxCooldown = 3,
		value = 800,
		range = 12,

		PROGRAM_LIST = 10,	

		acquireTargets = function( self, targets, game, sim, unit )
            		local targetHandler = targets.customAreaTarget( game, sim, {"+", 0, self.range, 1}, sim:getPC() )
            		targetHandler:setUnitPredicate(
                		function( u )
                    			return mainframe.canBreakIce( sim, u, self )
                		end )
            		targetHandler:setHiliteColor( { 0.33, 0.33, 0.33, 0.33 } )
            		return targetHandler
		end, 

        	startTargeting = function( self, sim, unit, userUnit, targetCell )
    			MOAIFmodDesigner.playSound( "SpySociety/Actions/mainframe_datablast_select" )
        	end,

        	getTargetUnits = function( self, sim, cellx, celly )
            		local cells = simquery.fillOrthogonalCross( sim, cellx, celly, 0, self.range, 1 )
            		local units = {}
            		for i, x, y in util.xypairs( cells ) do
                		local cell = sim:getCell( x, y )
                		if cell then
                    			for _, cellUnit in ipairs(cell.units) do
                        			if sim:getCurrentPlayer():getLastKnownCell( sim, x, y ) ~= nil then
                            				if mainframe.canBreakIce( sim, cellUnit, self ) then
                                				table.insert( units, cellUnit )
                            				end
                        			end
                    			end
                		end
            		end
            		return units
        	end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			local player = sim:getCurrentPlayer()
            		local currentProgram = player:getEquippedProgram()

    			player:equipProgram( sim, self:getID() )
            		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_datablast_use" )

            		local targetUnits = self:getTargetUnits( sim, targetCell[1], targetCell[2] )
            		local daemonUnits = {}

            		for _, targetUnit in ipairs(targetUnits) do
				local daemonProgram = targetUnit:getTraits().mainframe_program
				local firewallsToBreak = self.break_firewalls
				local x0, y0 = targetUnit:getLocation()

				if sim:getPC():getTraits().firewallBreakPenalty and firewallsToBreak > 0 then
	        			firewallsToBreak = math.max(firewallsToBreak - sim:getPC():getTraits().firewallBreakPenalty, 1)
				end

				if firewallsToBreak >= targetUnit:getTraits().mainframe_ice then
                			targetUnit:getTraits().mainframe_program = nil
				end

				sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = { targetUnit }, range = 1 } )	
                		mainframe.breakIce( sim,  targetUnit, firewallsToBreak )

                		if daemonProgram and targetUnit:getTraits().mainframe_ice <= 0 then
					targetUnit:getTraits().mainframe_program = daemonProgram
                    			table.insert( daemonUnits, targetUnit )
                		end
	        	end

            		self:useCPUs( sim )
			self:setCooldown( sim )
    			player:equipProgram( sim, currentProgram and currentProgram:getID() )

            		for i, daemonUnit in ipairs( daemonUnits ) do
                		mainframe.invokeDaemon( sim, daemonUnit )
            		end
		end,
	},

	W93_split = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.SPLIT.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.SPLIT.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.SPLIT.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.SPLIT.SHORT_DESC,
		tipdesc = STRINGS.PROGEXTEND.PROGRAMS.SPLIT.TIP_DESC,

		icon = "gui/icons/programs_icons/icon-program_split.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_split.png",
		cpu_cost = 4,
		break_firewalls = 2, 
		equip_program = true, 
		equipped = false, 
		value = 400,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 5,	

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)
			for i, unit in pairs(sim:getAllUnits()) do
				local x1, y1 = unit:getLocation()
				if unit:getName() == targetUnit:getName() and unit ~= targetUnit and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and sim:getPC():getLastKnownCell( sim, x1, y1 ) ~= nil then
					mainframe.breakIce( sim, unit, 1 )
				end
			end
		end,
	},

	W93_void = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.VOID.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.VOID.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.VOID.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.VOID.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_void.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_void.png",
		value = 0,
		passive = true,
		MH_exception = true,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				local itemdefs = include( "sim/unitdefs/itemdefs" )
				for _, agent in pairs(sim:getPC():getUnits()) do
					local item = simfactory.createUnit( itemdefs.W93_item_icebreaker_starter, sim )
					sim:spawnUnit(item)
					agent:addChild( item )
				end
				for _, ability in pairs(sim:getPC():getAbilities()) do
					if ability ~= self then
						sim:getPC():removeAbility( sim, ability )
					end
				end
				sim:getPC():removeAbility( sim, self )
			end
		end,
	},

	W93_volcano = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.DESC,
		huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.HUD_DESC, 0),
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_volcano.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_volcano.png",
		value = 450,
		passive = true,
		daemonReversalAdd = 0,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,

		onSpawnAbility = function( self, sim )
			sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:addTrigger( simdefs.TRG_DAEMON_REVERSE, self )
			self.huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.HUD_DESC, self.daemonReversalAdd)
		end, 

    		onDespawnAbility = function( self, sim, player )
			sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:removeTrigger( simdefs.TRG_DAEMON_REVERSE, self )	
    		end,

		canUseAbility = function( self, sim )
			return false 	
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_DAEMON_INSTALL then
				sim:getPC():addCPUs( 1 )
				self.daemonReversalAdd = self.daemonReversalAdd + 5
				self.huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.HUD_DESC, self.daemonReversalAdd)
			end
			if evType == simdefs.TRG_DAEMON_REVERSE then
				self.daemonReversalAdd = 0
				self.huddesc = string.format(STRINGS.PROGEXTEND.PROGRAMS.VOLCANO.HUD_DESC, self.daemonReversalAdd)
			end
		end,
	}
}

local ai_programs =
{
	W93_decryptor = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.DECRYPTOR.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.DECRYPTOR.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.DECRYPTOR.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.DECRYPTOR.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_decryptor.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_decryptor.png",
		cpu_cost = 2,
		cooldown = 0,
		maxCooldown = 5,		
		value = 400,
		MH_exception = true,

		PROGRAM_LIST = 20,
		PROGRAM_NO_BREAKER_LIST = 20,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim, player )
			player:getTraits().aiToken = player:getTraits().aiToken or 0
			player:getTraits().aiTokenMax = (player:getTraits().aiTokenMax or 8) + 6
		end,

    		onDespawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onDespawnAbility( self, sim, player )
			player:getTraits().aiTokenMax = player:getTraits().aiTokenMax - 6
    		end,

		canUseAbility = function( self, sim, abilityOwner )
			if not sim:getNPC():hasMainframeAbility( "W93_AI_assembly" ) then
				return false, STRINGS.PROGEXTEND.UI.REASON_NO_AI
			end

			if sim:getPC():getTraits().aiToken >= sim:getPC():getTraits().aiTokenMax then
				return false, STRINGS.PROGEXTEND.UI.REASON_MAX_TOKENS
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner )
		end,

		executeAbility = function( self, sim, unit, userUnit, targetCell )
			sim:getPC():getTraits().aiToken = math.min(sim:getPC():getTraits().aiToken + 4, sim:getPC():getTraits().aiTokenMax)

            		self:useCPUs( sim )
            		self:setCooldown( sim )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.DECRYPTOR.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate",icon=self.icon } )
		end,

		onTooltip = function( self, screen, sim, player )
			local tooltip = util.tooltip( screen )
			local section = tooltip:addSection()
			local sub = util.sformat( STRINGS.PROGRAMS.POWER, self:getCpuCost() )							

			section:addLine( "<ttheader>"..self.name.."</>",sub)
			section:addLine( util.sformat( STRINGS.PROGRAMS.COOLDOWN, self.maxCooldown )  )
			section:addAbility( self.shortdesc, self.desc, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

			if sim then
				section:addLine( string.format(STRINGS.PROGEXTEND.AI.TOKEN, sim:getPC():getTraits().aiToken, sim:getPC():getTraits().aiTokenMax) )
			    	local canUse, reason = self:canUseAbility( sim, player )
			    	if not canUse and reason then
				    	section:addRequirement( reason )
			    	end
	        	end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1],self.dlcFooter[2])
			end	        

			return tooltip
		end,
	},

	W93_equalizer = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.EQUALIZER.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.EQUALIZER.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.EQUALIZER.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.EQUALIZER.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_equalizer.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_equalizer.png",
		cpu_cost = 0,
		cooldown = 0,
		maxCooldown = 4,	
		value = 600,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		canUseAbility = function( self, sim, abilityOwner )
			if not sim:getNPC():hasMainframeAbility( "W93_AI_assembly" ) then
				return false, STRINGS.PROGEXTEND.UI.REASON_NO_AI
			end

			return DEFAULT_ABILITY.canUseAbility( self, sim, abilityOwner )
		end,

		executeAbility = function( self, sim, unit, userUnit )
            		self:useCPUs( sim )
            		self:setCooldown( sim )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.PROGRAMS.EQUALIZER.WARNING, color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate",icon=self.icon } )

			local av = math.floor((sim:getPC():getCpus() + sim:getNPC():getCpus())/2)
			sim:getPC():addCPUs(av - sim:getPC():getCpus())
			sim:getNPC():addCPUs(av - sim:getNPC():getCpus())
		end,
	},

	W93_hushpuppy = util.extend( DEFAULT_ABILITY )
	{
		name = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_hushpuppy.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_hushpuppy.png",
		passive = true,		
		value = 400,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim, player )
			player:getTraits().aiToken = player:getTraits().aiToken or 0
			player:getTraits().aiTokenMax = (player:getTraits().aiTokenMax or 8) + 2
		end,

		canUseAbility = function( self, sim, abilityOwner )
			return false
		end,
	},

	W93_hushpuppy_PWR = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PWR.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PWR.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PWR.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PWR.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_hushpuppy.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_hushpuppy.png",
		passive = true,		
		value = 300,
		MH_exception = true,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim, player )
			player:getTraits().aiToken = player:getTraits().aiToken or 0
			player:getTraits().aiTokenMax = (player:getTraits().aiTokenMax or 8) + 2
		end,

		canUseAbility = function( self, sim, abilityOwner )
			return false
		end,
	},

	W93_hushpuppy_PRO = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PRO.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PRO.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PRO.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_PRO.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_hushpuppy.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_hushpuppy.png",
		passive = true,		
		value = 300,
		MH_exception = true,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim, player )
			player:getTraits().aiToken = player:getTraits().aiToken or 0
			player:getTraits().aiTokenMax = (player:getTraits().aiTokenMax or 8) + 2
		end,

		canUseAbility = function( self, sim, abilityOwner )
			return false
		end,
	},

	W93_hushpuppy_REA = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_REA.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_REA.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_REA.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.HUSHPUPPY_REA.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_hushpuppy.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_hushpuppy.png",
		passive = true,		
		value = 300,
		MH_exception = true,

		onSpawnAbility = function( self, sim, player )
			DEFAULT_ABILITY.onSpawnAbility( self, sim, player )
			player:getTraits().aiToken = player:getTraits().aiToken or 0
			player:getTraits().aiTokenMax = (player:getTraits().aiTokenMax or 8) + 2
		end,

		canUseAbility = function( self, sim, abilityOwner )
			return false
		end,
	},

	W93_leech = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.LEECH.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.LEECH.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.LEECH.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.LEECH.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_leech.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_leech.png",
		passive = true,		
		value = 550,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		canUseAbility = function( self, sim, abilityOwner )
			return false
		end,

		onTrigger = function( self, sim, evType, evData )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and sim:getNPC():getCpus() > 0 then
				local pwr = math.min( 1, sim:getNPC():getCpus() )
				sim:getPC():addCPUs( pwr )
				sim:getNPC():addCPUs( -pwr )
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.PROGRAMS.LEECH.WARNING, pwr, pwr), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate",icon=self.icon } )
			end
		end,
	},

	W93_network_siphon = util.extend( DEFAULT_ABILITY )
	{

		name = STRINGS.PROGEXTEND.PROGRAMS.NETWORK_SIPHON.NAME,
		desc = STRINGS.PROGEXTEND.PROGRAMS.NETWORK_SIPHON.DESC,
		huddesc = STRINGS.PROGEXTEND.PROGRAMS.NETWORK_SIPHON.HUD_DESC,
		shortdesc = STRINGS.PROGEXTEND.PROGRAMS.NETWORK_SIPHON.SHORT_DESC,

		icon = "gui/icons/programs_icons/icon-program_network_siphon.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_network_siphon.png",
		passive = true,		
		value = 600,
		MH_exception = true,

		PROGRAM_LIST = 10,
		PROGRAM_NO_BREAKER_LIST = 10,

		canUseAbility = function( self, sim, abilityOwner )
			return false
		end,
	},
}

local function adjustAvailability( t )
	local availability = 0
	if type(t) == "boolean" and t == true then
		availability = 1
	end
	if type(t) == "number" then
		availability = t
	end

	if availability == 0 then
		mainframe_abilities.W93_avalanche.PROGRAM_NO_BREAKER_LIST = 2
		mainframe_abilities.W93_axe.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_bandwidth.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_bank.PROGRAM_LIST = 10
		mainframe_abilities.W93_bank.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_breach.PROGRAM_LIST = 20
		mainframe_abilities.W93_breach.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_breach_2.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_bribe.PROGRAM_LIST = 5
		mainframe_abilities.W93_bribe.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_club.PROGRAM_LIST = 20
		mainframe_abilities.W93_club.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_condense.PROGRAM_LIST = 5
		mainframe_abilities.W93_condense.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_convert.PROGRAM_LIST = 20
		mainframe_abilities.W93_convert.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_crossfeed.PROGRAM_LIST = 10
		mainframe_abilities.W93_dao.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_fade.PROGRAM_LIST = 20
		mainframe_abilities.W93_fission.PROGRAM_LIST = nil
		mainframe_abilities.W93_fission.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_flail.PROGRAM_LIST = 20
		mainframe_abilities.W93_flail.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_fortify.PROGRAM_LIST = 20
		mainframe_abilities.W93_gear_shift.PROGRAM_LIST = nil
		mainframe_abilities.W93_gear_shift.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_greed.PROGRAM_LIST = 20
		mainframe_abilities.W93_heartbreaker.PROGRAM_LIST = 15
		mainframe_abilities.W93_hitman.PROGRAM_LIST = 10
		mainframe_abilities.W93_lantern.PROGRAM_LIST = 20
		mainframe_abilities.W93_lantern.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_lottery.PROGRAM_LIST = 20
		mainframe_abilities.W93_lottery.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_nosferatu.PROGRAM_LIST = nil
		mainframe_abilities.W93_nosferatu.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_omniwrench.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_overview.PROGRAM_LIST = 15
		mainframe_abilities.W93_plague.PROGRAM_LIST = 20
		mainframe_abilities.W93_plague.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_poke.PROGRAM_LIST = 20
		mainframe_abilities.W93_poke.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_power_surge.PROGRAM_LIST = 15
		mainframe_abilities.W93_power_surge.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_power_uplink.PROGRAM_LIST = nil
		mainframe_abilities.W93_power_uplink.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_pwr_bill.PROGRAM_LIST = nil
		mainframe_abilities.W93_pwr_bill.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_rally.PROGRAM_LIST = 20
		mainframe_abilities.W93_rally.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_reflect.PROGRAM_LIST = 20
		mainframe_abilities.W93_reflect.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_rental.PROGRAM_LIST = 10
		mainframe_abilities.W93_resonance.PROGRAM_LIST = 10
		mainframe_abilities.W93_resonance.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_ring.PROGRAM_LIST = 10
		mainframe_abilities.W93_sap.PROGRAM_LIST = 10
		mainframe_abilities.W93_sap.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_scapegoat.PROGRAM_LIST = 10
		mainframe_abilities.W93_search.PROGRAM_LIST = 10
		mainframe_abilities.W93_sonar.PROGRAM_LIST = 20
		mainframe_abilities.W93_sonar.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_spear.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_split.PROGRAM_LIST = 20
		mainframe_abilities.W93_split.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_volcano.PROGRAM_LIST = 20
		mainframe_abilities.W93_volcano.PROGRAM_NO_BREAKER_LIST = 20

		ai_programs.W93_decryptor.PROGRAM_LIST = 20
		ai_programs.W93_decryptor.PROGRAM_NO_BREAKER_LIST = 20
		ai_programs.W93_equalizer.PROGRAM_LIST = 10
		ai_programs.W93_hushpuppy.PROGRAM_LIST = 10
		ai_programs.W93_leech.PROGRAM_LIST = 10
		ai_programs.W93_network_siphon.PROGRAM_LIST = 10
	elseif availability == 1 then
		mainframe_abilities.W93_avalanche.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_axe.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_bandwidth.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_bank.PROGRAM_LIST = nil
		mainframe_abilities.W93_bank.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_breach.PROGRAM_LIST = 20
		mainframe_abilities.W93_breach.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_breach_2.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_bribe.PROGRAM_LIST = 5
		mainframe_abilities.W93_bribe.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_club.PROGRAM_LIST = 20
		mainframe_abilities.W93_club.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_condense.PROGRAM_LIST = nil
		mainframe_abilities.W93_condense.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_convert.PROGRAM_LIST = nil
		mainframe_abilities.W93_convert.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_crossfeed.PROGRAM_LIST = nil
		mainframe_abilities.W93_dao.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_fade.PROGRAM_LIST = nil
		mainframe_abilities.W93_fission.PROGRAM_LIST = nil
		mainframe_abilities.W93_fission.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_flail.PROGRAM_LIST = 20
		mainframe_abilities.W93_flail.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_fortify.PROGRAM_LIST = nil
		mainframe_abilities.W93_gear_shift.PROGRAM_LIST = nil
		mainframe_abilities.W93_gear_shift.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_greed.PROGRAM_LIST = nil
		mainframe_abilities.W93_heartbreaker.PROGRAM_LIST = nil
		mainframe_abilities.W93_hitman.PROGRAM_LIST = nil
		mainframe_abilities.W93_lantern.PROGRAM_LIST = nil
		mainframe_abilities.W93_lantern.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_lottery.PROGRAM_LIST = nil
		mainframe_abilities.W93_lottery.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_nosferatu.PROGRAM_LIST = nil
		mainframe_abilities.W93_nosferatu.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_omniwrench.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_overview.PROGRAM_LIST = nil
		mainframe_abilities.W93_plague.PROGRAM_LIST = 20
		mainframe_abilities.W93_plague.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_poke.PROGRAM_LIST = nil
		mainframe_abilities.W93_poke.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_power_surge.PROGRAM_LIST = nil
		mainframe_abilities.W93_power_surge.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_power_uplink.PROGRAM_LIST = nil
		mainframe_abilities.W93_power_uplink.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_pwr_bill.PROGRAM_LIST = nil
		mainframe_abilities.W93_pwr_bill.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_rally.PROGRAM_LIST = 20
		mainframe_abilities.W93_rally.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_reflect.PROGRAM_LIST = nil
		mainframe_abilities.W93_reflect.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_rental.PROGRAM_LIST = nil
		mainframe_abilities.W93_resonance.PROGRAM_LIST = nil
		mainframe_abilities.W93_resonance.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_ring.PROGRAM_LIST = nil
		mainframe_abilities.W93_sap.PROGRAM_LIST = nil
		mainframe_abilities.W93_sap.PROGRAM_NO_BREAKER_LIST = 5
		mainframe_abilities.W93_scapegoat.PROGRAM_LIST = nil
		mainframe_abilities.W93_search.PROGRAM_LIST = nil
		mainframe_abilities.W93_sonar.PROGRAM_LIST = nil
		mainframe_abilities.W93_sonar.PROGRAM_NO_BREAKER_LIST = 20
		mainframe_abilities.W93_spear.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_split.PROGRAM_LIST = 20
		mainframe_abilities.W93_split.PROGRAM_NO_BREAKER_LIST = nil
		mainframe_abilities.W93_volcano.PROGRAM_LIST = nil
		mainframe_abilities.W93_volcano.PROGRAM_NO_BREAKER_LIST = 20

		ai_programs.W93_decryptor.PROGRAM_LIST = nil
		ai_programs.W93_decryptor.PROGRAM_NO_BREAKER_LIST = 20
		ai_programs.W93_equalizer.PROGRAM_LIST = nil
		ai_programs.W93_hushpuppy.PROGRAM_LIST = nil
		ai_programs.W93_leech.PROGRAM_LIST = nil
		ai_programs.W93_network_siphon.PROGRAM_LIST = nil
	elseif availability == 2 then
		mainframe_abilities.W93_avalanche.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_axe.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_bandwidth.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_bank.PROGRAM_LIST = 10
		mainframe_abilities.W93_bank.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_breach.PROGRAM_LIST = 10
		mainframe_abilities.W93_breach.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_breach_2.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_bribe.PROGRAM_LIST = 10
		mainframe_abilities.W93_bribe.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_club.PROGRAM_LIST = 10
		mainframe_abilities.W93_club.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_condense.PROGRAM_LIST = 10
		mainframe_abilities.W93_condense.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_convert.PROGRAM_LIST = 10
		mainframe_abilities.W93_convert.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_crossfeed.PROGRAM_LIST = 10
		mainframe_abilities.W93_dao.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_fade.PROGRAM_LIST = 10
		mainframe_abilities.W93_fission.PROGRAM_LIST = 10
		mainframe_abilities.W93_fission.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_flail.PROGRAM_LIST = 10
		mainframe_abilities.W93_flail.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_fortify.PROGRAM_LIST = 10
		mainframe_abilities.W93_gear_shift.PROGRAM_LIST = 10
		mainframe_abilities.W93_gear_shift.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_greed.PROGRAM_LIST = 10
		mainframe_abilities.W93_heartbreaker.PROGRAM_LIST = 10
		mainframe_abilities.W93_hitman.PROGRAM_LIST = 10
		mainframe_abilities.W93_lantern.PROGRAM_LIST = 10
		mainframe_abilities.W93_lantern.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_lottery.PROGRAM_LIST = 10
		mainframe_abilities.W93_lottery.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_nosferatu.PROGRAM_LIST = 10
		mainframe_abilities.W93_nosferatu.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_omniwrench.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_overview.PROGRAM_LIST = 10
		mainframe_abilities.W93_plague.PROGRAM_LIST = 10
		mainframe_abilities.W93_plague.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_poke.PROGRAM_LIST = 10
		mainframe_abilities.W93_poke.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_power_surge.PROGRAM_LIST = 10
		mainframe_abilities.W93_power_surge.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_power_uplink.PROGRAM_LIST = 10
		mainframe_abilities.W93_power_uplink.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_pwr_bill.PROGRAM_LIST = 10
		mainframe_abilities.W93_pwr_bill.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_rally.PROGRAM_LIST = 10
		mainframe_abilities.W93_rally.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_reflect.PROGRAM_LIST = 10
		mainframe_abilities.W93_reflect.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_rental.PROGRAM_LIST = 10
		mainframe_abilities.W93_resonance.PROGRAM_LIST = 10
		mainframe_abilities.W93_resonance.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_ring.PROGRAM_LIST = 10
		mainframe_abilities.W93_sap.PROGRAM_LIST = 10
		mainframe_abilities.W93_sap.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_scapegoat.PROGRAM_LIST = 10
		mainframe_abilities.W93_search.PROGRAM_LIST = 10
		mainframe_abilities.W93_sonar.PROGRAM_LIST = 10
		mainframe_abilities.W93_sonar.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_spear.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_split.PROGRAM_LIST = 10
		mainframe_abilities.W93_split.PROGRAM_NO_BREAKER_LIST = 10
		mainframe_abilities.W93_volcano.PROGRAM_LIST = 10
		mainframe_abilities.W93_volcano.PROGRAM_NO_BREAKER_LIST = 10

		ai_programs.W93_decryptor.PROGRAM_LIST = 10
		ai_programs.W93_decryptor.PROGRAM_NO_BREAKER_LIST = 10
		ai_programs.W93_equalizer.PROGRAM_LIST = 10
		ai_programs.W93_hushpuppy.PROGRAM_LIST = 10
		ai_programs.W93_leech.PROGRAM_LIST = 10
		ai_programs.W93_network_siphon.PROGRAM_LIST = 10
	end
end

return 
{
	mainframe_abilities = mainframe_abilities,
	ai_programs = ai_programs,
	adjustAvailability = adjustAvailability
}
