local util = include("modules/util")
local mainframe = include("sim/mainframe")
local simdefs = include("sim/simdefs")
local mainframe_common = include("sim/abilities/mainframe_common")
local nonbreakIceTooltip = include( "hud/tooltip_nonbreakice" )

local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY

			
local booster_tooltip = class(nonbreakIceTooltip)

function booster_tooltip:init( mainframePanel, targetWidget, unit, reason )
	util.tooltip.init( self, mainframePanel._hud._screen )
	self._targetWidget = targetWidget
	self.mainframePanel = mainframePanel

	local localPlayer = mainframePanel._hud._game:getLocalPlayer()
	local equippedProgram = nil
	if localPlayer then 
		equippedProgram = localPlayer:getEquippedProgram()
		if equippedProgram then
			local programWidget = mainframePanel._panel.binder.programsPanel:findWidget( equippedProgram:getID() )		
			if programWidget and programWidget:isVisible() then            	
				self._ux0, self._uy0 = programWidget.binder.btn:getAbsolutePosition()
				if equippedProgram:canUseAbility( mainframePanel._hud._game.simCore, localPlayer ) then
					self.programWidget = programWidget
					self.equippedProgram = equippedProgram
				end
			end
		end
	end

	local section = self:addSection()
	section:addLine( "<ttheader>"..util.sformat( "BOOST {1}", unit:getName() ).."</>" )

	if equippedProgram then
		section:addAbility( string.format(STRINGS.UI.TOOLTIPS.CURRENTLY_EQUIPPED, equippedProgram:getDef().name), util.sformat(equippedProgram:getDef().tipdesc, equippedProgram:getCpuCost()),  "gui/icons/arrow_small.png" )
	end 

	if reason then
		section:addRequirement( reason )
	end
end

local abilities = {
	_debug_breaker = util.extend( DEFAULT_ABILITY )
	{
		name = "DEBUG BREAKER",
		desc = "ABSOLUTELY DESTROYS FIREWALLS",
		huddesc = "DELETES FIREWALL",
		shortdesc = "DELETE FIREWALL",
		tipdesc = "DELETES FIREWALLS",

		icon = "gui/icons/programs_icons/icon-program_breach.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_breach.png",
		cpu_cost = -90,
		break_firewalls = 99, 
		coolDownMod = -90,
		pwrMod = -90,
		equip_program = true, 
		equipped = false, 
		value = 400,

		executeAbility = function( self, sim, targetUnit )
			targetUnit:getTraits().mainframe_program = nil
			sim:dispatchEvent( simdefs.EV_KILL_DAEMON, {unit = targetUnit})
			targetUnit:getTraits().daemonHost =nil
		
			mainframe.breakIce( sim, targetUnit, 99 )
		end,
	},
	_debug_booster = util.extend( DEFAULT_ABILITY )
	{
		name = "BOOSTER",
		desc = "Give an agent 6 AP. Cost 2 PWR\n2 turn cooldown.",
		huddesc = "GIVE AGENT 6 AP.",
		shortdesc = "Give agent 6 AP.",
		tipdesc = "GIVE AGENT 6 AP. COST: {1} PWR",

		icon = "gui/icons/programs_icons/icon-program_lightning.png",
		icon_100 = "gui/icons/programs_icons/store_icons/StorePrograms_lightning.png",
		cpu_cost = 2,
		
		targetingIcon = "gui/icons/item_icons/items_icon_small/icon-item_stim_small.png",
		targetingDisabledColor = util.color(0.45,0.45,0.5,0.6),
		targetingColor = util.color( 120/255, 244/255, 255/255 ),
		targetingHoverColor = util.color( 188/255, 250/255, 255/255 ),
		
		cooldown = 0,
		maxCooldown = 2,
		
		mp_restore = 6,
		equip_program = true, 
		equipped = false, 
		value = 600,	
		targetRawUnit = true,

		customTargetTooltip = function( mainframePanel, widget, unit, reason )
			return booster_tooltip( mainframePanel, widget, unit, reason )
		end,
		
		isTarget = function(self, sim, abilityOwner, targetUnit)
			if not targetUnit:getTraits().isAgent then 				
				return false
			end
			
			if targetUnit:getTraits().isGuard then 				
				return false
			end
			
			if targetUnit:isDown() then 				
				return false
			end
			
			local unit = abilityOwner:getLastKnownUnit( sim, targetUnit:getID() )
			local canSee = sim:canPlayerSeeUnit( abilityOwner, unit )
			return (unit:isGhost() or canSee) and sim:getCell( targetUnit:getLocation() )
		end,
		
		--[[shouldRawTarget = function( self, panel, widgets, unitRaw )
			local localPlayer = panel._hud._game:getLocalPlayer()
			local sim = panel._hud._game.simCore
		
			return self:isTarget(sim,localPlayer,unitRaw)
		end,]]

		canUseAbility = function( self, sim, abilityOwner, targetUnit )

			if targetUnit then
				if not self:isTarget(sim, abilityOwner, targetUnit) then
					return false
				end
			end

			local player = sim:getCurrentPlayer()
			if player == nil or player ~= abilityOwner then
				return false
			end

			if (self.cooldown or 0) > 0 then 
				return false, STRINGS.UI.REASON.EQUIPPED_ON_COOLDOWN
			end

			if player:getCpus() < self:getCpuCost() then 
				return false, STRINGS.UI.REASON.NOT_ENOUGH_PWR
			end

			if sim:getMainframeLockout() then 
				return false, STRINGS.UI.REASON.INCOGNITA_LOCKED_DOWN
			end 

			return true	
		end,

		executeAbility = function( self, sim, targetUnit )
			DEFAULT_ABILITY.executeAbility(self, sim, targetUnit)
			
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_lightning_strike" )			

			targetUnit:getTraits().mp =targetUnit:getTraits().mp + self.mp_restore

			local x1, y1 = targetUnit:getLocation()
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = targetUnit } )
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.MOVEMENT_BOOSTED,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
		end,			
	},	
}

return abilities