local cdefs = include( "client_defs" )
local combat_panel = include( "hud/combat_panel" )
local mainframe_panel = include( "hud/mainframe_panel" )
local mainframe = include( "sim/mainframe" )
local util = include( "modules/util" )

local null_tooltip = class()

function null_tooltip:init( hud, str, str2 )
	self._hud = hud
	self._str = str
	self._str2 = str2  
end

function null_tooltip:setPosition( wx, wy )
	self._panel:setPosition( self._hud._screen:wndToUI( wx, wy ))
end

function null_tooltip:getScreen()
	return self._hud._screen
end

function null_tooltip:activate( screen )
	local COLOR_GREY = util.color(180/255,180/255,180/255,150/255)

	self._panel = combat_panel( self._hud, self._hud._screen )
	self._panel:refreshPanelFromStr( self._str, self._str2, COLOR_GREY )
end

function null_tooltip:deactivate()
	self._panel:setVisible( false )
end

function mainframe_panel.panel:refreshBreakIceButton( widget, unit )
	local sim = self._hud._game.simCore
	local program = self:getCurrentProgram()
	local canUse, reason, target =  false, STRINGS.UI.REASON.NO_EQUIPPED_PROGRAM, nil
	
	if program then
		canUse, reason, target = mainframe.canBreakIce( sim, unit, program )
	end

	if self._hud._game:isReplaying() then
		canUse, reason = false, nil
	end
    
	local tooltipWidget = widget.binder.btn
	local daemonTooltip = nil
	widget:setAlias( "BreakIce"..unit:getID() )
	if not widget.iceBreak then
		widget.binder.btn:setText(unit:getTraits().mainframe_ice)
	end
	widget.binder.btn:setDisabled( not canUse )
	if not canUse then		
		widget.binder.anim:getProp():setRenderFilter( cdefs.RENDER_FILTERS["desat"] )
	else
		widget.binder.anim:getProp():setRenderFilter( cdefs.RENDER_FILTERS["normal"] )
	end

	if unit:getTraits().parasite then
		widget.binder.anim:setAnim( "idle_bugged" )
	else
		widget.binder.anim:setAnim( "idle" )
	end

	if sim:getParams().difficultyOptions.W93_AI then
		if unit:getTraits().flail_target then
			widget.binder.flailTarget:setVisible(true)
		else
			widget.binder.flailTarget:setVisible(false)
		end

		if unit:getTraits().crossfeed_source then
			widget.binder.crossfeedSource:setVisible(true)
		else
			widget.binder.crossfeedSource:setVisible(false)
		end

		if unit:getTraits().crossfeed_target then
			widget.binder.crossfeedTarget:setVisible(true)
		else
			widget.binder.crossfeedTarget:setVisible(false)
		end

		if unit:getTraits().uplink then
			widget.binder.pwrUplink:setVisible(true)
		else
			widget.binder.pwrUplink:setVisible(false)
		end

		if unit:getTraits().plague then
			widget.binder.plague:setVisible(true)
		else
			widget.binder.plague:setVisible(false)
		end
	end

	local programWidget = widget.binder.program

	if sim:getHideDaemons() and not unit:getTraits().daemon_sniffed then
		programWidget:setVisible( true )
		programWidget.binder.daemonUnknown:setVisible(false)
		programWidget.binder.daemonKnown:setVisible(false)
		programWidget.binder.daemonHidden:setVisible(true)
	elseif unit:getTraits().mainframe_program ~= nil then
		programWidget:setVisible( true )

		local npc_abilities = include( "sim/abilities/npc_abilities" )
		local ability = npc_abilities[ unit:getTraits().mainframe_program ]
		if unit:getTraits().daemon_sniffed then 
			programWidget.binder.daemonUnknown:setVisible(false)
			programWidget.binder.daemonKnown:setVisible(true)
			if unit:getTraits().daemon_sniffed_revealed == nil then
				unit:getTraits().daemon_sniffed_revealed = true
				programWidget.binder.daemonKnown.binder.txt:spoolText(ability.name, 12)			
			else
				programWidget.binder.daemonKnown.binder.txt:setText(ability.name)			
			end
           		daemonTooltip = programWidget.binder.daemonKnown.binder.bg
        	else
			programWidget.binder.daemonKnown:setVisible(false)
    			programWidget.binder.daemonUnknown:setVisible(true)
		end	
        	programWidget.binder.daemonHidden:setVisible(false)
    	else
        	programWidget:setVisible( false )
	end

    	if not canUse or program == nil or program.acquireTargets then
        	widget.binder.btn.onClick = nil
        	tooltipWidget:setTooltip( nil )
   	else
	    	widget.binder.btn.onClick = function( widget, ie )
                	if unit:isValid() and mainframe.canBreakIce( sim, unit, program ) then
				MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_MAINFRAME_CONFIRM_ACTION )
				self._hud._game:doAction( "mainframeAction", {action = "breakIce", unitID = unit:getID() } )
			end
		end
	end
	if reason == "nulldrone" then
		tooltipWidget:setTooltip( function()
			local str = STRINGS.UI.REASON.SECURED
			local str2 = string.upper(string.format(STRINGS.FUNCLIB.UI.REASON_NULLZONE_BY, target or ""))
			return null_tooltip( self._hud, str, str2 )
		end )
		widget.binder.btn:setText("-")
	else  
		tooltipWidget:setTooltip( function()
			local breakIceTooltip = include( "hud/tooltip_breakice" )
			return breakIceTooltip( self, widget, unit, reason )
		end )
	    
        	if daemonTooltip then 
        		daemonTooltip:setTooltip( function() 
        			local breakIceTooltip = include( "hud/tooltip_breakice_nolines" )
            			return breakIceTooltip( self, widget, unit, reason )
        		end )
        	end 
	end   
end