local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local array = include( "modules/array" )
local mui_defs = include( "mui/mui_defs")
local world_hud = include( "hud/hud-inworld" )
local hudtarget = include( "hud/targeting")
local rig_util = include( "gameplay/rig_util" )
local level = include( "sim/level" )
local mainframe = include( "sim/mainframe" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )

local oldCanBreakIce = mainframe.canBreakIce

function mainframe.canBreakIce( sim, targetUnit, equippedProgram )
	local ok, result = oldCanBreakIce( sim, targetUnit, equippedProgram )
	
	if type(ok) == "string" then
		return false, ok
	end
	
	return ok, result
end

local panel = include("hud/mainframe_panel").panel

local oldGetCurrentProgram = panel.getCurrentProgram

function panel:getCurrentProgram()
    local currentProgram = oldGetCurrentProgram(self)

	if not currentProgram then
		return {
			canUseAbility = function()
				return false, STRINGS.UI.REASON.NO_EQUIPPED_PROGRAM
			end,
			getCpuCost = function()
				return -1
			end
		}
	end
	
    return currentProgram
end

local function findWidget( widgets, unit )
	for _, widget in ipairs( widgets ) do		
		if widget.ownerID == unit:getID() then
			return widget
		end
	end

	return nil
end

panel.RAW_MF_T = "RAW_MF_T"

local oldOnSimEvent = panel.onSimEvent
local oldDestroyUI = panel.destroyUI

function panel:createRawTargetButton( widgets, unit )
	local sim = self._hud._game.simCore
    local program = self:getCurrentProgram()
	local canUse, reason = mainframe.canTargetUnit( sim, unit, program )
    if (not canUse and not reason) or not program.targetRawUnit then
        return
    end

	local wx, wy = self._hud._game:cellToWorld( unit:getLocation() )
	local widget = findWidget( widgets, unit )
	if widget == nil then
		local wz = (program.getTargetOffset and program:getTargetOffset(self, widgets, unit)) or unit:getTraits().breakIceOffset or 12
		widget = self._hud._world_hud:createWidget( self.RAW_MF_T, program.widgetName or "Target", { worldx = wx, worldy = wy, worldz = wz, ownerID = unit:getID() } )
	else
		array.removeElement( widgets, widget )
	end

    self:refreshTargetButton( widget, unit )
	
	local sim = self._hud._game.simCore
    local program = self:getCurrentProgram()
	local canUse, reason = mainframe.canTargetUnit( sim, unit, program )
    if self._hud._game:isReplaying() then
        canUse, reason = false, nil
    end
	
	widget.binder.btn:setImage( program.targetingIcon or "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )

	if canUse then
	    widget.binder.btn:setColor((program.targetingColor or cdefs.COLOR_FREE):unpack())			
	    widget.binder.btn:setColorInactive((program.targetingColor or cdefs.COLOR_FREE):unpack())
	    widget.binder.btn:setColorActive((program.targetingHoverColor or cdefs.COLOR_FREE_HOVER):unpack())
	    widget.binder.btn:setColorHover((program.targetingHoverColor or cdefs.COLOR_FREE_HOVER):unpack())
    else
		widget.binder.btn:setColor((program.targetingDisabledColor or {0.5,0.5,0.5,1}):unpack())		
		widget.binder.img:setColor((program.targetingDisabledColor or {0.5,0.5,0.5,1}):unpack())
	end
	
	
    widget.binder.btn:setTooltip( function()
		if program.customTargetTooltip then
			return program.customTargetTooltip( self, widget, unit, reason )
		else
			local nonbreakIceTooltip = include( "hud/tooltip_nonbreakice" )
			return nonbreakIceTooltip( self, widget, unit, reason )
		end
    end )

	-- Update navigation
	if self._hud.addToNavigation and widget.binder.btn:isDisabled() == false then
		self._hud:addToNavigation(self._hud.NAV_MAINFRAME2, widget.binder.btn, false)
	end

	return widget
end

function panel:defaultShouldRawTarget( program, widgets, unitRaw )
	local localPlayer = self._hud._game:getLocalPlayer()
	local sim = self._hud._game.simCore
	local unit = localPlayer:getLastKnownUnit( sim, unitRaw:getID() )
	local canSee = sim:canPlayerSeeUnit( localPlayer, unit )
	local canUse, reason = program:canUseAbility( sim, localPlayer, unit )
	return (unit:isGhost() or canSee) and sim:getCell( unitRaw:getLocation() ) and (canUse or reason)
end

function panel:rawTargeting()
	local widgets = util.tdupe( self._hud._world_hud:getWidgets( self.RAW_MF_T ) or {} )
	local localPlayer = self._hud._game:getLocalPlayer()
	
	if localPlayer and localPlayer:isPC() then
		local sim = self._hud._game.simCore

		for _, unitRaw in pairs(sim:getAllUnits()) do
		    if localPlayer:getEquippedProgram() and localPlayer:getEquippedProgram().targetRawUnit then
				if localPlayer:getEquippedProgram().shouldRawTarget then
					if localPlayer:getEquippedProgram().shouldRawTarget( localPlayer:getEquippedProgram(), self, widgets, unitRaw ) then
						self:createRawTargetButton( widgets, unitRaw )
					end
				elseif self:defaultShouldRawTarget( localPlayer:getEquippedProgram(), widgets, unitRaw ) then
				    self:createRawTargetButton( widgets, unitRaw )
			    end
			end
        end
	end
	
	-- Sweep any widgets that no longer exist.
	for i, widget in ipairs(widgets) do
		if widget.iceBreak == nil then
			self._hud._world_hud:destroyWidget( self.RAW_MF_T, table.remove( widgets ) )
		end
	end
end

function panel:updateButtonFromProgram( widget, ability, abilityOwner )
	local sim = self._hud._game.simCore
	local enabled, reason = ability:canUseAbility( sim, abilityOwner )

    local isEquipped = false
    if self._hud._state == self._hud.STATE_ABILITY_TARGET then
        isEquipped = ability == self._hud._stateData.ability
    else
        isEquipped = ability.equipped
    end

	widget:setVisible( true )
	widget.binder.powerTxt:setVisible(true)

	widget.binder.turnsTxt:setVisible(true)
	if isEquipped then
		widget.binder.powerTxt:setColor(1,1,1,1)
		widget.binder.turnsTxt:setColor(1,1,1,1)
	else
		widget.binder.powerTxt:setColor(72/255,128/255,128/255)
		widget.binder.turnsTxt:setColor(72/255,128/255,128/255)		
	end


	widget.binder.powerTxt:setText( ability:getDef():getCpuCost() or 0 )
	if ability.passive then
		widget.binder.powerTxt:setText("-")		
		widget.binder.turnsTxt:setVisible(false)
	end
	widget.binder.turnsTxt:setText(STRINGS.PROGRAMS.PWR)
	
	widget.binder.hazzardBG:setVisible(false)
	if ability.cooldown then
		if ability.cooldown > 0 then 
			widget.binder.powerTxt:setColor(140/255,1,1,1)
			widget.binder.turnsTxt:setColor(140/255,1,1,1)
		
			widget.binder.turnsTxt:setVisible(true)
			widget.binder.turnsTxt:setText(STRINGS.PROGRAMS.TURNS)
			widget.binder.powerTxt:setVisible(true)
			widget.binder.powerTxt:setText( ability.cooldown )
			widget.binder.hazzardBG:setVisible(true)
		end		
	end

	local txt = ability:getDef().huddesc 
	if ability:getDef().maxCooldown then
		txt	= txt.. "\n" .. util.sformat( STRINGS.PROGRAMS.COOLDOWN, ability:getDef().maxCooldown)
		--widget.binder.costTxt:setText(  )	
	end
	widget.binder.descTxt:setText(txt)	

	if ability:getDef():getCpuCost() then 
		for i, widget in widget.binder:forEach( "power" ) do	
			if i<=ability:getDef():getCpuCost() then
				if isEquipped then					
					widget:setColor(140/255,255/255,255/255)
				else
					widget:setColor(72/255,128/255,128/255)
				end
			else
				widget:setColor(17/255,29/255,29/255)
			end
		end
	else
		for i, widget in widget.binder:forEach( "power" ) do	
			widget:setColor(17/255,29/255,29/255)
		end
	end
	
	widget:setAlias( ability:getID() )
	widget.binder.btn:setTooltip( function() return ability:onTooltip( self._hud._screen, sim, abilityOwner ) end )
	widget.binder.btn:setDisabled( not enabled )

	self:programWidgetSetColor( widget, ability )

	if enabled then	
		widget.binder.btn.onClick = util.makeDelegate( nil, self.onClickMainframeAbility, self, ability, abilityOwner )
	end
		
	if ability:getDef().icon then
		widget.binder.img:setVisible(true)
		widget.binder.img:setImage(  ability:getDef().icon )
	end
end 

function panel:onClickMainframeAbility( ability, abilityOwner )
    if ability.equip_program and abilityOwner:getEquippedProgram() ~= ability then
        MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_MAINFRAME_SELECT_PROGRAM )
    end

	self._hud:transitionAbilityTarget( abilityOwner, abilityOwner, ability )
end

function panel:updateProgramButtons( widgetName, player, primeRefresh )
	-- Show all actionables owned by unit.
	local panel = self._panel
	local sim = self._hud._game.simCore
	local numAbilities = #sim:getPC():getAbilities()
	local MAX_EMPTY = simquery.getMaxPrograms( sim ) - self.programScrollHandler._scrollIndex

	for i, widget in panel.binder.programsPanel.binder:forEach( "program" ) do
		if i > MAX_EMPTY or i > self.programScrollHandler.maxPrograms then
			panel.binder.programsPanel.binder["empty"..i]:setVisible(false)
		else
			panel.binder.programsPanel.binder["empty"..i]:setVisible(true)
			if primeRefresh then
				panel.binder.programsPanel.binder["empty"..i]:createTransition( "activate_above")
			end		
		end
		widget:setVisible( false )
	end

	for i, ability in ipairs(player:getAbilities()) do
		local slotIdx = i - self.programScrollHandler._scrollIndex
	
		if slotIdx > 0 and slotIdx <= self.programScrollHandler.maxPrograms then
			local widget = panel.binder.programsPanel.binder["program"..slotIdx]
			panel.binder.programsPanel.binder["empty"..slotIdx]:setVisible(false)
			self:updateButtonFromProgram( widget, ability, player )
			widget:setVisible( true )
			if primeRefresh then
				widget:createTransition( "activate_above")
			end
		end
	end
end

local programScrollHandler = {
	programSlotCount = 0,
	maxPrograms = 8,
	_scrollIndex = 0,
	refreshing = true,
	
	scrollItems = function( self, idx )
		local maxIndex = self.programSlotCount - self.maxPrograms

		self._scrollIndex = math.max( 0, math.min( maxIndex, idx ))
		
		self.panel._panel.binder.programsPanel.binder.programScrollbar.binder.scrollbar:setValue( self._scrollIndex )
		
		if not self.refreshing then
			self.panel:refresh()
		end
	end,
}

function panel:refreshProgramScroller( primeRefresh )
	local sim = self._hud._game.simCore
	local pc = sim:getPC()

	if not self.programScrollHandler then
		self.programScrollHandler = util.tcopy(programScrollHandler)
		self.programScrollHandler.panel = self
		
		self._panel.binder.programsPanel.binder.programScroller._cont.handleInputEvent = function( container, ev )
			if ev.eventType == mui_defs.EVENT_MouseWheel and self.programScrollHandler.programSlotCount > self.programScrollHandler.maxPrograms then
				if ev.delta > 0 then
					self.programScrollHandler:scrollItems( self.programScrollHandler._scrollIndex - 1 )
				elseif ev.delta < 0 then
					self.programScrollHandler:scrollItems( self.programScrollHandler._scrollIndex + 1 )
				end
				return true
			end

			return false
		end

		self._panel.binder.programsPanel.binder.programScrollbar.binder.scrollbar.onValueChanged = function( scrollbar, value )
			self.programScrollHandler:scrollItems( value )
		end
	end
	
	self.programScrollHandler.refreshing = true
	
	local W, H = self._hud._screen:getResolution()
	local slots = math.max( #pc:getAbilities(), simquery.getMaxPrograms( sim ) )
		
	if slots ~= self.programScrollHandler.programSlotCount or H ~= self.programScrollHandler.resolution then
		self.programScrollHandler.resolution = H
		self.programScrollHandler.programSlotCount = slots
		H = math.min(1080, math.max(720, H))
		local max = math.floor((H - 372) / 57)
		self.programScrollHandler.maxPrograms = max
		self._panel.binder.programsPanel.binder.programScrollbar.binder.scrollbar:setSize(nil,57 * max - 5)
		self._panel.binder.programsPanel.binder.programScrollbar.binder.scrollbar:setPosition(nil,-28.5 * max + 151.5)
		self._panel.binder.programsPanel.binder.programScroller:setSize(nil,57 * max - 5)
		self._panel.binder.programsPanel.binder.programScroller:setPosition(nil,-28.5 * max + 9)
		
		local count = self.programScrollHandler.programSlotCount - self.programScrollHandler.maxPrograms
		if count > 0 then
			self._panel.binder.programsPanel.binder.programScrollbar.binder.scrollbar:setRange( 0, count )
			self._panel.binder.programsPanel.binder.programScrollbar:setVisible( true )
			self._panel.binder.programsPanel.binder.programScroller:setVisible( true )
			
			if primeRefresh then
				self._panel.binder.programsPanel.binder.programScroller:createTransition( "activate_above" )
			end
		else
			self._panel.binder.programsPanel.binder.programScrollbar.binder.scrollbar:setRange( 0, 0 )
			self._panel.binder.programsPanel.binder.programScrollbar:setVisible( false )
			self._panel.binder.programsPanel.binder.programScroller:setVisible( false )
		end
	end
	
	self:updateProgramButtons( "program", sim:getPC(), primeRefresh )
	self.programScrollHandler.refreshing = false
	
	if self._mode == 1 then--MODE_VISIBLE
		self:rawTargeting()
	end
end

panel.refreshBreakIce = upvalueUtil.find( panel.refresh, "refreshBreakIce" )
panel.updateDaemonButtons = upvalueUtil.find( panel.refresh, "updateDaemonButtons" )

local MODE_HIDDEN = 0

function panel:refresh( primeRefresh )
	self:updateDaemonButtons( "enemyAbility", self._hud._game.simCore:getNPC() )
	
	if self._mode == MODE_HIDDEN then
		self._panel.binder.programsPanel:setVisible( false )
		return
	end
    self._panel.binder.programsPanel:setVisible( true )
	
	self:refreshProgramScroller( primeRefresh )

	self:refreshBreakIce( primeRefresh )
end

function panel:onSimEvent( ev )
	oldOnSimEvent(self,ev)

	local simdefs = include( "sim/simdefs" )
	if ev.eventType == simdefs.EV_UNIT_UPDATE_ICE and ev.eventData.refreshAll then
		self:rawTargeting()
	end
end

function panel:destroyUI()
	oldDestroyUI(self)
	self._hud._world_hud:destroyWidgets( self.RAW_MF_T )
end