local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local mission_util = include( "sim/missions/mission_util" )

local mod_daemonsniff = 
	{
	name = STRINGS.ABILITIES.SCAN_DEVICE,
	-- profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_chip_daemonscan_small.png",
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_analyze_small.png",
	iconColor = util.color( 244/255, 255/255, 120/255 ),
	iconColorHover= util.color( 1,1,1 ),
    proxy = true,
	alwaysShow = true,
	HUDpriority = 3,
	noAI_PWRcost = 2,
	triggersOverwatch = true,

	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit( targetID )
		-- return abilityutil.formatToolTip(STRINGS.ABILITIES.SCAN, string.format( STRINGS.ABILITIES.TOOLTIPS.SCAN, targetUnit:getName() ))
		local targetUnit = sim:getUnit( targetID )
		
		local tooltip = nil
		
		self.iconColor= util.color( 244/255, 255/255, 120/255 )
		if self.balancedHacking then
			tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.DAEMONSNIFF_WITH_INCOG, self.noAI_PWRcost)
		else
			tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.DAEMONSNIFF)
		end
		
		local hackingskill = abilityUser:getSkillLevel("hacking")
		if (hackingskill >= 5) or targetUnit:getTraits().daemon_sniffed then
			self.iconColor= util.color( 163/255, 0/255, 0/255 )
			self.iconColorHover= util.color( 1,1,1 )
			if self.balancedHacking then
				tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.DAEMONKILL_WITH_INCOG, self.noAI_PWRcost)
			else
				tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.DAEMONKILL)
			end
		end
		
		if abilityUser:getTraits().mod_daemon_sniffing and ( abilityUser:getTraits().mod_daemon_sniffing == targetUnit:getID() ) then
			tooltip = util.sformat(STRINGS.MANUALHACKING.TOOLTIPS.STOP_DAEMONHUNT)
		end
		
		return tooltip

	end,
		
	getName = function( self, sim, unit )
		return STRINGS.MANUALHACKING.TOOLTIPS.SLOWHACKINGNAME
	end,

    getProfileIcon = function( self, sim, abilityOwner )
        return self.profile_icon
    end,

	isTarget = function( self, abilityOwner, unit, targetUnit )

		if targetUnit:getTraits().mainframe_status ~= "active" then
			return false
		end
		
		if not targetUnit:getTraits().mainframe_ice then
			return false
		end
		
		if targetUnit:getTraits().mainframe_ice then
			if not (targetUnit:getTraits().mainframe_ice > 0) then
				return false
			end
		end
		
		-- if targetUnit:getTraits().daemon_sniffed then
			-- return false
		-- end
		
		-- local sim = self:getSim()
		-- if not (targetUnit:getTraits().mainframe_program or sim:getHideDaemons()) or targetUnit:getTraits().daemon_sniffed then
			-- return false
		-- end
		
		-- if targetUnit:getTraits().mainframe_camera then
			-- return false
		-- end --commented out for now since you CAN use buster chips on cameras so why wouldn't you be able to hack them


		return true
	end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, unit )

			local x0, y0 = unit:getLocation()
			local units = {}
			for _, targetUnit in pairs(sim:getAllUnits()) do
				local x1, y1 = targetUnit:getLocation()
				if x1 and self:isTarget( abilityOwner, unit, targetUnit ) and (targetUnit:getTraits().mainframe_program or sim:getHideDaemons()) then
					local range = mathutil.dist2d( x0, y0, x1, y1 )
					
						if range <= 1 and simquery.isConnected( sim, sim:getCell( x0, y0 ), sim:getCell( x1, y1 ) ) then
							table.insert( units, targetUnit )
						end
					
				end
			end

			return targets.unitTarget( game, units, self, abilityOwner, unit )
		end,
		
	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
	
		 if abilityOwner ~= unit and abilityOwner:getUnitOwner() ~= unit then
                return false
            end
		
		local targetUnit = sim:getUnit( targetUnitID )
		if targetUnit then
			assert( self:isTarget( abilityOwner, unit, targetUnit ))
			local hackingskill = abilityOwner:getSkillLevel("hacking")

			if targetUnit:getTraits().mainframe_ice and targetUnit:getTraits().mainframe_ice <= 0 then
				return false
			end
			
			if targetUnit:getTraits().daemon_sniffed and (targetUnit:getTraits().mainframe_program == nil) then
				return false, STRINGS.MANUALHACKING.TOOLTIPS.NO_DAEMONS
			end
			
			if (targetUnit:getTraits().daemon_sniffed and (hackingskill < 5)) then
				return false, STRINGS.MANUALHACKING.TOOLTIPS.NEEDS_HACKING_5
			end			
			
			if hackingskill < 3 then
				return false, STRINGS.MANUALHACKING.TOOLTIPS.NEEDS_HACKING_3
			end		
			
			if targetUnit:getTraits().mod_beingsniffed and not (abilityOwner:getTraits().mod_daemon_sniffing and abilityOwner:getTraits().mod_daemon_sniffing == targetUnit:getID()) then
				return false, STRINGS.MANUALHACKING.TOOLTIPS.ALREADY_SNIFFING
			end			
			
			if targetUnit:getTraits().mod_beinghacked then
				return false, STRINGS.MANUALHACKING.TOOLTIPS.CANNOT_SNIFF_WHILE_HACKING
			end		
			
			if self.hackingDisabled then
				return false
			end
		end		
		
		return abilityutil.checkRequirements( abilityOwner, unit)
	end,
		
	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local mainframe = include( "sim/mainframe" )
		local userUnit = unit
		local targetUnit = sim:getUnit( targetUnitID )
		assert( targetUnit, "No target : "..tostring(targetUnitID))
		
		if userUnit:getTraits().mod_daemon_sniffing and ( userUnit:getTraits().mod_daemon_sniffing == targetUnit:getID() ) then
			--stop sniffing! cease, discontinue, desist
			userUnit:getTraits().mod_daemon_sniffing = nil
			userUnit:getTraits().data_hacking = nil
			targetUnit:getTraits().progress = nil
			targetUnit:getTraits().hacker = nil
			targetUnit:getTraits().mod_beingsniffed = nil
			userUnit:getSounds().spot = nil
			if not userUnit:getTraits().isDrone then
				sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = userUnit } )
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )					
		else
		
			local x0,y0 = userUnit:getLocation()
			local x1,y1 = targetUnit:getLocation()			
			local tempfacing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )
			-- userUnit:getTraits().data_hacking = true
			userUnit:getTraits().mod_daemon_sniffing = targetUnit:getID()
			targetUnit:getTraits().mod_beingsniffed = userUnit:getID()
			userUnit:setFacing(tempfacing)	
			if not userUnit:getTraits().isDrone then
				sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = userUnit:getID(), facing = tempfacing, sound = simdefs.SOUNDPATH_USE_CONSOLE , soundFrame = 10, useTinkerMonst3r=true } )
			end
				
			if not targetUnit:getTraits().progress then --can't remember if I still need this but... \o/
				targetUnit:getTraits().progress = 0	
				-- targetUnit:getTraits().progressMax = math.huge
			end

			if sim:isVersion("0.17.11") then
				userUnit:getTraits().isMeleeAiming = false
				userUnit:setAiming(false)	
			end		

			targetUnit:getTraits().hacker = userUnit:getID()
			userUnit:getTraits().data_hacking = targetUnit:getID()			
			userUnit:getTraits().data_hacking_spotsound = true
			userUnit:getSounds().spot = "SpySociety/Actions/monst3r_hacking"
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit })
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit })
			sim:processReactions( userUnit ) -- this will get you shot if you start hacking while being overwatched! :)
			
			--Age of Lever compatibility: reset status if anyone other than LEGIT is caught doing it
			if SCRIPT_PATHS.W93_FuncLibActive then
				simquery.LEVER_resetStatus( sim, userUnit, "LEGIT", true)
			end			

		end

	end,
	
	onSpawnAbility = function( self, sim, unit )
		self.abilityOwner = unit
		sim:addTrigger( simdefs.TRG_START_TURN, self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_END_TURN, self )
		sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )
		if (sim:getParams().difficultyOptions.MH_incognita == nil) and (sim:getParams() and sim:getParams().difficultyOptions.MH_weakenIncognita or (sim:getParams().difficultyOptions.incognita_active and ( sim:getParams().difficultyOptions.incognita_active == 1) )) then
			self.balancedHacking = true--legacy for backwards compatibility
		end
		-- if sim:getParams().difficultyOptions.MH_incognita and (sim:getParams().difficultyOptions.MH_incognita < 2) then
			-- self.balancedHacking = true
		-- end
		if sim:getParams().difficultyOptions.MH_PWRhack and (sim:getParams().difficultyOptions.MH_PWRhack > 0 ) then
			self.balancedHacking = true
			self.noAI_PWRcost = sim:getParams().difficultyOptions.MH_PWRhack
		end		
		if not (sim:getParams().difficultyOptions.icebreaker and (sim:getParams().difficultyOptions.icebreaker == true)) then 
			self.hackingDisabled = true
			self.alwaysShow = false
		end
	end,
	
	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_END_TURN, self )
		sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		self.abilityOwner = nil
		self.balancedHacking = nil
	end,
	
	onTrigger = function( self, sim, evType, evData )

		if evType == simdefs.TRG_UNIT_WARP and ( evData.unit:getTraits().mod_daemon_sniffing or evData.unit:getTraits().hacker ) then

			--warped unit could be agent OR device moving away on enemy turn
			local userUnit = nil
			local targetUnit = nil
			if evData.unit:getTraits().mod_daemon_sniffing then
				userUnit = evData.unit
				targetUnit = sim:getUnit(evData.unit:getTraits().mod_daemon_sniffing)
			elseif evData.unit:getTraits().hacker then
				userUnit = sim:getUnit(evData.unit:getTraits().hacker)
				targetUnit = evData.unit
			end
			
			-- log:write("LOG targetunit")
			-- log:write(util.stringize(targetUnit,2))
			
			if userUnit and targetUnit and userUnit:isValid() and userUnit:getLocation() and targetUnit:isValid() and targetUnit:getLocation() then
			local canReach = simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() )
			-- log:write("can reach")
			-- log:write(util.stringize(canReach,2))
			
				-- if not simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() ) then
				-- log:write("LOG targetUnit resetting")
					userUnit:getTraits().mod_daemon_sniffing = nil
					userUnit:getTraits().data_hacking = nil
					targetUnit:getTraits().progress = nil
					targetUnit:getTraits().hacker = nil
					targetUnit:getTraits().mod_beingsniffed = nil
					userUnit:getSounds().spot = nil
					-- sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = userUnit } ) -- this makes the agent refreshing smoother but makes the moving guard/drone animation stutter so... \o/
					
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
				-- end
			end
		
		elseif evType == simdefs.TRG_END_TURN then 
		-- block to reset hacking that's still happening by mistake
			if evData:isPC() then
				for _, Unit in pairs(sim:getAllUnits()) do
					if Unit:getTraits().mainframe_ice ~= nil and Unit:getTraits().mainframe_ice <= 0 then
						if Unit:getTraits().hacker then 
							Unit:getTraits().mod_beingsniffed = nil
							if not Unit:getTraits().databank then
								local hacker = sim:getUnit( Unit:getTraits().hacker ) 
								if hacker then
									hacker:getTraits().data_hacking = nil
									Unit:getTraits().hacker = nil
									hacker:getSounds().spot = nil
									if not hacker:getTraits().isDrone then
										sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = hacker } )
									end
									sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = hacker } )
								end
							end
							sim:dispatchEvent(simdefs.EV_UNIT_REFRESH, {unit = Unit } )
						end
					end
				end
			end
		elseif evType == simdefs.TRG_ICE_BROKEN then
			local hackedUnit = evData.unit 
			if hackedUnit:getTraits().mod_beingsniffed and (hackedUnit:getTraits().mainframe_ice and (hackedUnit:getTraits().mainframe_ice <= 0)) then
				local hackerman = sim:getUnit(hackedUnit:getTraits().mod_beingsniffed)
				if hackerman then
					-- log:write("LOG stop sniffing")
					hackerman:getTraits().mod_daemon_sniffing = nil
					hackerman:getSounds().spot = nil
					hackerman:getTraits().data_hacking = nil
					if not hackerman:getTraits().isDrone then
						sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = hackerman } )
					end
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= hackerman } )
				end
				hackedUnit:getTraits().mod_beingsniffed = nil
				hackedUnit:getSounds().spot = nil
				hackedUnit:getTraits().hacker = nil		
			end			
		elseif evType == simdefs.TRG_START_TURN then
			
			local userUnit = self.abilityOwner
			local mainframe = include( "sim/mainframe" )
			-- log:write("LOG: userUnit")
			-- log:write(util.stringize(userUnit,2))
		
			if userUnit:getTraits().mod_daemon_sniffing and userUnit:getTraits().data_hacking then 
				local targetID = userUnit:getTraits().data_hacking or 0
				-- log:write("LOG: targetunit")
				-- log:write(util.stringize(targetUnit,2))
				-- log:write(util.stringize(userUnit:getTraits().data_hacking,2))
				local targetUnit = sim:getUnit( targetID )
			
				if not simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() ) then
					userUnit:getTraits().mod_daemon_sniffing = nil
					userUnit:getTraits().data_hacking = nil
					targetUnit:getTraits().progress = nil
					userUnit:getSounds().spot = nil
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
					targetUnit:getTraits().hacker = nil
					targetUnit:getTraits().mod_beingsniffed = nil
					 -- stop hacking if target has moved out of range!
				end
			
				if targetUnit:getTraits().progress and evData:isPC() then

					local hackingskill = userUnit:getSkillLevel("hacking")
					-- log:write("LOG: hacking skill")
					-- log:write(util.stringize(hackingskill,2))
					local enoughPWR = true
					
					if self.balancedHacking and (sim:getPC():getCpus() < self.noAI_PWRcost) then
						enoughPWR = false
						local x1, y1 = targetUnit:getLocation()
						sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT, {
						unit = targetUnit,
						txt = util.sformat(STRINGS.UI.REASON.NOT_ENOUGH_PWR ),
						x = x1, y = y1,
						color = {r = 255/255, g = 255/255, b = 51/255, a = 1 },
					} )
					end
					
					-- Daemonsniffing execution block
					if enoughPWR and (hackingskill > 2) then
					
						sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = targetUnit, ice = targetUnit:getTraits().mainframe_ice, delta = 0} )
						local delay = 0.5
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*delay)
						if self.balancedHacking then
							sim:getPC():addCPUs( (-self.noAI_PWRcost
							), sim )
						end
						
						if (hackingskill == 5) then
							if targetUnit:getTraits().mainframe_program then
								targetUnit:getTraits().mainframe_program = nil
								sim:dispatchEvent( simdefs.EV_KILL_DAEMON, {unit = targetUnit})
								
								if targetUnit:getTraits().daemonHost then
									sim:getUnit(targetUnit:getTraits().daemonHost):killUnit(sim)
									targetUnit:getTraits().daemonHost =nil
								end
							end	
						else

							if not targetUnit:getTraits().mainframe_program then
								sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HUD_INCIDENT_NEGATIVE.path )
								mission_util.showDialog( sim, STRINGS.UI.DIALOGS.NO_DAEMON_TITLE, STRINGS.UI.DIALOGS.NO_DAEMON_BODY )
								targetUnit:getTraits().daemon_sniffed = true	
								sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = targetUnit, ice = targetUnit:getTraits().mainframe_ice, delta = 0} )	
							else
								targetUnit:getTraits().daemon_sniffed = true	
								sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = targetUnit, ice = targetUnit:getTraits().mainframe_ice, delta = 0} )		
								sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_DAEMON_REVEAL.path )
							end	
							
						end						
							

					
					end
					
					--refresh stuff
					userUnit:getTraits().mod_daemon_sniffing = nil
					userUnit:getTraits().data_hacking = nil
					targetUnit:getTraits().mod_beingsniffed = nil
					targetUnit:getTraits().hacker = nil
					userUnit:getSounds().spot = nil
					if not userUnit:getTraits().isDrone then
						sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = userUnit } )
					end
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit= userUnit } )
					targetUnit:getTraits().progress = nil
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )	
										
					-- // execution block
				end
			end
		end
	
	end,
	}

return mod_daemonsniff