local mission_scoring = include( "mission_scoring" )
local array = include( "modules/array" )
local util = include("client_util")
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include ( "sim/unitdefs" )

local function postMissionProcessing( sim, campaign )
	local player = sim:getPC()
	for agentID, deployData in pairs( player:getDeployed() ) do
		local agentUnit = deployData.escapedUnit or sim:getUnit( deployData.id )
		--take away the drones anarchy skill
		if agentUnit and agentUnit:getTraits().maa_isUmiDrone then
			for i=#agentUnit:getSkills(), 1, -1 do
				if agentUnit:getSkills()[i]:getID( ) == "anarchy" then
					table.remove( agentUnit:getSkills(), i )
				end
			end
		end
		if deployData.escapedUnit then
			local remove_childs = {}
			local add_childs = {}
			for _, childUnit in ipairs( deployData.escapedUnit:getChildren() ) do
				--reload auto-loading weapons
				if childUnit:getTraits().maa_autoreload and childUnit:getTraits().maa_autoreload > 0 then
					childUnit:getTraits().ammo = math.min( (childUnit:getTraits().ammo or 0) + childUnit:getTraits().maa_autoreload, (childUnit:getTraits().maxAmmo or 0) )
				end

				--swap monster's trader items back
				if childUnit:getTraits().maa_swap_at_end then
					table.insert( add_childs, childUnit:getTraits().maa_swap_at_end )
					table.insert( remove_childs, childUnit )
				end

				--take away sharp's skills
				if childUnit:getTraits().maa_power_allocated then
					for i=#deployData.escapedUnit:getSkills(), 1, -1 do
						if childUnit:getTraits().maa_power_allocated[ deployData.escapedUnit:getSkills()[i]:getID( ) ] then
							table.remove( deployData.escapedUnit:getSkills(), i )
						end
					end

					deployData.escapedUnit:giveSkill( "maa_sharp_reactor", (childUnit:getTraits().maa_reactor_level or 1), sim )
					table.insert( deployData.escapedUnit:getSkills(), 1, table.remove(deployData.escapedUnit:getSkills(), #deployData.escapedUnit:getSkills()) )
				end
			end
			
			for _, child in ipairs( remove_childs ) do
				deployData.escapedUnit:removeChild( child )
			end
			for _, child in ipairs( add_childs ) do
				deployData.escapedUnit:addChild( child )
			end
			
			--did the drone escape on its own? then return the deployable drone item to the agency stash
			if deployData.escapedUnit:getTraits().maa_droneCan then
				if not campaign.agency.upgrades then
					campaign.agency.upgrades = {}
				end
				table.insert( campaign.agency.upgrades, deployData.escapedUnit:getTraits().maa_droneCan:getUnitData().id ) --TODO: this is vulnerable to other stuff manipulating the item permanently (like tony's tools)
			end
		end

		--temporarily transfer the drone's agentDef from child to normal, so vanilla mission scoring can do its stuff with it
		if agentUnit and agentUnit:getTraits().maa_isUmiDrone then
			local droneDef = serverdefs.findChildAgent( campaign.agency, agentID )
			if not droneDef then --first time drone was deployed
				droneDef = deployData.agentDef
				table.insert( campaign.agency.unitDefs_childs, droneDef )
			end
			local k = array.find( campaign.agency.unitDefs_childs, droneDef )
			table.insert( campaign.agency.unitDefs, table.remove( campaign.agency.unitDefs_childs, k ) )
		end
	end
end

local function postMissionProcessing_post( sim, campaign )

end

local baseDoFinishMission = mission_scoring.DoFinishMission
function mission_scoring.DoFinishMission( sim, campaign )
	if sim.maa_add_agent_to_loot_presented then
		if not campaign.agency.maa_first_time_loot_presented then
			campaign.agency.maa_first_time_loot_presented = {}
		end
		table.insert ( campaign.agency.maa_first_time_loot_presented , sim.maa_add_agent_to_loot_presented )
	end
	
	if sim:getPC()._deployedSummons then
		util.tmerge( sim:getPC()._deployed, sim:getPC()._deployedSummons )
	end

	postMissionProcessing( sim, campaign )

	local result = baseDoFinishMission( sim, campaign )

	--transfer back Umis droneDef from normal to child
	for i, agent in ipairs( campaign.agency.unitDefs ) do
		local agentDef = unitdefs.lookupTemplate( agent.template )
		if agentDef and agentDef.traits.maa_isUmiDrone then
			table.insert( campaign.agency.unitDefs_childs, table.remove( campaign.agency.unitDefs, i ) )
			break
		end
	end
	--if the drone was killed, we have to retrieve it from unitDefPot instead
	for i, agent in ipairs( campaign.agency.unitDefsPotential ) do
		local agentDefPot = unitdefs.lookupTemplate( agent.template )
		if agentDefPot and agentDefPot.traits.maa_isUmiDrone then
			table.insert( campaign.agency.unitDefs_childs, table.remove( campaign.agency.unitDefsPotential, i ))
			break
		end
	end
	
	if campaign.agency.unitDefs_childs then
		util.tlog( campaign.agency.unitDefs, "unitDefs: ")
		util.tlog( campaign.agency.unitDefs_childs, "unitDefs_childs: ")
	end

	--local rescued_agents = {}
	--local mia_agents = {}
	--for i, agent in ipairs( sim._resultTable.agents ) do
	--	if agent.status == "RESCUED" then
	--		--find rescued agents
	--		agent.name = agent.name.template
	--		table.insert(rescued_agents, agent.name)
	--	elseif agent.status == "MIA" then
	--		--find mia agents
	--		table.insert(mia_agents, agent.name)
	--	end
	--end
	----delete the duplicates
	--for i=#sim._resultTable.agents, 1, -1 do
	--	local entry = sim._resultTable.agents[i]
	--	if array.find( rescued_agents, entry.name ) and not array.find( mia_agents, entry.name ) and entry.status ~= "RESCUED" then
	--		--delete active if rescued
	--		table.remove( sim._resultTable.agents, i )
	--	elseif array.find( rescued_agents, entry.name ) and array.find( mia_agents, entry.name ) and entry.status == "RESCUED" then
	--		--delete rescued if mia
    	--		table.remove( sim._resultTable.agents, i )
	--	end
	--end

	--remove the drone from the result table
	for i=#sim._resultTable.agents, 1, -1 do
		if sim._resultTable.agents[i].name == "maa_spider" then
			table.remove( sim._resultTable.agents, i )
		end
	end

	return result
end
