local mission_util = include( "sim/missions/mission_util" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )

local function checkAgentID(sim, agent_id)
	-- log:write("LOG: checking ID")
	-- log:write(util.stringize(agent_id,1))
	if not sim:getParams().difficultyOptions.SeparateArchivedBanters then
		if agent_id == 230 then	--SAA compatibility stuff >_>
			agent_id = 1 --decker
		elseif agent_id == 232 then
			agent_id = 2 --shalem
		elseif agent_id == 234 then
			agent_id = 3 --xu
		elseif agent_id == 233 then
			agent_id = 4 --banks
		elseif agent_id == 231 then
			agent_id = 5 --maria
		elseif agent_id == 235 then
			agent_id = 6 --nika
		elseif agent_id == 236 then
			agent_id = 7 --sharp
		elseif agent_id == 237 then
			agent_id = 8 --prism
		end
		-- log:write(util.stringize(agent_id,1))
	end
	return agent_id
	end
	
local function DroneBantersAvailable( sim, cross_script, agentList )
	local banters = {}
	for i, set in pairs(cross_script) do
		local keep = true
		for t,agent in ipairs(set.agents)do		

			if not agentList[agent] then
				keep = false
				break
			end
		end
		-------------- --MM nonsense
		if sim.MM_spawnedDroneFriend 
		-- and sim:nextRand() < 0.8 
		then
			local refit_drone = false
			for t, agent in ipairs(set.agents) do
				if agent == "MM_refitDroneFriend" then
					refit_drone = true
				end
			end
			if refit_drone == false then
				keep = false
			end
		end
		---------------
		if keep then						
			table.insert(banters,set)
		end
	end
	if #banters > 0 then
		return true
	end
	return false
end			

local function checkForRefitDrone( sim )
	for i, unit in pairs(sim:getPC():getUnits()) do
		if unit:getTraits().refitDroneFriend then
			sim.MM_spawnedDroneFriend = unit
			if unit:getTraits().customName then
				unit:getUnitData().name = unit:getTraits().customName
			end			
			break
		end
	end
end

mission_util.doAgentBanter = function(script,sim,cross_script,odds,returnIfFailed)
	log:write("LOG doing agent banter")
	local agents = sim:getCurrentPlayer():getAgents()
	
	if #agents	> 0 and sim:getParams().world ~= "omni" then

		checkForRefitDrone(sim) -- MM nonsense
		if sim.MM_spawnedDroneFriend then
			table.insert(agents,sim.MM_spawnedDroneFriend)
		end
		
		if not cross_script then
			cross_script = include('client/cross_scripts')
		end
		local cross_set = {}
		local speechQue = {}
		local agentList = {} -- the list of agent IDs on the mission
		for i,agent in ipairs(agents) do
			local agent_id = agent._unitData.agentID
			local agent_id_new = checkAgentID(sim, agent_id)	--reassign if SAA IDs apply	
			agentList[agent_id_new] = agent		
			-- agentList[agent._unitData.agentID] = agent --vanilla line	
		end
		
		local message_time = 5;
		local delay_time = 1;
		if sim:getParams().difficultyOptions.message_time then
			message_time = sim:getParams().difficultyOptions.message_time
		end 		
		if sim:getParams().difficultyOptions.delay_time then
			delay_time = sim:getParams().difficultyOptions.delay_time
		end 

		if sim:isVersion("0.17.5") then
			sim:nextRand()
			sim:nextRand()
			for i, set in pairs(cross_script) do
				local keep = true
				for t,agent in ipairs(set.agents)do		

					if not agentList[agent] then
						keep = false
						break
					end
				end
				-------------- --MM nonsense
				if sim.MM_spawnedDroneFriend and DroneBantersAvailable( sim, cross_script, agentList) 
				and sim:nextRand() < 0.9
				then
					local refit_drone = false
					for t, agent in ipairs(set.agents) do
						if agent == "MM_refitDroneFriend" then
							refit_drone = true
						end
					end
					if refit_drone == false then
						keep = false
					end
				end
				---------------
				if keep then						
					table.insert(cross_set,set)
				end
			end
		
			local dialogueSet = nil
			local agency = sim:getParams().agency

			while #cross_set > 0 and dialogueSet == nil do -- find a set not already used
				local rand = math.floor(sim:nextRand()*#cross_set)+1
				dialogueSet = cross_set[rand]

				-- this is not longer needed, but is left in place so that it preserves savegames that need it to get same seed result.
				for i,crossId in ipairs(agency.crossIds or {})do
					if dialogueSet and crossId == dialogueSet.id then -- if dialogueSet is not nil, and if crossId equals the id from dialogueSet (added below), remove this banter from banter pool!
						dialogueSet = nil
						table.remove(cross_set,rand)					
					end
				end				
			end
			if dialogueSet and sim:nextRand() < (odds or sim:getParams().difficultyOptions.banter_mod or sim:getParams().difficultyOptions.banter or 0.2) then	
				
				for i,event in ipairs(dialogueSet.dialogue) do				
					agentDef = {}
					for i,agent in ipairs(agents) do
										
						local agent_id_new = checkAgentID(sim, agent._unitData.agentID) -- SAA compatibility
						if agent_id_new == event[1] then
							agentDef = agent._unitData
						end
					end
					local enemy = false
					if (i % 2 == 0)then
						enemy = true
					end
					local anim = agentDef.profile_anim
					local build = agentDef.profile_anim
					if agentDef.profile_build then
						build = agentDef.profile_build
					end

			        local speech={ 
			            {
			                text = agentDef.speech.CROSS[event[2]] or (type(event[2]) == "string" and event[2]) or "",
			                anim = anim,
			                build = build,
			                name = util.toupper(agentDef.name),
			                timing = message_time,
			                enemy = enemy,
			            } 
			        }				
					table.insert(speechQue,speech)

					--makes lines for convo
					
					-- code to prevent banters from repeating during campaign ------------------------------------
					-- if agency.crossIds == nil then
						-- agency.crossIds  = {}
					-- end
					-- if dialogueSet.id then 
						-- table.insert (agency.crossIds, dialogueSet.id )
					-- end
					-- ^: old vanilla code, causes bugs on Rewind
					--second half of it was moved to doFinishMission in modinit
					
					if dialogueSet.id then					
					sim.usedbanterID = dialogueSet.id
					end
					-------------------------------------------------------------
				end			
			end
		end
		local selection = math.floor(sim:nextRand()*#agents)+1
		local agentDef = agents[selection]:getUnitData()
		local speechSet = agents[selection]:getSpeech()
		if speechSet == nil then
			log:write("ERROR: no speechdef found for "..util.stringize(agents[selection],2))
			return
		end
		local text = speechSet.START[math.floor(sim:nextRand()*#speechSet.START)+1]		

		if #speechQue == 0 and not returnIfFailed then

   			local anim = agentDef.profile_anim
			local build = agentDef.profile_anim
			if agentDef.profile_build then
				build = agentDef.profile_build
			end

	        local speech={ 
	            {
	                text = text or "",
	                anim = anim,
	                build = build,
	                name = util.toupper(agentDef.name),
	                timing = message_time,
	            } 
	        }

			
	        table.insert(speechQue,speech)
			
    		end
		
		if #speechQue == 0 then
			return
		end

		
		script:queue( delay_time*cdefs.SECONDS ) -- delay after mission's start  before banter starts

    		for i,que in ipairs(speechQue)do

	    		if que[1].enemy then
			    	script:queue( { body=que[1].text, header=que[1].name, type="enemyMessage", 
						profileAnim=que[1].anim,
						profileBuild=que[1].build,
					} )	
					script:queue( message_time*cdefs.SECONDS )					
					script:queue( { type="clearEnemyMessage" } )
			else
	        		script:queue( { script=que, type="newOperatorMessage" } )        		
	        	end        	
		end
	end
end