local util = include( "modules/util" )
local array = include("modules/array")
local simdefs = include( "sim/simdefs" )
local mission_util = include( "sim/missions/mission_util" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local store = include( "sim/units/store" )
local mainframe_abilities = include( "sim/abilities/mainframe_abilities" )
local serverdefs = include( "modules/serverdefs" )
local agentdefs = include("sim/unitdefs/agentdefs")
local inventory = include("sim/inventory")

local BOUNTY_BASE_VALUE = 550
local BOUNTY_ARMOR_VALUE = 100
local BOUNTY_VARIATION_VALUE = 100

local FOUND_BOUNTY =
{
	trigger = simdefs.TRG_UNIT_APPEARED,
	fn = function( sim, evData )
		local bounty = evData.unit
		local seerUnit = sim:getUnit(evData.seerID)
		if bounty:getTraits().maa_bounty and seerUnit and seerUnit:isPC() then
			return bounty
		else
			return nil
		end
	end,
}

local BOUNTY_KILLED =
{
	trigger = simdefs.TRG_UNIT_KILLED,
	fn = function( sim, evData )
		if evData.unit:getTraits().maa_bounty then
			evData.corpse:getTraits().maa_bounty = evData.unit:getTraits().maa_bounty
			return evData.corpse
		else
			return nil
		end
	end,
}

local function updateTab( bountyUnit )
	if bountyUnit and bountyUnit:getTraits().iscorpse then
		bountyUnit:createTab( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_HEADER, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_DEAD_BODY )
	elseif bountyUnit then
		bountyUnit:createTab( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_HEADER, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_BODY )
	end
end

local function refreshUnitTab( script, sim )
	while true do
		local ev, triggerData = script:waitFor( mission_util.UNIT_WARP )
		if triggerData.unit:getTraits().maa_bounty then
			triggerData.unit:destroyTab()
		end
	end
end

local function appendCorpseTooltip( corpse )
	--why here? what do I know.
	local baseOnWorldTooltip = corpse:getUnitData().onWorldTooltip
	corpse:getUnitData().onWorldTooltip = function( tooltip, unit )
		baseOnWorldTooltip( tooltip, unit )
		if unit and unit:getTraits().maa_bounty then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_GUARD_TOOL, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_GUARD_TOOL_DESC_2, unit:getTraits().maa_bounty ), "gui/icons/arrow_small.png" )
		end
	end
end

local function bountyHunt( script, sim )
	--find the bounty
	sim:addObjective( STRINGS.MORE_ARCHIVE_AGENTS.MISSIONS.BOUNTY_OBJECTIVE_1, "find_bounty" )
	local _, bounty = script:waitFor( FOUND_BOUNTY )
	updateTab( bounty )
	script:addHook( refreshUnitTab )
	sim:removeObjective( "find_bounty" )
	--kill the bounty
	sim:addObjective( STRINGS.MORE_ARCHIVE_AGENTS.MISSIONS.BOUNTY_OBJECTIVE_2, "kill_bounty" )
	local _, corpse = script:waitFor( BOUNTY_KILLED )
	corpse._sim = sim
	appendCorpseTooltip( corpse )
	updateTab( corpse )
	sim:removeObjective( "kill_bounty" )
	--validate the bounty kill
	sim:addObjective( STRINGS.MORE_ARCHIVE_AGENTS.MISSIONS.BOUNTY_OBJECTIVE_3, "validate_bounty" )
	script:waitFor( mission_util.PC_USED_ABILITY( "maa_validate_bounty_kill") )
	sim:removeObjective( "validate_bounty" )
	corpse:destroyTab()
	script:removeHook( refreshUnitTab )
end

local baseCMInit = mission_util.campaign_mission.init
function mission_util.campaign_mission:init( scriptMgr, sim, finalMission )
	if sim:getParams().difficultyOptions.maa_loaded then
		--monster/ central if final mission
		if self.finalMission then
			local specialAgents = {}
			local hasCentral = false
			local hasMonst3r = false
			for i, agent in pairs( sim:getPC():getAgents() ) do
				if agent:getTraits().central then
					hasCentral = true
				end
				if agent:getTraits().monst3r then
					hasMonst3r = true
				end
			end

			if not hasCentral and sim:getParams().agency.MAA_CENTRAL and agentdefs[sim:getParams().agency.MAA_CENTRAL] then
				table.insert( specialAgents, serverdefs.createAgent( sim:getParams().agency.MAA_CENTRAL, util.tcopy( agentdefs[sim:getParams().agency.MAA_CENTRAL].upgrades ) ) )
			end

			if not hasMonst3r and sim:getParams().agency.MAA_MONSTER and agentdefs[sim:getParams().agency.MAA_MONSTER] then
				table.insert( specialAgents, serverdefs.createAgent( sim:getParams().agency.MAA_MONSTER, util.tcopy( agentdefs[sim:getParams().agency.MAA_MONSTER].upgrades ) ) )
			end

			if #specialAgents > 0 then
				sim:getPC():reserveUnits( specialAgents )
				sim:getPC():deployUnits( sim, specialAgents )
			end
		end
	end

	baseCMInit( self, scriptMgr, sim, finalMission )


	local guard_potentials = {}
	local shalem_c = false
	local shalem_d = false
	local virus_injector = nil

	if sim:getParams().difficultyOptions.maa_loaded then
		for _, unit in pairs( sim:getAllUnits() ) do
			--find notable units
			if unit:getTraits().isGuard and not unit:getTraits().isDrone then
				table.insert( guard_potentials, unit )
			end
			if unit:isPC() and unit:getTraits().maa_bounty_hunter then
				shalem_c = true
			end
			if unit:getTraits().maa_virus_injector then
				virus_injector = unit
			end
			if unit:getTraits().maa_can_find_weapon_mods then
				shalem_d = true
			end

			--add monster 1's secret shop to all nanofabs
			if unit:getTraits().storeType and ( unit:getTraits().storeType == "standard" or unit:getTraits().storeType == "large" ) then
				local template = unitdefs.lookupTemplate("maa_monster_shop")
				local new_secret_shop = simfactory.createUnit( template, sim )
				--unit:addChild( new_secret_shop )
				sim:spawnUnit( new_secret_shop )
				new_secret_shop:onWarp( sim, nil, nil ) --fake onWarp call to create the items in the shop
				new_secret_shop:getTraits().storeType = nil
				--pick augment or weapon and remove it, so we only add 1 augment/ weapon and 1 item
				if sim:nextRand() < 0.5 then
					new_secret_shop.augments = {}
				else
					new_secret_shop.weapons = {}
				end

				unit:giveAbility( "maa_showSecretStore" )
				local ability = unit:hasAbility( "maa_showSecretStore" )
				if ability then
					ability.secretShop = new_secret_shop
				end
			end

			--monster 2's tradeables
			if unit:getTraits().maa_replace_item_type then
				local trader = unit:getUnitOwner()
				if trader then
					local can_swap = true
					if unit:getTraits().restrictedUse then
						local legal_user = false
						local userAgentID = trader:getUnitData().agentID
						if not userAgentID then
							can_swap = false
						end

						for _, user in ipairs( unit:getTraits().restrictedUse ) do
							if user.agentID == userAgentID then
								legal_user = true
								break
							end
						end

						if not legal_user then
							can_swap = false
						end
					end
					if can_swap then
						--go through the shoplist and pick something at random. Not restricted by soldAfter or floorWeight. But only pick something monster can use with his skills.
						local itemList = util.tdupe( store.STORE_ITEM[ (unit:getTraits().maa_replace_item_type or "itemList") ] )

						--Take out items that should be overwriten by mods and items monster can't use
						for i=#itemList,1,-1 do
							if itemList[i].markedForDeletion then
								table.remove( itemList, i )
							elseif not itemList[i].floorWeight then --only items that can also appear in shops
								table.remove( itemList, i )
							elseif itemList[i].traits and ( itemList[i].traits.disposable or itemList[i].traits.usesLeft ) then --no items that destroy themself after use
								table.remove( itemList, i )
							elseif array.find( itemList[i].abilities, "throw" ) and not ( itemList[i].traits and not itemList[i].traits.disposable ) then --same as above, since grenade are natively disposable
								table.remove( itemList, i )
							elseif itemList[i].traits and itemList[i].traits.ammo_clip and not itemList[i].traits.disposable then --same as above for ammo clips
								table.remove( itemList, i )
							elseif itemList[i].requirements then
								for skill, level in pairs( itemList[i].requirements ) do
									if not trader:hasSkill(skill, level) and not trader:getTraits().useAnyItem then
										table.remove( itemList, i )
										break
									end
								end
							end
						end

						local selectedItem = itemList[ sim:nextRand(1, #itemList ) ]
						local gadget = simfactory.createUnit( selectedItem, sim )

						gadget:getTraits().maa_swap_at_end = unit
						sim:spawnUnit( gadget )
						trader:addChild( gadget )
						inventory.autoEquip( unit )

						trader:removeChild( unit )
					end
				end
			end
		end

		--pick one
		local bounty = nil
		local weapon_mod_target = nil
		if #guard_potentials > 0 then
			bounty = guard_potentials[ sim:nextRand(#guard_potentials) ]
			weapon_mod_target = guard_potentials[ sim:nextRand(#guard_potentials) ]
		end

		--if shalem c is present, add the bounty
		if shalem_c and bounty then
			bounty:getTraits().maa_bounty = math.floor( BOUNTY_BASE_VALUE + (bounty:getTraits().armor or 0) * BOUNTY_ARMOR_VALUE + sim:nextRand() * BOUNTY_VARIATION_VALUE - BOUNTY_VARIATION_VALUE / 2 )
			log:write("MAA: Added bounty to mission with value CR: " .. tostring(bounty:getTraits().maa_bounty) )
			bounty:getTraits().cleanup = nil
			scriptMgr:addHook( "BOUNTY_HUNT", bountyHunt, nil, nil )
		end
		--if shalem d is present, add a random mod on one human guard
		if shalem_d and weapon_mod_target then
			local weapon_mod_list = {}
			for itemID, item in pairs( unitdefs.tool_templates ) do
				if item.traits and item.traits.maa_mod_slot then
					table.insert( weapon_mod_list, itemID )
				end
			end
			
			--pick one
			local selected_weapon_mod = weapon_mod_list[ sim:nextRand(1, #weapon_mod_list ) ]
			local template = unitdefs.lookupTemplate( selected_weapon_mod )
			local weapon_mod_unit = simfactory.createUnit( template, sim )
			sim:spawnUnit( weapon_mod_unit )
			weapon_mod_target:addChild( weapon_mod_unit )
		end

		--add derek 1's craftable programs --FIXME: This should be in ability:onSpawn. This should be a problem when rescuing from dc right now-
		if virus_injector then
			-- local player = sim:getPC()
			-- local virusUser = virus_injector:getUnitOwner()
			--
			-- local progList = util.tcopy( store.STORE_ITEM.progList )
			-- progList:addList( store.STORE_ITEM.progList_17_5 )
			-- local nobreakList = util.tcopy( store.STORE_ITEM.noBreakerProgList )
			-- nobreakList:addList( store.STORE_ITEM.noBreakerProgList_17_5 )
			--
			-- local progList_unweighted = {} -- I don't know how to remove elements out of a weighted list without making a big mess
			-- for i = 1, #progList, 2 do
			-- 	progList_unweighted[ progList[i+1] ] = progList[i]
			-- end
			-- for i = 1, #nobreakList, 2 do
			-- 	progList_unweighted[ nobreakList[i+1] ] = nobreakList[i]
			-- end
			--
			-- for program, weight in pairs( progList_unweighted ) do
			-- 	if player:hasMainframeAbility( program ) then
			-- 		progList_unweighted[program] = nil
			-- 	end
			-- end
			--
			-- progList = util.weighted_list( progList_unweighted )
			-- virus_injector.programs = {}
			-- for i=1, 4, 1 do
			-- 	local w = sim:nextRand(1, progList:getTotalWeight() )
			-- 	local programName = progList:removeChoice( w )
			-- 	local programData = createProgramUnitData( programName, sim )
			-- 	local programUnit = simfactory.createUnit( programData, sim )
			-- 	table.insert( virus_injector.programs, programUnit )
			-- end
		end
	end
end
