local function init( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" ) 	
	--KLEIResourceMgr.MountPackage( dataPath .. "/images.kwad", "data" ) 
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
	
	include( scriptPath .."/skilldefs" )
	include( scriptPath .."/wizard_spellscroll" )
	include( scriptPath .."/state-upgrade-screen" )
	include( scriptPath .."/targeting" )
	include( scriptPath .."/abilityutil" )
	include( scriptPath .."/simquery" )
	include( scriptPath .."/actions" )
	include( scriptPath .."/installAugment" )
	package.loaded["wizard-spell-preparation-dialog"] = include( scriptPath .."/spell-preparation-dialog" )
	package.loaded["wizard-spell-learn-dialog"] = include( scriptPath .."/spell-learn-dialog" )
	
  	modApi:addGenerationOption("wizard", STRINGS.WIZARD.OPTIONS.WIZARD, STRINGS.WIZARD.OPTIONS.WIZARD_TIP, { noUpdate = true } )
  	modApi:addGenerationOption("wizard_2", STRINGS.WIZARD.OPTIONS.WIZARD_2, STRINGS.WIZARD.OPTIONS.WIZARD_2_TIP, { noUpdate = true } )
end

local spells = {
	-- Level 1

	"fog_cloud",
	"magic_missile",
	"true_strike",
	"silent_image",
	"hypnotic_gaze",
	"enfeeblement",
	"enhance_ability",
	"knock",
	"faerie_fire",
	"mage_armor",
	--"mage_hand",
	"counterspell",
	
	-- Level 2
	
	"haste",
	"misty_step",
	"mind_spike",
	"locate_object",
	"shatter",
	"mirror_image",
	
	-- Level 3
	
	"animate_dead",
	"fireball",
	"nondetection",
	"clairvoyance",
	"hypnotic_pattern",
	"courier",
	"phantasmal_killer",
}

local function load( modApi, options, params, mod_options )
  	local scriptPath = modApi:getScriptPath()
	local simquery = include("sim/simquery")
	simquery.removeSimulacrumMovement()
	
	if options["wizard"].enabled or (options["wizard_2"] and options["wizard_2"].enabled) then
		local commondefs = include( scriptPath .. "/commondefs" )
		modApi:addTooltipDef( commondefs )
		
		modApi:addAbilityDef( "wizard_arcane_focus", scriptPath .."/wizard_arcane_focus" )
		modApi:addAbilityDef( "wizard_spellbook", scriptPath .."/wizard_spellbook" )
		modApi:addAbilityDef( "wizard_book_recharge", scriptPath .."/wizard_book_recharge" )
		modApi:addAbilityDef( "wizard_book_swap", scriptPath .."/wizard_book_swap" )
		modApi:addAbilityDef( "wizard_archived", scriptPath .."/wizard_archived" )
		modApi:addAbilityDef( "wizard_crystal", scriptPath .."/wizard_crystal" )
		
		for i, spell in ipairs( spells ) do
			modApi:addAbilityDef( spell, scriptPath .."/spells/" .. spell )
		end
		
		modApi:insertUIElements( include( scriptPath.."/screens/upgrade_screen" ) )
		modApi:insertUIElements( include( scriptPath.."/screens/hud" ) )
		modApi:addNewUIScreen( "spell-preparation-modal.lua", scriptPath.."/screens/spell-preparation" )
		modApi:addNewUIScreen( "spell-learn-modal.lua", scriptPath.."/screens/spell-learn" )
		modApi:addNewUIScreen( "locate-object.lua", scriptPath.."/screens/locate-object" )
		
		local agentdefs = include( scriptPath .. "/agentdefs" )
		
		local animdefs = include( scriptPath .. "/animdefs" )
		for name, animDef in pairs(animdefs) do
			modApi:addAnimDef( name, animDef )
		end

		local itemdefs = include( scriptPath .. "/itemdefs" )
		for name, itemDef in pairs(itemdefs) do
			modApi:addItemDef( name, itemDef )
		end
		
		for name, agentDef in pairs(agentdefs) do
			modApi:addAgentDef( name, agentDef )
		end
		
		if options["wizard"].enabled then 
			modApi:addRescueAgent( "wizard_agent", "wizard_agent", {"augment_wizard", "item_wiz_spellbook"} )
			modApi:addAgentLoadout( "wizard_agent", "wizard_agent" )
		end
		if options["wizard_2"] and options["wizard_2"].enabled then
			modApi:addRescueAgent( "wizard_agent", "wizard_agent_archived", {"augment_wizard_archived"} )
			modApi:addAgentLoadout( "wizard_agent", "wizard_agent_archived" )
		end
		
		--local banterdefs = include( scriptPath.."/banterdefs" )
		--for i, banter in pairs( banterdefs ) do
		--	modApi:addBanter( banter )
		--end	
		
		local simdefs = include("sim/simdefs")
		if rawget(simdefs,"CHEATS") then
			local cheats = include(scriptPath.."/cheats")
			for i, cheat in ipairs(cheats) do
				table.insert(simdefs.CHEATS,cheat())
			end
		end
		
		for eventType, handlerFile in pairs( include( scriptPath.."/viz/viz" ) ) do
			modApi:addVizEvHandler( eventType, include( scriptPath.."/viz/"..handlerFile ))
		end
	end
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "WIZARD", DLC_STRINGS )
end

return {
	init = init,
	initStrings = initStrings,
	load = load,
}