--- Helper functions
local function handleEmittedSound( boardRig, x0, y0, sound, altVisTiles )
	local sim = boardRig:getSim()
	local closestUnit, closestRange = nil, math.huge
	local simquery = include( "sim/simquery" )
	local hilite_radius = include( "gameplay/hilite_radius" )
	local sound_ring_rig = include( "gameplay/sound_ring_rig" )

	if boardRig:getLocalPlayer() then
		closestUnit, closestRange = simquery.findClosestUnit( boardRig:getLocalPlayer():getUnits(), x0, y0, simquery.canHear )
	end

	local hasListener = closestUnit ~= nil and closestRange <= sound.range
	local canSeeSource = boardRig:canPlayerSee( x0, y0 ) 

	if altVisTiles and not canSeeSource then
		for i,tile in ipairs(altVisTiles) do
			if boardRig:canPlayerSee( tile.x, tile.y ) then
				canSeeSource = true
				break
			end
		end
	end

	if sound.path and (hasListener or canSeeSource) then
		boardRig._world_sounds:playSound( sound.path, nil, x0, y0 )
	end

	if sound.range > 0 then
		boardRig._world_sounds:playRattles( x0, y0, sound.range )
		if hasListener or canSeeSource then
        		local soundRingRig = sound_ring_rig( boardRig, x0, y0, sound.range, 0.5, {140/255, 255/255, 255/255, 0.3} )
	    		table.insert( boardRig._dynamicRigs, soundRingRig )
            		if sound.hiliteCells then
                		local hilite = hilite_radius( x0, y0, sound.range )
                		hilite:setRate( 1, 40, 0.2 )
                		hilite:setColor( { 0.05, 0.05, 0.05, 0.05 } )
                		boardRig:hiliteRadius( hilite )
            		end
		end
	end
end

local overloadVisHandler = function(ev, eventType, eventData, boardRig, hud, thread)
	local hilite_radius = include( "gameplay/hilite_radius" )
	local mathutil = include( "modules/mathutil" )
	local util = include( "modules/util" )

	if eventData.mainframe then
		boardRig._game.hud:showMainframe()
	end
	boardRig._game:cameraPanToCell( eventData.x, eventData.y )

	handleEmittedSound( boardRig, eventData.x, eventData.y, {range=eventData.range, path="SpySociety/Actions/EMP_explo"} )

	local RING_PERIOD = 6
	local hilite = hilite_radius( eventData.x, eventData.y, eventData.range )
	hilite:setRate( RING_PERIOD )
	if eventData.cells then
		hilite:setCells(eventData.cells)
	end
	boardRig:hiliteRadius( hilite )
	
	local wx, wy = boardRig:cellToWorld( eventData.x, eventData.y )
	boardRig._game.fxmgr:addAnimFx( { kanim = "fx/emp_explosion", symbol = "character", anim = "active", x = wx, y = wy, facingMask = KLEIAnim.FACING_W } )

	if eventData.units then
		local targetUnits = util.tdupe(eventData.units)
		table.sort(targetUnits,
            		function(u1, u2) return mathutil.distSqr2d( eventData.x, eventData.y, u1:getLocation() ) <
                                    mathutil.distSqr2d( eventData.x, eventData.y, u2:getLocation() ) end )
		local lastRange = 0
		for i, unit in ipairs(targetUnits) do
			local range = math.floor( mathutil.dist2d( eventData.x, eventData.y, unit:getLocation() ) )
			if range > lastRange then
				boardRig:wait( (range - lastRange) * RING_PERIOD )
				lastRange = range
			end
			if eventData.fx then
    				local wx1, wy1 = boardRig:cellToWorld( unit:getLocation() )
				boardRig._game.fxmgr:addAnimFx( { kanim = eventData.fx.kanim, symbol = eventData.fx.symbol, anim = eventData.fx.anim, x = wx1, y = wy1, facingMask = KLEIAnim.FACING_W } )
			else
				boardRig:getUnitRig( unit:getID() ):performEMP()
			end
		end
	end
end
--- 

local function earlyInit( modApi )
	modApi.requirements = { "Contingency Plan", "Advanced Guard Protocol", "Mods Combo by Shirsh" }
end

local function init( modApi )
    	function mod_manager:findModByName( name )
            for i, modData in ipairs(self.mods) do
                if name and mod_manager:getModName( modData.id ) == name then
                        return modData
                end
            end
    	end

	local scriptPath = modApi:getScriptPath()
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})
    	SCRIPT_PATHS.W93_FuncLibActive = true

	modApi:addGenerationOption("enable_function_changes", STRINGS.FUNCLIB.OPTIONS.ENABLE_FUNCTION_CHANGES, STRINGS.FUNCLIB.OPTIONS.ENABLE_FUNCTION_CHANGES_TIP, {noUpdate = true})

	include( scriptPath .. "/abilityutil" )
	include( scriptPath .. "/aiplayer" )
	include( scriptPath .. "/boardrig" )
	include( scriptPath .. "/color")
	include( scriptPath .. "/combat_panel" )
	include( scriptPath .. "/doorrig2" )
	include( scriptPath .. "/inventory" )
	include( scriptPath .. "/line-of-sight" )
	include( scriptPath .. "/mainframe" )
	include( scriptPath .. "/mainframe_panel" )
	include( scriptPath .. "/mission_util" )
	include( scriptPath .. "/pcplayer" )
	include( scriptPath .. "/senses" )
	include( scriptPath .. "/shop_panel" )
	include( scriptPath .. "/simactions" )
	include( scriptPath .. "/simcamera" )
	include( scriptPath .. "/simdrone" )
	include( scriptPath .. "/simengine" )
	include( scriptPath .. "/simquery" )
	include( scriptPath .. "/simstore" )
	include( scriptPath .. "/simtrap" )
	include( scriptPath .. "/simunit" )
	include( scriptPath .. "/sound_ring_rig" )
	include( scriptPath .. "/targeting" )
	include( scriptPath .. "/tooltip_breakice" )
	include( scriptPath .. "/unitrig" )
	include( scriptPath .. "/wallrig2" )
	include( scriptPath .. "/weighted_list" )

	local mission_scoring = include("mission_scoring")
	local mission_scoring_alt = include(scriptPath .. "/mission_scoring")
	mission_scoring.DoFinishMission = mission_scoring_alt.DoFinishMission

	local simparams = include( "sim/simparams" )
	simparams.createParams = include( scriptPath .. "/simparams" )

	modApi:addAbilityDef( "activate_locked_console", scriptPath .."/abilities/activate_locked_console" )
	modApi:addAbilityDef( "alarmCPU", scriptPath .."/abilities/alarmCPU" )
	modApi:addAbilityDef( "breakDoor", scriptPath .."/abilities/breakDoor" )
	modApi:addAbilityDef( "compile_software", scriptPath .."/abilities/compile_software" )
	modApi:addAbilityDef( "deployable", scriptPath .."/abilities/deployable" )
	modApi:addAbilityDef( "disarmtrap", scriptPath .."/abilities/disarmtrap" )
	modApi:addAbilityDef( "disguise", scriptPath .."/abilities/disguise" )
	modApi:addAbilityDef( "doorMechanism", scriptPath .."/abilities/doorMechanism" )
	modApi:addAbilityDef( "equippable", scriptPath .."/abilities/equippable" )
	modApi:addAbilityDef( "hostage_rescuable", scriptPath .."/abilities/hostage_rescuable" )
	modApi:addAbilityDef( "icebreak", scriptPath .."/abilities/icebreak" )
	modApi:addAbilityDef( "installAugment", scriptPath .."/abilities/installAugment" )
	modApi:addAbilityDef( "jackin", scriptPath .."/abilities/jackin" )
	modApi:addAbilityDef( "jackin_charge", scriptPath .."/abilities/jackin_charge" )
	modApi:addAbilityDef( "jackin_root_console", scriptPath .."/abilities/jackin_root_console" )
	modApi:addAbilityDef( "manualHack", scriptPath .."/abilities/manualHack" )
	modApi:addAbilityDef( "melee", scriptPath .."/abilities/melee" )
	modApi:addAbilityDef( "moveBody", scriptPath .."/abilities/moveBody" )
	modApi:addAbilityDef( "observePath", scriptPath .."/abilities/observePath" )
	modApi:addAbilityDef( "open_detention_cells", scriptPath .."/abilities/open_detention_cells" )
	modApi:addAbilityDef( "overwatch", scriptPath .."/abilities/overwatch" )
	modApi:addAbilityDef( "overwatchMelee", scriptPath .."/abilities/overwatchMelee" )
	modApi:addAbilityDef( "paralyze", scriptPath .."/abilities/paralyze" )
	modApi:addAbilityDef( "peek", scriptPath .."/abilities/peek" )
	modApi:addAbilityDef( "prime_emp", scriptPath .."/abilities/prime_emp" )
	modApi:addAbilityDef( "recharge", scriptPath .."/abilities/recharge" )
	modApi:addAbilityDef( "reload", scriptPath .."/abilities/reload" )
	modApi:addAbilityDef( "scandevice", scriptPath .."/abilities/scandevice" )
	modApi:addAbilityDef( "shootSingle", scriptPath .."/abilities/shootSingle" )
	modApi:addAbilityDef( "showItemStore", scriptPath .."/abilities/showItemStore" )
	modApi:addAbilityDef( "sprint", scriptPath .."/abilities/sprint" )
	modApi:addAbilityDef( "stealCredits", scriptPath .."/abilities/stealCredits" )
	modApi:addAbilityDef( "throw", scriptPath .."/abilities/throw" )
	modApi:addAbilityDef( "usable", scriptPath .."/abilities/usable" )
	modApi:addAbilityDef( "useAugmentMachine", scriptPath .."/abilities/useAugmentMachine" )
	modApi:addAbilityDef( "useInvisiCloak", scriptPath .."/abilities/useInvisiCloak" )
	modApi:addAbilityDef( "use_aggression", scriptPath .."/abilities/use_aggression" )
	modApi:addAbilityDef( "use_injection", scriptPath .."/abilities/use_injection" )
	modApi:addAbilityDef( "use_medgel", scriptPath .."/abilities/use_medgel" )
	modApi:addAbilityDef( "use_stim", scriptPath .."/abilities/use_stim" )

	local mainframe_abilities = include("sim/abilities/mainframe_abilities")
	mainframe_abilities.lockpick_1.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.lockpick_2.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.wrench_2.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.wrench_3.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.wrench_4.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.wrench_5.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.hammer.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.dagger.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.brimstone.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.dagger_2.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.rapier.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.dataBlast.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.flare.tipdesc = STRINGS.FUNCLIB.UI.FLARE_TIPDESC
	mainframe_abilities.feast.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
	mainframe_abilities.mercenary.tipdesc = STRINGS.FUNCLIB.UI.DEFAULT_PROGRAM_TIPDESC
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "FUNCLIB", DLC_STRINGS )
end

local function load( modApi, options, params, mod_options )
	local scriptPath = modApi:getScriptPath()

	local commondefs = include( scriptPath .. "/commondefs" )
	local simdefs = include( "sim/simdefs" )
	modApi:addTooltipDef( commondefs )
	modApi:addVizEvHandler( simdefs.EV_OVERLOAD_VIZ, overloadVisHandler, true )

	include( scriptPath .. "/simemppack")
	include( scriptPath .. "/simgrenade" )
	include( scriptPath .. "/mainframe_abilities" )
	funcLibUpdatePrograms()

	if options["enable_function_changes"] and options["enable_function_changes"].enabled then
		SCRIPT_PATHS.W93_FuncLibChanges = true
	else
		SCRIPT_PATHS.W93_FuncLibChanges = false
	end

	if params and mod_manager:findModByName( "New Items And Augments" ) then
		params.W93_FuncLib_Tooltips = true
	end
end

local function earlyLoad( modApi )
	SCRIPT_PATHS.W93_FuncLibActive = true
end

local function earlyUnload( modApi )
	SCRIPT_PATHS.W93_FuncLibActive = false
end

return
{
	init = init,
	initStrings = initStrings,
	earlyInit = earlyInit,
	earlyLoad = earlyLoad,
	earlyUnload = earlyUnload,
	load = load,
}
