-- Include files to be used throughout this file.
-- Don't eager-load sim- or client-heavy things though, those may break translation.
-- local util = include("modules/util")

-- Private static variables can be used by load/unload to remember what has been done.
local ThisModLoaded = false

local function earlyInit(modApi)
    modApi.requirements = {
        "Contingency Plan",
        "Sim Constructor",
        "Function Library"
    }
end

local function init(modApi)
    do -- dependency check
        local cdefs = include("client_defs")
        assert(cdefs.ActionToolkitLoaded, "ERROR - MOD-LOAD FAILED: Heat Signature depends on Action Toolkit")
    end

    local dataPath = modApi:getDataPath()
    KLEIResourceMgr.MountPackage(dataPath .. "/gui.kwad", "data")
    KLEIResourceMgr.MountPackage(dataPath .. "/anims.kwad", "data")

    modApi:addGenerationOption(
        "onfile_fiasco",
        STRINGS.HEATSIGNATURE.OPTIONS.SADERFIASCO,
        STRINGS.HEATSIGNATURE.OPTIONS.SADERFIASCO_TIP,
        {noUpdate = true}
    )
    modApi:addGenerationOption(
        "archive_fiasco",
        STRINGS.HEATSIGNATURE.OPTIONS.SADERFIASCO_ARCHIVE,
        STRINGS.HEATSIGNATURE.OPTIONS.SADERFIASCO_ARCHIVE_TIP,
        {noUpdate = true}
    )
    modApi:addGenerationOption(
        "glitchtrap_store",
        STRINGS.HEATSIGNATURE.OPTIONS.GLITCHTRAP_STORE,
        STRINGS.HEATSIGNATURE.OPTIONS.GLITCHTRAP_STORE_TIP,
        {noUpdate = true}
    )
    -- modApi:addGenerationOption(
    --     "empbeam_store",
    --     STRINGS.HEATSIGNATURE.OPTIONS.EMPBEAM_STORE,
    --     STRINGS.HEATSIGNATURE.OPTIONS.EMPBEAM_STORE_TIP,
    --     {noUpdate = true}
    -- )

    do  -- EAA/Goose compatibility
        local simquery = include("sim/simquery")
        local couldUnitSee = simquery.couldUnitSee
        simquery.couldUnitSee = function(sim, unit, targetUnit, ...)
            if unit and targetUnit
                and unit:getPlayerOwner() ~= targetUnit:getPlayerOwner()
                and targetUnit:getTraits().hidden_at_distance
             then
                return false
            end
            return couldUnitSee(sim, unit, targetUnit, ...)
        end
    end
end

local function FindModOption(mod_options, modname, optionname)
    if not mod_options then
        return
    end
    for k, mod in pairs(mod_options) do
        if k == modname or mod.name == modname then
            if mod and mod.enabled and mod.options and mod.options[optionname] then
                return mod.options[optionname].value or mod.options[optionname].enabled
            end
            return -- can give up at this point
        end
    end
end

local function generationOptionEnabled(options, name, default)
    if options and options[name] then
        return options[name].enabled
    end
    return default or false
end

local function load(modApi, options, params, mod_options)
    ThisModLoaded = true

    local scriptPath = modApi:getScriptPath()

    do
        local simdefs = include("sim/simdefs")
        for name, simDef in pairs(include(scriptPath .. "/simdefs")) do
            simdefs[name] = simDef
        end
    end

    modApi:addTooltipDef( include( scriptPath .. "/tooltipdefs" ) )

    local mod_animdefs = include(scriptPath .. "/animdefs")
    for name, animDef in pairs(mod_animdefs) do
        modApi:addAnimDef( name, animDef )
    end

    mod_animdefs = FindModOption(mod_options, "Mods Combo by Shirsh", "noWireframes")
        and include(scriptPath .. "/animdefs_no_wireframes")
        or include(scriptPath .. "/animdefs_wireframes")
    for name, animDef in pairs(mod_animdefs) do
        modApi:addAnimDef( name, animDef )
    end

    do -- registering rigs is hacky…
        package.loaded["gameplay/glitchtraprig"] = include(scriptPath .. "/glitchtraprig")
    end

    include(scriptPath .. "/units/simgrenade_stickystun")

    mod_itemdefs = include(scriptPath .. "/itemdefs")

    if generationOptionEnabled(options, "glitchtrap_store") then
        modApi:addItemDef("item_glitchtrap_extremerange", mod_itemdefs.item_glitchtrap_extremerange)
        modApi:addItemDef("item_glitchtrap_highcap", mod_itemdefs.item_glitchtrap_highcap)
        modApi:addItemDef("item_glitchtrap_rechargable", mod_itemdefs.item_glitchtrap_rechargable)
        modApi:addItemDef("item_glitchtrap_remote", mod_itemdefs.item_glitchtrap_remote)
    end

    if generationOptionEnabled(options, "empbeam_store") then
        modApi:addItemDef("item_empbeam_extremerange", mod_itemdefs.item_empbeam_extremerange)
        modApi:addItemDef("item_empbeam_highcap", mod_itemdefs.item_empbeam_highcap)
        modApi:addItemDef("item_empbeam_selfcharging", mod_itemdefs.item_empbeam_selfcharging)
    end

    modApi:addItemDef("item_glitchtrap_selfcharging", mod_itemdefs.item_glitchtrap_selfcharging)
    modApi:addItemDef("item_facebreaker", mod_itemdefs.item_facebreaker)
    modApi:addItemDef("item_flashgrenade_sticky_launcher", mod_itemdefs.item_flashgrenade_sticky_launcher)
    modApi:addItemDef("item_flashgrenade_sticky", mod_itemdefs.item_flashgrenade_sticky)
    modApi:addItemDef("item_swapper", mod_itemdefs.item_swapper)
    -- modApi:addItemDef("item_empbeam_rechargable", mod_itemdefs.item_empbeam_rechargable)

    modApi:addAbilityDef("facebreaker", scriptPath .. "/abilities/facebreaker")
    modApi:addAbilityDef("glitchtrap", scriptPath .. "/abilities/glitchtrap")
    modApi:addAbilityDef("swapteleport", scriptPath .. "/abilities/swapteleport")
    modApi:addAbilityDef("primeGrenades", scriptPath .. "/abilities/primeGrenades")

    local mod_agentdefs = include(scriptPath .. "/agentdefs")
    modApi:addAgentDef("saderfiasco", mod_agentdefs.saderfiasco, {"saderfiasco", "saderfiasco_a"})
    modApi:addAgentDef("saderfiasco_a", mod_agentdefs.saderfiasco_a)

    if generationOptionEnabled(options, "onfile_fiasco") then
        modApi:addRescueAgent("saderfiasco", {"item_facebreaker"})
    end
    if generationOptionEnabled(options, "archive_fiasco") then
        modApi:addRescueAgent("saderfiasco_a", {"item_swapper"})
    end

    include(scriptPath .. "/banter")( modApi )
end

local function unload(modApi, options)
    ThisModLoaded = false
end

local function initStrings(modApi)
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
    local strings = include(scriptPath .. "/strings")
    modApi:addStrings(dataPath, "HEATSIGNATURE", strings)
end

return {
    init = init,
    earlyInit = earlyInit,
    load = load,
    unload = unload,
    initStrings = initStrings
}
