local util = include( "modules/util" )
local array = include( "modules/array" )

local function earlyInit( modApi )
	modApi.requirements = { "Contingency Plan", "Programs Extended", "Neptune Corporation", "Function Library", "Mods Combo by Shirsh" }
end

local function init( modApi )
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )

	if not SCRIPT_PATHS.W93_FuncLibActive or SCRIPT_PATHS.W93_FuncLibActive == false then
		log:write("Function Library disabled or missing! Worldgen Extended is disabled.")
		return
	end

	modApi:addGenerationOption("extended_worldgen", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_TIP,{noUpdate = true})
	modApi:addGenerationOption("extended_worldgen_ftm", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_FTM, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_FTM_TIP,{noUpdate = true})
	modApi:addGenerationOption("extended_worldgen_ko", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_KO, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_KO_TIP,{noUpdate = true})
	modApi:addGenerationOption("extended_worldgen_sankaku", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_SANKAKU, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_SANKAKU_TIP,{noUpdate = true})
	modApi:addGenerationOption("extended_worldgen_plastech", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_PLASTECH, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WORLDGEN_PLASTECH_TIP,{noUpdate = true})
	modApi:addGenerationOption("enable_weighting", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WEIGHTING, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_WEIGHTING_TIP,{noUpdate = true, enabled=false})
	modApi:addGenerationOption("enable_devices", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_DEVICES, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_DEVICES_TIP,{noUpdate = true})
	modApi:addGenerationOption("extended_security", STRINGS.WORLDEXTEND.OPTIONS.ENABLE_SECURITY, STRINGS.WORLDEXTEND.OPTIONS.ENABLE_SECURITY_TIP,{noUpdate = true, strings={"0%", "20%", "40%", "60%", "80%", "100%",}, values = {0,0.2,0.4,0.6,0.8,1}, value=0.2})

    	SCRIPT_PATHS.W93_prefabs = scriptPath
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "WORLDEXTEND", DLC_STRINGS )

	util.tmerge( STRINGS.LOADING_TIPS, STRINGS.WORLDEXTEND.LOADING_TIPS )
end

local function load( modApi, options, params, mod_options )
	if not SCRIPT_PATHS.W93_FuncLibActive or SCRIPT_PATHS.W93_FuncLibActive == false then
		log:write("Funcion Library disabled or missing! Worldgen Extended is disabled.")
		return
	end

	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/alter" )

	W93_WE_reset_security(scriptPath)

	local PE = mod_manager:findModByName("Programs Extended")
	if PE and mod_options[PE.id] and mod_options[PE.id].enabled then
		if mod_options[PE.id].options.shop_programs and mod_options[PE.id].options.shop_programs.enabled then
			PE = 1
		else
			PE = 0
		end
	else
		PE = 0
	end

	SCRIPT_PATHS.W93_prefabs_value = options["extended_security"].value
	SCRIPT_PATHS.W93_prefabs_PE = PE
	W93_WE_set_security(options["extended_security"].value,PE)

        if options["extended_worldgen"] and options["extended_worldgen"].enabled then
    		local sharedPrefabs = include( scriptPath .. "/prefabs/shared/prefabt" )
		modApi:addPrefabt(sharedPrefabs)

		local propdefs = include( "sim/unitdefs/propdefs" )
		propdefs.item_store_large.traits.noMandatoryItems = true

		include( scriptPath .. "/missions/mission_security" )
    	else
		local propdefs = include( "sim/unitdefs/propdefs" )
		propdefs.item_store_large.traits.noMandatoryItems = false
	end

        if options["extended_worldgen_ftm"] and options["extended_worldgen_ftm"].enabled then
    		local ftmPrefabs = include( scriptPath .. "/prefabs/ftm/prefabt" )
    		modApi:addWorldPrefabt( scriptPath, "ftm", ftmPrefabs)
    	end

        if options["extended_worldgen_ko"] and options["extended_worldgen_ko"].enabled then
    		local koPrefabs = include( scriptPath .. "/prefabs/ko/prefabt" )
    		modApi:addWorldPrefabt( scriptPath, "ko", koPrefabs)
    	end

        if options["extended_worldgen_sankaku"] and options["extended_worldgen_sankaku"].enabled then
    		local sankakuPrefabs = include( scriptPath .. "/prefabs/sankaku/prefabt" )
    		modApi:addWorldPrefabt( scriptPath, "sankaku", sankakuPrefabs)
    	end

        if options["extended_worldgen_plastech"] and options["extended_worldgen_plastech"].enabled then
    		local plastechPrefabs = include( scriptPath .. "/prefabs/plastech/prefabt" )
    		modApi:addWorldPrefabt( scriptPath, "plastech", plastechPrefabs)
    	end

	if options["enable_devices"] and options["enable_devices"].enabled then
    		local animdefs = include( scriptPath .. "/animdefs" )
    		for name, animDef in pairs(animdefs) do
        		modApi:addAnimDef( name, animDef )
    		end

    		local propdefs = include( scriptPath .. "/propdefs" )
    		for i,item in pairs(propdefs) do  
        		modApi:addPropDef( i, item, false )
    		end

		local commondefs = include( scriptPath .. "/commondefs" )
		modApi:addTooltipDef( commondefs )

    		local devicesPrefabs = include( scriptPath .. "/prefabs/devices/prefabt" )
		modApi:addPrefabt(devicesPrefabs)
		include( scriptPath .. "/AI_Core" )
		include( scriptPath .. "/daemon_server" )
		include( scriptPath .. "/drone_uplink" )
		include( scriptPath .. "/frostspire" )
		include( scriptPath .. "/shockmine" )
		include( scriptPath .. "/worldgen" )
	end
end

local function lateLoad( modApi, options, params, mod_options )
	local scriptPath = modApi:getScriptPath()

	if not SCRIPT_PATHS.W93_FuncLibActive or SCRIPT_PATHS.W93_FuncLibActive == false then
		log:write("Funcion Library disabled or missing! Worldgen Extended is disabled.")
		return
	end

	if options["enable_weighting"] and not options["enable_weighting"].enabled then

	local prefabsShared = include( "sim/prefabs/shared/prefabt" )
	for _, prefab in pairs(prefabsShared.PREFABT0) do
		if array.find(prefab.tags, "entry") then
			if not prefab.old_weight then
				prefab.old_weight = prefab.weight
			end
			prefab.weight = 0
		end
		if array.find(prefab.tags, "objective") then
			if not prefab.old_weight then
				prefab.old_weight = prefab.weight
			end
			prefab.weight = 1
		end
	end
	local prefabsFTM = include( "sim/prefabs/ftm/prefabt" )
	for _, prefab in pairs(prefabsFTM.PREFABT0) do
		if array.find(prefab.tags, "entry") or array.find(prefab.tags, "objective") then
			if not prefab.old_weight then
				prefab.old_weight = prefab.weight
			end
			prefab.weight = 1
		end
	end
	local prefabsKO = include( "sim/prefabs/ko/prefabt" )
	for _, prefab in pairs(prefabsKO.PREFABT0) do
		if array.find(prefab.tags, "entry") or array.find(prefab.tags, "objective") then
			if not prefab.old_weight then
				prefab.old_weight = prefab.weight
			end
			prefab.weight = 1
		end
	end
	local prefabsSankaku = include( "sim/prefabs/sankaku/prefabt" )
	for _, prefab in pairs(prefabsSankaku.PREFABT0) do
		if array.find(prefab.tags, "entry") or array.find(prefab.tags, "objective") then
			if not prefab.old_weight then
				prefab.old_weight = prefab.weight
			end
			prefab.weight = 1
		end
	end
	local prefabsPlastech = include( "sim/prefabs/plastech/prefabt" )
	for _, prefab in pairs(prefabsPlastech.PREFABT0) do
		if array.find(prefab.tags, "entry") or array.find(prefab.tags, "objective") then
			if not prefab.old_weight then
				prefab.old_weight = prefab.weight
			end
			prefab.weight = 1
		end
	end

	local prefabsModded = mod_manager.modPrefabs
	for _, category in pairs(prefabsModded) do
		for _, prefab in pairs(category.PREFABT0) do
			if array.find(prefab.tags, "entry") or array.find(prefab.tags, "objective") then
				if not prefab.old_weight then
					prefab.old_weight = prefab.weight
				end
				prefab.weight = 1
			end
		end
	end

	local prefabsModdedWorld = mod_manager.modWorldPrefabs
	for _, category in pairs(prefabsModdedWorld) do
		for _, prefab in pairs(category.PREFABT0) do
			if array.find(prefab.tags, "entry") or array.find(prefab.tags, "objective") then
				if not prefab.old_weight then
					prefab.old_weight = prefab.weight
				end
				prefab.weight = 1
			end
		end
	end

	end

	local NIAA = mod_manager:findModByName("New Items And Augments")
	local propdefs = include("sim/unitdefs/propdefs")
	if options["enable_devices"] and options["enable_devices"].enabled and NIAA and mod_options[NIAA.id] and mod_options[NIAA.id].enabled then
		if mod_options[NIAA.id].options.enable_guard_loot and mod_options[NIAA.id].options.enable_guard_loot.enabled and mod_options[NIAA.id].options.enable_guard_loot.value > 0 then
			propdefs.W93_daemon_server.children[1] = "W93_item_clip_small"
			propdefs.W93_drone_uplink.children[1] = "W93_item_clip_small"
			propdefs.W93_frost_spire.children[1] = "W93_item_clip_small"
		else
			propdefs.W93_daemon_server.children[1] = "item_clip"
			propdefs.W93_drone_uplink.children[1] = "item_clip"
			propdefs.W93_frost_spire.children[1] = "item_clip"
		end
	elseif options["enable_devices"] and options["enable_devices"].enabled then
		propdefs.W93_daemon_server.children[1] = "item_clip"
		propdefs.W93_drone_uplink.children[1] = "item_clip"
		propdefs.W93_frost_spire.children[1] = "item_clip"
	end
end

local function unload( modApi, mod_options, params )
	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/alter" )

	local propdefs = include( "sim/unitdefs/propdefs" )
	propdefs.item_store_large.traits.noMandatoryItems = false

	W93_WE_reset_security(scriptPath)
end

local function lateUnload( modApi, mod_options, params )
	local scriptPath = modApi:getScriptPath()

	local prefabsShared = include( "sim/prefabs/shared/prefabt" )
	for _, prefab in pairs(prefabsShared.PREFABT0) do
		if prefab.old_weight then
			prefab.weight = prefab.old_weight
		end
	end
	local prefabsFTM = include( "sim/prefabs/ftm/prefabt" )
	for _, prefab in pairs(prefabsFTM.PREFABT0) do
		if prefab.old_weight then
			prefab.weight = prefab.old_weight
		end
	end
	local prefabsKO = include( "sim/prefabs/ko/prefabt" )
	for _, prefab in pairs(prefabsKO.PREFABT0) do
		if prefab.old_weight then
			prefab.weight = prefab.old_weight
		end
	end
	local prefabsSankaku = include( "sim/prefabs/sankaku/prefabt" )
	for _, prefab in pairs(prefabsSankaku.PREFABT0) do
		if prefab.old_weight then
			prefab.weight = prefab.old_weight
		end
	end
	local prefabsPlastech = include( "sim/prefabs/plastech/prefabt" )
	for _, prefab in pairs(prefabsPlastech.PREFABT0) do
		if prefab.old_weight then
			prefab.weight = prefab.old_weight
		end
	end

	local prefabsModded = mod_manager.modPrefabs
	for _, category in pairs(prefabsModded) do
		for _, prefab in pairs(category.PREFABT0) do
			if prefab.old_weight then
				prefab.weight = prefab.old_weight
			end
		end
	end

	local prefabsModdedWorld = mod_manager.modWorldPrefabs
	for _, category in pairs(prefabsModdedWorld) do
		for _, prefab in pairs(category.PREFABT0) do
			if prefab.old_weight then
				prefab.weight = prefab.old_weight
			end
		end
	end
end

return {
    earlyInit = earlyInit,
    init = init,
    load = load,
    lateLoad = lateLoad,
    initStrings= initStrings,
    unload = unload,
    lateUnload = lateUnload,
}