local util = include("client_util")
local array = include("modules/array")

local function earlyInit(modApi)
	modApi.requirements =
	{
		"Sim Constructor",
		"Function Library",
		"More Archived Agents",
		"Advanced Guard Protocol",
	}
	
	function mod_manager:findModByName( name )
		for i, modData in ipairs(self.mods) do
			if name and mod_manager:getModName( modData.id ) == name then
				return modData
			end
		end
    	end
    	
    	local scriptPath = modApi:getScriptPath()
	
	--use MAA code if installed or duplicate it if not
	if not mod_manager:findModByName("More Archived Agents") then
		include( scriptPath .. "/agent_panel_override" )
	end
	
	--include( scriptPath .. "/abilityutil_funclib" )
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local COIT_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "CORPORATE_ITEMS", COIT_STRINGS )

	--rawset(_G,"COIT_SCRIPTPATH",rawget(_G,"COIT_SCRIPTPATH") or {})
	--COIT_SCRIPTPATH.path = scriptPath
end


local function init( modApi )
	local scriptPath = modApi:getScriptPath()
	local dataPath = modApi:getDataPath()

	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	
	modApi:addGenerationOption("add_items", STRINGS.CORPORATE_ITEMS.OPTIONS.ADD_ITEMS, STRINGS.CORPORATE_ITEMS.OPTIONS.ADD_ITEMS_DESC,
	{
		noUpdate = true,
	} )
	modApi:addGenerationOption("add_programs", STRINGS.CORPORATE_ITEMS.OPTIONS.ADD_PROGRAMS, STRINGS.CORPORATE_ITEMS.OPTIONS.ADD_PROGRAMS_DESC,
	{
		noUpdate = true,
	} )
	
	--use MAA code if installed or duplicate it if not
	if not mod_manager:findModByName("More Archived Agents") then
		include( scriptPath .. "/agent_panel" )
		include( scriptPath .. "/agent_actions" )
		include( scriptPath .. "/guiex_maa" )
	end
	
	include( scriptPath .. "/simstore" )
	include( scriptPath .. "/simparams" )
	include( scriptPath .. "/simplayer" )
	include( scriptPath .. "/inventory" )
	include( scriptPath .. "/abilityutil" )
	include( scriptPath .. "/acid_grenade_unit" )
	include( scriptPath .. "/engine" )
	include( scriptPath .. "/simquery" )
	include( scriptPath .. "/combat_panel" )
	include( scriptPath .. "/simactions" )
	include( scriptPath .. "/mainframe_panel" )
end

local function lateInit( modApi )
	local scriptPath = modApi:getScriptPath()

	include( scriptPath .. "/simunit" )
end

local function load( modApi, options, params, mod_options )
	local scriptPath = modApi:getScriptPath()
	
	modApi:addAbilityDef( "coit_daemon_siphon", scriptPath .."/abilities/coit_daemon_siphon" )
	modApi:addAbilityDef( "coit_daemon_blocker", scriptPath .."/abilities/coit_daemon_blocker" )
	modApi:addAbilityDef( "coit_weapon_power_buff", scriptPath .."/abilities/coit_weapon_power_buff" )
	modApi:addAbilityDef( "coit_laptop", scriptPath .."/abilities/coit_laptop" )
	modApi:addAbilityDef( "coit_use_chip", scriptPath .."/abilities/coit_use_chip" )
	modApi:addAbilityDef( "coit_multishot", scriptPath .."/abilities/coit_multishot" )
	modApi:addAbilityDef( "coit_overdrive", scriptPath .."/abilities/coit_overdrive" )
	modApi:addAbilityDef( "coit_conversion_item", scriptPath .. "/abilities/coit_conversion_item" )
	
	include( scriptPath .. "/abilities/deployable" )
	include( scriptPath .. "/abilities/shootSingle" )
	include( scriptPath .. "/abilities/overwatch" )
	--include( scriptPath .. "/abilities/reload" )
	include( scriptPath .. "/abilities/use_stim" )
	include( scriptPath .. "/abilities/melee" )
	include( scriptPath .. "/abilities/jackin" )
	
	--PE Flags--
	local PE_enabled = false
	local AI_enabled = false
	
	local PE = mod_manager:findModByName("Programs Extended")
	if PE and mod_options[PE.id] and mod_options[PE.id].enabled then
		PE_enabled = true
		if mod_options[PE.id].options.counter_ai and mod_options[PE.id].options.counter_ai.value >= 0 then
			AI_enabled = true
		end
	end
	
	--Add items
	if options["add_items"] and options["add_items"].enabled then
		package.loaded[scriptPath .. "/itemdefs"] = nil
		local itemdefs = include( scriptPath .. "/itemdefs" ).items
		
		if AI_enabled then
	    		itemdefs = util.tconcat( itemdefs, include( scriptPath .. "/itemdefs" ).items_PE )
		end
		for name, itemDef in pairs(itemdefs) do
			modApi:addItemDef( name, itemDef )
		end
	end
	
	--Add programs
	if options["add_programs"] and options["add_programs"].enabled then
		local programs = include( scriptPath .. "/abilities/mainframe_abilities" )
		for name, ability in pairs( programs ) do
			modApi:addMainframeAbility( name, ability )
        	end
        	
        	--add programs that suits as starters
        	modApi:addStartingGenerator("coit_faust_v2")
        	modApi:addStartingGenerator("coit_recycling")
        	modApi:addStartingGenerator("coit_compass")
        	modApi:addStartingGenerator("coit_soul_harvest")
        	modApi:addStartingGenerator("coit_siren_v2")
        	
        	modApi:addStartingBreaker("coit_brimstone_v2")
        	modApi:addStartingBreaker("coit_nail")
	end
	
	--Add Tooltips
	local tooltips = include( scriptPath .. "/tooltips" )
	modApi:addTooltipDef( tooltips )
end

local function lateLoad( modApi, options, mod_params, mod_options )
	local scriptPath = modApi:getScriptPath()
	
	local itemdefs = include( "sim/unitdefs/itemdefs" )
	
	for _, item in pairs( itemdefs ) do
		--add soundrange if NIaA
		if mod_manager:findModByName("New Items And Augments") then
			local NIAA_id = mod_manager:findModByName("New Items And Augments").id
			if NIAA_id and mod_options[NIAA_id] and mod_options[NIAA_id].enabled then
				if item.niaa_traits then
					util.extend( item.niaa_traits ) ( item.traits )
				end
			end
		end
		
		if item.abilities then
			--Conversion
			if array.find( item.abilities, "recharge" ) then
				table.insert( item.abilities, "coit_conversion_item" )
			end
		end
	end
end

return
{
	initStrings = initStrings,
	earlyInit = earlyInit,
	init = init,
	lateInit = lateInit,
	--earlyLoad = earlyLoad,
	load = load,
	lateLoad = lateLoad,
	--unload = unload,
}
