local util = include("modules/util")
local simdefs = include( "sim/simdefs" )
local cdefs = include( "client_defs" )

local function FindIndexOf(val, array)
    for k,v in pairs(array) do 
        if v == val then return k end
    end
end

local function init( modApi )
	modApi.requirements = { "Contingency Plan", "New Items And Augments" }

	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/simquery" )
	include( scriptPath .. "/state-upgrade-screen" )
	include( scriptPath .. "/engine" )
	include( scriptPath .. "/hud" )
	local simdefs = include( "sim/simdefs" )
	local simdefsOverride = include( scriptPath .. "/simdefs" )
	for i, t in pairs(simdefsOverride) do
		for k, item in pairs(t) do
			if not simdefs[i][k] then
				rawset(simdefs[i],k,item)
			end
		end
	end
	modApi:addGenerationOption("programCount", STRINGS.GEN_OPT.NEW.SLOTS , STRINGS.GEN_OPT.NEW.SLOTS_DESC, 
	{
		values = {2,3,4,5,6,7,8,9,10,12,14,15,16,18,20,22,25,28,30,35,40}, 
		value=5,
	})
	modApi:addGenerationOption("maxPwr", STRINGS.GEN_OPT.NEW.PWR_MAX , STRINGS.GEN_OPT.NEW.PWR_MAX_DESC, 
	{
		values = {0,1,2,5,10,12,15,18,20,22,25,30,35,40}, 
		value=20,
	})
	modApi:addGenerationOption("disableCpItems", STRINGS.GEN_OPT.NEW.DISABLE_CP , STRINGS.GEN_OPT.NEW.DISABLE_CP_DESC, 
	{
		enabled = false
	})
	
	modApi:addGenerationOption("agencyInventorySize", STRINGS.GEN_OPT.NEW.STORAGE , STRINGS.GEN_OPT.NEW.STORAGE_DESC, 
	{
		values = {0,1,2,4,6,8,10,12,16,20,24,32,40,math.huge}, 
		strings = STRINGS.GEN_OPT.NEW.STORAGE_STRINGS, 
		value=4, 
	})
	
	modApi:addGenerationOption("powerPerConsole", STRINGS.GEN_OPT.NEW.CONSOLE_PWR, STRINGS.GEN_OPT.NEW.CONSOLE_PWR_DESC, 
	{
		values = {0.25,0.5,0.75,1,1.5,2,2.5,3,4,5,6,7,8,9,10}, 
		value=2,
		requirements = {{mask = "mask_dynamic_powerPerLevel", requirement = true}},
		masks = {{mask = "mask_random_consolePower", requirement = "RANDOM"}}
	})
	modApi:addGenerationOption("powerPerLevelOverride", STRINGS.GEN_OPT.NEW.LEVEL_PWR, STRINGS.GEN_OPT.NEW.LEVEL_PWR_DESC, 
	{
		values = {"INDETERMINED","RANDOM",1,2,3,4,5,6,7,8,9,10,12,14,15,16,18,20,22,25,30,35,40,45,50,60,70,75,80,90,100,120,125,135,140,150,160,175,180,200,225,250},
		strings = STRINGS.GEN_OPT.NEW.LEVEL_PWR_STRINGS, 
		value="INDETERMINED",
		masks = {{mask = "mask_dynamic_powerPerLevel", requirement = "INDETERMINED"},{mask = "mask_random_levelPower", requirement = "RANDOM"}}
	})
	modApi:addGenerationOption("min_powerPerLevelOverride", STRINGS.GEN_OPT.NEW.MIN_LEVEL_PWR, STRINGS.GEN_OPT.NEW.MIN_LEVEL_PWR_DESC, 
	{
		values = {1,2,3,4,5,6,7,8,9,10,12,14,15,16,18,20,22,25,30,35,40,45,50,60,70,75,80,90,100,120,125,135,140,150,160,175,180,200,225,250}, 
		value=5,
		requirements = {{mask = "mask_random_levelPower", requirement = true}}
	})
	modApi:addGenerationOption("max_powerPerLevelOverride", STRINGS.GEN_OPT.NEW.MAX_LEVEL_PWR, STRINGS.GEN_OPT.NEW.MAX_LEVEL_PWR_DESC, 
	{
		values = {1,2,3,4,5,6,7,8,9,10,12,14,15,16,18,20,22,25,30,35,40,45,50,60,70,75,80,90,100,120,125,135,140,150,160,175,180,200,225,250}, 
		value=15,
		requirements = {{mask = "mask_random_levelPower", requirement = true}}
	})
	modApi:addGenerationOption("banter", STRINGS.GEN_OPT.NEW.BANTER, STRINGS.GEN_OPT.NEW.BANTER_DESC, 
	{
		values = {0,0.1,0.2,0.4,0.7,1}, 
		strings = STRINGS.GEN_OPT.NEW.BANTER_STRINGS, 
		value=0.2,
		noUpdate = true,
	})
	modApi:addGenerationOption("sideMissions", STRINGS.GEN_OPT.NEW.SIDEMISSIONS, STRINGS.GEN_OPT.NEW.SIDEMISSIONS_DESC, 
	{
		values = {0,5,10,15,25,35,50,75,100}, 
		value=25, 
		strings = STRINGS.GEN_OPT.NEW.SIDEMISSIONS_STRINGS
	})
	modApi:addGenerationOption("startingHours", STRINGS.GEN_OPT.NEW.START_HOURS, STRINGS.GEN_OPT.NEW.START_HOURS_DESC, 
	{
		values = {0,12,24,36,48,60,72,96,120,144,168,192,216,240,264,288,312,336,360,384,408,432,456,480}, 
		value=0, 
		strings = STRINGS.GEN_OPT.NEW.START_HOURS_STRINGS
	})
	modApi:addGenerationOption("finalEnforcers", STRINGS.GEN_OPT.NEW.ENFORCERS, STRINGS.GEN_OPT.NEW.ENFORCERS_DESC, 
	{
		values = {0,1,2,3,4}, 
		value=0,
		difficulties = {{3,1},{4,2}},
	})
	modApi:addGenerationOption("hourMultiplier", STRINGS.GEN_OPT.NEW.TRAVEL, STRINGS.GEN_OPT.NEW.TRAVEL_DESC, 
	{
		values = {0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1,1.1,1.2,1.3,1.5,1.7,2,2.5,3,3.5,4}, 
		value=1, 
	})
	modApi:addGenerationOption("sphericalEarth", STRINGS.GEN_OPT.NEW.SPHERICAL_EARTH, STRINGS.GEN_OPT.NEW.SPHERICAL_EARTH_DESC, 
	{
		enabled = false, 
	})
	modApi:addGenerationOption("baseTraveltime", STRINGS.GEN_OPT.NEW.BASE_TRAVEL, STRINGS.GEN_OPT.NEW.BASE_TRAVEL_DESC, 
	{
		values = {0,1,2,3,4,5,6,7,8,9,10}, 
		value=3, 
	})
	modApi:addGenerationOption("travelCeiling", STRINGS.GEN_OPT.NEW.TRAVEL_CEILING, STRINGS.GEN_OPT.NEW.TRAVEL_CEILING_DESC, 
	{
		values = {5,6,7,8,9,10,11,12,13,14,15,16,18,20,22,24,26,28,30,32,36,40,48}, 
		value=12, 
	})
	modApi:addGenerationOption("startingMissions", STRINGS.GEN_OPT.NEW.MISSIONS, STRINGS.GEN_OPT.NEW.MISSIONS_DESC, 
	{
		values = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16}, 
		value=1, 
		strings = STRINGS.GEN_OPT.NEW.MISSIONS_STRINGS
	})
	modApi:addGenerationOption("magnetic_reinforcement", STRINGS.GEN_OPT.NEW.MAG, STRINGS.GEN_OPT.NEW.MAG_DESC, 
	{
		values =  {-3,-2,-1.5,1.25,-1,-0.75,-0.5,-0.375,-0.25,-0.125,0,0.125,0.25,0.5,0.75,1}, 
		strings = STRINGS.GEN_OPT.NEW.MAG_STRINGS,
		value= -0.25,
	})
	modApi:addGenerationOption("magnetic_reinforcement_scaling", STRINGS.GEN_OPT.NEW.MAG_SCALE, STRINGS.GEN_OPT.NEW.MAG_SCALE_DESC, 
	{
		values = {0,0.0625,0.1,0.125,0.25,0.5,0.75,1}, 
		value= 0.25, 
	})
	modApi:addGenerationOption("magnetic_reinforcement_step", STRINGS.GEN_OPT.NEW.MAG_STEP, STRINGS.GEN_OPT.NEW.MAG_STEP_DESC, 
	{
		values =  {1,2,3,4},
		value= 2, 
	})	
	modApi:addGenerationOption("magnetic_reinforcement_day", STRINGS.GEN_OPT.NEW.MAG_DAY, STRINGS.GEN_OPT.NEW.MAG_DAY_DESC, 
	{
		values = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20}, 
		value= 6, 
	})
	modApi:addGenerationOption("firewalls", STRINGS.GEN_OPT.NEW.ICE, STRINGS.GEN_OPT.NEW.ICE_DESC, 
	{
		values = {"RANDOM",-2,-1,0.5,0,0.5,1,1.5,2,3,4,5,6,8,10}, 
		strings = STRINGS.GEN_OPT.NEW.ICE_STRINGS,
		value=1,
		masks = {{mask = "mask_random_firewalls", requirement = "RANDOM"}}
	})
	modApi:addGenerationOption("min_firewalls", STRINGS.GEN_OPT.NEW.MIN_ICE, STRINGS.GEN_OPT.NEW.MIN_ICE_DESC, 
	{
		values = {-2,-1,0.5,0,0.5,1,1.5,2,3,4,5,6,8,10}, 
		strings = STRINGS.GEN_OPT.NEW.MIN_ICE_STRINGS,
		value=0.5,
		requirements = {{mask = "mask_random_firewalls", requirement = true}}
	})
	modApi:addGenerationOption("max_firewalls", STRINGS.GEN_OPT.NEW.MAX_ICE, STRINGS.GEN_OPT.NEW.MAX_ICE_DESC, 
	{
		values = {-2,-1,0.5,0,0.5,1,1.5,2,3,4,5,6,8,10}, 
		strings = STRINGS.GEN_OPT.NEW.MAX_ICE_STRINGS,
		value=1.5,
		requirements = {{mask = "mask_random_firewalls", requirement = true}}
	})
	modApi:addGenerationOption("firewallscalar", STRINGS.GEN_OPT.NEW.ICE_SCALE, STRINGS.GEN_OPT.NEW.ICE_SCALE_DESC, 
	{
		values = {"RANDOM",0,0.15,0.25,0.35,0.5,0.6,0.75,0.9,1,1.25,1.5,1.75,2,2.25,2.5,2.75,3}, 
		strings = STRINGS.GEN_OPT.NEW.ICE_SCALE_STRINGS,
		value=1,
		masks = {{mask = "mask_random_firewallscalar", requirement = "RANDOM"}},
	})
	modApi:addGenerationOption("min_firewallscalar", STRINGS.GEN_OPT.NEW.MIN_ICE_SCALE, STRINGS.GEN_OPT.NEW.MIN_ICE_SCALE_DESC, 
	{
		values = {0,0.15,0.25,0.35,0.5,0.6,0.75,0.9,1,1.25,1.5,1.75,2,2.25,2.5,2.75,3}, 
		value=0.5,
		requirements = {{mask = "mask_random_firewallscalar", requirement = true}},
	})
	modApi:addGenerationOption("max_firewallscalar", STRINGS.GEN_OPT.NEW.MAX_ICE_SCALE, STRINGS.GEN_OPT.NEW.MAX_ICE_SCALE_DESC, 
	{
		values = {0,0.15,0.25,0.35,0.5,0.6,0.75,0.9,1,1.25,1.5,1.75,2,2.25,2.5,2.75,3}, 
		value=1.5,
		requirements = {{mask = "mask_random_firewallscalar", requirement = true}},
	})
	modApi:addGenerationOption("firewalldelta", STRINGS.GEN_OPT.NEW.ICE_DELTA, STRINGS.GEN_OPT.NEW.ICE_DELTA_DESC, 
	{
		values = {"RANDOM",-2,-1,0.5,0,0.5,1,1.5,2,3,4,5,6,8,10}, 
		strings = STRINGS.GEN_OPT.NEW.ICE_DELTA_STRINGS,
		value=1,
		masks = {{mask = "mask_random_firewalldelta", requirement = "RANDOM"}}
	})
	modApi:addGenerationOption("min_firewalldelta", STRINGS.GEN_OPT.NEW.MIN_ICE_DELTA, STRINGS.GEN_OPT.NEW.MIN_ICE_DELTA_DESC, 
	{
		values = {-2,-1,0.5,0,0.5,1,1.5,2,3,4,5,6,8,10},
		value=0.5,
		requirements = {{mask = "mask_random_firewalldelta", requirement = true}}
	})
	modApi:addGenerationOption("max_firewalldelta", STRINGS.GEN_OPT.NEW.MAX_ICE_DELTA, STRINGS.GEN_OPT.NEW.MAX_ICE_DELTA_DESC, 
	{
		values = {-2,-1,0.5,0,0.5,1,1.5,2,3,4,5,6,8,10},
		value=1.5,
		requirements = {{mask = "mask_random_firewalldelta", requirement = true}}
	})
	modApi:addGenerationOption("firewalldeltascalar", STRINGS.GEN_OPT.NEW.ICE_DELTA_SCALE, STRINGS.GEN_OPT.NEW.ICE_DELTA_SCALE_DESC, 
	{
		values = {"RANDOM",0,0.15,0.25,0.35,0.5,0.6,0.75,0.9,1,1.25,1.5,1.75,2,2.25,2.5,2.75,3}, 
		strings = STRINGS.GEN_OPT.NEW.ICE_DELTA_SCALE_STRINGS, 
		value=1,
		masks = {{mask = "mask_random_firewalldeltascalar", requirement = "RANDOM"}}
	})
	modApi:addGenerationOption("min_firewalldeltascalar", STRINGS.GEN_OPT.NEW.MIN_ICE_DELTA_SCALE, STRINGS.GEN_OPT.NEW.MIN_ICE_DELTA_SCALE_DESC, 
	{
		values = {0,0.15,0.25,0.35,0.5,0.6,0.75,0.9,1,1.25,1.5,1.75,2,2.25,2.5,2.75,3}, 
		value=0.5,
		requirements = {{mask = "mask_random_firewalldeltascalar", requirement = true}}
	})
	modApi:addGenerationOption("max_firewalldeltascalar", STRINGS.GEN_OPT.NEW.MAX_ICE_DELTA_SCALE, STRINGS.GEN_OPT.NEW.MAX_ICE_DELTA_SCALE_DESC, 
	{
		values = {0,0.15,0.25,0.35,0.5,0.6,0.75,0.9,1,1.25,1.5,1.75,2,2.25,2.5,2.75,3}, 
		value=1.5,
		requirements = {{mask = "mask_random_firewalldeltascalar", requirement = true}}
	})
	modApi:addGenerationOption("credit_scalar", STRINGS.GEN_OPT.NEW.CREDIT_SCALE, STRINGS.GEN_OPT.NEW.CREDIT_SCALE_DESC, 
	{
		values = {0,0.005,0.01,0.015,0.02,0.025,0.03,0.04,0.05,0.075,0.1,0.15,0.2,0.25,0.3,0.4,0.5,0.75,1}, 
		strings = STRINGS.GEN_OPT.NEW.CREDIT_SCALE_STRINGS,
		value=0.25,
	})
	modApi:addGenerationOption("credit_ceil", STRINGS.GEN_OPT.NEW.CREDIT_CEIL, STRINGS.GEN_OPT.NEW.CREDIT_CEIL_DESC, 
	{
		values = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20},
		value=4,
	})
	modApi:addGenerationOption("newday", STRINGS.GEN_OPT.NEW.NEWDAY, STRINGS.GEN_OPT.NEW.NEWDAY_DESC, 
	{
		values = {0,1,2,3,4,5,6,7,8,9,10},
		value=4,
	})
	modApi:addGenerationOption("extendedcampaign", STRINGS.GEN_OPT.NEW.EXTENDED_CAMPAIGN, STRINGS.GEN_OPT.NEW.EXTENDED_CAMPAIGN_DESC, 
	{
		values = {0,12,24,36,48,60,72,96,120,144,168,192,math.huge},
		strings = STRINGS.GEN_OPT.NEW.EXTENDED_CAMPAIGN_STRINGS,
		value=48,
	})
	modApi:addGenerationOption("finaleDifficulty", STRINGS.GEN_OPT.NEW.FINAL_DIFFICULTY , STRINGS.GEN_OPT.NEW.FINAL_DIFFICULTY_DESC, 
	{
		values = {"DEFAULT",1, 2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20},
		strings = STRINGS.GEN_OPT.NEW.FINAL_DIFFICULTY_STRINGS,
		value="DEFAULT",
	})
	modApi:addGenerationOption("customFinaleDifficulty", STRINGS.GEN_OPT.NEW.CUSTOM_FINAL_DIFFICULTY , STRINGS.GEN_OPT.NEW.CUSTOM_FINAL_DIFFICULTY_DESC, 
	{
		values = {"DEFAULT",1, 2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20}, 
		strings = STRINGS.GEN_OPT.NEW.CUSTOM_FINAL_DIFFICULTY_STRINGS,
		value="DEFAULT",
	})
	modApi:addGenerationOption("difficultyFix", STRINGS.GEN_OPT.NEW.FIX_DIFFICULTY , STRINGS.GEN_OPT.NEW.FIX_DIFFICULTY_DESC, 
	{
		noUpdate = true
	})
	
	--modApi:addGenerationOption("quickGame", "QUICK GAME" , "<c:FF8411>QUICK GAME</c>\nPlay a single random mission with a random loadout.", {value=false} )
	rawset(simdefs.CUSTOM_OPTIONS,"REWINDS",{
    	name = STRINGS.UI.DIFF_OPTION_REWINDS,
    	tip = STRINGS.UI.DIFF_OPTION_REWINDS_TIP,
        ironManCheck = true,
        values = { 0,1,2,3,4,5,6,7,8,9,10,12,15,20,25,30,40,50,75,99 },
        strings = STRINGS.GEN_OPT.ORIGINAL.REWINDS,
        apply = function( self, options, index )
            options.rewindsLeft = self.values[index]
        end,
        retrieve = function( self, options )
            return self.strings[FindIndexOf(options.rewindsLeft, self.values)]
        end,
    })
	rawset(simdefs.CUSTOM_OPTIONS,"ADDITIONAL KO TIME",{
    	name = STRINGS.UI.DIFF_OPTION_KOTIME,
    	tip = STRINGS.UI.DIFF_OPTION_KOTIME_TIP,
        strings = STRINGS.GEN_OPT.ORIGINAL.KOTIME,
        masks = {{mask="mask_random_kotime",requirement="RANDOM"}},
		values = {99,50,40,30,25,20,15,12,10,9,8,7,6,5,4,3,2,1,0,-1,-2,-3,-4,-5},
        apply = function( self, options, idx )
            options.koDuration = self.values[idx]
        end,
        retrieve = function( self, options )
            return self.strings[FindIndexOf(options.koDuration, self.values)]
        end,
    })
	rawset(simdefs.CUSTOM_OPTIONS,"CREDIT MULTIPLIER",{
    	name = STRINGS.UI.DIFF_OPTION_CREDITS,
    	tip = STRINGS.UI.DIFF_OPTION_CREDITS_TIP,
        values = { 0, 0.05, 0.1, 1.5, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85, 0.9, 0.95, 1, 1.1, 1.2, 1.25, 1.3, 1.4, 1.5, 1.6, 1.7, 1.75, 1.8, 1.9, 2, 2.25, 2.5, 2.75, 3, 3.5, 4, 4.5, 5, 6, 7, 8, 9, 10, 12, 15, 20, 25, 30, 40, 50, 75, 100, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 950, 1000  },
        masks = {{mask="mask_random_credits",requirement="RANDOM"}},
		apply = function( self, options, idx )
            options.creditMultiplier = self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.creditMultiplier)
        end,
    })
	rawset(simdefs.CUSTOM_OPTIONS,"CAMPAIGN HOURS",{
    	name = STRINGS.UI.DIFF_OPTION_HOURS,
    	tip = STRINGS.UI.DIFF_OPTION_HOURS_TIP,
        strings = STRINGS.GEN_OPT.ORIGINAL.HOURS,
        values = { 12, 24,48,72,96,120,144,168,192,216,240,264,288,312,336,360,384,408,432,456,480, math.huge},
        apply = function( self, options, index )
            options.maxHours = self.values[index]
        end,
        retrieve = function( self, options )
            return self.strings[FindIndexOf(options.maxHours, self.values)]
        end,
    })
	rawset(simdefs.CUSTOM_OPTIONS,"MISSION START PWR",{
    	name = STRINGS.UI.DIFF_OPTION_STARTPWR,
    	tip = STRINGS.UI.DIFF_OPTION_STARTPWR_TIP,
        values = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20 },
        masks = {{mask="mask_random_pwr",requirement="RANDOM"}},
		apply = function( self, options, idx )
            options.startingPower = self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.startingPower)
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"ALARM MULTIPLIER",{
    	name = STRINGS.UI.DIFF_OPTION_ALARM,
    	tip = STRINGS.UI.DIFF_OPTION_ALARM_TIP,
        values = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 20, 25, 30 },
        apply = function( self, options, idx )
            options.alarmMultiplier =  self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.alarmMultiplier)
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"STARTING MONEY",{
    	name = STRINGS.UI.DIFF_OPTION_MONEY,
    	tip = STRINGS.UI.DIFF_OPTION_MONEY_TIP,
        values = { 0, 10, 100, 200, 250, 500, 750, 1000, 2000, 5000, 10000, 20000, 50000, 100000, 500000, 1000000 },
        apply = function( self, options, idx )
            options.startingCredits = self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.startingCredits)
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"CONSOLES PER LEVEL",{
    	name = STRINGS.UI.DIFF_OPTION_CONSOLES,
    	tip = STRINGS.UI.DIFF_OPTION_CONSOLES_TIP,
        values = { "RANDOM", 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 25 },
        strings = STRINGS.GEN_OPT.ORIGINAL.PROPSPAWNS,
        masks = {{mask="mask_random_consoles",requirement="RANDOM"}},
		nestedOptions = {
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.MIN_CONSOLES,
				tip = STRINGS.UI.DIFF_OPTION_CONSOLES_TIP,
				values = { 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 25 },
				strings = STRINGS.GEN_OPT.EXTENSIONS.EX_PROPS,
				requirements = {{mask="mask_random_consoles",requirement=true}},
				apply = function( self, options, idx )
					options.min_consolesPerLevel = self.values[idx]
				end,
				retrieve = function( self, options )
					return self.strings[ FindIndexOf(options.min_consolesPerLevel, self.values)]
				end,
			},
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.MAX_CONSOLES,
				tip = STRINGS.UI.DIFF_OPTION_CONSOLES_TIP,
				values = { 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 25 },
				strings = STRINGS.GEN_OPT.EXTENSIONS.EX_PROPS,
				requirements = {{mask="mask_random_consoles",requirement=true}},
				apply = function( self, options, idx )
					options.max_consolesPerLevel = self.values[idx]
				end,
				retrieve = function( self, options )
					return self.strings[ FindIndexOf(options.max_consolesPerLevel, self.values)]
				end,
			}
		},
		apply = function( self, options, idx )
            options.consolesPerLevel = self.values[idx]
            --options.powerPerLevel = tonumber(options.consolesPerLevel) * 2
        end,
        retrieve = function( self, options )
            return self.strings[ FindIndexOf(options.consolesPerLevel, self.values)]
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"SAFES PER LEVEL",{
    	name = STRINGS.UI.DIFF_OPTION_SAFES,
    	tip = STRINGS.UI.DIFF_OPTION_SAFES_TIP,
        values = { "RANDOM", 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 25 },
        strings = STRINGS.GEN_OPT.ORIGINAL.PROPSPAWNS,
        masks = {{mask="mask_random_safes",requirement="RANDOM"}},
		nestedOptions = {
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.MIN_SAFES,
				tip = STRINGS.UI.DIFF_OPTION_SAFES_TIP,
				values = { 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 25 },
				strings = STRINGS.GEN_OPT.EXTENSIONS.EX_PROPS,
				requirements = {{mask="mask_random_safes",requirement=true}},
				apply = function( self, options, idx )
					options.min_safesPerLevel = self.values[idx]
				end,
				retrieve = function( self, options )
					return self.strings[ FindIndexOf(options.min_safesPerLevel, self.values)]
				end,
			},
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.MAX_SAFES,
				tip = STRINGS.UI.DIFF_OPTION_SAFES_TIP,
				values = { 0, 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 25 },
				strings = STRINGS.GEN_OPT.EXTENSIONS.EX_PROPS,
				requirements = {{mask="mask_random_safes",requirement=true}},
				apply = function( self, options, idx )
					options.max_safesPerLevel = self.values[idx]
				end,
				retrieve = function( self, options )
					return self.strings[ FindIndexOf(options.max_safesPerLevel, self.values)]
				end
			},
		},
		apply = function( self, options, idx )
            options.safesPerLevel = self.values[idx]
        end,
        retrieve = function( self, options )
            return self.strings[ FindIndexOf(options.safesPerLevel, self.values)]
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"ROOMS IN LEVEL",{
    	name = STRINGS.UI.DIFF_OPTION_ROOMS,
    	tip = STRINGS.UI.DIFF_OPTION_ROOMS_TIP,
        values = { "RANDOM", 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 25, 27, 30, 32, 35, 37, 40 },
		strings = STRINGS.GEN_OPT.ORIGINAL.ROOMS,
        masks = {{mask="mask_random_rooms",requirement="RANDOM"}},
		nestedOptions = {
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.MIN_ROOMS,
				tip = STRINGS.UI.DIFF_OPTION_ROOMS_TIP,
				values = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 25, 27, 30, 32, 35, 37, 40 },
				requirements = {{mask="mask_random_rooms",requirement=true}},
				apply = function( self, options, idx )
					options.min_roomCount = self.values[idx]
				end,
				retrieve = function( self, options )
					return tostring(options.min_roomCount)
				end,
			},
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.MAX_ROOMS,
				tip = STRINGS.UI.DIFF_OPTION_ROOMS_TIP,
				values = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 25, 27, 30, 32, 35, 37, 40 },
				requirements = {{mask="mask_random_rooms",requirement=true}},
				apply = function( self, options, idx )
					options.max_roomCount = self.values[idx]
				end,
				retrieve = function( self, options )
					return tostring(options.max_roomCount)
				end
			},
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.ROOMS_SCALE,
				tip = STRINGS.GEN_OPT.EXTENSIONS.ROOMS_SCALE_DESC,
				values = { "RANDOM", -1, -0.75, -0.5, -0.3,-0.25, -0.2,-0.1, 0, 0.1, 0.2, 0.25, 0.3, 0.4, 0.5, 0.6, 0.7, 0.75, 0.8, 0.9, 1,1.25, 1.5, 1.75, 2 },
				masks = {{mask="mask_random_rooms_scaling",requirement="RANDOM"}},
				apply = function( self, options, idx )
					options.roomCount_scaling = self.values[idx]
				end,
				retrieve = function( self, options )
					return tostring(options.roomCount_scaling)
				end
			},
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.ROOMS_SCALE_MIN,
				tip = STRINGS.GEN_OPT.EXTENSIONS.ROOMS_SCALE_DESC,
				values = { -1, -0.75, -0.5, -0.3,-0.25, -0.2,-0.1, 0, 0.1, 0.2, 0.25, 0.3, 0.4, 0.5, 0.6, 0.7, 0.75, 0.8, 0.9, 1,1.25, 1.5, 1.75, 2 },
				requirements = {{mask="mask_random_rooms_scaling",requirement=true}},
				apply = function( self, options, idx )
					options.min_roomCount_scaling = self.values[idx]
				end,
				retrieve = function( self, options )
					return tostring(options.min_roomCount_scaling)
				end,
			},
			{
				name = STRINGS.GEN_OPT.EXTENSIONS.ROOMS_SCALE_MAX,
				tip = STRINGS.GEN_OPT.EXTENSIONS.ROOMS_SCALE_DESC,
				values = { -1, -0.75, -0.5, -0.3,-0.25, -0.2,-0.1, 0, 0.1, 0.2, 0.25, 0.3, 0.4, 0.5, 0.6, 0.7, 0.75, 0.8, 0.9, 1,1.25, 1.5, 1.75, 2 },
				requirements = {{mask="mask_random_rooms_scaling",requirement=true}},
				apply = function( self, options, idx )
					options.max_roomCount_scaling = self.values[idx]
				end,
				retrieve = function( self, options )
					return tostring(options.max_roomCount_scaling)
				end
			},
		},
		apply = function( self, options, idx )
            options.roomCount = self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.roomCount)
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"GUARDS PER LEVEL",{
    	name = STRINGS.UI.DIFF_OPTION_GUARDSPAWNS,
    	tip = STRINGS.UI.DIFF_OPTION_GUARDSPAWNS_TIP,
        values = { "NONE", "FEW", "LESS", "NORMAL", "MORE", "MANY", "SWARMING" },
		strings = STRINGS.GEN_OPT.ORIGINAL.GUARDSPAWNS,
		masks = {{mask="mask_random_guards",requirement="RANDOM"}},
        apply = function( self, options, idx )
            options.spawnTable = self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.spawnTable)
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"DAEMON QUANTITY",{
    	name = STRINGS.UI.DIFF_OPTION_DAEMONS,
    	tip = STRINGS.UI.DIFF_OPTION_DAEMONS_TIP,
        values = { "NONE", "FEW", "LESS", "NORMAL", "MORE", "MANY", "VERY MANY" },
		strings = STRINGS.GEN_OPT.ORIGINAL.DAEMONS,
		masks = {{mask="mask_random_daemons",requirement="RANDOM"}},
        apply = function( self, options, idx )
            options.daemonQuantity = self.values[idx]
        end,
        retrieve = function( self, options )
            return tostring(options.daemonQuantity)
        end
    })
	rawset(simdefs.CUSTOM_OPTIONS,"TIME ATTACK MODE",{
		name = STRINGS.UI.DIFF_OPTION_TIMEATTACK,
		tip = STRINGS.UI.DIFF_OPTION_TIMEATTACK_TIP,
		strings = STRINGS.GEN_OPT.ORIGINAL.TIMEATTACK,
		values = { 0, 10*cdefs.SECONDS, 15*cdefs.SECONDS, 20*cdefs.SECONDS, 30*cdefs.SECONDS, 45*cdefs.SECONDS, 60*cdefs.SECONDS, 1.25*60*cdefs.SECONDS, 1.5*60*cdefs.SECONDS, 1.75*60*cdefs.SECONDS, 2*60*cdefs.SECONDS, 2.25*60*cdefs.SECONDS, 2.5*60*cdefs.SECONDS, 3*60*cdefs.SECONDS, 3*60*cdefs.SECONDS, 4*60*cdefs.SECONDS, 5*60*cdefs.SECONDS, 6*60*cdefs.SECONDS, 7*60*cdefs.SECONDS, 8*60*cdefs.SECONDS, 9*60*cdefs.SECONDS, 10*60*cdefs.SECONDS },
		masks = {{mask="mask_random_time_attack",requirement="RANDOM"}},
		apply = function( self, options, index )
			options.timeAttack = self.values[index]
		end,
		retrieve = function( self, options )
			return self.strings[FindIndexOf(options.timeAttack, self.values)]
		end
	})
end

local function earlyInit( modApi )
	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/procgen_context" )
end

local function load( modApi, options, params )
	local serverdefs = include("modules/serverdefs")

    local scriptPath = modApi:getScriptPath()
	if params then
		for option, value in pairs(options) do
			params[option] = value.value or value.enabled
		end
	end
	for name, daemonAbility in pairs(include(scriptPath.."/npc_abilities")) do
		modApi:addDaemonAbility( name, daemonAbility )
	end
	if options.sideMissions then
		SIDEMISSIONODDS = options.sideMissions.value
	else
		SIDEMISSIONODDS = 25
	end
	if options.sphericalEarth and options.sphericalEarth.enabled then
		rawset(simdefs,"sphericalEarth",true)
	end
	if options.hourMultiplier then
		rawset(simdefs,"hourMultiplier",options.hourMultiplier.value)
	else
		rawset(simdefs,"hourMultiplier",nil)
	end
	if options.baseTraveltime then
		rawset(simdefs,"baseTraveltime",options.baseTraveltime.value)
	else
		rawset(simdefs,"baseTraveltime",nil)
	end
	if options.travelCeiling then
		serverdefs.TRAVEL_CEILING = options.travelCeiling.value
	else
		serverdefs.TRAVEL_CEILING = 12
	end
	
	if options.agencyInventorySize then
		rawset(simdefs,"agencyInventorySize",options.agencyInventorySize.value)
	else
		rawset(simdefs,"agencyInventorySize",4)
	end
	
	if options.newday then
		rawset(simdefs,"NUM_MISSIONS_TO_SPAWN",options.newday.value)
	else
		rawset(simdefs,"NUM_MISSIONS_TO_SPAWN",4)
	end
	
	if options.magnetic_reinforcement then
		local NORMAL = {}
		
		for i = 1, 20 do
			if i >= options.magnetic_reinforcement_day.value then
				NORMAL[i] = options.magnetic_reinforcement.value + math.floor(i / options.magnetic_reinforcement_step.value) * options.magnetic_reinforcement_scaling.value
			else
				NORMAL[i] = 0
			end
		end
		
		rawset(simdefs,"MAGNETIC_REINFORCEMENTS_CHANCE",{
			NORMAL = NORMAL
		})
	else
		rawset(simdefs,"MAGNETIC_REINFORCEMENTS_CHANCE",{
			NORMAL = {
				[1] = 0, 
				[2] = 0, 
				[3] = 0, 
				[4] = 0,
				[5] = 0,
				[6] = 0.5, 
				[7] = 0.5, 
				[8] = 0.75, 
				[9] = 0.75, 
				[10] = 1,
				
				[11] = 1, 
				[17] = 1, 
				[12] = 1, 
				[13] = 1, 
				[14] = 1,
				[15] = 1,
				[16] = 1, 
				[18] = 1, 
				[19] = 1, 
				[20] = 1,	
			},
		})
	end
	
	if params then
		if options.finaleDifficulty and options.finaleDifficulty.value ~= "DEFAULT" then
			params.finalMissionDifficulty = options.finaleDifficulty.value
			params.removeMinDifficultyFromFinalMission = true
		end
		
		if options.customFinaleDifficulty and options.customFinaleDifficulty.value ~= "DEFAULT" then
			params.customFinalDifficulty = options.customFinaleDifficulty.value
			params.customEndlessDifficulty = options.customFinaleDifficulty.value
		end
	end
	
	rawset(simdefs,"MAX_DIFFICULTY",4)
	if options.difficultyFix and options.difficultyFix.enabled then
		rawset(simdefs,"MAX_DIFFICULTY",math.huge)
	end
	
	for k, event in pairs(modApi.mod_manager:getCampaignEvents()) do
		if event.eventType == simdefs.CAMPAIGN_EVENTS.ADD_MORE_HOURS and event.mission == "mid_1" and options.extendedcampaign then
			event.data = 24 + options.extendedcampaign.value
		end
	end
end

local function unload( modApi, params )
	local serverdefs = include("modules/serverdefs")
	SIDEMISSIONODDS = 25
	serverdefs.TRAVEL_CEILING = 12
	rawset(simdefs,"baseTraveltime",nil)
	rawset(simdefs,"hourMultiplier",nil)
	rawset(simdefs,"agencyInventorySize",4)
	rawset(simdefs,"NUM_MISSIONS_TO_SPAWN",4)
	rawset(simdefs,"MAX_DIFFICULTY",4)
	rawset(simdefs,"MAGNETIC_REINFORCEMENTS_CHANCE",{
		NORMAL = {
			[1] = 0, 
			[2] = 0, 
			[3] = 0, 
			[4] = 0,
			[5] = 0,
			[6] = 0.5, 
			[7] = 0.5, 
			[8] = 0.75, 
			[9] = 0.75, 
			[10] = 1,
			
			[11] = 1, 
			[17] = 1, 
			[12] = 1, 
			[13] = 1, 
			[14] = 1,
			[15] = 1,
			[16] = 1, 
			[18] = 1, 
			[19] = 1, 
			[20] = 1,	
		},
	})
end

local function earlyLoad( modApi, options )
end

local function lateUnload( modApi )
	for id, itemDef in pairs(include("sim/unitdefs/itemdefs")) do
		if itemDef.oldFloorWeight then
			itemDef.floorWeight = itemDef.oldFloorWeight
			itemDef.oldFloorWeight = nil
		end
	end
end

local function removeDLCFromList(list)
	for i = #list, 1, -1 do
		local itemDef = list[i]
		if itemDef.traits.dlcFooter and itemDef.traits.dlcFooter[1] == "Contingency Plan" then
			table.remove(list,i)
			if itemDef.floorWeight and itemDef.floorWeight > 2 then
				itemDef.oldFloorWeight = itemDef.floorWeight
				itemDef.floorWeight = 1
			end
		end
	end
end

local function lateLoad( modApi, options, params )
	lateUnload( modApi )
	if options.disableCpItems and options.disableCpItems.enabled then
		local simstore = include( "sim/units/store" )
		
		removeDLCFromList(simstore.STORE_ITEM.itemList)
		removeDLCFromList(simstore.STORE_ITEM.augmentList)
		removeDLCFromList(simstore.STORE_ITEM.weaponList)
		
		for id,itemDef in pairs(include("sim/unitdefs/itemdefs")) do
			if itemDef.traits.dlcFooter and itemDef.traits.dlcFooter[1] == "Contingency Plan" then
				if itemDef.floorWeight and itemDef.floorWeight > 2 then
					itemDef.oldFloorWeight = itemDef.floorWeight
					itemDef.floorWeight = 1
				end
			end
		end
	end
end

local function lateInit(modApi)
	local scriptPath = modApi:getScriptPath()
	include(scriptPath.."/worldgen")
end

local function initStrings( modApi )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()

    local DLC_STRINGS = include( scriptPath .. "/strings" )
    modApi:addStrings( dataPath, "GEN_OPT", DLC_STRINGS ) 
	SIDEMISSIONODDS	= 25
end

return {
	lateInit = lateInit,
    init = init,
    load = load,
    initStrings= initStrings,
	earlyInit = earlyInit,
	lateUnload = lateUnload,
	lateLoad = lateLoad,
	unload = unload,
}