local util = include( "modules/util" )
local serverdefs = include( "modules/serverdefs" )
local simdefs = include( "sim/simdefs" )

local EarlyAccessEnabled = true
local showloadingtips = 0

local LOADING_IMAGES =
{
	{ png="airplane", world="any" },
	{ png="infiltration", world="any" },
	{ png="nika", world="any" },
	{ png="VTOL_matte", world="any" },
	{ png="tony_matte", world="any" },
	{ png="KO_matte", world="ko" },
	{ png="prism_hologram", world="ko" },
	{ png="Plastek_matte", world="plastech" },
	{ png="Droid_matte", world="sankaku" },
}
local LOADING_IMAGES_EARLYACCESS =
{
	{ png="up_02_Active_Recon", world="ftm", titleBG=.7 },
	{ png="up_03_Grand_Bazaar", world="any", titleBG=.7 },
	{ png="up_06_Memory_Check", world="any", titleBG=.66 },
	{ png="up_08_Operation_Sherwood", world="any", titleBG=.66 },
	{ png="up_10_Deep_Breath", world="any", titleBG=.7 },
}

-- local function printchildren(widget)
	-- for k, v in pairs(widget._children) do
		-- log:write("child of ".. widget._name ..": ".. v._name)
		-- if v._children then
			-- printchildren(v)
		-- end
	-- end
-- end

local function init( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	
	KLEIResourceMgr.MountPackage( dataPath .. "/loading_screens_EA.kwad", "data" ) 
	
	modApi:addGenerationOption("ealoadingscreens", "Early Access Posters", "Adds official early access update posters to the loading screens.", {noUpdate = true})
	modApi:addGenerationOption("showloadingtips", "Loading Tips Bar", "'Uniform' uses default black bar.\n'Normal' - darker values for added posters.\n'Bright' makes the black bar less black.\n'Transparent' hides the black bar.\n'No Loading Tips' hides Tips entirely.", {noUpdate = true,
		values = {0,1,2,3,4},
		value = 1,
		strings = {"Uniform","Normal","Bright","Transparent","No Loading Tips"},
	})
	-- modApi:addGenerationOption("ealoadingscreens", "Early Access Posters", "Adds official early access update posters to the loading screens.", {noUpdate = true})

	--the code
	local state = include( "states/state-loading" )
	local onLoad_old = state.onLoad 
	state.onLoad = function( self, ... )
		onLoad_old( self, ... )
		if (EarlyAccessEnabled or not showloadingtips) and self.loadThread and self.loadParams then --when adding new options, test if any are active in advance here
			local params = self.loadParams[2]
			local campaign = self.loadParams[4]
			if type(params) == "table" and type(campaign) == "table" then --additional failsafe
				--wrap the coroutine
				local loadThread_old = self.loadThread
				self.loadThread = coroutine.create( function()
					--first, let the original coroutine initialise stuff
					local ok, result = coroutine.resume( loadThread_old, unpack(self.loadParams) )
					
					--Error handling
					if not ok then
						--Since this only runs once and then dies, unwrap the original coroutine again.
						--Not that it matters much at this point...
						self.loadThread = loadThread_old
						--Copying the error handling hides this mod from the pop-up, but that's a lot of locals and junk to copy. -M
						-- util.callDelegate( self.errorFn or onLoadError, self, result )
						--Instead, just crash this coroutine and append the actual error. Much easier for us! -M
						assert(ok, "+Loading Screens has detected an error caused by another mod:\n".. tostring(result))
					end
					
					--now do our stuff
					local corpData_world = params.world and serverdefs.CORP_DATA[ params.world ] and serverdefs.CORP_DATA[ params.world ].world
					if not campaign
					or (corpData_world ~= "omni" and corpData_world ~= "omni2"
					and campaign.campaignDifficulty ~= simdefs.TUTORIAL_DIFFICULTY 
					and campaign.missionCount ~= 0) then
						--only roll a random screen if the above conditions check out
						local images = {}
						for _, v in ipairs(LOADING_IMAGES) do
							if v.world == "any" or (v.world == corpData_world) then
								table.insert( images, v )
							end
						end
						if EarlyAccessEnabled then
							for _, v in ipairs(LOADING_IMAGES_EARLYACCESS) do
								if v.world == "any" or (v.world == corpData_world) then
									table.insert( images, v )
								end
							end
						end

						local idx = math.random(1,#images)
						state.screen.binder.loadingBG:setImage( "gui/menu pages/loading/"..images[idx].png..".png")
						if images[idx].titleBG and self.screen:findWidget( "titleBG" )._color then
							--default is roughly .58
							self.screen:findWidget( "titleBG" ):setColor( 0, 0, 0, images[idx].titleBG )
						end
					end
					-- log:write("PATH DUMP")
					-- log:write(self.screen:findWidget( "titleBG" ):getPath())
					-- log:write(self.screen:findWidget( "tipTxt" ):getPath())
					-- log:write(self.screen:findWidget( "TextGroup" ):getPath())
					-- log:write(self.screen:findWidget( "corpLogo" ):getPath())
					local titleBG = self.screen:findWidget( "titleBG" )
					if titleBG then
						if showloadingtips == 0 then						-- Uniform vanilla
							titleBG:setColor( 0, 0, 0, .58)
						elseif showloadingtips == 2 and titleBG._color then			-- Bright
							titleBG:setColor( 0, 0, 0, (titleBG._color.a or .58) * .5 )
						elseif showloadingtips == 3 then					-- Transparent
							titleBG:setVisible(false)
						elseif showloadingtips == 4 then					-- No tips
							self.screen:findWidget( "TextGroup" ):setVisible(false)
							titleBG:setVisible(false)
						end
					end
					--since this only runs once and then dies, unwrap the original coroutine again
					self.loadThread = loadThread_old
				end)
			end
		end
	end
	--

end

local function load( modApi, options )
	EarlyAccessEnabled = not options.ealoadingscreens or options.ealoadingscreens.enabled
	showloadingtips = options.showloadingtips and options.showloadingtips.value or 0
end
local function unload( modApi, options )
	EarlyAccessEnabled = false
	showloadingtips = 0
end


return {
	init = init,
	load = load,
	unload = unload,
}