local util = include("client_util")

local function earlyInit(modApi)
	modApi.requirements =
	{
		"Sim Constructor",
		"Function Library",
		"Contingency Plan",
		"Programs Extended",
	}
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local SITREP_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "SITREP", SITREP_STRINGS )
end

local function init( modApi )
	local scriptPath = modApi:getScriptPath()
	local dataPath = modApi:getDataPath()
	reinclude = include

	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	
	modApi:addGenerationOption( "INCOGNITA_UPGRADE", STRINGS.SITREP.OPTIONS.INCOGNITA_UPGRADE, STRINGS.SITREP.OPTIONS.INCOGNITA_UPGRADE_DESC, { noUpdate = true,} )
	modApi:addGenerationOption( "SLOT_OVERWRITE", STRINGS.SITREP.OPTIONS.SLOT_OVERWRITE, STRINGS.SITREP.OPTIONS.SLOT_OVERWRITE_DESC, { noUpdate = true,} )
	modApi:addGenerationOption( "SITREP", STRINGS.SITREP.OPTIONS.SITREP, STRINGS.SITREP.OPTIONS.SITREP_DESC, { noUpdate = true,} )
	modApi:addGenerationOption( "SITREP_DAY", STRINGS.SITREP.OPTIONS.SITREP_STARTING_DAY, STRINGS.SITREP.OPTIONS.SITREP_STARTING_DAY_DESC, {values={0,1,2,3,4}, value = 0, noUpdate = true, strings = {"1", "2", "3", "4", "5"} } )
	modApi:addGenerationOption( "SITREP_INCREASE", STRINGS.SITREP.OPTIONS.SITREP_DAY_INCREASE, STRINGS.SITREP.OPTIONS.SITREP_DAY_INCREASE_DESC, {values={1,2,3}, value = 1, noUpdate = true, strings = {"1", "2", "3"} } )
	modApi:addGenerationOption( "INCOGNITA_DAY", STRINGS.SITREP.OPTIONS.INCOGNITA_PENALTY_STARTING_DAY, STRINGS.SITREP.OPTIONS.INCOGNITA_PENALTY_STARTING_DAY_DESC, {values={-1,0,1,2,3,4,5,6,7,8,9}, value = 4, noUpdate = true, strings = {"OFF", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"} } )
	
	
	include( scriptPath.."/serverdefs" )
	include( scriptPath.."/simplayer" )
	include( scriptPath.."/mission_scoring" )
	include( scriptPath.."/simparams" )
	include( scriptPath.."/screens/shop_panel" )
	include( scriptPath.."/simactions" )
	include( scriptPath.."/screens/state-upgrade-screen" )
	include( scriptPath.."/locationpopup" )
	include( scriptPath.."/mainframe_common" )
	include( scriptPath.."/mainframe" )
	include( scriptPath.."/engine" )
	include( scriptPath.."/screens/state-map-screen" )
	include( scriptPath.."/inventory" )
	include( scriptPath.."/mission_util" )
end

local function unload( modApi, mod_options, params )
	if params then
		params.SITREP_program_storage = false
	end
end

local function earlyLoad ( modApi, options, params, mod_options )
	
end

local function load( modApi, options, params, mod_options )
	local scriptPath = modApi:getScriptPath()
	local dataPath = modApi:getDataPath()
	
	package.loaded[scriptPath .. "/serverdefs"] = nil
	include( scriptPath.."/serverdefs" )
	
	local serverdefs = include( "modules/serverdefs" )
	
	local mission_init = include( scriptPath .. "/mission_init" )
	modApi:addEscapeScripts(mission_init)
	
	modApi:insertUIElements( include( scriptPath .. "/screen_inserts" ) )
	
	package.loaded[scriptPath .. "/skilldefs"] = nil
	include( scriptPath.."/skilldefs" )
	--PE Incognita skills--
	local PE = mod_manager:findModByName("Programs Extended")
	if PE and mod_options[PE.id] and mod_options[PE.id].enabled then
		if mod_options[PE.id].options.counter_ai and mod_options[PE.id].options.counter_ai.value >= 0 then
			package.loaded[scriptPath .. "/skilldefs_PE"] = nil
    			include( scriptPath.."/skilldefs_PE" )
		end
	end
	
	--Tooltips--
	modApi:addTooltipDef( include( scriptPath .. "/tooltips" ) )
	
	-----------------
	----Abilities----
	-----------------
	
	------------------------------
	----sitrep daemon abilities----
	------------------------------
	local sitrep_daemons = include( scriptPath .. "/abilities/sitrep_npc_abilities" )
	for name, daemon in pairs(sitrep_daemons) do
		serverdefs.SITREP_DAEMONS[name] = daemon
		modApi:addDaemonAbility( name, daemon )
	end

	modApi:addAbilityDef( "sitrep_failsafe", scriptPath .. "/abilities/sitrep_failsafe" )
	modApi:addAbilityDef( "sitrep_chronjob", scriptPath .. "/abilities/sitrep_chronjob" )
	modApi:addAbilityDef( "sitrep_daemonitor", scriptPath .. "/abilities/sitrep_daemonitor" )
	
	----------------------
	
	--------------------------
	----Situation modifier----
	--------------------------
	local situation_modifier = include( scriptPath .. "/abilities/situation_modifiers" )
	for name, modifier in pairs(situation_modifier) do
		serverdefs.SITUATION_MODIFIERS[name] = modifier
	end
	--------------------------
	
	---------------------------
	----Incognita penalties----
	---------------------------
	local incognita_penalty = include( scriptPath .. "/abilities/npc_ipenalty_abilities" )
	for name, penalty in pairs(incognita_penalty) do
		serverdefs.INCOGNITA_PENALTIES[name] = penalty
		modApi:addDaemonAbility( name, penalty )
	end
	--------------------------
	
	if params then
		params.SITREP_program_storage = true
		
		local AR = mod_manager:findModByName("Agent Reserve")
		if AR and mod_options[AR.id] and mod_options[AR.id].enabled and mod_options[AR.id].options.reserve and mod_options[AR.id].options.reserve.enabled then
			if options["SITREP"].enabled then
				params.SITREP = true
				params.SITREP_day = options["SITREP_DAY"].value
				params.SITREP_increase = options["SITREP_INCREASE"].value
				
				params.SITREP_incognita_day = options["INCOGNITA_DAY"].value
			end
		end
		
		if options["SLOT_OVERWRITE"].enabled then
			params.SITREP_slot = true
		end
		
		if options["INCOGNITA_UPGRADE"].enabled then
			params.SITREP_incognita_upgrade = true
		end
	end
end

local function lateLoad( modApi, options, params, mod_options )
	
end

return {
	init = init,
	load = load,
	unload = unload,
	initStrings = initStrings,
	earlyInit = earlyInit,
	earlyLoad = earlyLoad,
	lateLoad = lateLoad,
}
