-- Include files to be used throughout this file.
-- Don't eager-load sim- or client-heavy things though, those may break translation.
-- local util = include( "modules/util" )

local function earlyInit(modApi)
    modApi.requirements = {
        "Contingency Plan",
        "Sim Constructor",
        "Function Library"
    }

    do -- SimConstructor does not supply a dependency system. A variable helps detect this mod.
        local cdefs = include("client_defs")
        cdefs.ActionToolkitLoaded = true
    end

    local scriptPath = modApi:getScriptPath()

    do -- This makes the game not wait to finish animations (partly implemented in modinit.load)
        local simdefs = include("sim/simdefs")
        simdefs.EV_NONBLOCKING_EVENT = "EV_NONBLOCKING_EVENT"
    end

    do
        local targeting = include("hud/targeting")
        for k, v in pairs(include(scriptPath .. "/targeting")) do
            targeting[k] = v
        end
    end

    do
        local mathutil = include("modules/mathutil")
        for k, v in pairs(include(scriptPath .. "/mathutil")) do
            mathutil[k] = v
        end
    end

    rawset(_G, "dashabilityutil", include(scriptPath .. "/dashabilityutil"))
end

local function init(modApi)
end

local function load(modApi, options, params)
    do -- This makes the game not wait to finish animations (partly implemented in modinit.init)
        -- https://discord.com/channels/313015356052471828/313015598789427202/1008400149309051001
        local simdefs = include("sim/simdefs")
        local function nonBlockingEvent(event, eventType, eventData, boardRig, hud, vizThread)
            vizThread:unblock()
            boardRig:onSimEvent(event, eventData.actualEventType, eventData)
        end
        modApi:addVizEvHandler(simdefs.EV_NONBLOCKING_EVENT, nonBlockingEvent)
    end

    local scriptPath = modApi:getScriptPath()
    include(scriptPath .. "/units/simmainframeunit")
end

local function unload(modApi, options)
    load(modApi, options) -- Unloading at this point makes no sense, so don't.
end

local function initStrings(modApi)
end

return {
    init = init,
    earlyInit = earlyInit,
    load = load,
    unload = unload,
    initStrings = initStrings
}
