local util = include( "modules/util" )

local function earlyInit(modApi)
	modApi.requirements = {"Sim Constructor"}
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})

	local scriptPath = modApi:getScriptPath()
end
-- init will be called once
local function init(modApi)
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})
	local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	
	modApi:addGenerationOption("advection_programs", STRINGS.AFPROGRAMS.FRONTIER.NAME, STRINGS.AFPROGRAMS.FRONTIER.DESC, {noUpdate = true})
end

local function initStrings(modApi)
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "AFPROGRAMS", DLC_STRINGS )
end

-- load will be refreshed and run each time a new campaign is started
local function load(modApi, options, params)
    local scriptPath = modApi:getScriptPath()
	
	local mainframe_abilities = include( scriptPath .. "/mainframe_abilities" )
	local serverdefs = include( scriptPath .. "/serverdefs" )
	
	if options["advection_programs"] and options["advection_programs"].enabled then
		for name, ability in pairs(mainframe_abilities.mainframe_abilities) do
			modApi:addMainframeAbility( name, ability )
		end
			
		for i,program in ipairs(serverdefs.SELECTABLE_PROGRAMS[1]) do 
            modApi:addStartingGenerator( program )        
        end
	end
end

return 
{
	earlyInit = earlyInit,
	init = init,
	load = load,
	initStrings = initStrings,
}