local util = include("modules/util")
local serverdefs = include( "modules/serverdefs" ) 

local function init( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

    KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
    KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )

	modApi:addGenerationOption("wraith_agent", STRINGS.APEX.OPTIONS.WRAITH , STRINGS.APEX.OPTIONS.WRAITH_TIP, { 
		enabled = true,
		noUpdate = true,
	})
	modApi:addGenerationOption("loba_agent", STRINGS.APEX.OPTIONS.LOBA , STRINGS.APEX.OPTIONS.LOBA_TIP, { 
		enabled = true,
		noUpdate = true,
	})
	modApi:addGenerationOption("wraith_rescuable", STRINGS.APEX.OPTIONS.WRAITH_RESCUABLE, STRINGS.APEX.OPTIONS.WRAITH_RESCUABLE_TIP, {
		enabled = true,
		noUpdate = true,
	})
	modApi:addGenerationOption("loba_rescuable", STRINGS.APEX.OPTIONS.LOBA_RESCUABLE, STRINGS.APEX.OPTIONS.LOBA_RESCUABLE_TIP, {
		enabled = true,
		noUpdate = true,
	})

	include(scriptPath.."/targeting")
end

local function loadLegend ( modApi, name, agentdef, archivedefs )
	-- process prereqs if necessary
	local shorthand = string.gsub(string.lower(name), "apex_", "")
	if agentdef.prereqs then
		local scriptPath = modApi:getScriptPath()
		for prereq, reqFunc in pairs(agentdef.prereqs) do
			local file = include(scriptPath.."/"..shorthand.."/"..prereq)
			if reqFunc == "addAbilityDef" and type(name) == "string" then
				local abilitydefs = include( "sim/abilitydefs" )
				for abilityname, ability in pairs(file) do
					abilitydefs._abilities[ abilityname ] = ability
				end
			else
				for reqname, req in pairs(file) do
					--log:write("[APEXLOG] Calling function "..reqFunc..", state of modApi: "..util.stringize(modApi, 2))
					modApi[reqFunc]( modApi, reqname, req )
				end
			end
			log:write("[APEXLOG] Added prereq {"..scriptPath.."/"..shorthand.."/"..prereq.."} using function "..reqFunc)
		end
	end
	-- load 'em up
	modApi:addAgentDef( name, agentdef )
	table.insert( serverdefs.SELECTABLE_AGENTS, name )

	if not serverdefs.LOADOUTS[name] then
		serverdefs.LOADOUTS[name] = {}
	end
	table.insert(serverdefs.LOADOUTS[ name ], name)
	if archivedefs then
		for lname, archdef in pairs(archivedefs) do
			modApi:addAgentDef( lname, archdef )
			table.insert(serverdefs.SELECTABLE_AGENTS, lname)
			table.insert(serverdefs.LOADOUTS[ name ], lname)
		end
	end
	log:write("[APEXLOG] "..shorthand:gsub("^%l", string.upper).." locked and loaded.")
end

local function load( modApi, options, params )
	local scriptPath = modApi:getScriptPath()


	-- The Smartlink
	-- loads only relevant legend parts based on genoption choices

	local agentdefs = include(scriptPath.."/agentdefs")
	-- fetch relevant agentdefs (there may be multiple)
	local onfileDef, targetdefs = nil, {}
	local agentsToLoad = {}
	for name, agentDef in pairs(agentdefs) do
		-- load agentdef and prereqs
		for optname, opt in pairs(options) do
			if string.find(optname, string.gsub(string.lower(name), "apex_", "")) and string.find(optname, "agent") and opt.enabled then
				if name == agentDef.agentID then
					agentsToLoad[name] = {}
					agentsToLoad[name]["onfileDef"] = agentDef
				else
					table.insert(agentsToLoad[name]["otherDefs"], agentDef)
				end
			end
		end
	end

	for agentName, agent in pairs(agentsToLoad) do
		local onfileDef = agent["onfileDef"]
		local targetdefs = agent["otherDefs"]
		loadLegend( modApi, agentName, onfileDef, targetdefs)
	end

	local agentdefs_loaded = include( "sim/unitdefs/agentdefs" )
	for name, agentDef in pairs(agentdefs_loaded) do
		-- set rescuable if needed
		for optname, opt in pairs(options) do
			if string.find(optname, string.lower(string.gsub(name, "APEX_", ""))) and string.find(optname, "rescuable") and opt.enabled then
				local serverdefs_extension = include( scriptPath .. "/serverdefs" )
				for templateName, equipment in ipairs(serverdefs_extension.RAW_TEMPLATE_AGENCY.unitDefsPotential) do
					local unitDef = serverdefs.createAgent( templateName, equipment )
					if unitDef then
						modApi:addRescueAgent( unitDef )
						log:write("[APEXLOG] Added "..templateName.." (version of  "..name..") to detention centers.")
					end
				end
			end
		end
	end
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "APEX", DLC_STRINGS )    
end

return {
	init = init,
	load = load,
	initStrings = initStrings,
}