----------------------------------------------------------------------
-- NEW CORP TACTICS for Invisible Inc.
--
-- This file is responsible for initializing the mod and telling the main program what's up

----------------------------------------------------------------------
-- Local functions defs, defined before load(), unload(), etc.
--

----------------------------------------------------------------------
-- Functions called by the modApi
--

-- init will be called once (technically, it gets called twice because Invisible Inc. 'proofchecks' mods on startup)
local function init(modApi)
	log:write("New Corporate Tactics - v1.5.5 -- RELEASE")
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	-- Mount data.
	KLEIResourceMgr.MountPackage(dataPath.."/gui.kwad", "data")

	-- Get feature information (only required for features that have generation options with multiple values)
	local spotLootedSafes = include(scriptPath.."/features/spotLootedSafes")

	-- Add generation options
	-- Agent modifying options
	modApi:addGenerationOption("nct_surrender", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SURRENDER, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SURRENDER_TIP, {noUpdate = true})
	modApi:addGenerationOption("nct_wounded", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_WOUNDED, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_WOUNDED_TIP,
	{
		values = {0, 1, 2, 3, 4, 8, 12, 100},
		value = 2,
		strings = {"None", "1", "2", "3", "4", "8", "12", "100"},
		noUpdate = true
	})
	modApi:addGenerationOption("nct_woundedMax", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_WOUNDED_MAX, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_WOUNDED_MAX_TIP,
	{
		values = {1, 2, 3, 4, 5, 6, 7, 8, 12, 100},
		value = 4,
		strings = {"1", "2", "3", "4", "5", "6", "7", "8", "12", "100"},
		noUpdate = true
	})
	-- Guard modifying options
	modApi:addGenerationOption("nct_dragAgents", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_DRAG_AGENTS, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_DRAG_AGENTS_TIP, {noUpdate = true})
	modApi:addGenerationOption("nct_ignoreMissionCriticalAgents", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_IGNORE_MISSION_CRITICAL_AGENTS, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_IGNORE_MISSION_CRITICAL_AGENTS_TIP, {noUpdate = true})
	-- modApi:addGenerationOption("nct_fairDrag", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_FAIR_DRAG, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_FAIR_DRAG_TIP)
	modApi:addGenerationOption("nct_koDragCost", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_KO_DRAG_COST, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_KO_DRAG_COST_TIP,
	{
		values = {0, 1, 2, 3, 4},
		value = 2,
		strings = {"None", "1", "2", "3", "4"},
		noUpdate = true
	})
	modApi:addGenerationOption("nct_spotSafesWhenLooted", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_WHEN_LOOTED, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_WHEN_LOOTED_TIP,
	{
		values =
		{
			spotLootedSafes.RESPONSE_VALUES.NONE,
			spotLootedSafes.RESPONSE_VALUES.INVESTIGATE,
			spotLootedSafes.RESPONSE_VALUES.ALERT
		},
		value = spotLootedSafes.RESPONSE_VALUES.INVESTIGATE,
		strings =
		{
			STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_RESPONSE_VALUES.NONE,
			STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_RESPONSE_VALUES.INVESTIGATE,
			STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_RESPONSE_VALUES.ALERT
		},
		noUpdate = true
	})
	modApi:addGenerationOption("nct_spotLootedSafes", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_LOOTED_SAFES, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_LOOTED_SAFES_TIP,
	{
		values =
		{
			spotLootedSafes.RESPONSE_VALUES.NONE,
			spotLootedSafes.RESPONSE_VALUES.INVESTIGATE,
			spotLootedSafes.RESPONSE_VALUES.ALERT
		},
		value = spotLootedSafes.RESPONSE_VALUES.NONE,
		strings =
		{
			STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_RESPONSE_VALUES.NONE,
			STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_RESPONSE_VALUES.INVESTIGATE,
			STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_SPOT_SAFES_RESPONSE_VALUES.ALERT
		},
		noUpdate = true
	})
	--modApi:addGenerationOption("nct_guardsHideOnAlert", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_GUARDS_HIDE_ON_ALERT, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_GUARDS_HIDE_ON_ALERT_TIP, {value = false, noUpdate = true})
	--modApi:addGenerationOption("nct_guardsPreferCover", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_GUARDS_PREFER_COVER, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_GUARDS_PREFER_COVER_TIP, {value = false, noUpdate = true})
	modApi:addGenerationOption("nct_civiliansAlertGuards", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_CIVILIANS_ALERT_GUARDS, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_CIVILIANS_ALERT_GUARDS_TIP, {noUpdate = true})
	-- Camera modifying options
	modApi:addGenerationOption("nct_trackingCameras", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_TRACKING_CAMERAS, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_TRACKING_CAMERAS_TIP, {noUpdate = true})
	modApi:addGenerationOption("nct_camerasDetectDownedGuards", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_CAMERAS_DETECT_DOWNED_GUARDS, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_CAMERAS_DETECT_DOWNED_GUARDS_TIP, {noUpdate = true})
	modApi:addGenerationOption("nct_cameraActivationCount", STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_CAMERA_ACTIVATION_COUNT, STRINGS.NEW_CORP_TACTICS.OPTIONS.ENABLE_CAMERA_ACTIVATION_COUNT_TIP,
	{
		values =
		{
			1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 100
		},
		value = 1,
		strings =
		{
			"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "100"
		},
		noUpdate = true
	})

	-- Initialize features (not all features need initializing)
	include(scriptPath.."/features/dragAgents")
	local surrender = include(scriptPath.."/features/surrender")
	surrender.init(scriptPath)
end

-- load will be refreshed and run each time a new campaign is started
local function load(modApi, options, params, mod_options)
	local scriptPath = modApi:getScriptPath()
	local surrender = include(scriptPath.."/features/surrender")
	local dragAgents = include(scriptPath.."/features/dragAgents")
	local spotLootedSafes = include(scriptPath.."/features/spotLootedSafes")
	--local guardsUseCover = include(scriptPath.."/features/guardsUseCover")
	local civiliansAlertGuards = include(scriptPath.."/features/civiliansAlertGuards")
	local eliteCameras = include(scriptPath.."/features/eliteCameras")

	surrender.setModInfo(true, options.nct_surrender.enabled == true, options.nct_koDragCost.value, options.nct_wounded.value, options.nct_woundedMax.value)
	dragAgents.setModInfo(options.nct_dragAgents.enabled == true, options.nct_ignoreMissionCriticalAgents.enabled == true)
	spotLootedSafes.setModInfo(options.nct_spotSafesWhenLooted.value, options.nct_spotLootedSafes.value)
	--guardsUseCover.setModInfo(options.nct_guardsHideOnAlert.enabled == true, options.nct_guardsPreferCover.enabled == true)
	civiliansAlertGuards.setModInfo(options.nct_civiliansAlertGuards.enabled == true)
	eliteCameras.setModInfo(options.nct_trackingCameras.enabled == true,
	                        options.nct_camerasDetectDownedGuards.enabled == true,
	                        options.nct_cameraActivationCount.value)

	surrender.load(scriptPath, modApi)
end

local function unload(modApi, mod_options, params)
	local scriptPath = modApi:getScriptPath()
	local surrender = include(scriptPath.."/features/surrender")
	local dragAgents = include(scriptPath.."/features/dragAgents")
	local spotLootedSafes = include(scriptPath.."/features/spotLootedSafes")
	--local guardsUseCover = include(scriptPath.."/features/guardsUseCover")
	local civiliansAlertGuards = include(scriptPath.."/features/civiliansAlertGuards")
	local eliteCameras = include(scriptPath.."/features/eliteCameras")

	surrender.setModInfo()
	dragAgents.setModInfo()
	spotLootedSafes.setModInfo()
	--guardsUseCover.setModInfo()
	civiliansAlertGuards.setModInfo()
	eliteCameras.setModInfo()
end

local function lateLoad(modApi, options)
	local scriptPath = modApi:getScriptPath()
	local surrender = include(scriptPath.."/features/surrender")
	local dragAgents = include(scriptPath.."/features/dragAgents")

	surrender.lateLoad()
	dragAgents.lateLoad()
end

local function lateUnload(modApi, options)
	local scriptPath = modApi:getScriptPath()
	local surrender = include(scriptPath.."/features/surrender")

	surrender.lateUnload()
end

local function initStrings(modApi)
	local scriptPath = modApi:getScriptPath()
	local dataPath = modApi:getDataPath()
	modApi:addStrings(dataPath, "NEW_CORP_TACTICS", include(scriptPath.."/strings"))
end

return {
	init = init,
	load = load,
	unload = unload,
	lateLoad = lateLoad,
	lateUnload = lateUnload,
	initStrings = initStrings
}
