local util = include( "modules/util" )
local array = include( "modules/array" )

local function init( modApi )
	local dataPath = modApi:getDataPath()

	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" ) 	
	KLEIResourceMgr.MountPackage( dataPath .. "/gui_carmen.kwad", "data" ) 
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
	
  	modApi:addGenerationOption("enable_carmen_start", STRINGS.CARMEN.OPTIONS.ENABLE_CARMEN_START, STRINGS.CARMEN.OPTIONS.ENABLE_CARMEN_START_TIP,{noUpdate = true})
  	modApi:addGenerationOption("enable_carmen_detention", STRINGS.CARMEN.OPTIONS.ENABLE_CARMEN_DETENTION, STRINGS.CARMEN.OPTIONS.ENABLE_CARMEN_DETENTION_TIP,{noUpdate = true})

	modApi.requirements = {}
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "CARMEN", DLC_STRINGS )
end

local function load( modApi, options, params, mod_options )
  	local scriptPath = modApi:getScriptPath()

    	modApi:addAbilityDef( "CARMEN_crack_safe", scriptPath .."/CARMEN_crack_safe" )
	include( scriptPath .. "/simunits" )

	local agentdefs = include( scriptPath .. "/agentdefs" )
        for name, agentDef in pairs(agentdefs) do
		modApi:addAgentDef( name, agentDef )
	end
	
	local animdefs = include( scriptPath .. "/animdefs" )
		for name, animDef in pairs(animdefs) do
		modApi:addAnimDef( name, animDef )
	end

        local itemdefs = include( scriptPath .. "/itemdefs" )
        for name, itemDef in pairs(itemdefs) do
		modApi:addItemDef( name, itemDef )
        end  

	if options["enable_carmen_start"] and options["enable_carmen_start"].enabled then
        	local serverdefs = include( "modules/serverdefs" )
		if array.find( serverdefs.SELECTABLE_AGENTS, "carmen_sandiego_o" ) == nil then
            		table.insert( serverdefs.SELECTABLE_AGENTS, "carmen_sandiego_o" )
		end
        	serverdefs.LOADOUTS[ "carmen_sandiego_o" ] = { "carmen_sandiego_o" }
	end

	if options["enable_carmen_detention"] and options["enable_carmen_detention"].enabled then
		local serverdefs = include( scriptPath .. "/serverdefs" )
		for i,unitDef in ipairs(serverdefs.TEMPLATE_AGENCY_ONFILE.unitDefsPotential) do
			if unitDef then
				modApi:addRescueAgent( unitDef )
			end
		end
	end
end

return
{
	init = init,
	initStrings = initStrings,
	load = load,
}