local util = include( "modules/util" )
local home_panel = include( "hud/home_panel" )

local function earlyInit( modApi )
	modApi.requirements = { "Contingency Plan", "Function Library" }
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})

	local scriptPath = modApi:getScriptPath()
	home_panel.panel = include( scriptPath .. "/home_panel" ).panel
	SCRIPT_PATHS.aiPanel = include( scriptPath .. "/ai_panel" )
	include( scriptPath .. "/hud" )
	include( scriptPath .. "/client_defs" )
end

local function init( modApi )
	rawset(_G,"SCRIPT_PATHS",rawget(_G,"SCRIPT_PATHS") or {})
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/gui2.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/anims2.kwad", "data" )

	if not SCRIPT_PATHS.W93_FuncLibActive or SCRIPT_PATHS.W93_FuncLibActive == false then
		log:write("Function Library disabled or missing! Programs Extended is disabled.")
		return
	end

	modApi:addGenerationOption("extended_programs", STRINGS.PROGEXTEND.OPTIONS.ENABLE_PROGRAMS, STRINGS.PROGEXTEND.OPTIONS.ENABLE_PROGRAMS_TIP,
	{
		noUpdate = true,
		difficulties = {{1,false},{2,true},{3,true},{4,true},{5,true},{6,true},{7,true},{8,true}}
	})
	modApi:addGenerationOption("extended_daemons", STRINGS.PROGEXTEND.OPTIONS.ENABLE_DAEMONS, STRINGS.PROGEXTEND.OPTIONS.ENABLE_DAEMONS_TIP,
	{
		noUpdate = true,
		difficulties = {{1,false},{2,false},{3,true},{4,true},{5,true},{6,true},{7,true},{8,true}}
	})
	modApi:addGenerationOption("extended_algorithms", STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALGORITHMS, STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALGORITHMS_TIP,
	{
		noUpdate = true,
		difficulties = {{1,false},{2,true},{3,true},{4,true},{5,true},{6,true},{7,true},{8,true}}
	})
	modApi:addGenerationOption("extended_alarms", STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS, STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_TIP,
	{
		values = {0,3,1,2},
		value = 0,
		noUpdate = true,
		strings = {STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_FALSE,STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_TRUE_3,STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_TRUE,STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_TRUE_2},
		difficulties = {{1,0},{2,0},{3,0},{4,0},{5,0},{6,0},{7,0},{8,0}},
		masks = {{mask = "mask_extended_alarms", raw = true}}
	})
	modApi:addGenerationOption("extended_alarms_countermeasures", STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_COUNTERMEASURES, STRINGS.PROGEXTEND.OPTIONS.ENABLE_ALARMS_COUNTERMEASURES_TIP,
	{
		noUpdate = true,
		difficulties = {{1,false},{2,false},{3,false},{4,true},{5,false},{6,true},{7,false},{8,false}},
		requirements = {{passFunction = function(masks)
    			return masks.mask_extended_alarms and masks.mask_extended_alarms > 0
		end}}
	})
	if MOAIEnvironment.OS ~= "NX" then
		modApi:addGenerationOption("counter_ai", STRINGS.PROGEXTEND.OPTIONS.ENABLE_COUNTER_AI, STRINGS.PROGEXTEND.OPTIONS.ENABLE_COUNTER_AI_TIP,
		{
			values = {-1,0,1,2,3,4,5},
			value = -1,
			noUpdate = true,
			strings = { STRINGS.PROGEXTEND.OPTIONS.ENABLE_COUNTER_AI_DISABLED, STRINGS.PROGEXTEND.OPTIONS.ENABLE_COUNTER_AI_OMNI, "1", "2", "3", "4", "5"},
			difficulties = {{1,-1},{2,-1},{3,-1},{4,-1},{5,-1},{6,-1},{7,-1},{8,-1}},
			masks = {{mask = "mask_counter_ai", raw = true}}
		})
		modApi:addGenerationOption("counter_ai_cap", STRINGS.PROGEXTEND.OPTIONS.COUNTER_AI_CAP, STRINGS.PROGEXTEND.OPTIONS.COUNTER_AI_CAP_TIP,
		{
			values = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15},
			value = 8,
			noUpdate = true,
			strings = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15"},
			difficulties={{1,4},{2,6},{3,8},{4,12},{5,8},{6,12},{7,8},{8,12}},
			requirements = {{passFunction = function(masks)
    				return masks.mask_counter_ai and masks.mask_counter_ai > 0
			end}}
		})
	end
	modApi:addGenerationOption("endless_daemons", STRINGS.PROGEXTEND.OPTIONS.ENDLESS_DAEMONS, STRINGS.PROGEXTEND.OPTIONS.ENDLESS_DAEMONS_TIP,
	{
		values = {50,2,4,6,8,10,12,14,16,18},
		value = 6,
		noUpdate = false,
		strings = { STRINGS.PROGEXTEND.OPTIONS.ENDLESS_DAEMONS_DISABLED, "2", "4", "6", "8", "10", "12", "14", "16", "18"},
		difficulties={{1,6},{2,6},{3,6},{4,6},{5,6},{6,6},{7,6},{8,6}}
	})
	modApi:addGenerationOption("shop_programs", STRINGS.PROGEXTEND.OPTIONS.ENABLE_SHOP, STRINGS.PROGEXTEND.OPTIONS.ENABLE_SHOP_TIP,
	{
		noUpdate = true,
		difficulties = {{1,false},{2,true},{3,true},{4,true},{5,true},{6,true},{7,true},{8,true}}
	})
	modApi:addGenerationOption("rarity_rebalance", STRINGS.PROGEXTEND.OPTIONS.ENABLE_RARITY, STRINGS.PROGEXTEND.OPTIONS.ENABLE_RARITY_TIP,
	{
		values = { 0, 1, 2 },
		value = 1,
		noUpdate = true,
		strings = { STRINGS.PROGEXTEND.OPTIONS.ENABLE_RARITY_DISABLED, STRINGS.PROGEXTEND.OPTIONS.ENABLE_RARITY_NORMAL, STRINGS.PROGEXTEND.OPTIONS.ENABLE_RARITY_EQUALIZED },
		difficulties = {{1,0},{2,1},{3,1},{4,1},{5,1},{6,1},{7,1},{8,1}}
	})

	include( scriptPath .. "/W93_snare_grenade" )
	include( scriptPath .. "/W93_translocator_grenade" )

	local alarm_states = include( scriptPath .. "/alarm_states" )
	for i,item in pairs(alarm_states) do    
        	modApi:addAlarmStates(i,item)
	end
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "PROGEXTEND", DLC_STRINGS )

	util.tmerge( STRINGS.LOADING_TIPS, STRINGS.PROGEXTEND.LOADING_TIPS )
end

local function load( modApi, options, params, mod_options )
	if not SCRIPT_PATHS.W93_FuncLibActive or SCRIPT_PATHS.W93_FuncLibActive == false then
		log:write("Funcion Library disabled or missing! Programs Extended is disabled.")
		return
	end

	local scriptPath = modApi:getScriptPath()
	local simdefs = include( "sim/simdefs" )
        local npc_abilities = include( scriptPath .. "/npc_abilities" )
        local mainframe_abilities = include( scriptPath .. "/mainframe_abilities" )
	local commondefs = include( scriptPath .. "/commondefs" )
    	local serverdefs = include( scriptPath .. "/serverdefs" )
	local metadefs = include( scriptPath .. "/metadefs" )
	local GuiModifications = include( scriptPath.."/screen_modifications" )
	include( scriptPath .. "/alter" )

	PE_setColors()
	modApi:modifyUIElements( GuiModifications[1] )
	modApi:insertUIElements( include( scriptPath.."/screen_inserts" ) )
	modApi:addTooltipDef( commondefs )
    	modApi:addAbilityDef( "W93_combatAI_gainHash", scriptPath .."/W93_combatAI_gainHash" )
    	modApi:addAbilityDef( "W93_combatAI_gainHashHeavy", scriptPath .."/W93_combatAI_gainHashHeavy" )
    	modApi:addAbilityDef( "W93_combatAI_drainPWR", scriptPath .."/W93_combatAI_drainPWR" )

	local itemdefs = include( scriptPath .. "/itemdefs" )
	for name, itemDef in pairs(itemdefs) do
        	modApi:addItemDef( name, itemDef )
	end

	if mod_manager:isInstalled( "dlc1" ) and mod_options["dlc1"].enabled then
		enabled_item = mod_options["dlc1"].options
		if enabled_item.programs and enabled_item.programs.enabled then
			PE_updateDlcPrograms()
		end
	end

        if options["rarity_rebalance"] and ( ( type(options["rarity_rebalance"].value) == "boolean" and options["rarity_rebalance"].value == true ) or ( type(options["rarity_rebalance"].value) == "number" and options["rarity_rebalance"].value > 0 ) ) then
		PE_setRarity(options["rarity_rebalance"].value)
		if mod_manager:isInstalled( "dlc1" ) and mod_options["dlc1"].enabled then
			enabled_item = mod_options["dlc1"].options
			if enabled_item.programs and enabled_item.programs.enabled then
				PE_setRarityDlc(options["rarity_rebalance"].value)
			end
		end
    	end

        if options["extended_programs"] and options["extended_programs"].enabled then
		mainframe_abilities.adjustAvailability( options["rarity_rebalance"].value )
		for name, ability in pairs(mainframe_abilities.mainframe_abilities) do
			modApi:addMainframeAbility( name, ability )
        	end
    	end

    	if serverdefs.SELECTABLE_PROGRAMS then   
        	if options["extended_programs"] and options["extended_programs"].enabled then 
            		for i,program in ipairs(serverdefs.SELECTABLE_PROGRAMS[1]) do 
                		modApi:addStartingGenerator( program )        
            		end
            		for i,program in ipairs(serverdefs.SELECTABLE_PROGRAMS[2]) do 
                		modApi:addStartingBreaker( program )        
            		end        
        	end
    	end

    	if metadefs.DLC_INSTALL_REWARDS then  
        	for i, reward in ipairs(metadefs.DLC_INSTALL_REWARDS) do                
            		if reward.unlockType == metadefs.PROGRAM_UNLOCK and options["extended_programs"].enabled then 
                		modApi:addInstallReward( reward )
            		end    
        	end        
    	end

    	if options["extended_daemons"] and options["extended_daemons"].enabled then
        	for name, ability in pairs(npc_abilities) do
            		modApi:addDaemonAbility( name, ability )
        	end    
    	else
		modApi:addDaemonAbility( "W93_alarm", npc_abilities[ "W93_alarm" ] )
	end

    	if options["extended_algorithms"] and options["extended_algorithms"].enabled then
        	local npc_abilities3 = include( scriptPath .. "/npc_abilities3" )
        	for name, ability in pairs(npc_abilities3) do
            		modApi:addDaemonAbility( name, ability )
        	end    
    	end

	if options["extended_alarms"].value == 2 then
		simdefs.ALARM_TYPES =
		{
			EASY =  { "booting", "firewalls", "guards", "cameras", "enforcers", "W93_speed", "W93_reboot", "W93_lockdown" }, 
			ADVANCED_EASY =   { "booting", "guards", "increaseFirewalls2", "guards", "enforcers", "W93_ko_armor", "W93_reboot2", "W93_lockdown" },
			NORMAL = { "guards", "firewalls", "guards", "cameras", "enforcers", "W93_speed", "W93_reboot", "W93_lockdown"},
			ADVANCED_NORMAL = { "guards", "increaseFirewalls2", "guards", "W93_owl_speed", "enforcers", "W93_ko_armor", "W93_reboot2", "W93_lockdown" },
			NONE = {},
		}
		simdefs.TRACKER_MAXCOUNT = 32
		simdefs.TRACKER_INCREMENT = 4 
    		simdefs.TRACKER_MAXSTAGE = 8
		if params then
			params.W93_easyAlarm = false
			if options["extended_alarms_countermeasures"] and options["extended_alarms_countermeasures"].enabled then
				params.W93_trackerCountermeasures = true
			else
				params.W93_trackerCountermeasures = false
			end
		end
		if MOAIEnvironment.OS == "NX" then
			modApi:modifyUIElements( GuiModifications[3] )
		else
			modApi:modifyUIElements( GuiModifications[2] )
		end
    	elseif options["extended_alarms"].value == 1 or options["extended_alarms"].value == 3 then
		simdefs.ALARM_TYPES =
		{
			EASY =  { "booting", "firewalls", "guards", "cameras", "enforcers", "W93_speed", "W93_reboot", "W93_lockdown" }, 
			ADVANCED_EASY =   { "booting", "guards", "increaseFirewalls2", "guards", "enforcers", "W93_ko_armor", "W93_reboot2", "W93_lockdown" }, 
			NORMAL = { "guards", "firewalls", "guards", "cameras", "enforcers", "W93_speed", "W93_reboot", "W93_lockdown"},
			ADVANCED_NORMAL = { "guards", "increaseFirewalls2", "guards", "W93_owl_speed", "enforcers", "W93_ko_armor", "W93_reboot2", "W93_lockdown" },
			NONE = {},
		}
		simdefs.TRACKER_MAXCOUNT = 40
		simdefs.TRACKER_INCREMENT = 5 
    		simdefs.TRACKER_MAXSTAGE = 8
		if params and options["extended_alarms"].value == 3 then
			params.W93_easyAlarm = true
			if options["extended_alarms_countermeasures"] and options["extended_alarms_countermeasures"].enabled then
				params.W93_trackerCountermeasures = true
			else
				params.W93_trackerCountermeasures = false
			end
		elseif params then
			params.W93_easyAlarm = false
			if options["extended_alarms_countermeasures"] and options["extended_alarms_countermeasures"].enabled then
				params.W93_trackerCountermeasures = true
			else
				params.W93_trackerCountermeasures = false
			end
		end
		if MOAIEnvironment.OS == "NX" then
			modApi:modifyUIElements( GuiModifications[3] )
		else
			modApi:modifyUIElements( GuiModifications[2] )
		end
	elseif options["extended_alarms"].value == 0 then
		simdefs.ALARM_TYPES =
		{
			EASY =  { "booting", "cameras", "firewalls", "guards", "enforcers", "enforcers" },
			ADVANCED_EASY =   {"booting", "patrols",         "guardBuff_armor",    "increaseFirewalls2", "guardBuff_ko", "enforcers"},
			NORMAL = { "cameras", "firewalls", "guards", "guards", "enforcers", "enforcers" },
			ADVANCED_NORMAL = {"patrols", "guardBuff_armor", "increaseFirewalls2", "guardBuff_ko",     "enforcers",    "enforcers"},
			NONE = {},
		}
		simdefs.TRACKER_MAXCOUNT = 30
		simdefs.TRACKER_INCREMENT = 5 
    		simdefs.TRACKER_MAXSTAGE = 6
		if params then
			params.W93_easyAlarm = nil
			params.W93_trackerCountermeasures = nil
		end
	end

        if options["shop_programs"] and options["shop_programs"].enabled then
    		local propdefs = include( scriptPath .. "/propdefs" )
    		for i,item in pairs(propdefs) do  
        		modApi:addPropDef( i, item, false )
    		end

		PE_setShops()
		PE_updateBasePrograms()
	end

	if params then
		params.W93_AI = -1
		
	end

        if options["counter_ai"] and options["counter_ai"].value >= 0 then
		if options["counter_ai"].value >= 1 then
			for name, ability in pairs(mainframe_abilities.ai_programs) do
				modApi:addMainframeAbility( name, ability )
        		end
		end
        	local npc_abilities2 = include( scriptPath .. "/npc_abilities2" )
        	for name, ability in pairs(npc_abilities2) do
            		modApi:addDaemonAbility( name, ability )
        	end
		if params then
			params.W93_AI = options["counter_ai"].value
			if options["counter_ai_cap"] then
				params.W93_AI_CAP = options["counter_ai_cap"].value
			else
				params.W93_AI_CAP = 5
			end
		end
    	end

        if options["endless_daemons"] and options["endless_daemons"].value and params then
		local serverdefs_default = include( "modules/serverdefs" )
		params.W93_endless_daemons = options["endless_daemons"].value
	end
end

local function lateLoad( modApi, options, params, mod_options )
	if not SCRIPT_PATHS.W93_FuncLibActive or SCRIPT_PATHS.W93_FuncLibActive == false then
		return
	end

	local scriptPath = modApi:getScriptPath()
	local simdefs = include( "sim/simdefs" )
	include( scriptPath .. "/alter" )

	if options["counter_ai"] and options["counter_ai"].enabled then
		local agentdefs = include( "sim/unitdefs/agentdefs" )
		PE_updateAgentAbilityAI( false, agentdefs )
	end

        if options["shop_programs"] and options["shop_programs"].enabled then
		PE_setServers()
    	else
		PE_resetServers()
	end
end

local function unload( modApi, mod_options, params )
	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/alter" )
	local simdefs = include( "sim/simdefs" )
	local GuiModifications = include( scriptPath.."/screen_modifications" )
	modApi:modifyUIElements( GuiModifications[1] )
	modApi:insertUIElements( include( scriptPath.."/screen_inserts" ) )

	simdefs.ALARM_TYPES =
	{
		EASY =  { "booting", "cameras", "firewalls", "guards", "enforcers", "enforcers" },
		ADVANCED_EASY =   {"booting", "patrols",         "guardBuff_armor",    "increaseFirewalls2", "guardBuff_ko", "enforcers"},
		NORMAL = { "cameras", "firewalls", "guards", "guards", "enforcers", "enforcers" },
		ADVANCED_NORMAL = {"patrols", "guardBuff_armor", "increaseFirewalls2", "guardBuff_ko",       "enforcers",    "enforcers"},
		NONE = {},
	}
	simdefs.TRACKER_MAXCOUNT = 30
	simdefs.TRACKER_INCREMENT = 5 
    	simdefs.TRACKER_MAXSTAGE = 6
	if params then
		params.W93_AI = -1
		params.W93_endless_daemons = 6
		params.W93_trackerCountermeasures = nil
	end
end

local function lateUnload( modApi, mod_options, params )
	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/alter" )

	local agentdefs = include( "sim/unitdefs/agentdefs" )
	PE_updateAgentAbilityAI( true, agentdefs )
	PE_resetServers()
end

return {
    earlyInit = earlyInit,
    init = init,
    load = load,
    lateLoad = lateLoad,
    initStrings= initStrings,
    unload = unload,
    lateUnload = lateUnload,
}