local array = include("modules/array")
local simquery = include("sim/simquery")
local itemdefs = include("sim/unitdefs/itemdefs")
local simdefs = include("sim/simdefs")

local function init( modApi )
	local dataPath = modApi:getDataPath()
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	-- KLEIResourceMgr.MountPackage( dataPath .. "/sound.kwad", "data" )

	--modApi:addGenerationOption("transportshroud",  STRINGS.SHROUD.OPTIONS.ENABLE , STRINGS.SHROUD.OPTIONS.ENABLE_TIP, {noUpdate = true} )
	modApi:addGenerationOption("shroudbackfire",  STRINGS.SHROUD.OPTIONS.BACKFIRE , STRINGS.SHROUD.OPTIONS.BACKFIRE_TIP, 
		{values = {0,1,2,3,4},
		value = 1,
		noUpdate = true,
		strings = {"OFF","1","2","3","4"},
		difficulties={{1,0}}, --disable the backfire for beginner
		})
	modApi:addGenerationOption("shroudbackfirecap",  STRINGS.SHROUD.OPTIONS.BACKFIRECAP , STRINGS.SHROUD.OPTIONS.BACKFIRECAP_TIP, { noUpdate = true} )
	modApi:addGenerationOption("shroudalarmrate",  STRINGS.SHROUD.OPTIONS.ALARMRATE , STRINGS.SHROUD.OPTIONS.ALARMRATE_TIP, 
		{values = {1,2,3,4,5},
		value = 1,
		noUpdate = true,
		strings = {"OFF","2","3","4","5"},
		})
	modApi:addGenerationOption("shroudalarmcap",  STRINGS.SHROUD.OPTIONS.ALARMCAP , STRINGS.SHROUD.OPTIONS.ALARMCAP_TIP, 
		{values = {0,1,2,3,4,5,6},
		value = 0,
		noUpdate = true,
		strings = {"0","1","2","3","4","5","6"},
		})
	modApi:addGenerationOption("shroudpwrpenaltystart",  STRINGS.SHROUD.OPTIONS.PWRPENALTYSTART , STRINGS.SHROUD.OPTIONS.PWRPENALTYSTART_TIP, 
		{values = {0,2,5,8,10},
		value = 5,
		noUpdate = true,
		strings = {"0","2","5","8","10"},
		})
	modApi:addGenerationOption("shroudnocountermeasures",  STRINGS.SHROUD.OPTIONS.NOCOUNTERMEASURES , STRINGS.SHROUD.OPTIONS.NOCOUNTERMEASURES_TIP, { noUpdate = true} )
	modApi:addGenerationOption("shroudallalert",  STRINGS.SHROUD.OPTIONS.ALLALERT , STRINGS.SHROUD.OPTIONS.ALLALERT_TIP, 
		{values = {0,1,2},
		value = 1,
		noUpdate = true,
		strings = {"LOW","MED","HIGH"},
		})

	local scriptPath = modApi:getScriptPath()

	include( scriptPath .. "/aiplayer" )
	include( scriptPath .. "/engine" )
end

local function load( modApi, options, params )
	rawset(simdefs, "shroudbackfire", (options.shroudbackfire and options.shroudbackfire.value or 0))
	rawset(simdefs, "shroudbackfirecap", (options.shroudbackfirecap and options.shroudbackfirecap.enabled or false))
	rawset(simdefs, "shroudalarmrate", (options.shroudalarmrate and options.shroudalarmrate.value or 0))
	rawset(simdefs, "shroudalarmcap", (options.shroudalarmcap and options.shroudalarmcap.value or 0))
	rawset(simdefs, "shroudpwrpenaltystart", (options.shroudpwrpenaltystart and options.shroudpwrpenaltystart.value or 0))
	rawset(simdefs, "shroudnocountermeasures", (options.shroudnocountermeasures and options.shroudnocountermeasures.enabled or false))
	rawset(simdefs, "shroudallalert", (options.shroudallalert and options.shroudallalert.value or 2))

	local scriptPath = modApi:getScriptPath()

	if not options.transportshroud or options.transportshroud.enabled then
		local modprograms = include( scriptPath .. "/mainframe_abilities" )
		modprograms.desc = modprograms.build_shroud_desc()
		
        modApi:addMainframeAbility( STRINGS.SHROUD.NAME, modprograms )
        modApi:addStartingGenerator( STRINGS.SHROUD.NAME )
	end
end

local function unload()
	rawset(simdefs, "shroudbackfire", nil)
	rawset(simdefs, "shroudbackfirecap", nil)
	rawset(simdefs, "shroudalarmrate", nil)
	rawset(simdefs, "shroudalarmcap", nil)
	rawset(simdefs, "shroudpwrpenaltystart", nil)
	rawset(simdefs, "shroudallalert", nil)
	rawset(simdefs, "shroudnocountermeasures", nil)
end

local function initStrings(modApi)
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	local MOD_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "SHROUD", MOD_STRINGS)
end

return {
    init = init,
    load = load,
    unload = unload,
    initStrings = initStrings,
}