local util = include( "modules/util" )

local function init( modApi )
	local scriptPath = modApi:getScriptPath()
	local dataPath = modApi:getDataPath()
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
end

local function earlyInit( modApi )
	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/serverdefs" )
	include( scriptPath .. "/select-dlc-dialog" )
	include( scriptPath .. "/state-generation-options" )
	include( scriptPath .. "/inventory" )
	include( scriptPath .. "/mission_util" )
	include( scriptPath .. "/mui" )
	include( scriptPath .. "/unitdefs" )
	include( scriptPath .. "/simactions")
	include( scriptPath .. "/locationpopup" )
	include( scriptPath .. "/state-map-screen")
	include( scriptPath .. "/wallrig2" )
	include( scriptPath .. "/mainframe_panel" )
	include( scriptPath .. "/modifiers" )
	include( scriptPath .. "/simquery" )
	include( scriptPath .. "/shop_panel" )
	include( scriptPath .. "/engine" )
	include( scriptPath .. "/state-upgrade-screen" )
	include( scriptPath .. "/state-team-preview" )
	include( scriptPath .. "/agentsadded" )
	include( scriptPath .. "/prefabs" )
	include( scriptPath .. "/pairs" )
	include( scriptPath .."/boardrig" )
	include( scriptPath .."/agentrig" )
	include( scriptPath .."/level" )
	include( scriptPath .."/viz_manager" )
	include( scriptPath .."/state-game" )
	include( scriptPath .."/simparams" )
	package.loaded["state-loadout-selector"] = include( scriptPath .."/state-loadout-selector" )
	
	local radial = include( scriptPath .. "/mui_radialprogress" )
	package.loaded[ "mui/widgets/mui_radialtexture" ] = radial.mui_radialtexture
	package.loaded[ "mui/widgets/mui_radialprogressbar" ] = radial.radialprogressbar
	modApi:registerWidgetFactory( "radialprogressbar", radial.radialprogressbar )
end

local function load( modApi, options, params, options_raw )
	local scriptPath = modApi:getScriptPath()
	
	--Not sure why, but this fixes certain bugs
	package.loaded[scriptPath .. "/serverdefs"] = nil
	include( scriptPath .. "/serverdefs" )
	
	modApi:addAbilityDef( "debug_emp", scriptPath.."/debug_emp" )
	modApi:addAbilityDef( "debug_nuke", scriptPath.."/debug_nuke" )
	modApi:addAbilityDef( "debug_nuke_2", scriptPath.."/debug_nuke_2" )
	
    local itemdefs = include( scriptPath .. "/itemdefs" )
    for name, itemDef in pairs(itemdefs) do
        modApi:addItemDef( name, itemDef )
    end
	
	local mainframe_abilities = include(scriptPath.."/mainframe_abilities")
	for name, ability in pairs(mainframe_abilities) do
		modApi:addMainframeAbility( name, ability )
    end
end

local function earlyUnload( modApi )
	local scriptPath = modApi:getScriptPath()
	modApi.mod_manager:resetCrossBanter()
	
	-- Putting these in load is enough for most uses, but these changes are important so they shouldn't ever be turned off
	modApi:insertUIElements( include( scriptPath.."/screen_inserts" ) )
	modApi:modifyUIElements( include( scriptPath.."/screen_modifications" ) )
	modApi:addNewUIScreen( "screen-loadout-selector.lua", scriptPath.."/screen-loadout-selector" )
	
	for i, scriptHook in ipairs( include( scriptPath .."/objective_triggers" ) ) do
		modApi:addScriptHook( scriptHook )
	end
end

local function earlyLoad( modApi )
	earlyUnload( modApi )
end

local function initStrings( modApi )
	local scriptPath = modApi:getScriptPath()
	
	local settings = savefiles.getSettings( "settings" )
    log:write("Pre-Language selection: ", settings.data.localeMod)
    if settings.data.localeMod then
        modApi.mod_manager:mountLanguageMod( settings.data.localeMod )
    end

	reinclude = include
	include( scriptPath .."/cdefs" )
	include( scriptPath .."/upvalue-util" )
	include( scriptPath .. "/procgen_context" )
	include( scriptPath .. "/strict" )
	include( scriptPath .. "/simhistory" )
	include( scriptPath .. "/mod-api" )
	include( scriptPath .. "/hud" )
	
	log:write("LUA VERSION ".._VERSION)
	
	local defaultResetContent = modApi.mod_manager.resetContent
	local mod_manager = include ( scriptPath .. "/mod_manager" )
	util.tmerge( modApi.mod_manager, mod_manager ) --THIS CONTAINS CODE NEEDED FOR GENERATION OPTIONS AND NEW API FUNCTIONS, BUT ALSO FOR LEMONHEAD'S SEQUENTIAL MOD LOADER
	modApi.mod_manager.lib_sequentialModLoader = {version="2.0.0"} --PROVIDES COMPABILITY WITH THE LIBRARY VERSION OF SEQUENTIAL MOD LOADER
	modApi.mod_manager.resetSimDefs = include( scriptPath .. "/simdefs" )
	modApi.mod_manager.defaultResetContent = defaultResetContent
	modApi.mod_manager._raaHack = scriptPath.."/raa-hack"
	
	rawset( _G, "mod_manager", modApi.mod_manager )
end

return {
    init = init,
    load = load,
	earlyInit = earlyInit,
	earlyLoad = earlyLoad,
	earlyUnload = earlyUnload,
    initStrings= initStrings,
}