local util = include( "modules/util" )
local array = include( "modules/array" )
local mathutil = include( "modules/mathutil" )
local WILSON_PEEK_COST = 1.5 --2
local WILSON_SAA_ID = "agent_wilson_mod_saa"	

-- HELPER FUNCTIONS
local function updateBeard( sim, unit, stage )
	local simdefs = include("sim/simdefs")

	-- --log:write("LOG updating beard with stage "..tostring(stage))
	local beardAnim = "portraits/wilson_face_beard"..tostring(stage)
	unit:getUnitData().profile_build = beardAnim	
	if not unit:getTraits().rescued then --would clash with Prisoner Outfits
		if stage > 4 then -- fewer fullbody kanims for beard stages than portraits
			stage = 4
		end
		local beardKanim = "kanim_wilson_beard"..tostring(stage)
		if unit:getTraits().fancyLabcoat then
			beardKanim = "kanim_wilson_labcoat_beard"..tostring(stage)
		end
		unit:changeKanim(beardKanim)
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
	end
end

local function despawnShadowMonsters( sim )
local simdefs = include("sim/simdefs")

	for i, unit in pairs(sim:getAllUnits()) do
		if unit:getTraits().Wilson_shadowMonster then
			local cell = sim:getCell(unit:getLocation())
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = unit, kanim = "fx/shadow_teleport_fx", anim="out", symbol = "effect", x = cell.x, y = cell.y, facingMask = KLEIAnim.FACING_W } )
			sim:warpUnit( unit, nil )
			sim:despawnUnit(unit)
		end
	end
end

local function getFinalNode( sim, unit )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local propdefs = include("sim/unitdefs/propdefs")
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")

	-- the only way to get where a guard will be at the end of the next guard turn is via this stupidity, i.e. pathrig \o/
	local action, prevNode
	local finalNode
	local path = unit:getPather():getPath(unit)
	if path.currentNode then
		action = unit:getPather():getActionForNode(path, path.currentNode)
		prevNode = path.currentNode
	end
	local finalNodeIndex
	local movePoints = unit:getTraits().mpMax
	for i, node in ipairs(path.path:getNodes() ) do
		if movePoints and node and prevNode then
			local moveCost = simquery.getMoveCost(prevNode.location, node.location)
			movePoints = movePoints - moveCost
			if movePoints < 0 then
				break	--that's all the path we have time for right now
			end
		end
		action = unit:getPather():getActionForNode(path, node)
		prevNode = node --these are updated until the loop breaks because we ran out of AP
		finalNode = node
		finalNodeIndex = i
	end
	local x,y = finalNode.location.x, finalNode.location.y
	local cell = sim:getCell(x, y)
	local penultimateNode = path.path:getNodes()[(finalNodeIndex-1)]
	local prevCell = sim:getCell( penultimateNode.location.x, penultimateNode.location.y )
	if cell == prevCell then
		--log:write("LOG searching for nonidentical node cell")
		repeat
			finalNodeIndex = finalNodeIndex - 1
			if path.path:getNodes()[(finalNodeIndex-1)] then
				penultimateNode = path.path:getNodes()[(finalNodeIndex-1)]
				--log:write(util.stringize(penultimateNode.location,2))
				prevCell = sim:getCell( penultimateNode.location.x, penultimateNode.location.y )
			else
				break
			end
		until cell ~= prevCell
	end
	return cell, prevCell
end


--delete "orphaned" shadows
local function despawnOutdatedShadows( sim )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local propdefs = include("sim/unitdefs/propdefs")
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")

	local deletedshadows = {}
	for i, shadow in pairs(sim:getAllUnits()) do
		if shadow:getTraits().Wilson_shadowMonster then
			local shadowSource = sim:getUnit(shadow:getTraits().WilsonShadowSource)
			if not shadowSource then
				table.insert( deletedshadows, shadow )
			else
				local cell, prevCell = getFinalNode( sim, shadowSource )
				local shadowCell = sim:getCell(shadow:getLocation())
				if cell ~= shadowCell then
					table.insert( deletedshadows, shadow )
				end
			end
		end
	end
	
	for i = #deletedshadows, 1, -1 do
		local shadow = deletedshadows[i]
		log:write("LOG despawning")
		log:write(util.stringize(shadow:getID(),2))		
		local cell = sim:getCell(shadow:getLocation())
		sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = shadow, kanim = "fx/shadow_teleport_fx", anim="out", symbol = "effect", x = cell.x, y = cell.y, facingMask = KLEIAnim.FACING_W } )
		sim:warpUnit( shadow, nil )
		sim:despawnUnit(shadow)
	end
end

local function spawnShadowMonster( sim, shadowSpawn )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local propdefs = include("sim/unitdefs/propdefs")
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")

	local unit = shadowSpawn[1]
	local cell = shadowSpawn[2]
	local prevCell = shadowSpawn[3]
	local children = unit:getChildren()
	local x1, y1 = unit:getLocation()
	local prop_template = propdefs.shadow_creature
	local shadowGuard = simfactory.createUnit( prop_template, sim )	
	local kanim = unit:getUnitData().kanim
	shadowGuard._unitData.kanim = kanim
	local facing = simquery.getDirectionFromDelta( cell.x - prevCell.x, cell.y - prevCell.y)
	sim:spawnUnit( shadowGuard )
	sim:warpUnit(shadowGuard, cell)
	shadowGuard:setFacing(facing)
	for i, item in pairs(children) do
		local item_template = item:getUnitData().id
		if item_template then
			local newItem = simfactory.createUnit( unitdefs.lookupTemplate(item_template), sim )
			sim:spawnUnit( newItem )
			shadowGuard:addChild( newItem )
		end
	end
	
	if unit:getTraits().WilsonShadowMonster and sim:getUnit(unit:getTraits().WilsonShadowMonster) then
		local oldShadow = sim:getUnit(unit:getTraits().WilsonShadowMonster)
		sim:warpUnit( oldShadow, nil )
		sim:despawnUnit( oldShadow )
	end
	
	shadowGuard:getTraits().shadowName = unit:getUnitData().name
	unit:getTraits().WilsonShadowMonster = shadowGuard:getID()
	shadowGuard:getTraits().WilsonShadowSource = unit:getID()
	
	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = shadowGuard } )
	sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = shadowGuard, kanim = "fx/shadow_teleport_fx", anim="in", symbol = "effect", x = cell.x, y = cell.y, facingMask = KLEIAnim.FACING_W } )

end

local refurbisherFood = { --non-exhaustive list
	["item_clip"] = true,
	["item_valuable_tech"] = true,
	["item_valuable_tech_2"] = true,
	["item_valuable_tech_3"] = true,
	["item_valuable_tech_mid2"] = true,
}

local function canUnitOrEyeballSeeUnit( sim, unit, targetUnit )
	if sim:canUnitSeeUnit( unit, targetUnit ) then
		return true
	end
	for _, eyeballUnit in pairs(sim:getAllUnits()) do
		if eyeballUnit:getTraits().peekID == unit:getID() and 
		sim:canUnitSeeUnit( eyeballUnit, targetUnit ) then
			return true
		end
	end	
	return false
end
-- MOD START

local function init( modApi )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local propdefs = include("sim/unitdefs/propdefs")
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")

	local dataPath = modApi:getDataPath()
	log:write("inside the Agent Wilson mod")
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" ) 	
	KLEIResourceMgr.MountPackage( dataPath .. "/images.kwad", "data" ) 
    KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
	
  	modApi:addGenerationOption("enable_agent_wilson", STRINGS.AGENT_WILSON.OPTIONS.ENABLE_WILSON, STRINGS.AGENT_WILSON.OPTIONS.ENABLE_WILSON_TIP,{noUpdate = true})

	modApi.requirements = {"Sim Constructor","Function Library", "Contingency Plan","Additional Banter",}

	modApi:addGenerationOption("wilson_rescuable", STRINGS.AGENT_WILSON.OPTIONS.ENABLE_DET_WILSON, STRINGS.AGENT_WILSON.OPTIONS.ENABLE_DET_WILSON_TIP, 
	{
		values = {0,1,2,3}, --"NONE","ONFILE","ARCHIVED","RANDOM"
		strings = STRINGS.AGENT_WILSON.OPTIONS.ENABLE_DET_WILSON_STRINGS, 
		value=3,
		noUpdate = true,
	})	
	
  	modApi:addGenerationOption("wilsonShadowFluff", STRINGS.AGENT_WILSON.OPTIONS.ENABLE_WILSON_FLUFF, STRINGS.AGENT_WILSON.OPTIONS.ENABLE_WILSON_FLUFF_TIP,{enabled = false, noUpdate = true})
	
 	modApi:addGenerationOption("separateWilsons", STRINGS.AGENT_WILSON.OPTIONS.SEPARATE_WILSONS, STRINGS.AGENT_WILSON.OPTIONS.SEPARATE_WILSONS_TIP,{enabled = false, noUpdate = true})	
	
	-- appends
	local simengine = include("sim/engine")
	local spawnUnit_old = simengine.spawnUnit 
	simengine.spawnUnit = function( self, unit, ... )

		spawnUnit_old( self, unit, ... )	
		if unit and self:getParams().difficultyOptions.enable_agent_wilson then
			local template = unit:getUnitData().id
			if (refurbisherFood[template]) or (unit:getUnitData().traits and unit:getUnitData().traits.ammo_clip) then
				unit:giveAbility("wilson_quantumrefurbisher")
				local value = unit:getTraits().cashInReward or unit:getUnitData().value or 0
				unit:getTraits().quantumRefurbisherValue = value
			end
			if unit:getTraits().cooldown or unit:getTraits().ammo or unit:getTraits().charges then
				unit:giveAbility("wilson_quantumrefurbisher_rev")
			end
			
			if unit:getUnitData().agentID and (unit:getUnitData().agentID == "agent_wilson_mod") then
			
				local count, augments = unit:countAugments( "augment_wilson" )
				if not (count > 0) then
					count, augments = unit:countAugments( "augment_wilson_2" )
				end
				if (count > 0) then
					local augment = augments[1]
					local campaignHours = self:getParams().campaignHours
					local diff = campaignHours - augment:getTraits().timeElapsed
					-- if not unit:getTraits().fancyLabcoat and (augment:getTraits().timeElapsed == 0) then
						-- diff = 24 --this ensures that OF Wilson starts campaign with the 5 o'clock shadow
					-- end
					augment:getTraits().timeElapsed = campaignHours
					augment:getTraits().beardTime = augment:getTraits().beardTime + diff
					-- --log:write(tostring(augment:getTraits().timeElapsed))
					-- --log:write(tostring(augment:getTraits().beardTime))
					local beardHours = augment:getTraits().beardTime
					if beardHours < 24 then
						updateBeard( self, unit, 1 )--clean shaven
					elseif (beardHours >= 24) and (beardHours < 72) then
						updateBeard( self, unit, 2 )--5 o'clock shadow
					elseif (beardHours >= 72) and (beardHours < 120) then
						updateBeard( self, unit, 3 )--hairy
					elseif (beardHours >= 120) and (beardHours < 360) then
						updateBeard( self, unit, 4 )--hairier	
					elseif beardHours >= 360 then
						updateBeard( self, unit, 5 )--hairiest					
					end		
				end
			end
		end
	end	
	
end

local function lateInit( modApi )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local propdefs = include("sim/unitdefs/propdefs")
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")

	local scriptPath = modApi:getScriptPath()
	include( scriptPath .. "/unitrig" )
	
	local simengine = include("sim/engine")
	local abilitydefs = include( "sim/abilitydefs" )
	local overwatchMelee = abilitydefs.lookupAbility("overwatchMelee")
	local ambush_canUse_old = overwatchMelee.canUseAbility
	overwatchMelee.canUseAbility = function( self, sim, unit, ... )
		if (unit:countAugments( "augment_wilson" ) > 0) and unit:getTraits().ap then
			unit:getTraits().ap = unit:getTraits().ap + 1
			local result, reason = ambush_canUse_old(self, sim, unit, ...)
			unit:getTraits().ap = unit:getTraits().ap -1
			if result == true then
				return result
			end
		end
		return ambush_canUse_old(self, sim, unit, ...)		
	end
	
	local melee = abilitydefs.lookupAbility("melee")
	local melee_canUse_old = melee.canUseAbility
	melee.canUseAbility = function( self, sim, unit, userUnit, targetID, ... )
		if unit and (unit:countAugments( "augment_wilson" ) > 0) and unit:getTraits().ap and (sim:getCurrentPlayer() ~= sim:getPC()) then --only during enemy turn i.e. for ambush!
			unit:getTraits().ap = unit:getTraits().ap + 1
			local result, reason1, reason2, reason3 = melee_canUse_old( self, sim, unit, userUnit, targetID, ... )
			unit:getTraits().ap = unit:getTraits().ap - 1
			if result == true then
				return result
			end
		end
		local result, reason1, reason2, reason3 = melee_canUse_old( self, sim, unit, userUnit, targetID, ... )
		return result, reason1, reason2, reason3
	end
	
	local throw = abilitydefs.lookupAbility("throw")
	local throw_executeAbility_old = throw.executeAbility
	throw.executeAbility = function( self, sim, grenadeUnit, userUnit, targetCell, ... )
		throw_executeAbility_old( self, sim, grenadeUnit, userUnit, targetCell, ... )
		if userUnit:getUnitData().agentID and (userUnit:getUnitData().agentID == "agent_wilson_mod") then --randomise gas colour every throw
			local unitdefs = include("sim/unitdefs")
			local props = unitdefs.prop_templates
			if props.smoke_cloud_wilson then
				props.smoke_cloud_wilson.traits.gasColor = props.smoke_cloud_wilson.traits.gascolours[sim:nextRand( 1, #props.smoke_cloud_wilson.traits.gascolours)]
			end
		end
	end
		
	local senses = include("sim/btree/senses")
	local simdefs = include("sim/simdefs")
	local old_processAppearedTrigger = senses.processAppearedTrigger
	senses.processAppearedTrigger = function(self, sim, evData)
		old_processAppearedTrigger(self, sim, evData)
		local seerUnit = sim:getUnit(evData.seerID)
		if seerUnit ~= self.unit then
			return -- the seer isn't us
		end

		if self.unit:isKO() or self.unit:isDead() then
			return
		end
		--log:write("LOG senses")
		--log:write(util.stringize(evData.unit:getUnitData().name,2))
		local x,y = evData.unit:getLocation()
		if evData.unit:getTraits().shouldMakeInvestigate then --for beard hair
			--log:write("saw beard hair")
			self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_NOTICED)
		end
		
		if self.unit:getTraits().scientist and evData.unit:getTraits().fancyLabcoat then
			self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_NOTICED)
		end
	end	
	
	local simengine_endTurn_old = simengine.endTurn
	simengine.endTurn = function( self, ... )
		simengine_endTurn_old( self, ... )
		despawnShadowMonsters( self )
	end
	
	local canPlayerSeeUnit_old = simengine.canPlayerSeeUnit
	simengine.canPlayerSeeUnit = function(self, player, unit, ... )
		if self:getParams().difficultyOptions.wilsonShadowFluff and (self:getParams().difficultyOptions.wilsonShadowFluff == true) then
			if unit and unit:getTraits().Wilson_shadowMonster and (self:getTags().WilsonSelected == nil) then
				return false
			end
		end
		return canPlayerSeeUnit_old(self, player, unit, ... )
	end

	local peek = abilitydefs.lookupAbility("peek")
	local peek_canUse_old = peek.canUseAbility
	peek.canUseAbility = function( self, sim, unit, ... )
		if unit:countAugments( "augment_wilson" ) > 0 then
			if unit:getMP() < WILSON_PEEK_COST then
				return false, util.sformat(STRINGS.AGENT_WILSON.ABILITIES.PEEK_REQUIRES_AP,WILSON_PEEK_COST)
			end
		end
		return peek_canUse_old( self, sim, unit, ... )
	end
	
	local peek_executeAbility_old = peek.executeAbility
	peek.executeAbility = function( self, sim, unit, userUnit, exitX, exitY, exitDir, ... )
		peek_executeAbility_old( self, sim, unit, userUnit, exitX, exitY, exitDir, ... )
		if unit:getTraits().Wilson_paranoiaRange then
			local peek_cost = WILSON_PEEK_COST - 1
			unit:useMP( peek_cost,sim )
			
			--reveal unseen units -- NOW UNUSED
			-- local range = unit:getTraits().Wilson_paranoiaRange
			-- local x0, y0 = unit:getLocation()
			-- local cells = simquery.rasterCircle( sim, x0, y0, range )
            -- for i, x, y in util.xypairs( cells ) do
                -- local cell = sim:getCell( x, y )
				-- if cell and cell.units then
					-- for k, seenUnit in pairs(cell.units) do
						-- if seenUnit:getTraits().isAgent and (seenUnit:getPlayerOwner() == sim:getNPC()) and not sim:canPlayerSeeUnit( unit:getPlayerOwner(), seenUnit ) then
							-- seenUnit:getTraits().oldFacing = seenUnit:getFacing()
							-- local x1, y1 = seenUnit:getLocation()
							-- local tempfacing = simquery.getDirectionFromDelta( x0 -x1, y0 - y1 )
							-- if not seenUnit:getTraits().pacifist and not seenUnit:getTraits().isDrone then
								-- seenUnit:setFacing(tempfacing)
								-- seenUnit:setAiming(true)
								-- sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = seenUnit } )
							-- end
							-- sim:getPC():glimpseUnit( sim, seenUnit:getID() )
							-- if not seenUnit:getTraits().pacifist and not seenUnit:getTraits().isDrone then
								-- seenUnit:setAiming(false)
								-- seenUnit:setFacing(seenUnit:getTraits().oldFacing)
								-- seenUnit:getTraits().oldFacing = nil
							-- end
						-- end
					-- end
				-- end
			-- end		
			
			-- reveal "shadows"
			local enemies = {}
			-- find unseen guards whose path ends in seen or peeked tiles
			for i, guard in pairs(sim:getNPC():getUnits()) do
				--log:write("LOG checking for guard"..tostring(guard:getUnitData().name))
				if guard:getBrain() and guard:getPather():getPath(guard) and guard:getTraits().mpMax and not guard:isKO() then
					local cell, prevCell = getFinalNode( sim, guard )
					local guard_cell = sim:getCell(guard:getLocation())
					if cell ~= guard_cell then
						--log:write("checking...")
						local canSee = false
						if simquery.couldUnitSeeCell(sim, unit, cell) then
							canSee = true
							--log:write("LOG unit could see cell")
						end
						if not canSee then
							--log:write("LOG checking eyeballs")
							for _, eyeballUnit in pairs(sim:getAllUnits()) do
								if eyeballUnit:getTraits().peekID == unit:getID() and simquery.couldUnitSeeCell( sim, eyeballUnit, cell ) then
									canSee = true
								end
							end
						end
						if canUnitOrEyeballSeeUnit( sim, unit, guard ) then
							--log:write("can see original guard")
							canSee = false --we only want shadows of guards who are out of sight
						end
						if canSee then
							local shadowSpawn = {guard, cell, prevCell}
							table.insert(enemies,shadowSpawn)
						end
					end
				end
			end
			despawnOutdatedShadows(sim)			
			if #enemies > 0 then
				-- despawnShadowMonsters(sim)
				for i, shadowSpawn in pairs(enemies) do
					spawnShadowMonster(sim, shadowSpawn)
				end
			end
		end
	end	
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "AGENT_WILSON", DLC_STRINGS )
end

local function load( modApi, options, params, mod_options )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local propdefs = include("sim/unitdefs/propdefs")
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")

  	local scriptPath = modApi:getScriptPath()
	local commondefs = include( scriptPath .. "/commondefs" )
	modApi:addAbilityDef( "wilson_shadowvisibility", scriptPath .."/wilson_shadowvisibility" )
	modApi:addAbilityDef( "wilson_quantumrefurbisher", scriptPath .."/wilson_quantumrefurbisher" )
	modApi:addAbilityDef( "wilson_quantumrefurbisher_rev", scriptPath .."/wilson_quantumrefurbisher_rev" )
	modApi:addAbilityDef( "wilson_shave", scriptPath .."/wilson_shave" )
	
	-- modApi:addTooltipDef( commondefs )

	-- modApi:addAbilityDef( "paranoia_scan", scriptPath .."/paranoia_scan" )
	
	include( scriptPath .. "/simgrenade" )
	
	-- local agentdefs = include( scriptPath .. "/agentdefs" )
	-- modApi:addAgentDef( "ds_wilson", include( scriptPath .. "/agentdefs" ).ds_wilson, {"ds_wilson, ds_wilson_arch"} )
	-- modApi:addAgentDef( "ds_wilson_arch", include( scriptPath .. "/agentdefs" ).ds_wilson_arch  )	
	
	-- local serverdefs = include( "modules/serverdefs" )
    -- table.insert( serverdefs.SELECTABLE_AGENTS, "ds_wilson" )
    -- serverdefs.LOADOUTS[ "ds_wilson" ] = { "ds_wilson","ds_wilson_arch" }	
	
	local animdefs = include( scriptPath .. "/animdefs" )
	for name, animDef in pairs(animdefs) do
		modApi:addAnimDef( name, animDef )
	end

	local itemdefs = include( scriptPath .. "/itemdefs" )
	for name, itemDef in pairs(itemdefs) do
		modApi:addItemDef( name, itemDef )
	end  

	local propdefs = include( scriptPath .. "/propdefs" )
	for i,item in pairs(propdefs) do  
		modApi:addPropDef( i, item, false )
	end			
		
	if params then 
		if options["wilsonShadowFluff"] and options["wilsonShadowFluff"].enabled then
			params.wilsonShadowFluff = true
		else
			params.wilsonShadowFluff = false
		end
	end
	
	if options["enable_agent_wilson"] and options["enable_agent_wilson"].enabled then
		local agentdefs = include( scriptPath .. "/agentdefs" )
		for name, agentDef in pairs(agentdefs) do
			modApi:addAgentDef( name, agentDef )
		end	
		local banterdefs = include( scriptPath.."/banterdefs" )
		log:write(util.stringize(banterdefs,3))
		for i, banter in pairs( banterdefs ) do
			modApi:addBanter( banter )
		end		
        local serverdefs = include( "modules/serverdefs" )
		if array.find( serverdefs.SELECTABLE_AGENTS, "ds_wilson" ) == nil then
            table.insert( serverdefs.SELECTABLE_AGENTS, "ds_wilson" )
		end
        serverdefs.LOADOUTS[ "ds_wilson" ] = { "ds_wilson","ds_wilson_arch" }
		if params then
			params.enable_agent_wilson = true
		end
		
		local agents = include("sim/unitdefs/agentdefs")
		if options["separateWilsons"] and options["separateWilsons"].enabled then
			agents.ds_wilson_arch.agentID = WILSON_SAA_ID
			serverdefs.LOADOUTS[ "ds_wilson" ] = { "ds_wilson" }
			serverdefs.LOADOUTS[ "ds_wilson_arch" ] = { "ds_wilson_arch" }
			if array.find( serverdefs.SELECTABLE_AGENTS, "ds_wilson_arch" ) == nil then
				table.insert( serverdefs.SELECTABLE_AGENTS, "ds_wilson_arch")
			end
		else
			agents.ds_wilson_arch.agentID = "agent_wilson_mod"
			serverdefs.LOADOUTS[ "ds_wilson" ] = { "ds_wilson","ds_wilson_arch" }
		end
	end

	local agentdefs = include("sim/unitdefs/agentdefs")
	if options["wilson_rescuable"] and options["wilson_rescuable"].value then
		if agentdefs.ds_wilson and agentdefs.ds_wilson_arch then	
			local serverdefs_mod = include( scriptPath .. "/serverdefs" ).TEMPLATE_AGENCY.unitDefsPotential
			if options["wilson_rescuable"].value == 1 then --onfile
				modApi:addRescueAgent( serverdefs_mod[1] )
			elseif options["wilson_rescuable"].value == 2 then --arch
				modApi:addRescueAgent( serverdefs_mod[2] )
			elseif options["wilson_rescuable"].value == 3 then --random
				if math.random() <= 0.5 then
					modApi:addRescueAgent( serverdefs_mod[1] )
				else
					modApi:addRescueAgent( serverdefs_mod[2] )
				end
			end
		end
	end
end

return
{
	init = init,
	lateInit = lateInit,
	initStrings = initStrings,
	load = load,
}