---------------------------------------------------------------------
-- Invisible Inc. Improved Shade mod

local util = include("modules/util")
local simdefs = include("sim/simdefs")
local array = include("modules/array")
local oldShade = nil

local function init( modApi )
    log:write("inside the ImprovedShade mod")
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	
	modApi.requirements = {"Sim Constructor","Contingency Plan"}	
	
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	
	modApi:addGenerationOption("shadefix" , "Enable Improved Shade" , "Shade will reduce guard vision arc instead of range, peripheral vision unchanged", {noUpdate = true} )
	
	local programs = include("sim/abilities/mainframe_abilities")
	
	oldShade = util.tcopy(programs.shade)

end


local function load( modApi, options, params, allOptions )

    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	local programs = include("sim/abilities/mainframe_abilities")
	local modifiers = include( "sim/modifiers" ) --need this for getModifiers to work
	local programs = include("sim/abilities/mainframe_abilities")
	
	local old_executeAbility = programs.shade.executeAbility 
	
	
	
	if options["shadefix"] and options["shadefix"].enabled then
		
		programs.shade.desc = "Reduces sight arc of guards.\n3 turn cooldown." --update description
		programs.shade.maxCooldown = 3 --increase CD by 1
		
		--replace default shade function with edited version
		programs.shade.executeAbility = function ( self, sim )
			self.used = true 
			local player = sim:getCurrentPlayer()
            self:useCPUs( sim )
			for _, unit in pairs(sim:getAllUnits()) do
				if unit:getPlayerOwner() ~= player and unit:getTraits().isGuard and unit:getTraits().hasSight then
				
                    if unit:getTraits().LOSarc < (math.pi/10) then 
						unit:getModifiers():add( "LOSarc", "shade", modifiers.ADD, -(unit:getTraits().LOSarc) )
					else
					
						unit:getModifiers():add( "LOSarc", "shade", modifiers.ADD, -(math.pi/10) ) --vanilla code except changing arc instead of range, note: change value is in Rad! Therefore pi=180° and modifier reduces LOSarc by a fraction of pi
					end
					
                    sim:refreshUnitLOS( unit )
				end
			end
			self:setCooldown( sim )
		end
	
	
	else
		programs.shade.desc = oldShade.desc
		programs.shade.maxCooldown = oldShade.maxCooldown 
		
		programs.shade.executeAbility = oldShade.executeAbility
	end
end



local function lateUnload(modApi, options, params, allOptions) --reset everything to vanilla values
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	local programs = include("sim/abilities/mainframe_abilities")
	
		programs.shade.desc = oldShade.desc
		programs.shade.maxCooldown = oldShade.maxCooldown 
		
		programs.shade.executeAbility = oldShade.executeAbility
end


-- local function initStrings( modApi )
    -- local dataPath = modApi:getDataPath()
    -- local scriptPath = modApi:getScriptPath()
    -- local DLC_STRINGS = include( scriptPath .. "/strings" )
    -- modApi:addStrings( dataPath, "Permadeath", DLC_STRINGS )  
-- end

return {
    init = init,
	-- unload = unload,
    load = load,
	-- lateLoad = lateLoad,
	lateUnload = lateUnload,
}
