---------------------------------------------------------------------
-- Invisible Inc. MOD.
local simdefs = include( "sim/simdefs" )
local util = include( "modules/util" )
local STRINGS = include("strings")
local array = include( "modules/array" )
local cdefs = include( "client_defs" )

------------- HELPER FUNCTIONS -------------------------
local frozenRenderFilter = { 
	shader=KLEIAnim.SHADER_FOW,
	r=85/255,
	g=222/255,
	b=227/255,
	a=0.9,
	lum=1.0, }
				
local function isInSmoke( agent, sim )
	local agentCell = sim:getCell( agent:getLocation() )
	for i, smokeUnit in pairs(sim:getAllUnits()) do
		if smokeUnit:getUnitData().type == "smoke_cloud" then
			local smokeCells = smokeUnit:getSmokeCells()
			for k, cell in pairs(smokeCells) do
				if cell == agentCell then
					return true
				end
			end
		end
	end
	return false		
end

local function CamilaIsThere( sim, x, y )
	local float_text_cell = sim:getCell( x, y )
	for i, unit in pairs(sim:getPC():getUnits()) do
		if unit:getTraits().ITB_smokeAttackBonus then
			local agent_cell = sim:getCell( unit:getLocation() )
			if agent_cell == float_text_cell then
				return true
			end
		end
	end
	return false
end

local function checkForUnits(unit,cell)
	local units = {}
		for i,checkUnit in ipairs(cell.units) do
			if checkUnit ~= unit and checkUnit:getTraits().isAgent then
				return true
			end
		end
	return false
end

local function GanaNotAlone( sim, GanaUnit )
	for i, unit in pairs(sim:getPC():getUnits()) do
		if (unit ~= GanaUnit) and not unit:isKO() then
			return true
		end
	end
	return false
end

local function cellContainsHenry( sim, x, y )
	local cell = sim:getCell(x, y)
	for i, unit in pairs(cell.units) do
		if unit:getTraits().ITB_HenryUntrackable then
			return true
		end
	end
	return false	
end

local function customCanPath( cellquery, unit, startcell, endcell ) -- simquery.canPath but with small changes
	-- Check for dynamic impass.  Note that even if dynamicImpass exists, if the unit owner doesn't know it's there, this won't fail.
 --   if unit == nil or unit:getTraits().dynamicImpass then
		for i,cellUnit in ipairs(endcell.units) do
			if cellUnit:getTraits().dynamicImpass then
				-- Owned emitters will turn themselves off, so they are not considered impassable.
				if cellUnit:getTraits().emitterID == nil or cellUnit:isGhost() or not cellUnit:canControl( unit ) then
					return false, simdefs.CANMOVE_DYNAMIC_IMPASS
				end
			end
		end
--    end

	return true
end

local function lethalLaserBlocking( cellquery, unit, startcell, endcell )
	for i,cellUnit in ipairs(endcell.units) do
		if cellUnit:getTraits().dynamicImpass then
			-- Owned emitters will turn themselves off, so they are not considered impassable.
			if cellUnit:getTraits().emitterID and cellUnit:getTraits().damage and not cellUnit:canControl( unit ) then
				return true
			end
		end
	end
	return false
end

local function cellHasProsperoVines( sim, cell )
	for i, cellUnit in pairs(cell.units) do
		if cellUnit:getTraits().ITB_ProsperoVines then
			return true
		end
	end
	return false
end
-------------------------------------------------------

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()

	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "BREACH_AGENTS", DLC_STRINGS )
end
	
-- init will be called once
local function init( modApi )
	log:write("inside the Breach Agents mod")
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	
	modApi.requirements = { "Sim Constructor","Contingency Plan", "Function Library","Programs Extended", "Advanced Guard Protocol"} 
	
	modApi:addGenerationOption("itb_camila", STRINGS.BREACH_AGENTS.OPTIONS.CAMILA, STRINGS.BREACH_AGENTS.OPTIONS.CAMILA_DESC,{noUpdate=true})

	modApi:addGenerationOption("itb_silica", STRINGS.BREACH_AGENTS.OPTIONS.SILICA, STRINGS.BREACH_AGENTS.OPTIONS.SILICA_DESC,{noUpdate=true})
	
	modApi:addGenerationOption("itb_bethany", STRINGS.BREACH_AGENTS.OPTIONS.BETHANY, STRINGS.BREACH_AGENTS.OPTIONS.BETHANY_DESC,{noUpdate=true})

	modApi:addGenerationOption("itb_archimedes", STRINGS.BREACH_AGENTS.OPTIONS.ARCHIMEDES, STRINGS.BREACH_AGENTS.OPTIONS.ARCHIMEDES_DESC,{noUpdate=true})

	modApi:addGenerationOption("itb_henry", STRINGS.BREACH_AGENTS.OPTIONS.HENRY, STRINGS.BREACH_AGENTS.OPTIONS.HENRY_DESC,{noUpdate=true})

	modApi:addGenerationOption("itb_lily", STRINGS.BREACH_AGENTS.OPTIONS.LILY, STRINGS.BREACH_AGENTS.OPTIONS.LILY_DESC,{noUpdate=true})	
	
	modApi:addGenerationOption("itb_gana", STRINGS.BREACH_AGENTS.OPTIONS.GANA, STRINGS.BREACH_AGENTS.OPTIONS.GANA_DESC,{noUpdate=true})		
	
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" ) 
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/images.kwad", "data" )
	
end
	
local function lateInit( modApi )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
		
	-- CAMILA
	
	local simquery = include("sim/simquery")
	local unitdefs = include("sim/unitdefs")
	local simfactory = include("sim/simfactory")
	
	local simunit = include("sim/simunit")
	local simunit_useAP_old = simunit.useAP

	-- simunit.useAP = function( self, sim, ... )
		-- simunit_useAP_old( self, sim, ... )
		-- if self:getTraits().ITB_smokeAttackBonus and isInSmoke( self, sim ) and not self:getTraits().usedSmokeBonus then
			-- self:getTraits().ap = self:getTraits().ap + 1
			-- self:getTraits().usedSmokeBonus = true
			-- sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self  } )
			-- local x1, y1 = self:getLocation()
			-- sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.EXTRA_ATTACK,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
		-- end
	-- end
	
	local simunit_addMPMax_old = simunit.addMPMax
	simunit.addMPMax = function( self, delta )		
		if self:getSim() and self:getSim():getTags().ITB_camilaCheck and self:getTraits().ITB_smokeAttackBonus then
			-- log:write("Camila: Skipping AP change")
			return
		end
		return simunit_addMPMax_old( self, delta )
	end	
	
	local simquery_calculateDamageAndArmorForMelee_old = simquery.calculateDamageAndArmorForMelee
	simquery.calculateDamageAndArmorForMelee = function( sim, sourceUnit, targetUnit, ... )
		local meleeDamage, armorPiercing, armor = simquery_calculateDamageAndArmorForMelee_old( sim, sourceUnit, targetUnit, ... )
		local unitOwner = sourceUnit:getUnitOwner()
		if unitOwner and unitOwner:getTraits().ITB_smokeAttackBonus and isInSmoke( unitOwner, sim ) then
			armorPiercing = armorPiercing + 1
		end
		return meleeDamage, armorPiercing, armor
	end

	local simunit_addMP_old = simunit.addMP
	simunit.addMP = function( self, delta )
		if self:getSim() and self:getSim():getTags().ITB_camilaCheck and self:getTraits().ITB_smokeAttackBonus then
			-- log:write("Camila: Skipping AP change")
			return
		end
		return simunit_addMP_old( self, delta )
	end
	
	-- Don't display the floaty AP loss notification for Camila. AP is given back in mainframe_abilities
	local simengine = include("sim/engine")
	local simengine_dispatchEvent_old = simengine.dispatchEvent
	simengine.dispatchEvent = function( self, evType, evData, noSkip, ... )
		assert( evType )
		if self:getTags().ITB_camilaCheck and (evType == simdefs.EV_UNIT_FLOAT_TXT) and evData.txt and
		((evData.txt == util.sformat("<c:FFFF33>"..STRINGS.DLC1.PROGRAMS.BURST.WARNING .."</>\n"..STRINGS.FORMATS.AP,-3)))
		and CamilaIsThere( self, evData.x, evData.y ) then
			return
		end
		
		-- GANA
		-- remove Gana from the units that get a teleportation effect
		-- for mid_1 and ending_1, pass the trigger for Gana's despawn here... super hacky but oh well
		if (evType == simdefs.EV_TELEPORT) and (evData.warpOut ~= nil) and (evData.warpOut == false) then
			if self:getTags().makingAgentConnection then
				for i = #evData.units, 1, -1 do
					local agent = evData.units[i]
					if agent:getTraits().ITB_Gana or (agent:getTraits().ITB_Gana_a and GanaNotAlone(self, agent)) and not agent:getTraits().GanaSpawned then
						table.remove(evData.units, i)
						self:dispatchEvent( simdefs.EV_UNIT_HIDE, { unit = agent, hide = true } )
						agent:getTraits().GanaHidden = true
						log:write("ITB hiding Gana A")
					end
				end

			elseif (self:getParams().situationName == "mid_1") or (self:getParams().situationName == "ending_1")
			or (self:getParams().situationName == "mid_2")  
			then
				local needTrigger = false
				-- log:write("ITB log story mode gana")
				for i = #evData.units, 1, -1 do
					local agent = evData.units[i]
					-- log:write("LOG agent traits")
					-- log:write(util.stringize(agent:getTraits(),2))
					if (agent:getTraits().ITB_Gana_a and GanaNotAlone(self, agent)) and not agent:getTraits().GanaSpawned then
						table.remove(evData.units, i)
						self:dispatchEvent( simdefs.EV_UNIT_HIDE, { unit = agent, hide = true } )
						needTrigger = true
						if not agent:getTraits().ITB_Gana_ult then
							agent:getTraits().GanaHidden = true
							log:write("ITB hiding Gana B")
						end
					end
				end
				if needTrigger then
					self:triggerEvent("Gana_a_finishedAgentConnection")
				end
			end
		end		
		
		-- BETHANY
		-- Freezegun: Redirect hit event to 	
		if (evType == simdefs.EV_UNIT_HIT) and evData.unit and evData.unit:getTraits().frozenStagger  then
			local targetUnit = evData.unit
			
			local sim = self
			-- hit the unit
			targetUnit:getTraits().frozenRenderFilter = frozenRenderFilter
			evData.unit:getTraits().frozenStagger = nil
			if not evData.unit:getTraits().wasKO then
				self:dispatchEvent( simdefs.EV_UNIT_HIT, {unit = evData.unit, result = 0} )
			end
			
			-- spawn the ice
			local iceCell = sim:getCell(targetUnit:getLocation())
			local unitData = unitdefs.lookupTemplate("ITB_iceblock_bethany")
			local iceUnit = simfactory.createUnit( unitData, sim )
			sim:spawnUnit( iceUnit )
			sim:warpUnit( iceUnit, iceCell )
			local newFacing = simquery.getReverseDirection(iceUnit:getFacing())
			if (sim:nextRand() <= 0.5) then
				iceUnit:setFacing( newFacing )
			end
			iceUnit:getTraits().frozenUnit = targetUnit:getID()
			
			evData.unit:getTraits().frozenStagger2 = true
			return
		end
		
		if (evType == simdefs.EV_UNIT_KO) and evData.unit and not evData.unit:getTraits().wasKO then 	
			if evData.unit:getTraits().frozenStagger2 then
				evData.unit:getTraits().frozenStagger2 = nil
				return
			end
			if evData.stand and evData.unit:getTraits().frozenByCryoGun then
				-- replace with stagger animation
				evType = simdefs.EV_UNIT_HIT
				evData = {unit = evData.unit, result = 0}
			end
			evData.unit:getTraits().wasKO = nil
		end
		
		return simengine_dispatchEvent_old( self, evType, evData, noSkip, ... )
	end	
--------------------------------------------------------	
	-- SILICA ONFILE	
	local simunit_removeChild_old = simunit.removeChild
	simunit.removeChild = function( self, childUnit, ... )
		simunit_removeChild_old( self, childUnit, ... )
		if childUnit:getTraits().SilicaXray then
			childUnit:getTraits().SilicaXray = nil
			childUnit:getTraits().xray = nil
		end
	end

	local inventory = include("sim/inventory")
	local unequipItem_old = inventory.unequipItem
	inventory.unequipItem = function( userUnit, item, ... )
		unequipItem_old( userUnit, item, ... )
		if item:getTraits().SilicaXray then
			item:getTraits().SilicaXray = nil
			item:getTraits().xray = nil
		end
	end

	local equipItem_old = inventory.equipItem
	inventory.equipItem = function( userUnit, item, ... )
		equipItem_old( userUnit, item, ... )
		local alreadyHadXray = false
		if item:getTraits().xray then
			alreadyHadXray = true
		end
		if userUnit:getTraits().OFSilica and item:getTraits().slot and (item:getTraits().slot == "gun") and not alreadyHadXray then
			item:getTraits().SilicaXray = true
			item:getTraits().xray = true
		end
	end

	local simengine_canUnitSeeUnit_old = simengine.canUnitSeeUnit
	simengine.canUnitSeeUnit = function( self, unit, targetUnit, ... )
		if self:getTags().OFSilica_shooting and unit:getTraits().OFSilica and self:canPlayerSeeUnit( unit:getPlayerOwner(), targetUnit ) then
			-- log:write("can see!")
			return true
		end
		-- new onfile HENRY
		if unit:isNPC() and targetUnit:getTraits().notSeenByEnemy then
			if self:getCell(unit:getLocation()) == self:getCell(targetUnit:getLocation()) then
				return false, false
			end
		end
		
		return simengine_canUnitSeeUnit_old( self, unit, targetUnit, ... )
	end

	-- SILICA ARCHIVE
	local simunit_useAP_old = simunit.useAP
	simunit.useAP = function( self, sim, ... )
		simunit_useAP_old( self, sim, ... )
		if self:getTraits().ARCHSilica and self:getTraits().notYetMoved and self:getLocation() then
			self:getTraits().ap = self:getTraits().apMax
			self:getTraits().unlimitedAttacks = true
			-- local x1, y1 = self:getLocation()
			-- sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.AMPED,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} ) 
			-- sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.BREACH_AGENTS.ITEMS.SILICA_AUGMENT2_NAME, x = x1, y = y1, color=cdefs.AUGMENT_TXT_COLOR} )
			-- sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit =self  } )			
		end
	end
--------------------------------------------------------
	-- BETHANY ONFILE
	local simunit_setInvisible_old = simunit.setInvisible
	simunit.setInvisible = function( self, state, duration, ... )
		simunit_setInvisible_old( self, state, duration, ... )
		if (state == false) and self:getTraits().BethanyCloak then
			self:getTraits().BethanyCloak = nil
		end
	end	
	
	-- BETHANY ARCHIVE
	-- for ice
	include( scriptPath .. "/simunits/ITB_iceblock" )
	include( scriptPath .. "/simunits/ITB_shieldingRig")
	include( scriptPath .. "/simunits/ITB_iceMine")
	include( scriptPath .. "/simunits/ITB_iceSmoke")
	include( scriptPath .. "/unitrig" )	
	
	local grenaderig = include("gameplay/grenaderig").rig
	local grenaderig_refresh_old = grenaderig.refresh

	-- this disables the flicker animation when someone is inside the ice "hologram"
	grenaderig.refresh = function( self, ... )
		grenaderig_refresh_old( self, ... )
		local unit = self:getUnit()
		if unit:getTraits().ITB_ice and unit:getTraits().holoProjector and unit:getTraits().deployed and not unit:getTraits().disruptAnim then
			local sim = unit:getSim()
			local cell = sim:getCell( unit:getLocation() )
			if cell and checkForUnits(unit,cell) then
				self:setCurrentAnim( "idle_hologram" )
			end
		end
	end	
	
	local simquery = include( "sim/simquery" )
	local simquery_calculateShotSuccess_old = simquery.calculateShotSuccess

	simquery.calculateShotSuccess = function( sim, sourceUnit, targetUnit, equipped, ... )
		local shot = simquery_calculateShotSuccess_old( sim, sourceUnit, targetUnit, equipped, ... )
		if shot and equipped and equipped:getTraits().freezeGun then
			shot.freezeGun = true
			if targetUnit:isKO() and targetUnit:getTraits().frozenByCryoGun then
				shot.alreadyFrozen = true
			end
		end
		return shot
	end
	
	local COLOR_RED_DARK = util.color(80/255,0/255,0/255,150/255)
	local COLOR_RED_LIGHT = util.color(163/255,0/255,0/255,1)
	
	local combat_panel = include( "hud/combat_panel" )
	local combat_panel_refreshShoot_old = combat_panel.refreshShoot
	combat_panel.refreshShoot = function( self, weaponUnit, unit, targetUnit )
		combat_panel_refreshShoot_old( self, weaponUnit, unit, targetUnit )
		local sim = self._hud._game.simCore
		local shot = sim:getQuery().calculateShotSuccess( sim, unit, targetUnit, weaponUnit )
		local color, textColor = COLOR_RED_DARK, COLOR_RED_LIGHT
		if shot.freezeGun and shot.alreadyFrozen then
			local txt = util.toupper(STRINGS.BREACH_AGENTS.UI.ALREADY_FROZEN)
			-- self._panel.binder.damageTxt:setText("<font1_24_r>"..txt.."</>")
			self._panel.binder.damageTxt:setText(txt)				
			self._panel:setVisible( true )
			self._panel.binder.hitBackground:setColor(color:unpack())
			self._panel.binder.targetTxt:setColor(textColor:unpack())
			self._panel.binder.damageTxt:setColor(textColor:unpack())
		end
	end	
	
	local simunit_setKO_old = simunit.setKO --reset freezegun-related statuses on wakeup, just in case
	simunit.setKO = function( self, sim, ticks, fx, ... )
		-- ARCHIVE PROSPERO
		-- local cell = sim:getCell( self:getLocation() )
		-- if cellHasProsperoVines( sim, cell ) then
			-- if self:isNPC() and (ticks == nil) then
				-- return
			-- end
		-- end
		if ticks == nil and self and self:getTraits() then
			self:getTraits().frozenStagger = nil
			self:getTraits().frozenAnim = nil
			self:getTraits().frozenByCryoGun = nil
			self:getTraits().frozenRenderFilter = nil
			-- this ought to get taken care of by the iceblock unit but just in case...		
		end		
		return simunit_setKO_old( self, sim, ticks, fx, ... )
	end	
	
--------------------------------------------------------	
	-- ARCHIMEDES + OF SILICA
	local abilitydefs = include("sim/abilitydefs")
	local shootSingle = abilitydefs.lookupAbility("shootSingle")
	
	-- BETHANY
	local simunit = include("sim/simunit")
	local simunit_canHide_old = simunit.canHide
	simunit.canHide = function( self, ... )
		if self:getTraits().frozenRenderFilter then
			return false --frozen guards are always visible if in 'cover'
		end
		return simunit_canHide_old( self, ... )
	end
	
	local shootSingle_canUse_old = shootSingle.canUseAbility
	shootSingle.canUseAbility = function( self, sim, ownerUnit, unit, targetUnitID, ... )
		local targetUnit = sim:getUnit( targetUnitID )
		local result = shootSingle_canUse_old( self, sim, ownerUnit, unit, targetUnitID, ... )
		if (result == true) and targetUnit and targetUnit:getTraits().frozenByCryoGun then
			return false
		end
		return result
	end
	
	local shootSingle_execute_old = shootSingle.executeAbility

	shootSingle.executeAbility = function( self, sim, ownerUnit, unit, targetUnitID, ... )
		local weaponUnit = ownerUnit
		local targetUnit = sim:getUnit(targetUnitID)
		if unit:getTraits().ITB_Archimedes then
			targetUnit:getTraits().ITB_ArchieShooting = true
		end
		if targetUnit and weaponUnit and weaponUnit:getTraits().freezeGun then
			-- log:write("target frozen stagger")
			targetUnit:getTraits().frozenStagger = true --Bethany cryogun, for FX
		end
		local wasKO = targetUnit:isKO()
		if wasKO then
			targetUnit:getTraits().wasKO = true
		end
		shootSingle_execute_old( self, sim, ownerUnit, unit, targetUnitID, ... )
		if targetUnit then
			targetUnit:getTraits().ITB_ArchieShooting = nil
		end
			
		-- BETHANY
		local weaponUnit = ownerUnit
		if weaponUnit and weaponUnit:getTraits().freezeGun and targetUnit and not wasKO then
			-- set anim on target
			local facing = targetUnit:getFacing()
			if (facing % 2 ~= 0) then
				local newOrthogFacing = facing + 1
				if newOrthogFacing > 7 then
					newOrthogFacing = 0
				end
				targetUnit:updateFacing( newOrthogFacing )
			end
			local frozenAnims = weaponUnit:getTraits().possibleFrozenAnims
			local frozenAnimation = frozenAnims[sim:nextRand(1,#frozenAnims)]
			targetUnit:getTraits().frozenAnim = frozenAnimation
			targetUnit:getTraits().frozenByCryoGun = true
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )

			-- -- BETHANY
			-- -- spawn the ice
			-- local iceCell = sim:getCell(targetUnit:getLocation())
			-- local unitData = unitdefs.lookupTemplate("ITB_iceblock_bethany")
			-- local iceUnit = simfactory.createUnit( unitData, sim )
			-- sim:spawnUnit( iceUnit )
			-- sim:warpUnit( iceUnit, iceCell )
			-- iceUnit:getTraits().frozenUnit = targetUnit:getID()
		end
	end		
	
	local shootOverwatch = abilitydefs.lookupAbility("shootOverwatch")
	local shootOverwatch_onTrigger_old = shootOverwatch.onTrigger
	shootOverwatch.onTrigger = function( self, sim, evType, evData, userUnit, ... )
		local targetUnit = evData
		if targetUnit and userUnit:getTraits().ITB_Archimedes then
			targetUnit:getTraits().ITB_ArchieShooting = true
		end
		if targetUnit and userUnit:getTraits().OFSilica then
			sim:getTags().OFSilica_shooting = true
		end
		shootOverwatch_onTrigger_old( self, sim, evType, evData, userUnit, ... )
		if targetUnit then
			targetUnit:getTraits().ITB_ArchieShooting = nil
		end
		sim:getTags().OFSilica_shooting = nil
	end

	local shootOverwatch_executeAbility_old = shootOverwatch.executeAbility
	shootOverwatch.executeAbility = function( self, sim, unit, userUnit, targetUnit, ... )
		local weaponUnit = unit
		if targetUnit and weaponUnit and weaponUnit:getTraits().freezeGun then
			-- log:write("target frozen stagger")
			targetUnit:getTraits().frozenStagger = true --Bethany cryogun, for FX
		end
		local wasKO = targetUnit:isKO()
		if wasKO then
			targetUnit:getTraits().wasKO = true
		end
		
		shootOverwatch_executeAbility_old( self, sim, unit, userUnit, targetUnit, ... )
		
		-- BETHANY
		if weaponUnit and weaponUnit:getTraits().freezeGun and targetUnit and not wasKO and targetUnit:isValid() and targetUnit:isKO() then
			-- set anim on target
			local facing = targetUnit:getFacing()
			if (facing % 2 ~= 0) then
				local newOrthogFacing = facing + 1
				if newOrthogFacing > 7 then
					newOrthogFacing = 0
				end
				targetUnit:updateFacing( newOrthogFacing )
			end			
			local frozenAnims = weaponUnit:getTraits().possibleFrozenAnims
			local frozenAnimation = frozenAnims[sim:nextRand(1,#frozenAnims)]
			targetUnit:getTraits().frozenAnim = frozenAnimation
			targetUnit:getTraits().frozenByCryoGun = true
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
		if targetUnit and targetUnit:isValid() and not targetUnit:isKO() then
			targetUnit:getTraits().frozenStagger = nil
		end
	end

	local simunit_killUnit_old = simunit.killUnit
	simunit.killUnit = function( self, sim, ... )
		if self:getTraits().ITB_ArchieShooting then
			sim:triggerEvent( "TRG_UNIT_KILLED_BY_ARCHIMEDES")
		end
		simunit_killUnit_old( self, sim, ... )
	end
	
	-- ARCHIMEDES ONFILE
	local inventory = include("sim/inventory")
	local inventory_useItem_old = inventory.useItem
	inventory.useItem = function( sim, unit, item, ... )
		local x1, y1 = unit:getLocation()
		local cooldown = item:getTraits().cooldownMax or 0
		if item:getTraits().cooldown then
			local mod = 0
			if unit:countAugments( "augment_torque_injectors" ) > 0 then			
				mod = mod +1
			end
			cooldown = math.max((item:getTraits().cooldownMax or 0) - mod, 0)
		end
		inventory_useItem_old( sim, unit, item, ... )
		-- careful: both agent and item may have despawned now
		if item and item:getTraits().cooldown then
			cooldown = item:getTraits().cooldown
		end
		if (cooldown > 0) and unit and (unit:countAugments("ITB_augment_archimedes") > 0 ) then
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = STRINGS.BREACH_AGENTS.ITEMS.ARCHIMEDES_AUGMENT_NAME, x = x1, y = y1,color={r=255/255,g=255/255,b=51/255,a=1}} )
			unit:addMP( cooldown )
		end
	end
--------------------------------------------------------	
	-- GANA
	local mission_util = include("sim/missions/mission_util")
	local makeAgentConnection_old = mission_util.makeAgentConnection
	mission_util.makeAgentConnection = function( script, sim, ... )
		log:write("ITB makeAgentConnection wrap")
		sim:getTags().makingAgentConnection = true -- flag for dispatchEvent
		makeAgentConnection_old( script, sim, ... )
		sim:getTags().makingAgentConnection = nil
		log:write("Triggering Gana finishedAgentConnection")
		sim:triggerEvent("Gana_finishedAgentConnection")
		sim:triggerEvent("Gana_a_finishedAgentConnection")
	end
	
	local storySituations = {["mid_1"] = true, ["mid_2"] = true, ["ending_1"] = true }
	
	local simengine_spawnUnit_old = simengine.spawnUnit
	simengine.spawnUnit = function( self, unit, ... )
		simengine_spawnUnit_old( self, unit, ... )
		if not storySituations[self:getParams().situationName] and unit:getTraits().ITB_Gana and (#self:getLevelScript().scripts == 0) then --last check is to make sure this wrap doesn't affect non-deployment spawns, such as debug mode or rescue
			unit:getTraits().hasSight = false
			unit._seenUnits = {}
		end	
	end
	
	-- GANA ARCHIVE
	local mainframe = include("sim/mainframe")
	local mainframe_init_old = mainframe.init
	mainframe.init = function( sim, ... )
		local result = mainframe_init_old( sim, ... )
		sim:triggerEvent( "MAINFRAME_INIT", {} )
		return result
	end
-----------------------------------------------------
	-- LILY
	local peek = abilitydefs.lookupAbility("peek")
	local peek_canUse_old = peek.canUseAbility
	peek.canUseAbility = function( self, sim, unit, ... )
		local alarm = sim:getTrackerStage()
		if unit:getTraits().freePeeking and (alarm < 1) then
			unit:getTraits().mp = unit:getTraits().mp + simdefs.DEFAULT_COST
			local result, reason = peek_canUse_old( self, sim, unit, ... ) --this hack determines if canUse would otherwise return true if not for AP
			unit:getTraits().mp = unit:getTraits().mp - simdefs.DEFAULT_COST
			if result == true then
				return result --safely allow free peeking
			end
		end
		
		return peek_canUse_old( self, sim, unit, ... )
	end

	local peek_execute_old = peek.executeAbility
	peek.executeAbility = function( self, sim, unit, userUnit, exitX, exitY, exitDir, ... )
		local alarm = sim:getTrackerStage()
		local oldMP = unit:getMP()
		peek_execute_old( self, sim, unit, userUnit, exitX, exitY, exitDir, ... )
		if unit:getTraits().freePeeking and (alarm < 1) then
			-- unit:addMP( simdefs.DEFAULT_COST ) -- this is exploity whoops
			unit:getTraits().mp = oldMP
		end		
	end
	
---------------------------------------------------------
	-- ARCHIVE HENRY (NEW VERSION)
	local turnToFace_old = simunit.turnToFace
	simunit.turnToFace = function( self, x, y, floatTxt, ... )
		local trackingHenry = false
		if floatTxt and (floatTxt == STRINGS.UI.TRACKED) and cellContainsHenry( self:getSim(), x, y ) then
		-- could do this but it's not very robust to rely on UI... eh...
			trackingHenry = true
		end
		if trackingHenry or self:getSim():getTags().doNotTrackThisAgent then --cover our bases
			if x and y then
				self:getSim():dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, { txt=STRINGS.BREACH_AGENTS.UI.HENRY_NO_TRACKING, x=x,y=y, color={r=1,g=1,b=41/255,a=1}, skipQue=true } )
			end
			return
		end
		return turnToFace_old( self, x, y, floatTxt, ... )
	end
	
	local Senses = include("sim/btree/senses")
	local processWarpTrigger_old = Senses.processWarpTrigger
	Senses.processWarpTrigger = function( self, sim, evData, ... )
		sim:getTags().doNotTrackThisAgent = nil --autoclear flag
		if evData.unit and evData.unit:getTraits().ITB_HenryUntrackable then
			sim:getTags().doNotTrackThisAgent = true
		end
		return processWarpTrigger_old( self, sim, evData, ... )
	end
-----------------------------------------------
	
	-- ONFILE HENRY (NEW VERSION)
	local melee = abilitydefs.lookupAbility("melee")
	local melee_canUse_old = melee.canUseAbility
	melee.canUseAbility = function( self, sim, unit, userUnit, targetID, ... )
		if unit:getTraits().ITB_HenryImpass then
			if unit:getTraits().dynamicImpass == false then
				unit:getTraits().dynamicImpass = true
				unit:getTraits().hasNoDynamicImpass = true
			end
			local res, str1, str2, str3, str4 = melee_canUse_old( self, sim, unit, userUnit, targetID, ... )
		
			if unit:getTraits().hasNoDynamicImpass then
				unit:getTraits().hasNoDynamicImpass = nil
				unit:getTraits().dynamicImpass = false
			end
			return res, str1, str2, str3, str4
		end
		return melee_canUse_old( self, sim, unit, userUnit, targetID, ... )
	end
	
	local melee_acquireTargets_old = melee.acquireTargets
	melee.acquireTargets = function( self, targets, game, sim, unit, ... )
		if unit:getTraits().ITB_HenryImpass then
			unit:getTraits().dynamicImpass = true
		end
		local result = melee_acquireTargets_old(self, targets, game, sim, unit, ... )
		if unit:getTraits().ITB_HenryImpass then
			unit:getTraits().dynamicImpass = false
		end		
		return result
	end

	local simactions = include("sim/simactions")
	local moveAction_old = simactions.moveAction

	simactions.moveAction = function( sim, unitID, moveTable, ... )
		
		-- log:write(util.stringize(moveTable,3))
		local unit = sim:getUnit( unitID )
		if unit:getTraits().ITB_HenryImpass then
			local endMove = moveTable[#moveTable]
			local endCell = sim:getCell( endMove.x, endMove.y )
			if not customCanPath( sim, unit, nil, endCell ) then
				local txt1 = util.sformat( STRINGS.UI.WARNING_CANT_MOVE, unit:getName() )
				sim:getLevelScript():queue( { type="showWarning", txt1=txt1, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
				return
			end
			if #moveTable > 0 then
				for i, cellData in pairs(moveTable) do
					local cell = sim:getCell( cellData.x, cellData.y )
					if lethalLaserBlocking( sim, unit, nil, cell ) then
						local txt1 = util.sformat( STRINGS.BREACH_AGENTS.UI.WARNING_CANT_MOVE_LASER, unit:getName() )
						sim:getLevelScript():queue( { type="showWarning", txt1=txt1, sound="SpySociety/HUD/voice/level1/alarmvoice_warning"  } )
						return
					end
				end
			end
		end
		return moveAction_old( sim, unitID, moveTable, ... )
	end
	
	local zonerig = include( "gameplay/zonerig" )
	local setMovementTiles_old = zonerig.setMovementTiles
	zonerig.setMovementTiles = function( self, tiles, grad, line, ... )
		local sim = self._simCore
		if sim:getTags().HenrySelected and sim.HenryAgent then
			if #tiles > 0 then
				for i = #tiles, 1, -1 do
					-- log:write("LOG processing ".. i )
					local cell = tiles[i]
					if not customCanPath(sim, sim.HenryAgent, nil, cell) then
						table.remove( tiles, i )
					end
				end
			end
		end
		return setMovementTiles_old( self, tiles, grad, line, ... )
	end
		
--------------------------------------------------------
	
end

local function load( modApi, options, params, mod_options )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	
	local agentdefs = include( scriptPath .. "/agentdefs" )
	for name, agentDef in pairs(agentdefs) do
		modApi:addAgentDef( name, agentDef )
	end		
	
	local itemdefs = include( scriptPath .. "/itemdefs" )
	for name, itemDef in pairs(itemdefs) do
		modApi:addItemDef( name, itemDef )
	end
	
	local propdefs = include( scriptPath .. "/propdefs" )
	for name, propDef in pairs(propdefs) do
		modApi:addPropDef( name, propDef, false )
	end
	
	local animdefs = include( scriptPath .. "/animdefs" )
	for name, animDef in pairs(animdefs) do
		modApi:addAnimDef( name, animDef )
	end	
	
	local commondefs = include( scriptPath .. "/commondefs" )
	modApi:addTooltipDef( commondefs )	
	
	-- abilities
	modApi:addAbilityDef( "ITB_CamilaAugment", scriptPath .."/abilities/ITB_CamilaAugment" )
	modApi:addAbilityDef( "ITB_SilicaAugment", scriptPath .."/abilities/ITB_SilicaAugment" )
	modApi:addAbilityDef( "ITB_BethanyAugment", scriptPath .."/abilities/ITB_BethanyAugment" )
	modApi:addAbilityDef( "ITB_BethanyAugment2", scriptPath .."/abilities/ITB_BethanyAugment2" )
	modApi:addAbilityDef( "ITB_ArchimedesAugment", scriptPath .."/abilities/ITB_ArchimedesAugment" )
	modApi:addAbilityDef( "ITB_LilyAugment", scriptPath .."/abilities/ITB_LilyAugment" )
	modApi:addAbilityDef( "ITB_giveHenryScurry", scriptPath .."/abilities/ITB_giveHenryScurry" )
	modApi:addAbilityDef( "ITB_GanaAugment", scriptPath .."/abilities/ITB_GanaAugment" )
	modApi:addAbilityDef( "ITB_GanaAugment2", scriptPath .."/abilities/ITB_GanaAugment2" )
	modApi:addAbilityDef( "ITB_GanaLockOpener", scriptPath .."/abilities/ITB_GanaLockOpener" )
	modApi:addAbilityDef( "ITB_HenryOverwatch", scriptPath .."/abilities/ITB_HenryOverwatch" ) --now unused
	modApi:addAbilityDef( "ITB_HenryImpassHelp", scriptPath .."/abilities/ITB_HenryImpassHelp" )
	modApi:addAbilityDef( "ITB_throwIceMine", scriptPath .."/abilities/ITB_throwIceMine" )
	-- modApi:addAbilityDef( "ITB_ProsperoAugment", scriptPath .."/abilities/ITB_ProsperoAugment" )
	-- modApi:addAbilityDef( "ITB_ProsperoAugment2", scriptPath .."/abilities/ITB_ProsperoAugment2" )
	
	local abilitydefs = include( "sim/abilitydefs" )
	local squeeze = include( scriptPath .. "/abilities/ITB_HenryScurry" ) --now unused
	abilitydefs._abilities[ "ITB_henry_squeeze_up" ] = squeeze.ITB_henry_squeeze_up
	abilitydefs._abilities[ "ITB_henry_squeeze_down" ] = squeeze.ITB_henry_squeeze_down
	abilitydefs._abilities[ "ITB_henry_squeeze_left" ] = squeeze.ITB_henry_squeeze_left
	abilitydefs._abilities[ "ITB_henry_squeeze_right" ] = squeeze.ITB_henry_squeeze_right

	local mainframe_abilities = include( scriptPath .. "/mainframe_abilities")
	for name, ability in pairs(mainframe_abilities) do
		modApi:addMainframeAbility( name, ability )
	end	
	
	local serverdefs = include("modules/serverdefs")
	local serverdefs_mod = include( scriptPath .. "/serverdefs" ).TEMPLATE_AGENCY.unitDefsPotential
	
	if options["itb_camila"] and options["itb_camila"].enabled then
		serverdefs.LOADOUTS[ "itb_camila" ] = { "itb_camila" }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_camila" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_camila")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_camila"] )
	end

	if options["itb_silica"] and options["itb_silica"].enabled then
		serverdefs.LOADOUTS[ "itb_silica" ] = { "itb_silica", "itb_silica_a" }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_silica" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_silica")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_silica"] )
	end
	
	if options["itb_bethany"] and options["itb_bethany"].enabled then
		serverdefs.LOADOUTS[ "itb_bethany" ] = {"itb_bethany","itb_bethany_a"  }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_bethany" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_bethany")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_bethany"] )
	end

	if options["itb_archimedes"] and options["itb_archimedes"].enabled then
		serverdefs.LOADOUTS[ "itb_archimedes" ] = { "itb_archimedes", "itb_archimedes_a" }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_archimedes" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_archimedes")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_archimedes"] )
	end

	if options["itb_henry"] and options["itb_henry"].enabled then
		serverdefs.LOADOUTS[ "itb_henry" ] = { "itb_henry", "itb_henry_a" }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_henry" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_henry")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_henry"] )
	end

	if options["itb_lily"] and options["itb_lily"].enabled then
		serverdefs.LOADOUTS[ "itb_lily" ] = { "itb_lily" }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_lily" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_lily")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_lily"] )
	end
	
	if options["itb_gana"] and options["itb_gana"].enabled then
		serverdefs.LOADOUTS[ "itb_gana" ] = { "itb_gana", "itb_gana_a" }
		if array.find( serverdefs.SELECTABLE_AGENTS, "itb_gana" ) == nil then
			table.insert( serverdefs.SELECTABLE_AGENTS, "itb_gana")
		end		
		modApi:addRescueAgent( serverdefs_mod["itb_gana"] )
	end		
	
	-- appends to vanilla programs
	local program_changes = include( scriptPath .. "/program_changes" )
	program_changes.runAppend()
	
end	

return {
    init = init,
	initStrings = initStrings,
	lateInit = lateInit,
    load = load,
}
