local util = include("client_util")
local mui = include( "mui/mui" )
local simdefs = include("sim/simdefs")
local mui_screen = include( "mui/mui_screen" )

local oldInitMui = mui.initMui

local function loadUI( filename )
	fullFilePath = mui.internals._internals.resolveFilename( filename )
	assert( fullFilePath ) -- Important assert, since loadfile's behaviour for nil filename is to read from stdin (can you say HANG?)
	local f,e = loadfile( fullFilePath )
	
	if not f then
		if mod_manager.screenModding.newFilePaths[filename] then
			-- Create new GUI file from a custom file path
			filename = mod_manager.screenModding.newFilePaths[filename]
			if not filename:find( ".lua" ) then
				filename = filename..".lua"
			end
			f,e = loadfile( filename )
		elseif mod_manager.screenModding.newFilesRaw[filename] then
			-- Create new GUI file from raw data
			return mod_manager.screenModding.newFilesRaw[filename]
		elseif simdefs.SCREEN_CUSTOMS[filename] then
			-- Legacy system for UI changes
			return simdefs.SCREEN_CUSTOMS[filename]
		end
	end
	
	assert( f, e )

	-- Setup an environment to load the UI in so as to not clobber the global env.
	local envmt = { __index = _G }
	setfenv(f, setmetatable({}, envmt))

	return f()
end


local function modify( t1, t2 )
	assert( type(t1) == "table" and type(t2) == "table", string.format("type(t1) == %s, type(t2) == %s", type(t1), type(t2)) )
	-- Merge all of t2 into t1
	for k,v in pairs( t2 ) do
		if t1[k] == nil or type(v) ~= "table" then
			t1[k] = v
		else
			modify( t1[k], v )
		end
	end
end

local function getElementAtPath( t, path, filename )
	for depth, pathKey in ipairs(path) do
		if t[pathKey] then
			t = t[pathKey]
		else
			local foundName = false
			if type(pathKey) == "string" then
				for i, item in ipairs(t) do
					if type(item) == "table" and item.name == pathKey then
						foundName = true
						t = item
						break
					end
				end
			end
		
			if not foundName then
				local pathstring = ""
				
				for d, pathElement in ipairs(path) do
					pathstring = pathstring..pathElement.."/"
				end
			
				log:write(string.format("[Sim Constructor] Failed to get element with path %s in screen %s, no element with key %s at depth %i", pathstring, filename, pathKey, depth))
				return nil
			end
		end
	end
	
	return t
end

local function createScreen( filename )
	local t = mui.loadUI( filename )
	
	-- Legacy system for UI changes
	if simdefs.SCREEN_CUSTOMS[filename] then
		t = util.extend( t )( util.tcopy(simdefs.SCREEN_CUSTOMS[filename]) )
	end
	
	-- Change existing hud elements
	if mod_manager.screenModding.changes[filename] then
		for i, change in ipairs(mod_manager.screenModding.changes[filename]) do
			local path = change[1]
			local changedElements = change[2]
			local elementToModify = getElementAtPath( t, path, filename )
			
			if elementToModify then
				modify( elementToModify, changedElements )
			end
		end
	end
	
	-- Insert new hud elements
	if mod_manager.screenModding.inserts[filename] then
		for i, insert in ipairs(mod_manager.screenModding.inserts[filename]) do
			local path = insert[1]
			local newElement = insert[2]
			local index = insert[3] or 0
			local parentElement = getElementAtPath( t, path, filename )
			
			if parentElement then
				if index <= 0 then
					index = #parentElement + 1 + index
				end
				
				table.insert(parentElement, index, newElement)
			end
		end
	end
	
	mui.parseUI( filename, t )

	-- Instantiate the widgets.
	if mui.internals._files[ filename ] then
		assert( mui.internals._files[ filename ].widgets, "Creating screen for " ..filename.. ", but no widgets specified.")

		local screen = mui_screen( mui.internals._internals, mui.internals._files[ filename ], filename )
		for i,def in ipairs(mui.internals._files[ filename ].widgets) do
			local widget = screen:createWidget( def )
			screen:addWidget( widget )
		end
		
		if screen.resetFocusedWidget then
			screen:resetFocusedWidget()
		end

		return screen
	end
end

local function initMui(...)
    oldInitMui(...)
	util.tmerge( mui.internals._widgetFactory, mod_manager.widgetFactories )
end

mui.loadUI = loadUI
mui.initMui = initMui
mui.createScreen = createScreen