local array = include( "modules/array" )
local util = include( "modules/util" )
local mui_widget = include( "mui/widgets/mui_widget" )
local mui_texture = include( "mui/widgets/mui_texture" )
local mui_component = include( "mui/widgets/mui_component" )
local mui_container = include( "mui/widgets/mui_container" )
include( "class" )

--------------------------------------------------------

local loadImages = upvalueUtil.find( mui_texture.init, "loadImages" )

--------------------------------------------------------

--local mui_radialtexture = class( mui_texture )
local mui_radialtexture = class( mui_component )

function mui_radialtexture:init( screen, def )
	self._screen = screen
	self._images = loadImages( screen, def.images )
	self._progress = def.value or 0
	self._angleOffset = def.angleOffset or 0
	self._reverse = def.reverse
	
	self._deck = MOAIGfxQuadListDeck2D.new()
	
	local prop = MOAIProp2D.new ()
	prop:setDepthMask( false )
	prop:setBlendMode( MOAIProp.BLEND_NORMAL )
	
	mui_component.init( self, prop, def )
    
    if self._images and self._images[1] then
        self:setImageIndex( 1 )
    end
	
	self:setProgress( self._progress )
end

local function getVertex( val )
	if val < 0.25 then
		return { 4 * val,          1,                0.25 }
	elseif val < 0.5 then
		return { 1,                4 * (0.5 - val),  0.5 }
	elseif val < 0.75 then
		return { 4 * (0.75 - val), 0,                0.75 }
	elseif val < 1 then
		return { 0,                4 * (val - 0.75), 1 }
	elseif val < 1.25 then
		return { 4 * (val - 1),    1,                1.25 }
	elseif val < 1.5 then
		return { 1,                4 * (1.5 - val),  1.5 }
	elseif val < 1.75 then
		return { 4 * (1.75 - val), 0,                1.75 }
	else
		return { 0,                4 * (val - 1.75), 2 }
	end
end

function mui_radialtexture:setProgress( val )
	self._progress = math.max( 0, math.min( 1.0, val ))
	
	local fromA = (self._angleOffset + 0.125) % 1
	local toA = fromA + self._progress
	
	local x, y, w, h = self:calculateBounds()
	local points = {}
	local v0 = getVertex(fromA)
	table.insert(points, v0)
	
	while true do
		if v0[3] < toA then
			v0 = getVertex(v0[3])
			table.insert(points, v0)
		else
			v0 = getVertex(toA)
			table.insert(points, v0)
			break
		end
	end
	
	if self._reverse then
		for i, vertex in ipairs(points) do
			vertex[1] = 1 - vertex[1]
		end
		array.reverse(points)
	end
	
	local numQuads = math.floor(#points / 2)
	table.insert(points, points[#points]) -- Append an extra vertex to handle odd number of triangles
	
	self._deck:reserveLists(1)
	self._deck:reserveQuads(numQuads)
	self._deck:reserveUVQuads(numQuads)
	self._deck:reservePairs(numQuads)
	self._deck:setList(1, 1, numQuads)
	
	for i = 1, numQuads do
		self._deck:setQuad(
			i,
			0, 0,
			w * (points[2 * i - 1][1] - 0.5), h * ( points[2 * i - 1][2] - 0.5),
			w * (points[2 * i    ][1] - 0.5), h * ( points[2 * i    ][2] - 0.5),
			w * (points[2 * i + 1][1] - 0.5), h * ( points[2 * i + 1][2] - 0.5)
		)
		
		self._deck:setUVQuad(
			i,
			0.5, 0.5,
			points[2 * i - 1][1], 1 - points[2 * i - 1][2],
			points[2 * i    ][1], 1 - points[2 * i    ][2],
			points[2 * i + 1][1], 1 - points[2 * i + 1][2]
		)
		
		self._deck:setPair(i, i, i)
	end
end

function mui_radialtexture:setAngleOffset( val )
	self._angleOffset = val
end

function mui_radialtexture:setImageIndex( idx )
	if self._images[idx] then
		self._deck:setTexture( self._images[idx].image )
		self._prop:setDeck( self._deck )
		if self._images[idx].color then
			self._prop:setColor( unpack(self._images[idx].color) )
		end
	else
		self._prop:setDeck(nil)
	end
end

function mui_radialtexture:setColor( r, g, b, a )
	if self._prop.setSymbolModulate then
		self._prop:setSymbolModulate( "", r, g, b, a )
	else
		-- Prop color uses pre-multiplied alpha.
		a = a or 1
		self._prop:setColor( r * a, g * a, b * a, a )
	end
end

--------------------------------------------------------

local mui_progressbar = class( mui_widget )

function mui_progressbar:init( screen, def )
	mui_widget.init( self, def )
	
	local insetX = def.inset_size and def.inset_size.w or 0
	local insetY = def.inset_size and def.inset_size.h or 0

	self._bgImg = mui_texture( screen, { x = 0, y = 0, w = def.w, wpx = def.wpx, h = def.h, hpx = def.hpx, images = def.bg_images })
	--self._progressImg = mui_texture( screen, { x = 0, y = 0, w = def.w, wpx = def.wpx, h = def.h, hpx = def.hpx, images = def.images })
	
	self._progressImg = mui_radialtexture( screen, { x = 0, y = 0, value = def.value, angleOffset = def.angleOffset, reverse = def.reverse,
		xpx = def.wpx, ypx = def.hpx, wpx = def.wpx, hpx = def.hpx,
		w = def.w - 2 * insetX, h = def.h - 2 * insetY, images = def.progress_images or def.images } )

	self._cont = mui_container( def )
	self._cont:addComponent( self._bgImg )
	self._cont:addComponent( self._progressImg )
	
	if def.spinSpeed then
		self:spin(def.spinSpeed, def.offsetSpinSpeed)
	end
end

function mui_progressbar:setProgressColor( r, g, b, a )
	self._progressImg:setColor( r, g, b, a )
end

function mui_progressbar:setBGColor( r, g, b, a )
	self._bgImg:setColor( r, g, b, a )
end

function mui_progressbar:setProgress( val )
	self._progressImg:setProgress( val )
end

function mui_progressbar:setAngleOffset( val )
	self._progressImg:setAngleOffset( val )
end

function mui_progressbar:reverse( val )
	if val == false then
		self._progressImg._reverse = false
	else
		self._progressImg._reverse = true
	end
end

function mui_progressbar:getProgress()
	return self._progressImg._progress
end

function mui_progressbar:getAngleOffset()
	return self._progressImg._angleOffset
end

function mui_progressbar:animateProgress( fromProgress, toProgress, speed )
	if self._thread then
		self._thread:stop()
	end
	
	fromProgress = math.max( 0, math.min( 1.0, fromProgress ))
	toProgress = math.max( 0, math.min( 1.0, toProgress ))
	
	if speed < 0 then
		local temp = fromProgress
		fromProgress = toProgress
		toProgress = temp
		speed = -speed
	end
	
	self:setProgress( fromProgress )
	self._thread = MOAICoroutine.new()
    self._thread:run(
		function()
			if toProgress > fromProgress then
				while fromProgress < toProgress do
					self:setProgress( fromProgress )
					coroutine.yield()
					fromProgress = self._progressImg._progress + speed / 60
				end
			else
				while fromProgress > toProgress do
					self:setProgress( fromProgress )
					coroutine.yield()
					fromProgress = self._progressImg._progress - speed / 60
				end
			end
			
			self:setProgress( toProgress )
			self._thread = nil
		end
	)
end

function mui_progressbar:spin( speed, offsetSpeed )
	if self._thread then
		self._thread:stop()
	end
	
	speed = speed or 1
	offsetSpeed = offsetSpeed or 0
	self._thread = MOAICoroutine.new()
    self._thread:run(
		function()
			while true do
				coroutine.yield()
				self:setAngleOffset( self._progressImg._angleOffset + offsetSpeed / 60 )
				local progress = self._progressImg._progress + speed / 60
				
				if progress > 1 then
					self:setProgress( 0 )
				elseif progress < 0 then
					self:setProgress( 1 )
				else
					self:setProgress( progress )
				end
			end
		end
	)
end

function mui_progressbar:stop()
	if self._thread then
		self._thread:stop()
		self._thread = nil
	end
end

return 
{
	radialtexture = mui_radialtexture,
	radialprogressbar = mui_progressbar,
}
