-----------------------------------------------------------
-- myUtil
-- Various functions available for use throughout the mod.
local util = include("modules/util")

local function addAbilityToAllAgents(abilityName)
	local agentdefs = include("sim/unitdefs/agentdefs")

	for name, agentdef in pairs(agentdefs) do
		-- Make sure we don't add the ability twice to the same agent
		if util.indexOf(agentdef.abilities, abilityName) == nil then
			table.insert(agentdef.abilities, abilityName)
		end
	end
end

local function removeAbilityFromAllAgents(abilityName)
	local removeIndex
	local agentdefs = include("sim/unitdefs/agentdefs")

	for name, agentdef in pairs(agentdefs) do
		removeIndex = util.indexOf(agentdef.abilities, abilityName)
		if removeIndex ~= nil then
			table.remove(agentdef.abilities, removeIndex)
		end
	end
	-- It would appear that at least one of the agent defs references commondefs (and not just a copy), so we need to make sure we remove the abilty from commondefs as well
	local commondefs = include("sim/unitdefs/commondefs")
	removeIndex = util.indexOf(commondefs.DEFAULT_AGENT_ABILITIES, abilityName)
	if removeIndex ~= nil then
		table.remove(commondefs.DEFAULT_AGENT_ABILITIES, removeIndex)
	end
end

return {
	addAbilityToAllAgents = addAbilityToAllAgents,
	removeAbilityFromAllAgents = removeAbilityFromAllAgents
}
