local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local cdefs = include( "client_defs" )
local serverdefs = include( "modules/serverdefs" )
local mainframe_common = include("sim/abilities/mainframe_common")
local speechdefs = include( "sim/speechdefs" )
local modifiers = include( "sim/modifiers" )

-------------------------------------------------------------------------------
-- These are NPC abilities.

local createDaemon = mainframe_common.createDaemon
local createReverseDaemon = mainframe_common.createReverseDaemon
local createCountermeasureInterest = mainframe_common.createCountermeasureInterest

local npc_abilities = {
	_testOwlMinus3 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Subtracts the value. Note that it won't reduce it below 1, while also not raising the value if it starts out below 1. The 8 doesn't do anything
				unit:getModifiers():add( "LOSrange", "_testOwlMinus3", modifiers.ADD, -3, 1, 8 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testOwlMinus3" )
			end
		end,		
	},
	_testOwlPlus2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Same as above
				unit:getModifiers():add( "LOSrange", "_testOwlPlus2", modifiers.ADD, 2, 1, 8 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testOwlPlus2" )
			end
		end,		
	},
	_testOwlClamp = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Clamps the value. Goes after addition and multiplication
				unit:getModifiers():add( "LOSrange", "_testOwlClamp", modifiers.CLA, 2, 2, 5 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testOwlClamp" )
			end
		end,		
	},
	_testPrioAdd = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Low priority add, goes after all other operations (unless they too are low priority)
				unit:getModifiers():add( "LOSrange", "_testPrioAdd", modifiers.ADD, 2, nil, nil, -1 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testPrioAdd" )
			end
		end,		
	},
	_testOpAdd = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Custom operation order add. Goes after other addition and multiplication, but before set.
				--Goes at the same time as clamp, although there's nothing stopping you from using a decimal value to get around that
				unit:getModifiers():add( "LOSrange", "_testOpAdd", modifiers.ADD, 2, nil, nil, nil, 2 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testOpAdd" )
			end
		end,		
	},
	_testMult = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Multiplication, by default, always goes after addition
				unit:getModifiers():add( "LOSrange", "_testMult", modifiers.MULT, 0.5 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testMult" )
			end
		end,		
	},
	_testCustom = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--Just a random test for arbitrary modifiers
				unit:getModifiers():add( "LOSrange", "_testCustom", modifiers.FN, function(val, modifier, isDebug) if type(val) == "number" then return (val % 3) + 1 end end )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testCustom" )
			end
		end,		
	},
	_testChanging = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				--
				unit:getModifiers():add( "LOSrange", "_testChanging", modifiers.ADD, 1 )
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,
		
    	onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN then
				for _, unit in ipairs(sim:getNPC():getUnits() ) do
					--This method works less well when there are multiple modifiers of the same name (they will all target the same modifier, unless you mess with priority), see below for fix
					unit:getModifiers():change( "LOSrange", "_testChanging", nil, 4 - self.duration, nil, nil, nil, self.duration + 1 )
				end
     		end
     	end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "_testChanging" )
			end
		end,		
	},
	_testChanging2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",

		onSpawnAbility = function( self, sim, player )
			self.duration = 3
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
			
			self.affectedUnits = {}

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				local modifier, idx = unit:getModifiers():add( "LOSrange", "_testChanging2", modifiers.ADD, 1 )
				
				self.affectedUnits[unit] = modifier
				
				sim:refreshUnitLOS( unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end

			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,
		
    	onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN then
				for unit, modifier in pairs(self.affectedUnits) do
					--trait (first) variable is optional, but can speed it up
					unit:getModifiers():change( nil, modifier, nil, 4 - self.duration, nil, nil, nil, self.duration + 1 )
					--Alternatively, if you are only changing the value, not the priority, you could do this:
					--modifier.val = newValue
					--unit:getModifiers():reeval("LOSrange")
				end
     		end
     	end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
			for unit, modifier in pairs(self.affectedUnits) do
				--More narrow remove pattern
				unit:getModifiers():remove( modifier, "LOSrange" )
			end
		end,
	},
}

return npc_abilities