local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local cdefs = include( "client_defs" )
local serverdefs = include( "modules/serverdefs" )
local mainframe_common = include("sim/abilities/mainframe_common")
local speechdefs = include( "sim/speechdefs" )
local modifiers = include( "sim/modifiers" )

-------------------------------------------------------------------------------
-- These are NPC abilities.

local createDaemon = mainframe_common.createDaemon
local createReverseDaemon = mainframe_common.createReverseDaemon
local createCountermeasureInterest = mainframe_common.createCountermeasureInterest

local npc_abilities =
{
	W93_alarm = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.ALARM ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_alarm.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		W93_AP_Bonus = 3,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc } )

			if sim:getParams().difficultyOptions.W93_easyAlarm then
				self.W93_AP_Bonus = 1
			end

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if sim:getNPC():getTraits().W93_alarm_alert then
					if unit:getTraits().mp and unit:getTraits().mpMax then
						if not unit:getModifiers():has( "mpMax", "alert" ) and unit:getTraits().mpMax then
							unit:getModifiers():add( "mpMax", "alert", modifiers.ADD, self.W93_AP_Bonus )
							unit:getTraits().mp = unit:getTraits().mp + self.W93_AP_Bonus
						end
					end
				end
				if sim:getNPC():getTraits().W93_alarm_owl then
					if unit:getTraits().hasSight and unit:getTraits().isGuard then
						if not sim:getParams().difficultyOptions.W93_easyAlarm then
							if not unit:getModifiers():has( "LOSarc", "owl" ) and unit:getTraits().LOSarc and unit:getTraits().LOSarc <= math.pi *1.5 then
								unit:getModifiers():add( "LOSarc", "owl", modifiers.ADD, math.pi/4 )
							end
							if not unit:getModifiers():has( "LOSperipheralArc", "owl" ) and unit:getTraits().LOSperipheralArc and unit:getTraits().LOSperipheralArc <= math.pi * 1.5 then
								unit:getModifiers():add( "LOSperipheralArc", "owl", modifiers.ADD, math.pi/4 )
							end
						end
						if not unit:getModifiers():has( "LOSrange", "owl" ) and unit:getTraits().LOSrange and unit:getTraits().LOSrange >= 2 then
							unit:getModifiers():add( "LOSrange", "owl", modifiers.MULT, 1.25 )
						end
						if not unit:getModifiers():has( "LOSperipheralRange", "owl" ) and unit:getTraits().LOSperipheralRange and unit:getTraits().LOSperipheralRange >= 2 then
							unit:getModifiers():add( "LOSperipheralRange", "owl", modifiers.MULT, 1.25 )
						end
						if not unit:getTraits().refreshingLOS then
							sim:refreshUnitLOS( unit )
						end
					end
				end
			end
			sim:processReactions()

			sim:addTrigger( "TRG_UNIT_SPAWNED", self )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			sim:addTrigger( simdefs.TRG_UNIT_USEDOOR, self )
			if sim:getNPC():getTraits().W93_alarm_chiton_watchdog then
				sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			end
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_UNIT_SPAWNED", self )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			sim:removeTrigger( simdefs.TRG_UNIT_USEDOOR, self )
			if sim:getNPC():getTraits().W93_alarm_chiton_watchdog then
				sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			end

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if unit:getModifiers():remove( "owl" ) and sim:getNPC():getTraits().W93_alarm_owl then
                        		sim:refreshUnitLOS( unit )
				end
				if sim:getNPC():getTraits().W93_alarm_alert then
					unit:getModifiers():remove( "alert" )
				end

				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end
			sim:processReactions()

			if sim:getNPC():getTraits().W93_alarm_chiton_watchdog and sim:getNPC():getTraits().boostArmor and sim:getNPC():getTraits().boostArmor > 0 then
				sim:getNPC():getTraits().boostArmor = sim:getNPC():getTraits().boostArmor - 1
				sim:getNPC():getTraits().boostKOresist = sim:getNPC():getTraits().boostKOresist - 1
			end
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard then
				if sim:getNPC():getTraits().W93_alarm_alert and evData:getTraits().mpMax then
					evData:getModifiers():add( "mpMax", "alert", modifiers.ADD, self.W93_AP_Bonus )
					evData:getTraits().mp = evData:getTraits().mp + self.W93_AP_Bonus
				end
				if sim:getNPC():getTraits().W93_alarm_owl and evData:getTraits().hasSight then
					if not sim:getParams().difficultyOptions.W93_easyAlarm then
						if evData:getTraits().LOSarc and evData:getTraits().LOSarc <= math.pi * 1.5 then
							evData:getModifiers():add( "LOSarc", "owl", modifiers.ADD, math.pi/4 )
						end
						if evData:getTraits().LOSperipheralArc and evData:getTraits().LOSperipheralArc <= math.pi * 1.5 then
							evData:getModifiers():add( "LOSperipheralArc", "owl", modifiers.ADD, math.pi/4 )
						end
					end
					if evData:getTraits().LOSrange and evData:getTraits().LOSrange >= 2 then
						evData:getModifiers():add( "LOSrange", "owl", modifiers.MULT, 1.25 )
					end
					if evData:getTraits().LOSperipheralRange and evData:getTraits().LOSperipheralRange >= 2 then
						evData:getModifiers():add( "LOSperipheralRange", "owl", modifiers.MULT, 1.25 )
					end
					if not evData:getTraits().refreshingLOS then
						sim:refreshUnitLOS( evData )
					end
				end
     			end
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				if evData == 6 and sim:getNPC():getTraits().W93_alarm_alert ~= nil then
					sim:getNPC():getTraits().W93_alarm_alert = true
					for _, unit in ipairs(sim:getNPC():getUnits() ) do
						if unit:getTraits().mp and unit:getTraits().mpMax then
							if not unit:getModifiers():has( "mpMax", "alert" ) and unit:getTraits().mpMax then
								unit:getModifiers():add( "mpMax", "alert", modifiers.ADD, self.W93_AP_Bonus )
								unit:getTraits().mp = unit:getTraits().mp + self.W93_AP_Bonus
							end
						end
					end
				end
				if evData == 6 and sim:getNPC():getTraits().W93_alarm_chiton_watchdog ~= nil then
					sim:getNPC():getTraits().W93_alarm_chiton_watchdog = true
					if not sim:getNPC():getTraits().boostArmor then
						sim:getNPC():getTraits().boostArmor = 1
					else
						sim:getNPC():getTraits().boostArmor = sim:getNPC():getTraits().boostArmor + 1
					end
					if not sim:getNPC():getTraits().boostKOresist then
						sim:getNPC():getTraits().boostKOresist = 1
					else
						sim:getNPC():getTraits().boostKOresist = sim:getNPC():getTraits().boostKOresist + 1
					end
				end
				if evData == 8 then
					self.name = STRINGS.PROGEXTEND.DAEMONS.ALARM.NAME2
					self.icon = "gui/icons/daemon_icons/icon-daemon_lockdownV2.png"
				end

				if evData > 8 and self.W93_AP_Bonus == 3 then
					sim:getNPC():huntAgents( 4 )
				end
			end

			if evType == simdefs.TRG_UNIT_USEDOOR and sim:getNPC():getTraits().W93_alarm_lockdown and sim:getCurrentPlayer():isPC() and evData.unit then
    	        		local x0, y0 = evData.unit:getLocation()
	            		sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_DOOR, evData.unit)
	            		sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.DAEMONS.ALARM.LOCKDOWN_WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
			end

			if evType == simdefs.TRG_UNIT_KO and sim:getNPC():getTraits().W93_alarm_chiton_watchdog and evData.unit:getPlayerOwner() == sim:getNPC() and evData.unit:getTraits().enforcer and evData.ticks then
				sim:addEnforcersToSpawn( 1 )
			end
     		end,

		onTooltip = function( self, hud, sim, player )
			local tooltip = util.tooltip( hud._screen )
			local section = tooltip:addSection()

			section:addLine( self.name )
			if sim:getNPC():getTraits().W93_alarm_owl then
				if sim:getParams().difficultyOptions.W93_easyAlarm then
	   				section:addAbility( STRINGS.PROGEXTEND.DAEMONS.ALARM.OWL, STRINGS.PROGEXTEND.DAEMONS.ALARM.OWL_DESC2, "gui/icons/daemon_icons/icon-daemon_owl.png" )
				else
	   				section:addAbility( STRINGS.PROGEXTEND.DAEMONS.ALARM.OWL, STRINGS.PROGEXTEND.DAEMONS.ALARM.OWL_DESC, "gui/icons/daemon_icons/icon-daemon_owl.png" )
				end
			end
			if sim:getNPC():getTraits().W93_alarm_alert then
	   			section:addAbility( STRINGS.PROGEXTEND.DAEMONS.ALARM.ALERT, string.format(STRINGS.PROGEXTEND.DAEMONS.ALARM.ALERT_DESC, self.W93_AP_Bonus), "gui/icons/daemon_icons/icon-daemon_alert.png" )
			end
			if sim:getNPC():getTraits().W93_alarm_chiton_watchdog and sim:getNPC():getTraits().W93_alarm_chiton_watchdog == 2 then
	   			section:addAbility( STRINGS.DLC1.DAEMONS.CHITON.NAME, STRINGS.DLC1.DAEMONS.CHITON.DESC, "gui/icons/daemon_icons/Daemons_chiten.png" )
	   			section:addAbility( STRINGS.DLC1.DAEMONS.SHOCK.NAME, STRINGS.DLC1.DAEMONS.SHOCK.DESC, "gui/icons/daemon_icons/Daemons_fortitude.png" )
			end
			if sim:getNPC():getTraits().W93_alarm_lockdown then
				if sim:getParams().difficultyOptions.W93_easyAlarm then
	   				section:addAbility( STRINGS.PROGEXTEND.DAEMONS.ALARM.LOCKDOWN, STRINGS.PROGEXTEND.DAEMONS.ALARM.LOCKDOWN_DESC, "gui/icons/daemon_icons/icon-daemon_lockdown.png" )
				else
	   				section:addAbility( STRINGS.PROGEXTEND.DAEMONS.ALARM.LOCKDOWN, STRINGS.PROGEXTEND.DAEMONS.ALARM.LOCKDOWN_DESC2, "gui/icons/daemon_icons/icon-daemon_lockdown.png" )
				end
			end

			return tooltip
		end,
	},


	W93_alert = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.ALERT ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_alert.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,
		alerted = 0,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,5))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if unit:getTraits().isGuard and unit:getTraits().mpMax then
					unit:getModifiers():add( "mpMax", "alert_small", modifiers.ADD, 4 )
					unit:getTraits().mp = unit:getTraits().mp + 4
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
				end
			end

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if not unit:getTraits().pacifist and not unit:isAlerted() and self.alerted == 0 then
					unit:setAlerted(true)
    	        			local x0, y0 = unit:getLocation()
	            			sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_HUNTING, unit)
					sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = unit:getID() } )
					unit:getSim():emitSpeech( unit, speechdefs.HUNT_NOISE)
					self.alerted = 1
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
				end
			end		

			sim:addTrigger( "TRG_UNIT_SPAWNED", self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_UNIT_SPAWNED", self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "alert_small" )
			end
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard and evData:getTraits().mpMax then
				evData:getModifiers():add( "mpMax", "alert_small", modifiers.ADD, 4 )
				evData:getTraits().mp = evData:getTraits().mp + 4
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_attention = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.ATTENTION ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_attention.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,
		used = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(2,3))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_NEWINTEREST and evData.interest and
			(evData.interest.reason == simdefs.REASON_NOISE or evData.interest.reason == simdefs.REASON_DOOR or evData.interest.reason == simdefs.REASON_NOTICED or evData.interest.reason == simdefs.REASON_SENSEDTARGET) and
			(evData.interest.sense == simdefs.SENSE_HEARING or evData.interest.sense == simdefs.SENSE_SIGHT or evData.interest.sense == simdefs.SENSE_PERIPHERAL) then
				sim:trackerAdvance( 1 )
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.DAEMONS.ATTENTION.WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_blindfold = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.BLINDFOLD ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_blindfold.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,5))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )	

		        if not sim:getPC():getTraits().blindfold_penalty then
				sim:getPC():getTraits().blindfold_penalty = 0
			end
			sim:getPC():getTraits().blindfold_penalty = sim:getPC():getTraits().blindfold_penalty + 1

			sim:addTrigger( simdefs.TRG_END_TURN, self )	
		end,

		onDespawnAbility = function( self, sim )
			sim:getPC():getTraits().blindfold_penalty = sim:getPC():getTraits().blindfold_penalty - 1

			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end	
	},

	W93_branch = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.BRANCH ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_branch.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,
		firewalls_broken = 0,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,5))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration)} )
			self.desc = util.sformat(STRINGS.PROGEXTEND.DAEMONS.BRANCH.DESC2, 5 )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ICE_BROKEN and sim:getCurrentPlayer():isPC() and evData.delta then
				self.firewalls_broken = self.firewalls_broken + math.min(evData.delta, 0)
				if self.firewalls_broken <= -5 or self.firewalls_broken >= 5 then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.DAEMONS.BRANCH.WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*0.65)
					self.firewalls_broken = 0
					local daemon = nil
					if sim:isVersion("0.17.5") then
						daemon = sim:getIcePrograms():getChoice( sim:nextRand( 1, sim:getIcePrograms():getTotalWeight() ))
					else
						daemon = programList[sim:nextRand(1, #serverdefs.PROGRAM_LIST)]			
					end	

					sim:getNPC():addMainframeAbility( sim, daemon )
				end
				self.desc = util.sformat(STRINGS.PROGEXTEND.DAEMONS.BRANCH.DESC2, 5 - math.abs(self.firewalls_broken) )
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,
	},

	W93_branchV2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.BRANCHV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_branch.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,
		firewalls_broken = 0,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, "-")
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc } )
			self.desc = util.sformat(STRINGS.PROGEXTEND.DAEMONS.BRANCHV2.DESC2, 10 )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ICE_BROKEN and sim:getCurrentPlayer():isPC() and evData.delta then
				self.firewalls_broken = self.firewalls_broken + math.min(evData.delta, 0)
				if self.firewalls_broken <= -10 or self.firewalls_broken >= 10 then
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.DAEMONS.BRANCH.WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*0.65)
					self.firewalls_broken = 0
					local daemon = nil
					if sim:isVersion("0.17.5") then
						daemon = sim:getIcePrograms():getChoice( sim:nextRand( 1, sim:getIcePrograms():getTotalWeight() ))
					else
						daemon = programList[sim:nextRand(1, #serverdefs.PROGRAM_LIST)]			
					end	

					sim:getNPC():addMainframeAbility( sim, daemon )
				end
				self.desc = util.sformat(STRINGS.PROGEXTEND.DAEMONS.BRANCHV2.DESC2, 10 - math.abs(self.firewalls_broken) )
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,
	},

	W93_chain = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.CHAIN ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_chain.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(4,6))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if (evType == simdefs.TRG_UNIT_ALERTED or evType == simdefs.TRG_UNIT_KILLED) and evData["unit"]:getTraits().isGuard then
				local daemon = nil
				if sim:isVersion("0.17.5") then
					daemon = sim:getIcePrograms():getChoice( sim:nextRand( 1, sim:getIcePrograms():getTotalWeight() ))
				else
					daemon = programList[sim:nextRand(1, #serverdefs.PROGRAM_LIST)]			
				end	
				sim:getNPC():addMainframeAbility( sim, daemon )
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_chainV2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.CHAINV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_chain.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )

			sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if (evType == simdefs.TRG_UNIT_ALERTED or evType == simdefs.TRG_UNIT_KILLED) and evData["unit"]:getTraits().isGuard then
				local daemon = nil
				if sim:isVersion("0.17.5") then
					daemon = sim:getIcePrograms():getChoice( sim:nextRand( 1, sim:getIcePrograms():getTotalWeight() ))
				else
					daemon = programList[sim:nextRand(1, #serverdefs.PROGRAM_LIST)]			
				end	
				sim:getNPC():addMainframeAbility( sim, daemon )
     			end
     		end,	
	},

	W93_clock = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.CLOCK ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_clock.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,
		
	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(4,8))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration) } )
			for i, ability in ipairs(sim:getPC():getAbilities())do
				if not ability.coolDownMod then
					ability.coolDownMod = 0
				end
				if not ability.clock_penalty then
					ability.clock_penalty = 0
				end
				ability.coolDownMod = ability.coolDownMod + 1
				ability.clock_penalty = ability.clock_penalty + 1
				break
			end
			sim:addTrigger( simdefs.TRG_END_TURN, self )	
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			for i, ability in ipairs(sim:getPC():getAbilities())do
				if ability.coolDownMod and ability.clock_penalty then
					ability.coolDownMod = ability.coolDownMod - 1
					ability.clock_penalty = ability.clock_penalty - 1
					if ability.clock_penalty <= 0 then
						ability.clock_penalty = nil
					end
				end
			end
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end	
	},

	W93_clockV2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.CLOCKV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_clock.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc } )
			for i, ability in ipairs(sim:getPC():getAbilities())do
				if not ability.coolDownMod then
					ability.coolDownMod = 0
				end
				if not ability.clock_penalty then
					ability.clock_penalty = 0
				end
				ability.coolDownMod = ability.coolDownMod + 1
				ability.clock_penalty = ability.clock_penalty + 1
				break
			end	
		end,

		onDespawnAbility = function( self, sim, unit )
			for i, ability in ipairs(sim:getPC():getAbilities())do
				if ability.coolDownMod and ability.clock_penalty then
					ability.coolDownMod = ability.coolDownMod - 1
					ability.clock_penalty = ability.clock_penalty - 1
					if ability.clock_penalty <= 0 then
						ability.clock_penalty = nil
					end
				end
			end
		end,	
	},

	W93_disarm = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.DISARM ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_disarm.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,5))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )


			for i, unit in ipairs(sim:getPC():getUnits())do
				if unit:hasTrait("ap") then
					if unit:getTraits().ap > 0 then
						unit:getTraits().ap = unit:getTraits().ap - 1
					end
				end
			end
			sim:addTrigger( simdefs.TRG_START_TURN, self )		
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer():isPC() then
				for i, unit in ipairs(sim:getPC():getUnits())do
					if unit:hasTrait("ap") then
						if unit:getTraits().ap > 0 then
							unit:getTraits().ap = unit:getTraits().ap - 1
						end
					end
				end
			end 
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_disarmV2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.DISARMV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_disarm.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )

			for i, unit in ipairs(sim:getPC():getUnits())do
				if unit:hasTrait("ap") then
					if unit:getTraits().ap > 0 then
						unit:getTraits().ap = unit:getTraits().ap - 1
					end
				end
			end
			sim:addTrigger( simdefs.TRG_START_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer():isPC() then
				for i, unit in ipairs(sim:getPC():getUnits())do
					if unit:hasTrait("ap") then
						if unit:getTraits().ap > 0 then
							unit:getTraits().ap = unit:getTraits().ap - 1
						end
					end
				end
			end 
     		end,
	},

	W93_format = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.FORMAT ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_format.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc } )
		
			sim:forEachCell(function( c )
				if not sim:canPlayerSee( sim:getPC(), c.x, c.y ) then
					sim:getPC():markSeen( sim, c.x, c.y )
				end
			end )
			sim:dispatchEvent( "W93_REFRESH_CELLS" )
			sim:dispatchEvent( simdefs.EV_SCRIPT_EXIT_MAINFRAME )
			sim:getNPC():removeAbility(sim, self )
        	end,

		onDespawnAbility = function( self, sim )
		end,		
	},

	W93_gatekeeper = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.GATEKEEPER ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_gatekeeper.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(7,10))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
		
			if not sim:getPC():getTraits().gatekeeper then
				sim:getPC():getTraits().gatekeeper = 1
				local shouldUpdateLock = false
				sim:forEachCell(function( c )
					for i, exit in pairs( c.exits ) do
						if exit.door and not exit.closed and (exit.keybits == simdefs.DOOR_KEYS.ELEVATOR or exit.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE) then
							local reverseExit = exit.cell.exits[ simquery.getReverseDirection( i ) ]
							exit.keybits = simdefs.DOOR_KEYS.ELEVATOR_INUSE						
							reverseExit.keybits = simdefs.DOOR_KEYS.ELEVATOR_INUSE
							sim:modifyExit( c, i, simdefs.EXITOP_CLOSE )
							sim:modifyExit( c, i, simdefs.EXITOP_LOCK )
							sim:dispatchEvent( simdefs.EV_EXIT_MODIFIED, {cell=c, dir=i} )
							shouldUpdateLock = true
						elseif exit.door and exit.closed and (exit.keybits == simdefs.DOOR_KEYS.ELEVATOR or exit.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE) then
							shouldUpdateLock = true
						elseif exit.door and not exit.closed and exit.keybits == simdefs.DOOR_KEYS.FINAL_LEVEL then 
							sim:modifyExit( c, i, simdefs.EXITOP_CLOSE )
							sim:modifyExit( c, i, simdefs.EXITOP_LOCK )
							sim:dispatchEvent( simdefs.EV_EXIT_MODIFIED, {cell=c, dir=i} )
						end
					end
				end )
				if shouldUpdateLock and (sim._elevator_inuse or 0) < self.duration then
					sim._elevator_inuse = self.duration
				end
			else
				sim:getPC():getTraits().gatekeeper = sim:getPC():getTraits().gatekeeper + 1
				if sim._elevator_inuse and sim._elevator_inuse < self.duration then
					sim._elevator_inuse = self.duration
				end
			end
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:getPC():getTraits().gatekeeper = sim:getPC():getTraits().gatekeeper - 1
			if sim:getPC():getTraits().gatekeeper <= 0 then
				sim:getPC():getTraits().gatekeeper = nil
			end
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_inspect = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.INSPECT ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_inspect.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = self.activedesc } )

			local idle = sim:getNPC():getIdleSituation()
			local guards = sim:getNPC():getUnits()

			for i, guard in ipairs(guards) do
				if guard:getBrain() and guard:getBrain():getSituation() and guard:getBrain():getSituation().ClassType == simdefs.SITUATION_IDLE then
					idle:generatePatrolPath( guard )
					if guard:getTraits().patrolPath and #guard:getTraits().patrolPath > 1 then
						local firstPoint = guard:getTraits().patrolPath[1]
						guard:getBrain():getSenses():addInterest(firstPoint.x, firstPoint.y, simdefs.SENSE_RADIO, simdefs.REASON_PATROLCHANGED, guard)
					end
				end
			end
			sim:processReactions()

			sim:getNPC():removeAbility(sim, self )
        	end,

		onDespawnAbility = function( self, sim )
		end,
	},

	W93_lockdown = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.LOCKDOWN ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_lockdown.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, 3)
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )

			sim:addTrigger( simdefs.TRG_OPEN_DOOR, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_OPEN_DOOR, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_OPEN_DOOR and sim:getCurrentPlayer():isPC() and evData.unit then
    	        		local x0, y0 = evData.unit:getLocation()
	            		sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_DOOR, evData.unit)
	            		sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.PROGEXTEND.DAEMONS.LOCKDOWN.WARNING, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
				sim:trackerAdvance( 1 )
			end 
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_mute = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.MUTE ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_mute.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(5,7))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
			sim:addTrigger( "TRG_UNIT_SPAWNED", self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if unit:getTraits().isGuard and unit:getTraits().dashSoundRange then
					unit:getModifiers():add( "dashSoundRange", "mute", modifiers.MULT, 0.25 )
				end
			end
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( "TRG_UNIT_SPAWNED", self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				unit:getModifiers():remove( "mute" )
			end
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard and evData:getTraits().dashSoundRange then
				evData:getModifiers():add( "dashSoundRange", "mute", modifiers.MULT, 0.25 )
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},

	W93_owl = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.OWL ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_owl.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,5))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration) } )

			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if unit:getTraits().hasSight and unit:getTraits().isGuard then
					if unit:getTraits().LOSarc and unit:getTraits().LOSarc < math.pi then
						unit:getModifiers():add( "LOSarc", "owl_small", modifiers.ADD, math.pi/4 )
					end
					if unit:getTraits().LOSperipheralArc and unit:getTraits().LOSperipheralArc < math.pi * 1.5 then
						unit:getModifiers():add( "LOSperipheralArc", "owl_small", modifiers.ADD, math.pi/4 )
					end
					if unit:getTraits().LOSrange and unit:getTraits().LOSrange > 2 then
						unit:getModifiers():add( "LOSrange", "owl_small", modifiers.ADD, 4 )
					end
					if unit:getTraits().LOSperipheralRange and unit:getTraits().LOSperipheralRange > 2 then
						unit:getModifiers():add( "LOSperipheralRange", "owl_small", modifiers.ADD, 4 )
					end
					if not unit:getTraits().refreshingLOS then
						sim:refreshUnitLOS( unit )
					end
				end
			end
			sim:processReactions()

			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( "TRG_UNIT_SPAWNED", self )
        	end,

		onDespawnAbility = function( self, sim, unit )
			for _, unit in ipairs(sim:getNPC():getUnits() ) do
				if unit:getModifiers():remove( "owl_small" ) then
                        		sim:refreshUnitLOS( unit )
				end
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end
			sim:processReactions()		
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( "TRG_UNIT_SPAWNED", self )	
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )

			if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard then
				if evData:getTraits().hasSight then
					if evData:getTraits().LOSarc and evData:getTraits().LOSarc < math.pi then
						evData:getModifiers():add( "LOSarc", "owl_small", modifiers.ADD, math.pi/4 )
					end
					if evData:getTraits().LOSperipheralArc and evData:getTraits().LOSperipheralArc < math.pi * 1.5 then
						evData:getModifiers():add( "LOSperipheralArc", "owl_small", modifiers.ADD, math.pi/4 )
					end
					if evData:getTraits().LOSrange and evData:getTraits().LOSrange > 2 then
						evData:getModifiers():add( "LOSrange", "owl_small", modifiers.ADD, 4 )
					end
					if evData:getTraits().LOSperipheralRange and evData:getTraits().LOSperipheralRange > 2 then
						evData:getModifiers():add( "LOSperipheralRange", "owl_small", modifiers.ADD, 4 )
					end
					if not evData:getTraits().refreshingLOS then
						sim:refreshUnitLOS( evData )
					end
				end
			end
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,	
	},

	W93_poison = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.POISON ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_poison.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,
		KOtime = 3,
		agent = nil,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, 4)

			sim:addTrigger( simdefs.TRG_END_TURN, self )

			local agents = sim:getPC():getAgents()
			for i, unit in ipairs(agents) do
	    			self.agent = agents[sim:nextRand( 1, #agents )]
				if not self.agent:getTraits().poisoned and not self.agent:isDown() and self.agent:getTraits().canKO == true then
					self.agent:getTraits().poisoned = self.KOtime
					break
				end
			end
			if self.agent and not self.agent:getTraits().poisoned then
				sim:getNPC():removeAbility(sim, self )
			end

			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration, string.upper(self.agent:getName())) } )	
		end,

		onDespawnAbility = function( self, sim, unit )
			if self.agent and self.agent:isValid() and self.agent:getLocation() and self.agent:getTraits().poisoned then
				self.agent:setKO( sim, self.agent:getTraits().poisoned, "emp" )
				self.agent:getTraits().poisoned = nil
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_lightning_strike" )
			end			
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end	
	},

	W93_poisonV2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.POISONV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_poison.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,
		KOtime = 3,
		agent = nil,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc) } )	
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )

	    		local agents = sim:getPC():getAgents()
			for i, unit in ipairs(agents) do
	    			self.agent = agents[ sim:nextRand( 1, #agents ) ]
				if not self.agent:getTraits().poisoned and not self.agent:isDown() and self.agent:getTraits().canKO == true then
					self.agent:getTraits().poisoned = self.KOtime
					break
				end
			end
		end,

		onDespawnAbility = function( self, sim, unit )			
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )	
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				if self.agent and self.agent:isValid() and self.agent:getLocation() and self.agent:getTraits().poisoned then
					self.agent:setKO( sim, self.agent:getTraits().poisoned, "emp" )
					self.agent:getTraits().poisoned = nil
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_lightning_strike" )
				end

				local agents = sim:getPC():getAgents()
				for i, unit in ipairs(agents) do
	    				self.agent = agents[ sim:nextRand( 1, #agents ) ]
					if not self.agent:getTraits().poisoned and not self.agent:isDown() and self.agent:getTraits().canKO == true then
						self.agent:getTraits().poisoned = self.KOtime
						break
					end
				end
			end
		end,
	
	},

	W93_pulse = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.PULSE ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_pulse.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(5,7))
			local pcplayer = sim:getPC()
			self.items = {}

			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration) } )	

			local pcplayer = sim:getPC()
			for i, unit in pairs( pcplayer:getUnits() ) do 
				for i, item in pairs( unit:getChildren() ) do 
					table.insert( self.items, item )
				end 
			end 

			for _, item in ipairs(self.items) do
				if item:getTraits().cooldownMax then 
					item:getModifiers():add( "cooldownMax", "pulse", modifiers.ADD, 3 )
				end 
			end

			sim:addTrigger( simdefs.TRG_END_TURN, self )	
		end,

		onDespawnAbility = function( self, sim, unit )
			for _, item in ipairs(self.items) do
				item:getModifiers():remove( "pulse" )
			end
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end	
	},

	W93_shackles = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.SHACKLES ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_schackles.png",

		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,4))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:getPC():getTraits().shackleDaemon = true
		end,

		onDespawnAbility = function( self, sim )		
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:getPC():getTraits().shackleDaemon = nil	
		end,
	
		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,	
	},

	W93_short = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.SHORT ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_short.png",

		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,
		short_amount = 10,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(4,6))
			self.desc = util.sformat(STRINGS.PROGEXTEND.DAEMONS.SHORT.DESC2,self.short_amount)
			if not sim:getPC():getTraits().PWRmaxBouns then
				sim:getPC():getTraits().PWRmaxBouns = 0
			end
			sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns - self.short_amount
			sim:getPC():addCPUs( 0, sim )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration, self.short_amount ) } )
			sim:addTrigger( simdefs.TRG_END_TURN, self )	
		end,

		onDespawnAbility = function( self, sim )		
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
			sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns + self.short_amount
		end,
	
		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,	
	},

	W93_shortV2 = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.SHORTV2 ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_short.png",

		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,
		short_amount = 10,

	     	ENDLESS_DAEMONS = true,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			if not sim:getPC():getTraits().PWRmaxBouns then
				sim:getPC():getTraits().PWRmaxBouns = 0
			end
			sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns - self.short_amount
			sim:getPC():addCPUs( 0, sim )
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc } )
			sim:addTrigger( simdefs.TRG_START_TURN, self )	
		end,

		onDespawnAbility = function( self, sim )		
			sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns + self.short_amount
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,
	
    		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer():isPC() then
				sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns + self.short_amount
				self.short_amount = 10 + math.min(sim:getTrackerStage(sim:getTracker()),simdefs.TRACKER_MAXSTAGE)
				sim:getPC():getTraits().PWRmaxBouns = sim:getPC():getTraits().PWRmaxBouns - self.short_amount
				sim:getPC():addCPUs( 0, sim )
     			end
     		end,
	},

	W93_shutdown = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.SHUTDOWN ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_shutdown.png",
		standardDaemon = true,
		reverseDaemon = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, "-")
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = self.activedesc } )
			local targets = {}

			for _, consoleUnit in pairs(sim:getAllUnits()) do
				if consoleUnit:getTraits().mainframe_console and consoleUnit:getTraits().hijacked == false and consoleUnit:getTraits().mainframe_status ~= "off" and (not consoleUnit:hasTag("W93_INCOG_LOCK")) then
					table.insert(targets, consoleUnit)
				end
			end
			if #targets < 1 then
				for _, consoleUnit in pairs(sim:getAllUnits()) do
					if consoleUnit:getTraits().mainframe_console and consoleUnit:getTraits().hijacked == true and consoleUnit:getTraits().mainframe_status ~= "off" then
						table.insert(targets, consoleUnit)
					end
				end	
			end
			if #targets > 0 then
				local target = sim:nextRand(1, #targets)
				targets[target]:getTraits().mainframe_status = "off"
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targets[target] } )
			end
			sim:getNPC():removeAbility( sim, self )
        	end,

		onDespawnAbility = function( self, sim )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility( sim, self )
		end,
	},

	W93_sleep = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.SLEEP ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_sleep.png",
		standardDaemon = true,
		reverseDaemon = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = true,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, 4)
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration) } )

			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_END_TURN and evData:isNPC() then
				self.duration = self.duration - 1
				if self.duration <= 0 then
					if sim:getPC():getTraits().skipTurn then
						sim:getPC():getTraits().skipTurn = sim:getPC():getTraits().skipTurn + 1
					else
						sim:getPC():getTraits().skipTurn = 1
					end

					sim:getNPC():removeAbility( sim, self )
     				end
			end
     		end,
	},

	W93_suspicion = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.SUSPICION ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_suspicion.png",
		standardDaemon = true,
		reverseDaemon = false,
		premanent = false,

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = true,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, sim:nextRand(3,5))
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
		
			sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			mainframe_common.DEFAULT_ABILITY_DAEMON.onTrigger( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_NEWINTEREST and evData.unit and (not evData.unit:isAlerted()) and (not evData.unit:getTraits().pacifist) then
				if evData.interest and
				(evData.interest.reason == simdefs.REASON_NOISE or evData.interest.reason == simdefs.REASON_DOOR or evData.interest.reason == simdefs.REASON_NOTICED or evData.interest.reason == simdefs.REASON_SENSEDTARGET) and
				(evData.interest.sense == simdefs.SENSE_HEARING or evData.interest.sense == simdefs.SENSE_SIGHT or evData.interest.sense == simdefs.SENSE_PERIPHERAL) then
	            			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.DAEMONS.SUSPICION.WARNING, evData.unit:getName()), color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*0.65)
					local x0, y0 = evData.interest.x, evData.interest.y
					evData.unit:setAlerted(true)
					evData.unit:getBrain():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_HUNTING, evData.interest.sourceUnit)
					sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = evData.unit:getID() } )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = evData.unit } )
					sim:emitSpeech( evData.unit, speechdefs.HUNT_NOISE )
				end
     			end
     		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end,		
	},
}

return npc_abilities