local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local mainframe_common = include("sim/abilities/mainframe_common")

local createDaemon = mainframe_common.createDaemon
local createCountermeasureInterest = mainframe_common.createCountermeasureInterest
local DEFAULT_ABILITY = mainframe_common.DEFAULT_ABILITY_DAEMON

return {
	panic = util.extend( createDaemon( STRINGS.DAEMONS.PANIC ) )
	{
		icon = "gui/icons/daemon_icons/Daemons0007.png",
		standardDaemon = false,

		onSpawnAbility = function( self, sim, player )
			self.duration = self.getDuration(self, sim, 3)
			sim:dispatchEvent( simdefs.EV_SHOW_DAEMON, { showMainframe=true, name = self.name, icon=self.icon, txt = util.sformat(self.activedesc, self.duration ) } )
			if sim:getParams().difficultyOptions.finalEnforcers then
				if sim:getParams().difficultyOptions.finalEnforcers > 0 then
					for i=1, sim:getParams().difficultyOptions.finalEnforcers do 
						sim:getNPC():doTrackerSpawn(sim, 1, simdefs.TRACKER_SPAWN_UNIT_ENFORCER_2 )
					end
				end
            elseif sim:getParams().campaignDifficulty == simdefs.HARD_DIFFICULTY then 
            	sim:getNPC():doTrackerSpawn(sim, 1, simdefs.TRACKER_SPAWN_UNIT_ENFORCER_2 )
            elseif sim:getParams().campaignDifficulty == simdefs.VERY_HARD_DIFFICULTY then 
            	for i=1, 2 do 
            		sim:getNPC():doTrackerSpawn(sim, 1, simdefs.TRACKER_SPAWN_UNIT_ENFORCER_2 )
            	end
            end 

            local npcPlayer = sim:getNPC()
			for _, unit in ipairs(npcPlayer:getUnits() ) do
				if unit:getBrain() and not unit:getTraits().enforcer then
					unit:setAlerted(true)
				end
			end		

			self._guardsTagged = {}

			local agent = nil
				for _, unit in pairs( sim:getPC():getAgents() ) do
					if unit:getTraits().monst3r then 
						agent = unit
					end 
				end 

			createCountermeasureInterest( self, sim, agent )

			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( simdefs.TRG_START_TURN, self )	
        end,

        onTrigger = function( self, sim, evType, evData, userUnit )
	    	if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer():isPC() then
    			local agent = nil
				for _, unit in pairs( sim:getPC():getAgents() ) do
					if unit:getTraits().monst3r then 
						agent = unit
					end 
				end 

				createCountermeasureInterest( self, sim, agent )
            else
                DEFAULT_ABILITY.onTrigger( self, sim, evType, evData, userUnit )
            end
        end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )	
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

		executeTimedAbility = function( self, sim )
			sim:getNPC():removeAbility(sim, self )
		end		
	},
}