local simunit = include("sim/simunit")
local simdefs = include("sim/simdefs")
local mission_util = include("sim/missions/mission_util")
local util = include("modules/util")
local serverdefs = include("modules/serverdefs")


local TRG_OBJ_NAME = simdefs.TRG_OBJ_COMPLETE
assert(TRG_OBJ_NAME)

--------------------------------------------------------------------------------------------

-- CEO OFFICE: When the brain scan is finished; done via a unit-tag append.

 -- nota bene: can also be done when the card is *taken* from the exec
 -- just swap addTag (addTag_old) for removeTag (removeTag_old)

 -- I opted for the brain scan because that way there's less ways to cheese
 -- when precisely you want the objective trigger to happen

local simunit_addTag_old = simunit.addTag
simunit.addTag = function ( self, tag, ... )
	if tag == "access_card_obj" then
		self:getSim():triggerEvent(TRG_OBJ_NAME, { missionType = "ceo_office"})
	end
	simunit_addTag_old( self, tag, ... )
end

-- CYBERLAB: When the first grafter/drill is activated.
local USED_AUGMENT_MACHINE =
{       
    trigger = simdefs.TRG_USE_AUGMENT_MACHINE, 	-- this trigger fires for both grafters and drills
    fn = function( sim, triggerData )			-- see sim/abilities/useAugmentMachine
        return true
    end,
}

local function signalCyberlabCompletion(script, sim, mission)
	script:waitFor( USED_AUGMENT_MACHINE )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "cyberlab" })
end

-- DET CENTRE: When the door console is used. There's already a hook for this!
local function signalDetentionCompletion( script, sim, mission)
	script:waitFor( mission_util.PC_USED_ABILITY( "open_detention_cells" ))
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "detention_centre" })
end

-- EXEC TERMINAL: When the intel is taken out of the terminal.
-- I *think* that technically creates a cheese-window, but no biggy.
local LOOTED_EXEC_TERMINAL =
{       
    trigger = simdefs.TRG_SAFE_LOOTED,
    fn = function( sim, triggerData )
		if triggerData.targetUnit and triggerData.targetUnit:getTraits().public_term then
			return true
		end
    end
}
	
local function signalExecTerminalCompletion(script, sim, mission)
	script:waitFor( LOOTED_EXEC_TERMINAL )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "executive_terminals" })
end

-- NANOFAB VESTIBULE: When the shop interface is closed.
local USE_NANOFAB = 
{
	trigger = simdefs.TRG_CLOSE_NANOFAB,
    fn = function( sim, triggerData )
        if triggerData.unit:getTraits().storeType == "large" then
            return triggerData.unit, triggerData.sourceUnit
       	end
    end
}

local function signalNanofabCompletion(script, sim, mission)
	script:waitFor( USE_NANOFAB )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "nanofab" })
end

-- SERVER FARM: When the player opens the Server Farm interface.
-- Sadly, I don't think *closing* the interface comes with a nifty TRG in this case
-- No one's about to dig into hud to fix that, it's no biggy
local USE_TERM =
{
    action = "abilityAction",
    --pre = true,
    fn = function( sim, ownerID, userID, abilityIdx, ... )
        local unit, ownerUnit = sim:getUnit( userID ), sim:getUnit( ownerID )
        
        if not unit or not unit:isPC() or not ownerUnit or not ownerUnit:getTraits().bigshopcat then
            return nil
        end

        if ownerUnit:getAbilities()[ abilityIdx ]:getID() == "showItemStore" then
            return ownerUnit
        end
    end,
}

local function signalServerFarmCompletion(script, sim, mission)
	script:waitFor( USE_TERM )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "server_farm" })
end

-- SECURITY DISPATCH: When the item is taken out of the locker.
local PC_LOOTED_SECURE_SAFE =
{
    trigger = simdefs.TRG_SAFE_LOOTED,
    fn = function( sim, triggerData )
        if triggerData.targetUnit:hasTag("topGear") then 	-- I was surprised to learn this is,
            return triggerData 								-- in fact,
        end													-- the tag.
    end,
}

local function signalDispatchCompletion(script, sim, mission)
	script:waitFor( PC_LOOTED_SECURE_SAFE )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "security" })
end

-- VAULT: When a lockbox is looted. We don't discriminate between inner and outer vaults.
local LOOT_VAULT =
{
    action = "abilityAction",
    --pre = true,
    fn = function( sim, ownerID, userID, abilityIdx, ... )
        local unit, ownerUnit = sim:getUnit( userID ), sim:getUnit( ownerID )
        
        if not unit or not unit:isPC() or not ownerUnit then
            return nil
        end
        if ownerUnit:getAbilities()[ abilityIdx ]:getID() == "stealCredits" then
            if ownerUnit:hasTag("vault") then
                return ownerUnit
            end
        end
    end,
}

local function signalVaultCompletion( script, sim, mission )
    local _, vault = script:waitFor( LOOT_VAULT )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "vault" })
end

--------------------------------------------------------------------------------------------

-- MID1: When both switches are thrown (and the BEEG elevator opens).
local MULTISWITCHES_USED = {
	trigger = simdefs.TRG_MAP_EVENT,
	fn = function( sim, eventData )
		if eventData.event == simdefs.MAP_EVENTS.SWITCH then 
			if eventData.data == "mid_1" then 
				return true
			end
		end 
	end,
}

local function signalMid1Completion( script, sim, mission )
	script:waitFor( MULTISWITCHES_USED )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "mid1" })
end

-- MID2: When the McGuffin (tm) is taken.
local PC_LOOTED_CASE =
{
    trigger = simdefs.TRG_SAFE_LOOTED,
    fn = function( sim, triggerData )
        if triggerData.targetUnit:getTraits().incognita_case then
            return triggerData 
        end
    end,
}

local function signalMid2Completion( script, sim, mission )
	script:waitFor( PC_LOOTED_CASE )
	sim:triggerEvent(TRG_OBJ_NAME, { missionType = "mid2" })
end

--------------------------------------------------------------------------------------------
-- now just match up which mission we're in with its corresponding trigger
local additional_esc_scripts = 
{
	{ "cyberlab", "OBJ_CHECK_CYBERLAB", signalCyberlabCompletion },
	{ "detention_centre", "OBJ_CHECK_DETENTION", signalDetentionCompletion },
	{ "executive_terminals", "OBJ_CHECK_EXECTERM", signalExecTerminalCompletion },
	{ "nanofab", "OBJ_CHECK_NANOFAB", signalNanofabCompletion },
	{ "server_farm", "OBJ_CHECK_SERVERFARM", signalServerFarmCompletion },
	{ "security", "OBJ_CHECK_SECURELOCKER", signalDispatchCompletion },
	{ "vault", "OBJ_CHECK_VAULT", signalVaultCompletion },
	{ "mid_1", "OBJ_CHECK_MID_1", signalMid1Completion },
	{ "mid_2", "OBJ_CHECK_MID_2", signalMid2Completion },
}

return additional_esc_scripts