local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local unitdefs = include("sim/unitdefs")
local unitghost = include( "sim/unitghost" )
local simfactory = include( "sim/simfactory" )
-------------------------------------------------------------------------
-- Trigger wireless onselect.

-- local function triggerWirelessScan( script, sim, abilityOwner )
    -- local level = include( "sim/level" )
    -- local userUnit = abilityOwner:getUnitOwner()
    -- if userUnit == nil then
        -- return -- Only triggers if carried by somebody.
    -- end

    -- local wirelessAbility = abilityOwner:hasAbility( "wireless_scan" )
    -- if not wirelessAbility:canUseAbility( sim, abilityOwner, userUnit ) then
        -- return
    -- end
    
    -- script:waitFor( { uiEvent = level.EV_UNIT_SELECTED,
        -- fn = function( sim, unitID )
            -- return userUnit:isValid() and unitID == userUnit:getID()
        -- end } )

    -- local cell = sim:getCell( userUnit:getLocation() )
    -- if cell then
        -- wirelessAbility:performScan( sim, abilityOwner, cell )
    -- end
-- end

-------------------------------------------------------------------------
-- Passive wireless scan; scans devices whenever the owning unit warps.

local function checkForGhosts( player, sim, cell )
	local ghost_units = player._ghost_units

	for i, ghost in ipairs(ghost_units) do
		if sim:getCell(ghost:getLocation()) == cell then
			return true		
		end
	end

	return false

end

--copies from simplayer.lua
local function getCellGhost( player, cellx, celly )
	assert( player._ghost_cells, util.stringize( player, 1 ))
	return player._ghost_cells[ simquery.toCellID( cellx, celly ) ]
end
local function addCellGhost( self, sim, cell )
	local ghost_units, ghost_cells = self._ghost_units, self._ghost_cells
	if ghost_cells[ cell.id ] ~= nil then
		return 		-- Never update ghost info if it's already ghosted.
	end

	self._nextGhostID = (self._nextGhostID or 0) + 1
	local cellghost = {}
	cellghost.id = cell.id
	cellghost.x = cell.x
	cellghost.y = cell.y
	cellghost.impass = cell.impass
	cellghost.exits = {}
	cellghost.units = {}	
	cellghost.exitID = cell.exitID
	-- keep track of relative ages of ghosts, given that nextID() is monotonically increasing.
	cellghost.ghostID = self._nextGhostID

	for dir,exit in pairs(cell.exits) do
		-- Only ghost doors.  Other stuff doesn't change, so reference the raw exit.  Reduces table junk.
		if exit.door then
			cellghost.exits[ dir ] = util.tmerge( {}, exit )
		else
			cellghost.exits[ dir ] = exit
		end
	end

	ghost_cells[ cell.id ] = cellghost
	return cellghost
end
local function addUnitGhost( ghost_units, ghost_cell, unit ) --some changes
	local ghost_unit = ghost_units[ unit:getID() ]
	if ghost_unit ~= nil then
		return false -- Never update ghost info if it's already ghosted.
	end
	if unit:getTraits().noghost then
		return false
	end
	
	ghost_unit = unitghost.createUnitGhost( unit )

	table.insert( ghost_cell.units, ghost_unit )
	ghost_units[ unit:getID() ] = ghost_unit

	return true
end
local function removeUnitGhost( player, cellghost, sim, unitID )
	local ghost_units = player._ghost_units
	local ghost = ghost_units[ unitID ]

	if ghost then
		if cellghost == nil then
			cellghost = getCellGhost( player, ghost:getLocation() )
		end

		assert( array.find( cellghost.units, ghost ))  -- If a unit ghost it exists, it must exist in a ghosted cell.

		array.removeElement( cellghost.units, ghost )
		local unit = player._sim:getUnit(unitID)
		ghost_units[ unitID ] = nil

	-- Handle the unghosting (whether or not a ghost actually existed).
	-- A unit is unghosted because (1) it was seen (2) its ghost was seen, in either case,
	-- its viz state must be updated.  Unghosting a unit is NOT equivalent to that unit becoming seen,
	-- but note that a unit becoming seen DOES always result in unghosting.

		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = ghost } )	
	end
end




local paranoia_scan =
{
	name = STRINGS.ABILITIES.WIRELESS_SCAN,
	getName = function( self, sim, unit )
		return self.name
	end,
	
	scaryGuards = {"npc_guard_enforcer_reinforcement_2", "npc_guard_enforcer"}, --to be populated more from guarddefs
	
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit
        sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:addTrigger( simdefs.TRG_START_TURN, self )
		log:write("paranoia scan onspawn")
		-- populate table with guards that are deemed scary enough.
			
		local params = sim:getParams()
		local guarddefs = include( "sim/unitdefs/guarddefs" )
		local worldgen = include("sim/worldgen")
		log:write(util.stringize(worldgen,2))
		
		local threat = nil
		
		log:write(util.stringize(params.world,2))
		if params.world == "plastech" then
			threat = worldgen.PLASTECH_THREAT
		elseif params.world == "ko" then
			threat = worldgen.KO_THREAT
		elseif params.world == "ftm" then
			threat = worldgen.FTM_THREAT
		elseif params.world == "sankaku" then
			threat = worldgen.SANKAKU_THREAT
		elseif params.world == "neptune" then
			threat = worldgen.NEPTUNE_THREAT
		end
		
		if threat then
			log:write(util.stringize(threat,2))
			log:write("LOG threat2")
			log:write(util.stringize(threat[1],2))
			for guard, value in pairs(threat) do
				log:write("LOG guard")
				log:write(util.stringize(guard,2))
				local guardData = unitdefs.lookupTemplate(guard)
				table.insert(self.scaryGuards, guardData.id)
			end
		end
		
		-- Check for omni guards in mission. If so, adding them to scaryGuards is fair game below
		local omni_protected = false
		local guards = sim:getNPC():getUnits()
		for i, guard in ipairs(guards) do
			if not guard:getTraits().isDrone and guard:getTraits().omni then
				omni_protected = true
				break
			end
		end
		
		-- check for scary guards with scary traits/abilities to populate the list
		log:write("LOG guarddefs")
		log:write(util.stringize(guarddefs, 2))
		for k,guard in ipairs(guarddefs) do
			if guard.traits and (
				(guard.traits.LOSarc > 0)
				or guard.traits.shieldArmor
				or guard.traits.enforcer
				or guard.traits.firewallSupport
				or (guard.abilities and #guard.abilities >3 )
				or (omni_protected == true and guard.traits.omni) )
				and not traits.pacifist
				and not trait.isDrone then
					table.insert(self.scaryGuards, guard.id)
			end
		end--this isn't working
		
		log:write("LOG scaryguards")
		log:write(util.stringize(self.scaryGuards,2))
						
	end,
        
	onDespawnAbility = function( self, sim, unit )
        sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
        self.abilityOwner = nil
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,

    onTrigger = function( self, sim, evType, evData )
        if (evType == simdefs.TRG_START_TURN or evType == simdefs.TRG_UNIT_WARP) and self.abilityOwner and evData.unit == self.abilityOwner:getUnitOwner() and evData.unit:getPlayerOwner() and evData.to_cell then
            if self:canUseAbility( sim, self.abilityOwner, evData.unit ) then
                self:performParanoiaScan( sim, self.abilityOwner, evData.to_cell )
				log:write("LOG on trigger paranoia scan")
            end
		elseif evType == simdefs.EV_UNIT_THROW then
				
				-- this is overkill but....
				local props = unitdefs.prop_templates
				if props.smoke_cloud_wilson then
				
					props.smoke_cloud_wilson.traits.gasColor = props.smoke_cloud_wilson.traits.gascolours[sim:nextRand( 1, #props.smoke_cloud_wilson.traits.gascolours)]
				end			
			
        end
    end,
	
	--basically a reskin of Internationale's wireless scan
    performParanoiaScan = function( self, sim, abilityOwner, cell )
        local player = abilityOwner:getPlayerOwner()
		local userUnit = self.abilityOwner:getUnitOwner()
		sim:forEachUnit(
		    function( enemyUnit )
				local wilson_ghosts = {}
			    local x1, y1 = enemyUnit:getLocation()
				
				    if x1 and y1 
					and not sim:canPlayerSee( player, x1, y1) 
					and (enemyUnit:getTraits().isGuard or enemyUnit:getTraits().isDrone) then
					local unit_cell = sim:getCell(x1, y1)
				    local distance = mathutil.dist2d( cell.x, cell.y, x1, y1 )
					local ghost_cell = getCellGhost( player, unit_cell.x, unit_cell.y )
					log:write("LOG ghost_cell")
					log:write(util.stringize(ghost_cell,3))
				    if distance <= abilityOwner:getTraits().paranoia_range then
					
					if ghost_cell and ghost_cell.units and (#ghost_cell.units > 0 ) then
					
						local x0, y0 = userUnit:getLocation()
						local newFacing = simquery.getDirectionFromDelta( x0-x1, y0-y1 )
						-- ghost_cell.units[1]:setFacing(newFacing)
						ghost_cell.units[1]._traits.isAiming = true
					end
					
					if ghost_cell and ghost_cell.units and not (#ghost_cell.units > 0) then
						-- if no ghosted unit already in cell, spawn in new ghost there
						local ghostEnemy = nil

							player:glimpseCell( sim, unit_cell ) 
							-- local cellghost = getCellGhost( player, unit_cell.x, unit_cell.y )
							
							if sim:nextRand()<0.1 then
							
							local enemy_template = enemyUnit:getUnitData().id

							local newTemplate = self.scaryGuards[sim:nextRand(1, #self.scaryGuards)]
							local newUnitData = unitdefs.lookupTemplate( newTemplate )
							
							-- log:write("LOg cellghost units")
							-- log:write(util.stringize(cellghost,2))
							-- log:write(#cellghost.units)
							
							if newUnitData and not enemyUnit:getTraits().isDrone then
							
								local enemyClone = simfactory.createUnit( newUnitData, sim )
								sim:spawnUnit( enemyClone )
								enemyClone:setPlayerOwner(sim:getNPC())
								enemyClone:setPather(sim:getNPC().pather)
								sim:warpUnit( enemyClone, unit_cell )
								enemyClone:getTraits().isAiming = true
								-- enemyClone:getTraits().tempKanim = "kanim_specops_ko"
								local x0, y0 = userUnit:getLocation()
								local newFacing = simquery.getDirectionFromDelta( x0-x1, y0-y1 )
								enemyClone:setFacing(newFacing)
								

								sim:dispatchEvent(simdefs.EV_UNIT_REFRESH, { unit = enemyClone } )
								
								player:glimpseUnit( sim, enemyClone:getID() )
								-- log:write(util.stringize(enemyClone,3))
								
								sim:warpUnit( enemyClone, nil )
								sim:despawnUnit( enemyClone )
								
								-- cellghost = getCellGhost( player, unit_cell.x, unit_cell.y )
							-- log:write("LOG cellghost")
							-- log:write(util.stringize(cellghost,3))
							-- -- log:write(util.stringize(cellghost._units,2))
														
								
							end
							
							
							else
								player:glimpseUnit( sim, enemyUnit:getID() )
							end							
					end	
				    end
			    end
		    end )
	end,
}

return paranoia_scan