local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local commondefs = include("sim/unitdefs/commondefs")
local tool_templates = include("sim/unitdefs/itemdefs")

local function onFooterTooltip( tooltip, unit, userUnit )
	if unit:getTraits().dlcFooter then
		tooltip:addFooter( unit:getTraits().dlcFooter[1],unit:getTraits().dlcFooter[2] )
	end
end


local prop_templates =
{
	ITB_smoke_cloud_camila =
    {
        type = "smoke_cloud",
        name = STRINGS.PROPS.SMOKE,
        rig = "smokerig",
		kanim = "kanim_smoke_plume",
        traits = { radius = 1, lifetime = 4, noghost = true, gasColor = {r=186/255,g=168/255,b=129/255} }
    },
	
	ITB_iceblock_bethany = 
	{
		type = "ITB_iceblock",
		name = STRINGS.BREACH_AGENTS.PROPS.ICEBLOCK_NAME,
		desc = STRINGS.BREACH_AGENTS.PROPS.ICEBLOCK_DESC,
		flavor = "",
		rig = "grenaderig",
		--icon = "itemrigs/FloorProp_Bandages.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_holo_grenade_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_holo_grenade.png",	
		kanim = "kanim_iceblock",
		-- sounds = {
		-- activate="SpySociety/Actions/holocover_activate",
		-- deactivate="SpySociety/Actions/holocover_deactivate",
		-- activeSpot="SpySociety/Actions/holocover_run_LP",
		-- },
		traits = { disposable = false, agent_filter=true, ITB_ice = true, dynamicImpass = 1, deploy_cover = true, cover = true, iceBlock = true, holoProjector = true},	-- holoprojector trait needed to play animation.
		value = 600,	
		locator=true,
		onTooltip = function( tooltip, unit, userUnit )
			onItemTooltip( tooltip, unit, userUnit )

			local damage = unit:getTraits().damage or 0

			if damage > 0 then
				tooltip:addAbility( STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE, util.sformat(STRINGS.ITEMS.TOOLTIPS.KO_DAMAGE_DESC, damage), "gui/icons/arrow_small.png" )
			end

			onFooterTooltip( tooltip, unit, userUnit )
		end		
	},		

	ITB_iceMine = util.extend(commondefs.item_template)
	{
		type = "ITB_iceMine",
		name = STRINGS.BREACH_AGENTS.PROPS.ICEMINE_NAME,
		desc = STRINGS.BREACH_AGENTS.PROPS.ICEMINE_DESC,
		-- flavor = STRINGS.DLC1.FLASH_PACK_FLAVOR,

		ITEM_LIST = false,
		
		icon = "itemrigs/ITB_iceMine.png",
		--profile_icon = "gui/items/icon-emp.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_flashpack_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_flashpack.png",	
		-- abilities = { "carryable","recharge","prime_emp", },
		-- requirements = { anarchy = 2 },
		-- uses_mainframe =
		-- {
			-- toggle =
			-- {
				-- name = STRINGS.DLC1.FLASH_PACK_USE,
				-- tooltip = STRINGS.DLC1.FLASH_PACK_USE_TIP,
				-- fn = "toggle" -- global script function
			-- }
		-- },		
		-- traits = { cooldown = 0, cooldownMax = 12, range = 3, attackNeedsLOS=true, canSleep = true, baseDamage = 2, flash_pack = true, trigger_mainframe=true},
		traits = { ko_duration = 3,
			possibleFrozenAnims = {
				{"hitfrt", 0},
				-- {"hitbck", 16},
				{"death", 11},
				-- {"body_fall", 0},
				-- {"body_fall", 3},
				{"throw", 2},
				-- {"lean_throw", 1}, -- guards don't have this
				{"door_kick_pre", 10}, --!!
				{"door_kick_pre", 2},
				{"door_kick_pre", 8},
				{"door_kick_pre", 16},
			},
			frozenRenderFilter = { 
				shader=KLEIAnim.SHADER_FOW,
				r=85/255,
				g=222/255,
				b=227/255,
				a=0.75,
				lum=1.0, },
		},
		value = 900,
		floorWeight = 2,	
	},
	
	ITB_ice_cloud =
    {
        type = "ITB_iceSmoke",
        name = STRINGS.PROPS.SMOKE,
        rig = "smokerig",
		kanim = "kanim_smoke_plume",
        traits = { radius = 0, lifetime = 1, noghost = true, gasColor = {r = 0.5, g = 1, b = 1, a = 0.05},},	
	},	


    ITB_smoke_cloud_small =
    {
        type = "smoke_cloud",
        name = STRINGS.PROPS.SMOKE,
        rig = "smokerig",
		kanim = "kanim_smoke_plume",
        traits = { radius = 1.5, lifetime = 2, noghost = true }
    },
	
	-- not done
    ITB_Prospero_Vines =
    {
        type = "ITB_vines",
        name = STRINGS.PROPS.SMOKE,
        rig = "smokerig",
		kanim = "kanim_smoke_plume",
        traits = { radius = 1.5, lifetime = 2, noghost = true, ITB_ProsperoVines = true, }
    },	
}

-- Reassign key name to value table.
for id, template in pairs(prop_templates) do
	template.id = id
end

return prop_templates
