local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local commondefs = include("sim/unitdefs/commondefs")
local tool_templates = include("sim/unitdefs/itemdefs")

-------------------------------------------------------------
--

local MAINFRAME_TRAITS = commondefs.MAINFRAME_TRAITS
local SAFE_TRAITS = commondefs.SAFE_TRAITS

local onMainframeTooltip = commondefs.onMainframeTooltip
local onSoundBugTooltip = commondefs.onSoundBugTooltip
local onBeamTooltip = commondefs.onBeamTooltip
local onConsoleTooltip = commondefs.onConsoleTooltip
local onStoreTooltip = commondefs.onStoreTooltip
local onDeviceTooltip = commondefs.onDeviceTooltip
local onSafeTooltip = commondefs.onSafeTooltip

local prop_templates =
{
	W93_infrared_camera_1x1 =
	{
		type = "simcamera",
		name = STRINGS.WORLDEXTEND.PROPS.INFRARED_CAMERA,
		kanim = "kanim_security_camera_infrared",
		onWorldTooltip = onMainframeTooltip,
		profile_anim = "portraits/camera_portrait",		
		facing = simdefs.DIR_N,	
		rig = "camerarig",
        	hit_fx = { z = 64 },
		traits = util.extend( MAINFRAME_TRAITS )
		{
			mainframe_camera = true,
			mainframe_no_daemon_spawn = true,
			hasSight = true,
			canBeShot = true,
			hit_metal = true,
			LOSrange = 8,
			breakIceOffset = 56,
			hasAttenuationHearing=true,
			seesHidden=true,
			recap_icon = "camera",
		},
		sounds = {appeared="SpySociety/HUD/gameplay/peek_negative",reboot_start="SpySociety/Actions/reboot_initiated_camera",reboot_end="SpySociety/Actions/reboot_complete_camera"},
	},

	W93_daemon_server =
	{
		type = "daemon_server",
		name = STRINGS.WORLDEXTEND.PROPS.DAEMON_SERVER,
		rig = "corerig",
		onWorldTooltip = onSafeTooltip,
		kanim = "kanim_plas_daemon_server",
		abilities = { "stealCredits" },
		traits = util.extend( MAINFRAME_TRAITS )	
		{
			moveToDevice=true, 
			cover = true,
			impass = {0,0},
            		mainframe_status = "inactive",
			mainframe_autodeactivate=true,
			daemon_server = true,
			sightable = true,
			spotSoundPowerDown = true,	
			startOn = true,
			mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = false,
			recap_icon = "W93_Daemon_Server",
		},
        	children = { "item_clip" },
		sounds = {appeared="SpySociety/HUD/gameplay/peek_negative", spot="SpySociety_DLC001/Objects/router",reboot_start="SpySociety/Actions/reboot_initiated_scanner",reboot_end="SpySociety/Actions/reboot_complete_scanner"},					
	},

	W93_drone_uplink =
	{
		type = "drone_uplink",
		name = STRINGS.WORLDEXTEND.PROPS.DRONE_UPLINK,
		rig = "corerig",
		onWorldTooltip = onSafeTooltip,
		kanim = "kanim_sk_uplink_node",
		abilities = { "stealCredits" },
		traits = util.extend( MAINFRAME_TRAITS )	
		{
			moveToDevice=true, 
			cover = true,
			impass = {0,0},
            		mainframe_status = "inactive",
			mainframe_autodeactivate=true,
			drone_uplink = 1,
			sightable = true,
			spotSoundPowerDown = true,	
			startOn = true,
			mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = false,
			recap_icon = "W93_Drone_Uplink_Node",
		},
        	children = { "item_clip" },
		sounds = { appeared="SpySociety/HUD/gameplay/peek_negative", spot="SpySociety/Objects/computer_types", reboot_start="SpySociety/Actions/reboot_initiated_generator",reboot_end="SpySociety/Actions/reboot_complete_generator" },
	},

	W93_frost_spire =
	{
		type = "frostspire",
		name = STRINGS.WORLDEXTEND.PROPS.FROST_SPIRE,
		rig = "corerig",
		onWorldTooltip = onSafeTooltip,
		kanim = "kanim_power_reversal",
		abilities = { "stealCredits" },
		traits = util.extend( MAINFRAME_TRAITS )	
		{
			moveToDevice=true, 
			cover = true,
			impass = {0,0},
            		mainframe_status = "inactive",
			mainframe_autodeactivate=true,
			freezeMP = 2,
			range = 9,
			sightable = true,
			spotSoundPowerDown = true,	
			startOn = true,
			mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = false,
			recap_icon = "W93_Drone_Uplink_Node",
		},
        	children = { "item_clip" },
		sounds = { appeared="SpySociety/HUD/gameplay/peek_negative", spot="SpySociety/Objects/computer_types", reboot_start="SpySociety/Actions/reboot_initiated_generator",reboot_end="SpySociety/Actions/reboot_complete_generator" },
	},

	W93_shockmine =
	{
		type = "shockmine",
		name =  STRINGS.WORLDEXTEND.PROPS.SHOCK_MINE,
		icon = "itemrigs/FloorProp_emp.png",
		kanim = "kanim_ko_shock_mine",
		onWorldTooltip = onMainframeTooltip,
		facing = simdefs.DIR_N,	
		rig = "unitrig",
		traits = util.extend( MAINFRAME_TRAITS )
		{
			breakIceOffset = 4,
			shockmine = true,
			hasSight = true,
			LOSrange = 1.35,
			LOSarc = 2*math.pi,
			detect_cloak = true,
			soundRange = 8,
			canBeShot = true,
			hit_metal = true,
			wounds = 0,
			woundsMax = 1,
		},				
		sounds =
		{
			appeared="SpySociety/HUD/gameplay/peek_negative",
			reboot_start="SpySociety/Actions/reboot_initiated_soundbug",
			reboot_end="SpySociety/Actions/reboot_complete_soundbug"
		},
	},

	W93_AI_Core =
	{
		type = "AI_Core",
		name = STRINGS.WORLDEXTEND.PROPS.AI_CORE,
		rig = "corerig",
		onWorldTooltip = onMainframeTooltip,
		kanim = "kanim_AI_powersource",
		traits = util.extend( MAINFRAME_TRAITS )	
		{ 
			cover = true,
			impass = {0,0},
            		mainframe_status = "inactive",
			mainframe_iceBonus = 1,
			mainframe_autodeactivate=true,
			sightable = true,
			spotSoundPowerDown = true,	
			startOn = true,
			recap_icon = "powersource",
			AI_PWR_Core = 1,
		},
		sounds = { appeared="SpySociety/HUD/gameplay/peek_negative", spot="SpySociety/HUD/gameplay/peek_negative", reboot_start="SpySociety/Actions/reboot_initiated_generator",reboot_end="SpySociety/Actions/reboot_complete_generator" },
	},
}

-- Reassign key name to value table.
for id, template in pairs(prop_templates) do
	template.id = id
end

return prop_templates