local function removeAgent( agency, agentDef )
	-- Remove all potentials with the same ID (or template)
	for i = #agency.unitDefsPotential, 1, -1 do
		local def2 = agency.unitDefsPotential[ i ]
		if (def2.id == agentDef.id) or (def2.template == agentDef.template) then
			table.remove( agency.unitDefsPotential, i )
		end
	end
end

local function load( modApi, options )
	local scriptPath = modApi:getScriptPath() 
	local serverdefs = include ( scriptPath .. "/serverdefs" )
	
	-- DRM'ed items
	local itemdefs = include( scriptPath .. "/itemdefs" )
	for name, itemDef in pairs(itemdefs) do
		modApi:addItemDef( name, itemDef )
	end
	
	if options.multiRescue and options.multiRescue.enabled then
		local simdefs = include("sim/simdefs")
		simdefs.ALLOW_LOADOUT_RESCUABLES = true
	end
	
	local agent_list = include( scriptPath .. "/agent_list" );
	
	for _, k in ipairs( agent_list.AGENT_LIST ) do
		local is_o = options[k .. '_o'].enabled
		local is_a = options[k .. '_a'].enabled
		
		-- remove on-file agent
		if not is_o then
			removeAgent( serverdefs.ORIGINAL_AGENCY, serverdefs.TEMPLATE_AGENCY.unitDefsPotential[k] )
		end
		-- add archive agent
		if is_a then
			modApi:addRescueAgent( serverdefs.TEMPLATE_AGENCY.unitDefsPotential[k] )
		end
	end
	-- Remove DLC agents (their prototypes may not exist, so we remove them by name. Also, they have no archive version)
	-- Luckily, their names are normal, not 'stealth_1' etc.
	for _, k in ipairs( agent_list.AGENT_LIST_DLC ) do
		if not options[k .. '_dlc'].enabled then 
			removeAgent( serverdefs.ORIGINAL_AGENCY, {id=-1, template=k} )
		end
	end
end

return load