local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

local function timeHopAug(unit)
	local res = false
    for i,childAug in ipairs(unit:getChildren( )) do
		if childAug:getTraits().isTimeTravelAug then
			unit:getTraits().timeHopAug = childAug
			res = childAug
		end
	end
	return res
end


local recharge_timeHop =
	{
		name = STRINGS.ABILITIES.RECHARGE,
		createToolTip = function( self,sim,unit,targetCell)
			if timeHopAug(unit):getTraits().usesCharges then
				return abilityutil.formatToolTip( STRINGS.ABILITIES.RECHARGE,  STRINGS.ABILITIES.RECHARGE_CHARGES_DESC, 1 )
			else
				return abilityutil.formatToolTip( STRINGS.ABILITIES.RECHARGE,  STRINGS.ABILITIES.RECHARGE_DESC, 1 )
			end
		end,

		--profile_icon = "gui/items/icon-item_ammo.png",
		profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",

		alwaysShow = false,

		getName = function( self, sim, unit )
			return  STRINGS.ABILITIES.RECHARGE
		end,

		canUseAbility = function( self, sim, itemUnit, unit )
			
				local augUnit = timeHopAug(unit)
			
			
			
			
		
			if augUnit:getTraits().usesCharges then
				if augUnit:getTraits().charges == augUnit:getTraits().chargesMax then
					return false
				end
				if unit:getTraits().leavesAtEndOfMission then
					return false, STRINGS.TIMEHOP.ABILITIES.UNSTABLE
				end
			end

			-- Unit must have an ammo clip, or be flagged infinite-ammo.
			local hasAmmo = array.findIf( unit:getChildren(), function( u ) return u:getTraits().ammo_clip ~= nil end )
			if not hasAmmo then 
				return false
			end

			return true
		end,
		
		executeAbility = function( self, sim, itemUnit, unit )
			local augUnit = timeHopAug(unit)
			local cooldownCheck = augUnit:getTraits().cooldown
			abilityutil.doRecharge( sim, augUnit )
			if augUnit:getTraits().cooldown == cooldownCheck and augUnit:getTraits().cooldown > 3 then
				augUnit:getTraits().cooldown = math.max (3, augUnit:getTraits().cooldown - 2)
			elseif augUnit:getTraits().cooldown ~= cooldownCheck and augUnit:getTraits().cooldown < 3 then
				augUnit:getTraits().cooldown = math.min( cooldownCheck, 3 )

			end
		end
	}
return recharge_timeHop