local mui = include("mui/mui")
local mui_defs = include("mui/mui_defs")
local scroll_text = include("hud/scroll_text")
local util = include("client_util")
local array = require( "modules/array" )
local cdefs = include("client_defs")
local simdefs = include("sim/simdefs")
local agentdefs = include("sim/unitdefs/agentdefs")
local serverdefs = include( "modules/serverdefs" )
local simfactory = include( "sim/simfactory" )
local unitdefs = include("sim/unitdefs")
local guiex = include( "client/guiex" )
local skilldefs = include( "sim/skilldefs" )
local modalDialog = include( "states/state-modal-dialog" )

local upgradeScreen = include( "states/state-upgrade-screen" )


local SET_COLOR = {r=244/255,g=255/255,b=120/255, a=1}
local POSSIBLE_COLOR = {r=56/255,g=96/255,b=96/255, a=1}
local BLANK_COLOR = {r=12/255,g=17/255,b=16/255, a=1}
local HOVER_COLOR = {r=255/255,g=255/255,b=255/255, a=1}
local HOVER_COLOR_FAIL = {r=178/255,g=0/255,b=0/255, a=1}
local TEST_COLOR = {r=0/255,g=184/255,b=0/255, a=1}
local LOCKED_COLOR = {r=255/255,g=150/255,b=19/255, a=1}
local LOCKED_BLANK = {r=12/255,g=17/255,b=17/255, a=1}
--------------------------------
----Incognita Skill Upgrades----
--------------------------------
local skillChanges = upgradeScreen.skillChanges

local baseSkillInit = skillChanges.init
function skillChanges:init( upgradeScreen )
	self.upgradeScreen = upgradeScreen
	
	baseSkillInit( self, upgradeScreen )
end

function skillChanges:getIncognitaSkillCost( skillID, level )
	local skillDef = skilldefs.sitrep_incognita_skills[ skillID ]
	local cost = skillDef[ level ].cost
	return cost
end

local oldSkillChangeInit = skillChanges.init
function skillChanges:init( upgradeScreen )
	oldSkillChangeInit( self, upgradeScreen )
	
	self.incognita = self.agency.sitrep_incognita
end

function skillChanges:countIncognitaChanges( skillId )
	local skill = self.incognita.skills[ skillId ]
	if skill and skill.skillChange then
		return skill.skillChange
	end
	
	return 0
end

function skillChanges:hasIncognitaChanges( skillId )
    return self:countIncognitaChanges( skillId ) > 0
end

function skillChanges:learnIncognitaSkill( skillId )
	local skill = self.incognita.skills[ skillId ]
	local cost = self:getIncognitaSkillCost( skillId, skill.level + 1 )

	-- Subtract cost, increase skill.
	if self.agency.cash >= cost then
		skill.skillChange = (skill.skillChange or 0) + 1
		skill.level = skill.level + 1
		self.agency.cash = self.agency.cash - cost
		return true

	else
		return false
	end
end

function skillChanges:undoIncognitaSkill( skillId )
	local skill = self.incognita.skills[ skillId ]
	local cost = self:getIncognitaSkillCost( skillId, skill.level )
	
	if skillId == "slots" and self.agency.programLoadout and #self.agency.programLoadout > serverdefs.getProgramSlots( self.upgradeScreen._campaign ) - 1 then
		return false
	end

	-- Return cost, decrease skill
	if skill.skillChange and skill.skillChange > 0 then
		skill.skillChange = skill.skillChange - 1
		skill.level = skill.level - 1
		self.agency.cash = self.agency.cash + cost
		return true
	else
		return false
	end
end

function upgradeScreen:onClickUndoIncognitaSkill( skillId )
	if self.changes:undoIncognitaSkill( skillId ) then
		MOAIFmodDesigner.playSound("SpySociety/HUD/gameplay/upgrade_cancel_unit")

		local oldCash = self._agency.cash 

		if self._cashUpdateThread then
			self._cashUpdateThread:stop()
		end
		self._cashUpdateThread = guiex.createCountUpThread( self.screen:findWidget("agencyCredits"), oldCash, self._agency.cash, 1, STRINGS.UI.UPGRADE_SCREEN_AVAILABLE_CREDITS)

		self:refreshIncognitaSkills()
	else
		MOAIFmodDesigner.playSound("SpySociety/HUD/gameplay/upgrade_cancel_unit")
		modalDialog.show( STRINGS.SITREP.UI.REASON.PROGRAM_SLOTS )
	end
end

function upgradeScreen:onClickLearnIncognitaSkill( skillId )
	if self.changes:learnIncognitaSkill( skillId ) then
		MOAIFmodDesigner.playSound("SpySociety/HUD/gameplay/upgrade_select_unit")

		self:refreshCredits()
		self:refreshIncognitaSkills()
	else
		MOAIFmodDesigner.playSound("SpySociety/HUD/gameplay/upgrade_cancel_unit")
		modalDialog.show( STRINGS.UI.REASON.NOT_ENOUGH_CREDITS )
	end
end

------------------------------------

local function onRollOverPlus( self, skill, skillDef, index )
	local level = skill.level + 1
	if skill.level < skillDef.levels then
		if self._agency.cash >= skillDef[level].cost then 				
			self.screen:findWidget("IncognitaSkill_"..index..".bar"..level..".bar"):setColor(HOVER_COLOR.r,HOVER_COLOR.g,HOVER_COLOR.b,HOVER_COLOR.a)			
		else
			self.screen:findWidget("IncognitaSkill_"..index..".bar"..level..".bar"):setColor(HOVER_COLOR_FAIL.r,HOVER_COLOR_FAIL.g,HOVER_COLOR_FAIL.b,HOVER_COLOR_FAIL.a)			
		end	
	end
end

local function onRollOutPlus( self, skill, skillDef, index )
	local level = skill.level + 1
	if skill.level < skillDef.levels then
		self.screen:findWidget("IncognitaSkill_"..index..".bar"..level..".bar"):setColor(POSSIBLE_COLOR.r,POSSIBLE_COLOR.g,POSSIBLE_COLOR.b,POSSIBLE_COLOR.a)
	end
end

local function onRollOverMinus( self, skill, skillDef, index )
	local level = skill.level
    if self.changes:hasIncognitaChanges( index ) then
		self.screen:findWidget("IncognitaSkill_"..index..".bar"..level..".bar"):setColor(HOVER_COLOR.r,HOVER_COLOR.g,HOVER_COLOR.b,HOVER_COLOR.a)			
	end
end

local function onRollOutMinus( self, skill, skillDef, index )
	local level = skill.level 
    if self.changes:hasIncognitaChanges( index ) then
		self.screen:findWidget("IncognitaSkill_"..index..".bar"..level..".bar"):setColor(TEST_COLOR.r,TEST_COLOR.g,TEST_COLOR.b,TEST_COLOR.a)
	end
end

----------------------------------------------------
local tooltipBtnPlus = class()

function tooltipBtnPlus:init( panel, skill, skillDef, index )
	self._panel = panel
 	self._skill = skill
 	self._skillDef = skillDef
 	self._index = index
end

function tooltipBtnPlus:activate( screen )
	onRollOverPlus(self._panel, self._skill, self._skillDef, self._index )
end

function tooltipBtnPlus:deactivate( screen )
	onRollOutPlus(self._panel, self._skill, self._skillDef, self._index )
end

function tooltipBtnPlus:setPosition( )
end


local tooltipBtnMinus = class()

function tooltipBtnMinus:init( panel, skill, skillDef, index )
	self._panel = panel
 	self._skill = skill
 	self._skillDef = skillDef
 	self._index = index
end

function tooltipBtnMinus:activate( screen )
	onRollOverMinus( self._panel, self._skill, self._skillDef, self._index )
end

function tooltipBtnMinus:deactivate( screen )
	onRollOutMinus( self._panel, self._skill, self._skillDef, self._index )
end

function tooltipBtnMinus:setPosition( )
end
----------------------------------------------------

local baseRefreshSkills = upgradeScreen.refreshSkills
function upgradeScreen:refreshSkills( unitDef, k )
	baseRefreshSkills( self, unitDef, k )

	if self._SITREP_enabled then	
		self:refreshIncognitaSkills()
	end
end

function upgradeScreen:refreshIncognitaSkills()
	local skills = self._agency.sitrep_incognita.skills
	
	self.screen:findWidget("programUpgradeBG"):setVisible(true)
	self.screen:findWidget("incognitaUpgradePnl"):setVisible(true)
		
	for id, skill in pairs(skills) do
		local skillDef = skilldefs.sitrep_incognita_skills[ id ]
		local skillWidget = self.screen:findWidget("IncognitaSkill_"..id)
		skillWidget:setVisible(true)
		
		skillWidget.binder.skillTitle:setText(util.toupper(skillDef.name))
		skillWidget.binder.btnBack.onClick = util.makeDelegate( nil, self.onClickUndoIncognitaSkill, self, id )	
		skillWidget.binder.btnFwd.onClick = util.makeDelegate( nil, self.onClickLearnIncognitaSkill, self, id )
		
		if skill.level < skillDef.levels then 			
			local currentLevel = skillDef[ skill.level +1 ]
			skillWidget.binder.costTxt:setText( util.sformat( STRINGS.FORMATS.CREDITS, currentLevel.cost ))
			skillWidget.binder.btn:setDisabled(true) 							
			skillWidget.binder.btnFwd:setVisible(true)
						
		else 
			skillWidget.binder.btnFwd:setVisible(false)
			skillWidget.binder.costTxt:setText( STRINGS.UI.UPGRADE_SCREEN_MAX )
			skillWidget.binder.btn:setDisabled(true) 				
		end
		
		skillWidget.binder.btnBack:setVisible( self.changes:hasIncognitaChanges( id ) )
		
		for j, bar in skillWidget.binder:forEach( "bar" ) do
			if j <= skill.level - self.changes:countIncognitaChanges( id ) then
				bar.binder.bar:setColor(SET_COLOR.r,SET_COLOR.g,SET_COLOR.b,SET_COLOR.a)
			elseif j <= skill.level then
				bar.binder.bar:setColor(TEST_COLOR.r,TEST_COLOR.g,TEST_COLOR.b,TEST_COLOR.a)
			elseif j <= skillDef.levels then
				bar.binder.bar:setColor(POSSIBLE_COLOR.r,POSSIBLE_COLOR.g,POSSIBLE_COLOR.b,POSSIBLE_COLOR.a)
			else
				bar.binder.bar:setColor(BLANK_COLOR.r,BLANK_COLOR.g,BLANK_COLOR.b,BLANK_COLOR.a)
			end
		end
		skillWidget.binder.btn:setColor(1,0,0,1)
		
		local tooltip_BtnPlus = tooltipBtnPlus( self, skill, skillDef, id )
		skillWidget.binder.btnFwd:setTooltip( tooltip_BtnPlus ) 
		local tooltip_BtnMinus = tooltipBtnMinus( self, skill, skillDef, id )
		skillWidget.binder.btnBack:setTooltip( tooltip_BtnMinus ) 
		
		if skillDef.onTooltip then
			local toolTip = util.tooltip( self.screen, nil )
			local section = toolTip:addSection()
			skillDef:onTooltip ( section, self._campaign )
			skillWidget:setTooltip(toolTip) 
		end
	end
end

function upgradeScreen:refreshIncognitaPrograms()
	local widget = self.screen:findWidget("progList")
	for i, program in ipairs( widget:getItems() ) do
		program.widget.binder.SITREP_star:setVisible(false)
		if array.find( self._agency.programLoadout, self._agency.abilities[i] ) then
			program.widget.binder.SITREP_star:setVisible(true)
		end
	end
end

function upgradeScreen:SITREP_selectProgram( index )
	local loadout_index = array.find( self._agency.programLoadout, self._agency.abilities[index] )
	
	if loadout_index then
		table.remove( self._agency.programLoadout, loadout_index )
	else
		if #self._agency.programLoadout < serverdefs.getProgramSlots( self._campaign ) then
			table.insert( self._agency.programLoadout, self._agency.abilities[index] )
		end
	end
	
	self:refreshIncognitaPrograms()
end

local baseSelectIncognita = upgradeScreen.selectIncognita
function upgradeScreen:selectIncognita( unitDef )
	baseSelectIncognita( self, unitDef )
	
	self._agency.programLoadout = self._agency.programLoadout or {}

	local widget = self.screen:findWidget("progList")
	for i, program in ipairs( widget:getItems() ) do
		program.widget.binder.SITREP_btn.onClick = util.makeDelegate( nil, upgradeScreen.SITREP_selectProgram, self, i )
	end
	
	self:refreshIncognitaPrograms()
end


local baseOnLoad = upgradeScreen.onLoad
function upgradeScreen:onLoad( agency, endless, is_post_mission, suppress_map_intro )
	local user = savefiles.getCurrentGame()	
	local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
	
	self._SITREP_enabled = campaign.difficultyOptions.SITREP_incognita_upgrade
	
	self._campaign = campaign

	baseOnLoad( self, agency, endless, is_post_mission, suppress_map_intro )
end
