local Senses = include("sim/btree/senses")
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )

function Senses:processAppearedTrigger(sim, evData)
	local seerUnit = sim:getUnit(evData.seerID)
	if seerUnit ~= self.unit then
		return
	end

	if self.unit:isKO() or self.unit:isDead() then
		return
	end

	local x,y = evData.unit:getLocation()
	local cell = sim:getCell(x, y)

    	if evData.unit:getTraits().smokeEdge then
		self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_SMOKE)
    	elseif evData.unit:getTraits().laptop and evData.unit:getTraits().deployed then
		self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_FOUNDOBJECT)
    	elseif evData.unit:getTraits().sightable and evData.unit:getTraits().deployed then
		self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_NOTICED)
	elseif evData.unit:isDead() or evData.unit:getTraits().interestSource or evData.unit:isKO() then
		local originalUnit = sim:getUnit(evData.unit:getTraits().unitID) or evData.unit
		if self:hasTarget(originalUnit) then
			self:removeTarget(originalUnit)
			evData.unit:setInvestigated(self.unit)
			self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_FOUNDCORPSE, evData.unit)
		elseif not evData.unit:hasBeenInvestigated(self.unit) then
			self:addInterest(x, y, simdefs.SENSE_SIGHT, evData.unit:getTraits().isDrone and simdefs.REASON_FOUNDDRONE or simdefs.REASON_FOUNDCORPSE, evData.unit)
		end
	elseif self:hasTarget(evData.unit) then
		self.targets[evData.unit:getID()].missing = nil
	elseif simquery.isEnemyTarget(self.unit:getPlayerOwner(), evData.unit) or simquery.isKnownTraitor(evData.unit, self.unit) then
		if not simquery.isAgent(evData.unit) and self.unit:getTraits().camera_drone then
			self:addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_NOTICED, evData.unit)
		else 
			self:addTarget(evData.unit)
		end
	end
end

function Senses:processDeployedTrigger(sim, evData)
	local canSee, canSense = sim:canUnitSeeUnit( self.unit, evData.unit )
	if (canSee or canSense) and evData.unit:getTraits().holoProjector then
		local x1,y1 = evData.unit:getLocation() 
		self:addInterest(x1, y1, canSee and simdefs.SENSE_SIGHT or simdefs.SENSE_PERIPHERAL, simdefs.REASON_FOUNDOBJECT, evData.unit)
	end
	if (canSee or canSense) and evData.unit:getTraits().sightable and not evData.unit:getTraits().laptop then
		local x1,y1 = evData.unit:getLocation() 
		self:addInterest(x1, y1, canSee and simdefs.SENSE_SIGHT or simdefs.SENSE_PERIPHERAL, simdefs.REASON_NOTICED, evData.unit)
	end
end

function Senses:processDoorTrigger(sim, evData)
	if evData.unit == self.unit then
		self:checkPeripheralVision()
		return
	end

	if (not simquery.isEnemyTarget(self.unit:getPlayerOwner(), evData.unit)) and not (evData.unit and evData.unit:getTraits().LEVER_status and evData.unit:getTraits().LEVER_status == "HARMLESS" ) then
		return
	end

	local canSeeCell, canSenseCell = sim:canUnitSee(self.unit, evData.cell.x, evData.cell.y)
	local canSeeToCell, canSenseToCell = sim:canUnitSee(self.unit, evData.tocell.x, evData.tocell.y)
	if not canSeeCell and not canSenseCell and not canSeeToCell and not canSenseToCell then
		return
	end

	local sense = simdefs.SENSE_PERIPHERAL
	if canSeeCell or canSeeToCell then
		sense = simdefs.SENSE_SIGHT
	end

	self:addInterest(evData.cell.x, evData.cell.y, sense, simdefs.REASON_DOOR, evData.unit)
end

function Senses:processAttackedTrigger(sim, evData)
	local x0, y0 = evData.sourceUnit:getLocation()
	local x1, y1 = evData.targetUnit:getLocation()

	if not evData.noTargetAlert then
		if evData.targetUnit == self.unit then
			if simquery.couldUnitSeeCell(sim, self.unit, sim:getCell( x0, y0 )) then
				if self:hasLostTarget(evData.sourceUnit) then
					self.unit:turnToFace(x0, y0)
					self:addInterest(x0, y0, simdefs.SENSE_HIT, simdefs.REASON_SHARED, self.unit)
				else
					self.unit:setAlerted(true)
					self:addInterest(x0, y0, simdefs.SENSE_HIT, simdefs.REASON_SHARED, self.unit)
				end
			else
				self.unit:setAlerted(true)
				self:addInterest(x1, y1, simdefs.SENSE_HIT, simdefs.REASON_SHARED, self.unit)
			end
		elseif evData.sourceUnit:getTraits().takenDrone and sim:canUnitSeeUnit(self.unit, evData.sourceUnit) then
			if evData.sourceUnit:getTraits().lastAttack then
				evData.sourceUnit:getTraits().lastAttack.witnesses[self.unit:getID()] = self.unit
			end
			self:addTarget(evData.sourceUnit)
		elseif sim:canUnitSeeUnit(self.unit, evData.targetUnit) then
			if simquery.isAgent(evData.targetUnit)
			 and (not evData.targetUnit:isValid() or evData.targetUnit:isDead() or evData.targetUnit:isKO() ) then
				self:addInterest(x1, y1, simdefs.SENSE_SIGHT, simdefs.REASON_FOUNDCORPSE, evData.targetUnit)
				self:removeTarget(evData.targetUnit)
			else
				self:addInterest(x1, y1, simdefs.SENSE_SIGHT, simdefs.REASON_WITNESS, evData.targetUnit)
			end
		elseif sim:canUnitSee(self.unit, evData.x, evData.y) then
			self:addInterest(evData.x, evData.y, simdefs.SENSE_SIGHT, simdefs.REASON_WITNESS, evData.targetUnit)
		end
	end
end