local array = include("modules/array")
local util = include( "modules/util" )
local serverdefs = include( "modules/serverdefs" )
local agentdefs = include("sim/unitdefs/agentdefs")
local simdefs = include( "sim/simdefs" )

------------------------
----THIS FIXES A BUG----
------------------------
function serverdefs.updateCompatibility( campaign )
	local version = include( "modules/version" )
	while version.isIncompatible( campaign.version ) do
		log:write( "UPDATING CAMPAIGN: v%s", tostring( campaign.version ))
		local major, minor, build = version.parseVersion( campaign.version )
		local updateFile = string.format( "%s/modules/updates/%d.%d.0.lua", config.SRC_MEDIA, major, minor )
		local res, fn = pcall( dofile, updateFile )
		if not res then
			log:write( "Could not find update file: '%s'", updateFile )
			break

		else
			local newVersion = fn( campaign )
			if newVersion and newVersion ~= campaign.version then
				campaign.version = newVersion
				if campaign.sim_history then
					log:write( "\tSUCCESS: v%s [cleared sim_history]", tostring( campaign.version ))
					campaign.sim_history, campaign.wasUpdated = nil, true
				else
					log:write( "\tSUCCESS: v%s", tostring( campaign.version ))
				end
			else
				log:write( "\tFAILED" )
			end
		end
	end

	-- Do a pass of updating difficulty options, so we don't have to ++version everytime we add a setting.

	if campaign.version == version.VERSION then
		campaign.previousDay = campaign.previousDay or (math.floor(campaign.hours / 24))
		campaign.currentDay = campaign.currentDay or (math.floor(campaign.hours / 24))
		campaign.missionsPlayedThisDay = campaign.missionsPlayedThisDay or 0
		campaign.miniserversSeen = campaign.miniserversSeen or 0
		-- Base missing settings off normal difficulty.  Eh.
		for setting, value in pairs( simdefs.DIFFICULTY_OPTIONS[ simdefs.NORMAL_DIFFICULTY ] ) do
			if campaign.difficultyOptions[ setting ] == nil then
				--campaign.difficultyOptions[ setting ] = valuecreateNewSituation--blech
				campaign.difficultyOptions[ setting ] = value
				log:write( "\tAssigning difficulty option: %s = %s", tostring(setting), tostring(value) )
			end
		end
	end

	campaign.incognitaLevel = campaign.incognitaLevel or campaign.currentDay 

	-- TODO: remove me.  fail safe for bugs causing 0 missions.
	if #campaign.situations == 0 and campaign.situation == nil then
		serverdefs.createCampaignSituations( campaign, 1 )
	end
end

------------------------
----HELPER FUNCTIONS----
------------------------
function serverdefs.checkTags( situationTags, tags )
	local tagCount = 0
	-- Each situation tag must exist in 'tags' for it to be a valid choice.
	for i, tag in ipairs(situationTags) do
		if array.find( tags, tag ) then
			tagCount = tagCount + 1
		end
	end

	return tagCount == #situationTags
end

function serverdefs.countSituations( campaign, situationName )
	local count = 0
	for i = 1, #campaign.situations do
		if campaign.situations[i].name == situationName then
			count = count + 1
		end
	end
	return count
end

function serverdefs.chooseSituation( campaign, tags, gen )
	local MAX_DUPLICATE_SITUATION = math.huge
	for k, tag in pairs( tags ) do
		if tag:sub(2) == "max" then
			MAX_DUPLICATE_SITUATION = tonumber(tag:sub(1,1))
		end
	end

	local situationNames = util.weighted_list()
	for name, situationData in pairs( serverdefs.SITUATIONS ) do
		if serverdefs.checkTags( situationData.tags, tags ) and serverdefs.countSituations( campaign, name ) < MAX_DUPLICATE_SITUATION then
			situationNames:addChoice( name, situationData.weight or 1 )
		end
	end

	-- Pick a random situation.
	if situationNames:getTotalWeight() > 0 then
		local wt = gen:nextInt( 1, situationNames:getTotalWeight())
		return situationNames:getChoice( wt )
	end
end

function serverdefs.defaultMapSelector( campaign, tags, tempLocation )
	if array.find( tags, tempLocation.corpName ) == nil then
		return false -- Not allowed corp.
	end

	local MIN_TRAVEL = 1
	local MAX_TRAVEL = 12
	if array.find( tags, "not_close" ) then
		MIN_TRAVEL = campaign.difficultyOptions.not_close_dist or 6
		if campaign.difficultyOptions.close_dist then
			MAX_TRAVEL = math.max(MIN_TRAVEL+1,campaign.difficultyOptions.close_dist)
		end
	end

	local safeDist = true
	local dist = serverdefs.trueCalculateTravelTime( serverdefs.MAP_LOCATIONS[ campaign.location ], tempLocation, campaign )
	if dist <= MIN_TRAVEL or dist > MAX_TRAVEL then
		safeDist = false
	end

	for i,situation in ipairs( campaign.situations ) do
		local travelTime = serverdefs.trueCalculateTravelTime( serverdefs.MAP_LOCATIONS[ situation.mapLocation ], tempLocation, campaign )
		if travelTime <= MIN_TRAVEL or travelTime > MAX_TRAVEL then
		 	safeDist = false
		 	break
		end
	end

	return safeDist
end

-----------------------
----WEIRD CONSTANTS----
-----------------------
function serverdefs.getHoursBeforeSpawn(campaign,hours)
	return simdefs.HOURS_BEFORE_SPAWN
end

function serverdefs.getNumMissionsToSpawn(campaign,hours)
	return simdefs.NUM_MISSIONS_TO_SPAWN
end

------------------------------
----DIFFICULTY CALCULATION----
------------------------------
function serverdefs.getDifficulty( campaign, gen )
	---- For the average mission...
	
	local scaling = campaign.difficultyOptions.difficultyScaling or 1
	local difficulty = 1 + math.floor( scaling * campaign.hours / 24)
	if campaign.campaignDifficulty == simdefs.NORMAL_DIFFICULTY and campaign.hours < 24 then
		-- No chance for increased difficulty on beginner: day 1
	else
		if gen:nextInt( 1, 10 ) <= 3 then
			difficulty = difficulty + 1
		end
	end

	return math.min( serverdefs.MAX_DIFFICULTY.MAX, difficulty )
end

function serverdefs.getAddEndlessMissionDifficulty( campaign, customFinalSituation )
	---- For the DLC mission in endless...
	
	local scaling = customFinalSituation.difficultyScaling or campaign.difficultyOptions.customEndlessDifficultyScaling or 1
	local difficulty = 2 + math.floor( scaling * campaign.hours / 24)
	
	if type(campaign.difficultyOptions.customEndlessDifficulty) == "number" and not customFinalSituation.useDefaultDifficulty then
		difficulty = campaign.difficultyOptions.customEndlessDifficulty
	end
	if customFinalSituation.difficulty then
		difficulty = customFinalSituation.difficulty
	elseif customFinalSituation.difficultyOffset then
		difficulty = difficulty + customFinalSituation.difficultyOffset
	end
	
	return math.min( serverdefs.MAX_DIFFICULTY.MAX, difficulty )
end

function serverdefs.getCustomFinalSituationDifficulty( campaign, customFinalSituation )
	---- For the DLC mission in campaign...
	
	local scaling = customFinalSituation.difficultyScaling or campaign.difficultyOptions.customFinalMissionScaling or 1
	local difficulty = 2 + math.floor( scaling * campaign.hours / 24)
	
	if type(campaign.difficultyOptions.customFinalDifficulty) == "number" and not customFinalSituation.useDefaultDifficulty then
		difficulty = campaign.difficultyOptions.customFinalDifficulty
	end
	if customFinalSituation.difficulty then
		difficulty = customFinalSituation.difficulty
	elseif customFinalSituation.difficultyOffset then
		difficulty = difficulty + customFinalSituation.difficultyOffset
	end
	
	return math.min( serverdefs.MAX_DIFFICULTY.MAX, difficulty )
end

function serverdefs.getFinalSituationDifficulty( campaign )
	---- For the final mission...
	
	local scaling = campaign.difficultyOptions.finalMissionScaling or 1
	local difficulty = 2 + math.floor( scaling * campaign.hours / 24)
	
	if type(campaign.difficultyOptions.finalMissionDifficulty) == "number" then
		difficulty = campaign.difficultyOptions.finalMissionDifficulty
	elseif campaign.difficultyOptions.finalMissionDifficultyOffset then
		difficulty = difficulty + campaign.difficultyOptions.finalMissionDifficultyOffset
	end
	
	if not campaign.difficultyOptions.removeMinDifficultyFromFinalMission then
		difficulty = math.max(difficulty,5)
	end
	
	return math.min( serverdefs.MAX_DIFFICULTY.MAX, difficulty )
end

---------------------------------
----ACTUAL MISSION GENERATION----
---------------------------------

function serverdefs.createNewSituation(campaign, gen, tags, difficulty )
	assert( tags and #tags > 0 )
	local seed = gen._seed

	-------------------------------------
	-- Choose from serverdefs.SITUATIONS.
	local situationName = serverdefs.chooseSituation( campaign, tags, gen )
	if situationName == nil then
		log:write( "No situations found for: hour %d, %s", campaign.hours, util.stringize(tags) )
		return nil
	end

	-------------------------------------
	-- Determine difficulty from hours into the campaign.
	difficulty = difficulty or serverdefs.getDifficulty( campaign, gen )

	-------------------------------------
	-- Determine map location (which in turn implies CORP)
	local situationData = serverdefs.SITUATIONS[ situationName ]
	local mapLocation = nil
	local availableLocations = {}
	for i, location in ipairs( serverdefs.MAP_LOCATIONS) do
		assert( location.name )
		if serverdefs.defaultMapSelector( campaign, tags, location ) then
			table.insert( availableLocations, i )
		end
	end
	if #availableLocations > 0 then
		mapLocation = availableLocations[ gen:nextInt(1,#availableLocations) ]
	end
	
	if mapLocation then
		log:write( "NEW SITUATION{ %s } -- %s, %d, %s, %d", util.stringize(tags), situationName, difficulty, serverdefs.MAP_LOCATIONS[ mapLocation ].name, seed )
		return { name = situationName, difficulty = difficulty, mapLocation = mapLocation, new = true }
	else
		log:write( "\tCould not find map location: %s", util.stringize( tags ) )
	end
end

function serverdefs.createCampaignSituations( campaign, count, tags, difficulty )
	local rand = include( "modules/rand" )
	local gen = rand.createGenerator( campaign.seed )
	
	for i = 1, count do
		local situationTags = nil 
		if campaign.missionCount == 0 and campaign.campaignDifficulty == simdefs.NORMAL_DIFFICULTY then 
			situationTags = tags or simdefs.DEFAULT_MISSION_TAGS_FIRST
		else 
			situationTags = tags or simdefs.DEFAULT_MISSION_TAGS
		end 
		local situation = serverdefs.createNewSituation( campaign, gen, situationTags, difficulty )
		if situation then
			table.insert( campaign.situations, situation )
			campaign.missionTotal = campaign.missionTotal + 1

		else
			log:write( "Found no mission with tags '%s'", util.stringize(situationTags) )
		end
	end

	campaign.seed = gen._seed
end

function serverdefs.createCustomFinalSituation( campaign, customFinalSituation )
	assert(customFinalSituation)
	local rand = include( "modules/rand" )
	local gen = rand.createGenerator( campaign.seed )
	local returnSituations = {}
	local situation = {}
	
	local difficulty = serverdefs.getCustomFinalSituationDifficulty( campaign, customFinalSituation )

	situation = serverdefs.createNewSituation( campaign, gen, util.tmerge( { customFinalSituation.name }, serverdefs.CORP_NAMES ), difficulty  )
	if customFinalSituation.corp then
		situation.corpName = customFinalSituation.corp
	end

	if not campaign.lastMissionIndex then
		campaign.lastMissionIndex = 0
	end

	campaign.lastMissionIndex = campaign.lastMissionIndex +1 
	
	table.insert(returnSituations, situation )
	return returnSituations
end

function serverdefs.createFinalSituations( campaign )
	local rand = include( "modules/rand" )
	local gen = rand.createGenerator( campaign.seed )
	local returnSituations = {}

	local difficulty = serverdefs.getFinalSituationDifficulty( campaign )

	local situation = serverdefs.createNewSituation( campaign, gen, util.tmerge( { "ending_1" }, serverdefs.CORP_NAMES ), difficulty )
	situation.corpName = "omni"
	table.insert(returnSituations, situation )

	return returnSituations
end

local function checkAdvanceCampaignTime( campaign, hours, stage )
	for i, event in pairs(mod_manager.advanceTimeEvents) do
		if event.stage == stage then
			event.fn( campaign, hours, stage )
		end
	end
end

function serverdefs.newDay(campaign, hours)
	local NUM_MISSIONS_TO_SPAWN = serverdefs.getNumMissionsToSpawn(campaign, hours)

	if campaign.customScriptIndex then
		if campaign.customScriptIndexDay then
			campaign.customScriptIndexDay = campaign.customScriptIndexDay +1
		else
			campaign.customScriptIndexDay = 1
		end				
	end

	campaign.missionsPlayedThisDay = 0
	
	local MAX_DIFFICULTY_LOC = serverdefs.MAX_DIFFICULTY.MAX
	if (campaign.difficultyOptions.maxHours ~= math.huge and simdefs.MAX_DIFFICULTY < MAX_DIFFICULTY_LOC) then
		MAX_DIFFICULTY_LOC = simdefs.MAX_DIFFICULTY
	end

	for i,situation in ipairs(campaign.situations) do
		situation.difficulty = math.min( MAX_DIFFICULTY_LOC, situation.difficulty + 1 )
	end 

	if campaign.campaignDifficulty == simdefs.NORMAL_DIFFICULTY and campaign.currentDay == 1 then
		-- Day 2 beginner: only difficulty 2.
		serverdefs.createCampaignSituations( campaign, NUM_MISSIONS_TO_SPAWN, nil, 2 )	
	else
		serverdefs.createCampaignSituations( campaign, NUM_MISSIONS_TO_SPAWN )	
	end

	if campaign.difficultyOptions.maxHours == math.huge and campaign.currentDay >= serverdefs.ENDLESS_ALERT_DAYS.DAY then 
		campaign.endlessAlert = true 
	end
end

function serverdefs.newFinalDay(campaign, hours)
	-- If if it's not a new day, but we've exceeded total hours... start the final leg.

	local customFinalSituation = nil
	if campaign.campaignEvents then
		for i,event in ipairs( campaign.campaignEvents ) do
			if event.eventType == simdefs.CAMPAIGN_EVENTS.CUSTOM_FINAL then
				customFinalSituation = event.data
				print("Removing campaign event",i,"CUSTOM_FINAL")
				table.remove(campaign.campaignEvents,i)

				local eventNew = {
					eventType = simdefs.CAMPAIGN_EVENTS.CUSTOM_SCRIPT,
					data = event.data.name,	
				}
				table.insert(campaign.campaignEvents,eventNew)   					
				break
			end		   
		end
	end
	if customFinalSituation then
		campaign.situations = serverdefs.createCustomFinalSituation( campaign,  customFinalSituation )
	else
		campaign.situations = serverdefs.createFinalSituations( campaign )
	end
end

function serverdefs.checkAddEndlessMission(campaign,hours)
	if campaign.campaignEvents then
		for i=#campaign.campaignEvents,0,-1 do
			local event = campaign.campaignEvents[i] 
			if event then
				if event.eventType == simdefs.CAMPAIGN_EVENTS.ADD_ENDLESS_MISSION then
					if campaign.difficultyOptions.maxHours == math.huge then
						if campaign.hours >= (event.day-1)*24 then
							
							local rand = include( "modules/rand" )
							local gen = rand.createGenerator( campaign.seed )

							local difficulty = serverdefs.getAddEndlessMissionDifficulty( campaign, event )

							local situation = serverdefs.createNewSituation( campaign, gen, util.tmerge( { event.mission }, serverdefs.CORP_NAMES ), difficulty )							
							campaign.situations = {situation}

							table.remove(campaign.campaignEvents,i)
						end			
					end
				end				
			end
		end
	end
end

function serverdefs.monst3rInject(campaign,hours)
	serverdefs.createCampaignSituations( campaign, 2 )
	campaign.monst3rInject = true
end

------------------------------------------------------
----THE MAIN FUNCTION FOR PROGRESSING THE CAMPAIGN----
------------------------------------------------------

function serverdefs.advanceCampaignTime( campaign, hours )
	checkAdvanceCampaignTime( campaign, hours, 0 )
	
	if campaign.hours >= campaign.difficultyOptions.maxHours then
		return -- No more advancement!
	end

	local HOURS_BEFORE_SPAWN = serverdefs.getHoursBeforeSpawn(campaign, hours)
	local adjustmentHours = hours
	local previousDay = campaign.currentDay
	campaign.previousDay, campaign.currentDay = math.floor(campaign.hours / HOURS_BEFORE_SPAWN), math.floor((campaign.hours + adjustmentHours) / HOURS_BEFORE_SPAWN)

	if previousDay ~= campaign.currentDay then
		campaign.incognitaLevel = campaign.incognitaLevel + 1
	end

	campaign.hours = campaign.hours + adjustmentHours
	campaign.seed = campaign.seed + 1
	campaign.lastEndlessAlert = false

	checkAdvanceCampaignTime( campaign, hours, 1 )

	-- Determine if a new day has dawned: if so, add new situations.
	if campaign.hours >= campaign.difficultyOptions.maxHours then
		serverdefs.newFinalDay(campaign,hours)
	elseif campaign.currentDay > campaign.previousDay then
		serverdefs.newDay(campaign,hours)
	end
	if campaign.customScriptIndex and not campaign.customScriptIndexDay then
		campaign.customScriptIndexDay = 1		
	end
	checkAdvanceCampaignTime( campaign, hours, 2 )

	--If no situations left, "monst3r inject"
	if #campaign.situations <= 0 then
		serverdefs.monst3rInject(campaign,hours)
	else
		campaign.monst3rInject = false
	end
	
	checkAdvanceCampaignTime( campaign, hours, 3 )
	serverdefs.checkAddEndlessMission( campaign, hours ) -- If it's endless, check if we want to insert the DLC mission
	checkAdvanceCampaignTime( campaign, hours, 4 )
	
	for i, situation in pairs(campaign.situations) do
		situation.difficulty = math.min(situation.difficulty, serverdefs.MAX_DIFFICULTY.MAX)
	end
end

------------------------------------------------------
--------THE FUNCTION THAT CREATES THE CAMPAIGN--------
------------------------------------------------------

function serverdefs.createNewCampaign( agency, campaignDifficulty, difficultyOptions )
	local version = include( "modules/version" )
	local campaign =
	{
		agency = agency,
		location = agency.startLocation,
		play_t = 0,
		hours = 0,
		previousDay = -1,
		currentDay = 0,
		incognitaLevel = 0,
		version = version.VERSION,
		situations = {},
		missionCount = 0,
		missionTotal = 0,
		miniserversSeen = 0,
		missionsPlayedThisDay = 0,
		recent_build_number = util.formatGameInfo(),
		save_time = os.time(),
		creation_time = os.time(),
		missionEvents = {},
	}

	campaign.seed = config.LOCALSEED()
	campaign.campaignEvents = mod_manager:getCampaignEvents()
	
	if not difficultyOptions then
		-- Initializes the campaign situation to the tutorial level.
		campaign.campaignDifficulty = simdefs.TUTORIAL_DIFFICULTY
		campaign.difficultyOptions = util.tcopy( simdefs.DIFFICULTY_OPTIONS[ simdefs.TUTORIAL_DIFFICULTY ])
		campaign.situation = { name = serverdefs.TUTORIAL_SITUATION, difficulty = 1, mapLocation = 2 } -- Starts in FTM (Vancouver)
		campaign.missionVersion = version.VERSION
	else
		-- Standard campaign startup.
		campaign.campaignDifficulty = campaignDifficulty
		campaign.difficultyOptions = util.tcopy( difficultyOptions )
		campaign.agency.cash = difficultyOptions.startingCredits
		campaign.agency.cpus = difficultyOptions.startingPower
		
		campaign.hours = difficultyOptions.startingHours or 0
		
		campaign.currentDay = math.floor((difficultyOptions.startingHours or 0)/24) 
		campaign.previousDay = campaign.currentDay-1

		if campaign.hours >= campaign.difficultyOptions.maxHours then
			-- Start with the final mission
			serverdefs.newFinalDay(campaign,campaign.hours)
		else
			serverdefs.createCampaignSituations( campaign, 1, serverdefs.MISSION_TAGS_INITIAL_EXEC, campaign.currentDay+1 )
			if difficultyOptions.startingMissions and difficultyOptions.startingMissions > 1 then
				serverdefs.createCampaignSituations( campaign, difficultyOptions.startingMissions-1 )
			end
		end
	
		serverdefs.checkAddEndlessMission( campaign, campaign.hours )
	end
	
	for i, situation in pairs(campaign.situations) do
		situation.difficulty = math.min(situation.difficulty, serverdefs.MAX_DIFFICULTY.MAX)
	end

	return campaign
end

-----------------------------
----RANDOMIZE RESCUEABLES----
-----------------------------

function serverdefs.createAgency( agentIDs, programIDs )
	agentIDs = agentIDs or { "stealth_1", "engineer_2" }
	programIDs = programIDs or { "remoteprocessor", "lockpick_1" }

	local agency = util.extend( serverdefs.TEMPLATE_AGENCY )
	{
		id = 1,
		abilities = programIDs,
		alwaysUnlocked = true, 
		startLocation = 23,
	}
	
	local useTemplateAsId = simdefs.ENABLE_FULLPARTY or simdefs.ALLOW_LOADOUT_RESCUABLES
	
	if useTemplateAsId then
		for i, def in ipairs( agency.unitDefsPotential ) do
			def.id = def.template
		end
	end
	
	if simdefs.ENABLE_FULLPARTY then
		local addAgents = {}
	
		for i, selectable in ipairs( serverdefs.SELECTABLE_AGENTS ) do
			local loadouts = serverdefs.LOADOUTS[ selectable ]
			if loadouts then
				for i, template in ipairs( loadouts ) do
					if not array.find( agentIDs, template ) then
						table.insert( addAgents, template )
					end
				end
			end
		end
		
		util.tmerge( agentIDs, addAgents )
		agency.unitDefsPotential = {}
	end

	for k,v in ipairs(agentIDs) do
		local agent = serverdefs.createAgent( v, util.tcopy( agentdefs[v].upgrades ))
		if useTemplateAsId then
			agent.id = agent.template
		end
		serverdefs.assignAgent( agency, agent )
	end
	
	local rescueablesByAgentID = {}
	for i = #agency.unitDefsPotential, 1, -1 do
		local def = agency.unitDefsPotential[i]
		rescueablesByAgentID[def.id] = rescueablesByAgentID[def.id] or {}
		table.insert(rescueablesByAgentID[def.id], def)
	end
	
	for i, defs in pairs(rescueablesByAgentID) do
		if #defs > 1 then
			local randomToKeep = math.random(1, #defs)
			for j = 1, #defs do
				if j ~= randomToKeep then
					defs[j].markedForRemoval = true
				end
			end
		end
	end
	
	for i = #agency.unitDefsPotential, 1, -1 do
		if agency.unitDefsPotential[i].markedForRemoval then
			table.remove( agency.unitDefsPotential, i )
		end
	end
	
	return agency
end

------------------------------------------------------
---------------------TRAVEL TIME----------------------
------------------------------------------------------

serverdefs.trueCalculateTravelTime = serverdefs.trueCalculateTravelTime or serverdefs.calculateTravelTime

serverdefs.TRAVEL_UNIT = 40
serverdefs.TRAVEL_UNIT_SPHERE = 15 / math.pi
serverdefs.WORLD_WIDTH = 1170 -- based on the widget size
serverdefs.TRAVEL_CEILING = 12

-- fromLocation, toLocation are both actual table entries of MAP_LOCATIONS, or indices therein.
function serverdefs.calculateTravelTime( fromLocation, toLocation, campaign )
	if not toLocation then 
		return 	0
	end
	
	-- If 
	if type(fromLocation) == "number" then
		fromLocation = serverdefs.MAP_LOCATIONS[ fromLocation ]
	end
	if type(toLocation) == "number" then
		toLocation = serverdefs.MAP_LOCATIONS[ toLocation ]
	end
	
	local x0, y0 = fromLocation.x, fromLocation.y
	local x1, y1 = toLocation.x, toLocation.y
	local hours
	
	-- Parameters/Generation Options
	local hourMultiplier = ((campaign and campaign.difficultyOptions.hourMultiplier) or rawget(simdefs,"hourMultiplier") or 1)
	local baseTraveltime = ((campaign and campaign.difficultyOptions.baseTraveltime) or rawget(simdefs,"baseTraveltime") or serverdefs.BASE_TRAVEL_TIME)
	local travelCeiling = (campaign and campaign.difficultyOptions.travelCeiling or serverdefs.TRAVEL_CEILING) 
	local sphericalEarth
	if campaign then
		sphericalEarth = campaign.difficultyOptions.sphericalEarth
	else
		sphericalEarth = rawget(simdefs,"sphericalEarth")
	end
	
	local scaleFactor = 1
	
	if sphericalEarth then
		local N0 = 2 * math.atan(math.exp((y0 + 90) / 170)) - math.pi/2
		local N1 = 2 * math.atan(math.exp((y1 + 90) / 170)) - math.pi/2
		
		if N0 == N1 then
			scaleFactor = math.cos(N1)
		else
			scaleFactor = (math.sin(N1) - math.sin(N0)) / (N1 - N0)
		end
	end
	
	-- This is the "standard" straight line distance
	local dx = math.min( math.abs(x0 - x1), math.abs(x0 + serverdefs.WORLD_WIDTH - x1), math.abs(x0 - serverdefs.WORLD_WIDTH - x1) )
	local dy = math.abs(y1 - y0)
	local dist = math.sqrt(dx * dx + dy * dy)*scaleFactor
	
	-- Apply parameters
	dist = dist * hourMultiplier
	hours = math.ceil(dist/serverdefs.TRAVEL_UNIT)
	hours = hours + baseTraveltime
	hours = math.min( travelCeiling, hours )
	
	return hours - serverdefs.BASE_TRAVEL_TIME
end

function serverdefs.calculateCurrentHours(campaign)

	--calculate the current hours including the current mission
	if campaign.location and campaign.situation then
		local situationTime = 0
		if campaign.situation.mapLocation then
			situationTime = serverdefs.calculateTravelTime( campaign.location, campaign.situation.mapLocation, campaign ) + serverdefs.BASE_TRAVEL_TIME
		end
		return campaign.hours + situationTime
	else
		return campaign.hours
	end
end

function serverdefs.createTutorialAgency()
	local agency = util.extend( serverdefs.TEMPLATE_AGENCY ) {
		name = "Tutorial Agency",
		abilities = { "lockpick_1" },
		startLocation = 23,
		cpus = 0,
		cash = 350,
	}
	
	agency.unitDefsPotential = {}
	serverdefs.assignAgent( agency, serverdefs.createAgent( "tutorial" ))
	mod_manager:resetContent()
	mod_manager:loadModContent()

	return agency
end