local util = include( "modules/util" )
local array = include("modules/array")
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local agentdefs = include("sim/unitdefs/agentdefs")

function serverdefs.assignChidAgent( agency, agentDef )
	if not agency.unitDefs_childs then
		agency.unitDefs_childs = {}
	end
	assert( array.find( agency.unitDefs_childs, agentDef ) == nil )
	table.insert( agency.unitDefs_childs, agentDef )
end

function serverdefs.findChildAgent( agency, agentID )
	if not agency.unitDefs_childs then
		agency.unitDefs_childs = {}
	end
	
	local pred = function( agentDef ) return agentDef.id == agentID end
	return array.findIf( agency.unitDefs_childs, pred )
end

local baseAssignAgent = serverdefs.assignAgent
function serverdefs.assignAgent( agency, agentDef )
	baseAssignAgent( agency, agentDef )
	
	local unitTemplate = unitdefs.lookupTemplate( agentDef.template )
	
	if unitTemplate.maa_summons then
		for _, summon in ipairs( unitTemplate.maa_summons ) do
			local summonDef = serverdefs.createChildAgent( summon, util.tcopy( agentdefs[summon].upgrades ), agentDef.id )
			serverdefs.assignChidAgent( agency, summonDef )
		end
	end
end

function serverdefs.createChildAgent( templateName, upgrades, parentID )
	assert( parentID )
	local unitDef = agentdefs[ templateName ]
	local agent = 
	{
		id = unitDef.agentID,
		template = templateName,
		upgrades = upgrades or {},
	}
	assert( agent.id and agent.template )
	return agent
end
