----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local mathutil = include( "modules/mathutil" )

-----------------------------------------------------
-- Local functions
local shockmine =
{
	ClassType = "shockmine"
}

function shockmine:toggle( sim )
	if self:getTraits().mainframe_status == "active" then
		shockmine:deactivate( sim )
	else
		shockmine:activate( sim )
	end
end

function shockmine:activate( sim )
	if self:getTraits().mainframe_status == "inactive" then
		self:getTraits().mainframe_status = "active"
		self:getTraits().hasSight = true
		self:getTraits().LOSrange = math.max(1.35, math.min(0.85 + self:getTraits().mainframe_ice * 0.25, 2.5))
		sim:refreshUnitLOS( self )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
	end
end

function shockmine:deactivate( sim )	
	if self:getTraits().mainframe_status == "active" then
		self:getTraits().mainframe_status = "inactive"
		self:getTraits().hasSight = nil
		sim:refreshUnitLOS( self )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
	end
end

function shockmine:onWarp( sim, oldcell, cell )
	if oldcell == nil then
		sim:addTrigger( simdefs.TRG_OVERWATCH, self)
		sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
		self:setPlayerOwner( sim:getNPC() )
		self:getTraits().LOSrange = math.max(1.35, math.min(0.85 + self:getTraits().mainframe_ice * 0.25, 2.5))
		sim:refreshUnitLOS( self )
	elseif cell == nil then
		sim:removeTrigger( simdefs.TRG_OVERWATCH, self)
		sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
	end
end 

function shockmine:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_OVERWATCH and evData then
		local isActive = (self:getTraits().mainframe_status == "active" and self:getPlayerOwner():isNPC())
		if isActive then
			if simquery.isEnemyAgent( self:getPlayerOwner(), evData ) and sim:canUnitSeeUnit( self, evData ) then
				if evData:getTraits().movingBody and evData:getTraits().movingBody:getPlayerOwner() == self:getPlayerOwner() then
					return
				end
				self:detonate(sim, evData)
			end
		end
	end
	if evType == simdefs.TRG_ICE_BROKEN and evData.unit == self then
		self:getTraits().LOSrange = math.max(1.35, math.min(0.85 + self:getTraits().mainframe_ice * 0.25, 2.5))
		sim:refreshUnitLOS( self )
	end
end

function shockmine:checkParasites( sim )
	local mainframe = include( "sim/mainframe" )
	if sim:isVersion("0.17.5") then
		if self:getTraits().parasite then
			mainframe.removeParasite(sim:getPC(), self)
		end
	end
end

function shockmine:onDamage( damage, sim )
	self:checkParasites(sim)

	local x1,y1 = self:getLocation()	
	sim:emitSound( simdefs.SOUND_CAMERA_DESTROYED, x1, y1, self )

	self:detonate(sim)
end

function shockmine:detonate( sim, targetUnit )
	local x0,y0 = self:getLocation()
	local currentCell = sim:getCell( x0, y0 )
	local cells = {currentCell}
	local fillCells = simquery.fillCircle( sim, x0, y0, self:getTraits().LOSrange + 0.25, 0)

	for i, cell in ipairs(fillCells) do
		if cell ~= currentCell then
			local raycastX, raycastY = sim:getLOS():raycast(x0, y0, cell.x, cell.y)
			if raycastX == cell.x and raycastY == cell.y then
				table.insert(cells, cell)
			end
		end
	end

	sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, units = nil, range = math.floor(self:getTraits().LOSrange) } )
	local soundRange = { path = nil, range = self:getTraits().soundRange }
	sim:emitSound( soundRange, x0, y0, self )
	local dmgt =
	{
		damage = 1,
		ko = false,
		noTargetAlert = true,
	}

	sim:startTrackerQueue(true)				
	sim:startDaemonQueue()	
	sim:dispatchEvent( simdefs.EV_KO_GROUP, true )

	for i, cell in pairs(cells) do
		local cellUnits = {}
		for j, cellUnit in pairs( cell.units ) do
			table.insert(cellUnits, cellUnit)
		end
		for j = #cellUnits, 1, -1 do
			if cellUnits[j]:getTraits().isAgent and sim:canUnitSeeUnit( self, cellUnits[j] ) then
				if cellUnits[j]:getTraits().movingBody then
					sim:hitUnit( self, cellUnits[j]:getTraits().movingBody, dmgt )
				end
				sim:hitUnit( self, cellUnits[j], dmgt )
			end
		end
	end
	if targetUnit and targetUnit:isValid() and not targetUnit:isDown() then
		sim:hitUnit( self, targetUnit, dmgt )
	end
	self:killUnit( sim )
	sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
	sim:startTrackerQueue(false)				
	sim:processDaemonQueue()
end
-----------------------------------------------------
-- Interface functions

local function createShockMine( unitData, sim )
	return simunit.createUnit( unitData, sim, shockmine )
end

simfactory.register( createShockMine )

return createShockMine

