local shop_panel = include("hud/shop_panel")

local cdefs = include("client_defs")
local guiex = include( "guiex" )
local modalDialog = include( "states/state-modal-dialog" )
local array = include("modules/array")
local util = include("modules/util")
local simquery = include("sim/simquery")
local simdefs = include("sim/simdefs")

local researchRefreshUserPrograms = shop_panel.research.refreshUserPrograms

function shop_panel.research:refreshUserPrograms( player, ability, widget, i, ... )
	researchRefreshUserPrograms( self, player, ability, widget, i, ... )

	-- Update empty slots to use simquery.getMaxPrograms
	if ability == nil then
		local sim = self._hud._game.simCore
		local maxPrograms = simquery.getMaxPrograms( sim )
		if i <= maxPrograms then
            guiex.updateButtonEmptySlot( widget )
			widget.binder.cost:setVisible(false)
			widget.binder.itemName:setVisible(false)
		else
			widget:setVisible( false )
		end
		return false

	else
		return true
	end
end

local serverRefreshUserPrograms = shop_panel.server.refreshUserPrograms
function shop_panel.server:refreshUserPrograms( player, ability, widget, i, ... )
	-- Insert scrollability
	i = i + (self.scrollProgramsIdx or 0)
	ability = self._unit:getAbilities()[i]

	serverRefreshUserPrograms( self, player, ability, widget, i, ... )

	-- Update empty slots to use simquery.getMaxPrograms
	if ability == nil then
		local sim = self._hud._game.simCore
		local maxPrograms = simquery.getMaxPrograms( sim )
		if i <= maxPrograms then
            guiex.updateButtonEmptySlot( widget )
			widget.binder.cost:setVisible(false)
			widget.binder.itemName:setVisible(false)
		else
			widget:setVisible( false )
		end
		return false

	else
		return true
	end
end

----- Helper function (extracted from onClickBuyItem/onClickTradeItem) -----
function shop_panel.canTakeItem( panel, item, itemType, player )
	local sim = panel._hud._game.simCore	

	if player ~= sim:getCurrentPlayer() then
		modalDialog.show( STRINGS.UI.TOOLTIP_CANT_PURCHASE )
		return false
	end

	-- Raise program cap using simquery.getMaxPrograms
	if item:getTraits().mainframe_program then
		local maxPrograms = simquery.getMaxPrograms( sim )
		if #player:getAbilities() >= maxPrograms then
			modalDialog.show( STRINGS.UI.TOOLTIP_PROGRAMS_FULL )
			return false
		end		
		if player:hasMainframeAbility( item:getTraits().mainframe_program ) then
			modalDialog.show( STRINGS.UI.TOOLTIP_ALREADY_OWN )
			return false
		end
	elseif panel._unit:getInventoryCount() >= 8 then  
		modalDialog.show( STRINGS.UI.TOOLTIP_INVENTORY_FULL )
		return false
	end
	
	return true
end

----- Helper function (extracted from onClickBuyItem) -----
function shop_panel.canBuyItem( panel, item, itemType, player )
	if not shop_panel.canTakeItem( panel, item, itemType, player ) then
		return false
	end

	local credits = player:getCredits()
	if credits < (item:getUnitData().value * panel._discount) then 
		modalDialog.show( STRINGS.UI.TOOLTIP_NOT_ENOUGH_CREDIT )
		return false
	end
	
	return true
end

-- Updated to use our helper functions
function shop_panel.onClickBuyItem( panel, item, itemType )
	local player = panel._unit
	if not panel._unit._isPlayer then	
	 	player = panel._unit:getPlayerOwner()
	end

	if shop_panel.canBuyItem( panel, item, itemType, player ) then
		local itemIndex = shop_panel.getItemIndex( panel, item, itemType )

		MOAIFmodDesigner.playSound(cdefs.SOUND_HUD_BUY)

		panel._hud._game:doAction( "buyItem", panel._unit:getID(), panel._shopUnit:getID(), itemIndex, panel._discount, itemType, panel._buyback )
		panel.last_action = panel._buyback and "buyback" or "buy"
		panel:refresh()
	end
end

-- Updated to use our helper functions
function shop_panel.onClickTradeItem( panel, item, itemType )
	local player = panel._unit
	if not panel._unit._isPlayer then	
	 	player = panel._unit:getPlayerOwner()
	end

	if shop_panel.canTakeItem( panel, item, itemType, player ) then
		local itemIndex = shop_panel.getItemIndex( panel, item, itemType )

		MOAIFmodDesigner.playSound(cdefs.SOUND_HUD_BUY)

		panel._hud._game:doAction( "tradeItem", panel._unit:getID(), panel._shopUnit:getID(), itemIndex, panel._discount, itemType, panel._buyback )
		panel.last_action = panel._buyback and "buyback" or "buy"
		panel:refresh()
	end
end

----- Helper function (Extracted from several different functions)
function shop_panel.getItem( panel, widget, i, itemType )
	local item = nil 
	if panel._buyback then
		if itemType == "item" or itemType == nil then 
			item = panel._shopUnit.buyback.items[i]
		elseif itemType == "weapon" then 
			item = panel._shopUnit.buyback.weapons[i]
		elseif itemType == "augment" then 
			item = panel._shopUnit.buyback.augments[i]
		end
	else 
		if itemType == "item" or itemType == nil then 
			item = panel._shopUnit.items[i]
		elseif itemType == "weapon" then 
			item = panel._shopUnit.weapons[i]
		elseif itemType == "augment" then 
			item = panel._shopUnit.augments[i]
		end
	end
	return item
end

----- Helper function (Extracted from several different functions)
function shop_panel.getItemIndex( panel, item, itemType )
	local itemIndex = nil 

	if panel._buyback then 
		if itemType == "item" or itemType == nil then 
			itemIndex = array.find( panel._shopUnit.buyback.items, item )
		elseif itemType == "weapon" then 
			itemIndex = array.find( panel._shopUnit.buyback.weapons, item )
		elseif itemType == "augment" then 
			itemIndex = array.find( panel._shopUnit.buyback.augments, item )
		end
	else 
		if itemType == "item" or itemType == nil then 
			itemIndex = array.find( panel._shopUnit.items, item )
		elseif itemType == "weapon" then 
			itemIndex = array.find( panel._shopUnit.weapons, item )
		elseif itemType == "augment" then 
			itemIndex = array.find( panel._shopUnit.augments, item )
		end
	end 
	return itemIndex
end

----- Update buttons to reference our new onClick functions -----

local oldRefreshShop = shop_panel.shop.refreshItem
local oldRefreshServer = shop_panel.server.refreshItem
local oldRefreshResearch = shop_panel.research.refreshItem

function shop_panel.shop:refreshItem( widget, i, ... )
	local ret = oldRefreshShop( self, widget, i, ... )

	local item = shop_panel.getItem( self, widget, i, ... )
	if item ~= nil then
		widget.binder.btn.onClick = util.makeDelegate( nil, shop_panel.onClickBuyItem, self, item, ... )
        widget.binder.btn.onDragStart = util.makeDelegate( self, "onDragFromBottom", item:getUnitData().profile_icon, widget.binder.btn.onClick )
	end
	
	return ret
end

function shop_panel.server:refreshItem( widget, i, ... )
	local ret = oldRefreshServer( self, widget, i, ... )

	local item = shop_panel.getItem( self, widget, i, ... )
	if item ~= nil then
		widget.binder.btn.onClick = util.makeDelegate( nil, shop_panel.onClickBuyItem, self, item, ... )
        widget.binder.btn.onDragStart = util.makeDelegate( self, "onDragFromBottom", item:getUnitData().profile_icon_100, widget.binder.btn.onClick )
	end
	
	return ret
end

function shop_panel.research:refreshItem( widget, i, ... )
	local ret = oldRefreshResearch( self, widget, i, ... )

	local item = shop_panel.getItem( self, widget, i, ... )
	if item ~= nil then
		widget.binder.btn.onClick = util.makeDelegate( nil, shop_panel.onClickTradeItem, self, item, ... )
        widget.binder.btn.onDragStart = util.makeDelegate( self, "onDragFromBottom", item:getUnitData().profile_icon_100, widget.binder.btn.onClick )
	end
	
	return ret
end

----- Insert scrolling -----
local oldRefreshServerFull = shop_panel.server.refresh

function shop_panel.server:scroll( offset )
	self.scrollProgramsIdx = self.scrollProgramsIdx + offset
	self:refresh()
end

function shop_panel.server:refresh( ... )
	local programCount = #self._unit:getAbilities()
	local rgtBtn = self._screen.binder.cycleIncognitaRightBtn
	local leftBtn = self._screen.binder.cycleIncognitaLeftBtn
	
	if programCount > 8 then
		self.scrollProgramsIdx = self.scrollProgramsIdx or 0
		if self.scrollProgramsIdx > 0 then
			leftBtn:setVisible(true)
		else
			self.scrollProgramsIdx = 0
			leftBtn:setVisible(false)
		end
		
		if self.scrollProgramsIdx >= programCount - 8 then
			rgtBtn:setVisible(false)
			self.scrollProgramsIdx = programCount - 8
		else
			rgtBtn:setVisible(true)
		end
		
		rgtBtn.onClick = util.makeDelegate( nil, self.scroll, self, 1 )
		leftBtn.onClick = util.makeDelegate( nil, self.scroll, self, -1 )
	else
		rgtBtn:setVisible(false)
		leftBtn:setVisible(false)
		self.scrollProgramsIdx = 0
	end

	return oldRefreshServerFull( self, ... )
end