local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local prop_templates = include( "sim/unitdefs/propdefs" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )

-----------------------------------------------------
-- Local functions

local sight_block = { ClassType = "sight_block" }

local function occludeSight( sim, targetCell )
    local segments = {}
    local los = sim:getLOS()
	local cell = targetCell
    for _, dir in ipairs( simdefs.DIR_SIDES ) do
        local dx, dy = simquery.getDeltaFromDirection( dir )
        local tocell = sim:getCell( cell.x + dx, cell.y + dy )
        if tocell then
            table.insert( segments, tocell )
            table.insert( segments, simquery.getReverseDirection( dir ) )
        end
        table.insert( segments, cell )
        table.insert( segments, dir )
    end

    sim:getLOS():insertSegments( unpack( segments ))

    return segments
end

-----------------------------------------------------
-- Interface functions

function sight_block:onWarp(sim, oldcell, cell)
    if self._segments then
        sim:getLOS():removeSegments( unpack( self._segments ))
        self._segments = nil
    end
    if cell then
        self._segments = occludeSight( sim, cell )    
    end
	
    for i, unit in pairs(sim:getAllUnits()) do
        sim:refreshUnitLOS( unit )
    end
    sim:dispatchEvent( simdefs.EV_EXIT_MODIFIED ) -- Update shadow map.
    sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } ) 
end

function sight_block:getSmokeCells()
	return {self:getSim():getCell(self:getLocation())}
end

function sight_block:getSmokeEdge()
	return {}
end
-----------------------------------------------------
-- Interface functions

local function createSightBlock( unitData, sim )
	return simunit.createUnit( unitData, sim, sight_block )
end

simfactory.register( createSightBlock )

return sight_block
