local inventory = include( "sim/inventory" )
local simactions = include( "sim/simactions" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )

function simactions.mainframeAction( sim, updates )
	if updates.action == "breakIce" then
		local player = sim:getCurrentPlayer()
        	local program = player:getEquippedProgram()
		if updates.unitID and program then
        		local unit = sim:getUnit(updates.unitID)
            		program:executeAbility( sim, unit )
		elseif program then
            		program:executeAbility( sim, updates.ability )
		end

		if program and program.cooldown and program.cooldown > 0 then
			player:equipProgram( sim, nil )			
		end
	elseif updates.action == "targetUnit" then
		local player = sim:getCurrentPlayer()
        	local program = player:getEquippedProgram()
		if updates.unitID and program then
        		local unit = sim:getUnit(updates.unitID)
            		program:executeAbility( sim, unit )
		elseif program then
            		program:executeAbility( sim, updates.ability )
		end

		if program and program.cooldown and program.cooldown> 0 then
			player:equipProgram( sim, nil )			
		end
	elseif updates.action == "use" then
		sim:getCurrentPlayer():glimpseUnit( sim, updates.unitID )
		sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units = {updates.unitID}} )	
    		local unit = sim:getUnit(updates.unitID)
		if type( unit[updates.fn] ) == "function" then
			unit[ updates.fn ]( unit, sim )
		end
	end
end

function simactions.lootItem( sim, unitID, itemID )
	local unit = sim:getUnit( unitID )
	local item = sim:getUnit( itemID )
	assert( unit:getPlayerOwner() == sim:getCurrentPlayer() )

	local cell = sim:getCell( item:getLocation() )
	local itemOwner
	if cell then
		if item:hasAbility( "carryable" ) then
			inventory.pickupItem( sim, unit, item )
			sim:emitSound( simdefs.SOUND_ITEM_PICKUP, cell.x, cell.y, unit )				
		elseif item:getTraits().cashOnHand then
			local credits = math.floor( simquery.calculateCashOnHand( sim, item ) * (1 + (unit:getTraits().stealBonus or 0)) )
			sim._resultTable.credits_gained.pickpocket = sim._resultTable.credits_gained.pickpocket and sim._resultTable.credits_gained.pickpocket + credits or credits
			unit:getPlayerOwner():addCredits( credits, sim, cell.x, cell.y )
			item:getTraits().cashOnHand = nil
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_CREDITS_PICKUP.path )
		elseif item:getTraits().credits then
			local credits = item:getTraits().credits
			sim._resultTable.credits_gained.safes = sim._resultTable.credits_gained.safes and sim._resultTable.credits_gained.safes + credits or credits
			unit:getPlayerOwner():addCredits( credits, sim, cell.x, cell.y )
			item:getTraits().credits = nil
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_CREDITS_PICKUP.path )
		elseif item:getTraits().PWROnHand then
			local PWR = simquery.calculatePWROnHand( sim, item )  
			unit:getPlayerOwner():addCPUs( PWR, sim, cell.x, cell.y)
			item:getTraits().PWROnHand = nil	
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/item_pickup" )		
		end
	else
		itemOwner = item:getUnitOwner()
		inventory.giveItem( itemOwner, unit, item)
		local itemDef = item:getUnitData()
		if itemDef.traits.showOnce then
            		local dialogParams =
            		{
                		STRINGS.UI.ITEM_ACQUIRED, itemDef.name, itemDef.desc, itemDef.profile_icon_100,
            		}
			sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { showOnce = itemDef.traits.showOnce, dialog = "programDialog", dialogParams = dialogParams } )
		end
	end
    	item:getTraits().anarchySpecialItem = nil
    	item:getTraits().largeSafeMapIntel = nil
	inventory.autoEquip( unit )
	if unit.recheckAllAiming then

		unit:recheckAllAiming()

	else

		simlog("CBF: unit did not define recheckAllAiming. Falling back to resetAllAiming. %s [%d]", unit:getName(), unit:getID())

		unit:resetAllAiming()

	end
	unit:checkOverload( sim )
	sim:dispatchEvent( simdefs.EV_ITEMS_PANEL )
	sim:triggerEvent( "FUNCLIB-ITEM-STOLEN", {unit=unit, targetUnit=item} )
end