local simquery = include("sim/simquery")
local simactions = include( "sim/simactions" )
local array = include( "modules/array" )
local abilitydefs = include	( "sim/abilitydefs" )
local simdefs = include("sim/simdefs")

local baseDoBuyItem = simactions.buyItem
function simactions.buyItem( sim, unitID, shopUnitID, itemIndex, discount, itemType, buyback )
	local handled_buy = false

	if sim:getParams().difficultyOptions.SITREP_program_storage then
		local unit = sim:getUnit( unitID ) or sim:getPlayerByID(unitID)
		local shopUnit = sim:getUnit( shopUnitID )
		local player = sim:getCurrentPlayer()
		assert( unit )
		assert( unit == player or unit:getPlayerOwner() == player )


		local item = nil
		if buyback then
			if itemType == "item" then
				item = shopUnit.buyback.items[itemIndex]
			elseif itemType == "weapon" then
				item = shopUnit.buyback.items[itemIndex]
			elseif itemType == "augment" then
				item = shopUnit.buyback.items[itemIndex]
			end
		else
			if itemType == "item" then
				item = shopUnit.items[itemIndex]
			elseif itemType == "weapon" then
				item = shopUnit.items[itemIndex]
			elseif itemType == "augment" then
				item = shopUnit.items[itemIndex]
			end
		end

		if item:getTraits().mainframe_program then
			if #player:getAbilities() >= simquery.getMaxPrograms( sim ) then
				handled_buy = true
				table.insert(sim._sitrepAgencyPrograms, item:getTraits().mainframe_program)
				table.insert(sim._resultTable.new_programs, item:getTraits().mainframe_program)
				sim:getStats():incStat( "programs_earned" )

				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/VoiceOver/Incognita/Pickups/NewProgram" )

				local mainframeDef = abilitydefs.lookupAbility(item:getTraits().mainframe_program)
				local dialogParams =
				{
					STRINGS.PROGRAMS.PURCHASED,
					item:getName(),
					string.format( STRINGS.SITREP.PROGRAMS.SEND_TO_AGENCY_DESC, mainframeDef.desc),
					mainframeDef.icon_100,
					color = {r=1,g=0,b=0,a=1}
				}
				sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )

				if buyback then
					if itemType == "item" then
						table.remove( shopUnit.buyback.items, itemIndex )
					elseif itemType == "weapon" then
						table.remove( shopUnit.buyback.weapons, itemIndex )
					elseif itemType == "augment" then
						table.remove( shopUnit.buyback.augments, itemIndex )
					end
				else
					if itemType == "item" then
						table.remove( shopUnit.items, itemIndex )
					elseif itemType == "weapon" then
						table.remove( shopUnit.weapons, itemIndex )
					elseif itemType == "augment" then
						table.remove( shopUnit.augments, itemIndex )
					end
				end

				sim:getStats():incStat( "items_earned" )

				-- Pay up.
				local credits = item:getUnitData().value * discount
				assert( player:getCredits() >= credits )
				player:addCredits( -credits )
				sim._resultTable.credits_lost.buying = sim._resultTable.credits_lost.buying and sim._resultTable.credits_lost.buying + credits or credits

				sim:getStats():sumStat( itemType .. "_purchases", -credits )

				sim:triggerEvent( simdefs.TRG_BUY_ITEM, {shopUnit = shopUnit, unit = unit, item = item} )

				sim:dispatchEvent( simdefs.EV_ITEMS_PANEL ) -- Triggers refresh.
			end
		end
	end

	if not handled_buy then
		baseDoBuyItem( sim, unitID, shopUnitID, itemIndex, discount, itemType, buyback )
	end
end

--same for tradeItem if its a research server
local baseDoTradeItem = simactions.tradeItem
function simactions.tradeItem( sim, unitID, shopUnitID, itemIndex, discount, itemType, buyback )
	local handled_buy = false

	if sim:getParams().difficultyOptions.SITREP_program_storage then
		local unit = sim:getUnit( unitID ) or sim:getPlayerByID(unitID)
		local shopUnit = sim:getUnit( shopUnitID )
		local player = sim:getCurrentPlayer()
		assert( unit )
		assert( unit == player or unit:getPlayerOwner() == player )


		local item = nil
		if buyback then
			if itemType == "item" then
				item = shopUnit.buyback.items[itemIndex]
			elseif itemType == "weapon" then
				item = shopUnit.buyback.items[itemIndex]
			elseif itemType == "augment" then
				item = shopUnit.buyback.items[itemIndex]
			end
		else
			if itemType == "item" then
				item = shopUnit.items[itemIndex]
			elseif itemType == "weapon" then
				item = shopUnit.items[itemIndex]
			elseif itemType == "augment" then
				item = shopUnit.items[itemIndex]
			end
		end

		if item:getTraits().mainframe_program then
			if #player:getAbilities() >= simquery.getMaxPrograms( sim ) then
				handled_buy = true
				table.insert(sim._sitrepAgencyPrograms, item:getTraits().mainframe_program)
				table.insert(sim._resultTable.new_programs, item:getTraits().mainframe_program)
				sim:getStats():incStat( "programs_earned" )

				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/VoiceOver/Incognita/Pickups/NewProgram" )

				local mainframeDef = abilitydefs.lookupAbility(item:getTraits().mainframe_program)
				local dialogParams =
				{
					STRINGS.PROGRAMS.INSTALLED,
					item:getName(),
					string.format( STRINGS.SITREP.PROGRAMS.SEND_TO_AGENCY_DESC, mainframeDef.desc),
					mainframeDef.icon_100,
					color = {r=1,g=0,b=0,a=1}
				}
				sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )

				if buyback then
					if itemType == "item" then
						table.remove( shopUnit.buyback.items, itemIndex )
					elseif itemType == "weapon" then
						table.remove( shopUnit.buyback.weapons, itemIndex )
					elseif itemType == "augment" then
						table.remove( shopUnit.buyback.augments, itemIndex )
					end
				else
					if itemType == "item" then
						table.remove( shopUnit.items, itemIndex )
					elseif itemType == "weapon" then
						table.remove( shopUnit.weapons, itemIndex )
					elseif itemType == "augment" then
						table.remove( shopUnit.augments, itemIndex )
					end
				end

				sim:getStats():incStat( "items_earned" )

				sim:triggerEvent( simdefs.TRG_BUY_ITEM, {shopUnit = shopUnit, unit = unit, item = item} )

				sim:dispatchEvent( simdefs.EV_ITEMS_PANEL ) -- Triggers refresh.
			end
		end
	end

	if not handled_buy then
		baseDoTradeItem( sim, unitID, shopUnitID, itemIndex, discount, itemType, buyback )
	end
end

--not needed anymore from v1.0.4 on
--local baseDoSellAbility = simactions.sellAbility
--function simactions.sellAbility( sim, index )
--	if sim:getParams().difficultyOptions.SITREP_program_storage then
--		local player = sim:getCurrentPlayer()
--		local ability = player:getAbilities()[index]
--		assert(ability)
--		
--		for i, mainframe_ability in ipairs( sim._sitrepAgencyPrograms ) do
--			if mainframe_ability == ability:getID() then
--				table.remove( sim._sitrepAgencyPrograms, i )
--				break
--			end
--		end
--	end
--	
--	baseDoSellAbility( sim, index )
--end

local baseDoMainframeAction = simactions.mainframeAction
function simactions.mainframeAction( sim, updates )
	local player = sim:getPC()
	local program = player:getEquippedProgram()
	
	baseDoMainframeAction( sim, updates )
	
	if program and ( updates.action == "breakIce" or updates.action == "targetUnit" ) then
		sim:triggerEvent( "TRG_SITREP_PROGRAM_USED", {program = program:getID()} )
	end
end
local baseDoAbilityAction = simactions.abilityAction
function simactions.abilityAction ( sim, ownerID, userID, abilityIdx, ... )
	local abilityOwner = sim:getUnit( ownerID ) or array.findIf( sim:getPlayers(), function( p ) return p:getID() == ownerID end )
	local ability = abilityOwner:getAbilities()[ abilityIdx ]
	baseDoAbilityAction( sim, ownerID, userID, abilityIdx, ... )
	if ability:getDef().program and not ability:getDef().equip_program then
		sim:triggerEvent( "TRG_SITREP_PROGRAM_USED", {program = ability:getID()} )
	end
end
