local cdefs = include( "client_defs" )
local serverdefs = include("modules/serverdefs")
local util = include("modules/util")
local _simdefs = include("sim/simdefs")

local function FindIndexOf(val, array)
    for k,v in pairs(array) do 
        if v == val then return k end
    end
end

local CORP_NAMES = util.tcopy(serverdefs.CORP_NAMES)
local ESCAPE_MISSION_TAGS = { "vault", "ceo_office", "security", "executive_terminals", "server_farm", "nanofab", "cyberlab", "detention_centre" }
local NO_CFO_MISSION_TAGS = { "vault", "security", "executive_terminals", "server_farm", "nanofab", "cyberlab", "detention_centre" }
local DEFAULT_MISSION_TAGS = util.tmerge( { "2max" }, CORP_NAMES, ESCAPE_MISSION_TAGS )
local DEFAULT_MISSION_TAGS_FIRST = util.tmerge( { "2max" }, CORP_NAMES, NO_CFO_MISSION_TAGS )

-- These simdefs are set once and then never reset
local init_simdefs = {

	TRG_OBJ_COMPLETE = "TRG_OBJ_COMPLETE",
	
	CUSTOM_OPTIONS = {
  
		------------------------------------
		--BROAD STROKE DIFFICULTY SETTINGS--
		------------------------------------
		["REWINDS"] = {
			name = STRINGS.UI.DIFF_OPTION_REWINDS,
			tip = STRINGS.UI.DIFF_OPTION_REWINDS_TIP,
			ironManCheck = true,
			values = { 0,1,3,5,10,99 },
			strings = STRINGS.UI.GENOPTIONS.REWINDS,
			apply = function( self, options, index )
				options.rewindsLeft = self.values[index]
			end,
			retrieve = function( self, options )
				return self.strings[FindIndexOf(options.rewindsLeft, self.values)]
			end,
		},

		["LEVEL RETRIES"] = {
			name = STRINGS.UI.DIFF_OPTION_LEVEL_RETRIES,
			tip = STRINGS.UI.DIFF_OPTION_LEVEL_RETRIES_TIP,
			check = true,
			apply = function( self, options, value )
				options.savescumming = value
			end,
			retrieve = function( self, options )
				return options.savescumming
			end
		},
		
		["AUTO-INCREMENT ALARM"] = {
			name = STRINGS.UI.DIFF_OPTION_AUTOALARM,
			tip = STRINGS.UI.DIFF_OPTION_AUTOALARM_TIP,
			check = true,
			apply = function( self, options, value )
				options.autoAlarm = value
			end,
			retrieve = function( self, options )
				return options.autoAlarm
			end
		},

		------------------------------------
		-----------TWEAKY SETTINGS----------
		------------------------------------
		["CAMPAIGN HOURS"] = {
			name = STRINGS.UI.DIFF_OPTION_HOURS,
			tip = STRINGS.UI.DIFF_OPTION_HOURS_TIP,
			strings = STRINGS.UI.GENOPTIONS.HOURS,
			values = { 24,48,72,120, math.huge},
			apply = function( self, options, index )
				options.maxHours = self.values[index]
			end,
			retrieve = function( self, options )
				return self.strings[FindIndexOf(options.maxHours, self.values)]
			end,
		},    

		["MISSION START PWR"] = {
			name = STRINGS.UI.DIFF_OPTION_STARTPWR,
			tip = STRINGS.UI.DIFF_OPTION_STARTPWR_TIP,
			values = { 0, 5, 10, 20 },
			apply = function( self, options, idx )
				options.startingPower = self.values[idx]
			end,
			retrieve = function( self, options )
				return tostring(options.startingPower)
			end
		},   
	  
		["ALARM STAGES"] = {
			name = STRINGS.UI.DIFF_OPTION_ALARMTYPE,
			tip = STRINGS.UI.DIFF_OPTION_ALARMTYPE_TIP,
			strings = STRINGS.UI.GENOPTIONS.ALARMTYPE,
			values = { "EASY", "NORMAL" },
			apply = function( self, options, idx )
				options.alarmTypes = self.values[idx]
			end,
			retrieve = function( self, options )
				return self.strings[FindIndexOf(options.alarmTypes, self.values)]
			end
		}, 

		["CREDIT MULTIPLIER"] = {
			name = STRINGS.UI.DIFF_OPTION_CREDITS,
			tip = STRINGS.UI.DIFF_OPTION_CREDITS_TIP,
			values = { 0.5, 0.75, 1, 1.25, 1.5, 2 },
			apply = function( self, options, idx )
				options.creditMultiplier = self.values[idx]
			end,
			retrieve = function( self, options )
				return tostring(options.creditMultiplier)
			end
		},

		["GUARDS PER LEVEL"] = {
			name = STRINGS.UI.DIFF_OPTION_GUARDSPAWNS,
			tip = STRINGS.UI.DIFF_OPTION_GUARDSPAWNS_TIP,
			values = { "LESS", "NORMAL", "MORE" },
			strings = STRINGS.UI.GENOPTIONS.GUARDSPAWNS,
			apply = function( self, options, idx )
				options.spawnTable = self.values[idx]
			end,
			retrieve = function( self, options )
				return self.strings[FindIndexOf(options.spawnTable, self.values)]
			end
		}, 
		
		["SAFE GUARD PATROLS"] = {
			name = STRINGS.UI.DIFF_OPTION_BEGINNER_PATROLS,
			tip = STRINGS.UI.DIFF_OPTION_BEGINNER_PATROLS_TIP,
			check = true,
			apply = function( self, options, value )
				options.beginnerPatrols = value
			end,
			retrieve = function( self, options )
				return options.beginnerPatrols
			end
		}, 
		
		["ADDITIONAL KO TIME"] = {
			name = STRINGS.UI.DIFF_OPTION_KOTIME,
			tip = STRINGS.UI.DIFF_OPTION_KOTIME_TIP,
			strings = STRINGS.UI.GENOPTIONS.KOTIME,
			values = {99,4,2,1,0,-1},
			apply = function( self, options, idx )
				options.koDuration = self.values[idx]
			end,
			retrieve = function( self, options )

				return self.strings[FindIndexOf(options.koDuration, self.values)]
			end,
		},

		["ALARM WHEN GUARDS KO"] = {
			name = STRINGS.UI.DIFF_OPTION_ALARMKO,
			tip = STRINGS.UI.DIFF_OPTION_ALARMKO_TIP,
			check = true,
			apply = function( self, options, value )
				options.alarmRaisedOnKO = value
			end,
			retrieve = function( self, options )
				return options.alarmRaisedOnKO
			end
		}, 
		
		["SHOW DANGER ZONES"] = {
			name = STRINGS.UI.DIFF_OPTION_DANGERZONE,
			tip = STRINGS.UI.DIFF_OPTION_DANGERZONE_TIP,
			check = true,
			apply = function( self, options, value )
				options.dangerZones = value
			end,
			retrieve = function( self, options )
				return options.dangerZones
			end
		},    

		["ALLOW MELEE WHILE SEEN"] = {
			name = STRINGS.UI.DIFF_OPTION_MELEEFRONT,
			tip = STRINGS.UI.DIFF_OPTION_MELEEFRONT_TIP,
			check = true,
			apply = function( self, options, value )
				options.meleeFromFront = value
			end,
			retrieve = function( self, options )
				return options.meleeFromFront
			end
		},

		["INCREASED FINAL RESISTANCE"] = {
			name = STRINGS.UI.DIFF_OPTION_COUNTERMEASURES,
			tip = STRINGS.UI.DIFF_OPTION_COUNTERMEASURES_TIP,
			check = true,
			apply = function( self, options, value )
				options.countermeasuresFinal = value
			end,
			retrieve = function( self, options )
				return options.countermeasuresFinal
			end
		},        

		["ALARM MULTIPLIER"] = {
			name = STRINGS.UI.DIFF_OPTION_ALARM,
			tip = STRINGS.UI.DIFF_OPTION_ALARM_TIP,
			values = { 0, 1, 2, 3, 4, 5 },
			apply = function( self, options, idx )
				options.alarmMultiplier =  self.values[idx]
			end,
			retrieve = function( self, options )
				return tostring(options.alarmMultiplier)
			end
		},

		["STARTING MONEY"] = {
			name = STRINGS.UI.DIFF_OPTION_MONEY,
			tip = STRINGS.UI.DIFF_OPTION_MONEY_TIP,
			values = { 0, 500, 1000, 2000, 10000 },
			apply = function( self, options, idx )
				options.startingCredits = self.values[idx]
			end,
			retrieve = function( self, options )
				return tostring(options.startingCredits)
			end
		},    
		
		["CONSOLES PER LEVEL"] = {
			name = STRINGS.UI.DIFF_OPTION_CONSOLES,
			tip = STRINGS.UI.DIFF_OPTION_CONSOLES_TIP,
			values = { 0, 3, 5, 7, 15 },
			strings = STRINGS.UI.GENOPTIONS.PROPSPAWNS,
			apply = function( self, options, idx )
				options.consolesPerLevel = self.values[idx or -1] or 5
				options.powerPerLevel = tonumber(options.consolesPerLevel) * 2
			end,
			retrieve = function( self, options )
				return self.strings[ FindIndexOf(options.consolesPerLevel, self.values)]
			end
		},
	   
		["SAFES PER LEVEL"] = {
			name = STRINGS.UI.DIFF_OPTION_SAFES,
			tip = STRINGS.UI.DIFF_OPTION_SAFES_TIP,
			values = { 0, 3, 5, 7, 15 },
			strings = STRINGS.UI.GENOPTIONS.PROPSPAWNS,
			apply = function( self, options, idx )
				options.safesPerLevel = self.values[idx]
			end,
			retrieve = function( self, options )
				return self.strings[ FindIndexOf(options.safesPerLevel, self.values)]
			end
		},

		["ROOMS IN LEVEL"] = {
			name = STRINGS.UI.DIFF_OPTION_ROOMS,
			tip = STRINGS.UI.DIFF_OPTION_ROOMS_TIP,
			values = { 8, 10, 12, 14, 16, 18, 20 },
			apply = function( self, options, idx )
				options.roomCount = self.values[idx]
			end,
			retrieve = function( self, options )
				return tostring(options.roomCount)
			end
		},
	   
		["DAEMON QUANTITY"] = {
			name = STRINGS.UI.DIFF_OPTION_DAEMONS,
			tip = STRINGS.UI.DIFF_OPTION_DAEMONS_TIP,
			values = { "LESS", "NORMAL", "MORE" },
			strings = STRINGS.UI.GENOPTIONS.DAEMONS,
			apply = function( self, options, idx )
				options.daemonQuantity = self.values[idx]
			end,
			retrieve = function( self, options )
				return self.strings[FindIndexOf(options.daemonQuantity, self.values)]
			end
		},

		------------------------------------
		-----------OTHER SETTINGS-----------
		------------------------------------
		["TIME ATTACK MODE"] = {
			name = STRINGS.UI.DIFF_OPTION_TIMEATTACK,
			tip = STRINGS.UI.DIFF_OPTION_TIMEATTACK_TIP,
			strings = STRINGS.UI.GENOPTIONS.TIMEATTACK,
			values = { 0, 30*cdefs.SECONDS, 60*cdefs.SECONDS, 2*60*cdefs.SECONDS, 5*60*cdefs.SECONDS },
			apply = function( self, options, index )
				options.timeAttack = self.values[index]
			end,
			retrieve = function( self, options )
				return self.strings[FindIndexOf(options.timeAttack, self.values)]
			end
		},    
	},
	
	CUSTOM_OPTIONS_ORDER =
	{
		"REWINDS",
		"LEVEL RETRIES",
		"AUTO-INCREMENT ALARM",
		"CAMPAIGN HOURS",
		"MISSION START PWR",
		"ALARM STAGES",
		"CREDIT MULTIPLIER",
		"GUARDS PER LEVEL",
		"SAFE GUARD PATROLS",
		"ADDITIONAL KO TIME",
		"ALARM WHEN GUARDS KO",
		"SHOW DANGER ZONES",
		"ALLOW MELEE WHILE SEEN",
		"INCREASED FINAL RESISTANCE",
		"ALARM MULTIPLIER",
		"STARTING MONEY",
		"CONSOLES PER LEVEL",
		"SAFES PER LEVEL",
		"ROOMS IN LEVEL",
		"DAEMON QUANTITY",
		"TIME ATTACK MODE",  
	},
	
	STATE_GENERATION_OPTIONS = {
		DEFAULT_DIFF = _simdefs.NORMAL_DIFFICULTY,
		ACTIVE_TXT = { 61/255,81/255,83/255,1 },
		INACTIVE_TXT = { 1,1,1,1 },

		ACTIVE_BG = { 244/255, 255/255, 120/255,1 },
		INACTIVE_BG = { 78/255, 136/255, 136/255,1 },

		MENU_UNSELECTED = { 78/255, 136/255, 136/255 },
		MENU_SELECTED = { 122/255, 215/255, 215/255 },
		
		
		CHARACTER_IMAGES = 
		{
			{ png="spec_ops" },
			{ png="internationale", unlock="sharpshooter_1" }, --lock them to the first unlocks
			{ png="character", unlock="sharpshooter_1" }, --lock them to the first unlocks
			{ png="shalem", unlock="sharpshooter_1" },
			{ png="banks", unlock="stealth_2" },    
			{ png="nika", unlock="sharpshooter_2" },    
			{ png="tony", unlock="engineer_1" },    
			{ png="sharp", unlock="cyborg_1" },
			{ png="prism", unlock="disguise_1" },
			{ png="central", unlock="central_pc" },
			{ png="monst3r", unlock="monst3r_pc" },
		},

		DIFFICULTY_MENU = {	
			{diff=_simdefs.NORMAL_DIFFICULTY,name=util.toupper(STRINGS.UI.NORMAL_DIFFICULTY), desc=STRINGS.UI.NORMAL_DESC, short_desc=STRINGS.UI.NORMAL_SHORT_DESC},
			{diff=_simdefs.EXPERIENCED_DIFFICULTY,name=util.toupper(STRINGS.UI.EXPERIENCED_DIFFICULTY), desc=STRINGS.UI.EXPERIENCED_DESC, short_desc=STRINGS.UI.EXPERIENCED_SHORT_DESC},
			{diff=_simdefs.HARD_DIFFICULTY,name=util.toupper(STRINGS.UI.HARD_DIFFICULTY), desc=STRINGS.UI.HARD_DESC, short_desc=STRINGS.UI.HARD_SHORT_DESC},
			{diff=_simdefs.VERY_HARD_DIFFICULTY,name=util.toupper(STRINGS.UI.VERY_HARD_DIFFICULTY), desc=STRINGS.UI.VERY_HARD_DESC, short_desc=STRINGS.UI.VERY_HARD_SHORT_DESC},
			{diff=_simdefs.ENDLESS_DIFFICULTY,name=util.toupper(STRINGS.UI.ENDLESS_DIFFICULTY), desc=STRINGS.UI.ENDLESS_DESC, short_desc=STRINGS.UI.ENDLESS_SHORT_DESC},		
			{diff=_simdefs.ENDLESS_PLUS_DIFFICULTY,name=util.toupper(STRINGS.UI.ENDLESS_PLUS_DIFFICULTY), desc=STRINGS.UI.ENDLESS_PLUS_DESC, short_desc=STRINGS.UI.ENDLESS_PLUS_SHORT_DESC},     
			{diff=_simdefs.TIME_ATTACK_DIFFICULTY,name=util.toupper(STRINGS.UI.TIME_ATTACK_DIFFICULTY), desc=STRINGS.UI.TIME_ATTACK_DESC, short_desc=STRINGS.UI.TIME_ATTACK_SHORT_DESC},     
			{diff=_simdefs.CUSTOM_DIFFICULTY,name=util.toupper(STRINGS.UI.CUSTOM_DIFFICULTY), desc=STRINGS.UI.CUSTOM_DESC, short_desc=STRINGS.UI.CUSTOM_SHORT_DESC},
		},
	},
	
	-- LEGACY: Use one of the following functions instead
	
	-- modApi:addNewUIScreen( filename, file )
	-- modApi:insertUIElement( filename, path, element, index = 0 )
	-- modApi:insertUIElementsAtPath( filename, path, elements, index = 0 )
	-- modApi:insertUIElements( elements )
	-- modApi:modifyUIElement( filename, path, modifications )
	-- modApi:modifyUIElements( elements ) 
	
	SCREEN_CUSTOMS = { },
}

-- These simdefs are reset before each time mods are loaded
local simdefs = {
	ENABLE_FULLPARTY = false,
	ALLOW_LOADOUT_RESCUABLES = false,
	MAX_ITEMS = 2,
	LOOT_TABLES =
	{
		lab_safe =
		{
			{ "item_icebreaker", 2 },
			{ "item_portabledrive", 1 },
			{ "item_paralyzer", 2 },
			{ "item_adrenaline", 3 },
			{ "item_cloakingrig_1", 1 },		
			{ "item_clip", 2 },
			{ "item_stim", 3 },
			{ "item_emp_pack", 2 },
			{ 50, 3 },
			{ 150, 3 },
			{ 400, 3 },
			{ nil, 40 }, -- cash out the rest of the value in credits
		},  	
	},

	ICE_LOWS = 
	{
		NORMAL = {
			[1] = 1, 
			[2] = 1, 
			[3] = 2, 
			[4] = 2,
			[5] = 2, 
			[6] = 2, 
			[7] = 3,
			[8] = 3, 
			[9] = 3, 
			[10] = 4,  

			[11] = 4, 
			[12] = 4, 
			[13] = 4, 
			[14] = 4,
			[15] = 4, 
			[16] = 5, 
			[17] = 5,
			[18] = 5, 
			[19] = 5, 
			[20] = 5,  	
		},
	},

	ICE_HIGHS = 
	{
		NORMAL = {
			[1] = 2,
			[2] = 2, 
			[3] = 3, 
			[4] = 4,
			[5] = 4, 
			[6] = 5, 
			[7] = 5, 
			[8] = 6, 
			[9] = 6, 
			[10] = 7, 

			[11] = 7,
			[12] = 7, 
			[13] = 7, 
			[14] = 8,
			[15] = 8, 
			[16] = 8, 
			[17] = 8, 
			[18] = 9, 
			[19] = 9, 
			[20] = 10, 	
		},
	},
	
	NUM_BOOSTED_ICE = 
	{
		NORMAL = {
			[1] = 1, 
			[2] = 2, 
			[3] = 3, 
			[4] = 3,
			[5] = 3,
			[6] = 4, 
			[7] = 4, 
			[8] = 5, 
			[9] = 5, 
			[10] = 6,

			[11] = 6, 
			[12] = 6, 
			[13] = 7, 
			[14] = 7,
			[15] = 7,
			[16] = 8, 
			[17] = 8, 
			[18] = 8, 
			[19] = 9, 
			[20] = 9,	
		}
	},

	MAGNETIC_REINFORCEMENTS_CHANCE = 
	{
		NORMAL = {
			[1] = 0, 
			[2] = 0, 
			[3] = 0, 
			[4] = 0,
			[5] = 0,
			[6] = 0.5, 
			[7] = 0.5, 
			[8] = 0.75, 
			[9] = 0.75, 
			[10] = 1,
			
			[11] = 1, 
			[17] = 1, 
			[12] = 1, 
			[13] = 1, 
			[14] = 1,
			[15] = 1,
			[16] = 1, 
			[18] = 1, 
			[19] = 1, 
			[20] = 1,	
		},
	},
	
	map_colours_normal =
	{
		asia = {14/255, 54/255, 79/255, 1},
		europe = {22/255, 56/255, 56/255, 1},
		sa = {42/255, 61/255, 30/255, 1},
		na = {28/255, 36/255, 64/255, 1},
		omni = {255/255,175/255,36/255, 1},	
		--atlantis = {68/255,59/255,13/255,1},
	},
	map_colours_highlight =
	{
		asia = {89/255,138/255,221/255, 1},
		europe = {180/255,180/255,180/255, 1},
		sa = {200/255, 125/255, 13/255, 1},
		na = {187/255,82/255,200/255, 1},
		omni = {255/255,175/255,36/255, 1},
		--atlantis = {155/255,209/255,65/255,1},
	},

	map_colours_unhighlight =
	{
		asia = {34/255,57/255,56/255,.5},
		europe = {34/255,57/255,56/255,.5},
		sa = {34/255,57/255,56/255,.5},
		na = {34/255,57/255,56/255,.5},
		omni = {255/255,175/255,36/255, 1},
		--atlantis = {34/255,57/255,56/255,.5},	
	},
	
	map_territories = {},
	
	TIER2_IDX = 0.2,
	TIER2_VALUE = 375,
	TIER1_VALUE = 150,
	
	NUM_MISSIONS_TO_SPAWN = 4,
	HOURS_BEFORE_SPAWN = 24,
	MAX_DIFFICULTY = 4,
	
	map_offset = {x = 0, y = 0},
	sphericalEarth = false,
	
	--CORP_NAMES = CORP_NAMES,
	ESCAPE_MISSION_TAGS = ESCAPE_MISSION_TAGS,
	NO_CFO_MISSION_TAGS = NO_CFO_MISSION_TAGS,
	DEFAULT_MISSION_TAGS = DEFAULT_MISSION_TAGS,
	DEFAULT_MISSION_TAGS_FIRST = DEFAULT_MISSION_TAGS_FIRST,
}

for k, v in pairs(init_simdefs) do
	rawset(_simdefs,k,v)
end

local function resetSimDefs()
	local loadSimdefs = util.tcopy(simdefs)
	for k, v in pairs(loadSimdefs) do
		if rawget(_simdefs,k) and type(_simdefs[k]) == "table" and type(v) == "table" then
			util.tclear(_simdefs[k])
			util.tmerge(_simdefs[k],v)
		else
			rawset(_simdefs,k,v)
		end
	end
end

resetSimDefs()

return resetSimDefs