local cdefs = include( "client_defs" )
local util = include( "modules/util" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )

local brokenDoor =
{
	door = true,
	closed = true,
        locked = nil,
        openSound = simdefs.SOUND_DOOR_OPEN,
        closeSound = simdefs.SOUND_DOOR_CLOSE,
        breakSound = simdefs.SOUND_DOOR_BREAK,
        keybits =  simdefs.DOOR_KEYS.OFFICE,
	wallUVs = { unlocked = cdefs.WALL_DOOR_UNLOCKED, locked = cdefs.WALL_DOOR_LOCKED, broken = cdefs.WALL_DOOR_BROKEN },
}

local tileDir =
{
	[0] = { 1, 0 },
	[2] = { 0, 1 },
	[4] = { -1, 0},
	[6] = { 0, -1},
}

local emp = { ClassType = "simemppack" }

function emp:detonate( sim )
	local x0,y0 = self:getLocation()

	if self:getTraits().flash_pack then
		local sim, player = self:getSim(), self:getPlayerOwner()

		sim:startTrackerQueue(true)				
		sim:startDaemonQueue()			
		sim:dispatchEvent( simdefs.EV_KO_GROUP, true )
		local cells = self:getExplodeCells()

		sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, units = nil, range = self:getTraits().range} )

		for i, cell in ipairs(cells) do
			for i, cellUnit in ipairs( cell.units ) do
				if self:getTraits().baseDamage and simquery.isEnemyAgent( player, cellUnit) and not cellUnit:getTraits().isDrone then
					if self:getTraits().canSleep then

						local damage = self:getTraits().baseDamage
						damage = cellUnit:processKOresist( damage )
				   			
						cellUnit:setKO(sim, damage)
					else
						sim:damageUnit(cellUnit, self:getTraits().baseDamage)
					end
				end
			end
		end

	    	sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
		sim:startTrackerQueue(false)				
		sim:processDaemonQueue()	
	elseif self:getTraits().scan_pack then
		local player = sim:getPC()
            	local cells = simquery.rasterCircle( sim, x0, y0, self:getTraits().range )
		for i, x, y in util.xypairs( cells ) do
                	local cell = sim:getCell( x, y )
			if cell then
				player:glimpseCell( sim, cell )
			end
		end
		local units = self:getTargets( x0, y0 )
	    	for i, unit in ipairs(units) do
			player:glimpseUnit( sim, unit:getID() )
	    	end
           	sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = nil, range = self:getTraits().range } )
           	sim:dispatchEvent( simdefs.EV_SCRIPT_ENTER_MAINFRAME )
	elseif self:getTraits().breaching_charges then
		local cell = sim:getCell( x0, y0 )

		for i=0, 6, 2 do
			if simquery.checkIsDoor( sim, cell, i ) and (cell.exits[i].keybits < simdefs.DOOR_KEYS.ELEVATOR or cell.exits[i].keybits == simdefs.DOOR_KEYS.VAULT or cell.exits[i].keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT) then
				if cell.exits[i].locked and (cell.exits[i].keybits == simdefs.DOOR_KEYS.VAULT or cell.exits[i].keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT or cell.exits[i].keybits == simdefs.DOOR_KEYS.SECURITY ) then
					sim:modifyExit( cell, i, simdefs.EXITOP_UNLOCK, self, false )
				end
				sim:modifyExit(cell, i, simdefs.EXITOP_BREAK_DOOR, self, false)
			elseif simquery.checkIsWall( sim, cell, i ) and sim:getCell( x0 + tileDir[i][1], y0 + tileDir[i][2] ) then
				local targetCell = sim:getCell( x0 + tileDir[i][1], y0 + tileDir[i][2] )

				if not targetCell.isSolid then
					cell.sides = util.tdupe(cell.sides)
					targetCell.sides = util.tdupe(targetCell.sides)

					cell.exits = util.tdupe(cell.exits)
					targetCell.exits = util.tdupe(targetCell.exits)

					cell.sides[i] = util.tcopy(brokenDoor)
					targetCell.sides[simquery.getReverseDirection(i)] = util.tcopy(brokenDoor)
					cell.exits[i] = util.tcopy(brokenDoor)
					targetCell.exits[simquery.getReverseDirection(i)] = util.tcopy(brokenDoor)
					targetCell.exits[simquery.getReverseDirection(i)].cell = cell
					cell.exits[i].cell = targetCell
					sim:dispatchEvent( "EV_WALL_DESTROYED", { cell = cell, rCell = targetCell, dir = i, rdir = simquery.getReverseDirection(i) } )
					sim:modifyExit(cell, i, simdefs.EXITOP_BREAK_DOOR, self, false)
				end
			end
		end
		local explodeCells = self:getExplodeCells(x0,y0)
		sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, units = nil, range = self:getTraits().soundRange } )

		sim:startTrackerQueue(true)
		sim:startDaemonQueue()
		local targetUnits = {}
		for j, explodeCell in pairs(explodeCells) do
			for k, cellUnit in ipairs( explodeCell.units ) do
				if cellUnit:hasTrait("isAgent") or cellUnit:hasTrait("isGuard") then
					table.insert(targetUnits, cellUnit)
				end
			end
		end
		for i=#targetUnits, 1, -1 do
			sim:damageUnit(targetUnits[i], 1)
		end
		sim:startTrackerQueue(false)				
		sim:processDaemonQueue()
	else
		local units = self:getTargets( x0, y0 )
		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = units, range = self:getTraits().range } )

		sim:startTrackerQueue(true)
		sim:startDaemonQueue()	
		for i, unit in ipairs(units) do
	        	unit:processEMP( self:getTraits().emp_duration, true, false, self:getTraits().ignoreMagReinf )
	    	end
		sim:startTrackerQueue(false)				
		sim:processDaemonQueue()
	end

	if not self:hasTrait("soundRange") then
		sim:emitSound( simdefs.SOUND_SMALL, x0, y0, nil )
	else
		local soundRange = { path = nil, range = self:getTraits().soundRange }
		sim:emitSound( soundRange, x0, y0, self )
	end

	if self:getTraits().breaching_charges then
		sim:trackerAdvance(5, STRINGS.FUNCLIB.UI.ALARM_BREACHING_CHARGE )
	end
			
	sim:processReactions(self)

	sim:warpUnit( self, nil )
	sim:despawnUnit( self )
end

function emp:onWarp(sim)
	if not self:getTraits().trigger_mainframe then
		if self:getLocation() then
			sim:addTrigger( simdefs.TRG_END_TURN, self )
		else
			sim:removeTrigger( simdefs.TRG_END_TURN, self )		
		end
	end
end 

function emp:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_END_TURN then
		if self:getTraits().primed then
			self:detonate( sim )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self} ) 		
		end
		
	end
end

function emp:getTargets( x0, y0 )
	local cells = {}

	if self:getTraits().flash_pack or self:getTraits().breaching_charges then
		local currentCell = self:getSim():getCell( x0, y0 )
		cells = {currentCell}
		if self:getTraits().range then
			local fillCells = simquery.fillCircle( self:getSim(), x0, y0, self:getTraits().range, 0)

			for i, cell in ipairs(fillCells) do
				if cell ~= currentCell then
					local raycastX, raycastY = self._sim:getLOS():raycast(x0, y0, cell.x, cell.y)
					if raycastX == cell.x and raycastY == cell.y then
						table.insert(cells, cell)
					end
				end
			end
		end	   
	else
		cells = simquery.rasterCircle( self._sim, x0, y0, self:getTraits().range )
	end

	local units = {}
	for i, x, y in util.xypairs( cells ) do
        	local cell = self._sim:getCell( x, y )
        	if cell then
            		for _, cellUnit in ipairs(cell.units) do
            			local player = self:getPlayerOwner()
                		if cellUnit ~= self and (cellUnit:getTraits().mainframe_status or cellUnit:getTraits().heartMonitor) and (not self:getTraits().flash_pack or (simquery.isEnemyAgent( player, cellUnit) and not cellUnit:getTraits().isDrone)) then
                    			table.insert( units, cellUnit )
                		end
            		end
		end
	end

	return units
end

-- get cells for FLASH PACK
function emp:getExplodeCells(x0,y0)
	assert(self:getTraits().range)

	if not x0 and not y0 then
    		x0, y0 = self:getLocation()
    	end

	local currentCell = self:getSim():getCell( x0, y0 )
	local cells = {currentCell}
	if self:getTraits().range then
		local coords = simquery.rasterCircle( self._sim, x0, y0, self:getTraits().range )

		for i=1,#coords-1,2 do
			local cell = self._sim:getCell(coords[i],coords[i+1])
			if cell then
			local raycastX, raycastY = self._sim:getLOS():raycast(x0, y0, cell.x, cell.y)			
				if raycastX == cell.x and raycastY == cell.y then
					table.insert(cells, cell)
				end			
			end
		end
	end
    	return cells
end

-- for FLASH PACK
function emp:toggle( sim )
	self:detonate( sim )
end

-----------------------------------------------------
-- Interface functions

local function createEMPPack( unitData, sim )
	return simunit.createUnit( unitData, sim, emp )
end

simfactory.register( createEMPPack )

return
{
	createEMPPack = createEMPPack,
}

