local cdefs = include("client_defs")
local simdefs = include( "sim/simdefs" )
local simengine = include( "sim/engine" )
local simquery = include( "sim/simquery" )
local util = include( "client_util" )

local oldRefreshUnitLOS = simengine.refreshUnitLOS
local oldMoveUnit = simengine.moveUnit
local oldSpawnUnit = simengine.spawnUnit
local oldInit = simengine.init

function simengine:tryShootAt( sourceUnit, targetUnit, dmgt0, equipped )
	local dmgt = simquery.calculateShotSuccess( self, sourceUnit, targetUnit, equipped )
	sourceUnit:setInvisible( false )
	sourceUnit:setDisguise(false,nil,true)

	local x0, y0 = sourceUnit:getLocation()
	local x1,y1 = targetUnit:getLocation()
	local evData = { unitID = sourceUnit:getID(), x0 = x0, y0 = y0, x1=x1, y1=y1, dmgt = dmgt0 } 	
	if x0 == x1 and y0 == y1 then
		evData.facing = targetUnit:getFacing()
	end

	self:dispatchEvent( simdefs.EV_UNIT_SHOT, evData )
	if equipped:getTraits().canTag then
		targetUnit:setTagged()		
	end
	if equipped:getTraits().canEmp then
		if targetUnit:getTraits().heartMonitor and targetUnit:getTraits().heartMonitor == "enabled" then
			targetUnit:getTraits().heartMonitor = "disabled"
		end
	end
	if not dmgt.armorBlocked then 
		self:hitUnit( sourceUnit, targetUnit, dmgt )
		if equipped:getTraits().canEmp then
			targetUnit:processEMP( equipped:getTraits().modeEmpdamage or 1, false, false, equipped:getTraits().ignoreMagReinf )
		end
		if equipped:getTraits().canSleep then
			targetUnit:getTraits().W93_rangedKO = true
			if sourceUnit:hasTrait("rangedKO_warpTarget") then
				targetUnit:getTraits().W93_rangedKO_warp = sourceUnit
			end
		end
	end
	if targetUnit:isValid() and targetUnit:getPlayerOwner() then
		if not self:isVersion("0.17.9") or not equipped:getTraits().doNotInterruptMove then			
			targetUnit:interruptMove( self )
			if equipped:getTraits().onHitMpDrain and targetUnit:getTraits().mp then
				targetUnit:useMP(targetUnit:getTraits().mp, self)
			end
		end
		self:triggerEvent( simdefs.TRG_UNIT_SHOT, { targetUnit = targetUnit, sourceUnit = sourceUnit } )
	end
	if equipped:getTraits().ammo and sourceUnit:countAugments( "W93_augment_subdermal_dart_pocket" ) > 0 and equipped:getTraits().canSleep then
		local number, aug = sourceUnit:countAugments( "W93_augment_subdermal_dart_pocket" )
		if aug[1]:getTraits().ammo > 0 then
			aug[1]:getTraits().ammo = aug[1]:getTraits().ammo - 1
			equipped:getTraits().ammo = equipped:getTraits().ammo + 1
		end
	end

	if equipped:getTraits().soundRange or not sourceUnit:getTraits().silencer then
		local soundRange = nil
		if equipped:getTraits().soundRange then
			soundRange = { path = nil, range = equipped:getTraits().soundRange }
		else
			soundRange = { path = nil, range = simdefs.SOUND_RANGE_2 }
		end
		self:emitSound( soundRange, x0, y0, sourceUnit )
		self:processReactions()
	end
end

function simengine:refreshUnitLOS( unit )
	oldRefreshUnitLOS( self, unit )
	if unit:getTraits().facingToTurnTo then
		unit:updateFacing(unit:getTraits().facingToTurnTo)
		if unit:getFacing() == unit:getTraits().facingToTurnTo then
			unit:getTraits().facingToTurnTo = nil
		end
	end
end

function simengine:trackerAdvance( delta, txt, scan )
	if self._trackerQueue then
		table.insert(self._trackerQueue,{delta=delta, txt=txt, scan=scan})
	else
	    	assert( delta > 0 )
	    	delta = delta * self._params.difficultyOptions.alarmMultiplier
	    	if delta > 0 then
	        	local oldTracker = self._tracker

	        	for i = 1, delta do
	            		self._tracker = self._tracker + 1
 				self:triggerEvent( simdefs.TRG_ALARM_INCREASE )
	            		local stage = self:getTrackerStage()
	            		if stage > self._trackerStage then
	                		self._trackerStage = stage
	    				self:dispatchEvent( simdefs.EV_ADVANCE_TRACKER, {txt=txt, delta = self._tracker - oldTracker, tracker = oldTracker })
	         	    		self:triggerEvent( simdefs.TRG_ALARM_STATE_CHANGE, self._trackerStage )
	                		oldTracker = self._tracker

	                		--HANDLE PROTOCOL 78
	                		if self:getTags().protocol78 then
	                			for _, unitRaw in pairs(self:getAllUnits() ) do
	                				if unitRaw:getTraits().safeUnit and unitRaw:getTraits().credits and unitRaw:getTraits().credits > 0 then
	                					unitRaw:getTraits().credits =  unitRaw:getTraits().credits +25
	                					local player = self:getPC()
	                					local x0,y0 = unitRaw:getLocation()
	                					if player:hasSeen(unitRaw) then									             											
									self:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = self:getTags().protocol78, x = x0, y = y0, color={r=163/255,g=243/255,b=248/255,a=1},skipQue=true} )									
	                					end
	                				end
	                			end
	                		end

					local pc = self:getPC()
					for i, unit in pairs( pc:getAgents() ) do
						for c,child in ipairs(unit:getChildren())do
							if child:getTraits().alarmMPmod then
								if (not self:isVersion("0.17.12") or not child:getTraits().augment or child:getTraits().installed) and stage <= simdefs.TRACKER_MAXSTAGE then
									self:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit  } )
						       			local x2,y2 = unit:getLocation()
							    		self:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = child:getTraits().alarmMPmodTxt, x = x2, y = y2, color=cdefs.AUGMENT_TXT_COLOR} )
							    		self:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )						
									unit:getTraits().mpMax = unit:getTraits().mpMax + child:getTraits().alarmMPmod
									unit:getTraits().mp = unit:getTraits().mp + child:getTraits().alarmMPmod
								end
							end
						end			
					end
	            		end
	       		end

	        	if self._tracker > oldTracker then
	            		self:dispatchEvent( simdefs.EV_ADVANCE_TRACKER, {txt=txt, delta = self._tracker - oldTracker, tracker = oldTracker })
	        	end
		end

		self._resultTable.max_alarm_level = self._resultTable.max_alarm_level and math.max(self._resultTable.max_alarm_level, self._tracker) or self._tracker
	    	self:getStats():maxStat( "max_tracker", self._tracker )
	end
end

function simengine:endTurn()
	self._turnState = simdefs.TURN_ENDING
	self:dispatchEvent( simdefs.EV_TURN_END, self:getCurrentPlayer() )
	self:getTags().nextProgFree = nil

	self:triggerEvent( "TRG_PRE_END_TURN", self:getCurrentPlayer() )

	for i,player in ipairs(self._players) do
		player:onEndTurn( self )
	end

	local units = util.tdupe( self._units )
	for unitID, unit in pairs( units ) do
        	if unit:isValid() then
		    	unit:onEndTurn( self )		   
        	end
	end
    
	self:triggerEvent( simdefs.TRG_TIMER )
	self:triggerEvent( simdefs.TRG_END_TURN, self:getCurrentPlayer() )

	local prevTurn = self._turn
	self._turn = (self._turn % #self._players) + 1
	self._turnCount = self._turnCount + 1

	self:forEachCell(
		function( c )
			for i, exit in pairs( c.exits ) do
				if exit.door then
                    			if exit.closeEndTurn and not exit.closed then
    						self:modifyExit( c, i, simdefs.EXITOP_CLOSE )
                    			end
                    			if exit.lockEndTurn and not exit.locked then
    						self:modifyExit( c, i, simdefs.EXITOP_LOCK )
                    			end
				end
			end
		end )
	self:closeGuardDoors()

	self:triggerEvent( "TRG_POST_END_TURN", self:getCurrentPlayer() )

	self._turnState = simdefs.TURN_STARTING
	if self:getCurrentPlayer() then
		self:dispatchEvent( simdefs.EV_TURN_START, {player = self:getCurrentPlayer() })
		self:triggerEvent( "TRG_PRE_START_TURN", self:getCurrentPlayer() )
		self:getCurrentPlayer():onStartTurn( self )
		self:triggerEvent( simdefs.TRG_START_TURN, self:getCurrentPlayer() )
		self:triggerEvent( "TRG_POST_START_TURN", self:getCurrentPlayer() )
	end
	self._turnState = simdefs.TURN_PLAYING

	if self:getCurrentPlayer() and (self:getCurrentPlayer():getTraits().skipTurn or 0) > 0 then
		self:getCurrentPlayer():getTraits().skipTurn = self:getCurrentPlayer():getTraits().skipTurn - 1
		if self:getCurrentPlayer():isPC() then
			self:endTurn()
			if self:getCurrentPlayer():isNPC() then
				self._players[1]:thinkHard( self )
			end
		elseif self:getCurrentPlayer():isNPC() then
			self:endTurn()
		end
	end
end

function simengine:moveUnit( unit, moveTable )
	local t1, t2 = oldMoveUnit( self, unit, moveTable )
	if unit:countAugments( "W93_augment_scanner_mesh" ) > 0 then
		self:dispatchEvent( "W93_REFRESH_CELLS" )
	end
	return t1, t2
end

function simengine:setMissionReward(missionReward)
	if self._missionReward then
		self._missionReward = self._missionReward + missionReward
	else
		self._missionReward = missionReward
	end
	self._resultTable.credits_gained.hostage = self._resultTable.credits_gained.hostage and self._resultTable.credits_gained.hostage + missionReward or missionReward
end

function simengine:spawnUnit( unit )
	oldSpawnUnit(self, unit)
	self:triggerEvent( "TRG_UNIT_SPAWNED", unit )
end

function simengine:init( params, levelData )
	oldInit( self, params, levelData )

	if self:getCurrentPlayer() and not EDITOR then
		self:triggerEvent( "TRG_POST_START_TURN", self:getCurrentPlayer() )
	end
end