local array = include("modules/array")
local simdefs = include("sim/simdefs")

local simplayer = include("sim/simplayer")
local pcplayer = include("sim/pcplayer")

function simplayer:deployUnits( sim, agentDefs )
	if not self.deploycells then
		self.deploycells = {}
		self.backupcells = {}
		sim:forEachCell(
			function( deploycell )
				if deploycell.deployID and #deploycell.units == 0 then
					table.insert(self.deploycells,deploycell)
					table.insert(self.backupcells,deploycell)
				end
			end )
	end
	
	local deploycells = self.deploycells
	local backupcells = self.backupcells
	
	for i, levelUnit in pairs(sim._levelOutput.units) do
		--Oof consoles have not been placed yet
		sim:getCell(levelUnit.x,levelUnit.y).plannedUnit = true
	end

	local maxAgentCount = sim:getParams().agency.equippedAgents or simdefs.MAX_AGENT_COUNT
		
	-- Deploy the agents as the last thing we do!
	for agentCount, agentDef in ipairs( agentDefs ) do
		if not maxAgentCount or agentCount <= maxAgentCount or self.deployedEquipAgents then
			local deploy_cell, i = array.findIf( deploycells, function( c ) return c.deployID == (agentDef.deployID or simdefs.DEFAULT_EXITID) end )
			if deploy_cell then
				deploy_cell = table.remove(deploycells, i)
				self:deployUnit( sim, agentDef.id, deploy_cell, deploy_cell.deployFacing )
			else
				log:write( "No deploycells for %s, generating random deploy", agentDef.template )
			
				local randomCells = {}
			
				for i, cell in ipairs(backupcells) do
					for dir, exit in pairs(cell.exits) do
						if #exit.cell.units == 0 and exit.cell.impass == 0 and (not exit.closed or exit.keybits == simdefs.DOOR_KEYS.BLAST_DOOR) and not exit.cell.plannedUnit then 
							table.insert(randomCells, exit.cell)
						end
					end
				end
				
				if #randomCells == 0 then
					--Diagonals?
					log:write( "No adjacent cells for %s, checking next adjacent", agentDef.template )
					for i, cell in ipairs(backupcells) do
						for dir, exit0 in pairs(cell.exits) do
							if not exit0.closed or exit0.keybits == simdefs.DOOR_KEYS.BLAST_DOOR then 
								for dir, exit in pairs(exit0.cell.exits) do
									if #exit.cell.units == 0 and exit.cell.impass == 0 and (not exit.closed or exit.keybits == simdefs.DOOR_KEYS.BLAST_DOOR) and not exit.cell.plannedUnit then 
										table.insert(randomCells, exit.cell)
									end
								end
							end
						end
					end
				end
				
				if #randomCells > 0 then
					deploy_cell = randomCells[sim:nextRand(1,#randomCells)]
					table.insert(backupcells,deploy_cell)
					self:deployUnit( sim, agentDef.id, deploy_cell, deploy_cell.deployFacing )
				else
					log:write( "WARNING: couldn't deploy %s, no deploycells", agentDef.template )
				end
			end
		end
	end
	
	--Make sure special agents (Central and Monst3r) are allowed to be deployed
	self.deployedEquipAgents = true
	
	--Detention Center
	simdefs.AGENT_LIMIT = #self:getAgents() + simdefs.MAX_AGENCY_SIZE - #sim:getParams().agency.unitDefs
end

pcplayer.deployUnits = simplayer.deployUnits