local util = include( "modules/util" )
local simplayer = include("sim/simplayer")
local pcplayer = include("sim/pcplayer")
local aiplayer = include("sim/aiplayer")
local unitdefs = include("sim/unitdefs")
local simability = include( "sim/simability" )
local simfactory = include( "sim/simfactory" )

local baseAddMainframeAbility = aiplayer.addMainframeAbility
function aiplayer:addMainframeAbility( sim, abilityID, hostUnit, reversalOdds )
	if self:isNPC() then --redundant, but whatever
		local ability = simability.create( abilityID )
		sim:triggerEvent( "TRG_MAA_TRUE_DAEMON_INSTALL", ( (not reversalOdds and not ability.noDaemonReversal) or (reversalOdds and reversalOdds > 0) ) )
		baseAddMainframeAbility( self, sim, abilityID, hostUnit, reversalOdds )
	end
end

local baseDeployUnit = simplayer.deployUnit
function simplayer:deployUnit( sim, agentID, ... )
	if agentID == 100 and unitdefs.lookupTemplate("ef_memory") then -- monst3r
		if sim:getParams().agency._keptItems and sim:getParams().agency._keptItems[100] then
			local agentDef = self._deployed[ agentID ].agentDef
			agentDef.upgrades = sim:getParams().agency._keptItems[100]
		end
	end

	return baseDeployUnit( self, sim, agentID, ... )
end
pcplayer.deployUnit = simplayer.deployUnit

function pcplayer:deploySummon( sim, agentDef )
	local aDef = agentDef
	assert( aDef )
	
	local unitData = unitdefs.createUnitData( aDef )
	local unit = simfactory.createUnit( unitData, sim )
	unit:setPlayerOwner( self )
	sim:spawnUnit( unit )
	unit:getTraits().maa_isSummon = true
	
	if not self._deployedSummons then
		self._deployedSummons = {}
	end
	self._deployedSummons[ aDef.id ] = { agentDef = aDef }
	self._deployedSummons[ aDef.id ].id = unit:getID()
	
	return unit
end

local baseInit = pcplayer.init
function pcplayer:init( sim, agency )
	baseInit( self, sim, agency )
	
	self._deployedSummons = {}
--	
--	if agency.unitDefs_childs then
--		self:deploySummons( sim, agency.unitDefs_childs )
--	end
end

--local baseGetAgents = pcplayer.getAgents
--function pcplayer:getAgents()
--	local agents = baseGetAgents( self )
--	
--	local agnets_cleaned = {}
--	for _, agent in ipairs( agents ) do
--		if not agent:getTraits().maa_isSummon then
--			table.insert( agnets_cleaned, agent )
--		end
--	end
--	
--	return agnets_cleaned
--end

function pcplayer:getSummons()
	local summons = {}
	if self._deployedSummons then
		for agentID, deployData in pairs( self._deployedSummons ) do
			local unit = self._sim:getUnit( deployData.id )
			if unit then
				table.insert( summons, unit )
			end
		end
	end

	return summons
end
