local util = include("client_util")
local array = include("modules/array")
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )

local baseCalculateShotSuccess = simquery.calculateShotSuccess
function simquery.calculateShotSuccess( sim, sourceUnit, targetUnit, equipped )
	if not equipped then
		return baseCalculateShotSuccess( sim, sourceUnit, targetUnit, equipped )
	end

	--alteration to handle Shalem_1's headshot ability
	local weapon_armor_ignore = equipped:getTraits().ignoreArmor
	local weapon_base_damage = equipped:getTraits().baseDamage
	--if sourceUnit:getTraits().maa_headshot_ready and equipped:getTraits().weaponType and equipped:getTraits().weaponType == "rifle" then
	if sourceUnit:getTraits().maa_tracked_target and targetUnit == sourceUnit:getTraits().maa_tracked_target then
		--target is tracked target; temporarily have the weapon ignore armor
		equipped:getTraits().ignoreArmor = true
		equipped:getTraits().baseDamage = (equipped:getTraits().baseDamage or 1) + 2
	end

	local shot = baseCalculateShotSuccess( sim, sourceUnit, targetUnit, equipped )

	--neural toxin dart stuff
	if equipped:getTraits().maa_is_toxin then
		shot.maa_is_toxin = true
		if targetUnit and ( not targetUnit:getTraits().isGuard or targetUnit:getTraits().isDrone ) then
			shot.maa_no_bio = true
		end
	end
	--umi drone's mark to reduce armor
	if equipped:getTraits().maa_is_umi_drone_toolkit_support then
		shot.maa_mark_armor = true
		if targetUnit and targetUnit:getArmor() <= 0 then
			shot.maa_mark_armor_noArmor = true
		end
	end

	--reset the weapon
	equipped:getTraits().ignoreArmor = weapon_armor_ignore
	equipped:getTraits().baseDamage = weapon_base_damage
	
	return shot
end

local baseCalculateDamageAndArmorForMelee = simquery.calculateDamageAndArmorForMelee
function simquery.calculateDamageAndArmorForMelee( sim, sourceUnit, targetUnit )
	local meleeDamage, armorPiercing, armor = baseCalculateDamageAndArmorForMelee( sim, sourceUnit, targetUnit )

	--Nika's and Decker's temporary buff
	local unitOwner = sourceUnit:getUnitOwner()
	if unitOwner then
		if unitOwner:getTraits().maa_buff_melee_damage then
			meleeDamage = meleeDamage + unitOwner:getTraits().maa_buff_melee_damage
		end
		if unitOwner:getTraits().maa_buff_melee_pierce then
			armorPiercing = armorPiercing + unitOwner:getTraits().maa_buff_melee_pierce
		end
		if unitOwner:getTraits().maa_buff_melee_damage_turn then
			meleeDamage = meleeDamage + unitOwner:getTraits().maa_buff_melee_damage_turn
		end
		if unitOwner:getTraits().maa_buff_melee_pierce_turn then
			armorPiercing = armorPiercing + unitOwner:getTraits().maa_buff_melee_pierce_turn
		end
	end

	return meleeDamage, armorPiercing, armor
end

local baseCouldUnitSee = simquery.couldUnitSee
function simquery.couldUnitSee( sim, unit, targetUnit, ignoreCover, targetCell )
	local result = baseCouldUnitSee( sim, unit, targetUnit, ignoreCover, targetCell )

	--Prism2 not visible by cameras
	if result and targetUnit and unit then
		local target_has_camera_disguise = false

		for _, child in ipairs( targetUnit:getChildren() ) do
			if child:getTraits() and child:getTraits().maa_camera_disguise then
				if child:getTraits().restrictedUse then
					for _, legalUser in ipairs( child:getTraits().restrictedUse ) do
						if legalUser.agentID == targetUnit:getUnitData().agentID then
							target_has_camera_disguise = true
							break
						end
					end
				else
					target_has_camera_disguise = true
					break
				end
			end
		end

		if target_has_camera_disguise and unit:getTraits() and unit:getTraits().mainframe_camera then
			return false
		end
	end

	--Banks1 not visible in peripheral
	if result and targetUnit and targetUnit:getTraits().maa_banks_no_peripheral then
		if not( unit:getBrain() and unit:getBrain():getTarget() and unit:getBrain():getTarget() == targetUnit ) then --disable bank's cloak when overwatched
			local x1, y1 = targetUnit:getLocation()
			if targetCell then
				x1, y1 = targetCell.x, targetCell.y
			end

			if x1 and y1 then
				local sight, peripheral = sim:canUnitSee( unit, x1, y1 )
				if peripheral and not sight then
					return false
				end
			end
		end
	end

	return result
end

local baseCanUnitReach = simquery.canUnitReach
function simquery.canUnitReach( sim, unit, x1, y1 )
	if unit:getTraits().maa_inside_mainframe then
		return true
	end

	return baseCanUnitReach( sim, unit, x1, y1 )
end

local baseGetMoveSoundRange = simquery.getMoveSoundRange
function simquery.getMoveSoundRange( unit, cell )
	local range = baseGetMoveSoundRange( unit, cell )
	if unit:getTraits().maa_isDrunk then
		range = math.max( range, simdefs.SOUND_RANGE_1 + (cell.noiseRadius or 0) )
	end

	return range
end

local baseCountFieldAgents = simquery.countFieldAgents
function simquery.countFieldAgents( sim )
	--remove the warning, when leaving behind the drone dummy
	local fieldUnits, escapingUnits = baseCountFieldAgents( sim )
	
	local droneCan = nil
	for _, escapeUnit in ipairs( escapingUnits ) do
		for _, childUnit in ipairs( escapeUnit:getChildren() ) do
			if childUnit:getTraits().maa_is_umi_drone_item then
				droneCan = childUnit
				break
			end			
		end
	end
	--if the drone is leaving in the inventory of an agent, remove the drone from the fieldUnits list
	if droneCan then
		for i, fieldUnit in ipairs( fieldUnits ) do
			if fieldUnit:getTraits().maa_isUmiDrone then
				table.remove( fieldUnits, i )
				break
			end
		end
	end
	
	return fieldUnits, escapingUnits
end

