local simquery = include("sim/simquery")
local simdefs = include("sim/simdefs")
local mathutil = include("modules/mathutil")
local CHEATS = include( "fe/cheatmenu" )

CHEATS.spawnProgram = function( programName )
    if sim then
        local function fn( sim, programName )
            local simquery = include("sim/simquery")
            local maxPrograms = simquery.getMaxPrograms( sim )

        	if #sim:getPC():getAbilities() < maxPrograms then
                sim:getPC():addMainframeAbility( sim, programName )
            end
        end
        game:doAction( "debugAction", fn, programName )
    end
end

function simquery.getMaxPrograms( sim )
	return simdefs.MAX_PROGRAMS + (sim:getParams().agency.extraPrograms or 0) + (sim:getParams().difficultyOptions.programCount or 5) + (sim:getTags().extraPrograms or 0) - 5
end

local oldCalculateMeleeDamage = simquery.calculateMeleeDamage
function simquery.calculateMeleeDamage( sim, sourceUnit, targetUnit, ... )
	if targetUnit or not sourceUnit then
		return oldCalculateMeleeDamage( sim, sourceUnit, targetUnit, ... )
	else
		return math.max( 1, oldCalculateMeleeDamage( sim, sourceUnit, targetUnit, ... ) )
	end
end

function simquery.getMoveAudioRange( unit, cell, innocuous )
	local range = simquery.getMoveSoundRange( unit, cell )
	if unit:isNPC() and range == unit:getUnitData().traits.dashSoundRange and not unit:getTraits().sneaking and not unit:getModifiers():hasAny("dashSoundRange") then             
		return simdefs.SOUND_RANGE_2
	end
	
	return range
end

function simquery.getWalkTime( unit, from_cell, to_cell )
	local WALKSPEED = 0.13 -- In cells/second
	if unit:getTraits().movingBody then
		WALKSPEED = 0.36
	elseif unit:getTraits().walk then
		WALKSPEED = 0.32
	elseif unit:isPC() then
		WALKSPEED = 0.16
	end
	local settingsFile = savefiles.getSettings( "settings" )
	if settingsFile.data.fastMode then
		WALKSPEED = WALKSPEED * 0.5
	end

	return mathutil.dist2d( from_cell.x, from_cell.y, to_cell.x, to_cell.y ) * config.WARPSPEED * WALKSPEED
end