local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )

local oldIsEnemyAgent = simquery.isEnemyAgent
local oldIsEnemyTarget = simquery.isEnemyTarget
local oldCanSoftPath = simquery.canSoftPath
local oldFillCircle = simquery.fillCircle

function simquery.isCellWatched( sim, player, cellx, celly )

	local isWatched, isNoticed ,isHidden = false, false, false


	if sim:canPlayerSee( player, cellx, celly ) then
		local seers = sim:getLOS():getSeers( cellx, celly )

		for _, seerID in ipairs(seers) do

			local seerUnit

			if seerID >= simdefs.SEERID_PERIPHERAL then

				seerUnit = sim:getUnit( seerID - simdefs.SEERID_PERIPHERAL )
			else
				seerUnit = sim:getUnit( seerID )
			end

			if seerUnit and seerUnit:getPlayerOwner() ~= player and simquery.canSeeLOS( sim, player, seerUnit ) then
				if simquery.checkCover( sim, seerUnit, cellx, celly ) and not seerUnit:getTraits().seesHidden and not seerUnit:getTraits().ultrasonic_echolocation then

					isHidden = true
				elseif seerID >= simdefs.SEERID_PERIPHERAL or (simquery.checkCover( sim, seerUnit, cellx, celly ) and not seerUnit:getTraits().seesHidden) then

					isNoticed = true
				else

					isWatched = true
					break

				end

			end
		end

	end

	if isWatched then
		return simdefs.CELL_WATCHED
	elseif isNoticed then

		return simdefs.CELL_NOTICED

	elseif isHidden then

		return simdefs.CELL_HIDDEN

	end

end

function simquery.couldUnitSee( sim, unit, targetUnit, ignoreCover, targetCell )
	if unit:getTraits().globalVision then
		return true
	end
	if not unit:getTraits().hasSight then
		return false
	end
	if unit:isKO() then
		return false
	end
	if unit:getTraits().grappler then
		return false
	end
	
	if targetCell and unit:getTraits().noPeripheralDetection then
		return false
	end

	if targetUnit and targetUnit ~= unit then

		if not targetUnit:getLocation() then
			return false
		end

		if targetUnit:getTraits().invisible and targetUnit:getPlayerOwner() ~= unit:getPlayerOwner() and not unit:getTraits().detect_cloak then
			return false
		end
        
		ignoreCover = ignoreCover or unit:isPC()
		if not ignoreCover then
			if not targetCell then
        			targetCell = sim:getCell(targetUnit:getLocation() )
			end
			if not unit:getTraits().seesHidden and targetUnit:canHide() and
				simquery.checkIfCellNextToCover(sim, targetCell) and simquery.checkCover( sim, unit, targetCell.x, targetCell.y ) then
				return false
			end
		end
	end

	return true
end

function simquery.isEnemyTarget( player, unit, ignoreDisguise )
	if unit and unit:hasTrait("LEVER_status") then
		if (not simquery.getEquippedGun( unit )) and (not unit:getTraits().movingBody) then
			return false
		elseif unit:getPlayerOwner() ~= player then
			simquery.LEVER_resetStatus( unit:getSim(), unit, "ANY", false )
			return true
		end
	end

	if player and player:isPC() and unit and unit:getTraits().invisible then
		return false
	end

	return oldIsEnemyTarget( player, unit, ignoreDisguise )
end

function simquery.isEnemyAgent( player, unit, ignoreDisguise )
	if not unit:hasTrait("isAgent") then
		return false
	end

	if unit:hasTrait("LEVER_status") then
		if (not simquery.getEquippedGun( unit )) and (not unit:getTraits().movingBody) then
			return false
		elseif unit:getPlayerOwner() ~= player then
			simquery.LEVER_resetStatus( unit:getSim(), unit, "ANY", false )
			return true
		end
	end

	return simquery.isEnemyTarget( player, unit, ignoreDisguise )
end

function simquery.calculateShotSuccess( sim, sourceUnit, targetUnit, equipped )
	if not equipped then
		return nil
	end
	 	
	local shot =
	{
		damage = equipped:getTraits().baseDamage or 1,
		ko = equipped:getTraits().canSleep,
		noTargetAlert = equipped:getTraits().noTargetAlert
	}
	
	if equipped:getTraits().canTag then
		shot.canTag = true
    	else
	    	local armor = 0
	    	for i,childUnit in ipairs(targetUnit:getChildren()) do 
		    	if childUnit:getTraits().armor and childUnit:getTraits().armor > armor then 
			    	armor = childUnit:getTraits().armor 
		    	end
	    	end

	    	if targetUnit:getArmor() then
		    	armor = armor + targetUnit:getArmor()
	    	end

		if equipped:getTraits().ignoreArmor then
			armor = 0
		else
	    		armor = math.max( 0, armor - sourceUnit:countArmorPiercingUpgrades("addArmorPiercingRanged") )
		end

		local proxyKObonus = 0
	    	local armorPiercing = equipped:getTraits().armorPiercing or 0

		if sourceUnit:countAugments( "LEVER_augment_unexpected_shot" ) > 0 and ( sourceUnit:getTraits().LEVER_status or sourceUnit:getTraits().disguiseOn or sourceUnit:getTraits().invisible ) then
			if shot.ko then
				armorPiercing = armorPiercing + 1
				proxyKObonus = proxyKObonus + 2
			else
				armorPiercing = armorPiercing + 2
			end
		end

		if equipped:getTraits().proxyRange then
			local x0, y0 = sourceUnit:getLocation()
			local x1, y1 = targetUnit:getLocation()
			if equipped:getTraits().proxyKObonus and x0 and x1 then
				if equipped:getTraits().proxyRange < 0 and mathutil.dist2d(x0, y0, x1, y1) <= math.abs(equipped:getTraits().proxyRange) then
					proxyKObonus = proxyKObonus + equipped:getTraits().proxyKObonus
				elseif equipped:getTraits().proxyRange > 0 and mathutil.dist2d(x0, y0, x1, y1) >= math.abs(equipped:getTraits().proxyRange) then
					proxyKObonus = proxyKObonus + equipped:getTraits().proxyKObonus
				end
			end
		end

	    	if shot.ko then
	    		shot.damage = targetUnit:processKOresist( shot.damage + (sourceUnit:getTraits().rangedKO_bonus or 0) + proxyKObonus )	
	    	end

	    	shot.armorBlocked = false 
	    	if armor > armorPiercing or (equipped:getTraits().targetNotAlerted and targetUnit:isAlerted()) or 
		(targetUnit:countAugments( "W93_augment_hostage_monitor" ) > 0 and targetUnit:getTraits().movingBody and targetUnit:getTraits().movingBody:isKO() and targetUnit:getTraits().movingBody:getTraits().isGuard and not sourceUnit:getTraits().enforcer and 
		(targetUnit:getFacing() - sourceUnit:getFacing() >= 7 or targetUnit:getFacing() - sourceUnit:getFacing() <= -7 or (targetUnit:getFacing() - sourceUnit:getFacing() >= -1 and targetUnit:getFacing() - sourceUnit:getFacing() <= 1) )) then 
		    	shot.armorBlocked = true 
	    	end
		shot.armorPiercing = armorPiercing
    	end

	return shot
end

function simquery.calculateDamageAndArmorForMelee(sim, sourceUnit, targetUnit)
	local meleeDamage, armorPiercing = 0, 0
	local armor = 0
    	local unitOwner = sourceUnit:getUnitOwner()

	if simquery.isAgent( unitOwner ) then
		meleeDamage = meleeDamage + (unitOwner:getTraits().meleeDamage or 0)
        	armorPiercing = armorPiercing + unitOwner:countArmorPiercingUpgrades("addArmorPiercingMelee") 

		if sourceUnit:getTraits().skillAugmentedStrength then
			meleeDamage = meleeDamage + (unitOwner:getSkillLevel( "inventory" ) or 0)
		end

        	if unitOwner:getTraits().kinetic_capacitor_bonus and unitOwner:getTraits().kinetic_capacitor_bonus_charged  then
        		armorPiercing = armorPiercing + unitOwner:getTraits().kinetic_capacitor_bonus
        		meleeDamage = meleeDamage + unitOwner:getTraits().kinetic_capacitor_bonus
        	end

		if unitOwner:countAugments( "augment_sharp_1" ) > 0 then
			local augmentCount = unitOwner:getAugmentCount()
			meleeDamage = meleeDamage + math.floor( augmentCount / 3 )  
		end

		if unitOwner:countAugments( "augment_sharp_2" ) > 0 then
    			local augmentCount = unitOwner:getAugmentCount()
			armorPiercing = armorPiercing + math.floor( augmentCount / 3 )    	
		end

		if unitOwner:countAugments( "LEVER_augment_sucker_punch" ) > 0 and ( unitOwner:getTraits().disguiseOn or unitOwner:getTraits().invisible or unitOwner:getTraits().LEVER_status ) then
    			local augmentCount = unitOwner:getAugmentCount()
			meleeDamage = meleeDamage + 3
			armorPiercing = armorPiercing + 1  	
		end

		local augCount, augType = unitOwner:countAugments("W93_augment_charged_actuators")
		if augCount > 0 and augType[1]:getTraits().ammo > 0 then
			meleeDamage = meleeDamage + augType[1]:getTraits().ammo
		end
	end

	meleeDamage = meleeDamage + (sourceUnit:getTraits().damage or 0)
    	armorPiercing = armorPiercing + (sourceUnit:getTraits().armorPiercing or 0)

	if targetUnit then
		armor = targetUnit:getArmor()
	end

	if sourceUnit:getTraits().ignoreArmor then
		armorPiercing = 99
	end

	return meleeDamage, armorPiercing, armor
end

function simquery.getMoveSoundRange( unit, cell )
	local range = 0
	if not unit:getTraits().sneaking then  
		range = unit:getTraits().dashSoundRange
	else
		range = simdefs.SOUND_RANGE_0
	end
	if unit:getPlayerOwner():getTraits().shackleDaemon and not unit:getTraits().isDrone then
		range = math.max(range, simdefs.SOUND_RANGE_1)
	end
	return range + (cell.noiseRadius or 0)	
end

function simquery.LEVER_resetStatus( sim, unit, status, reverse, forced )
	local x,y = unit:getLocation()

	if unit:getTraits().LEVER_status and (simquery.isCellWatched( sim, sim:getNPC(), x, y ) == simdefs.CELL_WATCHED or simquery.isCellWatched( sim, sim:getNPC(), x, y ) == simdefs.CELL_NOTICED or forced) then
		if status == "ANY" or ((status == unit:getTraits().LEVER_status and not reverse) or (status ~= unit:getTraits().LEVER_status and reverse)) then
			local cell = sim:getCell(x,y)
			unit:getTraits().LEVER_status = nil
			unit:getTraits().hidesInCover = true
			unit:getTraits().walk = false
			unit:getTraits().LEVER_stalling = nil
			local seers = sim:generateSeers( unit )
			for i,seer in ipairs(seers) do
		        	local seerUnit = sim:getUnit(seer)
	        		if seerUnit then
					if simquery.couldUnitSee( sim, seerUnit, unit, true ) then
				    		local x0,y0 = seerUnit:getLocation()
						seerUnit:setAlerted(true)
						sim:triggerEvent( simdefs.TRG_UNIT_APPEARED, { seerID = seer, unit = unit } )				   
					end
				end
	        	end
			sim:notifySeers()
			sim:triggerEvent( simdefs.TRG_UNIT_WARP, { unit = unit, from_cell = cell, to_cell = cell } )
			sim:processReactions()
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit })
		end
	end
end

function simquery.canSoftPath( sim, unit, startcell, endcell )
	if (not SCRIPT_PATHS.W93_FuncLibActive) or (SCRIPT_PATHS.W93_FuncLibActive and (not SCRIPT_PATHS.W93_FuncLibChanges)) then
		return oldCanSoftPath( sim, unit, startcell, endcell )
	end

    	assert( not endcell.ghostID )
    	assert( not startcell.ghostID )
    	assert( unit:isNPC() )
	
	if not unit:getTraits().rememberedImpass then
		unit:getTraits().rememberedImpass = {}
	end
	
	for i,cellUnit in ipairs(endcell.units) do
		if unit and (unit:getTraits().dynamicImpass or cellUnit:getTraits().emitterID) and cellUnit:getTraits().dynamicImpass then
			local x0, y0 = unit:getLocation()
            		if cellUnit:getTraits().emitterID and cellUnit:canControl( unit ) then

			elseif cellUnit:getPlayerOwner() == unit:getPlayerOwner() then

			elseif not cellUnit:getTraits().isAgent then
				return false, simdefs.CANMOVE_DYNAMIC_IMPASS
			elseif (startcell.x == x0 and startcell.y == y0 and sim:canUnitSee( unit, endcell.x, endcell.y )) or (math.abs(x0 - endcell.x) + math.abs(y0 - endcell.y) == 1 and unit:getTraits().rememberedImpass[endcell.id]) then
				unit:getTraits().rememberedImpass[endcell.id] = true
				return false, simdefs.CANMOVE_DYNAMIC_IMPASS
			else

			end
		end
	end
	
	unit:getTraits().rememberedImpass[endcell.id] = nil

	return simquery.canStaticPath( sim, unit, startcell, endcell )
end

function simquery.rasterCircle( sim, xOrigin, yOrigin, rangeMax, rangeMin )
	local cells = {}
	local x0 = math.min((xOrigin - rangeMax), 0)
	local y0 = math.min((yOrigin - rangeMax), 0)  
	local x1 = xOrigin + rangeMax
	local y1 = yOrigin + rangeMax

	for x = x0, x1 do 
		for y = y0, y1 do
			local distance = mathutil.dist2d( xOrigin, yOrigin, x, y )
			if distance <= rangeMax and distance >= (rangeMin or 0) then
				table.insert( cells, x )
				table.insert( cells, y )
			end
		end
	end

	return cells
end

function simquery.fillCircle( sim, xOrigin, yOrigin, rangeMax, rangeMin )
	local cells = {}
	local x0 = math.min((xOrigin - rangeMax), 0)
	local y0 = math.min((yOrigin - rangeMax), 0)  
	local x1 = xOrigin + rangeMax
	local y1 = yOrigin + rangeMax

	for x = x0, x1 do 
		for y = y0, y1 do
			local distance = 0
			if SCRIPT_PATHS.W93_FuncLibActive and SCRIPT_PATHS.W93_FuncLibChanges then
				distance = mathutil.dist2d( xOrigin, yOrigin, x, y )
			else
				distance = math.floor( mathutil.dist2d( xOrigin, yOrigin, x, y ) )
			end

			local cell = sim:getCell( x, y )
			if distance >= rangeMin and distance <= rangeMax and cell then
				table.insert( cells, sim:getCell( x, y ))
			end
		end
	end

	return cells
end

function simquery.rasterOrthogonalCross( sim, xOrigin, yOrigin, rangeMin, rangeMax, width )
	local cells = {}
	if rangeMin == 0 then
		table.insert( cells, xOrigin )
		table.insert( cells, yOrigin )
	end

	for i=math.max(rangeMin,1), math.max(rangeMin,rangeMax) do
		table.insert( cells, xOrigin + i )
		table.insert( cells, yOrigin )
		table.insert( cells, xOrigin - i )
		table.insert( cells, yOrigin )
		table.insert( cells, xOrigin )
		table.insert( cells, yOrigin + i )
		table.insert( cells, xOrigin )
		table.insert( cells, yOrigin - i )
	end

	for j=math.min(width+1,2), math.max(width,1), 1 do
		for i=math.max(rangeMin, j), math.max(rangeMin, rangeMax) do
			table.insert( cells, xOrigin + i )
			table.insert( cells, yOrigin + (j - 1))
			table.insert( cells, xOrigin + i )
			table.insert( cells, yOrigin - (j - 1))

			table.insert( cells, xOrigin - i )
			table.insert( cells, yOrigin + (j - 1))
			table.insert( cells, xOrigin - i )
			table.insert( cells, yOrigin - (j - 1))

			table.insert( cells, xOrigin + (j - 1))
			table.insert( cells, yOrigin + i )
			table.insert( cells, xOrigin - (j - 1))
			table.insert( cells, yOrigin + i )

			table.insert( cells, xOrigin + (j - 1))
			table.insert( cells, yOrigin - i )
			table.insert( cells, xOrigin - (j - 1))
			table.insert( cells, yOrigin - i )
		end
	end

	if width > 1 then
		util.tmerge(cells, simquery.rasterDiagonalCross(sim, xOrigin, yOrigin, math.max(rangeMin, 1), width - 1, 1))
	end

	return cells
end

function simquery.fillOrthogonalCross( sim, xOrigin, yOrigin, rangeMin, rangeMax, width )
	local cells = {}
	if rangeMin == 0 then
		table.insert( cells, sim:getCell( xOrigin, yOrigin ) )
	end

	for i=math.max(rangeMin,1), math.max(rangeMin,rangeMax) do
		if sim:getCell( xOrigin + i, yOrigin ) then table.insert( cells, sim:getCell( xOrigin + i, yOrigin ) ) end
		if sim:getCell( xOrigin - i, yOrigin ) then table.insert( cells, sim:getCell( xOrigin - i, yOrigin ) ) end
		if sim:getCell( xOrigin, yOrigin + i ) then table.insert( cells, sim:getCell( xOrigin, yOrigin + i ) ) end
		if sim:getCell( xOrigin, yOrigin - i ) then table.insert( cells, sim:getCell( xOrigin, yOrigin - i ) ) end
	end

	for j=math.min(width+1,2), math.max(width,1), 1 do
		for i=math.max(rangeMin, j), math.max(rangeMin, rangeMax) do
			if sim:getCell( xOrigin + i, yOrigin + (j - 1) ) then table.insert( cells, sim:getCell( xOrigin + i, yOrigin + (j - 1) ) ) end
			if sim:getCell( xOrigin + i, yOrigin - (j - 1) ) then table.insert( cells, sim:getCell( xOrigin + i, yOrigin - (j - 1) ) ) end

			if sim:getCell( xOrigin - i, yOrigin + (j - 1) ) then table.insert( cells, sim:getCell( xOrigin - i, yOrigin + (j - 1) ) ) end
			if sim:getCell( xOrigin - i, yOrigin - (j - 1) ) then table.insert( cells, sim:getCell( xOrigin - i, yOrigin - (j - 1) ) ) end

			if sim:getCell( xOrigin + (j - 1), yOrigin + i ) then table.insert( cells, sim:getCell( xOrigin + (j - 1), yOrigin + i ) ) end
			if sim:getCell( xOrigin - (j - 1), yOrigin + i ) then table.insert( cells, sim:getCell( xOrigin - (j - 1), yOrigin + i ) ) end

			if sim:getCell( xOrigin + (j - 1), yOrigin - i ) then table.insert( cells, sim:getCell( xOrigin + (j - 1), yOrigin - i ) ) end
			if sim:getCell( xOrigin - (j - 1), yOrigin - i ) then table.insert( cells, sim:getCell( xOrigin - (j - 1), yOrigin - i ) ) end
		end
	end

	if width > 1 then
		util.tmerge(cells, simquery.fillDiagonalCross(sim, xOrigin, yOrigin, math.max(rangeMin, 1), width - 1, 1))
	end

	return cells
end

function simquery.rasterDiagonalCross( sim, xOrigin, yOrigin, rangeMin, rangeMax, width )
	local cells = {}
	if rangeMin == 0 then
		table.insert( cells, xOrigin )
		table.insert( cells, yOrigin )
	end

	for i=math.max(rangeMin,1), math.max(rangeMin,rangeMax) do
		table.insert( cells, xOrigin + i )
		table.insert( cells, yOrigin + i )
		table.insert( cells, xOrigin + i )
		table.insert( cells, yOrigin - i )
		table.insert( cells, xOrigin - i )
		table.insert( cells, yOrigin + i )
		table.insert( cells, xOrigin - i )
		table.insert( cells, yOrigin - i )
	end

	for j=math.min(width+1,2), math.max(width,1), 1 do
		for i=math.max(rangeMin, 1), math.max(rangeMin, math.ceil(rangeMax - 0.5*(j - 1))) do
			table.insert( cells, xOrigin + i )
			table.insert( cells, yOrigin + i + (j - 1) )
			table.insert( cells, xOrigin + i + (j - 1) )
			table.insert( cells, yOrigin + i )

			table.insert( cells, xOrigin - i )
			table.insert( cells, yOrigin - i - (j - 1) )
			table.insert( cells, xOrigin - i - (j - 1) )
			table.insert( cells, yOrigin - i )

			table.insert( cells, xOrigin + i )
			table.insert( cells, yOrigin - i - (j - 1) )
			table.insert( cells, xOrigin + i + (j - 1) )
			table.insert( cells, yOrigin - i )

			table.insert( cells, xOrigin - i )
			table.insert( cells, yOrigin + i + (j - 1) )
			table.insert( cells, xOrigin - i - (j - 1) )
			table.insert( cells, yOrigin + i )
		end
	end

	if width > 1 then
		util.tmerge(cells, simquery.rasterOrthogonalCross(sim, xOrigin, yOrigin, math.max(rangeMin, 1), width - 1, 1))
	end

	return cells
end

function simquery.fillDiagonalCross( sim, xOrigin, yOrigin, rangeMin, rangeMax, width )
	local cells = {}
	if rangeMin == 0 then
		table.insert( cells, sim:getCell( xOrigin, yOrigin ) )
	end

	for i=math.max(rangeMin,1), math.max(rangeMin,rangeMax) do
		if sim:getCell( xOrigin + i, yOrigin + i ) then
			table.insert( cells, sim:getCell( xOrigin + i, yOrigin + i ) )
		end
		if sim:getCell( xOrigin + i, yOrigin - i ) then
			table.insert( cells, sim:getCell( xOrigin + i, yOrigin - i ) )
		end
		if sim:getCell( xOrigin - i, yOrigin + i ) then
			table.insert( cells, sim:getCell( xOrigin - i, yOrigin + i ) )
		end
		if sim:getCell( xOrigin - i, yOrigin - i ) then
			table.insert( cells, sim:getCell( xOrigin - i, yOrigin - i ) )
		end
	end

	for j=math.min(width+1,2), math.max(width,1), 1 do
		for i=math.max(rangeMin, 1), math.max(rangeMin, math.ceil(rangeMax - 0.5*(j - 1))) do
			if sim:getCell( xOrigin + i, yOrigin + i + (j - 1) ) then table.insert( cells, sim:getCell( xOrigin + i, yOrigin + i + (j - 1) ) ) end
			if sim:getCell( xOrigin + i + (j - 1), yOrigin + i ) then table.insert( cells, sim:getCell( xOrigin + i + (j - 1), yOrigin + i ) ) end

			if sim:getCell( xOrigin - i, yOrigin - i - (j - 1) ) then table.insert( cells, sim:getCell( xOrigin - i, yOrigin - i - (j - 1) ) ) end
			if sim:getCell( xOrigin - i - (j - 1), yOrigin - i ) then table.insert( cells, sim:getCell( xOrigin - i - (j - 1), yOrigin - i ) ) end

			if sim:getCell( xOrigin + i, yOrigin - i - (j - 1) ) then table.insert( cells, sim:getCell( xOrigin + i, yOrigin - i - (j - 1) ) ) end
			if sim:getCell( xOrigin + i + (j - 1), yOrigin - i ) then table.insert( cells, sim:getCell( xOrigin + i + (j - 1), yOrigin - i ) ) end

			if sim:getCell( xOrigin - i, yOrigin + i + (j - 1) ) then table.insert( cells, sim:getCell( xOrigin - i, yOrigin + i + (j - 1) ) ) end
			if sim:getCell( xOrigin - i - (j - 1), yOrigin + i ) then table.insert( cells, sim:getCell( xOrigin - i - (j - 1), yOrigin + i ) ) end
		end
	end

	if width > 1 then
		util.tmerge(cells, simquery.fillOrthogonalCross(sim, xOrigin, yOrigin, math.max(rangeMin, 1), width - 1, 1))
	end

	return cells
end