local simquery = include("sim/simquery")
local simdefs = include("sim/simdefs")

local canMoveUnit
local canPathBetween
local canStaticPath
local new_canMoveUnit
local new_canPathBetween
local new_canStaticPath

local function addDoorGhosting( sim )
	if not sim._simulacrumMovementCount then
		sim._simulacrumMovementCount = 1
		local modifyExit = sim.modifyExit
		local moveUnit = sim.moveUnit
		sim.modifyExit = function( self, cell, dir, exitOp, unit, ... ) 
			if unit and unit:getTraits().ghostsDoors and unit:getTraits().movePath then
				return
			end
		
			return modifyExit( self, cell, dir, exitOp, unit, ... )
		end
	else
		sim._simulacrumMovementCount = sim._simulacrumMovementCount + 1
	end
end

simquery.addSimulacrumMovement = function( localSim )
	addDoorGhosting( localSim )
	
	-- Update canMoveUnit (used by sim:moveUnit) to fix the specific case of ghosting through locked doors
	if not new_canMoveUnit then
		canMoveUnit = simquery.canMoveUnit
		new_canMoveUnit = function( sim, unit, x, y )
			if unit and unit:getTraits().ghostsDoors then
				local ok, reason = canMoveUnit( sim, unit, x, y )
				if reason == simdefs.CANMOVE_STATIC_IMPASS then
					local startcell = sim:getCell( unit:getLocation() )
					local endcell = sim:getCell( x, y )
					local dir = simquery.getDirectionFromDelta( endcell.x - startcell.x, endcell.y - startcell.y )
					local exit = startcell.exits[ dir ]
					
					if exit and exit.closed then
						exit.closed = false
						ok, reason = canMoveUnit( sim, unit, x, y )
						exit.closed = true
					end
				end
				
				return ok, reason
			end
			
			return canMoveUnit( sim, unit, x, y )
		end
		
		simquery.canMoveUnit = new_canMoveUnit
	end
	
	-- Update canPathBetween to allow pathing through doors and/or walls
	-- Also stops frightened guards from pathing through the phantasm
	if not new_canPathBetween then
		canPathBetween = simquery.canPathBetween
		new_canPathBetween = function( cellquery, unit, startcell, endcell )
			if unit and unit:getTraits().ghostsDoors then
				if unit:getTraits().ghostsAllMovement then
					return endcell
				elseif endcell and (startcell.x == endcell.x or startcell.y == endcell.y) then
					local dx, dy = endcell.x - startcell.x, endcell.y - startcell.y
					local dir = simquery.getDirectionFromDelta( dx, dy )
					local exit = startcell.exits[ dir ]
					
					return exit and exit.cell.x == endcell.x and exit.cell.y == endcell.y
				end
			end
			
			if unit and endcell and unit.huntedByPhantasmalKiller then
				local x1, y1 = unit.huntedByPhantasmalKiller:getLocation()
				if endcell.x == x1 and endcell.y == y1 then
					return false
				end
			end
			
			return canPathBetween( cellquery, unit, startcell, endcell )
		end
		
		simquery.canPathBetween = new_canPathBetween
	end
	
	if not new_canStaticPath then
		canStaticPath = simquery.canStaticPath
		new_canStaticPath = function( cellquery, unit, startcell, endcell )
			if unit and unit:getTraits().ghostsAllMovement then
				return endcell
			end
			
			return canStaticPath( cellquery, unit, startcell, endcell )
		end
		
		simquery.canStaticPath = new_canStaticPath
	end
end

simquery.removeSimulacrumMovement = function( sim )
	if sim then
		sim._simulacrumMovementCount = sim._simulacrumMovementCount - 1
	end
	
	if sim == nil or sim._simulacrumMovementCount <= 0 then
		if simquery.canMoveUnit == new_canMoveUnit then
			simquery.canMoveUnit = canMoveUnit
			new_canMoveUnit = nil
			canMoveUnit = nil
		end
		if simquery.canPathBetween == new_canPathBetween then
			simquery.canPathBetween = canPathBetween
			new_canPathBetween = nil
			canPathBetween = nil
		end
		if simquery.canStaticPath == new_canStaticPath then
			simquery.canStaticPath = canStaticPath
			new_canStaticPath = nil
			canStaticPath = nil
		end
	end
end

local _checkIsCover = simquery.checkIsCover
local _checkCellCover = simquery.checkCellCover
function simquery.checkIsCover( sim, cell, ... )
	if cell.selfCover and cell.selfCover > 0 then
		return true
	else
		return _checkIsCover( sim, cell, ... )
	end
end

function simquery.checkCellCover( sim, x0, y0, x1, y1 )
	local targetCell = sim:getCell( x1, y1 )
	if targetCell.selfCover and targetCell.selfCover > 0 then
		return true
	else
		return _checkCellCover( sim, x0, y0, x1, y1 )
	end
end