local util = include("client_util")
local array = include("modules/array")
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )

local baseCalculateShotSuccess = simquery.calculateShotSuccess
function simquery.calculateShotSuccess( sim, sourceUnit, targetUnit, equipped )
	if not equipped then
		return baseCalculateShotSuccess( sim, sourceUnit, targetUnit, equipped )
	end
	
	--local temporary_trait_buffs = {}
	--if not equipped:getTraits().coit_temporary_buffs_applied then
	--	--trace weapons bonus against tagged targets
	--	if equipped and equipped:getTraits().coit_weapon_trace_buffs and targetUnit and targetUnit:getTraits().tagged then
	--		for _, buff in ipairs( equipped:getTraits().coit_weapon_trace_buffs ) do
	--			if type(buff.value) == "boolean" and not equipped:getTraits()[buff.trait] then
	--				equipped:getTraits()[buff.trait] = buff.value
	--				table.insert( temporary_trait_buffs, {trait = buff.trait, value = buff.value } )
	--			elseif type(buff.value) == "number" then
	--				equipped:getTraits()[buff.trait] = (equipped:getTraits()[buff.trait] or 0) + buff.value
	--				table.insert( temporary_trait_buffs, {trait = buff.trait, value = buff.value } )
	--			end
	--		end
	--	end
	--end
	
	local shot = baseCalculateShotSuccess( sim, sourceUnit, targetUnit, equipped )
	
	if equipped:getTraits().coit_ammo_cost_per_armor then
		if targetUnit then
			local target_armor = 0
			local bonus_pierce = (shot.armorPiercing or equipped:getTraits().armorPiercing) - equipped:getTraits().armorPiercing
			
			if targetUnit:getArmor() then
			    	target_armor = target_armor + targetUnit:getArmor()
		    	end
			if equipped:getTraits().ignoreArmor then
				target_armor = 0
			else
		    		target_armor = math.max( 0, target_armor - sourceUnit:countArmorPiercingUpgrades("addArmorPiercingRanged") )
			end
			
			local pierce_needed = target_armor - bonus_pierce
			local ammo_cost = 1
			if pierce_needed > 0 then
				ammo_cost = math.max( math.ceil( pierce_needed * equipped:getTraits().coit_ammo_cost_per_armor ), 1 )
			end
			if equipped:getTraits().ammo and equipped:getTraits().ammo < ammo_cost then
				shot.armorBlocked = true
			end
		end
	end
	
	if equipped:getTraits().coit_can_mark then
		shot.coit_mark_armor = true
		if targetUnit and targetUnit:getArmor() <= 0 then
			shot.coit_mark_armor_noArmor = true
		end
	end
	
	----remove temporary buffs again
	--for _, buff in ipairs( temporary_trait_buffs ) do
	--	if type(buff.value) == "boolean" then
	--		equipped:getTraits()[buff.trait] = nil
	--	elseif type(buff.value) == "number" then
	--		assert( equipped:getTraits()[buff.trait] )
	--		equipped:getTraits()[buff.trait] = equipped:getTraits()[buff.trait] - buff.value
	--	end
	--end

	return shot
end

local baseCalculateDamageAndArmorForMelee = simquery.calculateDamageAndArmorForMelee
function simquery.calculateDamageAndArmorForMelee( sim, sourceUnit, targetUnit )
	local meleeDamage, armorPiercing, armor = baseCalculateDamageAndArmorForMelee( sim, sourceUnit, targetUnit )

	--Nika's and Decker's temporary buff
	local player = sourceUnit:getPlayerOwner()
	if player then
		if player:getTraits().coit_melee_damage then
			meleeDamage = meleeDamage + player:getTraits().coit_melee_damage
		end
	end

	return meleeDamage, armorPiercing, armor
end
