local util = include( "modules/util" )
local unitdefs = include( "sim/unitdefs" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local simunit = include( "sim/simunit" )
local simtrap = include( "sim/units/simtrap" )

local simtrap2 = { ClassType = "simtrap" }

function simtrap2:onWarp( sim, oldcell, cell )
	if oldcell == nil and cell ~= nil then
		sim:addTrigger( simdefs.TRG_UNIT_USEDOOR, self )

	elseif oldcell ~= nil and cell == nil then
        local exit = oldcell.exits[ self:getFacing() ]
        assert( exit.trapped )
        exit.trapped = nil

		sim:removeTrigger( simdefs.TRG_UNIT_USEDOOR, self )		
	end
end

function simtrap2:performTrap( sim, cell, unit )
	if self:getTraits().soundRange then
		sim:emitSound( { path = "SpySociety/HitResponse/hitby_shocktrap", range = self:getTraits().soundRange }, cell.x, cell.y, nil )
	else
		sim:emitSound( simdefs.SOUND_SHOCKTRAP, cell.x, cell.y, nil )
	end
	sim:dispatchEvent( simdefs.EV_CAM_PAN, { cell.x, cell.y } )	

	sim:startTrackerQueue(true)				
	sim:startDaemonQueue()			
	sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = self:getID() } )

	local x1, y1 = self:getLocation()
	local dmgt

	if self:getTraits().lethal and self:getTraits().lethal == true then
		dmgt = {ko = nil, damage = self:getTraits().stun}
	else
		dmgt = {ko = true, damage = self:getTraits().stun}
	end

	if self:getTraits().range then
		local cells = simquery.fillCircle( sim, x1, y1, self:getTraits().range, 0)
		local raycastCells = {}

		for i, cell in ipairs(cells) do
			if cell ~= sim:getCell(x1, y1) then
				local raycastX, raycastY = self._sim:getLOS():raycast(x1, y1, cell.x, cell.y)
				if raycastX == cell.x and raycastY == cell.y then
					table.insert(raycastCells, cell)
				end
			end
		end

		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x1, y = y1, range = self:getTraits().range, cells = raycastCells} )
		sim:dispatchEvent( simdefs.EV_KO_GROUP, true )

		for i, cell in ipairs(raycastCells) do
			for i, cellUnit in ipairs( cell.units ) do
				local localdmgt = dmgt

				if sim:isVersion("0.17.12") then
					localdmgt = util.tcopy(dmgt)
				end

				if not cellUnit:isKO() and cellUnit:getTraits().isAgent and ( ( self:getTraits().ignoreArmor == nil or self:getTraits().ignoreArmor == true ) or ( self:getTraits().ignoreArmor == false and cellUnit:getArmor() <= (self:getTraits().armorPiercing or 0) ) ) then
					if cellUnit:getPlayerOwner() == sim:getPC() and not self:getTraits().friendlyFire then

					elseif cellUnit:getTraits().isDrone then
						if localdmgt.ko == true then
							cellUnit:processEMP(localdmgt.damage, false, false, self:getTraits().ignoreMagReinf)
						else
							sim:hitUnit( self, cellUnit, localdmgt )
						end
					else
				    		if localdmgt.ko == true and sim:isVersion("0.17.5") then
							localdmgt.damage = unit:processKOresist( localdmgt.damage )
				    		end						
						sim:hitUnit( self, cellUnit, localdmgt )
					end
				end
			end
		end

        	sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
	elseif unit and unit:getTraits().isAgent and ( ( self:getTraits().ignoreArmor == nil or self:getTraits().ignoreArmor == true ) or ( self:getTraits().ignoreArmor == false and unit:getArmor() <= (self:getTraits().armorPiercing or 0) ) ) then
		if unit:getTraits().isDrone then
			if dmgt.ko == true then
				unit:processEMP(dmgt.damage, false, false, self:getTraits().ignoreMagReinf)
			else
				sim:hitUnit( self, unit, dmgt )
			end
		else
			if dmgt.ko == true and sim:isVersion("0.17.5") then
				dmgt.damage = unit:processKOresist( dmgt.damage )
			end						
			sim:hitUnit( self, unit, dmgt )
		end
	end

	local linkedTrap = sim:getUnit( self:getTraits().linkedTrap )
	local unitID, linkedUnitID = self:getID()

	sim:warpUnit( self, nil )

	if self:getTraits().range then
		if linkedTrap and linkedTrap:getLocation() then
			linkedTrap:performTrap( sim, sim:getCell(linkedTrap:getLocation()), unit )
		end
	else
		if linkedTrap then
			linkedUnitID = linkedTrap:getID()
			sim:warpUnit( linkedTrap, nil )
		end

		if linkedTrap then
			sim:despawnUnit( linkedTrap )
		end
	end
	sim:despawnUnit( self )

	for i,player in ipairs(sim:getPlayers()) do			
		player:glimpseUnit(sim, unitID)
		if linkedUnitID and self:getTraits().range then
			player:glimpseUnit(sim, linkedUnitID)
		end
	end

	sim:startTrackerQueue(false)				
	sim:processDaemonQueue()			
end

function simtrap2:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_UNIT_USEDOOR and evData.exitOp ~= simdefs.EXITOP_CLOSE then
		local cell = sim:getCell( self:getLocation() )
		if evData.cell == cell or evData.tocell == cell then
			self:performTrap( sim, cell, evData.unit )
		end
	end
end

local function createTrap( unitData, sim )
	return simunit.createUnit( unitData, sim, simtrap2 )
end

simfactory.register( createTrap )

function simtrap.applyToDoor( sim, cell, direction, unit, userUnit )
	assert( simquery.isDoorExit( cell.exits[ direction ] ))

	local player = userUnit:getPlayerOwner()
	local toCell = cell.exits[ direction ].cell
	local rdir = simquery.getReverseDirection( direction )

	local trap1 = simfactory.createUnit( unitdefs.prop_templates.trap_shock_door, sim )
	trap1:getTraits().stun = unit:getTraits().stun
	trap1:getTraits().range = unit:getTraits().range
	trap1:getTraits().lethal = unit:getTraits().lethal
	trap1:getTraits().ignoreArmor = unit:getTraits().ignoreArmor
	trap1:getTraits().armorPiercing = unit:getTraits().armorPiercing
	trap1:getTraits().friendlyFire = unit:getTraits().friendlyFire
	trap1:getTraits().ignoreMagReinf = unit:getTraits().ignoreMagReinf
	trap1:getTraits().soundRange = unit:getTraits().soundRange
	trap1:setFacing( direction )
	sim:spawnUnit( trap1 )
	sim:warpUnit( trap1, cell)
	trap1:setPlayerOwner(player)
	
	local trap2 = simfactory.createUnit( unitdefs.prop_templates.trap_shock_door, sim )
	trap2:getTraits().stun = unit:getTraits().stun
	trap2:getTraits().range = unit:getTraits().range
	trap2:getTraits().lethal = unit:getTraits().lethal
	trap2:getTraits().ignoreArmor = unit:getTraits().ignoreArmor
	trap2:getTraits().armorPiercing = unit:getTraits().armorPiercing
	trap2:getTraits().friendlyFire = unit:getTraits().friendlyFire
	trap2:getTraits().ignoreMagReinf = unit:getTraits().ignoreMagReinf
	trap2:getTraits().soundRange = unit:getTraits().soundRange
	trap2:setFacing( rdir )
	sim:spawnUnit( trap2 )
	sim:warpUnit( trap2, toCell )
	trap2:setPlayerOwner(player)

	trap1:getTraits().linkedTrap = trap2:getID()
	trap2:getTraits().linkedTrap = trap1:getID()

	cell.exits[ direction ].trapped = true
	toCell.exits[ rdir ].trapped = true
end