local util = include("client_util")
local simunit = include( "sim/simunit" )
local simskill = include( "sim/simskill" )
local simfactory = include( "sim/simfactory" )
local simdefs = include( "sim/simdefs" )

local baseCanUseAbility = simunit.canUseAbility
function simunit:canUseAbility( sim, abilityDef, abilityOwner, ... )
	if self:getTraits().maa_inside_mainframe and ( abilityDef.maa_mp_ability or abilityOwner:getTraits().mainframe_item ) then
		local playerOwner = self:getPlayerOwner()
		if playerOwner and sim:getCurrentPlayer() ~= playerOwner then
			return false
		end

		local canUse, reason = abilityDef:canUseAbility( sim, abilityOwner, self, ... )
		if not canUse then
			return canUse, reason
		end

		return true
	end

	return baseCanUseAbility( self, sim, abilityDef, abilityOwner, ... )
end

local baseOnStartTurn = simunit.onStartTurn
function simunit:onStartTurn( sim )
	baseOnStartTurn( self, sim )

	for i, childUnit in ipairs(self:getChildren()) do
		if childUnit:getTraits().maa_ap_refreshs_turn then
			childUnit:getTraits().maa_ap_refreshs_turn = nil
		end
	end
	
	self:getTraits().maa_armor_mark = nil
	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
end

function simunit:giveSkill( skillID, level, sim )
	local skill = self:hasSkill( skillID )
	if not skill then
		skill = simskill( skillID, level, sim, self )
		table.insert( self._skills, skill )
	end
	
	return skill
end

local baseGetArmor = simunit.getArmor
function simunit:getArmor()
	local armor = baseGetArmor( self )
	
	armor = math.max( armor - (self:getTraits().maa_armor_mark or 0), 0 )
	
	return armor
end


--local baseCreateUnit = simunit.createUnit
--function simunit.createUnit( unitData, sim, ... )
--	local t = baseCreateUnit( unitData, sim, ... )
--	
--	--late skill levelup for skills that need the unit's children to be instantiated
--	if t:getSkills( ) then
--		for _, skill in ipairs( t:getSkills( ) ) do
--			for i=1, skill:getCurrentLevel() do
--				if skill:getLevel( i ).onLearn_late then
--					skill:getLevel( i ).onLearn_late( sim, t )
--				end
--			end
--		end
--	end
--	
--	return t
--end
