local util = include( "modules/util" )
local mainframe = include( "sim/mainframe" )
local simdefs = include( "sim/simdefs" )
local simunit = include( "sim/simunit" )
local simquery = include( "sim/simquery" )

local AUGMENT_TXT_COLOR = {r=255/255,g=255/255,b=51/255,a=1 }

function simunit:onStartTurn( sim )
	if sim:isVersion("0.17.5") then
		if self:getPlayerOwner() == sim:getPC() then
			self:tickKO( sim )
		end
	end

	if sim:isVersion("0.17.9") then
		self:getTraits().augment_nika_2_triggered = nil	
	end	

	if self:getTraits().kinetic_capacitor_bonus_charged then
		self:getTraits().kinetic_capacitor_bonus_charged = nil
	end
	
	if self:getTraits().unlimitedAttacks then
		self:getTraits().unlimitedAttacks = nil
	end

	if self:getTraits().tempMeleeBoost then
		self:getTraits().tempMeleeBoost = 0
	end

	if self:getTraits().mpAttackRestrictMax then
		self:getTraits().mpAttackRestrict = self:getTraits().mpAttackRestrictMax
	end

    	if self:getBrain() then
        	self:getBrain():onStartTurn( sim )
   	end

	if self._abilities then
		for i,ability in ipairs(self._abilities) do
			if ability.automatic then
				ability:executeAbility( sim, self, self)			
			end		
		end
	end

	local traits = self:getTraits()

	if traits.shieldsMax ~= nil then
		if traits.shields < traits.shieldsMax then
			traits.shields = traits.shields + 1 
		end
	end
	
	if traits.apMax then
		traits.ap = traits.apMax
	end

    	if self:getPlayerOwner() ~= nil then
    		local power = 0
	   	for i = 1, self:countAugments( "augment_distributed_processing" ) do
		    	if sim:getTurnCount() % 2 == 0 then			   
				power = power + 1
		    	end
	    	end
	    	if power > 0 then
	    		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = util.sformat( STRINGS.UI.FLY_TXT.PLUS_PWR, power), unit = self, color={r=255/255,g=255/255,b=51/255,a=1}} )
	    		self:getPlayerOwner():addCPUs( power )
		end
    	end

	if traits.invisDuration then
		traits.invisDuration = traits.invisDuration - 1
		if traits.invisDuration <= 0 then
			traits.invisDuration = nil
			self:setInvisible( false )
			if traits.cloakDistance then
				traits.cloakDistance = nil
			end
		end		
	end

	if traits.genericPiercing then 
		traits.genericPiercing = nil
	end 

	for i,childUnit in ipairs(self:getChildren()) do
		local childTraits = childUnit:getTraits()

		if (childTraits.cooldown or 0) > 0 then
			childTraits.cooldown = childTraits.cooldown - 1
		end

		if childTraits.energyWeapon == "active" then
            		childTraits.energyWeapon = "used"
		end

		if childTraits.nopwr_guards then 
			childTraits.nopwr_guards = {}
		end 
	
		if childTraits.extraAPMax then		
			childTraits.extraAP = childTraits.extraAPMax
		end
	end

	if traits.mainframeRecapture and not self:isKO() then
		self:recaptureMainframeItems(sim,traits.mainframeRecapture)
	end

	traits.temporaryProtect = nil	

    	self:getTraits().overloadCount = nil
	self:checkOverload(sim)

	if self:getTraits().floatTxtQue then
		for i,floatItem in ipairs(self:getTraits().floatTxtQue) do
			local x0,y0 = self:getLocation()
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=floatItem.txt,x=x0,y=y0,color=floatItem.color} ) 
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = self } )
		end
		self:getTraits().floatTxtQue = {}
	end

	if traits.pulseScan and not self:isKO() and self:getPlayerOwner():isNPC() then		
	    	local cells = self:getAreaCells()
	    	sim:dispatchEvent( simdefs.EV_PULSE_SCAN, {unit=self, cells=cells, range=self:getTraits().range} ) 
		for i, cell in ipairs(cells) do
			for i, cellUnit in ipairs( cell.units ) do
				sim:scanCell(self, cell, true)
			end
		end		
	end
	
	if traits.data_hacking then
		if self:getPlayerOwner():isPC() then
			local hackUnit = sim:getUnit(traits.data_hacking)
			hackUnit:progressHack()
		end
	end	
end

function simunit:doAugmentUpgrade(childUnit, useAbilityIndex)
	childUnit:getTraits().installed = true

    	if self._sim and not childUnit:getUnitData().traits.installed then
		self._sim:getStats():incStat( "times_augmented" )
	end

	if childUnit:getTraits().addAbilities then
		if useAbilityIndex then
			self:giveAbility(childUnit:getTraits().addAbilities, self._abilityIndex)
			self._abilityIndex = self._abilityIndex + 1
		else
			self:giveAbility(childUnit:getTraits().addAbilities)
		end
		
	end

	if childUnit:getTraits().addTrait then
		for i,trait in ipairs(childUnit:getTraits().addTrait) do
			if trait[2] then
				self:getTraits()[trait[1]] = trait[2]
			else
				self:getTraits()[trait[1]] = false
			end
		end
	end

	if childUnit:getTraits().modSkill then
		local skill = self:getSkills()[childUnit:getTraits().modSkill]

		if skill then
			while skill._currentLevel > childUnit:getTraits().modSkillStat do
				skill:levelDown( self._sim, self )
			end

			while skill._currentLevel < childUnit:getTraits().modSkillStat do
				skill:levelUp( self._sim, self )
			end
		end
	end

	if childUnit:getTraits().modSkillLock then
		if not self:getTraits().skillLock then
			self:getTraits().skillLock = {}
		end		
		for i,skill in ipairs(childUnit:getTraits().modSkillLock) do
			self:getTraits().skillLock[skill] = true
		end
	end
	
	if childUnit:getTraits().modTrait then
		for i,trait in ipairs(childUnit:getTraits().modTrait) do
			if not self:getTraits()[trait[1]] then
				self:getTraits()[trait[1]] = 0
			end
			self:getTraits()[trait[1]] = self:getTraits()[trait[1]] + trait[2]			
			if trait[1] == "mpMax" then
				self:getTraits().mp = self:getTraits().mp + trait[2]
			end
		end	
	end


	if childUnit:getTraits().addInventory then
		self:getTraits().inventoryMaxSize = self:getTraits().inventoryMaxSize + childUnit:getTraits().addInventory
	end

	if self._sim and self:getAugmentCount() >= simdefs.DEFAULT_AUGMENT_CAPACITY then
		self._sim:dispatchEvent( simdefs.EV_ACHIEVEMENT, "MEAT_MACHINE" )
    	end
end

function simunit:tickKO( sim )
	if self:isKO() and not self:isDead() and not self:getTraits().paralyzed then
		local cell = sim:getCell( self:getLocation() )
		if cell then
			for i, cellUnit in ipairs( cell.units ) do
				if simquery.isAgent( cellUnit ) and not cellUnit:isKO() then
					return
				end
			end
		else
			return
		end

		self:getTraits().koTimer = self:getTraits().koTimer - 1
		if self:getTraits().koTimer <= 0 then
			if self:getTraits().mainframe_suppress_rangeMax then
				if not self:getPlayerOwner():isPC() then
					self:getTraits().mainframe_suppress_range = self:getTraits().mainframe_suppress_rangeMax
				end
			end

            		if self:getSounds().reboot_end then
	        		local x0,y0 = self:getLocation()
				self._sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound=self:getSounds().reboot_end, x=x0,y=y0} )                
        		end

 			if self:getTraits().koDaemon and self:getTraits().mainframe_device then 				
				mainframe.revokeDaemonHost( sim, self )
 			end		

			if self:getTraits().W93_rangedKO_warp then
				self:getTraits().W93_rangedKO = nil
			end

			if self:getTraits().W93_rangedKO_warp then
				self:getTraits().W93_rangedKO_warp = nil
			end

			self:setKO( sim, nil )
	
			sim:processReactions(self)

			if self:getPlayerOwner() ~= sim:getPC() then
				local pc = sim:getPC()
				for i, unit in pairs( pc:getAgents() ) do
					local count, augments = unit:countAugments( "boost_AP_on_wakeup" )
					if count > 0 then
			        		local x2,y2 = unit:getLocation()
			        		sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit  } )
					    	sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt = augments[1]:getTraits().boost_AP_on_wakeup_txt, x = x2, y = y2, color=AUGMENT_TXT_COLOR} )
					    	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
						if type(augments[1]:getTraits().boost_AP_on_wakeup) == "number" then						
							unit:getTraits().mp = unit:getTraits().mp + augments[1]:getTraits().boost_AP_on_wakeup
							sim:getPC():glimpseUnit( sim, self:getID() )
						else
							unit:getTraits().mp = unit:getTraits().mp + 2
						end
					end
				end
			end			
		end
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )			
	end
end

function simunit:processEMP( bootTime, noEmpFX, noAttack, ignoreMagRei )
	local empResisted = false
	local doEMPeffect = false

	if self:getTraits().mainframe_status == "off" then
		if self:getTraits().mainframe_booting then
        		self:getTraits().mainframe_booting = bootTime
    		end
	elseif self:getTraits().mainframe_status ~= nil and self:getTraits().mainframe_status ~= "off" then
		local EMP_FIREWALL_BREAK_STRENGTH = 2
		if self:getTraits().magnetic_reinforcement and self:getTraits().mainframe_ice > 2 and not ignoreMagRei then
			empResisted = true
			local x1,y1 = self:getLocation()
			local sim = self._sim 
			mainframe.breakIce( sim, self, EMP_FIREWALL_BREAK_STRENGTH )
			self._sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.TOOLTIPS.MAGNETIC_REINFOREMENTS,x=x1,y=y1,color={r=255/255,g=255/255,b=255/255,a=1}} )
		else
			self:getTraits().mainframe_status_old = self:getTraits().mainframe_status
	        	if self.deactivate then
			    	self:deactivate( self._sim )
			end

			if self:getTraits().firewallShield then
				self:getTraits().shields = 0			
			end	
			self:getTraits().mainframe_status = "off"
			self:getTraits().mainframe_booting = bootTime
	       		if self:getTraits().switched then
				self:getTraits().switched  = nil 
				self:setSwitchStage(self._sim,"off")
			end	
			if self:getTraits().progress then
				self:setSwitchStage(self._sim,"off")
			end			
			if self:getTraits().hacker then
				local hacker = self._sim:getUnit(self:getTraits().hacker)	
				hacker:getTraits().data_hacking = nil
				hacker:getSounds().spot = nil
				self._sim:dispatchEvent( simdefs.EV_UNIT_TINKER_END, { unit = hacker } ) 	
				self._sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = hacker })
				self:getTraits().hacker = nil 
			end
			if self:getTraits().mainframe_suppress_rangeMax then
				self:getTraits().mainframe_suppress_range = 0
			end		

			doEMPeffect = true
	    	end
	elseif not empResisted and self:getTraits().heartMonitor == "enabled" then
		doEMPeffect = true
	end
	if doEMPeffect then
		if not noEmpFX then
	   		local x0,y0 = self:getLocation()
			self._sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound="SpySociety/HitResponse/hitby_distrupter_flesh", x=x0,y=y0} )
			self._sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self, fx = "emp" } )			
	    	end		
	    	self._sim:triggerEvent( simdefs.TRG_UNIT_EMP, self )
	end

	if not empResisted and self:getTraits().heartMonitor=="enabled" then
		local x1,y1 = self:getLocation()

		if self:getTraits().improved_heart_monitor then
			self._sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.IMPROVED_HEART_MONITOR,x=x1,y=y1,color={r=1,g=1,b=41/255,a=1}} ) 
		else
			self:getTraits().heartMonitor="disabled"
			self._sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.MONITOR_DISABLED,x=x1,y=y1,color={r=255/255,g=255/255,b=255/255,a=1}} )			
		end
	end

	self._sim:getPC():glimpseUnit( self._sim, self:getID() )
end

function simunit:useAP( sim )
	local x1,y1 = self:getLocation()
	local apTxt = STRINGS.UI.FLY_TXT.ATTACK_USED

	sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=apTxt,x=x1,y=y1,color={r=1,g=1,b=1,a=1}} )
	sim:dispatchEvent( simdefs.EV_HUD_MPUSED, self )

	if self:getTraits().mpAttackRestrict then
		self:addMP( -self:getTraits().mpAttackRestrict )
	end

	if self:getTraits().actionAP then 
		sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = self } )
		if type(self:getTraits().actionAP) ~= "table" then
 			local ADRENAL_BONUS=3
			if self:getPlayerOwner() ~= sim:getCurrentPlayer() then
				if not self:getTraits().floatTxtQue then
					self:getTraits().floatTxtQue = {}
				end
				table.insert(self:getTraits().floatTxtQue,{txt=util.sformat(STRINGS.UI.FLY_TXT.ADRENAL_REGULATOR,ADRENAL_BONUS),color={r=255/255,g=178/255,b=102/255,a=1}})		
			else		
				sim:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT, { unit = self, txt=util.sformat(STRINGS.UI.FLY_TXT.ADRENAL_REGULATOR, ADRENAL_BONUS), x=x1,y=y1,color={r=255/255,g=178/255,b=102/255,a=1}  } )	-- 
			end
			self:addMP( ADRENAL_BONUS )
		else
			for _, actionTrait in pairs(self:getTraits().actionAP) do
				if actionTrait[1] == "mp" then
					self:addMP( actionTrait[2] )
				elseif actionTrait[1] == "ap" then
					self:getTraits().ap = self:getTraits().ap + actionTrait[2]
				elseif actionTrait[1] == "genericPiercing" then
					self:getTraits().genericPiercing = (self:getTraits().genericPiercing or 0) + actionTrait[2]
				elseif actionTrait[1] == "meleeDamage" then
					self:getTraits().meleeDamage = (self:getTraits().meleeDamage or 0) + actionTrait[2]
				elseif actionTrait[1] == "tempMeleeBoost" then
					self:getTraits().tempMeleeBoost = (self:getTraits().tempMeleeBoost or 0) + actionTrait[2]
				elseif actionTrait[1] == "mpAttackRestrict" then
					self:getTraits().mpAttackRestrict = (self:getTraits().mpAttackRestrict or 0) + actionTrait[2]
				elseif actionTrait[1] == "unlimitedAttacks" then
					self:getTraits().unlimitedAttacks = actionTrait[2]
				end
			end
		end
	end 

	local apUsed = false	
	for i,unit in ipairs(self:getChildren( )) do
		if unit:getTraits().extraAP and unit:getTraits().extraAP > 0 then
			unit:getTraits().extraAP = unit:getTraits().extraAP -1 
			apUsed = true
			break
		end
	end	

	if self:getTraits().unlimitedAttacks then
		apUsed = true
	end

	if apUsed == false then
		self:getTraits().ap = self:getTraits().ap - 1
	end

	if self:getTraits().monster_hacking then 
		self:getTraits().monster_hacking = false
		self:getSounds().spot = nil
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self })
	end

	self:stopHacking(sim)
	self:resetAllAiming()
	sim:processReactions( self )
end

function simunit:onEndTurn( sim )
	local traits = self:getTraits()

	traits.movePath = nil
	traits.interrupted = nil

	if traits.controlTimer and sim:getCurrentPlayer():isNPC() then 
		traits.controlTimer = traits.controlTimer - 1 
		if traits.controlTimer <= 0 then 
			if self.loseControl then 
				self:loseControl( sim )
			end
		end
	end

	if traits.mainframe_booting and traits.mainframe_status == "off" then
		if sim:getCurrentPlayer() and sim:getCurrentPlayer():isPC() then
			traits.mainframe_booting = traits.mainframe_booting - 1

			if traits.mainframe_booting <= 0 then
				traits.mainframe_booting = nil

				if self:getSounds().reboot_end then
					local x0,y0 = self:getLocation()
					self._sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound=self:getSounds().reboot_end, x=x0,y=y0} )                
				end

				if traits.firewallShield and traits.mainframe_ice > 0 then
					traits.shields = 1
				end	

				if traits.unArmor then 
					traits.armor = 0
					traits.unArmor = false 
				end 

				if self.activate then                	
					traits.mainframe_status = "inactive"
					if traits.mainframe_status_old and traits.mainframe_status_old == "active" and self:canActivate( sim ) then
						self:activate( sim )
					end
				else
					traits.mainframe_status = "active"
					self._sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
				end

				if traits.mainframe_suppress_rangeMax then					
					if not self:getPlayerOwner() or self:getPlayerOwner():isNPC() then
						traits.mainframe_suppress_range = traits.mainframe_suppress_rangeMax
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )	
					end
				end	

				if sim:getPC():hasSeen( self ) then
					sim:getPC():glimpseUnit( sim, self:getID() )
				end
			end
		end
	end

	if traits.patrolObserved then
		traits.patrolObserved = nil
		self:destroyTab()
	end

	if not traits.isDrone or self:getPlayerOwner():isNPC() then 
		traits.sneaking = self:getUnitData().traits.sneaking
	end

	if traits.investigated then
		traits.investigated = true
	end
    
	if sim:getCurrentPlayer() == self:getPlayerOwner() then
		if traits.mpMax then
			traits.mp = self:getMPMax()
			traits.netDownlinkMp = nil
			traits.mpUsed = nil

			local totalPenalty = 0
			for i,unit in ipairs (self:getChildren()) do
				if unit:getTraits().mp_penalty then
					traits.mp = traits.mp - unit:getTraits().mp_penalty
					totalPenalty = totalPenalty + unit:getTraits().mp_penalty
				end
			end
			if totalPenalty  > 0 then
				local x1,y1 = self:getLocation()
				self:getSim():dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, { txt=util.sformat(STRINGS.UI.FLY_TXT.HEAVY, totalPenalty ), x=x1,y=y1, color={r=163/255,g=0,b=0,a=1}, skipQue=false } ) 
			end

			if self:getPlayerOwner() ~= nil and self:getPlayerOwner() == sim:getPC() then 
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
			end             
		end

		traits.prismBonus = nil -- augment_prism_2.
		traits.tonyBonus = nil -- augment_tony_2.

		if sim:isVersion("0.17.5") then
			if self:getPlayerOwner() ~= sim:getPC() then
				self:tickKO( sim )
			end
		else 
			self:tickKO( sim )
		end
	end

	if traits.switched and not traits.unlocked then
		self:resetSwitch(sim)
		if traits.mainframe_status ~= "off" then
			sim:triggerEvent( simdefs.TRG_MAP_EVENT, { event =simdefs.MAP_EVENTS.RESET_SWITCH })
			for i,childunit in pairs(sim:getAllUnits())do
				if childunit.switched then
					childunit:resetSwitch(sim)
				end
			end	
		end
	end
end

function simunit:onDamage( damage )
	self:getTraits().wounds = self:getTraits().wounds + damage

	if self:getTraits().wounds >= self:getTraits().woundsMax then
		self:getTraits().dead = true		
	end

	if self:isDead() then
		if self:getTraits().canBeCritical then					
			self:setKO( self._sim, 3 )
            		self._sim:dispatchEvent( simdefs.EV_SHOW_MODAL_REWIND )
            		if self._sim:getTags().rewindsLeft > 0 and not self._sim:getPC():isNeutralized( self._sim ) then    	
            			if self._sim:getCurrentPlayer():isPC() then
            				self._sim:dispatchEvent( simdefs.EV_BLINK_REWIND )
        			else
					self._sim:getTags().blinkAtStartTurn = true
				end            	
        		end
		elseif self:getTraits().regenerative_nanocells then 
			self:setKO( self._sim, 1 )
		else 
			self:killUnit( self._sim )	
		end
		return
	end

	if damage ~= 0 and self:getTraits().isAgent and not self:getTraits().isDrone and not self:isDown() then
		self._sim:dispatchEvent( simdefs.EV_UNIT_HIT, { unit = self, result = 0 } )
	end
end