----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local mathutil = include( "modules/mathutil" )
local unitdefs = include("sim/unitdefs")
local propdefs = include("sim/unitdefs/propdefs")
local speechdefs = include("sim/speechdefs")

-----------------------------------------------------
-- Local functions
local frozenRenderFilter = { 
	shader=KLEIAnim.SHADER_FOW,
	r=85/255,
	g=222/255,
	b=227/255,
	a=0.9,
	lum=1.0, }

local function sameLocation( sim, self, targetUnit )
	if targetUnit:getLocation() and self:getLocation() then
		local selfCell = sim:getCell(self:getLocation())
		local unitCell = sim:getCell(targetUnit:getLocation())
		if selfCell == unitCell then
			-- log:write("same location true")
			return true
		end
	end
	-- log:write("same location false")
	return false
end

local function isViableTarget( unit )
	if (unit:getTraits().canKO or unit:getTraits().empKO) and not ((unit:getTraits().shields or 0) > 0) then
		if not unit:isKO() then
			return true
		end
	end
	return false
end

local function isBarrierTarget( unit )
	if (unit:getTraits().canKO or unit:getTraits().empKO) and ((unit:getTraits().shields or 0) > 0) then
		if not unit:isKO() then
			return true
		end
	end
	return false
end

local function spawnDecorativeIceCloud( sim, x0, y0 )
	local iceCloudUnit = simfactory.createUnit( propdefs.ITB_ice_cloud, sim )
	sim:spawnUnit( iceCloudUnit )
	sim:warpUnit( iceCloudUnit, sim:getCell(x0, y0) )
end

local icetrap = { ClassType = "ITB_iceMine" }

function icetrap:detonate( sim )
	local x0,y0 = self:getLocation()

	local units = self:getTargets( x0, y0 )

	local doFX = true
	-- spawnDecorativeIceCloud( sim, x0, y0 ) --handled by iceblock simunit instead
	
	for i, targetUnit in ipairs(units) do
		-- KO and spawn ice
		if doFX then
			sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = {targetUnit}, range = 0 } )
			doFX = false
		end
		sim:dispatchEvent( simdefs.EV_UNIT_HIT, {unit = targetUnit, result = 0})
			local iceCell = sim:getCell(targetUnit:getLocation())
			local unitData = unitdefs.lookupTemplate("ITB_iceblock_bethany")
			local iceUnit = simfactory.createUnit( unitData, sim )
			sim:spawnUnit( iceUnit )
			sim:warpUnit( iceUnit, iceCell )
			local newFacing = simquery.getReverseDirection(iceUnit:getFacing())
			if (sim:nextRand() <= 0.5) then
				iceUnit:setFacing( newFacing )
			end
			iceUnit:getTraits().frozenUnit = targetUnit:getID()

			targetUnit:setKO(sim, (self:getTraits().ko_duration or 3))

			local facing = targetUnit:getFacing()
			if (facing % 2 ~= 0) then
				local newOrthogFacing = facing + 1
				if newOrthogFacing > 7 then
					newOrthogFacing = 0
				end
				targetUnit:updateFacing( newOrthogFacing )
			end
			local frozenAnims = self:getTraits().possibleFrozenAnims
			local frozenAnimation = frozenAnims[sim:nextRand(1,#frozenAnims)]
			targetUnit:getTraits().frozenRenderFilter = frozenRenderFilter
			if not targetUnit:getTraits().isDrone then
				targetUnit:getTraits().frozenAnim = frozenAnimation
				targetUnit:getTraits().frozenByCryoGun = true
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
	end

	-- Destroy the DEVICE.
	sim:emitSound( simdefs.SOUND_SMALL, x0, y0, nil )			
	sim:processReactions(self)

	sim:warpUnit( self, nil )
	sim:despawnUnit( self )
end

function icetrap:disarm(sim, targetUnit) -- despawn without detonating, interrupt move and alert target
	local x0, y0 = self:getLocation()
	sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound="SpySociety/HitResponse/hitby_distrupter_flesh", x=x0,y=y0} )
	sim:emitSound( simdefs.SOUND_SMALL, x0, y0, nil )
	self:getTraits().sightable = true
	spawnDecorativeIceCloud( sim, x0, y0 )
	if targetUnit and targetUnit:isValid() and targetUnit:getBrain() and targetUnit:getBrain():getSenses() then
		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.BREACH_AGENTS.UI.IMMUNE,x=x0,y=y0,color={r=1,g=1,b=1,a=1}} )
		targetUnit:interruptMove(sim)
		local actions = include("sim/btree/actions")
		sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, units = {targetUnit}, range = 0 } )
		targetUnit:getBrain():getSenses():addInterest(x0, y0, simdefs.SENSE_SIGHT, simdefs.REASON_FOUNDOBJECT)
		-- sim:emitSpeech( targetUnit, speechdefs.HUNT_FOUNDOBJECT) -- dolookaround takes care of this
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		actions.DoLookAround(sim, targetUnit)
	end
	sim:warpUnit( self, nil )
	sim:despawnUnit( self )
end

function icetrap:onSpawn(sim)
	-- if not self:getTraits().trigger_mainframe then
		-- if self:getLocation() then
			-- log:write("adding warp trigger")
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self)
		-- else
			-- log:write("removing warp trigger")
			-- sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		-- end
	-- end
end 

function icetrap:onDespawn(sim)
	sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
end

function icetrap:onTrigger( sim, evType, evData )
	if evType == simdefs.TRG_UNIT_WARP then
		-- log:write("icemine warp trigger")
		if (evData.unit ~= self) and sameLocation(sim, self, evData.unit) then
			if isViableTarget( evData.unit ) then
			-- log:write("detonating ice mine")
				self:detonate( sim )	
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self} )
			elseif isBarrierTarget( evData.unit ) then
				self:disarm( sim, evData.unit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self} )
			end
		end
		
	end
end

function icetrap:getTargets( x0, y0 )

	local cells = {}
	-- currently no support for AoE ice mine.
	
	-- if self:getTraits().flash_pack then
		-- local currentCell = self:getSim():getCell( x0, y0 )
		-- cells = {currentCell}
		-- if self:getTraits().range then
			-- local fillCells = simquery.fillCircle( self:getSim(), x0, y0, self:getTraits().range, 0)

			-- for i, cell in ipairs(fillCells) do
				-- if cell ~= currentCell then
					-- local raycastX, raycastY = self._sim:getLOS():raycast(x0, y0, cell.x, cell.y)
					-- if raycastX == cell.x and raycastY == cell.y then
						-- table.insert(cells, cell)
					-- end
				-- end
			-- end
		-- end	   
	-- else
		-- cells = simquery.rasterCircle( self._sim, x0, y0, self:getTraits().range )
	-- end
	
	local currentCell = self:getSim():getCell( x0, y0 )
	cells = {currentCell}
	
    local units = {}
    for i, x, y in util.xypairs( cells ) do
        local cell = self._sim:getCell( x, y )
        if cell then
            for _, cellUnit in ipairs(cell.units) do
            	local player = self:getPlayerOwner()
                if cellUnit ~= self 
				-- and	simquery.isEnemyAgent( player, cellUnit)
				and isViableTarget( cellUnit )
				then
                    table.insert( units, cellUnit )
                end
            end
		end
	end

    return units
end

-- get cells for FLASH PACK
-- function icetrap:getExplodeCells(x0,y0)

	-- assert(self:getTraits().range)

	-- if not x0 and not y0 then
    	-- x0, y0 = self:getLocation()
    -- end

	-- local currentCell = self:getSim():getCell( x0, y0 )
	-- local cells = {currentCell}
	-- if self:getTraits().range then
		-- local coords = simquery.rasterCircle( self._sim, x0, y0, self:getTraits().range )

		-- for i=1,#coords-1,2 do
			-- local cell = self._sim:getCell(coords[i],coords[i+1])
			-- if cell then
			-- local raycastX, raycastY = self._sim:getLOS():raycast(x0, y0, cell.x, cell.y)			
				-- if raycastX == cell.x and raycastY == cell.y then
					-- table.insert(cells, cell)
				-- end			
			-- end
		-- end
	-- end
    -- return cells
-- end

-- for FLASH PACK
-- function icetrap:toggle( sim )
	-- self:detonate( sim )
-- end

-----------------------------------------------------
-- Interface functions

local function createicetrapPack( unitData, sim )
	return simunit.createUnit( unitData, sim, icetrap )
end

simfactory.register( createicetrapPack )

return
{
	createicetrapPack = createicetrapPack,
}

