----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local prop_templates = include( "sim/unitdefs/propdefs" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )

-----------------------------------------------------
-- Local functions
-- copied from Neptune with minor changes
local simfrostcloud = { ClassType = "ITB_iceSmoke" }

local function occludeSight( sim, targetCell, smokeRadius )
	local cells = simquery.floodFill( sim, nil, targetCell, smokeRadius, simquery.getManhattanMoveCost, simquery.canPathBetween )
	for i, cell in pairs(cells) do
		cell.fake_frost = (cell.fake_frost or 0) + 1
	end
	return cells
end

-----------------------------------------------------
-- Interface functions

function simfrostcloud:onWarp(sim, oldcell, cell)
	if oldcell then
		for i, cell in pairs(self._cells) do
			cell.fake_frost = cell.fake_frost - 1
		end
		self._cells = nil
	end
    if cell then
        self._cells = occludeSight( sim, cell, self:getTraits().radius )    
    end
    sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } ) 
end

function simfrostcloud:onEndTurn( sim )
    -- There is no simunit base behaviour we desire at this time.
    self:getTraits().lifetime = self:getTraits().lifetime - 1
    if self:getTraits().lifetime <= 0 then
        sim:warpUnit( self, nil )
        sim:despawnUnit( self )
    end
end

function simfrostcloud:getSmokeCells()
    return self._cells
end

function simfrostcloud:getSmokeEdge()
    return {}
end
-----------------------------------------------------
-- Interface functions

local function createSmokePlume( unitData, sim )
	return simunit.createUnit( unitData, sim, simfrostcloud )
end

simfactory.register( createSmokePlume )

return simfrostcloud
