----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "modules/util" )
local array = include( "modules/array" )
local unitdefs = include( "sim/unitdefs" )
local simunit = include( "sim/simunit" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local simfactory = include( "sim/simfactory" )
local propdefs = include("sim/unitdefs/propdefs")

-----------------------------------------------------
-- Grenade item
local function spawnDecorativeIceCloud( sim, self )
	local x0, y0 = self:getLocation()
	local iceCloudUnit = simfactory.createUnit( propdefs.ITB_ice_cloud, sim )
	sim:spawnUnit( iceCloudUnit )
	sim:warpUnit( iceCloudUnit, sim:getCell(x0, y0) )
	iceCloudUnit:getTraits().lifetime = 100
	self.iceCloud = iceCloudUnit
end

local iceblock = { ClassType = "ITB_iceblock" }
  
function iceblock:onSpawn( sim )
	sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
	sim:addTrigger( simdefs.TRG_UNIT_KO, self )
	sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
end

function iceblock:onDespawn( sim )
	sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
	sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
	sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self  )
	if self.iceCloud then
		sim:warpUnit( self.iceCloud, nil )
		sim:despawnUnit( self.iceCloud )
	end
	self.iceCloud = nil
end

function iceblock:onTrigger( sim, evType, evData )
	if (evType == simdefs.TRG_UNIT_KO) or (evType == simdefs.TRG_UNIT_KILLED) and self:getTraits().frozenUnit then
		local koUnit = sim:getUnit(self:getTraits().frozenUnit)
		if ((evData.unit == koUnit) and (evData.ticks == nil)) or (koUnit == nil) then -- woke up
			koUnit:getTraits().frozenAnim = nil
			koUnit:getTraits().frozenByCryoGun = nil
			koUnit:getTraits().frozenRenderFilter = nil
			self:getTraits().disruptAnim = true
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )
			-- sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= self, anim="distrupt_out", } )
			
			local cell = sim:getCell(self:getLocation())
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = self, kanim = "fx/ice_block_fx", anim="out", symbol = "effect", x = cell.x, y = cell.y, facingMask = KLEIAnim.FACING_W } )		
			-- sim:dispatchEvent( simdefs.EV_UNIT_ACTIVATE, { unit = self, } )				
			sim:warpUnit( self, nil )
			sim:getPC():glimpseCell( sim, cell )
			sim:despawnUnit( self )
		end
	elseif evType == simdefs.TRG_UNIT_WARP and (evData.unit == self) and not evData.from_cell then -- initial warp on spawn
		local x0, y0 = self:getLocation()
		spawnDecorativeIceCloud( sim, self )
		-- log:write("LOG iceblock onWarp")
		if self:getTraits().iceBlock then
			if self:getTraits().deploy_cover then
				-- local cell = sim:getCell(self:getLocation())
				-- sim:warpUnit( self )
				self:getTraits().cover = true			
				-- sim:warpUnit( self, cell)
				-- sim:dispatchEvent( simdefs.EV_UNIT_HIDE, { unit = self, hide = true } )
			end		
			-- self:getTraits().hologram=true --disable this, because it keeps distracting guards.
			self:getSounds().spot = self:getSounds().activeSpot
			sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_SPOTSOUND, { unit = self,  stop = false } )
		end
		
		-- self:getTraits().holoProjector = true
		self:getTraits().deployed = true --only keep these two if I want the flicker animation... do I?
		
		local cells = { sim:getCell(self:getLocation()) }
		sim:dispatchEvent( simdefs.EV_UNIT_ACTIVATE, { unit = self, } )
	
	end
end

-----------------------------------------------------
-- Interface functions

local function createIceblock( unitData, sim )
	return simunit.createUnit( unitData, sim, iceblock )
end

simfactory.register( createIceblock )

return
{
	createIceblock = createIceblock,
}


