local game = include( "modules/game" )
local util = include("client_util")
local array = include("modules/array")
local mui = include( "mui/mui" )
local mui_defs = include( "mui/mui_defs" )
local mui_tooltip = include( "mui/mui_tooltip" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local version = include( "modules/version" )
local agentdefs = include("sim/unitdefs/agentdefs")
local skilldefs = include( "sim/skilldefs" )
local simdefs = include( "sim/simdefs" )
local simactions = include( "sim/simactions" )
local modalDialog = include( "states/state-modal-dialog" )
local rig_util = include( "gameplay/rig_util" )
local metrics = include( "metrics" )
local cdefs = include("client_defs")
local scroll_text = include("hud/scroll_text")
local guiex = include( "client/guiex" )
local SCRIPTS = include('client/story_scripts')
local simengine = include('sim/engine')

local sitreps = include(SCRIPT_PATHS.advanced_cyberwarfare.."/sitreps/sitreps" )

local function fetchSitrep( name )
	for i, sitrep in pairs(sitreps) do
		if sitrep.name == name then
			return sitrep
		end
	end
end

-- actual mechanics of applying the grid
local oldinit = simengine.init
function simengine:init( params, levelData, ... )
	oldinit(self, params, levelData, ...)

	local grid = params.ACW_GRID

	if grid and grid.name then
		grid = fetchSitrep(grid.name)
		if not grid then return end
		if grid.initialize then
			-- grid makes a change to the sim once, and that change applies to the level
			grid:initialize( self )
			--log:write("[ACWLOG] Grid attached: "..grid.name)
		end
		if grid.trigger then
			for i, trg in pairs(grid.trigger) do
				-- grid adds a trigger and serves as the carrier of that trigger's effect
				--log:write("[ACWLOG] Attempting to add a trigger to "..util.stringize(grid, 2))
				self:addTrigger( trg, grid )
			end
			--log:write("[ACWLOG] Grid embedded: "..grid.name)
		end
	end
end

local function generateSitrep( mapScreen, situation, corpData )
	local sitrep_none = {}

	-- do not make sitreps on the first mission, mid1, mid2, or ending1
	--log:write("Generating sitrep based on situation: "..util.stringize(situation, 2))
	if (situation.name == "executive_terminals" and not situation.random_idx) or situation.name == "mid_1" or situation.name == "ending_1" then
		log:write("[ACWLOG] Story mission - no grid")
		return sitrep_none
	end

	--log:write("[ACWLOG] Generating sitrep")
	local result = math.random(100)
	local minimum = 100
	for i, DLC in pairs(mapScreen._campaign.difficultyOptions.enabledDLC) do
		if DLC.name == "Advanced Cyberwarfare" then
			minimum = 100 - (DLC.options.ac_sitreps.value or 0)*100
		end
	end
	--log:write("[ACWLOG] Result: "..tostring(result).."/"..tostring(minimum))

	if result > minimum then
		-- each sitrep has a weight to appear at each corp (default is 10)
		local total_weight = 0
		for i, sitrep in pairs(sitreps) do
			total_weight = total_weight + sitrep.weights[corpData.shortname]
		end
		local choice = math.random(total_weight)
		for i, sitrep in pairs(sitreps) do
			choice = choice - sitrep.weights[corpData.shortname]
			if choice < 0 then
				return sitrep
			end
		end
	else
		return sitrep_none
	end
end

-- map-screen grid label, along with actual mechanics of generating a grid
local stateMapScreen = include( "states/state-map-screen" )
local OnClickLocation_old = stateMapScreen.OnClickLocation

stateMapScreen.OnClickLocation = function( self, situation, ... )
	OnClickLocation_old( self, situation, ... )
	
	local screen

	for i, active_screen in pairs(mui.internals._activeScreens) do
		if active_screen:findWidget("SLF_Sitrep") then
			screen = active_screen
			break
		end
	end

	if screen and STRINGS.SLF then
		local corpData = serverdefs.getCorpData( situation )

		if not situation.sitrep then
			situation.sitrep = generateSitrep( self, situation, corpData )
			log:write("[ACWLOG] Grid generated: <"..(situation.sitrep.name or "None")..">")
			--log:write("Sitrep output: "..util.stringize(situation.sitrep, 1).."; length: "..#situation.sitrep)
		end
		if situation.sitrep.name then
			self._campaign.missionParams = self._campaign.missionParams or {}
			self._campaign.missionParams.ACW_GRID = situation.sitrep
			screen:findWidget("SLF_Sitrep"):setVisible( true )
			screen:findWidget("SLF_Sitrep.label"):setText("<font1_16_sb><c:F48134>Cybersec Grid detected</c>\n\n<font1_14_r><c:F5FF78>"..situation.sitrep.name..STRINGS.SLF.SITREPS.GRID_DEFAULT.."</c>\n<font1_12_r>"..situation.sitrep.desc)
		else
			screen:findWidget("SLF_Sitrep"):setVisible( false )
			if self._campaign.missionParams then
				self._campaign.missionParams.ACW_GRID = nil
			end
		end
	end
end

-- in-mission grid label
local hudFile = include("hud/hud")
local oldCreateHud = hudFile.createHud

function hudFile.createHud(...)
	local hud = oldCreateHud(...)

	local oldRefresh = hud.refreshHud

	function hud:refreshHud(...)
		oldRefresh(self, ...)

		local sim = self._game.simCore
		local showPanels = (sim:getCurrentPlayer() == self._game:getLocalPlayer())
		self._screen.binder.statsPnl:setVisible( showPanels and self:canShowElement( "statsPnl" ))

		local daysTxt = 0
		local turn = math.ceil( (sim:getTurnCount() + 1) / 2)
		if self._game.params.campaignHours then
			daysTxt = math.floor( self._game.params.campaignHours / 24 ) + 1
		end
		local gameModeStr = util.toupper( serverdefs.GAME_MODE_STRINGS[ self._game.params.campaignDifficulty ] )

	    local corpData = serverdefs.CORP_DATA[ self._game.params.world ]
	    local situationData = serverdefs.SITUATIONS[ self._game.params.situationName ]
	    if corpData and situationData then
	    	local locationName = situationData.ui.locationName
	    	if sim:getTags().newLocationName then
				locationName = sim:getTags().newLocationName
	    	end
	        local missionTxt = corpData.stringTable.SHORTNAME .." " .. locationName
		    self._screen.binder.statsPnl.binder.statsTxt:setText( string.format(STRINGS.UI.HUD_DAYS_TURN_ALARM, turn, daysTxt, gameModeStr, missionTxt ) )
		    local statsGridTxt = self._screen.binder.statsPnl.binder.statsGridTxt
		    if statsGridTxt and sim:getParams().ACW_GRID then
		    	local x, y = self._screen.binder.statsPnl.binder.statsTxt:getPosition()
		    	if y == 10 then
		    		y = 20
		    	end
				self._screen.binder.statsPnl.binder.statsTxt:setPosition(x, y)
				local grid = sim:getParams().ACW_GRID
				local tip =	mui_tooltip( STRINGS.SLF.WORLD.GRID.NAME .. grid.name, grid.desc )
				statsGridTxt:setText( "<c:8CFFFF>//</c>  <c:FF8411>"..grid.name..STRINGS.SLF.SITREPS.GRID_DEFAULT.."</c>" )
				statsGridTxt:setTooltip(tip)
				statsGridTxt:setVisible(true)
			else
		    	local x, y = self._screen.binder.statsPnl.binder.statsTxt:getPosition()
		    	if y == 20 then
		    		y = 10
		    	end
				self._screen.binder.statsPnl.binder.statsTxt:setPosition(x, y)
				statsGridTxt:setVisible(false)
			end
	    end
	end

	return hud
end