local simdefs = include("sim/simdefs")
local util = include("modules/util")
local cdefs = include("client_defs")

local sitreps = {
	-- SanSan City Grid: Whenever Alarm Level increases, add 1 tick to it.
	{
		name = STRINGS.SLF.SITREPS.SANSAN.NAME,
		desc = STRINGS.SLF.SITREPS.SANSAN.DESC,
		weights = {
			KO = 10,
			FTM = 20,
			SANKAKU = 10,
			PLASTECH = 10,
			NEPTUNE = 10,
			OMNI = 10,
		},
		trigger = { simdefs.TRG_ALARM_STATE_CHANGE },
		onTrigger = function( self, sim, evType, evData )
			if not sim._SLF_tracker_advancing then
				sim._SLF_tracker_advancing = true
				sim:trackerAdvance(1, STRINGS.SLF.SITREPS.SANSAN.TRIGGER)
				sim._SLF_tracker_advancing = nil
			end
		end,
	},

	-- Crisium Grid: Daemons cannot be reversed.
	{
		name = STRINGS.SLF.SITREPS.CRISIUM.NAME,
		desc = STRINGS.SLF.SITREPS.CRISIUM.DESC,
		weights = {
			KO = 10,
			FTM = 10,
			SANKAKU = 10,
			PLASTECH = 20,
			NEPTUNE = 10,
			OMNI = 10,
		},
		initialize = function( self, sim )
			local aiplayer = sim:getNPC()
			local reversible = aiplayer.addMainframeAbility
			function aiplayer.addMainframeAbility( self, sim, abilityID, hostUnit, reversalOdds, ... )
				-- Progextend Reflect coverage
				local PE_reverse = sim:getPC():getTraits().reverseAll
				sim:getPC():getTraits().reverseAll = nil

				-- determine whether the daemon would be reversed in order to display the pop-up
				-- this relies on the next sim:getRand call to be part of the base addMainframeAbility;
				-- so if another mod appends addMainframeAbility and calls a rand in there, this might well cease to be accurate
				local monst3rReverseOdds = reversalOdds or 10
				for _, ability in ipairs( sim:getPC():getAbilities() ) do 
					if ability.daemonReversalAdd and monst3rReverseOdds > 0 then 
						monst3rReverseOdds = monst3rReverseOdds + ability.daemonReversalAdd
					end 
				end 

				local seed = sim._seed
				local monst3rReverse = sim:nextRand(1, 100) < monst3rReverseOdds
				if monst3rReverse then
					--log:write("[ACWLOG] Should display warning here.")
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.SLF.SITREPS.CRISIUM.TRIGGER, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
				end
				-- undo the rand call to avoid messing with simhistory
				sim._seed = seed

				local result = reversible( self, sim, abilityID, hostUnit, 0, ... ) -- setting reversalOdds to 0 makes daemons irreversible

				sim:getPC():getTraits().reverseAll = PE_reverse

				return result
			end
		end,
	},

	-- Djupstad Grid: When the objective is completed, your units get -1 max AP for the rest of the mission.
	{
		name = STRINGS.SLF.SITREPS.DJUPSTAD.NAME,
		desc = STRINGS.SLF.SITREPS.DJUPSTAD.DESC,
		weights = {
			KO = 10,
			FTM = 10,
			SANKAKU = 10,
			PLASTECH = 10,
			NEPTUNE = 20,
			OMNI = 10,
		},
		initialize = function( self, sim )
			local oldClimax = sim.setClimax
			function sim:setClimax( state, ... )
				if state then
					local modifiers = include("sim/modifiers")
					for i, unit in pairs(sim:getPC():getUnits()) do
						unit:getModifiers():add("mpMax", "ACW_djupstad", modifiers.ADD, -2)
					end	
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.SLF.SITREPS.DJUPSTAD.TRIGGER, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
				end

				return oldClimax( self, state, ... )
			end
		end,
	},

	-- Heinlein Grid: After a device is captured, Incognita is locked for the rest of the turn.
	{
		name = STRINGS.SLF.SITREPS.HEINLEIN.NAME,
		desc = STRINGS.SLF.SITREPS.HEINLEIN.DESC,
		weights = {
			KO = 10,
			FTM = 20,
			SANKAKU = 10,
			PLASTECH = 10,
			NEPTUNE = 10,
			OMNI = 10,
		},
		trigger = { simdefs.TRG_ICE_BROKEN, simdefs.TRG_END_TURN },
		onTrigger = function( self, sim, evType, evData )
			-- lockout logic is weird. Leaving a bit of leeway for multiple paradox situations
			if evType == simdefs.TRG_END_TURN and sim._mainframeLockout > 995 and sim._mainframeLockout < 1005 then
				sim._mainframeLockout = 0
			end
			log:write(string.format("[ACW] HEINLEIN trigger. Lockout: %s.", tostring(sim._mainframeLockout)))
			if evType == simdefs.TRG_ICE_BROKEN and sim._mainframeLockout == 0 and evData.unit and evData.unit:getTraits().mainframe_ice < 1 then
				sim._mainframeLockout = 1001
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.SLF.SITREPS.HEINLEIN.TRIGGER, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
			end
		end,
	},

	-- Hokusai Grid: Level starts with no guards. Each turn, a guard arrives, until all are there. After that, an enforcer arrives.

	-- Expo Grid: Scanning Amps, Alarm and Countermeasures pinpoints detect an additional agent.
	-- POSTPONED - niche/specific, might become exactly Scanning Amps and spawn exclusively at FTM

	-- Satellite Grid: when the objective is completed, lock the exit elevator for 5 turns.
	{
		name = STRINGS.SLF.SITREPS.SATELLITE.NAME,
		desc = STRINGS.SLF.SITREPS.SATELLITE.DESC,
		weights = {
			KO = 20,
			FTM = 10,
			SANKAKU = 10,
			PLASTECH = 10,
			NEPTUNE = 10,
			OMNI = 20,
		},
		initialize = function( self, sim )
			local oldClimax = sim.setClimax
			function sim:setClimax( state, ... )
				if state then
					self:closeElevator()
					self._elevator_inuse = 5
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.SLF.SITREPS.SATELLITE.TRIGGER, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety/Actions/mainframe_deterrent_action" } )
				end

				return oldClimax( self, state, ... )
			end
		end,
	},

	-- Oaktown Grid: ALL units have +3 sprint noise.
	{
		name = STRINGS.SLF.SITREPS.SATELLITE.NAME,
		desc = STRINGS.SLF.SITREPS.SATELLITE.DESC,
		weights = {
			KO = 20,
			FTM = 10,
			SANKAKU = 10,
			PLASTECH = 10,
			NEPTUNE = 10,
			OMNI = 20,
		},
		initialize = function( self, sim )
			local oldSpawn = sim.spawnUnit
			function sim:spawnUnit( unit, ... )
				local results = {oldSpawn( self, unit, ... )}

				if unit and unit:getTraits().mp and unit:getTraits().dashSoundRange then
					unit:getModifiers():add("dashSoundRange", "ACW_oaktown", modifiers.ADD, 3)
				end

				return unpack(results)
			end
		end,
	},
}

return sitreps