local game = include( "modules/game" )
local util = include("client_util")
local array = include( "modules/array" )
local mui = include( "mui/mui" )
local mui_defs = include( "mui/mui_defs" )
local serverdefs = include( "modules/serverdefs" )
local agentdefs = include("sim/unitdefs/agentdefs")
local skilldefs = include( "sim/skilldefs" )
local modalDialog = include( "states/state-modal-dialog" )
local scroll_text = include("hud/scroll_text")
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local itemdefs = include( "sim/unitdefs/itemdefs" )
local guiex = include( "client/guiex" )
local cdefs = include( "client_defs" )
local rig_util = include( "gameplay/rig_util" )
local SCRIPTS = include('client/story_scripts')
local talkinghead = include('client/fe/talkinghead')
local abilitydefs = include( "sim/abilitydefs" )

----------------------------------------------------------------
-- Class setup

local dialog = class()

----------------------------------------------------------------
-- Local helpers
local skillRequired = {
	1,
	3,
	4,
	5,
	5,
}

local function getLockReason( self )
	if self.focus.upgradeParams.traits.nextReward then
		return STRINGS.WIZARD.UI.CANT_LEARN_NOT_NEXT 
	elseif self.magicLevel < skillRequired[ self.focus.upgradeParams.traits.nextRewardCount ] then
		return STRINGS.WIZARD.UI.CANT_LEARN_NEED_UPGRADE
	else
		return STRINGS.WIZARD.UI.CANT_LEARN_GENERIC
	end
end

local function updateSpellBtn( self, widget, spellId, disabled )
	local spell = abilitydefs.lookupAbility( spellId )
	local btn = widget.binder.item.binder.btn
	
	spell.smallFont = true
	widget.binder.itemName:setVisible( true )
	widget.binder.itemName:setText( spell:getName() )
	spell.smallFont = nil

	if disabled then
		btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.SORCERY_DID_NOT_LEARN, util.toupper( spell:getName() ) ) )
	elseif spell.onTooltip then
		btn:setTooltip( spell:onTooltip( self.screen ) )
	elseif spell.createToolTip then
		btn:setTooltip( spell:createToolTip() )
	end

	btn.spellName = spell:getName()
	btn:setImage( spell.profile_icon )
	btn:setDisabled( disabled )
	
	local color
	local colorInactive
	local colorActive
	local colorHover

	if disabled then
		color = util.color(0.5,0.5,0.5)
		colorInactive = util.color(0.5,0.5,0.5)
		colorActive = util.color(0.5,0.5,0.5)
		colorHover = util.color(0.5,0.5,0.5)
	elseif spell.iconColor then
		color = spell.iconColor
		colorInactive = spell.iconColor
		colorActive = spell.iconColorHover
		colorHover = spell.iconColorHover
	elseif spell.usesAction then
		color = cdefs.COLOR_ATTACK
		colorInactive = cdefs.COLOR_ATTACK
		colorActive = cdefs.COLOR_ATTACK_HOVER
		colorHover = cdefs.COLOR_ATTACK_HOVER
	elseif spell.usesMP then
		color = cdefs.COLOR_ACTION
		colorInactive = cdefs.COLOR_ACTION
		colorActive = cdefs.COLOR_ACTION_HOVER
		colorHover = cdefs.COLOR_ACTION_HOVER		
	else
		color = cdefs.COLOR_FREE
		colorInactive = cdefs.COLOR_FREE
		colorActive = cdefs.COLOR_FREE_HOVER
		colorHover = cdefs.COLOR_FREE_HOVER	
	end
	
	btn:setColor(color:unpack())
	btn:setColorInactive(colorInactive:unpack())
	btn:setColorActive(colorActive:unpack())
	btn:setColorHover(colorHover:unpack())
end

local function onClickLearnSpell( self, spellId, idx, rewardWidget, altSpellId )
	local spell = abilitydefs.lookupAbility( spellId )
	local altSpell = abilitydefs.lookupAbility( altSpellId )
	local body = util.sformat( STRINGS.WIZARD.UI.SORCERY_LEARN_CONFIRM, spell:getName(), altSpell:getName() )
	
	local result = modalDialog.showYesNo( body, STRINGS.WIZARD.UI.LEARN_SPELL )
	
	if result == modalDialog.OK then
		MOAIFmodDesigner.playSound( "SpySociety/HUD/gameplay/upgrade_confirm" )

		table.insert( self.focus.upgradeParams.traits.spellsKnown, spellId )
		table.insert( self.focus.upgradeParams.traits.pastRewards, self.focus.upgradeParams.traits.nextReward )
		self.focus.upgradeParams.traits.nextReward = nil
		
		self.panel.bodyTxtCentral:spoolText( STRINGS.WIZARD.UI.SORCERY_LEARN_RANDOM[math.random(#STRINGS.WIZARD.UI.SORCERY_LEARN_RANDOM)] )
		
		for i, widget in rewardWidget.binder:forEach("item_") do
			local btn = widget.binder.item.binder.btn
			widget.binder.slot_glow:setVisible( false )
			btn.onClick = nil
			
			if i ~= idx then
				btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.SORCERY_DID_NOT_LEARN, util.toupper( btn.spellName ) ) )
				btn:setDisabled( true )
				btn:setColor(0.5,0.5,0.5,1)
				btn:setColorInactive(0.5,0.5,0.5,1)
				btn:setColorActive(0.5,0.5,0.5,1)
				btn:setColorHover(0.5,0.5,0.5,1)
			end
		end
		
		if self.lockedRewardWidget then
			for i, widget in self.lockedRewardWidget.binder:forEach("item_") do
				local btn = widget.binder.item.binder.btn
				btn:setTooltip( getLockReason( self ) )
			end
		end
	end
end

function dialog:show( agency, focus, magicLevel )
	local screen = mui.createScreen( "spell-learn-modal.lua" )
	mui.activateScreen( screen )

	screen:createTransition("activate_left")

	self.agency = agency
	self.focus = focus
	self.magicLevel = magicLevel
	self.screen = screen
	self.panel = self.screen.binder.panel.binder
	
	self.panel.closeBtn.onClick = util.makeDelegate( self, "destroy" )
	
	local list = self.panel.spellList
	local scroller = list._scrollbar.binder.scrollbar
	list:clearItems()
	scroller:setPosition( nil, 0 )
	local w, h = list:getSize()
	scroller:setSize( nil, h )
	
	local pastRewards = focus.upgradeParams.traits.pastRewards
	local nextReward = focus.upgradeParams.traits.nextReward
	local nextRewardCount = focus.upgradeParams.traits.nextRewardCount
	local spellsKnown = focus.upgradeParams.traits.spellsKnown
	
	for i, pastReward in ipairs( pastRewards ) do
		local rewardWidget = list:addItem( self )
		
		for j, widget in rewardWidget.binder:forEach("item_") do
			if pastReward[j] then
				updateSpellBtn( self, widget, pastReward[j], not array.find( spellsKnown, pastReward[j] ) )
			else
				widget:setVisible(false)
			end
		end
	end
	
	if nextReward then
		self.panel.bodyTxtCentral:spoolText( STRINGS.WIZARD.UI.SORCERY_LEARN_CHOOSE )
	
		local rewardWidget = list:addItem( self )
		
		for i, widget in rewardWidget.binder:forEach("item_") do
			if nextReward[i] then
				local btn = widget.binder.item.binder.btn
				local altSpellId = nextReward[1 + (i % #nextReward)]
				updateSpellBtn( self, widget, nextReward[i], false )
				widget.binder.slot_glow:setVisible( true )
				btn.onClick = util.makeDelegate( nil, onClickLearnSpell, self, nextReward[i], i, rewardWidget, altSpellId )
			else
				widget:setVisible(false)
			end
		end
	else
		self.panel.bodyTxtCentral:spoolText( STRINGS.WIZARD.UI.SORCERY_LEARN_RANDOM[math.random(#STRINGS.WIZARD.UI.SORCERY_LEARN_RANDOM)] )
	end
	
	if nextRewardCount <= 5 then
		local rewardWidget = list:addItem( self )
		self.lockedRewardWidget = rewardWidget
		
		for j, widget in rewardWidget.binder:forEach("item_") do
			local btn = widget.binder.item.binder.btn
			btn:setTooltip( getLockReason( self ) )
			btn:setImage( "gui/icons/action_icons/Action_icon_Small/icon-action_lock_small.png" )
			btn:setDisabled( true )
			btn:setColor(0.5,0.5,0.5,1)
		end
	else
		self.lockedRewardWidget = nil
	end
	
	list:scrollItems( nextRewardCount )
end

function dialog:destroy()
	if self.screen then
		local screen = self.screen
		self.screen = nil
		
		screen:createTransition( "deactivate_left",
			function( transition )
				mui.deactivateScreen( screen )
			end,
		{ easeOut = true } )
	end
end


return dialog
