local game = include( "modules/game" )
local util = include("client_util")
local array = include( "modules/array" )
local mui = include( "mui/mui" )
local mui_defs = include( "mui/mui_defs" )
local serverdefs = include( "modules/serverdefs" )
local agentdefs = include("sim/unitdefs/agentdefs")
local skilldefs = include( "sim/skilldefs" )
local modalDialog = include( "states/state-modal-dialog" )
local scroll_text = include("hud/scroll_text")
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local itemdefs = include( "sim/unitdefs/itemdefs" )
local guiex = include( "client/guiex" )
local cdefs = include( "client_defs" )
local rig_util = include( "gameplay/rig_util" )
local SCRIPTS = include('client/story_scripts')
local talkinghead = include('client/fe/talkinghead')
local abilitydefs = include( "sim/abilitydefs" )

-------------------------------------------------------------
------------------- SHARED FUNCTIONALITY --------------------
-------------------------------------------------------------

local commonDialog = class()

----------------------------------------------------------------
-- Local helpers

-- For each spell slot in order, this is its level
local slotLevels = { 1, 1, 1, 2, 2, 3 }
-- This is the MAGIC level that the wizard must have to cast a spell of a specific level
local minimumLevelForSpell = { 1, 3, 5 }

local function hasSpellSlot( self, i )
	return i <= self.magicLevel + 1
end

local function skillNeededToHaveSpellSlot( self, i )
	return i - 1
end

local function updateSpellBtn( self, btn, spell )
	if spell.onTooltip then
		btn:setTooltip( spell:onTooltip( self.screen ) )
	elseif spell.createToolTip then
		btn:setTooltip( spell:createToolTip() )
	end

	btn:setImage( spell.profile_icon )
	btn:setDisabled( false )
	
	local color
	local colorInactive
	local colorActive
	local colorHover
	
	if spell.iconColor then
		color = spell.iconColor
		colorInactive = spell.iconColor
		colorActive = spell.iconColorHover
		colorHover = spell.iconColorHover
	elseif spell.usesAction then
		color = cdefs.COLOR_ATTACK
		colorInactive = cdefs.COLOR_ATTACK
		colorActive = cdefs.COLOR_ATTACK_HOVER
		colorHover = cdefs.COLOR_ATTACK_HOVER
	elseif spell.usesMP then
		color = cdefs.COLOR_ACTION
		colorInactive = cdefs.COLOR_ACTION
		colorActive = cdefs.COLOR_ACTION_HOVER
		colorHover = cdefs.COLOR_ACTION_HOVER		
	else
		color = cdefs.COLOR_FREE
		colorInactive = cdefs.COLOR_FREE
		colorActive = cdefs.COLOR_FREE_HOVER
		colorHover = cdefs.COLOR_FREE_HOVER	
	end
	
	btn:setColor(color:unpack())
	btn:setColorInactive(colorInactive:unpack())
	btn:setColorActive(colorActive:unpack())
	btn:setColorHover(colorHover:unpack())
end

local function hideSlotGlow( self )
	if self.slot_glow then
		self.slot_glow:setVisible( false )
		self.slot_glow = nil
	end
end

local function showSlotGlow( self, widget )
	self.slot_glow = widget.binder.slot_glow
	self.slot_glow:setVisible( true )
end

----------------------------------------------------------------
-- Spell preparation

function commonDialog:prepareSpell( slotIndex, spellId )
	hideSlotGlow( self )
	self.preparingSpellId = nil
	self.preparingSlotIndex = nil
	
	local slotLevel = slotLevels[ slotIndex ]
	local spell = abilitydefs.lookupAbility( spellId )
	
	if slotLevel < spell.spellLevel then
		--MOAIFmodDesigner.playSound("SpySociety/HUD/gameplay/upgrade_cancel_unit")
		modalDialog.show( util.sformat( STRINGS.WIZARD.UI.SPELL_SLOT_TOO_LOW, spell:getName(), spell.spellLevel, slotLevel ) )
		
		return false
	else
		return true
	end
end

function commonDialog:reorderSpells( i, j, spellId )
	local oldSpellIdx = array.find( self.book, spellId )
	local newSpellIdx = math.min( (i - 1) * 6 + j, #self.book )
	
	if oldSpellIdx ~= newSpellIdx then
		table.remove( self.book, oldSpellIdx )
		table.insert( self.book, newSpellIdx, spellId )
		
		local items = self.panel.spellBookList
		for i = math.min( oldSpellIdx, newSpellIdx ), math.max( oldSpellIdx, newSpellIdx ) do
			local row = math.ceil( i / 6 )
			local column = 1 + ( (i - 1) % 6 )
			local widget = items:getItem( row ).widget.binder["item_"..column]
			local spellId = self.book[i]
			
			self:updateSpellBookWidget( i, widget, spellId )
		end
	end
end

-------------------------------------------------------------
-- Show spell book (spells available to prepare)

function commonDialog:onDragSpellBook( spellId, spell )
	local widget = self.screen:startDragDrop( spell.profile_icon, "DragItem" )
	widget.binder.img:setImage( spell.profile_icon )
	
	for i, widget in self.panel.preparedSpells.binder:forEach("item_") do
		if hasSpellSlot( self, i ) then
			widget.binder.item.binder.dragzone.onDragDrop = function() util.coDelegate( self.prepareSpell, self, i, spellId ) end
		end
	end
	
	for i, bookList in ipairs( self.panel.spellBookList:getItems() ) do
		for j, widget in bookList.widget.binder:forEach("item_") do
			widget.binder.item.binder.dragzone.onDragDrop = function() util.coDelegate( self.reorderSpells, self, i, j, spellId ) end
		end
	end
	
	return true
end

function commonDialog:onClickSpellBook( spellId, spell, i, widget )
	hideSlotGlow( self )
	
	if self.preparingSlotIndex then
		self:prepareSpell( self.preparingSlotIndex, spellId )
	else
		if self.preparingSpellId == spellId then
			self.preparingSpellId = nil
		else
			MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/click" )
			self.preparingSpellId = spellId
			showSlotGlow( self, widget )
		end
	end

	return true
end

function commonDialog:updateSpellBookWidget( i, widget, spellId )
	local btn = widget.binder.item.binder.btn
	
	widget.binder.cost:setVisible( false )
	widget.binder.level:setVisible( false )
	
	if spellId then
		local spell = abilitydefs.lookupAbility( spellId )
		spell.smallFont = true
		widget.binder.itemName:setVisible( true )
		widget.binder.itemName:setText( spell:getName() )
		spell.smallFont = nil
		
		updateSpellBtn( self, btn, spell )
		
		btn.onClick = util.makeDelegate( nil, self.onClickSpellBook, self, spellId, spell, i, widget )
		btn.onDragStart = util.makeDelegate( nil, self.onDragSpellBook, self, spellId, spell )
	else
		btn:setImage( "" )
		btn:setDisabled( true )
		widget.binder.itemName:setVisible( false )
	end
end

function commonDialog:showBook()
	local list = self.panel.spellBookList
	local scroller = list._scrollbar.binder.scrollbar
	list._scrollbar:setVisible( false )
	list:clearItems()
	scroller:setPosition( nil, 0 )
	local w, h = list:getSize()
	scroller:setSize( nil, h )
	
	for i = 1, math.ceil( #self.book / 6 ) do
		local bookList = list:addItem( self )
		bookList:setTooltip( nil )
		
		for j, widget in bookList.binder:forEach("item_") do
			local spellId = self.book[(i - 1) * 6 + j]
			
			self:updateSpellBookWidget( (i - 1) * 6 + j, widget, spellId )
		end
	end
end

-------------------------------------------------------------
-- Show spell slots (prepared spells)

function commonDialog:onClickSpellSlot( slotIndex, widget )
	hideSlotGlow( self )
	if self.preparingSpellId then
		self:prepareSpell( slotIndex, self.preparingSpellId )
	else
		if self.preparingSlotIndex == slotIndex then
			self.preparingSlotIndex = nil
		else
			MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/click" )
			self.preparingSlotIndex = slotIndex
			showSlotGlow( self, widget )
		end
	end

	return true
end

function commonDialog:updatePreparedSpellWidget( i, widget, spellId )
	local btn = widget.binder.item.binder.btn
	
	widget.binder.cost:setVisible( false )
	widget.binder.level:setText( STRINGS.WIZARD.UI.SLOT_LEVELS[ slotLevels[i] ] )
	widget.binder.item.binder.encumbered:setVisible( not hasSpellSlot( self, i ) )
	
	if spellId and hasSpellSlot( self, i ) then
		local spell = abilitydefs.lookupAbility( spellId )
		spell.smallFont = true
		widget.binder.itemName:setVisible( true )
		widget.binder.itemName:setText( spell:getName() )
		btn.onClick = util.makeDelegate( nil, self.onClickSpellSlot, self, i, widget )
		spell.smallFont = nil
		updateSpellBtn( self, btn, spell )
	else
		btn:setImage( "" )
		widget.binder.itemName:setVisible( false )
		if hasSpellSlot( self, i ) then
			btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.EMPTY_SLOT_TIP, slotLevels[i] ) )
			btn.onClick = util.makeDelegate( nil, self.onClickSpellSlot, self, i, widget )
		else
			btn:setDisabled( true )
			btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.UNAVAILABLE_SLOT_TIP, slotLevels[i], skillNeededToHaveSpellSlot( self, i ) ) )
		end
	end
end

function commonDialog:showPreparedSpells( )
	for i, widget in self.panel.preparedSpells.binder:forEach("item_") do
		local spellId = self.focus[i]
		
		self:updatePreparedSpellWidget( i, widget, spellId )
	end
end

-------------------------------------------------------------
-- Interface

function commonDialog:hideScrollsScroller()
	local list = self.panel.spellScrollList
	local scroller = list._scrollbar.binder.scrollbar
	list._scrollbar:setVisible( false )
	list:clearItems()
end

function commonDialog:show( book, focus, magicLevel )
	local screen = mui.createScreen( "spell-preparation-modal.lua" )
	mui.activateScreen( screen )

	screen:createTransition("activate_left")
	screen:addEventHandler( self, mui_defs.EVENT_DragDrop )
	screen:addEventHandler( self, mui_defs.EVENT_MouseUp )
	screen:addEventHandler( self, mui_defs.EVENT_KeyDown )

	self.book = book
	self.focus = focus
	self.magicLevel = magicLevel
	self.screen = screen
	self.panel = self.screen.binder.panel.binder
	
	self.panel.closeBtn.onClick = util.makeDelegate( self, "destroy" )
	self.panel.creditsTxt:setVisible( false )
	
	self:hideScrollsScroller()
	self:showBook( )
	self:showPreparedSpells( )
end

function commonDialog:handleEvent( ev )
	if ev.eventType == mui_defs.EVENT_DragDrop and ev.dragData == nil then
		for i, widget in self.panel.preparedSpells.binder:forEach("item_") do
			widget.binder.item.binder.dragzone.onDragDrop = nil
		end
		
		for i, bookList in ipairs( self.panel.spellBookList:getItems() ) do
			for j, widget in bookList.widget.binder:forEach("item_") do
				widget.binder.item.binder.dragzone.onDragDrop = nil
			end
		end
	elseif (ev.eventType == mui_defs.EVENT_MouseUp and ev.button == mui_defs.MB_Right) or
		(ev.eventType == mui_defs.EVENT_KeyDown and ev.key == mui_defs.K_ESCAPE) then
		
		hideSlotGlow( self )
		self.preparingSlotIndex = nil
		self.preparingSpellId = nil
	end
end

function commonDialog:destroy()
	if self.screen then
		local screen = self.screen
		self.screen = nil
	
		if self._cashUpdateThreadMonster then
			self._cashUpdateThreadMonster:stop()
			self._cashUpdateThreadMonster = nil
		end
		
		if self.onClose then
			self:onClose()
		end
		
		screen:createTransition( "deactivate_left",
			function( transition )
				mui.deactivateScreen( screen )
			end,
		{ easeOut = true } )
	end
end

-------------------------------------------------------------
------------------- DOWNTIME PREPARATION --------------------
-------------------------------------------------------------

local prepareDialog = class( commonDialog )

-------------------------------------------------------------
-- Learning spells

function prepareDialog:onClickLearnSpell( scroll )
	local spellId = scroll.upgradeParams.traits.storedSpell
	local spell = abilitydefs.lookupAbility( spellId )
	local itemDef = itemdefs[ "item_wiz_spellscroll" ]
	local cost = itemDef.traits.copyingCostPerLevel * spell.spellLevel
	
	if self.agency.cash >= cost then
		local body = util.sformat( STRINGS.WIZARD.UI.LEARN_SPELL_CONFIRM, spell:getName(), cost )
		if self.magicLevel < minimumLevelForSpell[spell.spellLevel] then
			body = body..STRINGS.WIZARD.UI.HIGH_LEVEL_SPELL_WARNING
		end
		
		local result = modalDialog.showYesNo( body, STRINGS.WIZARD.UI.LEARN_SPELL )
		
		if result == modalDialog.OK then
			MOAIFmodDesigner.playSound(cdefs.SOUND_HUD_BUY)
			
			if self._cashUpdateThreadMonster then
				self._cashUpdateThreadMonster:stop()
				self._cashUpdateThreadMonster = nil
			end

			local oldCash = self.agency.cash 
			self.agency.cash = self.agency.cash - cost 
			self._cashUpdateThreadMonster = guiex.createCountUpThread( self.panel.creditsTxt, oldCash, self.agency.cash, 1, STRINGS.UI.UPGRADE_SCREEN_AVAILABLE_CREDITS)
			
			local row = math.ceil( (#self.book + 1) / 6 )
			local column = 1 + ( #self.book % 6 )
			
			if column == 1 then
				local bookList = self.panel.spellBookList:addItem( self )
				bookList:setTooltip( nil )
				
				for i, widget in bookList.binder:forEach("item_") do
					self:updateSpellBookWidget( (row - 1) * 6 + i, widget )
				end
			end
			
			local bookList = self.panel.spellBookList:getItem( row )
			local widget = bookList.widget.binder[ "item_"..column ]
			
			table.insert( self.book, spellId )
			
			self:updateSpellBookWidget( #self.book, widget, spellId )
			local list = self.panel.spellScrollList
			
			for i, widget in pairs( list:getItems() ) do
				if widget.user_data == scroll then
					list:removeItem( i )
					local count = list:getMaxVisibleItems()
					if #list._items > count then
						list._scrollbar.binder.scrollbar:setRange( 0, #list._items - count )
						list._scrollbar:setVisible( true )
					else
						list._scrollbar:setVisible( false )
					end
					
					break
				end
			end

			self.stateUpgradeScreen:searchAndDestroyItem( scroll )
		end
	else
		modalDialog.show( STRINGS.UI.REASON.NOT_ENOUGH_CREDITS ) 
		--MOAIFmodDesigner.playSound("SpySociety/HUD/gameplay/upgrade_cancel_unit")
	end
end

-------------------------------------------------------------
-- Show spell scrolls (can be added to your spellbook for a price)

function prepareDialog:showScrolls( )
	local list = self.panel.spellScrollList
	local scroller = list._scrollbar.binder.scrollbar
	local w, h = list:getSize()
	scroller:setPosition( nil, 0 )
	scroller:setSize( nil, h )

	local scrolls = self.stateUpgradeScreen:searchForItem( "item_wiz_spellscroll" )
	
	for i, scroll in ipairs( scrolls ) do
		local widget = list:addItem( scroll )
		local spellId = scroll.upgradeParams.traits.storedSpell
		local spell = abilitydefs.lookupAbility( spellId )
		local itemDef = itemdefs[ "item_wiz_spellscroll" ]
		local cost = itemDef.traits.copyingCostPerLevel * spell.spellLevel
		local btn = widget.binder.item.binder.btn
	
		widget.binder.cost:setText( string.format( "-%d", cost ))
		widget.binder.level:setVisible( false )
		
		spell.smallFont = true
		widget.binder.itemName:setVisible( true )
		widget.binder.itemName:setText( util.sformat( STRINGS.WIZARD.ITEMS.SCROLL_OF, spell:getName() ) )
		spell.smallFont = nil
		
		btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.SCROLL_DESC, spell:getName(), cost ) )
		widget:setTooltip( nil )

		btn:setImage( "gui/icons/item_icons/icon-item_wizard_page.png" )
		btn:setDisabled( false )

		btn:setColor(cdefs.COLOR_FREE:unpack())
		btn:setColorInactive(cdefs.COLOR_FREE:unpack())
		btn:setColorActive(cdefs.COLOR_FREE_HOVER:unpack())
		btn:setColorHover(cdefs.COLOR_FREE_HOVER:unpack())
		
		btn.onClick = util.makeDelegate( nil, self.onClickLearnSpell, self, scroll )
	end
end

----------------------------------------------------------------
-- Spell preparation

function prepareDialog:prepareSpell( slotIndex, spellId )
	if commonDialog.prepareSpell( self, slotIndex, spellId ) then
		MOAIFmodDesigner.playSound( "SpySociety/HUD/gameplay/upgrade_confirm" )
		
		self.focus[ slotIndex ] = spellId
		
		local widget = self.panel.preparedSpells.binder["item_"..tostring( slotIndex )]
		self:updatePreparedSpellWidget( slotIndex, widget, spellId )
	end
end

-------------------------------------------------------------
-- Interface

function prepareDialog:show( agency, stateUpgradeScreen, book, focus, magicLevel )
	commonDialog.show( self, book, focus, magicLevel )
	
	self.agency = agency
	self.stateUpgradeScreen = stateUpgradeScreen
	
	self.panel.creditsTxt:setVisible( true )
	self.panel.creditsTxt:setText( string.format( STRINGS.UI.UPGRADE_SCREEN_AVAILABLE_CREDITS, self.agency.cash) )
	self.panel.bodyTxtCentral:spoolText( STRINGS.WIZARD.UI.PREPARE_SPELLS )
	
	self:showScrolls( )
end


-------------------------------------------------------------
------------------- MID-MISSION SWAPPING --------------------
-------------------------------------------------------------

local swapDialog = class( commonDialog )

function swapDialog:show( book, focus, magicLevel, expendedSpells, maxMp )
	self.expendedSpells = expendedSpells
	self.maxMp = maxMp
	self.result = nil
	
	commonDialog.show( self, util.tdupe( book ), focus, magicLevel )
	
	self.panel.bodyTxtCentral:spoolText( STRINGS.WIZARD.UI.SWAP_SPELL )
end

function swapDialog:prepareSpell( slotIndex, spellId )
	if commonDialog.prepareSpell( self, slotIndex, spellId ) then
		local slotLevel = slotLevels[ slotIndex ]
		local apCost = 2 * slotLevel
		if self.expendedSpells[ slotIndex ] then
			modalDialog.show( util.sformat( STRINGS.WIZARD.UI.SLOT_NOT_SWAPPABLE ) ) 
		elseif apCost > self.maxMp then
			modalDialog.show( util.sformat( STRINGS.WIZARD.UI.NOT_ENOUGH_AP_TO_PREPARE ) ) 
		else
			local slotLevel = slotLevels[ slotIndex ]
			local oldSpellId = self.focus[slotIndex]
			local spell = abilitydefs.lookupAbility( spellId )
			
			local body
			if oldSpellId then
				if oldSpellId == spellId then
					return
				end
			
				local oldSpell = abilitydefs.lookupAbility( oldSpellId )
				body = util.sformat( STRINGS.WIZARD.UI.CONFIRM_SWITCH_SPELL, slotLevel, oldSpell:getName(), spell:getName(), apCost )
			else
				body = util.sformat( STRINGS.WIZARD.UI.CONFIRM_PREPARE_FROM_BOOK, slotLevel, spell:getName(), apCost )
			end
			
			local confirm = modalDialog.showYesNo( body )
			
			if confirm == modalDialog.OK then
				self.result = { slotIndex = slotIndex, spellId = spellId, apCost = apCost }
				self:destroy()
			end
		end
	end
end

function swapDialog:updatePreparedSpellWidget( i, widget, spellId )
	commonDialog.updatePreparedSpellWidget( self, i, widget, spellId )
	
	if self.expendedSpells[ i ] then
		local btn = widget.binder.item.binder.btn
		local spell = abilitydefs.lookupAbility( spellId )
		btn:setDisabled( true )
		btn:setColor(0.5,0.5,0.5,1)
		btn:setColorInactive(0.5,0.5,0.5,1)
		btn:setColorActive(0.5,0.5,0.5,1)
		btn:setColorHover(0.5,0.5,0.5,1)
		btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.CANT_SWAP_EXPENDED, util.toupper( spell:getName() ) ) )
	end
end

-------------------------------------------------------------
------------------- MID-MISSION RECHARGE --------------------
-------------------------------------------------------------

local rechargeDialog = class( commonDialog )

function rechargeDialog:show( focus, magicLevel, expendedSpells, maxMp )
	self.expendedSpells = expendedSpells
	self.maxMp = maxMp
	self.result = nil
	
	commonDialog.show( self, {}, focus, magicLevel )
	
	self.panel.bodyTxtCentral:spoolText( STRINGS.WIZARD.UI.RECHARGE_SPELL )
end

function rechargeDialog:onClickSpellSlot( slotIndex )
	local spell = self.expendedSpells[ slotIndex ]
	
	if spell then
		local slotLevel = slotLevels[ slotIndex ]
		local apCost = 5 * spell:getLevel()
		if apCost > self.maxMp then
			modalDialog.show( util.sformat( STRINGS.WIZARD.UI.NOT_ENOUGH_AP_TO_PREPARE ) ) 
		else
			local body = util.sformat( STRINGS.WIZARD.UI.CONFIRM_RECHARGE_FROM_BOOK, spell:getLevel(), spell:getName(), apCost )
			local confirm = modalDialog.showYesNo( body )
			
			if confirm == modalDialog.OK then
				self.result = { slotIndex = slotIndex, apCost = apCost }
				self:destroy()
			end
		end
	end
end

function rechargeDialog:updatePreparedSpellWidget( i, widget, spellId )
	commonDialog.updatePreparedSpellWidget( self, i, widget, spellId )
	
	if not self.expendedSpells[ i ] and spellId then
		local btn = widget.binder.item.binder.btn
		local spell = abilitydefs.lookupAbility( spellId )
		btn:setDisabled( true )
		btn:setColor(0.5,0.5,0.5,1)
		btn:setColorInactive(0.5,0.5,0.5,1)
		btn:setColorActive(0.5,0.5,0.5,1)
		btn:setColorHover(0.5,0.5,0.5,1)
		btn:setTooltip( util.sformat( STRINGS.WIZARD.UI.CANT_RECHARGE_NOT_EXPENDED, util.toupper( spell:getName() ) ) )
	end
end

return {
	prepareDialog = prepareDialog,
	swapDialog = swapDialog,
	rechargeDialog = rechargeDialog
}