local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local commondefs = include( "sim/unitdefs/commondefs" )
local simfactory = include( "sim/simfactory" )
local mui_anim = include("mui/widgets/mui_anim")

local ZOMBIE_SOUNDS = {
	appeared="SpySociety/HUD/gameplay/peek_negative", 
	alert ="SpySociety/Actions/guard/guard_alerted", 
	speech="SpySociety/Agents/dialogue_KO", 
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
	step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,

	getup = "SpySociety/Movement/foley_guard/getup",
	fall = "SpySociety/Movement/foley_guard/fall",
	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 34,					
	grabbed = "SpySociety/Movement/foley_guard/grabbed",
	pin = "SpySociety/Movement/foley_guard/pin_guard",
	pinned = "SpySociety/Movement/foley_guard/pinned",
	move ="SpySociety/Movement/foley_guard/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",	
}

local zombie_template = {
	type = "simunit",
	name = "Zombie",
	onWorldTooltip = function( tooltip, unit )
		commondefs.onAgentTooltip( tooltip, unit )
	
	    if unit:getTraits().neural_scanned then
			tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.NEURAL_SCANNED ), string.format( STRINGS.UI.TOOLTIPS.NEURAL_SCANNED_DESC), "gui/icons/arrow_small.png" )
	    end    		
	end,
	
	traits = util.extend( commondefs.basic_agent_traits ) {
		walk = true,
		enforcer = false,
		dashSoundRange = 8,
		canBeCritical = false,
		silencer = false,
		sneaking = true,
		LOSarc = math.pi * 2,
		leavesAtEndOfMission = true,
		isZombie = true,
	},
	
	skills = {},
	abilities = { "shootOverwatch", "overwatch", "peek", "escape", "melee", "disarmtrap", "overwatchMelee" , "meleeOverwatch", "moveBody" },
	children = {},
	sounds = ZOMBIE_SOUNDS,
}

local function giveAll( unit, targetUnit )
	for i, item in ipairs( unit:getChildren() ) do
		targetUnit:addChild( item )
	end
	util.tclear( unit:getChildren() )
end

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.ANIMATE_DEAD,

	profile_icon = "gui/icons/spells/animate_dead.png",
	spellLevel = 3,
	
	iconColor = util.color( 189/255, 243/255, 148/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not targetUnit:getTraits().iscorpse then
			return false
		end
		if targetUnit:getTraits().wasDrone then
			return false
		end
		
		return sim:canUnitSeeUnit( userUnit, targetUnit )
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 5 * (self:getLevel() - 2)
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
			local x0, y0 = targetUnit:getLocation()
			if x0 then
				local cell = sim:getCell( x0, y0 )
				for i, cellUnit in ipairs( cell.units ) do
					if simquery.isAgent( cellUnit ) and not cellUnit:isKO() then
						return false, STRINGS.UI.REASON.ENEMY_PINNED
					end
				end
			end
		end

		return true
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			if not mui_anim.zombiePortraits then
				mui_anim.zombiePortraits = true
				
				local oldBindBuild = mui_anim.bindBuild

				function mui_anim:bindBuild( build )
					if self.zombie then
						self._cont._prop:setSymbolVisibility( "eyeball", true )
						self.zombie = nil
					end
					
					if build:sub(1,6) == "zombie" then
						self.zombie = true
						self._cont._prop:setSymbolVisibility( "eyeball", false )
						build = build:sub(7)
					end
					
					return oldBindBuild( self, build )
				end
			end
			
			
			local corpseTemplate = util.extend( zombie_template )
			{
				name = string.sub( targetUnit:getName(), #STRINGS.UI.CORPSE_OF - 2 ) .. STRINGS.WIZARD.UNITS.ZOMBIE,
				kanim = targetUnit:getUnitData().kanim,
				profile_anim = targetUnit:getUnitData().profile_anim,
				profile_build = targetUnit:getUnitData().profile_build,
			}
			
			if corpseTemplate.profile_build and corpseTemplate.profile_build:sub(1,6) ~= "zombie" then
				corpseTemplate.profile_build = "zombie"..corpseTemplate.profile_build
			end

			local zombie = simfactory.createUnit( corpseTemplate, sim )

			zombie:getTraits().cashOnHand = targetUnit:getTraits().cashOnHand
			zombie:getTraits().neural_scanned = targetUnit:getTraits().neural_scanned 
			zombie:getTraits().koTimer = 1
			zombie:getTraits().disguiseOn = true
			zombie:setPlayerOwner( sim:getPC() )
			zombie.setDisguise = function() end
			self.zombie = zombie
			
			-- Don't put enemy weapons in storage
			local oldReturnItemsToStash = zombie.returnItemsToStash
			zombie.returnItemsToStash = function( zombieSelf, ... )
				local children = zombieSelf:getChildren()
				
				for i = #children, 1, -1 do
					local child = children[i]
					
					if not inventory.isCarryable( child ) then
						zombieSelf:removeChild( child )
						sim:despawnUnit( child )
					end
				end
			
				return oldReturnItemsToStash( zombieSelf, ... )
			end
			
			sim:spawnUnit( zombie )
			sim:warpUnit( zombie, sim:getCell( targetUnit:getLocation()), targetUnit:getFacing() )
			
			giveAll( targetUnit, zombie )

			sim:warpUnit( targetUnit, nil )
			sim:despawnUnit( targetUnit )
			
			zombie:tickKO( sim )
			
			sim:addTrigger( simdefs.TRG_UNIT_HIT, self )
			
			local oldUpdateWinners = sim.updateWinners
			
			sim.updateWinners = function()
				oldUpdateWinners(sim)
				if not sim:isGameOver() then
					if self.zombie and not self.concentrationEnded and not self.zombie:isValid() then
						self.arcaneFocus:executeAbility( sim )
					elseif self.concentrationEnded and self.zombie and self.zombie:isValid() then
						self:endConcentration( sim )
					end
				end
			end
			
			for i, laserUnit in pairs( sim:getAllUnits() ) do
				if laserUnit.canControl then
					local oldCanControl = laserUnit.canControl
					
					laserUnit.canControl = function( selfUnit, otherUnit )
						if otherUnit:getTraits().isZombie then
							return true
						else
							return oldCanControl( selfUnit, otherUnit )
						end
					end
				end
			end
		end

		self:endThrow( sim, abilityOwner, unit )
	end, 
	
	endConcentration = function( self, sim, escaped )
		self.concentrationEnded = true
		if self.zombie then
			if not escaped then
				if self.zombie:isValid() then
					self.zombie:killUnit( sim )
				end
				
				self.zombie = nil
				sim:removeTrigger( simdefs.TRG_UNIT_HIT, self )
			end
		end
	end,
	
	onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_UNIT_HIT and self.zombie and evData.sourceUnit == self.zombie then
			local seers = sim:generateSeers( self.zombie )
			local x0, y0 = self.zombie:getLocation()
			
			self.zombie:getTraits().disguiseOn = false
			self.zombie:getTraits().hidesInCover = true

	        for i,seer in ipairs(seers) do
	        	local unit = sim:getUnit(seer)
	        	if unit and unit:isNPC() then
				    if simquery.couldUnitSee( sim, unit, self.zombie, true ) then
				    	sim:triggerEvent( simdefs.TRG_UNIT_APPEARED, { seerID = seer, unit = self.zombie } )			   
				    end
				end
	        end
		end
	end,
}

return spellDef