local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local commondefs = include( "sim/unitdefs/commondefs" )

local eyeball360 = {
	type = "simunit",
	name = STRINGS.WIZARD.UNITS.EYEBALL,
	facing = simdefs.DIR_N,	
	rig = "corerig",
	kanim = "kanim_wizard_clairvoyance",
	traits = { hasSight = true, hasHearing = true, seesHidden = true, LOSarc = math.pi * 2, detect_cloak = true },
	onWorldTooltip = commondefs.onItemWorldTooltip,
}
local eyeball360_small = {
	type = "simunit",
	name = "",
	facing = simdefs.DIR_N,	
	rig = "eyeballrig",
	traits = { hasSight = true, hasHearing = true, seesHidden = true, LOSarc = math.pi * 2, detect_cloak = true },
}

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.CLAIRVOYANCE,

	profile_icon = "gui/icons/spells/clairvoyance.png",		
	spellLevel = 3,
	ignoreLOS = true,
	
	iconColor = util.color( 183/255, 215/255, 233/255 ),

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetCell )
		--return (5 * self:getLevel()) - 5
	end,
	
	getThrowRange = function( self )
		return 3 * self:getLevel() - 1
	end,
	
	getDescParams = function( self )
		return self:getThrowRange()
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
		
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/agent_peek" )
			self.eyeballUnits = {}
		
			self:spawnEyeBall( sim, sim:getCell( x1, y1 ), eyeball360, userUnit )
			
			for i, exit in pairs( sim:getCell( x1, y1 ).exits ) do
				self:spawnEyeBall( sim, exit.cell, eyeball360_small, userUnit )
			end
		end

		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	spawnEyeBall = function( self, sim, cell, template, userUnit )
		local eyeballUnit = simfactory.createUnit( template, sim )
		eyeballUnit:setPlayerOwner( userUnit:getPlayerOwner() )
        eyeballUnit:getTraits().peekID = userUnit:getID()
		table.insert( self.eyeballUnits, eyeballUnit )

		sim:spawnUnit( eyeballUnit )
		sim:warpUnit( eyeballUnit, cell )
	end,
	
	endConcentration = function( self, sim )
		for i, unit in ipairs(self.eyeballUnits) do
			if unit:isValid() then
				sim:warpUnit( unit, nil )
				sim:despawnUnit( unit )
			end
		end
		
		self.eyeballUnits = nil
	end,
}

return spellDef