local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local simunit = include( "sim/simunit" )

local courierDef = {
	type = "simunit",
	profile_anim = "wizard/courier_portrait",
	rig = "eyeballrig",
	name = STRINGS.WIZARD.UNITS.COURIER,
	traits = {hardInventoryLimit = true},
	children = {}
}

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.COURIER,

	profile_icon = "gui/icons/spells/speedy_courier.png",		
	spellLevel = 3,
	
	getMaxTransfers = function( self )
		return 2 * self:getLevel() - 2
	end,
	
	getDescParams = function( self )
		return self:getMaxTransfers()
	end,
	
	iconColor = util.color( 255/255, 244/255, 140/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if targetUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() then
			return false
		end
		if not targetUnit:getTraits().isAgent then
			return false
		end
		if targetUnit:getTraits().isDrone then
			return false
		end
		if (targetUnit:getTraits().inventoryMaxSize or 0) == 0 then
			return false
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetUnitID )
		if not sim._itemsTakenOutOfStorage then
			sim._itemsTakenOutOfStorage = {}
		end
		
		if targetUnitID < 100 then
			include("gameplay/eyeballrig").rig.setHidden = function( self, isHidden )
				if isHidden ~= self._isHidden then
					self._isHidden = isHidden
					if self.refreshLocation then
						self:refreshLocation()
					end
				end
			end
		
			local courierUnit = sim.courierUnit
			
			sim:warpUnit( courierUnit, sim:getCell( self.targetUnit:getLocation() ) )
			sim:dispatchEvent( simdefs.EV_TELEPORT, { units = { courierUnit }, warpOut = true } )
			sim:warpUnit( courierUnit, nil )
			
			for id, item in pairs(sim:getAllUnits()) do
				if item:getTraits().teleportedFromStorage and item:getUnitOwner() ~= courierUnit then
					sim._itemsTakenOutOfStorage[item:getTraits().teleportedFromStorageIndex] = item
					item:getTraits().teleportedFromStorage = nil
				end
				if item:getTraits().isObjective and item:getUnitOwner() == courierUnit then
					sim:getPC():setEscapedWithObjective(true)
				end
			end
			
			sim:triggerEvent( simdefs.TRG_UNIT_ESCAPED, courierUnit )
			sim:addTrigger( simdefs.TRG_GAME_OVER, self ).priority = 10
			
			self:castSpell( sim, self.wizard, self.wizard )
			self:endThrow( sim, self.wizard, self.wizard )
		else
			local targetUnit = sim:getUnit( targetUnitID )
			local courierUnit = sim.courierUnit
			if not courierUnit then
				local courierTemplate = util.tcopy( courierDef )
				local upgradedefs = include( "sim/upgradedefs" )
				for i, upgrade in ipairs(sim:getParams().agency.upgrades) do
					local t = { traits = {
						teleportedFromStorage = true,
						teleportedFromStorageIndex = i
					} }
					
					if type(upgrade) == "string" then
						upgradedefs.applyUpgrade( upgrade, courierTemplate, t )
					else
						upgradedefs.applyUpgrade( upgrade.upgradeName, courierTemplate, util.extend( upgrade.upgradeParams )( t ) )
					end
				end
				
				courierUnit = simunit.createUnit( courierTemplate, sim )
				courierUnit.initialChildren = util.tdupe(courierUnit:getChildren())
				sim.courierUnit = courierUnit
				sim:spawnUnit( courierUnit )
				
				if type(sim.exit_warning) == "function" then
					local oldWarning = sim.exit_warning
					sim.exit_warning = function(...)
						for i, child in ipairs(courierUnit:getChildren()) do
							child._parent = nil
						end
						
						local str = oldWarning(...)
						
						for i, child in ipairs(courierUnit:getChildren()) do
							child._parent = courierUnit
						end
						
						return str
					end
				end
			end
			
			courierUnit:getTraits().inventoryMaxSize = math.max((sim:getParams().difficultyOptions.agencyInventorySize or 4), self:getMaxTransfers())
			self.targetUnit = targetUnit
			self.wizard = userUnit
			courierUnit._parent = sim:getPC()
			sim:dispatchEvent( "EV_WIZ_COURIER", {unit = targetUnit, abilityOwner = abilityOwner, target = courierUnit, ability = self} )
		end
	end,
	
	onTrigger = function( self, sim )
		--if sim:getPlayers()[sim:getWinner()]:isPC() and sim.courierUnit then
		if sim.courierUnit then
			local courierUnit = sim.courierUnit
			local user = savefiles.getCurrentGame()
			local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
			local agency = campaign.agency
			sim.courierUnit = nil
			
			for i = #agency.upgrades, 1, -1 do
				if sim._itemsTakenOutOfStorage[i] and sim._itemsTakenOutOfStorage[i]:getUnitOwner() ~= courierUnit then
					table.remove(agency.upgrades, i)
				end
			end
			
			sim._itemsTakenOutOfStorage = {}
			
			for i, item in ipairs( courierUnit:getChildren() ) do
				if not item:getTraits().installed and not item:getTraits().teleportedFromStorageIndex then
					local upgradeName = item:getUnitData().upgradeName
					if upgradeName then
						local unitData = item:getUnitData()
						if item:getUnitData().upgradeOverride then
						   upgradeName = item:getUnitData().upgradeOverride
						   unitData =  unitdefs.lookupTemplate( upgradeName ) 
						end
						local upgradeParams = unitData.createUpgradeParams and unitData:createUpgradeParams( item )
						table.insert( agency.upgrades, { upgradeName = upgradeName, upgradeParams = upgradeParams })
					end
					
					-- New Locations
					if item:getTraits().newLocations then
						sim._newLocations = util.tmerge( sim._newLocations, item:getTraits().newLocations )
					end

					-- UNLOCK LOGS
					if item:getTraits().lostData then
						local potentialLogIds, potentialLogs = {}, {}
						user.data.logs = user.data.logs or {}
						
						for i, log in ipairs(serverdefs.LOGS) do
							potentialLogIds[log.id] = i
						end
						for i, log in ipairs(user.data.logs) do
							potentialLogIds[log.id] = nil
						end
						for id, idx in pairs(potentialLogIds) do
							table.insert(potentialLogs, serverdefs.LOGS[idx])
						end
						
						if #potentialLogs > 0 then
							local rand = include( "modules/rand" )
							local gen = rand.createGenerator( campaign.seed )
							local logID = gen:nextInt( 1,  #potentialLogs )
							local newLog = util.tcopy(potentialLogs[logID])    
							print("UNLOCKED LOG DATA",newLog.id)                    
							table.insert(user.data.logs,newLog)                        
							sim._resultTable.newLogData = newLog
						end
					end

					-- Cash reward items
					if item:getTraits().cashInReward then
						local value = sim:getQuery().scaleCredits(sim, item:getTraits().cashInReward )
						sim:addMissionReward(value)
						sim._resultTable.credits_gained.stolengoods = (sim._resultTable.credits_gained.stolengoods or 0) + value
					end
				end
			end
		end
	end,
}

return spellDef