local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local modifiers = include( "sim/modifiers" )

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.ENFEEBLEMENT,

	profile_icon = "gui/icons/spells/enfeeblement.png",
	spellLevel = 1,
	
	iconColor = util.color( 189/255, 243/255, 148/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not targetUnit:getTraits().isGuard or targetUnit:getPlayerOwner() == userUnit:getPlayerOwner() then
			return false
		end
		if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		
		return true
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 30
	end,
	
	getDescParams = function( self, sim, unit )
		return self:getPenalty()
	end,
	
	getPenalty = function( self, sim, unit )
		return 3 * self:getLevel()
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
		
			if targetUnit:getTraits().isDrone then
				return false, STRINGS.UI.TOOLTIPS.KO_IMMUNE
			end
		end
		return true
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			local penalty = self:getPenalty()
			self.targetUnit = targetUnit
			targetUnit:getTraits().mpMax = targetUnit:getTraits().mpMax - penalty
			targetUnit:getTraits().mp = targetUnit:getTraits().mp - penalty
			targetUnit:getModifiers():add( "LOSrange", "enfeeblement", modifiers.ADD, -penalty, 1, nil, -1 )
			targetUnit:getModifiers():add( "LOSperipheralRange", "enfeeblement", modifiers.ADD, -penalty, 1.5, nil, -1 )
			
			if targetUnit:isNPC() and targetUnit:getBrain() and not targetUnit:isKO() then
				if targetUnit:getTraits().vip then
					sim:emitSpeech( targetUnit, speechdefs.SOUND_SPEECH_GUARD_FLEE_COWER )
				else
					sim:emitSpeech( targetUnit, speechdefs.HUNT_WAKEUP )
				end
			end
			
			sim:refreshUnitLOS( targetUnit )
			
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_lightning_strike" )
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = targetUnit } )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
		
		self:endThrow( sim, abilityOwner, unit )
	end, 
	
	endConcentration = function( self, sim )
		local targetUnit = self.targetUnit
		if targetUnit and targetUnit:isValid() then
			local penalty = self:getPenalty()
			targetUnit:getTraits().mpMax = targetUnit:getTraits().mpMax + penalty
			targetUnit:getTraits().mp = targetUnit:getTraits().mp + penalty
			targetUnit:getModifiers():remove( "enfeeblement" )
			sim:refreshUnitLOS( targetUnit )
		end
	end,
}

return spellDef