local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local skilldefs = include( "sim/skilldefs" )

local skillOverrides = {
	hacking = {
		[6] = {
			-- Thinking a hacking bonus of 2 might be a bit too strong... but not sure
			tooltip = STRINGS.SKILLS.HACKING2_TOOLTIP,
			onLearn = function(sim, unit)
                unit:getTraits().hacking_bonus = unit:getTraits().hacking_bonus + 1
			end,
			onUnLearn = function(sim, unit)
                unit:getTraits().hacking_bonus = unit:getTraits().hacking_bonus - 1
			end,
		}
	},
	
	inventory = {
		[2] = {
			onUnLearn = function(sim, unit) 	
    			unit:getTraits().inventoryMaxSize = unit:getTraits().inventoryMaxSize - 1
    			unit:getTraits().dragCostMod = unit:getTraits().dragCostMod - 0.5
			end,
		},

		[3] = {
			onUnLearn = function(sim, unit) 	
    			unit:getTraits().inventoryMaxSize = unit:getTraits().inventoryMaxSize - 1
			end,
		},

		[4] = {
			onUnLearn = function(sim, unit) 	
    			unit:getTraits().inventoryMaxSize = unit:getTraits().inventoryMaxSize - 1
    			unit:getTraits().dragCostMod = unit:getTraits().dragCostMod - 0.5
			end,
		},

		[5] = {
			onUnLearn = function(sim, unit) 	
    			unit:getTraits().inventoryMaxSize = unit:getTraits().inventoryMaxSize - 1
    			unit:getTraits().meleeDamage = unit:getTraits().meleeDamage - 1
			end,
		},
		[6] = {
			tooltip = STRINGS.WIZARD.UI.STRENGTH6,
			onLearn = function(sim, unit)
    			unit:getTraits().inventoryMaxSize = unit:getTraits().inventoryMaxSize + 1
    			unit:getTraits().dragCostMod = unit:getTraits().dragCostMod + 0.5
				unit:getTraits().meleeDamage = unit:getTraits().meleeDamage + 1
			end,
			onUnLearn = function(sim, unit)
    			unit:getTraits().inventoryMaxSize = unit:getTraits().inventoryMaxSize - 1
    			unit:getTraits().dragCostMod = unit:getTraits().dragCostMod - 0.5
				unit:getTraits().meleeDamage = unit:getTraits().meleeDamage - 1
			end,
		},
		
		[7] = {
			tooltip = STRINGS.WIZARD.UI.STRENGTH7,
			onLearn = function(sim, unit)
				unit:getTraits().meleeDamage = unit:getTraits().meleeDamage + 1
			end,
			onUnLearn = function(sim, unit)
				unit:getTraits().meleeDamage = unit:getTraits().meleeDamage - 1
			end,
		},
	},
	
	anarchy = {
		[3] = {
			onUnLearn = function(sim, unit) 	
    			unit:getTraits().stealBonus = unit:getTraits().stealBonus - 0.15
			end, 
		},

		[4] =  {
			onUnLearn = function(sim, unit) 	
    			unit:getTraits().stealBonus = unit:getTraits().stealBonus - 0.20
			end, 
		},

		[5] =  {
			onUnLearn = function(sim, unit) 	
			    unit:getTraits().anarchyItemBonus = false
            end, 
		},
		
		[6] = {
			tooltip = STRINGS.SKILLS.ANARCHY4_TOOLTIP, 
			onLearn = function(sim, unit)
				unit:getTraits().stealBonus = (unit:getTraits().stealBonus or 0) + 0.20
			end,
			onUnLearn = function(sim, unit)
				unit:getTraits().stealBonus = (unit:getTraits().stealBonus or 0) - 0.20
			end,
		},
	},
}

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.ENHANCE_ABILITY,

	profile_icon = "gui/icons/spells/enhance_ability.png",
	spellLevel = 1,
	
	iconColor = util.color( 208/255, 134/255, 64/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if targetUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() then
			return false
		end
		if not targetUnit:getTraits().isAgent then
			return false
		end
		if targetUnit:getTraits().isDrone then
			return false
		end
		if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		
		return true
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 10
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		local lvl = self:getLevel()
		local skillId = sim:dispatchChoiceEvent( "EV_WIZ_CHOOSE_SKILL", { unit = targetUnit, level = lvl, skillOverrides = skillOverrides } )
		
		if skillId and skilldefs.lookupSkill( skillId ) then
			if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
				self.targetUnit = targetUnit
				self.limitBreak = 0
				
				targetUnit._skills = targetUnit._skills or {}
				local skill = targetUnit:hasSkill( skillId )
				if not skill then
					local simskill = include( "sim/simskill" )
					skill = simskill( skillId, 1, sim, targetUnit )
					skill._currentLevel = 0
					table.insert( targetUnit._skills, skill )
				end
				
				self.skill = skill
				local idx = array.find( targetUnit:getSkills(), skill )
				local def = util.tcopy( skill:getDef() )
				function skill:getDef()
					return def
				end
				targetUnit:getTraits().enhanceAbility = { skillId = skillId, level = lvl }
				targetUnit:getTraits().temp_skill_points = targetUnit:getTraits().temp_skill_points or {}
				
				for i = 1, lvl do
					if skill:getCurrentLevel() >= def.levels then
						def[ def.levels + 1 ] = util.tcopy( def[def.levels] )
						def.levels = def.levels + 1
						self.limitBreak = self.limitBreak + 1
					end
					
					local toLevel = skill:getCurrentLevel() + 1
					if skillOverrides[ skillId ] and skillOverrides[ skillId ][ toLevel ] then
						util.tmerge( def[ toLevel ], skillOverrides[ skillId ][ toLevel ] )
					end
					
					table.insert( targetUnit:getTraits().temp_skill_points, idx )
					skill:levelUp( sim, targetUnit )
				end
				
				
				local dialogParams = {
					STRINGS.WIZARD.UI.ENHANCE_ABILITY_MODAL_1,
					STRINGS.WIZARD.UI.ENHANCE_ABILITY_MODAL_2,
					util.sformat( STRINGS.WIZARD.UI.ENHANCE_ABILITY_MODAL_3, targetUnit:getName(), util.toupper( def.name ) ),
					"gui/icons/spells/enhance_ability.png",
				}
				
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/transferData" )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
				sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "programDialog", dialogParams = dialogParams } )
			end
			
			self:endThrow( sim, abilityOwner, unit )
		end
	end, 
	
	endConcentration = function( self, sim )
		local targetUnit = self.targetUnit
		if targetUnit then
			targetUnit:getTraits().enhanceAbility = nil
			
			local idx = array.find( targetUnit:getSkills(), self.skill )
			for i = 1, self:getLevel() do
				if self.skill:getCurrentLevel() <= 1 then
					array.removeElement( targetUnit:getSkills(), self.skill )
				else
					self.skill:levelDown( sim, targetUnit )
				end
				
				array.removeElement( targetUnit:getTraits().temp_skill_points, idx )
			end
			
			if self.limitBreak > 0 then
				local def = self.skill:getDef()
				for i = 1, self.limitBreak do
					def[ def.levels ] = nil
					def.levels = def.levels - 1
				end
			end
		end
	end,
}

return spellDef