local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local mainframe = include( "sim/mainframe" )

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.FAERIE_FIRE,

	profile_icon = "gui/icons/spells/faerie_fire.png",		
	spellLevel = 1,
	squareArea = true,
	
	iconColor = util.color( 232/255, 96/255, 69/255 ),
	
	getEffectRadius = function( self, sim )
		return 2 * self:getLevel()
	end,
	
	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 5 * self:getLevel()
	end,
	
	getDescParams = function( self, sim, unit )
		return self:getEffectRadius( sim )
	end,
	
	isAreaTargetUnit = function( self, sim, userUnit, cellUnit )
		return cellUnit:getTraits().isGuard
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
		
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			local targets = include( "hud/targeting" )
			local cells = targets.fillSquare( sim, x1, y1, self:getEffectRadius( sim ), userUnit, nil, self.areaPassesThroughWalls )
			local taggedUnits = {}
			local seerUnits = {}
			self.taggedUnits = taggedUnits
			
			for i = 1, #cells, 2 do
                local cell = sim:getCell( cells[i], cells[i+1] )
                if cell then
                    for j, cellUnit in ipairs(cell.units) do
                        if cellUnit:getTraits().isGuard and not cellUnit:getTraits().tagged then
							cellUnit:setTagged()
							table.insert( taggedUnits, cellUnit )
						end
					end
					
					local seers = sim:getLOS():getSeers( cell.x, cell.y )
					for _, unitID in ipairs(seers) do
						local seerUnit = sim:getUnit( unitID )
						if seerUnit and seerUnit:getTraits().isGuard and seerUnit:getBrain() then
							local sx, sy = seerUnit:getLocation()
							if seerUnits[unitID] == nil or mathutil.distSqr2d(sx, sy, cell.x, cell.y) < mathutil.distSqr2d(sx, sy, seerUnits[unitID].x, seerUnits[unitID].y) then
								seerUnits[unitID] = cell
							end
						end
					end
				end
			end
			
			sim:dispatchEvent( "EV_WIZ_FAERIE_FIRE", { cells = cells, x = x1, y = y1, range = self:getEffectRadius( sim ) } )
			
			for unitID, seenTile in pairs(seerUnits) do
				log:write(tostring(unitID))
				local seerUnit = sim:getUnit( unitID )
				seerUnit:getBrain():getSenses():addInterest( seenTile.x, seenTile.y, simdefs.SENSE_SIGHT, simdefs.REASON_DOOR )
			end
		end

		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	endConcentration = function( self, sim )
		for i, targetUnit in ipairs( self.taggedUnits ) do 
			if targetUnit:isValid() then
				targetUnit:setTagged( true )
			end
		end
	end,
}

return spellDef