local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local mainframe = include( "sim/mainframe" )

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.FIREBALL,

	profile_icon = "gui/icons/spells/fireball.png",		
	spellLevel = 3,
	effectRadius = 3,
	
	iconColor = util.color( 232/255, 96/255, 69/255 ),
	
	getArmorPiercing = function( self )
		return self:getLevel() - 2
	end,
	
	getArmorDamage = function( self )
		return self:getLevel() - 1
	end,
	
	getDescParams = function( self, sim, unit )
		return self:getArmorPiercing(), self:getArmorDamage()
	end,
	
	isAreaTargetUnit = function( self, sim, userUnit, cellUnit )
		return cellUnit:getWounds() or cellUnit:getTraits().canBeShot
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
		
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			-- Create a fiery cloud for visuals
			local smokeDef = util.extend( unitdefs.lookupTemplate( "smoke_cloud" ) ) {
				traits = {
					gasColor = util.color( 232/255, 153/255, 69/255 ),
					radius = self.effectRadius
				}
			}
			
			local smoke = simfactory.createUnit( smokeDef, sim )
			sim:spawnUnit( smoke )
			sim:emitSound( { path = "SpySociety/Grenades/flashbang_explo", range = simdefs.SOUND_RANGE_2, }, x1, y1 )
			
			-- Skip the LOS blocking effect
			local insertSegments = sim:getLOS().insertSegments
			local removeSegments = sim:getLOS().removeSegments
			sim:getLOS().insertSegments = function() end
			sim:getLOS().removeSegments = function() end
		
			sim:warpUnit( smoke, sim:getCell( x1, y1 ) )
			sim:warpUnit( smoke, nil )
			sim:despawnUnit( smoke )
			
			sim:getLOS().insertSegments = insertSegments
			sim:getLOS().removeSegments = removeSegments
			
			-- Do damage
			local fillCells = simquery.fillCircle( sim, x1, y1, self.effectRadius, 0)
			
			local armorPiercing = self:getArmorPiercing()
			local armorDamage = self:getArmorDamage()
			
			sim.wizSupressMainframe = true
			sim:startTrackerQueue(true)
			sim:dispatchEvent( "EV_WIZ_NO_MAINFRAME", { on = true } )
			sim:startDaemonQueue()
			sim:dispatchEvent( simdefs.EV_KO_GROUP, true )

			for i, cell in ipairs(fillCells) do
				local raycastX, raycastY = sim:getLOS():raycast( x1, y1, cell.x, cell.y )
				if raycastX == cell.x and raycastY == cell.y then
					for i, cellUnit in ipairs( util.tdupe( cell.units ) ) do
						if cellUnit:getWounds() or cellUnit:getTraits().canBeShot then
							if cellUnit:getArmor() > armorPiercing then
								local dmgLeft = armorDamage
								local shieldHit = false
								local fleshHit = false
								-- Defense Matrix from AGP blocks the damage first
								if cellUnit:getTraits().temporaryArmor and cellUnit:getTraits().temporaryArmor > 0 and dmgLeft > 0 then
									local d = dmgLeft
									dmgLeft = dmgLeft - math.ceil(0.5 * cellUnit:getTraits().temporaryArmor)
									cellUnit:getTraits().temporaryArmor = cellUnit:getTraits().temporaryArmor - 2 * d
									shieldHit = true
									if cellUnit:getTraits().temporaryArmor <= 0 then
										cellUnit:getTraits().temporaryArmor = nil
									end
								end
								
								-- Next is barrier shields
								if cellUnit:getTraits().firewallArmor and dmgLeft > 0 then
									local iceArmor = cellUnit:getTraits().mainframe_ice - (cellUnit:getTraits().weakBarrier or 0)
									
									if iceArmor > 0 and cellUnit:getTraits().mainframe_status ~= "off" and not cellUnit:isKO() then
										shieldHit = true
										mainframe.breakIce( sim, cellUnit, math.min( iceArmor, dmgLeft ) )
										dmgLeft = dmgLeft - iceArmor
									end
								end
								
								-- Lastly permanently damage armor
								if dmgLeft > 0 then
									cellUnit:getTraits().armor = (cellUnit:getTraits().armor or 0) - dmgLeft
									fleshHit = true
								end
								
								if shieldHit then
									sim:dispatchEvent( simdefs.EV_UNIT_HIT_SHIELD, {unit = cellUnit, shield = cellUnit:getTraits().shields or 0} )
								end
								if fleshHit then
									sim:dispatchEvent( simdefs.EV_UNIT_HIT, { unit = cellUnit, result = 0 } )
								end
								
								sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = cellUnit } )
							else
								-- Kill em!
								sim:damageUnit(cellUnit, 3)
							end
						end
					end
				end
			end
			
			sim.wizSupressMainframe = nil
			sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
			sim:dispatchEvent( "EV_WIZ_NO_MAINFRAME", { on = false } )
			sim:startTrackerQueue(false)				
			sim:processDaemonQueue()
		end

		self:endThrow( sim, abilityOwner, userUnit )
	end,
}

return spellDef