local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local prop_templates = include( "sim/unitdefs/propdefs" )

local smokeTemplate = {
	type = "simunit",
	name = STRINGS.PROPS.SMOKE,
	rig = "smokerig",
	kanim = "kanim_smoke_plume",
	traits = { noghost = true }
}

local function createSmokeEdge( sim, cell )
	local unit = simfactory.createUnit( prop_templates.smoke_edge, sim )
	sim:spawnUnit( unit )
	sim:warpUnit( unit, cell )
	return unit
end

local function occludeSight( sim, targetCell, smokeRadius )
	local cells = simquery.floodFill( sim, nil, targetCell, smokeRadius, simquery.getManhattanMoveCost, simquery.canPathBetween )
	local interestUnits, hasInterestUnit = {}, {}
	for _, cell in ipairs(cells) do
		cell.selfCover = (cell.selfCover or 0) + 1
	end
	for _, cell in ipairs(cells) do
		for _, dir in ipairs( simdefs.DIR_SIDES ) do
			local dx, dy = simquery.getDeltaFromDirection( dir )
			local tocell = sim:getCell( cell.x + dx, cell.y + dy )
			if tocell and (tocell.selfCover or 0) == 0 and not hasInterestUnit[tocell] then
				if simquery.isOpenExit( cell.exits[ dir ] )  then
					-- Spawn an interest point here.
					local interestUnit = createSmokeEdge( sim, tocell )
					hasInterestUnit[tocell] = interestUnit
					table.insert( interestUnits, interestUnit:getID() )
				elseif simquery.isDoorExit( cell.exits[ dir ] ) and not hasInterestUnit[cell] then
					-- Spawn an interest point here.
					local interestUnit = createSmokeEdge( sim, cell )
					hasInterestUnit[cell] = interestUnit
					table.insert( interestUnits, interestUnit:getID() )
				end
			end
		end
	end
	
	sim:dispatchEvent( simdefs.EV_EXIT_MODIFIED ) -- Update shadow map.

	return cells, interestUnits
end

local smokeClass = {
	onWarp = function(self, sim, oldcell, cell)
		if self._cells then
			for i, cell in pairs(self._cells) do
				cell.selfCover = cell.selfCover - 1
			end
			self._cells = nil
			sim:dispatchEvent( simdefs.EV_EXIT_MODIFIED ) -- Update shadow map.
		end
		if self._interestUnits then
			for i, unitID in ipairs( self._interestUnits) do
				local unit = sim:getUnit( unitID )
				if unit then
					sim:warpUnit( unit, nil )
					sim:despawnUnit( unit )
				end
			end
			self._interestUnits = nil
		end
		if cell then
			self._cells, self._interestUnits = occludeSight( sim, cell, self:getTraits().radius )
			for i, cell in ipairs(self._cells) do
				for i, unit in ipairs(cell.units) do
					if unit:getBrain() and unit:getTraits().hasSight then
						unit:getBrain():getSenses():addInterest(cell.x, cell.y, simdefs.SENSE_SIGHT, simdefs.REASON_SMOKE)
					end
				end
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self } )	 
		end
		
		for i, unit in pairs(sim:getAllUnits()) do
			sim:refreshUnitLOS( unit )
		end
	end,

	getSmokeCells = function( self )
		return self._cells
	end,

	getSmokeEdge = function( self )
		return self._interestUnits
	end,
}

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.FOG_CLOUD,

	profile_icon = "gui/icons/spells/fog_cloud.png",		
	spellLevel = 1,
	getEffectRadius = function( self, sim )
		return self:getLevel()
	end,
	
	iconColor = util.color( 255/255, 244/255, 140/255 ),

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetCell )
		return self:getLevel()
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
		
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			self.smoke = simfactory.createUnit( smokeTemplate, sim, smokeClass )
			self.smoke:getTraits().radius = self:getEffectRadius()
			sim:spawnUnit( self.smoke )
			sim:warpUnit( self.smoke, sim:getCell( x1, y1 ) )
		end

		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	endConcentration = function( self, sim )
		if self.smoke and self.smoke:isValid() then
			sim:warpUnit( self.smoke, nil )
			sim:despawnUnit( self.smoke )
		end
	end,
}

return spellDef