local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local modifiers = include( "sim/modifiers" )

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.HASTE,

	profile_icon = "gui/icons/spells/haste.png",
	spellLevel = 2,
	
	iconColor = util.color( 208/255, 134/255, 64/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if targetUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() then
			return false
		end
		if not targetUnit:getTraits().isAgent then
			return false
		end
		if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		
		return true
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 5
	end,
	
	getDescParams = function( self, sim, unit )
		return 2 * self:getLevel() - 1, self:getLevel()
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		return self:canCastSpell( sim, abilityOwner, unit )
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			local mpBonus = self:getLevel() * 2 - 1
			local sprintBonus = self:getLevel()
			local mpPenalty = 5
			self.targetUnit = targetUnit
			targetUnit:getTraits().mpMax = targetUnit:getTraits().mpMax + mpBonus
			targetUnit:getTraits().sprintBonus = (targetUnit:getTraits().sprintBonus or 0) + sprintBonus
			targetUnit:getTraits().apMax = targetUnit:getTraits().apMax + 1
			targetUnit:getTraits().ap = targetUnit:getTraits().ap + 1
			targetUnit:getTraits().hasteAbility = { mpBonus = mpBonus, sprintBonus = sprintBonus, mpPenalty = mpPenalty }
			if targetUnit:getTraits().sneaking then
				targetUnit:getTraits().mp = targetUnit:getTraits().mp + mpBonus
			else
				targetUnit:getTraits().mp = targetUnit:getTraits().mp + mpBonus + sprintBonus
			end
			
			for i, augment in ipairs(targetUnit:getChildren()) do
				if augment:getTraits().beardTime then
					augment:getTraits().beardTime = augment:getTraits().beardTime + 24
					local shaveAbility = targetUnit:ownsAbility("wilson_shave")
					if shaveAbility then
						local upvName, updateBeardStage
						local upvIdx = 1
						repeat
							upvName, updateBeardStage = debug.getupvalue(shaveAbility.executeAbility, upvIdx)
							upvIdx = upvIdx + 1
						until ( upvName == nil or upvName == "updateBeardStage" )
						
						if updateBeardStage then
							updateBeardStage( sim, targetUnit, augment )
						end
					end
				end
			end
			
            sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_wings_activate")
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = targetUnit } )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
		
		self:endThrow( sim, abilityOwner, unit )
	end, 
	
	endConcentration = function( self, sim )
		if self.targetUnit then
			local mpBonus = self:getLevel() * 2 - 1
			local sprintBonus = self:getLevel()
			local mpPenalty = 5
			local targetUnit = self.targetUnit
			targetUnit:getTraits().mpMax = targetUnit:getTraits().mpMax - mpBonus
			targetUnit:getTraits().sprintBonus = targetUnit:getTraits().sprintBonus - sprintBonus
			targetUnit:getTraits().apMax = targetUnit:getTraits().apMax - 1
			targetUnit:getTraits().hasteAbility = nil
			targetUnit:getTraits().hastePenalty = self
			
			if targetUnit:getTraits().ap < 2 then
				self.penalizeApNextTurn = true
			end
			targetUnit:getTraits().ap = 0
			
			mpPenalty = mpPenalty + mpBonus 
			if not targetUnit:getTraits().sneaking then
				mpPenalty = mpPenalty + sprintBonus
			end
			
			self.penalizeMPNextTurn = mpPenalty - targetUnit:getTraits().mp
			targetUnit:getTraits().mp = targetUnit:getTraits().mp - mpPenalty
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			
			sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = targetUnit } )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
	end,
	
	onTrigger = function( self, sim, evType, evData )
		if evData:isPC() then
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			self.targetUnit:getTraits().hastePenalty = nil
			
			if self.penalizeApNextTurn then
				self.targetUnit:getTraits().ap = 0
			end
			if self.penalizeMPNextTurn > 0 then
				self.targetUnit:getTraits().mp = self.targetUnit:getTraits().mp - self.penalizeMPNextTurn
			end
		end
	end,
}

return spellDef