local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local simfactory = include( "sim/simfactory" )
local modifiers = include( "sim/modifiers" )

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.HYPNOTIC_GAZE,

	profile_icon = "gui/icons/spells/hypnotic_gaze.png",
	spellLevel = 1,
	
	iconColor = util.color( 251/255, 184/255, 232/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not targetUnit:getTraits().isGuard then
			return false
		end
		if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		
		return true
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 5 + 5 * self:getLevel()
	end,
	
	getRange = function( self, sim )
		return 2 + (self:getLevel() - 1) * 4
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
		
			if targetUnit:isKO() then
				return false
			end
			if targetUnit:getTraits().isDrone then
				return false, STRINGS.UI.TOOLTIPS.KO_IMMUNE
			end
			
			local x0, y0 = unit:getLocation()
			local x1, y1 = targetUnit:getLocation()
			local distance = mathutil.dist2d( x0, y0, x1, y1 )
			
			if distance > self:getRange( sim ) then
				return false, util.sformat( STRINGS.UI.REASON.CANT_REACH_BY, math.ceil(distance - self:getRange( sim )) )
			end
		end
		return true
	end,
	
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return abilityutil.SPELL_UNITTARGET.onTooltip( self, hud, sim, abilityOwner, abilityUser, targetUnitID, {
			util.sformat( self.strings.range, self:getRange( sim ) )
		} )
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			self.targetUnit = targetUnit
			if targetUnit:getBrain() then
				targetUnit:getBrain():onDespawned()
				targetUnit._brain = nil
			end
			--targetUnit:getTraits().hasSight = false
			targetUnit:getTraits().patrolObserved = true
			targetUnit:getModifiers():add( "LOSrange", "hypnotic_gaze", modifiers.SET, 0.1 )
			targetUnit:getModifiers():add( "LOSperipheralRange", "hypnotic_gaze", modifiers.SET, 0.1 )
			targetUnit:setAiming( false )
			targetUnit:turnToFace( x0, y0 )
			sim:refreshUnitLOS( targetUnit )
			
			local params = { color = { util.extend( self.iconColor ) { symbol="wall" } } }
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Psionics/attack_one" )
    		--sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, { txt=util.sformat(STRINGS.WIZARD.UI.TRUE_STRIKE), x = x1, y = y1, color = self.iconColor } )
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = targetUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params=params} )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
			
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( simdefs.TRG_UNIT_DISAPPEARED, self, unit )
		end

		self:endThrow( sim, abilityOwner, unit )
	end, 
	
	endConcentration = function( self, sim )
		self.concentrationEnded = true
		
		if self.targetUnit:isValid() and not self.targetUnit:getBrain() then
			self.targetUnit._brain = simfactory.createBrain( self.targetUnit:getUnitData().brain, sim, self.targetUnit )
			self.targetUnit:getBrain():onSpawned( sim, self.targetUnit )
			if self.targetUnit:isAlerted() then
				local x0, y0 = self.targetUnit:getLocation()
				local interest = self.targetUnit:getBrain():getSenses():addInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_HUNTING, self.targetUnit)
				sim:getNPC():createOrJoinHuntSituation(self.targetUnit, interest)
			else
				self.targetUnit:getBrain():setSituation(sim:getNPC():getIdleSituation() )
			end
			
			sim:processReactions( self.targetUnit )
		end
		
		local params = { color = { util.extend( self.iconColor ) { symbol="wall" } } }
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Psionics/linkPerson" )
		--sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, { txt=util.sformat(STRINGS.WIZARD.UI.TRUE_STRIKE), x = x1, y = y1, color = self.iconColor } )
		sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = self.targetUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params=params} )	
	end,
	
	onTrigger = function( self, sim, evType, evData, abilityUser )
		if evType == simdefs.TRG_END_TURN then
			if self.targetUnit then
				self.targetUnit:getTraits().patrolObserved = true
			end
			
			if self.concentrationEnded and self.targetUnit then
				if self.targetUnit:isValid() then
					self.targetUnit:getModifiers():remove( "hypnotic_gaze" )
					sim:refreshUnitLOS( self.targetUnit )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.targetUnit } )
				end
				
				self.targetUnit = nil
				sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
				sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
				sim:removeTrigger( simdefs.TRG_END_TURN, self )
				sim:removeTrigger( simdefs.TRG_UNIT_DISAPPEARED, self )
			end
		elseif evType == simdefs.TRG_UNIT_DISAPPEARED then
			if not self.concentrationEnded and abilityUser:isValid() and evData.seerID == abilityUser:getID() and evData.unit == self.targetUnit then
				self.arcaneFocus:executeAbility( sim )
			end
		elseif evType == simdefs.TRG_UNIT_WARP then
			if not self.concentrationEnded then
				local x0, y0 = abilityUser:getLocation()
				local x1, y1 = self.targetUnit:getLocation()
				if x0 == nil or x1 == nil or mathutil.dist2d(x0,y0,x1,y1) > (self:getRange( sim ) + 0.5) then
					self.arcaneFocus:executeAbility( sim )
				else
					self.targetUnit:turnToFace( x0, y0 )
				end
			end
		elseif evType == simdefs.TRG_UNIT_KO then
			if evData.unit == self.targetUnit and not self.concentrationEnded then
				self.arcaneFocus:executeAbility( sim )
			end
		end
	end,
}

return spellDef