local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local simfactory = include( "sim/simfactory" )
local modifiers = include( "sim/modifiers" )

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.HYPNOTIC_PATTERN,

	profile_icon = "gui/icons/spells/hypnotic_pattern.png",
	spellLevel = 3,
	effectRadius = 4,
	
	iconColor = util.color( 251/255, 184/255, 232/255 ),
	
	isAreaTargetUnit = function( self, sim, userUnit, cellUnit )
		return cellUnit:getBrain()
	end,
	
	getDuration = function( self, sim )
		return self:getLevel() - 1
	end,
	
	getDescParams = function( self, sim )
		return self:getDuration()
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
		
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			
			-- Do damage
			local fillCells = simquery.fillCircle( sim, x1, y1, self.effectRadius, 0)
			local effectCells = {}
			self.units = {}
			
			for i, cell in ipairs(fillCells) do
				local raycastX, raycastY = sim:getLOS():raycast( x1, y1, cell.x, cell.y )
				if raycastX == cell.x and raycastY == cell.y then
					table.insert(effectCells, cell)
					for i, targetUnit in ipairs( util.tdupe( cell.units ) ) do
						self:addTarget( sim, targetUnit )
					end
				end
			end
			
			local params = { color = { util.extend( self.iconColor ) { symbol="wall" } } }
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Psionics/attack_blastHit" )
			sim:dispatchEvent( "EV_WIZ_HILITE_RADIUS", { range = self.effectRadius, x = x1, y = y1, cells = effectCells, color = { self.iconColor:unpack() }, rate = {20,200,0.25} } )
		end

		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	addTarget = function( self, sim, targetUnit )
		if targetUnit:getTraits().hypnoticPattern then
			targetUnit:getTraits().hypnoticPattern = math.max(targetUnit:getTraits().hypnoticPattern, self:getDuration())
		--if targetUnit:getBrain() and not targetUnit:getTraits().isDrone then
		elseif targetUnit:getBrain() then
			table.insert(self.units, targetUnit)
			targetUnit:getTraits().hypnoticPattern = self:getDuration()
			targetUnit:getBrain():onDespawned()
			targetUnit._brain = nil
			targetUnit:getModifiers():add( "LOSrange", "hypnotic_pattern", modifiers.SET, 0.1 )
			targetUnit:getModifiers():add( "LOSperipheralRange", "hypnotic_pattern", modifiers.SET, 0.1 )
			targetUnit:setAiming( false )
			targetUnit:getTraits().patrolObserved = true
			sim:refreshUnitLOS( targetUnit )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
	end,
	
	onTrigger = function( self, sim, evType, evData )
		if evData:isPC() then
			for i = #self.units, 1, -1 do
				local targetUnit = self.units[i]
				targetUnit:getTraits().hypnoticPattern = targetUnit:getTraits().hypnoticPattern - 1
				
				if targetUnit:getTraits().hypnoticPattern <= 0 then
					if not targetUnit:getBrain() then
						targetUnit._brain = simfactory.createBrain( targetUnit:getUnitData().brain, sim, targetUnit )
						targetUnit:getBrain():onSpawned( sim, targetUnit )
						if targetUnit:isAlerted() then
							local x0, y0 = targetUnit:getLocation()
							local interest = targetUnit:getBrain():getSenses():addInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_HUNTING, targetUnit)
							sim:getNPC():createOrJoinHuntSituation(targetUnit, interest)
						else
							targetUnit:getBrain():setSituation(sim:getNPC():getIdleSituation() )
						end
					end
					
					table.remove( self.units, i )
					targetUnit:getTraits().hypnoticPattern = nil
					targetUnit:getModifiers():remove( "hypnotic_pattern" )
					sim:refreshUnitLOS( targetUnit )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
				end
			end
			
			sim:processReactions()
			
			if #self.units == 0 then
				sim:removeTrigger( simdefs.TRG_START_TURN, self )
			end
		end
	end,
}

return spellDef