local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local mission_util = include( "sim/missions/mission_util" )

local function hasSeenRelaySwitch( sim, unit )
	return sim:hasTag("transformer_start")
end

local function hasSeenDataBank( sim, unit )
	for id, unit in pairs( sim:getAllUnits() ) do
		if unit:hasTag("databank") and sim:getPC():hasSeen( unit ) then
			return true
		end
	end
end

local function shouldShowSwitches( sim, unit )
	return not unit:hasTag("switch_subgoal_seen") or sim:hasTag("transformer_start")
end

local function shouldShowStore( sim, unit )
	return not unit:getTraits().luxuryNanofab
end

local function shouldShowCompiler( sim, unit )
	return sim:hasTag("seen_compile_objective")
end

local object_categories = {
    { objects = { "security_camera_1x1", "W93_infrared_camera_1x1" }, icon = "camera", name = STRINGS.PROPS.SECURITY_CAMERA },
    { objects = { "console" }, icon = "wizard_console", name = STRINGS.PROPS.CONSOLE },
    { objects = { "security_soundBug_1x1" }, icon = "sound_bug", name = STRINGS.PROPS.SOUND_BUG },
    { objects = { "turret" }, icon = "turret", name = STRINGS.PROPS.TURRET },
    { objects = { "security_laser_emitter_1x1" }, icon = "wizard_laser", name = STRINGS.PROPS.LASER_EMITTER },
    { objects = { "security_infrared_emitter_1x1" }, icon = "wizard_beam", name = STRINGS.PROPS.BEAM_EMITTER },
    { objects = { "security_infrared_wall_emitter_1x1" }, icon = "wizard_wall", name = STRINGS.PROPS.WALL_EMITTER },
    { objects = { "NEPTUNE_security_infrared_wall_emitter_1x1" }, icon = "wizard_wall", name = STRINGS.MOD_CORPS and STRINGS.MOD_CORPS.PROPS.WALL_EMITTER },
    { objects = { "turret_generator", "laser_generator" }, icon = "powersource", name = STRINGS.PROPS.LASER_GENERATOR },
    { objects = { "item_store", "MM_luxury_store", "item_store_large" }, icon = "nanofab", name = STRINGS.PROPS.STORE, shouldShow = shouldShowStore },
    { objects = { "lab_safe", "lab_safe_tier2" }, icon = "safe", name = STRINGS.PROPS.SAFE },
    { objects = { "server_terminal" }, icon = "big_program_shop", name = STRINGS.PROPS.SERVER_TERMINAL },
    { objects = { "mini_server_terminal" }, icon = "program_shop", name = STRINGS.PROPS.MINI_SERVER_TERMINAL },
    { objects = { "extra_server_terminal" }, icon = "big_program_shop", name = STRINGS.PROGEXTEND and STRINGS.PROGEXTEND.PROPS.EXTRA_SERVER_TERMINAL },
    { objects = { "camera_core", "map_core", "console_core", "daemon_core" }, icon = "db_console", name = STRINGS.WIZARD.UI.GENERIC_CORE },
	
    { objects = { "guard_locker" }, icon = "large_safe", name = STRINGS.PROPS.GUARD_LOCKER },
    { objects = { "augment_grafter", "augment_drill" }, icon = "augment_grafter", name = STRINGS.PROPS.AUGMENT_GRAFTER },
    { objects = { "public_terminal" }, icon = "executive_terminal", name = STRINGS.PROPS.PUBLIC_TERMINAL },
    { objects = { "vault_processor" }, icon = "vault_terminal", name = STRINGS.PROPS.VAULT_PROCESSOR },
    { objects = { "detention_processor" }, icon = "detention_cell", name = STRINGS.PROPS.DETENTION_PROCESSOR },
    { objects = { "vault_safe_2", "vault_safe_3" }, icon = "artifact_case", name = STRINGS.PROPS.SECURE_CASE },
    { objects = { "MM_vault_safe_1", "MM_vault_safe_2", "MM_vault_safe_3" }, icon = "artifact_case", name = STRINGS.MOREMISSIONS and STRINGS.MOREMISSIONS.PROPS.WEAPONSLOCKER },
    { objects = { "MM_vault_safe_nonweapon_1", "MM_vault_safe_nonweapon_2" }, icon = "artifact_case", name = STRINGS.MOREMISSIONS and STRINGS.MOREMISSIONS.PROPS.ITEMLOCKER },
    { objects = { "MM_personneldb" }, icon = "vault_terminal", name = STRINGS.MOREMISSIONS and STRINGS.MOREMISSIONS.PROPS.PERSONNEL_DB },
    { objects = { "MM_W93_incogRoom_terminal" }, icon = "detention_cell", name = STRINGS.MOREMISSIONS and STRINGS.MOREMISSIONS.PROPS.INCOGROOM_TERMINAL },
    { objects = { "MM_W93_incogRoom_ai_terminal" }, icon = "vault_terminal", name = STRINGS.MOREMISSIONS and STRINGS.MOREMISSIONS.PROPS.INCOGROOM_AI_TERMINAL },
	
    { objects = { "ftm_scanner" }, icon = "scanner", name = STRINGS.PROPS.FTM_SCANNER },
    { objects = { "pressure_core" }, icon = "sound_bug", name = STRINGS.MOD_CORPS and STRINGS.MOD_CORPS.PROPS.PRESSURE_CORE },
    { objects = { "ftm_router" }, icon = "FTM_router", name = STRINGS.DLC1 and STRINGS.DLC1.PROPS.FTM_ROUTER },
    { objects = { "W93_daemon_server" }, icon = "W93_daemon_server", name = STRINGS.WORLDEXTEND and STRINGS.WORLDEXTEND.PROPS.DAEMON_SERVER },
    { objects = { "W93_drone_uplink" }, icon = "W93_Drone_Uplink_Node", name = STRINGS.WORLDEXTEND and STRINGS.WORLDEXTEND.PROPS.DRONE_UPLINK },
    { objects = { "W93_frost_spire" }, icon = "W93_Drone_Uplink_Node", name = STRINGS.WORLDEXTEND and STRINGS.WORLDEXTEND.PROPS.FROST_SPIRE },
    { objects = { "W93_shockmine" }, icon = "sound_bug", name = STRINGS.WORLDEXTEND and STRINGS.WORLDEXTEND.PROPS.SHOCK_MINE },
    { objects = { "W93_AI_Core" }, icon = "powersource", name = STRINGS.WORLDEXTEND and STRINGS.WORLDEXTEND.PROPS.AI_CORE },
	
    { objects = { "power_core" }, icon = "power_core", name = STRINGS.DLC1 and STRINGS.DLC1.PROPS.POWER_CORE },
	
    { objects = { "research_processor" }, icon = "vault_terminal", name = STRINGS.PROPS.RESEARCH_PROCESSOR, shouldShow = shouldShowCompiler },
    { objects = { "transformer_terminal" }, icon = "vault_terminal", name = STRINGS.DLC1 and STRINGS.DLC1.PROPS.TRANSFORMER_TERMINAL, shouldShow = hasSeenRelaySwitch },
    { objects = { "data_node" }, icon = "data_bank", name = STRINGS.DLC1 and STRINGS.DLC1.PROPS.DATA_NODE, shouldShow = hasSeenDataBank },
    { objects = { "lock_switch", "MM_lock_switch" }, icon = "access_switch", name = STRINGS.DLC1 and STRINGS.DLC1.PROPS.LOCK_SWITCH, shouldShow = shouldShowSwitches },
}

local spellDef = util.extend( abilityutil.SPELL_TEMPLATE ) {
	strings = STRINGS.WIZARD.SPELLS.LOCATE_OBJECT,

	profile_icon = "gui/icons/spells/locate_object.png",		
	spellLevel = 2,
	
	iconColor = util.color( 183/255, 215/255, 233/255 ),
	
	getDescParams = function( self, sim, unit )
		return self:getRevealCount(), self:getRevealCount()
	end,
	
	getRevealCount = function( self, sim )
		return 3 * self:getLevel() - 5
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit )
		local x0,y0 = userUnit:getLocation()
		
		local candidates = {}
		local map = {}
		
		for i, category in ipairs(object_categories) do
			category.idx = i
			for j, deviceType in ipairs(category.objects) do
				map[deviceType] = {i,true}
			end
		end
		
		for unitID, unit in pairs(sim:getAllUnits()) do
			local unitType = unit:getUnitData().id
			if unitType and map[unitType] then
				if map[unitType][2] then
					local i = map[unitType][1]
					local category = object_categories[i]
					
					local ok = true
					
					if not category.hiddenNumber then
						local x1, y1 = unit:getLocation()
						if sim:getPC():getLastKnownCell( sim, x1, y1 ) then
							ok = false
						end
					end
					if category.shouldShow and not category.shouldShow( sim, unit ) then
						ok = false
					end
					
					if ok then
						table.insert( candidates, category )
						for j, deviceType in ipairs( category.objects ) do
							map[deviceType][2] = nil -- Prevent duplicate candidates
							map[deviceType][3] = #candidates
						end
					end
				end
			end
		end
		
		table.sort( candidates, function( c1, c2 ) return c1.idx < c2.idx end )
		
		local choice = sim:dispatchChoiceEvent( "EV_WIZ_LOCATE_OBJECT", { unit = userUnit, level = self:getLevel(), categories = candidates } )
		
		if choice and choice ~= 0 then
			if self:castSpell( sim, abilityOwner, userUnit ) then
				local category = candidates[choice]
				local foundUnits = {}
				local pc = sim:getPC()
			
				for unitID, unit in pairs(sim:getAllUnits()) do
					local unitType = unit:getUnitData().id
					for i, matchObject in ipairs( category.objects ) do
						if unitType == matchObject then
							if not pc:hasSeen( unit ) then
								table.insert(foundUnits, unitID)
							end
						end
					end
				end
				
				if #foundUnits > 0 then
					table.sort( foundUnits, function(id0, id1)
						local x1,y1 = sim:getUnit(id0):getLocation()
						local x2,y2 = sim:getUnit(id1):getLocation() 
						
						return mathutil.distSqr2d( x0, y0, x1, y1 ) < mathutil.distSqr2d( x0, y0, x2, y2 )
					end )
					
					if #foundUnits > self:getRevealCount() then
						for i = #foundUnits, self:getRevealCount() + 1, -1 do
							table.remove(foundUnits, i)
						end
					end
					
					for i, id in ipairs( foundUnits ) do
						pc:glimpseUnit( sim, id )
						
						--sim:dispatchEvent(simdefs.EV_PLAY_SOUND,"SpySociety/Actions/mainframe_objectsreveled")
						
						local str = STRINGS.ABILITIES.WIRELESS_SCAN
						STRINGS.ABILITIES.WIRELESS_SCAN = STRINGS.UI.FLY_TXT.REVEALED -- Ew
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, {unit = sim:getUnit( id ), reveal = true} )
						STRINGS.ABILITIES.WIRELESS_SCAN = str
						
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30 )
					end
				else
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, simdefs.SOUND_HUD_INCIDENT_NEGATIVE.path )
					--mission_util.showDialog( sim, STRINGS.WIZARD.UI.NO_OBJECT, STRINGS.WIZARD.UI.NO_OBJECT_BODY )
					sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "messageDialog", dialogParams = { STRINGS.WIZARD.UI.NO_OBJECT, STRINGS.WIZARD.UI.NO_OBJECT_BODY }})
				end
			end

			self:endThrow( sim, abilityOwner, userUnit )
		end
	end,
}

return spellDef