local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.MAGE_ARMOR,

	profile_icon = "gui/icons/spells/mage_armor.png",
	spellLevel = 1,
	
	iconColor = util.color( 120/255, 169/255, 231/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if targetUnit:getPlayerOwner() ~= userUnit:getPlayerOwner() then
			return false
		end
		if not targetUnit:getTraits().isAgent then
			return false
		end
		if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		
		return true
	end,
	
	getDescParams = function( self, sim, unit )
		return 2 * self:getLevel()
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
			local x0, y0 = unit:getLocation()
			local x1, y1 = targetUnit:getLocation()
			
			return simquery.canReach( sim, x0, y0, x1, y1 )
		end
		
		return true
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			if not targetUnit:getTraits().mageArmor then
				targetUnit:getTraits().mageArmor = true
				targetUnit:getTraits().resistKO = (targetUnit:getTraits().resistKO or 0) + 1
			end
			
			targetUnit:getTraits().neutralize_shield = self:getLevel() * 2
			targetUnit:getTraits().shields = 1
			self.targetUnit = targetUnit
			
			local params = { color = { util.extend( self.iconColor ) { symbol="wall" } } }
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, { sound= "SpySociety_DLC001/Actions/flackguard_armourincrease", x=x1, y=y1 } )
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = targetUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params=params} )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
			sim:addTrigger( simdefs.TRG_UNIT_HIT, self )
		end
		
		self:endThrow( sim, abilityOwner, unit )
	end,
	
	onTrigger = function( self, sim, evType, evData )
		self.targetUnit:getTraits().shields = self.targetUnit:getTraits().neutralize_shield
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.targetUnit } )
	end,
}

return spellDef