local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local commondefs = include( "sim/unitdefs/commondefs" )

local holoItem = {
	type = "simunit",
	rig = "agentrig",
	name = STRINGS.WIZARD.UNITS.MAGE_HAND,
	
	kanim = "kanim_drone_camera_SA",
	--profile_anim = userUnit:getUnitData().profile_anim,
	--profile_build = userUnit:getUnitData().profile_build,
	profile_icon = "gui/icons/spells/mage_hand.png",
	sounds = commondefs.SOUNDS.DRONE_HOVER,
	
	traits = {
		isAgent = true,
		isDrone = true,
		sightable = true,
		apMax = 0, 
		mpMax = 5,
		mp = 5,	
		ap = 0,
		silencer = true,
		meleeDamage = 0,
		maxThrow = 4,
		
		noghost = true,

		inventoryMaxSize = 1,
		--augmentMaxSize = 0,
		LOSarc = 0,
		LOSrange = 0.5,
		hasSight = true,
		isMageHand = true,
		
		canBeShot = true,
		canKO = true,
		doesNotProvideCover = true,

		wounds = 0,	
		woundsMax = 1,
		
		dashSoundRange = 5,
	 
		isAiming=false, 
		selectpriority = 8,	
		
		sneaking = true,
		hidesInCover = true,
		calcAutoFacing = true,
	},
	abilities = {"escape","disarmtrap"},
	skills = {"anarchy"},
	onWorldTooltip = commondefs.onAgentTooltip,
}

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.MAGE_HAND,

	profile_icon = "gui/icons/spells/mage_hand.png",		
	spellLevel = 1,
	staticImpass = true,
	
	iconColor = util.color( 255/255, 244/255, 140/255 ),
	
	getDescParams = function( self, sim, unit )
		return self:getHandMP( sim ), self:getHandInventory( sim )
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetCell )
		return 10
	end,
	
	getHandMP = function( self, sim )
		return 4 + self:getLevel()
	end,
	
	getHandInventory = function( self, sim )
		return self:getLevel()
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
	
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			local holoTemplate = util.extend( holoItem )
			{
				traits = {
					mp = self:getHandMP( sim ),
					mpMax = self:getHandMP( sim ),
					inventoryMaxSize = self:getHandInventory(),
				},
				
				startingSkills = { anarchy = userUnit:getSkillLevel( "anarchy" ) }
			}
			
			local facing = userUnit:getFacing()
			if x0 ~= x1 or y0 ~= y1 then
				facing = simquery.getDirectionFromDelta(x1 - x0, y1 - y0)
			end
			
			self.wizard = userUnit
			self.simulacrum = simfactory.createUnit( holoTemplate, sim )
			self.simulacrum:setPlayerOwner( sim:getPC() )
			self.unitID = self.simulacrum:getID()
			sim:spawnUnit( self.simulacrum )
			sim:warpUnit( self.simulacrum, sim:getCell( x1, y1 ) )
			simquery.suggestAgentFacing(self.simulacrum, facing)
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.simulacrum } )
			
			sim:dispatchEvent( simdefs.EV_UNIT_ACTIVATE, { unit = self.simulacrum } )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			simquery.addSimulacrumMovement()
			
			if not sim._canGhostDoors then
				sim._canGhostDoors = true
				local modifyExit = sim.modifyExit
				local moveUnit = sim.moveUnit
				sim.modifyExit = function( self, cell, dir, exitOp, unit, ... ) 
					if unit and unit:getTraits().ghostsDoors and unit:getTraits().movePath then
						return
					end
				
					return modifyExit( self, cell, dir, exitOp, unit, ... )
				end
			end
		end
		
		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	endConcentration = function( self, sim )
		if self.simulacrum and self.simulacrum:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_DEACTIVATE, { unit = self.simulacrum } )
			sim:warpUnit( self.simulacrum, nil )
			sim:despawnUnit( self.simulacrum )
		end
		
		self.simulacrum = nil
		sim:getPC():glimpseUnit( sim, self.unitID )
		sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		simquery.removeSimulacrumMovement()
	end,
	
	onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_START_TURN then
			self.simulacrum:getTraits().sneaking = true
			if not sim:canUnitSeeUnit( self.wizard, self.simulacrum ) then
				self.arcaneFocus:executeAbility( sim )
			end
		elseif evData.unit == self.simulacrum then
			self.arcaneFocus:executeAbility( sim )
		end
	end,
}

return spellDef