local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )

local COLOR_BLUE_DARK = util.color(50/255,87/255,87/255,150/255)
local COLOR_BLUE_LIGHT = util.color(150/255,255/255,255/255,1)

local COLOR_RED_DARK = util.color(80/255,0/255,0/255,150/255)
local COLOR_RED_LIGHT = util.color(163/255,0/255,0/255,1)

local shoot_tooltip = class()

function shoot_tooltip:init( spell, hud, abilityUser, targetUnitID )
	self._hud = hud
	self._spell = spell
	self._abilityUser = abilityUser
	self._targetUnitID = targetUnitID
end

function shoot_tooltip:setPosition( wx, wy )
	self._panel:setPosition( self._hud._screen:wndToUI( wx, wy ))
end

function shoot_tooltip:getScreen()
	return self._hud._screen
end

function shoot_tooltip:activate( screen )
	local combat_panel = include( "hud/combat_panel" )
	local sim = self._hud._game.simCore

	self._panel = combat_panel( self._hud, self._hud._screen )
	local binder = self._panel._panel.binder
	
	local unit = self._abilityUser
	local targetUnit = sim:getUnit( self._targetUnitID )
	
	local color, textColor = COLOR_BLUE_DARK, COLOR_BLUE_LIGHT

	if not sim:canUnitSeeUnit( unit, targetUnit ) then 
		binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_NO_LOS )		
		color = COLOR_RED_DARK
		textColor = COLOR_RED_LIGHT
	elseif targetUnit:getArmor() > self._spell:getArmorPiercing() then 
		binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_ARMORED )
		color = COLOR_RED_DARK
		textColor = COLOR_RED_LIGHT
	else
		binder.damageTxt:setText( STRINGS.UI.COMBAT_PANEL_KILL )
	end

	local targetTxt = targetUnit:getName()
	if targetUnit:isKO() then
		targetTxt = STRINGS.UI.COMBAT_PANEL_KILL_KO_TARGET .." ".. targetTxt
	end
	binder.targetTxt:setText( targetTxt )

	self._panel:setVisible( true )
	binder.hitBackground:setColor(color:unpack())  
	binder.targetTxt:setColor(textColor:unpack())
	binder.damageTxt:setColor(textColor:unpack())
	
	self._hud._game.boardRig:getUnitRig( self._targetUnitID )._prop:setRenderFilter( cdefs.RENDER_FILTERS["focus_highlite"] )
end

function shoot_tooltip:deactivate()
	self._hud._game.boardRig:getUnitRig( self._targetUnitID )._prop:setRenderFilter( nil )
	self._panel:setVisible( false )
end

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.MAGIC_MISSILE,

	profile_icon = "gui/icons/spells/magic_missile.png",
	spellLevel = 1,
	
	iconColor = util.color( 232/255, 96/255, 69/255 ),

	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		if targetUnitID then
			return shoot_tooltip( self, hud, abilityUser, targetUnitID )
		else
			local ap
			if self:getArmorPiercing() > 0 then
				ap = util.sformat( self.strings.ap, self:getArmorPiercing() )
			else
				ap = self.strings.noap
			end
		
			return abilityutil.SPELL_UNITTARGET.onTooltip( self, hud, sim, abilityOwner, abilityUser, targetUnitID, { ap } )
		end
	end,
	
	getArmorPiercing = function( self )
		return 3 * self:getLevel() - 3
	end,
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not simquery.isShootable( userUnit, targetUnit ) then
			return false
		end
		if not sim:canPlayerSeeUnit( userUnit:getPlayerOwner(), targetUnit ) then
			return false
		end
		
		local x0, y0 = userUnit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		return sim:canUnitSeeUnit( userUnit, targetUnit ) or mathutil.dist2d( x0, y0, x1, y1 ) <= 12
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
		
			if not sim:canUnitSeeUnit( unit, targetUnit ) then
				return false, STRINGS.UI.COMBAT_PANEL_NO_LOS
			end
		
			if targetUnit:getArmor() > self:getArmorPiercing() then
				return false, STRINGS.UI.COMBAT_PANEL_ARMORED
			end
		end
		return true
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			sim:hitUnit( unit, targetUnit, { damage = self:getLevel() } )

			if targetUnit:isValid() and targetUnit:getPlayerOwner() then
				targetUnit:interruptMove( self )
			end				
		end

		self:endThrow( sim, abilityOwner, unit )
	end
}

return spellDef