local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local simfactory = include( "sim/simfactory" )
local modifiers = include( "sim/modifiers" )
local mui_tooltip = include( "mui/mui_tooltip" )

local eyeballrig = class()

function eyeballrig:init( boardRig, unit )
	self._boardRig = boardRig
	self._unitID = unit:getID()
	self._targetID = unit:getTraits().mindSpikeID
end

function eyeballrig:getUnit()
	return self._boardRig:getLastKnownUnit( self._unitID )
end

function eyeballrig:onSimEvent( ev, eventType, eventData )
end

function eyeballrig:destroy()
	self._boardRig:refreshLOSCaster( self._unitID )
	self.destroyed = true
	local followRig = self._boardRig:getUnitRig( self._targetID )
	if followRig and self.animateLOS and followRig.animateLOS == self.newAnimateLOS then
		followRig.animateLOS = self.animateLOS
		self.animateLOS = nil
		self.newAnimateLOS = nil
	end
end

function eyeballrig:refresh()
	self._boardRig:refreshLOSCaster( self._unitID )
end

function eyeballrig:refreshLOSCaster( seerID )
	local followRig = self._boardRig:getUnitRig( self._targetID )
	local seer = self:getUnit()
	
	if not self.animateLOS and followRig then
		self.animateLOS = followRig.animateLOS
		self.newAnimateLOS = function(agRig, params)
			self.animateLOS(agRig, params)
			
			if not self.destroyed then
				self._boardRig._game.shadow_map:insertLOS( KLEIShadowMap.ALOS_DIRECT, seerID, params.right, params.left, params.range, self._boardRig:cellToWorld( seer:getLocation() ) )
			end
		end	
		
		followRig.animateLOS = self.newAnimateLOS
	end
	
	if followRig and followRig._state and followRig._state._losProp then
		local arcStart = seer:getFacingRad() - simquery.getLOSArc( seer )/2
		local arcEnd = seer:getFacingRad() + simquery.getLOSArc( seer )/2
		local range = seer:getTraits().LOSrange and self._boardRig:cellToWorldDistance( seer:getTraits().LOSrange )
	
		self._boardRig._game.shadow_map:insertLOS( KLEIShadowMap.ALOS_DIRECT, seerID, arcStart, arcEnd, range, followRig._state._losProp )
	
		return true
	elseif followRig and followRig._state and followRig._state.status then
		return true
	end
	return false
end

local rigrig = { rig = eyeballrig }

local eyeball_seeking = {
	type = "simunit",
	name = "Seeking Eyeball",
	facing = simdefs.DIR_N,	
	rig = "seeking_eyeballrig",
	traits = {hasSight = true, detect_cloak = true},
}

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.MIND_SPIKE,

	profile_icon = "gui/icons/spells/mind_spike.png",
	spellLevel = 2,
	
	iconColor = util.color( 251/255, 184/255, 232/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not targetUnit:getTraits().isGuard then
			return false
		end
		if targetUnit:isKO() then
			return false
		end
		if not sim:canUnitSeeUnit( userUnit, targetUnit ) then
			return false
		end
		
		return true
	end,
	
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, target )
		if self.targetUnit then
			local enabled, reason = self:canUseAbility( sim, abilityOwner, abilityUser )
			local tooltip = util.sformat(self.strings.endEarly, self.targetUnit:getName(), self:getKOTime( sim ) )
			if reason then
				return mui_tooltip(self.strings.name, string.format( "%s\n<tthotkey><c:FF0000>%s</>", tooltip, reason ) )
			else
				return mui_tooltip(self.strings.name, tooltip )
			end
		else
			return abilityutil.SPELL_UNITTARGET.onTooltip( self, hud, sim, abilityOwner, abilityUser, target )
		end
	end,
	
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		if self.targetUnit then
			return targets.unitTarget( game, { self.targetUnit }, self, unit, userUnit )
		else
			return abilityutil.SPELL_UNITTARGET.acquireTargets( self, targets, game, sim, unit, userUnit )
		end
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 10 * self:getLevel() - 10
	end,
	
	getKOTime = function( self, sim )
		return self:getLevel()
	end,
	
	getDescParams = function( self, sim, unit )
		return self:getKOTime( sim )
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		if self.targetUnit then
			if unit:getAP() < 1 then 
				return false, STRINGS.UI.REASON.ATTACK_USED
			end
		else
			local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
			if not canCast then
				return false, canCastReason
			end

			if targetUnitID then
				local targetUnit = sim:getUnit( targetUnitID )
			
				if targetUnit:getTraits().isDrone then
					return false, STRINGS.UI.TOOLTIPS.KO_IMMUNE
				end
			end
		end
		
		return true
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self.targetUnit then
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/HUD/gameplay/levelup" )
			self.targetUnit:setKO( sim, self:getKOTime( sim ) )
			unit:useAP( sim )
			self.arcaneFocus:executeAbility( sim )
		else
			if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
				self.sim = sim
				self.targetUnit = targetUnit
				self.showTargets = nil
			
				if not targetUnit:getTraits().tagged then
					self.addedTag = true
				end
				
				--sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/look" )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Psionics/attack_one" )
				targetUnit:setTagged()
				sim:addTrigger( simdefs.TRG_UNIT_KO, self )
				sim:addTrigger( simdefs.TRG_START_TURN, self )
				
				-- Spawn eyeball
				package.loaded["gameplay/seeking_eyeballrig"] = rigrig
				local eyeballUnit = simfactory.createUnit( eyeball_seeking, sim )
				eyeballUnit:setPlayerOwner( unit:getPlayerOwner() )
				eyeballUnit:getTraits().peekID = unit:getID()
				eyeballUnit:getTraits().mindSpikeID = targetUnit:getID()
				sim:spawnUnit( eyeballUnit )
				self.eyeballUnit = eyeballUnit
				self:refreshLOS()
				
				-- Function appends
				self.oldUpdateSeenUnits = targetUnit.updateSeenUnits
				self.newUpdateSeenUnits = function( ... )
					self:refreshLOS()
					self.oldUpdateSeenUnits( ... )
				end
				
				targetUnit.updateSeenUnits = self.newUpdateSeenUnits
			end

			self:endThrow( sim, abilityOwner, unit )
		end
	end,
	
	refreshLOS = function( self )
		local eyeballUnit = self.eyeballUnit
		local targetUnit = self.targetUnit
		local sim = self.sim
		
		if eyeballUnit then
			eyeballUnit:getTraits().LOSarc = math.max(targetUnit:getTraits().LOSarc or 0, targetUnit:getTraits().LOSperipheralArc or 0)
			eyeballUnit:getTraits().LOSrange = math.max(targetUnit:getTraits().LOSrange or 0, targetUnit:getTraits().LOSperipheralRange or 0)
			eyeballUnit:getTraits().LOSrads = targetUnit:getTraits().LOSrads
			eyeballUnit:getTraits().grappler = not simquery.couldUnitSee( sim, targetUnit )
			eyeballUnit:getTraits().hasHearing = simquery.canHear( targetUnit )
			eyeballUnit:setFacing( targetUnit:getFacing() )
			local fromCell = sim:getCell( eyeballUnit:getLocation() )
			local toCell = sim:getCell( targetUnit:getLocation() )
			if fromCell ~= toCell then
				sim:warpUnit( eyeballUnit, toCell )
			else
				sim:refreshUnitLOS( eyeballUnit )
			end
		end
	end,
	
	endConcentration = function( self, sim )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		
		if self.targetUnit and self.targetUnit:isValid() then
			if self.addedTag then
				self.targetUnit:setTagged( true )
			end
			
			if self.newUpdateSeenUnits == self.targetUnit.updateSeenUnits then
				self.targetUnit.updateSeenUnits = self.oldUpdateSeenUnits
				self.oldUpdateSeenUnits = nil
				self.newUpdateSeenUnits = nil
			end
		end
		
		if self.eyeballUnit and self.eyeballUnit:isValid() then
			sim:warpUnit( self.eyeballUnit, nil )
			sim:despawnUnit( self.eyeballUnit )
		end
		
		self.targetUnit = nil
		self.eyeballUnit = nil
	end,
	
	onTrigger = function( self, sim, evType, evData, abilityUser )
		if self.targetUnit then
			self.targetUnit:getTraits().patrolObserved = true
			if not self.targetUnit:isValid() then
				self.arcaneFocus:executeAbility( sim )
			end
		end
	end,
}

return spellDef