local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local commondefs = include( "sim/unitdefs/commondefs" )

local agentrig = include( "gameplay/agentrig" )
-- dronerig
--------------------------------------------------------------------

local doppelgangerrig = class( agentrig.rig )

local default = 		{ shader=KLEIAnim.SHADER_FOW, 			r= 235/255,	g= 210/255, b= 1, 	a=0.5, 	lum=0.5 }
local mainframe_agent = { shader=KLEIAnim.SHADER_FOW,			r= 216/255,	g= 162/255,	b= 0/255,	a=0.2,	lum=1.0-0.41 }
local cloak = 			{ shader=KLEIAnim.SHADER_FOW,			r= 0.9,  	g= 0.6,  	b= 1,  		a=0.3,  lum=1.0-0.0 }
	--local ghost = 	{ shader=KLEIAnim.SHADER_FOW, 			r= 0.8, 	g= 0.0, 	b= 0.0, 	a=0.5, 	lum=1.0-0.62 }
	--cloak = 			{ shader=KLEIAnim.SHADER_FOW,			r=	  0.9,  g=    0.8,  b=      1,  a=0.3,  lum=1.0-0.0 }
	--mainframe_agent =	{ shader=KLEIAnim.SHADER_FOW,			r=255/255,	g= 255/255,	b= 0/255,	a=0.2,	lum=1.0-0.41 }

function doppelgangerrig:refreshRenderFilter()
	local unit = self._boardRig:getLastKnownUnit( self._unitID )
	if unit then
		local gfxOptions = self._boardRig._game:getGfxOptions()
		if gfxOptions.bMainframeMode or gfxOptions.bTacticalView then
			self._prop:setRenderFilter( mainframe_agent )
		else
			self._prop:setPlayMode( self._playMode )

			if unit:getTraits().invisible then
				self._prop:setRenderFilter( cloak )
			else
				self._prop:setRenderFilter( default )
			end
		end
	end
end

function doppelgangerrig:init( boardRig, unit )
	agentrig.rig.init( self, boardRig, unit )
	
	local onEnter = self._walkState.onEnter
	local onSimEvent = self._walkState.onSimEvent
	
	function self._walkState:onEnter( ev )
		unit:getTraits().isDrone = false
		onEnter( self, ev )
		unit:getTraits().isDrone = true
	end
	
	function self._walkState:onSimEvent( ev, eventType, eventData )
		unit:getTraits().isDrone = false
		local ret = onSimEvent( self, ev, eventType, eventData )
		unit:getTraits().isDrone = true
		return ret
	end
end

package.loaded["gameplay/doppelgangerrig"] = {
	rig = doppelgangerrig
}

local holoItem = {
	type = "simunit",
	rig = "doppelgangerrig",
	name = STRINGS.WIZARD.UNITS.MIRROR_IMAGE,
	--sounds = { activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate" },
	traits = {
		isAgent = true,
		isDrone = true,
		sightable = true,
		apMax = 1, 
		mpMax=8,
		mp=8,	
		ap = 1,
		silencer = true,
		meleeDamage = 0,
		maxThrow = 10,
		
		noghost = true,
		--canUseDoor = false,
		ghostsDoors = true,

		inventoryMaxSize = 0,
		augmentMaxSize = 0,
		LOSarc = 0,
		LOSrange = 0.5,
		hasSight = true,
		
		canBeShot = true,
		canKO = true,
		doesNotProvideCover = true,

		wounds = 0,	
		woundsMax = 1,
		
		dashSoundRange = 5,
	 
		isAiming=false, 
		selectpriority = 8,	
		
		sneaking = true,
		hidesInCover = true,
		calcAutoFacing = true,
	},
	abilities = {"sprint"},
	skills = {"stealth"},
	onWorldTooltip = commondefs.onAgentTooltip,
}

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.MIRROR_IMAGE,

	profile_icon = "gui/icons/spells/mirror_image.png",		
	spellLevel = 2,
	staticImpass = true,
	
	iconColor = util.color( 216/255, 162/255, 255/255 ),

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetCell )
		return 2 * self:getLevel() - 2
	end,
	
	getName = function( self, sim, ownerUnit, userUnit, targetUnitID )
		local name = self.strings.name
		
		if self.smallFont then
			name = self.strings.name2
		end
	
		if self.spellSlotLevel then
			return util.sformat( STRINGS.WIZARD.SPELLS.LEVEL, self.spellSlotLevel, name )
		else
			return name
		end
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
	
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			local holoTemplate = util.extend( holoItem )
			{
				kanim = userUnit:getUnitData().kanim,
				profile_anim = userUnit:getUnitData().profile_anim,
				profile_build = userUnit:getUnitData().profile_build,
				sounds = userUnit:getUnitData().sounds,
				startingSkills = { stealth = userUnit:getSkillLevel( "stealth" ) }
			}
			
			local facing = userUnit:getFacing()
			if x0 ~= x1 or y0 ~= y1 then
				facing = simquery.getDirectionFromDelta(x1 - x0, y1 - y0)
			end
			
			self.simulacrum = simfactory.createUnit( holoTemplate, sim )
			self.simulacrum:setPlayerOwner( sim:getPC() )
			self.unitID = self.simulacrum:getID()
			sim:spawnUnit( self.simulacrum )
			sim:warpUnit( self.simulacrum, sim:getCell( x1, y1 ) )
			simquery.suggestAgentFacing(self.simulacrum, facing)
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.simulacrum } )
			
			sim:dispatchEvent( simdefs.EV_UNIT_ACTIVATE, { unit = self.simulacrum } )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			simquery.addSimulacrumMovement( sim )
		end
		
		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	endConcentration = function( self, sim )
		if self.simulacrum and self.simulacrum:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_DEACTIVATE, { unit = self.simulacrum } )
			sim:warpUnit( self.simulacrum, nil )
			sim:despawnUnit( self.simulacrum )
		end
		
		self.simulacrum = nil
		sim:getPC():glimpseUnit( sim, self.unitID )
		sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		simquery.removeSimulacrumMovement( sim )
	end,
	
	onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_START_TURN then
			self.simulacrum:getTraits().sneaking = true
		elseif evData.unit == self.simulacrum then
			self.arcaneFocus:executeAbility( sim )
		end
	end,
}

return spellDef