local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )

local smokeTemplate = {
	type = "simunit",
	name = STRINGS.PROPS.SMOKE,
    rig = "smokerig",
	kanim = "kanim_smoke_plume",
	traits = { noghost = true }
}

local smokeClass = {
	getSmokeCells = function( self )
		return { self:getSim():getCell( self:getLocation() ) }
	end,

	getSmokeEdge = function( self )
		return {}
	end
}

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.MISTY_STEP,

	profile_icon = "gui/icons/spells/misty_step.png",		
	spellLevel = 2,
	staticImpass = true,
	dynamicImpass = true,
	throwRange = 5,
	
	getProfileIcon = function( self, sim, abilityOwner )
		if self.multiActivations and self.multiActivations > 0 then
			return "gui/icons/spells/misty_step_multi.png"
		else
			return self.profile_icon
		end
	end,
	
	iconColor = util.color( 255/255, 244/255, 140/255 ),
	
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		if self.multiActivations then
			return abilityutil.SPELL_TEMPLATE.onTooltip( self, hud, sim, abilityOwner, abilityUser, targetUnitID, {
				util.sformat( self.strings.multi_active, 1 + self:maxMultiActions( sim ) - self.multiActivations )
			} )
		elseif self:maxMultiActions( sim ) > 0 then
			return abilityutil.SPELL_TEMPLATE.onTooltip( self, hud, sim, abilityOwner, abilityUser, targetUnitID, {
				util.sformat( self.strings.multi, self:maxMultiActions( sim ) )
			} )
		else
			return abilityutil.SPELL_TEMPLATE.onTooltip( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		end
	end,
	
	maxMultiActions = function( self, sim )
		return self:getLevel() - 2
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetCell )
		if self.spellSlotExpended and sim:getTurnCount() == self.activationTurn and self.multiActivations <= self:maxMultiActions() then
			self.noAttackRequired = true
			self.noSpellSlotRequired = true
		end
		
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit, targetCell )
		
		self.noAttackRequired = nil
		self.noSpellSlotRequired = nil
		
		return canCast, canCastReason
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local cell = sim:getCell( targetCell[1], targetCell[2] )
		
		if self.spellSlotExpended and sim:getTurnCount() == self.activationTurn and self.multiActivations <= self:maxMultiActions() then
			self.noAttackRequired = true
			self.noSpellSlotRequired = true
		end
		
		--if self:castSpell( sim, abilityOwner, userUnit, cell ) then
		if self:castSpell( sim, abilityOwner, userUnit ) then
			self.activationTurn = sim:getTurnCount()
			self.multiActivations = (self.multiActivations or 0) + 1
			
			local smoke = simfactory.createUnit( smokeTemplate, sim, smokeClass )
			sim:spawnUnit( smoke )
			sim:warpUnit( smoke, sim:getCell(x0, y0) )
			sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = userUnit:getID(), noSightingFx=true } )
			
			local x1, y1 = cell.x, cell.y
			local newFacing = simquery.getDirectionFromDelta( x1 - x0, y1 - y0 )

			if newFacing ~= simdefs.DIR_MAX then
				simquery.suggestAgentFacing( userUnit, newFacing )
			end

			sim:dispatchEvent( "EV_WIZ_MISTY_STEP", { unitID = userUnit:getID(), x1=x1, y1=y1, facing=newFacing } )
			
			--sim:dispatchEvent( simdefs.EV_TELEPORT, { units={userUnit}, warpOut =true } )
			
			sim:warpUnit( userUnit, cell )
			sim:warpUnit( smoke, cell )
			sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = userUnit:getID(), noSightingFx=true } )
			--sim:dispatchEvent( simdefs.EV_TELEPORT, { units={userUnit}, warpOut =false } )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
			
			sim:warpUnit( smoke, nil )
			sim:despawnUnit( smoke )
			
			sim:dispatchEvent( "EV_WIZ_MISTY_STEP", { unitID = userUnit:getID(), endThrow = true } )
		end

		self:endThrow( sim, abilityOwner, userUnit )
		
		self.noAttackRequired = nil
		self.noSpellSlotRequired = nil
	end,
}

return spellDef