local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local abilitydefs = include( "sim/abilitydefs" )

local scanAbilities = {
	ce_long_range_sensors_passive = true,
	ce_droneMonitor = true,
	ce_ultraviolet_spectrometer_passive = true,
	ce_search_tools = true,
	ce_ultrasonic_echolocation_passive = true,
	ce_holographic_pinpointing = true,
	ce_piercing_scope_passive_new = true,
}

local scanRoutines = {}

if STRINGS.PROGEXTEND then
	scanRoutines = {
		[STRINGS.PROGEXTEND.AI.PINPOINT] = true,
		[STRINGS.PROGEXTEND.AI.PROXY_SCANNER] = true,
		[STRINGS.PROGEXTEND.AI.SENTINEL] = true,
		[STRINGS.PROGEXTEND.AI.SCANNER_SWEEP] = true,
		[STRINGS.PROGEXTEND.AI.VIDEO_COPROCESSORS] = true,
		[STRINGS.PROGEXTEND.AI.PROXIMAL] = true,
	}
end

local spellDef = util.extend( abilityutil.SPELL_TEMPLATE ) {
	strings = STRINGS.WIZARD.SPELLS.NONDETECTION,

	profile_icon = "gui/icons/spells/nondetection.png",		
	spellLevel = 3,
	
	iconColor = util.color( 120/255, 169/255, 231/255 ),

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser )
		return self:getLevel() - 1
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit )
		local x0,y0 = userUnit:getLocation()
	
		if self:castSpell( sim, abilityOwner, userUnit ) then
			self.oldTrackerAdvance = sim.trackerAdvance
			self.oldScanCell = sim.scanCell
			self.userUnit = userUnit
			
			sim.nonDetectionActive = true
			sim.trackerAdvance = function() end
			sim.scanCell = function() end
			
			self.scannerUnits = {}
			
			for id, unit in pairs(sim:getAllUnits()) do
				local matched = false
				for i, ability in ipairs(unit:getAbilities()) do
					local abilityID = ability:getID()
					
					if scanAbilities[abilityID] then
						ability:onDespawnAbility( sim, unit )
						matched = true
					end
				end
				
				if unit:getModifiers():has( "LOSperipheralArc", "W93_AI_SCANNER_SWEEP" ) then
					unit:getModifiers():remove( "W93_AI_SCANNER_SWEEP" )
					matched = true
				end
				
				if matched then
					unit:getTraits().hasHoloTarget = nil
					unit:getTraits().ultrasonic_echolocation = nil
					unit:getTraits().detect_cloak = nil
					unit:getTraits().seesHidden = nil
					unit:getTraits().ignoresWalls = nil
					
					table.insert( self.scannerUnits, unit )
					sim:refreshUnitLOS( unit )
					
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
				end
			end
			
			local assembly = abilitydefs.lookupAbility( "W93_AI_assembly" ) and sim:getNPC():hasMainframeAbility("W93_AI_assembly")
			if assembly then
				local slots = util.tmerge( {}, assembly.slotsPWR, assembly.slotsPro, assembly.slotsRea, assembly.slotsUti )
				for i, routine in ipairs( slots ) do
					if scanRoutines[ routine.name ] then
						if routine.executeAbility then
							routine._wizardBakExecute = routine.executeAbility
							routine.executeAbility = function() end
						end
						if routine.removeAbility then
							routine._wizardBakRemove = routine.removeAbility
							routine.removeAbility( assembly, sim, routine )
							routine.removeAbility = function() end
						end
					end
				end
			end
			
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_deterrent_stopped" )
			local r,g,b,a = self.iconColor.r, self.iconColor.g, self.iconColor.b, 0.8
			local params = {color ={{symbol="inner_line",r=r,g=g,b=b,a=a},{symbol="wall_digital",r=r,g=g,b=b,a=a},{symbol="boxy_tail",r=r,g=g,b=b,a=a},{symbol="boxy",r=r,g=g,b=b,a=a}} }
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, {unit = userUnit, kanim = "fx/deamon_ko", symbol = "effect", anim = "out", above = true, params = params} )
			sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.WIZARD.UI.NONDETECTION_WARNING, color=cdefs.COLOR_PLAYER_WARNING, icon="gui/wizard_hud/nondetection_warning.png" } )
		end
		
		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	endConcentration = function( self, sim )
		sim.nonDetectionActive = nil
		sim.trackerAdvance = self.oldTrackerAdvance
		sim.scanCell = self.oldScanCell
		
		for i, unit in pairs( self.scannerUnits ) do
			if unit:isValid() then
				for i, ability in ipairs(unit:getAbilities()) do
					local abilityID = ability:getID()
					
					if scanAbilities[abilityID] then
						ability:onSpawnAbility( sim, unit )
					end
				end
				
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
			end
		end
		
		local assembly = abilitydefs.lookupAbility( "W93_AI_assembly" ) and sim:getNPC():hasMainframeAbility("W93_AI_assembly")
		if assembly then
			local slots = util.tmerge( {}, assembly.slotsPWR, assembly.slotsPro, assembly.slotsRea, assembly.slotsUti )
			for i, routine in ipairs( slots ) do
				if scanRoutines[ routine.name ] then
					if routine._wizardBakExecute then
						routine.executeAbility = routine._wizardBakExecute
						routine._wizardBakExecute = nil
					end
					if routine._wizardBakRemove then
						routine.removeAbility = routine._wizardBakRemove
						routine._wizardBakRemove = nil
					end
					if routine.onSpawnAbility then
						routine.onSpawnAbility( assembly, sim, routine )
					end
				end
			end
		end
		
		sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_deterrent_stopped" )
		local r,g,b,a = self.iconColor.r, self.iconColor.g, self.iconColor.b, 0.8
		local params = {color ={{symbol="inner_line",r=r,g=g,b=b,a=a},{symbol="wall_digital",r=r,g=g,b=b,a=a},{symbol="boxy_tail",r=r,g=g,b=b,a=a},{symbol="boxy",r=r,g=g,b=b,a=a}} }
		sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, {unit = self.userUnit, kanim = "fx/deamon_ko", symbol = "effect", anim = "in", above = true, params = params} )
		sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.WIZARD.UI.NONDETECTION_WARNING_END, color=cdefs.COLOR_PLAYER_WARNING, icon="gui/wizard_hud/nondetection_warning.png" } )
	end,
}

return spellDef