local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )
local commondefs = include( "sim/unitdefs/commondefs" )
local btree = include("sim/btree/btree")
local actions = include("sim/btree/actions")
local abilitydefs = include( "sim/abilitydefs" )
local geo_util = include( "geo_util" )

local function tooltip_activate( self, screen )
	abilityutil.hotkey_tooltip.activate( self, screen )
	
	if not self.prop then
		self.prop = MOAIProp.new()
		
		local x0, y0 = self._game:cellToWorld( self.x0, self.y0 )
		local x1, y1 = self._game:cellToWorld( self.x1, self.y1 )

		local msh = geo_util.generateArcMesh(self._game.boardRig, x0, y0, 0, x1, y1, 0, 30, 10)
		self.prop:setDeck(msh)
		self.prop:setLoc(x0, y0)
		self._game.boardRig:getLayer("ceiling"):insertProp(self.prop)
	end
end

local function tooltip_deactivate( self )
	abilityutil.hotkey_tooltip.deactivate( self )
	
	if self.prop then
		self._game.boardRig:getLayer("ceiling"):removeProp(self.prop)
		self.prop = nil
	end
end

local phantasm_retarget = {
	profile_icon = "gui/icons/spells/phantasmal_killer.png",
	name = STRINGS.WIZARD.ABILITIES.RETARGET_PHANTASM,
	
	getName = function( self )
		return STRINGS.WIZARD.ABILITIES.RETARGET_PHANTASM
	end,
	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit( targetID )
		return util.sformat( STRINGS.WIZARD.ABILITIES.RETARGET_PHANTASM, util.toupper( targetUnit:getName() ) )
	end,
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not targetUnit:getBrain() then
			return false
		end
		if not sim:canPlayerSeeUnit( userUnit:getPlayerOwner(), targetUnit ) then
			return false
		end
		if targetUnit:isDown() then
			return false
		end
		
		return true
	end,
	
	acquireTargets = function( self, targets, game, sim, unit, userUnit )
		local units = {}
		for _, targetUnit in pairs(sim:getAllUnits()) do
			if self:isTarget( sim, unit, userUnit, targetUnit ) then
				table.insert( units, targetUnit )
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,
	
	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		if self.spell.targetUnit then
			return false
		end
		
		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
			if targetUnit:getTraits().isDrone then
				return false, STRINGS.UI.TOOLTIPS.KO_IMMUNE
			end
			if targetUnit:getBrain():getTarget() then
				return false, STRINGS.UI.COMBAT_PANEL_SEEN
			end
			
			local x0, y0 = unit:getLocation()
			local x1, y1 = targetUnit:getLocation()
			local raycastX, raycastY = self._sim:getLOS():raycast(x0, y0, x1, y1)
			if raycastX ~= x1 or raycastY ~= y1 then
				return false, STRINGS.UI.COMBAT_PANEL_NO_LOS
			end
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		targetUnit.huntedByPhantasmalKiller = unit
		self.spell:setTarget( sim, targetUnit )
		self.spell:spookGuard( sim, targetUnit )
		unit:turnToFace( targetUnit:getLocation() )
	end
}

local holoItem = {
	type = "simunit",
	rig = "doppelgangerrig",
	name = STRINGS.WIZARD.UNITS.PHANTASM,
	--sounds = { activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate" },
	traits = {
		isAgent = true,
		isDrone = true,
		sightable = true,
		apMax = 1, 
		mpMax=10,
		mp=0,	
		ap = 1,
		silencer = true,
		meleeDamage = 0,
		maxThrow = 10,
		
		noghost = true,
		canUseDoor = false,
		ghostsDoors = true,
		ghostsAllMovement = true,

		inventoryMaxSize = 0,
		augmentMaxSize = 0,
		LOSarc = 0,
		LOSrange = 0.5,
		hasSight = true,
		
		canBeShot = true,
		canKO = true,
		doesNotProvideCover = true,
		walk = true,

		--wounds = 0,	
		--woundsMax = 1,
		
		dashSoundRange = 5,
	 
		isAiming=false, 
		selectpriority = 8,	
		sneaking = true,
	},
	abilities = {"phantasm_retarget"},
	--skills = {"stealth"},
	onWorldTooltip = commondefs.onAgentTooltip,
}

local function makeBtreeNode()
	local CantFail = class(btree.Decorator, function(self, name, child)
		if type(name) == "table" and not child then
			child = name
			name = "Can't Fail"
		end
		btree.Decorator.init(self, name, child)
	end)

	function CantFail:update()
		self.status = simdefs.BSTATE_RUNNING
		self.status = self.child:tick()
		if self.status == simdefs.BSTATE_FAILED then
			self.status = simdefs.BSTATE_COMPLETE
		end
		
		return self.status
	end

	return btree.Sequence("Flee Phantasm",
		{
			btree.Condition(actions.IsPhantasmalFrightened),
			CantFail(
				btree.Sequence({
					btree.Action(actions.ReactToPhantasm),
					actions.FleePhantasm(),
					btree.Action( actions.TurnToFacePhantasm ),
					btree.Action( actions.DismissPhantasm )
				})
			),
			btree.Condition(actions.IsPhantasmalFrightened),
		})
end

local function updateBrain( targetUnit )
	local brain = targetUnit:getBrain()
	if not brain.hasFleePhantasmBehaviour then
		brain.hasFleePhantasmBehaviour = true
		local node = makeBtreeNode()
		brain:setupNode(node)
		table.insert( brain.rootNode.children, 1, node )
	end
	
	local senses = brain:getSenses()
	if senses and not senses.hasFleePhantasmBehaviour then
		senses.hasFleePhantasmBehaviour = true
		local oldAddTarget = senses.addTarget
		
		senses.addTarget = function( self, ... )
			if self.unit:getTraits().huntedByPhantasmalKiller then
				return
			end
			
			return oldAddTarget( self, ... )
		end
	end
	
	targetUnit:resetAllAiming()
	brain:reset()
end

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.PHANTASMAL_KILLER,

	profile_icon = "gui/icons/spells/phantasmal_killer.png",		
	spellLevel = 3,
	
	iconColor = util.color( 216/255, 162/255, 255/255 ),

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 2 * self:getLevel() - 1
	end,
	
	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
		
			if targetUnit:isKO() or not targetUnit:getBrain() then
				return false
			end
			if targetUnit:getTraits().isDrone then
				return false, STRINGS.UI.TOOLTIPS.KO_IMMUNE
			end
			if targetUnit:getBrain():getTarget() then
				return false, STRINGS.UI.COMBAT_PANEL_SEEN
			end
			if not sim:canUnitSeeUnit( unit, targetUnit ) then
				return false, STRINGS.UI.COMBAT_PANEL_NO_LOS
			end
		end
		return true
	end,
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not targetUnit:getBrain() then
			return false
		end
		if not sim:canPlayerSeeUnit( userUnit:getPlayerOwner(), targetUnit ) then
			return false
		end
		if targetUnit:isDown() then
			return false
		end
		
		return true
	end,
	
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		local tooltip, section = abilityutil.SPELL_TEMPLATE.onTooltip( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		
		if targetUnitID and self:canUseAbility( sim, abilityOwner, abilityUser, targetUnitID ) then
			local targetUnit = sim:getUnit( targetUnitID )
			local cell = self:getPhantasmSpawnLocation( sim, abilityUser, targetUnit )
			if cell then
				tooltip._game = hud._game
				tooltip.x0, tooltip.y0 = abilityUser:getLocation()
				tooltip.x1, tooltip.y1 = cell.x, cell.y
				tooltip.activate = tooltip_activate
				tooltip.deactivate = tooltip_deactivate
			end
		end
		
		return tooltip, section
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetUnitID )
		local x0,y0 = userUnit:getLocation()
		local targetUnit = sim:getUnit( targetUnitID )
		local x1,y1 = targetUnit:getLocation()
	
		if self:castSpell( sim, abilityOwner, userUnit, targetUnit ) then
			-- Create guard frightened behaviour
			self:setTarget( sim, targetUnit )
			
			-- Calculate phantasm position
			local simulacrumCell = self:getPhantasmSpawnLocation( sim, userUnit, targetUnit )
			local x2, y2 = simulacrumCell.x, simulacrumCell.y
		
			-- Create the phantasm
			abilitydefs._abilities[ "phantasm_retarget" ] = phantasm_retarget
			local holoTemplate = util.extend( holoItem )
			{
				kanim = userUnit:getUnitData().kanim,
				profile_anim = userUnit:getUnitData().profile_anim,
				profile_build = userUnit:getUnitData().profile_build,
				sounds = userUnit:getUnitData().sounds,
			}
			
			local facing = userUnit:getFacing()
			if x1 ~= x2 or y1 ~= y2 then
				facing = simquery.getDirectionFromDelta(x1 - x2, y1 - y2)
			end
			
			self.simulacrum = simfactory.createUnit( holoTemplate, sim )
			self.simulacrum:setPlayerOwner( sim:getPC() )
			self.unitID = self.simulacrum:getID()
			sim:spawnUnit( self.simulacrum )
			sim:warpUnit( self.simulacrum, simulacrumCell )
			self.simulacrum:useMP(self.simulacrum:getMP(), sim)
			simquery.suggestAgentFacing(self.simulacrum, facing)
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.simulacrum } )
			
			self.simulacrum:hasAbility( "phantasm_retarget" ).spell = self
			sim:dispatchEvent( simdefs.EV_UNIT_ACTIVATE, { unit = self.simulacrum } )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, userUnit )
			sim:addTrigger( simdefs.TRG_UNIT_APPEARED, self, userUnit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, userUnit )
			simquery.addSimulacrumMovement( sim )
			
			-- Spook guard
			targetUnit.huntedByPhantasmalKiller = self.simulacrum
			self:spookGuard( sim, targetUnit )
		end
		
		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	getPhantasmSpawnLocation = function( self, sim, userUnit, targetUnit )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		local dist = mathutil.dist2d(x0, y0, x1, y1)
		
		local x2, y2 = math.floor( x0 + 0.5 + (x1 - x0) / dist ), math.floor( y0 + 0.5 + (y1 - y0) / dist )
		if dist < 2 or not sim:getCell( x2, y2 ) or not sim:getLOS():hasSight( userUnit, x2, y2 ) then
			x2, y2 = x0,y0
		end
		
		return sim:getCell( x2, y2 )
	end,
	
	setTarget = function( self, sim, targetUnit )
		self.targetUnit = targetUnit
		targetUnit:setAlerted(true)
		targetUnit:getTraits().ce_cantmelee = true
		--targetUnit.huntedByPhantasmalKiller = true
		
		if not targetUnit:getTraits().vip then
			self.madeVip = true
			targetUnit:getTraits().vip = true
			updateBrain( targetUnit )
		end
		
		self.overrideVoice = targetUnit:getTraits().voice
		targetUnit:getTraits().voice = "Executive"
	end,
	
	endConcentration = function( self, sim )
		-- Un-panic guard
		if self.targetUnit then
			self.targetUnit:getTraits().voice = self.overrideVoice
			self.targetUnit.huntedByPhantasmalKiller = nil
			self.targetUnit:getTraits().ce_cantmelee = nil
			self.targetUnit:getTraits().hasSeenSimulacrum = nil
			if self.madeVip then
				self.madeVip = nil
				self.targetUnit:getTraits().vip = nil
			end
			if self.targetUnit:getBrain() then
				self.targetUnit:getBrain():reset()
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
	
		-- Remove phantasm
		if self.simulacrum and self.simulacrum:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_DEACTIVATE, { unit = self.simulacrum } )
			sim:warpUnit( self.simulacrum, nil )
			sim:despawnUnit( self.simulacrum )
		end
		
		self.simulacrum = nil
		self.targetUnit = nil
		sim:getPC():glimpseUnit( sim, self.unitID )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		sim:removeTrigger( simdefs.TRG_UNIT_APPEARED, self )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		simquery.removeSimulacrumMovement( sim )
	end,
	
	onTrigger = function( self, sim, evType, evData, abilityUser )
		local targetUnit = self.targetUnit
		local simulacrum = self.simulacrum
		
		if evType == simdefs.TRG_UNIT_WARP and targetUnit and evData.unit == simulacrum then
			local x0, y0 = simulacrum:getLocation()
			local x1, y1 = targetUnit:getLocation()
			
			if x0 and x0 == x1 and y0 == y1 then
				targetUnit:getTraits().regenerative_nanocells = nil
				if targetUnit:getTraits().neutralize_shield then
					targetUnit:getTraits().neutralize_shield = 0
				end
				sim:damageUnit( targetUnit, 999, nil, nil, abilityUser )
				
				if self.madeVip then
					self.madeVip = nil
					targetUnit:getTraits().vip = nil
				end
				
				self.targetUnit = nil
				simulacrum:useMP(simulacrum:getMP(), sim)
			else
				local canSee, canSense = sim:canUnitSee(targetUnit, evData.from_cell.x, evData.from_cell.y)
				if canSee or canSense then
					-- Attempt to reface if the target moved
					self:spookGuard()
				end
			end
		elseif evType == simdefs.TRG_UNIT_APPEARED and targetUnit and targetUnit:getBrain() then
			if targetUnit:getID() == evData.seerID and evData.unit == simulacrum then
				local interest = self:spookGuard()
				--if interest and targetUnit:getTraits().movePath then
					interest.noticed = false
					targetUnit:interruptMove( sim )
					targetUnit:getBrain():reset()
				--end
			end
		elseif evType == simdefs.TRG_START_TURN and evData:isPC() then
			if not targetUnit or not targetUnit:getTraits().hasSeenSimulacrum then
				self.arcaneFocus:executeAbility( sim )
			elseif targetUnit and not sim:canUnitSeeUnit( targetUnit, simulacrum ) then
				targetUnit:getTraits().hasSeenSimulacrum = nil
				targetUnit:getBrain():reset()
			end
		end
	end,
	
	spookGuard = function( self )
		local targetUnit = self.targetUnit
		local hasSeen = targetUnit:getTraits().hasSeenSimulacrum
		local x,y = self.simulacrum:getLocation()
		
		if hasSeen then
			-- Interest may have moved
			hasSeen.x, hasSeen.y = x,y
			targetUnit:turnToFace( x, y )
		else
			hasSeen = targetUnit:getBrain():getSenses():addInterest( x, y, simdefs.SENSE_SIGHT, "SAW_PHANTASM", self.simulacrum )
			targetUnit:getTraits().hasSeenSimulacrum  = hasSeen
		end
		
		return hasSeen 
	end,
}

return spellDef