local array = include( "modules/array" )
local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )

local holoItem = {
	type = "simunit",
	rig = "grenaderig",
	name = STRINGS.WIZARD.SPELLS.SILENT_IMAGE.name,
	desc = STRINGS.ITEMS.GRENADE_HOLO_TOOLTIP,
	flavor = STRINGS.ITEMS.GRENADE_HOLO_FLAVOR,
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_holo_grenade_small.png",	
	profile_icon_100 = "gui/icons/item_icons/icon-item_holo_grenade.png",	
	kanim = "kanim_hologrenade",
	sounds = { activate="SpySociety/Actions/holocover_activate", deactivate="SpySociety/Actions/holocover_deactivate" },
	traits = { cover=true, holoProjector=true, agent_filter=true, deployed = true },	
	abilities = {}, 	
	onWorldTooltip = function( tooltip, unit )
		tooltip:addLine("<ttheader>"..util.toupper(unit:getName()).."</>")
		
		if unit:getTraits().concentrationLessDuration then
			tooltip:addDesc( util.sformat( STRINGS.WIZARD.TOOLTIPS.SILENT_IMAGE, unit:getTraits().concentrationLessDuration ) )
		elseif unit:getTraits().concentrationDuration then
			tooltip:addDesc( util.sformat( STRINGS.WIZARD.TOOLTIPS.SILENT_IMAGE_CONCENTRATION, unit:getTraits().concentrationDuration ) )
		end
	end,
	--locator=true,
}

local spellDef = util.extend( abilityutil.SPELL_CELLTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.SILENT_IMAGE,

	profile_icon = "gui/icons/spells/silent_image.png",		
	spellLevel = 1,
	
	iconColor = util.color( 216/255, 162/255, 255/255 ),

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetCell )
		local lvl = self:getLevel()
		if lvl >= 3 then
			return 0 -- No concentration required!
		else
			return 3 * lvl
		end
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = unpack(targetCell)
		local lvl = self:getLevel()
		
		if lvl >= 3 then
			self.concentrationLessDuration = 3 * lvl
		end
		
		sim:addTrigger( simdefs.TRG_START_TURN, self )
	
		if self:castSpell( sim, abilityOwner, userUnit, targetCell ) then
			self.smoke = simfactory.createUnit( holoItem, sim )
			self.smoke:getTraits().concentrationLessDuration = self.concentrationLessDuration
			self.smoke:getTraits().concentrationDuration = self:getConcentrationDuration()
			self.holoCell = sim:getCell( x1, y1 )
			sim:spawnUnit( self.smoke )
			sim:warpUnit( self.smoke, self.holoCell )
			
			sim:dispatchEvent( simdefs.EV_UNIT_ACTIVATE, { unit = self.smoke } )
			sim:triggerEvent( simdefs.TRG_UNIT_DEPLOYED, { unit = self.smoke })

			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
			self.interestUnits = {}
		end
		
		self:endThrow( sim, abilityOwner, userUnit )
	end,
	
	endConcentration = function( self, sim )
		if self.smoke and self.smoke:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_DEACTIVATE, { unit = self.smoke } )
			sim:warpUnit( self.smoke, nil )
			sim:despawnUnit( self.smoke )
		end
		
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
	end,
	
	onTrigger = function( self, sim, evType, evData )
		if evType == simdefs.TRG_START_TURN and evData:isPC() then
			if self.concentrationLessDuration then
				self.concentrationLessDuration = self.concentrationLessDuration - 1
				self.smoke:getTraits().concentrationLessDuration = self.concentrationLessDuration
				if self.concentrationLessDuration == 0 then
					self:endConcentration( sim )
				end
			else
				self.smoke:getTraits().concentrationDuration = self.smoke:getTraits().concentrationDuration - 1
			end
		elseif evType == simdefs.TRG_UNIT_WARP and evData.to_cell == self.holoCell then
			local brain = evData.unit:getBrain()
			if brain and not self.interestUnits[evData.unit] then
				local x1, y1 = self.holoCell.x, self.holoCell.y
				brain:getSenses():addInterest( x1, y1, simdefs.SENSE_SIGHT, simdefs.REASON_FOUNDOBJECT, self.smoke )
				self.interestUnits[evData.unit] = true
			end
		end
	end
}

return spellDef