local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )
local serverdefs = include( "modules/serverdefs" )

local spellDef = util.extend( abilityutil.SPELL_UNITTARGET ) {
	strings = STRINGS.WIZARD.SPELLS.TRUE_STRIKE,

	profile_icon = "gui/icons/spells/true_strike.png",
	spellLevel = 1,
	
	iconColor = util.color( 183/255, 215/255, 233/255 ),
	
	isTarget = function( self, sim, unit, userUnit, targetUnit)
		if not simquery.isShootable( userUnit, targetUnit ) or not targetUnit:getTraits().isAgent then
			return false
		end
		if not sim:canPlayerSeeUnit( userUnit:getPlayerOwner(), targetUnit ) then
			return false
		end
		
		local x0, y0 = userUnit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		return sim:canUnitSeeUnit( userUnit, targetUnit ) or mathutil.dist2d( x0, y0, x1, y1 ) <= 12
	end,

	getConcentrationDuration = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return 10
	end,
	
	getDescParams = function( self, sim, unit )
		return self:getStrength( self, sim, unit )
	end,
	
	getStrength = function( self, sim, unit )
		return 2 * self:getLevel()
	end,

	canUseAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local canCast, canCastReason = self:canCastSpell( sim, abilityOwner, unit )
		if not canCast then
			return false, canCastReason
		end

		if targetUnitID then
			local targetUnit = sim:getUnit( targetUnitID )
			
			if targetUnit and not sim:canUnitSeeUnit( unit, targetUnit ) then
				return false, STRINGS.UI.REASON.BLOCKED
			end
		end
		
		return true
	end,

	executeAbility = function( self, sim, abilityOwner, unit, targetUnitID )
		local targetUnit = sim:getUnit( targetUnitID )
		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		
		if self:castSpell( sim, abilityOwner, unit, targetUnit ) then
			if not targetUnit:getTraits().trueStrike then
				local processKOresist = targetUnit.processKOresist
				local getArmor = targetUnit.getArmor
				
				targetUnit.getArmor = function(guard, ...)
					return getArmor(guard,...) - guard:getTraits().trueStrike
				end
				targetUnit.processKOresist = function(guard, ...)
					guard:getTraits().resistKO = (guard:getTraits().resistKO or 0) - guard:getTraits().trueStrike
					local ko = processKOresist(guard, ...)
					guard:getTraits().resistKO = guard:getTraits().resistKO + guard:getTraits().trueStrike
					if guard:getTraits().resistKO == 0 then
						guard:getTraits().resistKO = nil
					end
					return ko
				end
			end
			
			targetUnit:getTraits().trueStrike = self:getStrength()
			self.targetUnit = targetUnit
			
			local params = { color = { util.extend( self.iconColor ) { symbol="wall" } } }
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/listen" )
    		--sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, { txt=util.sformat(STRINGS.WIZARD.UI.TRUE_STRIKE), x = x1, y = y1, color = self.iconColor } )
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = targetUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params=params} )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
		end
		
		self:endThrow( sim, abilityOwner, unit )
	end, 
	
	endConcentration = function( self, sim )
		if self.targetUnit then
			self.targetUnit:getTraits().trueStrike = 0
			
			local params = { color = { util.extend( self.iconColor ) { symbol="wall" } } }
			sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/listen" )
    		--sim:dispatchEvent( simdefs.EV_UNIT_FLY_TXT, { txt=util.sformat(STRINGS.WIZARD.UI.TRUE_STRIKE), x = x1, y = y1, color = self.iconColor } )
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = self.targetUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params=params} )	
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.targetUnit } )	
		end
	end,
}

return spellDef