local game = include("states/state-game")
local simactions = include( "sim/simactions" )

-- Create a new action type for combining multiple triggerActions
function simactions.multiTriggerAction( sim, triggerData )
	for i, action in ipairs( triggerData ) do
		sim:triggerEvent( action[1], action[2] )
	end
end

local oldRewindTurns = game.rewindTurns

-- When rewinding to turn 1, rewindTurns is hardcoded to always leave 1 action left
-- While this is fine most of the time, mid_1 needs 2 triggerActions for its mission initialization sequence
-- To get around the issue of the second trigger being deleted, we combine several triggerActions into 1 action
function game:rewindTurns()
	local combineTriggers = {}
	
	for i = 1, #self.simHistory do
		if self.simHistory[i] and self.simHistory[i].name == "triggerAction" then
			table.insert(combineTriggers,self.simHistory[i])
		else
			break
		end
	end
	
	if #combineTriggers > 1 then
		for i = #combineTriggers, 2, -1 do
			table.remove(self.simHistory,i)
		end
		
		self.simHistory[1] = { name = "multiTriggerAction", crc = self.simHistory[1].crc, playerIndex = self.simHistory[1].playerIndex, combineTriggers }
	end
	
	oldRewindTurns(self)
end