local game = include( "modules/game" )
local util = include("client_util")
local array = include( "modules/array" )
local mui = include( "mui/mui" )
local serverdefs = include( "modules/serverdefs" )
local modalDialog = include( "states/state-modal-dialog" )
local rig_util = include( "gameplay/rig_util" )
local cdefs = include("client_defs")
local skilldefs = include( "sim/skilldefs" )
local scroll_text = include("hud/scroll_text")
local unitdefs = include( "sim/unitdefs" )
local metadefs = include( "sim/metadefs" )
local stateTeamPreview = include( "states/state-team-preview" )

local function onLoadoutClicked( self, index )
	self.loadoutIndex = index
	self:destroy()
end

----------------------------------------------------------------
--

local dialog = class()

function dialog:showAgentFluff( agentWidget, agentDef )
	-- Agent Name
	agentWidget.binder.agentName:setText( agentDef.loadoutName )

	-- Agent Image
	if agentDef.team_select_img then
		agentWidget.binder.agentImg:setImage(agentDef.team_select_img[1])
		agentWidget.binder.agentImg:setColor(1,1,1)
		--agentWidget.binder.agentImg:createTransition("transition_none")
		--agentWidget.binder.agentImgBG:createTransition("transition_none")
	end
end

function dialog:showLockedAgent( agentWidget, agentDef )
	agentWidget.binder.agentName:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_NAME )
	
	agentWidget.binder.agentDescBody:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_DESC )
	if agentDef.team_select_img then
		agentWidget.binder.agentImg:setImage(agentDef.team_select_img[1])
		agentWidget.binder.agentImg:setColor(0,0,0)
		--agentWidget.binder.agentImg:createTransition("transition_none")
		--agentWidget.binder.agentImgBG:createTransition("transition_none")
	end

	for i, widget in agentWidget.binder:forEach( "item" ) do
		widget:setVisible( false )
	end
	for i, widget in agentWidget.binder:forEach( "skill" ) do
		if agentDef.skills[i] then
			local skill = skilldefs.lookupSkill(agentDef.skills[i])
			widget.binder.costTxt:setText(skill.name)

			for i, barWidget in widget.binder:forEach( "bar" ) do
				barWidget.binder.meterbarSmall.binder.bar:setColor(unpack(stateTeamPreview.COLORS.BLANK_COLOR))
			end
			widget:setVisible( true )
		else
			widget:setVisible(false)
		end
	end
end

function dialog:show( loadouts, teamPreview )
	self.loadoutIndex = nil
	
	local screen = mui.createScreen( "screen-loadout-selector.lua" )
	self.screen = screen 
	mui.activateScreen( screen )

	screen:createTransition("activate_left")
  
	self.screen:findWidget("closeBtn").onClick = util.makeDelegate( nil, self.destroy, self)
	
	for i=1, #loadouts do
		local loadout = loadouts[i]
		local widget = self.screen.binder.loadoutList:addItem( self )
		local agentDef = unitdefs.lookupTemplate( loadout )
		
		widget.binder.dlcIcon:setVisible(false)
		widget.binder.dlcName:setVisible(false)
		if agentDef.traits.dlcFooter then
			teamPreview:setFooter( widget.binder.dlcName, widget.binder.dlcIcon, agentDef.traits.dlcFooter )
		end
		
		if metadefs.isRewardUnlocked( loadout ) then
			widget.binder.btn.onClick = util.makeDelegate( nil, onLoadoutClicked, self, i )
			teamPreview:showAgentSkills( widget, agentDef, true )
			teamPreview:showAgentItems( widget, agentDef )
			self:showAgentFluff( widget, agentDef )
		else
			widget.binder.btn:setDisabled( true )
			widget.binder.btn:setVisible( false )
			self:showLockedAgent( widget, agentDef )
		end
	end
end

function dialog:destroy()
	MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/popdown" )

	if self.onClose then
		self.onClose()
	end
	
	self.screen:createTransition( "deactivate_left",
		function( transition )
			mui.deactivateScreen( self.screen )
		end,
	{ easeOut = true } )
end


return dialog


