local mui = include("mui/mui")
local cdefs = include("client_defs")
local util = include("modules/util")
local simdefs = include("sim/simdefs")
local SCRIPTS = include('client/story_scripts')
local talkinghead = include('client/fe/talkinghead')
local rig_util = include( "gameplay/rig_util" )
local serverdefs = include("modules/serverdefs")
local mui_defs = include( "mui/mui_defs")

local mapScreen = include("states/state-map-screen")

function mapScreen:UpdateMapColours( region_to_highlight )

	if self.highlight_region == region_to_highlight then
		return
	end

	
	self.colourRoutine = self.colourRoutine or MOAICoroutine.new()
	if self.colourRoutine then
		self.colourRoutine:stop()
	end

	
	self.highlight_region = region_to_highlight
	self.highlight_t = 0
	self.highlight_duration = 0.333

	self.start_region_colours = util.tcopy(self.current_region_colours)
	self.end_region_colours = self.end_region_colours or {}

	if region_to_highlight == nil then
		
		for k,v in pairs(simdefs.map_colours_normal) do
			self.end_region_colours[k] = v
		end	
	else
		self.end_region_colours = {}
		for k,v in pairs(self.regions) do
			
			if k == region_to_highlight then
				self.end_region_colours[k] = simdefs.map_colours_highlight[k]
			else
				self.end_region_colours[k] = simdefs.map_colours_unhighlight[k]
			end
		end
	end


	self.colourRoutine:run( function()
		while (self.highlight_t < self.highlight_duration) do
			self.highlight_t = math.min(self.highlight_t + 1/cdefs.SECONDS, self.highlight_duration)
			local t = self.highlight_t / self.highlight_duration
		    
		    for k,v in pairs(self.regions) do
		    	
		    	for n = 1, 4 do
					self.current_region_colours[k][n] = (self.start_region_colours[k][n] or 0) + ((self.end_region_colours[k][n] or 0) - (self.start_region_colours[k][n] or 0))*t
		    	end

		    	v:setColor(unpack(self.current_region_colours[k]))
		    end
		    coroutine.yield()
		 end
	end)
end

local oldGetMapLocation = mapScreen.getMapLocation

-- Translates coordinates in serverdefs to widget coordinates.
function mapScreen:getMapLocation( location, offx, offy )
	local x, y = oldGetMapLocation( self, location, offx, offy )
	return x + simdefs.map_offset.x, y + simdefs.map_offset.y
end

local oldHideControls = mapScreen.HideControls

function mapScreen:HideControls( ... )
	oldHideControls( self, ... )

	-- This is a good place to show modded corp regions, it's after the screen is activated but before we start showing any dialogue
	for region, widgetName in pairs(simdefs.map_territories) do
		local widget = self._screen:findWidget("territories."..widgetName)
		self.regions[region] = widget
	end
	
    self.current_region_colours = util.tcopy(simdefs.map_colours_normal)

    for region,widget in pairs(self.regions) do
		widget:setColor(unpack(simdefs.map_colours_normal[region]))
		local x, y = widget:getPosition()
		widget:setPosition( x + simdefs.map_offset.x, y + simdefs.map_offset.y )
    end
end

local oldOnClickLocation = mapScreen.OnClickLocation
	
function mapScreen:OnClickLocation( situation )
	oldOnClickLocation(self,situation)
	local corpData = serverdefs.getCorpData( situation )
	local screen
	for i, _screen in pairs(mui.internals._activeScreens) do
		if _screen._filename == "mission_preview_dialog.lua" or _screen._filename == "mission_preview_dialog_ps4.lua" then
			screen = _screen
			break
		end
	end
	if corpData.region then
		local c = simdefs.map_colours_highlight[corpData.region]
		if c then
			screen:findWidget("CorpDetails.logo"):setColor(44/255,1,1,1)
		end
	end
end

function mapScreen:GetIntroScript()
	local campaign = self._campaign
	local script = nil
	
	if campaign.difficultyOptions.maxHours == math.huge then
		--endless mode!
		script = SCRIPTS.ENDLESS_MAP.MISSIONS[campaign.currentDay+1] and SCRIPTS.ENDLESS_MAP.MISSIONS[campaign.currentDay+1][campaign.missionsPlayedThisDay+1]
		--no generics
	else
		local dialogIndex = campaign.currentDay+1
		local customScript = nil

  		if campaign.campaignEvents then
	        for i=#campaign.campaignEvents,0,-1 do
        		local event = campaign.campaignEvents[i] 
        		if event then
		        	if event.eventType == simdefs.CAMPAIGN_EVENTS.CUSTOM_SCRIPT then
		        		--this is removed after the mission in the state-postgame
						customScript = event.data
		        		break
		        	end	       
	        	end
	        end
    	end

    	if campaign.customScriptIndex then
    		dialogIndex = campaign.customScriptIndex..campaign.customScriptIndexDay
    	end

		if customScript then
			script = SCRIPTS.CAMPAIGN_MAP.MISSIONS[customScript]
		elseif campaign.hours >= campaign.difficultyOptions.maxHours then
			script = SCRIPTS.CAMPAIGN_MAP.MISSIONS[SCRIPTS.FINAL_LEVEL_SCRIPT][1]
		elseif dialogIndex and SCRIPTS.CAMPAIGN_MAP.MISSIONS[dialogIndex] then
			script = SCRIPTS.CAMPAIGN_MAP.MISSIONS[dialogIndex][campaign.missionsPlayedThisDay+1]
		end

		--generic stuff
		if not script then
			if campaign.currentDay > campaign.previousDay then 
				script = SCRIPTS.CAMPAIGN_MAP.GENERIC_NEWDAY
			else
				script = SCRIPTS.CAMPAIGN_MAP.GENERIC
			end
		end
	end
	
	return script
end

function mapScreen:ShouldShowAgentsAdded()
	local campaign = self._campaign
	return campaign.difficultyOptions.maxHours ~= math.huge and campaign.currentDay == 0 and campaign.missionsPlayedThisDay == 0
end

function mapScreen:ShowAgentsAdded()
	local campaign = self._campaign
	
	if self:ShouldShowAgentsAdded() then
        local agentsadded = include('client/fe/agentsadded')
		self.agentsadded = agentsadded(campaign.agency)

		while not self.agentsadded:IsDone() do
			coroutine.yield()
		end
	end
end

function mapScreen:ShownIntroCampaignEvents()
	local campaign = self._campaign
	
	-- RUN MISSION CAMPAIGN EVETNS
	if campaign.campaignEvents then
	   	for i=#campaign.campaignEvents,0,-1 do
    		local event = campaign.campaignEvents[i] 
        	if event then     

	            if event.eventType == simdefs.CAMPAIGN_EVENTS.MAP_POPUP then                 
		            local map_moodal = include('client/fe/map-modal')
					self.map_moodal = map_moodal(event.data)

					while not self.map_moodal:IsDone() do
						coroutine.yield()
					end				            	   
					table.remove(campaign.campaignEvents,i)  
	            end           		

        	end
    	end
	end	
end

function mapScreen:InsertLastMissionScript(script)
	local campaign = self._campaign
	
	local time_left = math.max(0, campaign.difficultyOptions.maxHours - campaign.hours)
	local last_mission = time_left > 0
	if time_left > 0 then
		for k,sit in pairs(campaign.situations) do
			local travelTime = serverdefs.calculateTravelTime( campaign.location, sit.mapLocation, campaign ) + serverdefs.BASE_TRAVEL_TIME 		
			if travelTime < time_left then
				last_mission = false
				break
			end
		end
	end

	if last_mission then					
		if not campaign.lastMissionIndex then
			campaign.lastMissionIndex = 1
		end
		table.insert(script, SCRIPTS.CAMPAIGN_MAP.LAST_MISSION[campaign.lastMissionIndex])
	end
end

function mapScreen:DoMonst3rInject()
	local campaign = self._campaign
	local do_monster_inject_speech = campaign.monst3rInject and campaign.currentDay == 0 and campaign.missionsPlayedThisDay == 1
	
	--add on an addendum
	if do_monster_inject_speech then		
		self.storytalkinghead:PlayScript(SCRIPTS.CAMPAIGN_MAP.MONSTER_INJECT)
	end
	
	while not self.storytalkinghead:IsDone() do
		coroutine.yield()
	end
end

function mapScreen:PlayIntroScript()

	rig_util.wait(0.5*cdefs.SECONDS)

	local script = self:GetIntroScript()
	local campaign = self._campaign
	
	if script then
		script = util.tcopy(script)
		local storyheadscreen = nil
		if MOAIEnvironment.OS == "PS4" then
			storyheadscreen = mui.createScreen( "modal-story_ps4.lua" )
		elseif MOAIEnvironment.OS == "XBO" then
			storyheadscreen = mui.createScreen( "modal-story_ps4.lua" )
		elseif MOAIEnvironment.OS == "NX" then
			storyheadscreen = mui.createScreen( "modal-story_ps4.lua" )
		else
			storyheadscreen = mui.createScreen( "modal-story.lua" )
		end
		mui.activateScreen( storyheadscreen )
		
		self.storytalkinghead = talkinghead(storyheadscreen, storyheadscreen.binder.Friends)

		self:InsertLastMissionScript(script)

		self.storytalkinghead:PlayScript(script)
		self.storytalkinghead:FadeBackground( 0.25 * cdefs.SECONDS )

		while not self.storytalkinghead:IsDone() do
			coroutine.yield()
		end
		
		self:DoMonst3rInject(script)

		mui.deactivateScreen( storyheadscreen ) 
		MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT )
		
		self:ShowAgentsAdded()
	end

	self:ShownIntroCampaignEvents()
end