
local statemodaldialog = include( "states/state-modal-dialog")


local resources = include( "resources")
local util = include( "modules/util" )
local mui = include( "mui/mui" )
local mui_defs = include( "mui/mui_defs" )
local cdefs = include( "client_defs" )
local rig_util = include( "gameplay/rig_util" )

local CANCEL = 0
local OK = 1
local AUX = 2
local AUX2 = 3

local _M =
{
}

_M.setText = function( self, str )
	self.screen.binder.bodyTxt:setText(str)
end

_M.onUnload = function ( self )
	mui.deactivateScreen( self.screen )
	self.screen = nil
end

local function showDialog( modalDialog, controller )
	statemgr.activate( modalDialog )

    controller = controller or statemodaldialog.findGameModalControl()

	while modalDialog.result == nil do
        if controller and controller.result then
            break
        end
		coroutine.yield()
	end

    -- It's possible for shutdown to preemptively deactivate active dialogs from under us.
    if statemgr.isActive( modalDialog ) then
	    statemgr.deactivate( modalDialog )
    end

    if controller and controller.result then
        return controller.result
    end

	return modalDialog.result
end

function statemodaldialog.showTimehop( ... )
	--MOAIFmodDesigner.playSound(  cdefs.SOUND_HUD_MENU_POPUP )
	local modalDialog = statemodaldialog.createTimehopDialog( ... )
	return showDialog( modalDialog )
end


function statemodaldialog.createTimehopDialog( dialogStr, headerStr, auxStr, continueTxt, cancelTxt, large )
	local t = util.tcopy( _M )

	t.onLoad = function( self )
		if large then
			self.screen = mui.createScreen( "modal-dialog-large.lua" )
		else
			self.screen = mui.createScreen( "modal-dialog.lua" )
		end

		mui.activateScreen( self.screen )
		
		if continueTxt then
			self.screen.binder.okBtn:setText( util.toupper(continueTxt) )
		end

		self.screen.binder.okBtn.onClick = statemodaldialog.rewindTurns(6)
		self.screen.binder.cancelBtn.onClick = util.makeDelegate( nil, function() t.result = CANCEL end )
		self.screen.binder.cancelBtn:setClickSound(cdefs.SOUND_HUD_MENU_CANCEL)
		if auxStr then
			self.screen.binder.auxBtn:setVisible( true )
			self.screen.binder.auxBtn:setText( auxStr )
			self.screen.binder.auxBtn.onClick = util.makeDelegate( nil, function() t.result = AUX end )
		end
		if cancelTxt then
			self.screen.binder.cancelBtn:setText( cancelTxt )
		end
		if headerStr then
			self.screen.binder.headerTxt:setText( headerStr )
		else
			self.screen.binder.headerTxt:setText("")
		end	
		self.screen.binder.bodyTxt:setText( dialogStr )
	end
	
	return t
end

local function findGameModalControl2()
    local states = statemgr.getStates()
    for i = 1, #states do
        if states[i].simCore and states[i].levelData and states[i].modalControl then
            return states[i].modalControl -- If it acts like a duck and quacks like a duck...
        end
    end
	return false
end

function statemodaldialog.rewindTurns( num )
    local numTurns = 3*2
	
	if not findGameModalControl2() and findGameModalControl2().simCore ~= nil and findGameModalControl2().simHistory and #findGameModalControl2().simHistory > 1 then
		return false
	end

    local rewindsLeft = findGameModalControl2().simCore:getTags().rewindsLeft
	-- Go back 'numTurns'.
    -- What this means is to pop-back to the Nth endTurnAction (without actually removing it), where N = numTurns. 
	while #findGameModalControl2().simHistory > 1 and numTurns > 0 do
        local action = table.remove( findGameModalControl2().simHistory )
        if action.name == "endTurnAction" then
            numTurns = numTurns - 1
            if numTurns <= 0 then
                table.insert( findGameModalControl2().simHistory, action )
            end
        end
	end
	findGameModalControl2():goto( #findGameModalControl2().simHistory )
	findGameModalControl2():doAction( "rewindAction", rewindsLeft )
end