----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local mui = include( "mui/mui" )
local mui_defs = include( "mui/mui_defs" )
local mui_util = include( "mui/mui_util" )
local cdefs = include( "client_defs" )
local util = include( "client_util" )
local array = include( "modules/array" )
local modalDialog = include( "states/state-modal-dialog" )

local select_preset_dialog = class()

local user = savefiles.getFile( KLEIPersistentStorage.PST_SaveGame, "gen-presets" )
user:load()

----------------------------------------------------------------
-- Local functions

local ACTIVE_TXT = { 61/255,81/255,83/255,1 }
local INACTIVE_TXT = { 1,1,1,1 }

local MENU_UNSELECTED = { 0, 0, 0, 0 }
local MENU_SELECTED = { 100/255, 176/255, 176/255 }

local function loadFn( baseData, savedData )
	if type(savedData) ~= "table" then
		return savedData
	elseif type(baseData) ~= "table" then
		return util.tcopy(savedData)
	end
	
	local data = {}
	
	for k, v in pairs(savedData) do
		data[k] = loadFn( baseData[k], v )
	end
	
	for k, v in pairs(baseData) do
		if data[k] == nil then
			if type(v) == "table" then
				data[k] = util.tcopy(v)
			else
				data[k] = v
			end
		end
	end
	
	return data
end

local function onChangedOption( self, modID )
	MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_MENU_CLICK )
	self._screen.binder.presetNameBox:setText(modID)
end

local function onClickSave( dialog )
	if not user.data.generationPresets then
		user.data.generationPresets = {}
	end
	
	local id = dialog._screen.binder.presetNameBox:getText()
	
	if (#id > 0) then
		local foundAt
		for i, v in pairs(dialog._list:getItems()) do
			if (v.user_data == id) then
				foundAt = i
				break
			end
		end
		
		if not foundAt then
			local widget = dialog._list:addItem( id, "PresetOption" )
			widget:setText( id )
			widget.onClick = util.makeDelegate( nil, onChangedOption, dialog, id )
			widget:setColorInactive( unpack(MENU_SELECTED) )
			widget:setTextColorInactive( unpack(ACTIVE_TXT) )
			widget:setStateInactive()
			foundAt = #dialog._list:getItems()
			dialog._list:selectIndex( foundAt )
		end
	
		user.data.generationPresets[foundAt] = {
			id = id,
			difficulty = dialog.difficulty,
			options = util.tcopy(dialog.options),
			dlcOptions = util.tcopy(dialog.dlcOptions)
		}
		user.data.lastGenerationPreset = id
	
		user:save()
	end
end

local function onClickLoad( dialog )
	if not user.data.generationPresets then
		user.data.generationPresets = {}
	end
	
	local id = dialog._screen.binder.presetNameBox:getText()
	
	if (#id > 0) then
		local foundAt
		for i, v in pairs(dialog._list:getItems()) do
			if (v.user_data == id) then
				foundAt = i
				break
			end
		end
		
		if foundAt then
			user.data.lastGenerationPreset = id
			
			dialog.difficulty = user.data.generationPresets[foundAt].difficulty
			dialog.options = loadFn( dialog.options, user.data.generationPresets[foundAt].options )
			dialog.dlcOptions = loadFn( dialog.dlcOptions, user.data.generationPresets[foundAt].dlcOptions )

			dialog.result = true
			
			user:save()
		end
	end
end

local function onClickDelete( dialog )
	if not user.data.generationPresets then
		user.data.generationPresets = {}
	end
	
	local id = dialog._screen.binder.presetNameBox:getText()
	
	if (#id > 0) then
		local foundAt
		for i, v in pairs(dialog._list:getItems()) do
			if (v.user_data == id) then
				foundAt = i
				break
			end
		end
		
		if foundAt then
			dialog._list:removeItem( foundAt )
			
			if dialog._list._scrollbar then
				local count = dialog._list:getMaxVisibleItems()
				if #dialog._list._items > count then
					dialog._list._scrollbar.binder.scrollbar:setRange( 0, #dialog._list._items - count )
					dialog._list._scrollbar:setVisible( true )
				else
					dialog._list._scrollbar:setVisible( false )
				end
			end
			
			dialog._screen.binder.presetNameBox:setText("")
			table.remove(user.data.generationPresets,foundAt)
			user.data.lastGenerationPreset = ""
	
			user:save()
		end
	end
end

local function onClickCancel( dialog )
	dialog.result = false
end

----------------------------------------------------------------
-- Interface functions

function select_preset_dialog:init()
	local screen = mui.createScreen( "modal-select-preset" )
	self._screen = screen
	self._list =  screen:findWidget("list")

	screen.binder.saveBtn.onClick = util.makeDelegate( nil, onClickSave, self )
	--screen.binder.okBtn:setText(STRINGS.UI.BUTTON_ACCEPT)
	
	screen.binder.loadBtn.onClick = util.makeDelegate( nil, onClickLoad, self )
	--screen.binder.okBtn:setText(STRINGS.UI.BUTTON_ACCEPT)
	
	screen.binder.deleteBtn.onClick = util.makeDelegate( nil, onClickDelete, self )
	--screen.binder.okBtn:setText(STRINGS.UI.BUTTON_ACCEPT)
	
	screen.binder.cancelBtn.onClick = util.makeDelegate( nil, onClickCancel, self )  
	screen.binder.cancelBtn:setHotkey( "pause" )
end

function select_preset_dialog:onLoad()
	MOAIFmodDesigner.playSound(  cdefs.SOUND_HUD_MENU_POPUP )
	mui.activateScreen( self._screen )
end

function select_preset_dialog:onUnload()
	MOAIFmodDesigner.playSound(  cdefs.SOUND_HUD_MENU_POPDOWN  )
	mui.deactivateScreen( self._screen )
end

function select_preset_dialog:show(difficulty, options, dlcOptions)
	self.result = nil

	if not self._screen then
		self:init()
	end

	self.difficulty = difficulty
	self.options = options
	self.dlcOptions = dlcOptions

	statemgr.activate( self )
	
	self._list:clearItems()
	self._list._scrollbar:setVisible( false )
	
	if user.data.lastGenerationPreset then
		self._screen.binder.presetNameBox:setText(user.data.lastGenerationPreset)
	end
	
	if user.data.generationPresets then
		for i, v in ipairs(user.data.generationPresets) do
			self:addPreset(v)
		end
	end
	
	self._txt = ""
	
	while self.result == nil do
		local txt = self._screen.binder.presetNameBox:getText()
		
		if self._txt ~= txt then
			self._txt = txt
			self._list:selectIndex( nil )
			
			for i, v in pairs(self._list:getItems()) do
				if (v.user_data == txt) then
					self._list:selectIndex( i )
					v.widget:setColorInactive( unpack(MENU_SELECTED) )
					v.widget:setTextColorInactive( unpack(ACTIVE_TXT) )
				else
					v.widget:setColorInactive( unpack(MENU_UNSELECTED) )
					v.widget:setTextColorInactive( unpack(INACTIVE_TXT) )
				end
				v.widget:updateImageState()
			end
		end
		
		coroutine.yield()
	end
    statemgr.deactivate( self )

	return self.result
end

function select_preset_dialog:peek(difficulty, options, dlcOptions)
	self.difficulty = difficulty
	self.options = options
	self.dlcOptions = dlcOptions
	
	if user.data.lastGenerationPreset and user.data.generationPresets then
		for i, preset in ipairs(user.data.generationPresets) do
			if preset.id == user.data.lastGenerationPreset then
				self.difficulty = preset.difficulty
				self.options = loadFn(self.options, preset.options)
				self.dlcOptions = loadFn(self.dlcOptions, preset.dlcOptions)
				
				return true
			end
		end
	end
	
	return false
end

function select_preset_dialog:isActive()
	return self._screen and self._screen:isActive()
end

function select_preset_dialog:addPreset(preset)
	
	local widget

	widget = self._list:addItem( preset.id, "PresetOption" )
	widget:setText( preset.id )
	widget.onClick = util.makeDelegate( nil, onChangedOption, self, preset.id )
	
	return widget
end

return select_preset_dialog
