----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------
local teamPreview = include( "states/state-team-preview" )
local game = include( "modules/game" )
local util = include("client_util")
local array = include( "modules/array" )
local mui = include( "mui/mui" )
local serverdefs = include( "modules/serverdefs" )
local modalDialog = include( "states/state-modal-dialog" )
local rig_util = include( "gameplay/rig_util" )
local cdefs = include("client_defs")
local skilldefs = include( "sim/skilldefs" )
local tool_templates = include("sim/unitdefs/itemdefs")
local mainframe_abilities = include( "sim/abilities/mainframe_abilities" )
local scroll_text = include("hud/scroll_text")
local unitdefs = include( "sim/unitdefs" )
local metadefs = include( "sim/metadefs" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )

----------------------------------------------------------------

local COLORS = {
	SET_COLOR = { 244/255, 255/255, 120/255, 1 },
	POSSIBLE_COLOR = { 0/255, 184/255, 0/255, 1 },
	BLANK_COLOR = { 56/255, 96/255, 96/255, 200/255 },
	--HOVER_COLOR = { 255/255, 255/255, 255/255, 1 },
	--HOVER_COLOR_FAIL = { 178/255,0/255, 0/255, 1 },
	
	--ACTIVE_TXT = { 61/255, 81/255, 83/255, 1 },
	--INACTIVE_TXT = { 1,1,1,1 },
	
	ACTIVE_BG = { 244/255, 255/255, 120/255, 1 },
	INACTIVE_BG = { 78/255, 136/255, 136/255, 1 },
}

teamPreview.COLORS = COLORS

----------------------------------------------------------------

function teamPreview:stopVoice()
	MOAIFmodDesigner.stopSound("voice" )
	if self._voiceCoroutine then
		self._voiceCoroutine:stop()
		self._voiceCoroutine = nil
	end
end

function teamPreview:lookupLoadouts( template )
	return serverdefs.LOADOUTS[ template ]
end

function teamPreview:lookupTemplate( name )
	return tool_templates[ name ]
end

function teamPreview:onClickBioPrev( nextBtn, prevBtn, textWidget, text )
	textWidget:setText(text)
	nextBtn:setVisible(true)

	textWidget._pageNumber = textWidget._pageNumber -1

	for i= textWidget._pageNumber,0, -1 do
		textWidget:nextPage()
		textWidget._cont._prop:forceUpdate()
	end

	if textWidget._pageNumber == 0 then
		prevBtn:setVisible(false)
	else
		prevBtn:setVisible(true)
	end
end

function teamPreview:onClickBioNext( nextBtn, prevBtn, textWidget )
	textWidget:nextPage()
	if textWidget:hasNextPage() then
		nextBtn:setVisible(true)
	else
		nextBtn:setVisible(false)
	end

	textWidget._pageNumber = textWidget._pageNumber +1

	prevBtn:setVisible(true)
end

function teamPreview:onClickShowLogs( logData )
	MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/popup" )

	local dialog = include( "fe/agent-logs" )
	dialog.onClose = function() end
	dialog:show( logData )
end

function teamPreview:setFooter(name, icon, footer)
	name:setVisible(true)
	icon:setVisible(true)
	name:setText(footer[1])
	icon:setImage(footer[2])

	print(STRINGS.UI.DLC_TAG, footer[1],footer[2],footer[3])

	local author = STRINGS.UI.DLC_TAG_TYPE_MOD
	if footer[3] == "Klei" then
		author = STRINGS.UI.DLC_TAG_TYPE_EXPANSION
	end

	name:setTooltip(util.sformat(STRINGS.UI.DLC_TAG,footer[1],author))
	icon:setTooltip(util.sformat(STRINGS.UI.DLC_TAG,footer[1],author))
end

function teamPreview:onClickCampaign()
	self:stopVoice()

	-- Verify that chosen loadout is unlocked
	local agentIDs = {}
	for k,v in ipairs(self._selectedAgents) do
		local agentName = serverdefs.SELECTABLE_AGENTS[v]
		local loadouts = self:lookupLoadouts( agentName )
		local agentTemplate = loadouts[ self._selectedLoadouts[k] ]
		if metadefs.isRewardUnlocked( agentTemplate ) then
			agentIDs[k] = agentTemplate
		else
			modalDialog.show( STRINGS.UI.TEAM_SELECT.LOCKED_LOADOUT )
			return
		end
	end

	local programIDs = {}
	for k, programName in ipairs(self._selectedPrograms) do
		if metadefs.isRewardUnlocked( programName ) then
			programIDs[k] = programName
		else
			modalDialog.show( STRINGS.UI.TEAM_SELECT.LOCKED_LOADOUT )
			return
		end
	end

	-- Create campaign!
	local selectedAgency = serverdefs.createAgency( agentIDs, programIDs )
	local campaign = serverdefs.createNewCampaign( selectedAgency, self._campaignDifficulty, self._campaignOptions )

	local user = savefiles.getCurrentGame()
	user.data.saveSlots[ user.data.currentSaveSlot ] = campaign
	user.data.num_campaigns = (user.data.num_campaigns or 0) + 1
	user.data.gamesStarted = true
	user:save()

	-- Transition to the intro screen (It's 2074, corporations rule...)
	statemgr.deactivate( self )
	MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT )

	local intro_screen = mui.createScreen( "modal-posttutorial.lua" )
	mui.activateScreen( intro_screen )

	local overlay = intro_screen:findWidget("overlay")

	local closeBtn = intro_screen.binder.closeBtn
	closeBtn:setHotkey( "pause" )
	closeBtn:setText( STRINGS.UI.NEW_GAME_CONFIRM )
	local clicked = false
	local fade_time = .5
	local c = 1
	closeBtn.onClick = function()
		if not clicked then
			-- Clicked: Fade out intro screen, then show the map screen
			clicked = true
			overlay:setVisible(true)

			local t = c*fade_time
			while t < fade_time do
				t = t + 1/cdefs.SECONDS
				c = math.min(t / fade_time, 1)
				overlay:setColor(0, 0, 0, c)
				coroutine.yield()
			end

			mui.deactivateScreen( intro_screen )
			MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT )
			MOAIFmodDesigner.stopSound("theme")

			local stateMapScreen = include( "states/state-map-screen" )
			statemgr.activate( stateMapScreen(), campaign )
		end
	end

	-- Fade in intro screen
	overlay:setVisible(true)
	local t = 0
	while t < fade_time and not clicked do
		t = t + 1/cdefs.SECONDS
		c = 1- math.min(t / fade_time, 1)
		overlay:setColor(0, 0, 0, c)
		coroutine.yield()
	end

	if not clicked then
		overlay:setVisible(false)
	end

	if self._dialog then
		self._dialog:close()
		self._dialog = nil
	end
end

function teamPreview:onClickCancel()
	MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_WOOSHOUT )

	--don't show the movie again if you've already seen it
	config.SHOW_MOVIE = false

	local stateGenerationOptions = include( "states/state-generation-options" )
	statemgr.deactivate( self )
	statemgr.activate( stateGenerationOptions(), self._campaignDifficulty, self._campaignOptions )

	if self._dialog then
		self._dialog:close()
		self._dialog = nil
	end
end

function teamPreview:selectProgram( programIdx, programID )
	local programPanel = self._panel.binder["program"..programIdx]
	local programData =  mainframe_abilities[programID]
	local PWRtooltip = STRINGS.UI.TOOLTIPS.TEAM_SELECT_PWR_DESC

	programPanel.binder["dlcName"]:setVisible(false)
	programPanel.binder["dlcIcon"]:setVisible(false)

	if programData.dlcFooter then
		self:setFooter( programPanel.binder["dlcName"], programPanel.binder["dlcIcon"], programData.dlcFooter)
	end

	programPanel.binder["firewallTooltip"]:setTooltip(PWRtooltip..STRINGS.UI.TOOLTIPS.TEAM_SELECT_NOPWR)

	if metadefs.isRewardUnlocked( programID ) then
		programPanel.binder["programIcon"]:setImage(programData.icon)
		programPanel.binder["programIcon"]:setColor(1,1,1)
		programPanel.binder["programName"]:setText(util.toupper(programData.name))
		programPanel.binder["programTxt"]:spoolText( programData.desc )
		programPanel.binder["programTxt"]:setColor(140/255,255/255,255/255)

		if programData.cpu_cost > 0 or not programData.passive then
			programPanel.binder["powerTxt"]:setText(tostring(programData.cpu_cost))
			programPanel.binder["firewallTooltip"]:setTooltip(PWRtooltip..util.sformat(STRINGS.UI.TOOLTIPS.TEAM_SELECT_PWR,programData.cpu_cost))
		else
			programPanel.binder["powerTxt"]:setText("-")
		end

	else
		programPanel.binder["programIcon"]:setImage(programData.icon)
		programPanel.binder["programIcon"]:setColor(0,0,0)
		programPanel.binder["programName"]:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_NAME )

		if programData.lockedText then
			programPanel.binder["programTxt"]:setText( programData.lockedText )
		else
			programPanel.binder["programTxt"]:setText( STRINGS.UI.TEAM_SELECT.UNLOCK_TO_USE )
		end
		programPanel.binder["programTxt"]:setColor( 1, 1, 1 )

		programPanel.binder["powerTxt"]:setText("?")


		for i, widget in programPanel.binder:forEach( "power" ) do
			widget:setColor(17/255,29/255,29/255)
		end
	end


	programPanel:setVisible(true)

	self._selectedPrograms[ programIdx ] = programID
end

----------------------------------------------------------------

function teamPreview:updateButtonFromLoadout( widget, agentID, loadout, selected, canDisable )
	if loadout then
		local agentDef = unitdefs.lookupTemplate( loadout )
		widget:setVisible( true )

		if metadefs.isRewardUnlocked( loadout ) then
			widget.binder.btn:setText( agentDef.loadoutName )
			widget.binder.btn:setDisabled( false )
			widget.binder.btn:setVisible( true )
		else
			widget.binder.btn:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_NAME )
			widget.binder.btn:setDisabled( canDisable )
			widget.binder.btn:setVisible( not canDisable )
		end

		if selected then
			widget.binder.btn:setColorInactive( unpack(COLORS.ACTIVE_BG) )
			widget.binder.btn:updateImageState()
			widget.binder.arrow:setVisible( true )
		else
			widget.binder.btn:setColorInactive( unpack(COLORS.INACTIVE_BG) )
			widget.binder.btn:updateImageState()
			widget.binder.arrow:setVisible( false )
		end
	else
		widget:setVisible( false )
	end
end

function teamPreview:updateLoadoutButtons( agentWidget, agentIdxInTeam, agentID )
	local MAX_LOADOUT_BUTTONS = 2

	local loadouts = self:lookupLoadouts( agentID )
	
	agentWidget.binder.loadoutSharedBtn:setVisible( false )
	agentWidget.binder.loadoutArrowLeft:setVisible( false )
	agentWidget.binder.loadoutArrowRight:setVisible( false )
	for i, widget in agentWidget.binder:forEach( "loadoutBtn" ) do
		widget:setVisible( false )
	end

	if not metadefs.isRewardUnlocked( agentID ) then
		-- Do nothing
	elseif #loadouts > MAX_LOADOUT_BUTTONS then
		local sharedWidget = agentWidget.binder.loadoutSharedBtn
		local loadoutIdx = self._selectedLoadouts[ agentIdxInTeam ]
		self:updateButtonFromLoadout( sharedWidget, agentID, loadouts[ loadoutIdx ], false, false )
		agentWidget.binder.loadoutArrowLeft:setVisible( true )
		agentWidget.binder.loadoutArrowRight:setVisible( true )
	else
		for i, widget in agentWidget.binder:forEach( "loadoutBtn" ) do
			local loadout = loadouts[ i ]
			self:updateButtonFromLoadout( widget, agentID, loadout, self._selectedLoadouts[ agentIdxInTeam ] == i, true )
		end
	end
end

function teamPreview:showAgentItems( agentWidget, agentDef )
	local SECOND_ROW_OFFSET = 13
	local ITEMS_IN_ROW = 3
	
	local itemCount = #agentDef.upgrades
	
	for i, widget in agentWidget.binder:forEach( "item" ) do
		if agentDef.upgrades[i] then
			widget:setVisible(true)

			local unitData = self:lookupTemplate( agentDef.upgrades[i] )
			local newItem = simfactory.createUnit( unitData, nil )
			widget:setImage( unitData.profile_icon )

			local tooltip = util.tooltip( self.screen )
			local section = tooltip:addSection()
			newItem:getUnitData().onTooltip( section, newItem )
			widget:setTooltip( tooltip )
			widget:setColor( unpack( COLORS.SET_COLOR ) )
			
			-- If there are more than 3 items, shift items up to make room for a second row
			if widget._rowOffset then
				local x, y = widget:getPosition()
				widget:setPosition( x, y - SECOND_ROW_OFFSET )
				widget._rowOffset = nil
			end
			if itemCount > ITEMS_IN_ROW then
				local x, y = widget:getPosition()
				widget:setPosition( x, y + SECOND_ROW_OFFSET )
				widget._rowOffset = SECOND_ROW_OFFSET
			end
		else
			widget:setVisible(false)
		end
	end
end

function teamPreview:showAgentSkills( agentWidget, agentDef, noSpool )
	for i, widget in agentWidget.binder:forEach( "skill" ) do
		if agentDef.skills[i] then

			skill = skilldefs.lookupSkill(agentDef.skills[i])

			if noSpool then
				widget.binder.costTxt:setText(skill.name)
			else
				widget.binder.costTxt:spoolText(skill.name,15)
			end

			for i, barWidget in widget.binder:forEach( "bar" ) do
				barWidget.binder.meterbarSmall.binder.bar:setColor(unpack(COLORS.BLANK_COLOR))
			end

			widget.binder.bar1.binder.meterbarSmall.binder.bar:setColor(unpack(COLORS.SET_COLOR))
			widget:setVisible( true )

			local tooltip = "<c:F4FF78>".. skill.name.. "</>\n"..skill.description
			widget.binder.tooltip:setTooltip(tooltip)

		else
			widget:setVisible(false)
		end
	end
	
	for i, skillUpgrade in pairs(agentDef.startingSkills) do
		for v, skill in ipairs(agentDef.skills) do
	  		for f=1,skillUpgrade-1 do
				if skill == i then
			 		 agentWidget.binder["skill"..v].binder["bar"..(1+f)].binder.meterbarSmall.binder.bar:setColor(unpack(COLORS.POSSIBLE_COLOR))
				end
			end
		end
	end
	
	for i, widget in agentWidget.binder:forEach( "iconSkill" ) do
		widget:setVisible(false)
		agentWidget.binder["skillTxt"..i]:setVisible(false)
	end
end

function teamPreview:showAgentFluff( agentWidget, agentDef )
	-- Agent Name
	agentWidget.binder.agentName:spoolText(util.toupper(agentDef.name),15)

	-- Agent Image
	if agentDef.team_select_img then
		agentWidget.binder.agentImg:setImage(agentDef.team_select_img[1])
		agentWidget.binder.agentImg:setColor(1,1,1)
		agentWidget.binder.agentImg:createTransition("transition_none")
		agentWidget.binder.agentImgBG:createTransition("transition_none")
	end

	-- Full name and age
	local fluff = util.sformat( STRINGS.UI.BIO_FORMAT, agentDef.fullname, agentDef.age )
	agentWidget.binder.fluffTxt:spoolText(util.toupper(fluff))
	agentWidget.binder.agentNameBody:setText( util.toupper(agentDef.codename or agentDef.fullname) )
	
	-- Backstory
	agentWidget.binder.agentDescBody:setText( agentDef.blurb )
	agentWidget.binder.prevBtn:setVisible(false)
	agentWidget.binder.agentDescBody._pageNumber = 0

	agentWidget.binder.prevBtn.onClick = util.makeDelegate( nil, self.onClickBioPrev, self, agentWidget.binder.nextBtn, agentWidget.binder.prevBtn, agentWidget.binder.agentDescBody, agentDef.blurb )
	agentWidget.binder.nextBtn.onClick =  util.makeDelegate( nil, self.onClickBioNext, self, agentWidget.binder.nextBtn, agentWidget.binder.prevBtn, agentWidget.binder.agentDescBody )

	if agentWidget.binder.agentDescBody:hasNextPage() then
		agentWidget.binder.nextBtn:setVisible(true)
	else
		agentWidget.binder.nextBtn:setVisible(false)
	end
end

function teamPreview:showLogBtn( agentWidget, agentDef )
	-- Show quick link to agent's log if they have one unlocked
	local user = savefiles.getCurrentGame()
	if agentDef.logs and user.data.logs then
		for i, logID in ipairs( agentDef.logs ) do
			local hasLog
			
			for i, logData in ipairs( user.data.logs ) do
				if logData.id == logID then
					hasLog = logData
					break
				end
			end
			
			if hasLog then
				agentWidget.binder.logBtn:setVisible(true)
				agentWidget.binder.logBtn.onClick = util.makeDelegate( nil, self.onClickShowLogs, self, hasLog )
				break
			end
		end
	end
end

function teamPreview:showLockedAgent( agentWidget, agentDef )
	agentWidget.binder.agentName:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_NAME )
	agentWidget.binder.agentNameBody:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_NAME )

	if agentDef.lockedText then
		agentWidget.binder.fluffTxt:setText( agentDef.lockedText )
	else
		agentWidget.binder.fluffTxt:setText( STRINGS.UI.TEAM_SELECT.UNLOCK_TO_USE )
	end
	agentWidget.binder.agentDescBody:setText( STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_DESC )
	if agentDef.team_select_img then
		agentWidget.binder.agentImg:setImage(agentDef.team_select_img[1])
		agentWidget.binder.agentImg:setColor(0,0,0)
		agentWidget.binder.agentImg:createTransition("transition_none")
		agentWidget.binder.agentImgBG:createTransition("transition_none")
	end

	for i, widget in agentWidget.binder:forEach( "item" ) do
		widget:setVisible( false )
	end
	for i, widget in agentWidget.binder:forEach( "skill" ) do
		if agentDef.skills[i] then
			local skill = skilldefs.lookupSkill(agentDef.skills[i])
			widget.binder.costTxt:spoolText(skill.name,15)

			for i, barWidget in widget.binder:forEach( "bar" ) do
				barWidget.binder.meterbarSmall.binder.bar:setColor(unpack(COLORS.BLANK_COLOR))
			end
			widget:setVisible( true )
		else
			widget:setVisible(false)
		end
	end
	for i, widget in agentWidget.binder:forEach( "iconSkill" ) do
		widget:setVisible( false )
	end
end

----------------------------------------------------------------

function teamPreview:selectAgent( agentIdxInTeam, loadoutIdx, agentSelectionIdx, transition, noVoice )
	-- Show team info
	-- Show the agents on the team
	
	local agentWidget = self._panel.binder[ "agent" .. agentIdxInTeam ]
	local agentID = serverdefs.SELECTABLE_AGENTS[ agentSelectionIdx ]
	local loadouts = self:lookupLoadouts( agentID )
	local agentDef = unitdefs.lookupTemplate( loadouts[ loadoutIdx ] )
	assert( agentDef )

	local oldSelectionIdx = self._selectedAgents[ agentIdxInTeam ]
	if oldSelectionIdx then
		self._isAgentSelected[ oldSelectionIdx ] = nil
	end
	
	self._selectedLoadouts[ agentIdxInTeam ] = loadoutIdx
	self._selectedAgents[ agentIdxInTeam ] = agentSelectionIdx
	self._isAgentSelected[ agentSelectionIdx ] = true

	-- Show DLC footer
	agentWidget.binder.dlcIcon:setVisible(false)
	agentWidget.binder.dlcName:setVisible(false)

	if agentDef.traits.dlcFooter then
		self:setFooter( agentWidget.binder.dlcName, agentWidget.binder.dlcIcon, agentDef.traits.dlcFooter)
	end

	-- Show widget
	self:updateLoadoutButtons( agentWidget, agentIdxInTeam, agentID )
	MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/popdown" )

	if transition then
		agentWidget:createTransition( "transition_none" )
	end

	agentWidget:setVisible(true)
	--agentWidget.binder.fadeBox:blinkWhiteTransition()
	agentWidget.binder.logBtn:setVisible(false)

	if metadefs.isRewardUnlocked( loadouts[ loadoutIdx ] ) then
		-- Actually show the agent!
		self:showAgentItems( agentWidget, agentDef )
		self:showAgentSkills( agentWidget, agentDef )
		self:showAgentFluff( agentWidget, agentDef )
		self:showLogBtn( agentWidget, agentDef )

		-- Central voice line
		if not noVoice and not self.mute then
			self:stopVoice()
			self._voiceCoroutine = MOAICoroutine.new()
			self._voiceCoroutine:run( function()
				rig_util.wait(0.5*cdefs.SECONDS)
				self._voiceCoroutine:stop()
				MOAIFmodDesigner.playSound(agentDef.sounds.bio,"voice" )
			end )
		end
	else
		self:showLockedAgent( agentWidget, agentDef )
	end
end

----------------------------------------------------------------

function teamPreview:updateSelectionState()
	local lb = self.screen:findWidget( "agentListbox" )
	for i = 1, lb:getItemCount() do
		local widget = lb:getItem( i ).widget
		if self._isAgentSelected[ i ] and metadefs.isRewardUnlocked( serverdefs.SELECTABLE_AGENTS[ i ] ) then
			widget:setVisible( true )
			widget.binder.img:setColor( 1, 1, 1, 1 )
			widget.binder.img:setVisible( true )
			widget.binder.bg:setVisible( true )
		elseif metadefs.isRewardUnlocked( serverdefs.SELECTABLE_AGENTS[ i ] ) and i <= #serverdefs.SELECTABLE_AGENTS then
			widget:setVisible( true )
			widget.binder.img:setColor( 60/255, 60/255, 60/255, 0.9 )
			widget.binder.img:setVisible( true )
			widget.binder.bg:setVisible( true )
		elseif i <= #serverdefs.SELECTABLE_AGENTS and self._isAgentSelected[ i ] then
			widget.binder.img:setColor( 0, 0, 0, 1)
			widget.binder.img:setVisible( true )
			widget.binder.bg:setVisible( true )
			widget:setVisible( true )
		elseif i <= #serverdefs.SELECTABLE_AGENTS then
			widget.binder.img:setColor( 0, 0, 0, 0.75)
			widget.binder.img:setVisible( true )
			widget.binder.bg:setVisible( true )
			widget:setVisible( true )
		else
			widget:setVisible( false )
		end
	end
end

function teamPreview:muteToggle()
	local settings = savefiles.getSettings( "settings" )

	if not self.mute then
		self.mute = true
		self:stopVoice()
		self._panel.binder.muteTxt:setVisible(true)
		settings.data.muteCentralTeamSelect = true
		settings:save()
	else
		self.mute = nil
		self._panel.binder.muteTxt:setVisible(false)
		settings.data.muteCentralTeamSelect = nil
		settings:save()
	end
end

function teamPreview:randomizeEverything(  )
	-- Select random agents
	local agentTable = {}
	for i,agent in ipairs( serverdefs.SELECTABLE_AGENTS ) do
		if metadefs.isRewardUnlocked( agent ) then
			table.insert( agentTable, i )
		end
	end
	
	util.shuffle( agentTable )
	
	for i = 1, self._selectAgentCount do
		local loadouts = self:lookupLoadouts( serverdefs.SELECTABLE_AGENTS[ agentTable[i] ] )
		local loadoutIdx = {}
		for j, loadout in ipairs( loadouts ) do
			if metadefs.isRewardUnlocked( loadout ) then
				table.insert( loadoutIdx, j )
			end
		end
		
		self:selectAgent( i, loadoutIdx[ math.random(#loadoutIdx) ], agentTable[i], true )
	end
	
	-- Select random programs
	for i, programList in ipairs( serverdefs.SELECTABLE_PROGRAMS ) do
		local programTable = {}
		for j, program in ipairs( programList ) do
			if metadefs.isRewardUnlocked( program ) then
				table.insert( programTable, program )
			end
		end
		
		self:selectProgram( i, programTable[ math.random(#programTable) ] )
	end
	
	self:updateSelectionState(  )
end

function teamPreview:onClickSharedLoadout( agentIdxInTeam )
	MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/popup" )

	local loadouts = self:lookupLoadouts( serverdefs.SELECTABLE_AGENTS[ self._selectedAgents[ agentIdxInTeam ] ] )
	local dialog = include( "state-loadout-selector" )
	dialog.onClose = function()
		if dialog.loadoutIndex then
			self:selectAgent( agentIdxInTeam, dialog.loadoutIndex, self._selectedAgents[ agentIdxInTeam ], false, true )
		end
	end
	dialog:show( loadouts, self )
end

function teamPreview:onClickLoadout( agentIdxInTeam, loadoutIdx )
	if self._selectedLoadouts[ agentIdxInTeam ] ~= loadoutIdx then
		self:selectAgent( agentIdxInTeam, loadoutIdx, self._selectedAgents[ agentIdxInTeam ], false, true )
	end
end

function teamPreview:onClickNextLoadout( direction, agentIdxInTeam )
	local loadoutIdx = self._selectedLoadouts[ agentIdxInTeam ]
	local loadouts = self:lookupLoadouts( serverdefs.SELECTABLE_AGENTS[ self._selectedAgents[ agentIdxInTeam ] ] )
	loadoutIdx = loadoutIdx + direction
	if loadoutIdx <= 0 then
		loadoutIdx = #loadouts
	elseif loadoutIdx > #loadouts then
		loadoutIdx = 1
	end
	
	self:selectAgent( agentIdxInTeam, loadoutIdx, self._selectedAgents[ agentIdxInTeam ], false, true )
end

function teamPreview:onClickAgentSelect( selectionIdx )
	if self._isAgentSelected[ selectionIdx ] then
		-- Do nothing
	else
		self:selectAgent( self._agentSelectIdx, 1, selectionIdx, nil )
		self:updateSelectionState(  )
		self._agentSelectIdx = ( self._agentSelectIdx % self._selectAgentCount ) + 1
	end
end

function teamPreview:onClickNextAgent( direction, agentIdxInTeam )
	local selectionIdx = self._selectedAgents[ agentIdxInTeam ]
	repeat
		selectionIdx = selectionIdx + direction
		if selectionIdx <= 0 then
			selectionIdx = #serverdefs.SELECTABLE_AGENTS
		elseif selectionIdx > #serverdefs.SELECTABLE_AGENTS then
			selectionIdx = 1
		end
	until not self._isAgentSelected[selectionIdx]
	
	self:selectAgent( agentIdxInTeam, 1, selectionIdx )
	self:updateSelectionState(  )
end

function teamPreview:onClickNextProgram( direction, programIdx )
	local programName = self._selectedPrograms[ programIdx ]
	local programs = serverdefs.SELECTABLE_PROGRAMS[ programIdx ]
	local selectionIdx = array.find( programs, programName ) - 1 -- 0-based
	selectionIdx = (selectionIdx + direction) % #programs + 1 -- back to 1-based
	self:selectProgram( programIdx, programs[ selectionIdx ] )
end

----------------------------------------------------------------

function teamPreview:initScreen()
	self.screen = mui.createScreen( "team_preview_screen.lua" )
	mui.activateScreen( self.screen )

	self._scroll_text = scroll_text.panel( self.screen.binder.bg )

	self._panel = self.screen.binder.pnl

	self._panel.binder.title_txt:setText("")
	self._panel.binder.title_txt:spoolText(STRINGS.UI.SCREEN_NAME_TEAM_SELECT)

	self._panel.binder.acceptBtn.onClick = util.makeDelegate( nil,  self.onClickCampaign, self)
	self._panel.binder.acceptBtn:setClickSound(cdefs.SOUND_HUD_MENU_CONFIRM)

	self._panel.binder.cancelBtn.onClick = util.makeDelegate( nil,  self.onClickCancel, self)
	self._panel.binder.cancelBtn:setClickSound(cdefs.SOUND_HUD_MENU_CANCEL)
	
	self._panel.binder.randomizeBtn.onClick = util.makeDelegate( nil, self.randomizeEverything, self )


	self._panel.binder.muteBtn.onClick = util.makeDelegate( nil, self.muteToggle, self )
	self._panel.binder.muteTxt:setVisible(false)

	local gameModeStr = serverdefs.GAME_MODE_STRINGS[ self._campaignDifficulty ]
	local ironmanStr = STRINGS.UI.HUD_OFF
	if self._campaignOptions.rewindsLeft == 0 then
		ironmanStr = STRINGS.UI.HUD_ON
	end
	local difficultyStr = string.format("%s: <c:8CFFFF>%s</>	%s: <c:8CFFFF>%s</>",
		util.toupper(STRINGS.UI.DIFFICULTY_STR),
		util.toupper(gameModeStr),
		STRINGS.UI.DIFF_OPTION_IRONMAN,
		ironmanStr )
	self._panel.binder.gameOptions:setText( difficultyStr)
end

function teamPreview:initProgramWidget( programIdx, programList )
	for i,program in ipairs( programList ) do
		programData =  mainframe_abilities[program]

		if programData.abilityOverride then
			programList[i] = programData.abilityOverride[1]
		end
	end
	
	local programWidget = self._panel.binder["program"..programIdx]
	programWidget.binder.arrowLeft.binder.btn.onClick = util.makeDelegate( nil,  self.onClickNextProgram, self, -1, programIdx )
	programWidget.binder.arrowRight.binder.btn.onClick = util.makeDelegate( nil,  self.onClickNextProgram, self, 1, programIdx )
	programWidget:setVisible(false)
	
	self:selectProgram( programIdx, programList[1] )
end

function teamPreview:initAgentWidget( agentIdxInTeam )
	local agentWidget = self._panel.binder["agent"..agentIdxInTeam]

	agentWidget.binder.arrowLeft.binder.btn.onClick = util.makeDelegate( nil,  self.onClickNextAgent, self, -1, agentIdxInTeam )
	agentWidget.binder.arrowRight.binder.btn.onClick = util.makeDelegate( nil,  self.onClickNextAgent, self, 1, agentIdxInTeam )
	
	agentWidget.binder.loadoutArrowLeft.binder.btn.onClick = util.makeDelegate( nil, self.onClickNextLoadout, self, -1, agentIdxInTeam )
	agentWidget.binder.loadoutArrowRight.binder.btn.onClick = util.makeDelegate( nil, self.onClickNextLoadout, self, 1, agentIdxInTeam )
	
	agentWidget.binder.loadoutSharedBtn.binder.btn.onClick = util.makeDelegate( nil, self.onClickSharedLoadout, self, agentIdxInTeam )
	
	for i, widget in agentWidget.binder:forEach( "loadoutBtn" ) do
		widget.binder.btn.onClick = util.makeDelegate( nil, self.onClickLoadout, self, agentIdxInTeam, i )
	end
	
	agentWidget:setVisible(false)
	
	if agentIdxInTeam <= self._selectAgentCount then
		self:selectAgent( agentIdxInTeam, 1, agentIdxInTeam, true, true )
	end
end

----------------------------------------------------------------

function teamPreview:init( campaignDifficulty, campaignOptions )
	assert( campaignDifficulty and campaignOptions )
	self._campaignOptions = campaignOptions
	self._campaignDifficulty = campaignDifficulty
	self._voiceCoroutine = nil
	self._selectAgentCount = 2
end

function teamPreview:onLoad(  )
	self._selectedAgents = {}
	self._isAgentSelected = {}
	self._selectedPrograms = {}
	self._selectedLoadouts = {}
	self._agentSelectIdx = 1
	
	for i = 1, self._selectAgentCount do
		self._selectedAgents[i] = i
		self._selectedLoadouts[i] = i
		self._isAgentSelected[i] = true
	end

	log:write("teamPreview:onLoad()")
	self:initScreen()

	local settings = savefiles.getSettings( "settings" )
	if settings.data.muteCentralTeamSelect then
		self:muteToggle()
	end

	-- Show agent selection list
	local lb = self.screen:findWidget( "agentListbox" )
	lb.onItemClicked = util.makeDelegate( nil, self.onClickAgentSelect, self )

	for i = 1, #serverdefs.SELECTABLE_AGENTS do
		local agentID = serverdefs.SELECTABLE_AGENTS[ i ]
		local agentDef = unitdefs.lookupTemplate( agentID )
		local widget = lb:addItem( agentID )
		widget.binder.img:setImage( agentDef.team_select_img[1] )
		widget.binder.img.onClick = util.makeDelegate( nil, self.onClickAgentSelect, self, i )
		if metadefs.isRewardUnlocked( serverdefs.SELECTABLE_AGENTS[ i ] ) then
			widget.binder.img:setTooltip( util.toupper(agentDef.name) )
		else
			widget.binder.img:setTooltip( util.toupper(STRINGS.UI.TEAM_SELECT.LOCKED_AGENT_NAME) )
		end
	end

	if #serverdefs.SELECTABLE_AGENTS < 14 then
		local bump = (14 - #serverdefs.SELECTABLE_AGENTS) * 54
		local x1,y1 = lb:getPosition()
		lb:setPosition(x1 + bump, y1)

		local title =self.screen:findWidget( "available_txt" )
		local x2,y2 = title:getPosition()
		title:setPosition(x2 + bump, y2)
	end
	
	-- Show programs
	for i, programList in ipairs( serverdefs.SELECTABLE_PROGRAMS ) do
		self:initProgramWidget( i, programList )
	end
	
	-- Show agents
	for i, widget in self._panel.binder:forEach( "agent" ) do
		self:initAgentWidget( i )
	end

	self:updateSelectionState(  )

	-- Play music
	if not MOAIFmodDesigner.isPlaying("theme") then
		MOAIFmodDesigner.playSound("SpySociety/Music/music_map_AMB","theme")
	end

	-- Show new DLC rewards
	for i,reward in ipairs(metadefs.DLC_INSTALL_REWARDS) do
		local settings = savefiles.getSettings( "settings" )
		if not settings.data[reward.unlocks[1].name.."_unlocked"] then
			if reward.unlockType == metadefs.PROGRAM_UNLOCK then
				modalDialog.showUnlockProgram( reward )
			elseif reward.unlockType == metadefs.AGENT_UNLOCK then
				modalDialog.showUnlockAgent( reward )
			end
			settings.data[reward.unlocks[1].name.."_unlocked"] = true
			settings:save()
		end
	end
end

function teamPreview:onUnload()
	self._scroll_text:destroy()
	self._scroll_text = nil
	self:stopVoice()

	mui.deactivateScreen( self.screen )
	self.screen = nil
end
