local util = include("modules/util")
local simdefs = include("sim/simdefs")
local agentdefs = include("sim/unitdefs/agentdefs")
local skilldefs = include( "sim/skilldefs" )

local stateUpgradeScreen = include( "states/state-upgrade-screen" )

local oldSelectAgent = stateUpgradeScreen.selectAgent

local skillChanges = class()
stateUpgradeScreen.skillChanges = skillChanges

skillChanges.AGENCY_RESERVE = true

function skillChanges:getSkillCost( agentDef, skill, level )
	local skillDef = skilldefs.lookupSkill( skill.skillID )
    local cost = skillDef[ level ].cost
	return cost
end

function skillChanges:init( upgradeScreen )
    self.agency = upgradeScreen._agency
	self.unitDefs = self.agency.unitDefs
end

function skillChanges:countChanges( agentIdx, skillIdx )
	local agentDef = self.unitDefs[ agentIdx ]
	local skill = agentDef.skills[ skillIdx ]
	if skill and skill.skillChange then
		return skill.skillChange
	end
	
    return 0
end

function skillChanges:hasChanges( agentIdx, skillIdx )
    return self:countChanges( agentIdx, skillIdx ) > 0
end

function skillChanges:learnSkill( agentIdx, skillIdx )
    local agentDef = self.unitDefs[ agentIdx ]
	local skill = agentDef.skills[ skillIdx ]
    local cost = self:getSkillCost( agentDef, skill, skill.level + 1 )

    -- Subtract cost, increase skill.
    if self.agency.cash >= cost then
        skill.skillChange = (skill.skillChange or 0) + 1
        skill.level = skill.level + 1
	    self.agency.cash = self.agency.cash - cost
        return true

    else
        return false
    end
end

function skillChanges:undoSkill( agentIdx, skillIdx )
	local agentDef = self.unitDefs[ agentIdx ]
	local skill = agentDef.skills[ skillIdx ]
    local cost = self:getSkillCost( agentDef, skill, skill.level )

    -- Return cost, decrease skill
    if skill.skillChange and skill.skillChange > 0 then
        skill.skillChange = skill.skillChange - 1
        skill.level = skill.level - 1
	    self.agency.cash = self.agency.cash + cost
        return true
    else
        return false
    end
end

function skillChanges:undoAll()
	for agentIdx, agentDef in pairs( self.unitDefs or {} ) do
		for skillIdx, skill in pairs( agentDef.skills or {} ) do
			while skill.skillChange and skill.skillChange > 0 do
				local cost = self:getSkillCost( agentDef, skill, skill.level )
				
				skill.skillChange = skill.skillChange - 1
				skill.level = skill.level - 1
				self.agency.cash = self.agency.cash + cost
			end
		end
	end
end

function stateUpgradeScreen:selectAgent( unitDef, index, ... )
	if not self.changes or not self.changes.AGENCY_RESERVE then
		self.changes = self.skillChanges( self )
	end

	oldSelectAgent( self, unitDef, index, ... )
end