local util = include("modules/util")
local simdefs = include("sim/simdefs")
local agentdefs = include("sim/unitdefs/agentdefs")
local skilldefs = include( "sim/skilldefs" )
local abilitydefs = include( "sim/abilitydefs" )
local itemdefs = include("sim/unitdefs/itemdefs")

local stateUpgradeScreen = include( "states/state-upgrade-screen" )

local oldSelectAgent = stateUpgradeScreen.selectAgent
local oldSelectIncognita = stateUpgradeScreen.selectIncognita
local oldUnload = stateUpgradeScreen.onUnload
local oldOnDragToAugments = stateUpgradeScreen.onDragToAugments

local function onClickSpells( self, book, focus, unitDef )
   	MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/popup" )
	
	for i, item in ipairs(self.newSpellScrolls ) do
		item.newScroll = nil
	end
	util.tclear( self.newSpellScrolls )

	local dialog = include( "wizard-spell-preparation-dialog" ).prepareDialog
	dialog.onClose = function()
		self:refreshCredits()
		self:refreshInventory( self._agency.unitDefs[ self._selectedIndex ], self._selectedIndex )
	end
	
	dialog:show( self._agency, self, book.upgradeParams.traits.spellsKnown, focus.upgradeParams.traits.spellsPrepared, unitDef.skills[3].level )
end

local function onClickSpells2( self, focus2, unitDef )
   	MOAIFmodDesigner.playSound( "SpySociety/HUD/menu/popup" )
	local dialog = include( "wizard-spell-learn-dialog" )
	dialog:show( self._agency, focus2, unitDef.skills[3].level )
end

function stateUpgradeScreen:searchForItem( itemName )
	local results = {}
	
	for i, agentDef in ipairs( self._agency.unitDefs ) do
		for j, upgrade in ipairs( agentDef.upgrades ) do
			if type(upgrade) == "table" and upgrade.upgradeName == itemName then
				table.insert( results, upgrade )
			end
		end
	end
	
	for j, upgrade in ipairs( self._agency.upgrades ) do
		if type(upgrade) == "table" and upgrade.upgradeName == itemName then
			table.insert( results, upgrade )
		end
	end
	
	return results
end

function stateUpgradeScreen:searchAndDestroyItem( item )
	for i, agentDef in ipairs( self._agency.unitDefs ) do
		for j, upgrade in ipairs( agentDef.upgrades ) do
			if upgrade == item then
				table.remove( agentDef.upgrades, j )
				return
			end
		end
	end
	
	for j, upgrade in ipairs( self._agency.upgrades ) do
		if upgrade == item then
			table.remove( self._agency.upgrades, j )
			return
		end
	end
end

local function checkFreeSpells( self, book, focus )
	if book then
		local freeScroll = self:searchForItem( "item_wiz_spellscroll_free" )[1]
	
		if freeScroll then
			self:searchAndDestroyItem( freeScroll )
			self:refreshInventory( self._agency.unitDefs[ self._selectedIndex ], self._selectedIndex )
			
			local fakeSim = {
				getParams = function()
					return {
						agency = self._agency
					}
				end,
				
				nextRand = function(sim,a,b)
					return math.random(a,b)
				end
			}
			
			local fakeBook = {
				getTraits = function()
					return book.upgradeParams.traits
				end
			}
			
			local bookAbility = abilitydefs.lookupAbility( "wizard_spellbook" )
			local scrollItem = itemdefs[ "item_wiz_spellscroll_free" ]
			local spells = bookAbility:getRandomSpells( fakeSim, fakeBook, scrollItem.traits.freeWizardSpells, 1 )
			
			util.tmerge( book.upgradeParams.traits.spellsKnown, spells )
			if focus then
				util.tmerge( focus.upgradeParams.traits.spellsPrepared, spells )
			end
		end
	end
end

local function findNewSpellScrolls( self )
	self.newSpellScrolls = {}
	
	for i, item in ipairs( self:searchForItem( "item_wiz_spellscroll" ) ) do
		if item.upgradeParams.traits.hiddenScroll then
			item.upgradeParams.traits.hiddenScroll = false
			item.newScroll = true
		end
		
		if item.newScroll then
			table.insert( self.newSpellScrolls, item )
		end
	end
end

function stateUpgradeScreen:selectAgent( unitDef, index, ... )
	local isSame = false
	local spellsWidget = self.screen:findWidget("spells")
    if self._selectedIndex == index then
        isSame = true
    end
	
	oldSelectAgent( self, unitDef, index, ... )
	if isSame or spellsWidget == nil or spellsWidget.isnull then
		return
	end
	
	if not self._agency.upgrades then
		self._agency.upgrades = {}
	end
	
	local focus
	local focus2
	local book
	
	if unitDef.template == "wizard_agent" then
		focus = self:searchForItem( "augment_wizard" )[1]
		book = self:searchForItem( "item_wiz_spellbook" )[1]
	elseif unitDef.template == "wizard_agent_archived" then
		focus2 = self:searchForItem( "augment_wizard_archived" )[1]
	end
	
	if not self.newSpellScrolls then
		findNewSpellScrolls( self )
		checkFreeSpells( self, book, focus )
		self:refreshInventory( self._agency.unitDefs[ self._selectedIndex ], self._selectedIndex )
	end
	
	if self._spellsBlinkThread then
		self._spellsBlinkThread:stop()
		self._spellsBlinkThread = nil
	end
	
	if focus and book then
		self.screen:findWidget("spells"):setVisible(true)
		self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.PREPARE_SPELLS_BTN)
		
		self._spellsBlinkThread = MOAICoroutine.new()
		self._spellsBlinkThread:run( function() 
			local i = 0
			while true do
				if i == 0 then
					local spellSlots = 1 + unitDef.skills[3].level
					local spellsPrepared = #focus.upgradeParams.traits.spellsPrepared
					local spellsKnown = #book.upgradeParams.traits.spellsKnown
					
					if spellsPrepared < spellSlots and spellsKnown > 0 then
						self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.SLOT_NOT_PREPARED)
					elseif #self.newSpellScrolls > 0 then
						self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.SCROLL_AVAILABLE)
					else
						i = -1
					end
				elseif i == 30 then
					self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.PREPARE_SPELLS_BTN)
					i = -30
				end
				
				i = i + 1

				coroutine.yield()
			end
		end )
		self._spellsBlinkThread:resume()	

		self.screen:findWidget("spellsBtn").onClick = util.makeDelegate( nil, onClickSpells, self, book, focus, unitDef )
	elseif focus2 then
		self.screen:findWidget("spells"):setVisible(true)
		self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.SORCERY_SPELLS_BTN)
		
		self._spellsBlinkThread = MOAICoroutine.new()
		self._spellsBlinkThread:run( function() 
			local i = 0
			local totalItems = #unitDef.upgrades + #self._agency.upgrades
			while true do
				if i == 0 then
					if focus2.upgradeParams.traits.nextReward then
						self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.SORCERY_SPELLS_AVAILABLE)
					end
				elseif i == 30 then
					self.screen:findWidget("spellsTxt"):setText(STRINGS.WIZARD.UI.SORCERY_SPELLS_BTN)
					i = -30
				end
				
				i = i + 1

				coroutine.yield()
			end
		end )
		self._spellsBlinkThread:resume()	

		self.screen:findWidget("spellsBtn").onClick = util.makeDelegate( nil, onClickSpells2, self, focus2, unitDef )
	else
		self.screen:findWidget("spells"):setVisible(false)
	end
end

function stateUpgradeScreen:onDragToAugments( upgrade, item )
	local agentDef = self._agency.unitDefs[ self._selectedIndex ]
	
	if item:getTraits().modSkill == 3 and (unitDef.template == "wizard_agent" or unitDef.template == "wizard_agent_archived") then
		return
	end
	
	oldOnDragToAugments( self, upgrade, item )
end

function stateUpgradeScreen:selectIncognita(...)
	local spellsWidget = self.screen:findWidget("spells")
	if spellsWidget and not spellsWidget.isnull then
		self.screen:findWidget("spells"):setVisible(false)
	end
	if self._spellsBlinkThread then
		self._spellsBlinkThread:stop()
		self._spellsBlinkThread = nil
	end
	
	return oldSelectIncognita( self, ... )
end

function stateUpgradeScreen:onUnload()
	if self._spellsBlinkThread then
		self._spellsBlinkThread:stop()
		self._spellsBlinkThread = nil
	end
	
	self.newSpellScrolls = nil

	oldUnload( self )
end