local util = include("modules/util")
local simdefs = include("sim/simdefs")
local agentdefs = include("sim/unitdefs/agentdefs")
local skilldefs = include( "sim/skilldefs" )

local stateUpgradeScreen = include( "states/state-upgrade-screen" )

local oldOnLoad = stateUpgradeScreen.onLoad
local oldSelectAgent = stateUpgradeScreen.selectAgent
local oldSelectIncognita = stateUpgradeScreen.selectIncognita

--Local helpers

local INACTIVE_BG = { 78/255, 136/255, 136/255,1 }
local SELECTED_BG = { 140/255, 255/255, 255/255,1 }

function stateUpgradeScreen:refreshAgentButton( widget, isSelected )
    if isSelected then
        widget.binder.img:setColor( 1, 1, 1, 1 )
        widget.binder.btn:setColorInactive( unpack(SELECTED_BG) )
    else
        widget.binder.img:setColor( 1, 1, 1, 0.5 )
        widget.binder.btn:setColorInactive( unpack(INACTIVE_BG) )
    end
    widget.binder.btn:updateImageState()
end

function stateUpgradeScreen:changeReserveIndex( delta )
	self._reserveIndex =  self._reserveIndex + delta
	self.screen:findWidget("reserveLeftBtn"):setVisible(self._reserveIndex > 0)
	self.screen:findWidget("reserveRightBtn"):setVisible(#self._agency.unitDefs > self._reserveIndex + 4)
	
	for i, widget in self.screen.binder:forEach( "agent" ) do
        local agentDef = self._agency.unitDefs[ i + self._reserveIndex ]
		if agentDef and agentDef ~= self.loneStart then
			local data = agentdefs[ agentDef.template ]
			widget.binder.img:setImage(data.profile_icon_64x64)
			widget.binder.btn.onClick = util.makeDelegate( nil, stateUpgradeScreen.selectAgent, self, agentDef, i + self._reserveIndex )
			widget.binder.check:setVisible( simdefs.ENABLE_RESERVE and i + self._reserveIndex <= self._agency.equippedAgents )
			self:refreshAgentButton( widget, i + self._reserveIndex == self._selectedIndex )
		end
	end
end

function stateUpgradeScreen:onClickAssign()
	if self._selectedIndex <= self._agency.equippedAgents then
		--Unassign
		self._agency.equippedAgents = self._agency.equippedAgents - 1
		self.screen:findWidget("assignBtn.btn"):setText(STRINGS.A_RES.UI.ASSIGN)
		
		if self._selectedIndex <= self._agency.equippedAgents then
			--Remove and re-add for order
			local unitDef = table.remove(self._agency.unitDefs, self._selectedIndex)
			table.insert(self._agency.unitDefs, self._agency.equippedAgents + 1, unitDef)
			self._selectedIndex = self._agency.equippedAgents + 1
		end
	elseif simdefs.MAX_AGENT_COUNT == 1 then
		--Swap
		local unitDef = table.remove(self._agency.unitDefs, self._selectedIndex)
		table.insert(self._agency.unitDefs, 1, unitDef)
		self._selectedIndex = 1
		
		self.screen:findWidget("assignBtn.btn"):setText(STRINGS.A_RES.UI.UNASSIGN)
		self.screen:findWidget("assignBtn.btn"):setDisabled(true)
		self.screen:findWidget("assignBtn.btn"):setTooltip(STRINGS.A_RES.UI.CANT_UNASSIGN)
	else
		--Assign
		self._agency.equippedAgents = self._agency.equippedAgents + 1
		self.screen:findWidget("assignBtn.btn"):setText(STRINGS.A_RES.UI.UNASSIGN)
		
		if self._selectedIndex > self._agency.equippedAgents then
			--Remove and re-add for order
			local unitDef = table.remove(self._agency.unitDefs, self._selectedIndex)
			table.insert(self._agency.unitDefs, self._agency.equippedAgents, unitDef)
			self._selectedIndex = self._agency.equippedAgents
		end
	end
	
	self:changeReserveIndex( 0 )--Just to update buttons
end

--Interface

function stateUpgradeScreen:onLoad( agency, ... )
	self._reserveIndex = 0
	
	for i = #agency.unitDefs, 1, -1 do
		local unitDef = agency.unitDefs[i]
		local unitData = agentdefs[ unitDef.template ]
		
		if unitData.traits.mod_lonestart then
			self.loneStart = table.remove(agency.unitDefs,i)
		else
			--Skills are missing before mission 1
			if not unitDef.skills then
				unitDef.skills = {}
				
				for i,skillData in ipairs(unitData.skills) do
					local level = 1
					if unitData.startingSkills then
						level = unitData.startingSkills[ skillData ] or 1
					end
					table.insert( unitDef.skills, { skillID = skillData, level = level } )
				end
			end
		end
	end
	if agency.equippedAgents == nil or agency.equippedAgents > #agency.unitDefs or agency.equippedAgents > simdefs.MAX_AGENT_COUNT then
		agency.equippedAgents = math.min(#agency.unitDefs,simdefs.MAX_AGENT_COUNT)
	end
	
	oldOnLoad( self, agency, ... )
	
	self.screen:findWidget("reserveLeftBtn"):setVisible(false)
	self.screen:findWidget("reserveLeftBtn").onClick = util.makeDelegate( nil, stateUpgradeScreen.changeReserveIndex, self, -1)
	self.screen:findWidget("reserveRightBtn"):setVisible(#agency.unitDefs > 4)
	self.screen:findWidget("reserveRightBtn").onClick = util.makeDelegate( nil, stateUpgradeScreen.changeReserveIndex, self, 1)
	self.screen:findWidget("assignBtn.btn").onClick = util.makeDelegate( nil, stateUpgradeScreen.onClickAssign, self)
	self.screen:findWidget("assignBtn.btn"):setText(STRINGS.A_RES.UI.UNASSIGN)
	for i, widget in self.screen.binder:forEach( "agent" ) do
        widget.binder.check:setVisible( simdefs.ENABLE_RESERVE and i <= agency.equippedAgents )
		if #agency.unitDefs > 4 then
			local x0, y0 = widget:getPosition()
			widget:setPosition(x0 - simdefs.RESERVE_UI_OFFSET,y0)
		end
    end
	
    self.screen:findWidget("incognita").binder.check:setVisible( false )
	if self.loneStart then
		table.insert(agency.unitDefs,self.loneStart)
	end
end

function stateUpgradeScreen:selectAgent( unitDef, index, ... )
	oldSelectAgent( self, unitDef, index, ... )
	
	self.screen:findWidget("assignBtn"):setVisible( simdefs.ENABLE_RESERVE )
		
	for i, widget in self.screen.binder:forEach( "agent" ) do
		widget.binder.check:setVisible( simdefs.ENABLE_RESERVE and i + self._reserveIndex <= self._agency.equippedAgents )
		self:refreshAgentButton( widget, i + self._reserveIndex == index )
	end
	
	if index > self._agency.equippedAgents then
		self.screen:findWidget("assignBtn.btn"):setText(STRINGS.A_RES.UI.ASSIGN)
		if self._agency.equippedAgents == simdefs.MAX_AGENT_COUNT and simdefs.MAX_AGENT_COUNT > 1 then
			self.screen:findWidget("assignBtn.btn"):setDisabled(true)
			self.screen:findWidget("assignBtn.btn"):setTooltip(STRINGS.A_RES.UI.CANT_ASSIGN)
		else
			self.screen:findWidget("assignBtn.btn"):setDisabled(false)
			self.screen:findWidget("assignBtn.btn"):setTooltip()
		end
	else
		self.screen:findWidget("assignBtn.btn"):setText(STRINGS.A_RES.UI.UNASSIGN)
		if self._agency.equippedAgents == 1 then
			self.screen:findWidget("assignBtn.btn"):setDisabled(true)
			self.screen:findWidget("assignBtn.btn"):setTooltip(STRINGS.A_RES.UI.CANT_UNASSIGN)
		else
			self.screen:findWidget("assignBtn.btn"):setDisabled(false)
			self.screen:findWidget("assignBtn.btn"):setTooltip()
		end
	end
end

function stateUpgradeScreen:selectIncognita( ... )
	oldSelectIncognita( self, ... )
	
	self.screen:findWidget("assignBtn"):setVisible( false )
end