local util = include("client_util")
local array = include( "modules/array" )
local unitdefs = include("sim/unitdefs")
local skilldefs = include( "sim/skilldefs" )
local guiex = include( "client/guiex" )
local simfactory = include( "sim/simfactory" )
local upgradeScreen = include( "states/state-upgrade-screen" )
local agentdefs = include("sim/unitdefs/agentdefs")
local modalDialog = include( "states/state-modal-dialog" )
local cdefs = include( "client_defs" )

function upgradeScreen:onClickLearnSkillPath( agentIdx, skillpathID )
	self.changes:learnSkillpath( agentIdx, skillpathID )

	local agentDef = nil
	if type(agentIdx) == "string" then
		for _, summonDef in ipairs( self._agency.unitDefs_childs ) do
			if summonDef.template == agentIdx then
				agentDef = summonDef
				break
			end
		end
	else
		agentDef = self._agency.unitDefs[ agentIdx ]
	end

	self:refreshCredits()
	self:refreshSkills( agentDef, agentIdx )
	self:refreshInventory( agentDef, agentIdx )
end

local function canSelectSkillpath( unitDef, skillPathID )
	local data_skillpath = skilldefs.lookupSkillpath( skillPathID )
	local agentData = unitdefs.lookupTemplate( unitDef.template )
	for _, skill in ipairs( unitDef.skills or {} ) do
		for _, skillpath in ipairs( data_skillpath.skills ) do

		end
	end
end

local function onClickRebuySummon( self, summonData, rebuyData, summonDef, summonIdx )
	if self._agency.cash >= ( rebuyData.value or 0 ) then
		local result = modalDialog.showYesNo( util.sformat( STRINGS.UI.UPGRADE_SCREEN_CONFIRM_BUY, rebuyData.name, rebuyData.value), STRINGS.UI.BUY_FROM_MONST3R, nil, STRINGS.UI.BUY_FROM_MONST3R )
		if result == modalDialog.OK then
			MOAIFmodDesigner.playSound(cdefs.SOUND_HUD_SELL)

			local oldCash = self._agency.cash
			self._agency.cash= self._agency.cash - rebuyData.value
			self._cashUpdateThreadMonster =  guiex.createCountUpThread( self.screen:findWidget("creditsTxt"), oldCash, self._agency.cash, 1, STRINGS.UI.UPGRADE_SCREEN_AVAILABLE_CREDITS)
			if not self._agency.upgrades then
				self._agency.upgrades = {}
			end
			table.insert(self._agency.upgrades, { upgradeName = rebuyData.id, upgradeParams = {} } )
			self:selectSummon( summonDef, summonIdx )
		else

		end
	else
		local result = modalDialog.show( STRINGS.UI.REASON.NOT_ENOUGH_CREDITS .. util.sformat(STRINGS.MORE_ARCHIVE_AGENTS.UI.SCREENS.UPGRADE.MONEY_NEEDED, rebuyData.value) )
	end
end

local baseSelectAgent = upgradeScreen.selectAgent
function upgradeScreen:selectAgent( unitDef, index )
	self._selectedParentIndex = nil
	self._selectedSummonIndex = nil
	for i, widget in self.screen.binder:forEach( "skillPath_" ) do
		widget:setVisible( false )
	end


	local data = unitdefs.lookupTemplate( unitDef.template )
	if self._selectedIndex == index then
		self.screen:findWidget("agentTitle"):spoolText(util.toupper(data.name))

		self:clearCurrentlyDisplayedSkill()

		self:refreshInventory( unitDef, index)
		self:refreshSkills( unitDef, index )
	end

	baseSelectAgent( self, unitDef, index )

	if self.screen:findWidget("switch_to_summon") then
		self.screen:findWidget("switch_to_summon"):setVisible( false )
		if data.maa_summons and data.maa_summons[1] then
			local summonDef = nil
			local summonIdx = nil
			for i, summon in ipairs( self._agency.unitDefs_childs ) do
				if summon.template == data.maa_summons[1] then
					summonDef = summon
					summonIdx = i
					break
				end
			end

			if summonDef and summonIdx then
				local summonData = unitdefs.lookupTemplate( summonDef.template )
				--show rebuy option instead?
				local showRebuy = false
				if summonData.upgrade_screen_rebuy then
					assert( type(summonData.upgrade_screen_rebuy == "string") )
					--can we find the rebuy item somewhere in the agency's inventories
					local rebuyData = unitdefs.lookupTemplate( summonData.upgrade_screen_rebuy )
					local itemFound = false
					for _, agentDef in pairs( self._agency.unitDefs ) do
						for _, item in pairs( agentDef.upgrades ) do
							local itemDef
							if type(item) == "string" then
								itemDef = unitdefs.lookupTemplate( item )
							else
								itemDef = unitdefs.lookupTemplate( item.upgradeName )
							end

							if itemDef and itemDef.id == rebuyData.id then
								itemFound = true
								break
							end
						end

						if itemFound then
							break
						end
					end
					if not itemFound then
						if self._agency.upgrades then
							for _, item in pairs( self._agency.upgrades ) do
								local itemDef
								if type(item) == "string" then
									itemDef = unitdefs.lookupTemplate( item )
								else
									itemDef = unitdefs.lookupTemplate( item.upgradeName )
								end

								if itemDef and itemDef.id == rebuyData.id then
									itemFound = true
									break
								end
							end
						end
					end

					if not itemFound then
						showRebuy = true
						self.screen:findWidget("switch_to_summon"):setVisible( true )
						self.screen:findWidget("switch_to_summon").binder.btn:setText( util.toupper(STRINGS.MORE_ARCHIVE_AGENTS.UI.SCREENS.UPGRADE.REBUY_SUMMON .. summonData.name) or "ERROR" )
						self.screen:findWidget("switch_to_summon").binder.btn.onClick = util.makeDelegate( nil, onClickRebuySummon, self, summonData, rebuyData, summonDef, summonIdx )
					end
				end

				if not showRebuy then
					self.screen:findWidget("switch_to_summon"):setVisible( true )
					self.screen:findWidget("switch_to_summon").binder.btn:setText( util.toupper(summonData.name) or "ERROR" )
					self.screen:findWidget("switch_to_summon").binder.btn.onClick = util.makeDelegate( nil, upgradeScreen.selectSummon, self, summonDef, summonIdx )
				end
			end
		end
	end
end

local baseRefreshSkills = upgradeScreen.refreshSkills
function upgradeScreen:refreshSkills( unitDef, agentIdx )
	if self._selectedSummonIndex then
		for _, summonDef in ipairs( self._agency.unitDefs_childs ) do
			if summonDef.template == self._selectedSummonIndex then
				unitDef = summonDef
				agentIdx = unitDef.template
				break
			end
		end
	end

	baseRefreshSkills( self, unitDef, agentIdx )
end

local baseRefreshInventory = upgradeScreen.refreshInventory
function upgradeScreen:refreshInventory( unitDef, agentIdx )
	if self._selectedSummonIndex then
		for _, summonDef in ipairs( self._agency.unitDefs_childs ) do
			if summonDef.template == self._selectedSummonIndex then
				unitDef = summonDef
				break
			end
		end
	end
	baseRefreshInventory( self, unitDef, agentIdx )

	for i, widget in self.screen.binder:forEach( "skillPath_" ) do
		widget:setVisible( false )
	end

	local data = unitdefs.lookupTemplate( unitDef.template )
	--local parentIdx = self._selectedParentIndex
	--local parentDef = self._agency.unitDefs[ parentIdx ]
	--local parentData = unitdefs.lookupTemplate( parentDef.template )

	if data.maa_skillpaths then
		if not unitDef.skillpath then
			for i, widget in self.screen.binder:forEach( "skillPath_" ) do
				if data.maa_skillpaths[i] then
					local data_skillpath = skilldefs.lookupSkillpath( data.maa_skillpaths[i] )
					if data_skillpath then
						local preview_itemDef = unitdefs.lookupTemplate( data_skillpath.preview )
						local itemUnit = simfactory.createUnit( preview_itemDef )
						guiex.updateButtonFromItem( self.screen, nil, widget, itemUnit )
						widget:setVisible( true )
						widget.binder.btn.onClick = util.makeDelegate( nil, self.onClickLearnSkillPath, self, unitDef.template, data.maa_skillpaths[i])
					end
				end
			end

			for i, widget in self.screen.binder:forEach( "aug_" ) do
				widget:setVisible( false )
			end
		end
	end
end

function upgradeScreen:selectSummon( unitDef, index )
	self.screen:findWidget("programPnl"):setVisible(false)
	self.screen:findWidget("agentPnl"):setVisible(true)

	local data = unitdefs.lookupTemplate( unitDef.template )
	self.screen:findWidget("agentTitle"):spoolText(util.toupper(data.name))

	self:clearCurrentlyDisplayedSkill()
	self._selectedParentIndex = self._selectedIndex
	self._selectedSummonIndex = unitDef.template

	local data = unitdefs.lookupTemplate( unitDef.template )
	local parentIdx = self._selectedParentIndex
	local parentDef = self._agency.unitDefs[ parentIdx ]
	local parentData = unitdefs.lookupTemplate( parentDef.template )
	self.screen:findWidget("switch_to_summon"):setVisible( true )
	self.screen:findWidget("switch_to_summon").binder.btn:setText( util.toupper(parentData.name) or "ERROR" )
	self.screen:findWidget("switch_to_summon").binder.btn.onClick = util.makeDelegate( nil, upgradeScreen.selectAgent, self, parentDef, parentIdx )

	self:refreshInventory( unitDef, nil )
	self:refreshSkills( unitDef, unitDef.template )
end

local baseInit = upgradeScreen.skillChanges.init
function upgradeScreen.skillChanges:init( upScreen )
	baseInit( self, upScreen )

	self.unitDefs_childs = self.agency.unitDefs_childs
end

function upgradeScreen.skillChanges:learnSkillpath( agentIdx, skillpathID )
	local agentDef = nil
	if type(agentIdx) == "string" then
		for _, summonDef in ipairs( self.unitDefs_childs ) do
			if summonDef.template == agentIdx then
				agentDef = summonDef
				break
			end
		end
	else
		agentDef = self.unitDefs[ agentIdx ]
	end

	if agentDef then
		local data_skillpath = skilldefs.lookupSkillpath( skillpathID )
		local data_agent = unitdefs.lookupTemplate( agentDef.template )

		--check if any skills from other paths must be removed first
		local all_path_skills = {}
		for _, skillpath in ipairs( data_agent.maa_skillpaths or {} ) do
			for _, path_skill in ipairs( skilldefs.lookupSkillpath( skillpath ).skills ) do
				table.insert( all_path_skills, path_skill )
			end
		end
		for i=#agentDef.skills, 1, -1 do
			if array.find( all_path_skills, agentDef.skills[i].skillID ) then
				table.remove( agentDef.skills, i )
			end
		end

		--add the skills from the skillpath
		for _, path_skill in ipairs( data_skillpath.skills ) do
			table.insert( agentDef.skills, { skillID = path_skill, level = 1 } )
		end
	end
end

local baseCountChanges = upgradeScreen.skillChanges.countChanges
function upgradeScreen.skillChanges:countChanges( agentIdx, skillIdx )
	if type(agentIdx) == "string" then
		local agentDef = nil
		for _, summonDef in ipairs( self.unitDefs_childs ) do
			if summonDef.template == agentIdx then
				agentDef = summonDef
				break
			end
		end
		if agentDef then
			local skill = agentDef.skills[ skillIdx ]
			if skill and skill.skillChange then
				return skill.skillChange
			end
		end

		return 0
	end

	return baseCountChanges( self, agentIdx, skillIdx )
end

local baseLearnSkill = upgradeScreen.skillChanges.learnSkill
function upgradeScreen.skillChanges:learnSkill( agentIdx, skillIdx )
	if type(agentIdx) == "string" then
		local agentDef = nil
		for _, summonDef in ipairs( self.unitDefs_childs ) do
			if summonDef.template == agentIdx then
				agentDef = summonDef
				break
			end
		end
		if agentDef then
			local skill = agentDef.skills[ skillIdx ]
			local cost = self:getSkillCost( agentDef, skill, skill.level + 1 )

			-- Subtract cost, increase skill.
			if self.agency.cash >= cost then
				skill.skillChange = (skill.skillChange or 0) + 1
				skill.level = skill.level + 1
				self.agency.cash = self.agency.cash - cost

				--check if the bought skill should lock the paths
				local data_agent = unitdefs.lookupTemplate( agentDef.template )
				if data_agent.maa_skillpaths then
					for _, skillpath in ipairs( data_agent.maa_skillpaths ) do
						local data_skillpath = skilldefs.lookupSkillpath( skillpath )
						if array.find( data_skillpath.skills, skill.skillID ) then
							agentDef.skillpath = skillpath
						end
					end
				end

				return true
			else
				return false
			end
		end

		return false
	end

	return baseLearnSkill( self, agentIdx, skillIdx )
end

local baseUndoSkill = upgradeScreen.skillChanges.undoSkill
function upgradeScreen.skillChanges:undoSkill( agentIdx, skillIdx )
	if type(agentIdx) == "string" then
		local agentDef = nil
		for _, summonDef in ipairs( self.unitDefs_childs ) do
			if summonDef.template == agentIdx then
				agentDef = summonDef
				break
			end
		end
		if agentDef then
			local skill = agentDef.skills[ skillIdx ]
			local cost = self:getSkillCost( agentDef, skill, skill.level )

			-- Return cost, decrease skill
			if skill.skillChange and skill.skillChange > 0 then
				skill.skillChange = skill.skillChange - 1
				skill.level = skill.level - 1
				self.agency.cash = self.agency.cash + cost

				--check if the sold skill enables skillpath selection
				if agentDef.skillpath then
					local unlock = true
					local data_skillpath = skilldefs.lookupSkillpath( agentDef.skillpath )
					for _, agent_skill in ipairs( agentDef.skills ) do
						if array.find( data_skillpath.skills, agent_skill.skillID ) then
							if agent_skill.level > 1 then
								unlock = false
								break
							end
						end
					end

					if unlock then
						agentDef.skillpath = nil
					end
				end

				return true
			else
				return false
			end
		end

		return false
	end

	return baseUndoSkill( self, agentIdx, skillIdx )
end

local baseUndoAll = upgradeScreen.skillChanges.undoAll
function upgradeScreen.skillChanges:undoAll()
	baseUndoAll( self )

	for agentIdx, agentDef in pairs( self.unitDefs_childs or {} ) do
		for skillIdx, skill in pairs( agentDef.skills or {} ) do
			while skill.skillChange and skill.skillChange > 0 do
				local cost = self:getSkillCost( agentDef, skill, skill.level )

				skill.skillChange = skill.skillChange - 1
				skill.level = skill.level - 1
				self.agency.cash = self.agency.cash + cost
			end
		end
	end
end

--2 reasons to do this: 1: same fix as in agent reserve for same case, 2: drones skills aren't set if drone isn't deployed
local baseOnLoad = upgradeScreen.onLoad
function upgradeScreen:onLoad( agency, ... )
	--add skills for summons
	if agency.unitDefs_childs then
		for i = #agency.unitDefs_childs, 1, -1 do
			local unitDef = agency.unitDefs_childs[i]
			local unitData = agentdefs[ unitDef.template ]

			if not unitDef.skills then
				unitDef.skills = {}

				for i, skillData in ipairs(unitData.skills) do
					local level = 1
					if unitData.startingSkills then
						level = unitData.startingSkills[ skillData ] or 1
					end
					table.insert( unitDef.skills, { skillID = skillData, level = level } )
				end
			end
		end
	end
	baseOnLoad( self, agency, ... )
end

local baseOnUnload = upgradeScreen.onUnload
function upgradeScreen:onUnload()
	--check if any pathskills must be taken away, since the path wasn't locked
	for _, summonDef in ipairs( self._agency.unitDefs_childs or {} ) do
		local data_summon = unitdefs.lookupTemplate( summonDef.template )
		if data_summon.maa_skillpaths and not summonDef.skillpath then
			for _, skillpath in ipairs( data_summon.maa_skillpaths ) do
				if summonDef.skills then
					local data_skillpath = skilldefs.lookupSkillpath( skillpath )
					for i = #summonDef.skills, 1, -1 do
						if array.find( data_skillpath.skills, summonDef.skills[i].skillID ) then
							table.remove( summonDef.skills, i )
						end
					end
				end
			end
		end
	end

	baseOnUnload( self )
end
