local util = include( "client_util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local mui_defs = include( "mui/mui_defs")
local array = include( "modules/array" )
local simquery = include( "sim/simquery" )
local targeting = include( "hud/targeting" )
local world_hud = include( "hud/hud-inworld" )
local guiex = include( "guiex" )

-- a targeter that returns unique icons for children of all targeted units
targeting.multiUnitTarget = class(targeting.unitTarget)

function targeting.multiUnitTarget:startTargeting( cellTargets )
	local agent_panel = include( "hud/agent_panel" )
	local sim = self._game.simCore
	for i, unit in ipairs( self._units ) do
		local cell = sim:getCell( unit:getLocation() )

		-- If targetting self in item-target mode, then the option shoudl go into the popup menu (eg. STIM)
		if unit == self._abilityUser and self._game.hud._state == self._game.hud.STATE_ITEM_TARGET then
			local realAgentPanel = self._game.hud._agent_panel
			table.insert(realAgentPanel._popUps,{ ability = self._ability, abilityOwner = self._abilityOwner, abilityUser = self._abilityUser, unitID = unit:getID() })
		else
			local wx, wy, wz = self._game:cellToWorld( cell.x, cell.y )
			wx, wy, wz = cellTargets:findLocation( wx, wy )

			wz = 0
			if unit:getTraits().breakIceOffset then
				wz = unit:getTraits().breakIceOffset
			end

			-- Check special 'credits item'
			local credits = simquery.calculateCashOnHand( self._game.simCore, unit )
			local verb = STRINGS.APEX.ITEMS.EYE.TOOLTIPS.CREDITS_NONGUARD
			if credits == 0 then
	            credits = unit:getTraits().credits or 0
	        end
	        local bonus = 0
	        if unit:getTraits().isGuard then
	        	verb = STRINGS.APEX.ITEMS.EYE.TOOLTIPS.CREDITS_GUARD
				bonus = math.floor( credits * (self._abilityUser:getTraits().stealBonus or 0))
				credits = credits + bonus
			end
			if credits > 0 then
				local widget = self._game.hud._world_hud:createWidget( world_hud.HUD, "Target", {  worldx = wx, worldy = wy, worldz = wz, layoutID = unit:getID() } )

				local tt = util.sformat( STRINGS.APEX.ITEMS.EYE.TOOLTIPS.CREDITS_ITEM, credits, verb )
				if unit:getTraits().isGuard then
					if bonus and bonus > 0 then
						tt = tt..util.sformat( STRINGS.APEX.ITEMS.EYE.TOOLTIPS.CREDITS_BONUS, bonus )
					end
				end
    			widget.binder.btn:setImage( cdefs.CREDITS_ICON )					
				widget.binder.btn:setTooltip( 
					function()
				        local tooltip = util.tooltip( self._game.hud._screen, self._game )
				        local section = tooltip:addSection()
				        section:addLine( "<ttheader>"..util.toupper(STRINGS.ITEMS.CREDCHIP).."</>" )
				        section:addDesc( tt.."." )
		                return tooltip
		            end )
				widget.binder.btn.onClick = nil
	            widget.binder.img:setColor(0.8,0.5,0.8,1)
				widget.binder.btn:setColor(0.8,0.5,0.8,1)
			end

			-- check other items
			for i, childUnit in pairs(unit:getChildren()) do
				--log:write("ENTERING QUALIFIER")
				if self._ability:targetingQualifier(childUnit) then
					local widget = self._game.hud._world_hud:createWidget( world_hud.HUD, "Target", {  worldx = wx, worldy = wy, worldz = wz, layoutID = unit:getID() } )
					--local guiex = include( "client/guiex" )
					--guiex.updateButtonFromItem( self._game.hud._screen, self._game, widget, childUnit, unit )

					local itemData = childUnit:getUnitData()

					widget._item = childUnit

					widget:setVisible( true )
					widget:setAlias( childUnit:getName() )
					if widget.binder.btn:getSize() > 36 then
				    	widget.binder.btn:setImage( itemData.profile_icon_100 )
				    else
				    	widget.binder.btn:setImage( itemData.profile_icon )
				    end
				    widget.binder.btn.onClick = nil -- Caller-defined.
				    if itemData.onTooltip then
				        widget.binder.btn:setTooltip(
				            function()
						        local tooltip = util.tooltip( self._game.hud._screen, self._game )
						        local section = tooltip:addSection()
						        itemData.onTooltip( section, childUnit, unit, self._game )
				                return tooltip
				            end )

				    else
				        widget.binder.btn:setTooltip( nil )
					end

					widget.binder.img:setColor(0.8,0.5,0.8,1)
					widget.binder.btn:setColor(0.8,0.5,0.8,1)
				end
			end
		end
	end
end


function targeting.multiUnitTarget:endTargeting( hud )
	hud._world_hud:destroyWidgets( world_hud.HUD )
end