local util = include( "client_util" )
local cdefs = include( "client_defs" )
local array = include( "modules/array" )
local mui_defs = include( "mui/mui_defs")
local world_hud = include( "hud/hud-inworld" )
local simquery = include( "sim/simquery" )
local mathutil = include( "modules/mathutil" )
local geo_util = include( "geo_util" )
local targeting = include( "hud/targeting" )

-----------------------------------------
--Copy pasta of vanilla local base class
-----------------------------------------
local areaTargetBase = class()

function areaTargetBase:init( game, range, sim )
	self._game = game
	self.sim = sim
	self._hiliteClr = { 0.5, 0.5, 0.5, 0.5 }
	self.mx = nil
	self.my = nil
	self.range = range
    self.unitTargets = {}
end

function areaTargetBase:setUnitPredicate( unitTargetFn )
    self.unitTargetFn = unitTargetFn
end

function areaTargetBase:setHiliteColor( clr )
    self._hiliteClr = clr
end

function areaTargetBase:hasTargets()
	return true
end

function areaTargetBase:setTargetCell( cellx, celly )
    self.mx, self.my = cellx, celly

	if self:isValidTargetLoc( cellx, celly ) then
		self.cells = simquery.rasterCircle( self.sim, cellx, celly, self.range )
    else
        self.cells = nil
    end

     if self.unitTargetFn then
        local count = #self.unitTargets
        if self.cells then
		    for i = 1, #self.cells, 2 do
                local cell = self.sim:getCell( self.cells[i], self.cells[i+1] )
                if cell then
                    for j, cellUnit in ipairs(cell.units) do
                        if self.unitTargetFn( cellUnit ) then
                            -- Hey: does this already exist in the list?
                            local idx = array.find( self.unitTargets, cellUnit:getID() )
                            if idx then
                                table.insert( self.unitTargets, table.remove( self.unitTargets, idx ))
                                count = count - 1
                            else
                                table.insert( self.unitTargets, cellUnit:getID() )
                                self._game.boardRig:getUnitRig( cellUnit:getID() ):getProp():setRenderFilter( cdefs.RENDER_FILTERS["focus_target"] )
                            end
                        end
                    end
                end
            end
		end

        while count > 0 do
            local unitRig = self._game.boardRig:getUnitRig( table.remove( self.unitTargets, 1 ))
            unitRig:refreshRenderFilter()
            count = count - 1
        end
    end
end

function areaTargetBase:endTargeting()
    self:setTargetCell( nil, nil )
end


function areaTargetBase:onInputEvent( event )
	if event.eventType == mui_defs.EVENT_MouseDown and event.button == mui_defs.MB_Left then
        self.ux, self.uy = event.wx, event.wy

	elseif event.eventType == mui_defs.EVENT_MouseUp and event.button == mui_defs.MB_Left then
        if mathutil.dist2d( self.ux or event.wx, self.uy or event.wy, event.wx, event.wy ) >= 16 then
            return nil
        else
		    local x, y = self._game:wndToSubCell( event.wx, event.wy )
		    local cellx, celly = math.floor(x), math.floor(y)
		    if self:isValidTargetLoc(cellx, celly) then
			    return {cellx, celly}
		    else
			    return nil
		    end
        end
	elseif event.eventType == mui_defs.EVENT_MouseMove then
		local x, y = self._game:wndToSubCell( event.wx, event.wy )
        x, y = math.floor(x), math.floor(y)
        if x ~= self.mx or y ~= self.my then
            self.tooltip = self:generateTooltip( x, y )
            self:setTargetCell( x, y )
        end
    end
end

function areaTargetBase:onDraw()
	MOAIGfxDevice.setPenColor(unpack(self._hiliteClr))
	if self.cells then
		for i = 1, #self.cells, 2 do
            local x, y = self.cells[i], self.cells[i+1]
			local x0, y0 = self._game:cellToWorld( x + 0.4, y + 0.4 )
			local x1, y1 = self._game:cellToWorld( x - 0.4, y - 0.4 )
			MOAIDraw.fillRect( x0, y0, x1, y1 )
		end
        return true
	end

    return false
end

function areaTargetBase:generateTooltip( x, y )
    return nil
end

function areaTargetBase:getTooltip( x, y )
    return self.tooltip
end

function areaTargetBase:getDefaultTarget()
	return nil
end

-----------------------------------------

local maa_areaCellTarget = class( areaTargetBase )

function maa_areaCellTarget:init( game, range, sim, cells )
  areaTargetBase.init( self, game, range, sim )
	self.targetcells = cells or {}
end

function maa_areaCellTarget:startTargeting( cellTargets )
end

function maa_areaCellTarget:endTargeting()
  areaTargetBase.endTargeting( self )
	if self.prop then
		self._game.boardRig:getLayer("ceiling"):removeProp(self.prop)
		self.prop = nil
	end
end

function maa_areaCellTarget:isValidTargetLoc(x, y)
	if not x or not y then
		return false
	end

	local find_cell = false
	for _, cell in ipairs( self.targetcells ) do
		if cell.x == x and cell.y == y then
			find_cell = true
		end
	end

  if not find_cell then
    return false
  end
	return true
end

-- function maa_areaCellTarget:onInputEvent( event )
-- 	local x, y = self.mx, self.my
-- 	local result = areaTargetBase.onInputEvent(self, event)
-- 	if result then
-- 		return result
-- 	end
-- 	if self.mx ~= x or self.my ~= y then
-- 		if self:isValidTargetLoc(self.mx, self.my) then
-- 			if not self.prop then
-- 				self.prop = MOAIProp.new()
-- 			end
-- 		    local x0, y0 = self._game:cellToWorld(self.unit:getLocation() )
-- 		    local x1, y1 = self._game:cellToWorld(self.mx, self.my)
--
-- 			local msh = geo_util.generateArcMesh(self._game.boardRig, x0, y0, 0, x1, y1, 0, 30, 10)
-- 			self.prop:setDeck(msh)
-- 			self.prop:setLoc(x0, y0)
-- 			self._game.boardRig:getLayer("ceiling"):insertProp(self.prop)
-- 		else
-- 			if self.prop then
-- 				self._game.boardRig:getLayer("ceiling"):removeProp(self.prop)
-- 				self.prop = nil
-- 			end
-- 		end
-- 	end
-- end

function maa_areaCellTarget:generateTooltip( x, y )
    if not self:isValidTargetLoc(x, y) then
        return "<c:ff0000>" .. STRINGS.UI.REASON.BAD_TARGET_RANGE .. "</>"
    end
end


-- function throwTarget:onDraw()
-- 	MOAIGfxDevice.setPenColor(unpack(self._hiliteClr))
-- 	if self.mx and self.my and self:isValidTargetLoc(self.mx, self.my) then
-- 		local cells = simquery.rasterCircle( self.sim, self.mx, self.my, self.range )
-- 		for i = 1, #cells, 2 do
--             local x, y = cells[i], cells[i+1]
-- 			local raycastX, raycastY = self.sim:getLOS():raycast(self.mx, self.my, x, y)
-- 			if ((raycastX == x and raycastY == y) or self._ignoreLOS ) and self.sim:canPlayerSee(self.unit:getPlayerOwner(), x, y) then
-- 				local x0, y0 = self._game:cellToWorld( x + 0.4, y + 0.4 )
-- 				local x1, y1 = self._game:cellToWorld( x - 0.4, y - 0.4 )
-- 				MOAIDraw.fillRect( x0, y0, x1, y1 )
-- 			end
-- 		end
--         return true
-- 	end
--
--     return false
-- end

targeting.maa_areaCellTarget = maa_areaCellTarget
