local unitdefs = include("sim/unitdefs")
local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local rig_util = include( "gameplay/rig_util" )
local inventory = include("sim/inventory")
local simfactory = include( "sim/simfactory" )


local TURNS_TO_JUMP = 2

local function getCurrentGameState()
    local states = statemgr.getStates()
    for i = 1, #states do
        if states[i].simCore and states[i].levelData then
            return states[i] -- If it acts like a duck and quacks like a duck...
        end
    end
end

local function timeHopAug(unit)
	local res = false
    for i,childAug in ipairs(unit:getChildren( )) do
		if childAug:getTraits().isTimeTravelAug then
			unit:getTraits().timeHopAug = childAug
			res = childAug
		end
	end
	return res
end


local function warpOutEffects( sim, unit )
	local cell = sim:getCell( unit:getLocation() )
	sim:dispatchEvent( simdefs.EV_CAM_PAN, { cell.x, cell.y } )	
	
	unit:getTraits().pinningOverride = true
	
	unit:getTraits().isFutureUnit = nil
	unit:getTraits().isPastUnit = nil
	unit:getTraits().isHoppingBackUnit = true
	
	if not unit:isKO() and not unit:isDead() then
		sim:dispatchEvent( simdefs.EV_UNIT_PICKUP, { unitID = unit:getID() } )
	end
		
	sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = unit:getID(), noSightingFx=true } )	
	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )KLEIRenderScene:pulseUIFuzz( .5 )
	
	
	unit:getTraits().customDefinedRenderFilter = { shader=KLEIAnim.SHADER_FOW, r=112/255, g=97/255, b=179/255, a=0.333,lum=1 }

	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit, fx = "emp" } )	
	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )

end

local function clearTimeHops( )
	local user = savefiles.getCurrentGame()
	local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
    campaign.timeHops = nil
	--log:write("clearing timeHop")
end

local function findUnitByTag( sim, tag )
	for _, unit in pairs( sim:getAllUnits() ) do
		if unit:hasTag( tag ) then
			return unit
		end
	end
	return false
end

local timeBranch =
	{

		name = STRINGS.TIMEHOP.ABILITIES.TIMEHOP,
		--hotkey = "abilityReaction",
		profile_icon = "gui/icons/icon-hopper_rewind.png",

		getProfileIcon =  function( self, sim, unit )
			local currentTurn = sim:getTurnCount()
	--		if not canUseAbility( self, sim, userUnit )  then
	--			return "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png"
	--		end
			return self.profile_icon
		end,

		alwaysShow = true,
		--HUDpriority = 1,

		getName = function( self, sim, unit )
			local txt = STRINGS.TIMEHOP.ABILITIES.TIMEHOP
			return txt
		end,
		
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			return abilityutil.overwatch_tooltip( hud, self, sim, abilityOwner, STRINGS.TIMEHOP.ABILITIES.TIMEHOP_TOOLTIP )
		end,
		
		confirmAbility = function( self, sim, ownerUnit )
            return STRINGS.TIMEHOP.ABILITIES.TIMEHOP_WARNING
        end,
		
		canUseAbility = function( self, sim, userUnit ) 
			local currentTurn = sim:getTurnCount()
			
			if userUnit:isKO() or userUnit:isDead() then
				return false, STRINGS.TIMEHOP.ABILITIES.UNCONCIOUS
			end

			if userUnit:getTraits().leavesAtEndOfMission == true then
				return false, STRINGS.TIMEHOP.ABILITIES.UNSTABLE -- HES THE CLONE SHOOT HIM
			end
			
			local augUnit = timeHopAug(userUnit)
			if augUnit:getTraits().cooldown > 0 then
				return false, STRINGS.TIMEHOP.ABILITIES.UNSTABLE
			end
			if augUnit:getTraits().charges < 1 then
				return false, STRINGS.TIMEHOP.ABILITIES.CHARGES
			end
			
			if (currentTurn-4) < 0 then
				return false, STRINGS.TIMEHOP.ABILITIES.TIMELINE -- you should never see this but just in case
			end
			
			return true
		end,
		
		executeAbility = function( self, sim, unit, userUnit, x0, y0, dir )
			
			local augUnit = timeHopAug(userUnit)
			augUnit:getTraits().cooldown = augUnit:getTraits().cooldownMax 
			augUnit:getTraits().charges = math.max(0, augUnit:getTraits().charges - 1)
			
		--	local mainUnit = userUnit
			local unitData = unitdefs.lookupTemplate( "schysm" )
			local copyUnit = simfactory.createUnit( unitData, sim )
			
			local skillTable = {}
			for i,skill in ipairs(userUnit:getSkills()) do
				skillTable[skill:getID()] = skill:getCurrentLevel()
			end
			
			for i, skill in ipairs(userUnit:getSkills()) do
				local targetLevel = skillTable[skill:getID()]
				if targetLevel and targetLevel > skill:getCurrentLevel() then
					for j = 1, targetLevel - skill:getCurrentLevel() do
						skill:levelUp(sim, copyUnit)
					end
				elseif targetLevel and targetLevel < skill:getCurrentLevel() then
					for j = 1, skill:getCurrentLevel() - targetLevel do
						skill:levelDown(sim, copyUnit)
					end
				end
			end
			
			copyUnit:setPlayerOwner(sim:getPC() )
			
			local cell = sim:getCell(userUnit:getLocation())
			local facing = userUnit:getFacing()
			sim:spawnUnit( copyUnit )		
			sim:warpUnit( copyUnit, cell, facing )
			
			sim:dispatchEvent( simdefs.EV_UNIT_PICKUP, { unitID = userUnit:getID() } )
			sim:dispatchEvent( simdefs.EV_UNIT_PICKUP, { unitID = copyUnit:getID() } )
			
			copyUnit:getTraits().customDefinedRenderFilter = { shader=KLEIAnim.SHADER_FOW, r=100/255, g=200/255, b=200/255, a=0.6,lum=1 }
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = copyUnit, fx = "emp" } )	
			
			userUnit:getTraits().customDefinedRenderFilter = { shader=KLEIAnim.SHADER_FOW, r=200/255, g=100/255, b=100/255, a=.6,lum=1 }
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )	
			
			userUnit:getUnitData().kanim = "kanim_schysm_R"
			copyUnit:getUnitData().kanim = "kanim_schysm_B"
			userUnit:getTraits().replaceKanim ="kanim_schysm_R"
			copyUnit:getTraits().replaceKanim ="kanim_schysm_B"
			
			userUnit:getTraits().isPastUnit = true
			copyUnit:getTraits().isFutureUnit = true
			
			copyUnit:getTraits().leavesAtEndOfMission = true
			
			for j,child in ipairs(copyUnit:getChildren()) do
				inventory.trashItem( sim, copyUnit, child )
			end
			
			for i,child in ipairs(userUnit:getChildren()) do
				local thisTemplate = child._unitData.id
				local newChild = inventory.giftUnit( sim, copyUnit, thisTemplate, true )
				if newChild then
					if child:getTraits().installed then
						local abilityDef = newChild:ownsAbility( "installAugment" )
						if abilityDef and abilityDef:canUseAbility( sim, newChild, copyUnit ) then 
							 copyUnit:doAugmentUpgrade( newChild )
						end 
					end
					if child:getTraits().equipped then
						local abilityDef = newChild:ownsAbility( "equippable" )
						if abilityDef and abilityDef:canUseAbility( sim, newChild, copyUnit ) then 
							abilityDef:executeAbility( sim, newChild, copyUnit )
						end 
					end
				
					for trait,value in pairs(child:getTraits()) do
						newChild:getTraits()[trait] = value
					end
					if child._tags then
						for tag,value in pairs(child._tags) do
								newChild:addTag( value )
						end
					end
					
					newChild:getTraits().isTimeDupUnit = true
					newChild:getTraits().timedupeOriginalItem = child
					child:getTraits().timedupeCopiedItem = newChild
				
				end
			end
			
			self._active = true
			
			
		end,


--------------

	onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_MAP_EVENT, self )
			self.abilityOwner = unit
			self._copyUnit = false
			self._active = false
		end,
	onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_MAP_EVENT, self )
		end,


	onTrigger = function ( self, sim, evType, evData )  -- need to get this working
			
			local abilityOwner = self.abilityOwner
			local player = abilityOwner:getPlayerOwner()
			
			if evType == simdefs.TRG_START_TURN  and sim:getCurrentPlayer() == player then --  evType == simdefs.TRG_END_TURN
				log:write("cleanup")
				self._active = false
				
				for j,testUnit in ipairs( sim:getAllUnits() ) do
					if testUnit:getTraits().isTimeDupUnit then
						inventory.trashItem( sim, testUnit:getUnitOwner(), testUnit )
					end
				end
				for j,testUnit in ipairs( sim:getAllUnits() ) do
					if testUnit:getTraits().isPastUnit then
						sim:warpUnit( testUnit, nil )	
						sim:despawnUnit( testUnit )
					end
				end
				for j,testUnit in ipairs( sim:getAllUnits() ) do
					if testUnit:getTraits().isFutureUnit then
						testUnit:getTraits().customDefinedRenderFilter = nil
						testUnit:getTraits().isFutureUnit = nil
					end
				end
				
				
			end
			
			if evType == simdefs.TRG_MAP_EVENT and evData.event == simdefs.MAP_EVENTS.TELEPORT then

				local futureUnitEscaped = false
				local pastUnitEscaped = false
				local futureUnit = false
				local pastUnit = false
				for i,escaperUnit in pairs(evData.units) do 
						
					--- confiscate dup units and check if they are a time traveller
					
					for j,child in ipairs(escaperUnit:getChildren()) do
						if child:getTraits().isTimeDupUnit then
							inventory.trashItem( sim, escaperUnit, child )
						end
					end

					if escaperUnit:getTraits().isFutureUnit == true then
						futureUnitEscaped = true
						futureUnit = escaperUnit
					end
					if escaperUnit:getTraits().isPastUnit == true then
						pastUnitEscaped = true
						pastUnit = escaperUnit
					end
					
				end
				
				if futureUnitEscaped then
					self._active = false
					if pastUnit and not pastUnitEscaped then
						inventory.dropAll(sim, pastUnit)
						sim:warpUnit( pastUnit, nil )	
						sim:despawnUnit( pastUnit )
						futureUnit:getTraits().isFutureUnit = false
					end
					for j,child in ipairs(escaperUnit:getChildren()) do
						if child:getTraits().isTimeDupUnit then
							inventory.trashItem( sim, escaperUnit, child )
						end
					end
				end
			end
			
		end,
	}
	
return timeBranch

