local unitdefs = include("sim/unitdefs")
local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local rig_util = include( "gameplay/rig_util" )
local inventory = include("sim/inventory")
local simfactory = include( "sim/simfactory" )


local TURNS_TO_JUMP = 2

local function getCurrentGameState()
    local states = statemgr.getStates()
    for i = 1, #states do
        if states[i].simCore and states[i].levelData then
            return states[i] -- If it acts like a duck and quacks like a duck...
        end
    end
end

local function timeHopAug(unit)
	local res = false
    for i,childAug in ipairs(unit:getChildren( )) do
		if childAug:getTraits().isTimeTravelAug then
			unit:getTraits().timeHopAug = childAug
			res = childAug
		end
	end
	return res
end


local function warpOutEffects( sim, unit )
	local cell = sim:getCell( unit:getLocation() )
	sim:dispatchEvent( simdefs.EV_CAM_PAN, { cell.x, cell.y } )	
	
	unit:getTraits().pinningOverride = true
	
	unit:getTraits().isFutureUnit = nil
	unit:getTraits().isPastUnit = nil
	unit:getTraits().isHoppingBackUnit = true
	
	if not unit:isKO() and not unit:isDead() then
		sim:dispatchEvent( simdefs.EV_UNIT_PICKUP, { unitID = unit:getID() } )
	end
		
	sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = unit:getID(), noSightingFx=true } )	
	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )KLEIRenderScene:pulseUIFuzz( .5 )
	
	
	unit:getTraits().customDefinedRenderFilter = { shader=KLEIAnim.SHADER_FOW, r=112/255, g=97/255, b=179/255, a=0.333,lum=1 }

	sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit, fx = "emp" } )	
	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )

end

local function clearTimeHops( )
	local user = savefiles.getCurrentGame()
	local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
    campaign.timeHops = nil
	--log:write("clearing timeHop")
end

local function findUnitByTag( sim, tag )
	for _, unit in pairs( sim:getAllUnits() ) do
		if unit:hasTag( tag ) then
			return unit
		end
	end
	return false
end

local timeHop =
	{

		name = STRINGS.TIMEHOP.ABILITIES.TIMEHOP,
		--hotkey = "abilityReaction",
		profile_icon = "gui/icons/icon-hopper_rewind.png",

		getProfileIcon =  function( self, sim, unit )
			local currentTurn = sim:getTurnCount()
	--		if not canUseAbility( self, sim, userUnit )  then
	--			return "gui/icons/action_icons/Action_icon_Small/actionicon_noentry.png"
	--		end
			return self.profile_icon
		end,

		alwaysShow = true,
		--HUDpriority = 1,

		getName = function( self, sim, unit )
			local txt = STRINGS.TIMEHOP.ABILITIES.TIMEHOP
			return txt
		end,
		
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			return abilityutil.overwatch_tooltip( hud, self, sim, abilityOwner, STRINGS.TIMEHOP.ABILITIES.TIMEHOP_TOOLTIP )
		end,
		
		confirmAbility = function( self, sim, ownerUnit )
            return STRINGS.TIMEHOP.ABILITIES.TIMEHOP_WARNING
        end,
		
		canUseAbility = function( self, sim, userUnit ) 
			local currentTurn = sim:getTurnCount()
			
			if userUnit:isKO() or userUnit:isDead() then
				return false, STRINGS.TIMEHOP.ABILITIES.UNCONCIOUS
			end

			if userUnit:getTraits().leavesAtEndOfMission == true then
				return false, STRINGS.TIMEHOP.ABILITIES.UNSTABLE -- HES THE CLONE SHOOT HIM
			end
			
			local augUnit = timeHopAug(userUnit)
			if augUnit:getTraits().cooldown > 0 then
				return false, STRINGS.TIMEHOP.ABILITIES.COOLDOWN
			end
			if augUnit:getTraits().charges < 1 then
				return false, STRINGS.TIMEHOP.ABILITIES.CHARGES
			end
			
			if (currentTurn-4) < 0 then
				return false, STRINGS.TIMEHOP.ABILITIES.TIMELINE -- you should never see this but just in case
			end
			
			return true
		end,
		
		
		executeAbility = function( self, sim, unit, userUnit, x0, y0, dir )
		
			local user = savefiles.getCurrentGame()
			local game = getCurrentGameState()
			local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]

			local timeSegment = 1
			local currentTurn = 1
			--log:write("initTimeHop")
			if campaign then
			--------- define the time segemt we're targeting ---------
				
				currentTurn = sim:getTurnCount()
				targetTurn =  math.max(currentTurn - TURNS_TO_JUMP,1) -- can only go back so far
				
				local numTurnsTravelled = TURNS_TO_JUMP
				
			--	timeSegment	= campaign.currentDay .. "_" .. campaign.hours ..  "_" .. campaign.seed .. "_" .. currentTurn
				timeSegment	= campaign.currentDay .. "_" .. campaign.hours ..  "_" .. campaign.seed

				--log:write("leaving "..timeSegment.."_"..currentTurn)
			--------- create a timestamped Snapshot for us to look for in rewinds/replays ---------

				local agentID = userUnit:getID()
				if not campaign.timeHops or not type(campaign.timeHops) == "table" then
					campaign.timeHops = {}
				end
				if campaign.timeHops and not campaign.timeHops[timeSegment .. "_" .. currentTurn] then
					campaign.timeHops[timeSegment .. "_" .. currentTurn] = {}
				end
				local timeStamp = campaign.timeHops[timeSegment .. "_" .. currentTurn]
				
				timeStamp.agentID = agentID
				timeStamp.agentMP = userUnit:getTraits().mp
				timeStamp.agentAP = userUnit:getAP( )
				timeStamp.sneaking = userUnit:getTraits().sneaking
				timeStamp.augmentMaxSize = userUnit:getTraits().augmentMaxSize
				timeStamp.exists = true
				timeStamp.originTurn = currentTurn
				timeStamp.agentPosX,timeStamp.agentPosY = userUnit:getLocation()
				timeStamp.agentFacing = userUnit:getFacing()
				timeStamp.agentSkills = {}
				timeStamp.agentInv = {}
				timeStamp.shoppingList = {}
				timeStamp.augMachUseList = {}
				
				for i,skill in ipairs(userUnit:getSkills()) do
					timeStamp.agentSkills[skill:getID()] = skill:getCurrentLevel()
				end
				
				

				local childrenIDs = {}
				local childTemplates = {}
				local childTraits = {}
				local childTags = {}
				local potentialShoppingList = {}
				
				-- get a list of items bought between hops to check for
				
				for i,purchase in pairs(self._shoppingList) do
					local purchaseTurn = purchase.purchaseTurn
					local ptemplate = purchase.itemTemplate
					--log:write("checking shoppinglist item - " .. ptemplate)
					if purchaseTurn <= currentTurn and purchaseTurn >= targetTurn then
						table.insert(potentialShoppingList,purchase)
				--		table.insert(timeStamp.shoppingList,purchase)
					end
				end
				for i,augMachUse in pairs(self._augMachineList) do
					local usageTurn = augMachUse.usageTurn
					--log:write("checking shoppinglist item - " .. ptemplate)
					if usageTurn <= currentTurn and usageTurn >= targetTurn then
						table.insert(timeStamp.augMachUseList,augMachUse)
					end
				end
				
				--build timeStamp.listOriginItemIDs
				for i,child in ipairs(userUnit:getChildren()) do
					local thisID = child:getID()
					table.insert(childrenIDs,thisID)
					childTemplates[thisID] = child._unitData.id
		--			--log:write("adding ID ".. child._unitData.id)
					childTraits[thisID] = {}
					local thisChildTraitRef = childTraits[thisID]
					--log:write("adding traits")
					for trait,value in pairs(child:getTraits()) do
						--log:write("adding trait ".. trait)
						if trait ~= "isTimeDupUnit" then
							thisChildTraitRef[trait] = value
						end
						--[==[if value:getID() ~= nil then
							thisChildTraitRef[trait] = {isUnitReference = true, unitRefID = thisChildTraitRef[trait]:getID()}
						else end]==]
					end
					
					childTags[thisID] = {}
					local thisChildTagRef = childTags[thisID]
					--log:write("adding traits")
					if child._tags then
						for tag,value in pairs(child._tags) do
						--	log:write(tag)
						--	log:write(value)
							thisChildTagRef[tag] = value
							--[==[if value:getID() ~= nil then
								thisChildTraitRef[trait] = {isUnitReference = true, unitRefID = thisChildTraitRef[trait]:getID()}
							else end]==]
						end
					end
					
					--- check to see if this item has a receipt.
					for s,purchase in pairs(potentialShoppingList) do
						local puchasedItem = purchase.item
						if puchasedItem and puchasedItem == child then
							table.insert(timeStamp.shoppingList,purchase)
						end
					end
				end
				timeStamp.childIDs = childrenIDs
				timeStamp.childTemplates = childTemplates
				timeStamp.childTraits = childTraits
				timeStamp.childTags = childTags
				
				if sim:getPC().soldItemList then
					timeStamp.soldItemList = sim:getPC().soldItemList
				end
			--------- we've got to go back, marty ---------	
				
			--	KLEIRenderScene:pulseUIFuzz( .5 )	
				
			
							
							--futureUnit:getTraits().invisible = true
				warpOutEffects( sim, userUnit )
				sim:warpUnit( userUnit, nil )	
				sim:despawnUnit( userUnit )
			--	sim:warpUnit( userUnit, nil )
				
			--	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )
				--KLEIRenderScene:pulseUIFuzz( .5 )

				

			--	KLEIRenderScene:setDesaturation( rig_util.linearEase("desat_ease") )
			--	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 1*cdefs.SECONDS )

				timeStamp.rewindsLeft = sim:getTags().rewindsLeft

				sim:dispatchEvent("TIME_HOP",{turns = TURNS_TO_JUMP})

			end
		end,


--------------

	onSpawnAbility = function( self, sim, unit )
			--self._game = savefiles.getCurrentGame()
			self._game = getCurrentGameState()
			sim:addTrigger( simdefs.TRG_START_TURN, self )
		--	sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( simdefs.TRG_GAME_OVER, self )
			sim:addTrigger( simdefs.TRG_MAP_EVENT, self )
			sim:addTrigger( simdefs.TRG_BUY_ITEM, self )
			sim:addTrigger( simdefs.TRG_USE_AUGMENT_MACHINE, self )
			self.abilityOwner = unit
			
			self.linkedAug = unit
			self._shoppingList = {}
			self._duplicatedUnits = {}
			self._augMachineList = {}
			--define augment <<>>
			local augUnit = timeHopAug(unit)
			if augUnit:getTraits().initialCooldown then	
				augUnit:getTraits().cooldown = augUnit:getTraits().initialCooldown
			end
			unit:getTraits().replaceKanim = "kanim_schysm"
			unit:getUnitData().profile_build = "schysm_face"
			unit:getUnitData().profile_icon_36x36 = "profile_icons/schysm_36.png"
		end,
	onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		--	sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( simdefs.TRG_GAME_OVER, self )
			sim:removeTrigger( simdefs.TRG_MAP_EVENT, self )
			sim:removeTrigger( simdefs.TRG_BUY_ITEM, self )
			sim:removeTrigger( simdefs.TRG_USE_AUGMENT_MACHINE, self )
			--self.abilityOwner = nil
		end,


	onTrigger = function ( self, sim, evType, evData )  -- need to get this working
			
			if evType == simdefs.TRG_START_TURN then --  evType == simdefs.TRG_END_TURN
				
				
				local abilityOwner = self.abilityOwner
				local player = abilityOwner:getPlayerOwner()
				local currentTurn = sim:getTurnCount()
				
				
				---- load the campaign stuff and check to see if we're expecting a guest
				
				
			
				local user = savefiles.getCurrentGame()
				local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]

				if campaign and campaign.timeHops and player and sim:getCurrentPlayer() == player then
				
				
				
					if evType == simdefs.TRG_START_TURN then
					
						sim:getPC().soldItemList = nil

						local targetTurn = currentTurn + TURNS_TO_JUMP -- 3 turns
						
						local numTurnsTravelled = currentTurn - targetTurn
						local timeSegment = campaign.currentDay .. "_" .. campaign.hours ..  "_" .. campaign.seed
						
						-- we check to see there is a jump back from three turns in the future
						local timeStamp = false
						--log:write("checking arrivals "..timeSegment .. "_" .. targetTurn)
						if campaign.timeHops[timeSegment .. "_" .. targetTurn] then
							timeStamp = campaign.timeHops[timeSegment .. "_" .. targetTurn]
						end
						
						if timeStamp and abilityOwner == sim:getUnit(timeStamp.agentID) then
							
							--log:write("arriving from "..timeSegment)
							-- prevent rewinds here
							sim:getTags().rewindsLeft = 0
							--- let's get the location 
							
							local futureUnit = nil
							local pastUnit = sim:getUnit(timeStamp.agentID)
							--log:write("getCell")
							local cell = sim:getCell(timeStamp.agentPosX,timeStamp.agentPosY)
							local facing = timeStamp.agentFacing
								
							--- check if we spawn ON someone and KO them for ease
							
							for i, cellUnit in ipairs( cell.units ) do
								if not cellUnit:isKO() then
									if cellUnit:getTraits().canKO then
										cellUnit:setKO( sim, 1 )
									elseif cellUnit:getTraits().isDrone and cellUnit.deactivate then
										cellUnit:deactivate(sim)
									end
								end
							end
							
							--- ok, spawn the new guy
							
							local unitData = unitdefs.lookupTemplate( "schysm" )
							local futureUnit = simfactory.createUnit( unitData, sim )
							
							for i, skill in ipairs(futureUnit:getSkills()) do
								local targetLevel = timeStamp.agentSkills[skill:getID()]
								if targetLevel and targetLevel > skill:getCurrentLevel() then
									for j = 1, targetLevel - skill:getCurrentLevel() do
										skill:levelUp(sim, futureUnit)
									end
								elseif targetLevel and targetLevel < skill:getCurrentLevel() then
									for j = 1, skill:getCurrentLevel() - targetLevel do
										skill:levelDown(sim, futureUnit)
									end
								end
							end
							
							futureUnit:setPlayerOwner(sim:getPC() )
							
							futureUnit:getTraits().customDefinedRenderFilter = "invis"
							futureUnit:getTraits().pinningOverride = true
							
							sim:spawnUnit( futureUnit )	
							
							
							-- swap the deployed agent for the new one
							
							for deployID, deployData in pairs(sim:getPC()._deployed) do
								if deployData.id == timeStamp.agentID then
									agentID = deployID
									break
								end
							end
							if agentID then
								sim:getPC()._deployed[ agentID ].id = futureUnit:getID()
							end
							
								
							sim:dispatchEvent( simdefs.EV_CAM_PAN, { cell.x, cell.y } )	sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )
							
							-- Spawn and warp the new unit with flashy fx
							
							sim:warpUnit( futureUnit, cell, facing )
							
							futureUnit:getTraits().isFutureUnit = true
							pastUnit:getTraits().isPastUnit = true
							
							KLEIRenderScene:pulseUIFuzz( .5 )
							
							futureUnit:getTraits().customDefinedRenderFilter = { shader=KLEIAnim.SHADER_FOW, r=100/255, g=200/255, b=200/255, a=0.333,lum=1 }
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = futureUnit, fx = "emp" } )	
							
							pastUnit:getTraits().customDefinedRenderFilter = { shader=KLEIAnim.SHADER_FOW, r=200/255, g=100/255, b=100/255, a=.75,lum=1 }
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = pastUnit } )	
							
							sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.5 * cdefs.SECONDS )

							KLEIRenderScene:pulseUIFuzz( .5 )
							sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cell.x, y = cell.y, range = 1} )
							
							
							for i, cellUnit in ipairs( cell.units ) do
								sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = cellUnit } )
							end
							
							for i,child in ipairs(futureUnit:getChildren()) do
								inventory.trashItem( sim, futureUnit, child )
							end
							
							for trait,value in ipairs(pastUnit:getTraits()) do
								futureUnit:getTraits()[trait] = value
							end
							
							futureUnit:getTraits().mp = timeStamp.agentMP
							futureUnit:getTraits().ap = timeStamp.agentAP
							futureUnit:getTraits().sneaking = timeStamp.sneaking
							futureUnit:getTraits().augmentMaxSize = timeStamp.augmentMaxSize
							
							--- fun stuff; we're going to find every item that has now been duplicated, make sure their IDs don't clash, and mark them for removal 
							
							if type(timeStamp.childIDs) == "table" and #timeStamp.childIDs > 0 then
							
								for i,futureChildID in pairs(timeStamp.childIDs) do
								--	gift copies of items
									
									local itemTemplateNam = timeStamp.childTemplates[futureChildID]
									--log:write(itemTemplateNam .. "template")
									local itemTemplate = unitdefs.lookupTemplate( timeStamp.childTemplates[futureChildID] )
									local newChild = inventory.giftUnit( sim, futureUnit, itemTemplate, true )
									
									
									if newChild then
										local installed = timeStamp.childTraits[futureChildID].installed
										if installed then
											newChild:getTraits().installed = false
											local abilityDef = newChild:ownsAbility( "installAugment" )
											if abilityDef and abilityDef:canUseAbility( sim, newChild, futureUnit ) then 
											--	abilityDef:executeAbility( sim, newChild, futureUnit )
												 futureUnit:doAugmentUpgrade( newChild )
											end 
										end
										local equipped = timeStamp.childTraits[futureChildID].equipped
										if equipped then
											local abilityDef = newChild:ownsAbility( "equippable" )
											if abilityDef and abilityDef:canUseAbility( sim, newChild, futureUnit ) then 
												abilityDef:executeAbility( sim, newChild, futureUnit )
											end 
										end
										local futureTraits = timeStamp.childTraits[futureChildID]
										for trait,value in pairs(futureTraits) do
											if type(value) == "table" and value.isUnitReference == true then
												newChild:getTraits()[trait] = sim:getUnit( value.unitRefID )
											else
												newChild:getTraits()[trait] = value
											end
										end
										local futureTags = timeStamp.childTags[futureChildID]
										if #futureTags > 0 then
											for tag,value in pairs(futureTags) do
											--		newChild:addTag( tag )
													newChild:addTag( value )
											--		log:write(tag)
											--		log:write(value)
													
											end
										end
									end
								end
								
								timeStamp._duplicatedUnitIDs = {}
								for i,duplicateUnit in pairs( timeStamp.childIDs ) do
									if duplicateUnit and sim:getUnit(duplicateUnit) then
										dupUnit = sim:getUnit(duplicateUnit)
										local futureTemplateNam = timeStamp.childTemplates[duplicateUnit]
										local itemTemplateNam = dupUnit._unitData.id
										if dupUnit and itemTemplateNam == futureTemplateNam then
											dupUnit:getTraits().isTimeDupUnit = true
											dupUnit:getTraits().cannotBeSold = true
											table.insert( timeStamp._duplicatedUnitIDs, dupUnit:getID())
										end
									end
								end
							end
						
						---- now we need to check if any shops need their stock tagged too
						
							if #timeStamp.shoppingList > 0 then
								for i,purchase in pairs(timeStamp.shoppingList) do
									--local purchaseTurn = purchase[1]
									local shopID = purchase.shopID
									local itemTemplate = purchase.itemTemplate
									
									local shopUnit = sim:getUnit(shopID)
									--log:write( "WINDOWSHOPPING: " .. shopUnit:getName() .. " " .. itemTemplate )
									if shopUnit then
										local found = false
										local itemType = purchase.itemType
										for j,template in pairs(shopUnit[itemType]) do
											if not found and template._unitData.id == itemTemplate then
												--log:write( "ITEM FOUND: " .. template:getName() .. " " .. itemTemplate )
												template:getTraits().isTimeDupUnit = true
												template:getTraits().cannotBeSold = true
												found = true
											end
										end
									end
									
								end
							end
							
							--- and tag any items that were sold
							--[==[
							if timeStamp.soldItemList and #timeStamp.soldItemList > 0 then
								for i,sale in pairs(timeStamp.soldItemList) do
									log:write( "SOLD ITEMS" )
									log:write( sale )
									dupUnit = sim:getUnit(sale)
									if dupUnit then
										dupUnit:getTraits().isTimeDupUnit = true
										dupUnit:getTraits().cannotBeSold = true
										table.insert( timeStamp._duplicatedUnitIDs, dupUnit:getID())
									end
									
								end
							end
							]==]
							--- this is some niche stuff that fixes obvious cheese strats, to be added to i'm sure
							
							--- disable used augmachines
							
							if #timeStamp.augMachUseList > 0 then
								for i,augMachUse in pairs(timeStamp.augMachUseList) do
									local shopID = augMachUse.shopID
									local shopUnit = sim:getUnit(shopID)
									if shopUnit then
										shopUnit:getTraits().used = true	
										shopUnit:getTraits().mainframe_status = "off"
										sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = shopUnit, fx = "emp" } )
									end
								end
							end
							-- kill spent CEOs
							for i,child in ipairs(futureUnit:getChildren()) do
								if child:hasTag("mission_loot") then
									if child:getUnitData().id == "vault_passcard" then
										local ceo = findUnitByTag( sim, "interrogate" )
										if ceo and ceo:isKO() and not sim:hasTag( "cardEjected") then
											local _, draggedBy = simquery.isUnitDragged( sim, ceo )
											if draggedBy then
												draggedBy:getTraits().movingBody = nil -- Really, stop moving me.
												sim:dispatchEvent( simdefs.EV_UNIT_BODYDROPPED, {unit = ceo} )
											end
											ceo:killUnit(sim)
										--	log:write( "KILL" )
										end
									end
								end
							end
						
							
							timeStamp.futureUnitID = futureUnit:getID()
							
							sim:dispatchEvent("SELECT_HOPPER",{unit = futureUnit})
							
							futureUnit:getTraits().pinningOverride = nil

							
							pastUnit:getTraits().customDefinedRenderFilter = nil
							futureUnit:getTraits().customDefinedRenderFilter = nil

							
							
							
							pastUnit:getUnitData().kanim = "kanim_schysm_R"
							futureUnit:getUnitData().kanim = "kanim_schysm_B"
							pastUnit:getTraits().replaceKanim ="kanim_schysm_R"
							futureUnit:getTraits().replaceKanim ="kanim_schysm_B"
							pastUnit:getUnitData().profile_build = "schysm_face_R"
							futureUnit:getUnitData().profile_build = "schysm_face_B"
							futureUnit:getUnitData().profile_icon_36x36 = "profile_icons/schysm_36_B.png"
							pastUnit:getUnitData().profile_icon_36x36 = "profile_icons/schysm_36_R.png"
							
							
							pastUnit:getUnitData().toolTip = STRINGS.TIMEHOP.AGENTS.SKYSM.ALT_1.DOPPLE_TOOLTIP
							
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = pastUnit } )	
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = futureUnit } )	
							sim:dispatchEvent( simdefs.EV_UNIT_GOTO_STAND, { unit = futureUnit, stand = false})
							
							sim:processReactions( futureUnit )
					
							
							local futureAugUnit = timeHopAug(futureUnit)
							if futureAugUnit then
								futureAugUnit:getTraits().cooldown = futureAugUnit:getTraits().cooldownMax
								futureAugUnit:getTraits().charges = math.max(0, futureAugUnit:getTraits().charges - 1)
							end
							----- re ID items and store unitid
							if pastUnit then
								pastUnit:getTraits().leavesAtEndOfMission = true
								local pastAugUnit = timeHopAug(pastUnit)
								pastAugUnit:getTraits().cooldown = 1
								pastAugUnit:getTraits().skipPWRcost = true
							end
							
							
						end
					end
					
					if evType == simdefs.TRG_START_TURN then -- evType == simdefs.TRG_END_TURN
						
						local currentTurn = sim:getTurnCount()
						local targetTurn = currentTurn -- 3 turns
						local timeSegment = campaign.currentDay .. "_" .. campaign.hours ..  "_" .. campaign.seed
						
						-- we check to see if we should be jumping back
						local timeStamp = " ... no timestamp"
						if campaign.timeHops[timeSegment .. "_" .. currentTurn] then
							timeStamp = campaign.timeHops[timeSegment .. "_" .. currentTurn]
						end
						--log:write("checking "..timeSegment)
						
						if timeStamp and timeStamp.exists == true and timeStamp.childIDs and #timeStamp.childIDs > 0 then
						--log:write("..found?")
						
							-- restore rewinds
							 sim:getTags().rewindsLeft = timeStamp.rewindsLeft
							--- run backwards through shopping list to make sure we remove any bought items
							for i,purchase in pairs(timeStamp.shoppingList) do
								
								local shopUnit = sim:getUnit(purchase.shopID)
								local itemType = purchase.itemType
								
								local found = false
								if shopUnit then
									--log:write("found purchase in: " .. itemType .. " of " .. shopUnit:getName() )
									for j,template in pairs(shopUnit[itemType]) do
										if template and not found and template._unitData.id == purchase.itemTemplate and template:getTraits().isTimeDupUnit then
											--log:write( "ITEM REMOVED: " .. template:getName() .. " " .. purchase.itemTemplate )
											local credits = template:getUnitData().value
											player:addCredits( -credits )
											table.remove( shopUnit[itemType], j )
											found = true
										end
									end
								end
								
							end
						
						
							for i,duplicateUnit in pairs( timeStamp.childIDs ) do
								local itemTemplateNam = timeStamp.childTemplates[duplicateUnit]
								if duplicateUnit and sim:getUnit(duplicateUnit) and sim:getUnit(duplicateUnit)._unitData.id == itemTemplateNam then -- << check if same template to avoid BIG mistakes >>
								
									dupUnit = sim:getUnit(duplicateUnit)
									if dupUnit:getUnitOwner() then
										inventory.dropItem(sim, dupUnit:getUnitOwner(), dupUnit)
									end
									
									sim:warpUnit( dupUnit, nil )	
									sim:despawnUnit( dupUnit )
								end
							end
							
							if timeStamp.agentID and sim:getUnit(timeStamp.agentID) then
								local pastUnit = sim:getUnit(timeStamp.agentID)
								for i,child in ipairs(pastUnit:getChildren()) do
									if child:getTraits().installed then
										child:getTraits().installed = false
									end
								end
								inventory.dropAll(sim, pastUnit)
								
								warpOutEffects( sim,pastUnit )
								
								sim:warpUnit( pastUnit, nil )	
								sim:despawnUnit( pastUnit )
							end
							
							if timeStamp.futureUnitID and sim:getUnit(timeStamp.futureUnitID) then
								local futureUnit = sim:getUnit(timeStamp.futureUnitID)
								futureUnit:getTraits().isFutureUnit = false
								futureUnit:getUnitData().kanim = "kanim_schysm"
							--	futureUnit:changeKanim(nil)
								futureUnit:getTraits().replaceKanim = "kanim_schysm"
								futureUnit:getUnitData().profile_build = "schysm_face"
								futureUnit:getUnitData().profile_icon_36x36 = "profile_icons/schysm_36.png"
								futureUnit:getTraits().customDefinedRenderFilter = nil
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = futureUnit } )
								--<<TOO>>
							end
						end
					end
				end			
			end	
			
			---- << map event buyItem
			---- add to list of bought items
			---- send back for item removal chek
			--- if not item, clear template from shop
			
			
			
			if evType == simdefs.TRG_MAP_EVENT and evData.event == simdefs.MAP_EVENTS.TELEPORT then
				local user = savefiles.getCurrentGame()
				local campaign = user.data.saveSlots[ user.data.currentSaveSlot ]
				
				if campaign and campaign.timeHops then
					local currentTurn = sim:getTurnCount()
				--	local nextTimeMarker = false
					local allDupdItems = {}
					local futureUnit = {}
					local pastUnit = {}
					local timeStamp = 0
					for i,testReality in pairs({0,1,2}) do
						local targetTurn = currentTurn + i -- up to 3 turns - need 5 turn cooldown minimum
						local numTurnsTravelled = currentTurn - targetTurn
						local timeSegment = campaign.currentDay .. "_" .. campaign.hours ..  "_" .. campaign.seed
						
						-- we check to see there is a jump back from three turns in the future
						local testTimeStamp = campaign.timeHops[timeSegment .. "_" .. targetTurn]
						
						if testTimeStamp then
							--nextTimeMarker = timestamp
							timeStamp = testTimeStamp
							for j,duplicateUnitID in pairs( timeStamp.childIDs ) do
								table.insert(allDupdItems,duplicateUnitID)
							end
							futureUnit = sim:getUnit(timeStamp.futureUnitID)
							pastUnit = sim:getUnit(timeStamp.agentID)
							
						end
					end
					
					local futureUnitEscaped = false
					local pastUnitEscaped = false
					
					for i,escaperUnit in pairs(evData.units) do 
						
						--- confiscate dup units and check if they are a time traveller
						
						for j,child in ipairs(escaperUnit:getChildren()) do
							if child:getTraits().isTimeDupUnit then
								inventory.trashItem( sim, child:getUnitOwner(), child )
							end
						end
						
						if futureUnit and escaperUnit == futureUnit then
							futureUnitEscaped = true
						end
						if pastUnit and escaperUnit == pastUnit then
							pastUnitEscaped = true
						end
						
					end
					
					if futureUnitEscaped then
						if pastUnit and not pastUnitEscaped then
							inventory.dropAll(sim, pastUnit)
								
						--	warpOutEffects( sim,pastUnit )
							
							sim:warpUnit( pastUnit, nil )	
							sim:despawnUnit( pastUnit )
							futureUnit:getTraits().isFutureUnit = false
						end
					elseif futureUnit and pastUnitEscaped then
					
						futureUnit:getTraits().isFutureUnit = false
						futureUnit:getUnitData().kanim = "kanim_schysm"
						futureUnit:getTraits().customDefinedRenderFilter = nil
					--	futureUnit:changeKanim( nil )
						futureUnit:getTraits().replaceKanim = "kanim_schysm"
						futureUnit:getUnitData().profile_build = "schysm_face"
						futureUnit:getUnitData().profile_icon_36x36 = "profile_icons/schysm_36.png"
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = futureUnit } )
					end
					--
				end
			end
			if evType == simdefs.TRG_BUY_ITEM and evData.shopUnit and evData.unit and evData.item  then
				
				local currentTurn = sim:getTurnCount()
				local shopID = evData.shopUnit:getID()
				local itemUnit = evData.item
				local itemID = itemUnit:getID()
				local itemTemplate = itemUnit:getUnitData().id
				local purchase = {}
				purchase.purchaseTurn = currentTurn
				purchase.shopID = shopID
				purchase.itemTemplate = itemTemplate 
				purchase.item = itemUnit 
				if itemUnit:getTraits().slot == "gun" or itemUnit:getTraits().slot == "melee" then
					purchase.itemType = "weapons"
				elseif itemUnit:getTraits().augment then
					purchase.itemType = "augments"
				else
					purchase.itemType = "items"
				end
				
				--log:write( "ITEM BOUGHT: " .. itemUnit:getName() .. " | temp: " .. purchase.itemTemplate .. " | from: " .. evData.shopUnit:getName() )
				table.insert(self._shoppingList, purchase)
			end
			if evType == simdefs.TRG_USE_AUGMENT_MACHINE and evData.unit and evData.unit == self.abilityOwner and evData.shop then
				
				local currentTurn = sim:getTurnCount()
				
				local shopID = evData.shop:getID()
				
				local augMachUse = {}
				augMachUse.usageTurn = currentTurn
				augMachUse.shopID = shopID
				
				--log:write( "AUG INSTALLED FROM: " .. evData.shop:getName() )
				table.insert(self._augMachineList, augMachUse)
			end
			if evType == simdefs.TRG_GAME_OVER then
				clearTimeHops( )
			end
		end,
	}
	
return timeHop

