local util = include("client_util")
local array = include("modules/array")

local tooltips =
{
	onItemTooltip = function ( tooltip, unit, userUnit )
		--General ammo rework. So we can use the ammo mechanic, but have it display as something else. --MAA code copy
		if unit:getTraits().coit_ammo_text then
			for _, widget in ipairs( tooltip._children ) do
				if widget.binder.lineRight and widget.binder.lineRight.getText and widget.binder.lineRight:getText() == string.format(STRINGS.ITEMS.AMMO, unit:getTraits().ammo, unit:getTraits().maxAmmo) then
					widget.binder.lineRight:setText( util.sformat( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.AMMO, unit:getTraits().coit_ammo_text, unit:getTraits().ammo, unit:getTraits().maxAmmo ) )
					break
				end
			end
		end
		
		--trigger
		if unit:getTraits().coit_trigger then
			if unit:getTraits().coit_trigger == "daemon" then
				tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.TRIGGER_DAEMON, STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.TRIGGER_DAEMON_DESC )
			end
		end
		
		--max procs
		if unit:getTraits().coit_proc_max then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.PROC_MAX, util.sformat(STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.PROC_MAX_DESC, unit:getTraits().coit_proc_max) )
		end
		
		--install daemon
		if unit:getTraits().coit_spawnsDaemon then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.INSTALLS_DAEMON, STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.INSTALLS_DAEMON_DESC )
		end
		--notify guard
		if unit:getTraits().coit_notifiyGuard then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.NOTIFIES_GUARD, STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.NOTIFIES_GUARD_DESC )
		end
		--increase alarm
		if unit:getTraits().coit_increase_alarm then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.INCREASES_ALARM, util.sformat(STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.INCREASES_ALARM_DESC, unit:getTraits().coit_increase_alarm) )
		end
		--alarm locked
		if unit:getTraits().coit_no_max_alarm then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.SEC_BLOCK, STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.SEC_BLOCK_DESC )
		end
		
		--automatic weapon
		if unit:getTraits().coit_multishot then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.MULTISHOT, STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.MULTISHOT_DESC )
		end
		
		--charge armor
		if unit:getTraits().coit_ammo_per_armor then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.CHARGE_PER_ARMOR, util.sformat(STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.CHARGE_PER_ARMOR_DESC, unit:getTraits().coit_ammo_per_armor) )
		end
		
		--power weapon buff
		if unit:getTraits().coit_weapon_power_buffs then
			local desc = ""
			if unit:getTraits().coit_weapon_power_buff_cost then
				desc = desc .. util.sformat( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.POWER_WEAPON_COST, unit:getTraits().coit_weapon_power_buff_cost) .. "\n"
			end
			for i, buff in ipairs( unit:getTraits().coit_weapon_power_buffs ) do
				local trait_key = buff.trait
				if trait_key and trait_key == "baseDamage" and unit:getTraits().canSleep then
					trait_key = "baseDamage_KO"
				end
				if buff.trait and STRINGS.CORPORATE_ITEMS.UI.TRAIT_MAP[trait_key] then
					if i > 1 then
						desc = desc .. "\n"
					end
					if type(buff.value) == "boolean" then
						desc = desc .. STRINGS.CORPORATE_ITEMS.UI.TRAIT_MAP[trait_key]
					else
						desc = desc .. util.sformat( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.POWER_WEAPON_BUFF, STRINGS.CORPORATE_ITEMS.UI.TRAIT_MAP[trait_key], buff.value or 0 )
					end
				end
			end
			
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.POWER_WEAPON, desc, "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png" )
		end
		--tracer weapon buff
		if unit:getTraits().coit_weapon_trace_buffs then
			local desc = STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.TRACER_WEAPON_DESC
			for i, buff in ipairs( unit:getTraits().coit_weapon_trace_buffs ) do
				local trait_key = buff.trait
				if trait_key and trait_key == "baseDamage" and unit:getTraits().canSleep then
					trait_key = "baseDamage_KO"
				end
				if buff.trait and STRINGS.CORPORATE_ITEMS.UI.TRAIT_MAP[trait_key] then
					if i > 1 then
						desc = desc .. "\n"
					end
					if type(buff.value) == "boolean" then
						desc = desc .. STRINGS.CORPORATE_ITEMS.UI.TRAIT_MAP[trait_key]
					else
						desc = desc .. util.sformat( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.TRACER_WEAPON_BUFF, STRINGS.CORPORATE_ITEMS.UI.TRAIT_MAP[trait_key], buff.value or 0 )
					end
				end
			end
			
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.TRACER_WEAPON, desc, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )
		end
		
		--ammo cost per armor
		if unit:getTraits().coit_ammo_cost_per_armor then
			local ammo = 1
			local armor = 1
			
			if unit:getTraits().coit_ammo_cost_per_armor >= 1 then
				ammo = unit:getTraits().coit_ammo_cost_per_armor
			else
				armor = math.ceil( 1 / unit:getTraits().coit_ammo_cost_per_armor )
			end
			
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.COMBINED_BULLETS, util.sformat(STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.COMBINED_BULLETS_DESC, ammo, armor) )
		end
		
		--chips
		if unit:getTraits().coit_gain_cpu then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.GAIN_PWR, util.sformat(STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.GAIN_PWR_DESC, unit:getTraits().coit_gain_cpu) )
		end
	end,
	
	onGuardTooltip = function( tooltip, unit, userUnit )
		--armor mark
		if unit and unit:getTraits().coit_armor_mark and #unit:getTraits().coit_armor_mark > 0 then
			tooltip:addAbility( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.ARMOR_MARK, util.sformat( STRINGS.CORPORATE_ITEMS.UI.TOOLTIP.ARMOR_MARK_DESC, #unit:getTraits().coit_armor_mark ) )
		end
	end,
}

return tooltips
