local util = include("client_util")
local array = include("modules/array")

local tooltips =
{
	onItemTooltip = function ( tooltip, unit, userUnit )
		--General ammo rework. So we can use the ammo mechanic, but have it display as something else.
		if unit:getTraits().maa_ammo_text then
			for _, widget in ipairs( tooltip._children ) do
				if widget.binder.lineRight and widget.binder.lineRight.getText and widget.binder.lineRight:getText() == string.format(STRINGS.ITEMS.AMMO, unit:getTraits().ammo, unit:getTraits().maxAmmo) then
					widget.binder.lineRight:setText( util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.AMMO, unit:getTraits().maa_ammo_text, unit:getTraits().ammo, unit:getTraits().maxAmmo ) )
					break
				end
			end
		end

		--Sharp D stuff
		if unit:getTraits().maa_power_allocated then
			if userUnit then
				for _, widget in ipairs( tooltip._children ) do
					local line_widget = widget:findWidget("line")
					if line_widget and line_widget.getText and line_widget:getText() == "<ttheader>"..util.toupper( unit:getName() ).."</>" then
						local rtxt = util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.SHARP_POWER, unit:getTraits().maa_available_power )
						if userUnit:getTraits().maa_sharp_reactor then
							rtxt = rtxt .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.SHARP_POWER_3, userUnit:getTraits().maa_sharp_reactor )
						end
						widget.binder.lineRight:setText( rtxt )
						break
					end
				end
			end
		end
		if unit:getTraits().maa_sharp_augment_type then
			if userUnit then
				local reactor_unit = nil
				if userUnit.getChildren then
					for _, child in ipairs( userUnit:getChildren() ) do
						if child:getTraits().maa_power_allocated then
							reactor_unit = child
							break
						end
					end
				end
				if reactor_unit then
					local power_is = reactor_unit:getTraits().maa_power_allocated[ unit:getTraits().maa_sharp_augment_type ]
					local power_target = unit:getTraits().maa_target_level or power_is

					for _, widget in ipairs( tooltip._children ) do
						local line_widget = widget:findWidget("line")
						if line_widget and line_widget.getText and line_widget:getText() == "<ttheader>"..util.toupper( unit:getName() ).."</>" then
							local rtxt = util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.SHARP_POWER, power_is )
							if power_target then
								local color_a = ""
								local color_b = ""
								if power_target < power_is then
									color_a = "<c:f55120>"
									color_b = "</>"
								elseif power_target > power_is then
									color_a = "<c:2a9614>"
									color_b = "</>"
								end
								rtxt = rtxt .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.SHARP_POWER_2, power_target, color_a, color_b)
							end
							widget.binder.lineRight:setText( rtxt )
							break
						end
					end
				end
			end
		end

		--Nika B adrenaline stuff
		if userUnit then
			if unit and unit:getTraits().maa_is_aamplifier then
				if userUnit:getTraits().maa_buff_melee_damage and userUnit:getTraits().maa_buff_melee_damage > 0 then
					tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ADRENALINE_BUFF_DAMAGE, util.sformat(STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ADRENALINE_BUFF_DAMAGE_DESC, userUnit:getTraits().maa_buff_melee_damage), "gui/icons/action_icons/Action_icon_Small/icon-item_melee_small-old.png" )
				end
				if userUnit:getTraits().maa_buff_melee_pierce and userUnit:getTraits().maa_buff_melee_pierce > 0 then
					tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ADRENALINE_BUFF_PIERCE, util.sformat(STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ADRENALINE_BUFF_PIERCE_DESC, userUnit:getTraits().maa_buff_melee_pierce), "gui/icons/action_icons/Action_icon_Small/icon-item_melee_small.png" )
				end
			end
		end

		--Prism B mainframe possession stuff
		if unit and unit:hasAbility( "maa_mainframe_possession" ) then
			local desc = util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POSSESS_MAINFRAME_DESC, unit:getTraits().cdMax_mp )
			local icon = unit:hasAbility( "maa_mainframe_possession" ).profile_icon
			if unit:getTraits().cd_mp and unit:getTraits().cd_mp > 0 then
				desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POSSESS_MAINFRAME_DESC_2, unit:getTraits().cd_mp )
			end
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POSSESS_MAINFRAME, desc, icon )
		end
		if unit and unit:hasAbility( "maa_mp_advance" ) then
			local desc = util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.SCOUT_MAINFRAME_DESC, unit:getTraits().pwr_cost_advance, unit:getTraits().cdMax_advance )
			local icon = unit:hasAbility( "maa_mp_advance" ).profile_icon
			if unit:getTraits().cd_advance and unit:getTraits().cd_advance > 0 then
				desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POSSESS_MAINFRAME_DESC_2, unit:getTraits().cd_advance )
			end
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.SCOUT_MAINFRAME, desc, icon )
		end
		if unit and unit:hasAbility( "maa_mp_control_device" ) then
			local desc = util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CONTROL_DEVICE_DESC, unit:getTraits().pwr_cost_control_base, unit:getTraits().pwr_cost_control_reduction_per_turn, unit:getTraits().cdMax_control )
			local icon = unit:hasAbility( "maa_mp_control_device" ).profile_icon
			if unit:getTraits().cd_control and unit:getTraits().cd_control > 0 then
				desc = desc .. util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POSSESS_MAINFRAME_DESC_2, unit:getTraits().cd_control )
			end
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CONTROL_DEVICE, desc, icon )
		end

		--Draco B Vampire Gun stuff
		if unit and unit:getTraits().maa_ap_refresh then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.VAMPIRE_AP, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.VAMPIRE_AP_DESC, unit:getTraits().maa_ap_refresh ), "gui/icons/arrow_small.png" )
		end
		if unit and unit:getTraits().maa_mp_boost then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.VAMPIRE_MP, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.VAMPIRE_MP_DESC, unit:getTraits().maa_mp_boost ), "gui/icons/arrow_small.png" )
		end
		if unit and unit:getTraits().maa_autoreload then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.VAMPIRE_AUTOLOAD, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.VAMPIRE_AUTOLOAD_DESC, unit:getTraits().maa_autoreload ), "gui/icons/arrow_small.png" )
		end

		--Decker C Defibrillator Stuff
		if unit and unit:getTraits().maa_count_used then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.DEFIBRILLATOR, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.DEFIBRILLATOR_DESC, unit:getTraits().maa_count_used ), "gui/icons/arrow_small.png" )
		end

		--Maria C Power Battery Stuff
		if unit and unit:getTraits().maa_powerMax and unit:getTraits().maa_power_generated then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POWER_BATTERY, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.POWER_BATTERY_DESC, unit:getTraits().maa_power_generated, unit:getTraits().maa_powerMax ), "gui/icons/arrow_small.png" )
		end

		--Tony C Stuff
		if unit and unit:getTraits().maa_modded_item then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ITEM_MODDED, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ITEM_MODDED_DESC, unit:getTraits().maa_modded_item ), "gui/icons/skills_icons/skills_icon_small/icon-item_technician_small.png" )
		end

		--Monster C Stuff
		if unit and unit:getTraits().maa_swap_at_end then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.TRADED_ITEM, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.TRADED_ITEM_DESC, "gui/icons/skills_icons/skills_icon_small/icon-item_inventory_small.png" )
		end

		--Draco C Stuff
		if unit and unit:getTraits().maa_thirst_state then
			--create a new desc in the tooltip section first
			local desc = ""
			local thirst_state = unit:getTraits().maa_thirst_state
			if thirst_state > 7 then
				thirst_state = 7
			end
			for i, state in ipairs( STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_THIRST.THIRST_STATES ) do
				if i > 1 then
					desc = desc .. "\n"
				end
				if thirst_state == i-1 then
					desc = desc .. "<c:FF8411>" .. util.toupper( state[1] ) .. "</c>" .. state[2]
				else
					desc = desc .. util.toupper( state[1] ) .. state[2]
				end
			end
			tooltip:addDesc( desc )
			--then find the position of the original desc and place it behind
			local index = nil
			for i, widget in ipairs( tooltip._children ) do
				local widget_desc = widget:findWidget("desc")
				if widget_desc and widget_desc.getText and widget_desc:getText() == STRINGS.MORE_ARCHIVE_AGENTS.ITEMS.VAMPIRE_THIRST.DESC then
					index = i + 1
					break
				end
			end

			if index then
				table.insert( tooltip._children, index, table.remove(tooltip._children, #tooltip._children) )
			end
		end

		--Central C Tazer
		if unit and unit:getTraits().maa_alarm_per_armor then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ARMOR_ALARM_TAZER, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ARMOR_ALARM_TAZER_DESC )
		end

		--Rush C Tazer
		if unit and unit:getTraits().maa_alarm_on_use then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ALARM_TAZER, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ALARM_TAZER_DESC )
		end

		--Derek C Particle Uplink
		if unit and unit:hasAbility( "maa_pu_push" ) then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_PUSH, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_PUSH_DESC, "gui/icons/action_icons/Action_icon_Small/icon-item_select_small.png" )
		end
		if unit and unit:hasAbility( "maa_pu_swap" ) then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_SWAP, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_SWAP_DESC, "gui/icons/action_icons/Action_icon_Small/icon-action_equip_small.png" )
		end
		if unit and unit:hasAbility( "maa_pu_linked_pockets" ) then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_LINKED_POCKETS, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_LINKED_POCKETS_DESC, "gui/icons/action_icons/Action_icon_Small/icon-action_trade.png" )
		end
		if unit and unit:hasAbility( "maa_pu_lifelink" ) then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_LIFELINK, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.PU_LIFELINK_DESC, "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png" )
		end

		--N-Umi B tazer
		if unit and unit:getTraits().maa_ammo_per_armor then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CHARGE_PER_ARMOR, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CHARGE_PER_ARMOR_DESC, unit:getTraits().maa_ammo_per_armor ) )
		end
		
		--Shalem D mod stuff
		if unit and unit:getTraits().maa_mod_slot then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.WEAPON_MOD, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.WEAPON_MOD_DESC, unit:getTraits().maa_mod_slot ) )
			if unit:getTraits().baseDamage and unit:getTraits().baseDamage > 0 then
				if unit:getTraits().maa_mod_slot == "clip" then
					if unit:getTraits().canSleep then
						tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.KO_DMG, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.KO_DMG_DESC, unit:getTraits().baseDamage ) )
					else
						tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.LETHAL_DMG, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.LETHAL_DMG_DESC, unit:getTraits().baseDamage ) )
					end
				else
					tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.INCREASE_DMG, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.INCREASE_DMG_DESC, unit:getTraits().baseDamage ) )
				end
			end
			if unit:getTraits().modeEmpdamage and unit:getTraits().modeEmpdamage > 0 then
				if unit:getTraits().maa_mod_slot == "clip" then
					tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.EMP_DMG, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.EMP_DMG_DESC, unit:getTraits().modeEmpdamage ) )
				end
			end
			if unit:getTraits().armorPiercing and unit:getTraits().armorPiercing > 0 then
				tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.INCREASE_AP, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.INCREASE_AP_DESC, unit:getTraits().armorPiercing ) )
			end
			if unit:getTraits().maa_mp_cost and unit:getTraits().maa_mp_cost > 0 then
				tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.MP_PENALTY, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.MP_PENALTY_DESC, unit:getTraits().maa_mp_cost ) )
			end
		end
		if unit and unit:getTraits().maa_is_shalem_mod_rifle then
			if #unit:getChildren() > 0 then
				for _, childUnit in ipairs( unit:getChildren() ) do
					if childUnit:getTraits().maa_mod_slot == "clip" then
						if childUnit:getTraits().canSleep then
							tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CLIP_TIP, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CLIP_KO, "gui/icons/action_icons/Action_icon_Small/icon-item_reload_small.png" )
						elseif childUnit:getTraits().modeEmpdamage then
							tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CLIP_TIP, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CLIP_EMP, "gui/icons/action_icons/Action_icon_Small/icon-item_reload_small.png" )
						else
							tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CLIP_TIP, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.CLIP_LETHAL, "gui/icons/action_icons/Action_icon_Small/icon-item_reload_small.png" )
						end
					end
				end
			end
		end
	end,

	onGuardTooltip = function( tooltip, unit, userUnit )
		--Tony B Trap Stuff
		if unit and unit:getTraits().maa_trap_installed then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.HEARTSHOCKTRAP, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.HEARTSHOCKTRAP_DESC, "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png" )
		end

		--Shalem C Bounty Stuff
		if unit and unit:getTraits().maa_bounty then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_GUARD_TOOL, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.BOUNTY_GUARD_TOOL_DESC, unit:getTraits().maa_bounty ), "gui/icons/arrow_small.png" )
		end

		--Olivia B Stuff
		if unit and unit:getTraits().maa_neural_interrogated then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.INTERROGATED, STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.INTERROGATED_DESC )
		end

		--Umi Drone Mark
		if unit and unit:getTraits().maa_armor_mark then
			tooltip:addAbility( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ARMOR_MARK, util.sformat( STRINGS.MORE_ARCHIVE_AGENTS.UI.TOOLTIP.ARMOR_MARK_DESC, unit:getTraits().maa_armor_mark ) )
		end
	end,
}

return tooltips
