local util = include( "modules/util" )
local unitrig = include( "gameplay/unitrig" )
local simquery = include( "sim/simquery" )

local oldrefreshProp = unitrig.rig.refreshProp
local olddestroy = unitrig.rig.destroy
local oldRefreshRenderFilter = unitrig.rig.refreshRenderFilter

unitrig.rig.refreshProp = function( self, refreshLoc )
	local unit = self:getUnit()
	local ret = oldrefreshProp( self, refreshLoc )
	if not self._psiFX then
		self._psiFX = self:createHUDProp("kanim_null_fx", "effect", "idle", self._boardRig:getLayer("floor"), self._prop )
		self._psiFX:setSymbolModulate("innercicrle",1,1,1,0.75)
		self._psiFX:setSymbolModulate("innerring",1,1,1,0.75)
		self._psiFX:setVisible(false)
	end
	if unit:getTraits().psiLink then
		self._psiFX:setVisible(true)
	else
		self._psiFX:setVisible(false)
	end
	
	if not self._holoFX then
		self._holoFX = self:createHUDProp("kanim_null_fx", "effect", "idle", self._boardRig:getLayer("floor"), self._prop )
		self._holoFX:setSymbolModulate("innercicrle",0.5,0.8,1,0.9)
		self._holoFX:setSymbolModulate("innerring",0.5,0.8,1,0.5)
		self._holoFX:setVisible(false)
	end
	if unit:getTraits().holoTarget and unit:getTraits().holoTarget > 0 then
		self._holoFX:setVisible(true)
	else
		self._holoFX:setVisible(false)
	end
	return ret
end

unitrig.rig.destroy = function( self )
	self:transitionUnitState( nil )

	if self._prop then
		if self._boardRig:getLayer() then
			self._boardRig:getLayer():removeProp( self._prop )
		end
	end

	self._boardRig:refreshLOSCaster( self._unitID )

	if self._spotSound then
		self._boardRig:getSounds():stopSound( self._spotSound )
		self._spotSound = nil
	end

	if self._nullFX then
		self._boardRig:getLayer("floor"):removeProp( self._nullFX  )			
	end

	if self._mag_resFX then
		self._boardRig:getLayer("ceiling"):removeProp( self._mag_resFX  )			
	end


	if self._HUDlocated then
		self._boardRig:getLayer("ceiling"):removeProp( self._HUDlocated )
	end

	if self._monst3rConsole then
		self._prop:removeProp( self._monst3rConsole, true)	
	end

	if self._switchFx then
		self._prop:removeProp( self._switchFx , true)	
	end

	if self._psiFX then
		self._boardRig:getLayer("floor"):removeProp( self._psiFX  )			
	end

	if self._holoFX then
		self._boardRig:getLayer("floor"):removeProp( self._holoFX  )			
	end
end

unitrig.rig.playSound = function( self, soundPath, alias )
	local unit = self:getUnit()
	if unit then
		local x1,y1 = unit:getLocation()
		if soundPath == "SpySociety/Weapons/Shells/shoot_death" and simquery.getEquippedGun( unit ):getTraits().canSleep then
			return
		end
		if unit:getTraits().voice then
			soundPath = string.gsub(soundPath, "<voice>", unit:getTraits().voice)
		end
		self._boardRig:getSounds():playSound( soundPath, alias, x1, y1 )
	end
end

function unitrig.refreshRenderFilter(self, ...)
	oldRefreshRenderFilter(self, ...)
    
	if self._renderFilterOverride then
		self._prop:setRenderFilter( self._renderFilterOverride )
	else
		local unit = self._boardRig:getLastKnownUnit( self._unitID )
		if unit then
			self._prop:setPlayMode( self._playMode )
			if unit:getTraits().customRenderFilter then
				local filter = unit:getTraits().customRenderFilter
				self._prop:setRenderFilter( filter )
			end
		end
	end
end