--by qoalabear

local util = include( "modules/util" )
local simunit = include( "sim/simunit" )
local unitdefs = include( "sim/unitdefs" )
local simfactory = include( "sim/simfactory" )


local maa_simcontainer = { ClassType = "maa_simcontainer" }

-- ===

function maa_simcontainer.createContainerUnit( unitData, sim )
    -- Expand all children defs and insert them into unitData before this unit is created.
    -- maaUpgrades should only exist if this was a serialized item, so the unitData was already
    -- defensively copied by unitdefs.createUnitData. Editing should be safe.
    if unitData and unitData.maaUpgrades then
        unitData.children = unitData.children or {}
        for _, upgrade in ipairs(unitData.maaUpgrades) do
            local upgradeName, template, childData
            if type(upgrade) == "string" then
                upgradeName = upgrade
                template = unitdefs.lookupTemplate(upgrade)
                childData = template and util.tcopy(template)
            else
                upgradeName = upgrade.upgradeName
                local params = upgrade.upgradeParams
                template = unitdefs.lookupTemplate(upgradeName)
                childData = util.extend(template)(params and util.tcopy(params))
            end
            if not template then
                simlog("[MAA] Missing template for %s inside %s", tostring(upgradeName), tostring(unitData.id))
            elseif childData then
                table.insert(unitData.children, childData)
            end
        end
    end

	local unit = simunit.createUnit( unitData, sim, maa_simcontainer )

    local refreshAbility = unit:hasAbility("maa_refresh_weapon_mods")
    if refreshAbility then
        -- simunit.createUnit() spawns abilities before children. Initialize the rifle now.
        -- This may be called on the agent-select and upgrade screen, in which case, sim is nil.
        refreshAbility:onTrigger(sim, "TRG_MAA_REFRESH_MOD_WEAPON", nil, unit)
    end

    return unit
end

simfactory.register( maa_simcontainer.createContainerUnit )

return maa_simcontainer
