local util = include( "client_util" )
local cdefs = include( "client_defs" )
local mui_defs = include( "mui/mui_defs" )
local mui = include( "mui/mui" )
local simfactory = include( "sim/simfactory" )
local itemdefs = include("sim/unitdefs/itemdefs")
local modal_thread = include( "gameplay/modal_thread" )
local guiex = include( "client/guiex" )
local mui_tooltip = include( "mui/mui_tooltip")
local skilldefs = include( "sim/skilldefs" )

local SET_COLOR = {r=244/255,g=255/255,b=120/255, a=1}
local POSSIBLE_COLOR = {r=0/255,g=184/255,b=0/255, a=1}
local BLANK_COLOR = {r=56/255,g=96/255,b=96/255, a=200/255}
local HOVER_COLOR = {r=255/255,g=255/255,b=255/255, a=1}
local HOVER_COLOR_FAIL = {r=178/255,g=0/255,b=0/255, a=1}
local LOCKED_COLOR = {r=255/255,g=150/255,b=19/255, a=1}
local LOCKED_BLANK = {r=12/255,g=17/255,b=17/255, a=1}
local TEMP_COLOR = {r=0/255,g=184/255,b=0/255, a=1}

local skillButton = {
    name = [[skillBtn]],
    isVisible = true,
    noInput = false,
    anchor = 1,
    rotation = 0,
    x = 731,
    xpx = true,
    y = -573,
    ypx = true,
    w = 166,
    wpx = true,
    h = 38,
    hpx = true,
    sx = 1,
    sy = 1,
    ctor = [[button]],
    clickSound = [[SpySociety/HUD/menu/click]],
    hoverSound = [[SpySociety/HUD/menu/rollover]],
    hoverScale = 1,
    str = [[STR_476357528]],
    hotkey = 27,
    halign = MOAITextBox.RIGHT_JUSTIFY,
    valign = MOAITextBox.CENTER_JUSTIFY,
    text_style = [[font1_24_r]],
    offset =
    {
        x = -15,
        xpx = true,
        y = 0,
        ypx = true,
    },
    images =
    {
        {
            file = [[white.png]],
            name = [[inactive]],
            color =
            {
                0.30588236451149,
                0.533333361148834,
                0.533333361148834,
                0.39215686917305,
            },
        },
        {
            file = [[white.png]],
            name = [[hover]],
            color =
            {
                0.549019634723663,
                1,
                1,
                0.39215686917305,
            },
        },
        {
            file = [[white.png]],
            name = [[active]],
            color =
            {
                0.549019634723663,
                1,
                1,
                0.39215686917305,
            },
        },
    },
}

local skills = {
	"stealth",
	"hacking",
	"inventory",
	"anarchy",
	"marksman"
}

local function chooseSkill( event, evType, evData, boardRig, hud, vizThread )
	local result = nil
	local agent = evData.unit
	
	local screen = mui.createScreen( "upgrade-dialog.lua" )
	MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_POPUP )
	hud._choice_dialog = screen
	mui.activateScreen( screen )

	if agent:getUnitData() and agent:getUnitData().file then
		file = agent:getUnitData().file
	end
	
	--screen:findWidget("agentName"):setText( util.toupper( agent:getName() ) )
	screen:findWidget("agentName"):setText( STRINGS.WIZARD.UI.CHOOSE_SKILL )
	screen:findWidget("apTxt"):setText( agent:getTraits().mpMax )

	local inventory ={}
	local augments = {}
	local invSize = agent:getTraits().inventoryMaxSize or 0 
	local augSize = agent:getTraits().augmentMaxSize or 0

	for i,childUnit in ipairs(agent:getChildren())do
		if childUnit:getTraits().augment and childUnit:getTraits().installed then
			table.insert(augments,childUnit)
		else
			table.insert(inventory,childUnit)
		end
	end

	for i, widget in screen.binder:forEach( "inv_" ) do
		if inventory[i] then
			guiex.updateButtonFromItem( screen, nil, widget, inventory[i],  agent )
			widget.binder.btn.onClick = nil
		elseif i > invSize then 
			widget:setVisible(false)
		else
			guiex.updateButtonEmptySlot( widget )
		end
	end	

	for i, widget in screen.binder:forEach( "aug_" ) do
		if augments[i] then
			guiex.updateButtonFromItem( screen, nil, widget, augments[i], agent )
			widget.binder.btn.onClick = nil
		elseif i > augSize then 
			widget:setVisible(false)
		else
			guiex.updateButtonEmptySlot( widget )
		end
	end	

	if agent:getUnitData().profile_anim then
		screen:findWidget("agentProfileImg"):setVisible(false)
		screen:findWidget("agentProfileAnim"):setVisible(true)
		screen:findWidget("agentProfileAnim"):bindBuild( agent:getUnitData().profile_build or agent:getUnitData().profile_anim )
		screen:findWidget("agentProfileAnim"):bindAnim( agent:getUnitData().profile_anim )
		if agent:isKO() or agent:getTraits().iscorpse then
			screen:findWidget("agentProfileAnim"):getProp():setRenderFilter( cdefs.RENDER_FILTERS.desat )
			screen:findWidget("agentProfileAnim"):setPlayMode( KLEIAnim.STOP )
		else
			screen:findWidget("agentProfileAnim"):getProp():setRenderFilter( nil )
			screen:findWidget("agentProfileAnim"):setPlayMode( KLEIAnim.LOOP )
		end
	else
		screen:findWidget("agentProfileImg"):setVisible(true)
		screen:findWidget("agentProfileAnim"):setVisible(false)
		screen:findWidget("agentProfileImg"):setImage( agent:getUnitData().profile_icon )	
	end

	local choiceIdx = 0
	for t, widget in screen.binder:forEach( "skill" ) do
		local skillDef
	
		while skillDef == nil and choiceIdx < #skills do
			choiceIdx = choiceIdx + 1
			local skillId = skills[choiceIdx]
			
			if agent:hasSkill( skillId ) or #(agent:getUnitData().skills or {}) == 0 or (agent:getUnitData().couldLearnSkill and agent:getUnitData().couldLearnSkill(agent, skillId) ) then
				skillDef = skilldefs.lookupSkill( skillId )
			end
		end

		if skillDef then
			widget:setVisible(true)
			
			local skillId = skills[choiceIdx]
			local skill = agent:hasSkill( skillId )
			local currentLevel = 0
			local spellLevel = evData.level
			if skill then
				currentLevel = skill._currentLevel
			else
				--spellLevel = spellLevel + 1
			end
			
			log:write(tostring(currentLevel))
			
			local tempPoints = 0
			if agent:getTraits().temp_skill_points then
				for p,tempskill in ipairs(agent:getTraits().temp_skill_points) do
					if tempskill == choiceIdx then
						tempPoints = tempPoints +1
					end
				end
			end
			
			if agent:getTraits().enhanceAbility and agent:getTraits().enhanceAbility.skillId == skillId then
				currentLevel = currentLevel - agent:getTraits().enhanceAbility.level
				tempPoints = tempPoints - agent:getTraits().enhanceAbility.level
			end
			
			----- Tooltip
			local tooltipBody = ""
			local actualLevel = 0
			
			for i = currentLevel + 1, currentLevel + spellLevel do
				--if skillDef[i] then
					actualLevel = actualLevel + 1
					
					if i > 1 then
						if #tooltipBody > 0 then
							tooltipBody = tooltipBody.."\n\n"
						end
						
						local txt
						local txtIdx = math.min(i, skillDef.levels)
						if skillDef[txtIdx] then
							txt = skillDef[txtIdx].tooltip
						end
						if evData.skillOverrides[skillId] then
							for j = txtIdx, i do
								if evData.skillOverrides[skillId][j] then
									txt = evData.skillOverrides[skillId][j].tooltip or txt
								end
							end
						end
						
						tooltipBody = tooltipBody..util.sformat(STRINGS.WIZARD.UI.SKILL_TOOLTIP_LEVEL,skillDef.name,i,txt)
					end
				--end
			end
			
			local tooltip = mui_tooltip( util.toupper(skillDef.name), tooltipBody )
			function tooltip:activate( screen )
				mui_tooltip.activate( self, screen )
				
				for i, bar in widget.binder:forEach( "metterBar" ) do
					if i > currentLevel and i <= currentLevel + actualLevel then					
						bar.binder.bar:setColor(TEMP_COLOR.r,TEMP_COLOR.g,TEMP_COLOR.b,TEMP_COLOR.a)
					end
				end
			end
			function tooltip:deactivate(  )
				mui_tooltip.deactivate( self )
				
				for i, bar in widget.binder:forEach( "metterBar" ) do
					if i > currentLevel and i <= currentLevel + actualLevel then					
						if (agent:getTraits().skillLock and agent:getTraits().skillLock[choiceIdx]) or skill == nil then
							bar.binder.bar:setColor(LOCKED_BLANK.r,LOCKED_BLANK.g,LOCKED_BLANK.b,LOCKED_BLANK.a)
						elseif i <= skillDef.levels then
							bar.binder.bar:setColor(BLANK_COLOR.r,BLANK_COLOR.g,BLANK_COLOR.b,BLANK_COLOR.a)
						else
							bar.binder.bar:setColor(0.1,0.1,0.1,1)
						end
					end
				end
			end
			----- End of tooltip
			
			----- Show the upgrade button
			--widget.binder.tipTitle:setText( util.toupper(skillDef.name) )
			local button = screen:createWidget( skillButton )
			local x0, y0 = widget.binder.tipTitle:getPosition()
			widget:addChild( button )
			button:setPosition( x0 + 80, y0 )
			button.onClick = util.makeDelegate( nil, function() result = skillId end )
			button:setText( util.toupper(skillDef.name) )
			button:setTooltip( tooltip )
			--button:setDisabled( actualLevel == 0 or (agent:getTraits().skillLock and agent:getTraits().skillLock[t]) )
			button:setDisabled( actualLevel == 0 )

			----- Show the current skill level
			for i, bar in widget.binder:forEach( "metterBar" ) do
				if i <= currentLevel then	
					if skill == nil then
						bar.binder.bar:setColor(LOCKED_COLOR.r,LOCKED_COLOR.g,LOCKED_COLOR.b,LOCKED_COLOR.a)
					elseif agent:getTraits().skillLock and agent:getTraits().skillLock[choiceIdx] then
						bar.binder.bar:setColor(LOCKED_COLOR.r,LOCKED_COLOR.g,LOCKED_COLOR.b,LOCKED_COLOR.a)
					else
						bar.binder.bar:setColor(SET_COLOR.r,SET_COLOR.g,SET_COLOR.b,SET_COLOR.a)
					end
					if i > currentLevel - tempPoints then
						bar.binder.bar:setColor(TEMP_COLOR.r,TEMP_COLOR.g,TEMP_COLOR.b,TEMP_COLOR.a)
					end
				elseif i <= skillDef.levels then
					if (agent:getTraits().skillLock and agent:getTraits().skillLock[choiceIdx]) or skill == nil then
						bar.binder.bar:setColor(LOCKED_BLANK.r,LOCKED_BLANK.g,LOCKED_BLANK.b,LOCKED_BLANK.a)
					else
						bar.binder.bar:setColor(BLANK_COLOR.r,BLANK_COLOR.g,BLANK_COLOR.b,BLANK_COLOR.a)
					end
				else
					bar.binder.bar:setColor(0.1,0.1,0.1,1)
				end


				if i <= skillDef.levels then
					bar.binder.cost:setVisible(false)
					bar.binder.txt:setVisible(true)
					bar.binder.txt:setText(skillDef[i].tooltip)	

					if i <= currentLevel then
						bar.binder.cost:setColor(0,0,0,1)
					else
						bar.binder.cost:setColor(140/255,1,1,1)
					end
				else
					bar.binder.cost:setVisible(false)
					bar.binder.txt:setVisible(false)				
				end		
			end
		else
			widget:setVisible(false)
		end
	end
	
	screen.binder.panel.binder.closeBtn.onClick = util.makeDelegate( nil, function() result = 0 end )
	screen.binder.panel.binder.closeBtn:setHotkey( "pause" )
	screen.binder.panel.binder.arrowLeft:setVisible( false )
	screen.binder.panel.binder.arrowRight:setVisible( false )
	
	-- We are running in the vizThread coroutine.  Yield until a response is chosen by the UI.
	-- Note that the click handler will be triggered by the main coroutine, but we use a closure
	-- to inform us what the chosen result is.
	while result == nil do
		coroutine.yield()
		result = result or modal_thread.checkAutoClose( hud, hud._game )
	end

	mui.deactivateScreen( screen )
	hud._choice_dialog = nil

	hud._game.simCore:setChoice( result )
end

return chooseSkill