----------------------------------------------------------------
-- Copyright (c) 2012 Klei Entertainment Inc.
-- All Rights Reserved.
-- SPY SOCIETY.
----------------------------------------------------------------

local util = include( "client_util" )
local cdefs = include( "client_defs" )
local guiex = include( "guiex" )
local array = include( "modules/array" )
local mui = include( "mui/mui")
local serverdefs = include( "modules/serverdefs" )
local simquery = include( "sim/simquery" )
local modalDialog = include( "states/state-modal-dialog" )
local inventory = include( "sim/inventory" )
local strings = include( "strings" )
local simdefs = include( "sim/simdefs" )
local mui_defs = include( "mui/mui_defs" )
local level = include( "sim/level" )

local items_panel = include( "hud/items_panel" ).base

--------------------------------------------------------------------

local function onClickTransferItem( panel, unit, targetUnit, item )
	local maxSize = 8
	if targetUnit:getTraits().hardInventoryLimit then
		maxSize = targetUnit:getTraits().inventoryMaxSize
	end
	
	local add = panel._originalOwner[item] ~= targetUnit
	if add and panel._transfersDone >= panel._maxTransfers then
		modalDialog.show(STRINGS.WIZARD.UI.MAX_TRANSFERS )
		return
	end
	
	if targetUnit:getInventoryCount() >= maxSize then
		modalDialog.show(STRINGS.UI.TOOLTIP_INVENTORY_FULL )
		return
	end
	
	local itemIndex = array.find( unit:getChildren(), item )
	panel._hud._game:doAction( "transferItem", unit:getID(), targetUnit:getID(), itemIndex )
	panel:refresh()
end

local function onClickClose( panel )
	MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_GAME_POPDOWN )
	local game = panel._hud._game
	panel:destroy()
end

local function setProfileImage( unit, panel )
	if unit and unit:getUnitData().profile_anim then

		panel.binder.profile:setVisible( true )
		panel.binder.agentProfileAnim:bindBuild( unit:getUnitData().profile_build or unit:getUnitData().profile_anim )
		panel.binder.agentProfileAnim:bindAnim( unit:getUnitData().profile_anim )
		panel.binder.agentProfileImg:setVisible( false )

		if unit:isKO() or unit:getTraits().iscorpse then
			panel.binder.agentProfileAnim:getProp():setRenderFilter( cdefs.RENDER_FILTERS.desat )
			panel.binder.agentProfileAnim:setPlayMode( KLEIAnim.STOP )
		else
			panel.binder.agentProfileAnim:getProp():setRenderFilter( nil )
			panel.binder.agentProfileAnim:setPlayMode( KLEIAnim.LOOP )
		end
		
	else
		panel.binder.profile:setVisible( false )
	end
end

local function setupWidgets( panelBinder, panel, unit )
	panelBinder.sell.binder.titleLbl:setText(STRINGS.UI.SHOP_INVENTORY)
	panelBinder.inventory.binder.titleLbl:setText(STRINGS.UI.SHOP_INVENTORY)
	panelBinder.inventory_bg.binder.closeBtn.onClick = util.makeDelegate( nil, onClickClose, panel )
	panelBinder.inventory_bg:setVisible(true)
	panelBinder.inventory:setVisible(true)
	panelBinder.shop_bg:setVisible(false)
	panelBinder.shop:setVisible(false)
end

-----------------------------------------------------------------------
-- Transfer panel (inventories between two units)

local transfer_panel = class( items_panel )

function transfer_panel:init( hud, unit, targetUnit, abilityOwner, ability )
	items_panel.init( self, hud, unit )

	self._unit = unit
	self._targetUnit = targetUnit
	self._ability = ability
	self._abilityOwner = abilityOwner
	self._transfersDone = 0
	self._maxTransfers = ability:getMaxTransfers()
	self._originalOwner = {}
	
	for i,childUnit in ipairs(targetUnit:getChildren()) do
		self._originalOwner[childUnit] = targetUnit
	end
	for i,childUnit in ipairs(unit:getChildren()) do
		self._originalOwner[childUnit] = unit
	end

	local panelBinder = self._screen.binder
	setupWidgets( panelBinder, self, unit )
	panelBinder.headerTxt:spoolText( util.toupper( util.sformat( STRINGS.WIZARD.UI.GIVE_COURIER, self._transfersDone, self._maxTransfers ) ) )
	setProfileImage( targetUnit, panelBinder.inventory )
end

function transfer_panel:refresh()
	local screen = self._screen
	local lootWidget = screen:findWidget( "inventory" )
    local userWidget = screen:findWidget( "sell" )
	
	self._transfersDone = 0
	-- Fill out the LOOT panel.
	self:refreshItems( self._targetUnit, self._unit, lootWidget )
    -- Fill out the UNIT's inventory.
	self:refreshItems( self._unit, self._targetUnit, userWidget )
	
    self:refreshCredits()
	self:refreshTransferCount()
end

function transfer_panel:refreshTransferCount()
	local txt = self._screen.binder.headerTxt
	txt:setText( util.toupper( util.sformat( STRINGS.WIZARD.UI.GIVE_COURIER, self._transfersDone, self._maxTransfers ) ) )
	if self._transfersDone == 0 then
		txt:setColor( 1, 1, 1, 1 )
	elseif self._transfersDone == self._maxTransfers then
		txt:setColor( 0.95, 0.25, 0.25, 1 )
	else
		txt:setColor( 0.95, 0.5, 0.25, 1 )
	end
end

function transfer_panel:refreshItems( fromUnit, toUnit, userWidget )
	local inventory = {}
	for i,childUnit in ipairs(fromUnit:getChildren()) do
		if not childUnit:getTraits().augment or not childUnit:getTraits().installed then
			table.insert(inventory,childUnit)
			
			if self._originalOwner[childUnit] ~= fromUnit then
				self._transfersDone = self._transfersDone + 1
			end
		end
	end

	for i, widget in userWidget.binder:forEach( "item" ) do 
		self:refreshUserItem( fromUnit, toUnit, inventory[i], widget, i )
	end
end

function transfer_panel:refreshUserItem( unit, toUnit, item, widget, i )
	if items_panel.refreshUserItem( self, unit, item, widget, i ) then
		local enabled,reason = true, nil
		
		if item:getTraits().pickupOnly and not toUnit:hasTag( item:getTraits().pickupOnly ) then
			enabled = false
			reason = util.sformat(STRINGS.UI.TOOLTIPS.PICK_UP_CONDITION_DESC, util.toupper(item:getTraits().pickupOnly) )
		end

		local onTransfer = util.makeDelegate( nil, onClickTransferItem, self, unit, toUnit, item )
		widget.binder.btn.onClick = onTransfer
		widget.binder.btn.onDragStart = util.makeDelegate( self, "onDragFromTop", item:getUnitData().profile_icon, onTransfer )
		
		if reason then
			widget.binder.btn:setDisabled(not enabled)
			widget.binder.btn:setTooltip(reason)
		end
	end

	return true
end

function transfer_panel:destroy()
	items_panel.destroy(self)

	if self._transfersDone > 0 then
		local abilityIndex = util.indexOf( self._abilityOwner:getAbilities(), self._ability )
		assert( abilityIndex, self._ability:getID() )
		self._hud._game:doAction( "abilityAction", self._abilityOwner:getID(), self._abilityOwner:getID(), abilityIndex, self._transfersDone )
	end
end

local function showCourier( event, evType, evData, boardRig, hud, vizThread )
	hud:showItemsPanel( transfer_panel( hud, evData.unit, evData.target, evData.abilityOwner, evData.ability ) )
end

return showCourier