local animmgr = include( "anim-manager" )
local util = include( "client_util" )
local cdefs = include( "client_defs" )
local mathutil = include( "modules/mathutil" )
local rig_util = include( "gameplay/rig_util" )
local unitrig = include( "gameplay/unitrig" )
local world_hud = include( "hud/hud-inworld" )
local flagui = include( "hud/flag_ui" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )

local misty_step_state = class( unitrig.base_state )

function misty_step_state:init( rig )
	unitrig.base_state.init( self, rig, "throw")
end

function misty_step_state:onEnter( ev )
	ev.thread.unblock = function( thread )
		thread.blocking = false
		thread.viz:releaseLocks( thread )
	end
	
	self._rig._throwState.onEnter( self, ev )
end

function misty_step_state:onSimEvent( ev, eventType, eventData )
	local unit = self._rig:getUnit()
	unitrig.base_state.onSimEvent( self, ev, eventType, eventData )

	if eventType == simdefs.EV_UNIT_REFRESH or eventType == simdefs.EV_UNIT_SEEN or eventType == simdefs.EV_UNIT_UNSEEN or eventType == simdefs.EV_UNIT_WARPED then
		-- Don't do a full refresh, we're in the middle of walking.
		self._rig:refreshHUD( self._rig:getUnit() )
		self._rig:refreshLocation()
        self._rig:refreshProp()

		return true
	end
end

local function mistyStep( event, eventType, eventData, boardRig, hud, vizThread )
	local rig = boardRig._unitRigs[ eventData.unitID ]
	
	if rig then
		vizThread:waitForLocks( eventData.unitID )
		
		if eventData.endThrow then
			while rig._mistyStepState and rig._state == rig._mistyStepState do
				coroutine.yield()
			end
			
			rig._mistyStepState = nil
		else
			rig._mistyStepState = misty_step_state( rig )
			rig:refreshLocation()
			rig:transitionUnitState( rig._mistyStepState, event )
		end
	end
end

return mistyStep