local viz_manager = include( "gameplay/viz_manager" )
local simdefs = include( "sim/simdefs" )

local oldInit = viz_manager.init
local oldProcessViz = viz_manager.processViz

local capturedEvent = {
	eventType = {}
}

function viz_manager:init( game )
	oldInit( self, game )
	
	self.moddedEvMap = {}
	self.eventMap[ capturedEvent.eventType ] = {}
	
	for i, handler in ipairs( mod_manager.vizEventHandlers ) do
		local evType = handler[1]
		local fn = handler[2]
		local captures = handler[3]
		
		if not self.moddedEvMap[evType] then
			self.moddedEvMap[evType] = {}
		end
		
		if captures then
			self.moddedEvMap[evType].captures = true
		end
		
		table.insert( self.moddedEvMap[evType], fn )
	end
end

function viz_manager:processViz( ev )
	local handlers = self.moddedEvMap[ev.eventType]
	if handlers then
		ev.viz = self
		for i, handler in ipairs( handlers ) do
			local closure = function( viz_thread, ev )
				handler( ev, ev.eventType, ev.eventData, self.game.boardRig, self.game.hud, viz_thread )
			end
		
			self:spawnViz( closure, ev )
		end
		
		if handlers.captures then
			local ret = oldProcessViz( self, capturedEvent )
			capturedEvent.viz = nil
			return ret
		end
	end

	return oldProcessViz( self, ev )
end