local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local wall_rig = include( "gameplay/wallrig2" )

local old_setRenderFilter = wall_rig.setRenderFilter
local old_refreshProp = wall_rig.refreshProp

function wall_rig:setRenderFilter( x, y, filter )
	local rf = cdefs.RENDER_FILTERS[filter]
	if self._pieces then
	for _,piece in pairs( self._pieces ) do
		if rf and piece.x == x and piece.y == y then
			self:setShader( piece.prop, rf.shader, rf.r, rf.g, rf.b, rf.a, rf.lum )
		elseif piece.x ==x and piece.y == y then
			self:setShader( piece.prop )
		end
	end end
end

function wall_rig:refreshProp()
	local gfxOptions = self._game:getGfxOptions()
	local offsetIdx
    	if gfxOptions.bMainframeMode then
        	offsetIdx = "mainframe"
    	elseif gfxOptions.bTacticalView then
        	offsetIdx = "tactical"
    	else
        	offsetIdx = "normal"
    	end
	local bShortWallMode = gfxOptions.bShortWallMode

	local camera_orientation = self._boardRig._game:getCamera():getOrientation()

	local cell = self._boardRig:getLastKnownCell( self:getLocation1() ) or self._boardRig:getLastKnownCell( self:getLocation2() )

	if  self._boardRig._game.simCore._showOutline then
		local cell1 = self._boardRig._game.simCore:getCell( self:getLocation1() )
		local cell2 = self._boardRig._game.simCore:getCell( self:getLocation2() )

		if not cell and (not cell1 or not cell2) then 
			cell = true
			offsetIdx = "tactical"
		end
	end

	if cell then
		local uvInfoOverload
		if offsetIdx == 'mainframe' then
			uvInfoOverload = cdefs.WALL_MAINFRAME
		end

		if self._pieces then
		for _,piece in pairs(self._pieces) do
			local prop, mesh = piece.prop, piece.mesh
			local offset,count = 0, 0
			if piece.geoOffsets and piece.geoOffsets[offsetIdx] then
				offset,count = unpack( piece.geoOffsets[offsetIdx] )
			end
			--if offsetIdx == "normal" then offset,count = 0,0 end
			mesh:setElementOffset( offset )
			mesh:setElementCount( count )
			
			self:setUVTransform( prop, uvInfoOverload or piece.UVInfo )


			if offsetIdx ~= 'mainframe' and (gfxOptions.bShortWallMode or gfxOptions.bTacticalView) then
				prop:setScl(1,1,cdefs.SHORT_WALL_SCALE)
			else
				prop:setScl(1,1,1)
			end

            		prop:setVisible( count > 0 )
			prop:scheduleUpdate()
		end end
	else
		if self._pieces then
		for _,piece in pairs(self._pieces) do
			local prop, mesh = piece.prop, piece.mesh
			mesh:setElementOffset( 0 )
			mesh:setElementCount( 0 )
            		prop:setVisible( false )
			prop:scheduleUpdate()
		end end
	end

	self:refreshRenderFilter()
end