local cdefs = include("client_defs")
local simdefs = include("sim/simdefs")

local wall_rig = include("gameplay/wallrig2")

function wall_rig:init( boardRig, x1, y1, simdir_1, wallVBO )
	local N,E,S,W = simdefs.DIR_N, simdefs.DIR_E, simdefs.DIR_S, simdefs.DIR_W
	
	local x2, y2 = x1, y1
	local simdir_2
	if simdir_1 == E then
		x2 = x1+1
		simdir_2 = W
	elseif simdir_1 == N then
		y2 = y1+1
		simdir_2 = S
	else
		assert( false )
	end

	local cellviz_1 = boardRig:getClientCellXY( x1, y1 )
	local geoOffsets_1 = cellviz_1._wallGeoInfo or {}
	table.insert( cellviz_1._dependentRigs, self )

	local cellviz_2 = boardRig:getClientCellXY( x2, y2 )
	local geoOffsets_2 = cellviz_2._wallGeoInfo or {}
	table.insert( cellviz_2._dependentRigs, self )

	self._boardRig = boardRig
	self._x1, self._y1 = x1, y1
	self._x2, self._y2 = x2, y2
	
	self._game = boardRig._game

	self._layer = boardRig:getLayer()

	local wallUV1 = (cellviz_1.tileIndex and cdefs.MAPTILES[ cellviz_1.tileIndex ].zone.wallUV) or cdefs.WALL_EXTERNAL
	local wallUV2 = (cellviz_2.tileIndex and cdefs.MAPTILES[ cellviz_2.tileIndex ].zone.wallUV) or cdefs.WALL_EXTERNAL
	
	local texFile1 = (cellviz_1.tileIndex and cdefs.MAPTILES[ cellviz_1.tileIndex ].zone.texFile) or cdefs.WALLTILES_FILE
	local texFile2 = (cellviz_2.tileIndex and cdefs.MAPTILES[ cellviz_2.tileIndex ].zone.texFile) or cdefs.WALLTILES_FILE

	local zx, zy = boardRig:cellToWorld( -0.5, -0.5 )

	local pieces = {}
	table.insert( pieces, { x = x1, y = y1, geoOffsets = geoOffsets_1[simdir_1], UVInfo = wallUV1, texFile = texFile1 } )
	table.insert( pieces, { x = x2, y = y2, geoOffsets = geoOffsets_2[simdir_2], UVInfo = wallUV2, texFile = texFile2 } )

	for _,piece in pairs(pieces) do
		local mt = MOAIMultiTexture.new()
		mt:reserve( 2 )
		mt:setTexture( 1, piece.texFile )
		mt:setTexture( 2, self._game.shadow_map )
	
		local mesh = MOAIMesh.new()
		mesh:setTexture( mt )
		mesh:setVertexBuffer( wallVBO )
		mesh:setPrimType( MOAIMesh.GL_TRIANGLES )
		mesh:setElementOffset( 0 )
		mesh:setElementCount( 0 )

		local prop = MOAIProp.new()
		prop:setDeck( mesh )
		prop:setShader( MOAIShaderMgr.getShader( MOAIShaderMgr.WALL_SHADER ) )
		prop:setDepthTest( false )
		prop:setDepthMask( false )
		prop:setCullMode( MOAIProp.CULL_NONE )
		prop:setLoc( zx, zy )

		self:setUVTransform( prop, piece.UVInfo )

		self._layer:insertProp( prop )
		piece.prop = prop
		piece.mesh = mesh
	end
    
    pieces[1].prop:setDebugName( string.format("wall-%d-%d-%d", x1, y1, simdir_1 ))
    pieces[2].prop:setDebugName( string.format("wall-%d-%d-%d", x2, y2, simdir_2 ))

    local cx, cy = boardRig:cellToWorld( (x1+x2)/2, (y1+y2)/2 )
    local W, H = self._game:getWorldSize()
    cx = cx - zx
    cy = cy - zy
     -- Omg this is magic.  These values are dependent on two things:
     -- (1) Walls joining at an intersection should not overlap.
     -- (2) Walls should not overlap with wall-decor bounds (which are tuned in animdefs to be SLIGHTLY in front of the wall)
    local THICKNESS, WIDTH = 1, 13.3
    if simdir_1 == E then
        pieces[1].prop:setWorldBounds( -THICKNESS + cx, -WIDTH + cy, 0, 0 + cx, WIDTH + cy, 64 )
        pieces[2].prop:setWorldBounds( cx, -WIDTH + cy, 0, THICKNESS + cx, WIDTH + cy, 64 )
    elseif simdir_1 == W then
        pieces[1].prop:setWorldBounds( 0 + cx, -WIDTH + cy, 0, THICKNESS + cx, WIDTH + cy, 64 )
        pieces[2].prop:setWorldBounds( -THICKNESS + cx, -WIDTH + cy, 0, 0 + cx, WIDTH + cy, 64 )
    elseif simdir_1 == N then
        pieces[1].prop:setWorldBounds( -WIDTH + cx, -THICKNESS + cy, 0, WIDTH + cx, 0 + cy, 64 )
        pieces[2].prop:setWorldBounds( -WIDTH + cx, 0 + cy, 0, WIDTH + cx, THICKNESS + cy, 64 )
    elseif simdir_1 == S then
        pieces[1].prop:setWorldBounds( -WIDTH + cx, 0 + cy, 0, WIDTH + cx, THICKNESS + cy, 64 )
        pieces[2].prop:setWorldBounds( -WIDTH + cx, -THICKNESS + cy, 0, WIDTH + cx, 0 + cy, 64 )
    end

	self._pieces = pieces
end