local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local unitdefs = include("sim/unitdefs")
local simfactory = include( "sim/simfactory" )
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )
local guarddefs = include("sim/unitdefs/guarddefs")

local disguise_tooltip = class( abilityutil.hotkey_tooltip )

function disguise_tooltip:init( hud, unit, range, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
end

function disguise_tooltip:activate( screen )
	abilityutil.hotkey_tooltip.activate( self, screen )

end

function disguise_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
end

local wilson_disguise = 
	{
		name = STRINGS.ABILITIES.DISGUISE,
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			return disguise_tooltip( hud, abilityUser, abilityOwner:getTraits().range, self, sim, abilityOwner, STRINGS.ABILITIES.DISGUISE_DESC )
		end,

		profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_hijack_small.png",
		alwaysShow = true,
		getName = function( self, sim, unit )
		
			if unit:getUnitOwner() and unit:getUnitOwner():getTraits().disguiseOn then
				return STRINGS.ABILITIES.DISGUISE_DEACTIVATE
			else
				return STRINGS.ABILITIES.DISGUISE

			end
		end,

		canUseAbility = function( self, sim, unit )
			-- Must have a user owner.
			local userUnit = unit:getUnitOwner()
			if not userUnit then
				return false
			end

			local canUseLabcoat = false
			if (sim:getStats().times_seen or 0) == 0 and sim:getTrackerStage( sim:getTracker() ) < 5 then
				canUseLabcoat = true
			end
				
			if unit:getTraits().restrictedUse then

				local userUnitAgentID = userUnit:getUnitData().agentID
				local canUse =false
				for i,set in pairs(unit:getTraits().restrictedUse )do
					if set.agentID == userUnitAgentID then
						canUse=true
					end
				end
				
				if not canUse then
					return false,  STRINGS.UI.REASON.RESTRUCTED_USE
				end				
			end
			
			if not canUseLabcoat then
				return false, "Cannot disguise"--STRINGS.AGENT_WILSON.UI.CANNOT_DISGUISE
			end
					
			return true
			-- return abilityutil.checkRequirements( unit, userUnit )
		end,

		executeAbility = function( self, sim, unit )
			local userUnit = unit:getUnitOwner()
			local labcoat_kanim = "kanim_wilson_labcoat"
			local canUseLabcoat = false
			
			if (sim:getStats().times_seen or 0) == 0 and sim:getTrackerStage( sim:getTracker() ) < 5 then
				canUseLabcoat = true
			end
			
				
			if userUnit:getTraits().disguiseOn then
				userUnit:changeKanim( "kanim_wilson", 0.05 )
				userUnit:setDisguise(false)
			else
			
				userUnit:setDisguise(true, "kanim_wilson")
				userUnit:changeKanim( labcoat_kanim, 0.05 )
				
				if sim:canPlayerSeeUnit( sim:getNPC(), userUnit ) then 
					userUnit:setDisguise(false)
				end
				
				if not canUseLabcoat then
					userUnit:setDisguise(false)
				end

				userUnit:resetAllAiming()
	
				inventory.useItem( sim, userUnit, unit )
			end
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )

		--	sim:processReactions(userUnit)			

		end,
		
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
			self.abilityOwner = unit:getUnitOwner()
			self.abilityUser = unit
		end,
		
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
		self.abilityOwner = self.abilityUser:getUnitOwner()
		local canUseLabcoat = false
			if (sim:getStats().times_seen or 0) == 0 and sim:getTrackerStage( sim:getTracker() ) < 4 then
				canUseLabcoat = true
			end
		
			if evType == simdefs.TRG_ALARM_STATE_CHANGE and self.abilityOwner then
				if self.abilityOwner:getTraits().disguiseOn and not canUseLabcoat then 
					self.abilityOwner:setDisguise(false)
				end
			elseif evType == simdefs.TRG_UNIT_WARP and evData.unit == self.abilityOwner then
				if not canUseLabcoat then
					self.abilityOwner:setDisguise(false)
				end
				
			-- elseif evType == simdefs.EV_UNIT_THROW then
				
				-- -- this is overkill but....
				-- local props = unitdefs.prop_templates
				-- if props.smoke_cloud_wilson then
				
				-- log:write("LOG smoke cloud wilson")
				-- log:write(util.stringize(props.smoke_cloud_wilson,2))
					-- props.smoke_cloud_wilson.traits.gasColor = props.smoke_cloud_wilson.traits.gascolours[sim:nextRand( 1, #props.smoke_cloud_wilson.traits.gascolours)]
				-- end
				
			end
		end,
		
	}
return wilson_disguise




