local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local simfactory = include( "sim/simfactory" )
local inventory = include("sim/inventory")
local unitdefs = include("sim/unitdefs")

local function canUseItem( itemdef, unit )
	-- log:write("LOG item")
	log:write(util.stringize(itemdef,2))
	local requirements = itemdef.requirements
	if requirements then
		for skill,level in pairs( requirements ) do
			if not unit:hasSkill(skill, level) and not unit:getTraits().useAnyItem then
				-- log:write("can't use")
				return false
			end 
		end	
	end
	
	return true
end

local function valueMatched( itemdef, value )
	local minValue = value - 100
	local maxValue = value + 150
	if (itemdef.value >= minValue) and (itemdef.value <= maxValue) then
		return true
	end
	return false
end

local function isBuyableItem(item_id) --do this check to preclude agent-unique items
	local simstore = include("sim/units/store")
	if simstore.STORE_ITEM.itemList and simstore.STORE_ITEM.weaponList and simstore.STORE_ITEM.augmentList then
		local buyableItems = {}
		util.tmerge(buyableItems, simstore.STORE_ITEM.itemList)
		util.tmerge(buyableItems, simstore.STORE_ITEM.weaponList)
		-- buyableItems = util.tcopy(simstore.STORE_ITEM.augmentList)
		for i, itemdef in pairs(buyableItems) do
			if (itemdef.id == item_id) and not itemdef.traits.MM_tech_expo_item then
				return true
			end
		end
	end
	return false
end

local tooltip = class( abilityutil.hotkey_tooltip )

function tooltip:init( hud, unit, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
end

local wilson_quantumrefurbisher =
	{
		name = STRINGS.AGENT_WILSON.ABILITIES.WILSON_TINKER,
		-- createToolTip = function( self,sim,unit,targetCell)
			-- if unit:getTraits().quantumRefurbisherValue then
				-- log:write(util.stringize(unit:getTraits().quantumRefurbisherValue,2))
				-- -- return abilityutil.formatToolTip( STRINGS.ABILITIES.WILSON_TINKER_TIP,  (util.sformat(STRINGS.ABILITIES.WILSON_TINKER_DESC, unit:getTraits().quantumRefurbisherValue)) )
				-- return (STRINGS.ABILITIES.WILSON_TINKER_TIP,  (util.sformat(STRINGS.ABILITIES.WILSON_TINKER_DESC, unit:getTraits().quantumRefurbisherValue)))
			-- end
		-- end,
		
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			local TOOLTIP = util.sformat( util.sformat(STRINGS.AGENT_WILSON.ABILITIES.WILSON_TINKER_DESC, abilityOwner:getTraits().quantumRefurbisherValue ) )
            return tooltip( hud, abilityUser, self, sim, abilityOwner, TOOLTIP )
		
		end,
		
		usesMP = true,

		--profile_icon = "gui/items/icon-item_ammo.png",
		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_technician_small.png",

		alwaysShow = true,

		confirmAbility = function( self, sim, ownerUnit )
            return STRINGS.AGENT_WILSON.CONFIRM_REFURBISH
        end,
		
		getName = function( self, sim, unit )
			return STRINGS.AGENT_WILSON.ABILITIES.WILSON_TINKER
		end,

		canUseAbility = function( self, sim, itemUnit, unit )
			if unit == nil then
				unit = itemUnit:getUnitOwner()
			end
			if not unit then
				return
			end

			if not (unit:countAugments( "augment_wilson_2" ) > 0) then
				return false, nil
			end
			
			if unit:getTraits().mp and unit:getTraits().mpMax and (unit:getTraits().mp < unit:getTraits().mpMax) then
				return false, STRINGS.AGENT_WILSON.ABILITIES.WILSON_TINKER_NOT_FULL_AP
			end

			return true
		end,
		
		executeAbility = function( self, sim, itemUnit, unit )
			local userUnit = itemUnit:getUnitOwner()
			local itemdefs = include("sim/unitdefs/itemdefs")
			local itemValue = itemUnit:getTraits().quantumRefurbisherValue or 0
			local possibleItems = {}
			for i, itemdef in pairs(itemdefs) do
				if itemdef.value and itemdef.traits and not (itemdef.traits.ammo_clip or itemdef.traits.installed) and valueMatched(itemdef, itemValue) and isBuyableItem(i) and canUseItem( itemdef, userUnit ) then
					table.insert(possibleItems, i)
				end
			end
			if #possibleItems > 0 then
				local randTemplate = possibleItems[sim:nextRand(1,#possibleItems)]
				local newItem = simfactory.createUnit( unitdefs.lookupTemplate(randTemplate), sim )
				-- fluff
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Weapons/LowBore/reload_handgun" )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/EMP_explo" )
					sim:dispatchEvent( simdefs.EV_UNIT_PLAY_ANIM, {unit= userUnit, anim="pick_up"} )
					-- sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 0.1 * cdefs.SECONDS )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/monst3r_jackin" )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
				--
				inventory.trashItem( sim, userUnit, itemUnit )
				sim:spawnUnit( newItem )
				userUnit:addChild( newItem )
				userUnit:useMP( unit:getMP(), sim )
			end
		end
	}
return wilson_quantumrefurbisher