local array = include( "modules/array" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
 local level = include( "sim/level" )
 
-------------------------------------------------------------------------
-- Trigger wireless onselect.

local PC_AGENT_SELECTED =
{
	uiEvent = level.EV_UNIT_SELECTED,
	fn = function( sim, unitID )
		local unit = sim:getUnit( unitID )
		if unit and unit:isPC() then
			return unit
		end
	end
}

local function onSelected(script, sim)
	while true do
		local _, unitID = script:waitFor( PC_AGENT_SELECTED )
		local unit = sim:getUnit( unitID )
		if unit:getTraits().seesShadowCreatures then
			sim:getTags().WilsonSelected = true
		else
			sim:getTags().WilsonSelected = nil
		end
		sim:forEachUnit(
			function( u )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = u } )
			end
		)
	end
	-- script:addHook(onSelected)
end


local wilson_shadowvisibility =
{
	name = "Shadow Visibility",
	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
        self.abilityOwner = unit:getUnitOwner()
        self.hook = sim:getLevelScript():addHook( "SHADOWVISIBILITY", onSelected, nil, unit )
		-- sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		self.alarmLevelChange = 0
	end,
        
	onDespawnAbility = function( self, sim, unit )
        self.abilityOwner = nil
        sim:getLevelScript():removeHook( self.hook )
		-- sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
	end,

    canUseAbility = function( self, sim, abilityOwner, abilityUser )
        return abilityutil.checkRequirements( abilityOwner, abilityUser )
    end,
	
	onTrigger = function( self, sim, evType, evData, userUnit )
		local params = sim:getParams()
		-- if evType == simdefs.TRG_UNIT_NEWTARGET then
			-- if evData.unit and not evData.unit:isPC() and not evData.unit:hasTrait("pacifist") and evData.target and (evData.target == self.abilityOwner) then
				-- if self.abilityOwner:getTraits().Wilson_paranoiaRange and (self.abilityOwner:getTraits().Wilson_paranoiaRange <= 3) then
					-- self.abilityOwner:getTraits().Wilson_paranoiaRange = self.abilityOwner:getTraits().Wilson_paranoiaRange + 0.5
				-- end
			-- end
		-- end
		if evType == simdefs.TRG_ALARM_STATE_CHANGE then
			-- self.alarmLevelChange = self.alarmLevelChange + 1
			-- if self.alarmLevelChange >= 2 then
				-- self.alarmLevelChange = 0
				-- if self.abilityOwner:getTraits().Wilson_paranoiaRange < 3 then
					-- self.abilityOwner:getTraits().Wilson_paranoiaRange = self.abilityOwner:getTraits().Wilson_paranoiaRange + 0.5
				-- end
			-- end
			
		elseif (evType == simdefs.TRG_UNIT_WARP) and evData.unit == self.abilityOwner then
			if evData.to_cell and evData.to_cell.units then
				for i, cellUnit in pairs(evData.to_cell.units) do
					if cellUnit:getTraits().Wilson_shadowMonster then
						sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = cellUnit, kanim = "fx/shadow_teleport_fx", anim="out", symbol = "effect", x = evData.to_cell.x, y = evData.to_cell.y, facingMask = KLEIAnim.FACING_W } )
						-- sim:dispatchEvent( simdefs.EV_PLAY_SOUND,"SpySociety/HUD/gameplay/peek_negative" )
						sim:warpUnit( cellUnit, nil )
						sim:despawnUnit(cellUnit)
					end
				end			
			end
		end
	end,
}

return wilson_shadowvisibility