local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local simfactory = include( "sim/simfactory" )
local unitdefs = include( "sim/unitdefs" )

-- HELPER FUNCTIONS
local function updateBeard( sim, unit, stage )
	local beardAnim = "portraits/wilson_face_beard"..tostring(stage)
	unit:getUnitData().profile_build = beardAnim
	if not unit:getTraits().rescued then --would clash with Prisoner Outfits
		if stage > 4 then -- fewer fullbody kanims for beard stages than portraits
			stage = 4
		end
		local beardKanim = "kanim_wilson_beard"..tostring(stage)
		if unit:getTraits().fancyLabcoat then
			beardKanim = "kanim_wilson_labcoat_beard"..tostring(stage)
		end
		unit:changeKanim(beardKanim)
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
	end
end

local function updateBeardStage( sim, unit, augment )
	local beardHours = augment:getTraits().beardTime
	if beardHours < 24 then
		updateBeard( sim, unit, 1 )--clean shaven
	elseif (beardHours >= 24) and (beardHours < 72) then
		updateBeard( sim, unit, 2 )--5 o'clock shadow
	elseif (beardHours >= 72) and (beardHours < 120) then
		updateBeard( sim, unit, 3 )--hairy
	elseif (beardHours >= 120) and (beardHours < 360) then
		updateBeard( sim, unit, 4 )--hairier	
	elseif beardHours >= 360 then
		updateBeard( sim, unit, 5 )--hairiest					
	end			
end

local tooltip = class( abilityutil.hotkey_tooltip )

function tooltip:init( hud, unit, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
end

local shave = 
	{
		name = STRINGS.AGENT_WILSON.ABILITIES.SHAVE,
        canUseWhileDragging = false,

		onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
			local TOOLTIP = STRINGS.AGENT_WILSON.ABILITIES.SHAVE_DESC
			return tooltip( hud, abilityUser, self, sim, abilityOwner, TOOLTIP )
		end,

		alwaysShow = true,
		-- iconColor= util.color( 163/255, 0/255, 0/255 ),
		-- iconColorHover= util.color( 1,1,1 )  ,
		usesMP = true,
		profile_icon = "gui/icons/small_icons/icon-WilsonShave.png",

		getName = function( self, sim, unit )
			return STRINGS.ABILITIES.LAST_WORDS
		end,

		-- acquireTargets = function( self, targets, game, sim, unit )
			-- return targets.unitTarget( game, { unit }, self, unit, unit )
		-- end,

		canUseAbility = function( self, sim, unit )
			local count, augments = unit:countAugments( "augment_wilson" )
			if not (count > 0) then
				count, augments = unit:countAugments( "augment_wilson_2" )
			end
			if not (count > 0) then
				return false, "INVALID"
			end
			if (count > 0) then			
				local augment = augments[1]
				if augment:getTraits().beardTime < 24 then
					return false, STRINGS.AGENT_WILSON.ABILITIES.SHAVE_TOO_LITTLE_HAIR
				end
			end
			if unit:getMP() < 2 then
				return false, string.format(STRINGS.UI.REASON.REQUIRES_AP,2)
			end			
			return true 
		end,

		executeAbility = function( self, sim, unit )
			local count, augments = unit:countAugments( "augment_wilson" )
			if not (count > 0) then
				count, augments = unit:countAugments( "augment_wilson_2" )
			end
			if (count > 0) and (unit:getUnitData().agentID and (unit:getUnitData().agentID == "agent_wilson_mod")) then
				local augment = augments[1]
				if augment:getTraits().beardTime >= 360 then
					augment:getTraits().beardTime = 120
				elseif augment:getTraits().beardTime >= 120 then
					augment:getTraits().beardTime = 72
				else
					augment:getTraits().beardTime = 0
				end
				updateBeardStage( sim, unit, augment )
				--spawn beard hair
				local beardHair = simfactory.createUnit( unitdefs.lookupTemplate("item_beardhair"), sim )
				local cell = sim:getCell(unit:getLocation())
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/equip" )
				sim:spawnUnit(beardHair)
				sim:warpUnit( beardHair, cell )
				unit:useMP( 2, sim )
			end
		end,
	}

return shave