local mui_tooltip = include( "mui/mui_tooltip" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simability = include( "sim/simability" )
local abilityutil = include( "sim/abilities/abilityutil" )

local spellSlotsPerLevel = {
	{2,0,0},
	{3,0,0},
	{3,1,0},
	{3,2,0},
	{3,2,1},
}

-- This is the ability that spawns in all other spell abilities
local wizard_arcane_focus = {
	name = STRINGS.WIZARD.ABILITIES.END_CONCENTRATION,
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
		return mui_tooltip( util.toupper( self:getName() ), util.sformat( STRINGS.WIZARD.ABILITIES.END_CONCENTRATION_DESC, self.concentrationSpell:getName( sim, abilityOwner, abilityUser ), self.concentrationDuration ) )
	end,

	getProfileIcon = function( self, sim, abilityOwner, abilityUser, hasTarget )
		return self.profile_icon
	end,

	profile_icon = "gui/icons/spells/end_concentration.png",
	--HUDpriority = 2,

	getName = function( self, sim, unit )
		return self.name
	end,
		
	onSpawnAbility = function( self, sim, unit )
		self.focus_item = unit
		sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
		local wizard = unit:getUnitOwner()
		local oldDoAugmentUpgrade = wizard.doAugmentUpgrade
		
		function wizard:doAugmentUpgrade(childUnit, useAbilityIndex)
			if childUnit:getTraits().modSkill == 3 then
				return
			end
			
			oldDoAugmentUpgrade(self, childUnit, useAbilityIndex)
		end
		
		if self.addSpellsOnSpawn and not self.hasSpellAbilities then
			self:addSpellAbilities( sim, unit )
		end
	end,
		
	onDespawnAbility = function( self, sim, wizard )
		for agentID, deployData in pairs(sim:getPC():getDeployed()) do
			if deployData.escapedUnit == wizard then
				self.escaped = true
			end
		end
	
		self:executeAbility( sim )
		sim:removeTrigger( simdefs.TRG_START_TURN, self )
		sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
	end,

	canUseAbility = function( self, sim, abilityOwner, abilityUser )
		if not self.concentrationSpell then
			return false
		end
		return true
	end,
	
	onTrigger = function( self, sim, evType, evData, focus_item )
		local wizard = focus_item:getUnitOwner()
		if evType == simdefs.TRG_START_TURN then
			if not self.hasSpellAbilities then
				self:addSpellAbilities( sim, focus_item )
			end
			
			if evData:isPC() and self.concentrationSpell then
				self.concentrationDuration = self.concentrationDuration - 1
				if self.concentrationDuration == 0 then
					self:executeAbility( sim )
				end
			end
		elseif wizard == evData.unit then
			self:executeAbility( sim )
		end
	end,
	
	getLevelOfSlot = function( self, slotIdx )
		local levels = spellSlotsPerLevel[#spellSlotsPerLevel]
		for i = 1, #levels do
			slotIdx = slotIdx - levels[i]
			if slotIdx <= 0 then
				return i
			end
		end
	end,
	
	addSpellAbilities = function( self, sim, focus_item )
		self.hasSpellAbilities = true
		self.spellAbilities = {}
		local wizard = focus_item:getUnitOwner()
		
		if wizard then
			table.insert( wizard._abilities, self )
			
			local magicSkill = wizard:getSkillLevel("magic")
			if magicSkill > 0 then
				-- Spawn spell abilities
			
				local spellIdx = 0
				self.HUDpriority = 10
				
				for spellLevel, spellSlotsOfLevel in ipairs( spellSlotsPerLevel[magicSkill] ) do
					for i = 1, spellSlotsOfLevel do
						spellIdx = spellIdx + 1
						local abilityID = focus_item:getTraits().spellsPrepared[ spellIdx ]
						self:spawnSpell( sim, wizard, spellIdx, abilityID, spellLevel )
					end
				end
			end
		end
	end,
	
	switchSpell = function( self, sim, focus_item, spellIdx, abilityID )
		local wizard = focus_item:getUnitOwner()
		
		if self.spellAbilities[ spellIdx ] then
			self.spellAbilities[ spellIdx ].spellSwitchedOut = true
		end
		
		focus_item:getTraits().spellsPrepared[ spellIdx ] = abilityID
		self:spawnSpell( sim, wizard, spellIdx, abilityID, self:getLevelOfSlot( spellIdx ) )
	end,
	
	rechargeSpell = function( self, sim, focus_item, spellIdx )
		assert( self.spellAbilities[ spellIdx ] )
		self:spawnSpell( sim, focus_item:getUnitOwner(), spellIdx, self.spellAbilities[ spellIdx ]:getID(), self:getLevelOfSlot( spellIdx ) )
	end,
	
	spawnSpell = function( self, sim, wizard, spellIdx, abilityID, spellLevel )
		if abilityID then
			local ability = simability.create( abilityID )
			if ability then
				self.spellAbilities[ spellIdx ] = ability
				ability.HUDpriority = 10 + spellIdx
				table.insert( wizard._abilities, ability )
				ability.spellSlotLevel = spellLevel
				ability.arcaneFocus = self
				ability:spawnAbility( sim, wizard )
			end
		end
	end,
	
	executeAbility = function( self, sim, abilityOwner, abilityUser )
		if self.concentrationSpell then
			self.concentrationSpell:endConcentration( sim, self.escaped )
			self.concentrationSpell = nil
			self.concentrationDuration = nil
		end
	end,
	
	startConcentration = function( self, sim, spell, duration )
		self:executeAbility( sim )
		self.concentrationSpell = spell
		self.concentrationDuration = duration
	end,
}

return wizard_arcane_focus