local cdefs = include("client_util")
local agent_actions = include( "hud/agent_actions" )
local hudtarget = include( "hud/targeting")
local mui_tooltip = include( "mui/mui_tooltip" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simability = include( "sim/simability" )
local abilityutil = include( "sim/abilities/abilityutil" )
local abilitydefs = include( "sim/abilitydefs" )
local simfactory = include( "sim/simfactory" )

local maxUpcast = {
	2,
	2,
	3,
	3,
	4
}

local spellRewards = {
	{minSkill = 1, minHours = 13, spellLevel = 1},
	{minSkill = 3, minHours = 30, spellLevel = 2},
	{minSkill = 4, minHours = 44, spellLevel = 2},
	{minSkill = 5, minHours = 64, spellLevel = 3},
	{minSkill = 5, minHours = 98, spellLevel = 3},
}

local upcastableSpellTarget = class()

function upcastableSpellTarget:updateButtonFromAbilityTarget( widget, ability, abilityOwner, abilityUser, manaCost, ... )
	local sim = self._game.simCore
	local enabled, reason = ability:canUseAbility( sim, abilityOwner, abilityUser )
	local targetArgs = {...}

	widget.binder.btn:setImage( "gui/icons/action_icons/icon-item_wizard_mana_"..tostring(manaCost)..".png" )
	widget.binder.btn:setDisabled( not enabled )
	widget.binder.btn:setHotkey( ability.hotkey )

	widget.binder.label:setText( util.toupper( ability:getName( sim, abilityOwner, abilityUser )))
	widget.binder.btn:setTooltip( function() return ability:onTooltip( self._hud, sim, abilityOwner, abilityUser ) end )

	widget.binder.btn.onClick = util.makeDelegate( nil,
		function( btn )
			if self.wrappedTargeting and self.wrappedTargeting.endTargeting then
				self.wrappedTargeting:endTargeting( self._hud )
			end
			
			for i, widget in self._hud._screen.binder.agentPanel.binder.spellPopup.binder:forEach( "action" ) do		
				if widget:isVisible() then
					widget:createTransition( "deactivate_below_popup",
							function( transition )
								widget:setVisible( false )
							end,
						{ easeOut = true } )
				end
			end
			
			self.wrappedTargeting = nil
			
			if ability.def.acquireTargets then
				self.wrappedTargeting = ability:acquireTargets( hudtarget, self._game, sim, abilityOwner, abilityUser )
				self.wrappedTargeting._manaSource = abilityOwner
				if not self:hasTargets() then
					self._hud:showWarning( STRINGS.UI.WARNING_NO_TARGETS )
					self.wrappedTargeting = nil
				elseif self.wrappedTargeting.startTargeting then
					self.wrappedTargeting:startTargeting( unpack(targetArgs) )
				end
			else
				agent_actions.performAbility( self._game, abilityOwner, abilityUser, ability )
				MOAIFmodDesigner.playSound( cdefs.SOUND_HUD_MENU_CONFIRM )
			end
		end )

	if enabled then
		widget.binder.img:setColor( 244/255, 255/255, 120/255 )
		widget.binder.btn:setColor( 244/255, 255/255, 120/255 )
		widget.binder.btn:setColorInactive( 244/255, 255/255, 120/255 )
		widget.binder.btn:setColorActive(255/255, 255/255, 255/255,1)
		widget.binder.btn:setColorHover(255/255, 255/255, 255/255,1)
	else
		widget.binder.btn:setColor(0.5,0.5,0.5,1)
		widget.binder.img:setColor(0.5,0.5,0.5,1)
	end
end

function upcastableSpellTarget:init( game, ability, wizard, minLevel, maxLevel )
	self._game = game
	self._hud = game.hud
	self._wizard = wizard
	self._ability = ability
	self._min = minLevel
	self._max = maxLevel
end

function upcastableSpellTarget:hasTargets()
	if self.wrappedTargeting and self.wrappedTargeting.hasTargets then
		return self.wrappedTargeting:hasTargets()
	else
		return true
	end
end

function upcastableSpellTarget:getDefaultTarget()
	return nil -- Spells never have a default target
end

function upcastableSpellTarget:getTooltip()
	if self.wrappedTargeting and self.wrappedTargeting.getTooltip then
		return self.wrappedTargeting:getTooltip()
	end
end

function upcastableSpellTarget:repeatStartTargeting()
	for i, widget in self._hud._screen.binder.agentPanel.binder.spellPopup.binder:forEach( "action" ) do		
		if widget:isVisible() then
			widget:createTransition( "deactivate_below_popup",
					function( transition )
						widget:setVisible( false )
					end,
				{ easeOut = true } )
		end
	end
	
	--self.transitioningOut = true
	self._hud:transitionNull()
	self._hud._screen.binder.agentPanel.binder.spellPopup:setVisible(true)
end

function upcastableSpellTarget:startTargeting( ... )
	if self._hud._state ~= self._hud.STATE_ABILITY_TARGET then
		return
	end
	
	if not self._hud.hasRepeatStartTargeting then
		self._hud.hasRepeatStartTargeting = true
		local transitionAbilityTarget = self._hud.transitionAbilityTarget
		
		function self._hud:transitionAbilityTarget( abilityOwner, abilityUser, ability, ... )
			if self._stateData and self._stateData.ability == ability and self._stateData.targetHandler and self._stateData.targetHandler.repeatStartTargeting then
				return self._stateData.targetHandler:repeatStartTargeting()
			else
				return transitionAbilityTarget( self, abilityOwner, abilityUser, ability, ... )
			end
		end
	end
	
	local sim = self._game.simCore
	local popUp = self._hud._screen.binder.agentPanel.binder.spellPopup
	popUp:setVisible( true )
	
	self._hud._screen.binder.agentPanel.binder.augmentGroup.binder.inventory_title.binder["New Widget"]._noInput = true
	self._hud._screen.binder.agentPanel.binder.augmentGroup.binder.inventory_title.binder["New Widget"]._cont._noInput = true
	self._hud._screen.binder.agentPanel.binder.actionsGroup.binder.actions_title.binder["New Widget 2"]._noInput = true
	self._hud._screen.binder.agentPanel.binder.actionsGroup.binder.actions_title.binder["New Widget 2"]._cont._noInput = true
	
	self._hud._screen.binder.agentPanel.binder.inventoryGroup:setVisible( false )
	
	for i, widget in popUp.binder:forEach( "action" ) do
		local manaCost = i + self._min - 1
		if manaCost <= self._max then
			widget:createTransition( "activate_below_popup" )
			widget:setVisible(true)
			
			self:updateButtonFromAbilityTarget( widget, self._ability, self._ability.arcaneFocus.spellUsers[manaCost], self._wizard, manaCost, ... )
		else
			widget:setVisible(false)
		end
	end
	
	local buttonWidget = nil
	for i, widget in self._hud._screen.binder.agentPanel.binder:forEach( "dynaction" ) do
		if widget:isVisible() and widget:getAlias() == self._ability:getID() then
			buttonWidget = widget
			break		
		end
	end

    if buttonWidget then
	    local lx,ly = buttonWidget:getPosition()
	    local lx1,ly1 = popUp:getPosition()
	    popUp:setPosition(lx,ly1)
    end
end

function upcastableSpellTarget:endTargeting( hud )
	--hud._world_hud:destroyWidgets( world_hud.HUD )
	self._hud._screen.binder.agentPanel.binder.inventoryGroup:setVisible( true )
	
	--if self.transitioningOut then
	--	self.transitioningOut = nil
	--else
		self._hud._screen.binder.agentPanel.binder.spellPopup:setVisible( false )
	--end
	
	if self.wrappedTargeting and self.wrappedTargeting.endTargeting then
		self.wrappedTargeting:endTargeting( hud )
		self.wrappedTargeting = nil
	end
end

function upcastableSpellTarget:onInputEvent( event )
	if self.wrappedTargeting and self.wrappedTargeting.onInputEvent then
		local target = self.wrappedTargeting:onInputEvent( event )
		
		if target then
			agent_actions.performAbility( self._game, self.wrappedTargeting._manaSource, self._wizard, self._ability, target )
		end
	end
end

function upcastableSpellTarget:onDraw()
	if self.wrappedTargeting and self.wrappedTargeting.onDraw then
		return self.wrappedTargeting:onDraw()
	end
end

local upcastable_spell
upcastable_spell = {
	usesAction = true,
	noDefaultTarget = true,
	triggersOverwatch  = true,
	showTargets = true,
	
	getName = function( self, sim, abilityOwner, abilityUser, targetUnitID )
		self:updateManaCost( abilityOwner, abilityUser )
		return self.def.getName( self )
	end,
	
	getProfileIcon = function( self, sim, abilityOwner, abilityUser, hasTarget )
		return self.profile_icon
	end,
	
	getID = function( self )
		return "upcastable_"..self.spellId
	end,
	
	getDef = function()
		return upcastable_spell
	end,
	
	updateManaCost = function( self, abilityOwner, abilityUser )
		if abilityUser == nil then
			return
		end
		
		if abilityOwner == abilityUser then
			self.spellSlotLevel = nil
		else
			self.spellSlotLevel = abilityOwner:getTraits().manaCost
		end
	end,
	
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, ... )
		self:updateManaCost( abilityOwner, abilityUser )
		return self.def.onTooltip( self, hud, sim, abilityOwner, abilityUser, ... )
	end,
	
	acquireTargets = function( self, targets, game, sim, abilityOwner, abilityUser )
		if abilityOwner == abilityUser then
			return upcastableSpellTarget( game, self, abilityUser, self.def.spellLevel, maxUpcast[abilityUser:getSkillLevel("magic2")] )
		else
			self.spellSlotLevel = abilityOwner:getTraits().manaCost
			return self.def.acquireTargets( self, targets, game, sim, abilityOwner, abilityUser )
		end
	end,
	
	canUseAbility = function( self, sim, abilityOwner, abilityUser, ... )
		local mana = self.arcaneFocus:getMana()
		self:updateManaCost( abilityOwner, abilityUser )
		if mana < self:getLevel() then
			return false, util.sformat(STRINGS.WIZARD.UI.NO_MANA, mana, self:getLevel())
		end
		
		return self.def.canUseAbility( self, sim, abilityUser, abilityUser, ... )
	end,
	
	spawnAbility = function( self, sim, wizard, spellId )
		self.spellId = spellId
		self.def = abilitydefs._abilities[ spellId ]
	end,
	
	despawnAbility = function( self, sim, wizard )
	
	end,
	
	executeAbility = function( self, sim, abilityOwner, userUnit, ... )
		local spell = simability.create( self.spellId )
		
		spell.manaCost = abilityOwner:getTraits().manaCost
		spell.spellSlotLevel = abilityOwner:getTraits().manaCost
		spell.HUDpriority = self.HUDpriority + 0.5
		table.insert( userUnit._abilities, spell )
		spell.arcaneFocus = self.arcaneFocus
		spell:spawnAbility( sim, userUnit )
		
		spell:executeAbility( sim, userUnit, userUnit, ... )
		spell.spellSlotExpended = true -- Some spells can be cancelled, but we don't want them to stick around in our hotbar
	end,
}

-- This is the ability that spawns in all other spell abilities
local wizard_arcane_focus_archived = util.extend( abilitydefs._abilities["wizard_arcane_focus"] ) {
	addSpellsOnSpawn = true,
	
	addSpellAbilities = function( self, sim, focus_item )
		self.hasSpellAbilities = true
		self.focus_item = focus_item
		self.spellAbilities = {}
		local wizard = focus_item:getUnitOwner()
		
		if wizard then
			table.insert( wizard._abilities, self )
			
			local magicSkill = wizard:getSkillLevel("magic2")
			if magicSkill > 0 then
				-- Spawn spell abilities
				
				self.spellUsers = {}
				
				for i = 1, maxUpcast[magicSkill] do
					local unitData = {
						type = "simunit",
						icon = "",
						name = "MANA SOURCE",
						traits = { manaCost = i },
					}
					
					local newUnit = simfactory.createUnit( unitData, sim )
					sim:spawnUnit( newUnit )
					self.spellUsers[i] = newUnit
					newUnit._abilities = self.spellAbilities
				end
				
				focus_item:getTraits().usesCharges = true
				focus_item:getTraits().charges = 2 * magicSkill
				focus_item:getTraits().chargesMax = 2 * magicSkill
				
				-- Initialize with 1 random spell
				local startSpellCount = 1
				if #focus_item:getTraits().spellsKnown == 0 then
					local spells = self:getRandomSpells( sim, focus_item, startSpellCount, 1 )
					focus_item:getTraits().spellsKnown = spells
					table.insert(focus_item:getTraits().pastRewards, spells)
				--end
				
				-- Offer 2 new spells as a reward
				elseif focus_item:getTraits().nextReward == nil then
					local nextReward = spellRewards[focus_item:getTraits().nextRewardCount]
					if nextReward and nextReward.minSkill <= magicSkill and nextReward.minHours <= sim:getParams().campaignHours then
						local spells = self:getRandomSpells( sim, focus_item, 2, nextReward.spellLevel, nextReward.spellLevel )
						focus_item:getTraits().nextReward = spells
						focus_item:getTraits().nextRewardCount = focus_item:getTraits().nextRewardCount + 1
					end
				end
				
				local spellIdx = 0
				self.HUDpriority = 10
				
				for i, abilityID in ipairs(focus_item:getTraits().spellsKnown) do
					spellIdx = spellIdx + 1
					self:spawnSpell( sim, wizard, focus_item, spellIdx, abilityID )
				end
			end
		end
	end,
	
	spawnSpell = function( self, sim, wizard, focus_item, spellIdx, abilityID )
		if abilityID then
			local ability = util.tcopy( abilitydefs._abilities[ abilityID ] )
			util.tmerge( ability, upcastable_spell )
			ability.def = abilitydefs._abilities[ abilityID ]
			
			self.spellAbilities[ spellIdx ] = ability
			ability.HUDpriority = 10 + spellIdx
			table.insert( wizard._abilities, ability )
			--ability.spellSlotLevel = spellLevel
			ability.arcaneFocus = self
			ability.focus_item = focus_item
			ability:spawnAbility( sim, wizard, abilityID )
		end
	end,
	
	getRandomSpells = function( self, sim, focus_item, count, maxLevel, minLevel )
		return abilitydefs._abilities["wizard_spellbook"].getRandomSpells( self, sim, focus_item, count, maxLevel, minLevel )
	end,
	
	getMana = function( self )
		return self.focus_item:getTraits().charges
	end,
	
	useMana = function( self, manaCost )
		self.focus_item:getTraits().charges = self.focus_item:getTraits().charges - manaCost
	end,
	
	restoreMana = function( self, manaRestored, unlimited )
		self.focus_item:getTraits().charges = self.focus_item:getTraits().charges + manaRestored
		
		if not unlimited then
			self.focus_item:getTraits().charges = math.min(self.focus_item:getTraits().charges, self.focus_item:getTraits().chargesMax)
		end
	end
}

return wizard_arcane_focus_archived