local simdefs = include("sim/simdefs")
local simfactory = include( "sim/simfactory" )
local abilitydefs = include( "sim/abilitydefs" )
local itemdefs = include( "sim/unitdefs/itemdefs" )
local abilityutil = include( "sim/abilities/abilityutil" )
local util = include("modules/util")

local wizard_readbook = {
	name = STRINGS.WIZARD.ABILITIES.READ_SWAP,
	usesMP = true,
	profile_icon = "gui/icons/action_icons/icon-item_wizard_read.png",
	--triggersOverwatch  = true,
	getName = function( self, sim, unit )
		return self.name
	end,
	createToolTip = function( self,sim, unit )
		return abilityutil.formatToolTip(STRINGS.WIZARD.ABILITIES.READ_SWAP, STRINGS.WIZARD.ABILITIES.READ_SWAP_DESC, 0)
	end,
		
	onSpawnAbility = function( self, sim, unit )
	end,
		
	onDespawnAbility = function( self, sim, unit )
	end,

	canUseAbility = function( self, sim, abilityOwner, abilityUser )
		if abilityOwner:getUnitOwner() ~= abilityUser then
			return false
		end
		
		local focus = abilityUser:ownsAbility("wizard_arcane_focus")
		if not focus then
			return false
		end
		
		if abilityOwner:getTraits().usesCharges and abilityOwner:getTraits().charges < 1 then
			return false, STRINGS.WIZARD.UI.CANT_USE_BOOKREAD
		end
		
		return true
	end,
	
	executeAbility = function( self, sim, abilityOwner, abilityUser )
		local spellsKnown = abilityOwner:getTraits().spellsKnown
		local focus = abilityUser:ownsAbility("wizard_arcane_focus")
		local spellsPrepared = focus.focus_item:getTraits().spellsPrepared
		local expendedSpells = {}
		
		for k, spell in pairs( focus.spellAbilities ) do
			if spell.spellSlotExpended then
				expendedSpells[k] = spell
			end
		end
		
		local choice = sim:dispatchChoiceEvent( "EV_WIZ_BOOK_SWAP", { unit = abilityUser, spellsKnown = spellsKnown, spellsPrepared = spellsPrepared, expendedSpells = expendedSpells } )
		
		if choice and choice ~= 0 then
			abilityOwner:getTraits().charges = abilityOwner:getTraits().charges - 1
			abilityUser:useMP( choice.apCost, sim )
			--abilityUser:useAP( sim )
			focus:switchSpell( sim, focus.focus_item, choice.slotIndex, choice.spellId )
		end
	end,
}

return wizard_readbook