local util = include( "modules/util" )
local array = include( "modules/array" )
local simunit = include( "sim/simunit" )
local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local simfactory = include( "sim/simfactory" )
local inventory = include( "sim/inventory" )
local abilitydefs = include( "sim/abilitydefs" )
local itemdefs = include( "sim/unitdefs/itemdefs" )

-----------------------------------------------------
local spellScroll = { ClassType = "wizard_spellscroll" }

function spellScroll:getName()
	local spell = abilitydefs.lookupAbility( self:getTraits().storedSpell )

	if self:getTraits().hiddenScroll then
		return util.sformat( STRINGS.WIZARD.ITEMS.SCROLL_LVL, spell.spellLevel )
	else
		return util.sformat( STRINGS.WIZARD.ITEMS.SCROLL_OF, spell:getName() )
	end
end

-----------------------------------------------------

local freeSpellScroll = { ClassType = "wizard_free_spellscroll" }

function freeSpellScroll:onSpawn( sim )
	local wizard = self:getUnitOwner()
	
	if wizard then
		local bookAbility, book = wizard:ownsAbility("wizard_spellbook")
		local focusAbility, focus = wizard:ownsAbility("wizard_arcane_focus")
		
		if book then
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, wizard )
		
			local spells = bookAbility:getRandomSpells( sim, book, self:getTraits().freeWizardSpells, 1 )
			util.tmerge( book:getTraits().spellsKnown, spells )
			if focus then
				util.tmerge( focus:getTraits().spellsPrepared, spells )
			end
		end
	end
end

function freeSpellScroll:onTrigger( sim, evType, evData, wizard )
	wizard:removeChild( self )
	sim:despawnUnit( self )
    sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
end

-----------------------------------------------------
-- Interface functions

local function createSpellScroll( unitData, sim )
	return simunit.createUnit( unitData, sim, spellScroll )
end

local function createFreeSpellScroll( unitData, sim )
	return simunit.createUnit( unitData, sim, freeSpellScroll )
end

local function createSpellBook( unitData, sim )
	updateBookValue( unitData )
	return simunit.createUnit( unitData, sim, spellBook )
end

simfactory.register( createSpellScroll )
simfactory.register( createFreeSpellScroll )