local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local CARMEN_crack_safe =
{
	name = STRINGS.CARMEN.ABILITIES.CRACK_SAFE,
	profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_heist_small.png",
    	proxy = true,

	createToolTip = function( self,sim, abilityOwner, abilityUser, targetID )
		local targetUnit = sim:getUnit( targetID )
		local headerTxt = string.format( STRINGS.CARMEN.ABILITIES.CRACK_SAFE, util.toupper( targetUnit:getName() ) )
		local bodyTxt = util.sformat( STRINGS.CARMEN.ABILITIES.CRACK_SAFE_TIP, abilityUser:getSkillLevel("anarchy") or 1 )
		return abilityutil.formatToolTip( headerTxt, bodyTxt, simdefs.DEFAULT_COST )
	end,
		
	getName = function( self, sim, unit )
		return self.name
	end,

	getProfileIcon = function( self, sim, abilityOwner )
		return abilityOwner:getUnitData().profile_icon or self.profile_icon
	end,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
        	assert( userUnit, tostring(unit and unit:getName())..", "..tostring(unit:getLocation()) )
		local cell = sim:getCell( userUnit:getLocation() )
        	local cells = { cell }
		for dir, exit in pairs(cell.exits) do
            		if simquery.isOpenExit( exit ) then
                		table.insert( cells, exit.cell )
            		end
        	end

		local units = {}
		for i, cell in pairs(cells) do
			for _, cellUnit in ipairs( cell.units ) do
				if (cellUnit:getTraits().mainframe_ice or 0) > 0 and cellUnit:getTraits().safeUnit then
					if cellUnit:getTraits().mainframe_status ~= "off" and not cellUnit:getTraits().mainframe_booting then
						table.insert( units, cellUnit )
					end
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,
		
	canUseAbility = function( self, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return false
		end
		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown)
		end
		
        	return abilityutil.checkRequirements( unit, userUnit )
	end,
		
	executeAbility = function( self, sim, unit, userUnit, target )
		local mainframe = include( "sim/mainframe" )
		local userUnit = unit:getUnitOwner()
		local target = sim:getUnit(target)			
			
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = target:getLocation()
  		local newFacing = simquery.getDirectionFromDelta(x1-x0,y1-y0) 

		sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = userUnit:getID(), targetID = target:getID(), facing = newFacing, sound = "SpySociety/Actions/monst3r_jackin" , soundFrame = 16, useTinker=true } )
	
		local iceCount = userUnit:getSkillLevel("anarchy") or 1
		mainframe.breakIce( sim, target, math.min( target:getTraits().mainframe_ice, iceCount ) )

		inventory.useItem( sim, userUnit, unit )

		if userUnit:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
		end
	end,
}

return CARMEN_crack_safe