local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local simfactory = include( "sim/simfactory" )
local cdefs = include( "client_defs" )
local unitdefs = include("sim/unitdefs")
local serverdefs = include( "modules/serverdefs" )
local mainframe_common = include("sim/abilities/mainframe_common")
local speechdefs = include( "sim/speechdefs" )
local modifiers = include( "sim/modifiers" )
local level = include( "sim/level" )
local mainframe = include( "sim/mainframe" )
local mission_util = include( "sim/missions/mission_util" )
-------------------------------------------------------------------------------
-- These are NPC abilities.

local createDaemon = mainframe_common.createDaemon
local createReverseDaemon = mainframe_common.createReverseDaemon
local createCountermeasureInterest = mainframe_common.createCountermeasureInterest

local function guardLooted( script, sim )
	local UI_LOOT_CLOSED =
    	{
        	uiEvent = level.EV_CLOSE_LOOT_UI,
        	fn = function( sim )
            		return true
        	end
	}

	local i, triggerData = script:waitFor(UI_LOOT_CLOSED)
    	script:queue( 1*cdefs.SECONDS )
	sim:triggerEvent( simdefs.TRG_UI_ACTION, {uiEvent = level.EV_CLOSE_LOOT_UI} )
    	script:addHook( guardLooted, true )
end

local subroutines =
{
	PWR =
	{
		{
			name = STRINGS.PROGEXTEND.AI.ALARM_SYNCHRONIZER,
			desc = STRINGS.PROGEXTEND.AI.ALARM_SYNCHRONIZER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_alarm_synchronizer.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = math.ceil(math.min(sim:getTrackerStage(sim:getTracker())/4, 2))

					if pwr > 0 then
						sim:getNPC():addCPUs( pwr )
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.ALARM_SYNCHRONIZER_DESC, pwr), nil, nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PWR_AMPLIFIER,
			desc = STRINGS.PROGEXTEND.AI.PWR_AMPLIFIER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_pwr_amplifier.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,
			requiresGens = true,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and evData:isNPC() and slot.cooldown <= 0 then
					if self.countGens(self, sim) > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PWR_AMPLIFIER_DESC, 1), nil, nil )
						sim:getNPC():addCPUs( 1 )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.FEEDBACK_GRID,
			desc = STRINGS.PROGEXTEND.AI.FEEDBACK_GRID_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_feedback_grid.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = 0
					local total = 0

					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().mainframe_status == "active" and unit:getTraits().mainframe_ice then
							if unit:getTraits().mainframe_ice > 0 then
								pwr = pwr + 1
							end
							total = total + 1
						end
					end

					pwr = math.floor((pwr*2 + 5) / total)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.FEEDBACK_GRID_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( math.ceil(pwr) )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.RAID,
			desc = STRINGS.PROGEXTEND.AI.RAID_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_raid.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = math.min(math.max(sim:getPC():getCpus() - sim:getNPC():getCpus(),0),2)
					
					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.RAID_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SWITCH,
			desc = STRINGS.PROGEXTEND.AI.SWITCH_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_switch.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = (sim:getTrackerStage(sim:getTracker()) % 2) * 2

					if pwr > 0 then
						sim:getNPC():addCPUs( pwr )
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SWITCH_DESC, pwr), nil, nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SOUNDWEAVE,
			desc = STRINGS.PROGEXTEND.AI.SOUNDWEAVE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_soundweave.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "sankaku",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = 0

					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().hasHearing and not unit:getTraits().isGuard and unit:getTraits().mainframe_status == "active" and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
							pwr = pwr + 0.5
						end
					end
					pwr = math.floor(pwr)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SOUNDWEAVE_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.OPTIC_DOWNLINK,
			desc = STRINGS.PROGEXTEND.AI.OPTIC_DOWNLINK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_optic_downlink.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "ftm",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = 0

					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().mainframe_camera and unit:getTraits().mainframe_status == "active" and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
							pwr = pwr + 0.5
						end
					end
					pwr = math.floor(pwr)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.OPTIC_DOWNLINK_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( math.floor(pwr) )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.DAEMON_CLUSTER,
			desc = STRINGS.PROGEXTEND.AI.DAEMON_CLUSTER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_daemon_cluster.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "plastech",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = 0

					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().mainframe_program and unit:getTraits().mainframe_status == "active" and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
							pwr = pwr + 0.334
						end
					end
					pwr = math.floor(pwr)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.DAEMON_CLUSTER_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.COUNTERWEIGHT,
			desc = STRINGS.PROGEXTEND.AI.COUNTERWEIGHT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_counterweight.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "neptune",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = 0

					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().plateType and unit:getTraits().mainframe_status == "active" and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
							pwr = pwr + 0.5
						end
					end
					pwr = math.floor(pwr)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.COUNTERWEIGHT_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.CYCLONE,
			desc = STRINGS.PROGEXTEND.AI.CYCLONE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_cyclone.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "ko",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_UNIT_WARP and sim:getCurrentPlayer() == sim:getPC() and evData.unit and evData.unit:getTraits().isAgent then
					self.CyclonePWR = 0
					for i, unit in pairs(sim:getPC():getUnits()) do
						if unit:getMP() and unit:getMPMax() then
							self.CyclonePWR = self.CyclonePWR + math.floor((unit:getMPMax() - unit:getMP())/10)
						end
					end
				end
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and self.CyclonePWR and slot.cooldown <= 0 then
					if self.CyclonePWR > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.CYCLONE_DESC,self.CyclonePWR), nil, nil )
						sim:getNPC():addCPUs( self.CyclonePWR )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TERMINAL_LINK,
			desc = STRINGS.PROGEXTEND.AI.TERMINAL_LINK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_terminal_link.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 7,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = 0

					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().mainframe_console and unit:getTraits().mainframe_status == "active" and not unit:getTraits().hijacked then
							pwr = pwr + 0.5
						end
					end
					pwr = math.floor(pwr)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.TERMINAL_LINK_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.LEECH,
			desc = STRINGS.PROGEXTEND.AI.LEECH_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_leech.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "general",
			minDifficulty = 7,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local pwr = math.min(1,sim:getPC():getCpus())

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.LEECH_DESC,pwr, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:getPC():addCPUs( -pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.DIRECT_FEED,
			desc = STRINGS.PROGEXTEND.AI.DIRECT_FEED_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_direct_feed.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_ICE_BROKEN and evData.delta and slot.cooldown <= 0 then
					local pwr = math.max( -evData.delta, 0)

					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.DIRECT_FEED_DESC, pwr), nil, nil )
						sim:getNPC():addCPUs( pwr )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.GENERATOR,
			desc = STRINGS.PROGEXTEND.AI.GENERATOR_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_generator.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.GENERATOR_DESC, 2), nil, nil )
					sim:getNPC():addCPUs( 2 )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BIO_COPROCESSORS,
			desc = STRINGS.PROGEXTEND.AI.BIO_COPROCESSORS_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PWR,
			icon = "gui/icons/ai_icons/icon-ai_biocoprocessors.png",
			iconType = "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png",
			type = "PWR",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,

			onTrigger = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.cooldown <= 0 then
					local guards = 0
					for i, unit in pairs(evData:getUnits()) do
						if unit:hasTrait("isGuard") and not unit:isDown() then
							guards = guards + 1
						end
					end

					if guards >= 3 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.BIO_COPROCESSORS_DESC, math.floor(guards/3)), nil, nil )
						sim:getNPC():addCPUs( math.floor(guards/3) )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
					end
				end
			end,
		},
	},	

	PROACTIVE =
	{
		{
			name = STRINGS.PROGEXTEND.AI.BUCKLER,
			desc = STRINGS.PROGEXTEND.AI.BUCKLER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_buckler.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 2,
			raiseIce = 1,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, sim:getParams().difficulty + 2, "LOWEST" )

				if device then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.BUCKLER_DESC, device:getName(), slot.raiseIce), nil, device )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					device:increaseIce(sim, slot.raiseIce)
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 120)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SAFE_MODE,
			desc = STRINGS.PROGEXTEND.AI.SAFE_MODE_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.SAFE_MODE_SPECIAL,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_safe_mode.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 2,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				local dist = 100
				for _, shop in pairs(sim:getAllUnits()) do
					if shop:getTraits().storeType then
						local x1, y1 = shop:getLocation()
						for _, agent in pairs(sim:getPC():getUnits()) do
							local x2, y2 = agent:getLocation()
							if agent:getMP() and x1 and x2 then
								local dt = mathutil.dist2d(x1, y1, x2, y2)
								if dt < dist then dist = dt end
							end
						end
					end
				end
				if dist >= 15 or (dist > 8 and dist < 15 and sim:nextRand(0, dist) > 4) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local raised = false
				for i, unit in pairs( sim:getAllUnits() ) do
					local x1, y1 = unit:getLocation()
					if unit:getTraits().storeType and sim:getPC():getLastKnownCell( sim, x1, y1 ) ~= nil then
						raised = true
						unit:processEMP( 3, true, false, true )
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit })
					end
				end

				if raised then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.SAFE_MODE_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 120)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BLOWFISH,
			desc = STRINGS.PROGEXTEND.AI.BLOWFISH_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/daemon_icons/Daemons00010.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if sim:getTrackerStage(sim:getTracker()) < simdefs.TRACKER_MAXSTAGE then
					local alarm = 1
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.BLOWFISH_DESC,alarm), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 20)
					sim:trackerAdvance( alarm )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 120)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.ECHO,
			desc = STRINGS.PROGEXTEND.AI.ECHO_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/daemon_icons/Daemons_echo.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice <= 0 then
						self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.ECHO_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 120)
						sim:triggerEvent(simdefs.TRG_RECAPTURE_DEVICES, { reboots = 1 } )
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
						break
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SEMAPHORE,
			desc = STRINGS.PROGEXTEND.AI.SEMAPHORE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_semaphore.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local validPrograms = {}
				for i, ability in pairs(sim:getPC():getAbilities()) do
					if not ability.passive and (ability.cooldown or 0) <= 0 then
						table.insert(validPrograms, ability)
					end
				end
				if #validPrograms <= 0 then
					for i, ability in pairs(sim:getPC():getAbilities()) do
						if not ability.passive then
							table.insert(validPrograms, ability)
						end
					end
				end

				if #validPrograms > 0 then
					local choice = sim:nextRand(1, #validPrograms)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SEMAPHORE_DESC, validPrograms[choice].name), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					validPrograms[choice].cooldown = (validPrograms[choice].cooldown or 0) + 2
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PINPOINT,
			desc = STRINGS.PROGEXTEND.AI.PINPOINT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_pinpoint.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local agents = {}
				for i, unit in pairs( sim:getPC():getUnits() ) do
					if unit:getMP() then
						table.insert(agents, unit)
					end
				end

				if #agents > 0 then
					local targetUnit = agents[sim:nextRand(1,#agents)]
					local x0, y0 = targetUnit:getLocation()
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PINPOINT_DESC, targetUnit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", targetUnit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
	            			sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_ALARMEDSAFE, targetUnit)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PING,
			desc = STRINGS.PROGEXTEND.AI.PING_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_ping.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				local dist = 100
				for _, guard in pairs(sim:getNPC():getUnits()) do
					if guard:getTraits().isGuard then
						local x1, y1 = guard:getLocation()
						for _, agent in pairs(sim:getPC():getUnits()) do
							local x2, y2 = agent:getLocation()
							if agent:getMP() and x1 and x2 then
								local dt = mathutil.dist2d(x1, y1, x2, y2)
								if dt < dist then dist = dt end
							end
						end
					end
				end
				if dist > 4 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.PING_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)

				for i, unit in pairs(sim:getPC():getUnits()) do
					if unit:getMP() then
						local x,y = unit:getLocation()
						sim:emitSound( { path = "SpySociety/Actions/mainframe_noisemaker", range = 4, distanceOffset = -4, ignoreSight = true }, x, y, nil, {{ x = x, y = y }} )
						sim:processReactions()
						if slot.identified then
							sim:dispatchEvent( simdefs.EV_SCANRING_VIS, { x=x,y=y, range=4 } )
						end
					end
				end

				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.CHITON,
			desc = STRINGS.PROGEXTEND.AI.CHITON_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/daemon_icons/Daemons_chiten.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "ko",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
    				if not sim:getNPC():getTraits().boostArmor then
    					sim:getNPC():getTraits().boostArmor = 0
    				end
				sim:getNPC():getTraits().boostArmor = sim:getNPC():getTraits().boostArmor + 1
				self.chitonCooldown[0] = 5
				self.chitonCooldown[1] = self.chitonCooldown[1] + 1
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)

				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.CHITON_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 120)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SUPERVISION,
			desc = STRINGS.PROGEXTEND.AI.SUPERVISION_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_supervision.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "ftm",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 2,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.supervisionCooldown and self.supervisionCooldown > 0 then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local dist = 100
				local camera = nil

				for _, device in pairs(sim:getAllUnits()) do
					if device:getTraits().mainframe_camera and device:getTraits().mainframe_ice > 0 and (not device:getTraits().iceImmune) and device:getTraits().mainframe_status ~= "off" then
						local x1, y1 = device:getLocation()
						for _, agent in pairs(sim:getPC():getUnits()) do
							local x2, y2 = agent:getLocation()
							if agent:getMP() and x1 and x2 then
								local dt = mathutil.dist2d(x1, y1, x2, y2)
								if dt < dist then
									dist = dt
									camera = device
								end
							end
						end
					end
				end

				if camera then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.supervisionUnit = camera
					self.supervisionUnit:getTraits().iceImmune = true
					self.supervisionCooldown = 3
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SUPERVISION_DESC, self.supervisionUnit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", camera )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.FRACTAL,
			desc = STRINGS.PROGEXTEND.AI.FRACTAL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/daemon_icons/Daemons0009.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "plastech",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, nil, "LOWEST", "NO DAEMON" )

				if device then
					if sim:isVersion("0.17.5") then
						local programList = sim:getIcePrograms()
						device:getTraits().mainframe_program = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
					else
						device:getTraits().mainframe_program = PROGRAM_LIST[ sim:nextRand(1, #PROGRAM_LIST) ]
					end

					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.FRACTAL_DESC,device:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", device )	
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = device, ice = device:getTraits().mainframe_ice, delta = 0} )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.RECALIBRATE,
			desc = STRINGS.PROGEXTEND.AI.RECALIBRATE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_recalibrate.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "sankaku",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local captured = {}
				for i, unit in pairs( sim:getAllUnits() ) do
					if unit:getName() == STRINGS.PROPS.SOUND_BUG and unit:getPlayerOwner() == sim:getPC() then
						table.insert(captured, unit)
					end
				end

				if #captured > 0 then
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.RECALIBRATE_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)

					for i, unit in pairs(captured) do
						mainframe.revertIce( sim, unit )
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.OBFUSCATE,
			desc = STRINGS.PROGEXTEND.AI.OBFUSCATE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_obfuscate.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "neptune",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				for i, unit in pairs( sim:getNPC():getUnits() ) do
					if unit:getTraits().tagged then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.OBFUSCATE_DESC, unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", unit )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
						unit:getTraits().tagged = nil
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
						break
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.HEATER,
			desc = STRINGS.PROGEXTEND.AI.HEATER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_heater.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, sim:getParams().difficulty + 1, "LOWEST" )

				if device then
					local pwr = math.max(math.ceil(sim:getNPC():getCpus()/5),1)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.HEATER_DESC,device:getName(), pwr), "SpySociety/Actions/mainframe_PWRreverse_off", device )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					device:increaseIce(sim, pwr)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.KITE,
			desc = STRINGS.PROGEXTEND.AI.KITE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_kite.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,
			raiseIce = 1,
			radius = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, sim:getParams().difficulty + 1, "LOWEST" )

				if device then
					local cellx, celly = device:getLocation()
            				local cells = simquery.rasterCircle( sim, cellx, celly, slot.radius )
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.KITE_DESC, device:getName(), slot.raiseIce), "SpySociety/Actions/mainframe_PWRreverse_off", device )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
            				for i, x, y in util.xypairs( cells ) do
                				local cell = sim:getCell( x, y )
                				if cell then
                    					for _, cellUnit in ipairs(cell.units) do
								cellUnit:increaseIce(sim, slot.raiseIce)
                            				end
                        			end
                    			end
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BLIZZARD,
			desc = STRINGS.PROGEXTEND.AI.BLIZZARD_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_blizzard.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_START_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.BLIZZARD_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				for i, unit in pairs(sim:getPC():getUnits()) do
					if unit:getTraits().mp and unit:getTraits().mp > 5 then
						local x,y = unit:getLocation()
						unit:getTraits().mp = unit:getTraits().mp - 1
						sim:dispatchEvent( simdefs.EV_SCANRING_VIS, { x=x,y=y, range=2 } )
					end
				end
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.NULL_PULSE,
			desc = STRINGS.PROGEXTEND.AI.NULL_PULSE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_null_pulse.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, nil, "HIGHEST", "NO NULL ZONE" )

				if device then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.NULL_PULSE_DESC, device:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", device )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					device:getTraits().mainframe_suppress_rangeMax = 4
					device:getTraits().mainframe_suppress_range = 4
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = device } )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.VAMPIRE,
			desc = STRINGS.PROGEXTEND.AI.VAMPIRE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_vampire.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 7,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if sim:getPC():getMaxCpus() <= 5 then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.VAMPIRE_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				sim:getPC():getTraits().PWRmaxBouns = (sim:getPC():getTraits().PWRmaxBouns or 0) - 1
				sim:getNPC():getTraits().PWRmaxBouns = (sim:getNPC():getTraits().PWRmaxBouns or 0) + 1
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PROXY_SCANNER,
			desc = STRINGS.PROGEXTEND.AI.PROXY_SCANNER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_proxy_scanner.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 4,
			targets = {},

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_START_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				slot.targets = {}
				for _, guard in pairs(sim:getNPC():getUnits()) do
					if guard:getTraits().isGuard and not guard:isDown() then
						for _, agent in pairs(sim:getPC():getUnits()) do
							if agent:getTraits().isAgent and (not agent:getTraits().isGuard) then
								local x1, y1 = guard:getLocation()
								local x2, y2 = agent:getLocation()
								if x1 and x2 and mathutil.dist2d(x1, y1, x2, y2) <= 2 then
									table.insert(slot.targets, agent)
								end
							end
						end
					end
				end

				if #slot.targets <= 0 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local target = slot.targets[sim:nextRand(1, #slot.targets)]
				slot.targets = {}

				if target then
					local x1,y1 = target:getLocation()
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PROXY_SCANNER_DESC, target:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", target )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					sim:getNPC():spawnInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_SENSEDTARGET, target)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TARGE,
			desc = STRINGS.PROGEXTEND.AI.TARGE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_targe.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 5,
			raiseIce = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, sim:getParams().difficulty, "LOWEST" )

				if device then
					local raised = 0
					local devices = {}
					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getName() == device:getName() and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
							table.insert( devices, unit )
							raised = raised + 1
						end
					end
					raised = math.max( 1, math.ceil(slot.raiseIce/raised))
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.TARGE_DESC, device:getName(), raised), "SpySociety/Actions/mainframe_PWRreverse_off", device )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					for i, unit in pairs(devices) do
						unit:increaseIce(sim, raised)
					end
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TREATY,
			desc = STRINGS.PROGEXTEND.AI.TREATY_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_treaty.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_START_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.lockdown = 2
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.TREATY_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				sim:setMainframeLockout( true )

				for i, slotA in pairs(self.slotsPWR) do
					if slotA.cooldown then
						slotA.cooldown = slotA.cooldown + 2
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.FIEND,
			desc = STRINGS.PROGEXTEND.AI.FIEND_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_fiend.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 6,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if not sim:getCurrentPlayer():isNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local disabled = 0
				local done = false

				for i, ability in pairs(sim:getPC():getAbilities()) do
					if not ability.passive and (not array.find(sim:getPC():getLockedAbilities(true),i) or array.find(sim:getPC():getLockedAbilities(true),i) ~= "fiend") then
						sim:getPC():lockdownMainframeAbility( i, "fiend" )
						done = true
						disabled = i
						break
					end
				end

				if done == true then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.FIEND_DESC,disabled), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SHIELD,
			desc = STRINGS.PROGEXTEND.AI.SHIELD_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.SHIELD_SPECIAL,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_shield.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local target

				for _, guard in pairs(sim:getNPC():getUnits()) do
					if (not target) and guard:getTraits().isGuard and guard:getTraits().woundsMax then
						target = guard
					elseif guard:getTraits().isGuard and guard:getTraits().woundsMax and guard:getTraits().woundsMax < target:getTraits().woundsMax then
						target = guard
					end
				end

				if target then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SHIELD_DESC, target:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", target )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					target:getModifiers():add( "woundsMax", "AI_Shield", modifiers.ADD, 1 )
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.INSTALL,
			desc = STRINGS.PROGEXTEND.AI.INSTALL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_install.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.INSTALL_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)

				local daemon = nil
				if sim:isVersion("0.17.5") then
					daemon = sim:getIcePrograms():getChoice( sim:nextRand( 1, sim:getIcePrograms():getTotalWeight() ))
				else
					daemon = programList[sim:nextRand(1, #serverdefs.PROGRAM_LIST)]			
				end	

				sim:getNPC():addMainframeAbility( sim, daemon )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PAVISE,
			desc = STRINGS.PROGEXTEND.AI.PAVISE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_pavise.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, sim:getParams().difficulty, "HIGHEST" )

				if device then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PAVISE_DESC,device:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", device )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					device:increaseIce(sim, device:getTraits().mainframe_ice)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.REROUTE,
			desc = STRINGS.PROGEXTEND.AI.REROUTE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_reroute.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 6,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local idle = sim:getNPC():getIdleSituation()
				local guards = sim:getNPC():getUnits()

				for i,guard in ipairs(guards) do
					if guard:getBrain() and guard:getBrain():getSituation().ClassType == simdefs.SITUATION_IDLE then
						idle:generatePatrolPath( guard )
						if guard:getTraits().patrolPath and #guard:getTraits().patrolPath > 1 then
							local firstPoint = guard:getTraits().patrolPath[1]
							guard:getBrain():getSenses():addInterest(firstPoint.x, firstPoint.y, simdefs.SENSE_RADIO, simdefs.REASON_PATROLCHANGED, guard)
						end
					end
				end
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.REROUTE_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				sim:processReactions()
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PHASE_SHIFT,
			desc = STRINGS.PROGEXTEND.AI.PHASE_SHIFT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_phase_shift.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_START_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local guards = {}
				for i, unit in pairs( sim:getNPC():getUnits() ) do
					if unit:getBrain() and unit:getBrain():getInterest() and not unit:getTraits().scientist then
						local path, dist = simquery.findPath( sim, unit, sim:getCell( unit:getLocation() ), sim:getCell( unit:getBrain():getInterest().x, unit:getBrain():getInterest().y ), unit:getMPMax(), nil )
						if path == nil or dist == nil then
							table.insert(guards, unit)
						end
					end
				end

				if #guards > 0 then
					local targetUnit = guards[sim:nextRand(1,#guards)]
					local cells = {}

					local x0 = targetUnit:getBrain():getInterest().x
					local y0 = targetUnit:getBrain():getInterest().y
					local cell = sim:getCell( x0, y0 )

					if cell.impass <= 0 and not sim:getQuery().checkDynamicImpass(sim, cell) then
						table.insert(cells, cell)
					end

					for i = 1, #simdefs.OFFSET_NEIGHBOURS, 2 do
    						local dx, dy = simdefs.OFFSET_NEIGHBOURS[i], simdefs.OFFSET_NEIGHBOURS[i+1]
                				local targetCell = sim:getCell( cell.x + dx, cell.y + dy )
                				if simquery.isConnected( sim, cell, targetCell ) and targetCell.impass <= 0 and not sim:getQuery().checkDynamicImpass(sim, targetCell) then
							table.insert( cells, targetCell )
						end
					end

					for j, targetCell in pairs(cells) do
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PHASE_SHIFT_DESC, targetUnit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", targetUnit )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
						sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = targetUnit:getID(), noSightingFx=true } )
						sim:dispatchEvent( simdefs.EV_TELEPORT, { units={targetUnit}, warpOut = true } )
						sim:warpUnit( targetUnit, targetCell )
						sim:dispatchEvent( simdefs.EV_CAM_PAN, { targetUnit:getLocation() } )
						sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = targetUnit:getID(), noSightingFx=true } )
						sim:dispatchEvent( simdefs.EV_TELEPORT, { units={targetUnit}, warpOut = false } )
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
						sim:processReactions( targetUnit )
						break
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TELESTRIKE,
			desc = STRINGS.PROGEXTEND.AI.TELESTRIKE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_telestrike.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if sim:getTrackerStage(sim:getTracker()) < 1 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local agents = {}
				for i, unit in pairs( sim:getPC():getUnits() ) do
					if unit:getTraits().isAgent and unit:getTraits().canKO and not unit:isDown() then
						table.insert(agents, unit)
					end
				end

				if #agents > 0 then
					local targetUnit = agents[sim:nextRand(1,#agents)]
					local templateUnit = simfactory.createUnit( unitdefs.lookupTemplate( "item_npc_flashgrenade" ), sim )

					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.TELESTRIKE_DESC, targetUnit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", targetUnit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					sim:spawnUnit( templateUnit )
					sim:warpUnit( templateUnit, sim:getCell( targetUnit:getLocation() ) )
					templateUnit:setPlayerOwner(sim:getNPC())
					sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = templateUnit:getID(), noSightingFx=true } )
					sim:dispatchEvent( simdefs.EV_TELEPORT, { units={templateUnit}, warpOut = false } )
					templateUnit:activate()
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = templateUnit } )
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.JUMPSTART,
			desc = STRINGS.PROGEXTEND.AI.JUMPSTART_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_jumpstart.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "general",
			minDifficulty = 7,
			maxDifficulty = 100,
			requiresGens = true,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,
			raiseIce = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				for _, genUnit in pairs(self.gens) do
					if mainframe.canRevertIce( sim, genUnit ) then
						self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.JUMPSTART_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", genUnit )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
						mainframe.revertIce( sim, genUnit )
						genUnit:increaseIce(sim, slot.raiseIce)
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
						break
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BUNKER,
			desc = STRINGS.PROGEXTEND.AI.BUNKER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_bunker.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 8,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local raised = 0
				for i, unit in pairs( sim:getAllUnits() ) do
					unit:increaseIce(sim, 1)
					raised = raised + 1
				end

				if raised > 0 then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.BUNKER_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.HASTE,
			desc = STRINGS.PROGEXTEND.AI.HASTE_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.HASTE_SPECIAL,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_haste.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_START_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local raised = 0
				for i, unit in pairs( evData:getUnits() ) do
					if unit:getMP() and not unit:getModifiers():has( "mpMax", "AI_Haste" ) then
						unit:getModifiers():add( "mpMax", "AI_Haste", modifiers.ADD, 4 )
						unit:addMP( 4 )
						raised = raised + 1
					elseif unit:getMP() and unit:getModifiers():has( "mpMax", "AI_Haste" ) then
						unit:getModifiers():add( "mpMax", "AI_Haste", modifiers.ADD, 1 )
						unit:addMP( 1 )
						raised = raised + 1
					end
				end

				if raised > 0 then
					self.hasteCooldown = 4
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.HASTE_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.OVERSIGHT,
			desc = STRINGS.PROGEXTEND.AI.OVERSIGHT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_oversight.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_START_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local targets = {}
				local targetCells = {}
				local guards = {}

				if sim:getParams().world == "omni" then
					if not sim:getTags().monster_finished_hacking then
						for i, unit in pairs(sim:getAllUnits()) do
							if unit:hasTag("ending_jackin") then
								table.insert(targets, unit)
								break
							end
						end
					else
						for i, unit in pairs(sim:getAllUnits()) do
							if unit:hasTrait("install_incognita") then
								table.insert(targets, unit)
								break
							end
						end
					end
				else
					if sim:hasObjective("locate_lab") or sim:hasObjective("find_case") then
						for i, unit in pairs(sim:getAllUnits()) do
							if unit:hasTag("detention_processor") then
								table.insert(targets, unit)
								break
							end
						end
					end
					if sim:hasObjective("disable_shield") then
						for i, unit in pairs(sim:getAllUnits()) do
							if unit:hasTag("mid2_shieldGenerator") then
								table.insert(targets, unit)
								break
							end
						end
					end
					if sim:hasObjective("escape") then
						util.tmerge(targetCells, simquery.findExitCells( sim ))
					end
				end

				for i, unit in pairs(sim:getNPC():getUnits()) do
					if unit:hasTrait("isGuard") and not unit:hasTrait("pacifist") and not unit:isDown() and unit:getBrain() then
						table.insert(guards, unit)
					end
				end

				if (#targets > 0 or #targetCells > 0) and #guards > 0 then
					local target
					local targetCell
					local x0, y0
					local guard = guards[sim:nextRand(1,#guards)]
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)

					if #targets > 0 then
						target = targets[sim:nextRand(1,#targets)]
						x0, y0 = target:getLocation()
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.OVERSIGHT_DESC, guard:getName(), target:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", guard )
						guard:getBrain():getSenses():addInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_SCANNED, target)
					elseif #targetCells > 0 then
						while true do
							targetCell = targetCells[sim:nextRand(1,#targetCells)]
							x0 = targetCell.x
							y0 = targetCell.y
							if x0 and y0 then
								self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.OVERSIGHT_DESC, guard:getName(), STRINGS.PROGEXTEND.AI.OVERSIGHT_DESC2), "SpySociety/Actions/mainframe_PWRreverse_off", guard )
								guard:getBrain():getSenses():addInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_SCANNED, guard)
								break
							end
						end
					else
						return
					end
					sim:processReactions(guard)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.STATIC_SHOCK,
			desc = STRINGS.PROGEXTEND.AI.STATIC_SHOCK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_PRO,
			icon = "gui/icons/ai_icons/icon-ai_static_shock.png",
			iconType = "gui/icons/thought_icons/status_channeling.png",
			type = "PRO",
			identified = false,
			corp = "omni",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 5,
			targets = {},

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if self.activePrograms > 1 and sim:nextRand(sim:getNPC():getCpus(),sim:getNPC():getMaxCpus()) < 15 then
					return false
				end

				slot.targets = {}
				for _, device in pairs(sim:getAllUnits()) do
					if device:getTraits().mainframe_status == "active" and (not device:getTraits().mainframe_booting) and device:getLocation() then
						for _, agent in pairs(sim:getPC():getUnits()) do
							if agent:getTraits().isAgent and (not agent:getTraits().isGuard) and (not agent:isDown()) and simquery.canUnitReach( sim, agent, device:getLocation() ) and agent:getTraits().canKO then
								table.insert(slot.targets, agent)
							end
						end
					end
				end

				if #slot.targets <= 0 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local target = slot.targets[sim:nextRand(1, #slot.targets)]

				if target then
					local x1,y1 = target:getLocation()
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.STATIC_SHOCK_DESC, target:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", target )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x1, y = y1, units = nil, range = 2 } )
					local dmgt =
					{
						damage = 2,
						ko = true,
						noTargetAlert = true,
					}
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					sim:hitUnit( target, target, dmgt )
					sim:getPC():glimpseUnit( sim, target:getID() )
				end
				slot.targets = {}
			end,
		},
	},

	REACTIVE =
	{
		{
			name = STRINGS.PROGEXTEND.AI.PAWN,
			desc = STRINGS.PROGEXTEND.AI.PAWN_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_pawn.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 2,
			raiseIce = 1,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 or evData.unit:getTraits().mainframe_ice >= 20 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PAWN_DESC, evData.unit:getName(), slot.raiseIce), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				evData.unit:increaseIce(sim, slot.raiseIce)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SAFEGUARD,
			desc = STRINGS.PROGEXTEND.AI.SAFEGUARD_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_safeguard.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 2,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_UNIT_HIJACKED then
					return false
				end

				if not evData.unit:getTraits().mainframe_console then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local consoles = {}
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_console and unit ~= evData.unit and unit:getTraits().mainframe_status == "active" then
						table.insert(consoles, unit)
					end
				end

				if #consoles > 0 then
					local dist = 100
					local targetUnit = nil
					for _, console in pairs(consoles) do
						local x1, y1 = console:getLocation()
						for _, agent in pairs(sim:getPC():getUnits()) do
							local x2, y2 = agent:getLocation()
							if agent:getMP() and x1 and x2 then
								local dt = mathutil.dist2d(x1, y1, x2, y2)
								if dt < dist then
									dist = dt
									targetUnit = console
								end
							end
						end
					end
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.SAFEGUARD_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", targetUnit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					targetUnit:processEMP( 6, true )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = targetUnit } )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.ROOK,
			desc = STRINGS.PROGEXTEND.AI.ROOK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/daemon_icons/Daemons_castle.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local device = self.findMainframeDevice( self, sim, slot, nil, "HIGHEST" )

				if device and device ~= evData.unit then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.ROOK_DESC,device:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					local delta = device:getTraits().mainframe_ice - evData.unit:getTraits().mainframe_ice
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					device:increaseIce(sim, -delta)
					evData.unit:increaseIce(sim, delta)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.QUEUE,
			desc = STRINGS.PROGEXTEND.AI.QUEUE_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.QUEUE_SPECIAL,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_queue.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 3,
			target = nil,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_END_TURN and sim:getCurrentPlayer() == sim:getNPC() and slot.target ~= nil then
					return true
				end

				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice > 0 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_ICE_BROKEN then
					local validPrograms = {}
					for i, ability in pairs(sim:getPC():getAbilities()) do
						if not ability.passive and (ability.cooldown or 0) <= 0 then
							table.insert(validPrograms, ability)
						end
					end
					if #validPrograms <= 0 then
						for i, ability in pairs(sim:getPC():getAbilities()) do
							if not ability.passive then
								table.insert(validPrograms, ability)
							end
						end
					end

					if #validPrograms > 0 then
						local choice = sim:nextRand(1, #validPrograms)
						if validPrograms[choice].equipped == true then
							self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.QUEUE_DESC2, validPrograms[choice].name), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
							slot.target =  validPrograms[choice]:getID()  -- validPrograms[choice]
						else
							self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.QUEUE_DESC, validPrograms[choice].name), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
							validPrograms[choice].cooldown = (validPrograms[choice].cooldown or 0) + 2
						end
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
					end
				elseif evType == simdefs.TRG_END_TURN then
					slot.target = sim:getPC():hasMainframeAbility( slot.target )
					slot.target.cooldown = (slot.target.cooldown or 0) + 2
					slot.target = nil
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.CIRCUIT_BREAKER,
			desc = STRINGS.PROGEXTEND.AI.CIRCUIT_BREAKER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_circuit_breaker.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_UNIT_HIJACKED then
					return false
				end

				if not evData.unit:getTraits().mainframe_console then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_UNIT_HIJACKED then
    	        			local x0, y0 = evData.unit:getLocation()
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.CIRCUIT_BREAKER_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
	            			sim:getNPC():spawnInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_ALARMEDSAFE, evData.unit)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PROCESS,
			desc = STRINGS.PROGEXTEND.AI.PROCESS_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_process.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 4,
			pwrDrain = -8,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_SET_OVERWATCH then
					return false
				end

				if not evData.unit then
					return false
				end

				if evData.unit:getPlayerOwner() ~= sim:getNPC() then
					return false
				end

				if not evData.unit:getTraits().isAiming then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if sim:getPC():getCpus() <= 0 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PROCESS_DESC, slot.pwrDrain), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					sim:getPC():addCPUs( slot.pwrDrain )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.KNIGHT,
			desc = STRINGS.PROGEXTEND.AI.KNIGHT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_knight.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 4,
			raiseIce = 2,
			radius = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 or evData.unit:getTraits().mainframe_ice >= 20 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local cellx, celly = evData.unit:getLocation()
            			local cells = simquery.rasterCircle( sim, cellx, celly, slot.radius )
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.KNIGHT_DESC, evData.unit:getName(), slot.raiseIce), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
           			sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = cellx, y = celly, units = nil, range = slot.radius } )	
            			for i, x, y in util.xypairs( cells ) do
                			local cell = sim:getCell( x, y )
                			if cell then
                    				for _, cellUnit in ipairs(cell.units) do
							cellUnit:increaseIce(sim, slot.raiseIce)
                            			end
                        		end
                    		end
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.NOISE_SCANNER,
			desc = STRINGS.PROGEXTEND.AI.NOISE_SCANNER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_noise_scanner.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_SOUND_EMITTED then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if not evData.sourceUnit then
					return false
				end

				if evData.sourceUnit:getPlayerOwner() == sim:getNPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.NOISE_SCANNER_DESC, evData.sourceUnit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				sim:trackerAdvance( 1 )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SUBTERFUGE,
			desc = STRINGS.PROGEXTEND.AI.SUBTERFUGE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_subterfuge.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "ftm",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_CAUGHT_BY_CAMERA then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_camera and unit:getTraits().mainframe_status == "active" and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
						for j, agentUnit in pairs(sim:getPC():getUnits()) do
							if sim:canUnitSeeUnit(unit, agentUnit) and agentUnit:getTraits().mp then
								self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SUBTERFUGE_DESC, agentUnit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", agentUnit )
								sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
								agentUnit:useMP( math.floor(agentUnit:getMP()/2), sim )
								self.usePwr(self, sim, slot)
								self.applyCooldown(self, slot)
							end
						end
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SENTINEL,
			desc = STRINGS.PROGEXTEND.AI.SENTINEL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_sentinel.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "ko",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_UNIT_WARP then
					return false
				end

				if not sim:getCurrentPlayer():isPC() then
					return false
				end

				if not evData.unit or not evData.unit:getTraits().isAgent then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_turret and evData.unit and evData.to_cell then
						local x1,y1 = evData.unit:getLocation()
						local x2,y2 = unit:getLocation()

						if unit:getPlayerOwner() == nil then
							unit:setPlayerOwner(sim:getNPC())
						end

						if simquery.couldUnitSeeCell(sim, unit, evData.to_cell) and mathutil.dist2d(x1, y1, x2, y2) <= 3 and unit:getPlayerOwner() == sim:getNPC() and unit:getTraits().mainframe_status == "active" then
							self.usePwr(self, sim, slot)
							self.applyCooldown(self, slot)
							self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.SENTINEL_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", unit )
							sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
							unit:turnToFace(x1, y1)
							sim:refreshUnitLOS( unit )
							sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit })
							break
						end
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.FAILSAFE,
			desc = STRINGS.PROGEXTEND.AI.FAILSAFE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_failsafe.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "sankaku",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if not sim:getCurrentPlayer():isPC() then
					return false
				end

				if not evData.unit:getTraits().isDrone then
					return false
				end

				if evData.unit:getTraits().mainframe_ice > 0 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
    	        		local x0, y0 = evData.unit:getLocation()
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.FAILSAFE_DESC,evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				sim:getNPC():spawnInterestWithReturn(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_ALARMEDSAFE, evData.unit, { evData.unit:getID() } )
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.FAKEOUT,
			desc = STRINGS.PROGEXTEND.AI.FAKEOUT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_fakeout.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "plastech",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if not sim:getCurrentPlayer():isPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice > 0 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit:getTraits().mainframe_program and evData.unit:getTraits().daemon_sniffed and evData.unit:getTraits().daemon_sniffed == true then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.FAKEOUT_DESC, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					if sim:isVersion("0.17.5") then
						local programList = sim:getIcePrograms()
						evData.unit:getTraits().mainframe_program = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
					else
						evData.unit:getTraits().mainframe_program = PROGRAM_LIST[ sim:nextRand(1, #PROGRAM_LIST) ]
					end	
					sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = evData.unit, ice = evData.unit:getTraits().mainframe_ice, delta = 0} )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.ELECTROSHOCK,
			desc = STRINGS.PROGEXTEND.AI.ELECTROSHOCK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_electroshock.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "neptune",
			minDifficulty = 3,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_SAFE_LOOTED then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit and evData.unit:getTraits().isAgent then
					local x0,y0 = evData.unit:getLocation()
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.ELECTROSHOCK_DESC,evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					sim:dispatchEvent( simdefs.EV_FLASH_VIZ, {x = x0, y = y0, units = nil, range = 2 } )
					if evData.unit:getTraits().canKO then
						evData.unit:setKO(sim, 1)
					end
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BISHOP,
			desc = STRINGS.PROGEXTEND.AI.BISHOP_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_bishop.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 or evData.unit:getTraits().mainframe_ice >= 20 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local increaseIce = math.ceil(sim:getNPC():getCpus()/5)
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.BISHOP_DESC, evData.unit:getName(), increaseIce), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				evData.unit:increaseIce(sim, increaseIce)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SCANNER_SWEEP,
			desc = STRINGS.PROGEXTEND.AI.SCANNER_SWEEP_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_scanner_sweep.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_UNIT_NEWINTEREST and evType ~= simdefs.TRG_END_TURN then
					return false
				end

				if slot.cooldown > 0 and evType == simdefs.TRG_UNIT_NEWINTEREST then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) and evType == simdefs.TRG_UNIT_NEWINTEREST then
					return false
				end

				if evType == simdefs.TRG_UNIT_NEWINTEREST and evData.interest and (not
				(evData.interest.reason == simdefs.REASON_NOISE or evData.interest.reason == simdefs.REASON_DOOR or evData.interest.reason == simdefs.REASON_NOTICED or evData.interest.reason == simdefs.REASON_SENSEDTARGET) or (not
				(evData.interest.sense == simdefs.SENSE_HEARING or evData.interest.sense == simdefs.SENSE_SIGHT or evData.interest.sense == simdefs.SENSE_PERIPHERAL))) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_UNIT_NEWINTEREST and not evData.unit:getTraits().refreshingLOS and evData.unit:getTraits().LOSperipheralArc then
					if not evData.unit:getModifiers():has( "LOSperipheralArc", "W93_AI_SCANNER_SWEEP" ) and evData.unit:getTraits().LOSperipheralArc < math.pi then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SCANNER_SWEEP_DESC, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
						evData.unit:getModifiers():add( "LOSperipheralArc", "W93_AI_SCANNER_SWEEP", modifiers.SET, math.pi )
                    				sim:refreshUnitLOS( evData.unit )
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
					end
				elseif evType == simdefs.TRG_END_TURN and evData:isNPC() then
					for i, guardUnit in pairs(sim:getNPC():getUnits()) do
                    				if guardUnit:getModifiers():has( "LOSperipheralArc", "W93_AI_SCANNER_SWEEP" ) and not guardUnit:getTraits().refreshingLOS then
							guardUnit:getModifiers():remove( "W93_AI_SCANNER_SWEEP" )
							sim:refreshUnitLOS( guardUnit )
						end
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SPEED_BUMP,
			desc = STRINGS.PROGEXTEND.AI.SPEED_BUMP_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_speed_bump.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_OPEN_DOOR then
					return false
				end

				if not sim:getCurrentPlayer():isPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SPEED_BUMP_DESC, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					evData.unit:useMP( 2, sim )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.QUEEN,
			desc = STRINGS.PROGEXTEND.AI.QUEEN_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_queen.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 4,
			increaseIce = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 or evData.unit:getTraits().mainframe_ice >= 20 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local reinforced = 0
				local devices = {}
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getName() == evData.unit:getName() and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 then
						table.insert( devices, unit )
						reinforced = reinforced + 1
					end
				end

				if reinforced > 0 then
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.QUEEN_DESC, evData.unit:getName(), math.max( 1, math.ceil(slot.increaseIce/reinforced) )), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					for i, device in pairs(devices) do
						reinforced = math.max( 1, math.ceil(slot.increaseIce/reinforced) )
						device:increaseIce(sim, reinforced )
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BACKUP_PROTOCOLS,
			desc = STRINGS.PROGEXTEND.AI.BACKUP_PROTOCOLS_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_backup_protocols.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if (evType ~= simdefs.TRG_UNIT_ALERTED and evType ~= simdefs.TRG_UNIT_KILLED) then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)

				local wt = util.weighted_list( sim._patrolGuard )
				local templateName = wt:getChoice( sim:nextRand( 1, wt:getTotalWeight() ))
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.BACKUP_PROTOCOLS_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				sim:getNPC():doTrackerSpawn( sim, 1, templateName, false )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.VERIFY,
			desc = STRINGS.PROGEXTEND.AI.VERIFY_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_verify.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_UNIT_NEWINTEREST then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if evType == simdefs.TRG_UNIT_NEWINTEREST and evData.interest and (not
				(evData.interest.reason == simdefs.REASON_NOISE or evData.interest.reason == simdefs.REASON_DOOR or evData.interest.reason == simdefs.REASON_NOTICED or evData.interest.reason == simdefs.REASON_SENSEDTARGET) or (not
				(evData.interest.sense == simdefs.SENSE_HEARING or evData.interest.sense == simdefs.SENSE_SIGHT or evData.interest.sense == simdefs.SENSE_PERIPHERAL))) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit:getMPMax() then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.VERIFY_DESC, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					evData.unit:addMPMax( 1 )
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.DAEMON_NEST,
			desc = STRINGS.PROGEXTEND.AI.DAEMON_NEST_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_daemon_nest.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 then
					return false
				end

				if not evData.unit:getTraits().mainframe_program then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.DAEMON_NEST_DESC, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				mainframe.invokeDaemon( sim, evData.unit )
				local programList = nil
				local daemon = nil
				if sim:isVersion("0.17.5") then
					programList = sim:getIcePrograms()
					evData.unit:getTraits().mainframe_program = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))
				else
					programList = serverdefs.PROGRAM_LIST
					evData.unit:getTraits().mainframe_program = programList[sim:nextRand(1, #programList)]			
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.CLONE,
			desc = STRINGS.PROGEXTEND.AI.CLONE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_clone.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_DAEMON_INSTALL then
					return false
				end

				if not evData or not evData._abilityID then
					return false
				end

				if evData.standardDaemon ~= true or evData.premanent == true then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.CLONE_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				sim:getNPC():addMainframeAbility(sim, evData._abilityID, nil, 0 )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.KING,
			desc = STRINGS.PROGEXTEND.AI.KING_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_king.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 or evData.unit:getTraits().mainframe_ice >= 10 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.KING_DESC,evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				evData.unit:increaseIce(sim, evData.unit:getTraits().mainframe_ice)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.HASHED_CRYPTO,
			desc = STRINGS.PROGEXTEND.AI.HASHED_CRYPTO_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_hashed_crypto.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 0,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_SAFE_LOOTED then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_SAFE_LOOTED then
					local robbed = 0
					for i, unit in pairs(sim:getAllUnits()) do
						if unit:getTraits().safeUnit and not unit:getTraits().open and unit:getTraits().credits then
							unit:getTraits().credits = math.floor(unit:getTraits().credits * 0.75)
							robbed = robbed + 1
						end
					end

					if robbed > 0 then
						self.usePwr(self, sim, slot)
						self.applyCooldown(self, slot)
						self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.HASHED_CRYPTO_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BLOCKADE,
			desc = STRINGS.PROGEXTEND.AI.BLOCKADE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_blockade.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 4,
			pwrCost = 5,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_OPEN_DOOR then
					return false
				end

				if not sim:getCurrentPlayer():isPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				local guards = {}
				for i, unit in pairs(sim:getNPC():getUnits()) do
					if not unit:isDown() and unit:getBrain() and unit:getBrain():getSituation().ClassType == simdefs.SITUATION_IDLE then
						table.insert(guards, unit)
					end
				end

				if #guards > 0 and evData.unit then
					local x0, y0 = evData.unit:getLocation()
					local guard = guards[sim:nextRand(1,#guards)]
					local idle = sim:getNPC():getIdleSituation()
					idle:generateStationaryPath( guard, x0, y0 )
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.BLOCKADE_DESC, guard:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					guard:getBrain():getSenses():addInterest(x0, y0, simdefs.SENSE_RADIO, simdefs.REASON_PATROLCHANGED, guard)
					sim:processReactions(guard)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TERMINATE,
			desc = STRINGS.PROGEXTEND.AI.TERMINATE_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_terminate.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 5,
			pwrCost = 10,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_SET_OVERWATCH then
					return false
				end

				if not evData.unit then
					return false
				end

				if evData.unit:getPlayerOwner() ~= sim:getNPC() then
					return false
				end

				if not evData.unit:getTraits().isAiming then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit and evData.unit:getBrain() then
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.TERMINATE_DESC, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
					evData.unit:getBrain():reset()
					sim:processReactions(evData.unit:getBrain().target)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BARRIER,
			desc = STRINGS.PROGEXTEND.AI.BARRIER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_barrier.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 6,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_UNIT_KILLED and evType ~= simdefs.TRG_UNIT_KO then
					return false
				end

				if evType == simdefs.TRG_UNIT_KO and not evData.ticks then
					return false
				end

				if not evData.unit:getTraits().isGuard then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evData.unit then
					local raised = 0
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.BARRIER_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					for _, guard in pairs(sim:getNPC():getUnits()) do
						if guard:getTraits().isGuard and guard ~= evData.unit then
							guard:getModifiers():add( "woundsMax", "W93_AiBarrier", modifiers.ADD, 1 )
						end
					end
					self.usePwr(self, sim, slot)
					self.applyCooldown(self, slot)
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.KEYCARD,
			desc = STRINGS.PROGEXTEND.AI.KEYCARD_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_keycard.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 3,
			pwrCost = 4,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_OPEN_DOOR then
					return false
				end

				if not sim:getCurrentPlayer():isPC() then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				if sim:getPC():getCpus() <= 0 then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.KEYCARD_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*0.65)
				sim:getPC():addCPUs(-2)
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SYNCHRONIZED_FIREWALL,
			desc = STRINGS.PROGEXTEND.AI.SYNCHRONIZED_FIREWALL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_REA,
			icon = "gui/icons/ai_icons/icon-ai_firewall_synchroniser.png",
			iconType = "gui/icons/thought_icons/status_hunt.png",
			type = "REA",
			identified = false,
			corp = "omni",
			minDifficulty = 1,
			maxDifficulty = 100,

			cooldown = 0,
			cooldownMax = 2,
			pwrCost = 3,

			canUseAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType ~= simdefs.TRG_ICE_BROKEN then
					return false
				end

				if evData.delta and evData.delta >= 0 then
					return false
				end

				if sim:getCurrentPlayer() ~= sim:getPC() then
					return false
				end

				if evData.unit:getTraits().mainframe_ice <= 0 then
					return false
				end

				if slot.cooldown > 0 then
					return false
				end

				if sim:getNPC():getCpus() < self.getPwrCost(self, slot) then
					return false
				end

				return true
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				self.usePwr(self, sim, slot)
				self.applyCooldown(self, slot)
				local raise = math.min(sim:getTrackerStage(sim:getTracker()), simdefs.TRACKER_MAXSTAGE) + 1
				self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.SYNCHRONIZED_FIREWALL_DESC, raise, evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
				sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				evData.unit:increaseIce(sim, raise)
			end,
		},
	},

	UTILITY =
	{
		{
			name = STRINGS.PROGEXTEND.AI.PADLOCK,
			desc = STRINGS.PROGEXTEND.AI.PADLOCK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_padlock.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 1,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_END_TURN and evData:isNPC() then
					local locked = 0
					sim:forEachCell(function( c )
						for i,exit in pairs(c.exits) do
							if not exit.locked and exit.keybits == simdefs.DOOR_KEYS.SECURITY then
								if not exit.closed then
									sim:modifyExit( c, i, simdefs.EXITOP_CLOSE)
								end
								sim:modifyExit( c, i, simdefs.EXITOP_LOCK)
                    						sim:getPC():glimpseExit( c.x, c.y, i )
								locked = locked + 1
							end
						end
					end )
					if locked > 0 then
						self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.PADLOCK_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					end
				end
			end,

			removeAbility = function( self, sim )

			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PAYWALL,
			desc = STRINGS.PROGEXTEND.AI.PAYWALL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_paywall.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 1,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				if not sim:getPC():getTraits().shopPenalty then
					sim:getPC():getTraits().shopPenalty = 1
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getPC():getTraits().shopPenalty = sim:getPC():getTraits().shopPenalty - 1
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TAG_DOWNLINK,
			desc = STRINGS.PROGEXTEND.AI.TAG_DOWNLINK_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.TAG_DOWNLINK_SPECIAL,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_tag_downlink.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and evData:isPC() then
					local pwr = 0
					for i, unit in pairs(sim:getNPC():getUnits()) do
						if unit:getTraits().tagged then
							pwr = pwr + 1
						end
					end
					if sim.MM_mole_bonus_tag and not slot.MM_tags then
						slot.MM_tags = {}
						for _, guard in pairs(sim:getNPC():getUnits()) do
							if guard:getTraits().tagged then
								table.insert(slot.MM_tags, guard)
								if #slot.MM_tags >= sim.MM_mole_bonus_tag then
									break
								end
							end
						end
					elseif not sim.MM_mole_bonus_tag then
						slot.MM_tags = nil
					end
					if slot.MM_tags and #slot.MM_tags > 0 then
						for i=#slot.MM_tags, 1, -1 do
							if slot.MM_tags[i] and slot.MM_tags[i]:isValid() then
								pwr = pwr - 1
							else
								table.remove(slot.MM_tags, i)
							end
						end
					end
					if pwr > 0 then
						self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.TAG_DOWNLINK_DESC,pwr), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
						sim:getPC():addCPUs(-pwr)
					end
				end
			end,

			removeAbility = function( self, sim )

			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.DEFECTIVE_TAG,
			desc = STRINGS.PROGEXTEND.AI.DEFECTIVE_TAG_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_defective_tag.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 2,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_START_TURN and evData:isPC() then
					local count = 0
					for i, unit in pairs(sim:getNPC():getUnits()) do
						if unit:getTraits().mp and unit:getTraits().tagged and not unit:getModifiers():has( "mpMax", "W93_DEFECTIVE_TAG" ) then
							unit:getModifiers():add( "mpMax", "W93_DEFECTIVE_TAG", modifiers.ADD, 4 )
						end
						if not unit:getTraits().tagged and unit:getModifiers():has( "mpMax", "W93_DEFECTIVE_TAG" ) then
							unit:getModifiers():remove( "W93_DEFECTIVE_TAG" )
						end
					end
					if count > 0 then
						self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.DEFECTIVE_TAG_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					end
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					unit:getModifiers():remove( "W93_DEFECTIVE_TAG" )
					if unit:getTraits().mp and unit:getTraits().mp > unit:getTraits().mpMax then
						unit:getTraits().mp = unit:getTraits().mpMax
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PASSWORD,
			desc = STRINGS.PROGEXTEND.AI.PASSWORD_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_password.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,
			excludeIfSingle = true,

			onSpawnAbility = function( self, sim, slot )
				self.token_reduced = (sim:getPC():getTraits().aiTokenMax or 8) - math.max( (sim:getPC():getTraits().aiTokenMax or 8) - 6, 2)
				sim:getPC():getTraits().aiTokenMax = (sim:getPC():getTraits().aiTokenMax or 8) - self.token_reduced
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getNPC():getTraits().AI_Tokens = nil
				sim:getPC():getTraits().aiTokenMax = sim:getPC():getTraits().aiTokenMax + self.token_reduced
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.EXTENSION,
			desc = STRINGS.PROGEXTEND.AI.EXTENSION_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_extension.png",
			iconType = "gui/icons/thought_icons/status_action.png",	
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 8,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				sim:getPC():getTraits().daemonDurationModd = (sim:getPC():getTraits().daemonDurationModd or 0) + 1
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getPC():getTraits().daemonDurationModd = sim:getPC():getTraits().daemonDurationModd - 1
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.JAMMER,
			desc = STRINGS.PROGEXTEND.AI.JAMMER_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_jammer.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_ICE_BROKEN and evData.unit:getTraits().mainframe_ice > 0 and evData.delta and evData.delta < 0 then
					if self.jammerDevice then
						if self.jammerDevice == evData.unit then
							return
						elseif self.jammerDevice:getTraits().iceImmune then
							self.jammerDevice:getTraits().iceImmune = nil
						end
					end
					self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.JAMMER_DESC,evData.unit:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					evData.unit:getTraits().iceImmune = true
					self.jammerDevice = evData.unit
				end
			end,

			removeAbility = function( self, sim )
				if self.jammerDevice then
					if self.jammerDevice:getTraits().iceImmune then
						self.jammerDevice:getTraits().iceImmune = nil
					end
				end
				self.jammerDevice = nil
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.NETWORKED_CONSCIOUSNESS,
			desc = STRINGS.PROGEXTEND.AI.NETWORKED_CONSCIOUSNESS_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_networked_consciousness.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "plastech",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					if unit:getTraits().isGuard and not unit:getTraits().isDrone and not unit:getTraits().consciousness_monitor then
						unit:getTraits().consciousness_monitor = true
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard and not evData:getTraits().isDrone and not evData:getTraits().consciousness_monitor then
					evData:getTraits().consciousness_monitor = true
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().consciousness_monitor then
						unit:getTraits().consciousness_monitor = nil
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.EM_SHIELDING,
			desc = STRINGS.PROGEXTEND.AI.EM_SHIELDING_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_em_shielding.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "sankaku",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_ice and not unit:getTraits().magnetic_reinforcement then
						unit:getTraits().magnetic_reinforcement = true
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().mainframe_ice and not evData:getTraits().magnetic_reinforcement then
					evData:getTraits().magnetic_reinforcement = true
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_ice and unit:getTraits().magnetic_reinforcement then
						unit:getTraits().magnetic_reinforcement = nil
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.NEURAL_FEEDBACK,
			desc = STRINGS.PROGEXTEND.AI.NEURAL_FEEDBACK_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_neural_feedback.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "ko",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_UNIT_KO and evData.unit:getTraits().isGuard and evData.ticks and evData.ticks > 0 then
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.NEURAL_FEEDBACK_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					if not evData.unit:getTraits().armor then
						evData.unit:getTraits().armor = 1
					else				
						evData.unit:getTraits().armor = evData.unit:getTraits().armor + 1
					end
				end
			end,

			removeAbility = function( self, sim )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.VIDEO_COPROCESSORS,
			desc = STRINGS.PROGEXTEND.AI.VIDEO_COPROCESSORS_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_video_coprocessors.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "ftm",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					if unit:getTraits().isGuard and not unit:getTraits().isDrone and not unit:getTraits().detect_cloak then
						unit:getTraits().detect_cloak = true
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard and not evData:getTraits().isDrone and not evData:getTraits().detect_cloak then
					evData:getTraits().detect_cloak = true
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().detect_cloak then
						unit:getTraits().detect_cloak = nil
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.EMERGENCY_SHUTDOWN,
			desc = STRINGS.PROGEXTEND.AI.EMERGENCY_SHUTDOWN_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_emergency_shutdown.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "neptune",
			minDifficulty = 3,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_UNIT_HIJACKED and evData.unit:getTraits().mainframe_console and evData.action then
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.EMERGENCY_SHUTDOWN_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", evData.unit )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
					evData.unit:getTraits().mainframe_status = "off"
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().mainframe_console and unit:getTraits().mainframe_status == "off" then
						unit:getTraits().mainframe_status = "active"
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.BLACKOUT,
			desc = STRINGS.PROGEXTEND.AI.BLACKOUT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_blackout.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				for _, agent in pairs(sim:getPC():getUnits()) do
					if agent:getTraits().isAgent and not agent:getTraits().isGuard then
						if not agent:getTraits().LOSrange and not agent:getModifiers():has( "LOSrange", "W93_blackout" ) then
							agent:getTraits().LOSrange = 50
						end
						if not agent:getModifiers():has( "LOSrange", "W93_blackout" ) then
							agent:getModifiers():add( "LOSrange", "W93_blackout", modifiers.SET, 3 )
							sim:refreshUnitLOS( agent )
						end
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isAgent and not evData:getTraits().isGuard then
					if not evData:getTraits().LOSrange and not evData:getModifiers():has( "LOSrange", "W93_blackout" ) then
						evData:getTraits().LOSrange = 50
					end
					if not evData:getModifiers():has( "LOSrange", "W93_blackout" ) then
						evData:getModifiers():add( "LOSrange", "W93_blackout", modifiers.SET, 3 )
						sim:refreshUnitLOS( evData )
					end
				end
			end,

			removeAbility = function( self, sim )
				for _, agent in pairs(sim:getPC():getUnits()) do
					if agent:getModifiers():has( "LOSrange", "W93_blackout" ) then
						agent:getModifiers():remove( "W93_blackout" )
						sim:refreshUnitLOS( agent )
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.TRANSLOCATOR_EQUIPMENT,
			desc = STRINGS.PROGEXTEND.AI.TRANSLOCATOR_EQUIPMENT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_translocator_grenades.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				local itemdefs = include( "sim/unitdefs/itemdefs" )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					if unit:getTraits().isGuard and not unit:getTraits().isDrone and not unit:hasChild( "W93_item_teleportGrenade" ) then
						local hasGrenade = false
						for i, child in pairs(unit:getChildren()) do
							if child:getTraits().slot and child:getTraits().slot == "grenade" then
								hasGrenade = true
							end
						end
						if hasGrenade == false then
							local item = simfactory.createUnit( itemdefs.W93_item_teleportGrenade, sim )
							sim:spawnUnit( item )
							unit:addChild( item )
							if not self.pc_grenade then
								local item = simfactory.createUnit( itemdefs.W93_item_teleportGrenade_pc , sim )
								sim:spawnUnit( item )
								unit:addChild( item )
								self.pc_grenade = true
							end
						end
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard and not evData:getTraits().isDrone then
					local hasGrenade = false
					for i, child in pairs(evData:getChildren()) do
						if child:getTraits().slot and child:getTraits().slot == "grenade" then
							hasGrenade = true
						end
					end
					if hasGrenade == false then
						local itemdefs = include( "sim/unitdefs/itemdefs" )
						local item = simfactory.createUnit( itemdefs.W93_item_teleportGrenade, sim )
						sim:spawnUnit( item )
						evData:addChild( item )
					end
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					for j, childUnit in pairs(unit:getChildren()) do
						if childUnit:hasTrait("guardBeacon") then
							unit:removeChild( childUnit )
							sim:despawnUnit( childUnit )
							if unit:getBrain() then
								unit:getBrain():reset()
							end
						end
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SNARE_EQUIPMENT,
			desc = STRINGS.PROGEXTEND.AI.SNARE_EQUIPMENT_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_snare_equipment.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 4,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				local itemdefs = include( "sim/unitdefs/itemdefs" )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					if unit:getTraits().isGuard and not unit:getTraits().isDrone and not unit:hasChild( "W93_item_snareGrenade" ) then
						local hasGrenade = false
						for i, child in pairs(unit:getChildren()) do
							if child:getTraits().slot and child:getTraits().slot == "grenade" then
								hasGrenade = true
							end
						end
						if hasGrenade == false then
							local item = simfactory.createUnit( itemdefs.W93_item_snareGrenade, sim )
							sim:spawnUnit( item )
							unit:addChild( item )
							if not self.pc_grenade2 then
								local item = simfactory.createUnit( itemdefs.W93_item_snareGrenade_pc, sim )
								sim:spawnUnit( item )
								unit:addChild( item )
								self.pc_grenade2 = true
							end
						end
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isGuard and not evData:getTraits().isDrone then
					local hasGrenade = false
					for i, child in pairs(evData:getChildren()) do
						if child:getTraits().slot and child:getTraits().slot == "grenade" then
							hasGrenade = true
						end
					end
					if hasGrenade == false then
						local itemdefs = include( "sim/unitdefs/itemdefs" )
						local item = simfactory.createUnit( itemdefs.W93_item_snareGrenade, sim )
						sim:spawnUnit( item )
						evData:addChild( item )
					end
				end
			end,

			removeAbility = function( self, sim )
				for i, unit in pairs(sim:getNPC():getUnits()) do
					for j, childUnit in pairs(unit:getChildren()) do
						if childUnit:hasTrait("snareGrenade") then
							unit:removeChild( childUnit )
							sim:despawnUnit( childUnit )
							if unit:getBrain() then
								unit:getBrain():reset()
							end
						end
					end
				end
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PROXIMAL,
			desc = STRINGS.PROGEXTEND.AI.PROXIMAL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_proximal.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				sim:getLevelScript():addHook( "GUARD LOOTED", guardLooted, true )
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "FUNCLIB-ITEM-STOLEN" then
					self.ProximalItemStolen = evData
				end
				if ( evType == simdefs.TRG_UI_ACTION and evData and evData.uiEvent and evData.uiEvent == level.EV_CLOSE_LOOT_UI ) and self.ProximalItemStolen then
					local guard = self.ProximalItemStolen.targetUnit
					local agent = self.ProximalItemStolen.unit
					if guard:getTraits().isGuard and agent:getTraits().isAgent and not guard:isDown() then
						local x0,y0 = agent:getLocation()
						local x1,y1 = guard:getLocation()
						local dist = mathutil.dist2d( x0, y0, x1, y1 )
						if dist <= 1 then
							self.displayNotification(self, sim, slot, string.format(STRINGS.PROGEXTEND.AI.PROXIMAL_DESC, guard:getName()), "SpySociety/Actions/mainframe_PWRreverse_off", guard )
							sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
							guard:turnToFace(x0, y0)
							self.ProximalItemStolen = nil
						end
					end
				end
			end,

			removeAbility = function( self, sim )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SHOCK_GROUNDING,
			desc = STRINGS.PROGEXTEND.AI.SHOCK_GROUNDING_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_anti_shock_grounding.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 5,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_ALARM_STATE_CHANGE then
					if not self.isKOBoosted then self.isKOBoosted = 0 end
					if not sim:getNPC():getTraits().boostKOresist then
						sim:getNPC():getTraits().boostKOresist = 1
					else
						sim:getNPC():getTraits().boostKOresist = sim:getNPC():getTraits().boostKOresist + 1
						self.isKOBoosted = self.isKOBoosted + 1
					end
				end
			end,

			removeAbility = function( self, sim )
				if sim:getNPC():getTraits().boostKOresist then
					sim:getNPC():getTraits().boostKOresist = sim:getNPC():getTraits().boostKOresist - (self.isKOBoosted or 0)
				end
				self.isKOBoosted = nil
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.ACCUMULATOR,
			desc = STRINGS.PROGEXTEND.AI.ACCUMULATOR_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_accumulator.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 100,
			hashKeysSpent = 0,
			excludeIfSingle = true,

			onSpawnAbility = function(  self, sim, slot )
				if not sim:getNPC():getTraits().PWRmaxBouns then
					sim:getNPC():getTraits().PWRmaxBouns = 0
				end
				sim:getNPC():getTraits().PWRmaxBouns = sim:getNPC():getTraits().PWRmaxBouns + 10
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getNPC():getTraits().PWRmaxBouns = sim:getNPC():getTraits().PWRmaxBouns - 10
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PWR_EFFICIENCY_1,
			desc = STRINGS.PROGEXTEND.AI.PWR_EFFICIENCY_1_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.PWR_EFFICIENCY_1_TIP2,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_pwr_efficiency.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 8,
			hashKeysSpent = 0,
			excludeIfSingle = true,

			onSpawnAbility = function( self, sim, slot )
				self.pwrDiscount = self.pwrDiscount + 1
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getNPC():getTraits().AI_Efficiency = nil
				self.pwrDiscount = self.pwrDiscount - 1
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.OVERCLOCK_1,
			desc = STRINGS.PROGEXTEND.AI.OVERCLOCK_1_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.OVERCLOCK_1_TIP2,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_overclock.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 6,
			maxDifficulty = 8,
			hashKeysSpent = 0,
			excludeIfSingle = true,

			onSpawnAbility = function( self, sim, slot )
				self.cooldownReduction = self.cooldownReduction + 1
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getNPC():getTraits().AI_Overclock = nil
				self.cooldownReduction = self.cooldownReduction - 1
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.PWR_EFFICIENCY_2,
			desc = STRINGS.PROGEXTEND.AI.PWR_EFFICIENCY_2_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.PWR_EFFICIENCY_1_TIP2,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_pwr_efficiency_plus.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 9,
			maxDifficulty = 100,
			hashKeysSpent = 0,
			excludeIfSingle = true,

			onSpawnAbility = function( self, sim, slot )
				self.pwrDiscount = self.pwrDiscount + 2
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				self.pwrDiscount = self.pwrDiscount - 2
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.OVERCLOCK_2,
			desc = STRINGS.PROGEXTEND.AI.OVERCLOCK_2_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.OVERCLOCK_1_TIP2,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_overclock_plus.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 9,
			maxDifficulty = 100,
			hashKeysSpent = 0,
			excludeIfSingle = true,

			onSpawnAbility = function( self, sim, slot )
				self.cooldownReduction = self.cooldownReduction + 2
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				self.cooldownReduction = self.cooldownReduction - 2
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.EXTENSION_2,
			desc = STRINGS.PROGEXTEND.AI.EXTENSION_2_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_extension_plus.png",
			iconType = "gui/icons/thought_icons/status_action.png",	
			type = "UTI",
			identified = false,
			corp = "general",
			minDifficulty = 9,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				sim:getPC():getTraits().daemonDurationModd = (sim:getPC():getTraits().daemonDurationModd or 0) + 2
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				sim:getPC():getTraits().daemonDurationModd = sim:getPC():getTraits().daemonDurationModd - 2
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.SELF_EVOLVING_PROGRAMMING,
			desc = STRINGS.PROGEXTEND.AI.SELF_EVOLVING_PROGRAMMING_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_self_evolving_programming.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "omni2",
			minDifficulty = 1,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )

			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == simdefs.TRG_ALARM_STATE_CHANGE and ((#self.getAiAbilities(self, sim) < 10 and sim:getParams().difficulty <= 6) or (#self.getAiAbilities(self, sim) < 15 and sim:getParams().difficulty >= 7)) then
					self.spawnAiAblility( self, sim )
					self.displayNotification(self, sim, slot, STRINGS.PROGEXTEND.AI.SELF_EVOLVING_PROGRAMMING_DESC, "SpySociety/Actions/mainframe_PWRreverse_off", nil )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60)
				end
			end,

			removeAbility = function( self, sim )
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.CATALYST,
			desc = STRINGS.PROGEXTEND.AI.CATALYST_TIP,
			specialdesc = STRINGS.PROGEXTEND.AI.CATALYST_TIP2,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_catalyst.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "omni",
			minDifficulty = 1,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				if not sim:getNPC():getTraits().PWRmaxBouns then
					sim:getNPC():getTraits().PWRmaxBouns = 0
				end

				self.cooldownReduction = self.cooldownReduction + 3
				self.pwrDiscount = self.pwrDiscount + 3
				sim:getNPC():getTraits().PWRmaxBouns = sim:getNPC():getTraits().PWRmaxBouns + 5
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )

			end,

			removeAbility = function( self, sim )
				self.cooldownReduction = self.cooldownReduction - 3
				self.pwrDiscount = self.pwrDiscount - 3
				sim:getNPC():getTraits().PWRmaxBouns = sim:getNPC():getTraits().PWRmaxBouns - 5
			end,
		},
		{
			name = STRINGS.PROGEXTEND.AI.CHILL,
			desc = STRINGS.PROGEXTEND.AI.CHILL_TIP,
			shortdesc = STRINGS.PROGEXTEND.AI.SUBROUTINE_TYPE_UTI,
			icon = "gui/icons/ai_icons/icon-ai_chill.png",
			iconType = "gui/icons/thought_icons/status_action.png",
			type = "UTI",
			identified = false,
			corp = "omni",
			minDifficulty = 1,
			maxDifficulty = 100,
			hashKeysSpent = 0,

			onSpawnAbility = function( self, sim, slot )
				for _, agent in pairs(sim:getPC():getUnits()) do
					if agent:getTraits().isAgent and not agent:getTraits().isGuard and agent:getTraits().mpMax then
						if not agent:getModifiers():has( "mpMax", "W93_chill" ) then
							agent:getModifiers():add( "mpMax", "W93_chill", modifiers.ADD, -3 )
							agent:useMP( 3, sim )
						end
					end
				end
			end,

			executeAbility = function( self, sim, evType, evData, userUnit, slot )
				if evType == "TRG_UNIT_SPAWNED" and evData:getTraits().isAgent and not evData:getTraits().isGuard then
					if evData:getTraits().mpMax and not evData:getModifiers():has( "mpMax", "W93_chill" ) then
						evData:getModifiers():add( "mpMax", "W93_chill", modifiers.ADD, -3 )
						evData:useMP( 3, sim )
					end
				end
			end,

			removeAbility = function( self, sim )
				for _, agent in pairs(sim:getPC():getUnits()) do
					if agent:getModifiers():has( "mpMax", "W93_chill" ) then
						agent:getModifiers():remove( "W93_chill" )
					end
				end
			end,
		},
	},
}

local npc_abilities2 =
{
	W93_AI_assembly = util.extend( createDaemon( STRINGS.PROGEXTEND.DAEMONS.ASSEMBLY ) )
	{
		icon = "gui/icons/daemon_icons/icon-daemon_AI.png",
		standardDaemon = false,
		reverseDaemon = false,
		premanent = true,
		slotsPWR = {},
		slotsPro = {},
		slotsRea = {},
		slotsUti = {},
		activePrograms = 0,
		pwrDiscount = 0,
		cooldownReduction = 0,
		deletionRequirement = 4,
		gens = {},
		PWRtable =
		{
			["PWR"] = 0,
			["PRO"] = 0,
			["REA"] = 0,
			["UTI"] = 1,
		},
		CooldownTable =
		{
			["PWR"] = 2,
			["PRO"] = 1,
			["REA"] = 1,
			["UTI"] = 0,
		},
		HashTable =
		{
			["PWR"] = 1,
			["PRO"] = 1,
			["REA"] = 1,
			["UTI"] = 2,
		},
		chitonCooldown =
		{
			[0] = 0,
			[1] = 0
		},

	     	ENDLESS_DAEMONS = false,
	     	PROGRAM_LIST = false,
	     	OMNI_PROGRAM_LIST_EASY = false,
	     	OMNI_PROGRAM_LIST = false,
	     	REVERSE_DAEMONS = false,

		onSpawnAbility = function( self, sim, player )
			sim:addTrigger( simdefs.TRG_START_TURN, self )
			sim:addTrigger( simdefs.TRG_END_TURN, self )
			sim:addTrigger( simdefs.TRG_UNIT_HIJACKED, self )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self )
			sim:addTrigger( simdefs.TRG_SAFE_LOOTED, self )
			sim:addTrigger( simdefs.TRG_BUY_ITEM, self )
			sim:addTrigger( simdefs.TRG_UI_ACTION, self )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
			sim:addTrigger( simdefs.TRG_CAUGHT_BY_CAMERA, self )
			sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:addTrigger( simdefs.TRG_OPEN_DOOR, self )
			sim:addTrigger( simdefs.TRG_SET_OVERWATCH, self )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			sim:addTrigger( simdefs.TRG_SOUND_EMITTED, self)
			sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:addTrigger( "TRG_UNIT_SPAWNED", self)
			sim:addTrigger( "FUNCLIB-ITEM-STOLEN", self)

			local slotNumber =
			{
				["sankaku"] =
				{
					[1] = { 0, 0, 1, 0 },
					[2] = { 0, 1, 1, 0 },
					[3] = { 0, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 1, 2, 1 },
					[6] = { 1, 2, 2, 1 },
					[7] = { 1, 2, 2, 2 },
					[8] = { 1, 2, 3, 2 },
					[9] = { 1, 3, 3, 2 },
					[10] = { 1, 3, 3, 3 },
					[11] = { 2, 3, 3, 3 },
					[12] = { 2, 3, 4, 3 },
					[13] = { 2, 4, 4, 3 },
					[14] = { 2, 4, 4, 4 },
					[15] = { 2, 4, 5, 4 },
				},
				["plastech"] =
				{
					[1] = { 0, 1, 0, 0 },
					[2] = { 0, 1, 0, 1 },
					[3] = { 0, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 2, 1, 1 },
					[6] = { 1, 2, 1, 2 },
					[7] = { 1, 2, 2, 2 },
					[8] = { 1, 3, 2, 2 },
					[9] = { 1, 3, 2, 3 },
					[10] = { 1, 3, 3, 3 },
					[11] = { 2, 3, 3, 3 },
					[12] = { 2, 4, 3, 3 },
					[13] = { 2, 4, 3, 4 },
					[14] = { 2, 4, 4, 4 },
					[15] = { 2, 5, 4, 4 },
				},
				["ftm"] =
				{
					[1] = { 0, 1, 0, 0 },
					[2] = { 0, 1, 1, 0 },
					[3] = { 0, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 2, 1, 1 },
					[6] = { 1, 2, 2, 1 },
					[7] = { 1, 2, 2, 2 },
					[8] = { 1, 3, 2, 2 },
					[9] = { 1, 3, 3, 2 },
					[10] = { 1, 3, 3, 3 },
					[11] = { 2, 3, 3, 3 },
					[12] = { 2, 4, 3, 3 },
					[13] = { 2, 4, 4, 3 },
					[14] = { 2, 4, 4, 4 },
					[15] = { 2, 5, 4, 4 },
				},
				["ko"] =
				{
					[1] = { 0, 0, 0, 1 },
					[2] = { 0, 1, 0, 1 },
					[3] = { 0, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 1, 1, 2 },
					[6] = { 1, 2, 1, 2 },
					[7] = { 1, 2, 2, 2 },
					[8] = { 1, 2, 2, 3 },
					[9] = { 1, 3, 2, 3 },
					[10] = { 1, 3, 3, 3 },
					[11] = { 2, 3, 3, 3 },
					[12] = { 2, 3, 3, 4 },
					[13] = { 2, 4, 3, 4 },
					[14] = { 2, 4, 4, 4 },
					[15] = { 2, 4, 4, 5 },
				},
				["neptune"] =
				{
					[1] = { 0, 0, 1, 0 },
					[2] = { 0, 0, 1, 1 },
					[3] = { 0, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 1, 2, 1 },
					[6] = { 1, 1, 2, 2 },
					[7] = { 1, 2, 2, 2 },
					[8] = { 1, 2, 3, 2 },
					[9] = { 1, 2, 3, 3 },
					[10] = { 1, 3, 3, 3 },
					[11] = { 2, 3, 3, 3 },
					[12] = { 2, 3, 4, 3 },
					[13] = { 2, 3, 4, 4 },
					[14] = { 2, 4, 4, 4 },
					[15] = { 2, 4, 5, 4 },
				},
				["omni"] =
				{
					[1] = { 1, 1, 1, 1 },
					[2] = { 1, 1, 1, 1 },
					[3] = { 1, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 1, 1, 1 },
					[6] = { 1, 1, 1, 1 },
					[7] = { 1, 2, 2, 2 },
					[8] = { 1, 2, 2, 2 },
					[9] = { 1, 2, 2, 2 },
					[10] = { 1, 2, 2, 2 },
					[11] = { 1, 2, 2, 2 },
					[12] = { 1, 2, 2, 2 },
					[13] = { 1, 2, 2, 2 },
					[14] = { 1, 2, 2, 2 },
					[15] = { 1, 2, 2, 2 },
				},
				["omni2"] =
				{
					[1] = { 1, 1, 1, 1 },
					[2] = { 1, 1, 1, 1 },
					[3] = { 1, 1, 1, 1 },
					[4] = { 1, 1, 1, 1 },
					[5] = { 1, 1, 1, 1 },
					[6] = { 1, 1, 1, 1 },
					[7] = { 1, 1, 1, 1 },
					[8] = { 1, 1, 1, 1 },
					[9] = { 1, 1, 1, 1 },
					[10] = { 1, 1, 1, 1 },
					[11] = { 1, 1, 1, 1 },
					[12] = { 1, 1, 1, 1 },
					[13] = { 1, 1, 1, 1 },
					[14] = { 1, 1, 1, 1 },
					[15] = { 1, 1, 1, 1 },
				}
			}
			local choice = {}
			local diff = math.min(sim:getParams().difficulty, 15)
			self.activePrograms = 0
			self.pwrDiscount = 0
			self.cooldownReduction = 0
			sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken or 0
			sim:getPC():getTraits().aiTokenMax = sim:getPC():getTraits().aiTokenMax or 8

			if sim:getParams().difficultyOptions.W93_AI > 0 then
				diff = diff - sim:getParams().difficultyOptions.W93_AI + 1
			end

			if sim:getParams().agency.MM_hostileAInerf and sim:getParams().agency.MM_hostileAInerf[sim:getParams().world] then
				diff = math.max(1, diff - sim:getParams().agency.MM_hostileAInerf[sim:getParams().world])
			end

			if not (sim:getParams().world == "omni" or sim:getParams().world == "omni2") then
				diff = math.min(diff, sim:getParams().difficultyOptions.W93_AI_CAP or 15)
			end

			for _, genUnit in pairs( sim:getAllUnits() ) do
				if genUnit:getTraits().AI_PWR_Core then
					table.insert(self.gens, genUnit)
				end
			end

			for i, routine in pairs(subroutines.PWR) do
				if ( routine.corp == sim:getParams().world or ( sim:getParams().world == "omni" and routine.corp == "omni2") or (routine.corp == "general" and not ( sim:getParams().world == "omni" or sim:getParams().world == "omni2" )) ) and diff >= routine.minDifficulty and diff <= routine.maxDifficulty then
					if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
						table.insert(choice, routine)
					end
				end
			end
			for i=1, slotNumber[sim:getParams().world][diff][1] do
				if #choice > 0 then
					local selected = sim:nextRand(1,#choice)
					choice[selected].identified = false
					choice[selected].cooldown = 1
					table.insert(self.slotsPWR,choice[selected])
					table.remove(choice,selected)
				end
			end

			util.tclear(choice)
			for i, routine in pairs(subroutines.PROACTIVE) do
				if ( routine.corp == sim:getParams().world or ( sim:getParams().world == "omni" and routine.corp == "omni2") or (routine.corp == "general" and not ( sim:getParams().world == "omni" or sim:getParams().world == "omni2" )) ) and diff >= routine.minDifficulty and diff <= routine.maxDifficulty then
					if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
						table.insert(choice, routine)
					end
				end
			end
			for i=1, slotNumber[sim:getParams().world][diff][2] do
				if #choice > 0 then
					local selected = sim:nextRand(1,#choice)
					choice[selected].identified = false
					choice[selected].cooldown = 2
					table.insert(self.slotsPro,choice[selected])
					table.remove(choice,selected)
					self.activePrograms = self.activePrograms + 1
				end
			end

			util.tclear(choice)
			for i, routine in pairs(subroutines.REACTIVE) do
				if ( routine.corp == sim:getParams().world or ( sim:getParams().world == "omni" and routine.corp == "omni2") or (routine.corp == "general" and not ( sim:getParams().world == "omni" or sim:getParams().world == "omni2" )) ) and diff >= routine.minDifficulty and diff <= routine.maxDifficulty then
					if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
						table.insert(choice, routine)
					end
				end
			end
			for i=1, slotNumber[sim:getParams().world][diff][3] do
				if #choice > 0 then
					local selected = sim:nextRand(1,#choice)
					choice[selected].identified = false
					choice[selected].cooldown = 0
					table.insert(self.slotsRea,choice[selected])
					table.remove(choice,selected)
					self.activePrograms = self.activePrograms + 1
				end
			end

			util.tclear(choice)
			for i, routine in pairs(subroutines.UTILITY) do
				if ( routine.corp == sim:getParams().world or ( sim:getParams().world == "omni" and routine.corp == "omni2") or (routine.corp == "general" and not ( sim:getParams().world == "omni" or sim:getParams().world == "omni2" )) ) and diff >= routine.minDifficulty and diff <= routine.maxDifficulty then
					if not (routine.excludeIfSingle and self.activePrograms <= 0) then
						if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
							table.insert(choice, routine)
						end
					end
				end
			end
			for i=1, slotNumber[sim:getParams().world][diff][4] do
				if #choice > 0 then
					local selected = sim:nextRand(1,#choice)
					choice[selected].identified = false
					choice[selected].hashKeysSpent = 0
					table.insert(self.slotsUti,choice[selected])
					table.remove(choice,selected)
				end
			end

			for i, routine in pairs(self.slotsUti) do
				routine.onSpawnAbility(self, sim, routine)
			end

			sim:getNPC():addCPUs( 20 - sim:getParams().difficultyOptions.startingPower )

			if sim:getParams().world == "omni2" or sim:getParams().world == "omni" then
				self.choicePWR = {}
				self.choicePRO = {}
				self.choiceREA = {}
				self.choiceUTI = {}
				for i, routine in pairs(subroutines.PWR) do
					if routine.corp == "general" then
						if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
							table.insert(self.choicePWR, routine)
						end
					end
				end
				for i, routine in pairs(subroutines.PROACTIVE) do
					if routine.corp == "general" then
						if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
							table.insert(self.choicePRO, routine)
						end
					end
				end
				for i, routine in pairs(subroutines.REACTIVE) do
					if routine.corp == "general" then
						if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
							table.insert(self.choiceREA, routine)
						end
					end
				end
				for i, routine in pairs(subroutines.UTILITY) do
					if routine.corp == "general" then
						if (not routine.requiresGens) or (routine.requiresGens and #self.gens > 0) then
							table.insert(self.choiceUTI, routine)
						end
					end
				end
			end
			self.diff = diff
			sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
        	end,

		onDespawnAbility = function( self, sim )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_HIJACKED, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_SAFE_LOOTED, self )
			sim:removeTrigger( simdefs.TRG_BUY_ITEM, self )
			sim:removeTrigger( simdefs.TRG_UI_ACTION , self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP , self )
			sim:removeTrigger( simdefs.TRG_CAUGHT_BY_CAMERA, self )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:removeTrigger( simdefs.TRG_OPEN_DOOR, self )
			sim:removeTrigger( simdefs.TRG_SET_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
			sim:removeTrigger( simdefs.TRG_SOUND_EMITTED, self)
			sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:removeTrigger( "TRG_UNIT_SPAWNED", self)
			sim:removeTrigger( "FUNCLIB-ITEM-STOLEN", self)
		end,

    		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				for i, slot in pairs(self.slotsPWR) do
					if slot.cooldown > 0 then
						slot.cooldown = slot.cooldown - 1
					end
				end
				for i, slot in pairs(self.slotsPro) do
					if slot.cooldown > 0 then
						slot.cooldown = slot.cooldown - 1
					end
				end
				for i, slot in pairs(self.slotsRea) do
					if slot.cooldown > 0 then
						slot.cooldown = slot.cooldown - 1
					end
				end
				if self.lockdown and self.lockdown > 0 then
					self.lockdown = self.lockdown - 1
					if self.lockdown <= 0 then
						self.lockdown = nil
						sim:setMainframeLockout( false )
					end
				end
				if self.supervisionCooldown and self.supervisionCooldown == 0 then
					self.supervisionUnit:getTraits().iceImmune = nil
					self.supervisionUnit = nil
					self.supervisionCooldown = nil
				elseif self.supervisionCooldown and self.supervisionCooldown > 0 then
					self.supervisionCooldown = self.supervisionCooldown - 1
				end

				if self.chitonCooldown and self.chitonCooldown[0] > 0 then
					self.chitonCooldown[0] = self.chitonCooldown[0] - 1
				end
				if self.chitonCooldown and self.chitonCooldown[0] == 0 and self.chitonCooldown[1] > 0 then
					sim:getNPC():getTraits().boostArmor = sim:getNPC():getTraits().boostArmor - 1
					self.chitonCooldown[1] = self.chitonCooldown[1] - 1
				end

				if self.hasteCooldown and self.hasteCooldown == 0 then
					self.hasteCooldown = nil
					for i, unit in pairs(sim:getNPC():getUnits()) do
						while unit:getModifiers():has( "mpMax", "AI_Haste" ) do
							unit:getModifiers():remove( "AI_Haste" )
						end
						unit:getTraits().mp = unit:getTraits().mpMax
					end
				elseif self.hasteCooldown and self.hasteCooldown > 0 then
					self.hasteCooldown = self.hasteCooldown - 1
				end

				if #self.gens <= 0 and self.diff > 1 then
					sim:getNPC():addCPUs( 1 )
				end
			end

			if evType == simdefs.TRG_UNIT_HIJACKED and evData.unit:getTraits().mainframe_console and not evData.abort and (not evData.action) then
				sim:getPC():getTraits().aiToken = math.min(sim:getPC():getTraits().aiToken + 1, sim:getPC():getTraits().aiTokenMax)
				sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
			end

			for i, routine in pairs(self.slotsPWR) do
				routine.onTrigger(self, sim, evType, evData, userUnit, routine)
			end

			for i, routine in pairs(self.slotsPro) do
				if routine.canUseAbility(self, sim, evType, evData, userUnit, routine) then
					routine.executeAbility(self, sim, evType, evData, userUnit, routine)
				end
			end

			for i, routine in pairs(self.slotsRea) do
				if routine.canUseAbility(self, sim, evType, evData, userUnit, routine) then
					routine.executeAbility(self, sim, evType, evData, userUnit, routine)
				end
			end

			for i, routine in pairs(self.slotsUti) do
				routine.executeAbility(self, sim, evType, evData, userUnit, routine)
			end
     		end,

		executeAbility = function(self, sim, abilityOwner, abilityUser, slot)
			local category, index = self.findAiAbility(self, slot.name, slot.type)

			if category == "PWR" then
				if not self.slotsPWR[index].identified then
					self.slotsPWR[index].identified = true
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - 1
				else
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.AI.ABILITY_DELAY_WARNING, self.slotsPWR[index].name), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.slotsPWR[index].icon or "gui/icons/ai_icons/icon-ai_unknown.png" } )
					if sim:getPC():hasMainframeAbility( "W93_hushpuppy" ) or sim:getPC():hasMainframeAbility( "W93_hushpuppy_PWR" ) then
						self.slotsPWR[index].cooldown = self.slotsPWR[index].cooldown + self.CooldownTable["PWR"] * 2
					else
						self.slotsPWR[index].cooldown = self.slotsPWR[index].cooldown + self.CooldownTable["PWR"]
					end
					sim:getPC():addCPUs( -self.PWRtable["PWR"] )
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - self.HashTable["PWR"]
				end
				sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
			elseif category == "PRO" then
				if not self.slotsPro[index].identified then
					self.slotsPro[index].identified = true
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - 1
				else
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.AI.ABILITY_DELAY_WARNING, self.slotsPro[index].name), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.slotsPro[index].icon or "gui/icons/ai_icons/icon-ai_unknown.png" } )
					if sim:getPC():hasMainframeAbility( "W93_hushpuppy" ) or sim:getPC():hasMainframeAbility( "W93_hushpuppy_PRO" ) then
						self.slotsPro[index].cooldown = self.slotsPro[index].cooldown + self.CooldownTable["PRO"] * 2
					else
						self.slotsPro[index].cooldown = self.slotsPro[index].cooldown + self.CooldownTable["PRO"]
					end
					sim:getPC():addCPUs( -self.PWRtable["PRO"] )
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - self.HashTable["PRO"]
				end
				sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
			elseif category == "REA" then
				if not self.slotsRea[index].identified then
					self.slotsRea[index].identified = true
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - 1
				else
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
					sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.AI.ABILITY_DELAY_WARNING, self.slotsRea[index].name), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.slotsRea[index].icon or "gui/icons/ai_icons/icon-ai_unknown.png" } )
					if sim:getPC():hasMainframeAbility( "W93_hushpuppy" ) or sim:getPC():hasMainframeAbility( "W93_hushpuppy_REA" ) then
						self.slotsRea[index].cooldown = self.slotsRea[index].cooldown + self.CooldownTable["REA"] * 2
					else
						self.slotsRea[index].cooldown = self.slotsRea[index].cooldown + self.CooldownTable["REA"]
					end
					sim:getPC():addCPUs( -self.PWRtable["REA"] )
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - self.HashTable["REA"]
				end
				sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
			elseif category == "UTI" then
				if not self.slotsUti[index].identified then
					self.slotsUti[index].identified = true
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - 1
				else
					self.removeAblility(self, sim, self.slotsUti[index])
					sim:getPC():addCPUs( -self.PWRtable["UTI"] )
					sim:getPC():getTraits().aiToken = sim:getPC():getTraits().aiToken - self.HashTable["UTI"]
				end
				sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
			end
		end,

		onTooltip = function( self, hud, sim, player )
			local tooltip = util.tooltip( hud._screen )
			local section = tooltip:addSection()

			section:addLine( "<ttheader>"..self.name.."</>" )
			section:addLine( string.format(STRINGS.PROGEXTEND.AI.PWR, sim:getNPC():getCpus(), sim:getNPC():getMaxCpus()) )
			section:addLine( string.format(STRINGS.PROGEXTEND.AI.TOKEN, sim:getPC():getTraits().aiToken, sim:getPC():getTraits().aiTokenMax) )
	   		section:addAbility( STRINGS.PROGEXTEND.AI.AI_TITLE, STRINGS.PROGEXTEND.AI.AI_DESC, "gui/icons/action_icons/Action_icon_Small/icon-action_chargeweapon_small.png" )
			section:addAbility( STRINGS.PROGEXTEND.AI.AI_SCAN_TITLE, STRINGS.PROGEXTEND.AI.AI_SCAN_DESC, "gui/icons/skills_icons/skills_icon_small/icon-item_analyze_small.png" )

			if STRINGS.MOREMISSIONS and sim:getParams().agency.MM_hostileAInerf and sim:getParams().agency.MM_hostileAInerf[sim:getParams().world] then
				section:addAbility( STRINGS.MOREMISSIONS.UI.TOOLTIPS.HOSTILE_AI_WEAKEN, util.sformat(STRINGS.MOREMISSIONS.UI.TOOLTIPS.HOSTILE_AI_WEAKEN_DESC, sim:getParams().agency.MM_hostileAInerf[sim:getParams().world]
)
, "gui/icons/mission_icons/mission_ai_terminal_small.png" )
			end

			if #self.gens <= 0 and self.diff > 1 then
				section:addAbility( STRINGS.PROGRAMS.POWER_DRIP.NAME, STRINGS.PROGRAMS.POWER_DRIP.DESC, "gui/icons/programs_icons/store_icons/StorePrograms_drip.png" )
			end

			if self.dlcFooter then
				section:addFooter(self.dlcFooter[1], self.dlcFooter[2])
			end	        

			return tooltip
		end,

		onAbilityTooltip = function( self, hud, sim, player, slot )
			local tooltip = util.tooltip( hud._screen )
			local section = tooltip:addSection()

			if not slot.identified then
				section:addLine( "<ttheader>"..STRINGS.PROGEXTEND.AI.UNKNOWN.."</>")
	   			section:addAbility( STRINGS.PROGEXTEND.AI.ABILITY_IDENTIFY, STRINGS.PROGEXTEND.AI.ABILITY_IDENTIFY_DESC, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )
			else
				local sub = ""
				if slot.pwrCost then sub = util.sformat( STRINGS.PROGRAMS.POWER, self.getPwrCost(self, slot)) end
				section:addLine( "<ttheader>"..slot.name.."</>",  sub )
				if slot.cooldownMax then section:addLine( util.sformat( STRINGS.PROGRAMS.COOLDOWN, self.getMaxCooldown(self, slot) )  ) end
				section:addAbility( slot.shortdesc, slot.desc, slot.iconType )
				if slot.specialdesc then
					section:addAbility( STRINGS.PROGEXTEND.AI.SPECIAL_TIP, slot.specialdesc, "gui/icons/skills_icons/skills_icon_small/icon-item_analyze_small.png" )
				end
				if slot.type == "PWR" or slot.type == "PRO" or slot.type == "REA" then
	   				section:addAbility( STRINGS.PROGEXTEND.AI.ABILITY_DELAY, string.format(STRINGS.PROGEXTEND.AI.ABILITY_DELAY_DESC, self.CooldownTable[slot.type]), "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )
				end
				if slot.type == "UTI" then
	   				section:addAbility( STRINGS.PROGEXTEND.AI.ABILITY_DELETE, STRINGS.PROGEXTEND.AI.ABILITY_DELETE_DESC, "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png" )
				end
			end
			section:addLine( string.format(STRINGS.PROGEXTEND.AI.PWR, sim:getNPC():getCpus(), sim:getNPC():getMaxCpus()) )
			section:addLine( string.format(STRINGS.PROGEXTEND.AI.TOKEN, sim:getPC():getTraits().aiToken, sim:getPC():getTraits().aiTokenMax) )
			if slot.hashKeysSpent and slot.identified then
				section:addLine( string.format(STRINGS.PROGEXTEND.AI.DELETION_PROGRESS, slot.hashKeysSpent, self.deletionRequirement) )
			end

			if slot.identified and sim:getPC():getTraits().aiToken < self.HashTable[slot.type] then
				section:addRequirement( util.sformat(STRINGS.PROGEXTEND.UI.REASON_NO_TOKEN, self.HashTable[slot.type]) )
			elseif (not slot.identified) and sim:getPC():getTraits().aiToken < 1 then
				section:addRequirement( util.sformat(STRINGS.PROGEXTEND.UI.REASON_NO_TOKEN, 1) )
			elseif self.PWRtable[slot.type] > sim:getPC():getCpus() and slot.identified then
				section:addRequirement( STRINGS.UI.REASON.NOT_ENOUGH_PWR )
			end

			return tooltip
		end,

		getPwrCost = function(self, slot)
			if type(slot.pwrCost) == "number" then
				return math.max(slot.pwrCost - self.pwrDiscount, 1)
			elseif not slot.pwrCost then
				return "-"
			else
				return math.max(slot.pwrCost(self, slot) - self.pwrDiscount, 1)
			end
		end,

		getMaxCooldown = function(self, slot)
			local cd = math.max(slot.cooldownMax - self.cooldownReduction, 1)
			if slot.type == "PRO" then
				cd = math.max(cd - 1, 0)
			end
			return cd
		end,

		usePwr = function(self, sim, slot)
			if type(slot.pwrCost) == "number" then
				sim:getNPC():addCPUs(-math.max(slot.pwrCost - self.pwrDiscount, 1))
			else
				sim:getNPC():addCPUs(-math.max(slot.pwrCost(self, slot) - self.pwrDiscount, 1))
			end
		end,

		applyCooldown = function(self, slot)
			slot.cooldown = math.max(slot.cooldownMax - self.cooldownReduction, 1)
		end,

		getAiAbilities = function(self)
			local abil = {}
			util.tmerge(abil, self.slotsPWR, self.slotsPro, self.slotsRea, self.slotsUti)
			return abil
		end,

		findAiAbility = function(self, ability, category)
			if category == "PWR" then
				for i, routine in pairs(self.slotsPWR) do
					if routine.name == ability then
						return category, i
					end
				end
			elseif category == "PRO" then
				for i, routine in pairs(self.slotsPro) do
					if routine.name == ability then
						return category, i
					end
				end
			elseif category == "REA" then
				for i, routine in pairs(self.slotsRea) do
					if routine.name == ability then
						return category, i
					end
				end
			elseif category == "UTI" then
				for i, routine in pairs(self.slotsUti) do
					if routine.name == ability then
						return category, i
					end
				end
			end
			return ""
		end,

		spawnAiAblility = function(self, sim)
			local abilityType = 0
			while true do
				if (#self.slotsPro + #self.slotsRea) >= 6 and #self.slotsPWR == 1 then
					abilityType = 1
				elseif ((#self.slotsPro + #self.slotsRea) / #self.slotsUti) > 2 then
					abilityType = 4
				else
					abilityType = sim:nextRand(2,3)
				end
				if abilityType == 1 then
					if #self.choicePWR > 0 then
						local selected = sim:nextRand(1, #self.choicePWR)
						self.choicePWR[selected].identified = false
						self.choicePWR[selected].cooldown = 1
						table.insert(self.slotsPWR, self.choicePWR[selected])
						table.remove(self.choicePWR, selected)
						sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
						break
					end
				elseif abilityType == 2 then
					if #self.choicePRO > 0 then
						local selected = sim:nextRand(1, #self.choicePRO)
						self.choicePRO[selected].identified = false
						self.choicePRO[selected].cooldown = 2
						table.insert(self.slotsPro, self.choicePRO[selected])
						self.activePrograms = self.activePrograms + 1
						table.remove(self.choicePRO, selected)
						sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
						break
					end
				elseif abilityType == 3 then
					if #self.choiceREA > 0 then
						local selected = sim:nextRand(1, #self.choiceREA)
						self.choiceREA[selected].identified = false
						self.choiceREA[selected].cooldown = 0
						table.insert(self.slotsRea, self.choiceREA[selected])
						self.activePrograms = self.activePrograms + 1
						table.remove(self.choiceREA, selected)
						sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
						break
					end
				else
					if #self.choiceUTI > 0 then
						local selected = sim:nextRand(1, #self.choiceUTI)
						self.choiceUTI[selected].identified = false
						self.choiceUTI[selected].hashKeysSpent = 0
						table.insert(self.slotsUti, self.choiceUTI[selected])
						table.remove(self.choiceUTI, selected)
						self.slotsUti[#self.slotsUti].onSpawnAbility(self, sim, self.slotsUti[#self.slotsUti])
						sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
						break
					end
				end
			end
		end,

		removeAblility = function(self, sim, slot)
			for i, ability in pairs(self.slotsUti) do
				if ability.name == slot.name then
					self.slotsUti[i].hashKeysSpent = self.slotsUti[i].hashKeysSpent + 1
					if self.slotsUti[i].hashKeysSpent >= self.deletionRequirement then
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
						sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.AI.ABILITY_DELETE_WARNING, self.slotsUti[i].name), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.slotsUti[i].icon or "gui/icons/ai_icons/icon-ai_unknown.png" } )
						self.slotsUti[i].removeAbility( self, sim )
						table.remove(self.slotsUti, i)
					else
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, cdefs.SOUND_HUD_MAINFRAME_PROGRAM_AUTO_RUN )
						sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.PROGEXTEND.AI.ABILITY_DELETE_WARNING2, self.slotsUti[i].name, self.slotsUti[i].hashKeysSpent, self.deletionRequirement), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_gainCPU", icon=self.slotsUti[i].icon or "gui/icons/ai_icons/icon-ai_unknown.png" } )
					end
					break
				end
			end
		end,

		countGens = function(self, sim)
			if #self.gens == 0 then
				return 0
			end

			local count = 0
			for _, gen in pairs(self.gens) do
				if self.isGenValid(sim, gen) then
					count = count + 1
				end
			end

			return count
		end,

		isGenValid = function(sim, genUnit)
			if genUnit:getTraits().mainframe_status == "active" and genUnit:getPlayerOwner() == sim:getNPC() then
				return true
			end
		end,

		isNotNullZone = function(sim, targetUnit)
			local x0, y0 = targetUnit:getLocation()
			for unitID, unit in pairs( sim:getAllUnits() ) do
				local range = unit:getTraits().mainframe_suppress_range
				if range and not unit:isKO() and unit:getLocation() and unit ~= targetUnit then
					local distSqr = mathutil.distSqr2d( x0, y0, unit:getLocation() )
					if distSqr <= range * range then
						return false
					end
				end
			end

			return true
		end,

		findMainframeDevice = function(self, sim, slot, upperLimit, mode, condition, isHacked)
			local device = nil

			for i, unit in pairs(sim:getAllUnits()) do
				if unit:getTraits().mainframe_ice and ((not isHacked and unit:getTraits().mainframe_ice > 0) or (isHacked and unit:getTraits().mainframe_ice <= 0)) then
					if (not upperLimit) or (upperLimit and unit:getTraits().mainframe_ice <= upperLimit) then
						if not condition then
							if device == nil then
								device = unit
							elseif device ~= nil and mode == "LOWEST" and not isHacked and unit:getTraits().mainframe_ice < device:getTraits().mainframe_ice then
								device = unit
							elseif device ~= nil and mode == "HIGHEST" and not isHacked and unit:getTraits().mainframe_ice > device:getTraits().mainframe_ice then
								device = unit
							end
						elseif condition == "PARASITE" then
							if device == nil and unit:getTraits().parasite then
								device = unit
							elseif device ~= nil and mode == "LOWEST" and not isHacked and unit:getTraits().mainframe_ice < device:getTraits().mainframe_ice and unit:getTraits().parasite then
								device = unit
							elseif device ~= nil and mode == "HIGHEST" and not isHacked and unit:getTraits().mainframe_ice > device:getTraits().mainframe_ice and unit:getTraits().parasite then
								device = unit
							end
						elseif condition == "DAEMON" then
							if device == nil and unit:getTraits().mainframe_program then
								device = unit
							elseif device ~= nil and mode == "LOWEST" and not isHacked and unit:getTraits().mainframe_ice < device:getTraits().mainframe_ice and unit:getTraits().mainframe_program then
								device = unit
							elseif device ~= nil and mode == "HIGHEST" and not isHacked and unit:getTraits().mainframe_ice > device:getTraits().mainframe_ice and unit:getTraits().mainframe_program then
								device = unit
							end
						elseif condition == "NO DAEMON" then
							if device == nil and not unit:getTraits().mainframe_program then
								device = unit
							elseif device ~= nil and mode == "LOWEST" and not isHacked and unit:getTraits().mainframe_ice < device:getTraits().mainframe_ice and not unit:getTraits().mainframe_program then
								device = unit
							elseif device ~= nil and mode == "HIGHEST" and not isHacked and unit:getTraits().mainframe_ice > device:getTraits().mainframe_ice and not unit:getTraits().mainframe_program then
								device = unit
							end
						elseif condition == "NO NULL ZONE" then
							if device == nil and not unit:getTraits().mainframe_suppress_range and self.isNotNullZone(sim, unit) == true then
								device = unit
							elseif device ~= nil and mode == "LOWEST" and not isHacked and unit:getTraits().mainframe_ice < device:getTraits().mainframe_ice and not unit:getTraits().mainframe_suppress_range and self.isNotNullZone(sim, unit) == true then
								device = unit
							elseif device ~= nil and mode == "HIGHEST" and not isHacked and unit:getTraits().mainframe_ice > device:getTraits().mainframe_ice and not unit:getTraits().mainframe_suppress_range and self.isNotNullZone(sim, unit) == true then
								device = unit
							end
						elseif condition == "CAMERA" then
							if device == nil and unit:getTraits().mainframe_camera and not unit:getTraits().iceImmune and unit:getTraits().mainframe_status ~= "off" then
								device = unit
							elseif device ~= nil and mode == "LOWEST" and not isHacked and unit:getTraits().mainframe_ice < device:getTraits().mainframe_ice and unit:getTraits().mainframe_camera and not unit:getTraits().iceImmune and unit:getTraits().mainframe_status ~= "off" then
								device = unit
							elseif device ~= nil and mode == "HIGHEST" and not isHacked and unit:getTraits().mainframe_ice > device:getTraits().mainframe_ice and unit:getTraits().mainframe_camera and not unit:getTraits().iceImmune and unit:getTraits().mainframe_status ~= "off" then
								device = unit
							end
						end
					end
				end
			end

			return device
		end,

		displayNotification = function(self, sim, slot, txt, sound, target )
			if slot.identified then
				if target then 
					local x1, y1 = target:getLocation()
					if x1 and sim:getPC():getLastKnownCell( sim, x1, y1 ) ~= nil then
						local params = {color ={{symbol="inner_line",r=1,g=0,b=0,a=0.75},{symbol="wall_digital",r=1,g=0,b=0,a=0.75},{symbol="boxy_tail",r=1,g=0,b=0,a=0.75},{symbol="boxy",r=1,g=0,b=0,a=0.75}} }
    						sim:dispatchEvent( simdefs.EV_CAM_PAN, { target:getLocation() } )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 30)
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/guard/MFghost_exitMF" )
						sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = target, kanim = "fx/deamon_ko", symbol = "effect", anim="in", above=true, params=params} )
						sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 10)
					end
				end
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, { txt=txt, color=cdefs.COLOR_AI_WARNING, sound = sound, icon=slot.icon } )
			else
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, { txt=STRINGS.PROGEXTEND.AI.UNKNOWN_DESC, color=cdefs.COLOR_AI_WARNING, sound = "SpySociety/Actions/mainframe_wisp_activate", icon="gui/icons/ai_icons/icon-ai_unknown.png" } )
			end
		end,

		identifySubroutine = function(self, sim)
			local unidentified = {}
			for i, routine in pairs(self.slotsPWR) do
				if routine.identified ~= true then
					table.insert(unidentified, routine)
				end
			end
			for i, routine in pairs(self.slotsPro) do
				if routine.identified ~= true then
					table.insert(unidentified, routine)
				end
			end
			for i, routine in pairs(self.slotsRea) do
				if routine.identified ~= true then
					table.insert(unidentified, routine)
				end
			end
			for i, routine in pairs(self.slotsUti) do
				if routine.identified ~= true then
					table.insert(unidentified, routine)
				end
			end

			if #unidentified > 0 then
				unidentified[sim:nextRand(1, #unidentified)].identified = true
				sim:dispatchEvent( "PE_UPDATE_AI_INFO" )
			end
		end,
	},
}

return npc_abilities2